/*
 * Accelerometer.c
 *
 *  Author: Claire Gaffney
 *
 *  Purpose: This function enables I2C communication between the RSL10 and KX126-1063 accelerometer.
 *
 */


#include "rsl10.h"
#include <stdio.h>
#include <stdlib.h>


//initialize variables
volatile uint8_t AccDataX    = 0;
volatile uint8_t AccDataY    = 0;
volatile uint8_t AccDataZ    = 0;
volatile uint8_t MovingFlag  = 0;
enum state
{
	I2C_IDLE, I2C_START, I2C_WRITE_FINISH, I2C_READ_FINISH
};
volatile enum state i2c_state=  I2C_IDLE;


void main(void)
{

// initialize i2c
I2C_Init();

if (i2c_state == I2C_START)
	{
		i2c_state = I2C_IDLE;
		while ((Sys_I2C_Get_Status() & (1 << I2C_STATUS_BUS_FREE_Pos)) !=
				I2C_BUS_FREE);
		tx_buffer_index = 0;
		Sys_I2C_StartWrite(0x00);
	}

	//After getting data from all 4 LEDs

	//fill transfer buffer with commands for x output
	I2C_DATA=0x08;
	Sys_I2C_STARTWRITE(0x00);
	Sys_I2C_ACK();
	Sys_I2C_STARTREAD(0x00);

	//get result from receive buffer
	AccDataX = I2C_DATA;
	Sys_I2C_ACK();


	//fill transfer buffer with commands for y output
	I2C_DATA=0x0A;
	Sys_I2C_STARTWRITE(0x00);
	Sys_I2C_ACK();
	SYS_I2C_STARTREAD(0x00);

	//get result from receive buffer
	AccDataY = I2C_DATA;
	Sys_I2C_ACK();

	//fill transfer buffer with commands for z output
	I2C_DATA=0x0C;
	Sys_I2C_STARTWRITE(0x00);
	Sys_I2C_ACK();
	Sys_I2C_STARTREAD(0x00);

	//get result from receive buffer: AccDataY=I2C_RX_BUF?
	AccDataZ = I2C_DATA;
	Sys_I2C_ACK();


	if (AccDataX!=0 || AccDataY!=0 || AccDataZ!=0){
		MovingFlag=1;}
		else {
			MovingFlag=0;

	}
}

void I2C_Init(void)
{
		//Mask all interrupts
	    __set_PRIMASK(PRIMASK_DISABLE_INTERRUPTS);

	    //Disable all existing interrupts, clearing all pending source
	    Sys_NVIC_DisableAllInt();
	    Sys_NVIC_ClearAllPendingInt();
	    Sys_I2C_Config(I2C_MASTER_SPEED_120  | I2C_CONTROLLER_CM3   |
	                   I2C_STOP_INT_ENABLE   | I2C_AUTO_ACK_DISABLE |
	                   I2C_SAMPLE_CLK_ENABLE | I2C_SLAVE_DISABLE);

	Sys_I2C_DIOConfig(APP_DIO_CFG,8,7);

		//Enable interrupts
	    NVIC_EnableIRQ(DIO0_IRQn);
	    NVIC_EnableIRQ(I2C_IRQn);

	    //Unmask all interrupts
	    __set_PRIMASK(PRIMASK_ENABLE_INTERRUPTS);
}
