/*
 * FlashStorage.c
 *
 *  Created on: May 1, 2018
 *      Author: clairegaffney
 */


/*
 * spiMemoryWrite.c
 *
 *  Created on: May 1, 2018
 *      Author: matt
 */
/*
 * Description: this program's task is to allow the RSL10 to write data to
 * the flash memory chip through SPI protocol. The chip has already been erased
 * and initialized.
 *
 * This program is loaded onto the RSL10.
 *
 * The data is written to the memory chip's data buffer and then that is
 * written to a location in memory.
 *
 *
 * System clock = 3MHz
 */

#define RSL10_CID			101
#include "rsl10.h"
#include <stdio.h> //standard input/output
#include <stdlib.h> //standard library

//definitions
#define CONCAT(x, y)		x##y //concatenation operator: token pasting; two tokens on either side of ## operator are combined into a single token
#define SPI1_WORD_SIZE(x)	CONCAT(SPI1_WORD_SIZE_, x) // defining word size
#define DIO_SRC(x)			CONCAT(DIO_SRC_DIO_, x) //selecting DIO as source
#define SPI_TX_DATA			"RSL10 includes two SPI"//----> this was the example data transmitted but that's gonna change oh yeah
#define TX_BUFFER_SIZE		(sizeof SPI_TX_DATA) // defining the size of the transmit buffer based on the size of the data being transmitted (NICE!)
#define RX_BUFFER_SIZE		(2 * TX_BUFFER_SIZE) //size of data reception buffer should be 2 times that of transmission (idk why)
#define APP_DIO_CFG			DIO_6X_DRIVE | DIO_LPF_DISABLE | DIO_WEAK_PULL_UP

#define SPI_CLK_DIO			8 //SPI clock
#define SPI_CS_DIO			1 //chip select
#define SPI_SERI_DIO			3 //SOMI
#define SPI_SERO_DIO			2 //SIMO
#define BUTTON_DIO			2 //6 does bluetooth, 2 does data collection
#define WORD_SIZE			8 //this is for 32 byte data but this may need to change idk

//declare/initialize variables
/*
 * volatile: qualifier applied to a variable; tells the compiler that the value of the variable
 * may change at any time--without any action being taken by the code the compiler finds nearby
 *
 * unit8_t = unsigned char
 * uint16_t = unsigned short
 * uint32_t = unsigned int
 * uint64_t = unsigned long long
 */

unsigned short col = 0x000;
unsigned short page = 0x0000;
unsigned short measure = 0x0000; //measurements get read from RX buffer into this variable -> this is transmitted to storage
int sleep = 0;
int memoryFull = 0; //is the flash memory full? (1)
int memoryFullRead = 0; //sets to (1) if all pages have been read

unsigned short pageReadOut = 0x0000; //for page we are currently reading out from
unsigned short colReadOut = 0x0000; //for column we are currently reading out from
char sendingByteArray[2048] = {0x00};
unsigned short finalCol = 0x000;
unsigned short finalPage = 0x0000;

unsigned short eraseProgPage = 0x0000; //what page (block marker) are we on
   	//marks which block to erase

 unsigned short pageReadOut = 0x0000; //for page we are currently reading out from
unsigned short colReadOut = 0x0000; //for column we are currently reading out from (might not need this??)

//declare functions
void SPI_Initialize(void); 				//initialize SPI protocol
void iteratePos(void);
void iterateReadPage(void);
//void allocateArray();
int SPIMemoryWrite(void);
int SPIMemoryRead(void);
int SPIMemoryErase(void);



int SPIMemoryWrite(void){
	/*Initialize the system*/
	SPI_Initialize();

	unsigned char dat1, dat2,col_a,col_b,page_a,page_b; //initialize 2 8 bit values to store data
	unsigned short unshiftedByte=measure&0xFF00; //take highest value 8bits
	dat1=unshiftedByte>>8; //dat1 = highest bits
	dat2=measure&0xFF; //dat2= lower bits

	unshiftedByte=col&0xFF00;
	col_a=unshiftedByte>>8; //1st half of column address
	col_b=col&0xFF; //2nd half of column address

	unshiftedByte=page&0xFF00;
	page_a=unshiftedByte>>8; //1st half of page address
	page_b=page&0xFF; //2nd half of page address

	Sys_GPIO_Set_Low(SPI_CS_DIO); //set /CS low
	SPI1->TX_DATA=0x06; //write enable
	Sys_SPI_Write(1,8); //transmit the byte
	Sys_GPIO_Set_High(SPI_CS_DIO); //set /CS high

	Sys_GPIO_Set_Low(SPI_CS_DIO); //set /CS low
	SPI1->TX_DATA=0x02; //load program data
	Sys_SPI_Write(1,8); //transmit the byte
	SPI1->TX_DATA=col_a; //16bit column address a
	Sys_SPI_Write(1,8); //transmit the byte
	SPI1->TX_DATA=col_b; //16bit column address b
	Sys_SPI_Write(1,8); //transmit the byte
	SPI1->TX_DATA=dat1; //load program data highest bits
	Sys_SPI_Write(1,8); //transmit the byte
	SPI1->TX_DATA=dat2; //load program data lowest bits
	Sys_SPI_Write(1,8); //transmit the byte
	Sys_GPIO_Set_High(SPI_CS_DIO); //set /CS high

	Sys_GPIO_Set_Low(SPI_CS_DIO); //set /CS low
	SPI1->TX_DATA=0x10; //load program data
	Sys_SPI_Write(1,8); //transmit the byte
	SPI1->TX_DATA=0x00; //8 dummy clocks
	Sys_SPI_Write(1,8); //transmit the byte
	SPI1->TX_DATA=page_a; //16bit page address a
	Sys_SPI_Write(1,8); //transmit the byte
	SPI1->TX_DATA=page_b; //16bit page address b
	Sys_GPIO_Set_High(SPI_CS_DIO); //set /CS high

	iteratePos(); //move to next (col,page) memory position

	//when the flow moves out of this program, address "sleep" status - 1 = stop collecting data
}

void iteratePos(void){
	col=col+0x2; //iterate column by 2 bytes
	if(col==0x800){ //not 0x840
		if(page==0xFFFF){ //final page in memory
			sleep=1;
			memoryFull=1;
		}else{ //any page that's not the last one
			col=0x000; //reset column
			page++; //iterate page by 1 page
		}
	}
}

void iterateReadPage(void){ //iterates page when it's not the last data page
	if(memoryFullRead==1){
		return;
	}
	if(pageReadOut==0xFFFF){ //was the last page
		memoryFullRead=1;
	} else{
		pageReadOut=pageReadOut+0x0001;
	}
	return;
}

int SPIMemoryRead(void)
{
	if(col==0x000 & page!=0x0000){ //so we can stop reading at the right memory value
			finalCol=0x7FF;
			finalPage=page-0x1;
		}else{
			if(col==0x000 & page==0x0000){
				finalCol=col;
			}else{
				finalCol=col-0x1;
			}
			finalPage=page;
		}

		//allocateArray();
		/*Initialize the system*/
		SPI_Initialize();

		//memory should default to BUF=1 -> buffer read (we want this)
		//if not then set with config register
		//write status reg (1Fh/01h) -> SR address (config reg, SR-2?) 8bit -> write all bits to register

		unsigned char pageRead_a,pageRead_b; //initialize 2 8 bit values to store data
		unsigned short unshiftedByte=pageReadOut&0xFF00; //take highest value 8bits
		pageRead_a=unshiftedByte>>8; //pageRead_a = highest bits
		pageRead_b=pageReadOut&0xFF; //pageRead_b= lower bits

		Sys_GPIO_Set_Low(SPI_CS_DIO); //set /CS low
		SPI1->TX_DATA=0x13; //page data read
		Sys_SPI_Write(1,8); //transmit the byte
		SPI1->TX_DATA=0x00; //8 bit dummy clk
		Sys_SPI_Write(1,8); //transmit the byte
		SPI1->TX_DATA=pageRead_a; //1st byte of page address to read into buffer
		Sys_SPI_Write(1,8); //transmit the byte
		SPI1->TX_DATA=pageRead_b; //1st byte of page address to read into buffer
		Sys_SPI_Write(1,8); //transmit the byte
		Sys_GPIO_Set_High(SPI_CS_DIO); //set /CS high

		Sys_GPIO_Set_Low(SPI_CS_DIO); //set /CS low
		SPI1->TX_DATA=0x03; //read data
		Sys_SPI_Write(1,8); //transmit the byte
		SPI1->TX_DATA=0x00; //MSB of starting column (0x000)
		Sys_SPI_Write(1,8); //transmit the byte
		SPI1->TX_DATA=0x00; //LSBs of starting column (0x000)
		Sys_SPI_Write(1,8); //transmit the byte
		SPI1->TX_DATA=0x00; //8 bit dummy clock
		for(int i=0; i<2048; i++){ //iterate through columns/bytes
			sendingByteArray[i]=SPI1->RX_DATA; //read data out from receive buffer
			Sys_SPI_Read(1,8); //why do we need this?
			if(i==(finalCol) && pageReadOut == finalPage){ //if the column just read is the last data
				memoryFullRead=1; //status: all memory has been read -> end read function
				break;
			}
			//for loop will end when i becomes 2048: only ends prematurely with
			//final memory position being read
		}
		Sys_GPIO_Set_High(SPI_CS_DIO); //set /CS high

		iterateReadPage();

		return(1);
}

void SPI_Initialize(){
	//run a bunch of functions from the firmware documentation to set up the SPI for the memory component

	//remember: we are writing code for the RSL10!!
	//clock stuff - LEDs flash rate?

	/*Mask all interrupts --not sure if I need this, but it was in the example code*/
		__set_PRIMASK(PRIMASK_DISABLE_INTERRUPTS); //??

		Sys_NVIC_DisableAllInt(); //disables all external interrupts
		Sys_NVIC_ClearAllPendingInt(); //clear pending status for all external interrupts

	/*Configure SPI1 as Master (SPI1 for RSL10/flash memory)
	 	 * disable overrun and underrun interrupts
		 * Configure SPI1 for master-mode writes of 32 bit data (controlled by the ARM Cortex-M3 processor - CM3),
		 * running the SPI1 at 1/4 of the system clock frequency (3 MHz -> .75 MHz = 750 kHz)
		 * clk polarity normal -> serial output changes on falling edge of clk -> serial input sampled at rising edge of clk */
		Sys_SPI_Config(1, SPI1_OVERRUN_INT_DISABLE | SPI1_UNDERRUN_INT_DISABLE | SPI1_CONTROLLER_CM3 | SPI1_SELECT_MASTER |
				SPI1_CLK_POLARITY_NORMAL | SPI1_MODE_SELECT_MANUAL | SPI1_ENABLE | SPI1_PRESCALE_4);

	/*Configure the type of transfer for SPI 1
		 * inputs: SPI interface to configure (0/1),
		 * SPI interface initializes to idle
		 * we are writing to flash through SPI
		 * chip signal high - disables memory SPI - this is fine, we'll pull low when we want to be active -
		 * NEED TO DEFINE "WORD SIZE"--how much data to send to memory--in this case, it's 8-bit data??? 1 byte*/
		Sys_SPI_TransferConfig(1, SPI1_IDLE | SPI1_WRITE_DATA | SPI1_CS_1 | SPI1_WORD_SIZE_8);

	/*Configure the DIOs for SPI1 as Master Mode
		 * Sys_SPI_DIOConfig(num, slave, cfg, clk, cs, serio, sero)
		 * num   - interface (0/1)                         = 1
		 * slave - master/slave configuration              = master
		 * cfg   - DIO pin configuration for the SPI pads  = ??
		 * clk   - DIO to use as the SPI clock pad         = 8
		 * cs 	 - DIO to use as the chip select pad       = 1
		 * seri  - DIO to use as the SPI serial input pad  = (SPISOMI) -master in 3
		 * sero  - DIO to use as the SPI serial output pad = (SPISIMO) -master out 2
		 *
		 * */
		Sys_SPI_DIOConfig(1, SPI1_SELECT_MASTER, APP_DIO_CFG, 8, 1, 3, 2);

		/*Enable interrupts*/
			NVIC_EnableIRQ(DIO1_IRQn); //what are any of these??
			NVIC_EnableIRQ(SPI1_RX_IRQn);
			NVIC_EnableIRQ(SPI1_TX_IRQn);

		/*Stop masking interrupts*/
			__set_PRIMASK(PRIMASK_ENABLE_INTERRUPTS); //what is primask
			__set_FAULTMASK(FAULTMASK_ENABLE_INTERRUPTS); //what is faultmask


}

int SPIMemoryErase(void){
	SPI_Initialize();

	    	unsigned char pageErase_a,pageErase_b; //initialize 2 8 bit values to store page address
	    	unsigned short unshiftedByte=eraseProgPage&0xFF00; //take highest value 8bits
	    	pageErase_a=unshiftedByte>>8; //pageErase_a = highest bits
	    	pageErase_b=eraseProgPage&0xFF; //pageErase_b= lower bits

	    	for(int i=0;i<1023;i++){
	    		Sys_GPIO_Set_Low(SPI_CS_DIO); //set /CS low
	    		SPI1->TX_DATA=0xD8; //128kB block erase
	    		Sys_SPI_Write(1,8); //transmit the byte
	    		SPI1->TX_DATA=0X00; //8 dummy clocks
	    		Sys_SPI_Write(1,8);
	    		SPI1->TX_DATA=pageErase_a; //transmit MSBs of page address
	    		Sys_SPI_Write(1,8);
	    		SPI1->TX_DATA=pageErase_b; //transmit LSBs of page address
	    		Sys_SPI_Write(1,8);
	    		Sys_GPIO_Set_High(SPI_CS_DIO); //set /CS high

	    		if(eraseProgPage==0xFFC0){ //has the last block been cleared? 65535-64=65472
	    			break;
	    		}else{ //more blocks to erase: increment by 64 pages to access next block
	    			eraseProgPage=eraseProgPage+64; //inc by 64 page addresses
	    		}

	    		unshiftedByte=eraseProgPage&0xFF00;
	    		pageErase_a=unshiftedByte>>8; //pageErase_a = highest bits
	    		pageErase_b=eraseProgPage&0xFF; //pageErase_b= lower bits
	    	}
}



