% Calculate AI, IPA, and PPT 
function [IPAAverage,AIAverage,PPTAverage]=Analysis(concHbO2)
Frequency = 500;
resample_factor= 1;
resample_factor_conc = 1;

%% Find peaks of concentration waveform

concHbO2_smooth=smooth(concHbO2,0.01,'loess');
length_concHbO2=length(concHbO2_smooth);
[peaksconc,locsconc]=findpeaks(concHbO2_smooth,1:length_concHbO2,'MinPeakProminence',0.00004);
findpeaks(concHbO2_smooth,1:length_concHbO2,'MinPeakProminence',0.00004);
numpeaksconc=length(peaksconc);
% Find minimums

conc_neg=concHbO2_smooth*(-1);
[minconc,locsminconc]=findpeaks(conc_neg,1:length_concHbO2,'MinPeakProminence',0.0001);
findpeaks(conc_neg,1:length_concHbO2,'MinPeakProminence',0.0001);
minconc=minconc*(-1);
numminsconc=length(minconc);

%% Isolate one waveform - make more robust by using concavity
RiseTimeAggregate=0;
PWDAggregate=0;
PPTAggregate=0;
DiastolicPhaseAggregate=0;
AIAggregate=0;
IPAAggregate=0;
count=0;
for n=1:(numminsconc-3)
    min1loc=locsminconc(n+1);
    min2loc=locsminconc(n+2);
    
    length_onewave= min2loc-min1loc;
    for i=1:length_onewave
        onewave(i)=concHbO2_smooth(i+min1loc);
    end
    %plot(onewave);
    [onewave_peaks,onewave_peaks_locs]=findpeaks(onewave,1:length_onewave,'MinPeakProminence',0.00002);
    
    systolic_peak=onewave_peaks(1);
    systolic_loc=onewave_peaks_locs(1);
    
    if ((length(onewave_peaks))>1)
        diastolic_peak=onewave_peaks(2);
        diastolic_loc=onewave_peaks_locs(2);

        baseline= onewave(1);

        % Find dicrotic notch, systolic peak = Pulse wave amplitude (PWA)
        onewave_neg=onewave*(-1);
        %findpeaks(onewave_neg,1:length_onewave,'MinPeakProminence',0.00003)
        [dicrotic_notch,locdicrotic]=findpeaks(onewave_neg,1:length_onewave,'MinPeakProminence',0.000003);
        dicrotic=dicrotic_notch*(-1);
        mag_dicrotic=dicrotic-baseline;
        mag_systolic=systolic_peak-baseline;
        mag_diastolic=diastolic_peak-baseline;
    
        %% Timing
        
        % Rise time = Systolic Phase
        RiseTime=systolic_loc*resample_factor_conc*(1/Frequency);
        
        % Pulse Wave Duration = Length (onewave)
        PWD = length_onewave*resample_factor_conc*(1/Frequency);
        
        % Pulse Propogation Time
        PPT = (diastolic_loc-systolic_loc)*resample_factor_conc*(1/Frequency);
        
        % Diastolic Phase
        DiastolicPhase = (length_onewave-systolic_loc)*resample_factor_conc*(1/Frequency);
        
        % Augmentation Index
        AI=mag_diastolic/mag_systolic;
        
        %% Integrate under signal
        
        %A1
        for k=1:locdicrotic
            A1(k)=onewave(k);
        end
        
        %A2
        A2_length=length_onewave-locdicrotic;
        for s=1:A2_length
            A2(s)=onewave(s-1+locdicrotic);
        end
        
        % Inflection Point Area Ratio
        A1_trap=trapz(A1);
        A2_trap=trapz(A2);
        
        IPA=A2_trap/A1_trap;
        
        
        %% Calculate Averages
        RiseTimeAggregate=RiseTimeAggregate+RiseTime;
        PWDAggregate = PWDAggregate + PWD;
        PPTAggregate = PPTAggregate + PPT;
        DiastolicPhaseAggregate = DiastolicPhaseAggregate+ DiastolicPhase;
        AIAggregate = AIAggregate + AI;
        IPAAggregate = IPAAggregate + IPA;
        count=count+1;
        
    end
    clear onewave;
end
    RiseTimeAverage=RiseTimeAggregate/count;
    PWDAverage=PWDAggregate/count;
    PPTAverage=PPTAggregate/count;
    DiastolicPhaseAverage=DiastolicPhaseAggregate/count;
    AIAverage=AIAggregate/count;
    IPAAverage=IPAAggregate/count;
end

