/* ----------------------------------------------------------------------------
 * Copyright (c) 2015-2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ----------------------------------------------------------------------------
 * rsl10_protocol.h
 * - BLE protocol support
 * ----------------------------------------------------------------------------
 * $Revision: 1.3 $
 * $Date: 2017/11/09 21:36:29 $
 * ------------------------------------------------------------------------- */

#ifndef RSL10_PROTOCOL_H
#define RSL10_PROTOCOL_H

/* ----------------------------------------------------------------------------
 * If building with a C++ compiler, make all of the definitions in this header
 * have a C binding.
 * ------------------------------------------------------------------------- */
#ifdef __cplusplus
extern "C"
{
#endif

/* ----------------------------------------------------------------------------
 * Global variables and types
 * --------------------------------------------------------------------------*/

typedef enum
{
    FLASH_PROVIDED_or_DFLT,
    APP_PROVIDED,
} DEVICE_PARAM_SRC_TYPE;

typedef enum
{
    PARAM_ID_PUBLIC_BLE_ADDRESS,
    PARAM_ID_IRK,
    PARAM_ID_CSRK,
    PARAM_ID_PRIVATE_KEY,
    PARAM_ID_PUBLIC_KEY,

    PARAM_ID_CLOCK_ACCURACY,

    PARAM_ID_ADV_IFS,

    PARAM_ID_BLE_CA_TIMER_DUR,
    PARAM_ID_BLE_CRA_TIMER_CNT,
    PARAM_ID_BLE_CA_MIN_THR,
    PARAM_ID_BLE_CA_MAX_THR,
    PARAM_ID_BLE_CA_NOISE_THR,
} DEVICE_PARAM_ID;

typedef struct
{
    uint8_t device_param_src_type;
    uint8_t bleAddress[6];
    uint8_t irk[16];
    uint8_t csrk[16];
    uint8_t privateKey[32];
    uint8_t publicKey_x[32];
    uint8_t publicKey_y[32];

    uint16_t clockAccuracy;

    uint32_t adv_ifs;

    uint8_t chnlAsses_param_src_type;
    uint16_t chnlAsses_timer_duration;
    uint8_t chnlAsses_timer_cnt;
    uint8_t chnlAsses_min_thr;
    uint8_t chnlAsses_max_thr;
    uint8_t chnlAsses_noise_thr;
} app_device_param_t;

/* ----------------------------------------------------------------------------
 * Function prototypes
 * ------------------------------------------------------------------------- */

extern void Device_Param_Prepare(app_device_param_t *param);
extern uint8_t Device_Param_Read(uint8_t requestedId, uint8_t *buf);

/* ----------------------------------------------------------------------------
 * Close the 'extern "C"' block
 * ------------------------------------------------------------------------- */
#ifdef __cplusplus
}
#endif

#endif /* RSL10_PROTOCOL_H */
