/* ----------------------------------------------------------------------------
 * Copyright (c) 2014-2017 Semiconductor Components Industries, LLC (d/b/a ON
 * Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ----------------------------------------------------------------------------
 * rsl10_hw.h
 * - ARM Cortex-M3 accessible hardware registers and bit-field definitions
 * ----------------------------------------------------------------------------
 * $Revision: 1.4 $
 * $Date: 2017/06/07 19:54:21 $
 * ------------------------------------------------------------------------- */

#ifndef RSL10_HW_H
#define RSL10_HW_H

/* Include the system library register header */
#include <rsl10_reg.h>

/* ----------------------------------------------------------------------------
 * Chip Identification
 * ------------------------------------------------------------------------- */

typedef struct
{
    __I  uint32_t CHIP_ID_NUM;          /* Chip ID number */
} AHBREGS_Type;

#define AHBREGS_BASE                    0x1FFFFFFC
#define AHBREGS                         ((AHBREGS_Type *) AHBREGS_BASE)

/* AHBREGS_CHIP_ID_NUM bit positions */
#define AHBREGS_CHIP_ID_NUM_CHIP_FAMILY_Pos 24
#define AHBREGS_CHIP_ID_NUM_CHIP_FAMILY_Mask ((uint32_t)(0xFFU << AHBREGS_CHIP_ID_NUM_CHIP_FAMILY_Pos))
#define AHBREGS_CHIP_ID_NUM_CHIP_VERSION_Pos 16
#define AHBREGS_CHIP_ID_NUM_CHIP_VERSION_Mask ((uint32_t)(0xFFU << AHBREGS_CHIP_ID_NUM_CHIP_VERSION_Pos))
#define AHBREGS_CHIP_ID_NUM_CHIP_MAJOR_REVISION_Pos 8
#define AHBREGS_CHIP_ID_NUM_CHIP_MAJOR_REVISION_Mask ((uint32_t)(0xFFU << AHBREGS_CHIP_ID_NUM_CHIP_MAJOR_REVISION_Pos))
#define AHBREGS_CHIP_ID_NUM_CHIP_MINOR_REVISION_Pos 3
#define AHBREGS_CHIP_ID_NUM_CHIP_MINOR_REVISION_Mask ((uint32_t)(0x1FU << AHBREGS_CHIP_ID_NUM_CHIP_MINOR_REVISION_Pos))
#define AHBREGS_CHIP_ID_NUM_OD_FEATURE_Pos 2
#define AHBREGS_CHIP_ID_NUM_LPDSP32_FEATURE_Pos 1
#define AHBREGS_CHIP_ID_NUM_AOBLE_FEATURE_Pos 0

/* AHBREGS_CHIP_ID_NUM sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __I  uint8_t CHIP_MAJOR_REVISION_BYTE;
    __I  uint8_t CHIP_VERSION_BYTE;    
    __I  uint8_t CHIP_FAMILY_BYTE;     
} AHBREGS_CHIP_ID_NUM_Type;

#define AHBREGS_CHIP_ID_NUM_BASE        0x1FFFFFFC
#define AHBREGS_CHIP_ID_NUM             ((AHBREGS_CHIP_ID_NUM_Type *) AHBREGS_CHIP_ID_NUM_BASE)

/* AHBREGS_CHIP_ID_NUM settings */
#define CHIP_ID_AOBLE_NOT_PRESENT       ((uint32_t)(0x0U << AHBREGS_CHIP_ID_NUM_AOBLE_FEATURE_Pos))
#define CHIP_ID_AOBLE_PRESENT           ((uint32_t)(0x1U << AHBREGS_CHIP_ID_NUM_AOBLE_FEATURE_Pos))

#define CHIP_ID_LPDSP32_NOT_PRESENT     ((uint32_t)(0x0U << AHBREGS_CHIP_ID_NUM_LPDSP32_FEATURE_Pos))
#define CHIP_ID_LPDSP32_PRESENT         ((uint32_t)(0x1U << AHBREGS_CHIP_ID_NUM_LPDSP32_FEATURE_Pos))

#define CHIP_ID_OD_NOT_PRESENT          ((uint32_t)(0x0U << AHBREGS_CHIP_ID_NUM_OD_FEATURE_Pos))
#define CHIP_ID_OD_PRESENT              ((uint32_t)(0x1U << AHBREGS_CHIP_ID_NUM_OD_FEATURE_Pos))

/* ----------------------------------------------------------------------------
 * System Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t DSS_CTRL;             /* DSS Control */
    __O  uint32_t DSS_CMD;              /* DSS Commands */
    __IO uint32_t FLASH_OVERLAY_CFG;    /* Flash Overlay Configuration */
    __IO uint32_t CSS_LOOP_CACHE_CFG;   /* CSS Loop Cache Configuration */
    __IO uint32_t DSS_LOOP_CACHE_CFG;   /* DSS Loop Cache Configuration */
    __IO uint32_t MEM_ERROR;            /* Memory Access Error Flags */
    __IO uint32_t MEM_POWER_CFG;        /* Memory Power Configuration */
    __IO uint32_t MEM_ACCESS_CFG;       /* Memory Access Configuration and Wakeup Restore Address in packed 7-bit format */
    __IO uint32_t WAKEUP_ADDR;          /* Wakeup Restore Address in Unpacked 32-bit Format */
    __IO uint32_t MEM_RETENTION_CFG;    /* Memory Retention Configuration */
    __IO uint32_t MEM_ARBITER_CFG;      /* Memory Arbiter Configuration */
    __IO uint32_t MEM_TIMING_CFG;       /* Memory Timing Configuration */
    __IO uint32_t CNT_CTRL;             /* Activity Counters Control */
    __IO uint32_t SYSCLK_CNT;           /* System Clock Counter Value */
    __IO uint32_t CM3_CNT;              /* CM3 Activity Counter Value */
    __IO uint32_t LPDSP32_CNT;          /* LPDSP32 Activity Counter Value */
    __IO uint32_t FLASH_READ_CNT;       /* Flash Read Access Counter Value */
         uint32_t RESERVED0;
    __IO uint32_t SPEED_MEASURE;        /* Critical Path Speed Measurement */
    __IO uint32_t LPDSP32_DEBUG_CFG;    /* LPDSP32 Debug Port Configuration */
    __IO uint32_t RF_POWER_CFG;         /* RF Power Configuration */
    __IO uint32_t RF_ACCESS_CFG;        /* RF Access Configuration */
    __I  uint32_t WAKEUP_PAD;           /* WAKEUP Pad Value */
         uint32_t RESERVED1[32];
    __IO uint32_t DBG_LOCK;             /* Debug Port Access Configuration */
    __IO uint32_t DBG_LOCK_KEY[4];      /* Debug Port Lock Key Part 0 to 3 */
} SYSCTRL_Type;

#define SYSCTRL_BASE                    0x40000000
#define SYSCTRL                         ((SYSCTRL_Type *) SYSCTRL_BASE)

/* SYSCTRL_DSS_CTRL bit positions */
#define SYSCTRL_DSS_CTRL_LPDSP32_STATUS_Pos 8
#define SYSCTRL_DSS_CTRL_DSS_CSS_INT_RESET_Pos 5
#define SYSCTRL_DSS_CTRL_DSS_DMA_INT_RESET_Pos 4
#define SYSCTRL_DSS_CTRL_DSS_RESET_Pos  3
#define SYSCTRL_DSS_CTRL_LPDSP32_PAUSE_Pos 2
#define SYSCTRL_DSS_CTRL_LPDSP32_RESUME_Pos 1
#define SYSCTRL_DSS_CTRL_LPDSP32_RUNNING_Pos 0

/* SYSCTRL_DSS_CTRL bit-band aliases */
typedef struct
{
    __I  uint32_t LPDSP32_RUNNING_ALIAS;/* LPDSP32 running status */
    __O  uint32_t LPDSP32_RESUME_ALIAS; /* Write a 1 to run LPDSP32 */
    __O  uint32_t LPDSP32_PAUSE_ALIAS;  /* Write a 1 to pause LPDSP32 */
    __O  uint32_t DSS_RESET_ALIAS;      /* Write a 1 to reset DSS */
    __O  uint32_t DSS_DMA_INT_RESET_ALIAS;/* Write a 1 to reset pending DMA interrupts in the DSS interrupt controller */
    __O  uint32_t DSS_CSS_INT_RESET_ALIAS;/* Write a 1 to reset pending CSS interrupts in the DSS interrupt controller */
         uint32_t RESERVED0[2];
    __I  uint32_t LPDSP32_STATUS_ALIAS; /* LPDSP32 feature status */
} SYSCTRL_DSS_CTRL_Type;

#define SYSCTRL_DSS_CTRL_BASE           SYS_CALC_BITBAND(0x40000000, 0)
#define SYSCTRL_DSS_CTRL                ((SYSCTRL_DSS_CTRL_Type *) SYSCTRL_DSS_CTRL_BASE)

/* SYSCTRL_DSS_CTRL settings */
#define DSS_LPDSP32_STATE_PAUSE_BITBAND 0x0
#define DSS_LPDSP32_STATE_RUN_BITBAND   0x1
#define DSS_LPDSP32_STATE_PAUSE         ((uint32_t)(DSS_LPDSP32_STATE_PAUSE_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_RUNNING_Pos))
#define DSS_LPDSP32_STATE_RUN           ((uint32_t)(DSS_LPDSP32_STATE_RUN_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_RUNNING_Pos))

#define DSS_LPDSP32_RESUME_BITBAND      0x1
#define DSS_LPDSP32_RESUME              ((uint32_t)(DSS_LPDSP32_RESUME_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_RESUME_Pos))

#define DSS_LPDSP32_PAUSE_BITBAND       0x1
#define DSS_LPDSP32_PAUSE               ((uint32_t)(DSS_LPDSP32_PAUSE_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_PAUSE_Pos))

#define DSS_RESET_BITBAND               0x1
#define DSS_RESET                       ((uint32_t)(DSS_RESET_BITBAND << SYSCTRL_DSS_CTRL_DSS_RESET_Pos))

#define DSS_DMA_INT_RESET_BITBAND       0x1
#define DSS_DMA_INT_RESET               ((uint32_t)(DSS_DMA_INT_RESET_BITBAND << SYSCTRL_DSS_CTRL_DSS_DMA_INT_RESET_Pos))

#define DSS_CSS_INT_RESET_BITBAND       0x1
#define DSS_CSS_INT_RESET               ((uint32_t)(DSS_CSS_INT_RESET_BITBAND << SYSCTRL_DSS_CTRL_DSS_CSS_INT_RESET_Pos))

#define LPDSP32_DISABLED_DEVICE_BITBAND 0x0
#define LPDSP32_ENABLED_DEVICE_BITBAND  0x1
#define LPDSP32_DISABLED_DEVICE         ((uint32_t)(LPDSP32_DISABLED_DEVICE_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_STATUS_Pos))
#define LPDSP32_ENABLED_DEVICE          ((uint32_t)(LPDSP32_ENABLED_DEVICE_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_STATUS_Pos))

/* SYSCTRL_DSS_CMD bit positions */
#define SYSCTRL_DSS_CMD_DSS_CMD_6_Pos   6
#define SYSCTRL_DSS_CMD_DSS_CMD_5_Pos   5
#define SYSCTRL_DSS_CMD_DSS_CMD_4_Pos   4
#define SYSCTRL_DSS_CMD_DSS_CMD_3_Pos   3
#define SYSCTRL_DSS_CMD_DSS_CMD_2_Pos   2
#define SYSCTRL_DSS_CMD_DSS_CMD_1_Pos   1
#define SYSCTRL_DSS_CMD_DSS_CMD_0_Pos   0

/* SYSCTRL_DSS_CMD bit-band aliases */
typedef struct
{
    __O  uint32_t DSS_CMD_0_ALIAS;      /* Write a 1 to issue DSS command 0 */
    __O  uint32_t DSS_CMD_1_ALIAS;      /* Write a 1 to issue DSS command 1 */
    __O  uint32_t DSS_CMD_2_ALIAS;      /* Write a 1 to issue DSS command 2 */
    __O  uint32_t DSS_CMD_3_ALIAS;      /* Write a 1 to issue DSS command 3 */
    __O  uint32_t DSS_CMD_4_ALIAS;      /* Write a 1 to issue DSS command 4 */
    __O  uint32_t DSS_CMD_5_ALIAS;      /* Write a 1 to issue DSS command 5 */
    __O  uint32_t DSS_CMD_6_ALIAS;      /* Write a 1 to issue DSS command 6 */
} SYSCTRL_DSS_CMD_Type;

#define SYSCTRL_DSS_CMD_BASE            SYS_CALC_BITBAND(0x40000004, 0)
#define SYSCTRL_DSS_CMD                 ((SYSCTRL_DSS_CMD_Type *) SYSCTRL_DSS_CMD_BASE)

/* SYSCTRL_DSS_CMD settings */
#define DSS_CMD_0_BITBAND               0x1
#define DSS_CMD_0                       ((uint32_t)(DSS_CMD_0_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_0_Pos))

#define DSS_CMD_1_BITBAND               0x1
#define DSS_CMD_1                       ((uint32_t)(DSS_CMD_1_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_1_Pos))

#define DSS_CMD_2_BITBAND               0x1
#define DSS_CMD_2                       ((uint32_t)(DSS_CMD_2_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_2_Pos))

#define DSS_CMD_3_BITBAND               0x1
#define DSS_CMD_3                       ((uint32_t)(DSS_CMD_3_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_3_Pos))

#define DSS_CMD_4_BITBAND               0x1
#define DSS_CMD_4                       ((uint32_t)(DSS_CMD_4_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_4_Pos))

#define DSS_CMD_5_BITBAND               0x1
#define DSS_CMD_5                       ((uint32_t)(DSS_CMD_5_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_5_Pos))

#define DSS_CMD_6_BITBAND               0x1
#define DSS_CMD_6                       ((uint32_t)(DSS_CMD_6_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_6_Pos))

/* SYSCTRL_FLASH_OVERLAY_CFG bit positions */
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM0_OVERLAY_CFG_Pos 7
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM1_OVERLAY_CFG_Pos 6
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM2_OVERLAY_CFG_Pos 5
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM3_OVERLAY_CFG_Pos 4
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM3_OVERLAY_CFG_Pos 3
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM2_OVERLAY_CFG_Pos 2
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM1_OVERLAY_CFG_Pos 1
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM0_OVERLAY_CFG_Pos 0

/* SYSCTRL_FLASH_OVERLAY_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t PRAM0_OVERLAY_CFG_ALIAS;/* PRAM0 Flash overlay configuration */
    __IO uint32_t PRAM1_OVERLAY_CFG_ALIAS;/* PRAM1 Flash overlay configuration */
    __IO uint32_t PRAM2_OVERLAY_CFG_ALIAS;/* PRAM2 Flash overlay configuration */
    __IO uint32_t PRAM3_OVERLAY_CFG_ALIAS;/* PRAM3 Flash overlay configuration */
    __IO uint32_t DSP_PRAM3_OVERLAY_CFG_ALIAS;/* DSP_PRAM3 Flash overlay configuration */
    __IO uint32_t DSP_PRAM2_OVERLAY_CFG_ALIAS;/* DSP_PRAM2 Flash overlay configuration */
    __IO uint32_t DSP_PRAM1_OVERLAY_CFG_ALIAS;/* DSP_PRAM1 Flash overlay configuration */
    __IO uint32_t DSP_PRAM0_OVERLAY_CFG_ALIAS;/* DSP_PRAM0 Flash overlay configuration */
} SYSCTRL_FLASH_OVERLAY_CFG_Type;

#define SYSCTRL_FLASH_OVERLAY_CFG_BASE  SYS_CALC_BITBAND(0x40000008, 0)
#define SYSCTRL_FLASH_OVERLAY_CFG       ((SYSCTRL_FLASH_OVERLAY_CFG_Type *) SYSCTRL_FLASH_OVERLAY_CFG_BASE)

/* SYSCTRL_FLASH_OVERLAY_CFG settings */
#define PRAM0_OVERLAY_DISABLE_BITBAND   0x0
#define PRAM0_OVERLAY_ENABLE_BITBAND    0x1
#define PRAM0_OVERLAY_DISABLE           ((uint32_t)(PRAM0_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM0_OVERLAY_CFG_Pos))
#define PRAM0_OVERLAY_ENABLE            ((uint32_t)(PRAM0_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM0_OVERLAY_CFG_Pos))

#define PRAM1_OVERLAY_DISABLE_BITBAND   0x0
#define PRAM1_OVERLAY_ENABLE_BITBAND    0x1
#define PRAM1_OVERLAY_DISABLE           ((uint32_t)(PRAM1_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM1_OVERLAY_CFG_Pos))
#define PRAM1_OVERLAY_ENABLE            ((uint32_t)(PRAM1_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM1_OVERLAY_CFG_Pos))

#define PRAM2_OVERLAY_DISABLE_BITBAND   0x0
#define PRAM2_OVERLAY_ENABLE_BITBAND    0x1
#define PRAM2_OVERLAY_DISABLE           ((uint32_t)(PRAM2_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM2_OVERLAY_CFG_Pos))
#define PRAM2_OVERLAY_ENABLE            ((uint32_t)(PRAM2_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM2_OVERLAY_CFG_Pos))

#define PRAM3_OVERLAY_DISABLE_BITBAND   0x0
#define PRAM3_OVERLAY_ENABLE_BITBAND    0x1
#define PRAM3_OVERLAY_DISABLE           ((uint32_t)(PRAM3_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM3_OVERLAY_CFG_Pos))
#define PRAM3_OVERLAY_ENABLE            ((uint32_t)(PRAM3_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM3_OVERLAY_CFG_Pos))

#define DSP_PRAM3_OVERLAY_DISABLE_BITBAND 0x0
#define DSP_PRAM3_OVERLAY_ENABLE_BITBAND 0x1
#define DSP_PRAM3_OVERLAY_DISABLE       ((uint32_t)(DSP_PRAM3_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM3_OVERLAY_CFG_Pos))
#define DSP_PRAM3_OVERLAY_ENABLE        ((uint32_t)(DSP_PRAM3_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM3_OVERLAY_CFG_Pos))

#define DSP_PRAM2_OVERLAY_DISABLE_BITBAND 0x0
#define DSP_PRAM2_OVERLAY_ENABLE_BITBAND 0x1
#define DSP_PRAM2_OVERLAY_DISABLE       ((uint32_t)(DSP_PRAM2_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM2_OVERLAY_CFG_Pos))
#define DSP_PRAM2_OVERLAY_ENABLE        ((uint32_t)(DSP_PRAM2_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM2_OVERLAY_CFG_Pos))

#define DSP_PRAM1_OVERLAY_DISABLE_BITBAND 0x0
#define DSP_PRAM1_OVERLAY_ENABLE_BITBAND 0x1
#define DSP_PRAM1_OVERLAY_DISABLE       ((uint32_t)(DSP_PRAM1_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM1_OVERLAY_CFG_Pos))
#define DSP_PRAM1_OVERLAY_ENABLE        ((uint32_t)(DSP_PRAM1_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM1_OVERLAY_CFG_Pos))

#define DSP_PRAM0_OVERLAY_DISABLE_BITBAND 0x0
#define DSP_PRAM0_OVERLAY_ENABLE_BITBAND 0x1
#define DSP_PRAM0_OVERLAY_DISABLE       ((uint32_t)(DSP_PRAM0_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM0_OVERLAY_CFG_Pos))
#define DSP_PRAM0_OVERLAY_ENABLE        ((uint32_t)(DSP_PRAM0_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM0_OVERLAY_CFG_Pos))

/* SYSCTRL_CSS_LOOP_CACHE_CFG bit positions */
#define SYSCTRL_CSS_LOOP_CACHE_CFG_CSS_LOOP_CACHE_ENABLE_Pos 0

/* SYSCTRL_CSS_LOOP_CACHE_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t CSS_LOOP_CACHE_ENABLE_ALIAS;/* CSS loop cache enable */
} SYSCTRL_CSS_LOOP_CACHE_CFG_Type;

#define SYSCTRL_CSS_LOOP_CACHE_CFG_BASE SYS_CALC_BITBAND(0x4000000C, 0)
#define SYSCTRL_CSS_LOOP_CACHE_CFG      ((SYSCTRL_CSS_LOOP_CACHE_CFG_Type *) SYSCTRL_CSS_LOOP_CACHE_CFG_BASE)

/* SYSCTRL_CSS_LOOP_CACHE_CFG settings */
#define CSS_LOOP_CACHE_DISABLE_BITBAND  0x0
#define CSS_LOOP_CACHE_ENABLE_BITBAND   0x1
#define CSS_LOOP_CACHE_DISABLE          ((uint32_t)(CSS_LOOP_CACHE_DISABLE_BITBAND << SYSCTRL_CSS_LOOP_CACHE_CFG_CSS_LOOP_CACHE_ENABLE_Pos))
#define CSS_LOOP_CACHE_ENABLE           ((uint32_t)(CSS_LOOP_CACHE_ENABLE_BITBAND << SYSCTRL_CSS_LOOP_CACHE_CFG_CSS_LOOP_CACHE_ENABLE_Pos))

/* SYSCTRL_DSS_LOOP_CACHE_CFG bit positions */
#define SYSCTRL_DSS_LOOP_CACHE_CFG_DSS_LOOP_CACHE_ENABLE_Pos 0

/* SYSCTRL_DSS_LOOP_CACHE_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t DSS_LOOP_CACHE_ENABLE_ALIAS;/* DSS loop cache enable */
} SYSCTRL_DSS_LOOP_CACHE_CFG_Type;

#define SYSCTRL_DSS_LOOP_CACHE_CFG_BASE SYS_CALC_BITBAND(0x40000010, 0)
#define SYSCTRL_DSS_LOOP_CACHE_CFG      ((SYSCTRL_DSS_LOOP_CACHE_CFG_Type *) SYSCTRL_DSS_LOOP_CACHE_CFG_BASE)

/* SYSCTRL_DSS_LOOP_CACHE_CFG settings */
#define DSS_LOOP_CACHE_DISABLE_BITBAND  0x0
#define DSS_LOOP_CACHE_ENABLE_BITBAND   0x1
#define DSS_LOOP_CACHE_DISABLE          ((uint32_t)(DSS_LOOP_CACHE_DISABLE_BITBAND << SYSCTRL_DSS_LOOP_CACHE_CFG_DSS_LOOP_CACHE_ENABLE_Pos))
#define DSS_LOOP_CACHE_ENABLE           ((uint32_t)(DSS_LOOP_CACHE_ENABLE_BITBAND << SYSCTRL_DSS_LOOP_CACHE_CFG_DSS_LOOP_CACHE_ENABLE_Pos))

/* SYSCTRL_MEM_ERROR bit positions */
#define SYSCTRL_MEM_ERROR_MEM_ERROR_CLEAR_Pos 5
#define SYSCTRL_MEM_ERROR_BB_MEM_ERROR_Pos 4
#define SYSCTRL_MEM_ERROR_FLASH_COPIER_MEM_ERROR_Pos 3
#define SYSCTRL_MEM_ERROR_DMA_MEM_ERROR_Pos 2
#define SYSCTRL_MEM_ERROR_LPDSP32_DMEM_ERROR_Pos 1
#define SYSCTRL_MEM_ERROR_LPDSP32_PMEM_ERROR_Pos 0

/* SYSCTRL_MEM_ERROR bit-band aliases */
typedef struct
{
    __I  uint32_t LPDSP32_PMEM_ERROR_ALIAS;/* LPDSP32 program memory error flag */
    __I  uint32_t LPDSP32_DMEM_ERROR_ALIAS;/* LPDSP32 data memory error flag */
    __I  uint32_t DMA_MEM_ERROR_ALIAS;  /* DMA memory error flag */
    __I  uint32_t FLASH_COPIER_MEM_ERROR_ALIAS;/* Flash copier memory error flag */
    __I  uint32_t BB_MEM_ERROR_ALIAS;   /* Baseband memory error flag */
    __O  uint32_t MEM_ERROR_CLEAR_ALIAS;/* Write a 1 to clear the memory error flags */
} SYSCTRL_MEM_ERROR_Type;

#define SYSCTRL_MEM_ERROR_BASE          SYS_CALC_BITBAND(0x40000014, 0)
#define SYSCTRL_MEM_ERROR               ((SYSCTRL_MEM_ERROR_Type *) SYSCTRL_MEM_ERROR_BASE)

/* SYSCTRL_MEM_ERROR settings */
#define LPDSP32_PMEM_NO_ERROR_DETECTED_BITBAND 0x0
#define LPDSP32_PMEM_ERROR_DETECTED_BITBAND 0x1
#define LPDSP32_PMEM_NO_ERROR_DETECTED  ((uint32_t)(LPDSP32_PMEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_LPDSP32_PMEM_ERROR_Pos))
#define LPDSP32_PMEM_ERROR_DETECTED     ((uint32_t)(LPDSP32_PMEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_LPDSP32_PMEM_ERROR_Pos))

#define LPDSP32_DMEM_NO_ERROR_DETECTED_BITBAND 0x0
#define LPDSP32_DMEM_ERROR_DETECTED_BITBAND 0x1
#define LPDSP32_DMEM_NO_ERROR_DETECTED  ((uint32_t)(LPDSP32_DMEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_LPDSP32_DMEM_ERROR_Pos))
#define LPDSP32_DMEM_ERROR_DETECTED     ((uint32_t)(LPDSP32_DMEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_LPDSP32_DMEM_ERROR_Pos))

#define DMA_MEM_NO_ERROR_DETECTED_BITBAND 0x0
#define DMA_MEM_ERROR_DETECTED_BITBAND  0x1
#define DMA_MEM_NO_ERROR_DETECTED       ((uint32_t)(DMA_MEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_DMA_MEM_ERROR_Pos))
#define DMA_MEM_ERROR_DETECTED          ((uint32_t)(DMA_MEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_DMA_MEM_ERROR_Pos))

#define FLASH_COPIER_MEM_NO_ERROR_DETECTED_BITBAND 0x0
#define FLASH_COPIER_MEM_ERROR_DETECTED_BITBAND 0x1
#define FLASH_COPIER_MEM_NO_ERROR_DETECTED ((uint32_t)(FLASH_COPIER_MEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_FLASH_COPIER_MEM_ERROR_Pos))
#define FLASH_COPIER_MEM_ERROR_DETECTED ((uint32_t)(FLASH_COPIER_MEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_FLASH_COPIER_MEM_ERROR_Pos))

#define BB_MEM_NO_ERROR_DETECTED_BITBAND 0x0
#define BB_MEM_ERROR_DETECTED_BITBAND   0x1
#define BB_MEM_NO_ERROR_DETECTED        ((uint32_t)(BB_MEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_BB_MEM_ERROR_Pos))
#define BB_MEM_ERROR_DETECTED           ((uint32_t)(BB_MEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_BB_MEM_ERROR_Pos))

#define MEM_ERROR_CLEAR_BITBAND         0x1
#define MEM_ERROR_CLEAR                 ((uint32_t)(MEM_ERROR_CLEAR_BITBAND << SYSCTRL_MEM_ERROR_MEM_ERROR_CLEAR_Pos))

/* SYSCTRL_MEM_POWER_CFG bit positions */
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM5_POWER_Pos 21
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM4_POWER_Pos 20
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM3_POWER_Pos 19
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM2_POWER_Pos 18
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM1_POWER_Pos 17
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM0_POWER_Pos 16
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM3_POWER_Pos 15
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM2_POWER_Pos 14
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM1_POWER_Pos 13
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM0_POWER_Pos 12
#define SYSCTRL_MEM_POWER_CFG_BB_DRAM1_POWER_Pos 11
#define SYSCTRL_MEM_POWER_CFG_BB_DRAM0_POWER_Pos 10
#define SYSCTRL_MEM_POWER_CFG_DRAM2_POWER_Pos 8
#define SYSCTRL_MEM_POWER_CFG_DRAM1_POWER_Pos 7
#define SYSCTRL_MEM_POWER_CFG_DRAM0_POWER_Pos 6
#define SYSCTRL_MEM_POWER_CFG_PRAM3_POWER_Pos 5
#define SYSCTRL_MEM_POWER_CFG_PRAM2_POWER_Pos 4
#define SYSCTRL_MEM_POWER_CFG_PRAM1_POWER_Pos 3
#define SYSCTRL_MEM_POWER_CFG_PRAM0_POWER_Pos 2
#define SYSCTRL_MEM_POWER_CFG_FLASH_POWER_Pos 1
#define SYSCTRL_MEM_POWER_CFG_PROM_POWER_Pos 0

/* SYSCTRL_MEM_POWER_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t PROM_POWER_ALIAS;     /* PROM power configuration */
    __IO uint32_t FLASH_POWER_ALIAS;    /* Flash power configuration */
    __IO uint32_t PRAM0_POWER_ALIAS;    /* PRAM0 power configuration */
    __IO uint32_t PRAM1_POWER_ALIAS;    /* PRAM1 power configuration */
    __IO uint32_t PRAM2_POWER_ALIAS;    /* PRAM2 power configuration */
    __IO uint32_t PRAM3_POWER_ALIAS;    /* PRAM3 power configuration */
    __IO uint32_t DRAM0_POWER_ALIAS;    /* DRAM0 power configuration */
    __IO uint32_t DRAM1_POWER_ALIAS;    /* DRAM1 power configuration */
    __IO uint32_t DRAM2_POWER_ALIAS;    /* DRAM2 power configuration */
         uint32_t RESERVED0;
    __IO uint32_t BB_DRAM0_POWER_ALIAS; /* Baseband DRAM0 power configuration */
    __IO uint32_t BB_DRAM1_POWER_ALIAS; /* Baseband DRAM1 power configuration */
    __IO uint32_t DSP_PRAM0_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_PRAM1_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_PRAM2_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_PRAM3_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_DRAM0_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_DRAM1_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_DRAM2_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_DRAM3_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_DRAM4_POWER_ALIAS;/* DSP PRAM0 power configuration */
    __IO uint32_t DSP_DRAM5_POWER_ALIAS;/* DSP PRAM0 power configuration */
} SYSCTRL_MEM_POWER_CFG_Type;

#define SYSCTRL_MEM_POWER_CFG_BASE      SYS_CALC_BITBAND(0x40000018, 0)
#define SYSCTRL_MEM_POWER_CFG           ((SYSCTRL_MEM_POWER_CFG_Type *) SYSCTRL_MEM_POWER_CFG_BASE)

/* SYSCTRL_MEM_POWER_CFG settings */
#define PROM_POWER_DISABLE_BITBAND      0x0
#define PROM_POWER_ENABLE_BITBAND       0x1
#define PROM_POWER_DISABLE              ((uint32_t)(PROM_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PROM_POWER_Pos))
#define PROM_POWER_ENABLE               ((uint32_t)(PROM_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PROM_POWER_Pos))

#define FLASH_POWER_DISABLE_BITBAND     0x0
#define FLASH_POWER_ENABLE_BITBAND      0x1
#define FLASH_POWER_DISABLE             ((uint32_t)(FLASH_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_FLASH_POWER_Pos))
#define FLASH_POWER_ENABLE              ((uint32_t)(FLASH_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_FLASH_POWER_Pos))

#define PRAM0_POWER_DISABLE_BITBAND     0x0
#define PRAM0_POWER_ENABLE_BITBAND      0x1
#define PRAM0_POWER_DISABLE             ((uint32_t)(PRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM0_POWER_Pos))
#define PRAM0_POWER_ENABLE              ((uint32_t)(PRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM0_POWER_Pos))

#define PRAM1_POWER_DISABLE_BITBAND     0x0
#define PRAM1_POWER_ENABLE_BITBAND      0x1
#define PRAM1_POWER_DISABLE             ((uint32_t)(PRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM1_POWER_Pos))
#define PRAM1_POWER_ENABLE              ((uint32_t)(PRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM1_POWER_Pos))

#define PRAM2_POWER_DISABLE_BITBAND     0x0
#define PRAM2_POWER_ENABLE_BITBAND      0x1
#define PRAM2_POWER_DISABLE             ((uint32_t)(PRAM2_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM2_POWER_Pos))
#define PRAM2_POWER_ENABLE              ((uint32_t)(PRAM2_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM2_POWER_Pos))

#define PRAM3_POWER_DISABLE_BITBAND     0x0
#define PRAM3_POWER_ENABLE_BITBAND      0x1
#define PRAM3_POWER_DISABLE             ((uint32_t)(PRAM3_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM3_POWER_Pos))
#define PRAM3_POWER_ENABLE              ((uint32_t)(PRAM3_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM3_POWER_Pos))

#define DRAM0_POWER_DISABLE_BITBAND     0x0
#define DRAM0_POWER_ENABLE_BITBAND      0x1
#define DRAM0_POWER_DISABLE             ((uint32_t)(DRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM0_POWER_Pos))
#define DRAM0_POWER_ENABLE              ((uint32_t)(DRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM0_POWER_Pos))

#define DRAM1_POWER_DISABLE_BITBAND     0x0
#define DRAM1_POWER_ENABLE_BITBAND      0x1
#define DRAM1_POWER_DISABLE             ((uint32_t)(DRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM1_POWER_Pos))
#define DRAM1_POWER_ENABLE              ((uint32_t)(DRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM1_POWER_Pos))

#define DRAM2_POWER_DISABLE_BITBAND     0x0
#define DRAM2_POWER_ENABLE_BITBAND      0x1
#define DRAM2_POWER_DISABLE             ((uint32_t)(DRAM2_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM2_POWER_Pos))
#define DRAM2_POWER_ENABLE              ((uint32_t)(DRAM2_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM2_POWER_Pos))

#define BB_DRAM0_POWER_DISABLE_BITBAND  0x0
#define BB_DRAM0_POWER_ENABLE_BITBAND   0x1
#define BB_DRAM0_POWER_DISABLE          ((uint32_t)(BB_DRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_BB_DRAM0_POWER_Pos))
#define BB_DRAM0_POWER_ENABLE           ((uint32_t)(BB_DRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_BB_DRAM0_POWER_Pos))

#define BB_DRAM1_POWER_DISABLE_BITBAND  0x0
#define BB_DRAM1_POWER_ENABLE_BITBAND   0x1
#define BB_DRAM1_POWER_DISABLE          ((uint32_t)(BB_DRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_BB_DRAM1_POWER_Pos))
#define BB_DRAM1_POWER_ENABLE           ((uint32_t)(BB_DRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_BB_DRAM1_POWER_Pos))

#define DSP_PRAM0_POWER_DISABLE_BITBAND 0x0
#define DSP_PRAM0_POWER_ENABLE_BITBAND  0x1
#define DSP_PRAM0_POWER_DISABLE         ((uint32_t)(DSP_PRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM0_POWER_Pos))
#define DSP_PRAM0_POWER_ENABLE          ((uint32_t)(DSP_PRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM0_POWER_Pos))

#define DSP_PRAM1_POWER_DISABLE_BITBAND 0x0
#define DSP_PRAM1_POWER_ENABLE_BITBAND  0x1
#define DSP_PRAM1_POWER_DISABLE         ((uint32_t)(DSP_PRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM1_POWER_Pos))
#define DSP_PRAM1_POWER_ENABLE          ((uint32_t)(DSP_PRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM1_POWER_Pos))

#define DSP_PRAM2_POWER_DISABLE_BITBAND 0x0
#define DSP_PRAM2_POWER_ENABLE_BITBAND  0x1
#define DSP_PRAM2_POWER_DISABLE         ((uint32_t)(DSP_PRAM2_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM2_POWER_Pos))
#define DSP_PRAM2_POWER_ENABLE          ((uint32_t)(DSP_PRAM2_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM2_POWER_Pos))

#define DSP_PRAM3_POWER_DISABLE_BITBAND 0x0
#define DSP_PRAM3_POWER_ENABLE_BITBAND  0x1
#define DSP_PRAM3_POWER_DISABLE         ((uint32_t)(DSP_PRAM3_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM3_POWER_Pos))
#define DSP_PRAM3_POWER_ENABLE          ((uint32_t)(DSP_PRAM3_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM3_POWER_Pos))

#define DSP_DRAM0_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM0_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM0_POWER_DISABLE         ((uint32_t)(DSP_DRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM0_POWER_Pos))
#define DSP_DRAM0_POWER_ENABLE          ((uint32_t)(DSP_DRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM0_POWER_Pos))

#define DSP_DRAM1_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM1_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM1_POWER_DISABLE         ((uint32_t)(DSP_DRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM1_POWER_Pos))
#define DSP_DRAM1_POWER_ENABLE          ((uint32_t)(DSP_DRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM1_POWER_Pos))

#define DSP_DRAM2_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM2_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM2_POWER_DISABLE         ((uint32_t)(DSP_DRAM2_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM2_POWER_Pos))
#define DSP_DRAM2_POWER_ENABLE          ((uint32_t)(DSP_DRAM2_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM2_POWER_Pos))

#define DSP_DRAM3_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM3_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM3_POWER_DISABLE         ((uint32_t)(DSP_DRAM3_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM3_POWER_Pos))
#define DSP_DRAM3_POWER_ENABLE          ((uint32_t)(DSP_DRAM3_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM3_POWER_Pos))

#define DSP_DRAM4_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM4_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM4_POWER_DISABLE         ((uint32_t)(DSP_DRAM4_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM4_POWER_Pos))
#define DSP_DRAM4_POWER_ENABLE          ((uint32_t)(DSP_DRAM4_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM4_POWER_Pos))

#define DSP_DRAM5_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM5_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM5_POWER_DISABLE         ((uint32_t)(DSP_DRAM5_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM5_POWER_Pos))
#define DSP_DRAM5_POWER_ENABLE          ((uint32_t)(DSP_DRAM5_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM5_POWER_Pos))

/* SYSCTRL_MEM_ACCESS_CFG bit positions */
#define SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_Pos 24
#define SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_Mask ((uint32_t)(0x7FU << SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_Pos))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM5_ACCESS_Pos 21
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM4_ACCESS_Pos 20
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM3_ACCESS_Pos 19
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM2_ACCESS_Pos 18
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM1_ACCESS_Pos 17
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM0_ACCESS_Pos 16
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM3_ACCESS_Pos 15
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM2_ACCESS_Pos 14
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM1_ACCESS_Pos 13
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM0_ACCESS_Pos 12
#define SYSCTRL_MEM_ACCESS_CFG_BB_DRAM1_ACCESS_Pos 11
#define SYSCTRL_MEM_ACCESS_CFG_BB_DRAM0_ACCESS_Pos 10
#define SYSCTRL_MEM_ACCESS_CFG_DRAM2_ACCESS_Pos 8
#define SYSCTRL_MEM_ACCESS_CFG_DRAM1_ACCESS_Pos 7
#define SYSCTRL_MEM_ACCESS_CFG_DRAM0_ACCESS_Pos 6
#define SYSCTRL_MEM_ACCESS_CFG_PRAM3_ACCESS_Pos 5
#define SYSCTRL_MEM_ACCESS_CFG_PRAM2_ACCESS_Pos 4
#define SYSCTRL_MEM_ACCESS_CFG_PRAM1_ACCESS_Pos 3
#define SYSCTRL_MEM_ACCESS_CFG_PRAM0_ACCESS_Pos 2
#define SYSCTRL_MEM_ACCESS_CFG_FLASH_ACCESS_Pos 1
#define SYSCTRL_MEM_ACCESS_CFG_PROM_ACCESS_Pos 0

/* SYSCTRL_MEM_ACCESS_CFG sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t WAKEUP_ADDR_PACKED_BYTE;
         uint8_t RESERVED[SYS_CALC_BITBAND(0x4000001C, 0) - (0x4000001C + 4)];
    __IO uint32_t PROM_ACCESS_ALIAS;    /* PROM access configuration */
    __IO uint32_t FLASH_ACCESS_ALIAS;   /* Flash access configuration */
    __IO uint32_t PRAM0_ACCESS_ALIAS;   /* PRAM0 access configuration */
    __IO uint32_t PRAM1_ACCESS_ALIAS;   /* PRAM1 access configuration */
    __IO uint32_t PRAM2_ACCESS_ALIAS;   /* PRAM2 access configuration */
    __IO uint32_t PRAM3_ACCESS_ALIAS;   /* PRAM3 access configuration */
    __IO uint32_t DRAM0_ACCESS_ALIAS;   /* DRAM0 access configuration */
    __IO uint32_t DRAM1_ACCESS_ALIAS;   /* DRAM1 access configuration */
    __IO uint32_t DRAM2_ACCESS_ALIAS;   /* DRAM2 access configuration */
         uint32_t RESERVED1;
    __IO uint32_t BB_DRAM0_ACCESS_ALIAS;/* Baseband DRAM0 access configuration */
    __IO uint32_t BB_DRAM1_ACCESS_ALIAS;/* Baseband DRAM1 access configuration */
    __IO uint32_t DSP_PRAM0_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_PRAM1_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_PRAM2_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_PRAM3_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_DRAM0_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_DRAM1_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_DRAM2_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_DRAM3_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_DRAM4_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
    __IO uint32_t DSP_DRAM5_ACCESS_ALIAS;/* DSP PRAM0 access configuration */
} SYSCTRL_MEM_ACCESS_CFG_Type;

#define SYSCTRL_MEM_ACCESS_CFG_BASE     0x4000001C
#define SYSCTRL_MEM_ACCESS_CFG          ((SYSCTRL_MEM_ACCESS_CFG_Type *) SYSCTRL_MEM_ACCESS_CFG_BASE)

/* SYSCTRL_MEM_ACCESS_CFG settings */
#define PROM_ACCESS_DISABLE_BITBAND     0x0
#define PROM_ACCESS_ENABLE_BITBAND      0x1
#define PROM_ACCESS_DISABLE             ((uint32_t)(PROM_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PROM_ACCESS_Pos))
#define PROM_ACCESS_ENABLE              ((uint32_t)(PROM_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PROM_ACCESS_Pos))

#define FLASH_ACCESS_DISABLE_BITBAND    0x0
#define FLASH_ACCESS_ENABLE_BITBAND     0x1
#define FLASH_ACCESS_DISABLE            ((uint32_t)(FLASH_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_FLASH_ACCESS_Pos))
#define FLASH_ACCESS_ENABLE             ((uint32_t)(FLASH_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_FLASH_ACCESS_Pos))

#define PRAM0_ACCESS_DISABLE_BITBAND    0x0
#define PRAM0_ACCESS_ENABLE_BITBAND     0x1
#define PRAM0_ACCESS_DISABLE            ((uint32_t)(PRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM0_ACCESS_Pos))
#define PRAM0_ACCESS_ENABLE             ((uint32_t)(PRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM0_ACCESS_Pos))

#define PRAM1_ACCESS_DISABLE_BITBAND    0x0
#define PRAM1_ACCESS_ENABLE_BITBAND     0x1
#define PRAM1_ACCESS_DISABLE            ((uint32_t)(PRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM1_ACCESS_Pos))
#define PRAM1_ACCESS_ENABLE             ((uint32_t)(PRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM1_ACCESS_Pos))

#define PRAM2_ACCESS_DISABLE_BITBAND    0x0
#define PRAM2_ACCESS_ENABLE_BITBAND     0x1
#define PRAM2_ACCESS_DISABLE            ((uint32_t)(PRAM2_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM2_ACCESS_Pos))
#define PRAM2_ACCESS_ENABLE             ((uint32_t)(PRAM2_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM2_ACCESS_Pos))

#define PRAM3_ACCESS_DISABLE_BITBAND    0x0
#define PRAM3_ACCESS_ENABLE_BITBAND     0x1
#define PRAM3_ACCESS_DISABLE            ((uint32_t)(PRAM3_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM3_ACCESS_Pos))
#define PRAM3_ACCESS_ENABLE             ((uint32_t)(PRAM3_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM3_ACCESS_Pos))

#define DRAM0_ACCESS_DISABLE_BITBAND    0x0
#define DRAM0_ACCESS_ENABLE_BITBAND     0x1
#define DRAM0_ACCESS_DISABLE            ((uint32_t)(DRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM0_ACCESS_Pos))
#define DRAM0_ACCESS_ENABLE             ((uint32_t)(DRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM0_ACCESS_Pos))

#define DRAM1_ACCESS_DISABLE_BITBAND    0x0
#define DRAM1_ACCESS_ENABLE_BITBAND     0x1
#define DRAM1_ACCESS_DISABLE            ((uint32_t)(DRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM1_ACCESS_Pos))
#define DRAM1_ACCESS_ENABLE             ((uint32_t)(DRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM1_ACCESS_Pos))

#define DRAM2_ACCESS_DISABLE_BITBAND    0x0
#define DRAM2_ACCESS_ENABLE_BITBAND     0x1
#define DRAM2_ACCESS_DISABLE            ((uint32_t)(DRAM2_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM2_ACCESS_Pos))
#define DRAM2_ACCESS_ENABLE             ((uint32_t)(DRAM2_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM2_ACCESS_Pos))

#define BB_DRAM0_ACCESS_DISABLE_BITBAND 0x0
#define BB_DRAM0_ACCESS_ENABLE_BITBAND  0x1
#define BB_DRAM0_ACCESS_DISABLE         ((uint32_t)(BB_DRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_BB_DRAM0_ACCESS_Pos))
#define BB_DRAM0_ACCESS_ENABLE          ((uint32_t)(BB_DRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_BB_DRAM0_ACCESS_Pos))

#define BB_DRAM1_ACCESS_DISABLE_BITBAND 0x0
#define BB_DRAM1_ACCESS_ENABLE_BITBAND  0x1
#define BB_DRAM1_ACCESS_DISABLE         ((uint32_t)(BB_DRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_BB_DRAM1_ACCESS_Pos))
#define BB_DRAM1_ACCESS_ENABLE          ((uint32_t)(BB_DRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_BB_DRAM1_ACCESS_Pos))

#define DSP_PRAM0_ACCESS_DISABLE_BITBAND 0x0
#define DSP_PRAM0_ACCESS_ENABLE_BITBAND 0x1
#define DSP_PRAM0_ACCESS_DISABLE        ((uint32_t)(DSP_PRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM0_ACCESS_Pos))
#define DSP_PRAM0_ACCESS_ENABLE         ((uint32_t)(DSP_PRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM0_ACCESS_Pos))

#define DSP_PRAM1_ACCESS_DISABLE_BITBAND 0x0
#define DSP_PRAM1_ACCESS_ENABLE_BITBAND 0x1
#define DSP_PRAM1_ACCESS_DISABLE        ((uint32_t)(DSP_PRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM1_ACCESS_Pos))
#define DSP_PRAM1_ACCESS_ENABLE         ((uint32_t)(DSP_PRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM1_ACCESS_Pos))

#define DSP_PRAM2_ACCESS_DISABLE_BITBAND 0x0
#define DSP_PRAM2_ACCESS_ENABLE_BITBAND 0x1
#define DSP_PRAM2_ACCESS_DISABLE        ((uint32_t)(DSP_PRAM2_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM2_ACCESS_Pos))
#define DSP_PRAM2_ACCESS_ENABLE         ((uint32_t)(DSP_PRAM2_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM2_ACCESS_Pos))

#define DSP_PRAM3_ACCESS_DISABLE_BITBAND 0x0
#define DSP_PRAM3_ACCESS_ENABLE_BITBAND 0x1
#define DSP_PRAM3_ACCESS_DISABLE        ((uint32_t)(DSP_PRAM3_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM3_ACCESS_Pos))
#define DSP_PRAM3_ACCESS_ENABLE         ((uint32_t)(DSP_PRAM3_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM3_ACCESS_Pos))

#define DSP_DRAM0_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM0_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM0_ACCESS_DISABLE        ((uint32_t)(DSP_DRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM0_ACCESS_Pos))
#define DSP_DRAM0_ACCESS_ENABLE         ((uint32_t)(DSP_DRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM0_ACCESS_Pos))

#define DSP_DRAM1_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM1_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM1_ACCESS_DISABLE        ((uint32_t)(DSP_DRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM1_ACCESS_Pos))
#define DSP_DRAM1_ACCESS_ENABLE         ((uint32_t)(DSP_DRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM1_ACCESS_Pos))

#define DSP_DRAM2_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM2_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM2_ACCESS_DISABLE        ((uint32_t)(DSP_DRAM2_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM2_ACCESS_Pos))
#define DSP_DRAM2_ACCESS_ENABLE         ((uint32_t)(DSP_DRAM2_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM2_ACCESS_Pos))

#define DSP_DRAM3_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM3_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM3_ACCESS_DISABLE        ((uint32_t)(DSP_DRAM3_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM3_ACCESS_Pos))
#define DSP_DRAM3_ACCESS_ENABLE         ((uint32_t)(DSP_DRAM3_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM3_ACCESS_Pos))

#define DSP_DRAM4_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM4_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM4_ACCESS_DISABLE        ((uint32_t)(DSP_DRAM4_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM4_ACCESS_Pos))
#define DSP_DRAM4_ACCESS_ENABLE         ((uint32_t)(DSP_DRAM4_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM4_ACCESS_Pos))

#define DSP_DRAM5_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM5_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM5_ACCESS_DISABLE        ((uint32_t)(DSP_DRAM5_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM5_ACCESS_Pos))
#define DSP_DRAM5_ACCESS_ENABLE         ((uint32_t)(DSP_DRAM5_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM5_ACCESS_Pos))

/* SYSCTRL_MEM_ACCESS_CFG sub-register bit positions */
#define SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_BYTE_Pos 0
#define SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_BYTE_Mask ((uint32_t)(0x7FU << SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_BYTE_Pos))

/* SYSCTRL_WAKEUP_ADDR bit positions */
#define SYSCTRL_WAKEUP_ADDR_WAKEUP_ADDR_Pos 0
#define SYSCTRL_WAKEUP_ADDR_WAKEUP_ADDR_Mask ((uint32_t)(0xFFFFFFFFU << SYSCTRL_WAKEUP_ADDR_WAKEUP_ADDR_Pos))

/* SYSCTRL_MEM_RETENTION_CFG bit positions */
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM5_RETENTION_Pos 21
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM4_RETENTION_Pos 20
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM3_RETENTION_Pos 19
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM2_RETENTION_Pos 18
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM1_RETENTION_Pos 17
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM0_RETENTION_Pos 16
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM3_RETENTION_Pos 15
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM2_RETENTION_Pos 14
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM1_RETENTION_Pos 13
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM0_RETENTION_Pos 12
#define SYSCTRL_MEM_RETENTION_CFG_BB_DRAM1_RETENTION_Pos 11
#define SYSCTRL_MEM_RETENTION_CFG_BB_DRAM0_RETENTION_Pos 10
#define SYSCTRL_MEM_RETENTION_CFG_DRAM2_RETENTION_Pos 8
#define SYSCTRL_MEM_RETENTION_CFG_DRAM1_RETENTION_Pos 7
#define SYSCTRL_MEM_RETENTION_CFG_DRAM0_RETENTION_Pos 6
#define SYSCTRL_MEM_RETENTION_CFG_PRAM3_RETENTION_Pos 5
#define SYSCTRL_MEM_RETENTION_CFG_PRAM2_RETENTION_Pos 4
#define SYSCTRL_MEM_RETENTION_CFG_PRAM1_RETENTION_Pos 3
#define SYSCTRL_MEM_RETENTION_CFG_PRAM0_RETENTION_Pos 2

/* SYSCTRL_MEM_RETENTION_CFG bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[2];
    __IO uint32_t PRAM0_RETENTION_ALIAS;/* PRAM0 retention configuration */
    __IO uint32_t PRAM1_RETENTION_ALIAS;/* PRAM1 retention configuration */
    __IO uint32_t PRAM2_RETENTION_ALIAS;/* PRAM2 retention configuration */
    __IO uint32_t PRAM3_RETENTION_ALIAS;/* PRAM3 retention configuration */
    __IO uint32_t DRAM0_RETENTION_ALIAS;/* DRAM0 retention configuration */
    __IO uint32_t DRAM1_RETENTION_ALIAS;/* DRAM1 retention configuration */
    __IO uint32_t DRAM2_RETENTION_ALIAS;/* DRAM2 retention configuration */
         uint32_t RESERVED1;
    __IO uint32_t BB_DRAM0_RETENTION_ALIAS;/* Baseband DRAM0 retention configuration */
    __IO uint32_t BB_DRAM1_RETENTION_ALIAS;/* Baseband DRAM1 retention configuration */
    __IO uint32_t DSP_PRAM0_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_PRAM1_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_PRAM2_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_PRAM3_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_DRAM0_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_DRAM1_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_DRAM2_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_DRAM3_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_DRAM4_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
    __IO uint32_t DSP_DRAM5_RETENTION_ALIAS;/* DSP PRAM0 retention configuration */
} SYSCTRL_MEM_RETENTION_CFG_Type;

#define SYSCTRL_MEM_RETENTION_CFG_BASE  SYS_CALC_BITBAND(0x40000024, 0)
#define SYSCTRL_MEM_RETENTION_CFG       ((SYSCTRL_MEM_RETENTION_CFG_Type *) SYSCTRL_MEM_RETENTION_CFG_BASE)

/* SYSCTRL_MEM_RETENTION_CFG settings */
#define PRAM0_NORMAL_MODE_BITBAND       0x0
#define PRAM0_RETENTION_MODE_BITBAND    0x1
#define PRAM0_NORMAL_MODE               ((uint32_t)(PRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM0_RETENTION_Pos))
#define PRAM0_RETENTION_MODE            ((uint32_t)(PRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM0_RETENTION_Pos))

#define PRAM1_NORMAL_MODE_BITBAND       0x0
#define PRAM1_RETENTION_MODE_BITBAND    0x1
#define PRAM1_NORMAL_MODE               ((uint32_t)(PRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM1_RETENTION_Pos))
#define PRAM1_RETENTION_MODE            ((uint32_t)(PRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM1_RETENTION_Pos))

#define PRAM2_NORMAL_MODE_BITBAND       0x0
#define PRAM2_RETENTION_MODE_BITBAND    0x1
#define PRAM2_NORMAL_MODE               ((uint32_t)(PRAM2_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM2_RETENTION_Pos))
#define PRAM2_RETENTION_MODE            ((uint32_t)(PRAM2_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM2_RETENTION_Pos))

#define PRAM3_NORMAL_MODE_BITBAND       0x0
#define PRAM3_RETENTION_MODE_BITBAND    0x1
#define PRAM3_NORMAL_MODE               ((uint32_t)(PRAM3_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM3_RETENTION_Pos))
#define PRAM3_RETENTION_MODE            ((uint32_t)(PRAM3_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM3_RETENTION_Pos))

#define DRAM0_NORMAL_MODE_BITBAND       0x0
#define DRAM0_RETENTION_MODE_BITBAND    0x1
#define DRAM0_NORMAL_MODE               ((uint32_t)(DRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM0_RETENTION_Pos))
#define DRAM0_RETENTION_MODE            ((uint32_t)(DRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM0_RETENTION_Pos))

#define DRAM1_NORMAL_MODE_BITBAND       0x0
#define DRAM1_RETENTION_MODE_BITBAND    0x1
#define DRAM1_NORMAL_MODE               ((uint32_t)(DRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM1_RETENTION_Pos))
#define DRAM1_RETENTION_MODE            ((uint32_t)(DRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM1_RETENTION_Pos))

#define DRAM2_NORMAL_MODE_BITBAND       0x0
#define DRAM2_RETENTION_MODE_BITBAND    0x1
#define DRAM2_NORMAL_MODE               ((uint32_t)(DRAM2_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM2_RETENTION_Pos))
#define DRAM2_RETENTION_MODE            ((uint32_t)(DRAM2_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM2_RETENTION_Pos))

#define BB_DRAM0_NORMAL_MODE_BITBAND    0x0
#define BB_DRAM0_RETENTION_MODE_BITBAND 0x1
#define BB_DRAM0_NORMAL_MODE            ((uint32_t)(BB_DRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_BB_DRAM0_RETENTION_Pos))
#define BB_DRAM0_RETENTION_MODE         ((uint32_t)(BB_DRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_BB_DRAM0_RETENTION_Pos))

#define BB_DRAM1_NORMAL_MODE_BITBAND    0x0
#define BB_DRAM1_RETENTION_MODE_BITBAND 0x1
#define BB_DRAM1_NORMAL_MODE            ((uint32_t)(BB_DRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_BB_DRAM1_RETENTION_Pos))
#define BB_DRAM1_RETENTION_MODE         ((uint32_t)(BB_DRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_BB_DRAM1_RETENTION_Pos))

#define DSP_PRAM0_NORMAL_MODE_BITBAND   0x0
#define DSP_PRAM0_RETENTION_MODE_BITBAND 0x1
#define DSP_PRAM0_NORMAL_MODE           ((uint32_t)(DSP_PRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM0_RETENTION_Pos))
#define DSP_PRAM0_RETENTION_MODE        ((uint32_t)(DSP_PRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM0_RETENTION_Pos))

#define DSP_PRAM1_NORMAL_MODE_BITBAND   0x0
#define DSP_PRAM1_RETENTION_MODE_BITBAND 0x1
#define DSP_PRAM1_NORMAL_MODE           ((uint32_t)(DSP_PRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM1_RETENTION_Pos))
#define DSP_PRAM1_RETENTION_MODE        ((uint32_t)(DSP_PRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM1_RETENTION_Pos))

#define DSP_PRAM2_NORMAL_MODE_BITBAND   0x0
#define DSP_PRAM2_RETENTION_MODE_BITBAND 0x1
#define DSP_PRAM2_NORMAL_MODE           ((uint32_t)(DSP_PRAM2_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM2_RETENTION_Pos))
#define DSP_PRAM2_RETENTION_MODE        ((uint32_t)(DSP_PRAM2_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM2_RETENTION_Pos))

#define DSP_PRAM3_NORMAL_MODE_BITBAND   0x0
#define DSP_PRAM3_RETENTION_MODE_BITBAND 0x1
#define DSP_PRAM3_NORMAL_MODE           ((uint32_t)(DSP_PRAM3_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM3_RETENTION_Pos))
#define DSP_PRAM3_RETENTION_MODE        ((uint32_t)(DSP_PRAM3_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM3_RETENTION_Pos))

#define DSP_DRAM0_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM0_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM0_NORMAL_MODE           ((uint32_t)(DSP_DRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM0_RETENTION_Pos))
#define DSP_DRAM0_RETENTION_MODE        ((uint32_t)(DSP_DRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM0_RETENTION_Pos))

#define DSP_DRAM1_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM1_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM1_NORMAL_MODE           ((uint32_t)(DSP_DRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM1_RETENTION_Pos))
#define DSP_DRAM1_RETENTION_MODE        ((uint32_t)(DSP_DRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM1_RETENTION_Pos))

#define DSP_DRAM2_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM2_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM2_NORMAL_MODE           ((uint32_t)(DSP_DRAM2_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM2_RETENTION_Pos))
#define DSP_DRAM2_RETENTION_MODE        ((uint32_t)(DSP_DRAM2_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM2_RETENTION_Pos))

#define DSP_DRAM3_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM3_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM3_NORMAL_MODE           ((uint32_t)(DSP_DRAM3_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM3_RETENTION_Pos))
#define DSP_DRAM3_RETENTION_MODE        ((uint32_t)(DSP_DRAM3_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM3_RETENTION_Pos))

#define DSP_DRAM4_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM4_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM4_NORMAL_MODE           ((uint32_t)(DSP_DRAM4_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM4_RETENTION_Pos))
#define DSP_DRAM4_RETENTION_MODE        ((uint32_t)(DSP_DRAM4_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM4_RETENTION_Pos))

#define DSP_DRAM5_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM5_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM5_NORMAL_MODE           ((uint32_t)(DSP_DRAM5_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM5_RETENTION_Pos))
#define DSP_DRAM5_RETENTION_MODE        ((uint32_t)(DSP_DRAM5_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM5_RETENTION_Pos))

/* SYSCTRL_MEM_ARBITER_CFG bit positions */
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_Pos 28
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_Pos 26
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_Pos 24
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_Pos 22
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_Pos 20
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_Pos 18
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_Pos 16
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_Pos 10
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_Pos 8
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_Pos 4
#define SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DRAM0_ARBITER_Pos 2
#define SYSCTRL_MEM_ARBITER_CFG_PRAM_ARBITER_Pos 1
#define SYSCTRL_MEM_ARBITER_CFG_ROUND_ROBIN_TOKEN_Pos 0

/* SYSCTRL_MEM_ARBITER_CFG sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t BB_BYTE;              
    __IO uint16_t DSP_SHORT;            
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000028, 0) - (0x40000028 + 4)];
    __IO uint32_t ROUND_ROBIN_TOKEN_ALIAS;/* Round-robin token generation configuration */
    __IO uint32_t PRAM_ARBITER_ALIAS;   /* PRAM0 to PRAM3 arbiter configuration */
    __IO uint32_t DRAM0_ARBITER_ALIAS;  /* DRAM0 arbiter configuration */
} SYSCTRL_MEM_ARBITER_CFG_Type;

#define SYSCTRL_MEM_ARBITER_CFG_BASE    0x40000028
#define SYSCTRL_MEM_ARBITER_CFG         ((SYSCTRL_MEM_ARBITER_CFG_Type *) SYSCTRL_MEM_ARBITER_CFG_BASE)

/* SYSCTRL_MEM_ARBITER_CFG settings */
#define REALTIME_DMA_MODE_BITBAND       0x0
#define ROUND_ROBIN_MODE_BITBAND        0x1
#define REALTIME_DMA_MODE               ((uint32_t)(REALTIME_DMA_MODE_BITBAND << SYSCTRL_MEM_ARBITER_CFG_ROUND_ROBIN_TOKEN_Pos))
#define ROUND_ROBIN_MODE                ((uint32_t)(ROUND_ROBIN_MODE_BITBAND << SYSCTRL_MEM_ARBITER_CFG_ROUND_ROBIN_TOKEN_Pos))

#define PRAM_CM3_PRIORITY_BITBAND       0x0
#define PRAM_ROUND_ROBIN_PRIORITY_BITBAND 0x1
#define PRAM_CM3_PRIORITY               ((uint32_t)(PRAM_CM3_PRIORITY_BITBAND << SYSCTRL_MEM_ARBITER_CFG_PRAM_ARBITER_Pos))
#define PRAM_ROUND_ROBIN_PRIORITY       ((uint32_t)(PRAM_ROUND_ROBIN_PRIORITY_BITBAND << SYSCTRL_MEM_ARBITER_CFG_PRAM_ARBITER_Pos))

#define DRAM0_CM3_PRIORITY_BITBAND      0x0
#define DRAM0_ROUND_ROBIN_PRIORITY_BITBAND 0x1
#define DRAM0_CM3_PRIORITY              ((uint32_t)(DRAM0_CM3_PRIORITY_BITBAND << SYSCTRL_MEM_ARBITER_CFG_DRAM0_ARBITER_Pos))
#define DRAM0_ROUND_ROBIN_PRIORITY      ((uint32_t)(DRAM0_ROUND_ROBIN_PRIORITY_BITBAND << SYSCTRL_MEM_ARBITER_CFG_DRAM0_ARBITER_Pos))

#define DRAM12_CM3_PRIORITY             ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_Pos))
#define DRAM12_DSP_PRIORITY             ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_Pos))
#define DRAM12_ROUND_ROBIN_PRIORITY     ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_Pos))

#define BB_DRAM0_BB_PRIORITY            ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_Pos))
#define BB_DRAM0_CM3_PRIORITY           ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_Pos))
#define BB_DRAM0_ROUND_ROBIN_PRIORITY   ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_Pos))
#define BB_DRAM0_SMART_PRIORITY         ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_Pos))

#define BB_DRAM1_BB_PRIORITY            ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_Pos))
#define BB_DRAM1_CM3_PRIORITY           ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_Pos))
#define BB_DRAM1_ROUND_ROBIN_PRIORITY   ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_Pos))
#define BB_DRAM1_SMART_PRIORITY         ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_Pos))

#define DSP_PRAM0_DSP_PRIORITY          ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_Pos))
#define DSP_PRAM0_CM3_PRIORITY          ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_Pos))
#define DSP_PRAM0_ROUND_ROBIN_PRIORITY  ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_Pos))

#define DSP_PRAM1_DSP_PRIORITY          ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_Pos))
#define DSP_PRAM1_CM3_PRIORITY          ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_Pos))
#define DSP_PRAM1_ROUND_ROBIN_PRIORITY  ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_Pos))

#define DSP_PRAM2_DSP_PRIORITY          ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_Pos))
#define DSP_PRAM2_CM3_PRIORITY          ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_Pos))
#define DSP_PRAM2_ROUND_ROBIN_PRIORITY  ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_Pos))

#define DSP_PRAM3_DSP_PRIORITY          ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_Pos))
#define DSP_PRAM3_CM3_PRIORITY          ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_Pos))
#define DSP_PRAM3_ROUND_ROBIN_PRIORITY  ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_Pos))

#define DSP_DRAM01_DSP_PRIORITY         ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_Pos))
#define DSP_DRAM01_CM3_PRIORITY         ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_Pos))
#define DSP_DRAM01_ROUND_ROBIN_PRIORITY ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_Pos))

#define DSP_DRAM23_DSP_PRIORITY         ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_Pos))
#define DSP_DRAM23_CM3_PRIORITY         ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_Pos))
#define DSP_DRAM23_ROUND_ROBIN_PRIORITY ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_Pos))

#define DSP_DRAM45_DSP_PRIORITY         ((uint32_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_Pos))
#define DSP_DRAM45_CM3_PRIORITY         ((uint32_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_Pos))
#define DSP_DRAM45_ROUND_ROBIN_PRIORITY ((uint32_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_Pos))

/* SYSCTRL_MEM_ARBITER_CFG sub-register bit positions */
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_BYTE_Pos 0
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_BYTE_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_BYTE_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_BYTE_Pos 2
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_BYTE_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_BYTE_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_SHORT_Pos 0
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_SHORT_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_SHORT_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_SHORT_Pos 2
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_SHORT_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_SHORT_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_SHORT_Pos 4
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_SHORT_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_SHORT_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_SHORT_Pos 6
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_SHORT_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_SHORT_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_SHORT_Pos 8
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_SHORT_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_SHORT_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_SHORT_Pos 10
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_SHORT_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_SHORT_Pos))
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_SHORT_Pos 12
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_SHORT_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_SHORT_Pos))

/* SYSCTRL_MEM_ARBITER_CFG sub-register settings */
#define BB_DRAM0_BB_PRIORITY_BYTE       ((uint8_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_BYTE_Pos))
#define BB_DRAM0_CM3_PRIORITY_BYTE      ((uint8_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_BYTE_Pos))
#define BB_DRAM0_ROUND_ROBIN_PRIORITY_BYTE ((uint8_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_BYTE_Pos))
#define BB_DRAM0_SMART_PRIORITY_BYTE    ((uint8_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_BYTE_Pos))

#define BB_DRAM1_BB_PRIORITY_BYTE       ((uint8_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_BYTE_Pos))
#define BB_DRAM1_CM3_PRIORITY_BYTE      ((uint8_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_BYTE_Pos))
#define BB_DRAM1_ROUND_ROBIN_PRIORITY_BYTE ((uint8_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_BYTE_Pos))
#define BB_DRAM1_SMART_PRIORITY_BYTE    ((uint8_t)(0x3U << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_BYTE_Pos))

#define DSP_PRAM0_DSP_PRIORITY_SHORT    ((uint16_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_SHORT_Pos))
#define DSP_PRAM0_CM3_PRIORITY_SHORT    ((uint16_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_SHORT_Pos))
#define DSP_PRAM0_ROUND_ROBIN_PRIORITY_SHORT ((uint16_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_SHORT_Pos))

#define DSP_PRAM1_DSP_PRIORITY_SHORT    ((uint16_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_SHORT_Pos))
#define DSP_PRAM1_CM3_PRIORITY_SHORT    ((uint16_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_SHORT_Pos))
#define DSP_PRAM1_ROUND_ROBIN_PRIORITY_SHORT ((uint16_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_SHORT_Pos))

#define DSP_PRAM2_DSP_PRIORITY_SHORT    ((uint16_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_SHORT_Pos))
#define DSP_PRAM2_CM3_PRIORITY_SHORT    ((uint16_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_SHORT_Pos))
#define DSP_PRAM2_ROUND_ROBIN_PRIORITY_SHORT ((uint16_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_SHORT_Pos))

#define DSP_PRAM3_DSP_PRIORITY_SHORT    ((uint16_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_SHORT_Pos))
#define DSP_PRAM3_CM3_PRIORITY_SHORT    ((uint16_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_SHORT_Pos))
#define DSP_PRAM3_ROUND_ROBIN_PRIORITY_SHORT ((uint16_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_SHORT_Pos))

#define DSP_DRAM01_DSP_PRIORITY_SHORT   ((uint16_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_SHORT_Pos))
#define DSP_DRAM01_CM3_PRIORITY_SHORT   ((uint16_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_SHORT_Pos))
#define DSP_DRAM01_ROUND_ROBIN_PRIORITY_SHORT ((uint16_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_SHORT_Pos))

#define DSP_DRAM23_DSP_PRIORITY_SHORT   ((uint16_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_SHORT_Pos))
#define DSP_DRAM23_CM3_PRIORITY_SHORT   ((uint16_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_SHORT_Pos))
#define DSP_DRAM23_ROUND_ROBIN_PRIORITY_SHORT ((uint16_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_SHORT_Pos))

#define DSP_DRAM45_DSP_PRIORITY_SHORT   ((uint16_t)(0x0U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_SHORT_Pos))
#define DSP_DRAM45_CM3_PRIORITY_SHORT   ((uint16_t)(0x1U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_SHORT_Pos))
#define DSP_DRAM45_ROUND_ROBIN_PRIORITY_SHORT ((uint16_t)(0x2U << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_SHORT_Pos))

/* SYSCTRL_MEM_TIMING_CFG bit positions */
#define SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_Pos 8
#define SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_Mask ((uint32_t)(0x3U << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_Pos))
#define SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_Pos 4
#define SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_Mask ((uint32_t)(0x7U << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_Pos))
#define SYSCTRL_MEM_TIMING_CFG_PROM_KEN_Pos 3
#define SYSCTRL_MEM_TIMING_CFG_PROM_EMA_Pos 0
#define SYSCTRL_MEM_TIMING_CFG_PROM_EMA_Mask ((uint32_t)(0x7U << SYSCTRL_MEM_TIMING_CFG_PROM_EMA_Pos))

/* SYSCTRL_MEM_TIMING_CFG bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[3];
    __IO uint32_t PROM_KEN_ALIAS;       /* PROM bitlines keeper configuration */
} SYSCTRL_MEM_TIMING_CFG_Type;

#define SYSCTRL_MEM_TIMING_CFG_BASE     SYS_CALC_BITBAND(0x4000002C, 0)
#define SYSCTRL_MEM_TIMING_CFG          ((SYSCTRL_MEM_TIMING_CFG_Type *) SYSCTRL_MEM_TIMING_CFG_BASE)

/* SYSCTRL_MEM_TIMING_CFG settings */
#define PROM_EMA_MIN                    ((uint32_t)(0x0U << SYSCTRL_MEM_TIMING_CFG_PROM_EMA_Pos))
#define PROM_EMA_DEFAULT                ((uint32_t)(0x5U << SYSCTRL_MEM_TIMING_CFG_PROM_EMA_Pos))
#define PROM_EMA_MAX                    ((uint32_t)(0x7U << SYSCTRL_MEM_TIMING_CFG_PROM_EMA_Pos))

#define PROM_KEN_ENABLED_BITBAND        0x0
#define PROM_KEN_DISABLED_BITBAND       0x1
#define PROM_KEN_ENABLED                ((uint32_t)(PROM_KEN_ENABLED_BITBAND << SYSCTRL_MEM_TIMING_CFG_PROM_KEN_Pos))
#define PROM_KEN_DISABLED               ((uint32_t)(PROM_KEN_DISABLED_BITBAND << SYSCTRL_MEM_TIMING_CFG_PROM_KEN_Pos))

#define DSP_PRAM_EMA_MIN                ((uint32_t)(0x0U << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_Pos))
#define DSP_PRAM_EMA_DEFAULT            ((uint32_t)(0x2U << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_Pos))
#define DSP_PRAM_EMA_MAX                ((uint32_t)(0x7U << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_Pos))

#define DSP_PRAM_EMAW_DEFAULT           ((uint32_t)(0x0U << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_Pos))
#define DSP_PRAM_EMAW_MAX               ((uint32_t)(0x3U << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_Pos))

/* SYSCTRL_CNT_CTRL bit positions */
#define SYSCTRL_CNT_CTRL_CNT_STATUS_Pos 3
#define SYSCTRL_CNT_CTRL_CNT_CLEAR_Pos  2
#define SYSCTRL_CNT_CTRL_CNT_STOP_Pos   1
#define SYSCTRL_CNT_CTRL_CNT_START_Pos  0

/* SYSCTRL_CNT_CTRL bit-band aliases */
typedef struct
{
    __O  uint32_t CNT_START_ALIAS;      /* Start activity counters */
    __O  uint32_t CNT_STOP_ALIAS;       /* Stop activity counters */
    __O  uint32_t CNT_CLEAR_ALIAS;      /* Clear activity counters */
    __I  uint32_t CNT_STATUS_ALIAS;     /* Activity counters status bit */
} SYSCTRL_CNT_CTRL_Type;

#define SYSCTRL_CNT_CTRL_BASE           SYS_CALC_BITBAND(0x40000030, 0)
#define SYSCTRL_CNT_CTRL                ((SYSCTRL_CNT_CTRL_Type *) SYSCTRL_CNT_CTRL_BASE)

/* SYSCTRL_CNT_CTRL settings */
#define CNT_START_BITBAND               0x1
#define CNT_START                       ((uint32_t)(CNT_START_BITBAND << SYSCTRL_CNT_CTRL_CNT_START_Pos))

#define CNT_STOP_BITBAND                0x1
#define CNT_STOP                        ((uint32_t)(CNT_STOP_BITBAND << SYSCTRL_CNT_CTRL_CNT_STOP_Pos))

#define CNT_CLEAR_BITBAND               0x1
#define CNT_CLEAR                       ((uint32_t)(CNT_CLEAR_BITBAND << SYSCTRL_CNT_CTRL_CNT_CLEAR_Pos))

#define CNT_STOPPED_BITBAND             0x0
#define CNT_RUNNING_BITBAND             0x1
#define CNT_STOPPED                     ((uint32_t)(CNT_STOPPED_BITBAND << SYSCTRL_CNT_CTRL_CNT_STATUS_Pos))
#define CNT_RUNNING                     ((uint32_t)(CNT_RUNNING_BITBAND << SYSCTRL_CNT_CTRL_CNT_STATUS_Pos))

/* SYSCTRL_SYSCLK_CNT bit positions */
#define SYSCTRL_SYSCLK_CNT_SYSCLK_CNT_Pos 0
#define SYSCTRL_SYSCLK_CNT_SYSCLK_CNT_Mask ((uint32_t)(0xFFFFFFFFU << SYSCTRL_SYSCLK_CNT_SYSCLK_CNT_Pos))

/* SYSCTRL_CM3_CNT bit positions */
#define SYSCTRL_CM3_CNT_CM3_CNT_Pos     0
#define SYSCTRL_CM3_CNT_CM3_CNT_Mask    ((uint32_t)(0xFFFFFFFFU << SYSCTRL_CM3_CNT_CM3_CNT_Pos))

/* SYSCTRL_LPDSP32_CNT bit positions */
#define SYSCTRL_LPDSP32_CNT_LPDSP32_CNT_Pos 0
#define SYSCTRL_LPDSP32_CNT_LPDSP32_CNT_Mask ((uint32_t)(0xFFFFFFFFU << SYSCTRL_LPDSP32_CNT_LPDSP32_CNT_Pos))

/* SYSCTRL_FLASH_READ_CNT bit positions */
#define SYSCTRL_FLASH_READ_CNT_FLASH_READ_CNT_Pos 0
#define SYSCTRL_FLASH_READ_CNT_FLASH_READ_CNT_Mask ((uint32_t)(0xFFFFFFFFU << SYSCTRL_FLASH_READ_CNT_FLASH_READ_CNT_Pos))

/* SYSCTRL_SPEED_MEASURE bit positions */
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_START_Pos 4
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_STATUS_Pos 3
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_RESULT_Pos 0
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_RESULT_Mask ((uint32_t)(0x7U << SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_RESULT_Pos))

/* SYSCTRL_SPEED_MEASURE bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[3];
    __I  uint32_t SPEED_MEASURE_STATUS_ALIAS;/* Critical path speed measurement status */
    __O  uint32_t SPEED_MEASURE_START_ALIAS;/* Start critical path speed measurement */
} SYSCTRL_SPEED_MEASURE_Type;

#define SYSCTRL_SPEED_MEASURE_BASE      SYS_CALC_BITBAND(0x40000048, 0)
#define SYSCTRL_SPEED_MEASURE           ((SYSCTRL_SPEED_MEASURE_Type *) SYSCTRL_SPEED_MEASURE_BASE)

/* SYSCTRL_SPEED_MEASURE settings */
#define SPEED_MEASURE_IDLE_BITBAND      0x0
#define SPEED_MEASURE_BUSY_BITBAND      0x1
#define SPEED_MEASURE_IDLE              ((uint32_t)(SPEED_MEASURE_IDLE_BITBAND << SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_STATUS_Pos))
#define SPEED_MEASURE_BUSY              ((uint32_t)(SPEED_MEASURE_BUSY_BITBAND << SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_STATUS_Pos))

#define SPEED_MEASURE_START_BITBAND     0x1
#define SPEED_MEASURE_START             ((uint32_t)(SPEED_MEASURE_START_BITBAND << SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_START_Pos))

/* SYSCTRL_LPDSP32_DEBUG_CFG bit positions */
#define SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_Pos 1
#define SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_DEBUG_ENABLE_Pos 0

/* SYSCTRL_LPDSP32_DEBUG_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t LPDSP32_DEBUG_ENABLE_ALIAS;/* LPDSP32 debug port enable */
    __IO uint32_t LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_ALIAS;/* LPDSP32 exit powerdown mode configuration when halted */
} SYSCTRL_LPDSP32_DEBUG_CFG_Type;

#define SYSCTRL_LPDSP32_DEBUG_CFG_BASE  SYS_CALC_BITBAND(0x4000004C, 0)
#define SYSCTRL_LPDSP32_DEBUG_CFG       ((SYSCTRL_LPDSP32_DEBUG_CFG_Type *) SYSCTRL_LPDSP32_DEBUG_CFG_BASE)

/* SYSCTRL_LPDSP32_DEBUG_CFG settings */
#define LPDSP32_DEBUG_DISABLED_BITBAND  0x0
#define LPDSP32_DEBUG_ENABLED_BITBAND   0x1
#define LPDSP32_DEBUG_DISABLED          ((uint32_t)(LPDSP32_DEBUG_DISABLED_BITBAND << SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_DEBUG_ENABLE_Pos))
#define LPDSP32_DEBUG_ENABLED           ((uint32_t)(LPDSP32_DEBUG_ENABLED_BITBAND << SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_DEBUG_ENABLE_Pos))

#define LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_DISABLED_BITBAND 0x0
#define LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_ENABLED_BITBAND 0x1
#define LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_DISABLED ((uint32_t)(LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_DISABLED_BITBAND << SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_Pos))
#define LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_ENABLED ((uint32_t)(LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_ENABLED_BITBAND << SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_Pos))

/* SYSCTRL_RF_POWER_CFG bit positions */
#define SYSCTRL_RF_POWER_CFG_RF_POWER_Pos 0

/* SYSCTRL_RF_POWER_CFG sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t RF_POWER_BYTE;        
         uint8_t RESERVED0[3];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000050, 0) - (0x40000050 + 4)];
    __IO uint32_t RF_POWER_ALIAS;       /* RF power configuration */
} SYSCTRL_RF_POWER_CFG_Type;

#define SYSCTRL_RF_POWER_CFG_BASE       0x40000050
#define SYSCTRL_RF_POWER_CFG            ((SYSCTRL_RF_POWER_CFG_Type *) SYSCTRL_RF_POWER_CFG_BASE)

/* SYSCTRL_RF_POWER_CFG settings */
#define RF_POWER_DISABLE_BITBAND        0x0
#define RF_POWER_ENABLE_BITBAND         0x1
#define RF_POWER_DISABLE                ((uint32_t)(RF_POWER_DISABLE_BITBAND << SYSCTRL_RF_POWER_CFG_RF_POWER_Pos))
#define RF_POWER_ENABLE                 ((uint32_t)(RF_POWER_ENABLE_BITBAND << SYSCTRL_RF_POWER_CFG_RF_POWER_Pos))

/* SYSCTRL_RF_POWER_CFG sub-register bit positions */
#define SYSCTRL_RF_POWER_CFG_RF_POWER_BYTE_Pos 0

/* SYSCTRL_RF_POWER_CFG sub-register settings */
#define RF_POWER_DISABLE_BYTE           ((uint8_t)(RF_POWER_DISABLE_BITBAND << SYSCTRL_RF_POWER_CFG_RF_POWER_BYTE_Pos))
#define RF_POWER_ENABLE_BYTE            ((uint8_t)(RF_POWER_ENABLE_BITBAND << SYSCTRL_RF_POWER_CFG_RF_POWER_BYTE_Pos))

/* SYSCTRL_RF_ACCESS_CFG bit positions */
#define SYSCTRL_RF_ACCESS_CFG_RF_IRQ_ACCESS_Pos 1
#define SYSCTRL_RF_ACCESS_CFG_RF_ACCESS_Pos 0

/* SYSCTRL_RF_ACCESS_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t RF_ACCESS_ALIAS;      /* RF access configuration */
    __IO uint32_t RF_IRQ_ACCESS_ALIAS;  /* RF IRQ access configuration */
} SYSCTRL_RF_ACCESS_CFG_Type;

#define SYSCTRL_RF_ACCESS_CFG_BASE      SYS_CALC_BITBAND(0x40000054, 0)
#define SYSCTRL_RF_ACCESS_CFG           ((SYSCTRL_RF_ACCESS_CFG_Type *) SYSCTRL_RF_ACCESS_CFG_BASE)

/* SYSCTRL_RF_ACCESS_CFG settings */
#define RF_ACCESS_DISABLE_BITBAND       0x0
#define RF_ACCESS_ENABLE_BITBAND        0x1
#define RF_ACCESS_DISABLE               ((uint32_t)(RF_ACCESS_DISABLE_BITBAND << SYSCTRL_RF_ACCESS_CFG_RF_ACCESS_Pos))
#define RF_ACCESS_ENABLE                ((uint32_t)(RF_ACCESS_ENABLE_BITBAND << SYSCTRL_RF_ACCESS_CFG_RF_ACCESS_Pos))

#define RF_IRQ_ACCESS_DISABLE_BITBAND   0x0
#define RF_IRQ_ACCESS_ENABLE_BITBAND    0x1
#define RF_IRQ_ACCESS_DISABLE           ((uint32_t)(RF_IRQ_ACCESS_DISABLE_BITBAND << SYSCTRL_RF_ACCESS_CFG_RF_IRQ_ACCESS_Pos))
#define RF_IRQ_ACCESS_ENABLE            ((uint32_t)(RF_IRQ_ACCESS_ENABLE_BITBAND << SYSCTRL_RF_ACCESS_CFG_RF_IRQ_ACCESS_Pos))

/* SYSCTRL_WAKEUP_PAD bit positions */
#define SYSCTRL_WAKEUP_PAD_WAKEUP_PAD_VALUE_Pos 0

/* SYSCTRL_WAKEUP_PAD bit-band aliases */
typedef struct
{
    __I  uint32_t WAKEUP_PAD_VALUE_ALIAS;/* WAKEUP pad value */
} SYSCTRL_WAKEUP_PAD_Type;

#define SYSCTRL_WAKEUP_PAD_BASE         SYS_CALC_BITBAND(0x40000058, 0)
#define SYSCTRL_WAKEUP_PAD              ((SYSCTRL_WAKEUP_PAD_Type *) SYSCTRL_WAKEUP_PAD_BASE)

/* SYSCTRL_WAKEUP_PAD settings */
#define WAKEUP_PAD_LOW_BITBAND          0x0
#define WAKEUP_PAD_HIGH_BITBAND         0x1
#define WAKEUP_PAD_LOW                  ((uint32_t)(WAKEUP_PAD_LOW_BITBAND << SYSCTRL_WAKEUP_PAD_WAKEUP_PAD_VALUE_Pos))
#define WAKEUP_PAD_HIGH                 ((uint32_t)(WAKEUP_PAD_HIGH_BITBAND << SYSCTRL_WAKEUP_PAD_WAKEUP_PAD_VALUE_Pos))

/* SYSCTRL_DBG_LOCK bit positions */
#define SYSCTRL_DBG_LOCK_DBG_LOCK_WR_Pos 0
#define SYSCTRL_DBG_LOCK_DBG_LOCK_WR_Mask ((uint32_t)(0xFFFFFFFFU << SYSCTRL_DBG_LOCK_DBG_LOCK_WR_Pos))
#define SYSCTRL_DBG_LOCK_DBG_LOCK_RD_Pos 0

/* SYSCTRL_DBG_LOCK bit-band aliases */
typedef struct
{
    __I  uint32_t DBG_LOCK_RD_ALIAS;    /* Debug port access state */
} SYSCTRL_DBG_LOCK_Type;

#define SYSCTRL_DBG_LOCK_BASE           SYS_CALC_BITBAND(0x400000DC, 0)
#define SYSCTRL_DBG_LOCK                ((SYSCTRL_DBG_LOCK_Type *) SYSCTRL_DBG_LOCK_BASE)

/* SYSCTRL_DBG_LOCK settings */
#define DBG_ACCESS_UNLOCKED_BITBAND     0x0
#define DBG_ACCESS_LOCKED_BITBAND       0x1
#define DBG_ACCESS_UNLOCKED             ((uint32_t)(DBG_ACCESS_UNLOCKED_BITBAND << SYSCTRL_DBG_LOCK_DBG_LOCK_RD_Pos))
#define DBG_ACCESS_LOCKED               ((uint32_t)(DBG_ACCESS_LOCKED_BITBAND << SYSCTRL_DBG_LOCK_DBG_LOCK_RD_Pos))

#define DBG_ACCESS_UNLOCK               ((uint32_t)(0xB3909CB5U << SYSCTRL_DBG_LOCK_DBG_LOCK_WR_Pos))
#define DBG_ACCESS_LOCK                 ((uint32_t)(0x4C6F634BU << SYSCTRL_DBG_LOCK_DBG_LOCK_WR_Pos))

/* SYSCTRL_DBG_LOCK_KEY[] bit positions */
#define SYSCTRL_DBG_LOCK_KEY_DBG_LOCK_KEY_Pos 0
#define SYSCTRL_DBG_LOCK_KEY_DBG_LOCK_KEY_Mask ((uint32_t)(0xFFFFFFFFU << SYSCTRL_DBG_LOCK_KEY_DBG_LOCK_KEY_Pos))

/* ----------------------------------------------------------------------------
 * Clock Generation
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t SYS_CFG;              /* System Clock Configuration Register */
    __IO uint32_t DIV_CFG0;             /* Prescale register for SLOWCLK, BBCLK and USRCLK clocks */
    __IO uint32_t DIV_CFG1;             /* Prescale register for PWM clock, UART and DMIC clocks */
    __IO uint32_t DIV_CFG2;             /* Prescale register for DC-DC converter and charge pump clocks */
    __IO uint32_t DET_CFG;              /* External clock detector configuration register (including interrupt) */
    __I  uint32_t DET_STATUS;           /* External clock detector status register */
} CLK_Type;

#define CLK_BASE                        0x40000100
#define CLK                             ((CLK_Type *) CLK_BASE)

/* CLK_SYS_CFG bit positions */
#define CLK_SYS_CFG_JTCK_PRESCALE_Pos   16
#define CLK_SYS_CFG_JTCK_PRESCALE_Mask  ((uint32_t)(0xFU << CLK_SYS_CFG_JTCK_PRESCALE_Pos))
#define CLK_SYS_CFG_EXTCLK_PRESCALE_Pos 8
#define CLK_SYS_CFG_EXTCLK_PRESCALE_Mask ((uint32_t)(0xFU << CLK_SYS_CFG_EXTCLK_PRESCALE_Pos))
#define CLK_SYS_CFG_SYSCLK_SRC_SEL_Pos  0
#define CLK_SYS_CFG_SYSCLK_SRC_SEL_Mask ((uint32_t)(0x7U << CLK_SYS_CFG_SYSCLK_SRC_SEL_Pos))

/* CLK_SYS_CFG sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t SYSCLK_SRC_SEL_BYTE;  
    __IO uint8_t EXTCLK_PRESCALE_BYTE; 
    __IO uint8_t JTCK_PRESCALE_BYTE;   
         uint8_t RESERVED0[1];
} CLK_SYS_CFG_Type;

#define CLK_SYS_CFG_BASE                0x40000100
#define CLK_SYS_CFG                     ((CLK_SYS_CFG_Type *) CLK_SYS_CFG_BASE)

/* CLK_SYS_CFG settings */
#define SYSCLK_CLKSRC_RCCLK             ((uint32_t)(0x0U << CLK_SYS_CFG_SYSCLK_SRC_SEL_Pos))
#define SYSCLK_CLKSRC_STANDBYCLK        ((uint32_t)(0x1U << CLK_SYS_CFG_SYSCLK_SRC_SEL_Pos))
#define SYSCLK_CLKSRC_RFCLK             ((uint32_t)(0x2U << CLK_SYS_CFG_SYSCLK_SRC_SEL_Pos))
#define SYSCLK_CLKSRC_EXTCLK            ((uint32_t)(0x3U << CLK_SYS_CFG_SYSCLK_SRC_SEL_Pos))
#define SYSCLK_CLKSRC_JTCK              ((uint32_t)(0x4U << CLK_SYS_CFG_SYSCLK_SRC_SEL_Pos))

#define EXTCLK_PRESCALE_1               ((uint32_t)(0x0U << CLK_SYS_CFG_EXTCLK_PRESCALE_Pos))
#define EXTCLK_PRESCALE_2               ((uint32_t)(0x1U << CLK_SYS_CFG_EXTCLK_PRESCALE_Pos))
#define EXTCLK_PRESCALE_15              ((uint32_t)(0xEU << CLK_SYS_CFG_EXTCLK_PRESCALE_Pos))
#define EXTCLK_PRESCALE_16              ((uint32_t)(0xFU << CLK_SYS_CFG_EXTCLK_PRESCALE_Pos))

#define JTCK_PRESCALE_1                 ((uint32_t)(0x0U << CLK_SYS_CFG_JTCK_PRESCALE_Pos))
#define JTCK_PRESCALE_2                 ((uint32_t)(0x1U << CLK_SYS_CFG_JTCK_PRESCALE_Pos))
#define JTCK_PRESCALE_15                ((uint32_t)(0xEU << CLK_SYS_CFG_JTCK_PRESCALE_Pos))
#define JTCK_PRESCALE_16                ((uint32_t)(0xFU << CLK_SYS_CFG_JTCK_PRESCALE_Pos))

/* CLK_SYS_CFG sub-register bit positions */
#define CLK_SYS_CFG_SYSCLK_SRC_SEL_BYTE_Pos 0
#define CLK_SYS_CFG_SYSCLK_SRC_SEL_BYTE_Mask ((uint32_t)(0x7U << CLK_SYS_CFG_SYSCLK_SRC_SEL_BYTE_Pos))
#define CLK_SYS_CFG_EXTCLK_PRESCALE_BYTE_Pos 0
#define CLK_SYS_CFG_EXTCLK_PRESCALE_BYTE_Mask ((uint32_t)(0xFU << CLK_SYS_CFG_EXTCLK_PRESCALE_BYTE_Pos))
#define CLK_SYS_CFG_JTCK_PRESCALE_BYTE_Pos 0
#define CLK_SYS_CFG_JTCK_PRESCALE_BYTE_Mask ((uint32_t)(0xFU << CLK_SYS_CFG_JTCK_PRESCALE_BYTE_Pos))

/* CLK_SYS_CFG sub-register settings */
#define SYSCLK_CLKSRC_RCCLK_BYTE        ((uint8_t)(0x0U << CLK_SYS_CFG_SYSCLK_SRC_SEL_BYTE_Pos))
#define SYSCLK_CLKSRC_STANDBYCLK_BYTE   ((uint8_t)(0x1U << CLK_SYS_CFG_SYSCLK_SRC_SEL_BYTE_Pos))
#define SYSCLK_CLKSRC_RFCLK_BYTE        ((uint8_t)(0x2U << CLK_SYS_CFG_SYSCLK_SRC_SEL_BYTE_Pos))
#define SYSCLK_CLKSRC_EXTCLK_BYTE       ((uint8_t)(0x3U << CLK_SYS_CFG_SYSCLK_SRC_SEL_BYTE_Pos))
#define SYSCLK_CLKSRC_JTCK_BYTE         ((uint8_t)(0x4U << CLK_SYS_CFG_SYSCLK_SRC_SEL_BYTE_Pos))

#define EXTCLK_PRESCALE_1_BYTE          ((uint8_t)(0x0U << CLK_SYS_CFG_EXTCLK_PRESCALE_BYTE_Pos))
#define EXTCLK_PRESCALE_2_BYTE          ((uint8_t)(0x1U << CLK_SYS_CFG_EXTCLK_PRESCALE_BYTE_Pos))
#define EXTCLK_PRESCALE_15_BYTE         ((uint8_t)(0xEU << CLK_SYS_CFG_EXTCLK_PRESCALE_BYTE_Pos))
#define EXTCLK_PRESCALE_16_BYTE         ((uint8_t)(0xFU << CLK_SYS_CFG_EXTCLK_PRESCALE_BYTE_Pos))

#define JTCK_PRESCALE_1_BYTE            ((uint8_t)(0x0U << CLK_SYS_CFG_JTCK_PRESCALE_BYTE_Pos))
#define JTCK_PRESCALE_2_BYTE            ((uint8_t)(0x1U << CLK_SYS_CFG_JTCK_PRESCALE_BYTE_Pos))
#define JTCK_PRESCALE_15_BYTE           ((uint8_t)(0xEU << CLK_SYS_CFG_JTCK_PRESCALE_BYTE_Pos))
#define JTCK_PRESCALE_16_BYTE           ((uint8_t)(0xFU << CLK_SYS_CFG_JTCK_PRESCALE_BYTE_Pos))

/* CLK_DIV_CFG0 bit positions */
#define CLK_DIV_CFG0_USRCLK_PRESCALE_Pos 16
#define CLK_DIV_CFG0_USRCLK_PRESCALE_Mask ((uint32_t)(0xFFFU << CLK_DIV_CFG0_USRCLK_PRESCALE_Pos))
#define CLK_DIV_CFG0_BBCLK_PRESCALE_Pos 8
#define CLK_DIV_CFG0_BBCLK_PRESCALE_Mask ((uint32_t)(0x7U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))
#define CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos 0
#define CLK_DIV_CFG0_SLOWCLK_PRESCALE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))

/* CLK_DIV_CFG0 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t SLOWCLK_PRESCALE_BYTE;
    __IO uint8_t BBCLK_PRESCALE_BYTE;  
    __IO uint16_t USRCLK_PRESCALE_SHORT;
} CLK_DIV_CFG0_Type;

#define CLK_DIV_CFG0_BASE               0x40000104
#define CLK_DIV_CFG0                    ((CLK_DIV_CFG0_Type *) CLK_DIV_CFG0_BASE)

/* CLK_DIV_CFG0 settings */
#define SLOWCLK_PRESCALE_1              ((uint32_t)(0x0U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_2              ((uint32_t)(0x1U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_3              ((uint32_t)(0x2U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_4              ((uint32_t)(0x3U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_8              ((uint32_t)(0x7U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_10             ((uint32_t)(0x9U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_12             ((uint32_t)(0xBU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_16             ((uint32_t)(0xFU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_24             ((uint32_t)(0x17U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_48             ((uint32_t)(0x2FU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_63             ((uint32_t)(0x3EU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))
#define SLOWCLK_PRESCALE_64             ((uint32_t)(0x3FU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_Pos))

#define BBCLK_PRESCALE_1                ((uint32_t)(0x0U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))
#define BBCLK_PRESCALE_2                ((uint32_t)(0x1U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))
#define BBCLK_PRESCALE_3                ((uint32_t)(0x2U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))
#define BBCLK_PRESCALE_4                ((uint32_t)(0x3U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))
#define BBCLK_PRESCALE_5                ((uint32_t)(0x4U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))
#define BBCLK_PRESCALE_6                ((uint32_t)(0x5U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))
#define BBCLK_PRESCALE_7                ((uint32_t)(0x6U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))
#define BBCLK_PRESCALE_8                ((uint32_t)(0x7U << CLK_DIV_CFG0_BBCLK_PRESCALE_Pos))

#define USRCLK_PRESCALE_1               ((uint32_t)(0x0U << CLK_DIV_CFG0_USRCLK_PRESCALE_Pos))
#define USRCLK_PRESCALE_2               ((uint32_t)(0x1U << CLK_DIV_CFG0_USRCLK_PRESCALE_Pos))
#define USRCLK_PRESCALE_3               ((uint32_t)(0x2U << CLK_DIV_CFG0_USRCLK_PRESCALE_Pos))
#define USRCLK_PRESCALE_4095            ((uint32_t)(0xFFEU << CLK_DIV_CFG0_USRCLK_PRESCALE_Pos))
#define USRCLK_PRESCALE_4096            ((uint32_t)(0xFFFU << CLK_DIV_CFG0_USRCLK_PRESCALE_Pos))

/* CLK_DIV_CFG0 sub-register bit positions */
#define CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos 0
#define CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos 0
#define CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Mask ((uint32_t)(0x7U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))
#define CLK_DIV_CFG0_USRCLK_PRESCALE_SHORT_Pos 0
#define CLK_DIV_CFG0_USRCLK_PRESCALE_SHORT_Mask ((uint32_t)(0xFFFU << CLK_DIV_CFG0_USRCLK_PRESCALE_SHORT_Pos))

/* CLK_DIV_CFG0 sub-register settings */
#define SLOWCLK_PRESCALE_1_BYTE         ((uint8_t)(0x0U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_2_BYTE         ((uint8_t)(0x1U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_3_BYTE         ((uint8_t)(0x2U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_4_BYTE         ((uint8_t)(0x3U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_8_BYTE         ((uint8_t)(0x7U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_10_BYTE        ((uint8_t)(0x9U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_12_BYTE        ((uint8_t)(0xBU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_16_BYTE        ((uint8_t)(0xFU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_24_BYTE        ((uint8_t)(0x17U << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_48_BYTE        ((uint8_t)(0x2FU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_63_BYTE        ((uint8_t)(0x3EU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))
#define SLOWCLK_PRESCALE_64_BYTE        ((uint8_t)(0x3FU << CLK_DIV_CFG0_SLOWCLK_PRESCALE_BYTE_Pos))

#define BBCLK_PRESCALE_1_BYTE           ((uint8_t)(0x0U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))
#define BBCLK_PRESCALE_2_BYTE           ((uint8_t)(0x1U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))
#define BBCLK_PRESCALE_3_BYTE           ((uint8_t)(0x2U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))
#define BBCLK_PRESCALE_4_BYTE           ((uint8_t)(0x3U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))
#define BBCLK_PRESCALE_5_BYTE           ((uint8_t)(0x4U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))
#define BBCLK_PRESCALE_6_BYTE           ((uint8_t)(0x5U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))
#define BBCLK_PRESCALE_7_BYTE           ((uint8_t)(0x6U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))
#define BBCLK_PRESCALE_8_BYTE           ((uint8_t)(0x7U << CLK_DIV_CFG0_BBCLK_PRESCALE_BYTE_Pos))

#define USRCLK_PRESCALE_1_SHORT         ((uint16_t)(0x0U << CLK_DIV_CFG0_USRCLK_PRESCALE_SHORT_Pos))
#define USRCLK_PRESCALE_2_SHORT         ((uint16_t)(0x1U << CLK_DIV_CFG0_USRCLK_PRESCALE_SHORT_Pos))
#define USRCLK_PRESCALE_3_SHORT         ((uint16_t)(0x2U << CLK_DIV_CFG0_USRCLK_PRESCALE_SHORT_Pos))
#define USRCLK_PRESCALE_4095_SHORT      ((uint16_t)(0xFFEU << CLK_DIV_CFG0_USRCLK_PRESCALE_SHORT_Pos))
#define USRCLK_PRESCALE_4096_SHORT      ((uint16_t)(0xFFFU << CLK_DIV_CFG0_USRCLK_PRESCALE_SHORT_Pos))

/* CLK_DIV_CFG1 bit positions */
#define CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_Pos 30
#define CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_Mask ((uint32_t)(0x3U << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_Pos))
#define CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos 24
#define CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define CLK_DIV_CFG1_UARTCLK_PRESCALE_Pos 16
#define CLK_DIV_CFG1_UARTCLK_PRESCALE_Mask ((uint32_t)(0x1FU << CLK_DIV_CFG1_UARTCLK_PRESCALE_Pos))
#define CLK_DIV_CFG1_PWM1CLK_PRESCALE_Pos 8
#define CLK_DIV_CFG1_PWM1CLK_PRESCALE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG1_PWM1CLK_PRESCALE_Pos))
#define CLK_DIV_CFG1_PWM0CLK_PRESCALE_Pos 0
#define CLK_DIV_CFG1_PWM0CLK_PRESCALE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG1_PWM0CLK_PRESCALE_Pos))

/* CLK_DIV_CFG1 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t PWM0CLK_PRESCALE_BYTE;
    __IO uint8_t PWM1CLK_PRESCALE_BYTE;
    __IO uint8_t UARTCLK_PRESCALE_BYTE;
         uint8_t RESERVED0[1];
} CLK_DIV_CFG1_Type;

#define CLK_DIV_CFG1_BASE               0x40000108
#define CLK_DIV_CFG1                    ((CLK_DIV_CFG1_Type *) CLK_DIV_CFG1_BASE)

/* CLK_DIV_CFG1 settings */
#define PWM0CLK_PRESCALE_1              ((uint32_t)(0x0U << CLK_DIV_CFG1_PWM0CLK_PRESCALE_Pos))
#define PWM0CLK_PRESCALE_2              ((uint32_t)(0x1U << CLK_DIV_CFG1_PWM0CLK_PRESCALE_Pos))
#define PWM0CLK_PRESCALE_63             ((uint32_t)(0x3EU << CLK_DIV_CFG1_PWM0CLK_PRESCALE_Pos))
#define PWM0CLK_PRESCALE_64             ((uint32_t)(0x3FU << CLK_DIV_CFG1_PWM0CLK_PRESCALE_Pos))

#define PWM1CLK_PRESCALE_1              ((uint32_t)(0x0U << CLK_DIV_CFG1_PWM1CLK_PRESCALE_Pos))
#define PWM1CLK_PRESCALE_2              ((uint32_t)(0x1U << CLK_DIV_CFG1_PWM1CLK_PRESCALE_Pos))
#define PWM1CLK_PRESCALE_63             ((uint32_t)(0x3EU << CLK_DIV_CFG1_PWM1CLK_PRESCALE_Pos))
#define PWM1CLK_PRESCALE_64             ((uint32_t)(0x3FU << CLK_DIV_CFG1_PWM1CLK_PRESCALE_Pos))

#define UARTCLK_PRESCALE_1              ((uint32_t)(0x0U << CLK_DIV_CFG1_UARTCLK_PRESCALE_Pos))
#define UARTCLK_PRESCALE_2              ((uint32_t)(0x1U << CLK_DIV_CFG1_UARTCLK_PRESCALE_Pos))
#define UARTCLK_PRESCALE_31             ((uint32_t)(0x1EU << CLK_DIV_CFG1_UARTCLK_PRESCALE_Pos))
#define UARTCLK_PRESCALE_32             ((uint32_t)(0x1FU << CLK_DIV_CFG1_UARTCLK_PRESCALE_Pos))

#define AUDIOCLK_PRESCALE_1             ((uint32_t)(0x0U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_2             ((uint32_t)(0x1U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_3             ((uint32_t)(0x2U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_4             ((uint32_t)(0x3U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_5             ((uint32_t)(0x4U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_6             ((uint32_t)(0x5U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_7             ((uint32_t)(0x6U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_8             ((uint32_t)(0x7U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_9             ((uint32_t)(0x8U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_10            ((uint32_t)(0x9U << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_11            ((uint32_t)(0xAU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_12            ((uint32_t)(0xBU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_13            ((uint32_t)(0xCU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_14            ((uint32_t)(0xDU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_15            ((uint32_t)(0xEU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_16            ((uint32_t)(0xFU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_63            ((uint32_t)(0x3EU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))
#define AUDIOCLK_PRESCALE_64            ((uint32_t)(0x3FU << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_Pos))

#define AUDIOSLOWCLK_PRESCALE_1         ((uint32_t)(0x0U << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_Pos))
#define AUDIOSLOWCLK_PRESCALE_2         ((uint32_t)(0x1U << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_Pos))
#define AUDIOSLOWCLK_PRESCALE_3         ((uint32_t)(0x2U << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_Pos))
#define AUDIOSLOWCLK_PRESCALE_4         ((uint32_t)(0x3U << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_Pos))

/* CLK_DIV_CFG1 sub-register bit positions */
#define CLK_DIV_CFG1_PWM0CLK_PRESCALE_BYTE_Pos 0
#define CLK_DIV_CFG1_PWM0CLK_PRESCALE_BYTE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG1_PWM0CLK_PRESCALE_BYTE_Pos))
#define CLK_DIV_CFG1_PWM1CLK_PRESCALE_BYTE_Pos 0
#define CLK_DIV_CFG1_PWM1CLK_PRESCALE_BYTE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG1_PWM1CLK_PRESCALE_BYTE_Pos))
#define CLK_DIV_CFG1_UARTCLK_PRESCALE_BYTE_Pos 0
#define CLK_DIV_CFG1_UARTCLK_PRESCALE_BYTE_Mask ((uint32_t)(0x1FU << CLK_DIV_CFG1_UARTCLK_PRESCALE_BYTE_Pos))

/* CLK_DIV_CFG1 sub-register settings */
#define PWM0CLK_PRESCALE_1_BYTE         ((uint8_t)(0x0U << CLK_DIV_CFG1_PWM0CLK_PRESCALE_BYTE_Pos))
#define PWM0CLK_PRESCALE_2_BYTE         ((uint8_t)(0x1U << CLK_DIV_CFG1_PWM0CLK_PRESCALE_BYTE_Pos))
#define PWM0CLK_PRESCALE_63_BYTE        ((uint8_t)(0x3EU << CLK_DIV_CFG1_PWM0CLK_PRESCALE_BYTE_Pos))
#define PWM0CLK_PRESCALE_64_BYTE        ((uint8_t)(0x3FU << CLK_DIV_CFG1_PWM0CLK_PRESCALE_BYTE_Pos))

#define PWM1CLK_PRESCALE_1_BYTE         ((uint8_t)(0x0U << CLK_DIV_CFG1_PWM1CLK_PRESCALE_BYTE_Pos))
#define PWM1CLK_PRESCALE_2_BYTE         ((uint8_t)(0x1U << CLK_DIV_CFG1_PWM1CLK_PRESCALE_BYTE_Pos))
#define PWM1CLK_PRESCALE_63_BYTE        ((uint8_t)(0x3EU << CLK_DIV_CFG1_PWM1CLK_PRESCALE_BYTE_Pos))
#define PWM1CLK_PRESCALE_64_BYTE        ((uint8_t)(0x3FU << CLK_DIV_CFG1_PWM1CLK_PRESCALE_BYTE_Pos))

#define UARTCLK_PRESCALE_1_BYTE         ((uint8_t)(0x0U << CLK_DIV_CFG1_UARTCLK_PRESCALE_BYTE_Pos))
#define UARTCLK_PRESCALE_2_BYTE         ((uint8_t)(0x1U << CLK_DIV_CFG1_UARTCLK_PRESCALE_BYTE_Pos))
#define UARTCLK_PRESCALE_31_BYTE        ((uint8_t)(0x1EU << CLK_DIV_CFG1_UARTCLK_PRESCALE_BYTE_Pos))
#define UARTCLK_PRESCALE_32_BYTE        ((uint8_t)(0x1FU << CLK_DIV_CFG1_UARTCLK_PRESCALE_BYTE_Pos))

/* CLK_DIV_CFG2 bit positions */
#define CLK_DIV_CFG2_CPCLK_DISABLE_Pos  15
#define CLK_DIV_CFG2_CPCLK_PRESCALE_Pos 8
#define CLK_DIV_CFG2_CPCLK_PRESCALE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CLK_DIV_CFG2_DCCLK_DISABLE_Pos  7
#define CLK_DIV_CFG2_DCCLK_PRESCALE_Pos 0
#define CLK_DIV_CFG2_DCCLK_PRESCALE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))

/* CLK_DIV_CFG2 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DCCLK_BYTE;           
    __IO uint8_t CPCLK_BYTE;           
         uint8_t RESERVED0[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x4000010C, 0) - (0x4000010C + 4)];
         uint32_t RESERVED1[7];
    __IO uint32_t DCCLK_DISABLE_ALIAS;  /* DC-DC converter clock disable */
         uint32_t RESERVED2[7];
    __IO uint32_t CPCLK_DISABLE_ALIAS;  /* Charge pump clock disable */
} CLK_DIV_CFG2_Type;

#define CLK_DIV_CFG2_BASE               0x4000010C
#define CLK_DIV_CFG2                    ((CLK_DIV_CFG2_Type *) CLK_DIV_CFG2_BASE)

/* CLK_DIV_CFG2 settings */
#define DCCLK_PRESCALE_1                ((uint32_t)(0x0U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_2                ((uint32_t)(0x1U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_3                ((uint32_t)(0x2U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_4                ((uint32_t)(0x3U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_5                ((uint32_t)(0x4U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_6                ((uint32_t)(0x5U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_7                ((uint32_t)(0x6U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_8                ((uint32_t)(0x7U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_9                ((uint32_t)(0x8U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_10               ((uint32_t)(0x9U << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_12               ((uint32_t)(0xBU << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_63               ((uint32_t)(0x3EU << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))
#define DCCLK_PRESCALE_64               ((uint32_t)(0x3FU << CLK_DIV_CFG2_DCCLK_PRESCALE_Pos))

#define DCCLK_ENABLE_BITBAND            0x0
#define DCCLK_DISABLE_BITBAND           0x1
#define DCCLK_ENABLE                    ((uint32_t)(DCCLK_ENABLE_BITBAND << CLK_DIV_CFG2_DCCLK_DISABLE_Pos))
#define DCCLK_DISABLE                   ((uint32_t)(DCCLK_DISABLE_BITBAND << CLK_DIV_CFG2_DCCLK_DISABLE_Pos))

#define CPCLK_PRESCALE_1                ((uint32_t)(0x0U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_2                ((uint32_t)(0x1U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_3                ((uint32_t)(0x2U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_4                ((uint32_t)(0x3U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_5                ((uint32_t)(0x4U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_6                ((uint32_t)(0x5U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_7                ((uint32_t)(0x6U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_8                ((uint32_t)(0x7U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_9                ((uint32_t)(0x8U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_10               ((uint32_t)(0x9U << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_63               ((uint32_t)(0x3EU << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))
#define CPCLK_PRESCALE_64               ((uint32_t)(0x3FU << CLK_DIV_CFG2_CPCLK_PRESCALE_Pos))

#define CPCLK_ENABLE_BITBAND            0x0
#define CPCLK_DISABLE_BITBAND           0x1
#define CPCLK_ENABLE                    ((uint32_t)(CPCLK_ENABLE_BITBAND << CLK_DIV_CFG2_CPCLK_DISABLE_Pos))
#define CPCLK_DISABLE                   ((uint32_t)(CPCLK_DISABLE_BITBAND << CLK_DIV_CFG2_CPCLK_DISABLE_Pos))

/* CLK_DIV_CFG2 sub-register bit positions */
#define CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos 0
#define CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define CLK_DIV_CFG2_DCCLK_DISABLE_BYTE_Pos 7
#define CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos 0
#define CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Mask ((uint32_t)(0x3FU << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CLK_DIV_CFG2_CPCLK_DISABLE_BYTE_Pos 7

/* CLK_DIV_CFG2 sub-register settings */
#define DCCLK_PRESCALE_1_BYTE           ((uint8_t)(0x0U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_2_BYTE           ((uint8_t)(0x1U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_3_BYTE           ((uint8_t)(0x2U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_4_BYTE           ((uint8_t)(0x3U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_5_BYTE           ((uint8_t)(0x4U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_6_BYTE           ((uint8_t)(0x5U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_7_BYTE           ((uint8_t)(0x6U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_8_BYTE           ((uint8_t)(0x7U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_9_BYTE           ((uint8_t)(0x8U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_10_BYTE          ((uint8_t)(0x9U << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_12_BYTE          ((uint8_t)(0xBU << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_63_BYTE          ((uint8_t)(0x3EU << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))
#define DCCLK_PRESCALE_64_BYTE          ((uint8_t)(0x3FU << CLK_DIV_CFG2_DCCLK_PRESCALE_BYTE_Pos))

#define DCCLK_ENABLE_BYTE               ((uint8_t)(DCCLK_ENABLE_BITBAND << CLK_DIV_CFG2_DCCLK_DISABLE_BYTE_Pos))
#define DCCLK_DISABLE_BYTE              ((uint8_t)(DCCLK_DISABLE_BITBAND << CLK_DIV_CFG2_DCCLK_DISABLE_BYTE_Pos))

#define CPCLK_PRESCALE_1_BYTE           ((uint8_t)(0x0U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_2_BYTE           ((uint8_t)(0x1U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_3_BYTE           ((uint8_t)(0x2U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_4_BYTE           ((uint8_t)(0x3U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_5_BYTE           ((uint8_t)(0x4U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_6_BYTE           ((uint8_t)(0x5U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_7_BYTE           ((uint8_t)(0x6U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_8_BYTE           ((uint8_t)(0x7U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_9_BYTE           ((uint8_t)(0x8U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_10_BYTE          ((uint8_t)(0x9U << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_63_BYTE          ((uint8_t)(0x3EU << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))
#define CPCLK_PRESCALE_64_BYTE          ((uint8_t)(0x3FU << CLK_DIV_CFG2_CPCLK_PRESCALE_BYTE_Pos))

#define CPCLK_ENABLE_BYTE               ((uint8_t)(CPCLK_ENABLE_BITBAND << CLK_DIV_CFG2_CPCLK_DISABLE_BYTE_Pos))
#define CPCLK_DISABLE_BYTE              ((uint8_t)(CPCLK_DISABLE_BITBAND << CLK_DIV_CFG2_CPCLK_DISABLE_BYTE_Pos))

/* CLK_DET_CFG bit positions */
#define CLK_DET_CFG_CLK_DET_SEL_Pos     5
#define CLK_DET_CFG_CLK_DET_INT_SEL_Pos 3
#define CLK_DET_CFG_CLK_DET_INT_SEL_Mask ((uint32_t)(0x3U << CLK_DET_CFG_CLK_DET_INT_SEL_Pos))
#define CLK_DET_CFG_CLK_DET_DIV_Pos     1
#define CLK_DET_CFG_CLK_DET_DIV_Mask    ((uint32_t)(0x3U << CLK_DET_CFG_CLK_DET_DIV_Pos))
#define CLK_DET_CFG_CLK_DET_ENABLE_Pos  0

/* CLK_DET_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t CLK_DET_ENABLE_ALIAS; /* Clock detector enable/disable */
         uint32_t RESERVED0[4];
    __IO uint32_t CLK_DET_SEL_ALIAS;    /* Clock detector source selection */
} CLK_DET_CFG_Type;

#define CLK_DET_CFG_BASE                SYS_CALC_BITBAND(0x40000110, 0)
#define CLK_DET_CFG                     ((CLK_DET_CFG_Type *) CLK_DET_CFG_BASE)

/* CLK_DET_CFG settings */
#define CLK_DET_DISABLE_BITBAND         0x0
#define CLK_DET_ENABLE_BITBAND          0x1
#define CLK_DET_DISABLE                 ((uint32_t)(CLK_DET_DISABLE_BITBAND << CLK_DET_CFG_CLK_DET_ENABLE_Pos))
#define CLK_DET_ENABLE                  ((uint32_t)(CLK_DET_ENABLE_BITBAND << CLK_DET_CFG_CLK_DET_ENABLE_Pos))

#define CLK_DET_SLOWCLK_DIV32           ((uint32_t)(0x0U << CLK_DET_CFG_CLK_DET_DIV_Pos))
#define CLK_DET_SLOWCLK_DIV64           ((uint32_t)(0x1U << CLK_DET_CFG_CLK_DET_DIV_Pos))
#define CLK_DET_SLOWCLK_DIV96           ((uint32_t)(0x2U << CLK_DET_CFG_CLK_DET_DIV_Pos))
#define CLK_DET_SLOWCLK_DIV128          ((uint32_t)(0x3U << CLK_DET_CFG_CLK_DET_DIV_Pos))

#define CLK_DET_INT_DISABLE             ((uint32_t)(0x0U << CLK_DET_CFG_CLK_DET_INT_SEL_Pos))
#define CLK_DET_INT_ACTIVATED           ((uint32_t)(0x1U << CLK_DET_CFG_CLK_DET_INT_SEL_Pos))
#define CLK_DET_INT_DEACTIVATED         ((uint32_t)(0x2U << CLK_DET_CFG_CLK_DET_INT_SEL_Pos))
#define CLK_DET_INT_ACTIVITY_CHANGE     ((uint32_t)(0x3U << CLK_DET_CFG_CLK_DET_INT_SEL_Pos))

#define CLK_DET_SEL_EXTCLK_BITBAND      0x0
#define CLK_DET_SEL_JTCK_BITBAND        0x1
#define CLK_DET_SEL_EXTCLK              ((uint32_t)(CLK_DET_SEL_EXTCLK_BITBAND << CLK_DET_CFG_CLK_DET_SEL_Pos))
#define CLK_DET_SEL_JTCK                ((uint32_t)(CLK_DET_SEL_JTCK_BITBAND << CLK_DET_CFG_CLK_DET_SEL_Pos))

/* CLK_DET_STATUS bit positions */
#define CLK_DET_STATUS_CLK_DET_INT_STATUS_Pos 1
#define CLK_DET_STATUS_CLK_DET_STATUS_Pos 0

/* CLK_DET_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t CLK_DET_STATUS_ALIAS; /* Clock detector status */
    __I  uint32_t CLK_DET_INT_STATUS_ALIAS;/* Clock detector interrupt status (cleared when read) */
} CLK_DET_STATUS_Type;

#define CLK_DET_STATUS_BASE             SYS_CALC_BITBAND(0x40000114, 0)
#define CLK_DET_STATUS                  ((CLK_DET_STATUS_Type *) CLK_DET_STATUS_BASE)

/* CLK_DET_STATUS settings */
#define CLK_NOT_ACTIVE_BITBAND          0x0
#define CLK_ACTIVE_BITBAND              0x1
#define CLK_NOT_ACTIVE                  ((uint32_t)(CLK_NOT_ACTIVE_BITBAND << CLK_DET_STATUS_CLK_DET_STATUS_Pos))
#define CLK_ACTIVE                      ((uint32_t)(CLK_ACTIVE_BITBAND << CLK_DET_STATUS_CLK_DET_STATUS_Pos))

#define CLK_DET_INT_FALSE_BITBAND       0x0
#define CLK_DET_INT_TRUE_BITBAND        0x1
#define CLK_DET_INT_FALSE               ((uint32_t)(CLK_DET_INT_FALSE_BITBAND << CLK_DET_STATUS_CLK_DET_INT_STATUS_Pos))
#define CLK_DET_INT_TRUE                ((uint32_t)(CLK_DET_INT_TRUE_BITBAND << CLK_DET_STATUS_CLK_DET_INT_STATUS_Pos))

/* ----------------------------------------------------------------------------
 * Reset
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t RESET_STATUS;         /* Reset status register */
} DIG_Type;

#define DIG_BASE                        0x40000200
#define DIG                             ((DIG_Type *) DIG_BASE)

/* DIG_RESET_STATUS bit positions */
#define DIG_RESET_STATUS_LOCKUP_RESET_FLAG_CLEAR_Pos 7
#define DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_CLEAR_Pos 6
#define DIG_RESET_STATUS_CM3_SW_RESET_FLAG_CLEAR_Pos 5
#define DIG_RESET_STATUS_ACS_RESET_FLAG_CLEAR_Pos 4
#define DIG_RESET_STATUS_LOCKUP_FLAG_Pos 3
#define DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_Pos 2
#define DIG_RESET_STATUS_CM3_SW_RESET_FLAG_Pos 1
#define DIG_RESET_STATUS_ACS_RESET_FLAG_Pos 0

/* DIG_RESET_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t ACS_RESET_FLAG_ALIAS; /* Sticky flag that detects that a ACS reset occurred */
    __I  uint32_t CM3_SW_RESET_FLAG_ALIAS;/* Sticky flag that detects that a CM3 software reset occurred */
    __I  uint32_t WATCHDOG_RESET_FLAG_ALIAS;/* Sticky flag that detects that a Watchdog time-out reset occurred */
    __I  uint32_t LOCKUP_FLAG_ALIAS;    /* Sticky flag that detects that a LOCKUP occurred */
    __O  uint32_t ACS_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky ACS reset flag */
    __O  uint32_t CM3_SW_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky CM3 software reset flag */
    __O  uint32_t WATCHDOG_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky Watchdog time-out reset flag */
    __O  uint32_t LOCKUP_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky LOCKUP flag */
} DIG_RESET_STATUS_Type;

#define DIG_RESET_STATUS_BASE           SYS_CALC_BITBAND(0x40000200, 0)
#define DIG_RESET_STATUS                ((DIG_RESET_STATUS_Type *) DIG_RESET_STATUS_BASE)

/* DIG_RESET_STATUS settings */
#define ACS_RESET_NOT_SET_BITBAND       0x0
#define ACS_RESET_SET_BITBAND           0x1
#define ACS_RESET_NOT_SET               ((uint32_t)(ACS_RESET_NOT_SET_BITBAND << DIG_RESET_STATUS_ACS_RESET_FLAG_Pos))
#define ACS_RESET_SET                   ((uint32_t)(ACS_RESET_SET_BITBAND << DIG_RESET_STATUS_ACS_RESET_FLAG_Pos))

#define CM3_SW_RESET_NOT_SET_BITBAND    0x0
#define CM3_SW_RESET_SET_BITBAND        0x1
#define CM3_SW_RESET_NOT_SET            ((uint32_t)(CM3_SW_RESET_NOT_SET_BITBAND << DIG_RESET_STATUS_CM3_SW_RESET_FLAG_Pos))
#define CM3_SW_RESET_SET                ((uint32_t)(CM3_SW_RESET_SET_BITBAND << DIG_RESET_STATUS_CM3_SW_RESET_FLAG_Pos))

#define WATCHDOG_RESET_NOT_SET_BITBAND  0x0
#define WATCHDOG_RESET_SET_BITBAND      0x1
#define WATCHDOG_RESET_NOT_SET          ((uint32_t)(WATCHDOG_RESET_NOT_SET_BITBAND << DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_Pos))
#define WATCHDOG_RESET_SET              ((uint32_t)(WATCHDOG_RESET_SET_BITBAND << DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_Pos))

#define LOCKUP_NOT_SET_BITBAND          0x0
#define LOCKUP_SET_BITBAND              0x1
#define LOCKUP_NOT_SET                  ((uint32_t)(LOCKUP_NOT_SET_BITBAND << DIG_RESET_STATUS_LOCKUP_FLAG_Pos))
#define LOCKUP_SET                      ((uint32_t)(LOCKUP_SET_BITBAND << DIG_RESET_STATUS_LOCKUP_FLAG_Pos))

#define ACS_RESET_FLAG_CLEAR_BITBAND    0x1
#define ACS_RESET_FLAG_CLEAR            ((uint32_t)(ACS_RESET_FLAG_CLEAR_BITBAND << DIG_RESET_STATUS_ACS_RESET_FLAG_CLEAR_Pos))

#define CM3_SW_RESET_FLAG_CLEAR_BITBAND 0x1
#define CM3_SW_RESET_FLAG_CLEAR         ((uint32_t)(CM3_SW_RESET_FLAG_CLEAR_BITBAND << DIG_RESET_STATUS_CM3_SW_RESET_FLAG_CLEAR_Pos))

#define WATCHDOG_RESET_FLAG_CLEAR_BITBAND 0x1
#define WATCHDOG_RESET_FLAG_CLEAR       ((uint32_t)(WATCHDOG_RESET_FLAG_CLEAR_BITBAND << DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_CLEAR_Pos))

#define LOCKUP_FLAG_CLEAR_BITBAND       0x1
#define LOCKUP_FLAG_CLEAR               ((uint32_t)(LOCKUP_FLAG_CLEAR_BITBAND << DIG_RESET_STATUS_LOCKUP_RESET_FLAG_CLEAR_Pos))

/* ----------------------------------------------------------------------------
 * Watchdog Timer
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CFG;                  /* Watchdog timer Configuration Register */
    __O  uint32_t CTRL;                 /* Watchdog Refresh Control Register */
} WATCHDOG_Type;

#define WATCHDOG_BASE                   0x40000300
#define WATCHDOG                        ((WATCHDOG_Type *) WATCHDOG_BASE)

/* WATCHDOG_CFG bit positions */
#define WATCHDOG_CFG_TIMEOUT_VALUE_Pos  0
#define WATCHDOG_CFG_TIMEOUT_VALUE_Mask ((uint32_t)(0xFU << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))

/* WATCHDOG_CFG settings */
#define WATCHDOG_TIMEOUT_2M048          ((uint32_t)(0x0U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_4M096          ((uint32_t)(0x1U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_8M2            ((uint32_t)(0x2U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_16M4           ((uint32_t)(0x3U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_32M8           ((uint32_t)(0x4U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_65M5           ((uint32_t)(0x5U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_131M1          ((uint32_t)(0x6U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_262M1          ((uint32_t)(0x7U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_524M3          ((uint32_t)(0x8U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_1048M6         ((uint32_t)(0x9U << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_2097M1         ((uint32_t)(0xAU << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))
#define WATCHDOG_TIMEOUT_4194M3         ((uint32_t)(0xBU << WATCHDOG_CFG_TIMEOUT_VALUE_Pos))

/* WATCHDOG_CTRL bit positions */
#define WATCHDOG_CTRL_WATCHDOG_REFRESH_Pos 0
#define WATCHDOG_CTRL_WATCHDOG_REFRESH_Mask ((uint32_t)(0xFFFFFFFFU << WATCHDOG_CTRL_WATCHDOG_REFRESH_Pos))

/* WATCHDOG_CTRL settings */
#define WATCHDOG_REFRESH                ((uint32_t)(0x2B1E211U << WATCHDOG_CTRL_WATCHDOG_REFRESH_Pos))

/* ----------------------------------------------------------------------------
 * General-Purpose Timers 0, 1, 2 and 3
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CFG[4];               /* Timer Configuration Registers */
    __IO uint32_t CTRL[4];              /* General-Purpose timer Control / Status Register */
    __I  uint32_t VAL[4];               /* Timer Current Value Register */
} TIMER_Type;

#define TIMER_BASE                      0x40000400
#define TIMER                           ((TIMER_Type *) TIMER_BASE)

/* TIMER_CFG[] bit positions */
#define TIMER_CFG_MULTI_COUNT_Pos       29
#define TIMER_CFG_MULTI_COUNT_Mask      ((uint32_t)(0x7U << TIMER_CFG_MULTI_COUNT_Pos))
#define TIMER_CFG_MODE_Pos              28
#define TIMER_CFG_CLK_SRC_Pos           27
#define TIMER_CFG_PRESCALE_Pos          24
#define TIMER_CFG_PRESCALE_Mask         ((uint32_t)(0x7U << TIMER_CFG_PRESCALE_Pos))
#define TIMER_CFG_TIMEOUT_VALUE_Pos     0
#define TIMER_CFG_TIMEOUT_VALUE_Mask    ((uint32_t)(0xFFFFFFU << TIMER_CFG_TIMEOUT_VALUE_Pos))

/* TIMER_CFG[] bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[27];
    __IO uint32_t CLK_SRC_ALIAS;        /* Clock source */
    __IO uint32_t MODE_ALIAS;           /* Timer mode */
         uint32_t RESERVED_SPAN[3];
} TIMER_CFG_Type;

#define TIMER_CFG_BASE                  SYS_CALC_BITBAND(0x40000400, 0)
#define TIMER_CFG                       ((TIMER_CFG_Type *) TIMER_CFG_BASE)

/* TIMER_CFG settings */
#define TIMER_PRESCALE_1                ((uint32_t)(0x0U << TIMER_CFG_PRESCALE_Pos))
#define TIMER_PRESCALE_2                ((uint32_t)(0x1U << TIMER_CFG_PRESCALE_Pos))
#define TIMER_PRESCALE_4                ((uint32_t)(0x2U << TIMER_CFG_PRESCALE_Pos))
#define TIMER_PRESCALE_8                ((uint32_t)(0x3U << TIMER_CFG_PRESCALE_Pos))
#define TIMER_PRESCALE_16               ((uint32_t)(0x4U << TIMER_CFG_PRESCALE_Pos))
#define TIMER_PRESCALE_32               ((uint32_t)(0x5U << TIMER_CFG_PRESCALE_Pos))
#define TIMER_PRESCALE_64               ((uint32_t)(0x6U << TIMER_CFG_PRESCALE_Pos))
#define TIMER_PRESCALE_128              ((uint32_t)(0x7U << TIMER_CFG_PRESCALE_Pos))

#define TIMER_SLOWCLK_DIV32_BITBAND     0x0
#define TIMER_SLOWCLK_DIV2_BITBAND      0x1
#define TIMER_SLOWCLK_DIV32             ((uint32_t)(TIMER_SLOWCLK_DIV32_BITBAND << TIMER_CFG_CLK_SRC_Pos))
#define TIMER_SLOWCLK_DIV2              ((uint32_t)(TIMER_SLOWCLK_DIV2_BITBAND << TIMER_CFG_CLK_SRC_Pos))

#define TIMER_SHOT_MODE_BITBAND         0x0
#define TIMER_FREE_RUN_BITBAND          0x1
#define TIMER_SHOT_MODE                 ((uint32_t)(TIMER_SHOT_MODE_BITBAND << TIMER_CFG_MODE_Pos))
#define TIMER_FREE_RUN                  ((uint32_t)(TIMER_FREE_RUN_BITBAND << TIMER_CFG_MODE_Pos))

#define TIMER_MULTI_COUNT_1             ((uint32_t)(0x0U << TIMER_CFG_MULTI_COUNT_Pos))
#define TIMER_MULTI_COUNT_2             ((uint32_t)(0x1U << TIMER_CFG_MULTI_COUNT_Pos))
#define TIMER_MULTI_COUNT_3             ((uint32_t)(0x2U << TIMER_CFG_MULTI_COUNT_Pos))
#define TIMER_MULTI_COUNT_4             ((uint32_t)(0x3U << TIMER_CFG_MULTI_COUNT_Pos))
#define TIMER_MULTI_COUNT_5             ((uint32_t)(0x4U << TIMER_CFG_MULTI_COUNT_Pos))
#define TIMER_MULTI_COUNT_6             ((uint32_t)(0x5U << TIMER_CFG_MULTI_COUNT_Pos))
#define TIMER_MULTI_COUNT_7             ((uint32_t)(0x6U << TIMER_CFG_MULTI_COUNT_Pos))
#define TIMER_MULTI_COUNT_8             ((uint32_t)(0x7U << TIMER_CFG_MULTI_COUNT_Pos))

/* TIMER_CTRL[] bit positions */
#define TIMER_CTRL_TIMER_STATUS_Pos     2
#define TIMER_CTRL_TIMER_START_Pos      1
#define TIMER_CTRL_TIMER_STOP_Pos       0

/* TIMER_CTRL[] bit-band aliases */
typedef struct
{
    __O  uint32_t TIMER_STOP_ALIAS;     /* Stop the timer */
    __O  uint32_t TIMER_START_ALIAS;    /* Start or restart the timer */
    __I  uint32_t TIMER_STATUS_ALIAS;   /* Indicate if the timer is active or not */
         uint32_t RESERVED_SPAN[29];
} TIMER_CTRL_Type;

#define TIMER_CTRL_BASE                 SYS_CALC_BITBAND(0x40000410, 0)
#define TIMER_CTRL                      ((TIMER_CTRL_Type *) TIMER_CTRL_BASE)

/* TIMER_CTRL settings */
#define TIMER_STOP_BITBAND              0x1
#define TIMER_STOP                      ((uint32_t)(TIMER_STOP_BITBAND << TIMER_CTRL_TIMER_STOP_Pos))

#define TIMER_START_BITBAND             0x1
#define TIMER_START                     ((uint32_t)(TIMER_START_BITBAND << TIMER_CTRL_TIMER_START_Pos))

#define TIMER_INACTIVE_BITBAND          0x0
#define TIMER_ACTIVE_BITBAND            0x1
#define TIMER_INACTIVE                  ((uint32_t)(TIMER_INACTIVE_BITBAND << TIMER_CTRL_TIMER_STATUS_Pos))
#define TIMER_ACTIVE                    ((uint32_t)(TIMER_ACTIVE_BITBAND << TIMER_CTRL_TIMER_STATUS_Pos))

/* TIMER_VAL[] bit positions */
#define TIMER_VAL_TIMER_VALUE_Pos       0
#define TIMER_VAL_TIMER_VALUE_Mask      ((uint32_t)(0xFFFFFFU << TIMER_VAL_TIMER_VALUE_Pos))

/* ----------------------------------------------------------------------------
 * Flash Interface Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t IF_CTRL;              /* Flash Interface Control Register */
    __O  uint32_t MAIN_WRITE_UNLOCK;    /* Flash Main Write Unlock Register */
    __IO uint32_t MAIN_CTRL;            /* Flash Main Write Control Register */
         uint32_t RESERVED0;
    __IO uint32_t DELAY_CTRL;           /* Flash, Memory and RF Power-Up Delay Configuration */
         uint32_t RESERVED1[8];
    __IO uint32_t CMD_CTRL;             /* Flash Command Control Register */
    __I  uint32_t IF_STATUS;            /* Flash Interface Status Register */
    __IO uint32_t ADDR;                 /* Flash Address Register */
    __IO uint32_t DATA[2];              /* Flash Read/Write Data Register */
    __O  uint32_t NVR_WRITE_UNLOCK;     /* Flash NVR Write Unlock Register */
    __IO uint32_t NVR_CTRL;             /* Flash NVR Control Register */
         uint32_t RESERVED2[6];
    __IO uint32_t PATCH_ADDR[4];        /* Flash Patch Address Register */
         uint32_t RESERVED3[2];
    __IO uint32_t COPY_CFG;             /* Flash Copier Config Register */
         uint32_t RESERVED4[17];
    __IO uint32_t COPY_CTRL;            /* Flash-to-Memory Copier Control and Status */
         uint32_t RESERVED5;
    __IO uint32_t COPY_SRC_ADDR_PTR;    /* Flash-to-Memory Copier Source Address Pointer */
    __IO uint32_t COPY_DST_ADDR_PTR;    /* Flash-to-Memory Copier Destination Address Pointer */
    __IO uint32_t COPY_WORD_CNT;        /* Flash-to-Memory Copier Word Count */
    __IO uint32_t ECC_CTRL;             /* Flash ECC Control Register */
    __IO uint32_t ECC_STATUS;           /* Flash ECC Status Register */
    __I  uint32_t ECC_ERROR_ADDR;       /* Flash Address of the Latest Detected Error */
    __I  uint32_t ECC_UNCOR_ERROR_CNT;  /* Flash ECC Uncorrected Error Counter */
    __I  uint32_t ECC_COR_ERROR_CNT;    /* Flash ECC Corrected Error Counter */
} FLASH_Type;

#define FLASH_BASE                      0x40000500
#define FLASH                           ((FLASH_Type *) FLASH_BASE)

/* FLASH_IF_CTRL bit positions */
#define FLASH_IF_CTRL_PREFETCH_DBUS_Pos 18
#define FLASH_IF_CTRL_PREFETCH_IBUS_Pos 17
#define FLASH_IF_CTRL_NOT_LOAD_AUTO_Pos 16
#define FLASH_IF_CTRL_VREAD1_MODE_Pos   12
#define FLASH_IF_CTRL_VREAD0_MODE_Pos   11
#define FLASH_IF_CTRL_RECALL_Pos        10
#define FLASH_IF_CTRL_RETRY_Pos         8
#define FLASH_IF_CTRL_RETRY_Mask        ((uint32_t)(0x3U << FLASH_IF_CTRL_RETRY_Pos))
#define FLASH_IF_CTRL_LP_MODE_Pos       0

/* FLASH_IF_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t LP_MODE_BYTE;         
         uint8_t RESERVED0[3];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000500, 0) - (0x40000500 + 4)];
    __IO uint32_t LP_MODE_ALIAS;        /* Set the low power mode. Changing this bit will execute the CMD_SET_LOW_POWER or CMD_UNSET_LOW_POWER command. */
         uint32_t RESERVED1[9];
    __IO uint32_t RECALL_ALIAS;         /* Set the recall pins mode during CMD_READ. Changing this bit will execute the CMD_SET_RECALL or CMD_UNSET_RECALL command. */
    __IO uint32_t VREAD0_MODE_ALIAS;    /* Control VREAD0: Read data after program with more stringent condition than normal read. Changing this bit will execute the CMD_SET_VREAD0 or CMD_UNSET_VREAD0 command. */
    __IO uint32_t VREAD1_MODE_ALIAS;    /* Control VREAD1: Read data after erase with more stringent condition than normal read. Changing this bit will execute the CMD_SET_VREAD1 or CMD_UNSET_VREAD1 command. */
         uint32_t RESERVED2[3];
    __IO uint32_t NOT_LOAD_AUTO_ALIAS;  /* Do not automatically load the configuration registers and the patch information from NVR4 sector after the command WAKEUP is completed. */
    __IO uint32_t PREFETCH_IBUS_ALIAS;  /* Pre-fetch on I-Bus control */
    __IO uint32_t PREFETCH_DBUS_ALIAS;  /* Pre-fetch on D-Bus control */
} FLASH_IF_CTRL_Type;

#define FLASH_IF_CTRL_BASE              0x40000500
#define FLASH_IF_CTRL                   ((FLASH_IF_CTRL_Type *) FLASH_IF_CTRL_BASE)

/* FLASH_IF_CTRL settings */
#define FLASH_LOW_POWER_DISABLE_BITBAND 0x0
#define FLASH_LOW_POWER_ENABLE_BITBAND  0x1
#define FLASH_LOW_POWER_DISABLE         ((uint32_t)(FLASH_LOW_POWER_DISABLE_BITBAND << FLASH_IF_CTRL_LP_MODE_Pos))
#define FLASH_LOW_POWER_ENABLE          ((uint32_t)(FLASH_LOW_POWER_ENABLE_BITBAND << FLASH_IF_CTRL_LP_MODE_Pos))

#define FLASH_RETRY_1                   ((uint32_t)(0x0U << FLASH_IF_CTRL_RETRY_Pos))
#define FLASH_RETRY_2                   ((uint32_t)(0x1U << FLASH_IF_CTRL_RETRY_Pos))
#define FLASH_RETRY_3                   ((uint32_t)(0x2U << FLASH_IF_CTRL_RETRY_Pos))
#define FLASH_RETRY_4                   ((uint32_t)(0x3U << FLASH_IF_CTRL_RETRY_Pos))

#define FLASH_RECALL_DISABLE_BITBAND    0x0
#define FLASH_RECALL_ENABLE_BITBAND     0x1
#define FLASH_RECALL_DISABLE            ((uint32_t)(FLASH_RECALL_DISABLE_BITBAND << FLASH_IF_CTRL_RECALL_Pos))
#define FLASH_RECALL_ENABLE             ((uint32_t)(FLASH_RECALL_ENABLE_BITBAND << FLASH_IF_CTRL_RECALL_Pos))

#define FLASH_VREAD0_DISABLE_BITBAND    0x0
#define FLASH_VREAD0_ENABLE_BITBAND     0x1
#define FLASH_VREAD0_DISABLE            ((uint32_t)(FLASH_VREAD0_DISABLE_BITBAND << FLASH_IF_CTRL_VREAD0_MODE_Pos))
#define FLASH_VREAD0_ENABLE             ((uint32_t)(FLASH_VREAD0_ENABLE_BITBAND << FLASH_IF_CTRL_VREAD0_MODE_Pos))

#define FLASH_VREAD1_DISABLE_BITBAND    0x0
#define FLASH_VREAD1_ENABLE_BITBAND     0x1
#define FLASH_VREAD1_DISABLE            ((uint32_t)(FLASH_VREAD1_DISABLE_BITBAND << FLASH_IF_CTRL_VREAD1_MODE_Pos))
#define FLASH_VREAD1_ENABLE             ((uint32_t)(FLASH_VREAD1_ENABLE_BITBAND << FLASH_IF_CTRL_VREAD1_MODE_Pos))

#define FLASH_LOAD_AUTO_ENABLE_BITBAND  0x0
#define FLASH_LOAD_AUTO_DISABLE_BITBAND 0x1
#define FLASH_LOAD_AUTO_ENABLE          ((uint32_t)(FLASH_LOAD_AUTO_ENABLE_BITBAND << FLASH_IF_CTRL_NOT_LOAD_AUTO_Pos))
#define FLASH_LOAD_AUTO_DISABLE         ((uint32_t)(FLASH_LOAD_AUTO_DISABLE_BITBAND << FLASH_IF_CTRL_NOT_LOAD_AUTO_Pos))

#define FLASH_PREFETCH_IBUS_DISABLE_BITBAND 0x0
#define FLASH_PREFETCH_IBUS_ENABLE_BITBAND 0x1
#define FLASH_PREFETCH_IBUS_DISABLE     ((uint32_t)(FLASH_PREFETCH_IBUS_DISABLE_BITBAND << FLASH_IF_CTRL_PREFETCH_IBUS_Pos))
#define FLASH_PREFETCH_IBUS_ENABLE      ((uint32_t)(FLASH_PREFETCH_IBUS_ENABLE_BITBAND << FLASH_IF_CTRL_PREFETCH_IBUS_Pos))

#define FLASH_PREFETCH_DBUS_DISABLE_BITBAND 0x0
#define FLASH_PREFETCH_DBUS_ENABLE_BITBAND 0x1
#define FLASH_PREFETCH_DBUS_DISABLE     ((uint32_t)(FLASH_PREFETCH_DBUS_DISABLE_BITBAND << FLASH_IF_CTRL_PREFETCH_DBUS_Pos))
#define FLASH_PREFETCH_DBUS_ENABLE      ((uint32_t)(FLASH_PREFETCH_DBUS_ENABLE_BITBAND << FLASH_IF_CTRL_PREFETCH_DBUS_Pos))

/* FLASH_IF_CTRL sub-register bit positions */
#define FLASH_IF_CTRL_LP_MODE_BYTE_Pos  0

/* FLASH_IF_CTRL sub-register settings */
#define FLASH_LOW_POWER_DISABLE_BYTE    ((uint8_t)(FLASH_LOW_POWER_DISABLE_BITBAND << FLASH_IF_CTRL_LP_MODE_BYTE_Pos))
#define FLASH_LOW_POWER_ENABLE_BYTE     ((uint8_t)(FLASH_LOW_POWER_ENABLE_BITBAND << FLASH_IF_CTRL_LP_MODE_BYTE_Pos))

/* FLASH_MAIN_WRITE_UNLOCK bit positions */
#define FLASH_MAIN_WRITE_UNLOCK_UNLOCK_KEY_Pos 0
#define FLASH_MAIN_WRITE_UNLOCK_UNLOCK_KEY_Mask ((uint32_t)(0xFFFFFFFFU << FLASH_MAIN_WRITE_UNLOCK_UNLOCK_KEY_Pos))

/* FLASH_MAIN_WRITE_UNLOCK settings */
#define FLASH_MAIN_KEY                  ((uint32_t)(0xDBC8264EU << FLASH_MAIN_WRITE_UNLOCK_UNLOCK_KEY_Pos))

/* FLASH_MAIN_CTRL bit positions */
#define FLASH_MAIN_CTRL_MAIN_HIGH_W_EN_Pos 2
#define FLASH_MAIN_CTRL_MAIN_MIDDLE_W_EN_Pos 1
#define FLASH_MAIN_CTRL_MAIN_LOW_W_EN_Pos 0

/* FLASH_MAIN_CTRL bit-band aliases */
typedef struct
{
    __IO uint32_t MAIN_LOW_W_EN_ALIAS;  /* Authorize the write access to the lower part of the Flash MAIN block through the FLASH_IF registers. */
    __IO uint32_t MAIN_MIDDLE_W_EN_ALIAS;/* Authorize the write access to the middle part of the Flash MAIN block through the FLASH_IF registers. */
    __IO uint32_t MAIN_HIGH_W_EN_ALIAS; /* Authorize the write access to the high part of the Flash MAIN block through the FLASH_IF registers. */
} FLASH_MAIN_CTRL_Type;

#define FLASH_MAIN_CTRL_BASE            SYS_CALC_BITBAND(0x40000508, 0)
#define FLASH_MAIN_CTRL                 ((FLASH_MAIN_CTRL_Type *) FLASH_MAIN_CTRL_BASE)

/* FLASH_MAIN_CTRL settings */
#define MAIN_LOW_W_DISABLE_BITBAND      0x0
#define MAIN_LOW_W_ENABLE_BITBAND       0x1
#define MAIN_LOW_W_DISABLE              ((uint32_t)(MAIN_LOW_W_DISABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_LOW_W_EN_Pos))
#define MAIN_LOW_W_ENABLE               ((uint32_t)(MAIN_LOW_W_ENABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_LOW_W_EN_Pos))

#define MAIN_MIDDLE_W_DISABLE_BITBAND   0x0
#define MAIN_MIDDLE_W_ENABLE_BITBAND    0x1
#define MAIN_MIDDLE_W_DISABLE           ((uint32_t)(MAIN_MIDDLE_W_DISABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_MIDDLE_W_EN_Pos))
#define MAIN_MIDDLE_W_ENABLE            ((uint32_t)(MAIN_MIDDLE_W_ENABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_MIDDLE_W_EN_Pos))

#define MAIN_HIGH_W_DISABLE_BITBAND     0x0
#define MAIN_HIGH_W_ENABLE_BITBAND      0x1
#define MAIN_HIGH_W_DISABLE             ((uint32_t)(MAIN_HIGH_W_DISABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_HIGH_W_EN_Pos))
#define MAIN_HIGH_W_ENABLE              ((uint32_t)(MAIN_HIGH_W_ENABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_HIGH_W_EN_Pos))

/* FLASH_DELAY_CTRL bit positions */
#define FLASH_DELAY_CTRL_READ_MARGIN_Pos 7
#define FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos 0
#define FLASH_DELAY_CTRL_SYSCLK_FREQ_Mask ((uint32_t)(0xFU << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))

/* FLASH_DELAY_CTRL bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[7];
    __IO uint32_t READ_MARGIN_ALIAS;    /* Flash Read access time margin */
} FLASH_DELAY_CTRL_Type;

#define FLASH_DELAY_CTRL_BASE           SYS_CALC_BITBAND(0x40000510, 0)
#define FLASH_DELAY_CTRL                ((FLASH_DELAY_CTRL_Type *) FLASH_DELAY_CTRL_BASE)

/* FLASH_DELAY_CTRL settings */
#define FLASH_DELAY_FOR_SYSCLK_3MHZ     ((uint32_t)(0x0U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_4MHZ     ((uint32_t)(0x1U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_5MHZ     ((uint32_t)(0x2U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_8MHZ     ((uint32_t)(0x3U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_10MHZ    ((uint32_t)(0x4U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_12MHZ    ((uint32_t)(0x5U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_16MHZ    ((uint32_t)(0x6U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_20MHZ    ((uint32_t)(0x7U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_24MHZ    ((uint32_t)(0x8U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))
#define FLASH_DELAY_FOR_SYSCLK_48MHZ    ((uint32_t)(0x9U << FLASH_DELAY_CTRL_SYSCLK_FREQ_Pos))

#define DEFAULT_READ_MARGIN_BITBAND     0x0
#define FAST_READ_MARGIN_BITBAND        0x1
#define DEFAULT_READ_MARGIN             ((uint32_t)(DEFAULT_READ_MARGIN_BITBAND << FLASH_DELAY_CTRL_READ_MARGIN_Pos))
#define FAST_READ_MARGIN                ((uint32_t)(FAST_READ_MARGIN_BITBAND << FLASH_DELAY_CTRL_READ_MARGIN_Pos))

/* FLASH_CMD_CTRL bit positions */
#define FLASH_CMD_CTRL_CMD_END_Pos      5
#define FLASH_CMD_CTRL_CMD_END_Mask     ((uint32_t)(0x3U << FLASH_CMD_CTRL_CMD_END_Pos))
#define FLASH_CMD_CTRL_COMMAND_Pos      0
#define FLASH_CMD_CTRL_COMMAND_Mask     ((uint32_t)(0x1FU << FLASH_CMD_CTRL_COMMAND_Pos))

/* FLASH_CMD_CTRL settings */
#define CMD_IDLE                        ((uint32_t)(0x0U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_WAKE_UP                     ((uint32_t)(0x1U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_LOAD_TRIM                   ((uint32_t)(0x2U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_READ                        ((uint32_t)(0x5U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_PROGRAM_NOSEQ               ((uint32_t)(0x6U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_PROGRAM_SEQ                 ((uint32_t)(0x7U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_SECTOR_ERASE                ((uint32_t)(0x8U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_MASS_ERASE                  ((uint32_t)(0x9U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_SET_LOW_POWER               ((uint32_t)(0xAU << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_UNSET_LOW_POWER             ((uint32_t)(0xBU << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_SET_RECALL                  ((uint32_t)(0xCU << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_UNSET_RECALL                ((uint32_t)(0xDU << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_SET_VREAD1                  ((uint32_t)(0xEU << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_UNSET_VREAD1                ((uint32_t)(0xFU << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_SET_VREAD0                  ((uint32_t)(0x10U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_UNSET_VREAD0                ((uint32_t)(0x11U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_WRITE_USER_RED1             ((uint32_t)(0x12U << FLASH_CMD_CTRL_COMMAND_Pos))
#define CMD_WRITE_USER_RED2             ((uint32_t)(0x13U << FLASH_CMD_CTRL_COMMAND_Pos))

#define CMD_END                         ((uint32_t)(0x3U << FLASH_CMD_CTRL_CMD_END_Pos))

/* FLASH_IF_STATUS bit positions */
#define FLASH_IF_STATUS_TRIMMED_STATUS_Pos 13
#define FLASH_IF_STATUS_ISOLATE_STATUS_Pos 12
#define FLASH_IF_STATUS_PROG_SEQ_DATA_REQ_Pos 11
#define FLASH_IF_STATUS_BUSY_Pos        10
#define FLASH_IF_STATUS_RED2_W_UNLOCK_Pos 9
#define FLASH_IF_STATUS_RED1_W_UNLOCK_Pos 8
#define FLASH_IF_STATUS_NVR3_W_UNLOCK_Pos 6
#define FLASH_IF_STATUS_NVR2_W_UNLOCK_Pos 5
#define FLASH_IF_STATUS_NVR1_W_UNLOCK_Pos 4
#define FLASH_IF_STATUS_MAIN_HIGH_W_UNLOCK_Pos 2
#define FLASH_IF_STATUS_MAIN_MIDDLE_W_UNLOCK_Pos 1
#define FLASH_IF_STATUS_MAIN_LOW_W_UNLOCK_Pos 0

/* FLASH_IF_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t MAIN_LOW_W_UNLOCK_ALIAS;/* Write unlock status bit of the lower part of the Flash MAIN block */
    __I  uint32_t MAIN_MIDDLE_W_UNLOCK_ALIAS;/* Write unlock status bit of the middle part of the Flash MAIN block */
    __I  uint32_t MAIN_HIGH_W_UNLOCK_ALIAS;/* Write unlock status bit of the high part of the Flash MAIN block */
         uint32_t RESERVED0;
    __I  uint32_t NVR1_W_UNLOCK_ALIAS;  /* Flash NVR1 write unlock status bit */
    __I  uint32_t NVR2_W_UNLOCK_ALIAS;  /* Flash NVR2 write unlock status bit */
    __I  uint32_t NVR3_W_UNLOCK_ALIAS;  /* Flash NVR3 write unlock status bit */
         uint32_t RESERVED1;
    __I  uint32_t RED1_W_UNLOCK_ALIAS;  /* Flash RED1 write unlock status bit */
    __I  uint32_t RED2_W_UNLOCK_ALIAS;  /* Flash RED2 write unlock status bit */
    __I  uint32_t BUSY_ALIAS;           /* Flash interface busy status bit */
    __I  uint32_t PROG_SEQ_DATA_REQ_ALIAS;/* Request new data while in sequential program mode */
    __I  uint32_t ISOLATE_STATUS_ALIAS; /* Flash isolate status */
    __I  uint32_t TRIMMED_STATUS_ALIAS; /* Flash trimming status */
} FLASH_IF_STATUS_Type;

#define FLASH_IF_STATUS_BASE            SYS_CALC_BITBAND(0x40000538, 0)
#define FLASH_IF_STATUS                 ((FLASH_IF_STATUS_Type *) FLASH_IF_STATUS_BASE)

/* FLASH_IF_STATUS settings */
#define FLASH_MAIN_LOW_W_LOCKED_BITBAND 0x0
#define FLASH_MAIN_LOW_W_UNLOCKED_BITBAND 0x1
#define FLASH_MAIN_LOW_W_LOCKED         ((uint32_t)(FLASH_MAIN_LOW_W_LOCKED_BITBAND << FLASH_IF_STATUS_MAIN_LOW_W_UNLOCK_Pos))
#define FLASH_MAIN_LOW_W_UNLOCKED       ((uint32_t)(FLASH_MAIN_LOW_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_MAIN_LOW_W_UNLOCK_Pos))

#define FLASH_MAIN_MIDDLE_W_LOCKED_BITBAND 0x0
#define FLASH_MAIN_MIDDLE_W_UNLOCKED_BITBAND 0x1
#define FLASH_MAIN_MIDDLE_W_LOCKED      ((uint32_t)(FLASH_MAIN_MIDDLE_W_LOCKED_BITBAND << FLASH_IF_STATUS_MAIN_MIDDLE_W_UNLOCK_Pos))
#define FLASH_MAIN_MIDDLE_W_UNLOCKED    ((uint32_t)(FLASH_MAIN_MIDDLE_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_MAIN_MIDDLE_W_UNLOCK_Pos))

#define FLASH_MAIN_HIGH_W_LOCKED_BITBAND 0x0
#define FLASH_MAIN_HIGH_W_UNLOCKED_BITBAND 0x1
#define FLASH_MAIN_HIGH_W_LOCKED        ((uint32_t)(FLASH_MAIN_HIGH_W_LOCKED_BITBAND << FLASH_IF_STATUS_MAIN_HIGH_W_UNLOCK_Pos))
#define FLASH_MAIN_HIGH_W_UNLOCKED      ((uint32_t)(FLASH_MAIN_HIGH_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_MAIN_HIGH_W_UNLOCK_Pos))

#define FLASH_NVR1_W_LOCKED_BITBAND     0x0
#define FLASH_NVR1_W_UNLOCKED_BITBAND   0x1
#define FLASH_NVR1_W_LOCKED             ((uint32_t)(FLASH_NVR1_W_LOCKED_BITBAND << FLASH_IF_STATUS_NVR1_W_UNLOCK_Pos))
#define FLASH_NVR1_W_UNLOCKED           ((uint32_t)(FLASH_NVR1_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_NVR1_W_UNLOCK_Pos))

#define FLASH_NVR2_W_LOCKED_BITBAND     0x0
#define FLASH_NVR2_W_UNLOCKED_BITBAND   0x1
#define FLASH_NVR2_W_LOCKED             ((uint32_t)(FLASH_NVR2_W_LOCKED_BITBAND << FLASH_IF_STATUS_NVR2_W_UNLOCK_Pos))
#define FLASH_NVR2_W_UNLOCKED           ((uint32_t)(FLASH_NVR2_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_NVR2_W_UNLOCK_Pos))

#define FLASH_NVR3_W_LOCKED_BITBAND     0x0
#define FLASH_NVR3_W_UNLOCKED_BITBAND   0x1
#define FLASH_NVR3_W_LOCKED             ((uint32_t)(FLASH_NVR3_W_LOCKED_BITBAND << FLASH_IF_STATUS_NVR3_W_UNLOCK_Pos))
#define FLASH_NVR3_W_UNLOCKED           ((uint32_t)(FLASH_NVR3_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_NVR3_W_UNLOCK_Pos))

#define FLASH_RED1_W_LOCKED_BITBAND     0x0
#define FLASH_RED1_W_UNLOCKED_BITBAND   0x1
#define FLASH_RED1_W_LOCKED             ((uint32_t)(FLASH_RED1_W_LOCKED_BITBAND << FLASH_IF_STATUS_RED1_W_UNLOCK_Pos))
#define FLASH_RED1_W_UNLOCKED           ((uint32_t)(FLASH_RED1_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_RED1_W_UNLOCK_Pos))

#define FLASH_RED2_W_LOCKED_BITBAND     0x0
#define FLASH_RED2_W_UNLOCKED_BITBAND   0x1
#define FLASH_RED2_W_LOCKED             ((uint32_t)(FLASH_RED2_W_LOCKED_BITBAND << FLASH_IF_STATUS_RED2_W_UNLOCK_Pos))
#define FLASH_RED2_W_UNLOCKED           ((uint32_t)(FLASH_RED2_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_RED2_W_UNLOCK_Pos))

#define FLASH_IF_IDLE_BITBAND           0x0
#define FLASH_IF_BUSY_BITBAND           0x1
#define FLASH_IF_IDLE                   ((uint32_t)(FLASH_IF_IDLE_BITBAND << FLASH_IF_STATUS_BUSY_Pos))
#define FLASH_IF_BUSY                   ((uint32_t)(FLASH_IF_BUSY_BITBAND << FLASH_IF_STATUS_BUSY_Pos))

#define FLASH_PROG_SEQ_IDLE_BITBAND     0x0
#define FLASH_PROG_SEQ_REQ_NEW_DATA_BITBAND 0x1
#define FLASH_PROG_SEQ_IDLE             ((uint32_t)(FLASH_PROG_SEQ_IDLE_BITBAND << FLASH_IF_STATUS_PROG_SEQ_DATA_REQ_Pos))
#define FLASH_PROG_SEQ_REQ_NEW_DATA     ((uint32_t)(FLASH_PROG_SEQ_REQ_NEW_DATA_BITBAND << FLASH_IF_STATUS_PROG_SEQ_DATA_REQ_Pos))

#define FLASH_ACCESSIBLE_BITBAND        0x0
#define FLASH_ISOLATE_BITBAND           0x1
#define FLASH_ACCESSIBLE                ((uint32_t)(FLASH_ACCESSIBLE_BITBAND << FLASH_IF_STATUS_ISOLATE_STATUS_Pos))
#define FLASH_ISOLATE                   ((uint32_t)(FLASH_ISOLATE_BITBAND << FLASH_IF_STATUS_ISOLATE_STATUS_Pos))

#define FLASH_UNTRIMMED_BITBAND         0x0
#define FLASH_TRIMMED_BITBAND           0x1
#define FLASH_UNTRIMMED                 ((uint32_t)(FLASH_UNTRIMMED_BITBAND << FLASH_IF_STATUS_TRIMMED_STATUS_Pos))
#define FLASH_TRIMMED                   ((uint32_t)(FLASH_TRIMMED_BITBAND << FLASH_IF_STATUS_TRIMMED_STATUS_Pos))

/* FLASH_DATA[] bit positions */
#define FLASH_DATA_DATA_Pos             0
#define FLASH_DATA_DATA_Mask            ((uint32_t)(0xFFFFFFFFU << FLASH_DATA_DATA_Pos))

/* FLASH_NVR_WRITE_UNLOCK bit positions */
#define FLASH_NVR_WRITE_UNLOCK_UNLOCK_KEY_Pos 0
#define FLASH_NVR_WRITE_UNLOCK_UNLOCK_KEY_Mask ((uint32_t)(0xFFFFFFFFU << FLASH_NVR_WRITE_UNLOCK_UNLOCK_KEY_Pos))

/* FLASH_NVR_WRITE_UNLOCK settings */
#define FLASH_NVR_KEY                   ((uint32_t)(0x71B371F5U << FLASH_NVR_WRITE_UNLOCK_UNLOCK_KEY_Pos))

/* FLASH_NVR_CTRL bit positions */
#define FLASH_NVR_CTRL_NVR3_W_EN_Pos    3
#define FLASH_NVR_CTRL_NVR2_W_EN_Pos    2
#define FLASH_NVR_CTRL_NVR1_W_EN_Pos    1

/* FLASH_NVR_CTRL bit-band aliases */
typedef struct
{
         uint32_t RESERVED0;
    __IO uint32_t NVR1_W_EN_ALIAS;      /* Authorize Write access to the Flash NVR1 sector through the FLASH_IF registers. */
    __IO uint32_t NVR2_W_EN_ALIAS;      /* Authorize Write access to the Flash NVR2 sector through the FLASH_IF registers. */
    __IO uint32_t NVR3_W_EN_ALIAS;      /* Authorize Write access to the Flash NVR3 sector through the FLASH_IF registers. */
} FLASH_NVR_CTRL_Type;

#define FLASH_NVR_CTRL_BASE             SYS_CALC_BITBAND(0x4000054C, 0)
#define FLASH_NVR_CTRL                  ((FLASH_NVR_CTRL_Type *) FLASH_NVR_CTRL_BASE)

/* FLASH_NVR_CTRL settings */
#define NVR1_WRITE_DISABLE_BITBAND      0x0
#define NVR1_WRITE_ENABLE_BITBAND       0x1
#define NVR1_WRITE_DISABLE              ((uint32_t)(NVR1_WRITE_DISABLE_BITBAND << FLASH_NVR_CTRL_NVR1_W_EN_Pos))
#define NVR1_WRITE_ENABLE               ((uint32_t)(NVR1_WRITE_ENABLE_BITBAND << FLASH_NVR_CTRL_NVR1_W_EN_Pos))

#define NVR2_WRITE_DISABLE_BITBAND      0x0
#define NVR2_WRITE_ENABLE_BITBAND       0x1
#define NVR2_WRITE_DISABLE              ((uint32_t)(NVR2_WRITE_DISABLE_BITBAND << FLASH_NVR_CTRL_NVR2_W_EN_Pos))
#define NVR2_WRITE_ENABLE               ((uint32_t)(NVR2_WRITE_ENABLE_BITBAND << FLASH_NVR_CTRL_NVR2_W_EN_Pos))

#define NVR3_WRITE_DISABLE_BITBAND      0x0
#define NVR3_WRITE_ENABLE_BITBAND       0x1
#define NVR3_WRITE_DISABLE              ((uint32_t)(NVR3_WRITE_DISABLE_BITBAND << FLASH_NVR_CTRL_NVR3_W_EN_Pos))
#define NVR3_WRITE_ENABLE               ((uint32_t)(NVR3_WRITE_ENABLE_BITBAND << FLASH_NVR_CTRL_NVR3_W_EN_Pos))

/* FLASH_PATCH_ADDR[] bit positions */
#define FLASH_PATCH_ADDR_PATCH_ADDR_Pos 11
#define FLASH_PATCH_ADDR_PATCH_ADDR_Mask ((uint32_t)(0x3FFU << FLASH_PATCH_ADDR_PATCH_ADDR_Pos))

/* FLASH_COPY_CFG bit positions */
#define FLASH_COPY_CFG_COMP_ADDR_STEP_Pos 18
#define FLASH_COPY_CFG_COMP_ADDR_DIR_Pos 17
#define FLASH_COPY_CFG_COMP_MODE_Pos    16
#define FLASH_COPY_CFG_COPY_DEST_Pos    9
#define FLASH_COPY_CFG_COPY_MODE_Pos    8
#define FLASH_COPY_CFG_MODE_Pos         0

/* FLASH_COPY_CFG sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t MODE_BYTE;            
    __IO uint8_t COPY_BYTE;            
    __IO uint8_t COMP_BYTE;            
         uint8_t RESERVED0[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000580, 0) - (0x40000580 + 4)];
    __IO uint32_t MODE_ALIAS;           /* Copier or Comparator Mode Configuration */
         uint32_t RESERVED1[7];
    __IO uint32_t COPY_MODE_ALIAS;      /* Select copier mode (32-bit or 40-bit) */
    __IO uint32_t COPY_DEST_ALIAS;      /* Destination copier is the CRC or memories */
         uint32_t RESERVED2[6];
    __IO uint32_t COMP_MODE_ALIAS;      /* Comparator Mode */
    __IO uint32_t COMP_ADDR_DIR_ALIAS;  /* Comparator address-up or address-down */
    __IO uint32_t COMP_ADDR_STEP_ALIAS; /* Comparator address increment/decrement by 1 or 2 */
} FLASH_COPY_CFG_Type;

#define FLASH_COPY_CFG_BASE             0x40000580
#define FLASH_COPY_CFG                  ((FLASH_COPY_CFG_Type *) FLASH_COPY_CFG_BASE)

/* FLASH_COPY_CFG settings */
#define COPY_MODE_BITBAND               0x0
#define COMPARATOR_MODE_BITBAND         0x1
#define COPY_MODE                       ((uint32_t)(COPY_MODE_BITBAND << FLASH_COPY_CFG_MODE_Pos))
#define COMPARATOR_MODE                 ((uint32_t)(COMPARATOR_MODE_BITBAND << FLASH_COPY_CFG_MODE_Pos))

#define COPY_TO_32BIT_BITBAND           0x0
#define COPY_TO_40BIT_BITBAND           0x1
#define COPY_TO_32BIT                   ((uint32_t)(COPY_TO_32BIT_BITBAND << FLASH_COPY_CFG_COPY_MODE_Pos))
#define COPY_TO_40BIT                   ((uint32_t)(COPY_TO_40BIT_BITBAND << FLASH_COPY_CFG_COPY_MODE_Pos))

#define COPY_TO_MEM_BITBAND             0x0
#define COPY_TO_CRC_BITBAND             0x1
#define COPY_TO_MEM                     ((uint32_t)(COPY_TO_MEM_BITBAND << FLASH_COPY_CFG_COPY_DEST_Pos))
#define COPY_TO_CRC                     ((uint32_t)(COPY_TO_CRC_BITBAND << FLASH_COPY_CFG_COPY_DEST_Pos))

#define COMP_MODE_CONSTANT_BITBAND      0x0
#define COMP_MODE_CHBK_BITBAND          0x1
#define COMP_MODE_CONSTANT              ((uint32_t)(COMP_MODE_CONSTANT_BITBAND << FLASH_COPY_CFG_COMP_MODE_Pos))
#define COMP_MODE_CHBK                  ((uint32_t)(COMP_MODE_CHBK_BITBAND << FLASH_COPY_CFG_COMP_MODE_Pos))

#define COMP_ADDR_DOWN_BITBAND          0x0
#define COMP_ADDR_UP_BITBAND            0x1
#define COMP_ADDR_DOWN                  ((uint32_t)(COMP_ADDR_DOWN_BITBAND << FLASH_COPY_CFG_COMP_ADDR_DIR_Pos))
#define COMP_ADDR_UP                    ((uint32_t)(COMP_ADDR_UP_BITBAND << FLASH_COPY_CFG_COMP_ADDR_DIR_Pos))

#define COMP_ADDR_STEP_1_BITBAND        0x0
#define COMP_ADDR_STEP_2_BITBAND        0x1
#define COMP_ADDR_STEP_1                ((uint32_t)(COMP_ADDR_STEP_1_BITBAND << FLASH_COPY_CFG_COMP_ADDR_STEP_Pos))
#define COMP_ADDR_STEP_2                ((uint32_t)(COMP_ADDR_STEP_2_BITBAND << FLASH_COPY_CFG_COMP_ADDR_STEP_Pos))

/* FLASH_COPY_CFG sub-register bit positions */
#define FLASH_COPY_CFG_MODE_BYTE_Pos    0
#define FLASH_COPY_CFG_COPY_MODE_BYTE_Pos 0
#define FLASH_COPY_CFG_COPY_DEST_BYTE_Pos 1
#define FLASH_COPY_CFG_COMP_MODE_BYTE_Pos 0
#define FLASH_COPY_CFG_COMP_ADDR_DIR_BYTE_Pos 1
#define FLASH_COPY_CFG_COMP_ADDR_STEP_BYTE_Pos 2

/* FLASH_COPY_CFG sub-register settings */
#define COPY_MODE_BYTE                  ((uint8_t)(COPY_MODE_BITBAND << FLASH_COPY_CFG_MODE_BYTE_Pos))
#define COMPARATOR_MODE_BYTE            ((uint8_t)(COMPARATOR_MODE_BITBAND << FLASH_COPY_CFG_MODE_BYTE_Pos))

#define COPY_TO_32BIT_BYTE              ((uint8_t)(COPY_TO_32BIT_BITBAND << FLASH_COPY_CFG_COPY_MODE_BYTE_Pos))
#define COPY_TO_40BIT_BYTE              ((uint8_t)(COPY_TO_40BIT_BITBAND << FLASH_COPY_CFG_COPY_MODE_BYTE_Pos))

#define COPY_TO_MEM_BYTE                ((uint8_t)(COPY_TO_MEM_BITBAND << FLASH_COPY_CFG_COPY_DEST_BYTE_Pos))
#define COPY_TO_CRC_BYTE                ((uint8_t)(COPY_TO_CRC_BITBAND << FLASH_COPY_CFG_COPY_DEST_BYTE_Pos))

#define COMP_MODE_CONSTANT_BYTE         ((uint8_t)(COMP_MODE_CONSTANT_BITBAND << FLASH_COPY_CFG_COMP_MODE_BYTE_Pos))
#define COMP_MODE_CHBK_BYTE             ((uint8_t)(COMP_MODE_CHBK_BITBAND << FLASH_COPY_CFG_COMP_MODE_BYTE_Pos))

#define COMP_ADDR_DOWN_BYTE             ((uint8_t)(COMP_ADDR_DOWN_BITBAND << FLASH_COPY_CFG_COMP_ADDR_DIR_BYTE_Pos))
#define COMP_ADDR_UP_BYTE               ((uint8_t)(COMP_ADDR_UP_BITBAND << FLASH_COPY_CFG_COMP_ADDR_DIR_BYTE_Pos))

#define COMP_ADDR_STEP_1_BYTE           ((uint8_t)(COMP_ADDR_STEP_1_BITBAND << FLASH_COPY_CFG_COMP_ADDR_STEP_BYTE_Pos))
#define COMP_ADDR_STEP_2_BYTE           ((uint8_t)(COMP_ADDR_STEP_2_BITBAND << FLASH_COPY_CFG_COMP_ADDR_STEP_BYTE_Pos))

/* FLASH_COPY_CTRL bit positions */
#define FLASH_COPY_CTRL_ERROR_Pos       3
#define FLASH_COPY_CTRL_STOP_Pos        2
#define FLASH_COPY_CTRL_START_Pos       1
#define FLASH_COPY_CTRL_BUSY_Pos        0

/* FLASH_COPY_CTRL bit-band aliases */
typedef struct
{
    __I  uint32_t BUSY_ALIAS;           /* Busy status */
    __O  uint32_t START_ALIAS;          /* Start the transfer */
    __O  uint32_t STOP_ALIAS;           /* Stop the transfer */
    __I  uint32_t ERROR_ALIAS;          /* Error status */
} FLASH_COPY_CTRL_Type;

#define FLASH_COPY_CTRL_BASE            SYS_CALC_BITBAND(0x400005C8, 0)
#define FLASH_COPY_CTRL                 ((FLASH_COPY_CTRL_Type *) FLASH_COPY_CTRL_BASE)

/* FLASH_COPY_CTRL settings */
#define COPY_IDLE_BITBAND               0x0
#define COPY_BUSY_BITBAND               0x1
#define COPY_IDLE                       ((uint32_t)(COPY_IDLE_BITBAND << FLASH_COPY_CTRL_BUSY_Pos))
#define COPY_BUSY                       ((uint32_t)(COPY_BUSY_BITBAND << FLASH_COPY_CTRL_BUSY_Pos))

#define COPY_START_BITBAND              0x1
#define COPY_START                      ((uint32_t)(COPY_START_BITBAND << FLASH_COPY_CTRL_START_Pos))

#define COPY_STOP_BITBAND               0x1
#define COPY_STOP                       ((uint32_t)(COPY_STOP_BITBAND << FLASH_COPY_CTRL_STOP_Pos))

#define COPY_NO_ERROR_BITBAND           0x0
#define COPY_ERROR_BITBAND              0x1
#define COPY_NO_ERROR                   ((uint32_t)(COPY_NO_ERROR_BITBAND << FLASH_COPY_CTRL_ERROR_Pos))
#define COPY_ERROR                      ((uint32_t)(COPY_ERROR_BITBAND << FLASH_COPY_CTRL_ERROR_Pos))

/* FLASH_COPY_SRC_ADDR_PTR bit positions */
#define FLASH_COPY_SRC_ADDR_PTR_COPY_SRC_ADDR_PTR_Pos 0
#define FLASH_COPY_SRC_ADDR_PTR_COPY_SRC_ADDR_PTR_Mask ((uint32_t)(0x1FFFFFU << FLASH_COPY_SRC_ADDR_PTR_COPY_SRC_ADDR_PTR_Pos))

/* FLASH_COPY_DST_ADDR_PTR bit positions */
#define FLASH_COPY_DST_ADDR_PTR_COPY_DST_ADDR_PTR_Pos 2
#define FLASH_COPY_DST_ADDR_PTR_COPY_DST_ADDR_PTR_Mask ((uint32_t)(0x3FFFFFFFU << FLASH_COPY_DST_ADDR_PTR_COPY_DST_ADDR_PTR_Pos))

/* FLASH_COPY_WORD_CNT bit positions */
#define FLASH_COPY_WORD_CNT_COPY_WORD_CNT_Pos 0
#define FLASH_COPY_WORD_CNT_COPY_WORD_CNT_Mask ((uint32_t)(0x1FFFFU << FLASH_COPY_WORD_CNT_COPY_WORD_CNT_Pos))

/* FLASH_ECC_CTRL bit positions */
#define FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_Pos 8
#define FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_Mask ((uint32_t)(0xFFU << FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_Pos))
#define FLASH_ECC_CTRL_COPIER_ECC_CTRL_Pos 3
#define FLASH_ECC_CTRL_CMD_ECC_CTRL_Pos 2
#define FLASH_ECC_CTRL_IDBUS_ECC_CTRL_Pos 0

/* FLASH_ECC_CTRL sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t ECC_COR_CNT_INT_THRESHOLD_BYTE;
         uint8_t RESERVED1[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x400005DC, 0) - (0x400005DC + 4)];
    __IO uint32_t IDBUS_ECC_CTRL_ALIAS; /* Select the operating mode of the Flash ECC */
         uint32_t RESERVED2;
    __IO uint32_t CMD_ECC_CTRL_ALIAS;   
    __IO uint32_t COPIER_ECC_CTRL_ALIAS;
} FLASH_ECC_CTRL_Type;

#define FLASH_ECC_CTRL_BASE             0x400005DC
#define FLASH_ECC_CTRL                  ((FLASH_ECC_CTRL_Type *) FLASH_ECC_CTRL_BASE)

/* FLASH_ECC_CTRL settings */
#define FLASH_IDBUS_ECC_DISABLE_BITBAND 0x0
#define FLASH_IDBUS_ECC_ENABLE_BITBAND  0x1
#define FLASH_IDBUS_ECC_DISABLE         ((uint32_t)(FLASH_IDBUS_ECC_DISABLE_BITBAND << FLASH_ECC_CTRL_IDBUS_ECC_CTRL_Pos))
#define FLASH_IDBUS_ECC_ENABLE          ((uint32_t)(FLASH_IDBUS_ECC_ENABLE_BITBAND << FLASH_ECC_CTRL_IDBUS_ECC_CTRL_Pos))

#define FLASH_CMD_ECC_DISABLE_BITBAND   0x0
#define FLASH_CMD_ECC_ENABLE_BITBAND    0x1
#define FLASH_CMD_ECC_DISABLE           ((uint32_t)(FLASH_CMD_ECC_DISABLE_BITBAND << FLASH_ECC_CTRL_CMD_ECC_CTRL_Pos))
#define FLASH_CMD_ECC_ENABLE            ((uint32_t)(FLASH_CMD_ECC_ENABLE_BITBAND << FLASH_ECC_CTRL_CMD_ECC_CTRL_Pos))

#define FLASH_COPIER_ECC_DISABLE_BITBAND 0x0
#define FLASH_COPIER_ECC_ENABLE_BITBAND 0x1
#define FLASH_COPIER_ECC_DISABLE        ((uint32_t)(FLASH_COPIER_ECC_DISABLE_BITBAND << FLASH_ECC_CTRL_COPIER_ECC_CTRL_Pos))
#define FLASH_COPIER_ECC_ENABLE         ((uint32_t)(FLASH_COPIER_ECC_ENABLE_BITBAND << FLASH_ECC_CTRL_COPIER_ECC_CTRL_Pos))

#define FLASH_ECC_COR_INT_THRESHOLD_DISABLED ((uint32_t)(0x0U << FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_Pos))
#define FLASH_ECC_COR_INT_THRESHOLD_1   ((uint32_t)(0x1U << FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_Pos))
#define FLASH_ECC_COR_INT_THRESHOLD_255 ((uint32_t)(0xFFU << FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_Pos))

/* FLASH_ECC_CTRL sub-register settings */
#define FLASH_ECC_COR_INT_THRESHOLD_DISABLED_BYTE ((uint8_t)0x0U)
#define FLASH_ECC_COR_INT_THRESHOLD_1_BYTE ((uint8_t)0x1U)
#define FLASH_ECC_COR_INT_THRESHOLD_255_BYTE ((uint8_t)0xFFU)

/* FLASH_ECC_STATUS bit positions */
#define FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_CLEAR_Pos 6
#define FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_CLEAR_Pos 5
#define FLASH_ECC_STATUS_ECC_ERROR_ADDR_CLEAR_Pos 4
#define FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_STATUS_Pos 1
#define FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_STATUS_Pos 0

/* FLASH_ECC_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t ECC_UNCOR_ERROR_CNT_STATUS_ALIAS;/* FLASH_ECC_ERROR_UNCOR_CNT status */
    __I  uint32_t ECC_COR_ERROR_CNT_STATUS_ALIAS;/* FLASH_ECC_ERROR_COR_CNT status */
         uint32_t RESERVED0[2];
    __O  uint32_t ECC_ERROR_ADDR_CLEAR_ALIAS;/* Reset the Flash address of the last detected error */
    __O  uint32_t ECC_UNCOR_ERROR_CNT_CLEAR_ALIAS;/* Reset the Flash uncorrected errors counter */
    __O  uint32_t ECC_COR_ERROR_CNT_CLEAR_ALIAS;/* Reset the Flash corrected errors counter */
} FLASH_ECC_STATUS_Type;

#define FLASH_ECC_STATUS_BASE           SYS_CALC_BITBAND(0x400005E0, 0)
#define FLASH_ECC_STATUS                ((FLASH_ECC_STATUS_Type *) FLASH_ECC_STATUS_BASE)

/* FLASH_ECC_STATUS settings */
#define FLASH_ECC_NO_UNCORRECTED_ERROR_BITBAND 0x0
#define FLASH_ECC_UNCORRECTED_ERROR_BITBAND 0x1
#define FLASH_ECC_NO_UNCORRECTED_ERROR  ((uint32_t)(FLASH_ECC_NO_UNCORRECTED_ERROR_BITBAND << FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_STATUS_Pos))
#define FLASH_ECC_UNCORRECTED_ERROR     ((uint32_t)(FLASH_ECC_UNCORRECTED_ERROR_BITBAND << FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_STATUS_Pos))

#define FLASH_ECC_NO_CORRECTED_ERROR_BITBAND 0x0
#define FLASH_ECC_CORRECTED_ERROR_BITBAND 0x1
#define FLASH_ECC_NO_CORRECTED_ERROR    ((uint32_t)(FLASH_ECC_NO_CORRECTED_ERROR_BITBAND << FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_STATUS_Pos))
#define FLASH_ECC_CORRECTED_ERROR       ((uint32_t)(FLASH_ECC_CORRECTED_ERROR_BITBAND << FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_STATUS_Pos))

#define FLASH_ECC_ERROR_ADDR_CLEAR_BITBAND 0x1
#define FLASH_ECC_ERROR_ADDR_CLEAR      ((uint32_t)(FLASH_ECC_ERROR_ADDR_CLEAR_BITBAND << FLASH_ECC_STATUS_ECC_ERROR_ADDR_CLEAR_Pos))

#define FLASH_ECC_UNCOR_ERROR_CNT_CLEAR_BITBAND 0x1
#define FLASH_ECC_UNCOR_ERROR_CNT_CLEAR ((uint32_t)(FLASH_ECC_UNCOR_ERROR_CNT_CLEAR_BITBAND << FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_CLEAR_Pos))

#define FLASH_ECC_COR_ERROR_CNT_CLEAR_BITBAND 0x1
#define FLASH_ECC_COR_ERROR_CNT_CLEAR   ((uint32_t)(FLASH_ECC_COR_ERROR_CNT_CLEAR_BITBAND << FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_CLEAR_Pos))

/* FLASH_ECC_ERROR_ADDR bit positions */
#define FLASH_ECC_ERROR_ADDR_ECC_ERROR_ADDR_Pos 2
#define FLASH_ECC_ERROR_ADDR_ECC_ERROR_ADDR_Mask ((uint32_t)(0x7FFFFU << FLASH_ECC_ERROR_ADDR_ECC_ERROR_ADDR_Pos))

/* FLASH_ECC_UNCOR_ERROR_CNT bit positions */
#define FLASH_ECC_UNCOR_ERROR_CNT_ECC_UNCOR_ERROR_CNT_Pos 0
#define FLASH_ECC_UNCOR_ERROR_CNT_ECC_UNCOR_ERROR_CNT_Mask ((uint32_t)(0xFFU << FLASH_ECC_UNCOR_ERROR_CNT_ECC_UNCOR_ERROR_CNT_Pos))

/* FLASH_ECC_COR_ERROR_CNT bit positions */
#define FLASH_ECC_COR_ERROR_CNT_ECC_COR_ERROR_CNT_Pos 0
#define FLASH_ECC_COR_ERROR_CNT_ECC_COR_ERROR_CNT_Mask ((uint32_t)(0xFFU << FLASH_ECC_COR_ERROR_CNT_ECC_COR_ERROR_CNT_Pos))

/* ----------------------------------------------------------------------------
 * DMA Controller Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL0[8];             /* DMA Channel Control and Configuration */
    __IO uint32_t SRC_BASE_ADDR[8];     /* DMA Channel Source Base Address */
    __IO uint32_t DEST_BASE_ADDR[8];    /* DMA Channel Destination Base Address */
    __IO uint32_t CTRL1[8];             /* DMA Channel Transfer Control */
    __I  uint32_t NEXT_SRC_ADDR[8];     /* DMA Channel Next Source Address */
    __I  uint32_t NEXT_DEST_ADDR[8];    /* DMA Channel Next Destination Address */
    __I  uint32_t WORD_CNT[8];          /* DMA Channel Word Count */
    __IO uint32_t STATUS[8];            /* DMA Status channel */
} DMA_Type;

#define DMA_BASE                        0x40000600
#define DMA                             ((DMA_Type *) DMA_BASE)

/* DMA_CTRL0[] bit positions */
#define DMA_CTRL0_DEST_ADDR_STEP_SIZE_Pos 30
#define DMA_CTRL0_DEST_ADDR_STEP_SIZE_Mask ((uint32_t)(0x3U << DMA_CTRL0_DEST_ADDR_STEP_SIZE_Pos))
#define DMA_CTRL0_SRC_ADDR_STEP_SIZE_Pos 28
#define DMA_CTRL0_SRC_ADDR_STEP_SIZE_Mask ((uint32_t)(0x3U << DMA_CTRL0_SRC_ADDR_STEP_SIZE_Pos))
#define DMA_CTRL0_DEST_ADDR_STEP_MODE_Pos 27
#define DMA_CTRL0_SRC_ADDR_STEP_MODE_Pos 26
#define DMA_CTRL0_BYTE_ORDER_Pos        25
#define DMA_CTRL0_DISABLE_INT_ENABLE_Pos 24
#define DMA_CTRL0_ERROR_INT_ENABLE_Pos  23
#define DMA_CTRL0_COMPLETE_INT_ENABLE_Pos 22
#define DMA_CTRL0_COUNTER_INT_ENABLE_Pos 21
#define DMA_CTRL0_START_INT_ENABLE_Pos  20
#define DMA_CTRL0_DEST_WORD_SIZE_Pos    18
#define DMA_CTRL0_DEST_WORD_SIZE_Mask   ((uint32_t)(0x3U << DMA_CTRL0_DEST_WORD_SIZE_Pos))
#define DMA_CTRL0_SRC_WORD_SIZE_Pos     16
#define DMA_CTRL0_SRC_WORD_SIZE_Mask    ((uint32_t)(0x3U << DMA_CTRL0_SRC_WORD_SIZE_Pos))
#define DMA_CTRL0_DEST_SELECT_Pos       12
#define DMA_CTRL0_DEST_SELECT_Mask      ((uint32_t)(0x7U << DMA_CTRL0_DEST_SELECT_Pos))
#define DMA_CTRL0_SRC_SELECT_Pos        9
#define DMA_CTRL0_SRC_SELECT_Mask       ((uint32_t)(0x7U << DMA_CTRL0_SRC_SELECT_Pos))
#define DMA_CTRL0_CHANNEL_PRIORITY_Pos  6
#define DMA_CTRL0_CHANNEL_PRIORITY_Mask ((uint32_t)(0x3U << DMA_CTRL0_CHANNEL_PRIORITY_Pos))
#define DMA_CTRL0_TRANSFER_TYPE_Pos     4
#define DMA_CTRL0_TRANSFER_TYPE_Mask    ((uint32_t)(0x3U << DMA_CTRL0_TRANSFER_TYPE_Pos))
#define DMA_CTRL0_DEST_ADDR_INC_Pos     3
#define DMA_CTRL0_SRC_ADDR_INC_Pos      2
#define DMA_CTRL0_ADDR_MODE_Pos         1
#define DMA_CTRL0_ENABLE_Pos            0

/* DMA_CTRL0[] bit-band aliases */
typedef struct
{
    __IO uint32_t ENABLE_ALIAS;         /* Enable DMA Channel */
    __IO uint32_t ADDR_MODE_ALIAS;      /* Select the addressing mode for this channel */
    __IO uint32_t SRC_ADDR_INC_ALIAS;   /* Configure whether the source address should increment */
    __IO uint32_t DEST_ADDR_INC_ALIAS;  /* Configure whether the destination address should increment */
         uint32_t RESERVED0[16];
    __IO uint32_t START_INT_ENABLE_ALIAS;/* Raise an interrupt when the DMA transfer starts */
    __IO uint32_t COUNTER_INT_ENABLE_ALIAS;/* Raise an interrupt when the DMA transfer reaches the counter value */
    __IO uint32_t COMPLETE_INT_ENABLE_ALIAS;/* Raise an interrupt when the DMA transfer completes */
    __IO uint32_t ERROR_INT_ENABLE_ALIAS;/* Raise an interrupt when a state machine error occurs during a DMA transfer */
    __IO uint32_t DISABLE_INT_ENABLE_ALIAS;/* Raise an interrupt when the DMA channel is disabled */
    __IO uint32_t BYTE_ORDER_ALIAS;     /* Select the byte ordering for the DMA channel */
    __IO uint32_t SRC_ADDR_STEP_MODE_ALIAS;/* Configure the source address to either increment or decrement */
    __IO uint32_t DEST_ADDR_STEP_MODE_ALIAS;/* Configure the destination address to either increment or decrement */
         uint32_t RESERVED_SPAN[4];
} DMA_CTRL0_Type;

#define DMA_CTRL0_BASE                  SYS_CALC_BITBAND(0x40000600, 0)
#define DMA_CTRL0                       ((DMA_CTRL0_Type *) DMA_CTRL0_BASE)

/* DMA_CTRL0 settings */
#define DMA_DISABLE_BITBAND             0x0
#define DMA_ENABLE_BITBAND              0x1
#define DMA_DISABLE                     ((uint32_t)(DMA_DISABLE_BITBAND << DMA_CTRL0_ENABLE_Pos))
#define DMA_ENABLE                      ((uint32_t)(DMA_ENABLE_BITBAND << DMA_CTRL0_ENABLE_Pos))

#define DMA_ADDR_CIRC_BITBAND           0x0
#define DMA_ADDR_LIN_BITBAND            0x1
#define DMA_ADDR_CIRC                   ((uint32_t)(DMA_ADDR_CIRC_BITBAND << DMA_CTRL0_ADDR_MODE_Pos))
#define DMA_ADDR_LIN                    ((uint32_t)(DMA_ADDR_LIN_BITBAND << DMA_CTRL0_ADDR_MODE_Pos))

#define DMA_SRC_ADDR_STATIC_BITBAND     0x0
#define DMA_SRC_ADDR_INC_BITBAND        0x1
#define DMA_SRC_ADDR_STATIC             ((uint32_t)(DMA_SRC_ADDR_STATIC_BITBAND << DMA_CTRL0_SRC_ADDR_INC_Pos))
#define DMA_SRC_ADDR_INC                ((uint32_t)(DMA_SRC_ADDR_INC_BITBAND << DMA_CTRL0_SRC_ADDR_INC_Pos))

#define DMA_DEST_ADDR_STATIC_BITBAND    0x0
#define DMA_DEST_ADDR_INC_BITBAND       0x1
#define DMA_DEST_ADDR_STATIC            ((uint32_t)(DMA_DEST_ADDR_STATIC_BITBAND << DMA_CTRL0_DEST_ADDR_INC_Pos))
#define DMA_DEST_ADDR_INC               ((uint32_t)(DMA_DEST_ADDR_INC_BITBAND << DMA_CTRL0_DEST_ADDR_INC_Pos))

#define DMA_TRANSFER_M_TO_M             ((uint32_t)(0x0U << DMA_CTRL0_TRANSFER_TYPE_Pos))
#define DMA_TRANSFER_M_TO_P             ((uint32_t)(0x1U << DMA_CTRL0_TRANSFER_TYPE_Pos))
#define DMA_TRANSFER_P_TO_M             ((uint32_t)(0x2U << DMA_CTRL0_TRANSFER_TYPE_Pos))
#define DMA_TRANSFER_P_TO_P             ((uint32_t)(0x3U << DMA_CTRL0_TRANSFER_TYPE_Pos))

#define DMA_PRIORITY_0                  ((uint32_t)(0x0U << DMA_CTRL0_CHANNEL_PRIORITY_Pos))
#define DMA_PRIORITY_1                  ((uint32_t)(0x1U << DMA_CTRL0_CHANNEL_PRIORITY_Pos))
#define DMA_PRIORITY_2                  ((uint32_t)(0x2U << DMA_CTRL0_CHANNEL_PRIORITY_Pos))
#define DMA_PRIORITY_3                  ((uint32_t)(0x3U << DMA_CTRL0_CHANNEL_PRIORITY_Pos))

#define DMA_SRC_I2C                     ((uint32_t)(0x0U << DMA_CTRL0_SRC_SELECT_Pos))
#define DMA_SRC_SPI0                    ((uint32_t)(0x1U << DMA_CTRL0_SRC_SELECT_Pos))
#define DMA_SRC_SPI1                    ((uint32_t)(0x2U << DMA_CTRL0_SRC_SELECT_Pos))
#define DMA_SRC_PCM                     ((uint32_t)(0x3U << DMA_CTRL0_SRC_SELECT_Pos))
#define DMA_SRC_UART                    ((uint32_t)(0x4U << DMA_CTRL0_SRC_SELECT_Pos))
#define DMA_SRC_ASRC                    ((uint32_t)(0x5U << DMA_CTRL0_SRC_SELECT_Pos))
#define DMA_SRC_PBUS                    ((uint32_t)(0x6U << DMA_CTRL0_SRC_SELECT_Pos))
#define DMA_SRC_DMIC                    ((uint32_t)(0x7U << DMA_CTRL0_SRC_SELECT_Pos))

#define DMA_DEST_I2C                    ((uint32_t)(0x0U << DMA_CTRL0_DEST_SELECT_Pos))
#define DMA_DEST_SPI0                   ((uint32_t)(0x1U << DMA_CTRL0_DEST_SELECT_Pos))
#define DMA_DEST_SPI1                   ((uint32_t)(0x2U << DMA_CTRL0_DEST_SELECT_Pos))
#define DMA_DEST_PCM                    ((uint32_t)(0x3U << DMA_CTRL0_DEST_SELECT_Pos))
#define DMA_DEST_UART                   ((uint32_t)(0x4U << DMA_CTRL0_DEST_SELECT_Pos))
#define DMA_DEST_ASRC                   ((uint32_t)(0x5U << DMA_CTRL0_DEST_SELECT_Pos))
#define DMA_DEST_PBUS                   ((uint32_t)(0x6U << DMA_CTRL0_DEST_SELECT_Pos))
#define DMA_DEST_OD                     ((uint32_t)(0x7U << DMA_CTRL0_DEST_SELECT_Pos))

#define DMA_SRC_WORD_SIZE_8             ((uint32_t)(0x0U << DMA_CTRL0_SRC_WORD_SIZE_Pos))
#define DMA_SRC_WORD_SIZE_16            ((uint32_t)(0x1U << DMA_CTRL0_SRC_WORD_SIZE_Pos))
#define DMA_SRC_WORD_SIZE_32            ((uint32_t)(0x2U << DMA_CTRL0_SRC_WORD_SIZE_Pos))
#define DMA_SRC_WORD_SIZE_4             ((uint32_t)(0x3U << DMA_CTRL0_SRC_WORD_SIZE_Pos))

#define DMA_DEST_WORD_SIZE_8            ((uint32_t)(0x0U << DMA_CTRL0_DEST_WORD_SIZE_Pos))
#define DMA_DEST_WORD_SIZE_16           ((uint32_t)(0x1U << DMA_CTRL0_DEST_WORD_SIZE_Pos))
#define DMA_DEST_WORD_SIZE_32           ((uint32_t)(0x2U << DMA_CTRL0_DEST_WORD_SIZE_Pos))
#define DMA_DEST_WORD_SIZE_4            ((uint32_t)(0x3U << DMA_CTRL0_DEST_WORD_SIZE_Pos))

#define DMA_START_INT_DISABLE_BITBAND   0x0
#define DMA_START_INT_ENABLE_BITBAND    0x1
#define DMA_START_INT_DISABLE           ((uint32_t)(DMA_START_INT_DISABLE_BITBAND << DMA_CTRL0_START_INT_ENABLE_Pos))
#define DMA_START_INT_ENABLE            ((uint32_t)(DMA_START_INT_ENABLE_BITBAND << DMA_CTRL0_START_INT_ENABLE_Pos))

#define DMA_COUNTER_INT_DISABLE_BITBAND 0x0
#define DMA_COUNTER_INT_ENABLE_BITBAND  0x1
#define DMA_COUNTER_INT_DISABLE         ((uint32_t)(DMA_COUNTER_INT_DISABLE_BITBAND << DMA_CTRL0_COUNTER_INT_ENABLE_Pos))
#define DMA_COUNTER_INT_ENABLE          ((uint32_t)(DMA_COUNTER_INT_ENABLE_BITBAND << DMA_CTRL0_COUNTER_INT_ENABLE_Pos))

#define DMA_COMPLETE_INT_DISABLE_BITBAND 0x0
#define DMA_COMPLETE_INT_ENABLE_BITBAND 0x1
#define DMA_COMPLETE_INT_DISABLE        ((uint32_t)(DMA_COMPLETE_INT_DISABLE_BITBAND << DMA_CTRL0_COMPLETE_INT_ENABLE_Pos))
#define DMA_COMPLETE_INT_ENABLE         ((uint32_t)(DMA_COMPLETE_INT_ENABLE_BITBAND << DMA_CTRL0_COMPLETE_INT_ENABLE_Pos))

#define DMA_ERROR_INT_DISABLE_BITBAND   0x0
#define DMA_ERROR_INT_ENABLE_BITBAND    0x1
#define DMA_ERROR_INT_DISABLE           ((uint32_t)(DMA_ERROR_INT_DISABLE_BITBAND << DMA_CTRL0_ERROR_INT_ENABLE_Pos))
#define DMA_ERROR_INT_ENABLE            ((uint32_t)(DMA_ERROR_INT_ENABLE_BITBAND << DMA_CTRL0_ERROR_INT_ENABLE_Pos))

#define DMA_DISABLE_INT_DISABLE_BITBAND 0x0
#define DMA_DISABLE_INT_ENABLE_BITBAND  0x1
#define DMA_DISABLE_INT_DISABLE         ((uint32_t)(DMA_DISABLE_INT_DISABLE_BITBAND << DMA_CTRL0_DISABLE_INT_ENABLE_Pos))
#define DMA_DISABLE_INT_ENABLE          ((uint32_t)(DMA_DISABLE_INT_ENABLE_BITBAND << DMA_CTRL0_DISABLE_INT_ENABLE_Pos))

#define DMA_LITTLE_ENDIAN_BITBAND       0x0
#define DMA_BIG_ENDIAN_BITBAND          0x1
#define DMA_LITTLE_ENDIAN               ((uint32_t)(DMA_LITTLE_ENDIAN_BITBAND << DMA_CTRL0_BYTE_ORDER_Pos))
#define DMA_BIG_ENDIAN                  ((uint32_t)(DMA_BIG_ENDIAN_BITBAND << DMA_CTRL0_BYTE_ORDER_Pos))

#define DMA_SRC_ADDR_POS_BITBAND        0x0
#define DMA_SRC_ADDR_NEG_BITBAND        0x1
#define DMA_SRC_ADDR_POS                ((uint32_t)(DMA_SRC_ADDR_POS_BITBAND << DMA_CTRL0_SRC_ADDR_STEP_MODE_Pos))
#define DMA_SRC_ADDR_NEG                ((uint32_t)(DMA_SRC_ADDR_NEG_BITBAND << DMA_CTRL0_SRC_ADDR_STEP_MODE_Pos))

#define DMA_DEST_ADDR_POS_BITBAND       0x0
#define DMA_DEST_ADDR_NEG_BITBAND       0x1
#define DMA_DEST_ADDR_POS               ((uint32_t)(DMA_DEST_ADDR_POS_BITBAND << DMA_CTRL0_DEST_ADDR_STEP_MODE_Pos))
#define DMA_DEST_ADDR_NEG               ((uint32_t)(DMA_DEST_ADDR_NEG_BITBAND << DMA_CTRL0_DEST_ADDR_STEP_MODE_Pos))

#define DMA_SRC_ADDR_STEP_SIZE_1        ((uint32_t)(0x0U << DMA_CTRL0_SRC_ADDR_STEP_SIZE_Pos))
#define DMA_SRC_ADDR_STEP_SIZE_2        ((uint32_t)(0x1U << DMA_CTRL0_SRC_ADDR_STEP_SIZE_Pos))
#define DMA_SRC_ADDR_STEP_SIZE_3        ((uint32_t)(0x2U << DMA_CTRL0_SRC_ADDR_STEP_SIZE_Pos))
#define DMA_SRC_ADDR_STEP_SIZE_4        ((uint32_t)(0x3U << DMA_CTRL0_SRC_ADDR_STEP_SIZE_Pos))

#define DMA_DEST_ADDR_STEP_SIZE_1       ((uint32_t)(0x0U << DMA_CTRL0_DEST_ADDR_STEP_SIZE_Pos))
#define DMA_DEST_ADDR_STEP_SIZE_2       ((uint32_t)(0x1U << DMA_CTRL0_DEST_ADDR_STEP_SIZE_Pos))
#define DMA_DEST_ADDR_STEP_SIZE_3       ((uint32_t)(0x2U << DMA_CTRL0_DEST_ADDR_STEP_SIZE_Pos))
#define DMA_DEST_ADDR_STEP_SIZE_4       ((uint32_t)(0x3U << DMA_CTRL0_DEST_ADDR_STEP_SIZE_Pos))

/* DMA_CTRL1[] bit positions */
#define DMA_CTRL1_COUNTER_INT_VALUE_Pos 16
#define DMA_CTRL1_COUNTER_INT_VALUE_Mask ((uint32_t)(0xFFFFU << DMA_CTRL1_COUNTER_INT_VALUE_Pos))
#define DMA_CTRL1_TRANSFER_LENGTH_Pos   0
#define DMA_CTRL1_TRANSFER_LENGTH_Mask  ((uint32_t)(0xFFFFU << DMA_CTRL1_TRANSFER_LENGTH_Pos))

/* DMA_CTRL1[] sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t TRANSFER_LENGTH_SHORT;
    __IO uint16_t COUNTER_INT_VALUE_SHORT;
} DMA_CTRL1_Type;

#define DMA_CTRL1_BASE                  0x40000660
#define DMA_CTRL1                       ((DMA_CTRL1_Type *) DMA_CTRL1_BASE)

/* DMA_STATUS[] bit positions */
#define DMA_STATUS_ERROR_INT_CLEAR_Pos  12
#define DMA_STATUS_COMPLETE_INT_CLEAR_Pos 11
#define DMA_STATUS_COUNTER_INT_CLEAR_Pos 10
#define DMA_STATUS_START_INT_CLEAR_Pos  9
#define DMA_STATUS_DISABLE_INT_CLEAR_Pos 8
#define DMA_STATUS_STATE_Pos            5
#define DMA_STATUS_STATE_Mask           ((uint32_t)(0x7U << DMA_STATUS_STATE_Pos))
#define DMA_STATUS_ERROR_INT_STATUS_Pos 4
#define DMA_STATUS_COMPLETE_INT_STATUS_Pos 3
#define DMA_STATUS_COUNTER_INT_STATUS_Pos 2
#define DMA_STATUS_START_INT_STATUS_Pos 1
#define DMA_STATUS_DISABLE_INT_STATUS_Pos 0

/* DMA_STATUS[] bit-band aliases */
typedef struct
{
    __I  uint32_t DISABLE_INT_STATUS_ALIAS;/* Indicate if a channel disable interrupt has occurred on DMA channel */
    __I  uint32_t START_INT_STATUS_ALIAS;/* Indicate if a start interrupt has occurred on DMA channel */
    __I  uint32_t COUNTER_INT_STATUS_ALIAS;/* Indicate if a counter interrupt has occurred on DMA channel */
    __I  uint32_t COMPLETE_INT_STATUS_ALIAS;/* Indicate if a complete interrupt has occurred on DMA channel */
    __I  uint32_t ERROR_INT_STATUS_ALIAS;/* Indicate if a state machine error interrupt has occurred on DMA channel */
         uint32_t RESERVED0[3];
    __O  uint32_t DISABLE_INT_CLEAR_ALIAS;/* Clear the channel disable flag */
    __O  uint32_t START_INT_CLEAR_ALIAS;/* Clear the start interrupt flag */
    __O  uint32_t COUNTER_INT_CLEAR_ALIAS;/* Clear the counter interrupt flag */
    __O  uint32_t COMPLETE_INT_CLEAR_ALIAS;/* Clear the complete interrupt flag */
    __O  uint32_t ERROR_INT_CLEAR_ALIAS;/* Clear the state machine error interrupt flag */
         uint32_t RESERVED_SPAN[19];
} DMA_STATUS_Type;

#define DMA_STATUS_BASE                 SYS_CALC_BITBAND(0x400006E0, 0)
#define DMA_STATUS                      ((DMA_STATUS_Type *) DMA_STATUS_BASE)

/* DMA_STATUS settings */
#define DMA_DISABLE_INT_STATUS_BITBAND  0x1
#define DMA_DISABLE_INT_STATUS          ((uint32_t)(DMA_DISABLE_INT_STATUS_BITBAND << DMA_STATUS_DISABLE_INT_STATUS_Pos))

#define DMA_START_INT_STATUS_BITBAND    0x1
#define DMA_START_INT_STATUS            ((uint32_t)(DMA_START_INT_STATUS_BITBAND << DMA_STATUS_START_INT_STATUS_Pos))

#define DMA_COUNTER_INT_STATUS_BITBAND  0x1
#define DMA_COUNTER_INT_STATUS          ((uint32_t)(DMA_COUNTER_INT_STATUS_BITBAND << DMA_STATUS_COUNTER_INT_STATUS_Pos))

#define DMA_COMPLETE_INT_STATUS_BITBAND 0x1
#define DMA_COMPLETE_INT_STATUS         ((uint32_t)(DMA_COMPLETE_INT_STATUS_BITBAND << DMA_STATUS_COMPLETE_INT_STATUS_Pos))

#define DMA_ERROR_INT_STATUS_BITBAND    0x1
#define DMA_ERROR_INT_STATUS            ((uint32_t)(DMA_ERROR_INT_STATUS_BITBAND << DMA_STATUS_ERROR_INT_STATUS_Pos))

#define DMA_IDLE                        ((uint32_t)(0x0U << DMA_STATUS_STATE_Pos))
#define DMA_WAIT_SRC_DST_RDY            ((uint32_t)(0x1U << DMA_STATUS_STATE_Pos))
#define DMA_WAIT_SRC_RDY                ((uint32_t)(0x2U << DMA_STATUS_STATE_Pos))
#define DMA_WAIT_RD_COMP                ((uint32_t)(0x3U << DMA_STATUS_STATE_Pos))
#define DMA_CHK_DST                     ((uint32_t)(0x4U << DMA_STATUS_STATE_Pos))
#define DMA_DIS_CHAN                    ((uint32_t)(0x5U << DMA_STATUS_STATE_Pos))
#define DMA_WAIT_DST_RDY                ((uint32_t)(0x6U << DMA_STATUS_STATE_Pos))
#define DMA_WAIT_WR_COMP                ((uint32_t)(0x7U << DMA_STATUS_STATE_Pos))

#define DMA_DISABLE_INT_CLEAR_BITBAND   0x1
#define DMA_DISABLE_INT_CLEAR           ((uint32_t)(DMA_DISABLE_INT_CLEAR_BITBAND << DMA_STATUS_DISABLE_INT_CLEAR_Pos))

#define DMA_START_INT_CLEAR_BITBAND     0x1
#define DMA_START_INT_CLEAR             ((uint32_t)(DMA_START_INT_CLEAR_BITBAND << DMA_STATUS_START_INT_CLEAR_Pos))

#define DMA_COUNTER_INT_CLEAR_BITBAND   0x1
#define DMA_COUNTER_INT_CLEAR           ((uint32_t)(DMA_COUNTER_INT_CLEAR_BITBAND << DMA_STATUS_COUNTER_INT_CLEAR_Pos))

#define DMA_COMPLETE_INT_CLEAR_BITBAND  0x1
#define DMA_COMPLETE_INT_CLEAR          ((uint32_t)(DMA_COMPLETE_INT_CLEAR_BITBAND << DMA_STATUS_COMPLETE_INT_CLEAR_Pos))

#define DMA_ERROR_INT_CLEAR_BITBAND     0x1
#define DMA_ERROR_INT_CLEAR             ((uint32_t)(DMA_ERROR_INT_CLEAR_BITBAND << DMA_STATUS_ERROR_INT_CLEAR_Pos))

/* ----------------------------------------------------------------------------
 * DIO Interface and Digital Pad control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CFG[16];              /* Digital IO Configuration (ADC function only for pads 0 to 3) */
    __IO uint32_t DATA;                 /* Digital IOs Data Access Register */
    __IO uint32_t DIR;                  /* Digital IOs Direction State */
    __I  uint32_t MODE;                 /* Digital IOs Mode State */
    __IO uint32_t INT_CFG[4];           /* DIO Interrupt Configuration */
    __IO uint32_t INT_DEBOUNCE;         /* DIO Interrupt Button Debounce Filter Time Configuration */
    __IO uint32_t PCM_SRC;              /* PCM Input Selection */
    __IO uint32_t SPI_SRC[2];           /* SPI[1:0] Interface Input Selection */
    __IO uint32_t UART_SRC;             /* UART Interface Input Selection */
    __IO uint32_t I2C_SRC;              /* I2C Input Selection */
    __IO uint32_t AUDIOSINK_SRC;        /* Audio Sink Input Selection */
    __IO uint32_t NMI_SRC;              /* NMI Input Selection */
    __IO uint32_t BB_RX_SRC;            /* Baseband Controller RX Data And Clock Input Selection */
    __IO uint32_t BB_SPI_SRC;           /* Baseband Controller SPI Input Selection */
    __IO uint32_t RF_SPI_SRC;           /* RF Front-End SPI Input Selection */
    __IO uint32_t RF_GPIO03_SRC;        /* RF Front-End GPIOs 0-3 Input Selection */
    __IO uint32_t RF_GPIO47_SRC;        /* RF Front-End GPIOs 4-7 Input Selection */
    __IO uint32_t RF_GPIO89_SRC;        /* RF Front-End GPIOs 8-9 Input Selection */
    __IO uint32_t DMIC_SRC;             /* DMIC Data Input Selection Register */
    __IO uint32_t LPDSP32_JTAG_SRC;     /* LPDSP32 JTAG Configuration Register */
    __IO uint32_t JTAG_SW_PAD_CFG;      /* JTAG / SW Pad Configuration Register */
    __IO uint32_t EXTCLK_CFG;           /* External Clock Pad Configuration Register */
    __IO uint32_t PAD_CFG;              /* Global Pads Configuration Register */
} DIO_Type;

#define DIO_BASE                        0x40000700
#define DIO                             ((DIO_Type *) DIO_BASE)

/* DIO_CFG[] bit positions */
#define DIO_CFG_DRIVE_Pos               12
#define DIO_CFG_DRIVE_Mask              ((uint32_t)(0x3U << DIO_CFG_DRIVE_Pos))
#define DIO_CFG_LPF_Pos                 10
#define DIO_CFG_PULL_CTRL_Pos           8
#define DIO_CFG_PULL_CTRL_Mask          ((uint32_t)(0x3U << DIO_CFG_PULL_CTRL_Pos))
#define DIO_CFG_IO_MODE_Pos             0
#define DIO_CFG_IO_MODE_Mask            ((uint32_t)(0x3FU << DIO_CFG_IO_MODE_Pos))

/* DIO_CFG[] sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t IO_MODE_BYTE;         
         uint8_t RESERVED0[3];
} DIO_CFG_Type;

#define DIO_CFG_BASE                    0x40000700
#define DIO_CFG                         ((DIO_CFG_Type *) DIO_CFG_BASE)

/* DIO_CFG settings */
#define DIO_MODE_GPIO_IN_0              ((uint32_t)(0x0U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_GPIO_IN_1              ((uint32_t)(0x1U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_GPIO_OUT_0             ((uint32_t)(0x2U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_GPIO_OUT_1             ((uint32_t)(0x3U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_USRCLK                 ((uint32_t)(0x4U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SLOWCLK                ((uint32_t)(0x5U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SYSCLK                 ((uint32_t)(0x6U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_PCM_SERO               ((uint32_t)(0x7U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_PCM_FRAME              ((uint32_t)(0x8U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SPI0_SERO              ((uint32_t)(0x9U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SPI0_CS                ((uint32_t)(0xAU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SPI0_CLK               ((uint32_t)(0xBU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SPI1_SERO              ((uint32_t)(0xCU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SPI1_CS                ((uint32_t)(0xDU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SPI1_CLK               ((uint32_t)(0xEU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_UART_TX                ((uint32_t)(0xFU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SCL                    ((uint32_t)(0x10U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_SDA                    ((uint32_t)(0x11U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_PWM0                   ((uint32_t)(0x12U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_PWM0_INV               ((uint32_t)(0x13U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_PWM1                   ((uint32_t)(0x14U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_PWM1_INV               ((uint32_t)(0x15U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_LPDSP32_TDO            ((uint32_t)(0x16U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RFCLK                  ((uint32_t)(0x17U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RCCLK                  ((uint32_t)(0x18U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_JTCK_DIV               ((uint32_t)(0x19U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_EXTCLK_DIV             ((uint32_t)(0x1AU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_STANDBYCLK             ((uint32_t)(0x1BU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_TX_DATA             ((uint32_t)(0x1CU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_TX_DATA_VALID       ((uint32_t)(0x1DU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_SYNC_P              ((uint32_t)(0x1EU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_AUDIO0_SYNC_P       ((uint32_t)(0x1FU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_AUDIO1_SYNC_P       ((uint32_t)(0x20U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_AUDIO2_SYNC_P       ((uint32_t)(0x21U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_SPI_CSN             ((uint32_t)(0x22U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_SPI_CLK             ((uint32_t)(0x23U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_SPI_MOSI            ((uint32_t)(0x24U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_DBG0_0              ((uint32_t)(0x25U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_DBG0_1              ((uint32_t)(0x26U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_DBG0_2              ((uint32_t)(0x27U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_DBG0_3              ((uint32_t)(0x28U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_DBG0_4              ((uint32_t)(0x29U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_DBG0_5              ((uint32_t)(0x2AU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_DBG0_6              ((uint32_t)(0x2BU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_BB_DBG0_7              ((uint32_t)(0x2CU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_SPI_MISO            ((uint32_t)(0x2DU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO0               ((uint32_t)(0x2EU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO1               ((uint32_t)(0x2FU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO2               ((uint32_t)(0x30U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO3               ((uint32_t)(0x31U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO4               ((uint32_t)(0x32U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO5               ((uint32_t)(0x33U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO6               ((uint32_t)(0x34U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO7               ((uint32_t)(0x35U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO8               ((uint32_t)(0x36U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_RF_GPIO9               ((uint32_t)(0x37U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_AUDIOCLK               ((uint32_t)(0x38U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_AUDIOSLOWCLK           ((uint32_t)(0x39U << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_OD_P                   ((uint32_t)(0x3AU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_OD_N                   ((uint32_t)(0x3BU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_AUDIO_SYNC_PULSE       ((uint32_t)(0x3CU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_AUDIO_SYNC_MISSED      ((uint32_t)(0x3DU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_INPUT                  ((uint32_t)(0x3EU << DIO_CFG_IO_MODE_Pos))
#define DIO_MODE_DISABLE                ((uint32_t)(0x3FU << DIO_CFG_IO_MODE_Pos))

#define DIO_NO_PULL                     ((uint32_t)(0x0U << DIO_CFG_PULL_CTRL_Pos))
#define DIO_WEAK_PULL_UP                ((uint32_t)(0x1U << DIO_CFG_PULL_CTRL_Pos))
#define DIO_WEAK_PULL_DOWN              ((uint32_t)(0x2U << DIO_CFG_PULL_CTRL_Pos))
#define DIO_STRONG_PULL_UP              ((uint32_t)(0x3U << DIO_CFG_PULL_CTRL_Pos))

#define DIO_LPF_DISABLE_BITBAND         0x0
#define DIO_LPF_ENABLE_BITBAND          0x1
#define DIO_LPF_DISABLE                 ((uint32_t)(DIO_LPF_DISABLE_BITBAND << DIO_CFG_LPF_Pos))
#define DIO_LPF_ENABLE                  ((uint32_t)(DIO_LPF_ENABLE_BITBAND << DIO_CFG_LPF_Pos))

#define DIO_2X_DRIVE                    ((uint32_t)(0x0U << DIO_CFG_DRIVE_Pos))
#define DIO_3X_DRIVE                    ((uint32_t)(0x1U << DIO_CFG_DRIVE_Pos))
#define DIO_5X_DRIVE                    ((uint32_t)(0x2U << DIO_CFG_DRIVE_Pos))
#define DIO_6X_DRIVE                    ((uint32_t)(0x3U << DIO_CFG_DRIVE_Pos))

/* DIO_CFG sub-register bit positions */
#define DIO_CFG_IO_MODE_BYTE_Pos        0
#define DIO_CFG_IO_MODE_BYTE_Mask       ((uint32_t)(0x3FU << DIO_CFG_IO_MODE_BYTE_Pos))

/* DIO_CFG sub-register settings */
#define DIO_MODE_GPIO_IN_0_BYTE         ((uint8_t)(0x0U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_GPIO_IN_1_BYTE         ((uint8_t)(0x1U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_GPIO_OUT_0_BYTE        ((uint8_t)(0x2U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_GPIO_OUT_1_BYTE        ((uint8_t)(0x3U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_USRCLK_BYTE            ((uint8_t)(0x4U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SLOWCLK_BYTE           ((uint8_t)(0x5U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SYSCLK_BYTE            ((uint8_t)(0x6U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_PCM_SERO_BYTE          ((uint8_t)(0x7U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_PCM_FRAME_BYTE         ((uint8_t)(0x8U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SPI0_SERO_BYTE         ((uint8_t)(0x9U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SPI0_CS_BYTE           ((uint8_t)(0xAU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SPI0_CLK_BYTE          ((uint8_t)(0xBU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SPI1_SERO_BYTE         ((uint8_t)(0xCU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SPI1_CS_BYTE           ((uint8_t)(0xDU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SPI1_CLK_BYTE          ((uint8_t)(0xEU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_UART_TX_BYTE           ((uint8_t)(0xFU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SCL_BYTE               ((uint8_t)(0x10U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_SDA_BYTE               ((uint8_t)(0x11U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_PWM0_BYTE              ((uint8_t)(0x12U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_PWM0_INV_BYTE          ((uint8_t)(0x13U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_PWM1_BYTE              ((uint8_t)(0x14U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_PWM1_INV_BYTE          ((uint8_t)(0x15U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_LPDSP32_TDO_BYTE       ((uint8_t)(0x16U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RFCLK_BYTE             ((uint8_t)(0x17U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RCCLK_BYTE             ((uint8_t)(0x18U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_JTCK_DIV_BYTE          ((uint8_t)(0x19U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_EXTCLK_DIV_BYTE        ((uint8_t)(0x1AU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_STANDBYCLK_BYTE        ((uint8_t)(0x1BU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_TX_DATA_BYTE        ((uint8_t)(0x1CU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_TX_DATA_VALID_BYTE  ((uint8_t)(0x1DU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_SYNC_P_BYTE         ((uint8_t)(0x1EU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_AUDIO0_SYNC_P_BYTE  ((uint8_t)(0x1FU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_AUDIO1_SYNC_P_BYTE  ((uint8_t)(0x20U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_AUDIO2_SYNC_P_BYTE  ((uint8_t)(0x21U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_SPI_CSN_BYTE        ((uint8_t)(0x22U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_SPI_CLK_BYTE        ((uint8_t)(0x23U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_SPI_MOSI_BYTE       ((uint8_t)(0x24U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_DBG0_0_BYTE         ((uint8_t)(0x25U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_DBG0_1_BYTE         ((uint8_t)(0x26U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_DBG0_2_BYTE         ((uint8_t)(0x27U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_DBG0_3_BYTE         ((uint8_t)(0x28U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_DBG0_4_BYTE         ((uint8_t)(0x29U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_DBG0_5_BYTE         ((uint8_t)(0x2AU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_DBG0_6_BYTE         ((uint8_t)(0x2BU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_BB_DBG0_7_BYTE         ((uint8_t)(0x2CU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_SPI_MISO_BYTE       ((uint8_t)(0x2DU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO0_BYTE          ((uint8_t)(0x2EU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO1_BYTE          ((uint8_t)(0x2FU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO2_BYTE          ((uint8_t)(0x30U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO3_BYTE          ((uint8_t)(0x31U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO4_BYTE          ((uint8_t)(0x32U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO5_BYTE          ((uint8_t)(0x33U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO6_BYTE          ((uint8_t)(0x34U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO7_BYTE          ((uint8_t)(0x35U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO8_BYTE          ((uint8_t)(0x36U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_RF_GPIO9_BYTE          ((uint8_t)(0x37U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_AUDIOCLK_BYTE          ((uint8_t)(0x38U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_AUDIOSLOWCLK_BYTE      ((uint8_t)(0x39U << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_OD_P_BYTE              ((uint8_t)(0x3AU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_OD_N_BYTE              ((uint8_t)(0x3BU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_AUDIO_SYNC_PULSE_BYTE  ((uint8_t)(0x3CU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_AUDIO_SYNC_MISSED_BYTE ((uint8_t)(0x3DU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_INPUT_BYTE             ((uint8_t)(0x3EU << DIO_CFG_IO_MODE_BYTE_Pos))
#define DIO_MODE_DISABLE_BYTE           ((uint8_t)(0x3FU << DIO_CFG_IO_MODE_BYTE_Pos))

/* DIO_DATA bit positions */
#define DIO_DATA_DIO_Pos                0
#define DIO_DATA_DIO_Mask               ((uint32_t)(0xFFFFU << DIO_DATA_DIO_Pos))
#define DIO_DATA_GPIO_Pos               0
#define DIO_DATA_GPIO_Mask              ((uint32_t)(0xFFFFU << DIO_DATA_GPIO_Pos))

/* DIO_DATA settings */
#define DIO0_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO1_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO2_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO3_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO4_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO5_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO6_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO7_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO8_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO9_LOW                        ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO10_LOW                       ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO11_LOW                       ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO12_LOW                       ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO13_LOW                       ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO14_LOW                       ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO15_LOW                       ((uint32_t)(0x0U << DIO_DATA_DIO_Pos))
#define DIO0_HIGH                       ((uint32_t)(0x1U << DIO_DATA_DIO_Pos))
#define DIO1_HIGH                       ((uint32_t)(0x2U << DIO_DATA_DIO_Pos))
#define DIO2_HIGH                       ((uint32_t)(0x4U << DIO_DATA_DIO_Pos))
#define DIO3_HIGH                       ((uint32_t)(0x8U << DIO_DATA_DIO_Pos))
#define DIO4_HIGH                       ((uint32_t)(0x10U << DIO_DATA_DIO_Pos))
#define DIO5_HIGH                       ((uint32_t)(0x20U << DIO_DATA_DIO_Pos))
#define DIO6_HIGH                       ((uint32_t)(0x40U << DIO_DATA_DIO_Pos))
#define DIO7_HIGH                       ((uint32_t)(0x80U << DIO_DATA_DIO_Pos))
#define DIO8_HIGH                       ((uint32_t)(0x100U << DIO_DATA_DIO_Pos))
#define DIO9_HIGH                       ((uint32_t)(0x200U << DIO_DATA_DIO_Pos))
#define DIO10_HIGH                      ((uint32_t)(0x400U << DIO_DATA_DIO_Pos))
#define DIO11_HIGH                      ((uint32_t)(0x800U << DIO_DATA_DIO_Pos))
#define DIO12_HIGH                      ((uint32_t)(0x1000U << DIO_DATA_DIO_Pos))
#define DIO13_HIGH                      ((uint32_t)(0x2000U << DIO_DATA_DIO_Pos))
#define DIO14_HIGH                      ((uint32_t)(0x4000U << DIO_DATA_DIO_Pos))
#define DIO15_HIGH                      ((uint32_t)(0x8000U << DIO_DATA_DIO_Pos))

#define GPIO0_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO1_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO2_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO3_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO4_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO5_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO6_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO7_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO8_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO9_LOW                       ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO10_LOW                      ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO11_LOW                      ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO12_LOW                      ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO13_LOW                      ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO14_LOW                      ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO15_LOW                      ((uint32_t)(0x0U << DIO_DATA_GPIO_Pos))
#define GPIO0_HIGH                      ((uint32_t)(0x1U << DIO_DATA_GPIO_Pos))
#define GPIO1_HIGH                      ((uint32_t)(0x2U << DIO_DATA_GPIO_Pos))
#define GPIO2_HIGH                      ((uint32_t)(0x4U << DIO_DATA_GPIO_Pos))
#define GPIO3_HIGH                      ((uint32_t)(0x8U << DIO_DATA_GPIO_Pos))
#define GPIO4_HIGH                      ((uint32_t)(0x10U << DIO_DATA_GPIO_Pos))
#define GPIO5_HIGH                      ((uint32_t)(0x20U << DIO_DATA_GPIO_Pos))
#define GPIO6_HIGH                      ((uint32_t)(0x40U << DIO_DATA_GPIO_Pos))
#define GPIO7_HIGH                      ((uint32_t)(0x80U << DIO_DATA_GPIO_Pos))
#define GPIO8_HIGH                      ((uint32_t)(0x100U << DIO_DATA_GPIO_Pos))
#define GPIO9_HIGH                      ((uint32_t)(0x200U << DIO_DATA_GPIO_Pos))
#define GPIO10_HIGH                     ((uint32_t)(0x400U << DIO_DATA_GPIO_Pos))
#define GPIO11_HIGH                     ((uint32_t)(0x800U << DIO_DATA_GPIO_Pos))
#define GPIO12_HIGH                     ((uint32_t)(0x1000U << DIO_DATA_GPIO_Pos))
#define GPIO13_HIGH                     ((uint32_t)(0x2000U << DIO_DATA_GPIO_Pos))
#define GPIO14_HIGH                     ((uint32_t)(0x4000U << DIO_DATA_GPIO_Pos))
#define GPIO15_HIGH                     ((uint32_t)(0x8000U << DIO_DATA_GPIO_Pos))

/* DIO_DIR bit positions */
#define DIO_DIR_DIO_Pos                 0
#define DIO_DIR_DIO_Mask                ((uint32_t)(0xFFFFU << DIO_DIR_DIO_Pos))
#define DIO_DIR_GPIO_Pos                0
#define DIO_DIR_GPIO_Mask               ((uint32_t)(0xFFFFU << DIO_DIR_GPIO_Pos))

/* DIO_DIR settings */
#define DIO0_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO1_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO2_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO3_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO4_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO5_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO6_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO7_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO8_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO9_INPUT                      ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO10_INPUT                     ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO11_INPUT                     ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO12_INPUT                     ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO13_INPUT                     ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO14_INPUT                     ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO15_INPUT                     ((uint32_t)(0x0U << DIO_DIR_DIO_Pos))
#define DIO0_OUTPUT                     ((uint32_t)(0x1U << DIO_DIR_DIO_Pos))
#define DIO1_OUTPUT                     ((uint32_t)(0x2U << DIO_DIR_DIO_Pos))
#define DIO2_OUTPUT                     ((uint32_t)(0x4U << DIO_DIR_DIO_Pos))
#define DIO3_OUTPUT                     ((uint32_t)(0x8U << DIO_DIR_DIO_Pos))
#define DIO4_OUTPUT                     ((uint32_t)(0x10U << DIO_DIR_DIO_Pos))
#define DIO5_OUTPUT                     ((uint32_t)(0x20U << DIO_DIR_DIO_Pos))
#define DIO6_OUTPUT                     ((uint32_t)(0x40U << DIO_DIR_DIO_Pos))
#define DIO7_OUTPUT                     ((uint32_t)(0x80U << DIO_DIR_DIO_Pos))
#define DIO8_OUTPUT                     ((uint32_t)(0x100U << DIO_DIR_DIO_Pos))
#define DIO9_OUTPUT                     ((uint32_t)(0x200U << DIO_DIR_DIO_Pos))
#define DIO10_OUTPUT                    ((uint32_t)(0x400U << DIO_DIR_DIO_Pos))
#define DIO11_OUTPUT                    ((uint32_t)(0x800U << DIO_DIR_DIO_Pos))
#define DIO12_OUTPUT                    ((uint32_t)(0x1000U << DIO_DIR_DIO_Pos))
#define DIO13_OUTPUT                    ((uint32_t)(0x2000U << DIO_DIR_DIO_Pos))
#define DIO14_OUTPUT                    ((uint32_t)(0x4000U << DIO_DIR_DIO_Pos))
#define DIO15_OUTPUT                    ((uint32_t)(0x8000U << DIO_DIR_DIO_Pos))

#define GPIO0_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO1_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO2_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO3_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO4_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO5_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO6_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO7_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO8_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO9_INPUT                     ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO10_INPUT                    ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO11_INPUT                    ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO12_INPUT                    ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO13_INPUT                    ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO14_INPUT                    ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO15_INPUT                    ((uint32_t)(0x0U << DIO_DIR_GPIO_Pos))
#define GPIO0_OUTPUT                    ((uint32_t)(0x1U << DIO_DIR_GPIO_Pos))
#define GPIO1_OUTPUT                    ((uint32_t)(0x2U << DIO_DIR_GPIO_Pos))
#define GPIO2_OUTPUT                    ((uint32_t)(0x4U << DIO_DIR_GPIO_Pos))
#define GPIO3_OUTPUT                    ((uint32_t)(0x8U << DIO_DIR_GPIO_Pos))
#define GPIO4_OUTPUT                    ((uint32_t)(0x10U << DIO_DIR_GPIO_Pos))
#define GPIO5_OUTPUT                    ((uint32_t)(0x20U << DIO_DIR_GPIO_Pos))
#define GPIO6_OUTPUT                    ((uint32_t)(0x40U << DIO_DIR_GPIO_Pos))
#define GPIO7_OUTPUT                    ((uint32_t)(0x80U << DIO_DIR_GPIO_Pos))
#define GPIO8_OUTPUT                    ((uint32_t)(0x100U << DIO_DIR_GPIO_Pos))
#define GPIO9_OUTPUT                    ((uint32_t)(0x200U << DIO_DIR_GPIO_Pos))
#define GPIO10_OUTPUT                   ((uint32_t)(0x400U << DIO_DIR_GPIO_Pos))
#define GPIO11_OUTPUT                   ((uint32_t)(0x800U << DIO_DIR_GPIO_Pos))
#define GPIO12_OUTPUT                   ((uint32_t)(0x1000U << DIO_DIR_GPIO_Pos))
#define GPIO13_OUTPUT                   ((uint32_t)(0x2000U << DIO_DIR_GPIO_Pos))
#define GPIO14_OUTPUT                   ((uint32_t)(0x4000U << DIO_DIR_GPIO_Pos))
#define GPIO15_OUTPUT                   ((uint32_t)(0x8000U << DIO_DIR_GPIO_Pos))

/* DIO_MODE bit positions */
#define DIO_MODE_GPIO_Pos               0
#define DIO_MODE_GPIO_Mask              ((uint32_t)(0xFFFFU << DIO_MODE_GPIO_Pos))

/* DIO_MODE settings */
#define DIO0_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO1_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO2_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO3_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO4_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO5_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO6_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO7_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO8_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO9_IS_NOT_GPIO                ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO10_IS_NOT_GPIO               ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO11_IS_NOT_GPIO               ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO12_IS_NOT_GPIO               ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO13_IS_NOT_GPIO               ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO14_IS_NOT_GPIO               ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO15_IS_NOT_GPIO               ((uint32_t)(0x0U << DIO_MODE_GPIO_Pos))
#define DIO0_IS_GPIO                    ((uint32_t)(0x1U << DIO_MODE_GPIO_Pos))
#define DIO1_IS_GPIO                    ((uint32_t)(0x2U << DIO_MODE_GPIO_Pos))
#define DIO2_IS_GPIO                    ((uint32_t)(0x4U << DIO_MODE_GPIO_Pos))
#define DIO3_IS_GPIO                    ((uint32_t)(0x8U << DIO_MODE_GPIO_Pos))
#define DIO4_IS_GPIO                    ((uint32_t)(0x10U << DIO_MODE_GPIO_Pos))
#define DIO5_IS_GPIO                    ((uint32_t)(0x20U << DIO_MODE_GPIO_Pos))
#define DIO6_IS_GPIO                    ((uint32_t)(0x40U << DIO_MODE_GPIO_Pos))
#define DIO7_IS_GPIO                    ((uint32_t)(0x80U << DIO_MODE_GPIO_Pos))
#define DIO8_IS_GPIO                    ((uint32_t)(0x100U << DIO_MODE_GPIO_Pos))
#define DIO9_IS_GPIO                    ((uint32_t)(0x200U << DIO_MODE_GPIO_Pos))
#define DIO10_IS_GPIO                   ((uint32_t)(0x400U << DIO_MODE_GPIO_Pos))
#define DIO11_IS_GPIO                   ((uint32_t)(0x800U << DIO_MODE_GPIO_Pos))
#define DIO12_IS_GPIO                   ((uint32_t)(0x1000U << DIO_MODE_GPIO_Pos))
#define DIO13_IS_GPIO                   ((uint32_t)(0x2000U << DIO_MODE_GPIO_Pos))
#define DIO14_IS_GPIO                   ((uint32_t)(0x4000U << DIO_MODE_GPIO_Pos))
#define DIO15_IS_GPIO                   ((uint32_t)(0x8000U << DIO_MODE_GPIO_Pos))

/* DIO_INT_CFG[] bit positions */
#define DIO_INT_CFG_DEBOUNCE_ENABLE_Pos 11
#define DIO_INT_CFG_EVENT_Pos           8
#define DIO_INT_CFG_EVENT_Mask          ((uint32_t)(0x7U << DIO_INT_CFG_EVENT_Pos))
#define DIO_INT_CFG_SRC_Pos             0
#define DIO_INT_CFG_SRC_Mask            ((uint32_t)(0x1FU << DIO_INT_CFG_SRC_Pos))

/* DIO_INT_CFG[] sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t SRC_BYTE;             
         uint8_t RESERVED0[3];
} DIO_INT_CFG_Type;

#define DIO_INT_CFG_BASE                0x4000074C
#define DIO_INT_CFG                     ((DIO_INT_CFG_Type *) DIO_INT_CFG_BASE)

/* DIO_INT_CFG settings */
#define DIO_SRC_DIO_0                   ((uint32_t)(0x0U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_1                   ((uint32_t)(0x1U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_2                   ((uint32_t)(0x2U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_3                   ((uint32_t)(0x3U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_4                   ((uint32_t)(0x4U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_5                   ((uint32_t)(0x5U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_6                   ((uint32_t)(0x6U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_7                   ((uint32_t)(0x7U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_8                   ((uint32_t)(0x8U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_9                   ((uint32_t)(0x9U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_10                  ((uint32_t)(0xAU << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_11                  ((uint32_t)(0xBU << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_12                  ((uint32_t)(0xCU << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_13                  ((uint32_t)(0xDU << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_14                  ((uint32_t)(0xEU << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_DIO_15                  ((uint32_t)(0xFU << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_EXTCLK                  ((uint32_t)(0x10U << DIO_INT_CFG_SRC_Pos))
#define DIO_SRC_WAKEUP                  ((uint32_t)(0x11U << DIO_INT_CFG_SRC_Pos))

#define DIO_EVENT_NONE                  ((uint32_t)(0x0U << DIO_INT_CFG_EVENT_Pos))
#define DIO_EVENT_HIGH_LEVEL            ((uint32_t)(0x1U << DIO_INT_CFG_EVENT_Pos))
#define DIO_EVENT_LOW_LEVEL             ((uint32_t)(0x2U << DIO_INT_CFG_EVENT_Pos))
#define DIO_EVENT_RISING_EDGE           ((uint32_t)(0x3U << DIO_INT_CFG_EVENT_Pos))
#define DIO_EVENT_FALLING_EDGE          ((uint32_t)(0x4U << DIO_INT_CFG_EVENT_Pos))
#define DIO_EVENT_TRANSITION            ((uint32_t)(0x5U << DIO_INT_CFG_EVENT_Pos))

#define DIO_DEBOUNCE_DISABLE_BITBAND    0x0
#define DIO_DEBOUNCE_ENABLE_BITBAND     0x1
#define DIO_DEBOUNCE_DISABLE            ((uint32_t)(DIO_DEBOUNCE_DISABLE_BITBAND << DIO_INT_CFG_DEBOUNCE_ENABLE_Pos))
#define DIO_DEBOUNCE_ENABLE             ((uint32_t)(DIO_DEBOUNCE_ENABLE_BITBAND << DIO_INT_CFG_DEBOUNCE_ENABLE_Pos))

/* DIO_INT_CFG sub-register bit positions */
#define DIO_INT_CFG_SRC_BYTE_Pos        0
#define DIO_INT_CFG_SRC_BYTE_Mask       ((uint32_t)(0x1FU << DIO_INT_CFG_SRC_BYTE_Pos))

/* DIO_INT_CFG sub-register settings */
#define DIO_SRC_DIO_0_BYTE              ((uint8_t)(0x0U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_1_BYTE              ((uint8_t)(0x1U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_2_BYTE              ((uint8_t)(0x2U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_3_BYTE              ((uint8_t)(0x3U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_4_BYTE              ((uint8_t)(0x4U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_5_BYTE              ((uint8_t)(0x5U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_6_BYTE              ((uint8_t)(0x6U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_7_BYTE              ((uint8_t)(0x7U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_8_BYTE              ((uint8_t)(0x8U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_9_BYTE              ((uint8_t)(0x9U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_10_BYTE             ((uint8_t)(0xAU << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_11_BYTE             ((uint8_t)(0xBU << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_12_BYTE             ((uint8_t)(0xCU << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_13_BYTE             ((uint8_t)(0xDU << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_14_BYTE             ((uint8_t)(0xEU << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_DIO_15_BYTE             ((uint8_t)(0xFU << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_EXTCLK_BYTE             ((uint8_t)(0x10U << DIO_INT_CFG_SRC_BYTE_Pos))
#define DIO_SRC_WAKEUP_BYTE             ((uint8_t)(0x11U << DIO_INT_CFG_SRC_BYTE_Pos))

/* DIO_INT_DEBOUNCE bit positions */
#define DIO_INT_DEBOUNCE_DEBOUNCE_CLK_Pos 8
#define DIO_INT_DEBOUNCE_DEBOUNCE_COUNT_Pos 0
#define DIO_INT_DEBOUNCE_DEBOUNCE_COUNT_Mask ((uint32_t)(0xFFU << DIO_INT_DEBOUNCE_DEBOUNCE_COUNT_Pos))

/* DIO_INT_DEBOUNCE sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DEBOUNCE_COUNT_BYTE;  
         uint8_t RESERVED0[3];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x4000075C, 0) - (0x4000075C + 4)];
         uint32_t RESERVED1[8];
    __IO uint32_t DEBOUNCE_CLK_ALIAS;   /* Interrupt button debounce filter clock */
} DIO_INT_DEBOUNCE_Type;

#define DIO_INT_DEBOUNCE_BASE           0x4000075C
#define DIO_INT_DEBOUNCE                ((DIO_INT_DEBOUNCE_Type *) DIO_INT_DEBOUNCE_BASE)

/* DIO_INT_DEBOUNCE settings */
#define DIO_DEBOUNCE_SLOWCLK_DIV32_BITBAND 0x0
#define DIO_DEBOUNCE_SLOWCLK_DIV1024_BITBAND 0x1
#define DIO_DEBOUNCE_SLOWCLK_DIV32      ((uint32_t)(DIO_DEBOUNCE_SLOWCLK_DIV32_BITBAND << DIO_INT_DEBOUNCE_DEBOUNCE_CLK_Pos))
#define DIO_DEBOUNCE_SLOWCLK_DIV1024    ((uint32_t)(DIO_DEBOUNCE_SLOWCLK_DIV1024_BITBAND << DIO_INT_DEBOUNCE_DEBOUNCE_CLK_Pos))

/* DIO_PCM_SRC bit positions */
#define DIO_PCM_SRC_SERI_Pos            16
#define DIO_PCM_SRC_SERI_Mask           ((uint32_t)(0x1FU << DIO_PCM_SRC_SERI_Pos))
#define DIO_PCM_SRC_FRAME_Pos           8
#define DIO_PCM_SRC_FRAME_Mask          ((uint32_t)(0x1FU << DIO_PCM_SRC_FRAME_Pos))
#define DIO_PCM_SRC_CLK_Pos             0
#define DIO_PCM_SRC_CLK_Mask            ((uint32_t)(0x1FU << DIO_PCM_SRC_CLK_Pos))

/* DIO_PCM_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t CLK_BYTE;             
    __IO uint8_t FRAME_BYTE;           
    __IO uint8_t SERI_BYTE;            
         uint8_t RESERVED0[1];
} DIO_PCM_SRC_Type;

#define DIO_PCM_SRC_BASE                0x40000760
#define DIO_PCM_SRC                     ((DIO_PCM_SRC_Type *) DIO_PCM_SRC_BASE)

/* DIO_PCM_SRC settings */
#define PCM_CLK_SRC_DIO_0               ((uint32_t)(0x0U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_1               ((uint32_t)(0x1U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_2               ((uint32_t)(0x2U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_3               ((uint32_t)(0x3U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_4               ((uint32_t)(0x4U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_5               ((uint32_t)(0x5U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_6               ((uint32_t)(0x6U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_7               ((uint32_t)(0x7U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_8               ((uint32_t)(0x8U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_9               ((uint32_t)(0x9U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_10              ((uint32_t)(0xAU << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_11              ((uint32_t)(0xBU << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_12              ((uint32_t)(0xCU << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_13              ((uint32_t)(0xDU << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_14              ((uint32_t)(0xEU << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_DIO_15              ((uint32_t)(0xFU << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_CONST_LOW           ((uint32_t)(0x10U << DIO_PCM_SRC_CLK_Pos))
#define PCM_CLK_SRC_CONST_HIGH          ((uint32_t)(0x11U << DIO_PCM_SRC_CLK_Pos))

#define PCM_FRAME_SRC_DIO_0             ((uint32_t)(0x0U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_1             ((uint32_t)(0x1U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_2             ((uint32_t)(0x2U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_3             ((uint32_t)(0x3U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_4             ((uint32_t)(0x4U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_5             ((uint32_t)(0x5U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_6             ((uint32_t)(0x6U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_7             ((uint32_t)(0x7U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_8             ((uint32_t)(0x8U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_9             ((uint32_t)(0x9U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_10            ((uint32_t)(0xAU << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_11            ((uint32_t)(0xBU << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_12            ((uint32_t)(0xCU << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_13            ((uint32_t)(0xDU << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_14            ((uint32_t)(0xEU << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_DIO_15            ((uint32_t)(0xFU << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_CONST_LOW         ((uint32_t)(0x10U << DIO_PCM_SRC_FRAME_Pos))
#define PCM_FRAME_SRC_CONST_HIGH        ((uint32_t)(0x11U << DIO_PCM_SRC_FRAME_Pos))

#define PCM_SERI_SRC_DIO_0              ((uint32_t)(0x0U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_1              ((uint32_t)(0x1U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_2              ((uint32_t)(0x2U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_3              ((uint32_t)(0x3U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_4              ((uint32_t)(0x4U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_5              ((uint32_t)(0x5U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_6              ((uint32_t)(0x6U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_7              ((uint32_t)(0x7U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_8              ((uint32_t)(0x8U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_9              ((uint32_t)(0x9U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_10             ((uint32_t)(0xAU << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_11             ((uint32_t)(0xBU << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_12             ((uint32_t)(0xCU << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_13             ((uint32_t)(0xDU << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_14             ((uint32_t)(0xEU << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_DIO_15             ((uint32_t)(0xFU << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_PCM_SRC_SERI_Pos))
#define PCM_SERI_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_PCM_SRC_SERI_Pos))

/* DIO_PCM_SRC sub-register bit positions */
#define DIO_PCM_SRC_CLK_BYTE_Pos        0
#define DIO_PCM_SRC_CLK_BYTE_Mask       ((uint32_t)(0x1FU << DIO_PCM_SRC_CLK_BYTE_Pos))
#define DIO_PCM_SRC_FRAME_BYTE_Pos      0
#define DIO_PCM_SRC_FRAME_BYTE_Mask     ((uint32_t)(0x1FU << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define DIO_PCM_SRC_SERI_BYTE_Pos       0
#define DIO_PCM_SRC_SERI_BYTE_Mask      ((uint32_t)(0x1FU << DIO_PCM_SRC_SERI_BYTE_Pos))

/* DIO_PCM_SRC sub-register settings */
#define PCM_CLK_SRC_DIO_0_BYTE          ((uint8_t)(0x0U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_1_BYTE          ((uint8_t)(0x1U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_2_BYTE          ((uint8_t)(0x2U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_3_BYTE          ((uint8_t)(0x3U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_4_BYTE          ((uint8_t)(0x4U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_5_BYTE          ((uint8_t)(0x5U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_6_BYTE          ((uint8_t)(0x6U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_7_BYTE          ((uint8_t)(0x7U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_8_BYTE          ((uint8_t)(0x8U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_9_BYTE          ((uint8_t)(0x9U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_10_BYTE         ((uint8_t)(0xAU << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_11_BYTE         ((uint8_t)(0xBU << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_12_BYTE         ((uint8_t)(0xCU << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_13_BYTE         ((uint8_t)(0xDU << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_14_BYTE         ((uint8_t)(0xEU << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_DIO_15_BYTE         ((uint8_t)(0xFU << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_CONST_LOW_BYTE      ((uint8_t)(0x10U << DIO_PCM_SRC_CLK_BYTE_Pos))
#define PCM_CLK_SRC_CONST_HIGH_BYTE     ((uint8_t)(0x11U << DIO_PCM_SRC_CLK_BYTE_Pos))

#define PCM_FRAME_SRC_DIO_0_BYTE        ((uint8_t)(0x0U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_1_BYTE        ((uint8_t)(0x1U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_2_BYTE        ((uint8_t)(0x2U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_3_BYTE        ((uint8_t)(0x3U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_4_BYTE        ((uint8_t)(0x4U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_5_BYTE        ((uint8_t)(0x5U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_6_BYTE        ((uint8_t)(0x6U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_7_BYTE        ((uint8_t)(0x7U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_8_BYTE        ((uint8_t)(0x8U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_9_BYTE        ((uint8_t)(0x9U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_10_BYTE       ((uint8_t)(0xAU << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_11_BYTE       ((uint8_t)(0xBU << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_12_BYTE       ((uint8_t)(0xCU << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_13_BYTE       ((uint8_t)(0xDU << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_14_BYTE       ((uint8_t)(0xEU << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_DIO_15_BYTE       ((uint8_t)(0xFU << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_CONST_LOW_BYTE    ((uint8_t)(0x10U << DIO_PCM_SRC_FRAME_BYTE_Pos))
#define PCM_FRAME_SRC_CONST_HIGH_BYTE   ((uint8_t)(0x11U << DIO_PCM_SRC_FRAME_BYTE_Pos))

#define PCM_SERI_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_PCM_SRC_SERI_BYTE_Pos))
#define PCM_SERI_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_PCM_SRC_SERI_BYTE_Pos))

/* DIO_SPI_SRC[] bit positions */
#define DIO_SPI_SRC_SERI_Pos            16
#define DIO_SPI_SRC_SERI_Mask           ((uint32_t)(0x1FU << DIO_SPI_SRC_SERI_Pos))
#define DIO_SPI_SRC_CS_Pos              8
#define DIO_SPI_SRC_CS_Mask             ((uint32_t)(0x1FU << DIO_SPI_SRC_CS_Pos))
#define DIO_SPI_SRC_CLK_Pos             0
#define DIO_SPI_SRC_CLK_Mask            ((uint32_t)(0x1FU << DIO_SPI_SRC_CLK_Pos))

/* DIO_SPI_SRC[] sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t CLK_BYTE;             
    __IO uint8_t CS_BYTE;              
    __IO uint8_t SERI_BYTE;            
         uint8_t RESERVED0[1];
} DIO_SPI_SRC_Type;

#define DIO_SPI_SRC_BASE                0x40000764
#define DIO_SPI_SRC                     ((DIO_SPI_SRC_Type *) DIO_SPI_SRC_BASE)

/* DIO_SPI_SRC settings */
#define SPI_CLK_SRC_DIO_0               ((uint32_t)(0x0U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_1               ((uint32_t)(0x1U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_2               ((uint32_t)(0x2U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_3               ((uint32_t)(0x3U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_4               ((uint32_t)(0x4U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_5               ((uint32_t)(0x5U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_6               ((uint32_t)(0x6U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_7               ((uint32_t)(0x7U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_8               ((uint32_t)(0x8U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_9               ((uint32_t)(0x9U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_10              ((uint32_t)(0xAU << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_11              ((uint32_t)(0xBU << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_12              ((uint32_t)(0xCU << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_13              ((uint32_t)(0xDU << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_14              ((uint32_t)(0xEU << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_DIO_15              ((uint32_t)(0xFU << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_CONST_LOW           ((uint32_t)(0x10U << DIO_SPI_SRC_CLK_Pos))
#define SPI_CLK_SRC_CONST_HIGH          ((uint32_t)(0x11U << DIO_SPI_SRC_CLK_Pos))

#define SPI_CS_SRC_DIO_0                ((uint32_t)(0x0U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_1                ((uint32_t)(0x1U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_2                ((uint32_t)(0x2U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_3                ((uint32_t)(0x3U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_4                ((uint32_t)(0x4U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_5                ((uint32_t)(0x5U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_6                ((uint32_t)(0x6U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_7                ((uint32_t)(0x7U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_8                ((uint32_t)(0x8U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_9                ((uint32_t)(0x9U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_10               ((uint32_t)(0xAU << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_11               ((uint32_t)(0xBU << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_12               ((uint32_t)(0xCU << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_13               ((uint32_t)(0xDU << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_14               ((uint32_t)(0xEU << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_DIO_15               ((uint32_t)(0xFU << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_CONST_LOW            ((uint32_t)(0x10U << DIO_SPI_SRC_CS_Pos))
#define SPI_CS_SRC_CONST_HIGH           ((uint32_t)(0x11U << DIO_SPI_SRC_CS_Pos))

#define SPI_SERI_SRC_DIO_0              ((uint32_t)(0x0U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_1              ((uint32_t)(0x1U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_2              ((uint32_t)(0x2U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_3              ((uint32_t)(0x3U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_4              ((uint32_t)(0x4U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_5              ((uint32_t)(0x5U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_6              ((uint32_t)(0x6U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_7              ((uint32_t)(0x7U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_8              ((uint32_t)(0x8U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_9              ((uint32_t)(0x9U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_10             ((uint32_t)(0xAU << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_11             ((uint32_t)(0xBU << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_12             ((uint32_t)(0xCU << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_13             ((uint32_t)(0xDU << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_14             ((uint32_t)(0xEU << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_DIO_15             ((uint32_t)(0xFU << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_SPI_SRC_SERI_Pos))
#define SPI_SERI_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_SPI_SRC_SERI_Pos))

/* DIO_SPI_SRC sub-register bit positions */
#define DIO_SPI_SRC_CLK_BYTE_Pos        0
#define DIO_SPI_SRC_CLK_BYTE_Mask       ((uint32_t)(0x1FU << DIO_SPI_SRC_CLK_BYTE_Pos))
#define DIO_SPI_SRC_CS_BYTE_Pos         0
#define DIO_SPI_SRC_CS_BYTE_Mask        ((uint32_t)(0x1FU << DIO_SPI_SRC_CS_BYTE_Pos))
#define DIO_SPI_SRC_SERI_BYTE_Pos       0
#define DIO_SPI_SRC_SERI_BYTE_Mask      ((uint32_t)(0x1FU << DIO_SPI_SRC_SERI_BYTE_Pos))

/* DIO_SPI_SRC sub-register settings */
#define SPI_CLK_SRC_DIO_0_BYTE          ((uint8_t)(0x0U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_1_BYTE          ((uint8_t)(0x1U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_2_BYTE          ((uint8_t)(0x2U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_3_BYTE          ((uint8_t)(0x3U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_4_BYTE          ((uint8_t)(0x4U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_5_BYTE          ((uint8_t)(0x5U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_6_BYTE          ((uint8_t)(0x6U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_7_BYTE          ((uint8_t)(0x7U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_8_BYTE          ((uint8_t)(0x8U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_9_BYTE          ((uint8_t)(0x9U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_10_BYTE         ((uint8_t)(0xAU << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_11_BYTE         ((uint8_t)(0xBU << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_12_BYTE         ((uint8_t)(0xCU << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_13_BYTE         ((uint8_t)(0xDU << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_14_BYTE         ((uint8_t)(0xEU << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_DIO_15_BYTE         ((uint8_t)(0xFU << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_CONST_LOW_BYTE      ((uint8_t)(0x10U << DIO_SPI_SRC_CLK_BYTE_Pos))
#define SPI_CLK_SRC_CONST_HIGH_BYTE     ((uint8_t)(0x11U << DIO_SPI_SRC_CLK_BYTE_Pos))

#define SPI_CS_SRC_DIO_0_BYTE           ((uint8_t)(0x0U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_1_BYTE           ((uint8_t)(0x1U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_2_BYTE           ((uint8_t)(0x2U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_3_BYTE           ((uint8_t)(0x3U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_4_BYTE           ((uint8_t)(0x4U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_5_BYTE           ((uint8_t)(0x5U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_6_BYTE           ((uint8_t)(0x6U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_7_BYTE           ((uint8_t)(0x7U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_8_BYTE           ((uint8_t)(0x8U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_9_BYTE           ((uint8_t)(0x9U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_10_BYTE          ((uint8_t)(0xAU << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_11_BYTE          ((uint8_t)(0xBU << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_12_BYTE          ((uint8_t)(0xCU << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_13_BYTE          ((uint8_t)(0xDU << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_14_BYTE          ((uint8_t)(0xEU << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_DIO_15_BYTE          ((uint8_t)(0xFU << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_CONST_LOW_BYTE       ((uint8_t)(0x10U << DIO_SPI_SRC_CS_BYTE_Pos))
#define SPI_CS_SRC_CONST_HIGH_BYTE      ((uint8_t)(0x11U << DIO_SPI_SRC_CS_BYTE_Pos))

#define SPI_SERI_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_SPI_SRC_SERI_BYTE_Pos))
#define SPI_SERI_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_SPI_SRC_SERI_BYTE_Pos))

/* DIO_UART_SRC bit positions */
#define DIO_UART_SRC_RX_Pos             0
#define DIO_UART_SRC_RX_Mask            ((uint32_t)(0x1FU << DIO_UART_SRC_RX_Pos))

/* DIO_UART_SRC settings */
#define UART_RX_SRC_DIO_0               ((uint32_t)(0x0U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_1               ((uint32_t)(0x1U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_2               ((uint32_t)(0x2U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_3               ((uint32_t)(0x3U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_4               ((uint32_t)(0x4U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_5               ((uint32_t)(0x5U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_6               ((uint32_t)(0x6U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_7               ((uint32_t)(0x7U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_8               ((uint32_t)(0x8U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_9               ((uint32_t)(0x9U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_10              ((uint32_t)(0xAU << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_11              ((uint32_t)(0xBU << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_12              ((uint32_t)(0xCU << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_13              ((uint32_t)(0xDU << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_14              ((uint32_t)(0xEU << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_DIO_15              ((uint32_t)(0xFU << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_CONST_LOW           ((uint32_t)(0x10U << DIO_UART_SRC_RX_Pos))
#define UART_RX_SRC_CONST_HIGH          ((uint32_t)(0x11U << DIO_UART_SRC_RX_Pos))

/* DIO_I2C_SRC bit positions */
#define DIO_I2C_SRC_SDA_Pos             8
#define DIO_I2C_SRC_SDA_Mask            ((uint32_t)(0x1FU << DIO_I2C_SRC_SDA_Pos))
#define DIO_I2C_SRC_SCL_Pos             0
#define DIO_I2C_SRC_SCL_Mask            ((uint32_t)(0x1FU << DIO_I2C_SRC_SCL_Pos))

/* DIO_I2C_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t SCL_BYTE;             
    __IO uint8_t SDA_BYTE;             
         uint8_t RESERVED0[2];
} DIO_I2C_SRC_Type;

#define DIO_I2C_SRC_BASE                0x40000770
#define DIO_I2C_SRC                     ((DIO_I2C_SRC_Type *) DIO_I2C_SRC_BASE)

/* DIO_I2C_SRC settings */
#define SCL_SRC_DIO_0                   ((uint32_t)(0x0U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_1                   ((uint32_t)(0x1U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_2                   ((uint32_t)(0x2U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_3                   ((uint32_t)(0x3U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_4                   ((uint32_t)(0x4U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_5                   ((uint32_t)(0x5U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_6                   ((uint32_t)(0x6U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_7                   ((uint32_t)(0x7U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_8                   ((uint32_t)(0x8U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_9                   ((uint32_t)(0x9U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_10                  ((uint32_t)(0xAU << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_11                  ((uint32_t)(0xBU << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_12                  ((uint32_t)(0xCU << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_13                  ((uint32_t)(0xDU << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_14                  ((uint32_t)(0xEU << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_DIO_15                  ((uint32_t)(0xFU << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_CONST_LOW               ((uint32_t)(0x10U << DIO_I2C_SRC_SCL_Pos))
#define SCL_SRC_CONST_HIGH              ((uint32_t)(0x11U << DIO_I2C_SRC_SCL_Pos))

#define SDA_SRC_DIO_0                   ((uint32_t)(0x0U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_1                   ((uint32_t)(0x1U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_2                   ((uint32_t)(0x2U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_3                   ((uint32_t)(0x3U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_4                   ((uint32_t)(0x4U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_5                   ((uint32_t)(0x5U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_6                   ((uint32_t)(0x6U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_7                   ((uint32_t)(0x7U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_8                   ((uint32_t)(0x8U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_9                   ((uint32_t)(0x9U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_10                  ((uint32_t)(0xAU << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_11                  ((uint32_t)(0xBU << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_12                  ((uint32_t)(0xCU << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_13                  ((uint32_t)(0xDU << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_14                  ((uint32_t)(0xEU << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_DIO_15                  ((uint32_t)(0xFU << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_CONST_LOW               ((uint32_t)(0x10U << DIO_I2C_SRC_SDA_Pos))
#define SDA_SRC_CONST_HIGH              ((uint32_t)(0x11U << DIO_I2C_SRC_SDA_Pos))

/* DIO_I2C_SRC sub-register bit positions */
#define DIO_I2C_SRC_SCL_BYTE_Pos        0
#define DIO_I2C_SRC_SCL_BYTE_Mask       ((uint32_t)(0x1FU << DIO_I2C_SRC_SCL_BYTE_Pos))
#define DIO_I2C_SRC_SDA_BYTE_Pos        0
#define DIO_I2C_SRC_SDA_BYTE_Mask       ((uint32_t)(0x1FU << DIO_I2C_SRC_SDA_BYTE_Pos))

/* DIO_I2C_SRC sub-register settings */
#define SCL_SRC_DIO_0_BYTE              ((uint8_t)(0x0U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_1_BYTE              ((uint8_t)(0x1U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_2_BYTE              ((uint8_t)(0x2U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_3_BYTE              ((uint8_t)(0x3U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_4_BYTE              ((uint8_t)(0x4U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_5_BYTE              ((uint8_t)(0x5U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_6_BYTE              ((uint8_t)(0x6U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_7_BYTE              ((uint8_t)(0x7U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_8_BYTE              ((uint8_t)(0x8U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_9_BYTE              ((uint8_t)(0x9U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_10_BYTE             ((uint8_t)(0xAU << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_11_BYTE             ((uint8_t)(0xBU << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_12_BYTE             ((uint8_t)(0xCU << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_13_BYTE             ((uint8_t)(0xDU << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_14_BYTE             ((uint8_t)(0xEU << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_DIO_15_BYTE             ((uint8_t)(0xFU << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_CONST_LOW_BYTE          ((uint8_t)(0x10U << DIO_I2C_SRC_SCL_BYTE_Pos))
#define SCL_SRC_CONST_HIGH_BYTE         ((uint8_t)(0x11U << DIO_I2C_SRC_SCL_BYTE_Pos))

#define SDA_SRC_DIO_0_BYTE              ((uint8_t)(0x0U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_1_BYTE              ((uint8_t)(0x1U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_2_BYTE              ((uint8_t)(0x2U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_3_BYTE              ((uint8_t)(0x3U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_4_BYTE              ((uint8_t)(0x4U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_5_BYTE              ((uint8_t)(0x5U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_6_BYTE              ((uint8_t)(0x6U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_7_BYTE              ((uint8_t)(0x7U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_8_BYTE              ((uint8_t)(0x8U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_9_BYTE              ((uint8_t)(0x9U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_10_BYTE             ((uint8_t)(0xAU << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_11_BYTE             ((uint8_t)(0xBU << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_12_BYTE             ((uint8_t)(0xCU << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_13_BYTE             ((uint8_t)(0xDU << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_14_BYTE             ((uint8_t)(0xEU << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_DIO_15_BYTE             ((uint8_t)(0xFU << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_CONST_LOW_BYTE          ((uint8_t)(0x10U << DIO_I2C_SRC_SDA_BYTE_Pos))
#define SDA_SRC_CONST_HIGH_BYTE         ((uint8_t)(0x11U << DIO_I2C_SRC_SDA_BYTE_Pos))

/* DIO_AUDIOSINK_SRC bit positions */
#define DIO_AUDIOSINK_SRC_CLK_Pos       0
#define DIO_AUDIOSINK_SRC_CLK_Mask      ((uint32_t)(0x1FU << DIO_AUDIOSINK_SRC_CLK_Pos))

/* DIO_AUDIOSINK_SRC settings */
#define AUDIOSINK_CLK_SRC_DIO_0         ((uint32_t)(0x0U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_1         ((uint32_t)(0x1U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_2         ((uint32_t)(0x2U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_3         ((uint32_t)(0x3U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_4         ((uint32_t)(0x4U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_5         ((uint32_t)(0x5U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_6         ((uint32_t)(0x6U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_7         ((uint32_t)(0x7U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_8         ((uint32_t)(0x8U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_9         ((uint32_t)(0x9U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_10        ((uint32_t)(0xAU << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_11        ((uint32_t)(0xBU << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_12        ((uint32_t)(0xCU << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_13        ((uint32_t)(0xDU << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_14        ((uint32_t)(0xEU << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DIO_15        ((uint32_t)(0xFU << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_CONST_LOW     ((uint32_t)(0x10U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_CONST_HIGH    ((uint32_t)(0x11U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_STANDBYCLK    ((uint32_t)(0x12U << DIO_AUDIOSINK_SRC_CLK_Pos))
#define AUDIOSINK_CLK_SRC_DMIC_OD       ((uint32_t)(0x13U << DIO_AUDIOSINK_SRC_CLK_Pos))

/* DIO_NMI_SRC bit positions */
#define DIO_NMI_SRC_NMI_POLARITY_Pos    5
#define DIO_NMI_SRC_NMI_Pos             0
#define DIO_NMI_SRC_NMI_Mask            ((uint32_t)(0x1FU << DIO_NMI_SRC_NMI_Pos))

/* DIO_NMI_SRC bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[5];
    __IO uint32_t NMI_POLARITY_ALIAS;   /* NMI polarity */
} DIO_NMI_SRC_Type;

#define DIO_NMI_SRC_BASE                SYS_CALC_BITBAND(0x40000778, 0)
#define DIO_NMI_SRC                     ((DIO_NMI_SRC_Type *) DIO_NMI_SRC_BASE)

/* DIO_NMI_SRC settings */
#define NMI_SRC_DIO_0                   ((uint32_t)(0x0U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_1                   ((uint32_t)(0x1U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_2                   ((uint32_t)(0x2U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_3                   ((uint32_t)(0x3U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_4                   ((uint32_t)(0x4U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_5                   ((uint32_t)(0x5U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_6                   ((uint32_t)(0x6U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_7                   ((uint32_t)(0x7U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_8                   ((uint32_t)(0x8U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_9                   ((uint32_t)(0x9U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_10                  ((uint32_t)(0xAU << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_11                  ((uint32_t)(0xBU << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_12                  ((uint32_t)(0xCU << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_13                  ((uint32_t)(0xDU << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_14                  ((uint32_t)(0xEU << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_DIO_15                  ((uint32_t)(0xFU << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_CONST_LOW               ((uint32_t)(0x10U << DIO_NMI_SRC_NMI_Pos))
#define NMI_SRC_CONST_HIGH              ((uint32_t)(0x11U << DIO_NMI_SRC_NMI_Pos))

#define NMI_ACTIVE_LOW_BITBAND          0x0
#define NMI_ACTIVE_HIGH_BITBAND         0x1
#define NMI_ACTIVE_LOW                  ((uint32_t)(NMI_ACTIVE_LOW_BITBAND << DIO_NMI_SRC_NMI_POLARITY_Pos))
#define NMI_ACTIVE_HIGH                 ((uint32_t)(NMI_ACTIVE_HIGH_BITBAND << DIO_NMI_SRC_NMI_POLARITY_Pos))

/* DIO_BB_RX_SRC bit positions */
#define DIO_BB_RX_SRC_RF_SYNC_P_Pos     16
#define DIO_BB_RX_SRC_RF_SYNC_P_Mask    ((uint32_t)(0x1FU << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define DIO_BB_RX_SRC_CLK_Pos           8
#define DIO_BB_RX_SRC_CLK_Mask          ((uint32_t)(0x1FU << DIO_BB_RX_SRC_CLK_Pos))
#define DIO_BB_RX_SRC_DATA_Pos          0
#define DIO_BB_RX_SRC_DATA_Mask         ((uint32_t)(0x1FU << DIO_BB_RX_SRC_DATA_Pos))

/* DIO_BB_RX_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DATA_BYTE;            
    __IO uint8_t CLK_BYTE;             
    __IO uint8_t RF_SYNC_P_BYTE;       
         uint8_t RESERVED0[1];
} DIO_BB_RX_SRC_Type;

#define DIO_BB_RX_SRC_BASE              0x4000077C
#define DIO_BB_RX_SRC                   ((DIO_BB_RX_SRC_Type *) DIO_BB_RX_SRC_BASE)

/* DIO_BB_RX_SRC settings */
#define BB_RX_DATA_SRC_DIO_0            ((uint32_t)(0x0U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_1            ((uint32_t)(0x1U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_2            ((uint32_t)(0x2U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_3            ((uint32_t)(0x3U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_4            ((uint32_t)(0x4U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_5            ((uint32_t)(0x5U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_6            ((uint32_t)(0x6U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_7            ((uint32_t)(0x7U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_8            ((uint32_t)(0x8U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_9            ((uint32_t)(0x9U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_10           ((uint32_t)(0xAU << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_11           ((uint32_t)(0xBU << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_12           ((uint32_t)(0xCU << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_13           ((uint32_t)(0xDU << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_14           ((uint32_t)(0xEU << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_DIO_15           ((uint32_t)(0xFU << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_CONST_LOW        ((uint32_t)(0x10U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_CONST_HIGH       ((uint32_t)(0x11U << DIO_BB_RX_SRC_DATA_Pos))
#define BB_RX_DATA_SRC_RF_GPIO0         ((uint32_t)(0x12U << DIO_BB_RX_SRC_DATA_Pos))

#define BB_RX_CLK_SRC_DIO_0             ((uint32_t)(0x0U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_1             ((uint32_t)(0x1U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_2             ((uint32_t)(0x2U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_3             ((uint32_t)(0x3U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_4             ((uint32_t)(0x4U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_5             ((uint32_t)(0x5U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_6             ((uint32_t)(0x6U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_7             ((uint32_t)(0x7U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_8             ((uint32_t)(0x8U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_9             ((uint32_t)(0x9U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_10            ((uint32_t)(0xAU << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_11            ((uint32_t)(0xBU << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_12            ((uint32_t)(0xCU << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_13            ((uint32_t)(0xDU << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_14            ((uint32_t)(0xEU << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_DIO_15            ((uint32_t)(0xFU << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_CONST_LOW         ((uint32_t)(0x10U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_CONST_HIGH        ((uint32_t)(0x11U << DIO_BB_RX_SRC_CLK_Pos))
#define BB_RX_CLK_SRC_RF_GPIO1          ((uint32_t)(0x12U << DIO_BB_RX_SRC_CLK_Pos))

#define BB_RF_SYNC_P_SRC_DIO_0          ((uint32_t)(0x0U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_1          ((uint32_t)(0x1U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_2          ((uint32_t)(0x2U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_3          ((uint32_t)(0x3U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_4          ((uint32_t)(0x4U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_5          ((uint32_t)(0x5U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_6          ((uint32_t)(0x6U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_7          ((uint32_t)(0x7U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_8          ((uint32_t)(0x8U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_9          ((uint32_t)(0x9U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_10         ((uint32_t)(0xAU << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_11         ((uint32_t)(0xBU << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_12         ((uint32_t)(0xCU << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_13         ((uint32_t)(0xDU << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_14         ((uint32_t)(0xEU << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_DIO_15         ((uint32_t)(0xFU << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_CONST_LOW      ((uint32_t)(0x10U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_CONST_HIGH     ((uint32_t)(0x11U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))
#define BB_RF_SYNC_P_SRC_RF             ((uint32_t)(0x12U << DIO_BB_RX_SRC_RF_SYNC_P_Pos))

/* DIO_BB_RX_SRC sub-register bit positions */
#define DIO_BB_RX_SRC_DATA_BYTE_Pos     0
#define DIO_BB_RX_SRC_DATA_BYTE_Mask    ((uint32_t)(0x1FU << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define DIO_BB_RX_SRC_CLK_BYTE_Pos      0
#define DIO_BB_RX_SRC_CLK_BYTE_Mask     ((uint32_t)(0x1FU << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos 0
#define DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Mask ((uint32_t)(0x1FU << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))

/* DIO_BB_RX_SRC sub-register settings */
#define BB_RX_DATA_SRC_DIO_0_BYTE       ((uint8_t)(0x0U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_1_BYTE       ((uint8_t)(0x1U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_2_BYTE       ((uint8_t)(0x2U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_3_BYTE       ((uint8_t)(0x3U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_4_BYTE       ((uint8_t)(0x4U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_5_BYTE       ((uint8_t)(0x5U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_6_BYTE       ((uint8_t)(0x6U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_7_BYTE       ((uint8_t)(0x7U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_8_BYTE       ((uint8_t)(0x8U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_9_BYTE       ((uint8_t)(0x9U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_10_BYTE      ((uint8_t)(0xAU << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_11_BYTE      ((uint8_t)(0xBU << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_12_BYTE      ((uint8_t)(0xCU << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_13_BYTE      ((uint8_t)(0xDU << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_14_BYTE      ((uint8_t)(0xEU << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_DIO_15_BYTE      ((uint8_t)(0xFU << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_CONST_LOW_BYTE   ((uint8_t)(0x10U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_CONST_HIGH_BYTE  ((uint8_t)(0x11U << DIO_BB_RX_SRC_DATA_BYTE_Pos))
#define BB_RX_DATA_SRC_RF_GPIO0_BYTE    ((uint8_t)(0x12U << DIO_BB_RX_SRC_DATA_BYTE_Pos))

#define BB_RX_CLK_SRC_DIO_0_BYTE        ((uint8_t)(0x0U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_1_BYTE        ((uint8_t)(0x1U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_2_BYTE        ((uint8_t)(0x2U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_3_BYTE        ((uint8_t)(0x3U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_4_BYTE        ((uint8_t)(0x4U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_5_BYTE        ((uint8_t)(0x5U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_6_BYTE        ((uint8_t)(0x6U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_7_BYTE        ((uint8_t)(0x7U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_8_BYTE        ((uint8_t)(0x8U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_9_BYTE        ((uint8_t)(0x9U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_10_BYTE       ((uint8_t)(0xAU << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_11_BYTE       ((uint8_t)(0xBU << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_12_BYTE       ((uint8_t)(0xCU << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_13_BYTE       ((uint8_t)(0xDU << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_14_BYTE       ((uint8_t)(0xEU << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_DIO_15_BYTE       ((uint8_t)(0xFU << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_CONST_LOW_BYTE    ((uint8_t)(0x10U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_CONST_HIGH_BYTE   ((uint8_t)(0x11U << DIO_BB_RX_SRC_CLK_BYTE_Pos))
#define BB_RX_CLK_SRC_RF_GPIO1_BYTE     ((uint8_t)(0x12U << DIO_BB_RX_SRC_CLK_BYTE_Pos))

#define BB_RF_SYNC_P_SRC_DIO_0_BYTE     ((uint8_t)(0x0U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_1_BYTE     ((uint8_t)(0x1U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_2_BYTE     ((uint8_t)(0x2U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_3_BYTE     ((uint8_t)(0x3U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_4_BYTE     ((uint8_t)(0x4U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_5_BYTE     ((uint8_t)(0x5U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_6_BYTE     ((uint8_t)(0x6U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_7_BYTE     ((uint8_t)(0x7U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_8_BYTE     ((uint8_t)(0x8U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_9_BYTE     ((uint8_t)(0x9U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_10_BYTE    ((uint8_t)(0xAU << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_11_BYTE    ((uint8_t)(0xBU << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_12_BYTE    ((uint8_t)(0xCU << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_13_BYTE    ((uint8_t)(0xDU << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_14_BYTE    ((uint8_t)(0xEU << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_DIO_15_BYTE    ((uint8_t)(0xFU << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_CONST_LOW_BYTE ((uint8_t)(0x10U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_CONST_HIGH_BYTE ((uint8_t)(0x11U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))
#define BB_RF_SYNC_P_SRC_RF_BYTE        ((uint8_t)(0x12U << DIO_BB_RX_SRC_RF_SYNC_P_BYTE_Pos))

/* DIO_BB_SPI_SRC bit positions */
#define DIO_BB_SPI_SRC_MISO_Pos         0
#define DIO_BB_SPI_SRC_MISO_Mask        ((uint32_t)(0x1FU << DIO_BB_SPI_SRC_MISO_Pos))

/* DIO_BB_SPI_SRC settings */
#define BB_SPI_MISO_SRC_DIO_0           ((uint32_t)(0x0U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_1           ((uint32_t)(0x1U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_2           ((uint32_t)(0x2U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_3           ((uint32_t)(0x3U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_4           ((uint32_t)(0x4U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_5           ((uint32_t)(0x5U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_6           ((uint32_t)(0x6U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_7           ((uint32_t)(0x7U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_8           ((uint32_t)(0x8U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_9           ((uint32_t)(0x9U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_10          ((uint32_t)(0xAU << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_11          ((uint32_t)(0xBU << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_12          ((uint32_t)(0xCU << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_13          ((uint32_t)(0xDU << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_14          ((uint32_t)(0xEU << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_DIO_15          ((uint32_t)(0xFU << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_CONST_LOW       ((uint32_t)(0x10U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_CONST_HIGH      ((uint32_t)(0x11U << DIO_BB_SPI_SRC_MISO_Pos))
#define BB_SPI_MISO_SRC_RF_SPI_MISO     ((uint32_t)(0x12U << DIO_BB_SPI_SRC_MISO_Pos))

/* DIO_RF_SPI_SRC bit positions */
#define DIO_RF_SPI_SRC_MOSI_Pos         16
#define DIO_RF_SPI_SRC_MOSI_Mask        ((uint32_t)(0x1FU << DIO_RF_SPI_SRC_MOSI_Pos))
#define DIO_RF_SPI_SRC_CSN_Pos          8
#define DIO_RF_SPI_SRC_CSN_Mask         ((uint32_t)(0x1FU << DIO_RF_SPI_SRC_CSN_Pos))
#define DIO_RF_SPI_SRC_CLK_Pos          0
#define DIO_RF_SPI_SRC_CLK_Mask         ((uint32_t)(0x1FU << DIO_RF_SPI_SRC_CLK_Pos))

/* DIO_RF_SPI_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t CLK_BYTE;             
    __IO uint8_t CSN_BYTE;             
    __IO uint8_t MOSI_BYTE;            
         uint8_t RESERVED0[1];
} DIO_RF_SPI_SRC_Type;

#define DIO_RF_SPI_SRC_BASE             0x40000784
#define DIO_RF_SPI_SRC                  ((DIO_RF_SPI_SRC_Type *) DIO_RF_SPI_SRC_BASE)

/* DIO_RF_SPI_SRC settings */
#define RF_SPI_CLK_SRC_DIO_0            ((uint32_t)(0x0U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_1            ((uint32_t)(0x1U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_2            ((uint32_t)(0x2U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_3            ((uint32_t)(0x3U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_4            ((uint32_t)(0x4U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_5            ((uint32_t)(0x5U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_6            ((uint32_t)(0x6U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_7            ((uint32_t)(0x7U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_8            ((uint32_t)(0x8U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_9            ((uint32_t)(0x9U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_10           ((uint32_t)(0xAU << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_11           ((uint32_t)(0xBU << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_12           ((uint32_t)(0xCU << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_13           ((uint32_t)(0xDU << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_14           ((uint32_t)(0xEU << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_DIO_15           ((uint32_t)(0xFU << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_CONST_LOW        ((uint32_t)(0x10U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_CONST_HIGH       ((uint32_t)(0x11U << DIO_RF_SPI_SRC_CLK_Pos))
#define RF_SPI_CLK_SRC_BB_SPI_CLK       ((uint32_t)(0x12U << DIO_RF_SPI_SRC_CLK_Pos))

#define RF_SPI_CSN_SRC_DIO_0            ((uint32_t)(0x0U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_1            ((uint32_t)(0x1U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_2            ((uint32_t)(0x2U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_3            ((uint32_t)(0x3U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_4            ((uint32_t)(0x4U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_5            ((uint32_t)(0x5U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_6            ((uint32_t)(0x6U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_7            ((uint32_t)(0x7U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_8            ((uint32_t)(0x8U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_9            ((uint32_t)(0x9U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_10           ((uint32_t)(0xAU << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_11           ((uint32_t)(0xBU << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_12           ((uint32_t)(0xCU << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_13           ((uint32_t)(0xDU << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_14           ((uint32_t)(0xEU << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_DIO_15           ((uint32_t)(0xFU << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_CONST_LOW        ((uint32_t)(0x10U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_CONST_HIGH       ((uint32_t)(0x11U << DIO_RF_SPI_SRC_CSN_Pos))
#define RF_SPI_CSN_SRC_BB_SPI_CSN       ((uint32_t)(0x12U << DIO_RF_SPI_SRC_CSN_Pos))

#define RF_SPI_MOSI_SRC_DIO_0           ((uint32_t)(0x0U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_1           ((uint32_t)(0x1U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_2           ((uint32_t)(0x2U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_3           ((uint32_t)(0x3U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_4           ((uint32_t)(0x4U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_5           ((uint32_t)(0x5U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_6           ((uint32_t)(0x6U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_7           ((uint32_t)(0x7U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_8           ((uint32_t)(0x8U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_9           ((uint32_t)(0x9U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_10          ((uint32_t)(0xAU << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_11          ((uint32_t)(0xBU << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_12          ((uint32_t)(0xCU << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_13          ((uint32_t)(0xDU << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_14          ((uint32_t)(0xEU << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_DIO_15          ((uint32_t)(0xFU << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_CONST_LOW       ((uint32_t)(0x10U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_CONST_HIGH      ((uint32_t)(0x11U << DIO_RF_SPI_SRC_MOSI_Pos))
#define RF_SPI_MOSI_SRC_BB_SPI_MOSI     ((uint32_t)(0x12U << DIO_RF_SPI_SRC_MOSI_Pos))

/* DIO_RF_SPI_SRC sub-register bit positions */
#define DIO_RF_SPI_SRC_CLK_BYTE_Pos     0
#define DIO_RF_SPI_SRC_CLK_BYTE_Mask    ((uint32_t)(0x1FU << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define DIO_RF_SPI_SRC_CSN_BYTE_Pos     0
#define DIO_RF_SPI_SRC_CSN_BYTE_Mask    ((uint32_t)(0x1FU << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define DIO_RF_SPI_SRC_MOSI_BYTE_Pos    0
#define DIO_RF_SPI_SRC_MOSI_BYTE_Mask   ((uint32_t)(0x1FU << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))

/* DIO_RF_SPI_SRC sub-register settings */
#define RF_SPI_CLK_SRC_DIO_0_BYTE       ((uint8_t)(0x0U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_1_BYTE       ((uint8_t)(0x1U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_2_BYTE       ((uint8_t)(0x2U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_3_BYTE       ((uint8_t)(0x3U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_4_BYTE       ((uint8_t)(0x4U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_5_BYTE       ((uint8_t)(0x5U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_6_BYTE       ((uint8_t)(0x6U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_7_BYTE       ((uint8_t)(0x7U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_8_BYTE       ((uint8_t)(0x8U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_9_BYTE       ((uint8_t)(0x9U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_10_BYTE      ((uint8_t)(0xAU << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_11_BYTE      ((uint8_t)(0xBU << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_12_BYTE      ((uint8_t)(0xCU << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_13_BYTE      ((uint8_t)(0xDU << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_14_BYTE      ((uint8_t)(0xEU << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_DIO_15_BYTE      ((uint8_t)(0xFU << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_CONST_LOW_BYTE   ((uint8_t)(0x10U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_CONST_HIGH_BYTE  ((uint8_t)(0x11U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))
#define RF_SPI_CLK_SRC_BB_SPI_CLK_BYTE  ((uint8_t)(0x12U << DIO_RF_SPI_SRC_CLK_BYTE_Pos))

#define RF_SPI_CSN_SRC_DIO_0_BYTE       ((uint8_t)(0x0U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_1_BYTE       ((uint8_t)(0x1U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_2_BYTE       ((uint8_t)(0x2U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_3_BYTE       ((uint8_t)(0x3U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_4_BYTE       ((uint8_t)(0x4U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_5_BYTE       ((uint8_t)(0x5U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_6_BYTE       ((uint8_t)(0x6U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_7_BYTE       ((uint8_t)(0x7U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_8_BYTE       ((uint8_t)(0x8U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_9_BYTE       ((uint8_t)(0x9U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_10_BYTE      ((uint8_t)(0xAU << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_11_BYTE      ((uint8_t)(0xBU << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_12_BYTE      ((uint8_t)(0xCU << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_13_BYTE      ((uint8_t)(0xDU << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_14_BYTE      ((uint8_t)(0xEU << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_DIO_15_BYTE      ((uint8_t)(0xFU << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_CONST_LOW_BYTE   ((uint8_t)(0x10U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_CONST_HIGH_BYTE  ((uint8_t)(0x11U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))
#define RF_SPI_CSN_SRC_BB_SPI_CSN_BYTE  ((uint8_t)(0x12U << DIO_RF_SPI_SRC_CSN_BYTE_Pos))

#define RF_SPI_MOSI_SRC_DIO_0_BYTE      ((uint8_t)(0x0U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_1_BYTE      ((uint8_t)(0x1U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_2_BYTE      ((uint8_t)(0x2U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_3_BYTE      ((uint8_t)(0x3U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_4_BYTE      ((uint8_t)(0x4U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_5_BYTE      ((uint8_t)(0x5U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_6_BYTE      ((uint8_t)(0x6U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_7_BYTE      ((uint8_t)(0x7U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_8_BYTE      ((uint8_t)(0x8U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_9_BYTE      ((uint8_t)(0x9U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_10_BYTE     ((uint8_t)(0xAU << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_11_BYTE     ((uint8_t)(0xBU << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_12_BYTE     ((uint8_t)(0xCU << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_13_BYTE     ((uint8_t)(0xDU << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_14_BYTE     ((uint8_t)(0xEU << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_DIO_15_BYTE     ((uint8_t)(0xFU << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_CONST_LOW_BYTE  ((uint8_t)(0x10U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_CONST_HIGH_BYTE ((uint8_t)(0x11U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))
#define RF_SPI_MOSI_SRC_BB_SPI_MOSI_BYTE ((uint8_t)(0x12U << DIO_RF_SPI_SRC_MOSI_BYTE_Pos))

/* DIO_RF_GPIO03_SRC bit positions */
#define DIO_RF_GPIO03_SRC_GPIO3_Pos     24
#define DIO_RF_GPIO03_SRC_GPIO3_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define DIO_RF_GPIO03_SRC_GPIO2_Pos     16
#define DIO_RF_GPIO03_SRC_GPIO2_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define DIO_RF_GPIO03_SRC_GPIO1_Pos     8
#define DIO_RF_GPIO03_SRC_GPIO1_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define DIO_RF_GPIO03_SRC_GPIO0_Pos     0
#define DIO_RF_GPIO03_SRC_GPIO0_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO03_SRC_GPIO0_Pos))

/* DIO_RF_GPIO03_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t GPIO0_BYTE;           
    __IO uint8_t GPIO1_BYTE;           
    __IO uint8_t GPIO2_BYTE;           
    __IO uint8_t GPIO3_BYTE;           
} DIO_RF_GPIO03_SRC_Type;

#define DIO_RF_GPIO03_SRC_BASE          0x40000788
#define DIO_RF_GPIO03_SRC               ((DIO_RF_GPIO03_SRC_Type *) DIO_RF_GPIO03_SRC_BASE)

/* DIO_RF_GPIO03_SRC settings */
#define RF_GPIO0_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO03_SRC_GPIO0_Pos))
#define RF_GPIO0_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO03_SRC_GPIO0_Pos))

#define RF_GPIO1_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO03_SRC_GPIO1_Pos))
#define RF_GPIO1_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO03_SRC_GPIO1_Pos))

#define RF_GPIO2_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO03_SRC_GPIO2_Pos))
#define RF_GPIO2_SRC_BB_SYNC_P          ((uint32_t)(0x12U << DIO_RF_GPIO03_SRC_GPIO2_Pos))

#define RF_GPIO3_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO03_SRC_GPIO3_Pos))
#define RF_GPIO3_SRC_BB_TX_DATA         ((uint32_t)(0x12U << DIO_RF_GPIO03_SRC_GPIO3_Pos))

/* DIO_RF_GPIO03_SRC sub-register bit positions */
#define DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos 0
#define DIO_RF_GPIO03_SRC_GPIO0_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos 0
#define DIO_RF_GPIO03_SRC_GPIO1_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos 0
#define DIO_RF_GPIO03_SRC_GPIO2_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos 0
#define DIO_RF_GPIO03_SRC_GPIO3_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))

/* DIO_RF_GPIO03_SRC sub-register settings */
#define RF_GPIO0_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))
#define RF_GPIO0_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO03_SRC_GPIO0_BYTE_Pos))

#define RF_GPIO1_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))
#define RF_GPIO1_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO03_SRC_GPIO1_BYTE_Pos))

#define RF_GPIO2_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))
#define RF_GPIO2_SRC_BB_SYNC_P_BYTE     ((uint8_t)(0x12U << DIO_RF_GPIO03_SRC_GPIO2_BYTE_Pos))

#define RF_GPIO3_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))
#define RF_GPIO3_SRC_BB_TX_DATA_BYTE    ((uint8_t)(0x12U << DIO_RF_GPIO03_SRC_GPIO3_BYTE_Pos))

/* DIO_RF_GPIO47_SRC bit positions */
#define DIO_RF_GPIO47_SRC_GPIO7_Pos     24
#define DIO_RF_GPIO47_SRC_GPIO7_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define DIO_RF_GPIO47_SRC_GPIO6_Pos     16
#define DIO_RF_GPIO47_SRC_GPIO6_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define DIO_RF_GPIO47_SRC_GPIO5_Pos     8
#define DIO_RF_GPIO47_SRC_GPIO5_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define DIO_RF_GPIO47_SRC_GPIO4_Pos     0
#define DIO_RF_GPIO47_SRC_GPIO4_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO47_SRC_GPIO4_Pos))

/* DIO_RF_GPIO47_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t GPIO4_BYTE;           
    __IO uint8_t GPIO5_BYTE;           
    __IO uint8_t GPIO6_BYTE;           
    __IO uint8_t GPIO7_BYTE;           
} DIO_RF_GPIO47_SRC_Type;

#define DIO_RF_GPIO47_SRC_BASE          0x4000078C
#define DIO_RF_GPIO47_SRC               ((DIO_RF_GPIO47_SRC_Type *) DIO_RF_GPIO47_SRC_BASE)

/* DIO_RF_GPIO47_SRC settings */
#define RF_GPIO4_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO47_SRC_GPIO4_Pos))
#define RF_GPIO4_SRC_BB_TX_DATA_VALID   ((uint32_t)(0x12U << DIO_RF_GPIO47_SRC_GPIO4_Pos))

#define RF_GPIO5_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO47_SRC_GPIO5_Pos))
#define RF_GPIO5_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO47_SRC_GPIO5_Pos))

#define RF_GPIO6_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO47_SRC_GPIO6_Pos))
#define RF_GPIO6_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO47_SRC_GPIO6_Pos))

#define RF_GPIO7_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO47_SRC_GPIO7_Pos))
#define RF_GPIO7_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO47_SRC_GPIO7_Pos))

/* DIO_RF_GPIO47_SRC sub-register bit positions */
#define DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos 0
#define DIO_RF_GPIO47_SRC_GPIO4_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos 0
#define DIO_RF_GPIO47_SRC_GPIO5_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos 0
#define DIO_RF_GPIO47_SRC_GPIO6_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos 0
#define DIO_RF_GPIO47_SRC_GPIO7_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))

/* DIO_RF_GPIO47_SRC sub-register settings */
#define RF_GPIO4_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))
#define RF_GPIO4_SRC_BB_TX_DATA_VALID_BYTE ((uint8_t)(0x12U << DIO_RF_GPIO47_SRC_GPIO4_BYTE_Pos))

#define RF_GPIO5_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))
#define RF_GPIO5_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO47_SRC_GPIO5_BYTE_Pos))

#define RF_GPIO6_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))
#define RF_GPIO6_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO47_SRC_GPIO6_BYTE_Pos))

#define RF_GPIO7_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))
#define RF_GPIO7_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO47_SRC_GPIO7_BYTE_Pos))

/* DIO_RF_GPIO89_SRC bit positions */
#define DIO_RF_GPIO89_SRC_GPIO9_Pos     8
#define DIO_RF_GPIO89_SRC_GPIO9_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define DIO_RF_GPIO89_SRC_GPIO8_Pos     0
#define DIO_RF_GPIO89_SRC_GPIO8_Mask    ((uint32_t)(0x1FU << DIO_RF_GPIO89_SRC_GPIO8_Pos))

/* DIO_RF_GPIO89_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t GPIO8_BYTE;           
    __IO uint8_t GPIO9_BYTE;           
         uint8_t RESERVED0[2];
} DIO_RF_GPIO89_SRC_Type;

#define DIO_RF_GPIO89_SRC_BASE          0x40000790
#define DIO_RF_GPIO89_SRC               ((DIO_RF_GPIO89_SRC_Type *) DIO_RF_GPIO89_SRC_BASE)

/* DIO_RF_GPIO89_SRC settings */
#define RF_GPIO8_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO89_SRC_GPIO8_Pos))
#define RF_GPIO8_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO89_SRC_GPIO8_Pos))

#define RF_GPIO9_SRC_DIO_0              ((uint32_t)(0x0U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_1              ((uint32_t)(0x1U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_2              ((uint32_t)(0x2U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_3              ((uint32_t)(0x3U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_4              ((uint32_t)(0x4U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_5              ((uint32_t)(0x5U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_6              ((uint32_t)(0x6U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_7              ((uint32_t)(0x7U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_8              ((uint32_t)(0x8U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_9              ((uint32_t)(0x9U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_10             ((uint32_t)(0xAU << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_11             ((uint32_t)(0xBU << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_12             ((uint32_t)(0xCU << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_13             ((uint32_t)(0xDU << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_14             ((uint32_t)(0xEU << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_DIO_15             ((uint32_t)(0xFU << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_RF_GPIO89_SRC_GPIO9_Pos))
#define RF_GPIO9_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_RF_GPIO89_SRC_GPIO9_Pos))

/* DIO_RF_GPIO89_SRC sub-register bit positions */
#define DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos 0
#define DIO_RF_GPIO89_SRC_GPIO8_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos 0
#define DIO_RF_GPIO89_SRC_GPIO9_BYTE_Mask ((uint32_t)(0x1FU << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))

/* DIO_RF_GPIO89_SRC sub-register settings */
#define RF_GPIO8_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))
#define RF_GPIO8_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO89_SRC_GPIO8_BYTE_Pos))

#define RF_GPIO9_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))
#define RF_GPIO9_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_RF_GPIO89_SRC_GPIO9_BYTE_Pos))

/* DIO_DMIC_SRC bit positions */
#define DIO_DMIC_SRC_CLK_Pos            8
#define DIO_DMIC_SRC_CLK_Mask           ((uint32_t)(0x1FU << DIO_DMIC_SRC_CLK_Pos))
#define DIO_DMIC_SRC_DATA_Pos           0
#define DIO_DMIC_SRC_DATA_Mask          ((uint32_t)(0x1FU << DIO_DMIC_SRC_DATA_Pos))

/* DIO_DMIC_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DATA_BYTE;            
    __IO uint8_t CLK_BYTE;             
         uint8_t RESERVED0[2];
} DIO_DMIC_SRC_Type;

#define DIO_DMIC_SRC_BASE               0x40000794
#define DIO_DMIC_SRC                    ((DIO_DMIC_SRC_Type *) DIO_DMIC_SRC_BASE)

/* DIO_DMIC_SRC settings */
#define DMIC_DATA_SRC_DIO_0             ((uint32_t)(0x0U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_1             ((uint32_t)(0x1U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_2             ((uint32_t)(0x2U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_3             ((uint32_t)(0x3U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_4             ((uint32_t)(0x4U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_5             ((uint32_t)(0x5U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_6             ((uint32_t)(0x6U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_7             ((uint32_t)(0x7U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_8             ((uint32_t)(0x8U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_9             ((uint32_t)(0x9U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_10            ((uint32_t)(0xAU << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_11            ((uint32_t)(0xBU << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_12            ((uint32_t)(0xCU << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_13            ((uint32_t)(0xDU << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_14            ((uint32_t)(0xEU << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_DIO_15            ((uint32_t)(0xFU << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_CONST_LOW         ((uint32_t)(0x10U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_CONST_HIGH        ((uint32_t)(0x11U << DIO_DMIC_SRC_DATA_Pos))
#define DMIC_DATA_SRC_EXTCLK            ((uint32_t)(0x12U << DIO_DMIC_SRC_DATA_Pos))

#define DMIC_CLK_SRC_DIO_0              ((uint32_t)(0x0U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_1              ((uint32_t)(0x1U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_2              ((uint32_t)(0x2U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_3              ((uint32_t)(0x3U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_4              ((uint32_t)(0x4U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_5              ((uint32_t)(0x5U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_6              ((uint32_t)(0x6U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_7              ((uint32_t)(0x7U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_8              ((uint32_t)(0x8U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_9              ((uint32_t)(0x9U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_10             ((uint32_t)(0xAU << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_11             ((uint32_t)(0xBU << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_12             ((uint32_t)(0xCU << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_13             ((uint32_t)(0xDU << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_14             ((uint32_t)(0xEU << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_DIO_15             ((uint32_t)(0xFU << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_CONST_LOW          ((uint32_t)(0x10U << DIO_DMIC_SRC_CLK_Pos))
#define DMIC_CLK_SRC_CONST_HIGH         ((uint32_t)(0x11U << DIO_DMIC_SRC_CLK_Pos))

/* DIO_DMIC_SRC sub-register bit positions */
#define DIO_DMIC_SRC_DATA_BYTE_Pos      0
#define DIO_DMIC_SRC_DATA_BYTE_Mask     ((uint32_t)(0x1FU << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DIO_DMIC_SRC_CLK_BYTE_Pos       0
#define DIO_DMIC_SRC_CLK_BYTE_Mask      ((uint32_t)(0x1FU << DIO_DMIC_SRC_CLK_BYTE_Pos))

/* DIO_DMIC_SRC sub-register settings */
#define DMIC_DATA_SRC_DIO_0_BYTE        ((uint8_t)(0x0U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_1_BYTE        ((uint8_t)(0x1U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_2_BYTE        ((uint8_t)(0x2U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_3_BYTE        ((uint8_t)(0x3U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_4_BYTE        ((uint8_t)(0x4U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_5_BYTE        ((uint8_t)(0x5U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_6_BYTE        ((uint8_t)(0x6U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_7_BYTE        ((uint8_t)(0x7U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_8_BYTE        ((uint8_t)(0x8U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_9_BYTE        ((uint8_t)(0x9U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_10_BYTE       ((uint8_t)(0xAU << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_11_BYTE       ((uint8_t)(0xBU << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_12_BYTE       ((uint8_t)(0xCU << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_13_BYTE       ((uint8_t)(0xDU << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_14_BYTE       ((uint8_t)(0xEU << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_DIO_15_BYTE       ((uint8_t)(0xFU << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_CONST_LOW_BYTE    ((uint8_t)(0x10U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_CONST_HIGH_BYTE   ((uint8_t)(0x11U << DIO_DMIC_SRC_DATA_BYTE_Pos))
#define DMIC_DATA_SRC_EXTCLK_BYTE       ((uint8_t)(0x12U << DIO_DMIC_SRC_DATA_BYTE_Pos))

#define DMIC_CLK_SRC_DIO_0_BYTE         ((uint8_t)(0x0U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_1_BYTE         ((uint8_t)(0x1U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_2_BYTE         ((uint8_t)(0x2U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_3_BYTE         ((uint8_t)(0x3U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_4_BYTE         ((uint8_t)(0x4U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_5_BYTE         ((uint8_t)(0x5U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_6_BYTE         ((uint8_t)(0x6U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_7_BYTE         ((uint8_t)(0x7U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_8_BYTE         ((uint8_t)(0x8U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_9_BYTE         ((uint8_t)(0x9U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_10_BYTE        ((uint8_t)(0xAU << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_11_BYTE        ((uint8_t)(0xBU << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_12_BYTE        ((uint8_t)(0xCU << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_13_BYTE        ((uint8_t)(0xDU << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_14_BYTE        ((uint8_t)(0xEU << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_DIO_15_BYTE        ((uint8_t)(0xFU << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_CONST_LOW_BYTE     ((uint8_t)(0x10U << DIO_DMIC_SRC_CLK_BYTE_Pos))
#define DMIC_CLK_SRC_CONST_HIGH_BYTE    ((uint8_t)(0x11U << DIO_DMIC_SRC_CLK_BYTE_Pos))

/* DIO_LPDSP32_JTAG_SRC bit positions */
#define DIO_LPDSP32_JTAG_SRC_TDI_Pos    16
#define DIO_LPDSP32_JTAG_SRC_TDI_Mask   ((uint32_t)(0x1FU << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define DIO_LPDSP32_JTAG_SRC_TMS_Pos    8
#define DIO_LPDSP32_JTAG_SRC_TMS_Mask   ((uint32_t)(0x1FU << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define DIO_LPDSP32_JTAG_SRC_TCK_Pos    0
#define DIO_LPDSP32_JTAG_SRC_TCK_Mask   ((uint32_t)(0x1FU << DIO_LPDSP32_JTAG_SRC_TCK_Pos))

/* DIO_LPDSP32_JTAG_SRC sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TCK_BYTE;             
    __IO uint8_t TMS_BYTE;             
    __IO uint8_t TDI_BYTE;             
         uint8_t RESERVED0[1];
} DIO_LPDSP32_JTAG_SRC_Type;

#define DIO_LPDSP32_JTAG_SRC_BASE       0x40000798
#define DIO_LPDSP32_JTAG_SRC            ((DIO_LPDSP32_JTAG_SRC_Type *) DIO_LPDSP32_JTAG_SRC_BASE)

/* DIO_LPDSP32_JTAG_SRC settings */
#define LPDSP32_TCK_SRC_DIO_0           ((uint32_t)(0x0U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_1           ((uint32_t)(0x1U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_2           ((uint32_t)(0x2U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_3           ((uint32_t)(0x3U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_4           ((uint32_t)(0x4U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_5           ((uint32_t)(0x5U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_6           ((uint32_t)(0x6U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_7           ((uint32_t)(0x7U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_8           ((uint32_t)(0x8U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_9           ((uint32_t)(0x9U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_10          ((uint32_t)(0xAU << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_11          ((uint32_t)(0xBU << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_12          ((uint32_t)(0xCU << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_13          ((uint32_t)(0xDU << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_14          ((uint32_t)(0xEU << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_DIO_15          ((uint32_t)(0xFU << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_CONST_LOW       ((uint32_t)(0x10U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))
#define LPDSP32_TCK_SRC_CONST_HIGH      ((uint32_t)(0x11U << DIO_LPDSP32_JTAG_SRC_TCK_Pos))

#define LPDSP32_TMS_SRC_DIO_0           ((uint32_t)(0x0U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_1           ((uint32_t)(0x1U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_2           ((uint32_t)(0x2U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_3           ((uint32_t)(0x3U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_4           ((uint32_t)(0x4U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_5           ((uint32_t)(0x5U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_6           ((uint32_t)(0x6U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_7           ((uint32_t)(0x7U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_8           ((uint32_t)(0x8U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_9           ((uint32_t)(0x9U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_10          ((uint32_t)(0xAU << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_11          ((uint32_t)(0xBU << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_12          ((uint32_t)(0xCU << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_13          ((uint32_t)(0xDU << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_14          ((uint32_t)(0xEU << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_DIO_15          ((uint32_t)(0xFU << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_CONST_LOW       ((uint32_t)(0x10U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))
#define LPDSP32_TMS_SRC_CONST_HIGH      ((uint32_t)(0x11U << DIO_LPDSP32_JTAG_SRC_TMS_Pos))

#define LPDSP32_TDI_SRC_DIO_0           ((uint32_t)(0x0U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_1           ((uint32_t)(0x1U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_2           ((uint32_t)(0x2U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_3           ((uint32_t)(0x3U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_4           ((uint32_t)(0x4U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_5           ((uint32_t)(0x5U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_6           ((uint32_t)(0x6U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_7           ((uint32_t)(0x7U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_8           ((uint32_t)(0x8U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_9           ((uint32_t)(0x9U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_10          ((uint32_t)(0xAU << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_11          ((uint32_t)(0xBU << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_12          ((uint32_t)(0xCU << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_13          ((uint32_t)(0xDU << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_14          ((uint32_t)(0xEU << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_DIO_15          ((uint32_t)(0xFU << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_CONST_LOW       ((uint32_t)(0x10U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))
#define LPDSP32_TDI_SRC_CONST_HIGH      ((uint32_t)(0x11U << DIO_LPDSP32_JTAG_SRC_TDI_Pos))

/* DIO_LPDSP32_JTAG_SRC sub-register bit positions */
#define DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos 0
#define DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Mask ((uint32_t)(0x1FU << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos 0
#define DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Mask ((uint32_t)(0x1FU << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos 0
#define DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Mask ((uint32_t)(0x1FU << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))

/* DIO_LPDSP32_JTAG_SRC sub-register settings */
#define LPDSP32_TCK_SRC_DIO_0_BYTE      ((uint8_t)(0x0U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_1_BYTE      ((uint8_t)(0x1U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_2_BYTE      ((uint8_t)(0x2U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_3_BYTE      ((uint8_t)(0x3U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_4_BYTE      ((uint8_t)(0x4U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_5_BYTE      ((uint8_t)(0x5U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_6_BYTE      ((uint8_t)(0x6U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_7_BYTE      ((uint8_t)(0x7U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_8_BYTE      ((uint8_t)(0x8U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_9_BYTE      ((uint8_t)(0x9U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_10_BYTE     ((uint8_t)(0xAU << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_11_BYTE     ((uint8_t)(0xBU << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_12_BYTE     ((uint8_t)(0xCU << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_13_BYTE     ((uint8_t)(0xDU << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_14_BYTE     ((uint8_t)(0xEU << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_DIO_15_BYTE     ((uint8_t)(0xFU << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_CONST_LOW_BYTE  ((uint8_t)(0x10U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))
#define LPDSP32_TCK_SRC_CONST_HIGH_BYTE ((uint8_t)(0x11U << DIO_LPDSP32_JTAG_SRC_TCK_BYTE_Pos))

#define LPDSP32_TMS_SRC_DIO_0_BYTE      ((uint8_t)(0x0U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_1_BYTE      ((uint8_t)(0x1U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_2_BYTE      ((uint8_t)(0x2U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_3_BYTE      ((uint8_t)(0x3U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_4_BYTE      ((uint8_t)(0x4U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_5_BYTE      ((uint8_t)(0x5U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_6_BYTE      ((uint8_t)(0x6U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_7_BYTE      ((uint8_t)(0x7U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_8_BYTE      ((uint8_t)(0x8U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_9_BYTE      ((uint8_t)(0x9U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_10_BYTE     ((uint8_t)(0xAU << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_11_BYTE     ((uint8_t)(0xBU << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_12_BYTE     ((uint8_t)(0xCU << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_13_BYTE     ((uint8_t)(0xDU << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_14_BYTE     ((uint8_t)(0xEU << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_DIO_15_BYTE     ((uint8_t)(0xFU << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_CONST_LOW_BYTE  ((uint8_t)(0x10U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))
#define LPDSP32_TMS_SRC_CONST_HIGH_BYTE ((uint8_t)(0x11U << DIO_LPDSP32_JTAG_SRC_TMS_BYTE_Pos))

#define LPDSP32_TDI_SRC_DIO_0_BYTE      ((uint8_t)(0x0U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_1_BYTE      ((uint8_t)(0x1U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_2_BYTE      ((uint8_t)(0x2U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_3_BYTE      ((uint8_t)(0x3U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_4_BYTE      ((uint8_t)(0x4U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_5_BYTE      ((uint8_t)(0x5U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_6_BYTE      ((uint8_t)(0x6U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_7_BYTE      ((uint8_t)(0x7U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_8_BYTE      ((uint8_t)(0x8U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_9_BYTE      ((uint8_t)(0x9U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_10_BYTE     ((uint8_t)(0xAU << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_11_BYTE     ((uint8_t)(0xBU << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_12_BYTE     ((uint8_t)(0xCU << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_13_BYTE     ((uint8_t)(0xDU << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_14_BYTE     ((uint8_t)(0xEU << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_DIO_15_BYTE     ((uint8_t)(0xFU << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_CONST_LOW_BYTE  ((uint8_t)(0x10U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))
#define LPDSP32_TDI_SRC_CONST_HIGH_BYTE ((uint8_t)(0x11U << DIO_LPDSP32_JTAG_SRC_TDI_BYTE_Pos))

/* DIO_JTAG_SW_PAD_CFG bit positions */
#define DIO_JTAG_SW_PAD_CFG_JTCK_LPF_Pos 9
#define DIO_JTAG_SW_PAD_CFG_JTMS_LPF_Pos 8
#define DIO_JTAG_SW_PAD_CFG_CM3_JTAG_DATA_EN_Pos 7
#define DIO_JTAG_SW_PAD_CFG_CM3_JTAG_TRST_EN_Pos 6
#define DIO_JTAG_SW_PAD_CFG_JTCK_PULL_Pos 4
#define DIO_JTAG_SW_PAD_CFG_JTCK_PULL_Mask ((uint32_t)(0x3U << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_Pos))
#define DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_Pos 2
#define DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_Mask ((uint32_t)(0x3U << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_Pos))
#define DIO_JTAG_SW_PAD_CFG_JTMS_PULL_Pos 0
#define DIO_JTAG_SW_PAD_CFG_JTMS_PULL_Mask ((uint32_t)(0x3U << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_Pos))

/* DIO_JTAG_SW_PAD_CFG bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[6];
    __IO uint32_t CM3_JTAG_TRST_EN_ALIAS;/* CM3 JTAG TRST on DIO13 */
    __IO uint32_t CM3_JTAG_DATA_EN_ALIAS;/* CM3 JTAG on DIO[14:15] */
    __IO uint32_t JTMS_LPF_ALIAS;       /* JTMS Low-Pass-Filter enable / disable */
    __IO uint32_t JTCK_LPF_ALIAS;       /* JTCK Low-Pass-Filter enable / disable */
} DIO_JTAG_SW_PAD_CFG_Type;

#define DIO_JTAG_SW_PAD_CFG_BASE        SYS_CALC_BITBAND(0x4000079C, 0)
#define DIO_JTAG_SW_PAD_CFG             ((DIO_JTAG_SW_PAD_CFG_Type *) DIO_JTAG_SW_PAD_CFG_BASE)

/* DIO_JTAG_SW_PAD_CFG settings */
#define JTMS_NO_PULL                    ((uint32_t)(0x0U << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_Pos))
#define JTMS_WEAK_PULL_UP               ((uint32_t)(0x1U << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_Pos))
#define JTMS_WEAK_PULL_DOWN             ((uint32_t)(0x2U << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_Pos))
#define JTMS_STRONG_PULL_UP             ((uint32_t)(0x3U << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_Pos))

#define JTMS_2X_DRIVE                   ((uint32_t)(0x0U << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_Pos))
#define JTMS_3X_DRIVE                   ((uint32_t)(0x1U << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_Pos))
#define JTMS_5X_DRIVE                   ((uint32_t)(0x2U << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_Pos))
#define JTMS_6X_DRIVE                   ((uint32_t)(0x3U << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_Pos))

#define JTCK_NO_PULL                    ((uint32_t)(0x0U << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_Pos))
#define JTCK_WEAK_PULL_UP               ((uint32_t)(0x1U << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_Pos))
#define JTCK_WEAK_PULL_DOWN             ((uint32_t)(0x2U << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_Pos))
#define JTCK_STRONG_PULL_UP             ((uint32_t)(0x3U << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_Pos))

#define CM3_JTAG_TRST_DISABLED_BITBAND  0x0
#define CM3_JTAG_TRST_ENABLED_BITBAND   0x1
#define CM3_JTAG_TRST_DISABLED          ((uint32_t)(CM3_JTAG_TRST_DISABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_CM3_JTAG_TRST_EN_Pos))
#define CM3_JTAG_TRST_ENABLED           ((uint32_t)(CM3_JTAG_TRST_ENABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_CM3_JTAG_TRST_EN_Pos))

#define CM3_JTAG_DATA_DISABLED_BITBAND  0x0
#define CM3_JTAG_DATA_ENABLED_BITBAND   0x1
#define CM3_JTAG_DATA_DISABLED          ((uint32_t)(CM3_JTAG_DATA_DISABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_CM3_JTAG_DATA_EN_Pos))
#define CM3_JTAG_DATA_ENABLED           ((uint32_t)(CM3_JTAG_DATA_ENABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_CM3_JTAG_DATA_EN_Pos))

#define JTMS_LPF_DISABLED_BITBAND       0x0
#define JTMS_LPF_ENABLED_BITBAND        0x1
#define JTMS_LPF_DISABLED               ((uint32_t)(JTMS_LPF_DISABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_JTMS_LPF_Pos))
#define JTMS_LPF_ENABLED                ((uint32_t)(JTMS_LPF_ENABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_JTMS_LPF_Pos))

#define JTCK_LPF_DISABLED_BITBAND       0x0
#define JTCK_LPF_ENABLED_BITBAND        0x1
#define JTCK_LPF_DISABLED               ((uint32_t)(JTCK_LPF_DISABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_JTCK_LPF_Pos))
#define JTCK_LPF_ENABLED                ((uint32_t)(JTCK_LPF_ENABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_JTCK_LPF_Pos))

/* DIO_EXTCLK_CFG bit positions */
#define DIO_EXTCLK_CFG_LPF_Pos          2
#define DIO_EXTCLK_CFG_PULL_CTRL_Pos    0
#define DIO_EXTCLK_CFG_PULL_CTRL_Mask   ((uint32_t)(0x3U << DIO_EXTCLK_CFG_PULL_CTRL_Pos))

/* DIO_EXTCLK_CFG bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[2];
    __IO uint32_t LPF_ALIAS;            /* Low Pass Filter enable */
} DIO_EXTCLK_CFG_Type;

#define DIO_EXTCLK_CFG_BASE             SYS_CALC_BITBAND(0x400007A0, 0)
#define DIO_EXTCLK_CFG                  ((DIO_EXTCLK_CFG_Type *) DIO_EXTCLK_CFG_BASE)

/* DIO_EXTCLK_CFG settings */
#define EXTCLK_NO_PULL                  ((uint32_t)(0x0U << DIO_EXTCLK_CFG_PULL_CTRL_Pos))
#define EXTCLK_WEAK_PULL_UP             ((uint32_t)(0x1U << DIO_EXTCLK_CFG_PULL_CTRL_Pos))
#define EXTCLK_WEAK_PULL_DOWN           ((uint32_t)(0x2U << DIO_EXTCLK_CFG_PULL_CTRL_Pos))
#define EXTCLK_STRONG_PULL_UP           ((uint32_t)(0x3U << DIO_EXTCLK_CFG_PULL_CTRL_Pos))

#define EXTCLK_LPF_DISABLE_BITBAND      0x0
#define EXTCLK_LPF_ENABLE_BITBAND       0x1
#define EXTCLK_LPF_DISABLE              ((uint32_t)(EXTCLK_LPF_DISABLE_BITBAND << DIO_EXTCLK_CFG_LPF_Pos))
#define EXTCLK_LPF_ENABLE               ((uint32_t)(EXTCLK_LPF_ENABLE_BITBAND << DIO_EXTCLK_CFG_LPF_Pos))

/* DIO_PAD_CFG bit positions */
#define DIO_PAD_CFG_DRIVE_Pos           0

/* DIO_PAD_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t DRIVE_ALIAS;          /* Drive strength configuration (scales the individual drive strengths) */
} DIO_PAD_CFG_Type;

#define DIO_PAD_CFG_BASE                SYS_CALC_BITBAND(0x400007A4, 0)
#define DIO_PAD_CFG                     ((DIO_PAD_CFG_Type *) DIO_PAD_CFG_BASE)

/* DIO_PAD_CFG settings */
#define PAD_LOW_DRIVE_BITBAND           0x0
#define PAD_HIGH_DRIVE_BITBAND          0x1
#define PAD_LOW_DRIVE                   ((uint32_t)(PAD_LOW_DRIVE_BITBAND << DIO_PAD_CFG_DRIVE_Pos))
#define PAD_HIGH_DRIVE                  ((uint32_t)(PAD_HIGH_DRIVE_BITBAND << DIO_PAD_CFG_DRIVE_Pos))

/* ----------------------------------------------------------------------------
 * SPI Interface Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL0;                /* SPI0 Control and Configuration Register */
    __IO uint32_t CTRL1;                /* SPI0 Transaction Control Register */
    __IO uint32_t TX_DATA;              /* SPI0 Transmit Data */
    __I  uint32_t RX_DATA;              /* SPI0 Received Data */
    __IO uint32_t STATUS;               /* SPI0 Status */
} SPI0_Type;

#define SPI0_BASE                       0x40000800
#define SPI0                            ((SPI0_Type *) SPI0_BASE)

/* SPI0_CTRL0 bit positions */
#define SPI0_CTRL0_SPI0_OVERRUN_INT_ENABLE_Pos 10
#define SPI0_CTRL0_SPI0_UNDERRUN_INT_ENABLE_Pos 9
#define SPI0_CTRL0_SPI0_CONTROLLER_Pos  8
#define SPI0_CTRL0_SPI0_SLAVE_Pos       7
#define SPI0_CTRL0_SPI0_CLK_POLARITY_Pos 6
#define SPI0_CTRL0_SPI0_MODE_SELECT_Pos 5
#define SPI0_CTRL0_SPI0_ENABLE_Pos      4
#define SPI0_CTRL0_SPI0_PRESCALE_Pos    0
#define SPI0_CTRL0_SPI0_PRESCALE_Mask   ((uint32_t)(0xFU << SPI0_CTRL0_SPI0_PRESCALE_Pos))

/* SPI0_CTRL0 bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[4];
    __IO uint32_t SPI0_ENABLE_ALIAS;    /* Enable/disable the SPI interface */
    __IO uint32_t SPI0_MODE_SELECT_ALIAS;/* Select between manual and auto transaction handling modes for SPI master transactions */
    __IO uint32_t SPI0_CLK_POLARITY_ALIAS;/* Select the polarity of the SPI clock */
    __IO uint32_t SPI0_SLAVE_ALIAS;     /* Use the SPI interface as master or slave */
    __IO uint32_t SPI0_CONTROLLER_ALIAS;/* Select whether data transfer will be controlled by the CM3 or the DMA for SPI */
    __IO uint32_t SPI0_UNDERRUN_INT_ENABLE_ALIAS;/* Enable/disable SPI underrun interrupts */
    __IO uint32_t SPI0_OVERRUN_INT_ENABLE_ALIAS;/* Enable/disable SPI overrun interrupts */
} SPI0_CTRL0_Type;

#define SPI0_CTRL0_BASE                 SYS_CALC_BITBAND(0x40000800, 0)
#define SPI0_CTRL0                      ((SPI0_CTRL0_Type *) SPI0_CTRL0_BASE)

/* SPI0_CTRL0 settings */
#define SPI0_PRESCALE_2                 ((uint32_t)(0x0U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_4                 ((uint32_t)(0x1U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_8                 ((uint32_t)(0x2U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_16                ((uint32_t)(0x3U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_32                ((uint32_t)(0x4U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_64                ((uint32_t)(0x5U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_128               ((uint32_t)(0x6U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_256               ((uint32_t)(0x7U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_512               ((uint32_t)(0x8U << SPI0_CTRL0_SPI0_PRESCALE_Pos))
#define SPI0_PRESCALE_1024              ((uint32_t)(0x9U << SPI0_CTRL0_SPI0_PRESCALE_Pos))

#define SPI0_DISABLE_BITBAND            0x0
#define SPI0_ENABLE_BITBAND             0x1
#define SPI0_DISABLE                    ((uint32_t)(SPI0_DISABLE_BITBAND << SPI0_CTRL0_SPI0_ENABLE_Pos))
#define SPI0_ENABLE                     ((uint32_t)(SPI0_ENABLE_BITBAND << SPI0_CTRL0_SPI0_ENABLE_Pos))

#define SPI0_MODE_SELECT_MANUAL_BITBAND 0x0
#define SPI0_MODE_SELECT_AUTO_BITBAND   0x1
#define SPI0_MODE_SELECT_MANUAL         ((uint32_t)(SPI0_MODE_SELECT_MANUAL_BITBAND << SPI0_CTRL0_SPI0_MODE_SELECT_Pos))
#define SPI0_MODE_SELECT_AUTO           ((uint32_t)(SPI0_MODE_SELECT_AUTO_BITBAND << SPI0_CTRL0_SPI0_MODE_SELECT_Pos))

#define SPI0_CLK_POLARITY_NORMAL_BITBAND 0x0
#define SPI0_CLK_POLARITY_INVERSE_BITBAND 0x1
#define SPI0_CLK_POLARITY_NORMAL        ((uint32_t)(SPI0_CLK_POLARITY_NORMAL_BITBAND << SPI0_CTRL0_SPI0_CLK_POLARITY_Pos))
#define SPI0_CLK_POLARITY_INVERSE       ((uint32_t)(SPI0_CLK_POLARITY_INVERSE_BITBAND << SPI0_CTRL0_SPI0_CLK_POLARITY_Pos))

#define SPI0_SELECT_MASTER_BITBAND      0x0
#define SPI0_SELECT_SLAVE_BITBAND       0x1
#define SPI0_SELECT_MASTER              ((uint32_t)(SPI0_SELECT_MASTER_BITBAND << SPI0_CTRL0_SPI0_SLAVE_Pos))
#define SPI0_SELECT_SLAVE               ((uint32_t)(SPI0_SELECT_SLAVE_BITBAND << SPI0_CTRL0_SPI0_SLAVE_Pos))

#define SPI0_CONTROLLER_CM3_BITBAND     0x0
#define SPI0_CONTROLLER_DMA_BITBAND     0x1
#define SPI0_CONTROLLER_CM3             ((uint32_t)(SPI0_CONTROLLER_CM3_BITBAND << SPI0_CTRL0_SPI0_CONTROLLER_Pos))
#define SPI0_CONTROLLER_DMA             ((uint32_t)(SPI0_CONTROLLER_DMA_BITBAND << SPI0_CTRL0_SPI0_CONTROLLER_Pos))

#define SPI0_UNDERRUN_INT_DISABLE_BITBAND 0x0
#define SPI0_UNDERRUN_INT_ENABLE_BITBAND 0x1
#define SPI0_UNDERRUN_INT_DISABLE       ((uint32_t)(SPI0_UNDERRUN_INT_DISABLE_BITBAND << SPI0_CTRL0_SPI0_UNDERRUN_INT_ENABLE_Pos))
#define SPI0_UNDERRUN_INT_ENABLE        ((uint32_t)(SPI0_UNDERRUN_INT_ENABLE_BITBAND << SPI0_CTRL0_SPI0_UNDERRUN_INT_ENABLE_Pos))

#define SPI0_OVERRUN_INT_DISABLE_BITBAND 0x0
#define SPI0_OVERRUN_INT_ENABLE_BITBAND 0x1
#define SPI0_OVERRUN_INT_DISABLE        ((uint32_t)(SPI0_OVERRUN_INT_DISABLE_BITBAND << SPI0_CTRL0_SPI0_OVERRUN_INT_ENABLE_Pos))
#define SPI0_OVERRUN_INT_ENABLE         ((uint32_t)(SPI0_OVERRUN_INT_ENABLE_BITBAND << SPI0_CTRL0_SPI0_OVERRUN_INT_ENABLE_Pos))

/* SPI0_CTRL1 bit positions */
#define SPI0_CTRL1_SPI0_START_BUSY_Pos  8
#define SPI0_CTRL1_SPI0_BUSY_STATUS_Pos 8
#define SPI0_CTRL1_SPI0_RW_CMD_Pos      6
#define SPI0_CTRL1_SPI0_RW_CMD_Mask     ((uint32_t)(0x3U << SPI0_CTRL1_SPI0_RW_CMD_Pos))
#define SPI0_CTRL1_SPI0_CS_Pos          5
#define SPI0_CTRL1_SPI0_WORD_SIZE_Pos   0
#define SPI0_CTRL1_SPI0_WORD_SIZE_Mask  ((uint32_t)(0x1FU << SPI0_CTRL1_SPI0_WORD_SIZE_Pos))

/* SPI0_CTRL1 bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[5];
    __IO uint32_t SPI0_CS_ALIAS;        /* Set the chip-select line for SPI (master mode), read the chip-select line for SPI (slave mode) */
         uint32_t RESERVED1[2];
    __O  uint32_t SPI0_START_BUSY_ALIAS;/* Start an SPI data transfer and indicate if a transfer is in progress */
    __I  uint32_t SPI0_BUSY_STATUS_ALIAS;/* SPI data transfer status read */
} SPI0_CTRL1_Type;

#define SPI0_CTRL1_BASE                 SYS_CALC_BITBAND(0x40000804, 0)
#define SPI0_CTRL1                      ((SPI0_CTRL1_Type *) SPI0_CTRL1_BASE)

/* SPI0_CTRL1 settings */
#define SPI0_WORD_SIZE_1                ((uint32_t)(0x0U << SPI0_CTRL1_SPI0_WORD_SIZE_Pos))
#define SPI0_WORD_SIZE_8                ((uint32_t)(0x7U << SPI0_CTRL1_SPI0_WORD_SIZE_Pos))
#define SPI0_WORD_SIZE_16               ((uint32_t)(0xFU << SPI0_CTRL1_SPI0_WORD_SIZE_Pos))
#define SPI0_WORD_SIZE_24               ((uint32_t)(0x17U << SPI0_CTRL1_SPI0_WORD_SIZE_Pos))
#define SPI0_WORD_SIZE_32               ((uint32_t)(0x1FU << SPI0_CTRL1_SPI0_WORD_SIZE_Pos))

#define SPI0_CS_0_BITBAND               0x0
#define SPI0_CS_1_BITBAND               0x1
#define SPI0_CS_0                       ((uint32_t)(SPI0_CS_0_BITBAND << SPI0_CTRL1_SPI0_CS_Pos))
#define SPI0_CS_1                       ((uint32_t)(SPI0_CS_1_BITBAND << SPI0_CTRL1_SPI0_CS_Pos))

#define SPI0_NOP                        ((uint32_t)(0x0U << SPI0_CTRL1_SPI0_RW_CMD_Pos))
#define SPI0_WRITE_DATA                 ((uint32_t)(0x1U << SPI0_CTRL1_SPI0_RW_CMD_Pos))
#define SPI0_READ_DATA                  ((uint32_t)(0x2U << SPI0_CTRL1_SPI0_RW_CMD_Pos))
#define SPI0_RW_DATA                    ((uint32_t)(0x3U << SPI0_CTRL1_SPI0_RW_CMD_Pos))

#define SPI0_IDLE_BITBAND               0x0
#define SPI0_START_BITBAND              0x1
#define SPI0_IDLE                       ((uint32_t)(SPI0_IDLE_BITBAND << SPI0_CTRL1_SPI0_START_BUSY_Pos))
#define SPI0_START                      ((uint32_t)(SPI0_START_BITBAND << SPI0_CTRL1_SPI0_START_BUSY_Pos))

#define SPI0_BUSY_BITBAND               0x1
#define SPI0_BUSY                       ((uint32_t)(SPI0_BUSY_BITBAND << SPI0_CTRL1_SPI0_BUSY_STATUS_Pos))

/* SPI0_STATUS bit positions */
#define SPI0_STATUS_SPI0_TRANSMIT_STATUS_Pos 3
#define SPI0_STATUS_SPI0_RECEIVE_STATUS_Pos 2
#define SPI0_STATUS_SPI0_OVERRUN_STATUS_Pos 1
#define SPI0_STATUS_SPI0_UNDERRUN_STATUS_Pos 0

/* SPI0_STATUS bit-band aliases */
typedef struct
{
    __IO uint32_t SPI0_UNDERRUN_STATUS_ALIAS;/* Indicate that an underrun has occurred when transmitting data on the SPI interface */
    __IO uint32_t SPI0_OVERRUN_STATUS_ALIAS;/* Indicate that an overrun has occurred when receiving data on the SPI interface */
    __IO uint32_t SPI0_RECEIVE_STATUS_ALIAS;/* Indicate that new data has been received */
    __IO uint32_t SPI0_TRANSMIT_STATUS_ALIAS;/* Indicate that the transmission of the data is completed */
} SPI0_STATUS_Type;

#define SPI0_STATUS_BASE                SYS_CALC_BITBAND(0x40000810, 0)
#define SPI0_STATUS                     ((SPI0_STATUS_Type *) SPI0_STATUS_BASE)

/* SPI0_STATUS settings */
#define SPI0_UNDERRUN_FALSE_BITBAND     0x0
#define SPI0_UNDERRUN_TRUE_BITBAND      0x1
#define SPI0_UNDERRUN_CLEAR_BITBAND     0x1
#define SPI0_UNDERRUN_FALSE             ((uint32_t)(SPI0_UNDERRUN_FALSE_BITBAND << SPI0_STATUS_SPI0_UNDERRUN_STATUS_Pos))
#define SPI0_UNDERRUN_TRUE              ((uint32_t)(SPI0_UNDERRUN_TRUE_BITBAND << SPI0_STATUS_SPI0_UNDERRUN_STATUS_Pos))
#define SPI0_UNDERRUN_CLEAR             ((uint32_t)(SPI0_UNDERRUN_CLEAR_BITBAND << SPI0_STATUS_SPI0_UNDERRUN_STATUS_Pos))

#define SPI0_OVERRUN_FALSE_BITBAND      0x0
#define SPI0_OVERRUN_TRUE_BITBAND       0x1
#define SPI0_OVERRUN_CLEAR_BITBAND      0x1
#define SPI0_OVERRUN_FALSE              ((uint32_t)(SPI0_OVERRUN_FALSE_BITBAND << SPI0_STATUS_SPI0_OVERRUN_STATUS_Pos))
#define SPI0_OVERRUN_TRUE               ((uint32_t)(SPI0_OVERRUN_TRUE_BITBAND << SPI0_STATUS_SPI0_OVERRUN_STATUS_Pos))
#define SPI0_OVERRUN_CLEAR              ((uint32_t)(SPI0_OVERRUN_CLEAR_BITBAND << SPI0_STATUS_SPI0_OVERRUN_STATUS_Pos))

#define SPI0_RECEIVE_FALSE_BITBAND      0x0
#define SPI0_RECEIVE_TRUE_BITBAND       0x1
#define SPI0_RECEIVE_CLEAR_BITBAND      0x1
#define SPI0_RECEIVE_FALSE              ((uint32_t)(SPI0_RECEIVE_FALSE_BITBAND << SPI0_STATUS_SPI0_RECEIVE_STATUS_Pos))
#define SPI0_RECEIVE_TRUE               ((uint32_t)(SPI0_RECEIVE_TRUE_BITBAND << SPI0_STATUS_SPI0_RECEIVE_STATUS_Pos))
#define SPI0_RECEIVE_CLEAR              ((uint32_t)(SPI0_RECEIVE_CLEAR_BITBAND << SPI0_STATUS_SPI0_RECEIVE_STATUS_Pos))

#define SPI0_TRANSMIT_FALSE_BITBAND     0x0
#define SPI0_TRANSMIT_TRUE_BITBAND      0x1
#define SPI0_TRANSMIT_CLEAR_BITBAND     0x1
#define SPI0_TRANSMIT_FALSE             ((uint32_t)(SPI0_TRANSMIT_FALSE_BITBAND << SPI0_STATUS_SPI0_TRANSMIT_STATUS_Pos))
#define SPI0_TRANSMIT_TRUE              ((uint32_t)(SPI0_TRANSMIT_TRUE_BITBAND << SPI0_STATUS_SPI0_TRANSMIT_STATUS_Pos))
#define SPI0_TRANSMIT_CLEAR             ((uint32_t)(SPI0_TRANSMIT_CLEAR_BITBAND << SPI0_STATUS_SPI0_TRANSMIT_STATUS_Pos))

/* ----------------------------------------------------------------------------
 * SPI Interface Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL0;                /* SPI1 Control and Configuration Register */
    __IO uint32_t CTRL1;                /* SPI1 Transaction Control Register */
    __IO uint32_t TX_DATA;              /* SPI1 Transmit Data */
    __I  uint32_t RX_DATA;              /* SPI1 Received Data */
    __IO uint32_t STATUS;               /* SPI1 Status */
} SPI1_Type;

#define SPI1_BASE                       0x40000900
#define SPI1                            ((SPI1_Type *) SPI1_BASE)

/* SPI1_CTRL0 bit positions */
#define SPI1_CTRL0_SPI1_OVERRUN_INT_ENABLE_Pos 10
#define SPI1_CTRL0_SPI1_UNDERRUN_INT_ENABLE_Pos 9
#define SPI1_CTRL0_SPI1_CONTROLLER_Pos  8
#define SPI1_CTRL0_SPI1_SLAVE_Pos       7
#define SPI1_CTRL0_SPI1_CLK_POLARITY_Pos 6
#define SPI1_CTRL0_SPI1_MODE_SELECT_Pos 5
#define SPI1_CTRL0_SPI1_ENABLE_Pos      4
#define SPI1_CTRL0_SPI1_PRESCALE_Pos    0
#define SPI1_CTRL0_SPI1_PRESCALE_Mask   ((uint32_t)(0xFU << SPI1_CTRL0_SPI1_PRESCALE_Pos))

/* SPI1_CTRL0 bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[4];
    __IO uint32_t SPI1_ENABLE_ALIAS;    /* Enable/disable the SPI interface */
    __IO uint32_t SPI1_MODE_SELECT_ALIAS;/* Select between manual and auto transaction handling modes for SPI master transactions */
    __IO uint32_t SPI1_CLK_POLARITY_ALIAS;/* Select the polarity of the SPI clock */
    __IO uint32_t SPI1_SLAVE_ALIAS;     /* Use the SPI interface as master or slave */
    __IO uint32_t SPI1_CONTROLLER_ALIAS;/* Select whether data transfer will be controlled by the CM3 or the DMA for SPI */
    __IO uint32_t SPI1_UNDERRUN_INT_ENABLE_ALIAS;/* Enable/disable SPI underrun interrupts */
    __IO uint32_t SPI1_OVERRUN_INT_ENABLE_ALIAS;/* Enable/disable SPI overrun interrupts */
} SPI1_CTRL0_Type;

#define SPI1_CTRL0_BASE                 SYS_CALC_BITBAND(0x40000900, 0)
#define SPI1_CTRL0                      ((SPI1_CTRL0_Type *) SPI1_CTRL0_BASE)

/* SPI1_CTRL0 settings */
#define SPI1_PRESCALE_2                 ((uint32_t)(0x0U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_4                 ((uint32_t)(0x1U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_8                 ((uint32_t)(0x2U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_16                ((uint32_t)(0x3U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_32                ((uint32_t)(0x4U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_64                ((uint32_t)(0x5U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_128               ((uint32_t)(0x6U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_256               ((uint32_t)(0x7U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_512               ((uint32_t)(0x8U << SPI1_CTRL0_SPI1_PRESCALE_Pos))
#define SPI1_PRESCALE_1024              ((uint32_t)(0x9U << SPI1_CTRL0_SPI1_PRESCALE_Pos))

#define SPI1_DISABLE_BITBAND            0x0
#define SPI1_ENABLE_BITBAND             0x1
#define SPI1_DISABLE                    ((uint32_t)(SPI1_DISABLE_BITBAND << SPI1_CTRL0_SPI1_ENABLE_Pos))
#define SPI1_ENABLE                     ((uint32_t)(SPI1_ENABLE_BITBAND << SPI1_CTRL0_SPI1_ENABLE_Pos))

#define SPI1_MODE_SELECT_MANUAL_BITBAND 0x0
#define SPI1_MODE_SELECT_AUTO_BITBAND   0x1
#define SPI1_MODE_SELECT_MANUAL         ((uint32_t)(SPI1_MODE_SELECT_MANUAL_BITBAND << SPI1_CTRL0_SPI1_MODE_SELECT_Pos))
#define SPI1_MODE_SELECT_AUTO           ((uint32_t)(SPI1_MODE_SELECT_AUTO_BITBAND << SPI1_CTRL0_SPI1_MODE_SELECT_Pos))

#define SPI1_CLK_POLARITY_NORMAL_BITBAND 0x0
#define SPI1_CLK_POLARITY_INVERSE_BITBAND 0x1
#define SPI1_CLK_POLARITY_NORMAL        ((uint32_t)(SPI1_CLK_POLARITY_NORMAL_BITBAND << SPI1_CTRL0_SPI1_CLK_POLARITY_Pos))
#define SPI1_CLK_POLARITY_INVERSE       ((uint32_t)(SPI1_CLK_POLARITY_INVERSE_BITBAND << SPI1_CTRL0_SPI1_CLK_POLARITY_Pos))

#define SPI1_SELECT_MASTER_BITBAND      0x0
#define SPI1_SELECT_SLAVE_BITBAND       0x1
#define SPI1_SELECT_MASTER              ((uint32_t)(SPI1_SELECT_MASTER_BITBAND << SPI1_CTRL0_SPI1_SLAVE_Pos))
#define SPI1_SELECT_SLAVE               ((uint32_t)(SPI1_SELECT_SLAVE_BITBAND << SPI1_CTRL0_SPI1_SLAVE_Pos))

#define SPI1_CONTROLLER_CM3_BITBAND     0x0
#define SPI1_CONTROLLER_DMA_BITBAND     0x1
#define SPI1_CONTROLLER_CM3             ((uint32_t)(SPI1_CONTROLLER_CM3_BITBAND << SPI1_CTRL0_SPI1_CONTROLLER_Pos))
#define SPI1_CONTROLLER_DMA             ((uint32_t)(SPI1_CONTROLLER_DMA_BITBAND << SPI1_CTRL0_SPI1_CONTROLLER_Pos))

#define SPI1_UNDERRUN_INT_DISABLE_BITBAND 0x0
#define SPI1_UNDERRUN_INT_ENABLE_BITBAND 0x1
#define SPI1_UNDERRUN_INT_DISABLE       ((uint32_t)(SPI1_UNDERRUN_INT_DISABLE_BITBAND << SPI1_CTRL0_SPI1_UNDERRUN_INT_ENABLE_Pos))
#define SPI1_UNDERRUN_INT_ENABLE        ((uint32_t)(SPI1_UNDERRUN_INT_ENABLE_BITBAND << SPI1_CTRL0_SPI1_UNDERRUN_INT_ENABLE_Pos))

#define SPI1_OVERRUN_INT_DISABLE_BITBAND 0x0
#define SPI1_OVERRUN_INT_ENABLE_BITBAND 0x1
#define SPI1_OVERRUN_INT_DISABLE        ((uint32_t)(SPI1_OVERRUN_INT_DISABLE_BITBAND << SPI1_CTRL0_SPI1_OVERRUN_INT_ENABLE_Pos))
#define SPI1_OVERRUN_INT_ENABLE         ((uint32_t)(SPI1_OVERRUN_INT_ENABLE_BITBAND << SPI1_CTRL0_SPI1_OVERRUN_INT_ENABLE_Pos))

/* SPI1_CTRL1 bit positions */
#define SPI1_CTRL1_SPI1_START_BUSY_Pos  8
#define SPI1_CTRL1_SPI1_RW_CMD_Pos      6
#define SPI1_CTRL1_SPI1_RW_CMD_Mask     ((uint32_t)(0x3U << SPI1_CTRL1_SPI1_RW_CMD_Pos))
#define SPI1_CTRL1_SPI1_CS_Pos          5
#define SPI1_CTRL1_SPI1_WORD_SIZE_Pos   0
#define SPI1_CTRL1_SPI1_WORD_SIZE_Mask  ((uint32_t)(0x1FU << SPI1_CTRL1_SPI1_WORD_SIZE_Pos))

/* SPI1_CTRL1 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t SPI1_START_BUSY_BYTE; 
         uint8_t RESERVED1[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000904, 0) - (0x40000904 + 4)];
         uint32_t RESERVED2[5];
    __IO uint32_t SPI1_CS_ALIAS;        /* Set the chip-select line for SPI (master mode), read the chip-select line for SPI (slave mode) */
         uint32_t RESERVED3[2];
    __IO uint32_t SPI1_START_BUSY_ALIAS;/* Start an SPI data transfer and indicate if a transfer is in progress */
} SPI1_CTRL1_Type;

#define SPI1_CTRL1_BASE                 0x40000904
#define SPI1_CTRL1                      ((SPI1_CTRL1_Type *) SPI1_CTRL1_BASE)

/* SPI1_CTRL1 settings */
#define SPI1_WORD_SIZE_1                ((uint32_t)(0x0U << SPI1_CTRL1_SPI1_WORD_SIZE_Pos))
#define SPI1_WORD_SIZE_8                ((uint32_t)(0x7U << SPI1_CTRL1_SPI1_WORD_SIZE_Pos))
#define SPI1_WORD_SIZE_16               ((uint32_t)(0xFU << SPI1_CTRL1_SPI1_WORD_SIZE_Pos))
#define SPI1_WORD_SIZE_24               ((uint32_t)(0x17U << SPI1_CTRL1_SPI1_WORD_SIZE_Pos))
#define SPI1_WORD_SIZE_32               ((uint32_t)(0x1FU << SPI1_CTRL1_SPI1_WORD_SIZE_Pos))

#define SPI1_CS_0_BITBAND               0x0
#define SPI1_CS_1_BITBAND               0x1
#define SPI1_CS_0                       ((uint32_t)(SPI1_CS_0_BITBAND << SPI1_CTRL1_SPI1_CS_Pos))
#define SPI1_CS_1                       ((uint32_t)(SPI1_CS_1_BITBAND << SPI1_CTRL1_SPI1_CS_Pos))

#define SPI1_NOP                        ((uint32_t)(0x0U << SPI1_CTRL1_SPI1_RW_CMD_Pos))
#define SPI1_WRITE_DATA                 ((uint32_t)(0x1U << SPI1_CTRL1_SPI1_RW_CMD_Pos))
#define SPI1_READ_DATA                  ((uint32_t)(0x2U << SPI1_CTRL1_SPI1_RW_CMD_Pos))
#define SPI1_RW_DATA                    ((uint32_t)(0x3U << SPI1_CTRL1_SPI1_RW_CMD_Pos))

#define SPI1_IDLE_BITBAND               0x0
#define SPI1_START_BITBAND              0x1
#define SPI1_BUSY_BITBAND               0x1
#define SPI1_IDLE                       ((uint32_t)(SPI1_IDLE_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_Pos))
#define SPI1_START                      ((uint32_t)(SPI1_START_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_Pos))
#define SPI1_BUSY                       ((uint32_t)(SPI1_BUSY_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_Pos))

/* SPI1_CTRL1 sub-register bit positions */
#define SPI1_CTRL1_SPI1_START_BUSY_BYTE_Pos 0

/* SPI1_CTRL1 sub-register settings */
#define SPI1_IDLE_BYTE                  ((uint8_t)(SPI1_IDLE_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_BYTE_Pos))
#define SPI1_START_BYTE                 ((uint8_t)(SPI1_START_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_BYTE_Pos))
#define SPI1_BUSY_BYTE                  ((uint8_t)(SPI1_BUSY_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_BYTE_Pos))

/* SPI1_STATUS bit positions */
#define SPI1_STATUS_SPI1_TRANSMIT_STATUS_Pos 3
#define SPI1_STATUS_SPI1_RECEIVE_STATUS_Pos 2
#define SPI1_STATUS_SPI1_OVERRUN_STATUS_Pos 1
#define SPI1_STATUS_SPI1_UNDERRUN_STATUS_Pos 0

/* SPI1_STATUS bit-band aliases */
typedef struct
{
    __IO uint32_t SPI1_UNDERRUN_STATUS_ALIAS;/* Indicate that an underrun has occurred when transmitting data on the SPI interface */
    __IO uint32_t SPI1_OVERRUN_STATUS_ALIAS;/* Indicate that an overrun has occurred when receiving data on the SPI interface */
    __IO uint32_t SPI1_RECEIVE_STATUS_ALIAS;/* Indicate that new data has been received */
    __IO uint32_t SPI1_TRANSMIT_STATUS_ALIAS;/* Indicate that the transmission of the data is completed */
} SPI1_STATUS_Type;

#define SPI1_STATUS_BASE                SYS_CALC_BITBAND(0x40000910, 0)
#define SPI1_STATUS                     ((SPI1_STATUS_Type *) SPI1_STATUS_BASE)

/* SPI1_STATUS settings */
#define SPI1_UNDERRUN_FALSE_BITBAND     0x0
#define SPI1_UNDERRUN_TRUE_BITBAND      0x1
#define SPI1_UNDERRUN_CLEAR_BITBAND     0x1
#define SPI1_UNDERRUN_FALSE             ((uint32_t)(SPI1_UNDERRUN_FALSE_BITBAND << SPI1_STATUS_SPI1_UNDERRUN_STATUS_Pos))
#define SPI1_UNDERRUN_TRUE              ((uint32_t)(SPI1_UNDERRUN_TRUE_BITBAND << SPI1_STATUS_SPI1_UNDERRUN_STATUS_Pos))
#define SPI1_UNDERRUN_CLEAR             ((uint32_t)(SPI1_UNDERRUN_CLEAR_BITBAND << SPI1_STATUS_SPI1_UNDERRUN_STATUS_Pos))

#define SPI1_OVERRUN_FALSE_BITBAND      0x0
#define SPI1_OVERRUN_TRUE_BITBAND       0x1
#define SPI1_OVERRUN_CLEAR_BITBAND      0x1
#define SPI1_OVERRUN_FALSE              ((uint32_t)(SPI1_OVERRUN_FALSE_BITBAND << SPI1_STATUS_SPI1_OVERRUN_STATUS_Pos))
#define SPI1_OVERRUN_TRUE               ((uint32_t)(SPI1_OVERRUN_TRUE_BITBAND << SPI1_STATUS_SPI1_OVERRUN_STATUS_Pos))
#define SPI1_OVERRUN_CLEAR              ((uint32_t)(SPI1_OVERRUN_CLEAR_BITBAND << SPI1_STATUS_SPI1_OVERRUN_STATUS_Pos))

#define SPI1_RECEIVE_FALSE_BITBAND      0x0
#define SPI1_RECEIVE_TRUE_BITBAND       0x1
#define SPI1_RECEIVE_CLEAR_BITBAND      0x1
#define SPI1_RECEIVE_FALSE              ((uint32_t)(SPI1_RECEIVE_FALSE_BITBAND << SPI1_STATUS_SPI1_RECEIVE_STATUS_Pos))
#define SPI1_RECEIVE_TRUE               ((uint32_t)(SPI1_RECEIVE_TRUE_BITBAND << SPI1_STATUS_SPI1_RECEIVE_STATUS_Pos))
#define SPI1_RECEIVE_CLEAR              ((uint32_t)(SPI1_RECEIVE_CLEAR_BITBAND << SPI1_STATUS_SPI1_RECEIVE_STATUS_Pos))

#define SPI1_TRANSMIT_FALSE_BITBAND     0x0
#define SPI1_TRANSMIT_TRUE_BITBAND      0x1
#define SPI1_TRANSMIT_CLEAR_BITBAND     0x1
#define SPI1_TRANSMIT_FALSE             ((uint32_t)(SPI1_TRANSMIT_FALSE_BITBAND << SPI1_STATUS_SPI1_TRANSMIT_STATUS_Pos))
#define SPI1_TRANSMIT_TRUE              ((uint32_t)(SPI1_TRANSMIT_TRUE_BITBAND << SPI1_STATUS_SPI1_TRANSMIT_STATUS_Pos))
#define SPI1_TRANSMIT_CLEAR             ((uint32_t)(SPI1_TRANSMIT_CLEAR_BITBAND << SPI1_STATUS_SPI1_TRANSMIT_STATUS_Pos))

/* ----------------------------------------------------------------------------
 * PCM Interface Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL;                 /* PCM Control */
    __IO uint32_t TX_DATA;              /* PCM Transmit Data */
    __I  uint32_t RX_DATA;              /* PCM Receive Data */
    __IO uint32_t STATUS;               /* PCM Status */
} PCM_Type;

#define PCM_BASE                        0x40000A00
#define PCM                             ((PCM_Type *) PCM_BASE)

/* PCM_CTRL bit positions */
#define PCM_CTRL_PCM_CLK_POL_Pos        14
#define PCM_CTRL_BIT_ORDER_Pos          12
#define PCM_CTRL_TX_ALIGN_Pos           11
#define PCM_CTRL_WORD_SIZE_Pos          9
#define PCM_CTRL_WORD_SIZE_Mask         ((uint32_t)(0x3U << PCM_CTRL_WORD_SIZE_Pos))
#define PCM_CTRL_FRAME_ALIGN_Pos        8
#define PCM_CTRL_FRAME_WIDTH_Pos        7
#define PCM_CTRL_FRAME_LENGTH_Pos       4
#define PCM_CTRL_FRAME_LENGTH_Mask      ((uint32_t)(0x7U << PCM_CTRL_FRAME_LENGTH_Pos))
#define PCM_CTRL_FRAME_SUBFRAMES_Pos    3
#define PCM_CTRL_CONTROLLER_Pos         2
#define PCM_CTRL_ENABLE_Pos             1
#define PCM_CTRL_SLAVE_Pos              0

/* PCM_CTRL bit-band aliases */
typedef struct
{
    __IO uint32_t SLAVE_ALIAS;          /* Use the PCM interface as a master/slave */
    __IO uint32_t ENABLE_ALIAS;         /* Enable/disable the PCM interface */
    __IO uint32_t CONTROLLER_ALIAS;     /* Select whether data transfer will be controlled by the CM3 or the DMA for PCM */
    __IO uint32_t FRAME_SUBFRAMES_ALIAS;/* Enable the frame duration for each word */
         uint32_t RESERVED0[3];
    __IO uint32_t FRAME_WIDTH_ALIAS;    /* Use a long/short PCM frame signal */
    __IO uint32_t FRAME_ALIGN_ALIAS;    /* Align the PCM frame signal to the first/last bit */
         uint32_t RESERVED1[2];
    __IO uint32_t TX_ALIGN_ALIAS;       /* Select what bits to use for transmit data */
    __IO uint32_t BIT_ORDER_ALIAS;      /* Select whether the data will be transmitted starting with the MSB or LSB */
         uint32_t RESERVED2;
    __IO uint32_t PCM_CLK_POL_ALIAS;    /* PCM clock polarity */
} PCM_CTRL_Type;

#define PCM_CTRL_BASE                   SYS_CALC_BITBAND(0x40000A00, 0)
#define PCM_CTRL                        ((PCM_CTRL_Type *) PCM_CTRL_BASE)

/* PCM_CTRL settings */
#define PCM_SELECT_MASTER_BITBAND       0x0
#define PCM_SELECT_SLAVE_BITBAND        0x1
#define PCM_SELECT_MASTER               ((uint32_t)(PCM_SELECT_MASTER_BITBAND << PCM_CTRL_SLAVE_Pos))
#define PCM_SELECT_SLAVE                ((uint32_t)(PCM_SELECT_SLAVE_BITBAND << PCM_CTRL_SLAVE_Pos))

#define PCM_DISABLE_BITBAND             0x0
#define PCM_ENABLE_BITBAND              0x1
#define PCM_DISABLE                     ((uint32_t)(PCM_DISABLE_BITBAND << PCM_CTRL_ENABLE_Pos))
#define PCM_ENABLE                      ((uint32_t)(PCM_ENABLE_BITBAND << PCM_CTRL_ENABLE_Pos))

#define PCM_CONTROLLER_CM3_BITBAND      0x0
#define PCM_CONTROLLER_DMA_BITBAND      0x1
#define PCM_CONTROLLER_CM3              ((uint32_t)(PCM_CONTROLLER_CM3_BITBAND << PCM_CTRL_CONTROLLER_Pos))
#define PCM_CONTROLLER_DMA              ((uint32_t)(PCM_CONTROLLER_DMA_BITBAND << PCM_CTRL_CONTROLLER_Pos))

#define PCM_SUBFRAME_DISABLE_BITBAND    0x0
#define PCM_SUBFRAME_ENABLE_BITBAND     0x1
#define PCM_SUBFRAME_DISABLE            ((uint32_t)(PCM_SUBFRAME_DISABLE_BITBAND << PCM_CTRL_FRAME_SUBFRAMES_Pos))
#define PCM_SUBFRAME_ENABLE             ((uint32_t)(PCM_SUBFRAME_ENABLE_BITBAND << PCM_CTRL_FRAME_SUBFRAMES_Pos))

#define PCM_MULTIWORD_2                 ((uint32_t)(0x0U << PCM_CTRL_FRAME_LENGTH_Pos))
#define PCM_MULTIWORD_4                 ((uint32_t)(0x1U << PCM_CTRL_FRAME_LENGTH_Pos))
#define PCM_MULTIWORD_6                 ((uint32_t)(0x2U << PCM_CTRL_FRAME_LENGTH_Pos))
#define PCM_MULTIWORD_8                 ((uint32_t)(0x3U << PCM_CTRL_FRAME_LENGTH_Pos))
#define PCM_MULTIWORD_10                ((uint32_t)(0x4U << PCM_CTRL_FRAME_LENGTH_Pos))
#define PCM_MULTIWORD_12                ((uint32_t)(0x5U << PCM_CTRL_FRAME_LENGTH_Pos))
#define PCM_MULTIWORD_14                ((uint32_t)(0x6U << PCM_CTRL_FRAME_LENGTH_Pos))
#define PCM_MULTIWORD_16                ((uint32_t)(0x7U << PCM_CTRL_FRAME_LENGTH_Pos))

#define PCM_FRAME_WIDTH_SHORT_BITBAND   0x0
#define PCM_FRAME_WIDTH_LONG_BITBAND    0x1
#define PCM_FRAME_WIDTH_SHORT           ((uint32_t)(PCM_FRAME_WIDTH_SHORT_BITBAND << PCM_CTRL_FRAME_WIDTH_Pos))
#define PCM_FRAME_WIDTH_LONG            ((uint32_t)(PCM_FRAME_WIDTH_LONG_BITBAND << PCM_CTRL_FRAME_WIDTH_Pos))

#define PCM_FRAME_ALIGN_LAST_BITBAND    0x0
#define PCM_FRAME_ALIGN_FIRST_BITBAND   0x1
#define PCM_FRAME_ALIGN_LAST            ((uint32_t)(PCM_FRAME_ALIGN_LAST_BITBAND << PCM_CTRL_FRAME_ALIGN_Pos))
#define PCM_FRAME_ALIGN_FIRST           ((uint32_t)(PCM_FRAME_ALIGN_FIRST_BITBAND << PCM_CTRL_FRAME_ALIGN_Pos))

#define PCM_WORD_SIZE_8                 ((uint32_t)(0x0U << PCM_CTRL_WORD_SIZE_Pos))
#define PCM_WORD_SIZE_16                ((uint32_t)(0x1U << PCM_CTRL_WORD_SIZE_Pos))
#define PCM_WORD_SIZE_24                ((uint32_t)(0x2U << PCM_CTRL_WORD_SIZE_Pos))
#define PCM_WORD_SIZE_32                ((uint32_t)(0x3U << PCM_CTRL_WORD_SIZE_Pos))

#define PCM_TX_ALIGN_MSB_BITBAND        0x0
#define PCM_TX_ALIGN_LSB_BITBAND        0x1
#define PCM_TX_ALIGN_MSB                ((uint32_t)(PCM_TX_ALIGN_MSB_BITBAND << PCM_CTRL_TX_ALIGN_Pos))
#define PCM_TX_ALIGN_LSB                ((uint32_t)(PCM_TX_ALIGN_LSB_BITBAND << PCM_CTRL_TX_ALIGN_Pos))

#define PCM_BIT_ORDER_MSB_FIRST_BITBAND 0x0
#define PCM_BIT_ORDER_LSB_FIRST_BITBAND 0x1
#define PCM_BIT_ORDER_MSB_FIRST         ((uint32_t)(PCM_BIT_ORDER_MSB_FIRST_BITBAND << PCM_CTRL_BIT_ORDER_Pos))
#define PCM_BIT_ORDER_LSB_FIRST         ((uint32_t)(PCM_BIT_ORDER_LSB_FIRST_BITBAND << PCM_CTRL_BIT_ORDER_Pos))

#define PCM_SAMPLE_FALLING_EDGE_BITBAND 0x0
#define PCM_SAMPLE_RISING_EDGE_BITBAND  0x1
#define PCM_SAMPLE_FALLING_EDGE         ((uint32_t)(PCM_SAMPLE_FALLING_EDGE_BITBAND << PCM_CTRL_PCM_CLK_POL_Pos))
#define PCM_SAMPLE_RISING_EDGE          ((uint32_t)(PCM_SAMPLE_RISING_EDGE_BITBAND << PCM_CTRL_PCM_CLK_POL_Pos))

/* PCM_STATUS bit positions */
#define PCM_STATUS_TRANSMIT_STATUS_Pos  3
#define PCM_STATUS_RECEIVE_STATUS_Pos   2
#define PCM_STATUS_OVERRUN_STATUS_Pos   1
#define PCM_STATUS_UNDERRUN_STATUS_Pos  0

/* PCM_STATUS bit-band aliases */
typedef struct
{
    __IO uint32_t UNDERRUN_STATUS_ALIAS;/* Indicate that an underrun has occurred when transmitting data on the PCM interface */
    __IO uint32_t OVERRUN_STATUS_ALIAS; /* Indicate that an overrun has occurred when receiving data on the PCM interface */
    __IO uint32_t RECEIVE_STATUS_ALIAS; /* Indicate that PCM data has been received */
    __IO uint32_t TRANSMIT_STATUS_ALIAS;/* Indicate that PCM data has been sent */
} PCM_STATUS_Type;

#define PCM_STATUS_BASE                 SYS_CALC_BITBAND(0x40000A0C, 0)
#define PCM_STATUS                      ((PCM_STATUS_Type *) PCM_STATUS_BASE)

/* PCM_STATUS settings */
#define PCM_UNDERRUN_FALSE_BITBAND      0x0
#define PCM_UNDERRUN_TRUE_BITBAND       0x1
#define PCM_UNDERRUN_CLEAR_BITBAND      0x1
#define PCM_UNDERRUN_FALSE              ((uint32_t)(PCM_UNDERRUN_FALSE_BITBAND << PCM_STATUS_UNDERRUN_STATUS_Pos))
#define PCM_UNDERRUN_TRUE               ((uint32_t)(PCM_UNDERRUN_TRUE_BITBAND << PCM_STATUS_UNDERRUN_STATUS_Pos))
#define PCM_UNDERRUN_CLEAR              ((uint32_t)(PCM_UNDERRUN_CLEAR_BITBAND << PCM_STATUS_UNDERRUN_STATUS_Pos))

#define PCM_OVERRUN_FALSE_BITBAND       0x0
#define PCM_OVERRUN_TRUE_BITBAND        0x1
#define PCM_OVERRUN_CLEAR_BITBAND       0x1
#define PCM_OVERRUN_FALSE               ((uint32_t)(PCM_OVERRUN_FALSE_BITBAND << PCM_STATUS_OVERRUN_STATUS_Pos))
#define PCM_OVERRUN_TRUE                ((uint32_t)(PCM_OVERRUN_TRUE_BITBAND << PCM_STATUS_OVERRUN_STATUS_Pos))
#define PCM_OVERRUN_CLEAR               ((uint32_t)(PCM_OVERRUN_CLEAR_BITBAND << PCM_STATUS_OVERRUN_STATUS_Pos))

#define PCM_RECEIVE_FALSE_BITBAND       0x0
#define PCM_RECEIVE_TRUE_BITBAND        0x1
#define PCM_RECEIVE_CLEAR_BITBAND       0x1
#define PCM_RECEIVE_FALSE               ((uint32_t)(PCM_RECEIVE_FALSE_BITBAND << PCM_STATUS_RECEIVE_STATUS_Pos))
#define PCM_RECEIVE_TRUE                ((uint32_t)(PCM_RECEIVE_TRUE_BITBAND << PCM_STATUS_RECEIVE_STATUS_Pos))
#define PCM_RECEIVE_CLEAR               ((uint32_t)(PCM_RECEIVE_CLEAR_BITBAND << PCM_STATUS_RECEIVE_STATUS_Pos))

#define PCM_TRANSMIT_FALSE_BITBAND      0x0
#define PCM_TRANSMIT_TRUE_BITBAND       0x1
#define PCM_TRANSMIT_CLEAR_BITBAND      0x1
#define PCM_TRANSMIT_FALSE              ((uint32_t)(PCM_TRANSMIT_FALSE_BITBAND << PCM_STATUS_TRANSMIT_STATUS_Pos))
#define PCM_TRANSMIT_TRUE               ((uint32_t)(PCM_TRANSMIT_TRUE_BITBAND << PCM_STATUS_TRANSMIT_STATUS_Pos))
#define PCM_TRANSMIT_CLEAR              ((uint32_t)(PCM_TRANSMIT_CLEAR_BITBAND << PCM_STATUS_TRANSMIT_STATUS_Pos))

/* ----------------------------------------------------------------------------
 * I2C Interface Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL0;                /* I2C Interface Configuration and Control */
    __O  uint32_t CTRL1;                /* I2C Interface Status and Control */
    __IO uint32_t DATA;                 /* I2C Interface Data */
    __IO uint32_t DATA_M;               /* I2C Interface Data (Mirror) */
    __O  uint32_t ADDR_START;           /* I2C Master Address and Start */
    __I  uint32_t STATUS;               /* I2C Status */
} I2C_Type;

#define I2C_BASE                        0x40000B00
#define I2C                             ((I2C_Type *) I2C_BASE)

/* I2C_CTRL0 bit positions */
#define I2C_CTRL0_SPEED_Pos             16
#define I2C_CTRL0_SPEED_Mask            ((uint32_t)(0xFFU << I2C_CTRL0_SPEED_Pos))
#define I2C_CTRL0_SLAVE_ADDRESS_Pos     8
#define I2C_CTRL0_SLAVE_ADDRESS_Mask    ((uint32_t)(0x7FU << I2C_CTRL0_SLAVE_ADDRESS_Pos))
#define I2C_CTRL0_CONTROLLER_Pos        4
#define I2C_CTRL0_STOP_INT_ENABLE_Pos   3
#define I2C_CTRL0_AUTO_ACK_ENABLE_Pos   2
#define I2C_CTRL0_I2C_SAMPLE_CLK_ENABLE_Pos 1
#define I2C_CTRL0_SLAVE_ENABLE_Pos      0

/* I2C_CTRL0 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[2];
    __IO uint8_t SPEED_BYTE;           
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000B00, 0) - (0x40000B00 + 4)];
    __IO uint32_t SLAVE_ENABLE_ALIAS;   /* Select whether the I2C interface will be enabled for slave mode or not */
    __IO uint32_t I2C_SAMPLE_CLK_ENABLE_ALIAS;/* Enable/disable the I2C sample clock (mandatory to enable the I2C) */
    __IO uint32_t AUTO_ACK_ENABLE_ALIAS;/* Select whether acknowledgement is automatically generated or not */
    __IO uint32_t STOP_INT_ENABLE_ALIAS;/* Configure whether stop interrupts will be generated by the I2C interface */
    __IO uint32_t CONTROLLER_ALIAS;     /* Select whether data transfer will be controlled by the CM3 or the DMA for I2C */
} I2C_CTRL0_Type;

#define I2C_CTRL0_BASE                  0x40000B00
#define I2C_CTRL0                       ((I2C_CTRL0_Type *) I2C_CTRL0_BASE)

/* I2C_CTRL0 settings */
#define I2C_SLAVE_DISABLE_BITBAND       0x0
#define I2C_SLAVE_ENABLE_BITBAND        0x1
#define I2C_SLAVE_DISABLE               ((uint32_t)(I2C_SLAVE_DISABLE_BITBAND << I2C_CTRL0_SLAVE_ENABLE_Pos))
#define I2C_SLAVE_ENABLE                ((uint32_t)(I2C_SLAVE_ENABLE_BITBAND << I2C_CTRL0_SLAVE_ENABLE_Pos))

#define I2C_SAMPLE_CLK_DISABLE_BITBAND  0x0
#define I2C_SAMPLE_CLK_ENABLE_BITBAND   0x1
#define I2C_SAMPLE_CLK_DISABLE          ((uint32_t)(I2C_SAMPLE_CLK_DISABLE_BITBAND << I2C_CTRL0_I2C_SAMPLE_CLK_ENABLE_Pos))
#define I2C_SAMPLE_CLK_ENABLE           ((uint32_t)(I2C_SAMPLE_CLK_ENABLE_BITBAND << I2C_CTRL0_I2C_SAMPLE_CLK_ENABLE_Pos))

#define I2C_AUTO_ACK_DISABLE_BITBAND    0x0
#define I2C_AUTO_ACK_ENABLE_BITBAND     0x1
#define I2C_AUTO_ACK_DISABLE            ((uint32_t)(I2C_AUTO_ACK_DISABLE_BITBAND << I2C_CTRL0_AUTO_ACK_ENABLE_Pos))
#define I2C_AUTO_ACK_ENABLE             ((uint32_t)(I2C_AUTO_ACK_ENABLE_BITBAND << I2C_CTRL0_AUTO_ACK_ENABLE_Pos))

#define I2C_STOP_INT_DISABLE_BITBAND    0x0
#define I2C_STOP_INT_ENABLE_BITBAND     0x1
#define I2C_STOP_INT_DISABLE            ((uint32_t)(I2C_STOP_INT_DISABLE_BITBAND << I2C_CTRL0_STOP_INT_ENABLE_Pos))
#define I2C_STOP_INT_ENABLE             ((uint32_t)(I2C_STOP_INT_ENABLE_BITBAND << I2C_CTRL0_STOP_INT_ENABLE_Pos))

#define I2C_CONTROLLER_CM3_BITBAND      0x0
#define I2C_CONTROLLER_DMA_BITBAND      0x1
#define I2C_CONTROLLER_CM3              ((uint32_t)(I2C_CONTROLLER_CM3_BITBAND << I2C_CTRL0_CONTROLLER_Pos))
#define I2C_CONTROLLER_DMA              ((uint32_t)(I2C_CONTROLLER_DMA_BITBAND << I2C_CTRL0_CONTROLLER_Pos))

#define I2C_MASTER_SPEED_3              ((uint32_t)(0x0U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_6              ((uint32_t)(0x1U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_9              ((uint32_t)(0x2U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_12             ((uint32_t)(0x3U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_15             ((uint32_t)(0x4U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_18             ((uint32_t)(0x5U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_21             ((uint32_t)(0x6U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_24             ((uint32_t)(0x7U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_27             ((uint32_t)(0x8U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_30             ((uint32_t)(0x9U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_33             ((uint32_t)(0xAU << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_36             ((uint32_t)(0xBU << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_39             ((uint32_t)(0xCU << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_42             ((uint32_t)(0xDU << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_45             ((uint32_t)(0xEU << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_48             ((uint32_t)(0xFU << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_51             ((uint32_t)(0x10U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_54             ((uint32_t)(0x11U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_57             ((uint32_t)(0x12U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_60             ((uint32_t)(0x13U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_120            ((uint32_t)(0x27U << I2C_CTRL0_SPEED_Pos))
#define I2C_MASTER_SPEED_768            ((uint32_t)(0xFFU << I2C_CTRL0_SPEED_Pos))
#define I2C_SLAVE_SPEED_1               ((uint32_t)(0x0U << I2C_CTRL0_SPEED_Pos))
#define I2C_SLAVE_SPEED_2               ((uint32_t)(0x1U << I2C_CTRL0_SPEED_Pos))
#define I2C_SLAVE_SPEED_3               ((uint32_t)(0x2U << I2C_CTRL0_SPEED_Pos))
#define I2C_SLAVE_SPEED_4               ((uint32_t)(0x3U << I2C_CTRL0_SPEED_Pos))
#define I2C_SLAVE_SPEED_5               ((uint32_t)(0x4U << I2C_CTRL0_SPEED_Pos))
#define I2C_SLAVE_SPEED_6               ((uint32_t)(0x5U << I2C_CTRL0_SPEED_Pos))
#define I2C_SLAVE_SPEED_7               ((uint32_t)(0x6U << I2C_CTRL0_SPEED_Pos))
#define I2C_SLAVE_SPEED_14              ((uint32_t)(0xDU << I2C_CTRL0_SPEED_Pos))

/* I2C_CTRL0 sub-register settings */
#define I2C_MASTER_SPEED_3_BYTE         ((uint8_t)0x0U)
#define I2C_MASTER_SPEED_6_BYTE         ((uint8_t)0x1U)
#define I2C_MASTER_SPEED_9_BYTE         ((uint8_t)0x2U)
#define I2C_MASTER_SPEED_12_BYTE        ((uint8_t)0x3U)
#define I2C_MASTER_SPEED_15_BYTE        ((uint8_t)0x4U)
#define I2C_MASTER_SPEED_18_BYTE        ((uint8_t)0x5U)
#define I2C_MASTER_SPEED_21_BYTE        ((uint8_t)0x6U)
#define I2C_MASTER_SPEED_24_BYTE        ((uint8_t)0x7U)
#define I2C_MASTER_SPEED_27_BYTE        ((uint8_t)0x8U)
#define I2C_MASTER_SPEED_30_BYTE        ((uint8_t)0x9U)
#define I2C_MASTER_SPEED_33_BYTE        ((uint8_t)0xAU)
#define I2C_MASTER_SPEED_36_BYTE        ((uint8_t)0xBU)
#define I2C_MASTER_SPEED_39_BYTE        ((uint8_t)0xCU)
#define I2C_MASTER_SPEED_42_BYTE        ((uint8_t)0xDU)
#define I2C_MASTER_SPEED_45_BYTE        ((uint8_t)0xEU)
#define I2C_MASTER_SPEED_48_BYTE        ((uint8_t)0xFU)
#define I2C_MASTER_SPEED_51_BYTE        ((uint8_t)0x10U)
#define I2C_MASTER_SPEED_54_BYTE        ((uint8_t)0x11U)
#define I2C_MASTER_SPEED_57_BYTE        ((uint8_t)0x12U)
#define I2C_MASTER_SPEED_60_BYTE        ((uint8_t)0x13U)
#define I2C_MASTER_SPEED_120_BYTE       ((uint8_t)0x27U)
#define I2C_MASTER_SPEED_768_BYTE       ((uint8_t)0xFFU)
#define I2C_SLAVE_SPEED_1_BYTE          ((uint8_t)0x0U)
#define I2C_SLAVE_SPEED_2_BYTE          ((uint8_t)0x1U)
#define I2C_SLAVE_SPEED_3_BYTE          ((uint8_t)0x2U)
#define I2C_SLAVE_SPEED_4_BYTE          ((uint8_t)0x3U)
#define I2C_SLAVE_SPEED_5_BYTE          ((uint8_t)0x4U)
#define I2C_SLAVE_SPEED_6_BYTE          ((uint8_t)0x5U)
#define I2C_SLAVE_SPEED_7_BYTE          ((uint8_t)0x6U)
#define I2C_SLAVE_SPEED_14_BYTE         ((uint8_t)0xDU)

/* I2C_CTRL1 bit positions */
#define I2C_CTRL1_RESET_Pos             5
#define I2C_CTRL1_LAST_DATA_Pos         4
#define I2C_CTRL1_STOP_Pos              3
#define I2C_CTRL1_NACK_Pos              1
#define I2C_CTRL1_ACK_Pos               0

/* I2C_CTRL1 bit-band aliases */
typedef struct
{
    __O  uint32_t ACK_ALIAS;            /* Issue an acknowledge on the I2C interface bus */
    __O  uint32_t NACK_ALIAS;           /* Issue a not acknowledge on the I2C interface bus */
         uint32_t RESERVED0;
    __O  uint32_t STOP_ALIAS;           /* Issue a stop condition on the I2C interface bus */
    __O  uint32_t LAST_DATA_ALIAS;      /* Indicate that the current data is the last byte of a data transfer */
    __O  uint32_t RESET_ALIAS;          /* Reset the I2C interface */
} I2C_CTRL1_Type;

#define I2C_CTRL1_BASE                  SYS_CALC_BITBAND(0x40000B04, 0)
#define I2C_CTRL1                       ((I2C_CTRL1_Type *) I2C_CTRL1_BASE)

/* I2C_CTRL1 settings */
#define I2C_ACK_BITBAND                 0x1
#define I2C_ACK                         ((uint32_t)(I2C_ACK_BITBAND << I2C_CTRL1_ACK_Pos))

#define I2C_NACK_BITBAND                0x1
#define I2C_NACK                        ((uint32_t)(I2C_NACK_BITBAND << I2C_CTRL1_NACK_Pos))

#define I2C_STOP_BITBAND                0x1
#define I2C_STOP                        ((uint32_t)(I2C_STOP_BITBAND << I2C_CTRL1_STOP_Pos))

#define I2C_LAST_DATA_BITBAND           0x1
#define I2C_LAST_DATA                   ((uint32_t)(I2C_LAST_DATA_BITBAND << I2C_CTRL1_LAST_DATA_Pos))

#define I2C_RESET_BITBAND               0x1
#define I2C_RESET                       ((uint32_t)(I2C_RESET_BITBAND << I2C_CTRL1_RESET_Pos))

/* I2C_ADDR_START bit positions */
#define I2C_ADDR_START_ADDRESS_Pos      1
#define I2C_ADDR_START_ADDRESS_Mask     ((uint32_t)(0x7FU << I2C_ADDR_START_ADDRESS_Pos))
#define I2C_ADDR_START_READ_WRITE_Pos   0

/* I2C_ADDR_START bit-band aliases */
typedef struct
{
    __O  uint32_t READ_WRITE_ALIAS;     /* Select whether a read or a write transaction is started */
} I2C_ADDR_START_Type;

#define I2C_ADDR_START_BASE             SYS_CALC_BITBAND(0x40000B10, 0)
#define I2C_ADDR_START                  ((I2C_ADDR_START_Type *) I2C_ADDR_START_BASE)

/* I2C_ADDR_START settings */
#define I2C_START_WRITE_BITBAND         0x0
#define I2C_START_READ_BITBAND          0x1
#define I2C_START_WRITE                 ((uint32_t)(I2C_START_WRITE_BITBAND << I2C_ADDR_START_READ_WRITE_Pos))
#define I2C_START_READ                  ((uint32_t)(I2C_START_READ_BITBAND << I2C_ADDR_START_READ_WRITE_Pos))

/* I2C_STATUS bit positions */
#define I2C_STATUS_ERROR_S_Pos          15
#define I2C_STATUS_BUS_ERROR_S_Pos      14
#define I2C_STATUS_START_PENDING_Pos    13
#define I2C_STATUS_MASTER_MODE_Pos      12
#define I2C_STATUS_DMA_REQ_Pos          11
#define I2C_STATUS_STOP_DETECT_Pos      10
#define I2C_STATUS_DATA_EVENT_Pos       9
#define I2C_STATUS_ERROR_Pos            8
#define I2C_STATUS_BUS_ERROR_Pos        7
#define I2C_STATUS_BUFFER_FULL_Pos      6
#define I2C_STATUS_CLK_STRETCH_Pos      5
#define I2C_STATUS_BUS_FREE_Pos         4
#define I2C_STATUS_ADDR_DATA_Pos        3
#define I2C_STATUS_READ_WRITE_Pos       2
#define I2C_STATUS_GEN_CALL_Pos         1
#define I2C_STATUS_ACK_STATUS_Pos       0

/* I2C_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t ACK_STATUS_ALIAS;     /* Indicate whether an acknowledge or a not acknowledge has been received */
    __I  uint32_t GEN_CALL_ALIAS;       /* Indicate whether the I2C bus transfer is using the general call address or another address */
    __I  uint32_t READ_WRITE_ALIAS;     /* Indicate whether the I2C bus transfer is a read or a write */
    __I  uint32_t ADDR_DATA_ALIAS;      /* Indicate if the I2C data register holds an address or data byte */
    __I  uint32_t BUS_FREE_ALIAS;       /* Indicate if the I2C interface bus is free */
    __I  uint32_t CLK_STRETCH_ALIAS;    /* Indicate if the I2C interface is holding the clock signal */
    __I  uint32_t BUFFER_FULL_ALIAS;    /* Indicate if the I2C data buffer is full */
    __I  uint32_t BUS_ERROR_ALIAS;      /* Indicate if the I2C interface has detected a bus error (automatically cleared when a stop condition is detected) */
    __I  uint32_t ERROR_ALIAS;          /* Same as BUS_ERROR: Indicate if the I2C interface has detected a bus error (automatically cleared when a stop condition is detected) */
    __I  uint32_t DATA_EVENT_ALIAS;     /* Indicate that I2C interface either needs data to transmit or has received data */
    __I  uint32_t STOP_DETECT_ALIAS;    /* Indicate if an I2C stop bit has been detected */
    __I  uint32_t DMA_REQ_ALIAS;        /* Indicate if the I2C interface is currently requesting DMA data */
    __I  uint32_t MASTER_MODE_ALIAS;    /* Master mode status bit */
    __I  uint32_t START_PENDING_ALIAS;  /* Master frame start pending status bit */
    __I  uint32_t BUS_ERROR_S_ALIAS;    /* Bus error status bit (sticky) - This status bit is automatically reset when the I2C_STATUS register is read. */
    __I  uint32_t ERROR_S_ALIAS;        /* Same as BUS_ERROR_S: Bus error status bit (sticky) - This status bit is automatically reset when the I2C_STATUS register is read. */
} I2C_STATUS_Type;

#define I2C_STATUS_BASE                 SYS_CALC_BITBAND(0x40000B14, 0)
#define I2C_STATUS                      ((I2C_STATUS_Type *) I2C_STATUS_BASE)

/* I2C_STATUS settings */
#define I2C_HAS_ACK_BITBAND             0x0
#define I2C_HAS_NACK_BITBAND            0x1
#define I2C_HAS_ACK                     ((uint32_t)(I2C_HAS_ACK_BITBAND << I2C_STATUS_ACK_STATUS_Pos))
#define I2C_HAS_NACK                    ((uint32_t)(I2C_HAS_NACK_BITBAND << I2C_STATUS_ACK_STATUS_Pos))

#define I2C_ADDR_OTHER_BITBAND          0x0
#define I2C_ADDR_GEN_CALL_BITBAND       0x1
#define I2C_ADDR_OTHER                  ((uint32_t)(I2C_ADDR_OTHER_BITBAND << I2C_STATUS_GEN_CALL_Pos))
#define I2C_ADDR_GEN_CALL               ((uint32_t)(I2C_ADDR_GEN_CALL_BITBAND << I2C_STATUS_GEN_CALL_Pos))

#define I2C_IS_WRITE_BITBAND            0x0
#define I2C_IS_READ_BITBAND             0x1
#define I2C_IS_WRITE                    ((uint32_t)(I2C_IS_WRITE_BITBAND << I2C_STATUS_READ_WRITE_Pos))
#define I2C_IS_READ                     ((uint32_t)(I2C_IS_READ_BITBAND << I2C_STATUS_READ_WRITE_Pos))

#define I2C_DATA_IS_DATA_BITBAND        0x0
#define I2C_DATA_IS_ADDR_BITBAND        0x1
#define I2C_DATA_IS_DATA                ((uint32_t)(I2C_DATA_IS_DATA_BITBAND << I2C_STATUS_ADDR_DATA_Pos))
#define I2C_DATA_IS_ADDR                ((uint32_t)(I2C_DATA_IS_ADDR_BITBAND << I2C_STATUS_ADDR_DATA_Pos))

#define I2C_BUS_BUSY_BITBAND            0x0
#define I2C_BUS_FREE_BITBAND            0x1
#define I2C_BUS_BUSY                    ((uint32_t)(I2C_BUS_BUSY_BITBAND << I2C_STATUS_BUS_FREE_Pos))
#define I2C_BUS_FREE                    ((uint32_t)(I2C_BUS_FREE_BITBAND << I2C_STATUS_BUS_FREE_Pos))

#define I2C_CLK_NOT_STRETCHED_BITBAND   0x0
#define I2C_CLK_STRETCHED_BITBAND       0x1
#define I2C_CLK_NOT_STRETCHED           ((uint32_t)(I2C_CLK_NOT_STRETCHED_BITBAND << I2C_STATUS_CLK_STRETCH_Pos))
#define I2C_CLK_STRETCHED               ((uint32_t)(I2C_CLK_STRETCHED_BITBAND << I2C_STATUS_CLK_STRETCH_Pos))

#define I2C_BUFFER_EMPTY_BITBAND        0x0
#define I2C_BUFFER_FULL_BITBAND         0x1
#define I2C_BUFFER_EMPTY                ((uint32_t)(I2C_BUFFER_EMPTY_BITBAND << I2C_STATUS_BUFFER_FULL_Pos))
#define I2C_BUFFER_FULL                 ((uint32_t)(I2C_BUFFER_FULL_BITBAND << I2C_STATUS_BUFFER_FULL_Pos))

#define I2C_NO_BUS_ERROR_BITBAND        0x0
#define I2C_BUS_ERROR_BITBAND           0x1
#define I2C_NO_BUS_ERROR                ((uint32_t)(I2C_NO_BUS_ERROR_BITBAND << I2C_STATUS_BUS_ERROR_Pos))
#define I2C_BUS_ERROR                   ((uint32_t)(I2C_BUS_ERROR_BITBAND << I2C_STATUS_BUS_ERROR_Pos))

#define I2C_NO_ERROR_BITBAND            0x0
#define I2C_ERROR_BITBAND               0x1
#define I2C_NO_ERROR                    ((uint32_t)(I2C_NO_ERROR_BITBAND << I2C_STATUS_ERROR_Pos))
#define I2C_ERROR                       ((uint32_t)(I2C_ERROR_BITBAND << I2C_STATUS_ERROR_Pos))

#define I2C_NON_DATA_EVENT_BITBAND      0x0
#define I2C_DATA_EVENT_BITBAND          0x1
#define I2C_NON_DATA_EVENT              ((uint32_t)(I2C_NON_DATA_EVENT_BITBAND << I2C_STATUS_DATA_EVENT_Pos))
#define I2C_DATA_EVENT                  ((uint32_t)(I2C_DATA_EVENT_BITBAND << I2C_STATUS_DATA_EVENT_Pos))

#define I2C_NO_STOP_DETECTED_BITBAND    0x0
#define I2C_STOP_DETECTED_BITBAND       0x1
#define I2C_NO_STOP_DETECTED            ((uint32_t)(I2C_NO_STOP_DETECTED_BITBAND << I2C_STATUS_STOP_DETECT_Pos))
#define I2C_STOP_DETECTED               ((uint32_t)(I2C_STOP_DETECTED_BITBAND << I2C_STATUS_STOP_DETECT_Pos))

#define I2C_NO_DMA_REQUEST_BITBAND      0x0
#define I2C_DMA_REQUEST_BITBAND         0x1
#define I2C_NO_DMA_REQUEST              ((uint32_t)(I2C_NO_DMA_REQUEST_BITBAND << I2C_STATUS_DMA_REQ_Pos))
#define I2C_DMA_REQUEST                 ((uint32_t)(I2C_DMA_REQUEST_BITBAND << I2C_STATUS_DMA_REQ_Pos))

#define I2C_MASTER_INACTIVE_BITBAND     0x0
#define I2C_MASTER_ACTIVE_BITBAND       0x1
#define I2C_MASTER_INACTIVE             ((uint32_t)(I2C_MASTER_INACTIVE_BITBAND << I2C_STATUS_MASTER_MODE_Pos))
#define I2C_MASTER_ACTIVE               ((uint32_t)(I2C_MASTER_ACTIVE_BITBAND << I2C_STATUS_MASTER_MODE_Pos))

#define I2C_START_NOT_PENDING_BITBAND   0x0
#define I2C_START_PENDING_BITBAND       0x1
#define I2C_START_NOT_PENDING           ((uint32_t)(I2C_START_NOT_PENDING_BITBAND << I2C_STATUS_START_PENDING_Pos))
#define I2C_START_PENDING               ((uint32_t)(I2C_START_PENDING_BITBAND << I2C_STATUS_START_PENDING_Pos))

#define I2C_NO_BUS_ERROR_S_BITBAND      0x0
#define I2C_BUS_ERROR_S_BITBAND         0x1
#define I2C_NO_BUS_ERROR_S              ((uint32_t)(I2C_NO_BUS_ERROR_S_BITBAND << I2C_STATUS_BUS_ERROR_S_Pos))
#define I2C_BUS_ERROR_S                 ((uint32_t)(I2C_BUS_ERROR_S_BITBAND << I2C_STATUS_BUS_ERROR_S_Pos))

#define I2C_NO_ERROR_S_BITBAND          0x0
#define I2C_ERROR_S_BITBAND             0x1
#define I2C_NO_ERROR_S                  ((uint32_t)(I2C_NO_ERROR_S_BITBAND << I2C_STATUS_ERROR_S_Pos))
#define I2C_ERROR_S                     ((uint32_t)(I2C_ERROR_S_BITBAND << I2C_STATUS_ERROR_S_Pos))

/* ----------------------------------------------------------------------------
 * UART Interface Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CFG;                  /* UART Configuration Register */
    __IO uint32_t TX_DATA;              /* UART Transmit Data Register */
    __I  uint32_t RX_DATA;              /* UART Receive Data Register */
    __IO uint32_t STATUS;               /* UART Status Register */
} UART_Type;

#define UART_BASE                       0x40000C00
#define UART                            ((UART_Type *) UART_BASE)

/* UART_CFG bit positions */
#define UART_CFG_PRESCALE_Pos           8
#define UART_CFG_PRESCALE_Mask          ((uint32_t)(0xFFFFU << UART_CFG_PRESCALE_Pos))
#define UART_CFG_PRESCALE_ENABLE_Pos    4
#define UART_CFG_DMA_ENABLE_Pos         1
#define UART_CFG_ENABLE_Pos             0

/* UART_CFG bit-band aliases */
typedef struct
{
    __IO uint32_t ENABLE_ALIAS;         /* Enable/disable the UART interface */
    __IO uint32_t DMA_ENABLE_ALIAS;     /* DMA mode enable */
         uint32_t RESERVED0[2];
    __IO uint32_t PRESCALE_ENABLE_ALIAS;/* Enable/disable a fixed prescaler by 12 */
} UART_CFG_Type;

#define UART_CFG_BASE                   SYS_CALC_BITBAND(0x40000C00, 0)
#define UART_CFG                        ((UART_CFG_Type *) UART_CFG_BASE)

/* UART_CFG settings */
#define UART_DISABLE_BITBAND            0x0
#define UART_ENABLE_BITBAND             0x1
#define UART_DISABLE                    ((uint32_t)(UART_DISABLE_BITBAND << UART_CFG_ENABLE_Pos))
#define UART_ENABLE                     ((uint32_t)(UART_ENABLE_BITBAND << UART_CFG_ENABLE_Pos))

#define UART_DMA_MODE_DISABLE_BITBAND   0x0
#define UART_DMA_MODE_ENABLE_BITBAND    0x1
#define UART_DMA_MODE_DISABLE           ((uint32_t)(UART_DMA_MODE_DISABLE_BITBAND << UART_CFG_DMA_ENABLE_Pos))
#define UART_DMA_MODE_ENABLE            ((uint32_t)(UART_DMA_MODE_ENABLE_BITBAND << UART_CFG_DMA_ENABLE_Pos))

#define UART_PRESCALE_DISABLE_BITBAND   0x0
#define UART_PRESCALE_ENABLE_BITBAND    0x1
#define UART_PRESCALE_DISABLE           ((uint32_t)(UART_PRESCALE_DISABLE_BITBAND << UART_CFG_PRESCALE_ENABLE_Pos))
#define UART_PRESCALE_ENABLE            ((uint32_t)(UART_PRESCALE_ENABLE_BITBAND << UART_CFG_PRESCALE_ENABLE_Pos))

/* UART_STATUS bit positions */
#define UART_STATUS_UART_RX_OVERRUN_STATUS_Pos 0

/* UART_STATUS bit-band aliases */
typedef struct
{
    __IO uint32_t UART_RX_OVERRUN_STATUS_ALIAS;/* Indicate that an overrun has occurred when receiving data on the UART interface */
} UART_STATUS_Type;

#define UART_STATUS_BASE                SYS_CALC_BITBAND(0x40000C0C, 0)
#define UART_STATUS                     ((UART_STATUS_Type *) UART_STATUS_BASE)

/* UART_STATUS settings */
#define UART_RX_OVERRUN_FALSE_BITBAND   0x0
#define UART_RX_OVERRUN_TRUE_BITBAND    0x1
#define UART_RX_OVERRUN_CLEAR_BITBAND   0x1
#define UART_RX_OVERRUN_FALSE           ((uint32_t)(UART_RX_OVERRUN_FALSE_BITBAND << UART_STATUS_UART_RX_OVERRUN_STATUS_Pos))
#define UART_RX_OVERRUN_TRUE            ((uint32_t)(UART_RX_OVERRUN_TRUE_BITBAND << UART_STATUS_UART_RX_OVERRUN_STATUS_Pos))
#define UART_RX_OVERRUN_CLEAR           ((uint32_t)(UART_RX_OVERRUN_CLEAR_BITBAND << UART_STATUS_UART_RX_OVERRUN_STATUS_Pos))

/* ----------------------------------------------------------------------------
 * PWM 0 and 1 Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CFG[2];               /* PWM Configuration Register */
    __IO uint32_t CTRL;                 /* PWM 0 and 1 Control Register */
} PWM_Type;

#define PWM_BASE                        0x40000D00
#define PWM                             ((PWM_Type *) PWM_BASE)

/* PWM_CFG[] bit positions */
#define PWM_CFG_PWM_HIGH_Pos            8
#define PWM_CFG_PWM_HIGH_Mask           ((uint32_t)(0xFFU << PWM_CFG_PWM_HIGH_Pos))
#define PWM_CFG_PWM_PERIOD_Pos          0
#define PWM_CFG_PWM_PERIOD_Mask         ((uint32_t)(0xFFU << PWM_CFG_PWM_PERIOD_Pos))

/* PWM_CFG[] sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t PWM_PERIOD_BYTE;      
    __IO uint8_t PWM_HIGH_BYTE;        
         uint8_t RESERVED0[2];
} PWM_CFG_Type;

#define PWM_CFG_BASE                    0x40000D00
#define PWM_CFG                         ((PWM_CFG_Type *) PWM_CFG_BASE)

/* PWM_CTRL bit positions */
#define PWM_CTRL_PWM_OFFSET_ENABLE_Pos  16
#define PWM_CTRL_PWM_OFFSET_Pos         8
#define PWM_CTRL_PWM_OFFSET_Mask        ((uint32_t)(0xFFU << PWM_CTRL_PWM_OFFSET_Pos))
#define PWM_CTRL_PWM1_ENABLE_Pos        4
#define PWM_CTRL_PWM0_ENABLE_Pos        0

/* PWM_CTRL bit-band aliases */
typedef struct
{
    __IO uint32_t PWM0_ENABLE_ALIAS;    /* Enable/disable the PWM0 block */
         uint32_t RESERVED0[3];
    __IO uint32_t PWM1_ENABLE_ALIAS;    /* Enable/disable the PWM1 block */
         uint32_t RESERVED1[11];
    __IO uint32_t PWM_OFFSET_ENABLE_ALIAS;/* Enable/disable the PWM offset function */
} PWM_CTRL_Type;

#define PWM_CTRL_BASE                   SYS_CALC_BITBAND(0x40000D08, 0)
#define PWM_CTRL                        ((PWM_CTRL_Type *) PWM_CTRL_BASE)

/* PWM_CTRL settings */
#define PWM0_DISABLE_BITBAND            0x0
#define PWM0_ENABLE_BITBAND             0x1
#define PWM0_DISABLE                    ((uint32_t)(PWM0_DISABLE_BITBAND << PWM_CTRL_PWM0_ENABLE_Pos))
#define PWM0_ENABLE                     ((uint32_t)(PWM0_ENABLE_BITBAND << PWM_CTRL_PWM0_ENABLE_Pos))

#define PWM1_DISABLE_BITBAND            0x0
#define PWM1_ENABLE_BITBAND             0x1
#define PWM1_DISABLE                    ((uint32_t)(PWM1_DISABLE_BITBAND << PWM_CTRL_PWM1_ENABLE_Pos))
#define PWM1_ENABLE                     ((uint32_t)(PWM1_ENABLE_BITBAND << PWM_CTRL_PWM1_ENABLE_Pos))

#define PWM_OFFSET_DISABLE_BITBAND      0x0
#define PWM_OFFSET_ENABLE_BITBAND       0x1
#define PWM_OFFSET_DISABLE              ((uint32_t)(PWM_OFFSET_DISABLE_BITBAND << PWM_CTRL_PWM_OFFSET_ENABLE_Pos))
#define PWM_OFFSET_ENABLE               ((uint32_t)(PWM_OFFSET_ENABLE_BITBAND << PWM_CTRL_PWM_OFFSET_ENABLE_Pos))

/* ----------------------------------------------------------------------------
 * DMIC Input and Output Driver Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CFG;                  /* DMIC and OD Configuration Register */
    __IO uint32_t STATUS;               /* DMIC and OD Status Register */
    __IO uint32_t DMIC_CFG;             /* DMIC Configuration Register */
    __IO uint32_t DMIC0_GAIN;           /* DMIC0 Gain Configuration Register */
    __IO uint32_t DMIC1_GAIN;           /* DMIC1 Gain Configuration Register */
    __I  uint32_t DMIC_DATA;            /* DMIC0 and DMIC1 Data Register */
    __I  uint32_t DMIC0_DATA;           /* DMIC0 Data Register */
    __I  uint32_t DMIC1_DATA;           /* DMIC1 Data Register */
    __IO uint32_t OD_CFG;               /* Output Driver Configuration Register */
    __IO uint32_t OD_GAIN;              /* Output Driver Gain Configuration Register */
    __IO uint32_t OD_DATA;              /* Output Driver Data Register */
         uint32_t RESERVED0;
    __IO uint32_t SDM_CFG;              /* Output Driver Sigma-delta Modulator Configuration Register */
} AUDIO_Type;

#define AUDIO_BASE                      0x40000E00
#define AUDIO                           ((AUDIO_Type *) AUDIO_BASE)

/* AUDIO_CFG bit positions */
#define AUDIO_CFG_OD_CLK_SRC_Pos        25
#define AUDIO_CFG_DMIC_CLK_SRC_Pos      24
#define AUDIO_CFG_DEC_RATE_Pos          16
#define AUDIO_CFG_DEC_RATE_Mask         ((uint32_t)(0x1FU << AUDIO_CFG_DEC_RATE_Pos))
#define AUDIO_CFG_OD_UNDERRUN_PROTECT_Pos 12
#define AUDIO_CFG_OD_DMA_REQ_EN_Pos     11
#define AUDIO_CFG_OD_INT_GEN_EN_Pos     10
#define AUDIO_CFG_OD_DATA_ALIGN_Pos     9
#define AUDIO_CFG_OD_ENABLE_Pos         8
#define AUDIO_CFG_DMIC1_DMA_REQ_EN_Pos  7
#define AUDIO_CFG_DMIC1_INT_GEN_EN_Pos  6
#define AUDIO_CFG_DMIC1_DATA_ALIGN_Pos  5
#define AUDIO_CFG_DMIC1_ENABLE_Pos      4
#define AUDIO_CFG_DMIC0_DMA_REQ_EN_Pos  3
#define AUDIO_CFG_DMIC0_INT_GEN_EN_Pos  2
#define AUDIO_CFG_DMIC0_DATA_ALIGN_Pos  1
#define AUDIO_CFG_DMIC0_ENABLE_Pos      0

/* AUDIO_CFG sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[2];
    __IO uint8_t DEC_RATE_BYTE;        
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000E00, 0) - (0x40000E00 + 4)];
    __IO uint32_t DMIC0_ENABLE_ALIAS;   /* Enable DMIC0 input */
    __IO uint32_t DMIC0_DATA_ALIGN_ALIAS;/* Data alignment in AUDIO_DMIC_DATA_0 */
    __IO uint32_t DMIC0_INT_GEN_EN_ALIAS;/* Enable the interrupt generation when a new DMIC0 sample is ready */
    __IO uint32_t DMIC0_DMA_REQ_EN_ALIAS;/* Enable the DMA request when a new DMIC0 sample is ready */
    __IO uint32_t DMIC1_ENABLE_ALIAS;   /* Enable DMIC1 input */
    __IO uint32_t DMIC1_DATA_ALIGN_ALIAS;/* Data alignment in AUDIO_DMIC_DATA_1 */
    __IO uint32_t DMIC1_INT_GEN_EN_ALIAS;/* Enable the interrupt generation when a new DMIC1 sample is ready */
    __IO uint32_t DMIC1_DMA_REQ_EN_ALIAS;/* Enable the DMA request when a new DMIC1 sample is ready */
    __IO uint32_t OD_ENABLE_ALIAS;      /* Enable output driver output */
    __IO uint32_t OD_DATA_ALIGN_ALIAS;  /* Data alignment in AUDIO_OD_DATA */
    __IO uint32_t OD_INT_GEN_EN_ALIAS;  /* Enable the interrupt generation when a new output driver sample is required */
    __IO uint32_t OD_DMA_REQ_EN_ALIAS;  /* Enable the DMA request when a new output driver sample is required */
    __IO uint32_t OD_UNDERRUN_PROTECT_ALIAS;/* Enable OD_DATA underrun protection (automatically resets OD_DATA if it hasn't been updated during 16 sample periods) */
         uint32_t RESERVED2[11];
    __IO uint32_t DMIC_CLK_SRC_ALIAS;   /* DMIC clock selection (the same clock must be output to the DMIC_CLK DIO pad) */
    __IO uint32_t OD_CLK_SRC_ALIAS;     /* Output driver clock selection */
} AUDIO_CFG_Type;

#define AUDIO_CFG_BASE                  0x40000E00
#define AUDIO_CFG                       ((AUDIO_CFG_Type *) AUDIO_CFG_BASE)

/* AUDIO_CFG settings */
#define DMIC0_DISABLE_BITBAND           0x0
#define DMIC0_ENABLE_BITBAND            0x1
#define DMIC0_DISABLE                   ((uint32_t)(DMIC0_DISABLE_BITBAND << AUDIO_CFG_DMIC0_ENABLE_Pos))
#define DMIC0_ENABLE                    ((uint32_t)(DMIC0_ENABLE_BITBAND << AUDIO_CFG_DMIC0_ENABLE_Pos))

#define DMIC0_DATA_LSB_ALIGNED_BITBAND  0x0
#define DMIC0_DATA_MSB_ALIGNED_BITBAND  0x1
#define DMIC0_DATA_LSB_ALIGNED          ((uint32_t)(DMIC0_DATA_LSB_ALIGNED_BITBAND << AUDIO_CFG_DMIC0_DATA_ALIGN_Pos))
#define DMIC0_DATA_MSB_ALIGNED          ((uint32_t)(DMIC0_DATA_MSB_ALIGNED_BITBAND << AUDIO_CFG_DMIC0_DATA_ALIGN_Pos))

#define DMIC0_INT_GEN_DISABLE_BITBAND   0x0
#define DMIC0_INT_GEN_ENABLE_BITBAND    0x1
#define DMIC0_INT_GEN_DISABLE           ((uint32_t)(DMIC0_INT_GEN_DISABLE_BITBAND << AUDIO_CFG_DMIC0_INT_GEN_EN_Pos))
#define DMIC0_INT_GEN_ENABLE            ((uint32_t)(DMIC0_INT_GEN_ENABLE_BITBAND << AUDIO_CFG_DMIC0_INT_GEN_EN_Pos))

#define DMIC0_DMA_REQ_DISABLE_BITBAND   0x0
#define DMIC0_DMA_REQ_ENABLE_BITBAND    0x1
#define DMIC0_DMA_REQ_DISABLE           ((uint32_t)(DMIC0_DMA_REQ_DISABLE_BITBAND << AUDIO_CFG_DMIC0_DMA_REQ_EN_Pos))
#define DMIC0_DMA_REQ_ENABLE            ((uint32_t)(DMIC0_DMA_REQ_ENABLE_BITBAND << AUDIO_CFG_DMIC0_DMA_REQ_EN_Pos))

#define DMIC1_DISABLE_BITBAND           0x0
#define DMIC1_ENABLE_BITBAND            0x1
#define DMIC1_DISABLE                   ((uint32_t)(DMIC1_DISABLE_BITBAND << AUDIO_CFG_DMIC1_ENABLE_Pos))
#define DMIC1_ENABLE                    ((uint32_t)(DMIC1_ENABLE_BITBAND << AUDIO_CFG_DMIC1_ENABLE_Pos))

#define DMIC1_DATA_LSB_ALIGNED_BITBAND  0x0
#define DMIC1_DATA_MSB_ALIGNED_BITBAND  0x1
#define DMIC1_DATA_LSB_ALIGNED          ((uint32_t)(DMIC1_DATA_LSB_ALIGNED_BITBAND << AUDIO_CFG_DMIC1_DATA_ALIGN_Pos))
#define DMIC1_DATA_MSB_ALIGNED          ((uint32_t)(DMIC1_DATA_MSB_ALIGNED_BITBAND << AUDIO_CFG_DMIC1_DATA_ALIGN_Pos))

#define DMIC1_INT_GEN_DISABLE_BITBAND   0x0
#define DMIC1_INT_GEN_ENABLE_BITBAND    0x1
#define DMIC1_INT_GEN_DISABLE           ((uint32_t)(DMIC1_INT_GEN_DISABLE_BITBAND << AUDIO_CFG_DMIC1_INT_GEN_EN_Pos))
#define DMIC1_INT_GEN_ENABLE            ((uint32_t)(DMIC1_INT_GEN_ENABLE_BITBAND << AUDIO_CFG_DMIC1_INT_GEN_EN_Pos))

#define DMIC1_DMA_REQ_DISABLE_BITBAND   0x0
#define DMIC1_DMA_REQ_ENABLE_BITBAND    0x1
#define DMIC1_DMA_REQ_DISABLE           ((uint32_t)(DMIC1_DMA_REQ_DISABLE_BITBAND << AUDIO_CFG_DMIC1_DMA_REQ_EN_Pos))
#define DMIC1_DMA_REQ_ENABLE            ((uint32_t)(DMIC1_DMA_REQ_ENABLE_BITBAND << AUDIO_CFG_DMIC1_DMA_REQ_EN_Pos))

#define OD_DISABLE_BITBAND              0x0
#define OD_ENABLE_BITBAND               0x1
#define OD_DISABLE                      ((uint32_t)(OD_DISABLE_BITBAND << AUDIO_CFG_OD_ENABLE_Pos))
#define OD_ENABLE                       ((uint32_t)(OD_ENABLE_BITBAND << AUDIO_CFG_OD_ENABLE_Pos))

#define OD_DATA_LSB_ALIGNED_BITBAND     0x0
#define OD_DATA_MSB_ALIGNED_BITBAND     0x1
#define OD_DATA_LSB_ALIGNED             ((uint32_t)(OD_DATA_LSB_ALIGNED_BITBAND << AUDIO_CFG_OD_DATA_ALIGN_Pos))
#define OD_DATA_MSB_ALIGNED             ((uint32_t)(OD_DATA_MSB_ALIGNED_BITBAND << AUDIO_CFG_OD_DATA_ALIGN_Pos))

#define OD_INT_GEN_DISABLE_BITBAND      0x0
#define OD_INT_GEN_ENABLE_BITBAND       0x1
#define OD_INT_GEN_DISABLE              ((uint32_t)(OD_INT_GEN_DISABLE_BITBAND << AUDIO_CFG_OD_INT_GEN_EN_Pos))
#define OD_INT_GEN_ENABLE               ((uint32_t)(OD_INT_GEN_ENABLE_BITBAND << AUDIO_CFG_OD_INT_GEN_EN_Pos))

#define OD_DMA_REQ_DISABLE_BITBAND      0x0
#define OD_DMA_REQ_ENABLE_BITBAND       0x1
#define OD_DMA_REQ_DISABLE              ((uint32_t)(OD_DMA_REQ_DISABLE_BITBAND << AUDIO_CFG_OD_DMA_REQ_EN_Pos))
#define OD_DMA_REQ_ENABLE               ((uint32_t)(OD_DMA_REQ_ENABLE_BITBAND << AUDIO_CFG_OD_DMA_REQ_EN_Pos))

#define OD_UNDERRUN_PROTECT_DISABLE_BITBAND 0x0
#define OD_UNDERRUN_PROTECT_ENABLE_BITBAND 0x1
#define OD_UNDERRUN_PROTECT_DISABLE     ((uint32_t)(OD_UNDERRUN_PROTECT_DISABLE_BITBAND << AUDIO_CFG_OD_UNDERRUN_PROTECT_Pos))
#define OD_UNDERRUN_PROTECT_ENABLE      ((uint32_t)(OD_UNDERRUN_PROTECT_ENABLE_BITBAND << AUDIO_CFG_OD_UNDERRUN_PROTECT_Pos))

#define DECIMATE_BY_64                  ((uint32_t)(0x0U << AUDIO_CFG_DEC_RATE_Pos))
#define DECIMATE_BY_72                  ((uint32_t)(0x1U << AUDIO_CFG_DEC_RATE_Pos))
#define DECIMATE_BY_80                  ((uint32_t)(0x2U << AUDIO_CFG_DEC_RATE_Pos))
#define DECIMATE_BY_128                 ((uint32_t)(0x8U << AUDIO_CFG_DEC_RATE_Pos))
#define DECIMATE_BY_136                 ((uint32_t)(0x9U << AUDIO_CFG_DEC_RATE_Pos))
#define DECIMATE_BY_256                 ((uint32_t)(0x16U << AUDIO_CFG_DEC_RATE_Pos))
#define DECIMATE_BY_288                 ((uint32_t)(0x1CU << AUDIO_CFG_DEC_RATE_Pos))

#define DMIC_AUDIOCLK_BITBAND           0x0
#define DMIC_AUDIOSLOWCLK_BITBAND       0x1
#define DMIC_AUDIOCLK                   ((uint32_t)(DMIC_AUDIOCLK_BITBAND << AUDIO_CFG_DMIC_CLK_SRC_Pos))
#define DMIC_AUDIOSLOWCLK               ((uint32_t)(DMIC_AUDIOSLOWCLK_BITBAND << AUDIO_CFG_DMIC_CLK_SRC_Pos))

#define OD_AUDIOCLK_BITBAND             0x0
#define OD_AUDIOSLOWCLK_BITBAND         0x1
#define OD_AUDIOCLK                     ((uint32_t)(OD_AUDIOCLK_BITBAND << AUDIO_CFG_OD_CLK_SRC_Pos))
#define OD_AUDIOSLOWCLK                 ((uint32_t)(OD_AUDIOSLOWCLK_BITBAND << AUDIO_CFG_OD_CLK_SRC_Pos))

/* AUDIO_CFG sub-register bit positions */
#define AUDIO_CFG_DEC_RATE_BYTE_Pos     0
#define AUDIO_CFG_DEC_RATE_BYTE_Mask    ((uint32_t)(0x1FU << AUDIO_CFG_DEC_RATE_BYTE_Pos))

/* AUDIO_CFG sub-register settings */
#define DECIMATE_BY_64_BYTE             ((uint8_t)(0x0U << AUDIO_CFG_DEC_RATE_BYTE_Pos))
#define DECIMATE_BY_72_BYTE             ((uint8_t)(0x1U << AUDIO_CFG_DEC_RATE_BYTE_Pos))
#define DECIMATE_BY_80_BYTE             ((uint8_t)(0x2U << AUDIO_CFG_DEC_RATE_BYTE_Pos))
#define DECIMATE_BY_128_BYTE            ((uint8_t)(0x8U << AUDIO_CFG_DEC_RATE_BYTE_Pos))
#define DECIMATE_BY_136_BYTE            ((uint8_t)(0x9U << AUDIO_CFG_DEC_RATE_BYTE_Pos))
#define DECIMATE_BY_256_BYTE            ((uint8_t)(0x16U << AUDIO_CFG_DEC_RATE_BYTE_Pos))
#define DECIMATE_BY_288_BYTE            ((uint8_t)(0x1CU << AUDIO_CFG_DEC_RATE_BYTE_Pos))

/* AUDIO_STATUS bit positions */
#define AUDIO_STATUS_OD_STATUS_Pos      11
#define AUDIO_STATUS_OD_UNDERRUN_FLAG_CLEAR_Pos 10
#define AUDIO_STATUS_OD_UNDERRUN_FLAG_Pos 9
#define AUDIO_STATUS_OD_DATA_REQ_FLAG_Pos 8
#define AUDIO_STATUS_DMIC1_OVERRUN_FLAG_CLEAR_Pos 6
#define AUDIO_STATUS_DMIC1_OVERRUN_FLAG_Pos 5
#define AUDIO_STATUS_DMIC1_DATA_RDY_FLAG_Pos 4
#define AUDIO_STATUS_DMIC0_OVERRUN_FLAG_CLEAR_Pos 2
#define AUDIO_STATUS_DMIC0_OVERRUN_FLAG_Pos 1
#define AUDIO_STATUS_DMIC0_DATA_RDY_FLAG_Pos 0

/* AUDIO_STATUS sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t OD_BYTE;              
         uint8_t RESERVED1[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000E04, 0) - (0x40000E04 + 4)];
    __I  uint32_t DMIC0_DATA_RDY_FLAG_ALIAS;/* Flag indicating the availability of a new DMIC0 sample */
    __I  uint32_t DMIC0_OVERRUN_FLAG_ALIAS;/* Sticky bit indicating the detection of a DMIC0 overrun */
    __O  uint32_t DMIC0_OVERRUN_FLAG_CLEAR_ALIAS;/* Reset the DMIC0 overrun detection sticky bit */
         uint32_t RESERVED2;
    __I  uint32_t DMIC1_DATA_RDY_FLAG_ALIAS;/* Flag indicating the availability of a new DMIC1 sample */
    __I  uint32_t DMIC1_OVERRUN_FLAG_ALIAS;/* Sticky bit indicating the detection of a DMIC1 overrun */
    __O  uint32_t DMIC1_OVERRUN_FLAG_CLEAR_ALIAS;/* Reset the DMIC1 overrun detection sticky bit */
         uint32_t RESERVED3;
    __I  uint32_t OD_DATA_REQ_FLAG_ALIAS;/* Flag indicating that a new output driver sample is required */
    __I  uint32_t OD_UNDERRUN_FLAG_ALIAS;/* Sticky bit indicating the detection of an output driver underrun */
    __O  uint32_t OD_UNDERRUN_FLAG_CLEAR_ALIAS;/* Reset the output driver underrun detection sticky bit */
    __I  uint32_t OD_STATUS_ALIAS;      /* Output driver feature status */
} AUDIO_STATUS_Type;

#define AUDIO_STATUS_BASE               0x40000E04
#define AUDIO_STATUS                    ((AUDIO_STATUS_Type *) AUDIO_STATUS_BASE)

/* AUDIO_STATUS settings */
#define DMIC0_DATA_NOT_READY_BITBAND    0x0
#define DMIC0_DATA_READY_BITBAND        0x1
#define DMIC0_DATA_NOT_READY            ((uint32_t)(DMIC0_DATA_NOT_READY_BITBAND << AUDIO_STATUS_DMIC0_DATA_RDY_FLAG_Pos))
#define DMIC0_DATA_READY                ((uint32_t)(DMIC0_DATA_READY_BITBAND << AUDIO_STATUS_DMIC0_DATA_RDY_FLAG_Pos))

#define DMIC0_OVERRUN_NOT_DETECTED_BITBAND 0x0
#define DMIC0_OVERRUN_DETECTED_BITBAND  0x1
#define DMIC0_OVERRUN_NOT_DETECTED      ((uint32_t)(DMIC0_OVERRUN_NOT_DETECTED_BITBAND << AUDIO_STATUS_DMIC0_OVERRUN_FLAG_Pos))
#define DMIC0_OVERRUN_DETECTED          ((uint32_t)(DMIC0_OVERRUN_DETECTED_BITBAND << AUDIO_STATUS_DMIC0_OVERRUN_FLAG_Pos))

#define DMIC0_OVERRUN_FLAG_CLEAR_BITBAND 0x1
#define DMIC0_OVERRUN_FLAG_CLEAR        ((uint32_t)(DMIC0_OVERRUN_FLAG_CLEAR_BITBAND << AUDIO_STATUS_DMIC0_OVERRUN_FLAG_CLEAR_Pos))

#define DMIC1_DATA_NOT_READY_BITBAND    0x0
#define DMIC1_DATA_READY_BITBAND        0x1
#define DMIC1_DATA_NOT_READY            ((uint32_t)(DMIC1_DATA_NOT_READY_BITBAND << AUDIO_STATUS_DMIC1_DATA_RDY_FLAG_Pos))
#define DMIC1_DATA_READY                ((uint32_t)(DMIC1_DATA_READY_BITBAND << AUDIO_STATUS_DMIC1_DATA_RDY_FLAG_Pos))

#define DMIC1_OVERRUN_NOT_DETECTED_BITBAND 0x0
#define DMIC1_OVERRUN_DETECTED_BITBAND  0x1
#define DMIC1_OVERRUN_NOT_DETECTED      ((uint32_t)(DMIC1_OVERRUN_NOT_DETECTED_BITBAND << AUDIO_STATUS_DMIC1_OVERRUN_FLAG_Pos))
#define DMIC1_OVERRUN_DETECTED          ((uint32_t)(DMIC1_OVERRUN_DETECTED_BITBAND << AUDIO_STATUS_DMIC1_OVERRUN_FLAG_Pos))

#define DMIC1_OVERRUN_FLAG_CLEAR_BITBAND 0x1
#define DMIC1_OVERRUN_FLAG_CLEAR        ((uint32_t)(DMIC1_OVERRUN_FLAG_CLEAR_BITBAND << AUDIO_STATUS_DMIC1_OVERRUN_FLAG_CLEAR_Pos))

#define OD_DATA_NOT_REQUIRED_BITBAND    0x0
#define OD_DATA_REQUIRED_BITBAND        0x1
#define OD_DATA_NOT_REQUIRED            ((uint32_t)(OD_DATA_NOT_REQUIRED_BITBAND << AUDIO_STATUS_OD_DATA_REQ_FLAG_Pos))
#define OD_DATA_REQUIRED                ((uint32_t)(OD_DATA_REQUIRED_BITBAND << AUDIO_STATUS_OD_DATA_REQ_FLAG_Pos))

#define OD_UNDERRUN_NOT_DETECTED_BITBAND 0x0
#define OD_UNDERRUN_DETECTED_BITBAND    0x1
#define OD_UNDERRUN_NOT_DETECTED        ((uint32_t)(OD_UNDERRUN_NOT_DETECTED_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_Pos))
#define OD_UNDERRUN_DETECTED            ((uint32_t)(OD_UNDERRUN_DETECTED_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_Pos))

#define OD_UNDERRUN_FLAG_CLEAR_BITBAND  0x1
#define OD_UNDERRUN_FLAG_CLEAR          ((uint32_t)(OD_UNDERRUN_FLAG_CLEAR_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_CLEAR_Pos))

#define OD_DISABLED_DEVICE_BITBAND      0x0
#define OD_ENABLED_DEVICE_BITBAND       0x1
#define OD_DISABLED_DEVICE              ((uint32_t)(OD_DISABLED_DEVICE_BITBAND << AUDIO_STATUS_OD_STATUS_Pos))
#define OD_ENABLED_DEVICE               ((uint32_t)(OD_ENABLED_DEVICE_BITBAND << AUDIO_STATUS_OD_STATUS_Pos))

/* AUDIO_STATUS sub-register bit positions */
#define AUDIO_STATUS_OD_DATA_REQ_FLAG_BYTE_Pos 0
#define AUDIO_STATUS_OD_UNDERRUN_FLAG_BYTE_Pos 1
#define AUDIO_STATUS_OD_UNDERRUN_FLAG_CLEAR_BYTE_Pos 2
#define AUDIO_STATUS_OD_STATUS_BYTE_Pos 3

/* AUDIO_STATUS sub-register settings */
#define OD_DATA_NOT_REQUIRED_BYTE       ((uint8_t)(OD_DATA_NOT_REQUIRED_BITBAND << AUDIO_STATUS_OD_DATA_REQ_FLAG_BYTE_Pos))
#define OD_DATA_REQUIRED_BYTE           ((uint8_t)(OD_DATA_REQUIRED_BITBAND << AUDIO_STATUS_OD_DATA_REQ_FLAG_BYTE_Pos))

#define OD_UNDERRUN_NOT_DETECTED_BYTE   ((uint8_t)(OD_UNDERRUN_NOT_DETECTED_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_BYTE_Pos))
#define OD_UNDERRUN_DETECTED_BYTE       ((uint8_t)(OD_UNDERRUN_DETECTED_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_BYTE_Pos))

#define OD_UNDERRUN_FLAG_CLEAR_BYTE     ((uint8_t)(OD_UNDERRUN_FLAG_CLEAR_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_CLEAR_BYTE_Pos))

#define OD_DISABLED_DEVICE_BYTE         ((uint8_t)(OD_DISABLED_DEVICE_BITBAND << AUDIO_STATUS_OD_STATUS_BYTE_Pos))
#define OD_ENABLED_DEVICE_BYTE          ((uint8_t)(OD_ENABLED_DEVICE_BITBAND << AUDIO_STATUS_OD_STATUS_BYTE_Pos))

/* AUDIO_DMIC_CFG bit positions */
#define AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_Pos 24
#define AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_Mask ((uint32_t)(0x1FU << AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_Pos))
#define AUDIO_DMIC_CFG_DMIC1_DELAY_Pos  16
#define AUDIO_DMIC_CFG_DMIC1_DELAY_Mask ((uint32_t)(0xFU << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define AUDIO_DMIC_CFG_DMIC1_DCRM_Pos   12
#define AUDIO_DMIC_CFG_DMIC1_DCRM_Mask  ((uint32_t)(0x7U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))
#define AUDIO_DMIC_CFG_DMIC0_DCRM_Pos   8
#define AUDIO_DMIC_CFG_DMIC0_DCRM_Mask  ((uint32_t)(0x7U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))
#define AUDIO_DMIC_CFG_DMIC1_CLK_EDGE_Pos 1
#define AUDIO_DMIC_CFG_DMIC0_CLK_EDGE_Pos 0

/* AUDIO_DMIC_CFG sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t DMIC1_FRAC_DELAY_BYTE;
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000E08, 0) - (0x40000E08 + 4)];
    __IO uint32_t DMIC0_CLK_EDGE_ALIAS; /* DMIC0 input clock edge */
    __IO uint32_t DMIC1_CLK_EDGE_ALIAS; /* DMIC1 input clock edge */
} AUDIO_DMIC_CFG_Type;

#define AUDIO_DMIC_CFG_BASE             0x40000E08
#define AUDIO_DMIC_CFG                  ((AUDIO_DMIC_CFG_Type *) AUDIO_DMIC_CFG_BASE)

/* AUDIO_DMIC_CFG settings */
#define DMIC0_FALLING_EDGE_BITBAND      0x0
#define DMIC0_RISING_EDGE_BITBAND       0x1
#define DMIC0_FALLING_EDGE              ((uint32_t)(DMIC0_FALLING_EDGE_BITBAND << AUDIO_DMIC_CFG_DMIC0_CLK_EDGE_Pos))
#define DMIC0_RISING_EDGE               ((uint32_t)(DMIC0_RISING_EDGE_BITBAND << AUDIO_DMIC_CFG_DMIC0_CLK_EDGE_Pos))

#define DMIC1_FALLING_EDGE_BITBAND      0x0
#define DMIC1_RISING_EDGE_BITBAND       0x1
#define DMIC1_FALLING_EDGE              ((uint32_t)(DMIC1_FALLING_EDGE_BITBAND << AUDIO_DMIC_CFG_DMIC1_CLK_EDGE_Pos))
#define DMIC1_RISING_EDGE               ((uint32_t)(DMIC1_RISING_EDGE_BITBAND << AUDIO_DMIC_CFG_DMIC1_CLK_EDGE_Pos))

#define DMIC0_DCRM_CUTOFF_5HZ           ((uint32_t)(0x0U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))
#define DMIC0_DCRM_CUTOFF_10HZ          ((uint32_t)(0x1U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))
#define DMIC0_DCRM_CUTOFF_20HZ          ((uint32_t)(0x2U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))
#define DMIC0_DCRM_CUTOFF_40HZ          ((uint32_t)(0x3U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))
#define DMIC0_DCRM_CUTOFF_80HZ          ((uint32_t)(0x4U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))
#define DMIC0_DCRM_CUTOFF_160HZ         ((uint32_t)(0x5U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))
#define DMIC0_DCRM_CUTOFF_320HZ         ((uint32_t)(0x6U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))
#define DMIC0_DCRM_DISABLE              ((uint32_t)(0x7U << AUDIO_DMIC_CFG_DMIC0_DCRM_Pos))

#define DMIC1_DCRM_CUTOFF_5HZ           ((uint32_t)(0x0U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))
#define DMIC1_DCRM_CUTOFF_10HZ          ((uint32_t)(0x1U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))
#define DMIC1_DCRM_CUTOFF_20HZ          ((uint32_t)(0x2U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))
#define DMIC1_DCRM_CUTOFF_40HZ          ((uint32_t)(0x3U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))
#define DMIC1_DCRM_CUTOFF_80HZ          ((uint32_t)(0x4U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))
#define DMIC1_DCRM_CUTOFF_160HZ         ((uint32_t)(0x5U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))
#define DMIC1_DCRM_CUTOFF_320HZ         ((uint32_t)(0x6U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))
#define DMIC1_DCRM_DISABLE              ((uint32_t)(0x7U << AUDIO_DMIC_CFG_DMIC1_DCRM_Pos))

#define DMIC1_DELAY_DISABLE             ((uint32_t)(0x0U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_0P125               ((uint32_t)(0x1U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_0P25                ((uint32_t)(0x2U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_0P375               ((uint32_t)(0x3U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_0P5                 ((uint32_t)(0x4U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_0P625               ((uint32_t)(0x5U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_0P75                ((uint32_t)(0x6U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_0P875               ((uint32_t)(0x7U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_1P0                 ((uint32_t)(0x8U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_1P125               ((uint32_t)(0x9U << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_1P25                ((uint32_t)(0xAU << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_1P375               ((uint32_t)(0xBU << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_1P5                 ((uint32_t)(0xCU << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_1P625               ((uint32_t)(0xDU << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_1P75                ((uint32_t)(0xEU << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))
#define DMIC1_DELAY_1P875               ((uint32_t)(0xFU << AUDIO_DMIC_CFG_DMIC1_DELAY_Pos))

/* AUDIO_DMIC_CFG sub-register bit positions */
#define AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_BYTE_Pos 0
#define AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_BYTE_Mask ((uint32_t)(0x1FU << AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_BYTE_Pos))

/* AUDIO_DMIC0_GAIN bit positions */
#define AUDIO_DMIC0_GAIN_GAIN_Pos       0
#define AUDIO_DMIC0_GAIN_GAIN_Mask      ((uint32_t)(0xFFFU << AUDIO_DMIC0_GAIN_GAIN_Pos))

/* AUDIO_DMIC0_GAIN settings */
#define DMIC1_NOMINAL_GAIN              ((uint32_t)(0x800U << AUDIO_DMIC0_GAIN_GAIN_Pos))

/* AUDIO_DMIC1_GAIN bit positions */
#define AUDIO_DMIC1_GAIN_GAIN_Pos       0
#define AUDIO_DMIC1_GAIN_GAIN_Mask      ((uint32_t)(0xFFFU << AUDIO_DMIC1_GAIN_GAIN_Pos))

/* AUDIO_DMIC1_GAIN settings */
#define DMIC0_NOMINAL_GAIN              ((uint32_t)(0x800U << AUDIO_DMIC1_GAIN_GAIN_Pos))

/* AUDIO_DMIC_DATA bit positions */
#define AUDIO_DMIC_DATA_DMIC1_DATA_Pos  16
#define AUDIO_DMIC_DATA_DMIC1_DATA_Mask ((uint32_t)(0xFFFFU << AUDIO_DMIC_DATA_DMIC1_DATA_Pos))
#define AUDIO_DMIC_DATA_DMIC0_DATA_Pos  0
#define AUDIO_DMIC_DATA_DMIC0_DATA_Mask ((uint32_t)(0xFFFFU << AUDIO_DMIC_DATA_DMIC0_DATA_Pos))

/* AUDIO_DMIC_DATA sub-register and bit-band aliases */
typedef struct
{
    __I  uint16_t DMIC0_DATA_SHORT;     
    __I  uint16_t DMIC1_DATA_SHORT;     
} AUDIO_DMIC_DATA_Type;

#define AUDIO_DMIC_DATA_BASE            0x40000E14
#define AUDIO_DMIC_DATA                 ((AUDIO_DMIC_DATA_Type *) AUDIO_DMIC_DATA_BASE)

/* AUDIO_DMIC0_DATA bit positions */
#define AUDIO_DMIC0_DATA_DATA_Pos       0
#define AUDIO_DMIC0_DATA_DATA_Mask      ((uint32_t)(0xFFFFFFFFU << AUDIO_DMIC0_DATA_DATA_Pos))

/* AUDIO_DMIC1_DATA bit positions */
#define AUDIO_DMIC1_DATA_DATA_Pos       0
#define AUDIO_DMIC1_DATA_DATA_Mask      ((uint32_t)(0xFFFFFFFFU << AUDIO_DMIC1_DATA_DATA_Pos))

/* AUDIO_OD_CFG bit positions */
#define AUDIO_OD_CFG_DCRM_Pos           16
#define AUDIO_OD_CFG_DCRM_Mask          ((uint32_t)(0xFU << AUDIO_OD_CFG_DCRM_Pos))
#define AUDIO_OD_CFG_DITHER_Pos         10
#define AUDIO_OD_CFG_CLK_EDGE_Pos       0

/* AUDIO_OD_CFG sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t CLK_EDGE_BYTE;        
    __IO uint8_t DITHER_BYTE;          
    __IO uint8_t DCRM_BYTE;            
         uint8_t RESERVED0[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40000E20, 0) - (0x40000E20 + 4)];
    __IO uint32_t CLK_EDGE_ALIAS;       /* Output driver output clock edge */
         uint32_t RESERVED1[9];
    __IO uint32_t DITHER_ALIAS;         /* Sigma-delta modulator dithering enable */
} AUDIO_OD_CFG_Type;

#define AUDIO_OD_CFG_BASE               0x40000E20
#define AUDIO_OD_CFG                    ((AUDIO_OD_CFG_Type *) AUDIO_OD_CFG_BASE)

/* AUDIO_OD_CFG settings */
#define OD_FALLING_EDGE_BITBAND         0x0
#define OD_RISING_EDGE_BITBAND          0x1
#define OD_FALLING_EDGE                 ((uint32_t)(OD_FALLING_EDGE_BITBAND << AUDIO_OD_CFG_CLK_EDGE_Pos))
#define OD_RISING_EDGE                  ((uint32_t)(OD_RISING_EDGE_BITBAND << AUDIO_OD_CFG_CLK_EDGE_Pos))

#define DITHER_DISABLE_BITBAND          0x0
#define DITHER_ENABLE_BITBAND           0x1
#define DITHER_DISABLE                  ((uint32_t)(DITHER_DISABLE_BITBAND << AUDIO_OD_CFG_DITHER_Pos))
#define DITHER_ENABLE                   ((uint32_t)(DITHER_ENABLE_BITBAND << AUDIO_OD_CFG_DITHER_Pos))

#define DCRM_DISABLE                    ((uint32_t)(0x0U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_1P25HZ              ((uint32_t)(0x1U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_2P5HZ               ((uint32_t)(0x2U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_3P75HZ              ((uint32_t)(0x3U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_5HZ                 ((uint32_t)(0x4U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_7P5HZ               ((uint32_t)(0x5U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_10HZ                ((uint32_t)(0x6U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_15HZ                ((uint32_t)(0x7U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_20HZ                ((uint32_t)(0x8U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_30HZ                ((uint32_t)(0x9U << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_40HZ                ((uint32_t)(0xAU << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_60HZ                ((uint32_t)(0xBU << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_80HZ                ((uint32_t)(0xCU << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_120HZ               ((uint32_t)(0xDU << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_160HZ               ((uint32_t)(0xEU << AUDIO_OD_CFG_DCRM_Pos))
#define DCRM_CUTOFF_240HZ               ((uint32_t)(0xFU << AUDIO_OD_CFG_DCRM_Pos))

/* AUDIO_OD_CFG sub-register bit positions */
#define AUDIO_OD_CFG_CLK_EDGE_BYTE_Pos  0
#define AUDIO_OD_CFG_DITHER_BYTE_Pos    2
#define AUDIO_OD_CFG_DCRM_BYTE_Pos      0
#define AUDIO_OD_CFG_DCRM_BYTE_Mask     ((uint32_t)(0xFU << AUDIO_OD_CFG_DCRM_BYTE_Pos))

/* AUDIO_OD_CFG sub-register settings */
#define OD_FALLING_EDGE_BYTE            ((uint8_t)(OD_FALLING_EDGE_BITBAND << AUDIO_OD_CFG_CLK_EDGE_BYTE_Pos))
#define OD_RISING_EDGE_BYTE             ((uint8_t)(OD_RISING_EDGE_BITBAND << AUDIO_OD_CFG_CLK_EDGE_BYTE_Pos))

#define DITHER_DISABLE_BYTE             ((uint8_t)(DITHER_DISABLE_BITBAND << AUDIO_OD_CFG_DITHER_BYTE_Pos))
#define DITHER_ENABLE_BYTE              ((uint8_t)(DITHER_ENABLE_BITBAND << AUDIO_OD_CFG_DITHER_BYTE_Pos))

#define DCRM_DISABLE_BYTE               ((uint8_t)(0x0U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_1P25HZ_BYTE         ((uint8_t)(0x1U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_2P5HZ_BYTE          ((uint8_t)(0x2U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_3P75HZ_BYTE         ((uint8_t)(0x3U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_5HZ_BYTE            ((uint8_t)(0x4U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_7P5HZ_BYTE          ((uint8_t)(0x5U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_10HZ_BYTE           ((uint8_t)(0x6U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_15HZ_BYTE           ((uint8_t)(0x7U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_20HZ_BYTE           ((uint8_t)(0x8U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_30HZ_BYTE           ((uint8_t)(0x9U << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_40HZ_BYTE           ((uint8_t)(0xAU << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_60HZ_BYTE           ((uint8_t)(0xBU << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_80HZ_BYTE           ((uint8_t)(0xCU << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_120HZ_BYTE          ((uint8_t)(0xDU << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_160HZ_BYTE          ((uint8_t)(0xEU << AUDIO_OD_CFG_DCRM_BYTE_Pos))
#define DCRM_CUTOFF_240HZ_BYTE          ((uint8_t)(0xFU << AUDIO_OD_CFG_DCRM_BYTE_Pos))

/* AUDIO_OD_GAIN bit positions */
#define AUDIO_OD_GAIN_GAIN_Pos          0
#define AUDIO_OD_GAIN_GAIN_Mask         ((uint32_t)(0xFFFU << AUDIO_OD_GAIN_GAIN_Pos))

/* AUDIO_OD_GAIN settings */
#define OD_NOMINAL_GAIN                 ((uint32_t)(0x800U << AUDIO_OD_GAIN_GAIN_Pos))

/* AUDIO_OD_DATA bit positions */
#define AUDIO_OD_DATA_DATA_Pos          0
#define AUDIO_OD_DATA_DATA_Mask         ((uint32_t)(0xFFFFFFFFU << AUDIO_OD_DATA_DATA_Pos))
#define AUDIO_OD_DATA_DATA_RD_Pos       0
#define AUDIO_OD_DATA_DATA_RD_Mask      ((uint32_t)(0xFFFFFFFFU << AUDIO_OD_DATA_DATA_RD_Pos))

/* AUDIO_SDM_CFG bit positions */
#define AUDIO_SDM_CFG_SDM_CFG_Pos       0
#define AUDIO_SDM_CFG_SDM_CFG_Mask      ((uint32_t)(0xFFFFFFFFU << AUDIO_SDM_CFG_SDM_CFG_Pos))

/* AUDIO_SDM_CFG settings */
#define SDM_CFG_NORMAL                  ((uint32_t)(0x50012U << AUDIO_SDM_CFG_SDM_CFG_Pos))

/* ----------------------------------------------------------------------------
 * CRC Generator Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL;                 /* CRC Generator Control */
    __IO uint32_t VALUE;                /* CRC Generator Current Value */
    __O  uint32_t ADD_1;                /* CRC Generator - Add 1 Bit */
    __O  uint32_t ADD_8;                /* CRC Generator - Add 1 Byte */
    __O  uint32_t ADD_16;               /* CRC Generator - Add 1 Half-word */
    __O  uint32_t ADD_24;               /* CRC Generator - Add 3 Bytes */
    __O  uint32_t ADD_32;               /* CRC Generator - Add 1 Word */
    __I  uint32_t FINAL;                /* CRC Generator Final Value */
} CRC_Type;

#define CRC_BASE                        0x40000F00
#define CRC                             ((CRC_Type *) CRC_BASE)

/* CRC_CTRL bit positions */
#define CRC_CTRL_FINAL_CRC_XOR_Pos      4
#define CRC_CTRL_FINAL_CRC_REVERSE_Pos  3
#define CRC_CTRL_BIT_ORDER_Pos          2
#define CRC_CTRL_CRC_TYPE_Pos           1
#define CRC_CTRL_BYTE_ORDER_Pos         0

/* CRC_CTRL bit-band aliases */
typedef struct
{
    __IO uint32_t BYTE_ORDER_ALIAS;     /* Selects the endianness for bytes added to the CRC */
    __IO uint32_t CRC_TYPE_ALIAS;       /* Selects the CRC type */
    __IO uint32_t BIT_ORDER_ALIAS;      /* Selects the bit order for bytes added to the CRC */
    __IO uint32_t FINAL_CRC_REVERSE_ALIAS;/* Selects the final CRC reversal mode */
    __IO uint32_t FINAL_CRC_XOR_ALIAS;  /* Selects the final CRC XOR mode */
} CRC_CTRL_Type;

#define CRC_CTRL_BASE                   SYS_CALC_BITBAND(0x40000F00, 0)
#define CRC_CTRL                        ((CRC_CTRL_Type *) CRC_CTRL_BASE)

/* CRC_CTRL settings */
#define CRC_BIG_ENDIAN_BITBAND          0x0
#define CRC_LITTLE_ENDIAN_BITBAND       0x1
#define CRC_BIG_ENDIAN                  ((uint32_t)(CRC_BIG_ENDIAN_BITBAND << CRC_CTRL_BYTE_ORDER_Pos))
#define CRC_LITTLE_ENDIAN               ((uint32_t)(CRC_LITTLE_ENDIAN_BITBAND << CRC_CTRL_BYTE_ORDER_Pos))

#define CRC_CCITT_BITBAND               0x0
#define CRC_32_BITBAND                  0x1
#define CRC_CCITT                       ((uint32_t)(CRC_CCITT_BITBAND << CRC_CTRL_CRC_TYPE_Pos))
#define CRC_32                          ((uint32_t)(CRC_32_BITBAND << CRC_CTRL_CRC_TYPE_Pos))

#define CRC_BIT_ORDER_STANDARD_BITBAND  0x0
#define CRC_BIT_ORDER_NON_STANDARD_BITBAND 0x1
#define CRC_BIT_ORDER_STANDARD          ((uint32_t)(CRC_BIT_ORDER_STANDARD_BITBAND << CRC_CTRL_BIT_ORDER_Pos))
#define CRC_BIT_ORDER_NON_STANDARD      ((uint32_t)(CRC_BIT_ORDER_NON_STANDARD_BITBAND << CRC_CTRL_BIT_ORDER_Pos))

#define CRC_FINAL_REVERSE_STANDARD_BITBAND 0x0
#define CRC_FINAL_REVERSE_NON_STANDARD_BITBAND 0x1
#define CRC_FINAL_REVERSE_STANDARD      ((uint32_t)(CRC_FINAL_REVERSE_STANDARD_BITBAND << CRC_CTRL_FINAL_CRC_REVERSE_Pos))
#define CRC_FINAL_REVERSE_NON_STANDARD  ((uint32_t)(CRC_FINAL_REVERSE_NON_STANDARD_BITBAND << CRC_CTRL_FINAL_CRC_REVERSE_Pos))

#define CRC_FINAL_XOR_STANDARD_BITBAND  0x0
#define CRC_FINAL_XOR_NON_STANDARD_BITBAND 0x1
#define CRC_FINAL_XOR_STANDARD          ((uint32_t)(CRC_FINAL_XOR_STANDARD_BITBAND << CRC_CTRL_FINAL_CRC_XOR_Pos))
#define CRC_FINAL_XOR_NON_STANDARD      ((uint32_t)(CRC_FINAL_XOR_NON_STANDARD_BITBAND << CRC_CTRL_FINAL_CRC_XOR_Pos))

/* CRC_VALUE bit positions */
#define CRC_VALUE_CURRENT_CRC_Pos       0
#define CRC_VALUE_CURRENT_CRC_Mask      ((uint32_t)(0xFFFFFFFFU << CRC_VALUE_CURRENT_CRC_Pos))

/* CRC_VALUE settings */
#define CRC_CCITT_INIT_VALUE            ((uint32_t)(0xFFFFU << CRC_VALUE_CURRENT_CRC_Pos))
#define CRC_32_INIT_VALUE               ((uint32_t)(0xFFFFFFFFU << CRC_VALUE_CURRENT_CRC_Pos))

/* CRC_ADD_1 bit positions */
#define CRC_ADD_1_CRC_ADD_1_BIT_Pos     0

/* CRC_ADD_1 bit-band aliases */
typedef struct
{
    __O  uint32_t CRC_ADD_1_BIT_ALIAS;  /* Add 1 bit to the CRC calculation */
} CRC_ADD_1_Type;

#define CRC_ADD_1_BASE                  SYS_CALC_BITBAND(0x40000F08, 0)
#define CRC_ADD_1                       ((CRC_ADD_1_Type *) CRC_ADD_1_BASE)

/* CRC_ADD_24 bit positions */
#define CRC_ADD_24_CRC_ADD_24_BITS_Pos  0
#define CRC_ADD_24_CRC_ADD_24_BITS_Mask ((uint32_t)(0xFFFFFFU << CRC_ADD_24_CRC_ADD_24_BITS_Pos))

/* CRC_FINAL bit positions */
#define CRC_FINAL_FINAL_CRC_Pos         0
#define CRC_FINAL_FINAL_CRC_Mask        ((uint32_t)(0xFFFFFFFFU << CRC_FINAL_FINAL_CRC_Pos))

/* ----------------------------------------------------------------------------
 * Audio Sink Clock Counters
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL;                 /* Audio Sink Clock Control Register */
    __IO uint32_t CFG;                  /* Audio Sink Clock Configuration Register */
    __I  uint32_t CNT;                  /* Audio Sink Clock Counter Register */
    __IO uint32_t PHASE_CNT;            /* Audio Sink Clock Phase Counter Register */
    __IO uint32_t PERIOD_CNT;           /* Audio Sink Clock Period Counter Register */
} AUDIOSINK_Type;

#define AUDIOSINK_BASE                  0x40001000
#define AUDIOSINK                       ((AUDIOSINK_Type *) AUDIOSINK_BASE)

/* AUDIOSINK_CTRL bit positions */
#define AUDIOSINK_CTRL_PHASE_CNT_START_NO_WAIT_Pos 8
#define AUDIOSINK_CTRL_PERIOD_CNT_STATUS_Pos 7
#define AUDIOSINK_CTRL_PERIOD_CNT_STOP_Pos 6
#define AUDIOSINK_CTRL_PERIOD_CNT_START_Pos 5
#define AUDIOSINK_CTRL_PHASE_CNT_MISSED_STATUS_Pos 4
#define AUDIOSINK_CTRL_PHASE_CNT_STATUS_Pos 3
#define AUDIOSINK_CTRL_PHASE_CNT_STOP_Pos 2
#define AUDIOSINK_CTRL_PHASE_CNT_START_Pos 1
#define AUDIOSINK_CTRL_CNT_RESET_Pos    0

/* AUDIOSINK_CTRL sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t PHASE_CNT_START_NO_WAIT_BYTE;
         uint8_t RESERVED1[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001000, 0) - (0x40001000 + 4)];
    __O  uint32_t CNT_RESET_ALIAS;      /* Reset audio sink clock counter */
    __O  uint32_t PHASE_CNT_START_ALIAS;/* Start the audio sink clock PHASE counter mechanism and wait for sync pulse */
    __O  uint32_t PHASE_CNT_STOP_ALIAS; /* Stop the audio sink clock phase counter mechanism */
    __I  uint32_t PHASE_CNT_STATUS_ALIAS;/* Audio sink clock phase counter status */
    __I  uint32_t PHASE_CNT_MISSED_STATUS_ALIAS;/* Audio sink clock phase counter missed status */
    __O  uint32_t PERIOD_CNT_START_ALIAS;/* Start the audio sink clock period counter mechanism */
    __O  uint32_t PERIOD_CNT_STOP_ALIAS;/* Stop the audio sink clock period counter mechanism */
    __I  uint32_t PERIOD_CNT_STATUS_ALIAS;/* Audio sink clock period counter status */
    __O  uint32_t PHASE_CNT_START_NO_WAIT_ALIAS;/* Start the audio sink clock phase counter mechanism without waiting on a sync pulse */
} AUDIOSINK_CTRL_Type;

#define AUDIOSINK_CTRL_BASE             0x40001000
#define AUDIOSINK_CTRL                  ((AUDIOSINK_CTRL_Type *) AUDIOSINK_CTRL_BASE)

/* AUDIOSINK_CTRL settings */
#define CNT_RESET_BITBAND               0x1
#define CNT_RESET                       ((uint32_t)(CNT_RESET_BITBAND << AUDIOSINK_CTRL_CNT_RESET_Pos))

#define PHASE_CNT_START_BITBAND         0x1
#define PHASE_CNT_START                 ((uint32_t)(PHASE_CNT_START_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_START_Pos))

#define PHASE_CNT_STOP_BITBAND          0x1
#define PHASE_CNT_STOP                  ((uint32_t)(PHASE_CNT_STOP_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_STOP_Pos))

#define PHASE_CNT_IDLE_BITBAND          0x0
#define PHASE_CNT_BUSY_BITBAND          0x1
#define PHASE_CNT_IDLE                  ((uint32_t)(PHASE_CNT_IDLE_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_STATUS_Pos))
#define PHASE_CNT_BUSY                  ((uint32_t)(PHASE_CNT_BUSY_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_STATUS_Pos))

#define PHASE_CNT_NORMAL_BITBAND        0x0
#define PHASE_CNT_MISSED_BITBAND        0x1
#define PHASE_CNT_NORMAL                ((uint32_t)(PHASE_CNT_NORMAL_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_MISSED_STATUS_Pos))
#define PHASE_CNT_MISSED                ((uint32_t)(PHASE_CNT_MISSED_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_MISSED_STATUS_Pos))

#define PERIOD_CNT_START_BITBAND        0x1
#define PERIOD_CNT_START                ((uint32_t)(PERIOD_CNT_START_BITBAND << AUDIOSINK_CTRL_PERIOD_CNT_START_Pos))

#define PERIOD_CNT_STOP_BITBAND         0x1
#define PERIOD_CNT_STOP                 ((uint32_t)(PERIOD_CNT_STOP_BITBAND << AUDIOSINK_CTRL_PERIOD_CNT_STOP_Pos))

#define PERIOD_CNT_IDLE_BITBAND         0x0
#define PERIOD_CNT_BUSY_BITBAND         0x1
#define PERIOD_CNT_IDLE                 ((uint32_t)(PERIOD_CNT_IDLE_BITBAND << AUDIOSINK_CTRL_PERIOD_CNT_STATUS_Pos))
#define PERIOD_CNT_BUSY                 ((uint32_t)(PERIOD_CNT_BUSY_BITBAND << AUDIOSINK_CTRL_PERIOD_CNT_STATUS_Pos))

#define PHASE_CNT_START_NO_WAIT_BITBAND 0x1
#define PHASE_CNT_START_NO_WAIT         ((uint32_t)(PHASE_CNT_START_NO_WAIT_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_START_NO_WAIT_Pos))

/* AUDIOSINK_CTRL sub-register bit positions */
#define AUDIOSINK_CTRL_PHASE_CNT_START_NO_WAIT_BYTE_Pos 0

/* AUDIOSINK_CTRL sub-register settings */
#define PHASE_CNT_START_NO_WAIT_BYTE    ((uint8_t)(PHASE_CNT_START_NO_WAIT_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_START_NO_WAIT_BYTE_Pos))

/* AUDIOSINK_CFG bit positions */
#define AUDIOSINK_CFG_PERIODS_CFG_Pos   0
#define AUDIOSINK_CFG_PERIODS_CFG_Mask  ((uint32_t)(0xFU << AUDIOSINK_CFG_PERIODS_CFG_Pos))

/* AUDIOSINK_CFG settings */
#define AUDIO_SINK_PERIODS_1            ((uint32_t)(0x0U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_2            ((uint32_t)(0x1U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_3            ((uint32_t)(0x2U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_4            ((uint32_t)(0x3U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_5            ((uint32_t)(0x4U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_6            ((uint32_t)(0x5U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_7            ((uint32_t)(0x6U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_8            ((uint32_t)(0x7U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_9            ((uint32_t)(0x8U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_10           ((uint32_t)(0x9U << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_11           ((uint32_t)(0xAU << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_12           ((uint32_t)(0xBU << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_13           ((uint32_t)(0xCU << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_14           ((uint32_t)(0xDU << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_15           ((uint32_t)(0xEU << AUDIOSINK_CFG_PERIODS_CFG_Pos))
#define AUDIO_SINK_PERIODS_16           ((uint32_t)(0xFU << AUDIOSINK_CFG_PERIODS_CFG_Pos))

/* AUDIOSINK_CNT bit positions */
#define AUDIOSINK_CNT_CNT_Pos           0
#define AUDIOSINK_CNT_CNT_Mask          ((uint32_t)(0xFFFU << AUDIOSINK_CNT_CNT_Pos))

/* AUDIOSINK_PHASE_CNT bit positions */
#define AUDIOSINK_PHASE_CNT_PHASE_CNT_Pos 0
#define AUDIOSINK_PHASE_CNT_PHASE_CNT_Mask ((uint32_t)(0xFFFFU << AUDIOSINK_PHASE_CNT_PHASE_CNT_Pos))

/* AUDIOSINK_PERIOD_CNT bit positions */
#define AUDIOSINK_PERIOD_CNT_PERIOD_CNT_Pos 0
#define AUDIOSINK_PERIOD_CNT_PERIOD_CNT_Mask ((uint32_t)(0xFFFFU << AUDIOSINK_PERIOD_CNT_PERIOD_CNT_Pos))

/* ----------------------------------------------------------------------------
 * ASRC Configuration and Control
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL;                 /* ASRC Control Register */
    __IO uint32_t INT_ENABLE;           /* ASRC Interrupt Mask Register */
    __IO uint32_t OUT;                  /* ASRC Output Data Register */
    __IO uint32_t IN;                   /* ASRC Input Data Register */
    __IO uint32_t CFG;                  /* ASRC Configuration Register */
    __IO uint32_t OUTPUT_CNT;           /* ASRC output sample counter */
    __IO uint32_t PHASE_INC;            /* ASRC phase counter increment value */
    __IO uint32_t PHASE_CNT;            /* ASRC phase counter */
    __IO uint32_t STATE_MEM[30];        /* ASRC State Memory 0 to 29 (32 bit) */
} ASRC_Type;

#define ASRC_BASE                       0x40001100
#define ASRC                            ((ASRC_Type *) ASRC_BASE)

/* ASRC_CTRL bit positions */
#define ASRC_CTRL_ASRC_PROC_STATUS_Pos  14
#define ASRC_CTRL_ASRC_OUT_REQ_Pos      13
#define ASRC_CTRL_ASRC_IN_REQ_Pos       12
#define ASRC_CTRL_ASRC_UPDATE_ERR_Pos   11
#define ASRC_CTRL_ASRC_IN_ERR_Pos       10
#define ASRC_CTRL_ASRC_UPDATE_ERR_CLR_Pos 9
#define ASRC_CTRL_ASRC_IN_ERR_CLR_Pos   8
#define ASRC_CTRL_ASRC_RESET_Pos        3
#define ASRC_CTRL_ASRC_EN_STATUS_Pos    2
#define ASRC_CTRL_ASRC_DISABLE_Pos      1
#define ASRC_CTRL_ASRC_ENABLE_Pos       0

/* ASRC_CTRL bit-band aliases */
typedef struct
{
    __O  uint32_t ASRC_ENABLE_ALIAS;    /* Enable the re-sampler block */
    __O  uint32_t ASRC_DISABLE_ALIAS;   /* Disable the re-sampler block */
    __I  uint32_t ASRC_EN_STATUS_ALIAS; /* Enable status of the re-sampler block */
    __O  uint32_t ASRC_RESET_ALIAS;     /* Write a 1 to reset ASRC */
         uint32_t RESERVED0[4];
    __O  uint32_t ASRC_IN_ERR_CLR_ALIAS;/* Clear the ASRC input interface error interrupt */
    __O  uint32_t ASRC_UPDATE_ERR_CLR_ALIAS;/* Clear the ASRC update/configuration error interrupt status */
    __I  uint32_t ASRC_IN_ERR_ALIAS;    /* The ASRC input interface error interrupt status */
    __I  uint32_t ASRC_UPDATE_ERR_ALIAS;/* The ASRC state/configuration update error interrupt status */
    __I  uint32_t ASRC_IN_REQ_ALIAS;    /* The ASRC_IN register status */
    __I  uint32_t ASRC_OUT_REQ_ALIAS;   /* The ASRC_OUT register status */
    __I  uint32_t ASRC_PROC_STATUS_ALIAS;/* The ASRC processing state */
} ASRC_CTRL_Type;

#define ASRC_CTRL_BASE                  SYS_CALC_BITBAND(0x40001100, 0)
#define ASRC_CTRL                       ((ASRC_CTRL_Type *) ASRC_CTRL_BASE)

/* ASRC_CTRL settings */
#define ASRC_ENABLE_BITBAND             0x1
#define ASRC_ENABLE                     ((uint32_t)(ASRC_ENABLE_BITBAND << ASRC_CTRL_ASRC_ENABLE_Pos))

#define ASRC_DISABLE_BITBAND            0x1
#define ASRC_DISABLE                    ((uint32_t)(ASRC_DISABLE_BITBAND << ASRC_CTRL_ASRC_DISABLE_Pos))

#define ASRC_DISABLED_BITBAND           0x0
#define ASRC_ENABLED_BITBAND            0x1
#define ASRC_DISABLED                   ((uint32_t)(ASRC_DISABLED_BITBAND << ASRC_CTRL_ASRC_EN_STATUS_Pos))
#define ASRC_ENABLED                    ((uint32_t)(ASRC_ENABLED_BITBAND << ASRC_CTRL_ASRC_EN_STATUS_Pos))

#define ASRC_RESET_BITBAND              0x1
#define ASRC_RESET                      ((uint32_t)(ASRC_RESET_BITBAND << ASRC_CTRL_ASRC_RESET_Pos))

#define CLR_ASRC_IN_ERR_BITBAND         0x1
#define CLR_ASRC_IN_ERR                 ((uint32_t)(CLR_ASRC_IN_ERR_BITBAND << ASRC_CTRL_ASRC_IN_ERR_CLR_Pos))

#define CLR_ASRC_UPDATE_ERR_BITBAND     0x1
#define CLR_ASRC_UPDATE_ERR             ((uint32_t)(CLR_ASRC_UPDATE_ERR_BITBAND << ASRC_CTRL_ASRC_UPDATE_ERR_CLR_Pos))

#define INT_IDLE_ASRC_IN_ERR_BITBAND    0x0
#define INT_PENDING_ASRC_IN_ERR_BITBAND 0x1
#define INT_IDLE_ASRC_IN_ERR            ((uint32_t)(INT_IDLE_ASRC_IN_ERR_BITBAND << ASRC_CTRL_ASRC_IN_ERR_Pos))
#define INT_PENDING_ASRC_IN_ERR         ((uint32_t)(INT_PENDING_ASRC_IN_ERR_BITBAND << ASRC_CTRL_ASRC_IN_ERR_Pos))

#define INT_IDLE_ASRC_PH_CNT_ERR_BITBAND 0x0
#define INT_PENDING_ASRC_PH_CNT_ERR_BITBAND 0x1
#define INT_IDLE_ASRC_PH_CNT_ERR        ((uint32_t)(INT_IDLE_ASRC_PH_CNT_ERR_BITBAND << ASRC_CTRL_ASRC_UPDATE_ERR_Pos))
#define INT_PENDING_ASRC_PH_CNT_ERR     ((uint32_t)(INT_PENDING_ASRC_PH_CNT_ERR_BITBAND << ASRC_CTRL_ASRC_UPDATE_ERR_Pos))

#define INT_IDLE_ASRC_IN_BITBAND        0x0
#define INT_RDY_ASRC_IN_BITBAND         0x1
#define INT_IDLE_ASRC_IN                ((uint32_t)(INT_IDLE_ASRC_IN_BITBAND << ASRC_CTRL_ASRC_IN_REQ_Pos))
#define INT_RDY_ASRC_IN                 ((uint32_t)(INT_RDY_ASRC_IN_BITBAND << ASRC_CTRL_ASRC_IN_REQ_Pos))

#define INT_IDLE_ASRC_OUT_BITBAND       0x0
#define INT_RDY_ASRC_OUT_BITBAND        0x1
#define INT_IDLE_ASRC_OUT               ((uint32_t)(INT_IDLE_ASRC_OUT_BITBAND << ASRC_CTRL_ASRC_OUT_REQ_Pos))
#define INT_RDY_ASRC_OUT                ((uint32_t)(INT_RDY_ASRC_OUT_BITBAND << ASRC_CTRL_ASRC_OUT_REQ_Pos))

#define ASRC_IDLE_BITBAND               0x0
#define ASRC_BUSY_BITBAND               0x1
#define ASRC_IDLE                       ((uint32_t)(ASRC_IDLE_BITBAND << ASRC_CTRL_ASRC_PROC_STATUS_Pos))
#define ASRC_BUSY                       ((uint32_t)(ASRC_BUSY_BITBAND << ASRC_CTRL_ASRC_PROC_STATUS_Pos))

/* ASRC_INT_ENABLE bit positions */
#define ASRC_INT_ENABLE_ASRC_UPDATE_ERR_Pos 3
#define ASRC_INT_ENABLE_ASRC_IN_ERR_Pos 2
#define ASRC_INT_ENABLE_ASRC_OUT_REQ_Pos 1
#define ASRC_INT_ENABLE_ASRC_IN_REQ_Pos 0

/* ASRC_INT_ENABLE bit-band aliases */
typedef struct
{
    __IO uint32_t ASRC_IN_REQ_ALIAS;    /* The ASRC_IN register interrupt status */
    __IO uint32_t ASRC_OUT_REQ_ALIAS;   /* The ASRC_OUT register interrupt status */
    __IO uint32_t ASRC_IN_ERR_ALIAS;    /* The ASRC input interface error interrupt mask */
    __IO uint32_t ASRC_UPDATE_ERR_ALIAS;/* The ASRC state/configuration update error interrupt mask */
} ASRC_INT_ENABLE_Type;

#define ASRC_INT_ENABLE_BASE            SYS_CALC_BITBAND(0x40001104, 0)
#define ASRC_INT_ENABLE                 ((ASRC_INT_ENABLE_Type *) ASRC_INT_ENABLE_BASE)

/* ASRC_INT_ENABLE settings */
#define INT_DIS_ASRC_IN_BITBAND         0x0
#define INT_EBL_ASRC_IN_BITBAND         0x1
#define INT_DIS_ASRC_IN                 ((uint32_t)(INT_DIS_ASRC_IN_BITBAND << ASRC_INT_ENABLE_ASRC_IN_REQ_Pos))
#define INT_EBL_ASRC_IN                 ((uint32_t)(INT_EBL_ASRC_IN_BITBAND << ASRC_INT_ENABLE_ASRC_IN_REQ_Pos))

#define INT_DIS_ASRC_OUT_BITBAND        0x0
#define INT_EBL_ASRC_OUT_BITBAND        0x1
#define INT_DIS_ASRC_OUT                ((uint32_t)(INT_DIS_ASRC_OUT_BITBAND << ASRC_INT_ENABLE_ASRC_OUT_REQ_Pos))
#define INT_EBL_ASRC_OUT                ((uint32_t)(INT_EBL_ASRC_OUT_BITBAND << ASRC_INT_ENABLE_ASRC_OUT_REQ_Pos))

#define INT_DIS_ASRC_IN_ERR_BITBAND     0x0
#define INT_EBL_ASRC_IN_ERR_BITBAND     0x1
#define INT_DIS_ASRC_IN_ERR             ((uint32_t)(INT_DIS_ASRC_IN_ERR_BITBAND << ASRC_INT_ENABLE_ASRC_IN_ERR_Pos))
#define INT_EBL_ASRC_IN_ERR             ((uint32_t)(INT_EBL_ASRC_IN_ERR_BITBAND << ASRC_INT_ENABLE_ASRC_IN_ERR_Pos))

#define INT_DIS_ASRC_UPDATE_ERR_BITBAND 0x0
#define INT_EBL_ASRC_UPDATE_ERR_BITBAND 0x1
#define INT_DIS_ASRC_UPDATE_ERR         ((uint32_t)(INT_DIS_ASRC_UPDATE_ERR_BITBAND << ASRC_INT_ENABLE_ASRC_UPDATE_ERR_Pos))
#define INT_EBL_ASRC_UPDATE_ERR         ((uint32_t)(INT_EBL_ASRC_UPDATE_ERR_BITBAND << ASRC_INT_ENABLE_ASRC_UPDATE_ERR_Pos))

/* ASRC_CFG bit positions */
#define ASRC_CFG_WDF_TYPE_Pos           2
#define ASRC_CFG_ASRC_MODE_Pos          0
#define ASRC_CFG_ASRC_MODE_Mask         ((uint32_t)(0x3U << ASRC_CFG_ASRC_MODE_Pos))

/* ASRC_CFG bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[2];
    __IO uint32_t WDF_TYPE_ALIAS;       /* WDF Type Selection */
} ASRC_CFG_Type;

#define ASRC_CFG_BASE                   SYS_CALC_BITBAND(0x40001110, 0)
#define ASRC_CFG                        ((ASRC_CFG_Type *) ASRC_CFG_BASE)

/* ASRC_CFG settings */
#define ASRC_INT_MODE                   ((uint32_t)(0x0U << ASRC_CFG_ASRC_MODE_Pos))
#define ASRC_DEC_MODE1                  ((uint32_t)(0x1U << ASRC_CFG_ASRC_MODE_Pos))
#define ASRC_DEC_MODE2                  ((uint32_t)(0x2U << ASRC_CFG_ASRC_MODE_Pos))
#define ASRC_DEC_MODE3                  ((uint32_t)(0x3U << ASRC_CFG_ASRC_MODE_Pos))

#define LOW_DELAY_BITBAND               0x0
#define WIDE_BAND_BITBAND               0x1
#define LOW_DELAY                       ((uint32_t)(LOW_DELAY_BITBAND << ASRC_CFG_WDF_TYPE_Pos))
#define WIDE_BAND                       ((uint32_t)(WIDE_BAND_BITBAND << ASRC_CFG_WDF_TYPE_Pos))

/* ASRC_PHASE_INC bit positions */
#define ASRC_PHASE_INC_ASRC_STEP_Pos    0
#define ASRC_PHASE_INC_ASRC_STEP_Mask   ((uint32_t)(0xFFFFFFFFU << ASRC_PHASE_INC_ASRC_STEP_Pos))

/* ----------------------------------------------------------------------------
 * Analog-to-Digital Converter and Battery Monitoring
 * ------------------------------------------------------------------------- */

typedef struct
{
    __I  uint32_t DATA_TRIM_CH[8];      /* ADC conversion result for channel 0 to 7 in trimmer mode */
    __I  uint32_t DATA_AUDIO_CH[8];     /* ADC conversion result for channel 0 to 7 in audio mode (signed) */
    __IO uint32_t INPUT_SEL[8];         /* ADC input selection for channel 0 to 7 */
    __IO uint32_t CFG;                  /* ADC Configuration Register */
    __IO uint32_t OFFSET;               /* ADC conversion result for ADC GND */
         uint32_t RESERVED0[2];
    __IO uint32_t BATMON_CFG;           /* Battery Monitoring Configuration Register */
    __IO uint32_t BATMON_INT_ENABLE;    /* ADC / BATMON Interrupt Mask Register */
    __I  uint32_t BATMON_COUNT_VAL;     /* Battery Monitoring Status Register */
    __IO uint32_t BATMON_STATUS;        /* ADC / BATMON Status Register */
} ADC_Type;

#define ADC_BASE                        0x40001200
#define ADC                             ((ADC_Type *) ADC_BASE)

/* ADC_DATA_TRIM_CH[] bit positions */
#define ADC_DATA_TRIM_CH_DATA_Pos       0
#define ADC_DATA_TRIM_CH_DATA_Mask      ((uint32_t)(0x3FFFU << ADC_DATA_TRIM_CH_DATA_Pos))

/* ADC_DATA_AUDIO_CH[] bit positions */
#define ADC_DATA_AUDIO_CH_DATA_Pos      0
#define ADC_DATA_AUDIO_CH_DATA_Mask     ((uint32_t)(0xFFFFFFFFU << ADC_DATA_AUDIO_CH_DATA_Pos))

/* ADC_INPUT_SEL[] bit positions */
#define ADC_INPUT_SEL_POS_INPUT_SEL_Pos 4
#define ADC_INPUT_SEL_POS_INPUT_SEL_Mask ((uint32_t)(0x7U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))
#define ADC_INPUT_SEL_NEG_INPUT_SEL_Pos 0
#define ADC_INPUT_SEL_NEG_INPUT_SEL_Mask ((uint32_t)(0x7U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))

/* ADC_INPUT_SEL settings */
#define ADC_NEG_INPUT_DIO0              ((uint32_t)(0x0U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))
#define ADC_NEG_INPUT_DIO1              ((uint32_t)(0x1U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))
#define ADC_NEG_INPUT_DIO2              ((uint32_t)(0x2U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))
#define ADC_NEG_INPUT_DIO3              ((uint32_t)(0x3U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))
#define ADC_NEG_INPUT_AOUT              ((uint32_t)(0x4U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))
#define ADC_NEG_INPUT_VDDC              ((uint32_t)(0x5U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))
#define ADC_NEG_INPUT_VBAT_DIV2         ((uint32_t)(0x6U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))
#define ADC_NEG_INPUT_GND               ((uint32_t)(0x7U << ADC_INPUT_SEL_NEG_INPUT_SEL_Pos))

#define ADC_POS_INPUT_DIO0              ((uint32_t)(0x0U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))
#define ADC_POS_INPUT_DIO1              ((uint32_t)(0x1U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))
#define ADC_POS_INPUT_DIO2              ((uint32_t)(0x2U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))
#define ADC_POS_INPUT_DIO3              ((uint32_t)(0x3U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))
#define ADC_POS_INPUT_AOUT              ((uint32_t)(0x4U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))
#define ADC_POS_INPUT_VDDC              ((uint32_t)(0x5U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))
#define ADC_POS_INPUT_VBAT_DIV2         ((uint32_t)(0x6U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))
#define ADC_POS_INPUT_GND               ((uint32_t)(0x7U << ADC_INPUT_SEL_POS_INPUT_SEL_Pos))

/* ADC_CFG bit positions */
#define ADC_CFG_DUTY_DIVIDER_Pos        5
#define ADC_CFG_CONTINUOUS_MODE_Pos     4
#define ADC_CFG_FREQ_Pos                0
#define ADC_CFG_FREQ_Mask               ((uint32_t)(0xFU << ADC_CFG_FREQ_Pos))

/* ADC_CFG bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[4];
    __IO uint32_t CONTINUOUS_MODE_ALIAS;/* ADC continuously sampling the channel selected as interrupt source (ADC_INT_CH_NUM) */
    __IO uint32_t DUTY_DIVIDER_ALIAS;   /* Duty cycling VDD divider */
} ADC_CFG_Type;

#define ADC_CFG_BASE                    SYS_CALC_BITBAND(0x40001260, 0)
#define ADC_CFG                         ((ADC_CFG_Type *) ADC_CFG_BASE)

/* ADC_CFG settings */
#define ADC_DISABLE                     ((uint32_t)(0x0U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_200                ((uint32_t)(0x1U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_400                ((uint32_t)(0x2U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_640                ((uint32_t)(0x3U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_800                ((uint32_t)(0x4U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_1600               ((uint32_t)(0x5U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_3200               ((uint32_t)(0x6U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_6400               ((uint32_t)(0x7U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_20H                ((uint32_t)(0x8U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_40H                ((uint32_t)(0x9U << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_80H                ((uint32_t)(0xAU << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_128H               ((uint32_t)(0xBU << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_160H               ((uint32_t)(0xCU << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_320H               ((uint32_t)(0xDU << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_640H               ((uint32_t)(0xEU << ADC_CFG_FREQ_Pos))
#define ADC_PRESCALE_1280H              ((uint32_t)(0xFU << ADC_CFG_FREQ_Pos))

#define ADC_NORMAL_BITBAND              0x0
#define ADC_CONTINUOUS_BITBAND          0x1
#define ADC_NORMAL                      ((uint32_t)(ADC_NORMAL_BITBAND << ADC_CFG_CONTINUOUS_MODE_Pos))
#define ADC_CONTINUOUS                  ((uint32_t)(ADC_CONTINUOUS_BITBAND << ADC_CFG_CONTINUOUS_MODE_Pos))

#define ADC_VBAT_DIV2_NORMAL_BITBAND    0x0
#define ADC_VBAT_DIV2_DUTY_BITBAND      0x1
#define ADC_VBAT_DIV2_NORMAL            ((uint32_t)(ADC_VBAT_DIV2_NORMAL_BITBAND << ADC_CFG_DUTY_DIVIDER_Pos))
#define ADC_VBAT_DIV2_DUTY              ((uint32_t)(ADC_VBAT_DIV2_DUTY_BITBAND << ADC_CFG_DUTY_DIVIDER_Pos))

/* ADC_OFFSET bit positions */
#define ADC_OFFSET_DATA_Pos             0
#define ADC_OFFSET_DATA_Mask            ((uint32_t)(0x7FFFU << ADC_OFFSET_DATA_Pos))

/* ADC_BATMON_CFG bit positions */
#define ADC_BATMON_CFG_ALARM_COUNT_VALUE_Pos 16
#define ADC_BATMON_CFG_ALARM_COUNT_VALUE_Mask ((uint32_t)(0xFFU << ADC_BATMON_CFG_ALARM_COUNT_VALUE_Pos))
#define ADC_BATMON_CFG_SUPPLY_THRESHOLD_Pos 8
#define ADC_BATMON_CFG_SUPPLY_THRESHOLD_Mask ((uint32_t)(0xFFU << ADC_BATMON_CFG_SUPPLY_THRESHOLD_Pos))
#define ADC_BATMON_CFG_SUPPLY_SRC_Pos   0

/* ADC_BATMON_CFG sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t SUPPLY_THRESHOLD_BYTE;
    __IO uint8_t ALARM_COUNT_VALUE_BYTE;
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001270, 0) - (0x40001270 + 4)];
    __IO uint32_t SUPPLY_SRC_ALIAS;     /* Selects the power supply voltage source to be monitored */
} ADC_BATMON_CFG_Type;

#define ADC_BATMON_CFG_BASE             0x40001270
#define ADC_BATMON_CFG                  ((ADC_BATMON_CFG_Type *) ADC_BATMON_CFG_BASE)

/* ADC_BATMON_CFG settings */
#define BATMON_CH6_BITBAND              0x0
#define BATMON_CH7_BITBAND              0x1
#define BATMON_CH6                      ((uint32_t)(BATMON_CH6_BITBAND << ADC_BATMON_CFG_SUPPLY_SRC_Pos))
#define BATMON_CH7                      ((uint32_t)(BATMON_CH7_BITBAND << ADC_BATMON_CFG_SUPPLY_SRC_Pos))

#define SUPPLY_THRESHOLD_LOW            ((uint32_t)(0x0U << ADC_BATMON_CFG_SUPPLY_THRESHOLD_Pos))
#define SUPPLY_THRESHOLD_MID            ((uint32_t)(0x80U << ADC_BATMON_CFG_SUPPLY_THRESHOLD_Pos))
#define SUPPLY_THRESHOLD_HIGH           ((uint32_t)(0xFFU << ADC_BATMON_CFG_SUPPLY_THRESHOLD_Pos))

#define BATMON_ALARM_NONE               ((uint32_t)(0x0U << ADC_BATMON_CFG_ALARM_COUNT_VALUE_Pos))
#define BATMON_ALARM_COUNT1             ((uint32_t)(0x1U << ADC_BATMON_CFG_ALARM_COUNT_VALUE_Pos))
#define BATMON_ALARM_COUNT255           ((uint32_t)(0xFFU << ADC_BATMON_CFG_ALARM_COUNT_VALUE_Pos))

/* ADC_BATMON_CFG sub-register settings */
#define SUPPLY_THRESHOLD_LOW_BYTE       ((uint8_t)0x0U)
#define SUPPLY_THRESHOLD_MID_BYTE       ((uint8_t)0x80U)
#define SUPPLY_THRESHOLD_HIGH_BYTE      ((uint8_t)0xFFU)

#define BATMON_ALARM_NONE_BYTE          ((uint8_t)0x0U)
#define BATMON_ALARM_COUNT1_BYTE        ((uint8_t)0x1U)
#define BATMON_ALARM_COUNT255_BYTE      ((uint8_t)0xFFU)

/* ADC_BATMON_INT_ENABLE bit positions */
#define ADC_BATMON_INT_ENABLE_BATMON_ALARM_INT_ENABLE_Pos 4
#define ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos 1
#define ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Mask ((uint32_t)(0x7U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))
#define ADC_BATMON_INT_ENABLE_ADC_INT_ENABLE_Pos 0

/* ADC_BATMON_INT_ENABLE bit-band aliases */
typedef struct
{
    __IO uint32_t ADC_INT_ENABLE_ALIAS; /* The ADC new sample ready interrupt mask */
         uint32_t RESERVED0[3];
    __IO uint32_t BATMON_ALARM_INT_ENABLE_ALIAS;/* The BATMON Alarm interrupt mask */
} ADC_BATMON_INT_ENABLE_Type;

#define ADC_BATMON_INT_ENABLE_BASE      SYS_CALC_BITBAND(0x40001274, 0)
#define ADC_BATMON_INT_ENABLE           ((ADC_BATMON_INT_ENABLE_Type *) ADC_BATMON_INT_ENABLE_BASE)

/* ADC_BATMON_INT_ENABLE settings */
#define INT_DIS_ADC_BITBAND             0x0
#define INT_EBL_ADC_BITBAND             0x1
#define INT_DIS_ADC                     ((uint32_t)(INT_DIS_ADC_BITBAND << ADC_BATMON_INT_ENABLE_ADC_INT_ENABLE_Pos))
#define INT_EBL_ADC                     ((uint32_t)(INT_EBL_ADC_BITBAND << ADC_BATMON_INT_ENABLE_ADC_INT_ENABLE_Pos))

#define ADC_INT_CH0                     ((uint32_t)(0x0U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))
#define ADC_INT_CH1                     ((uint32_t)(0x1U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))
#define ADC_INT_CH2                     ((uint32_t)(0x2U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))
#define ADC_INT_CH3                     ((uint32_t)(0x3U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))
#define ADC_INT_CH4                     ((uint32_t)(0x4U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))
#define ADC_INT_CH5                     ((uint32_t)(0x5U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))
#define ADC_INT_CH6                     ((uint32_t)(0x6U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))
#define ADC_INT_CH7                     ((uint32_t)(0x7U << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_Pos))

#define INT_DIS_BATMON_ALARM_BITBAND    0x0
#define INT_EBL_BATMON_ALARM_BITBAND    0x1
#define INT_DIS_BATMON_ALARM            ((uint32_t)(INT_DIS_BATMON_ALARM_BITBAND << ADC_BATMON_INT_ENABLE_BATMON_ALARM_INT_ENABLE_Pos))
#define INT_EBL_BATMON_ALARM            ((uint32_t)(INT_EBL_BATMON_ALARM_BITBAND << ADC_BATMON_INT_ENABLE_BATMON_ALARM_INT_ENABLE_Pos))

/* ADC_BATMON_COUNT_VAL bit positions */
#define ADC_BATMON_COUNT_VAL_SUPPLY_COUNT_VALUE_Pos 0
#define ADC_BATMON_COUNT_VAL_SUPPLY_COUNT_VALUE_Mask ((uint32_t)(0xFFU << ADC_BATMON_COUNT_VAL_SUPPLY_COUNT_VALUE_Pos))

/* ADC_BATMON_STATUS bit positions */
#define ADC_BATMON_STATUS_BATMON_ALARM_CLEAR_Pos 12
#define ADC_BATMON_STATUS_ADC_OVERRUN_CLEAR_Pos 9
#define ADC_BATMON_STATUS_ADC_READY_CLEAR_Pos 8
#define ADC_BATMON_STATUS_BATMON_ALARM_STAT_Pos 4
#define ADC_BATMON_STATUS_ADC_OVERRUN_STAT_Pos 1
#define ADC_BATMON_STATUS_ADC_READY_STAT_Pos 0

/* ADC_BATMON_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t ADC_READY_STAT_ALIAS; /* ADC new sample Ready status bit */
    __I  uint32_t ADC_OVERRUN_STAT_ALIAS;/* ADC Overrun condition */
         uint32_t RESERVED0[2];
    __I  uint32_t BATMON_ALARM_STAT_ALIAS;/* Battery monitoring alarm status bit */
         uint32_t RESERVED1[3];
    __O  uint32_t ADC_READY_CLEAR_ALIAS;/* ADC new sample Ready status bit */
    __O  uint32_t ADC_OVERRUN_CLEAR_ALIAS;/* ADC Overrun condition */
         uint32_t RESERVED2[2];
    __O  uint32_t BATMON_ALARM_CLEAR_ALIAS;/* Battery monitoring alarm status bit */
} ADC_BATMON_STATUS_Type;

#define ADC_BATMON_STATUS_BASE          SYS_CALC_BITBAND(0x4000127C, 0)
#define ADC_BATMON_STATUS               ((ADC_BATMON_STATUS_Type *) ADC_BATMON_STATUS_BASE)

/* ADC_BATMON_STATUS settings */
#define ADC_READY_FALSE_BITBAND         0x0
#define ADC_READY_TRUE_BITBAND          0x1
#define ADC_READY_FALSE                 ((uint32_t)(ADC_READY_FALSE_BITBAND << ADC_BATMON_STATUS_ADC_READY_STAT_Pos))
#define ADC_READY_TRUE                  ((uint32_t)(ADC_READY_TRUE_BITBAND << ADC_BATMON_STATUS_ADC_READY_STAT_Pos))

#define ADC_OVERRUN_FALSE_BITBAND       0x0
#define ADC_OVERRUN_TRUE_BITBAND        0x1
#define ADC_OVERRUN_FALSE               ((uint32_t)(ADC_OVERRUN_FALSE_BITBAND << ADC_BATMON_STATUS_ADC_OVERRUN_STAT_Pos))
#define ADC_OVERRUN_TRUE                ((uint32_t)(ADC_OVERRUN_TRUE_BITBAND << ADC_BATMON_STATUS_ADC_OVERRUN_STAT_Pos))

#define BATMON_ALARM_FALSE_BITBAND      0x0
#define BATMON_ALARM_TRUE_BITBAND       0x1
#define BATMON_ALARM_FALSE              ((uint32_t)(BATMON_ALARM_FALSE_BITBAND << ADC_BATMON_STATUS_BATMON_ALARM_STAT_Pos))
#define BATMON_ALARM_TRUE               ((uint32_t)(BATMON_ALARM_TRUE_BITBAND << ADC_BATMON_STATUS_BATMON_ALARM_STAT_Pos))

#define ADC_READY_CLEAR_BITBAND         0x1
#define ADC_READY_CLEAR                 ((uint32_t)(ADC_READY_CLEAR_BITBAND << ADC_BATMON_STATUS_ADC_READY_CLEAR_Pos))

#define ADC_OVERRUN_CLEAR_BITBAND       0x1
#define ADC_OVERRUN_CLEAR               ((uint32_t)(ADC_OVERRUN_CLEAR_BITBAND << ADC_BATMON_STATUS_ADC_OVERRUN_CLEAR_Pos))

#define BATMON_ALARM_CLEAR_BITBAND      0x1
#define BATMON_ALARM_CLEAR              ((uint32_t)(BATMON_ALARM_CLEAR_BITBAND << ADC_BATMON_STATUS_BATMON_ALARM_CLEAR_Pos))

/* ----------------------------------------------------------------------------
 * ACS domain (Analog Bridge Access)
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t BG_CTRL;              /* Bandgap Configuration / Control register */
    __IO uint32_t VCC_CTRL;             /* DC-DC / LDO Supply Configuration / Control register */
    __IO uint32_t VDDA_CP_CTRL;         /* Analog Voltage and Flash Charge Pump Configuration / Control register */
    __IO uint32_t VDDC_CTRL;            /* Digital Core Voltage Regulator Configuration / Control register */
    __IO uint32_t VDDM_CTRL;            /* Memories Voltage Regulator Configuration / Control register */
    __IO uint32_t VDDRF_CTRL;           /* RF Block Regulator Configuration / Control register */
    __IO uint32_t VDDPA_CTRL;           /* RF Block Regulator Configuration / Control register */
    __IO uint32_t VDDRET_CTRL;          /* Retention Regulator Configuration / Control register */
    __IO uint32_t RCOSC_CTRL;           /* RC Oscillator Configuration / Control register */
    __IO uint32_t XTAL32K_CTRL;         /* XTAL 32 kHz configuration register */
    __IO uint32_t BB_TIMER_CTRL;        /* Baseband timer and standby clock configuration register */
    __IO uint32_t CLK_DET_CTRL;         /* Clock Detector configuration register */
    __IO uint32_t RTC_CFG;              /* RTC Timer Counter Preload */
    __I  uint32_t RTC_COUNT;            /* RTC Timer Counter Current Value (only reset by pmu reset or by writting 1 at ACS_RTC_CTRL.RESET) */
    __IO uint32_t RTC_CTRL;             /* RTC Control Register */
         uint32_t RESERVED0;
    __O  uint32_t PWR_MODES_CTRL;       /* Power Modes Control Register */
    __IO uint32_t WAKEUP_CTRL;          /* Wake-Up Control / Status Register */
    __IO uint32_t WAKEUP_CFG;           /* Wakeup configuration */
    __I  uint32_t WAKEUP_STATE;         /* RTC Timer wakeup value and wakeup source */
    __IO uint32_t WAKEUP_GP_DATA;       /* Wake-Up Control Register / JIC RW registers */
    __IO uint32_t RESET_STATUS;         /* ACS reset source status registers */
    __IO uint32_t AOUT_CTRL;            /* Analog output configuration register */
    __I  uint32_t JIC_READ;             /* Just In Case register in the ACS block */
} ACS_Type;

#define ACS_BASE                        0x40001300
#define ACS                             ((ACS_Type *) ACS_BASE)

/* ACS_BG_CTRL bit positions */
#define ACS_BG_CTRL_SLOPE_TRIM_Pos      8
#define ACS_BG_CTRL_SLOPE_TRIM_Mask     ((uint32_t)(0x1FU << ACS_BG_CTRL_SLOPE_TRIM_Pos))
#define ACS_BG_CTRL_VTRIM_Pos           0
#define ACS_BG_CTRL_VTRIM_Mask          ((uint32_t)(0x3FU << ACS_BG_CTRL_VTRIM_Pos))

/* ACS_BG_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t VTRIM_BYTE;           
    __IO uint8_t SLOPE_TRIM_BYTE;      
         uint8_t RESERVED0[2];
} ACS_BG_CTRL_Type;

#define ACS_BG_CTRL_BASE                0x40001300
#define ACS_BG_CTRL                     ((ACS_BG_CTRL_Type *) ACS_BG_CTRL_BASE)

/* ACS_BG_CTRL settings */
#define BG_TRIM_0P675V                  ((uint32_t)(0x0U << ACS_BG_CTRL_VTRIM_Pos))
#define BG_TRIM_0P678V                  ((uint32_t)(0x1U << ACS_BG_CTRL_VTRIM_Pos))
#define BG_TRIM_0P748V                  ((uint32_t)(0x1DU << ACS_BG_CTRL_VTRIM_Pos))
#define BG_TRIM_0P750V                  ((uint32_t)(0x1EU << ACS_BG_CTRL_VTRIM_Pos))
#define BG_TRIM_0P753V                  ((uint32_t)(0x1FU << ACS_BG_CTRL_VTRIM_Pos))
#define BG_TRIM_0P830V                  ((uint32_t)(0x3EU << ACS_BG_CTRL_VTRIM_Pos))
#define BG_TRIM_0P833V                  ((uint32_t)(0x3FU << ACS_BG_CTRL_VTRIM_Pos))

#define BG_SLOPE_TRIM_VALUE             ((uint32_t)(0xBU << ACS_BG_CTRL_SLOPE_TRIM_Pos))

/* ACS_BG_CTRL sub-register bit positions */
#define ACS_BG_CTRL_VTRIM_BYTE_Pos      0
#define ACS_BG_CTRL_VTRIM_BYTE_Mask     ((uint32_t)(0x3FU << ACS_BG_CTRL_VTRIM_BYTE_Pos))
#define ACS_BG_CTRL_SLOPE_TRIM_BYTE_Pos 0
#define ACS_BG_CTRL_SLOPE_TRIM_BYTE_Mask ((uint32_t)(0x1FU << ACS_BG_CTRL_SLOPE_TRIM_BYTE_Pos))

/* ACS_BG_CTRL sub-register settings */
#define BG_TRIM_0P675V_BYTE             ((uint8_t)(0x0U << ACS_BG_CTRL_VTRIM_BYTE_Pos))
#define BG_TRIM_0P678V_BYTE             ((uint8_t)(0x1U << ACS_BG_CTRL_VTRIM_BYTE_Pos))
#define BG_TRIM_0P748V_BYTE             ((uint8_t)(0x1DU << ACS_BG_CTRL_VTRIM_BYTE_Pos))
#define BG_TRIM_0P750V_BYTE             ((uint8_t)(0x1EU << ACS_BG_CTRL_VTRIM_BYTE_Pos))
#define BG_TRIM_0P753V_BYTE             ((uint8_t)(0x1FU << ACS_BG_CTRL_VTRIM_BYTE_Pos))
#define BG_TRIM_0P830V_BYTE             ((uint8_t)(0x3EU << ACS_BG_CTRL_VTRIM_BYTE_Pos))
#define BG_TRIM_0P833V_BYTE             ((uint8_t)(0x3FU << ACS_BG_CTRL_VTRIM_BYTE_Pos))

#define BG_SLOPE_TRIM_VALUE_BYTE        ((uint8_t)(0xBU << ACS_BG_CTRL_SLOPE_TRIM_BYTE_Pos))

/* ACS_VCC_CTRL bit positions */
#define ACS_VCC_CTRL_ICH_TRIM_Pos       16
#define ACS_VCC_CTRL_ICH_TRIM_Mask      ((uint32_t)(0xFU << ACS_VCC_CTRL_ICH_TRIM_Pos))
#define ACS_VCC_CTRL_CCM_ENABLE_Pos     11
#define ACS_VCC_CTRL_PULSE_CTRL_Pos     10
#define ACS_VCC_CTRL_CHARGE_CTRL_Pos    9
#define ACS_VCC_CTRL_BUCK_ENABLE_Pos    8
#define ACS_VCC_CTRL_VTRIM_Pos          0
#define ACS_VCC_CTRL_VTRIM_Mask         ((uint32_t)(0x1FU << ACS_VCC_CTRL_VTRIM_Pos))

/* ACS_VCC_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t VTRIM_BYTE;           
         uint8_t RESERVED0[1];
    __IO uint8_t ICH_TRIM_BYTE;        
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001304, 0) - (0x40001304 + 4)];
         uint32_t RESERVED2[8];
    __IO uint32_t BUCK_ENABLE_ALIAS;    /* Enable buck converter mode */
    __IO uint32_t CHARGE_CTRL_ALIAS;    /* Charge mode control */
    __IO uint32_t PULSE_CTRL_ALIAS;     /* Pulse mode control */
    __IO uint32_t CCM_ENABLE_ALIAS;     /* Enable CCM mode */
} ACS_VCC_CTRL_Type;

#define ACS_VCC_CTRL_BASE               0x40001304
#define ACS_VCC_CTRL                    ((ACS_VCC_CTRL_Type *) ACS_VCC_CTRL_BASE)

/* ACS_VCC_CTRL settings */
#define VCC_TRIM_1P00V                  ((uint32_t)(0x0U << ACS_VCC_CTRL_VTRIM_Pos))
#define VCC_TRIM_1P05V                  ((uint32_t)(0x5U << ACS_VCC_CTRL_VTRIM_Pos))
#define VCC_TRIM_1P10V                  ((uint32_t)(0xAU << ACS_VCC_CTRL_VTRIM_Pos))
#define VCC_TRIM_1P15V                  ((uint32_t)(0xFU << ACS_VCC_CTRL_VTRIM_Pos))
#define VCC_TRIM_1P20V                  ((uint32_t)(0x14U << ACS_VCC_CTRL_VTRIM_Pos))
#define VCC_TRIM_1P25V                  ((uint32_t)(0x19U << ACS_VCC_CTRL_VTRIM_Pos))
#define VCC_TRIM_1P31V                  ((uint32_t)(0x1FU << ACS_VCC_CTRL_VTRIM_Pos))

#define VCC_LDO_BITBAND                 0x0
#define VCC_BUCK_BITBAND                0x1
#define VCC_LDO                         ((uint32_t)(VCC_LDO_BITBAND << ACS_VCC_CTRL_BUCK_ENABLE_Pos))
#define VCC_BUCK                        ((uint32_t)(VCC_BUCK_BITBAND << ACS_VCC_CTRL_BUCK_ENABLE_Pos))

#define VCC_CONSTANT_CHARGE_BITBAND     0x0
#define VCC_CONSTANT_IMAX_BITBAND       0x1
#define VCC_CONSTANT_CHARGE             ((uint32_t)(VCC_CONSTANT_CHARGE_BITBAND << ACS_VCC_CTRL_CHARGE_CTRL_Pos))
#define VCC_CONSTANT_IMAX               ((uint32_t)(VCC_CONSTANT_IMAX_BITBAND << ACS_VCC_CTRL_CHARGE_CTRL_Pos))

#define VCC_SINGLE_PULSE_BITBAND        0x0
#define VCC_MULTI_PULSE_BITBAND         0x1
#define VCC_SINGLE_PULSE                ((uint32_t)(VCC_SINGLE_PULSE_BITBAND << ACS_VCC_CTRL_PULSE_CTRL_Pos))
#define VCC_MULTI_PULSE                 ((uint32_t)(VCC_MULTI_PULSE_BITBAND << ACS_VCC_CTRL_PULSE_CTRL_Pos))

#define VCC_DCM_MODE_BITBAND            0x0
#define VCC_CCM_MODE_BITBAND            0x1
#define VCC_DCM_MODE                    ((uint32_t)(VCC_DCM_MODE_BITBAND << ACS_VCC_CTRL_CCM_ENABLE_Pos))
#define VCC_CCM_MODE                    ((uint32_t)(VCC_CCM_MODE_BITBAND << ACS_VCC_CTRL_CCM_ENABLE_Pos))

#define VCC_ICHTRIM_16MA                ((uint32_t)(0x0U << ACS_VCC_CTRL_ICH_TRIM_Pos))
#define VCC_ICHTRIM_32MA                ((uint32_t)(0x1U << ACS_VCC_CTRL_ICH_TRIM_Pos))
#define VCC_ICHTRIM_64MA                ((uint32_t)(0x3U << ACS_VCC_CTRL_ICH_TRIM_Pos))
#define VCC_ICHTRIM_80MA                ((uint32_t)(0x4U << ACS_VCC_CTRL_ICH_TRIM_Pos))
#define VCC_ICHTRIM_256MA               ((uint32_t)(0xFU << ACS_VCC_CTRL_ICH_TRIM_Pos))

/* ACS_VCC_CTRL sub-register bit positions */
#define ACS_VCC_CTRL_VTRIM_BYTE_Pos     0
#define ACS_VCC_CTRL_VTRIM_BYTE_Mask    ((uint32_t)(0x1FU << ACS_VCC_CTRL_VTRIM_BYTE_Pos))
#define ACS_VCC_CTRL_ICH_TRIM_BYTE_Pos  0
#define ACS_VCC_CTRL_ICH_TRIM_BYTE_Mask ((uint32_t)(0xFU << ACS_VCC_CTRL_ICH_TRIM_BYTE_Pos))

/* ACS_VCC_CTRL sub-register settings */
#define VCC_TRIM_1P00V_BYTE             ((uint8_t)(0x0U << ACS_VCC_CTRL_VTRIM_BYTE_Pos))
#define VCC_TRIM_1P05V_BYTE             ((uint8_t)(0x5U << ACS_VCC_CTRL_VTRIM_BYTE_Pos))
#define VCC_TRIM_1P10V_BYTE             ((uint8_t)(0xAU << ACS_VCC_CTRL_VTRIM_BYTE_Pos))
#define VCC_TRIM_1P15V_BYTE             ((uint8_t)(0xFU << ACS_VCC_CTRL_VTRIM_BYTE_Pos))
#define VCC_TRIM_1P20V_BYTE             ((uint8_t)(0x14U << ACS_VCC_CTRL_VTRIM_BYTE_Pos))
#define VCC_TRIM_1P25V_BYTE             ((uint8_t)(0x19U << ACS_VCC_CTRL_VTRIM_BYTE_Pos))
#define VCC_TRIM_1P31V_BYTE             ((uint8_t)(0x1FU << ACS_VCC_CTRL_VTRIM_BYTE_Pos))

#define VCC_ICHTRIM_16MA_BYTE           ((uint8_t)(0x0U << ACS_VCC_CTRL_ICH_TRIM_BYTE_Pos))
#define VCC_ICHTRIM_32MA_BYTE           ((uint8_t)(0x1U << ACS_VCC_CTRL_ICH_TRIM_BYTE_Pos))
#define VCC_ICHTRIM_64MA_BYTE           ((uint8_t)(0x3U << ACS_VCC_CTRL_ICH_TRIM_BYTE_Pos))
#define VCC_ICHTRIM_80MA_BYTE           ((uint8_t)(0x4U << ACS_VCC_CTRL_ICH_TRIM_BYTE_Pos))
#define VCC_ICHTRIM_256MA_BYTE          ((uint8_t)(0xFU << ACS_VCC_CTRL_ICH_TRIM_BYTE_Pos))

/* ACS_VDDA_CP_CTRL bit positions */
#define ACS_VDDA_CP_CTRL_PTRIM_Pos      0
#define ACS_VDDA_CP_CTRL_PTRIM_Mask     ((uint32_t)(0x3U << ACS_VDDA_CP_CTRL_PTRIM_Pos))

/* ACS_VDDA_CP_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t PTRIM_BYTE;           
         uint8_t RESERVED0[3];
} ACS_VDDA_CP_CTRL_Type;

#define ACS_VDDA_CP_CTRL_BASE           0x40001308
#define ACS_VDDA_CP_CTRL                ((ACS_VDDA_CP_CTRL_Type *) ACS_VDDA_CP_CTRL_BASE)

/* ACS_VDDA_CP_CTRL settings */
#define VDDA_PTRIM_4MA                  ((uint32_t)(0x0U << ACS_VDDA_CP_CTRL_PTRIM_Pos))
#define VDDA_PTRIM_8MA                  ((uint32_t)(0x1U << ACS_VDDA_CP_CTRL_PTRIM_Pos))
#define VDDA_PTRIM_12MA                 ((uint32_t)(0x2U << ACS_VDDA_CP_CTRL_PTRIM_Pos))
#define VDDA_PTRIM_16MA                 ((uint32_t)(0x3U << ACS_VDDA_CP_CTRL_PTRIM_Pos))

/* ACS_VDDA_CP_CTRL sub-register bit positions */
#define ACS_VDDA_CP_CTRL_PTRIM_BYTE_Pos 0
#define ACS_VDDA_CP_CTRL_PTRIM_BYTE_Mask ((uint32_t)(0x3U << ACS_VDDA_CP_CTRL_PTRIM_BYTE_Pos))

/* ACS_VDDA_CP_CTRL sub-register settings */
#define VDDA_PTRIM_4MA_BYTE             ((uint8_t)(0x0U << ACS_VDDA_CP_CTRL_PTRIM_BYTE_Pos))
#define VDDA_PTRIM_8MA_BYTE             ((uint8_t)(0x1U << ACS_VDDA_CP_CTRL_PTRIM_BYTE_Pos))
#define VDDA_PTRIM_12MA_BYTE            ((uint8_t)(0x2U << ACS_VDDA_CP_CTRL_PTRIM_BYTE_Pos))
#define VDDA_PTRIM_16MA_BYTE            ((uint8_t)(0x3U << ACS_VDDA_CP_CTRL_PTRIM_BYTE_Pos))

/* ACS_VDDC_CTRL bit positions */
#define ACS_VDDC_CTRL_STANDBY_VTRIM_Pos 16
#define ACS_VDDC_CTRL_STANDBY_VTRIM_Mask ((uint32_t)(0x3FU << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define ACS_VDDC_CTRL_ENABLE_LOW_BIAS_Pos 13
#define ACS_VDDC_CTRL_SLEEP_CLAMP_Pos   12
#define ACS_VDDC_CTRL_VTRIM_Pos         0
#define ACS_VDDC_CTRL_VTRIM_Mask        ((uint32_t)(0x3FU << ACS_VDDC_CTRL_VTRIM_Pos))

/* ACS_VDDC_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t VTRIM_BYTE;           
         uint8_t RESERVED0[1];
    __IO uint8_t STANDBY_VTRIM_BYTE;   
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x4000130C, 0) - (0x4000130C + 4)];
         uint32_t RESERVED2[12];
    __IO uint32_t SLEEP_CLAMP_ALIAS;    /* Sleep mode clamp control */
    __IO uint32_t ENABLE_LOW_BIAS_ALIAS;/* Low power mode control */
} ACS_VDDC_CTRL_Type;

#define ACS_VDDC_CTRL_BASE              0x4000130C
#define ACS_VDDC_CTRL                   ((ACS_VDDC_CTRL_Type *) ACS_VDDC_CTRL_BASE)

/* ACS_VDDC_CTRL settings */
#define VDDC_TRIM_0P75V                 ((uint32_t)(0x0U << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_0P76V                 ((uint32_t)(0x1U << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_1P00V                 ((uint32_t)(0x19U << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_1P08V                 ((uint32_t)(0x21U << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_1P10V                 ((uint32_t)(0x23U << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_1P15V                 ((uint32_t)(0x28U << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_1P20V                 ((uint32_t)(0x2DU << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_1P25V                 ((uint32_t)(0x32U << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_1P32V                 ((uint32_t)(0x39U << ACS_VDDC_CTRL_VTRIM_Pos))
#define VDDC_TRIM_1P38V                 ((uint32_t)(0x3FU << ACS_VDDC_CTRL_VTRIM_Pos))

#define VDDC_SLEEP_HIZ_BITBAND          0x0
#define VDDC_SLEEP_GND_BITBAND          0x1
#define VDDC_SLEEP_HIZ                  ((uint32_t)(VDDC_SLEEP_HIZ_BITBAND << ACS_VDDC_CTRL_SLEEP_CLAMP_Pos))
#define VDDC_SLEEP_GND                  ((uint32_t)(VDDC_SLEEP_GND_BITBAND << ACS_VDDC_CTRL_SLEEP_CLAMP_Pos))

#define VDDC_NOMINAL_BIAS_BITBAND       0x0
#define VDDC_LOW_BIAS_BITBAND           0x1
#define VDDC_NOMINAL_BIAS               ((uint32_t)(VDDC_NOMINAL_BIAS_BITBAND << ACS_VDDC_CTRL_ENABLE_LOW_BIAS_Pos))
#define VDDC_LOW_BIAS                   ((uint32_t)(VDDC_LOW_BIAS_BITBAND << ACS_VDDC_CTRL_ENABLE_LOW_BIAS_Pos))

#define VDDC_STANDBY_TRIM_0P75V         ((uint32_t)(0x0U << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_0P76V         ((uint32_t)(0x1U << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_1P00V         ((uint32_t)(0x19U << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_1P08V         ((uint32_t)(0x21U << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_1P10V         ((uint32_t)(0x23U << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_1P15V         ((uint32_t)(0x28U << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_1P20V         ((uint32_t)(0x2DU << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_1P25V         ((uint32_t)(0x32U << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_1P32V         ((uint32_t)(0x39U << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))
#define VDDC_STANDBY_TRIM_1P38V         ((uint32_t)(0x3FU << ACS_VDDC_CTRL_STANDBY_VTRIM_Pos))

/* ACS_VDDC_CTRL sub-register bit positions */
#define ACS_VDDC_CTRL_VTRIM_BYTE_Pos    0
#define ACS_VDDC_CTRL_VTRIM_BYTE_Mask   ((uint32_t)(0x3FU << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos 0
#define ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Mask ((uint32_t)(0x3FU << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))

/* ACS_VDDC_CTRL sub-register settings */
#define VDDC_TRIM_0P75V_BYTE            ((uint8_t)(0x0U << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_0P76V_BYTE            ((uint8_t)(0x1U << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_1P00V_BYTE            ((uint8_t)(0x19U << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_1P08V_BYTE            ((uint8_t)(0x21U << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_1P10V_BYTE            ((uint8_t)(0x23U << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_1P15V_BYTE            ((uint8_t)(0x28U << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_1P20V_BYTE            ((uint8_t)(0x2DU << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_1P25V_BYTE            ((uint8_t)(0x32U << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_1P32V_BYTE            ((uint8_t)(0x39U << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))
#define VDDC_TRIM_1P38V_BYTE            ((uint8_t)(0x3FU << ACS_VDDC_CTRL_VTRIM_BYTE_Pos))

#define VDDC_STANDBY_TRIM_0P75V_BYTE    ((uint8_t)(0x0U << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_0P76V_BYTE    ((uint8_t)(0x1U << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_1P00V_BYTE    ((uint8_t)(0x19U << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_1P08V_BYTE    ((uint8_t)(0x21U << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_1P10V_BYTE    ((uint8_t)(0x23U << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_1P15V_BYTE    ((uint8_t)(0x28U << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_1P20V_BYTE    ((uint8_t)(0x2DU << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_1P25V_BYTE    ((uint8_t)(0x32U << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_1P32V_BYTE    ((uint8_t)(0x39U << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDC_STANDBY_TRIM_1P38V_BYTE    ((uint8_t)(0x3FU << ACS_VDDC_CTRL_STANDBY_VTRIM_BYTE_Pos))

/* ACS_VDDM_CTRL bit positions */
#define ACS_VDDM_CTRL_STANDBY_VTRIM_Pos 16
#define ACS_VDDM_CTRL_STANDBY_VTRIM_Mask ((uint32_t)(0x3FU << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define ACS_VDDM_CTRL_ENABLE_LOW_BIAS_Pos 13
#define ACS_VDDM_CTRL_SLEEP_CLAMP_Pos   12
#define ACS_VDDM_CTRL_VTRIM_Pos         0
#define ACS_VDDM_CTRL_VTRIM_Mask        ((uint32_t)(0x3FU << ACS_VDDM_CTRL_VTRIM_Pos))

/* ACS_VDDM_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t VTRIM_BYTE;           
         uint8_t RESERVED0[1];
    __IO uint8_t STANDBY_VTRIM_BYTE;   
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001310, 0) - (0x40001310 + 4)];
         uint32_t RESERVED2[12];
    __IO uint32_t SLEEP_CLAMP_ALIAS;    /* Sleep mode clamp control */
    __IO uint32_t ENABLE_LOW_BIAS_ALIAS;/* Low power mode control */
} ACS_VDDM_CTRL_Type;

#define ACS_VDDM_CTRL_BASE              0x40001310
#define ACS_VDDM_CTRL                   ((ACS_VDDM_CTRL_Type *) ACS_VDDM_CTRL_BASE)

/* ACS_VDDM_CTRL settings */
#define VDDM_TRIM_0P75V                 ((uint32_t)(0x0U << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_0P76V                 ((uint32_t)(0x1U << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_1P00V                 ((uint32_t)(0x19U << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_1P08V                 ((uint32_t)(0x21U << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_1P10V                 ((uint32_t)(0x23U << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_1P15V                 ((uint32_t)(0x28U << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_1P20V                 ((uint32_t)(0x2DU << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_1P25V                 ((uint32_t)(0x32U << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_1P32V                 ((uint32_t)(0x39U << ACS_VDDM_CTRL_VTRIM_Pos))
#define VDDM_TRIM_1P38V                 ((uint32_t)(0x3FU << ACS_VDDM_CTRL_VTRIM_Pos))

#define VDDM_SLEEP_HIZ_BITBAND          0x0
#define VDDM_SLEEP_GND_BITBAND          0x1
#define VDDM_SLEEP_HIZ                  ((uint32_t)(VDDM_SLEEP_HIZ_BITBAND << ACS_VDDM_CTRL_SLEEP_CLAMP_Pos))
#define VDDM_SLEEP_GND                  ((uint32_t)(VDDM_SLEEP_GND_BITBAND << ACS_VDDM_CTRL_SLEEP_CLAMP_Pos))

#define VDDM_NOMINAL_BIAS_BITBAND       0x0
#define VDDM_LOW_BIAS_BITBAND           0x1
#define VDDM_NOMINAL_BIAS               ((uint32_t)(VDDM_NOMINAL_BIAS_BITBAND << ACS_VDDM_CTRL_ENABLE_LOW_BIAS_Pos))
#define VDDM_LOW_BIAS                   ((uint32_t)(VDDM_LOW_BIAS_BITBAND << ACS_VDDM_CTRL_ENABLE_LOW_BIAS_Pos))

#define VDDM_STANDBY_TRIM_0P75V         ((uint32_t)(0x0U << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_0P76V         ((uint32_t)(0x1U << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_1P00V         ((uint32_t)(0x19U << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_1P08V         ((uint32_t)(0x21U << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_1P10V         ((uint32_t)(0x23U << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_1P15V         ((uint32_t)(0x28U << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_1P20V         ((uint32_t)(0x2DU << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_1P25V         ((uint32_t)(0x32U << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_1P32V         ((uint32_t)(0x39U << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))
#define VDDM_STANDBY_TRIM_1P38V         ((uint32_t)(0x3FU << ACS_VDDM_CTRL_STANDBY_VTRIM_Pos))

/* ACS_VDDM_CTRL sub-register bit positions */
#define ACS_VDDM_CTRL_VTRIM_BYTE_Pos    0
#define ACS_VDDM_CTRL_VTRIM_BYTE_Mask   ((uint32_t)(0x3FU << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos 0
#define ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Mask ((uint32_t)(0x3FU << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))

/* ACS_VDDM_CTRL sub-register settings */
#define VDDM_TRIM_0P75V_BYTE            ((uint8_t)(0x0U << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_0P76V_BYTE            ((uint8_t)(0x1U << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_1P00V_BYTE            ((uint8_t)(0x19U << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_1P08V_BYTE            ((uint8_t)(0x21U << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_1P10V_BYTE            ((uint8_t)(0x23U << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_1P15V_BYTE            ((uint8_t)(0x28U << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_1P20V_BYTE            ((uint8_t)(0x2DU << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_1P25V_BYTE            ((uint8_t)(0x32U << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_1P32V_BYTE            ((uint8_t)(0x39U << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))
#define VDDM_TRIM_1P38V_BYTE            ((uint8_t)(0x3FU << ACS_VDDM_CTRL_VTRIM_BYTE_Pos))

#define VDDM_STANDBY_TRIM_0P75V_BYTE    ((uint8_t)(0x0U << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_0P76V_BYTE    ((uint8_t)(0x1U << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_1P00V_BYTE    ((uint8_t)(0x19U << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_1P08V_BYTE    ((uint8_t)(0x21U << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_1P10V_BYTE    ((uint8_t)(0x23U << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_1P15V_BYTE    ((uint8_t)(0x28U << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_1P20V_BYTE    ((uint8_t)(0x2DU << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_1P25V_BYTE    ((uint8_t)(0x32U << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_1P32V_BYTE    ((uint8_t)(0x39U << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))
#define VDDM_STANDBY_TRIM_1P38V_BYTE    ((uint8_t)(0x3FU << ACS_VDDM_CTRL_STANDBY_VTRIM_BYTE_Pos))

/* ACS_VDDRF_CTRL bit positions */
#define ACS_VDDRF_CTRL_READY_Pos        24
#define ACS_VDDRF_CTRL_CLAMP_Pos        12
#define ACS_VDDRF_CTRL_ENABLE_Pos       8
#define ACS_VDDRF_CTRL_VTRIM_Pos        0
#define ACS_VDDRF_CTRL_VTRIM_Mask       ((uint32_t)(0x3FU << ACS_VDDRF_CTRL_VTRIM_Pos))

/* ACS_VDDRF_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t VTRIM_BYTE;           
         uint8_t RESERVED0[2];
    __I  uint8_t READY_BYTE;           
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001314, 0) - (0x40001314 + 4)];
         uint32_t RESERVED1[8];
    __IO uint32_t ENABLE_ALIAS;         /* Enable control */
         uint32_t RESERVED2[3];
    __IO uint32_t CLAMP_ALIAS;          /* Disable mode clamp control */
         uint32_t RESERVED3[11];
    __I  uint32_t READY_ALIAS;          /* Supply ready */
} ACS_VDDRF_CTRL_Type;

#define ACS_VDDRF_CTRL_BASE             0x40001314
#define ACS_VDDRF_CTRL                  ((ACS_VDDRF_CTRL_Type *) ACS_VDDRF_CTRL_BASE)

/* ACS_VDDRF_CTRL settings */
#define VDDRF_TRIM_0P75V                ((uint32_t)(0x0U << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_0P76V                ((uint32_t)(0x1U << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_1P00V                ((uint32_t)(0x19U << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_1P08V                ((uint32_t)(0x21U << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_1P10V                ((uint32_t)(0x23U << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_1P15V                ((uint32_t)(0x28U << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_1P20V                ((uint32_t)(0x2DU << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_1P25V                ((uint32_t)(0x32U << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_1P32V                ((uint32_t)(0x39U << ACS_VDDRF_CTRL_VTRIM_Pos))
#define VDDRF_TRIM_1P38V                ((uint32_t)(0x3FU << ACS_VDDRF_CTRL_VTRIM_Pos))

#define VDDRF_DISABLE_BITBAND           0x0
#define VDDRF_ENABLE_BITBAND            0x1
#define VDDRF_DISABLE                   ((uint32_t)(VDDRF_DISABLE_BITBAND << ACS_VDDRF_CTRL_ENABLE_Pos))
#define VDDRF_ENABLE                    ((uint32_t)(VDDRF_ENABLE_BITBAND << ACS_VDDRF_CTRL_ENABLE_Pos))

#define VDDRF_DISABLE_HIZ_BITBAND       0x0
#define VDDRF_DISABLE_GND_BITBAND       0x1
#define VDDRF_DISABLE_HIZ               ((uint32_t)(VDDRF_DISABLE_HIZ_BITBAND << ACS_VDDRF_CTRL_CLAMP_Pos))
#define VDDRF_DISABLE_GND               ((uint32_t)(VDDRF_DISABLE_GND_BITBAND << ACS_VDDRF_CTRL_CLAMP_Pos))

#define VDDRF_NOT_READY_BITBAND         0x0
#define VDDRF_READY_BITBAND             0x1
#define VDDRF_NOT_READY                 ((uint32_t)(VDDRF_NOT_READY_BITBAND << ACS_VDDRF_CTRL_READY_Pos))
#define VDDRF_READY                     ((uint32_t)(VDDRF_READY_BITBAND << ACS_VDDRF_CTRL_READY_Pos))

/* ACS_VDDRF_CTRL sub-register bit positions */
#define ACS_VDDRF_CTRL_VTRIM_BYTE_Pos   0
#define ACS_VDDRF_CTRL_VTRIM_BYTE_Mask  ((uint32_t)(0x3FU << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define ACS_VDDRF_CTRL_READY_BYTE_Pos   0

/* ACS_VDDRF_CTRL sub-register settings */
#define VDDRF_TRIM_0P75V_BYTE           ((uint8_t)(0x0U << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_0P76V_BYTE           ((uint8_t)(0x1U << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_1P00V_BYTE           ((uint8_t)(0x19U << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_1P08V_BYTE           ((uint8_t)(0x21U << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_1P10V_BYTE           ((uint8_t)(0x23U << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_1P15V_BYTE           ((uint8_t)(0x28U << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_1P20V_BYTE           ((uint8_t)(0x2DU << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_1P25V_BYTE           ((uint8_t)(0x32U << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_1P32V_BYTE           ((uint8_t)(0x39U << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))
#define VDDRF_TRIM_1P38V_BYTE           ((uint8_t)(0x3FU << ACS_VDDRF_CTRL_VTRIM_BYTE_Pos))

#define VDDRF_NOT_READY_BYTE            ((uint8_t)(VDDRF_NOT_READY_BITBAND << ACS_VDDRF_CTRL_READY_BYTE_Pos))
#define VDDRF_READY_BYTE                ((uint8_t)(VDDRF_READY_BITBAND << ACS_VDDRF_CTRL_READY_BYTE_Pos))

/* ACS_VDDPA_CTRL bit positions */
#define ACS_VDDPA_CTRL_VDDPA_SW_CTRL_Pos 12
#define ACS_VDDPA_CTRL_ENABLE_ISENSE_Pos 9
#define ACS_VDDPA_CTRL_ENABLE_Pos       8
#define ACS_VDDPA_CTRL_VTRIM_Pos        0
#define ACS_VDDPA_CTRL_VTRIM_Mask       ((uint32_t)(0x3FU << ACS_VDDPA_CTRL_VTRIM_Pos))

/* ACS_VDDPA_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t VTRIM_BYTE;           
         uint8_t RESERVED0[3];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001318, 0) - (0x40001318 + 4)];
         uint32_t RESERVED1[8];
    __IO uint32_t ENABLE_ALIAS;         /* Enable control */
    __IO uint32_t ENABLE_ISENSE_ALIAS;  /* Enable current sensing circuit */
         uint32_t RESERVED2[2];
    __IO uint32_t VDDPA_SW_CTRL_ALIAS;  /* Power amplifier supply control */
} ACS_VDDPA_CTRL_Type;

#define ACS_VDDPA_CTRL_BASE             0x40001318
#define ACS_VDDPA_CTRL                  ((ACS_VDDPA_CTRL_Type *) ACS_VDDPA_CTRL_BASE)

/* ACS_VDDPA_CTRL settings */
#define VDDPA_TRIM_1P05V                ((uint32_t)(0x0U << ACS_VDDPA_CTRL_VTRIM_Pos))
#define VDDPA_TRIM_1P06V                ((uint32_t)(0x1U << ACS_VDDPA_CTRL_VTRIM_Pos))
#define VDDPA_TRIM_1P59V                ((uint32_t)(0x36U << ACS_VDDPA_CTRL_VTRIM_Pos))
#define VDDPA_TRIM_1P60V                ((uint32_t)(0x37U << ACS_VDDPA_CTRL_VTRIM_Pos))
#define VDDPA_TRIM_1P61V                ((uint32_t)(0x38U << ACS_VDDPA_CTRL_VTRIM_Pos))
#define VDDPA_TRIM_1P68V                ((uint32_t)(0x3FU << ACS_VDDPA_CTRL_VTRIM_Pos))

#define VDDPA_DISABLE_BITBAND           0x0
#define VDDPA_ENABLE_BITBAND            0x1
#define VDDPA_DISABLE                   ((uint32_t)(VDDPA_DISABLE_BITBAND << ACS_VDDPA_CTRL_ENABLE_Pos))
#define VDDPA_ENABLE                    ((uint32_t)(VDDPA_ENABLE_BITBAND << ACS_VDDPA_CTRL_ENABLE_Pos))

#define VDDPA_ISENSE_DISABLE_BITBAND    0x0
#define VDDPA_ISENSE_ENABLE_BITBAND     0x1
#define VDDPA_ISENSE_DISABLE            ((uint32_t)(VDDPA_ISENSE_DISABLE_BITBAND << ACS_VDDPA_CTRL_ENABLE_ISENSE_Pos))
#define VDDPA_ISENSE_ENABLE             ((uint32_t)(VDDPA_ISENSE_ENABLE_BITBAND << ACS_VDDPA_CTRL_ENABLE_ISENSE_Pos))

#define VDDPA_SW_HIZ_BITBAND            0x0
#define VDDPA_SW_VDDRF_BITBAND          0x1
#define VDDPA_SW_HIZ                    ((uint32_t)(VDDPA_SW_HIZ_BITBAND << ACS_VDDPA_CTRL_VDDPA_SW_CTRL_Pos))
#define VDDPA_SW_VDDRF                  ((uint32_t)(VDDPA_SW_VDDRF_BITBAND << ACS_VDDPA_CTRL_VDDPA_SW_CTRL_Pos))

/* ACS_VDDPA_CTRL sub-register bit positions */
#define ACS_VDDPA_CTRL_VTRIM_BYTE_Pos   0
#define ACS_VDDPA_CTRL_VTRIM_BYTE_Mask  ((uint32_t)(0x3FU << ACS_VDDPA_CTRL_VTRIM_BYTE_Pos))

/* ACS_VDDPA_CTRL sub-register settings */
#define VDDPA_TRIM_1P05V_BYTE           ((uint8_t)(0x0U << ACS_VDDPA_CTRL_VTRIM_BYTE_Pos))
#define VDDPA_TRIM_1P06V_BYTE           ((uint8_t)(0x1U << ACS_VDDPA_CTRL_VTRIM_BYTE_Pos))
#define VDDPA_TRIM_1P59V_BYTE           ((uint8_t)(0x36U << ACS_VDDPA_CTRL_VTRIM_BYTE_Pos))
#define VDDPA_TRIM_1P60V_BYTE           ((uint8_t)(0x37U << ACS_VDDPA_CTRL_VTRIM_BYTE_Pos))
#define VDDPA_TRIM_1P61V_BYTE           ((uint8_t)(0x38U << ACS_VDDPA_CTRL_VTRIM_BYTE_Pos))
#define VDDPA_TRIM_1P68V_BYTE           ((uint8_t)(0x3FU << ACS_VDDPA_CTRL_VTRIM_BYTE_Pos))

/* ACS_VDDRET_CTRL bit positions */
#define ACS_VDDRET_CTRL_VDDMRET_VTRIM_Pos 17
#define ACS_VDDRET_CTRL_VDDMRET_VTRIM_Mask ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDMRET_VTRIM_Pos))
#define ACS_VDDRET_CTRL_VDDMRET_ENABLE_Pos 16
#define ACS_VDDRET_CTRL_VDDTRET_VTRIM_Pos 9
#define ACS_VDDRET_CTRL_VDDTRET_VTRIM_Mask ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDTRET_VTRIM_Pos))
#define ACS_VDDRET_CTRL_VDDTRET_ENABLE_Pos 8
#define ACS_VDDRET_CTRL_VDDCRET_VTRIM_Pos 1
#define ACS_VDDRET_CTRL_VDDCRET_VTRIM_Mask ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDCRET_VTRIM_Pos))
#define ACS_VDDRET_CTRL_VDDCRET_ENABLE_Pos 0

/* ACS_VDDRET_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t VDDCRET_BYTE;         
    __IO uint8_t VDDTRET_BYTE;         
    __IO uint8_t VDDMRET_BYTE;         
         uint8_t RESERVED0[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x4000131C, 0) - (0x4000131C + 4)];
    __IO uint32_t VDDCRET_ENABLE_ALIAS; /* Enable/Disable the VDDCRET retention regulator */
         uint32_t RESERVED1[7];
    __IO uint32_t VDDTRET_ENABLE_ALIAS; /* Enable/Disable the VDDTRET retention regulator */
         uint32_t RESERVED2[7];
    __IO uint32_t VDDMRET_ENABLE_ALIAS; /* Enable/Disable the VDDMRET retention regulator */
} ACS_VDDRET_CTRL_Type;

#define ACS_VDDRET_CTRL_BASE            0x4000131C
#define ACS_VDDRET_CTRL                 ((ACS_VDDRET_CTRL_Type *) ACS_VDDRET_CTRL_BASE)

/* ACS_VDDRET_CTRL settings */
#define VDDCRET_DISABLE_BITBAND         0x0
#define VDDCRET_ENABLE_BITBAND          0x1
#define VDDCRET_DISABLE                 ((uint32_t)(VDDCRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDCRET_ENABLE_Pos))
#define VDDCRET_ENABLE                  ((uint32_t)(VDDCRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDCRET_ENABLE_Pos))

#define VDDCRET_TRIM_VALUE              ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDCRET_VTRIM_Pos))

#define VDDTRET_DISABLE_BITBAND         0x0
#define VDDTRET_ENABLE_BITBAND          0x1
#define VDDTRET_DISABLE                 ((uint32_t)(VDDTRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDTRET_ENABLE_Pos))
#define VDDTRET_ENABLE                  ((uint32_t)(VDDTRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDTRET_ENABLE_Pos))

#define VDDTRET_TRIM_VALUE              ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDTRET_VTRIM_Pos))

#define VDDMRET_DISABLE_BITBAND         0x0
#define VDDMRET_ENABLE_BITBAND          0x1
#define VDDMRET_DISABLE                 ((uint32_t)(VDDMRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDMRET_ENABLE_Pos))
#define VDDMRET_ENABLE                  ((uint32_t)(VDDMRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDMRET_ENABLE_Pos))

#define VDDMRET_TRIM_VALUE              ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDMRET_VTRIM_Pos))

/* ACS_VDDRET_CTRL sub-register bit positions */
#define ACS_VDDRET_CTRL_VDDCRET_ENABLE_BYTE_Pos 0
#define ACS_VDDRET_CTRL_VDDCRET_VTRIM_BYTE_Pos 1
#define ACS_VDDRET_CTRL_VDDCRET_VTRIM_BYTE_Mask ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDCRET_VTRIM_BYTE_Pos))
#define ACS_VDDRET_CTRL_VDDTRET_ENABLE_BYTE_Pos 0
#define ACS_VDDRET_CTRL_VDDTRET_VTRIM_BYTE_Pos 1
#define ACS_VDDRET_CTRL_VDDTRET_VTRIM_BYTE_Mask ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDTRET_VTRIM_BYTE_Pos))
#define ACS_VDDRET_CTRL_VDDMRET_ENABLE_BYTE_Pos 0
#define ACS_VDDRET_CTRL_VDDMRET_VTRIM_BYTE_Pos 1
#define ACS_VDDRET_CTRL_VDDMRET_VTRIM_BYTE_Mask ((uint32_t)(0x3U << ACS_VDDRET_CTRL_VDDMRET_VTRIM_BYTE_Pos))

/* ACS_VDDRET_CTRL sub-register settings */
#define VDDCRET_DISABLE_BYTE            ((uint8_t)(VDDCRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDCRET_ENABLE_BYTE_Pos))
#define VDDCRET_ENABLE_BYTE             ((uint8_t)(VDDCRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDCRET_ENABLE_BYTE_Pos))

#define VDDCRET_TRIM_VALUE_BYTE         ((uint8_t)(0x3U << ACS_VDDRET_CTRL_VDDCRET_VTRIM_BYTE_Pos))

#define VDDTRET_DISABLE_BYTE            ((uint8_t)(VDDTRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDTRET_ENABLE_BYTE_Pos))
#define VDDTRET_ENABLE_BYTE             ((uint8_t)(VDDTRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDTRET_ENABLE_BYTE_Pos))

#define VDDTRET_TRIM_VALUE_BYTE         ((uint8_t)(0x3U << ACS_VDDRET_CTRL_VDDTRET_VTRIM_BYTE_Pos))

#define VDDMRET_DISABLE_BYTE            ((uint8_t)(VDDMRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDMRET_ENABLE_BYTE_Pos))
#define VDDMRET_ENABLE_BYTE             ((uint8_t)(VDDMRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDMRET_ENABLE_BYTE_Pos))

#define VDDMRET_TRIM_VALUE_BYTE         ((uint8_t)(0x3U << ACS_VDDRET_CTRL_VDDMRET_VTRIM_BYTE_Pos))

/* ACS_RCOSC_CTRL bit positions */
#define ACS_RCOSC_CTRL_CLOCK_MULT_Pos   18
#define ACS_RCOSC_CTRL_RC_OSC_EN_Pos    16
#define ACS_RCOSC_CTRL_FTRIM_FLAG_Pos   15
#define ACS_RCOSC_CTRL_FTRIM_START_Pos  8
#define ACS_RCOSC_CTRL_FTRIM_START_Mask ((uint32_t)(0x3FU << ACS_RCOSC_CTRL_FTRIM_START_Pos))
#define ACS_RCOSC_CTRL_FTRIM_32K_ADJ_Pos 6
#define ACS_RCOSC_CTRL_FTRIM_32K_Pos    0
#define ACS_RCOSC_CTRL_FTRIM_32K_Mask   ((uint32_t)(0x3FU << ACS_RCOSC_CTRL_FTRIM_32K_Pos))

/* ACS_RCOSC_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t FTRIM_32K_BYTE;       
         uint8_t RESERVED0[3];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001320, 0) - (0x40001320 + 4)];
         uint32_t RESERVED1[6];
    __IO uint32_t FTRIM_32K_ADJ_ALIAS;  /* Adjust 32 kHz oscillator frequency range */
         uint32_t RESERVED2[8];
    __IO uint32_t FTRIM_FLAG_ALIAS;     /* Trimming flag */
    __IO uint32_t RC_OSC_EN_ALIAS;      /* Enable/Disable the 32 kHz RC Oscillator */
         uint32_t RESERVED3;
    __IO uint32_t CLOCK_MULT_ALIAS;     /* Enable 12 MHz mode of startup oscillator */
} ACS_RCOSC_CTRL_Type;

#define ACS_RCOSC_CTRL_BASE             0x40001320
#define ACS_RCOSC_CTRL                  ((ACS_RCOSC_CTRL_Type *) ACS_RCOSC_CTRL_BASE)

/* ACS_RCOSC_CTRL settings */
#define RC_OSC_M48                      ((uint32_t)(0x0U << ACS_RCOSC_CTRL_FTRIM_32K_Pos))
#define RC_OSC_M46P5                    ((uint32_t)(0x1U << ACS_RCOSC_CTRL_FTRIM_32K_Pos))
#define RC_OSC_NOM                      ((uint32_t)(0x20U << ACS_RCOSC_CTRL_FTRIM_32K_Pos))
#define RC_OSC_P46P5                    ((uint32_t)(0x3FU << ACS_RCOSC_CTRL_FTRIM_32K_Pos))

#define RC_OSC_RANGE_NOM_BITBAND        0x0
#define RC_OSC_RANGE_M25_BITBAND        0x1
#define RC_OSC_RANGE_NOM                ((uint32_t)(RC_OSC_RANGE_NOM_BITBAND << ACS_RCOSC_CTRL_FTRIM_32K_ADJ_Pos))
#define RC_OSC_RANGE_M25                ((uint32_t)(RC_OSC_RANGE_M25_BITBAND << ACS_RCOSC_CTRL_FTRIM_32K_ADJ_Pos))

#define RC_START_OSC_M48                ((uint32_t)(0x0U << ACS_RCOSC_CTRL_FTRIM_START_Pos))
#define RC_START_OSC_M46P5              ((uint32_t)(0x1U << ACS_RCOSC_CTRL_FTRIM_START_Pos))
#define RC_START_OSC_NOM                ((uint32_t)(0x20U << ACS_RCOSC_CTRL_FTRIM_START_Pos))
#define RC_START_OSC_P46P5              ((uint32_t)(0x3FU << ACS_RCOSC_CTRL_FTRIM_START_Pos))

#define RC_OSC_UNCALIBRATED_BITBAND     0x0
#define RC_OSC_CALIBRATED_BITBAND       0x1
#define RC_OSC_UNCALIBRATED             ((uint32_t)(RC_OSC_UNCALIBRATED_BITBAND << ACS_RCOSC_CTRL_FTRIM_FLAG_Pos))
#define RC_OSC_CALIBRATED               ((uint32_t)(RC_OSC_CALIBRATED_BITBAND << ACS_RCOSC_CTRL_FTRIM_FLAG_Pos))

#define RC_OSC_DISABLE_BITBAND          0x0
#define RC_OSC_ENABLE_BITBAND           0x1
#define RC_OSC_DISABLE                  ((uint32_t)(RC_OSC_DISABLE_BITBAND << ACS_RCOSC_CTRL_RC_OSC_EN_Pos))
#define RC_OSC_ENABLE                   ((uint32_t)(RC_OSC_ENABLE_BITBAND << ACS_RCOSC_CTRL_RC_OSC_EN_Pos))

#define RC_START_OSC_3MHZ_BITBAND       0x0
#define RC_START_OSC_12MHZ_BITBAND      0x1
#define RC_START_OSC_3MHZ               ((uint32_t)(RC_START_OSC_3MHZ_BITBAND << ACS_RCOSC_CTRL_CLOCK_MULT_Pos))
#define RC_START_OSC_12MHZ              ((uint32_t)(RC_START_OSC_12MHZ_BITBAND << ACS_RCOSC_CTRL_CLOCK_MULT_Pos))

/* ACS_RCOSC_CTRL sub-register bit positions */
#define ACS_RCOSC_CTRL_FTRIM_32K_BYTE_Pos 0
#define ACS_RCOSC_CTRL_FTRIM_32K_BYTE_Mask ((uint32_t)(0x3FU << ACS_RCOSC_CTRL_FTRIM_32K_BYTE_Pos))
#define ACS_RCOSC_CTRL_FTRIM_32K_ADJ_BYTE_Pos 6

/* ACS_RCOSC_CTRL sub-register settings */
#define RC_OSC_M48_BYTE                 ((uint8_t)(0x0U << ACS_RCOSC_CTRL_FTRIM_32K_BYTE_Pos))
#define RC_OSC_M46P5_BYTE               ((uint8_t)(0x1U << ACS_RCOSC_CTRL_FTRIM_32K_BYTE_Pos))
#define RC_OSC_NOM_BYTE                 ((uint8_t)(0x20U << ACS_RCOSC_CTRL_FTRIM_32K_BYTE_Pos))
#define RC_OSC_P46P5_BYTE               ((uint8_t)(0x3FU << ACS_RCOSC_CTRL_FTRIM_32K_BYTE_Pos))

#define RC_OSC_RANGE_NOM_BYTE           ((uint8_t)(RC_OSC_RANGE_NOM_BITBAND << ACS_RCOSC_CTRL_FTRIM_32K_ADJ_BYTE_Pos))
#define RC_OSC_RANGE_M25_BYTE           ((uint8_t)(RC_OSC_RANGE_M25_BITBAND << ACS_RCOSC_CTRL_FTRIM_32K_ADJ_BYTE_Pos))

/* ACS_XTAL32K_CTRL bit positions */
#define ACS_XTAL32K_CTRL_READY_Pos      24
#define ACS_XTAL32K_CTRL_XIN_CAP_BYPASS_EN_Pos 18
#define ACS_XTAL32K_CTRL_EN_AMPL_CTRL_Pos 17
#define ACS_XTAL32K_CTRL_FORCE_READY_Pos 16
#define ACS_XTAL32K_CTRL_CLOAD_TRIM_Pos 8
#define ACS_XTAL32K_CTRL_CLOAD_TRIM_Mask ((uint32_t)(0x3FU << ACS_XTAL32K_CTRL_CLOAD_TRIM_Pos))
#define ACS_XTAL32K_CTRL_ITRIM_Pos      4
#define ACS_XTAL32K_CTRL_ITRIM_Mask     ((uint32_t)(0xFU << ACS_XTAL32K_CTRL_ITRIM_Pos))
#define ACS_XTAL32K_CTRL_IBOOST_Pos     1
#define ACS_XTAL32K_CTRL_ENABLE_Pos     0

/* ACS_XTAL32K_CTRL sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t CLOAD_TRIM_BYTE;      
         uint8_t RESERVED1[1];
    __I  uint8_t READY_BYTE;           
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001324, 0) - (0x40001324 + 4)];
    __IO uint32_t ENABLE_ALIAS;         /* Enable the XTAL 32 kHz oscillator */
    __IO uint32_t IBOOST_ALIAS;         /* XTAL current boosting (4x) */
         uint32_t RESERVED2[14];
    __IO uint32_t FORCE_READY_ALIAS;    /* XTAL bypass the ready detector */
    __IO uint32_t EN_AMPL_CTRL_ALIAS;   /* XTAL enable amplitude control (regulation) */
    __IO uint32_t XIN_CAP_BYPASS_EN_ALIAS;/* Switch to bypass the added XIN serial cap to reduce the leakage */
         uint32_t RESERVED3[5];
    __I  uint32_t READY_ALIAS;          /* XTAL ready status */
} ACS_XTAL32K_CTRL_Type;

#define ACS_XTAL32K_CTRL_BASE           0x40001324
#define ACS_XTAL32K_CTRL                ((ACS_XTAL32K_CTRL_Type *) ACS_XTAL32K_CTRL_BASE)

/* ACS_XTAL32K_CTRL settings */
#define XTAL32K_DISABLE_BITBAND         0x0
#define XTAL32K_ENABLE_BITBAND          0x1
#define XTAL32K_DISABLE                 ((uint32_t)(XTAL32K_DISABLE_BITBAND << ACS_XTAL32K_CTRL_ENABLE_Pos))
#define XTAL32K_ENABLE                  ((uint32_t)(XTAL32K_ENABLE_BITBAND << ACS_XTAL32K_CTRL_ENABLE_Pos))

#define XTAL32K_IBOOST_DISABLE_BITBAND  0x0
#define XTAL32K_IBOOST_ENABLE_BITBAND   0x1
#define XTAL32K_IBOOST_DISABLE          ((uint32_t)(XTAL32K_IBOOST_DISABLE_BITBAND << ACS_XTAL32K_CTRL_IBOOST_Pos))
#define XTAL32K_IBOOST_ENABLE           ((uint32_t)(XTAL32K_IBOOST_ENABLE_BITBAND << ACS_XTAL32K_CTRL_IBOOST_Pos))

#define XTAL32K_ITRIM_20NA              ((uint32_t)(0x0U << ACS_XTAL32K_CTRL_ITRIM_Pos))
#define XTAL32K_ITRIM_80NA              ((uint32_t)(0x3U << ACS_XTAL32K_CTRL_ITRIM_Pos))
#define XTAL32K_ITRIM_160NA             ((uint32_t)(0x7U << ACS_XTAL32K_CTRL_ITRIM_Pos))
#define XTAL32K_ITRIM_320NA             ((uint32_t)(0xFU << ACS_XTAL32K_CTRL_ITRIM_Pos))

#define XTAL32K_CTRIM_0P0PF             ((uint32_t)(0x0U << ACS_XTAL32K_CTRL_CLOAD_TRIM_Pos))
#define XTAL32K_CTRIM_0P4PF             ((uint32_t)(0x1U << ACS_XTAL32K_CTRL_CLOAD_TRIM_Pos))
#define XTAL32K_CTRIM_3P6PF             ((uint32_t)(0x9U << ACS_XTAL32K_CTRL_CLOAD_TRIM_Pos))
#define XTAL32K_CTRIM_25P2PF            ((uint32_t)(0x3FU << ACS_XTAL32K_CTRL_CLOAD_TRIM_Pos))

#define XTAL32K_NOT_FORCE_READY_BITBAND 0x0
#define XTAL32K_FORCE_READY_BITBAND     0x1
#define XTAL32K_NOT_FORCE_READY         ((uint32_t)(XTAL32K_NOT_FORCE_READY_BITBAND << ACS_XTAL32K_CTRL_FORCE_READY_Pos))
#define XTAL32K_FORCE_READY             ((uint32_t)(XTAL32K_FORCE_READY_BITBAND << ACS_XTAL32K_CTRL_FORCE_READY_Pos))

#define XTAL32K_AMPL_CTRL_DISABLE_BITBAND 0x0
#define XTAL32K_AMPL_CTRL_ENABLE_BITBAND 0x1
#define XTAL32K_AMPL_CTRL_DISABLE       ((uint32_t)(XTAL32K_AMPL_CTRL_DISABLE_BITBAND << ACS_XTAL32K_CTRL_EN_AMPL_CTRL_Pos))
#define XTAL32K_AMPL_CTRL_ENABLE        ((uint32_t)(XTAL32K_AMPL_CTRL_ENABLE_BITBAND << ACS_XTAL32K_CTRL_EN_AMPL_CTRL_Pos))

#define XTAL32K_XIN_CAP_BYPASS_DISABLE_BITBAND 0x0
#define XTAL32K_XIN_CAP_BYPASS_ENABLE_BITBAND 0x1
#define XTAL32K_XIN_CAP_BYPASS_DISABLE  ((uint32_t)(XTAL32K_XIN_CAP_BYPASS_DISABLE_BITBAND << ACS_XTAL32K_CTRL_XIN_CAP_BYPASS_EN_Pos))
#define XTAL32K_XIN_CAP_BYPASS_ENABLE   ((uint32_t)(XTAL32K_XIN_CAP_BYPASS_ENABLE_BITBAND << ACS_XTAL32K_CTRL_XIN_CAP_BYPASS_EN_Pos))

#define XTAL32K_NOT_OK_BITBAND          0x0
#define XTAL32K_OK_BITBAND              0x1
#define XTAL32K_NOT_OK                  ((uint32_t)(XTAL32K_NOT_OK_BITBAND << ACS_XTAL32K_CTRL_READY_Pos))
#define XTAL32K_OK                      ((uint32_t)(XTAL32K_OK_BITBAND << ACS_XTAL32K_CTRL_READY_Pos))

/* ACS_XTAL32K_CTRL sub-register bit positions */
#define ACS_XTAL32K_CTRL_CLOAD_TRIM_BYTE_Pos 0
#define ACS_XTAL32K_CTRL_CLOAD_TRIM_BYTE_Mask ((uint32_t)(0x3FU << ACS_XTAL32K_CTRL_CLOAD_TRIM_BYTE_Pos))
#define ACS_XTAL32K_CTRL_READY_BYTE_Pos 0

/* ACS_XTAL32K_CTRL sub-register settings */
#define XTAL32K_CTRIM_0P0PF_BYTE        ((uint8_t)(0x0U << ACS_XTAL32K_CTRL_CLOAD_TRIM_BYTE_Pos))
#define XTAL32K_CTRIM_0P4PF_BYTE        ((uint8_t)(0x1U << ACS_XTAL32K_CTRL_CLOAD_TRIM_BYTE_Pos))
#define XTAL32K_CTRIM_3P6PF_BYTE        ((uint8_t)(0x9U << ACS_XTAL32K_CTRL_CLOAD_TRIM_BYTE_Pos))
#define XTAL32K_CTRIM_25P2PF_BYTE       ((uint8_t)(0x3FU << ACS_XTAL32K_CTRL_CLOAD_TRIM_BYTE_Pos))

#define XTAL32K_NOT_OK_BYTE             ((uint8_t)(XTAL32K_NOT_OK_BITBAND << ACS_XTAL32K_CTRL_READY_BYTE_Pos))
#define XTAL32K_OK_BYTE                 ((uint8_t)(XTAL32K_OK_BITBAND << ACS_XTAL32K_CTRL_READY_BYTE_Pos))

/* ACS_BB_TIMER_CTRL bit positions */
#define ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_Pos 8
#define ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_Mask ((uint32_t)(0x3U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_Pos))
#define ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_Pos 0

/* ACS_BB_TIMER_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t BB_TIMER_NRESET_BYTE; 
    __IO uint8_t BB_CLK_PRESCALE_BYTE; 
         uint8_t RESERVED0[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001328, 0) - (0x40001328 + 4)];
    __IO uint32_t BB_TIMER_NRESET_ALIAS;/* nReset signal for the baseband timer */
} ACS_BB_TIMER_CTRL_Type;

#define ACS_BB_TIMER_CTRL_BASE          0x40001328
#define ACS_BB_TIMER_CTRL               ((ACS_BB_TIMER_CTRL_Type *) ACS_BB_TIMER_CTRL_BASE)

/* ACS_BB_TIMER_CTRL settings */
#define BB_TIMER_RESET_BITBAND          0x0
#define BB_TIMER_NRESET_BITBAND         0x1
#define BB_TIMER_RESET                  ((uint32_t)(BB_TIMER_RESET_BITBAND << ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_Pos))
#define BB_TIMER_NRESET                 ((uint32_t)(BB_TIMER_NRESET_BITBAND << ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_Pos))

#define BB_CLK_PRESCALE_1               ((uint32_t)(0x0U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_Pos))
#define BB_CLK_PRESCALE_2               ((uint32_t)(0x1U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_Pos))
#define BB_CLK_PRESCALE_4               ((uint32_t)(0x2U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_Pos))
#define BB_CLK_PRESCALE_8               ((uint32_t)(0x3U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_Pos))

/* ACS_BB_TIMER_CTRL sub-register bit positions */
#define ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_BYTE_Pos 0
#define ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_BYTE_Pos 0
#define ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_BYTE_Mask ((uint32_t)(0x3U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_BYTE_Pos))

/* ACS_BB_TIMER_CTRL sub-register settings */
#define BB_TIMER_RESET_BYTE             ((uint8_t)(BB_TIMER_RESET_BITBAND << ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_BYTE_Pos))
#define BB_TIMER_NRESET_BYTE            ((uint8_t)(BB_TIMER_NRESET_BITBAND << ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_BYTE_Pos))

#define BB_CLK_PRESCALE_1_BYTE          ((uint8_t)(0x0U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_BYTE_Pos))
#define BB_CLK_PRESCALE_2_BYTE          ((uint8_t)(0x1U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_BYTE_Pos))
#define BB_CLK_PRESCALE_4_BYTE          ((uint8_t)(0x2U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_BYTE_Pos))
#define BB_CLK_PRESCALE_8_BYTE          ((uint8_t)(0x3U << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_BYTE_Pos))

/* ACS_CLK_DET_CTRL bit positions */
#define ACS_CLK_DET_CTRL_CLOCK_PRESENT_Pos 8
#define ACS_CLK_DET_CTRL_RESET_IGNORE_Pos 1
#define ACS_CLK_DET_CTRL_ENABLE_Pos     0

/* ACS_CLK_DET_CTRL sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __I  uint8_t CLOCK_PRESENT_BYTE;   
         uint8_t RESERVED1[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x4000132C, 0) - (0x4000132C + 4)];
    __IO uint32_t ENABLE_ALIAS;         /* Clock detector enable */
    __IO uint32_t RESET_IGNORE_ALIAS;   /* Clock detector reset condition ignore */
         uint32_t RESERVED2[6];
    __I  uint32_t CLOCK_PRESENT_ALIAS;  /* Clock present flag */
} ACS_CLK_DET_CTRL_Type;

#define ACS_CLK_DET_CTRL_BASE           0x4000132C
#define ACS_CLK_DET_CTRL                ((ACS_CLK_DET_CTRL_Type *) ACS_CLK_DET_CTRL_BASE)

/* ACS_CLK_DET_CTRL settings */
#define ACS_CLK_DET_ENABLE_BITBAND      0x1
#define ACS_CLK_DET_DISABLE_BITBAND     0x0
#define ACS_CLK_DET_ENABLE              ((uint32_t)(ACS_CLK_DET_ENABLE_BITBAND << ACS_CLK_DET_CTRL_ENABLE_Pos))
#define ACS_CLK_DET_DISABLE             ((uint32_t)(ACS_CLK_DET_DISABLE_BITBAND << ACS_CLK_DET_CTRL_ENABLE_Pos))

#define ACS_CLK_DET_RESET_DISABLE_BITBAND 0x1
#define ACS_CLK_DET_RESET_ENABLE_BITBAND 0x0
#define ACS_CLK_DET_RESET_DISABLE       ((uint32_t)(ACS_CLK_DET_RESET_DISABLE_BITBAND << ACS_CLK_DET_CTRL_RESET_IGNORE_Pos))
#define ACS_CLK_DET_RESET_ENABLE        ((uint32_t)(ACS_CLK_DET_RESET_ENABLE_BITBAND << ACS_CLK_DET_CTRL_RESET_IGNORE_Pos))

#define ACS_CLK_DET_NO_CLOCK_BITBAND    0x0
#define ACS_CLK_DET_CLOCK_PRESENT_BITBAND 0x1
#define ACS_CLK_DET_NO_CLOCK            ((uint32_t)(ACS_CLK_DET_NO_CLOCK_BITBAND << ACS_CLK_DET_CTRL_CLOCK_PRESENT_Pos))
#define ACS_CLK_DET_CLOCK_PRESENT       ((uint32_t)(ACS_CLK_DET_CLOCK_PRESENT_BITBAND << ACS_CLK_DET_CTRL_CLOCK_PRESENT_Pos))

/* ACS_CLK_DET_CTRL sub-register bit positions */
#define ACS_CLK_DET_CTRL_CLOCK_PRESENT_BYTE_Pos 0

/* ACS_CLK_DET_CTRL sub-register settings */
#define ACS_CLK_DET_NO_CLOCK_BYTE       ((uint8_t)(ACS_CLK_DET_NO_CLOCK_BITBAND << ACS_CLK_DET_CTRL_CLOCK_PRESENT_BYTE_Pos))
#define ACS_CLK_DET_CLOCK_PRESENT_BYTE  ((uint8_t)(ACS_CLK_DET_CLOCK_PRESENT_BITBAND << ACS_CLK_DET_CTRL_CLOCK_PRESENT_BYTE_Pos))

/* ACS_RTC_CFG bit positions */
#define ACS_RTC_CFG_START_VALUE_Pos     0
#define ACS_RTC_CFG_START_VALUE_Mask    ((uint32_t)(0xFFFFFFFFU << ACS_RTC_CFG_START_VALUE_Pos))

/* ACS_RTC_CFG settings */
#define RTC_CNT_START_0                 ((uint32_t)(0x0U << ACS_RTC_CFG_START_VALUE_Pos))
#define RTC_CNT_START_1                 ((uint32_t)(0x1U << ACS_RTC_CFG_START_VALUE_Pos))
#define RTC_CNT_START_32767             ((uint32_t)(0x7FFFU << ACS_RTC_CFG_START_VALUE_Pos))
#define RTC_CNT_START_4294967295        ((uint32_t)(0xFFFFFFFFU << ACS_RTC_CFG_START_VALUE_Pos))

/* ACS_RTC_COUNT bit positions */
#define ACS_RTC_COUNT_VALUE_Pos         0
#define ACS_RTC_COUNT_VALUE_Mask        ((uint32_t)(0xFFFFFFFFU << ACS_RTC_COUNT_VALUE_Pos))

/* ACS_RTC_CTRL bit positions */
#define ACS_RTC_CTRL_FORCE_CLOCK_Pos    25
#define ACS_RTC_CTRL_RESET_Pos          24
#define ACS_RTC_CTRL_ALARM_CFG_Pos      4
#define ACS_RTC_CTRL_ALARM_CFG_Mask     ((uint32_t)(0xFU << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define ACS_RTC_CTRL_CLK_SRC_SEL_Pos    1
#define ACS_RTC_CTRL_CLK_SRC_SEL_Mask   ((uint32_t)(0x7U << ACS_RTC_CTRL_CLK_SRC_SEL_Pos))
#define ACS_RTC_CTRL_ENABLE_Pos         0

/* ACS_RTC_CTRL bit-band aliases */
typedef struct
{
    __IO uint32_t ENABLE_ALIAS;         /* Enable counter and RTC interrupt every 1s */
         uint32_t RESERVED0[23];
    __O  uint32_t RESET_ALIAS;          /* Reset the RTC timer */
    __O  uint32_t FORCE_CLOCK_ALIAS;    /* Force a clock on RTC timer (Test Purpose) */
} ACS_RTC_CTRL_Type;

#define ACS_RTC_CTRL_BASE               SYS_CALC_BITBAND(0x40001338, 0)
#define ACS_RTC_CTRL                    ((ACS_RTC_CTRL_Type *) ACS_RTC_CTRL_BASE)

/* ACS_RTC_CTRL settings */
#define RTC_DISABLE_BITBAND             0x0
#define RTC_ENABLE_BITBAND              0x1
#define RTC_DISABLE                     ((uint32_t)(RTC_DISABLE_BITBAND << ACS_RTC_CTRL_ENABLE_Pos))
#define RTC_ENABLE                      ((uint32_t)(RTC_ENABLE_BITBAND << ACS_RTC_CTRL_ENABLE_Pos))

#define RTC_CLK_SRC_RC_OSC              ((uint32_t)(0x0U << ACS_RTC_CTRL_CLK_SRC_SEL_Pos))
#define RTC_CLK_SRC_XTAL32K             ((uint32_t)(0x1U << ACS_RTC_CTRL_CLK_SRC_SEL_Pos))
#define RTC_CLK_SRC_DIO0                ((uint32_t)(0x4U << ACS_RTC_CTRL_CLK_SRC_SEL_Pos))
#define RTC_CLK_SRC_DIO1                ((uint32_t)(0x5U << ACS_RTC_CTRL_CLK_SRC_SEL_Pos))
#define RTC_CLK_SRC_DIO2                ((uint32_t)(0x6U << ACS_RTC_CTRL_CLK_SRC_SEL_Pos))
#define RTC_CLK_SRC_DIO3                ((uint32_t)(0x7U << ACS_RTC_CTRL_CLK_SRC_SEL_Pos))

#define RTC_ALARM_DISABLE               ((uint32_t)(0x0U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_7P8125MS              ((uint32_t)(0x1U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_15P625MS              ((uint32_t)(0x2U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_31P25MS               ((uint32_t)(0x3U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_62P5MS                ((uint32_t)(0x4U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_125MS                 ((uint32_t)(0x5U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_250MS                 ((uint32_t)(0x6U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_500MS                 ((uint32_t)(0x7U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_1S                    ((uint32_t)(0x8U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_2S                    ((uint32_t)(0x9U << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_4S                    ((uint32_t)(0xAU << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_8S                    ((uint32_t)(0xBU << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_16S                   ((uint32_t)(0xCU << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_32S                   ((uint32_t)(0xDU << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_64S                   ((uint32_t)(0xEU << ACS_RTC_CTRL_ALARM_CFG_Pos))
#define RTC_ALARM_ZERO                  ((uint32_t)(0xFU << ACS_RTC_CTRL_ALARM_CFG_Pos))

#define RTC_RESET_BITBAND               0x1
#define RTC_RESET                       ((uint32_t)(RTC_RESET_BITBAND << ACS_RTC_CTRL_RESET_Pos))

#define RTC_FORCE_CLOCK_BITBAND         0x1
#define RTC_FORCE_CLOCK                 ((uint32_t)(RTC_FORCE_CLOCK_BITBAND << ACS_RTC_CTRL_FORCE_CLOCK_Pos))

/* ACS_PWR_MODES_CTRL bit positions */
#define ACS_PWR_MODES_CTRL_POWER_MODE_Pos 0
#define ACS_PWR_MODES_CTRL_POWER_MODE_Mask ((uint32_t)(0xFFFFFFFFU << ACS_PWR_MODES_CTRL_POWER_MODE_Pos))

/* ACS_PWR_MODES_CTRL settings */
#define PWR_RUN_MODE                    ((uint32_t)(0x0U << ACS_PWR_MODES_CTRL_POWER_MODE_Pos))
#define PWR_STANDBY_MODE                ((uint32_t)(0x9B1D79A0U << ACS_PWR_MODES_CTRL_POWER_MODE_Pos))
#define PWR_SLEEP_MODE                  ((uint32_t)(0xE0045650U << ACS_PWR_MODES_CTRL_POWER_MODE_Pos))

/* ACS_WAKEUP_CTRL bit positions */
#define ACS_WAKEUP_CTRL_PADS_RETENTION_EN_Pos 24
#define ACS_WAKEUP_CTRL_BOOT_FLASH_APP_REBOOT_Pos 20
#define ACS_WAKEUP_CTRL_RC_CLOCK_MULT_Pos 19
#define ACS_WAKEUP_CTRL_RC_FTRIM_FLAG_Pos 18
#define ACS_WAKEUP_CTRL_BOOT_SELECT_Pos 16
#define ACS_WAKEUP_CTRL_BOOT_SELECT_Mask ((uint32_t)(0x3U << ACS_WAKEUP_CTRL_BOOT_SELECT_Pos))
#define ACS_WAKEUP_CTRL_DCDC_OVERLOAD_WAKEUP_Pos 15
#define ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_Pos 14
#define ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_Pos 13
#define ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_Pos 12
#define ACS_WAKEUP_CTRL_DIO3_WAKEUP_Pos 11
#define ACS_WAKEUP_CTRL_DIO2_WAKEUP_Pos 10
#define ACS_WAKEUP_CTRL_DIO1_WAKEUP_Pos 9
#define ACS_WAKEUP_CTRL_DIO0_WAKEUP_Pos 8
#define ACS_WAKEUP_CTRL_DCDC_OVERLOAD_CLEAR_Pos 7
#define ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_CLEAR_Pos 6
#define ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_CLEAR_Pos 5
#define ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_CLEAR_Pos 4
#define ACS_WAKEUP_CTRL_DIO3_WAKEUP_CLEAR_Pos 3
#define ACS_WAKEUP_CTRL_DIO2_WAKEUP_CLEAR_Pos 2
#define ACS_WAKEUP_CTRL_DIO1_WAKEUP_CLEAR_Pos 1
#define ACS_WAKEUP_CTRL_DIO0_WAKEUP_CLEAR_Pos 0

/* ACS_WAKEUP_CTRL sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t PADS_RETENTION_EN_BYTE;
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001344, 0) - (0x40001344 + 4)];
    __O  uint32_t DIO0_WAKEUP_CLEAR_ALIAS;
    __O  uint32_t DIO1_WAKEUP_CLEAR_ALIAS;
    __O  uint32_t DIO2_WAKEUP_CLEAR_ALIAS;
    __O  uint32_t DIO3_WAKEUP_CLEAR_ALIAS;
    __O  uint32_t BB_TIMER_WAKEUP_CLEAR_ALIAS;
    __O  uint32_t RTC_ALARM_WAKEUP_CLEAR_ALIAS;
    __O  uint32_t WAKEUP_PAD_WAKEUP_CLEAR_ALIAS;
    __O  uint32_t DCDC_OVERLOAD_CLEAR_ALIAS;
    __I  uint32_t DIO0_WAKEUP_ALIAS;    
    __I  uint32_t DIO1_WAKEUP_ALIAS;    
    __I  uint32_t DIO2_WAKEUP_ALIAS;    
    __I  uint32_t DIO3_WAKEUP_ALIAS;    
    __I  uint32_t BB_TIMER_WAKEUP_ALIAS;
    __I  uint32_t RTC_ALARM_WAKEUP_ALIAS;
    __I  uint32_t WAKEUP_PAD_WAKEUP_ALIAS;
    __I  uint32_t DCDC_OVERLOAD_WAKEUP_ALIAS;
         uint32_t RESERVED1[2];
    __I  uint32_t RC_FTRIM_FLAG_ALIAS;  /* RC oscillator trimming read only flag (mirror of FTRIM_FLAG of ACS_RCOSC_CTRL register */
    __I  uint32_t RC_CLOCK_MULT_ALIAS;  /* RC oscillator clock multiplier read only flag (mirror of CLOCK_MULT of ACS_RCOSC_CTRL register) */
    __IO uint32_t BOOT_FLASH_APP_REBOOT_ALIAS;/* Boot mode flag */
         uint32_t RESERVED2[3];
    __IO uint32_t PADS_RETENTION_EN_ALIAS;/* Enable / Disable the retention mode of the pads */
} ACS_WAKEUP_CTRL_Type;

#define ACS_WAKEUP_CTRL_BASE            0x40001344
#define ACS_WAKEUP_CTRL                 ((ACS_WAKEUP_CTRL_Type *) ACS_WAKEUP_CTRL_BASE)

/* ACS_WAKEUP_CTRL settings */
#define WAKEUP_DIO0_EVENT_CLEAR_BITBAND 0x1
#define WAKEUP_DIO0_EVENT_CLEAR         ((uint32_t)(WAKEUP_DIO0_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DIO0_WAKEUP_CLEAR_Pos))

#define WAKEUP_DIO1_EVENT_CLEAR_BITBAND 0x1
#define WAKEUP_DIO1_EVENT_CLEAR         ((uint32_t)(WAKEUP_DIO1_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DIO1_WAKEUP_CLEAR_Pos))

#define WAKEUP_DIO2_EVENT_CLEAR_BITBAND 0x1
#define WAKEUP_DIO2_EVENT_CLEAR         ((uint32_t)(WAKEUP_DIO2_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DIO2_WAKEUP_CLEAR_Pos))

#define WAKEUP_DIO3_EVENT_CLEAR_BITBAND 0x1
#define WAKEUP_DIO3_EVENT_CLEAR         ((uint32_t)(WAKEUP_DIO3_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DIO3_WAKEUP_CLEAR_Pos))

#define WAKEUP_BB_TIMER_CLEAR_BITBAND   0x1
#define WAKEUP_BB_TIMER_CLEAR           ((uint32_t)(WAKEUP_BB_TIMER_CLEAR_BITBAND << ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_CLEAR_Pos))

#define WAKEUP_RTC_ALARM_CLEAR_BITBAND  0x1
#define WAKEUP_RTC_ALARM_CLEAR          ((uint32_t)(WAKEUP_RTC_ALARM_CLEAR_BITBAND << ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_CLEAR_Pos))

#define WAKEUP_PAD_EVENT_CLEAR_BITBAND  0x1
#define WAKEUP_PAD_EVENT_CLEAR          ((uint32_t)(WAKEUP_PAD_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_CLEAR_Pos))

#define WAKEUP_DCDC_OVERLOAD_CLEAR_BITBAND 0x1
#define WAKEUP_DCDC_OVERLOAD_CLEAR      ((uint32_t)(WAKEUP_DCDC_OVERLOAD_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DCDC_OVERLOAD_CLEAR_Pos))

#define WAKEUP_DIO0_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_DIO0_EVENT_SET_BITBAND   0x1
#define WAKEUP_DIO0_EVENT_NOT_SET       ((uint32_t)(WAKEUP_DIO0_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO0_WAKEUP_Pos))
#define WAKEUP_DIO0_EVENT_SET           ((uint32_t)(WAKEUP_DIO0_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO0_WAKEUP_Pos))

#define WAKEUP_DIO1_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_DIO1_EVENT_SET_BITBAND   0x1
#define WAKEUP_DIO1_EVENT_NOT_SET       ((uint32_t)(WAKEUP_DIO1_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO1_WAKEUP_Pos))
#define WAKEUP_DIO1_EVENT_SET           ((uint32_t)(WAKEUP_DIO1_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO1_WAKEUP_Pos))

#define WAKEUP_DIO2_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_DIO2_EVENT_SET_BITBAND   0x1
#define WAKEUP_DIO2_EVENT_NOT_SET       ((uint32_t)(WAKEUP_DIO2_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO2_WAKEUP_Pos))
#define WAKEUP_DIO2_EVENT_SET           ((uint32_t)(WAKEUP_DIO2_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO2_WAKEUP_Pos))

#define WAKEUP_DIO3_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_DIO3_EVENT_SET_BITBAND   0x1
#define WAKEUP_DIO3_EVENT_NOT_SET       ((uint32_t)(WAKEUP_DIO3_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO3_WAKEUP_Pos))
#define WAKEUP_DIO3_EVENT_SET           ((uint32_t)(WAKEUP_DIO3_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO3_WAKEUP_Pos))

#define WAKEUP_BB_TIMER_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_BB_TIMER_EVENT_SET_BITBAND 0x1
#define WAKEUP_BB_TIMER_EVENT_NOT_SET   ((uint32_t)(WAKEUP_BB_TIMER_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_Pos))
#define WAKEUP_BB_TIMER_EVENT_SET       ((uint32_t)(WAKEUP_BB_TIMER_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_Pos))

#define WAKEUP_RTC_ALARM_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_RTC_ALARM_EVENT_SET_BITBAND 0x1
#define WAKEUP_RTC_ALARM_EVENT_NOT_SET  ((uint32_t)(WAKEUP_RTC_ALARM_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_Pos))
#define WAKEUP_RTC_ALARM_EVENT_SET      ((uint32_t)(WAKEUP_RTC_ALARM_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_Pos))

#define WAKEUP_PAD_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_PAD_EVENT_SET_BITBAND    0x1
#define WAKEUP_PAD_EVENT_NOT_SET        ((uint32_t)(WAKEUP_PAD_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_Pos))
#define WAKEUP_PAD_EVENT_SET            ((uint32_t)(WAKEUP_PAD_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_Pos))

#define WAKEUP_DCDC_OVERLOAD_NOT_SET_BITBAND 0x0
#define WAKEUP_DCDC_OVERLOAD_SET_BITBAND 0x1
#define WAKEUP_DCDC_OVERLOAD_NOT_SET    ((uint32_t)(WAKEUP_DCDC_OVERLOAD_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DCDC_OVERLOAD_WAKEUP_Pos))
#define WAKEUP_DCDC_OVERLOAD_SET        ((uint32_t)(WAKEUP_DCDC_OVERLOAD_SET_BITBAND << ACS_WAKEUP_CTRL_DCDC_OVERLOAD_WAKEUP_Pos))

#define BOOT_FLASH_XTAL_DISABLE         ((uint32_t)(0x0U << ACS_WAKEUP_CTRL_BOOT_SELECT_Pos))
#define BOOT_CUSTOM                     ((uint32_t)(0x1U << ACS_WAKEUP_CTRL_BOOT_SELECT_Pos))
#define BOOT_FLASH_XTAL_DEFAULT_TRIM    ((uint32_t)(0x2U << ACS_WAKEUP_CTRL_BOOT_SELECT_Pos))
#define BOOT_FLASH_XTAL_CUSTOM_TRIM     ((uint32_t)(0x3U << ACS_WAKEUP_CTRL_BOOT_SELECT_Pos))

#define RC_OSC_STATUS_UNCALIBRATED_BITBAND 0x0
#define RC_OSC_STATUS_CALIBRATED_BITBAND 0x1
#define RC_OSC_STATUS_UNCALIBRATED      ((uint32_t)(RC_OSC_STATUS_UNCALIBRATED_BITBAND << ACS_WAKEUP_CTRL_RC_FTRIM_FLAG_Pos))
#define RC_OSC_STATUS_CALIBRATED        ((uint32_t)(RC_OSC_STATUS_CALIBRATED_BITBAND << ACS_WAKEUP_CTRL_RC_FTRIM_FLAG_Pos))

#define RC_START_OSC_STATUS_3MHZ_BITBAND 0x0
#define RC_START_OSC_STATUS_12MHZ_BITBAND 0x1
#define RC_START_OSC_STATUS_3MHZ        ((uint32_t)(RC_START_OSC_STATUS_3MHZ_BITBAND << ACS_WAKEUP_CTRL_RC_CLOCK_MULT_Pos))
#define RC_START_OSC_STATUS_12MHZ       ((uint32_t)(RC_START_OSC_STATUS_12MHZ_BITBAND << ACS_WAKEUP_CTRL_RC_CLOCK_MULT_Pos))

#define BOOT_FLASH_APP_REBOOT_DISABLE_BITBAND 0x0
#define BOOT_FLASH_APP_REBOOT_ENABLE_BITBAND 0x1
#define BOOT_FLASH_APP_REBOOT_DISABLE   ((uint32_t)(BOOT_FLASH_APP_REBOOT_DISABLE_BITBAND << ACS_WAKEUP_CTRL_BOOT_FLASH_APP_REBOOT_Pos))
#define BOOT_FLASH_APP_REBOOT_ENABLE    ((uint32_t)(BOOT_FLASH_APP_REBOOT_ENABLE_BITBAND << ACS_WAKEUP_CTRL_BOOT_FLASH_APP_REBOOT_Pos))

#define PADS_RETENTION_DISABLE_BITBAND  0x0
#define PADS_RETENTION_ENABLE_BITBAND   0x1
#define PADS_RETENTION_DISABLE          ((uint32_t)(PADS_RETENTION_DISABLE_BITBAND << ACS_WAKEUP_CTRL_PADS_RETENTION_EN_Pos))
#define PADS_RETENTION_ENABLE           ((uint32_t)(PADS_RETENTION_ENABLE_BITBAND << ACS_WAKEUP_CTRL_PADS_RETENTION_EN_Pos))

/* ACS_WAKEUP_CTRL sub-register bit positions */
#define ACS_WAKEUP_CTRL_PADS_RETENTION_EN_BYTE_Pos 0

/* ACS_WAKEUP_CTRL sub-register settings */
#define PADS_RETENTION_DISABLE_BYTE     ((uint8_t)(PADS_RETENTION_DISABLE_BITBAND << ACS_WAKEUP_CTRL_PADS_RETENTION_EN_BYTE_Pos))
#define PADS_RETENTION_ENABLE_BYTE      ((uint8_t)(PADS_RETENTION_ENABLE_BITBAND << ACS_WAKEUP_CTRL_PADS_RETENTION_EN_BYTE_Pos))

/* ACS_WAKEUP_CFG bit positions */
#define ACS_WAKEUP_CFG_DELAY_Pos        16
#define ACS_WAKEUP_CFG_DELAY_Mask       ((uint32_t)(0x7U << ACS_WAKEUP_CFG_DELAY_Pos))
#define ACS_WAKEUP_CFG_DCDC_OVERLOAD_EN_Pos 9
#define ACS_WAKEUP_CFG_WAKEUP_PAD_POL_Pos 8
#define ACS_WAKEUP_CFG_DIO3_POL_Pos     7
#define ACS_WAKEUP_CFG_DIO2_POL_Pos     6
#define ACS_WAKEUP_CFG_DIO1_POL_Pos     5
#define ACS_WAKEUP_CFG_DIO0_POL_Pos     4
#define ACS_WAKEUP_CFG_DIO3_EN_Pos      3
#define ACS_WAKEUP_CFG_DIO2_EN_Pos      2
#define ACS_WAKEUP_CFG_DIO1_EN_Pos      1
#define ACS_WAKEUP_CFG_DIO0_EN_Pos      0

/* ACS_WAKEUP_CFG sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[2];
    __IO uint8_t DELAY_BYTE;           
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001348, 0) - (0x40001348 + 4)];
    __IO uint32_t DIO0_EN_ALIAS;        /* Enable / Disable the Wake-up functionality on the DIO0 pad */
    __IO uint32_t DIO1_EN_ALIAS;        /* Enable / Disable the Wake-up functionality on the DIO1 pad */
    __IO uint32_t DIO2_EN_ALIAS;        /* Enable / Disable the Wake-up functionality on the DIO2 pad */
    __IO uint32_t DIO3_EN_ALIAS;        /* Enable / Disable the Wake-up functionality on the DIO3 pad */
    __IO uint32_t DIO0_POL_ALIAS;       /* Wake-up polarity on the DIO0 pad */
    __IO uint32_t DIO1_POL_ALIAS;       /* Wake-up polarity on the DIO1 pad */
    __IO uint32_t DIO2_POL_ALIAS;       /* Wake-up polarity on the DIO2 pad */
    __IO uint32_t DIO3_POL_ALIAS;       /* Wake-up polarity on the DIO3 pad */
    __IO uint32_t WAKEUP_PAD_POL_ALIAS; /* Wake-up polarity on the WAKEUP pad */
    __IO uint32_t DCDC_OVERLOAD_EN_ALIAS;/* Enable / Disable the Wake-up functionality on the DCDC overload flag */
} ACS_WAKEUP_CFG_Type;

#define ACS_WAKEUP_CFG_BASE             0x40001348
#define ACS_WAKEUP_CFG                  ((ACS_WAKEUP_CFG_Type *) ACS_WAKEUP_CFG_BASE)

/* ACS_WAKEUP_CFG settings */
#define WAKEUP_DIO0_DISABLE_BITBAND     0x0
#define WAKEUP_DIO0_ENABLE_BITBAND      0x1
#define WAKEUP_DIO0_DISABLE             ((uint32_t)(WAKEUP_DIO0_DISABLE_BITBAND << ACS_WAKEUP_CFG_DIO0_EN_Pos))
#define WAKEUP_DIO0_ENABLE              ((uint32_t)(WAKEUP_DIO0_ENABLE_BITBAND << ACS_WAKEUP_CFG_DIO0_EN_Pos))

#define WAKEUP_DIO1_DISABLE_BITBAND     0x0
#define WAKEUP_DIO1_ENABLE_BITBAND      0x1
#define WAKEUP_DIO1_DISABLE             ((uint32_t)(WAKEUP_DIO1_DISABLE_BITBAND << ACS_WAKEUP_CFG_DIO1_EN_Pos))
#define WAKEUP_DIO1_ENABLE              ((uint32_t)(WAKEUP_DIO1_ENABLE_BITBAND << ACS_WAKEUP_CFG_DIO1_EN_Pos))

#define WAKEUP_DIO2_DISABLE_BITBAND     0x0
#define WAKEUP_DIO2_ENABLE_BITBAND      0x1
#define WAKEUP_DIO2_DISABLE             ((uint32_t)(WAKEUP_DIO2_DISABLE_BITBAND << ACS_WAKEUP_CFG_DIO2_EN_Pos))
#define WAKEUP_DIO2_ENABLE              ((uint32_t)(WAKEUP_DIO2_ENABLE_BITBAND << ACS_WAKEUP_CFG_DIO2_EN_Pos))

#define WAKEUP_DIO3_DISABLE_BITBAND     0x0
#define WAKEUP_DIO3_ENABLE_BITBAND      0x1
#define WAKEUP_DIO3_DISABLE             ((uint32_t)(WAKEUP_DIO3_DISABLE_BITBAND << ACS_WAKEUP_CFG_DIO3_EN_Pos))
#define WAKEUP_DIO3_ENABLE              ((uint32_t)(WAKEUP_DIO3_ENABLE_BITBAND << ACS_WAKEUP_CFG_DIO3_EN_Pos))

#define WAKEUP_DIO0_RISING_BITBAND      0x0
#define WAKEUP_DIO0_FALLING_BITBAND     0x1
#define WAKEUP_DIO0_RISING              ((uint32_t)(WAKEUP_DIO0_RISING_BITBAND << ACS_WAKEUP_CFG_DIO0_POL_Pos))
#define WAKEUP_DIO0_FALLING             ((uint32_t)(WAKEUP_DIO0_FALLING_BITBAND << ACS_WAKEUP_CFG_DIO0_POL_Pos))

#define WAKEUP_DIO1_RISING_BITBAND      0x0
#define WAKEUP_DIO1_FALLING_BITBAND     0x1
#define WAKEUP_DIO1_RISING              ((uint32_t)(WAKEUP_DIO1_RISING_BITBAND << ACS_WAKEUP_CFG_DIO1_POL_Pos))
#define WAKEUP_DIO1_FALLING             ((uint32_t)(WAKEUP_DIO1_FALLING_BITBAND << ACS_WAKEUP_CFG_DIO1_POL_Pos))

#define WAKEUP_DIO2_RISING_BITBAND      0x0
#define WAKEUP_DIO2_FALLING_BITBAND     0x1
#define WAKEUP_DIO2_RISING              ((uint32_t)(WAKEUP_DIO2_RISING_BITBAND << ACS_WAKEUP_CFG_DIO2_POL_Pos))
#define WAKEUP_DIO2_FALLING             ((uint32_t)(WAKEUP_DIO2_FALLING_BITBAND << ACS_WAKEUP_CFG_DIO2_POL_Pos))

#define WAKEUP_DIO3_RISING_BITBAND      0x0
#define WAKEUP_DIO3_FALLING_BITBAND     0x1
#define WAKEUP_DIO3_RISING              ((uint32_t)(WAKEUP_DIO3_RISING_BITBAND << ACS_WAKEUP_CFG_DIO3_POL_Pos))
#define WAKEUP_DIO3_FALLING             ((uint32_t)(WAKEUP_DIO3_FALLING_BITBAND << ACS_WAKEUP_CFG_DIO3_POL_Pos))

#define WAKEUP_WAKEUP_PAD_RISING_BITBAND 0x0
#define WAKEUP_WAKEUP_PAD_FALLING_BITBAND 0x1
#define WAKEUP_WAKEUP_PAD_RISING        ((uint32_t)(WAKEUP_WAKEUP_PAD_RISING_BITBAND << ACS_WAKEUP_CFG_WAKEUP_PAD_POL_Pos))
#define WAKEUP_WAKEUP_PAD_FALLING       ((uint32_t)(WAKEUP_WAKEUP_PAD_FALLING_BITBAND << ACS_WAKEUP_CFG_WAKEUP_PAD_POL_Pos))

#define WAKEUP_DCDC_OVERLOAD_DISABLE_BITBAND 0x0
#define WAKEUP_DCDC_OVERLOAD_ENABLE_BITBAND 0x1
#define WAKEUP_DCDC_OVERLOAD_DISABLE    ((uint32_t)(WAKEUP_DCDC_OVERLOAD_DISABLE_BITBAND << ACS_WAKEUP_CFG_DCDC_OVERLOAD_EN_Pos))
#define WAKEUP_DCDC_OVERLOAD_ENABLE     ((uint32_t)(WAKEUP_DCDC_OVERLOAD_ENABLE_BITBAND << ACS_WAKEUP_CFG_DCDC_OVERLOAD_EN_Pos))

#define WAKEUP_DELAY_1                  ((uint32_t)(0x0U << ACS_WAKEUP_CFG_DELAY_Pos))
#define WAKEUP_DELAY_2                  ((uint32_t)(0x1U << ACS_WAKEUP_CFG_DELAY_Pos))
#define WAKEUP_DELAY_4                  ((uint32_t)(0x2U << ACS_WAKEUP_CFG_DELAY_Pos))
#define WAKEUP_DELAY_8                  ((uint32_t)(0x3U << ACS_WAKEUP_CFG_DELAY_Pos))
#define WAKEUP_DELAY_16                 ((uint32_t)(0x4U << ACS_WAKEUP_CFG_DELAY_Pos))
#define WAKEUP_DELAY_32                 ((uint32_t)(0x5U << ACS_WAKEUP_CFG_DELAY_Pos))
#define WAKEUP_DELAY_64                 ((uint32_t)(0x6U << ACS_WAKEUP_CFG_DELAY_Pos))
#define WAKEUP_DELAY_128                ((uint32_t)(0x7U << ACS_WAKEUP_CFG_DELAY_Pos))

/* ACS_WAKEUP_CFG sub-register bit positions */
#define ACS_WAKEUP_CFG_DELAY_BYTE_Pos   0
#define ACS_WAKEUP_CFG_DELAY_BYTE_Mask  ((uint32_t)(0x7U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))

/* ACS_WAKEUP_CFG sub-register settings */
#define WAKEUP_DELAY_1_BYTE             ((uint8_t)(0x0U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))
#define WAKEUP_DELAY_2_BYTE             ((uint8_t)(0x1U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))
#define WAKEUP_DELAY_4_BYTE             ((uint8_t)(0x2U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))
#define WAKEUP_DELAY_8_BYTE             ((uint8_t)(0x3U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))
#define WAKEUP_DELAY_16_BYTE            ((uint8_t)(0x4U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))
#define WAKEUP_DELAY_32_BYTE            ((uint8_t)(0x5U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))
#define WAKEUP_DELAY_64_BYTE            ((uint8_t)(0x6U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))
#define WAKEUP_DELAY_128_BYTE           ((uint8_t)(0x7U << ACS_WAKEUP_CFG_DELAY_BYTE_Pos))

/* ACS_WAKEUP_STATE bit positions */
#define ACS_WAKEUP_STATE_WAKEUP_SRC_Pos 16
#define ACS_WAKEUP_STATE_WAKEUP_SRC_Mask ((uint32_t)(0x7U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))
#define ACS_WAKEUP_STATE_RTC_VALUE_Pos  0
#define ACS_WAKEUP_STATE_RTC_VALUE_Mask ((uint32_t)(0xFFU << ACS_WAKEUP_STATE_RTC_VALUE_Pos))

/* ACS_WAKEUP_STATE sub-register and bit-band aliases */
typedef struct
{
    __I  uint8_t RTC_VALUE_BYTE;       
         uint8_t RESERVED0[1];
    __I  uint8_t WAKEUP_SRC_BYTE;      
         uint8_t RESERVED1[1];
} ACS_WAKEUP_STATE_Type;

#define ACS_WAKEUP_STATE_BASE           0x4000134C
#define ACS_WAKEUP_STATE                ((ACS_WAKEUP_STATE_Type *) ACS_WAKEUP_STATE_BASE)

/* ACS_WAKEUP_STATE settings */
#define WAKEUP_DUE_TO_DCDC_OVERLOAD     ((uint32_t)(0x7U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))
#define WAKEUP_DUE_TO_WAKEUP_PAD        ((uint32_t)(0x6U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))
#define WAKEUP_DUE_TO_RTC_ALARM         ((uint32_t)(0x5U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))
#define WAKEUP_DUE_TO_BB_TIMER          ((uint32_t)(0x4U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))
#define WAKEUP_DUE_TO_DIO3              ((uint32_t)(0x3U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))
#define WAKEUP_DUE_TO_DIO2              ((uint32_t)(0x2U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))
#define WAKEUP_DUE_TO_DIO1              ((uint32_t)(0x1U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))
#define WAKEUP_DUE_TO_DIO0              ((uint32_t)(0x0U << ACS_WAKEUP_STATE_WAKEUP_SRC_Pos))

/* ACS_WAKEUP_STATE sub-register bit positions */
#define ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos 0
#define ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Mask ((uint32_t)(0x7U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))

/* ACS_WAKEUP_STATE sub-register settings */
#define WAKEUP_DUE_TO_DCDC_OVERLOAD_BYTE ((uint8_t)(0x7U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))
#define WAKEUP_DUE_TO_WAKEUP_PAD_BYTE   ((uint8_t)(0x6U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))
#define WAKEUP_DUE_TO_RTC_ALARM_BYTE    ((uint8_t)(0x5U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))
#define WAKEUP_DUE_TO_BB_TIMER_BYTE     ((uint8_t)(0x4U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))
#define WAKEUP_DUE_TO_DIO3_BYTE         ((uint8_t)(0x3U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))
#define WAKEUP_DUE_TO_DIO2_BYTE         ((uint8_t)(0x2U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))
#define WAKEUP_DUE_TO_DIO1_BYTE         ((uint8_t)(0x1U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))
#define WAKEUP_DUE_TO_DIO0_BYTE         ((uint8_t)(0x0U << ACS_WAKEUP_STATE_WAKEUP_SRC_BYTE_Pos))

/* ACS_WAKEUP_GP_DATA bit positions */
#define ACS_WAKEUP_GP_DATA_GP_DATA_Pos  0
#define ACS_WAKEUP_GP_DATA_GP_DATA_Mask ((uint32_t)(0xFFFFFFFFU << ACS_WAKEUP_GP_DATA_GP_DATA_Pos))

/* ACS_RESET_STATUS bit positions */
#define ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_Pos 14
#define ACS_RESET_STATUS_CLK_DET_RESET_FLAG_Pos 13
#define ACS_RESET_STATUS_VDDA_RESET_FLAG_Pos 12
#define ACS_RESET_STATUS_VDDM_RESET_FLAG_Pos 11
#define ACS_RESET_STATUS_VDDC_RESET_FLAG_Pos 10
#define ACS_RESET_STATUS_PAD_RESET_FLAG_Pos 9
#define ACS_RESET_STATUS_POR_RESET_FLAG_Pos 8
#define ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_CLEAR_Pos 6
#define ACS_RESET_STATUS_CLK_DET_RESET_FLAG_CLEAR_Pos 5
#define ACS_RESET_STATUS_VDDA_RESET_FLAG_CLEAR_Pos 4
#define ACS_RESET_STATUS_VDDM_RESET_FLAG_CLEAR_Pos 3
#define ACS_RESET_STATUS_VDDC_RESET_FLAG_CLEAR_Pos 2
#define ACS_RESET_STATUS_PAD_RESET_FLAG_CLEAR_Pos 1
#define ACS_RESET_STATUS_POR_RESET_FLAG_CLEAR_Pos 0

/* ACS_RESET_STATUS bit-band aliases */
typedef struct
{
    __O  uint32_t POR_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky POR_RESET flag. */
    __O  uint32_t PAD_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky PAD_RESET flag. */
    __O  uint32_t VDDC_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky VDDC_RESET flag. */
    __O  uint32_t VDDM_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky VDDM_RESET flag. */
    __O  uint32_t VDDA_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky VDDA_RESET flag. */
    __O  uint32_t CLK_DET_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky CLK_DET_RESET flag. */
    __O  uint32_t TIMEOUT_RESET_FLAG_CLEAR_ALIAS;/* Reset the sticky TIMEOUT_RESET flag. */
         uint32_t RESERVED0;
    __I  uint32_t POR_RESET_FLAG_ALIAS; /* Sticky flag that detects that a POR reset occurred */
    __I  uint32_t PAD_RESET_FLAG_ALIAS; /* Sticky flag that detects that a reset occurred due to pad NRESET */
    __I  uint32_t VDDC_RESET_FLAG_ALIAS;/* Sticky flag that detects that a VDDC reset occurred (triggered by vddc_ready = 0) */
    __I  uint32_t VDDM_RESET_FLAG_ALIAS;/* Sticky flag that detects that a VDDM reset occurred (triggered by vddm_ready = 0) */
    __I  uint32_t VDDA_RESET_FLAG_ALIAS;/* Sticky flag that detects that a VDDA reset occurred (triggered by vdda_ready = 0) */
    __I  uint32_t CLK_DET_RESET_FLAG_ALIAS;/* Sticky flag that detects that a clock detector reset occurred */
    __I  uint32_t TIMEOUT_RESET_FLAG_ALIAS;/* Sticky flag that detects that a timeout in the power up sequence */
} ACS_RESET_STATUS_Type;

#define ACS_RESET_STATUS_BASE           SYS_CALC_BITBAND(0x40001354, 0)
#define ACS_RESET_STATUS                ((ACS_RESET_STATUS_Type *) ACS_RESET_STATUS_BASE)

/* ACS_RESET_STATUS settings */
#define POR_RESET_FLAG_CLEAR_BITBAND    0x1
#define POR_RESET_FLAG_CLEAR            ((uint32_t)(POR_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_POR_RESET_FLAG_CLEAR_Pos))

#define PAD_RESET_FLAG_CLEAR_BITBAND    0x1
#define PAD_RESET_FLAG_CLEAR            ((uint32_t)(PAD_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_PAD_RESET_FLAG_CLEAR_Pos))

#define VDDC_RESET_FLAG_CLEAR_BITBAND   0x1
#define VDDC_RESET_FLAG_CLEAR           ((uint32_t)(VDDC_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_VDDC_RESET_FLAG_CLEAR_Pos))

#define VDDM_RESET_FLAG_CLEAR_BITBAND   0x1
#define VDDM_RESET_FLAG_CLEAR           ((uint32_t)(VDDM_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_VDDM_RESET_FLAG_CLEAR_Pos))

#define VDDA_RESET_FLAG_CLEAR_BITBAND   0x1
#define VDDA_RESET_FLAG_CLEAR           ((uint32_t)(VDDA_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_VDDA_RESET_FLAG_CLEAR_Pos))

#define CLK_DET_RESET_FLAG_CLEAR_BITBAND 0x1
#define CLK_DET_RESET_FLAG_CLEAR        ((uint32_t)(CLK_DET_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_CLK_DET_RESET_FLAG_CLEAR_Pos))

#define TIMEOUT_RESET_FLAG_CLEAR_BITBAND 0x1
#define TIMEOUT_RESET_FLAG_CLEAR        ((uint32_t)(TIMEOUT_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_CLEAR_Pos))

#define POR_RESET_FLAG_NOT_SET_BITBAND  0x0
#define POR_RESET_FLAG_SET_BITBAND      0x1
#define POR_RESET_FLAG_NOT_SET          ((uint32_t)(POR_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_POR_RESET_FLAG_Pos))
#define POR_RESET_FLAG_SET              ((uint32_t)(POR_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_POR_RESET_FLAG_Pos))

#define PAD_RESET_FLAG_NOT_SET_BITBAND  0x0
#define PAD_RESET_FLAG_SET_BITBAND      0x1
#define PAD_RESET_FLAG_NOT_SET          ((uint32_t)(PAD_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_PAD_RESET_FLAG_Pos))
#define PAD_RESET_FLAG_SET              ((uint32_t)(PAD_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_PAD_RESET_FLAG_Pos))

#define VDDC_RESET_FLAG_NOT_SET_BITBAND 0x0
#define VDDC_RESET_FLAG_SET_BITBAND     0x1
#define VDDC_RESET_FLAG_NOT_SET         ((uint32_t)(VDDC_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_VDDC_RESET_FLAG_Pos))
#define VDDC_RESET_FLAG_SET             ((uint32_t)(VDDC_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_VDDC_RESET_FLAG_Pos))

#define VDDM_RESET_FLAG_NOT_SET_BITBAND 0x0
#define VDDM_RESET_FLAG_SET_BITBAND     0x1
#define VDDM_RESET_FLAG_NOT_SET         ((uint32_t)(VDDM_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_VDDM_RESET_FLAG_Pos))
#define VDDM_RESET_FLAG_SET             ((uint32_t)(VDDM_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_VDDM_RESET_FLAG_Pos))

#define VDDA_RESET_FLAG_NOT_SET_BITBAND 0x0
#define VDDA_RESET_FLAG_SET_BITBAND     0x1
#define VDDA_RESET_FLAG_NOT_SET         ((uint32_t)(VDDA_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_VDDA_RESET_FLAG_Pos))
#define VDDA_RESET_FLAG_SET             ((uint32_t)(VDDA_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_VDDA_RESET_FLAG_Pos))

#define CLK_DET_RESET_FLAG_NOT_SET_BITBAND 0x0
#define CLK_DET_RESET_FLAG_SET_BITBAND  0x1
#define CLK_DET_RESET_FLAG_NOT_SET      ((uint32_t)(CLK_DET_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_CLK_DET_RESET_FLAG_Pos))
#define CLK_DET_RESET_FLAG_SET          ((uint32_t)(CLK_DET_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_CLK_DET_RESET_FLAG_Pos))

#define TIMEOUT_RESET_FLAG_NOT_SET_BITBAND 0x0
#define TIMEOUT_RESET_FLAG_SET_BITBAND  0x1
#define TIMEOUT_RESET_FLAG_NOT_SET      ((uint32_t)(TIMEOUT_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_Pos))
#define TIMEOUT_RESET_FLAG_SET          ((uint32_t)(TIMEOUT_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_Pos))

/* ACS_AOUT_CTRL bit positions */
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_Pos 13
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_Pos 11
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_Mask ((uint32_t)(0x3U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_Pos))
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos 8
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Mask ((uint32_t)(0x7U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))
#define ACS_AOUT_CTRL_TEST_AOUT_Pos     0
#define ACS_AOUT_CTRL_TEST_AOUT_Mask    ((uint32_t)(0x1FU << ACS_AOUT_CTRL_TEST_AOUT_Pos))

/* ACS_AOUT_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TEST_AOUT_BYTE;       
    __IO uint8_t RTC_CLOCK_DIO0_BYTE;  
         uint8_t RESERVED0[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001358, 0) - (0x40001358 + 4)];
         uint32_t RESERVED1[13];
    __IO uint32_t RTC_CLOCK_DIO0_STOP_EDGE_ALIAS;/* Stop edge for RTC clock output on AOUT */
} ACS_AOUT_CTRL_Type;

#define ACS_AOUT_CTRL_BASE              0x40001358
#define ACS_AOUT_CTRL                   ((ACS_AOUT_CTRL_Type *) ACS_AOUT_CTRL_BASE)

/* ACS_AOUT_CTRL settings */
#define AOUT_VSSA                       ((uint32_t)(0x0U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VCC_SENSE                  ((uint32_t)(0x1U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VREF_0P75V_OUTPUT          ((uint32_t)(0x2U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VREF_0P67V_OUTPUT          ((uint32_t)(0x3U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_IREF_50N_OUTPUT            ((uint32_t)(0x4U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_IREF_1N_OUTPUT             ((uint32_t)(0x5U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDACS_OUTPUT              ((uint32_t)(0x6U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VREF_0P75V_BUF_OUTPUT      ((uint32_t)(0x7U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VREG_BG                    ((uint32_t)(0x8U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDRF_SW                   ((uint32_t)(0x9U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDRF_SENSE                ((uint32_t)(0xAU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDT                       ((uint32_t)(0xBU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDC_SENSE                 ((uint32_t)(0xCU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDA_SW                    ((uint32_t)(0xDU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDA_SENSE                 ((uint32_t)(0xEU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDM_SENSE                 ((uint32_t)(0xFU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_NC                         ((uint32_t)(0x10U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDPA_SENSE                ((uint32_t)(0x11U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDPA_ISENSE               ((uint32_t)(0x12U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_TM0                        ((uint32_t)(0x13U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_BG_READY                   ((uint32_t)(0x14U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VCC_READY                  ((uint32_t)(0x15U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_DCDC_OVERLOAD              ((uint32_t)(0x16U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_DCDC_ACTIVATED             ((uint32_t)(0x17U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDRF_READY                ((uint32_t)(0x18U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDC_READY                 ((uint32_t)(0x19U << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDM_READY                 ((uint32_t)(0x1AU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_VDDA_READY                 ((uint32_t)(0x1BU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_CLK_PRESENT                ((uint32_t)(0x1CU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_XTAL_OK                    ((uint32_t)(0x1DU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_XTAL_CLK                   ((uint32_t)(0x1EU << ACS_AOUT_CTRL_TEST_AOUT_Pos))
#define AOUT_CLK_32K                    ((uint32_t)(0x1FU << ACS_AOUT_CTRL_TEST_AOUT_Pos))

#define DIO0_RTC_CLK_DISABLE            ((uint32_t)(0x0U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))
#define DIO0_RTC_CLK_125MS              ((uint32_t)(0x1U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))
#define DIO0_RTC_CLK_250MS              ((uint32_t)(0x2U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))
#define DIO0_RTC_CLK_500MS              ((uint32_t)(0x3U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))
#define DIO0_RTC_CLK_1S                 ((uint32_t)(0x4U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))
#define DIO0_RTC_CLK_2S                 ((uint32_t)(0x5U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))
#define DIO0_RTC_CLK_4S                 ((uint32_t)(0x6U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))
#define DIO0_RTC_CLK_8S                 ((uint32_t)(0x7U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_Pos))

#define DIO0_RTC_CLK_STOP_DIO0          ((uint32_t)(0x0U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_Pos))
#define DIO0_RTC_CLK_STOP_DIO1          ((uint32_t)(0x1U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_Pos))
#define DIO0_RTC_CLK_STOP_DIO2          ((uint32_t)(0x2U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_Pos))
#define DIO0_RTC_CLK_STOP_DIO3          ((uint32_t)(0x3U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_Pos))

#define DIO0_RTC_CLK_STOP_RISING_BITBAND 0x0
#define DIO0_RTC_CLK_STOP_FALLING_BITBAND 0x1
#define DIO0_RTC_CLK_STOP_RISING        ((uint32_t)(DIO0_RTC_CLK_STOP_RISING_BITBAND << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_Pos))
#define DIO0_RTC_CLK_STOP_FALLING       ((uint32_t)(DIO0_RTC_CLK_STOP_FALLING_BITBAND << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_Pos))

/* ACS_AOUT_CTRL sub-register bit positions */
#define ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos 0
#define ACS_AOUT_CTRL_TEST_AOUT_BYTE_Mask ((uint32_t)(0x1FU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos 0
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Mask ((uint32_t)(0x7U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_BYTE_Pos 3
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_BYTE_Mask ((uint32_t)(0x3U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_BYTE_Pos))
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_BYTE_Pos 5

/* ACS_AOUT_CTRL sub-register settings */
#define AOUT_VSSA_BYTE                  ((uint8_t)(0x0U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VCC_SENSE_BYTE             ((uint8_t)(0x1U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VREF_0P75V_OUTPUT_BYTE     ((uint8_t)(0x2U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VREF_0P67V_OUTPUT_BYTE     ((uint8_t)(0x3U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_IREF_50N_OUTPUT_BYTE       ((uint8_t)(0x4U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_IREF_1N_OUTPUT_BYTE        ((uint8_t)(0x5U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDACS_OUTPUT_BYTE         ((uint8_t)(0x6U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VREF_0P75V_BUF_OUTPUT_BYTE ((uint8_t)(0x7U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VREG_BG_BYTE               ((uint8_t)(0x8U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDRF_SW_BYTE              ((uint8_t)(0x9U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDRF_SENSE_BYTE           ((uint8_t)(0xAU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDT_BYTE                  ((uint8_t)(0xBU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDC_SENSE_BYTE            ((uint8_t)(0xCU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDA_SW_BYTE               ((uint8_t)(0xDU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDA_SENSE_BYTE            ((uint8_t)(0xEU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDM_SENSE_BYTE            ((uint8_t)(0xFU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_NC_BYTE                    ((uint8_t)(0x10U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDPA_SENSE_BYTE           ((uint8_t)(0x11U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDPA_ISENSE_BYTE          ((uint8_t)(0x12U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_TM0_BYTE                   ((uint8_t)(0x13U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_BG_READY_BYTE              ((uint8_t)(0x14U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VCC_READY_BYTE             ((uint8_t)(0x15U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_DCDC_OVERLOAD_BYTE         ((uint8_t)(0x16U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_DCDC_ACTIVATED_BYTE        ((uint8_t)(0x17U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDRF_READY_BYTE           ((uint8_t)(0x18U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDC_READY_BYTE            ((uint8_t)(0x19U << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDM_READY_BYTE            ((uint8_t)(0x1AU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_VDDA_READY_BYTE            ((uint8_t)(0x1BU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_CLK_PRESENT_BYTE           ((uint8_t)(0x1CU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_XTAL_OK_BYTE               ((uint8_t)(0x1DU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_XTAL_CLK_BYTE              ((uint8_t)(0x1EU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))
#define AOUT_CLK_32K_BYTE               ((uint8_t)(0x1FU << ACS_AOUT_CTRL_TEST_AOUT_BYTE_Pos))

#define DIO0_RTC_CLK_DISABLE_BYTE       ((uint8_t)(0x0U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))
#define DIO0_RTC_CLK_125MS_BYTE         ((uint8_t)(0x1U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))
#define DIO0_RTC_CLK_250MS_BYTE         ((uint8_t)(0x2U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))
#define DIO0_RTC_CLK_500MS_BYTE         ((uint8_t)(0x3U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))
#define DIO0_RTC_CLK_1S_BYTE            ((uint8_t)(0x4U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))
#define DIO0_RTC_CLK_2S_BYTE            ((uint8_t)(0x5U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))
#define DIO0_RTC_CLK_4S_BYTE            ((uint8_t)(0x6U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))
#define DIO0_RTC_CLK_8S_BYTE            ((uint8_t)(0x7U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_BYTE_Pos))

#define DIO0_RTC_CLK_STOP_DIO0_BYTE     ((uint8_t)(0x0U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_BYTE_Pos))
#define DIO0_RTC_CLK_STOP_DIO1_BYTE     ((uint8_t)(0x1U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_BYTE_Pos))
#define DIO0_RTC_CLK_STOP_DIO2_BYTE     ((uint8_t)(0x2U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_BYTE_Pos))
#define DIO0_RTC_CLK_STOP_DIO3_BYTE     ((uint8_t)(0x3U << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_BYTE_Pos))

#define DIO0_RTC_CLK_STOP_RISING_BYTE   ((uint8_t)(DIO0_RTC_CLK_STOP_RISING_BITBAND << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_BYTE_Pos))
#define DIO0_RTC_CLK_STOP_FALLING_BYTE  ((uint8_t)(DIO0_RTC_CLK_STOP_FALLING_BITBAND << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_BYTE_Pos))

/* ACS_JIC_READ bit positions */
#define ACS_JIC_READ_BYTE0_RO_Pos       0
#define ACS_JIC_READ_BYTE0_RO_Mask      ((uint32_t)(0xFFU << ACS_JIC_READ_BYTE0_RO_Pos))

/* ----------------------------------------------------------------------------
 * Baseband Controller Interface
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t CTRL;                 /* Baseband controller control register */
    __I  uint32_t STATUS;               /* Baseband controller status register */
    __IO uint32_t COEX_CTRL;            /* Coexistence control register */
    __I  uint32_t COEX_STATUS;          /* Coexistence status register */
    __IO uint32_t COEX_INT_CFG;         /* Coexistence interrupts configuration register */
    __I  uint32_t COEX_INT_STATUS;      /* Coexistence interrupt status register */
    __IO uint32_t SYNC_CFG;             /* BLE and RF link synchronization configuration register */
} BBIF_Type;

#define BBIF_BASE                       0x40001400
#define BBIF                            ((BBIF_Type *) BBIF_BASE)

/* BBIF_CTRL bit positions */
#define BBIF_CTRL_WAKEUP_REQ_Pos        16
#define BBIF_CTRL_CLK_SEL_Pos           4
#define BBIF_CTRL_CLK_SEL_Mask          ((uint32_t)(0x3FU << BBIF_CTRL_CLK_SEL_Pos))
#define BBIF_CTRL_CLK_ENABLE_Pos        0

/* BBIF_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t CLK_SHORT;            
    __IO uint8_t WAKEUP_REQ_BYTE;      
         uint8_t RESERVED0[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001400, 0) - (0x40001400 + 4)];
    __IO uint32_t CLK_ENABLE_ALIAS;     /* Enable the baseband controller clocks generation */
         uint32_t RESERVED1[15];
    __IO uint32_t WAKEUP_REQ_ALIAS;     /* External wake up request used to sort-out sleep modes */
} BBIF_CTRL_Type;

#define BBIF_CTRL_BASE                  0x40001400
#define BBIF_CTRL                       ((BBIF_CTRL_Type *) BBIF_CTRL_BASE)

/* BBIF_CTRL settings */
#define BB_CLK_DISABLE_BITBAND          0x0
#define BB_CLK_ENABLE_BITBAND           0x1
#define BB_CLK_DISABLE                  ((uint32_t)(BB_CLK_DISABLE_BITBAND << BBIF_CTRL_CLK_ENABLE_Pos))
#define BB_CLK_ENABLE                   ((uint32_t)(BB_CLK_ENABLE_BITBAND << BBIF_CTRL_CLK_ENABLE_Pos))

#define BBCLK_DIVIDER_6                 ((uint32_t)(0x6U << BBIF_CTRL_CLK_SEL_Pos))
#define BBCLK_DIVIDER_8                 ((uint32_t)(0x8U << BBIF_CTRL_CLK_SEL_Pos))
#define BBCLK_DIVIDER_12                ((uint32_t)(0xCU << BBIF_CTRL_CLK_SEL_Pos))
#define BBCLK_DIVIDER_16                ((uint32_t)(0x10U << BBIF_CTRL_CLK_SEL_Pos))
#define BBCLK_DIVIDER_24                ((uint32_t)(0x18U << BBIF_CTRL_CLK_SEL_Pos))

#define BB_DEEP_SLEEP_BITBAND           0x0
#define BB_WAKEUP_BITBAND               0x1
#define BB_DEEP_SLEEP                   ((uint32_t)(BB_DEEP_SLEEP_BITBAND << BBIF_CTRL_WAKEUP_REQ_Pos))
#define BB_WAKEUP                       ((uint32_t)(BB_WAKEUP_BITBAND << BBIF_CTRL_WAKEUP_REQ_Pos))

/* BBIF_CTRL sub-register bit positions */
#define BBIF_CTRL_CLK_ENABLE_SHORT_Pos  0
#define BBIF_CTRL_CLK_SEL_SHORT_Pos     4
#define BBIF_CTRL_CLK_SEL_SHORT_Mask    ((uint32_t)(0x3FU << BBIF_CTRL_CLK_SEL_SHORT_Pos))
#define BBIF_CTRL_WAKEUP_REQ_BYTE_Pos   0

/* BBIF_CTRL sub-register settings */
#define BB_CLK_DISABLE_SHORT            ((uint16_t)(BB_CLK_DISABLE_BITBAND << BBIF_CTRL_CLK_ENABLE_SHORT_Pos))
#define BB_CLK_ENABLE_SHORT             ((uint16_t)(BB_CLK_ENABLE_BITBAND << BBIF_CTRL_CLK_ENABLE_SHORT_Pos))

#define BBCLK_DIVIDER_6_SHORT           ((uint16_t)(0x6U << BBIF_CTRL_CLK_SEL_SHORT_Pos))
#define BBCLK_DIVIDER_8_SHORT           ((uint16_t)(0x8U << BBIF_CTRL_CLK_SEL_SHORT_Pos))
#define BBCLK_DIVIDER_12_SHORT          ((uint16_t)(0xCU << BBIF_CTRL_CLK_SEL_SHORT_Pos))
#define BBCLK_DIVIDER_16_SHORT          ((uint16_t)(0x10U << BBIF_CTRL_CLK_SEL_SHORT_Pos))
#define BBCLK_DIVIDER_24_SHORT          ((uint16_t)(0x18U << BBIF_CTRL_CLK_SEL_SHORT_Pos))

#define BB_DEEP_SLEEP_BYTE              ((uint8_t)(BB_DEEP_SLEEP_BITBAND << BBIF_CTRL_WAKEUP_REQ_BYTE_Pos))
#define BB_WAKEUP_BYTE                  ((uint8_t)(BB_WAKEUP_BITBAND << BBIF_CTRL_WAKEUP_REQ_BYTE_Pos))

/* BBIF_STATUS bit positions */
#define BBIF_STATUS_LINK_FORMAT_Pos     11
#define BBIF_STATUS_LINK_FORMAT_Mask    ((uint32_t)(0x1FU << BBIF_STATUS_LINK_FORMAT_Pos))
#define BBIF_STATUS_LINK_LABEL_Pos      4
#define BBIF_STATUS_LINK_LABEL_Mask     ((uint32_t)(0x1FU << BBIF_STATUS_LINK_LABEL_Pos))
#define BBIF_STATUS_AOBLE_STATUS_Pos    3
#define BBIF_STATUS_CLK_STATUS_Pos      2
#define BBIF_STATUS_OSC_EN_Pos          1
#define BBIF_STATUS_RADIO_EN_Pos        0

/* BBIF_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t RADIO_EN_ALIAS;       /* RF front-end enabling */
    __I  uint32_t OSC_EN_ALIAS;         /* Oscillator front-end enabling */
    __I  uint32_t CLK_STATUS_ALIAS;     /* Clock status defining the current active clock in use */
    __I  uint32_t AOBLE_STATUS_ALIAS;   /* Audio over BLE feature status */
} BBIF_STATUS_Type;

#define BBIF_STATUS_BASE                SYS_CALC_BITBAND(0x40001404, 0)
#define BBIF_STATUS                     ((BBIF_STATUS_Type *) BBIF_STATUS_BASE)

/* BBIF_STATUS settings */
#define RF_DISABLED_BITBAND             0x0
#define RF_ENABLED_BITBAND              0x1
#define RF_DISABLED                     ((uint32_t)(RF_DISABLED_BITBAND << BBIF_STATUS_RADIO_EN_Pos))
#define RF_ENABLED                      ((uint32_t)(RF_ENABLED_BITBAND << BBIF_STATUS_RADIO_EN_Pos))

#define OSC_DISABLED_BITBAND            0x0
#define OSC_ENABLED_BITBAND             0x1
#define OSC_DISABLED                    ((uint32_t)(OSC_DISABLED_BITBAND << BBIF_STATUS_OSC_EN_Pos))
#define OSC_ENABLED                     ((uint32_t)(OSC_ENABLED_BITBAND << BBIF_STATUS_OSC_EN_Pos))

#define MASTER_CLK_BITBAND              0x0
#define LOW_POWER_CLK_BITBAND           0x1
#define MASTER_CLK                      ((uint32_t)(MASTER_CLK_BITBAND << BBIF_STATUS_CLK_STATUS_Pos))
#define LOW_POWER_CLK                   ((uint32_t)(LOW_POWER_CLK_BITBAND << BBIF_STATUS_CLK_STATUS_Pos))

#define AOBLE_DISABLED_DEVICE_BITBAND   0x0
#define AOBLE_ENABLED_DEVICE_BITBAND    0x1
#define AOBLE_DISABLED_DEVICE           ((uint32_t)(AOBLE_DISABLED_DEVICE_BITBAND << BBIF_STATUS_AOBLE_STATUS_Pos))
#define AOBLE_ENABLED_DEVICE            ((uint32_t)(AOBLE_ENABLED_DEVICE_BITBAND << BBIF_STATUS_AOBLE_STATUS_Pos))

#define LINK_FORMAT_RESET               ((uint32_t)(0x0U << BBIF_STATUS_LINK_FORMAT_Pos))
#define MASTER_CONNECT                  ((uint32_t)(0x2U << BBIF_STATUS_LINK_FORMAT_Pos))
#define SLAVE_CONNECT                   ((uint32_t)(0x3U << BBIF_STATUS_LINK_FORMAT_Pos))
#define LOW_DUTY_ADVERTISER             ((uint32_t)(0x4U << BBIF_STATUS_LINK_FORMAT_Pos))
#define HIGH_DUTY_ADVERTISER            ((uint32_t)(0x5U << BBIF_STATUS_LINK_FORMAT_Pos))
#define PASSIVE_SCANNER                 ((uint32_t)(0x8U << BBIF_STATUS_LINK_FORMAT_Pos))
#define ACTIVE_SCANNER                  ((uint32_t)(0x9U << BBIF_STATUS_LINK_FORMAT_Pos))
#define INITIATOR                       ((uint32_t)(0xFU << BBIF_STATUS_LINK_FORMAT_Pos))
#define TX_TEST_MODE                    ((uint32_t)(0x1CU << BBIF_STATUS_LINK_FORMAT_Pos))
#define RX_TEST_MODE                    ((uint32_t)(0x1DU << BBIF_STATUS_LINK_FORMAT_Pos))
#define TX_RX_TEST_MODE                 ((uint32_t)(0x1EU << BBIF_STATUS_LINK_FORMAT_Pos))

/* BBIF_COEX_CTRL bit positions */
#define BBIF_COEX_CTRL_TX_Pos           4
#define BBIF_COEX_CTRL_RX_Pos           0

/* BBIF_COEX_CTRL bit-band aliases */
typedef struct
{
    __IO uint32_t RX_ALIAS;             /* Indicates if the  RF front-end performs a non-BLE Rx activity */
         uint32_t RESERVED0[3];
    __IO uint32_t TX_ALIAS;             /* Indicates if the RF front-end performs a non-BLE Tx activity */
} BBIF_COEX_CTRL_Type;

#define BBIF_COEX_CTRL_BASE             SYS_CALC_BITBAND(0x40001408, 0)
#define BBIF_COEX_CTRL                  ((BBIF_COEX_CTRL_Type *) BBIF_COEX_CTRL_BASE)

/* BBIF_COEX_CTRL settings */
#define COEX_RX_IDLE_BITBAND            0x0
#define COEX_RX_BUSY_BITBAND            0x1
#define COEX_RX_IDLE                    ((uint32_t)(COEX_RX_IDLE_BITBAND << BBIF_COEX_CTRL_RX_Pos))
#define COEX_RX_BUSY                    ((uint32_t)(COEX_RX_BUSY_BITBAND << BBIF_COEX_CTRL_RX_Pos))

#define COEX_TX_IDLE_BITBAND            0x0
#define COEX_TX_BUSY_BITBAND            0x1
#define COEX_TX_IDLE                    ((uint32_t)(COEX_TX_IDLE_BITBAND << BBIF_COEX_CTRL_TX_Pos))
#define COEX_TX_BUSY                    ((uint32_t)(COEX_TX_BUSY_BITBAND << BBIF_COEX_CTRL_TX_Pos))

/* BBIF_COEX_STATUS bit positions */
#define BBIF_COEX_STATUS_BLE_PTI_Pos    12
#define BBIF_COEX_STATUS_BLE_PTI_Mask   ((uint32_t)(0xFU << BBIF_COEX_STATUS_BLE_PTI_Pos))
#define BBIF_COEX_STATUS_BLE_IN_PROCESS_Pos 8
#define BBIF_COEX_STATUS_BLE_TX_Pos     4
#define BBIF_COEX_STATUS_BLE_RX_Pos     0

/* BBIF_COEX_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t BLE_RX_ALIAS;         /* Indicates if the RW-BLE core is busy and performs Rx activity, active high */
         uint32_t RESERVED0[3];
    __I  uint32_t BLE_TX_ALIAS;         /* Indicates if the RW-BLE core is busy and performs Tx activity, active high. */
         uint32_t RESERVED1[3];
    __I  uint32_t BLE_IN_PROCESS_ALIAS; /* Indicate if the RW-BLE core has an event in process, active high. */
} BBIF_COEX_STATUS_Type;

#define BBIF_COEX_STATUS_BASE           SYS_CALC_BITBAND(0x4000140C, 0)
#define BBIF_COEX_STATUS                ((BBIF_COEX_STATUS_Type *) BBIF_COEX_STATUS_BASE)

/* BBIF_COEX_STATUS settings */
#define BLE_RX_IDLE_BITBAND             0x0
#define BLE_RX_BUSY_BITBAND             0x1
#define BLE_RX_IDLE                     ((uint32_t)(BLE_RX_IDLE_BITBAND << BBIF_COEX_STATUS_BLE_RX_Pos))
#define BLE_RX_BUSY                     ((uint32_t)(BLE_RX_BUSY_BITBAND << BBIF_COEX_STATUS_BLE_RX_Pos))

#define BLE_TX_IDLE_BITBAND             0x0
#define BLE_TX_BUSY_BITBAND             0x1
#define BLE_TX_IDLE                     ((uint32_t)(BLE_TX_IDLE_BITBAND << BBIF_COEX_STATUS_BLE_TX_Pos))
#define BLE_TX_BUSY                     ((uint32_t)(BLE_TX_BUSY_BITBAND << BBIF_COEX_STATUS_BLE_TX_Pos))

#define BLE_IDLE_BITBAND                0x0
#define BLE_IN_PROCESS_BITBAND          0x1
#define BLE_IDLE                        ((uint32_t)(BLE_IDLE_BITBAND << BBIF_COEX_STATUS_BLE_IN_PROCESS_Pos))
#define BLE_IN_PROCESS                  ((uint32_t)(BLE_IN_PROCESS_BITBAND << BBIF_COEX_STATUS_BLE_IN_PROCESS_Pos))

#define BLE_PTI_PRIORITY_0              ((uint32_t)(0x0U << BBIF_COEX_STATUS_BLE_PTI_Pos))
#define BLE_PTI_PRIORITY_15             ((uint32_t)(0xFU << BBIF_COEX_STATUS_BLE_PTI_Pos))

/* BBIF_COEX_INT_CFG bit positions */
#define BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_Pos 8
#define BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_Mask ((uint32_t)(0x3U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_Pos))
#define BBIF_COEX_INT_CFG_BLE_TX_EVENT_Pos 4
#define BBIF_COEX_INT_CFG_BLE_TX_EVENT_Mask ((uint32_t)(0x3U << BBIF_COEX_INT_CFG_BLE_TX_EVENT_Pos))
#define BBIF_COEX_INT_CFG_BLE_RX_EVENT_Pos 0
#define BBIF_COEX_INT_CFG_BLE_RX_EVENT_Mask ((uint32_t)(0x3U << BBIF_COEX_INT_CFG_BLE_RX_EVENT_Pos))

/* BBIF_COEX_INT_CFG sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t BLE_IN_PROCESS_EVENT_BYTE;
         uint8_t RESERVED1[2];
} BBIF_COEX_INT_CFG_Type;

#define BBIF_COEX_INT_CFG_BASE          0x40001410
#define BBIF_COEX_INT_CFG               ((BBIF_COEX_INT_CFG_Type *) BBIF_COEX_INT_CFG_BASE)

/* BBIF_COEX_INT_CFG settings */
#define BLE_RX_EVENT_NONE               ((uint32_t)(0x0U << BBIF_COEX_INT_CFG_BLE_RX_EVENT_Pos))
#define BLE_RX_EVENT_RISING_EDGE        ((uint32_t)(0x1U << BBIF_COEX_INT_CFG_BLE_RX_EVENT_Pos))
#define BLE_RX_EVENT_FALLING_EDGE       ((uint32_t)(0x2U << BBIF_COEX_INT_CFG_BLE_RX_EVENT_Pos))
#define BLE_RX_EVENT_TRANSITION         ((uint32_t)(0x3U << BBIF_COEX_INT_CFG_BLE_RX_EVENT_Pos))

#define BLE_TX_EVENT_NONE               ((uint32_t)(0x0U << BBIF_COEX_INT_CFG_BLE_TX_EVENT_Pos))
#define BLE_TX_EVENT_RISING_EDGE        ((uint32_t)(0x1U << BBIF_COEX_INT_CFG_BLE_TX_EVENT_Pos))
#define BLE_TX_EVENT_FALLING_EDGE       ((uint32_t)(0x2U << BBIF_COEX_INT_CFG_BLE_TX_EVENT_Pos))
#define BLE_TX_EVENT_TRANSITION         ((uint32_t)(0x3U << BBIF_COEX_INT_CFG_BLE_TX_EVENT_Pos))

#define BLE_IN_PROCESS_EVENT_NONE       ((uint32_t)(0x0U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_Pos))
#define BLE_IN_PROCESS_EVENT_RISING_EDGE ((uint32_t)(0x1U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_Pos))
#define BLE_IN_PROCESS_EVENT_FALLING_EDGE ((uint32_t)(0x2U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_Pos))
#define BLE_IN_PROCESS_EVENT_TRANSITION ((uint32_t)(0x3U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_Pos))

/* BBIF_COEX_INT_CFG sub-register bit positions */
#define BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_BYTE_Pos 0
#define BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_BYTE_Mask ((uint32_t)(0x3U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_BYTE_Pos))

/* BBIF_COEX_INT_CFG sub-register settings */
#define BLE_IN_PROCESS_EVENT_NONE_BYTE  ((uint8_t)(0x0U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_BYTE_Pos))
#define BLE_IN_PROCESS_EVENT_RISING_EDGE_BYTE ((uint8_t)(0x1U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_BYTE_Pos))
#define BLE_IN_PROCESS_EVENT_FALLING_EDGE_BYTE ((uint8_t)(0x2U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_BYTE_Pos))
#define BLE_IN_PROCESS_EVENT_TRANSITION_BYTE ((uint8_t)(0x3U << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_BYTE_Pos))

/* BBIF_COEX_INT_STATUS bit positions */
#define BBIF_COEX_INT_STATUS_BLE_TX_EVENT_FLAG_Pos 4
#define BBIF_COEX_INT_STATUS_BLE_RX_EVENT_FLAG_Pos 0

/* BBIF_COEX_INT_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t BLE_RX_EVENT_FLAG_ALIAS;/* Indicates if a BLE_RX_EVENT interrupt has been generated */
         uint32_t RESERVED0[3];
    __I  uint32_t BLE_TX_EVENT_FLAG_ALIAS;/* Indicates if a BLE_TX_EVENT interrupt has been generated */
} BBIF_COEX_INT_STATUS_Type;

#define BBIF_COEX_INT_STATUS_BASE       SYS_CALC_BITBAND(0x40001414, 0)
#define BBIF_COEX_INT_STATUS            ((BBIF_COEX_INT_STATUS_Type *) BBIF_COEX_INT_STATUS_BASE)

/* BBIF_COEX_INT_STATUS settings */
#define BLE_RX_EVENT_NO_INT_BITBAND     0x0
#define BLE_RX_EVENT_INT_BITBAND        0x1
#define BLE_RX_EVENT_NO_INT             ((uint32_t)(BLE_RX_EVENT_NO_INT_BITBAND << BBIF_COEX_INT_STATUS_BLE_RX_EVENT_FLAG_Pos))
#define BLE_RX_EVENT_INT                ((uint32_t)(BLE_RX_EVENT_INT_BITBAND << BBIF_COEX_INT_STATUS_BLE_RX_EVENT_FLAG_Pos))

#define BLE_TX_EVENT_NO_INT_BITBAND     0x0
#define BLE_TX_EVENT_INT_BITBAND        0x1
#define BLE_TX_EVENT_NO_INT             ((uint32_t)(BLE_TX_EVENT_NO_INT_BITBAND << BBIF_COEX_INT_STATUS_BLE_TX_EVENT_FLAG_Pos))
#define BLE_TX_EVENT_INT                ((uint32_t)(BLE_TX_EVENT_INT_BITBAND << BBIF_COEX_INT_STATUS_BLE_TX_EVENT_FLAG_Pos))

/* BBIF_SYNC_CFG bit positions */
#define BBIF_SYNC_CFG_RF_RX_Pos         17
#define BBIF_SYNC_CFG_RF_ACTIVE_Pos     16
#define BBIF_SYNC_CFG_LINK_FORMAT_Pos   11
#define BBIF_SYNC_CFG_LINK_FORMAT_Mask  ((uint32_t)(0x1FU << BBIF_SYNC_CFG_LINK_FORMAT_Pos))
#define BBIF_SYNC_CFG_LINK_LABEL_Pos    4
#define BBIF_SYNC_CFG_LINK_LABEL_Mask   ((uint32_t)(0x1FU << BBIF_SYNC_CFG_LINK_LABEL_Pos))
#define BBIF_SYNC_CFG_SOURCE_Pos        1
#define BBIF_SYNC_CFG_SOURCE_Mask       ((uint32_t)(0x7U << BBIF_SYNC_CFG_SOURCE_Pos))
#define BBIF_SYNC_CFG_ENABLE_Pos        0

/* BBIF_SYNC_CFG sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[2];
    __IO uint8_t RF_BYTE;              
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001418, 0) - (0x40001418 + 4)];
    __IO uint32_t ENABLE_ALIAS;         /* Enable the frame synchronization pulse filter */
         uint32_t RESERVED2[15];
    __IO uint32_t RF_ACTIVE_ALIAS;      /* Specify if the RF front-end is currently processing the audio link */
    __IO uint32_t RF_RX_ALIAS;          /* Specify if the RF front-end is currently receiving the audio link */
} BBIF_SYNC_CFG_Type;

#define BBIF_SYNC_CFG_BASE              0x40001418
#define BBIF_SYNC_CFG                   ((BBIF_SYNC_CFG_Type *) BBIF_SYNC_CFG_BASE)

/* BBIF_SYNC_CFG settings */
#define SYNC_DISABLE_BITBAND            0x0
#define SYNC_ENABLE_BITBAND             0x1
#define SYNC_DISABLE                    ((uint32_t)(SYNC_DISABLE_BITBAND << BBIF_SYNC_CFG_ENABLE_Pos))
#define SYNC_ENABLE                     ((uint32_t)(SYNC_ENABLE_BITBAND << BBIF_SYNC_CFG_ENABLE_Pos))

#define SYNC_SOURCE_BLE_RX              ((uint32_t)(0x0U << BBIF_SYNC_CFG_SOURCE_Pos))
#define SYNC_SOURCE_BLE_RX_AUDIO0       ((uint32_t)(0x1U << BBIF_SYNC_CFG_SOURCE_Pos))
#define SYNC_SOURCE_BLE_RX_AUDIO1       ((uint32_t)(0x2U << BBIF_SYNC_CFG_SOURCE_Pos))
#define SYNC_SOURCE_BLE_RX_AUDIO2       ((uint32_t)(0x3U << BBIF_SYNC_CFG_SOURCE_Pos))
#define SYNC_SOURCE_RF_RX               ((uint32_t)(0x4U << BBIF_SYNC_CFG_SOURCE_Pos))
#define SYNC_SOURCE_BLE_TX              ((uint32_t)(0x5U << BBIF_SYNC_CFG_SOURCE_Pos))

#define IDLE_BITBAND                    0x0
#define ACTIVE_BITBAND                  0x1
#define IDLE                            ((uint32_t)(IDLE_BITBAND << BBIF_SYNC_CFG_RF_ACTIVE_Pos))
#define ACTIVE                          ((uint32_t)(ACTIVE_BITBAND << BBIF_SYNC_CFG_RF_ACTIVE_Pos))

#define RX_IDLE_BITBAND                 0x0
#define RX_ACTIVE_BITBAND               0x1
#define RX_IDLE                         ((uint32_t)(RX_IDLE_BITBAND << BBIF_SYNC_CFG_RF_RX_Pos))
#define RX_ACTIVE                       ((uint32_t)(RX_ACTIVE_BITBAND << BBIF_SYNC_CFG_RF_RX_Pos))

/* BBIF_SYNC_CFG sub-register bit positions */
#define BBIF_SYNC_CFG_RF_ACTIVE_BYTE_Pos 0
#define BBIF_SYNC_CFG_RF_RX_BYTE_Pos    1

/* BBIF_SYNC_CFG sub-register settings */
#define IDLE_BYTE                       ((uint8_t)(IDLE_BITBAND << BBIF_SYNC_CFG_RF_ACTIVE_BYTE_Pos))
#define ACTIVE_BYTE                     ((uint8_t)(ACTIVE_BITBAND << BBIF_SYNC_CFG_RF_ACTIVE_BYTE_Pos))

#define RX_IDLE_BYTE                    ((uint8_t)(RX_IDLE_BITBAND << BBIF_SYNC_CFG_RF_RX_BYTE_Pos))
#define RX_ACTIVE_BYTE                  ((uint8_t)(RX_ACTIVE_BITBAND << BBIF_SYNC_CFG_RF_RX_BYTE_Pos))

/* ----------------------------------------------------------------------------
 * Baseband Controller
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t RWBBCNTL;             /* Baseband control register */
    __I  uint32_t VERSION;              /* BLE revision register */
    __I  uint32_t RWBLEBCONF;           /* Baseband configuration register (compilation options dependant) */
    __IO uint32_t INTCNTL;              /* Interrupts control register */
    __I  uint32_t INTSTAT;              /* Interrupts status register */
    __I  uint32_t INTRAWSTAT;           /* Interrupts raw status register */
    __IO uint32_t INTACK;               /* Interrupts acknowledgement register */
    __IO uint32_t BASETIMECNT;          /* base timer configuration register */
    __I  uint32_t FINETIMECNT;          /* Fine timer configuration register */
    __IO uint32_t BDADDRL;              /* BLE device address (LSB part) register */
    __IO uint32_t BDADDRU;              /* BLE device address (MSB part) register */
    __IO uint32_t ET_CURRENTRXDESCPTR;  /* Rx descriptor pointer register */
    __IO uint32_t DEEPSLCNTL;           /* Deep sleep control register */
    __IO uint32_t DEEPSLWKUP;           /* Deep sleep wakeup register */
    __I  uint32_t DEEPSLSTAT;           /* Deep sleep status register */
    __IO uint32_t ENBPRESET;            /* Stabilization times */
    __IO uint32_t FINECNTCORR;          /* Fine timer correction register */
    __IO uint32_t BASETIMECNTCORR;      /* Base timer correction register */
         uint32_t RESERVED0[2];
    __IO uint32_t DIAGCNTL;             /* Diagnostic ports control register */
    __I  uint32_t DIAGSTAT;             /* Diagnostic ports status register */
    __IO uint32_t DEBUGADDMAX;          /* Diagnostic ports upper limit */
    __IO uint32_t DEBUGADDMIN;          /* Diagnostic ports lower limit */
    __I  uint32_t ERRORTYPESTAT;        /* Diagnostic ports errors register */
    __IO uint32_t SWPROFILING;          /* Software profiling register */
         uint32_t RESERVED1[2];
    __IO uint32_t RADIOCNTL0;           /* Principal control register for the radio interface */
    __IO uint32_t RADIOCNTL1;           /* Second control register for the radio interface */
    __IO uint32_t RADIOCNTL2;           /* Third control register for the radio interface */
         uint32_t RESERVED2;
    __IO uint32_t RADIOPWRUPDN0;        /* Principal control register for the radio interface power up/down delays */
    __IO uint32_t RADIOPWRUPDN1;        /* Second control register for the radio interface power up/down delays */
         uint32_t RESERVED3[2];
    __IO uint32_t RADIOTXRXTIM0;        /* Principal control register for the radio interface timing compensation delays */
    __IO uint32_t RADIOTXRXTIM1;        /* Second control register for the radio interface timing compensation delays */
         uint32_t RESERVED4[2];
    __IO uint32_t SPIPTRCNTL0;          /* First control register for the radio interface SPI pointers */
    __IO uint32_t SPIPTRCNTL1;          /* Second control register for the radio interface SPI pointers */
    __IO uint32_t SPIPTRCNTL2;          /* Third control register for the radio interface SPI pointers */
         uint32_t RESERVED5;
    __IO uint32_t ADVCHMAP;             /* Advertising channel map register */
         uint32_t RESERVED6[3];
    __IO uint32_t ADVTIM;               /* Delay information register handling advertising event timers */
    __I  uint32_t ACTSCANSTAT;          /* Active scan mode control register */
         uint32_t RESERVED7[2];
    __IO uint32_t WLPUBADDPTR;          /* Address pointer of public devices */
    __IO uint32_t WLPRIVADDPTR;         /* Address pointer of private devices */
    __IO uint32_t WLNBDEV;              /* Devices in white list */
         uint32_t RESERVED8;
    __IO uint32_t AESCNTL;              /* AES-128 ciphering control register */
    __IO uint32_t AESKEY31_0;           /* AES encryption 128-bit key register (bits 31:0) */
    __IO uint32_t AESKEY63_32;          /* AES encryption 128-bit key register (bits 63:32) */
    __IO uint32_t AESKEY95_64;          /* AES encryption 128-bit key register (bits 95:64) */
    __IO uint32_t AESKEY127_96;         /* AES encryption 128-bit key register (bits 127:96) */
    __IO uint32_t AESPTR;               /* AES memory zone pointer */
    __I  uint32_t TXMICVAL;             /* AES-CCM plain MIC value register in Tx */
    __I  uint32_t RXMICVAL;             /* AES-CCM plain MIC value register in Rx */
    __IO uint32_t RFTESTCNTL;           /* RF testing and regulatory body support register */
    __IO uint32_t RFTESTTXSTAT;         /* Number of transmitted packet during test modes */
    __IO uint32_t RFTESTRXSTAT;         /* Number of correctly received packet during test modes */
         uint32_t RESERVED9;
    __IO uint32_t TIMGENCNTL;           /* Timing generator control register */
    __IO uint32_t GROSSTIMTGT;          /* Gross timer control register */
    __IO uint32_t FINETIMTGT;           /* Fine timer control register */
         uint32_t RESERVED10;
    __IO uint32_t COEXIFCNTL0;          /* Coexistence control register 0 */
    __IO uint32_t COEXIFCNTL1;          /* Coexistence control register 1 */
    __IO uint32_t COEXIFCNTL2;          /* Coexistence control register 2 */
    __IO uint32_t BBMPRIO0;             /* Priority control register 0 */
    __IO uint32_t BBMPRIO1;             /* Priority control register 1 */
         uint32_t RESERVED11[3];
    __IO uint32_t RALPTR;               /* Register used by the Resolving Address List engine */
    __IO uint32_t RALNBDEV;             /* Register used by the Resolving Address List engine */
    __IO uint32_t RAL_LOCAL_RND;        /* Register used by the Resolving Address List engine */
    __IO uint32_t RAL_PEER_RND;         /* Register used by the Resolving Address List engine */
    __IO uint32_t ISOCHANCNTL0;         /* ISO Channel 0 control */
    __IO uint32_t ISOMUTECNTL0;         /* ISO Channel 0 mute control */
    __IO uint32_t ISOCURRENTTXPTR0;     /* ISO Channel 0 current Tx pointer */
    __IO uint32_t ISOCURRENTRXPTR0;     /* ISO Channel 0 current Rx pointer */
    __IO uint32_t ISOTRCNL0;            /* ISO Channel 0 payloads */
    __IO uint32_t ISOEVTCNTLOFFSETL0;   /* ISO Channel 0 mute control */
    __IO uint32_t ISOEVTCNTLOFFSETU0;   /* ISO Channel 0 mute control */
         uint32_t RESERVED12;
    __IO uint32_t ISOCHANCNTL1;         /* ISO Channel 1 control */
    __IO uint32_t ISOMUTECNTL1;         /* ISO Channel 1 mute control */
    __IO uint32_t ISOCURRENTTXPTR1;     /* ISO Channel 1 current Tx pointer */
    __IO uint32_t ISOCURRENTRXPTR1;     /* ISO Channel 1 current Rx pointer */
    __IO uint32_t ISOTRCNL1;            /* ISO Channel 1 payloads */
    __IO uint32_t ISOEVTCNTLOFFSETL1;   /* ISO Channel 1 mute control */
    __IO uint32_t ISOEVTCNTLOFFSETU1;   /* ISO Channel 1 mute control */
         uint32_t RESERVED13;
    __IO uint32_t ISOCHANCNTL2;         /* ISO Channel 2 control */
    __IO uint32_t ISOMUTECNTL2;         /* ISO Channel 2 mute control */
    __IO uint32_t ISOCURRENTTXPTR2;     /* ISO Channel 2 current Tx pointer */
    __IO uint32_t ISOCURRENTRXPTR2;     /* ISO Channel 2 current Rx pointer */
    __IO uint32_t ISOTRCNL2;            /* ISO Channel 2 payloads */
    __IO uint32_t ISOEVTCNTLOFFSETL2;   /* ISO Channel 2 mute control */
    __IO uint32_t ISOEVTCNTLOFFSETU2;   /* ISO Channel 2 mute control */
         uint32_t RESERVED14;
    __IO uint32_t BBPRIOSCHARB;         /* Register controlling the decision instant for priority scheduling arbitration */
} BB_Type;

#define BB_BASE                         0x40001500
#define BB                              ((BB_Type *) BB_BASE)

/* BB_RWBBCNTL bit positions */
#define BB_RWBBCNTL_MASTER_SOFT_RST_Pos 31
#define BB_RWBBCNTL_MASTER_TGSOFT_RST_Pos 30
#define BB_RWBBCNTL_REG_SOFT_RST_Pos    29
#define BB_RWBBCNTL_SWINT_REQ_Pos       28
#define BB_RWBBCNTL_RFTEST_ABORT_Pos    26
#define BB_RWBBCNTL_ADVERT_ABORT_Pos    25
#define BB_RWBBCNTL_SCAN_ABORT_Pos      24
#define BB_RWBBCNTL_MD_DSB_Pos          22
#define BB_RWBBCNTL_SN_DSB_Pos          21
#define BB_RWBBCNTL_NESN_DSB_Pos        20
#define BB_RWBBCNTL_CRYPT_DSB_Pos       19
#define BB_RWBBCNTL_WHIT_DSB_Pos        18
#define BB_RWBBCNTL_CRC_DSB_Pos         17
#define BB_RWBBCNTL_HOP_REMAP_DSB_Pos   16
#define BB_RWBBCNTL_ADVERTFILT_EN_Pos   9
#define BB_RWBBCNTL_RWBLE_EN_Pos        8
#define BB_RWBBCNTL_RXWINSZDEF_Pos      4
#define BB_RWBBCNTL_RXWINSZDEF_Mask     ((uint32_t)(0xFU << BB_RWBBCNTL_RXWINSZDEF_Pos))
#define BB_RWBBCNTL_SYNCERR_Pos         0
#define BB_RWBBCNTL_SYNCERR_Mask        ((uint32_t)(0x7U << BB_RWBBCNTL_SYNCERR_Pos))

/* BB_RWBBCNTL bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[8];
    __IO uint32_t RWBLE_EN_ALIAS;       /* Enable RW-BLE core exchange table pre-fetch mechanism */
    __IO uint32_t ADVERTFILT_EN_ALIAS;  /* Advertising channels error filtering enable control */
         uint32_t RESERVED1[6];
    __IO uint32_t HOP_REMAP_DSB_ALIAS;  /* Disable frequency hopping remapping algorithm */
    __IO uint32_t CRC_DSB_ALIAS;        /* Disable CRC stripping */
    __IO uint32_t WHIT_DSB_ALIAS;       /* Disable whitening */
    __IO uint32_t CRYPT_DSB_ALIAS;      /* Disable encryption / decryption */
    __IO uint32_t NESN_DSB_ALIAS;       /* Disable acknowledge scheme */
    __IO uint32_t SN_DSB_ALIAS;         /* Disable sequence number management */
    __IO uint32_t MD_DSB_ALIAS;         /* Allow a single Tx/Rx exchange whatever the MD bits are */
         uint32_t RESERVED2;
    __IO uint32_t SCAN_ABORT_ALIAS;     /* Abort the current advertising event */
    __IO uint32_t ADVERT_ABORT_ALIAS;   /* Abort the current scan window */
    __IO uint32_t RFTEST_ABORT_ALIAS;   /* Abort the current RF testing defined as per CS-FORMAT */
         uint32_t RESERVED3;
    __IO uint32_t SWINT_REQ_ALIAS;      /* Forces the generation of ble_sw_irq */
    __IO uint32_t REG_SOFT_RST_ALIAS;   /* Reset the complete register block */
    __IO uint32_t MASTER_TGSOFT_RST_ALIAS;/* Reset the timing generator */
    __IO uint32_t MASTER_SOFT_RST_ALIAS;/* Reset the complete system except registers and timing generator */
} BB_RWBBCNTL_Type;

#define BB_RWBBCNTL_BASE                SYS_CALC_BITBAND(0x40001500, 0)
#define BB_RWBBCNTL                     ((BB_RWBBCNTL_Type *) BB_RWBBCNTL_BASE)

/* BB_RWBBCNTL settings */
#define SYNCERR_0                       ((uint32_t)(0x0U << BB_RWBBCNTL_SYNCERR_Pos))

#define RXWINSZDEF_0                    ((uint32_t)(0x0U << BB_RWBBCNTL_RXWINSZDEF_Pos))

#define RWBLE_EN_0_BITBAND              0x0
#define RWBLE_EN_1_BITBAND              0x1
#define RWBLE_EN_0                      ((uint32_t)(RWBLE_EN_0_BITBAND << BB_RWBBCNTL_RWBLE_EN_Pos))
#define RWBLE_EN_1                      ((uint32_t)(RWBLE_EN_1_BITBAND << BB_RWBBCNTL_RWBLE_EN_Pos))

#define ADVERTFILT_EN_0_BITBAND         0x0
#define ADVERTFILT_EN_1_BITBAND         0x1
#define ADVERTFILT_EN_0                 ((uint32_t)(ADVERTFILT_EN_0_BITBAND << BB_RWBBCNTL_ADVERTFILT_EN_Pos))
#define ADVERTFILT_EN_1                 ((uint32_t)(ADVERTFILT_EN_1_BITBAND << BB_RWBBCNTL_ADVERTFILT_EN_Pos))

#define HOP_REMAP_DSB_0_BITBAND         0x0
#define HOP_REMAP_DSB_1_BITBAND         0x1
#define HOP_REMAP_DSB_0                 ((uint32_t)(HOP_REMAP_DSB_0_BITBAND << BB_RWBBCNTL_HOP_REMAP_DSB_Pos))
#define HOP_REMAP_DSB_1                 ((uint32_t)(HOP_REMAP_DSB_1_BITBAND << BB_RWBBCNTL_HOP_REMAP_DSB_Pos))

#define CRC_DSB_0_BITBAND               0x0
#define CRC_DSB_1_BITBAND               0x1
#define CRC_DSB_0                       ((uint32_t)(CRC_DSB_0_BITBAND << BB_RWBBCNTL_CRC_DSB_Pos))
#define CRC_DSB_1                       ((uint32_t)(CRC_DSB_1_BITBAND << BB_RWBBCNTL_CRC_DSB_Pos))

#define WHIT_DSB_0_BITBAND              0x0
#define WHIT_DSB_1_BITBAND              0x1
#define WHIT_DSB_0                      ((uint32_t)(WHIT_DSB_0_BITBAND << BB_RWBBCNTL_WHIT_DSB_Pos))
#define WHIT_DSB_1                      ((uint32_t)(WHIT_DSB_1_BITBAND << BB_RWBBCNTL_WHIT_DSB_Pos))

#define CRYPT_DSB_0_BITBAND             0x0
#define CRYPT_DSB_1_BITBAND             0x1
#define CRYPT_DSB_0                     ((uint32_t)(CRYPT_DSB_0_BITBAND << BB_RWBBCNTL_CRYPT_DSB_Pos))
#define CRYPT_DSB_1                     ((uint32_t)(CRYPT_DSB_1_BITBAND << BB_RWBBCNTL_CRYPT_DSB_Pos))

#define NESN_DSB_0_BITBAND              0x0
#define NESN_DSB_1_BITBAND              0x1
#define NESN_DSB_0                      ((uint32_t)(NESN_DSB_0_BITBAND << BB_RWBBCNTL_NESN_DSB_Pos))
#define NESN_DSB_1                      ((uint32_t)(NESN_DSB_1_BITBAND << BB_RWBBCNTL_NESN_DSB_Pos))

#define SN_DSB_0_BITBAND                0x0
#define SN_DSB_1_BITBAND                0x1
#define SN_DSB_0                        ((uint32_t)(SN_DSB_0_BITBAND << BB_RWBBCNTL_SN_DSB_Pos))
#define SN_DSB_1                        ((uint32_t)(SN_DSB_1_BITBAND << BB_RWBBCNTL_SN_DSB_Pos))

#define MD_DSB_0_BITBAND                0x0
#define MD_DSB_1_BITBAND                0x1
#define MD_DSB_0                        ((uint32_t)(MD_DSB_0_BITBAND << BB_RWBBCNTL_MD_DSB_Pos))
#define MD_DSB_1                        ((uint32_t)(MD_DSB_1_BITBAND << BB_RWBBCNTL_MD_DSB_Pos))

#define SCAN_ABORT_0_BITBAND            0x0
#define SCAN_ABORT_1_BITBAND            0x1
#define SCAN_ABORT_0                    ((uint32_t)(SCAN_ABORT_0_BITBAND << BB_RWBBCNTL_SCAN_ABORT_Pos))
#define SCAN_ABORT_1                    ((uint32_t)(SCAN_ABORT_1_BITBAND << BB_RWBBCNTL_SCAN_ABORT_Pos))

#define ADVERT_ABORT_0_BITBAND          0x0
#define ADVERT_ABORT_1_BITBAND          0x1
#define ADVERT_ABORT_0                  ((uint32_t)(ADVERT_ABORT_0_BITBAND << BB_RWBBCNTL_ADVERT_ABORT_Pos))
#define ADVERT_ABORT_1                  ((uint32_t)(ADVERT_ABORT_1_BITBAND << BB_RWBBCNTL_ADVERT_ABORT_Pos))

#define RFTEST_ABORT_0_BITBAND          0x0
#define RFTEST_ABORT_1_BITBAND          0x1
#define RFTEST_ABORT_0                  ((uint32_t)(RFTEST_ABORT_0_BITBAND << BB_RWBBCNTL_RFTEST_ABORT_Pos))
#define RFTEST_ABORT_1                  ((uint32_t)(RFTEST_ABORT_1_BITBAND << BB_RWBBCNTL_RFTEST_ABORT_Pos))

#define SWINT_REQ_0_BITBAND             0x0
#define SWINT_REQ_1_BITBAND             0x1
#define SWINT_REQ_0                     ((uint32_t)(SWINT_REQ_0_BITBAND << BB_RWBBCNTL_SWINT_REQ_Pos))
#define SWINT_REQ_1                     ((uint32_t)(SWINT_REQ_1_BITBAND << BB_RWBBCNTL_SWINT_REQ_Pos))

#define REG_SOFT_RST_0_BITBAND          0x0
#define REG_SOFT_RST_1_BITBAND          0x1
#define REG_SOFT_RST_0                  ((uint32_t)(REG_SOFT_RST_0_BITBAND << BB_RWBBCNTL_REG_SOFT_RST_Pos))
#define REG_SOFT_RST_1                  ((uint32_t)(REG_SOFT_RST_1_BITBAND << BB_RWBBCNTL_REG_SOFT_RST_Pos))

#define MASTER_TGSOFT_RST_0_BITBAND     0x0
#define MASTER_TGSOFT_RST_1_BITBAND     0x1
#define MASTER_TGSOFT_RST_0             ((uint32_t)(MASTER_TGSOFT_RST_0_BITBAND << BB_RWBBCNTL_MASTER_TGSOFT_RST_Pos))
#define MASTER_TGSOFT_RST_1             ((uint32_t)(MASTER_TGSOFT_RST_1_BITBAND << BB_RWBBCNTL_MASTER_TGSOFT_RST_Pos))

#define MASTER_SOFT_RST_0_BITBAND       0x0
#define MASTER_SOFT_RST_1_BITBAND       0x1
#define MASTER_SOFT_RST_0               ((uint32_t)(MASTER_SOFT_RST_0_BITBAND << BB_RWBBCNTL_MASTER_SOFT_RST_Pos))
#define MASTER_SOFT_RST_1               ((uint32_t)(MASTER_SOFT_RST_1_BITBAND << BB_RWBBCNTL_MASTER_SOFT_RST_Pos))

/* BB_VERSION bit positions */
#define BB_VERSION_TYP_Pos              24
#define BB_VERSION_TYP_Mask             ((uint32_t)(0xFFU << BB_VERSION_TYP_Pos))
#define BB_VERSION_REL_Pos              16
#define BB_VERSION_REL_Mask             ((uint32_t)(0xFFU << BB_VERSION_REL_Pos))
#define BB_VERSION_UPG_Pos              8
#define BB_VERSION_UPG_Mask             ((uint32_t)(0xFFU << BB_VERSION_UPG_Pos))
#define BB_VERSION_BUILD_Pos            0
#define BB_VERSION_BUILD_Mask           ((uint32_t)(0xFFU << BB_VERSION_BUILD_Pos))

/* BB_VERSION sub-register and bit-band aliases */
typedef struct
{
    __I  uint8_t BUILD_BYTE;           
    __I  uint8_t UPG_BYTE;             
    __I  uint8_t REL_BYTE;             
    __I  uint8_t TYP_BYTE;             
} BB_VERSION_Type;

#define BB_VERSION_BASE                 0x40001504
#define BB_VERSION                      ((BB_VERSION_Type *) BB_VERSION_BASE)

/* BB_VERSION settings */
#define BUILD_1                         ((uint32_t)(0x1U << BB_VERSION_BUILD_Pos))

#define UPG_09                          ((uint32_t)(0x9U << BB_VERSION_UPG_Pos))

#define REL_0                           ((uint32_t)(0x0U << BB_VERSION_REL_Pos))

#define TYP_8                           ((uint32_t)(0x8U << BB_VERSION_TYP_Pos))

/* BB_VERSION sub-register settings */
#define BUILD_1_BYTE                    ((uint8_t)0x1U)

#define UPG_09_BYTE                     ((uint8_t)0x9U)

#define REL_0_BYTE                      ((uint8_t)0x0U)

#define TYP_8_BYTE                      ((uint8_t)0x8U)

/* BB_RWBLEBCONF bit positions */
#define BB_RWBLEBCONF_DMMODE_Pos        31
#define BB_RWBLEBCONF_ISOPORTNB_Pos     24
#define BB_RWBLEBCONF_ISOPORTNB_Mask    ((uint32_t)(0x3U << BB_RWBLEBCONF_ISOPORTNB_Pos))
#define BB_RWBLEBCONF_DECIPHER_Pos      23
#define BB_RWBLEBCONF_COEX_Pos          21
#define BB_RWBLEBCONF_RFIF_Pos          16
#define BB_RWBLEBCONF_RFIF_Mask         ((uint32_t)(0x1FU << BB_RWBLEBCONF_RFIF_Pos))
#define BB_RWBLEBCONF_USEDBG_Pos        15
#define BB_RWBLEBCONF_USECRYPT_Pos      14
#define BB_RWBLEBCONF_CLK_SEL_Pos       8
#define BB_RWBLEBCONF_CLK_SEL_Mask      ((uint32_t)(0x3FU << BB_RWBLEBCONF_CLK_SEL_Pos))
#define BB_RWBLEBCONF_INTMODE_Pos       7
#define BB_RWBLEBCONF_BUSTYPE_Pos       6
#define BB_RWBLEBCONF_DATA_WIDTH_Pos    5
#define BB_RWBLEBCONF_ADD_WIDTH_Pos     0
#define BB_RWBLEBCONF_ADD_WIDTH_Mask    ((uint32_t)(0x1FU << BB_RWBLEBCONF_ADD_WIDTH_Pos))

/* BB_RWBLEBCONF bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[5];
    __I  uint32_t DATA_WIDTH_ALIAS;     /* Processor bus width */
    __I  uint32_t BUSTYPE_ALIAS;        /* Processor bus type */
    __I  uint32_t INTMODE_ALIAS;        /* Interruption mode */
         uint32_t RESERVED1[6];
    __I  uint32_t USECRYPT_ALIAS;       /* AES-CCM encryption */
    __I  uint32_t USEDBG_ALIAS;         /* Diagnostic port */
         uint32_t RESERVED2[5];
    __I  uint32_t COEX_ALIAS;           /* Coexistence mechanism */
         uint32_t RESERVED3;
    __I  uint32_t DECIPHER_ALIAS;       /* AES deciphering present */
         uint32_t RESERVED4[7];
    __I  uint32_t DMMODE_ALIAS;         /* RW-BLE core dual mode */
} BB_RWBLEBCONF_Type;

#define BB_RWBLEBCONF_BASE              SYS_CALC_BITBAND(0x40001508, 0)
#define BB_RWBLEBCONF                   ((BB_RWBLEBCONF_Type *) BB_RWBLEBCONF_BASE)

/* BB_RWBLEBCONF settings */
#define ADD_WIDTH_14                    ((uint32_t)(0xEU << BB_RWBLEBCONF_ADD_WIDTH_Pos))

#define DATA_WIDTH_0_BITBAND            0x0
#define DATA_WIDTH_1_BITBAND            0x1
#define DATA_WIDTH_0                    ((uint32_t)(DATA_WIDTH_0_BITBAND << BB_RWBLEBCONF_DATA_WIDTH_Pos))
#define DATA_WIDTH_1                    ((uint32_t)(DATA_WIDTH_1_BITBAND << BB_RWBLEBCONF_DATA_WIDTH_Pos))

#define BUSTYPE_0_BITBAND               0x0
#define BUSTYPE_1_BITBAND               0x1
#define BUSTYPE_0                       ((uint32_t)(BUSTYPE_0_BITBAND << BB_RWBLEBCONF_BUSTYPE_Pos))
#define BUSTYPE_1                       ((uint32_t)(BUSTYPE_1_BITBAND << BB_RWBLEBCONF_BUSTYPE_Pos))

#define INTMODE_0_BITBAND               0x0
#define INTMODE_1_BITBAND               0x1
#define INTMODE_0                       ((uint32_t)(INTMODE_0_BITBAND << BB_RWBLEBCONF_INTMODE_Pos))
#define INTMODE_1                       ((uint32_t)(INTMODE_1_BITBAND << BB_RWBLEBCONF_INTMODE_Pos))

#define CLK_SEL_8                       ((uint32_t)(0x8U << BB_RWBLEBCONF_CLK_SEL_Pos))

#define USECRYPT_0_BITBAND              0x0
#define USECRYPT_1_BITBAND              0x1
#define USECRYPT_0                      ((uint32_t)(USECRYPT_0_BITBAND << BB_RWBLEBCONF_USECRYPT_Pos))
#define USECRYPT_1                      ((uint32_t)(USECRYPT_1_BITBAND << BB_RWBLEBCONF_USECRYPT_Pos))

#define USEDBG_0_BITBAND                0x0
#define USEDBG_1_BITBAND                0x1
#define USEDBG_0                        ((uint32_t)(USEDBG_0_BITBAND << BB_RWBLEBCONF_USEDBG_Pos))
#define USEDBG_1                        ((uint32_t)(USEDBG_1_BITBAND << BB_RWBLEBCONF_USEDBG_Pos))

#define RFIF_RIPPLE                     ((uint32_t)(0x0U << BB_RWBLEBCONF_RFIF_Pos))
#define RFIF_EXT                        ((uint32_t)(0x1U << BB_RWBLEBCONF_RFIF_Pos))
#define RFIF_ATLAS                      ((uint32_t)(0x2U << BB_RWBLEBCONF_RFIF_Pos))
#define RFIF_ICYTRX                     ((uint32_t)(0x3U << BB_RWBLEBCONF_RFIF_Pos))

#define COEX_0_BITBAND                  0x0
#define COEX_1_BITBAND                  0x1
#define COEX_0                          ((uint32_t)(COEX_0_BITBAND << BB_RWBLEBCONF_COEX_Pos))
#define COEX_1                          ((uint32_t)(COEX_1_BITBAND << BB_RWBLEBCONF_COEX_Pos))

#define DECIPHER_0_BITBAND              0x0
#define DECIPHER_1_BITBAND              0x1
#define DECIPHER_0                      ((uint32_t)(DECIPHER_0_BITBAND << BB_RWBLEBCONF_DECIPHER_Pos))
#define DECIPHER_1                      ((uint32_t)(DECIPHER_1_BITBAND << BB_RWBLEBCONF_DECIPHER_Pos))

#define NO_ISO_CH                       ((uint32_t)(0x0U << BB_RWBLEBCONF_ISOPORTNB_Pos))
#define ONE_ISO_CH                      ((uint32_t)(0x1U << BB_RWBLEBCONF_ISOPORTNB_Pos))
#define TWO_ISO_CH                      ((uint32_t)(0x2U << BB_RWBLEBCONF_ISOPORTNB_Pos))
#define THREE_ISO_CH                    ((uint32_t)(0x3U << BB_RWBLEBCONF_ISOPORTNB_Pos))

#define DMMODE_0_BITBAND                0x0
#define DMMODE_1_BITBAND                0x1
#define DMMODE_0                        ((uint32_t)(DMMODE_0_BITBAND << BB_RWBLEBCONF_DMMODE_Pos))
#define DMMODE_1                        ((uint32_t)(DMMODE_1_BITBAND << BB_RWBLEBCONF_DMMODE_Pos))

/* BB_INTCNTL bit positions */
#define BB_INTCNTL_CSCNTDEVMSK_Pos      15
#define BB_INTCNTL_AUDIOINT2MSK_Pos     12
#define BB_INTCNTL_AUDIOINT1MSK_Pos     11
#define BB_INTCNTL_AUDIOINT0MSK_Pos     10
#define BB_INTCNTL_SWINTMSK_Pos         9
#define BB_INTCNTL_EVENTAPFAINTMSK_Pos  8
#define BB_INTCNTL_FINETGTIMINTMSK_Pos  7
#define BB_INTCNTL_GROSSTGTIMINTMSK_Pos 6
#define BB_INTCNTL_ERRORINTMSK_Pos      5
#define BB_INTCNTL_CRYPTINTMSK_Pos      4
#define BB_INTCNTL_EVENTINTMSK_Pos      3
#define BB_INTCNTL_SLPINTMSK_Pos        2
#define BB_INTCNTL_RXINTMSK_Pos         1
#define BB_INTCNTL_CSCNTINTMSK_Pos      0

/* BB_INTCNTL bit-band aliases */
typedef struct
{
    __IO uint32_t CSCNTINTMSK_ALIAS;    /* 625us base time interrupt mask */
    __IO uint32_t RXINTMSK_ALIAS;       /* Rx interrupt mask */
    __IO uint32_t SLPINTMSK_ALIAS;      /* Sleep mode interrupt mask */
    __IO uint32_t EVENTINTMSK_ALIAS;    /* End of event interrupt mask */
    __IO uint32_t CRYPTINTMSK_ALIAS;    /* Encryption engine interrupt mask */
    __IO uint32_t ERRORINTMSK_ALIAS;    /* Error interrupt mask */
    __IO uint32_t GROSSTGTIMINTMSK_ALIAS;/* Gross target timer mask */
    __IO uint32_t FINETGTIMINTMSK_ALIAS;/* Fine target timer mask */
    __IO uint32_t EVENTAPFAINTMSK_ALIAS;/* End of event / anticipated pre-fetch abort interrupt mask */
    __IO uint32_t SWINTMSK_ALIAS;       /* SW triggered interrupt mask */
    __IO uint32_t AUDIOINT0MSK_ALIAS;   /* Audio channel 0 interrupt mask */
    __IO uint32_t AUDIOINT1MSK_ALIAS;   /* Audio channel 1 interrupt mask */
    __IO uint32_t AUDIOINT2MSK_ALIAS;   /* Audio channel 2 interrupt mask */
         uint32_t RESERVED0[2];
    __IO uint32_t CSCNTDEVMSK_ALIAS;    /* CSCNT interrupt mask during event allowing to enable CSCNT interrupt generation during events */
} BB_INTCNTL_Type;

#define BB_INTCNTL_BASE                 SYS_CALC_BITBAND(0x4000150C, 0)
#define BB_INTCNTL                      ((BB_INTCNTL_Type *) BB_INTCNTL_BASE)

/* BB_INTCNTL settings */
#define CSCNTINTMSK_0_BITBAND           0x0
#define CSCNTINTMSK_1_BITBAND           0x1
#define CSCNTINTMSK_0                   ((uint32_t)(CSCNTINTMSK_0_BITBAND << BB_INTCNTL_CSCNTINTMSK_Pos))
#define CSCNTINTMSK_1                   ((uint32_t)(CSCNTINTMSK_1_BITBAND << BB_INTCNTL_CSCNTINTMSK_Pos))

#define RXINTMSK_0_BITBAND              0x0
#define RXINTMSK_1_BITBAND              0x1
#define RXINTMSK_0                      ((uint32_t)(RXINTMSK_0_BITBAND << BB_INTCNTL_RXINTMSK_Pos))
#define RXINTMSK_1                      ((uint32_t)(RXINTMSK_1_BITBAND << BB_INTCNTL_RXINTMSK_Pos))

#define SLPINTMSK_0_BITBAND             0x0
#define SLPINTMSK_1_BITBAND             0x1
#define SLPINTMSK_0                     ((uint32_t)(SLPINTMSK_0_BITBAND << BB_INTCNTL_SLPINTMSK_Pos))
#define SLPINTMSK_1                     ((uint32_t)(SLPINTMSK_1_BITBAND << BB_INTCNTL_SLPINTMSK_Pos))

#define EVENTINTMSK_0_BITBAND           0x0
#define EVENTINTMSK_1_BITBAND           0x1
#define EVENTINTMSK_0                   ((uint32_t)(EVENTINTMSK_0_BITBAND << BB_INTCNTL_EVENTINTMSK_Pos))
#define EVENTINTMSK_1                   ((uint32_t)(EVENTINTMSK_1_BITBAND << BB_INTCNTL_EVENTINTMSK_Pos))

#define CRYPTINTMSK_0_BITBAND           0x0
#define CRYPTINTMSK_1_BITBAND           0x1
#define CRYPTINTMSK_0                   ((uint32_t)(CRYPTINTMSK_0_BITBAND << BB_INTCNTL_CRYPTINTMSK_Pos))
#define CRYPTINTMSK_1                   ((uint32_t)(CRYPTINTMSK_1_BITBAND << BB_INTCNTL_CRYPTINTMSK_Pos))

#define ERRORINTMSK_0_BITBAND           0x0
#define ERRORINTMSK_1_BITBAND           0x1
#define ERRORINTMSK_0                   ((uint32_t)(ERRORINTMSK_0_BITBAND << BB_INTCNTL_ERRORINTMSK_Pos))
#define ERRORINTMSK_1                   ((uint32_t)(ERRORINTMSK_1_BITBAND << BB_INTCNTL_ERRORINTMSK_Pos))

#define GROSSTGTIMINTMSK_0_BITBAND      0x0
#define GROSSTGTIMINTMSK_1_BITBAND      0x1
#define GROSSTGTIMINTMSK_0              ((uint32_t)(GROSSTGTIMINTMSK_0_BITBAND << BB_INTCNTL_GROSSTGTIMINTMSK_Pos))
#define GROSSTGTIMINTMSK_1              ((uint32_t)(GROSSTGTIMINTMSK_1_BITBAND << BB_INTCNTL_GROSSTGTIMINTMSK_Pos))

#define FINETGTIMINTMSK_0_BITBAND       0x0
#define FINETGTIMINTMSK_1_BITBAND       0x1
#define FINETGTIMINTMSK_0               ((uint32_t)(FINETGTIMINTMSK_0_BITBAND << BB_INTCNTL_FINETGTIMINTMSK_Pos))
#define FINETGTIMINTMSK_1               ((uint32_t)(FINETGTIMINTMSK_1_BITBAND << BB_INTCNTL_FINETGTIMINTMSK_Pos))

#define EVENTAPFAINTMSK_0_BITBAND       0x0
#define EVENTAPFAINTMSK_1_BITBAND       0x1
#define EVENTAPFAINTMSK_0               ((uint32_t)(EVENTAPFAINTMSK_0_BITBAND << BB_INTCNTL_EVENTAPFAINTMSK_Pos))
#define EVENTAPFAINTMSK_1               ((uint32_t)(EVENTAPFAINTMSK_1_BITBAND << BB_INTCNTL_EVENTAPFAINTMSK_Pos))

#define SWINTMSK_0_BITBAND              0x0
#define SWINTMSK_1_BITBAND              0x1
#define SWINTMSK_0                      ((uint32_t)(SWINTMSK_0_BITBAND << BB_INTCNTL_SWINTMSK_Pos))
#define SWINTMSK_1                      ((uint32_t)(SWINTMSK_1_BITBAND << BB_INTCNTL_SWINTMSK_Pos))

#define AUDIOINT0MSK_0_BITBAND          0x0
#define AUDIOINT0MSK_1_BITBAND          0x1
#define AUDIOINT0MSK_0                  ((uint32_t)(AUDIOINT0MSK_0_BITBAND << BB_INTCNTL_AUDIOINT0MSK_Pos))
#define AUDIOINT0MSK_1                  ((uint32_t)(AUDIOINT0MSK_1_BITBAND << BB_INTCNTL_AUDIOINT0MSK_Pos))

#define AUDIOINT1MSK_0_BITBAND          0x0
#define AUDIOINT1MSK_1_BITBAND          0x1
#define AUDIOINT1MSK_0                  ((uint32_t)(AUDIOINT1MSK_0_BITBAND << BB_INTCNTL_AUDIOINT1MSK_Pos))
#define AUDIOINT1MSK_1                  ((uint32_t)(AUDIOINT1MSK_1_BITBAND << BB_INTCNTL_AUDIOINT1MSK_Pos))

#define AUDIOINT2MSK_0_BITBAND          0x0
#define AUDIOINT2MSK_1_BITBAND          0x1
#define AUDIOINT2MSK_0                  ((uint32_t)(AUDIOINT2MSK_0_BITBAND << BB_INTCNTL_AUDIOINT2MSK_Pos))
#define AUDIOINT2MSK_1                  ((uint32_t)(AUDIOINT2MSK_1_BITBAND << BB_INTCNTL_AUDIOINT2MSK_Pos))

#define CSCNTDEVMSK_0_BITBAND           0x0
#define CSCNTDEVMSK_1_BITBAND           0x1
#define CSCNTDEVMSK_0                   ((uint32_t)(CSCNTDEVMSK_0_BITBAND << BB_INTCNTL_CSCNTDEVMSK_Pos))
#define CSCNTDEVMSK_1                   ((uint32_t)(CSCNTDEVMSK_1_BITBAND << BB_INTCNTL_CSCNTDEVMSK_Pos))

/* BB_INTSTAT bit positions */
#define BB_INTSTAT_AUDIOINT2STAT_Pos    12
#define BB_INTSTAT_AUDIOINT1STAT_Pos    11
#define BB_INTSTAT_AUDIOINT0STAT_Pos    10
#define BB_INTSTAT_SWINTSTAT_Pos        9
#define BB_INTSTAT_EVENTAPFAINTSTAT_Pos 8
#define BB_INTSTAT_FINETGTIMINTSTAT_Pos 7
#define BB_INTSTAT_GROSSTGTIMINTSTAT_Pos 6
#define BB_INTSTAT_ERRORINTSTAT_Pos     5
#define BB_INTSTAT_CRYPTINTSTAT_Pos     4
#define BB_INTSTAT_EVENTINTSTAT_Pos     3
#define BB_INTSTAT_SLPINTSTAT_Pos       2
#define BB_INTSTAT_RXINTSTAT_Pos        1
#define BB_INTSTAT_CSCNTINTSTAT_Pos     0

/* BB_INTSTAT bit-band aliases */
typedef struct
{
    __I  uint32_t CSCNTINTSTAT_ALIAS;   /* Masked 625us base time reference interrupt status */
    __I  uint32_t RXINTSTAT_ALIAS;      /* Masked packet reception interrupt status */
    __I  uint32_t SLPINTSTAT_ALIAS;     /* Masked sleep interrupt status */
    __I  uint32_t EVENTINTSTAT_ALIAS;   /* Masked end of event interrupt status */
    __I  uint32_t CRYPTINTSTAT_ALIAS;   /* Masked encryption engine interrupt status */
    __I  uint32_t ERRORINTSTAT_ALIAS;   /* Masked error interrupt status */
    __I  uint32_t GROSSTGTIMINTSTAT_ALIAS;/* Masked gross target timer interrupt status */
    __I  uint32_t FINETGTIMINTSTAT_ALIAS;/* Masked fine target timer error interrupt status */
    __I  uint32_t EVENTAPFAINTSTAT_ALIAS;/* End of event / anticipated pre-fetch abort interrupt status */
    __I  uint32_t SWINTSTAT_ALIAS;      /* SW triggered interrupt status */
    __I  uint32_t AUDIOINT0STAT_ALIAS;  /* Audio channel 0 interrupt status */
    __I  uint32_t AUDIOINT1STAT_ALIAS;  /* Audio channel 1 interrupt status */
    __I  uint32_t AUDIOINT2STAT_ALIAS;  /* Audio channel 2 interrupt status */
} BB_INTSTAT_Type;

#define BB_INTSTAT_BASE                 SYS_CALC_BITBAND(0x40001510, 0)
#define BB_INTSTAT                      ((BB_INTSTAT_Type *) BB_INTSTAT_BASE)

/* BB_INTSTAT settings */
#define CSCNTINTSTAT_0_BITBAND          0x0
#define CSCNTINTSTAT_1_BITBAND          0x1
#define CSCNTINTSTAT_0                  ((uint32_t)(CSCNTINTSTAT_0_BITBAND << BB_INTSTAT_CSCNTINTSTAT_Pos))
#define CSCNTINTSTAT_1                  ((uint32_t)(CSCNTINTSTAT_1_BITBAND << BB_INTSTAT_CSCNTINTSTAT_Pos))

#define RXINTSTAT_0_BITBAND             0x0
#define RXINTSTAT_1_BITBAND             0x1
#define RXINTSTAT_0                     ((uint32_t)(RXINTSTAT_0_BITBAND << BB_INTSTAT_RXINTSTAT_Pos))
#define RXINTSTAT_1                     ((uint32_t)(RXINTSTAT_1_BITBAND << BB_INTSTAT_RXINTSTAT_Pos))

#define SLPINTSTAT_0_BITBAND            0x0
#define SLPINTSTAT_1_BITBAND            0x1
#define SLPINTSTAT_0                    ((uint32_t)(SLPINTSTAT_0_BITBAND << BB_INTSTAT_SLPINTSTAT_Pos))
#define SLPINTSTAT_1                    ((uint32_t)(SLPINTSTAT_1_BITBAND << BB_INTSTAT_SLPINTSTAT_Pos))

#define EVENTINTSTAT_0_BITBAND          0x0
#define EVENTINTSTAT_1_BITBAND          0x1
#define EVENTINTSTAT_0                  ((uint32_t)(EVENTINTSTAT_0_BITBAND << BB_INTSTAT_EVENTINTSTAT_Pos))
#define EVENTINTSTAT_1                  ((uint32_t)(EVENTINTSTAT_1_BITBAND << BB_INTSTAT_EVENTINTSTAT_Pos))

#define CRYPTINTSTAT_0_BITBAND          0x0
#define CRYPTINTSTAT_1_BITBAND          0x1
#define CRYPTINTSTAT_0                  ((uint32_t)(CRYPTINTSTAT_0_BITBAND << BB_INTSTAT_CRYPTINTSTAT_Pos))
#define CRYPTINTSTAT_1                  ((uint32_t)(CRYPTINTSTAT_1_BITBAND << BB_INTSTAT_CRYPTINTSTAT_Pos))

#define ERRORINTSTAT_0_BITBAND          0x0
#define ERRORINTSTAT_1_BITBAND          0x1
#define ERRORINTSTAT_0                  ((uint32_t)(ERRORINTSTAT_0_BITBAND << BB_INTSTAT_ERRORINTSTAT_Pos))
#define ERRORINTSTAT_1                  ((uint32_t)(ERRORINTSTAT_1_BITBAND << BB_INTSTAT_ERRORINTSTAT_Pos))

#define GROSSTGTIMINTSTAT_0_BITBAND     0x0
#define GROSSTGTIMINTSTAT_1_BITBAND     0x1
#define GROSSTGTIMINTSTAT_0             ((uint32_t)(GROSSTGTIMINTSTAT_0_BITBAND << BB_INTSTAT_GROSSTGTIMINTSTAT_Pos))
#define GROSSTGTIMINTSTAT_1             ((uint32_t)(GROSSTGTIMINTSTAT_1_BITBAND << BB_INTSTAT_GROSSTGTIMINTSTAT_Pos))

#define FINETGTIMINTSTAT_0_BITBAND      0x0
#define FINETGTIMINTSTAT_1_BITBAND      0x1
#define FINETGTIMINTSTAT_0              ((uint32_t)(FINETGTIMINTSTAT_0_BITBAND << BB_INTSTAT_FINETGTIMINTSTAT_Pos))
#define FINETGTIMINTSTAT_1              ((uint32_t)(FINETGTIMINTSTAT_1_BITBAND << BB_INTSTAT_FINETGTIMINTSTAT_Pos))

#define EVENTAPFAINTSTAT_0_BITBAND      0x0
#define EVENTAPFAINTSTAT_1_BITBAND      0x1
#define EVENTAPFAINTSTAT_0              ((uint32_t)(EVENTAPFAINTSTAT_0_BITBAND << BB_INTSTAT_EVENTAPFAINTSTAT_Pos))
#define EVENTAPFAINTSTAT_1              ((uint32_t)(EVENTAPFAINTSTAT_1_BITBAND << BB_INTSTAT_EVENTAPFAINTSTAT_Pos))

#define SWINTSTAT_0_BITBAND             0x0
#define SWINTSTAT_1_BITBAND             0x1
#define SWINTSTAT_0                     ((uint32_t)(SWINTSTAT_0_BITBAND << BB_INTSTAT_SWINTSTAT_Pos))
#define SWINTSTAT_1                     ((uint32_t)(SWINTSTAT_1_BITBAND << BB_INTSTAT_SWINTSTAT_Pos))

#define AUDIOINT0STAT_0_BITBAND         0x0
#define AUDIOINT0STAT_1_BITBAND         0x1
#define AUDIOINT0STAT_0                 ((uint32_t)(AUDIOINT0STAT_0_BITBAND << BB_INTSTAT_AUDIOINT0STAT_Pos))
#define AUDIOINT0STAT_1                 ((uint32_t)(AUDIOINT0STAT_1_BITBAND << BB_INTSTAT_AUDIOINT0STAT_Pos))

#define AUDIOINT1STAT_0_BITBAND         0x0
#define AUDIOINT1STAT_1_BITBAND         0x1
#define AUDIOINT1STAT_0                 ((uint32_t)(AUDIOINT1STAT_0_BITBAND << BB_INTSTAT_AUDIOINT1STAT_Pos))
#define AUDIOINT1STAT_1                 ((uint32_t)(AUDIOINT1STAT_1_BITBAND << BB_INTSTAT_AUDIOINT1STAT_Pos))

#define AUDIOINT2STAT_0_BITBAND         0x0
#define AUDIOINT2STAT_1_BITBAND         0x1
#define AUDIOINT2STAT_0                 ((uint32_t)(AUDIOINT2STAT_0_BITBAND << BB_INTSTAT_AUDIOINT2STAT_Pos))
#define AUDIOINT2STAT_1                 ((uint32_t)(AUDIOINT2STAT_1_BITBAND << BB_INTSTAT_AUDIOINT2STAT_Pos))

/* BB_INTRAWSTAT bit positions */
#define BB_INTRAWSTAT_AUDIOINT2RAWSTAT_Pos 12
#define BB_INTRAWSTAT_AUDIOINT1RAWSTAT_Pos 11
#define BB_INTRAWSTAT_AUDIOINT0RAWSTAT_Pos 10
#define BB_INTRAWSTAT_SWINTRAWSTAT_Pos  9
#define BB_INTRAWSTAT_EVENTAPFAINTRAWSTAT_Pos 8
#define BB_INTRAWSTAT_FINETGTIMINTRAWSTAT_Pos 7
#define BB_INTRAWSTAT_GROSSTGTIMINTRAWSTAT_Pos 6
#define BB_INTRAWSTAT_ERRORINTRAWSTAT_Pos 5
#define BB_INTRAWSTAT_CRYPTINTRAWSTAT_Pos 4
#define BB_INTRAWSTAT_EVENTINTRAWSTAT_Pos 3
#define BB_INTRAWSTAT_SLPINTRAWSTAT_Pos 2
#define BB_INTRAWSTAT_RXINTRAWSTAT_Pos  1
#define BB_INTRAWSTAT_CSCNTINTRAWSTAT_Pos 0

/* BB_INTRAWSTAT bit-band aliases */
typedef struct
{
    __I  uint32_t CSCNTINTRAWSTAT_ALIAS;/* Masked 625us base time reference interrupt raw status */
    __I  uint32_t RXINTRAWSTAT_ALIAS;   /* Masked packet reception interrupt raw status */
    __I  uint32_t SLPINTRAWSTAT_ALIAS;  /* Masked sleep interrupt raw status */
    __I  uint32_t EVENTINTRAWSTAT_ALIAS;/* Masked end of event interrupt raw status */
    __I  uint32_t CRYPTINTRAWSTAT_ALIAS;/* Masked encryption engine interrupt raw status */
    __I  uint32_t ERRORINTRAWSTAT_ALIAS;/* Masked error interrupt raw status */
    __I  uint32_t GROSSTGTIMINTRAWSTAT_ALIAS;/* Masked gross target timer interrupt raw status */
    __I  uint32_t FINETGTIMINTRAWSTAT_ALIAS;/* Masked fine target timer error interrupt raw status */
    __I  uint32_t EVENTAPFAINTRAWSTAT_ALIAS;/* End of event / anticipated pre-fetch abort interrupt raw status */
    __I  uint32_t SWINTRAWSTAT_ALIAS;   /* SW triggered interrupt raw status */
    __I  uint32_t AUDIOINT0RAWSTAT_ALIAS;/* Audio channel 0 interrupt raw status */
    __I  uint32_t AUDIOINT1RAWSTAT_ALIAS;/* Audio channel 1 interrupt raw status */
    __I  uint32_t AUDIOINT2RAWSTAT_ALIAS;/* Audio channel 2 interrupt raw status */
} BB_INTRAWSTAT_Type;

#define BB_INTRAWSTAT_BASE              SYS_CALC_BITBAND(0x40001514, 0)
#define BB_INTRAWSTAT                   ((BB_INTRAWSTAT_Type *) BB_INTRAWSTAT_BASE)

/* BB_INTRAWSTAT settings */
#define CSCNTINTRAWSTAT_0_BITBAND       0x0
#define CSCNTINTRAWSTAT_1_BITBAND       0x1
#define CSCNTINTRAWSTAT_0               ((uint32_t)(CSCNTINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_CSCNTINTRAWSTAT_Pos))
#define CSCNTINTRAWSTAT_1               ((uint32_t)(CSCNTINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_CSCNTINTRAWSTAT_Pos))

#define RXINTRAWSTAT_0_BITBAND          0x0
#define RXINTRAWSTAT_1_BITBAND          0x1
#define RXINTRAWSTAT_0                  ((uint32_t)(RXINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_RXINTRAWSTAT_Pos))
#define RXINTRAWSTAT_1                  ((uint32_t)(RXINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_RXINTRAWSTAT_Pos))

#define SLPINTRAWSTAT_0_BITBAND         0x0
#define SLPINTRAWSTAT_1_BITBAND         0x1
#define SLPINTRAWSTAT_0                 ((uint32_t)(SLPINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_SLPINTRAWSTAT_Pos))
#define SLPINTRAWSTAT_1                 ((uint32_t)(SLPINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_SLPINTRAWSTAT_Pos))

#define EVENTINTRAWSTAT_0_BITBAND       0x0
#define EVENTINTRAWSTAT_1_BITBAND       0x1
#define EVENTINTRAWSTAT_0               ((uint32_t)(EVENTINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_EVENTINTRAWSTAT_Pos))
#define EVENTINTRAWSTAT_1               ((uint32_t)(EVENTINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_EVENTINTRAWSTAT_Pos))

#define CRYPTINTRAWSTAT_0_BITBAND       0x0
#define CRYPTINTRAWSTAT_1_BITBAND       0x1
#define CRYPTINTRAWSTAT_0               ((uint32_t)(CRYPTINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_CRYPTINTRAWSTAT_Pos))
#define CRYPTINTRAWSTAT_1               ((uint32_t)(CRYPTINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_CRYPTINTRAWSTAT_Pos))

#define ERRORINTRAWSTAT_0_BITBAND       0x0
#define ERRORINTRAWSTAT_1_BITBAND       0x1
#define ERRORINTRAWSTAT_0               ((uint32_t)(ERRORINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_ERRORINTRAWSTAT_Pos))
#define ERRORINTRAWSTAT_1               ((uint32_t)(ERRORINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_ERRORINTRAWSTAT_Pos))

#define GROSSTGTIMINTRAWSTAT_0_BITBAND  0x0
#define GROSSTGTIMINTRAWSTAT_1_BITBAND  0x1
#define GROSSTGTIMINTRAWSTAT_0          ((uint32_t)(GROSSTGTIMINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_GROSSTGTIMINTRAWSTAT_Pos))
#define GROSSTGTIMINTRAWSTAT_1          ((uint32_t)(GROSSTGTIMINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_GROSSTGTIMINTRAWSTAT_Pos))

#define FINETGTIMINTRAWSTAT_0_BITBAND   0x0
#define FINETGTIMINTRAWSTAT_1_BITBAND   0x1
#define FINETGTIMINTRAWSTAT_0           ((uint32_t)(FINETGTIMINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_FINETGTIMINTRAWSTAT_Pos))
#define FINETGTIMINTRAWSTAT_1           ((uint32_t)(FINETGTIMINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_FINETGTIMINTRAWSTAT_Pos))

#define EVENTAPFAINTRAWSTAT_0_BITBAND   0x0
#define EVENTAPFAINTRAWSTAT_1_BITBAND   0x1
#define EVENTAPFAINTRAWSTAT_0           ((uint32_t)(EVENTAPFAINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_EVENTAPFAINTRAWSTAT_Pos))
#define EVENTAPFAINTRAWSTAT_1           ((uint32_t)(EVENTAPFAINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_EVENTAPFAINTRAWSTAT_Pos))

#define SWINTRAWSTAT_0_BITBAND          0x0
#define SWINTRAWSTAT_1_BITBAND          0x1
#define SWINTRAWSTAT_0                  ((uint32_t)(SWINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_SWINTRAWSTAT_Pos))
#define SWINTRAWSTAT_1                  ((uint32_t)(SWINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_SWINTRAWSTAT_Pos))

#define AUDIOINT0RAWSTAT_0_BITBAND      0x0
#define AUDIOINT0RAWSTAT_1_BITBAND      0x1
#define AUDIOINT0RAWSTAT_0              ((uint32_t)(AUDIOINT0RAWSTAT_0_BITBAND << BB_INTRAWSTAT_AUDIOINT0RAWSTAT_Pos))
#define AUDIOINT0RAWSTAT_1              ((uint32_t)(AUDIOINT0RAWSTAT_1_BITBAND << BB_INTRAWSTAT_AUDIOINT0RAWSTAT_Pos))

#define AUDIOINT1RAWSTAT_0_BITBAND      0x0
#define AUDIOINT1RAWSTAT_1_BITBAND      0x1
#define AUDIOINT1RAWSTAT_0              ((uint32_t)(AUDIOINT1RAWSTAT_0_BITBAND << BB_INTRAWSTAT_AUDIOINT1RAWSTAT_Pos))
#define AUDIOINT1RAWSTAT_1              ((uint32_t)(AUDIOINT1RAWSTAT_1_BITBAND << BB_INTRAWSTAT_AUDIOINT1RAWSTAT_Pos))

#define AUDIOINT2RAWSTAT_0_BITBAND      0x0
#define AUDIOINT2RAWSTAT_1_BITBAND      0x1
#define AUDIOINT2RAWSTAT_0              ((uint32_t)(AUDIOINT2RAWSTAT_0_BITBAND << BB_INTRAWSTAT_AUDIOINT2RAWSTAT_Pos))
#define AUDIOINT2RAWSTAT_1              ((uint32_t)(AUDIOINT2RAWSTAT_1_BITBAND << BB_INTRAWSTAT_AUDIOINT2RAWSTAT_Pos))

/* BB_INTACK bit positions */
#define BB_INTACK_AUDIOINT2ACK_Pos      12
#define BB_INTACK_AUDIOINT1ACK_Pos      11
#define BB_INTACK_AUDIOINT0ACK_Pos      10
#define BB_INTACK_SWINTACK_Pos          9
#define BB_INTACK_EVENTAPFAINTACK_Pos   8
#define BB_INTACK_FINETGTIMINTACK_Pos   7
#define BB_INTACK_GROSSTGTIMINTACK_Pos  6
#define BB_INTACK_ERRORINTACK_Pos       5
#define BB_INTACK_CRYPTINTACK_Pos       4
#define BB_INTACK_EVENTINTACK_Pos       3
#define BB_INTACK_SLPINTACK_Pos         2
#define BB_INTACK_RXINTACK_Pos          1
#define BB_INTACK_CSCNTINTACK_Pos       0

/* BB_INTACK bit-band aliases */
typedef struct
{
    __O  uint32_t CSCNTINTACK_ALIAS;    /* 625us base time reference interrupt acknowledgment bit */
    __O  uint32_t RXINTACK_ALIAS;       /* Packet reception interrupt acknowledgment bit */
    __O  uint32_t SLPINTACK_ALIAS;      /* End of deep sleep interrupt acknowledgment bit */
    __O  uint32_t EVENTINTACK_ALIAS;    /* End of event interrupt acknowledgment bit */
    __O  uint32_t CRYPTINTACK_ALIAS;    /* Encryption engine interrupt acknowledgement bit */
    __O  uint32_t ERRORINTACK_ALIAS;    /* Error interrupt acknowledgement bit */
    __O  uint32_t GROSSTGTIMINTACK_ALIAS;/* Gross target timer interrupt acknowledgement bit */
    __O  uint32_t FINETGTIMINTACK_ALIAS;/* Fine target timer interrupt acknowledgement bit */
    __O  uint32_t EVENTAPFAINTACK_ALIAS;/* End of event / anticipated pre-fetch abort interrupt acknowledgement bit */
    __O  uint32_t SWINTACK_ALIAS;       /* SW triggered interrupt acknowledgement bit */
    __I  uint32_t AUDIOINT0ACK_ALIAS;   /* Audio channel 0 interrupt acknowledgement bit */
    __I  uint32_t AUDIOINT1ACK_ALIAS;   /* Audio channel 1 interrupt acknowledgement bit */
    __I  uint32_t AUDIOINT2ACK_ALIAS;   /* Audio channel 2 interrupt acknowledgement bit */
} BB_INTACK_Type;

#define BB_INTACK_BASE                  SYS_CALC_BITBAND(0x40001518, 0)
#define BB_INTACK                       ((BB_INTACK_Type *) BB_INTACK_BASE)

/* BB_INTACK settings */
#define CSCNTINTACK_0_BITBAND           0x0
#define CSCNTINTACK_1_BITBAND           0x1
#define CSCNTINTACK_0                   ((uint32_t)(CSCNTINTACK_0_BITBAND << BB_INTACK_CSCNTINTACK_Pos))
#define CSCNTINTACK_1                   ((uint32_t)(CSCNTINTACK_1_BITBAND << BB_INTACK_CSCNTINTACK_Pos))

#define RXINTACK_0_BITBAND              0x0
#define RXINTACK_1_BITBAND              0x1
#define RXINTACK_0                      ((uint32_t)(RXINTACK_0_BITBAND << BB_INTACK_RXINTACK_Pos))
#define RXINTACK_1                      ((uint32_t)(RXINTACK_1_BITBAND << BB_INTACK_RXINTACK_Pos))

#define SLPINTACK_0_BITBAND             0x0
#define SLPINTACK_1_BITBAND             0x1
#define SLPINTACK_0                     ((uint32_t)(SLPINTACK_0_BITBAND << BB_INTACK_SLPINTACK_Pos))
#define SLPINTACK_1                     ((uint32_t)(SLPINTACK_1_BITBAND << BB_INTACK_SLPINTACK_Pos))

#define EVENTINTACK_0_BITBAND           0x0
#define EVENTINTACK_1_BITBAND           0x1
#define EVENTINTACK_0                   ((uint32_t)(EVENTINTACK_0_BITBAND << BB_INTACK_EVENTINTACK_Pos))
#define EVENTINTACK_1                   ((uint32_t)(EVENTINTACK_1_BITBAND << BB_INTACK_EVENTINTACK_Pos))

#define CRYPTINTACK_0_BITBAND           0x0
#define CRYPTINTACK_1_BITBAND           0x1
#define CRYPTINTACK_0                   ((uint32_t)(CRYPTINTACK_0_BITBAND << BB_INTACK_CRYPTINTACK_Pos))
#define CRYPTINTACK_1                   ((uint32_t)(CRYPTINTACK_1_BITBAND << BB_INTACK_CRYPTINTACK_Pos))

#define ERRORINTACK_0_BITBAND           0x0
#define ERRORINTACK_1_BITBAND           0x1
#define ERRORINTACK_0                   ((uint32_t)(ERRORINTACK_0_BITBAND << BB_INTACK_ERRORINTACK_Pos))
#define ERRORINTACK_1                   ((uint32_t)(ERRORINTACK_1_BITBAND << BB_INTACK_ERRORINTACK_Pos))

#define GROSSTGTIMINTACK_0_BITBAND      0x0
#define GROSSTGTIMINTACK_1_BITBAND      0x1
#define GROSSTGTIMINTACK_0              ((uint32_t)(GROSSTGTIMINTACK_0_BITBAND << BB_INTACK_GROSSTGTIMINTACK_Pos))
#define GROSSTGTIMINTACK_1              ((uint32_t)(GROSSTGTIMINTACK_1_BITBAND << BB_INTACK_GROSSTGTIMINTACK_Pos))

#define FINETGTIMINTACK_0_BITBAND       0x0
#define FINETGTIMINTACK_1_BITBAND       0x1
#define FINETGTIMINTACK_0               ((uint32_t)(FINETGTIMINTACK_0_BITBAND << BB_INTACK_FINETGTIMINTACK_Pos))
#define FINETGTIMINTACK_1               ((uint32_t)(FINETGTIMINTACK_1_BITBAND << BB_INTACK_FINETGTIMINTACK_Pos))

#define EVENTAPFAINTACK_0_BITBAND       0x0
#define EVENTAPFAINTACK_1_BITBAND       0x1
#define EVENTAPFAINTACK_0               ((uint32_t)(EVENTAPFAINTACK_0_BITBAND << BB_INTACK_EVENTAPFAINTACK_Pos))
#define EVENTAPFAINTACK_1               ((uint32_t)(EVENTAPFAINTACK_1_BITBAND << BB_INTACK_EVENTAPFAINTACK_Pos))

#define SWINTACK_0_BITBAND              0x0
#define SWINTACK_1_BITBAND              0x1
#define SWINTACK_0                      ((uint32_t)(SWINTACK_0_BITBAND << BB_INTACK_SWINTACK_Pos))
#define SWINTACK_1                      ((uint32_t)(SWINTACK_1_BITBAND << BB_INTACK_SWINTACK_Pos))

#define AUDIOINT0ACK_0_BITBAND          0x0
#define AUDIOINT0ACK_1_BITBAND          0x1
#define AUDIOINT0ACK_0                  ((uint32_t)(AUDIOINT0ACK_0_BITBAND << BB_INTACK_AUDIOINT0ACK_Pos))
#define AUDIOINT0ACK_1                  ((uint32_t)(AUDIOINT0ACK_1_BITBAND << BB_INTACK_AUDIOINT0ACK_Pos))

#define AUDIOINT1ACK_0_BITBAND          0x0
#define AUDIOINT1ACK_1_BITBAND          0x1
#define AUDIOINT1ACK_0                  ((uint32_t)(AUDIOINT1ACK_0_BITBAND << BB_INTACK_AUDIOINT1ACK_Pos))
#define AUDIOINT1ACK_1                  ((uint32_t)(AUDIOINT1ACK_1_BITBAND << BB_INTACK_AUDIOINT1ACK_Pos))

#define AUDIOINT2ACK_0_BITBAND          0x0
#define AUDIOINT2ACK_1_BITBAND          0x1
#define AUDIOINT2ACK_0                  ((uint32_t)(AUDIOINT2ACK_0_BITBAND << BB_INTACK_AUDIOINT2ACK_Pos))
#define AUDIOINT2ACK_1                  ((uint32_t)(AUDIOINT2ACK_1_BITBAND << BB_INTACK_AUDIOINT2ACK_Pos))

/* BB_BASETIMECNT bit positions */
#define BB_BASETIMECNT_SAMP_Pos         31
#define BB_BASETIMECNT_BASETIMECNT_Pos  0
#define BB_BASETIMECNT_BASETIMECNT_Mask ((uint32_t)(0x7FFFFFFU << BB_BASETIMECNT_BASETIMECNT_Pos))

/* BB_BASETIMECNT bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[31];
    __IO uint32_t SAMP_ALIAS;           /* Sample the base time counter */
} BB_BASETIMECNT_Type;

#define BB_BASETIMECNT_BASE             SYS_CALC_BITBAND(0x4000151C, 0)
#define BB_BASETIMECNT                  ((BB_BASETIMECNT_Type *) BB_BASETIMECNT_BASE)

/* BB_BASETIMECNT settings */
#define BASETIMECNT_0                   ((uint32_t)(0x0U << BB_BASETIMECNT_BASETIMECNT_Pos))

#define SAMP_0_BITBAND                  0x0
#define SAMP_1_BITBAND                  0x1
#define SAMP_0                          ((uint32_t)(SAMP_0_BITBAND << BB_BASETIMECNT_SAMP_Pos))
#define SAMP_1                          ((uint32_t)(SAMP_1_BITBAND << BB_BASETIMECNT_SAMP_Pos))

/* BB_FINETIMECNT bit positions */
#define BB_FINETIMECNT_FINECNT_Pos      0
#define BB_FINETIMECNT_FINECNT_Mask     ((uint32_t)(0x3FFU << BB_FINETIMECNT_FINECNT_Pos))

/* BB_FINETIMECNT settings */
#define FINECNT_0                       ((uint32_t)(0x0U << BB_FINETIMECNT_FINECNT_Pos))

/* BB_BDADDRL bit positions */
#define BB_BDADDRL_BDADDRL_Pos          0
#define BB_BDADDRL_BDADDRL_Mask         ((uint32_t)(0xFFFFFFFFU << BB_BDADDRL_BDADDRL_Pos))

/* BB_BDADDRL settings */
#define BDADDRL_0                       ((uint32_t)(0x0U << BB_BDADDRL_BDADDRL_Pos))

/* BB_BDADDRU bit positions */
#define BB_BDADDRU_PRIV_NPUB_Pos        16
#define BB_BDADDRU_BDADDRU_Pos          0
#define BB_BDADDRU_BDADDRU_Mask         ((uint32_t)(0xFFFFU << BB_BDADDRU_BDADDRU_Pos))

/* BB_BDADDRU sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[2];
    __IO uint8_t PRIV_NPUB_BYTE;       
         uint8_t RESERVED1[1];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001528, 0) - (0x40001528 + 4)];
         uint32_t RESERVED2[16];
    __IO uint32_t PRIV_NPUB_ALIAS;      /* BLE device address privacy indicator */
} BB_BDADDRU_Type;

#define BB_BDADDRU_BASE                 0x40001528
#define BB_BDADDRU                      ((BB_BDADDRU_Type *) BB_BDADDRU_BASE)

/* BB_BDADDRU settings */
#define BDADDRU_0                       ((uint32_t)(0x0U << BB_BDADDRU_BDADDRU_Pos))

#define PRIV_NPUB_0_BITBAND             0x0
#define PRIV_NPUB_1_BITBAND             0x1
#define PRIV_NPUB_0                     ((uint32_t)(PRIV_NPUB_0_BITBAND << BB_BDADDRU_PRIV_NPUB_Pos))
#define PRIV_NPUB_1                     ((uint32_t)(PRIV_NPUB_1_BITBAND << BB_BDADDRU_PRIV_NPUB_Pos))

/* BB_BDADDRU sub-register bit positions */
#define BB_BDADDRU_PRIV_NPUB_BYTE_Pos   0

/* BB_BDADDRU sub-register settings */
#define PRIV_NPUB_0_BYTE                ((uint8_t)(PRIV_NPUB_0_BITBAND << BB_BDADDRU_PRIV_NPUB_BYTE_Pos))
#define PRIV_NPUB_1_BYTE                ((uint8_t)(PRIV_NPUB_1_BITBAND << BB_BDADDRU_PRIV_NPUB_BYTE_Pos))

/* BB_ET_CURRENTRXDESCPTR bit positions */
#define BB_ET_CURRENTRXDESCPTR_ETPTR_Pos 16
#define BB_ET_CURRENTRXDESCPTR_ETPTR_Mask ((uint32_t)(0xFFFFU << BB_ET_CURRENTRXDESCPTR_ETPTR_Pos))
#define BB_ET_CURRENTRXDESCPTR_CURRENTRXDESCPTR_Pos 0
#define BB_ET_CURRENTRXDESCPTR_CURRENTRXDESCPTR_Mask ((uint32_t)(0x7FFFU << BB_ET_CURRENTRXDESCPTR_CURRENTRXDESCPTR_Pos))

/* BB_ET_CURRENTRXDESCPTR settings */
#define CURRENTRXDESCPTR_0              ((uint32_t)(0x0U << BB_ET_CURRENTRXDESCPTR_CURRENTRXDESCPTR_Pos))

#define ETPTR_0                         ((uint32_t)(0x0U << BB_ET_CURRENTRXDESCPTR_ETPTR_Pos))

/* BB_DEEPSLCNTL bit positions */
#define BB_DEEPSLCNTL_EXTWKUPDSB_Pos    31
#define BB_DEEPSLCNTL_DEEP_SLEEP_STAT_Pos 15
#define BB_DEEPSLCNTL_SOFT_WAKEUP_REQ_Pos 4
#define BB_DEEPSLCNTL_DEEP_SLEEP_CORR_EN_Pos 3
#define BB_DEEPSLCNTL_DEEP_SLEEP_ON_Pos 2
#define BB_DEEPSLCNTL_RADIO_SLEEP_EN_Pos 1
#define BB_DEEPSLCNTL_OSC_SLEEP_EN_Pos  0

/* BB_DEEPSLCNTL sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t EXTWKUPDSB_BYTE;      
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001530, 0) - (0x40001530 + 4)];
    __IO uint32_t OSC_SLEEP_EN_ALIAS;   /* Controls the RF high frequency crystal oscillator */
    __IO uint32_t RADIO_SLEEP_EN_ALIAS; /* Controls the radio module */
    __IO uint32_t DEEP_SLEEP_ON_ALIAS;  /* RW-BLE core power mode control */
    __IO uint32_t DEEP_SLEEP_CORR_EN_ALIAS;/* 625us base time reference integer and fractional part correction applying when system has been woken-up from deep sleep mode */
    __IO uint32_t SOFT_WAKEUP_REQ_ALIAS;/* Wake up request from RW-BLE software applying when system is in deep sleep mode */
         uint32_t RESERVED1[10];
    __I  uint32_t DEEP_SLEEP_STAT_ALIAS;/* Indicator of current deep sleep clock mux status */
         uint32_t RESERVED2[15];
    __IO uint32_t EXTWKUPDSB_ALIAS;     /* External wake-up disable */
} BB_DEEPSLCNTL_Type;

#define BB_DEEPSLCNTL_BASE              0x40001530
#define BB_DEEPSLCNTL                   ((BB_DEEPSLCNTL_Type *) BB_DEEPSLCNTL_BASE)

/* BB_DEEPSLCNTL settings */
#define OSC_SLEEP_EN_0_BITBAND          0x0
#define OSC_SLEEP_EN_1_BITBAND          0x1
#define OSC_SLEEP_EN_0                  ((uint32_t)(OSC_SLEEP_EN_0_BITBAND << BB_DEEPSLCNTL_OSC_SLEEP_EN_Pos))
#define OSC_SLEEP_EN_1                  ((uint32_t)(OSC_SLEEP_EN_1_BITBAND << BB_DEEPSLCNTL_OSC_SLEEP_EN_Pos))

#define RADIO_SLEEP_EN_0_BITBAND        0x0
#define RADIO_SLEEP_EN_1_BITBAND        0x1
#define RADIO_SLEEP_EN_0                ((uint32_t)(RADIO_SLEEP_EN_0_BITBAND << BB_DEEPSLCNTL_RADIO_SLEEP_EN_Pos))
#define RADIO_SLEEP_EN_1                ((uint32_t)(RADIO_SLEEP_EN_1_BITBAND << BB_DEEPSLCNTL_RADIO_SLEEP_EN_Pos))

#define DEEP_SLEEP_ON_0_BITBAND         0x0
#define DEEP_SLEEP_ON_1_BITBAND         0x1
#define DEEP_SLEEP_ON_0                 ((uint32_t)(DEEP_SLEEP_ON_0_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_ON_Pos))
#define DEEP_SLEEP_ON_1                 ((uint32_t)(DEEP_SLEEP_ON_1_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_ON_Pos))

#define DEEP_SLEEP_CORR_EN_0_BITBAND    0x0
#define DEEP_SLEEP_CORR_EN_1_BITBAND    0x1
#define DEEP_SLEEP_CORR_EN_0            ((uint32_t)(DEEP_SLEEP_CORR_EN_0_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_CORR_EN_Pos))
#define DEEP_SLEEP_CORR_EN_1            ((uint32_t)(DEEP_SLEEP_CORR_EN_1_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_CORR_EN_Pos))

#define SOFT_WAKEUP_REQ_0_BITBAND       0x0
#define SOFT_WAKEUP_REQ_1_BITBAND       0x1
#define SOFT_WAKEUP_REQ_0               ((uint32_t)(SOFT_WAKEUP_REQ_0_BITBAND << BB_DEEPSLCNTL_SOFT_WAKEUP_REQ_Pos))
#define SOFT_WAKEUP_REQ_1               ((uint32_t)(SOFT_WAKEUP_REQ_1_BITBAND << BB_DEEPSLCNTL_SOFT_WAKEUP_REQ_Pos))

#define DEEP_SLEEP_STAT_0_BITBAND       0x0
#define DEEP_SLEEP_STAT_1_BITBAND       0x1
#define DEEP_SLEEP_STAT_0               ((uint32_t)(DEEP_SLEEP_STAT_0_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_STAT_Pos))
#define DEEP_SLEEP_STAT_1               ((uint32_t)(DEEP_SLEEP_STAT_1_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_STAT_Pos))

#define EXTWKUPDSB_0_BITBAND            0x0
#define EXTWKUPDSB_1_BITBAND            0x1
#define EXTWKUPDSB_0                    ((uint32_t)(EXTWKUPDSB_0_BITBAND << BB_DEEPSLCNTL_EXTWKUPDSB_Pos))
#define EXTWKUPDSB_1                    ((uint32_t)(EXTWKUPDSB_1_BITBAND << BB_DEEPSLCNTL_EXTWKUPDSB_Pos))

/* BB_DEEPSLCNTL sub-register bit positions */
#define BB_DEEPSLCNTL_EXTWKUPDSB_BYTE_Pos 7

/* BB_DEEPSLCNTL sub-register settings */
#define EXTWKUPDSB_0_BYTE               ((uint8_t)(EXTWKUPDSB_0_BITBAND << BB_DEEPSLCNTL_EXTWKUPDSB_BYTE_Pos))
#define EXTWKUPDSB_1_BYTE               ((uint8_t)(EXTWKUPDSB_1_BITBAND << BB_DEEPSLCNTL_EXTWKUPDSB_BYTE_Pos))

/* BB_DEEPSLWKUP bit positions */
#define BB_DEEPSLWKUP_DEEPSLTIME_Pos    0
#define BB_DEEPSLWKUP_DEEPSLTIME_Mask   ((uint32_t)(0xFFFFFFFFU << BB_DEEPSLWKUP_DEEPSLTIME_Pos))

/* BB_DEEPSLWKUP settings */
#define DEEPSLTIME_0                    ((uint32_t)(0x0U << BB_DEEPSLWKUP_DEEPSLTIME_Pos))

/* BB_DEEPSLSTAT bit positions */
#define BB_DEEPSLSTAT_DEEPSLDUR_Pos     0
#define BB_DEEPSLSTAT_DEEPSLDUR_Mask    ((uint32_t)(0xFFFFFFFFU << BB_DEEPSLSTAT_DEEPSLDUR_Pos))

/* BB_DEEPSLSTAT settings */
#define DEEPSLDUR_0                     ((uint32_t)(0x0U << BB_DEEPSLSTAT_DEEPSLDUR_Pos))

/* BB_ENBPRESET bit positions */
#define BB_ENBPRESET_TWOSC_Pos          10
#define BB_ENBPRESET_TWOSC_Mask         ((uint32_t)(0x7FFU << BB_ENBPRESET_TWOSC_Pos))

/* BB_ENBPRESET settings */
#define TWOSC_0                         ((uint32_t)(0x0U << BB_ENBPRESET_TWOSC_Pos))

/* BB_FINECNTCORR bit positions */
#define BB_FINECNTCORR_FINECNTCORR_Pos  0
#define BB_FINECNTCORR_FINECNTCORR_Mask ((uint32_t)(0x3FFU << BB_FINECNTCORR_FINECNTCORR_Pos))

/* BB_FINECNTCORR settings */
#define FINECNTCORR_0                   ((uint32_t)(0x0U << BB_FINECNTCORR_FINECNTCORR_Pos))

/* BB_BASETIMECNTCORR bit positions */
#define BB_BASETIMECNTCORR_BASETIMECNTCORR_Pos 0
#define BB_BASETIMECNTCORR_BASETIMECNTCORR_Mask ((uint32_t)(0x7FFFFFFU << BB_BASETIMECNTCORR_BASETIMECNTCORR_Pos))

/* BB_BASETIMECNTCORR settings */
#define BASETIMECNTCORR_0               ((uint32_t)(0x0U << BB_BASETIMECNTCORR_BASETIMECNTCORR_Pos))

/* BB_DIAGCNTL bit positions */
#define BB_DIAGCNTL_DIAG3_EN_Pos        31
#define BB_DIAGCNTL_DIAG3_Pos           24
#define BB_DIAGCNTL_DIAG3_Mask          ((uint32_t)(0x3FU << BB_DIAGCNTL_DIAG3_Pos))
#define BB_DIAGCNTL_DIAG2_EN_Pos        23
#define BB_DIAGCNTL_DIAG2_Pos           16
#define BB_DIAGCNTL_DIAG2_Mask          ((uint32_t)(0x3FU << BB_DIAGCNTL_DIAG2_Pos))
#define BB_DIAGCNTL_DIAG1_EN_Pos        15
#define BB_DIAGCNTL_DIAG1_Pos           8
#define BB_DIAGCNTL_DIAG1_Mask          ((uint32_t)(0x3FU << BB_DIAGCNTL_DIAG1_Pos))
#define BB_DIAGCNTL_DIAG0_EN_Pos        7
#define BB_DIAGCNTL_DIAG0_Pos           0
#define BB_DIAGCNTL_DIAG0_Mask          ((uint32_t)(0x3FU << BB_DIAGCNTL_DIAG0_Pos))

/* BB_DIAGCNTL sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DIAG0_BYTE;           
    __IO uint8_t DIAG1_BYTE;           
    __IO uint8_t DIAG2_BYTE;           
    __IO uint8_t DIAG3_BYTE;           
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001550, 0) - (0x40001550 + 4)];
         uint32_t RESERVED0[7];
    __IO uint32_t DIAG0_EN_ALIAS;       /* Enable diagnostic port 0 output */
         uint32_t RESERVED1[7];
    __IO uint32_t DIAG1_EN_ALIAS;       /* Enable diagnostic port 1 output */
         uint32_t RESERVED2[7];
    __IO uint32_t DIAG2_EN_ALIAS;       /* Enable diagnostic port 2 output */
         uint32_t RESERVED3[7];
    __IO uint32_t DIAG3_EN_ALIAS;       /* Enable diagnostic port 3 output */
} BB_DIAGCNTL_Type;

#define BB_DIAGCNTL_BASE                0x40001550
#define BB_DIAGCNTL                     ((BB_DIAGCNTL_Type *) BB_DIAGCNTL_BASE)

/* BB_DIAGCNTL settings */
#define DIAG0_0                         ((uint32_t)(0x0U << BB_DIAGCNTL_DIAG0_Pos))

#define DIAG0_EN_0_BITBAND              0x0
#define DIAG0_EN_1_BITBAND              0x1
#define DIAG0_EN_0                      ((uint32_t)(DIAG0_EN_0_BITBAND << BB_DIAGCNTL_DIAG0_EN_Pos))
#define DIAG0_EN_1                      ((uint32_t)(DIAG0_EN_1_BITBAND << BB_DIAGCNTL_DIAG0_EN_Pos))

#define DIAG1_0                         ((uint32_t)(0x0U << BB_DIAGCNTL_DIAG1_Pos))

#define DIAG1_EN_0_BITBAND              0x0
#define DIAG1_EN_1_BITBAND              0x1
#define DIAG1_EN_0                      ((uint32_t)(DIAG1_EN_0_BITBAND << BB_DIAGCNTL_DIAG1_EN_Pos))
#define DIAG1_EN_1                      ((uint32_t)(DIAG1_EN_1_BITBAND << BB_DIAGCNTL_DIAG1_EN_Pos))

#define DIAG2_0                         ((uint32_t)(0x0U << BB_DIAGCNTL_DIAG2_Pos))

#define DIAG2_EN_0_BITBAND              0x0
#define DIAG2_EN_1_BITBAND              0x1
#define DIAG2_EN_0                      ((uint32_t)(DIAG2_EN_0_BITBAND << BB_DIAGCNTL_DIAG2_EN_Pos))
#define DIAG2_EN_1                      ((uint32_t)(DIAG2_EN_1_BITBAND << BB_DIAGCNTL_DIAG2_EN_Pos))

#define DIAG3_0                         ((uint32_t)(0x0U << BB_DIAGCNTL_DIAG3_Pos))

#define DIAG3_EN_0_BITBAND              0x0
#define DIAG3_EN_1_BITBAND              0x1
#define DIAG3_EN_0                      ((uint32_t)(DIAG3_EN_0_BITBAND << BB_DIAGCNTL_DIAG3_EN_Pos))
#define DIAG3_EN_1                      ((uint32_t)(DIAG3_EN_1_BITBAND << BB_DIAGCNTL_DIAG3_EN_Pos))

/* BB_DIAGCNTL sub-register bit positions */
#define BB_DIAGCNTL_DIAG0_BYTE_Pos      0
#define BB_DIAGCNTL_DIAG0_BYTE_Mask     ((uint32_t)(0x3FU << BB_DIAGCNTL_DIAG0_BYTE_Pos))
#define BB_DIAGCNTL_DIAG0_EN_BYTE_Pos   7
#define BB_DIAGCNTL_DIAG1_BYTE_Pos      0
#define BB_DIAGCNTL_DIAG1_BYTE_Mask     ((uint32_t)(0x3FU << BB_DIAGCNTL_DIAG1_BYTE_Pos))
#define BB_DIAGCNTL_DIAG1_EN_BYTE_Pos   7
#define BB_DIAGCNTL_DIAG2_BYTE_Pos      0
#define BB_DIAGCNTL_DIAG2_BYTE_Mask     ((uint32_t)(0x3FU << BB_DIAGCNTL_DIAG2_BYTE_Pos))
#define BB_DIAGCNTL_DIAG2_EN_BYTE_Pos   7
#define BB_DIAGCNTL_DIAG3_BYTE_Pos      0
#define BB_DIAGCNTL_DIAG3_BYTE_Mask     ((uint32_t)(0x3FU << BB_DIAGCNTL_DIAG3_BYTE_Pos))
#define BB_DIAGCNTL_DIAG3_EN_BYTE_Pos   7

/* BB_DIAGCNTL sub-register settings */
#define DIAG0_0_BYTE                    ((uint8_t)(0x0U << BB_DIAGCNTL_DIAG0_BYTE_Pos))

#define DIAG0_EN_0_BYTE                 ((uint8_t)(DIAG0_EN_0_BITBAND << BB_DIAGCNTL_DIAG0_EN_BYTE_Pos))
#define DIAG0_EN_1_BYTE                 ((uint8_t)(DIAG0_EN_1_BITBAND << BB_DIAGCNTL_DIAG0_EN_BYTE_Pos))

#define DIAG1_0_BYTE                    ((uint8_t)(0x0U << BB_DIAGCNTL_DIAG1_BYTE_Pos))

#define DIAG1_EN_0_BYTE                 ((uint8_t)(DIAG1_EN_0_BITBAND << BB_DIAGCNTL_DIAG1_EN_BYTE_Pos))
#define DIAG1_EN_1_BYTE                 ((uint8_t)(DIAG1_EN_1_BITBAND << BB_DIAGCNTL_DIAG1_EN_BYTE_Pos))

#define DIAG2_0_BYTE                    ((uint8_t)(0x0U << BB_DIAGCNTL_DIAG2_BYTE_Pos))

#define DIAG2_EN_0_BYTE                 ((uint8_t)(DIAG2_EN_0_BITBAND << BB_DIAGCNTL_DIAG2_EN_BYTE_Pos))
#define DIAG2_EN_1_BYTE                 ((uint8_t)(DIAG2_EN_1_BITBAND << BB_DIAGCNTL_DIAG2_EN_BYTE_Pos))

#define DIAG3_0_BYTE                    ((uint8_t)(0x0U << BB_DIAGCNTL_DIAG3_BYTE_Pos))

#define DIAG3_EN_0_BYTE                 ((uint8_t)(DIAG3_EN_0_BITBAND << BB_DIAGCNTL_DIAG3_EN_BYTE_Pos))
#define DIAG3_EN_1_BYTE                 ((uint8_t)(DIAG3_EN_1_BITBAND << BB_DIAGCNTL_DIAG3_EN_BYTE_Pos))

/* BB_DIAGSTAT bit positions */
#define BB_DIAGSTAT_DIAG3STAT_Pos       24
#define BB_DIAGSTAT_DIAG3STAT_Mask      ((uint32_t)(0xFFU << BB_DIAGSTAT_DIAG3STAT_Pos))
#define BB_DIAGSTAT_DIAG2STAT_Pos       16
#define BB_DIAGSTAT_DIAG2STAT_Mask      ((uint32_t)(0xFFU << BB_DIAGSTAT_DIAG2STAT_Pos))
#define BB_DIAGSTAT_DIAG1STAT_Pos       8
#define BB_DIAGSTAT_DIAG1STAT_Mask      ((uint32_t)(0xFFU << BB_DIAGSTAT_DIAG1STAT_Pos))
#define BB_DIAGSTAT_DIAG0STAT_Pos       0
#define BB_DIAGSTAT_DIAG0STAT_Mask      ((uint32_t)(0xFFU << BB_DIAGSTAT_DIAG0STAT_Pos))

/* BB_DIAGSTAT sub-register and bit-band aliases */
typedef struct
{
    __I  uint8_t DIAG0STAT_BYTE;       
    __I  uint8_t DIAG1STAT_BYTE;       
    __I  uint8_t DIAG2STAT_BYTE;       
    __I  uint8_t DIAG3STAT_BYTE;       
} BB_DIAGSTAT_Type;

#define BB_DIAGSTAT_BASE                0x40001554
#define BB_DIAGSTAT                     ((BB_DIAGSTAT_Type *) BB_DIAGSTAT_BASE)

/* BB_DIAGSTAT settings */
#define DIAG0STAT_0                     ((uint32_t)(0x0U << BB_DIAGSTAT_DIAG0STAT_Pos))

#define DIAG1STAT_0                     ((uint32_t)(0x0U << BB_DIAGSTAT_DIAG1STAT_Pos))

#define DIAG2STAT_0                     ((uint32_t)(0x0U << BB_DIAGSTAT_DIAG2STAT_Pos))

#define DIAG3STAT_0                     ((uint32_t)(0x0U << BB_DIAGSTAT_DIAG3STAT_Pos))

/* BB_DIAGSTAT sub-register settings */
#define DIAG0STAT_0_BYTE                ((uint8_t)0x0U)

#define DIAG1STAT_0_BYTE                ((uint8_t)0x0U)

#define DIAG2STAT_0_BYTE                ((uint8_t)0x0U)

#define DIAG3STAT_0_BYTE                ((uint8_t)0x0U)

/* BB_DEBUGADDMAX bit positions */
#define BB_DEBUGADDMAX_REG_ADDMAX_Pos   16
#define BB_DEBUGADDMAX_REG_ADDMAX_Mask  ((uint32_t)(0xFFFFU << BB_DEBUGADDMAX_REG_ADDMAX_Pos))
#define BB_DEBUGADDMAX_EM_ADDMAX_Pos    0
#define BB_DEBUGADDMAX_EM_ADDMAX_Mask   ((uint32_t)(0xFFFFU << BB_DEBUGADDMAX_EM_ADDMAX_Pos))

/* BB_DEBUGADDMAX sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t EM_ADDMAX_SHORT;      
    __IO uint16_t REG_ADDMAX_SHORT;     
} BB_DEBUGADDMAX_Type;

#define BB_DEBUGADDMAX_BASE             0x40001558
#define BB_DEBUGADDMAX                  ((BB_DEBUGADDMAX_Type *) BB_DEBUGADDMAX_BASE)

/* BB_DEBUGADDMAX settings */
#define EM_ADDMAX_0                     ((uint32_t)(0x0U << BB_DEBUGADDMAX_EM_ADDMAX_Pos))

#define REG_ADDMAX_0                    ((uint32_t)(0x0U << BB_DEBUGADDMAX_REG_ADDMAX_Pos))

/* BB_DEBUGADDMAX sub-register settings */
#define EM_ADDMAX_0_SHORT               ((uint16_t)0x0U)

#define REG_ADDMAX_0_SHORT              ((uint16_t)0x0U)

/* BB_DEBUGADDMIN bit positions */
#define BB_DEBUGADDMIN_REG_ADDMIN_Pos   16
#define BB_DEBUGADDMIN_REG_ADDMIN_Mask  ((uint32_t)(0xFFFFU << BB_DEBUGADDMIN_REG_ADDMIN_Pos))
#define BB_DEBUGADDMIN_EM_ADDMIN_Pos    0
#define BB_DEBUGADDMIN_EM_ADDMIN_Mask   ((uint32_t)(0xFFFFU << BB_DEBUGADDMIN_EM_ADDMIN_Pos))

/* BB_DEBUGADDMIN sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t EM_ADDMIN_SHORT;      
    __IO uint16_t REG_ADDMIN_SHORT;     
} BB_DEBUGADDMIN_Type;

#define BB_DEBUGADDMIN_BASE             0x4000155C
#define BB_DEBUGADDMIN                  ((BB_DEBUGADDMIN_Type *) BB_DEBUGADDMIN_BASE)

/* BB_DEBUGADDMIN settings */
#define EM_ADDMIN_0                     ((uint32_t)(0x0U << BB_DEBUGADDMIN_EM_ADDMIN_Pos))

#define REG_ADDMIN_0                    ((uint32_t)(0x0U << BB_DEBUGADDMIN_REG_ADDMIN_Pos))

/* BB_DEBUGADDMIN sub-register settings */
#define EM_ADDMIN_0_SHORT               ((uint16_t)0x0U)

#define REG_ADDMIN_0_SHORT              ((uint16_t)0x0U)

/* BB_ERRORTYPESTAT bit positions */
#define BB_ERRORTYPESTAT_RAL_UNDERRUN_Pos 19
#define BB_ERRORTYPESTAT_RAL_ERROR_Pos  18
#define BB_ERRORTYPESTAT_CONCEVTIRQ_ERROR_Pos 17
#define BB_ERRORTYPESTAT_RXDATA_PTR_ERROR_Pos 16
#define BB_ERRORTYPESTAT_TXDATA_PTR_ERROR_Pos 15
#define BB_ERRORTYPESTAT_RXDESC_EMPTY_ERROR_Pos 14
#define BB_ERRORTYPESTAT_TXDESC_EMPTY_ERROR_Pos 13
#define BB_ERRORTYPESTAT_CSFORMAT_ERROR_Pos 12
#define BB_ERRORTYPESTAT_LLCHMAP_ERROR_Pos 11
#define BB_ERRORTYPESTAT_ADV_UNDERRUN_Pos 10
#define BB_ERRORTYPESTAT_IFS_UNDERRUN_Pos 9
#define BB_ERRORTYPESTAT_WHITELIST_ERROR_Pos 8
#define BB_ERRORTYPESTAT_EVT_CNTL_APFM_ERROR_Pos 7
#define BB_ERRORTYPESTAT_EVT_SCHDL_APFM_ERROR_Pos 6
#define BB_ERRORTYPESTAT_EVT_SCHDL_ENTRY_ERROR_Pos 5
#define BB_ERRORTYPESTAT_EVT_SCHDL_EMACC_ERROR_Pos 4
#define BB_ERRORTYPESTAT_RADIO_EMACC_ERROR_Pos 3
#define BB_ERRORTYPESTAT_PKTCNTL_EMACC_ERROR_Pos 2
#define BB_ERRORTYPESTAT_RXCRYPT_ERROR_Pos 1
#define BB_ERRORTYPESTAT_TXCRYPT_ERROR_Pos 0

/* BB_ERRORTYPESTAT bit-band aliases */
typedef struct
{
    __I  uint32_t TXCRYPT_ERROR_ALIAS;  /* Indicates real time encryption error, happens when AES-CCM encryption is too slow compared to packet controller requests */
    __I  uint32_t RXCRYPT_ERROR_ALIAS;  /* Indicates real time decryption error, happens when AES-CCM decryption is too slow compared to packet controller requests */
    __I  uint32_t PKTCNTL_EMACC_ERROR_ALIAS;/* Indicates packet controller exchange memory access error, happens when exchange memory accesses are not served in time and Tx/Rx data are corrupted */
    __I  uint32_t RADIO_EMACC_ERROR_ALIAS;/* Indicates radio controller exchange memory access error, happens when exchange memory accesses are not served in time and data are corrupted */
    __I  uint32_t EVT_SCHDL_EMACC_ERROR_ALIAS;/* Indicates event scheduler exchange memory access error, happens when exchange memory accesses are not served in time, and blocks the exchange table entry read */
    __I  uint32_t EVT_SCHDL_ENTRY_ERROR_ALIAS;/* Indicates event scheduler faced invalid timing programing on two consecutive ET entries (e.g first one with 624us offset and second one with no offset) */
    __I  uint32_t EVT_SCHDL_APFM_ERROR_ALIAS;/* Indicates anticipated pre-fetch mechanism error: happens when 2 consecutive events are programmed, and when the first event is not completely finished while second pre-fetch instant is reached */
    __I  uint32_t EVT_CNTL_APFM_ERROR_ALIAS;/* Indicates anticipated pre-fetch mechanism error: happens when 2 consecutive events are programmed, and when the first event is not completely finished while second pre-fetch instant is reached */
    __I  uint32_t WHITELIST_ERROR_ALIAS;/* Indicates white list timeout error, occurs if white list parsing is not finished on time */
    __I  uint32_t IFS_UNDERRUN_ALIAS;   /* Indicates inter frame space under run, occurs if IFS time is not enough to update and read control structure / descriptors, and/or white list parsing is not finished and/or decryption time is too long to be finished on time */
    __I  uint32_t ADV_UNDERRUN_ALIAS;   /* Indicates advertising interval under run */
    __I  uint32_t LLCHMAP_ERROR_ALIAS;  /* Indicates Link Layer channel map error, happens when actual number of CS-LLCHMAP bit set to one is different from CS-NBCHGOOD at the beginning of frequency hopping process */
    __I  uint32_t CSFORMAT_ERROR_ALIAS; /* Indicates whether CS-FORMAT has been programmed with an invalid value (major failure) */
    __I  uint32_t TXDESC_EMPTY_ERROR_ALIAS;/* Indicates whether Tx descriptor pointer value programmed in control structure is null during advertising / scanning / initiating events (major failure) */
    __I  uint32_t RXDESC_EMPTY_ERROR_ALIAS;/* Indicates whether Rx descriptor pointer value programmed in register is null (major failure) */
    __I  uint32_t TXDATA_PTR_ERROR_ALIAS;/* Indicates whether Tx data buffer pointer value programmed is null during advertising / scanning / initiating events, or during master / slave connections with non-null packet length (major failure) */
    __I  uint32_t RXDATA_PTR_ERROR_ALIAS;/* Indicates whether Rx data buffer pointer value programmed is null (major failure) */
    __I  uint32_t CONCEVTIRQ_ERROR_ALIAS;/* Indicates whether two consecutive and concurrent ble_event_irq have been generated, and not acknowledged in time by the RW-BLE software */
    __I  uint32_t RAL_ERROR_ALIAS;      /* Indicates Resolving Address List engine faced a bad setting */
    __I  uint32_t RAL_UNDERRUN_ALIAS;   /* Indicates Resolving Address List engine Under run issue, happens when RAL List parsing not finished on time */
} BB_ERRORTYPESTAT_Type;

#define BB_ERRORTYPESTAT_BASE           SYS_CALC_BITBAND(0x40001560, 0)
#define BB_ERRORTYPESTAT                ((BB_ERRORTYPESTAT_Type *) BB_ERRORTYPESTAT_BASE)

/* BB_ERRORTYPESTAT settings */
#define TXCRYPT_ERROR_0_BITBAND         0x0
#define TXCRYPT_ERROR_1_BITBAND         0x1
#define TXCRYPT_ERROR_0                 ((uint32_t)(TXCRYPT_ERROR_0_BITBAND << BB_ERRORTYPESTAT_TXCRYPT_ERROR_Pos))
#define TXCRYPT_ERROR_1                 ((uint32_t)(TXCRYPT_ERROR_1_BITBAND << BB_ERRORTYPESTAT_TXCRYPT_ERROR_Pos))

#define RXCRYPT_ERROR_0_BITBAND         0x0
#define RXCRYPT_ERROR_1_BITBAND         0x1
#define RXCRYPT_ERROR_0                 ((uint32_t)(RXCRYPT_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RXCRYPT_ERROR_Pos))
#define RXCRYPT_ERROR_1                 ((uint32_t)(RXCRYPT_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RXCRYPT_ERROR_Pos))

#define PKTCNTL_EMACC_ERROR_0_BITBAND   0x0
#define PKTCNTL_EMACC_ERROR_1_BITBAND   0x1
#define PKTCNTL_EMACC_ERROR_0           ((uint32_t)(PKTCNTL_EMACC_ERROR_0_BITBAND << BB_ERRORTYPESTAT_PKTCNTL_EMACC_ERROR_Pos))
#define PKTCNTL_EMACC_ERROR_1           ((uint32_t)(PKTCNTL_EMACC_ERROR_1_BITBAND << BB_ERRORTYPESTAT_PKTCNTL_EMACC_ERROR_Pos))

#define RADIO_EMACC_ERROR_0_BITBAND     0x0
#define RADIO_EMACC_ERROR_1_BITBAND     0x1
#define RADIO_EMACC_ERROR_0             ((uint32_t)(RADIO_EMACC_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RADIO_EMACC_ERROR_Pos))
#define RADIO_EMACC_ERROR_1             ((uint32_t)(RADIO_EMACC_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RADIO_EMACC_ERROR_Pos))

#define EVT_SCHDL_EMACC_ERROR_0_BITBAND 0x0
#define EVT_SCHDL_EMACC_ERROR_1_BITBAND 0x1
#define EVT_SCHDL_EMACC_ERROR_0         ((uint32_t)(EVT_SCHDL_EMACC_ERROR_0_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_EMACC_ERROR_Pos))
#define EVT_SCHDL_EMACC_ERROR_1         ((uint32_t)(EVT_SCHDL_EMACC_ERROR_1_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_EMACC_ERROR_Pos))

#define EVT_SCHDL_ENTRY_ERROR_0_BITBAND 0x0
#define EVT_SCHDL_ENTRY_ERROR_1_BITBAND 0x1
#define EVT_SCHDL_ENTRY_ERROR_0         ((uint32_t)(EVT_SCHDL_ENTRY_ERROR_0_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_ENTRY_ERROR_Pos))
#define EVT_SCHDL_ENTRY_ERROR_1         ((uint32_t)(EVT_SCHDL_ENTRY_ERROR_1_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_ENTRY_ERROR_Pos))

#define EVT_SCHDL_APFM_ERROR_0_BITBAND  0x0
#define EVT_SCHDL_APFM_ERROR_1_BITBAND  0x1
#define EVT_SCHDL_APFM_ERROR_0          ((uint32_t)(EVT_SCHDL_APFM_ERROR_0_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_APFM_ERROR_Pos))
#define EVT_SCHDL_APFM_ERROR_1          ((uint32_t)(EVT_SCHDL_APFM_ERROR_1_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_APFM_ERROR_Pos))

#define EVT_CNTL_APFM_ERROR_0_BITBAND   0x0
#define EVT_CNTL_APFM_ERROR_1_BITBAND   0x1
#define EVT_CNTL_APFM_ERROR_0           ((uint32_t)(EVT_CNTL_APFM_ERROR_0_BITBAND << BB_ERRORTYPESTAT_EVT_CNTL_APFM_ERROR_Pos))
#define EVT_CNTL_APFM_ERROR_1           ((uint32_t)(EVT_CNTL_APFM_ERROR_1_BITBAND << BB_ERRORTYPESTAT_EVT_CNTL_APFM_ERROR_Pos))

#define WHITELIST_ERROR_0_BITBAND       0x0
#define WHITELIST_ERROR_1_BITBAND       0x1
#define WHITELIST_ERROR_0               ((uint32_t)(WHITELIST_ERROR_0_BITBAND << BB_ERRORTYPESTAT_WHITELIST_ERROR_Pos))
#define WHITELIST_ERROR_1               ((uint32_t)(WHITELIST_ERROR_1_BITBAND << BB_ERRORTYPESTAT_WHITELIST_ERROR_Pos))

#define IFS_UNDERRUN_0_BITBAND          0x0
#define IFS_UNDERRUN_1_BITBAND          0x1
#define IFS_UNDERRUN_0                  ((uint32_t)(IFS_UNDERRUN_0_BITBAND << BB_ERRORTYPESTAT_IFS_UNDERRUN_Pos))
#define IFS_UNDERRUN_1                  ((uint32_t)(IFS_UNDERRUN_1_BITBAND << BB_ERRORTYPESTAT_IFS_UNDERRUN_Pos))

#define ADV_UNDERRUN_0_BITBAND          0x0
#define ADV_UNDERRUN_1_BITBAND          0x1
#define ADV_UNDERRUN_0                  ((uint32_t)(ADV_UNDERRUN_0_BITBAND << BB_ERRORTYPESTAT_ADV_UNDERRUN_Pos))
#define ADV_UNDERRUN_1                  ((uint32_t)(ADV_UNDERRUN_1_BITBAND << BB_ERRORTYPESTAT_ADV_UNDERRUN_Pos))

#define LLCHMAP_ERROR_0_BITBAND         0x0
#define LLCHMAP_ERROR_1_BITBAND         0x1
#define LLCHMAP_ERROR_0                 ((uint32_t)(LLCHMAP_ERROR_0_BITBAND << BB_ERRORTYPESTAT_LLCHMAP_ERROR_Pos))
#define LLCHMAP_ERROR_1                 ((uint32_t)(LLCHMAP_ERROR_1_BITBAND << BB_ERRORTYPESTAT_LLCHMAP_ERROR_Pos))

#define CSFORMAT_ERROR_0_BITBAND        0x0
#define CSFORMAT_ERROR_1_BITBAND        0x1
#define CSFORMAT_ERROR_0                ((uint32_t)(CSFORMAT_ERROR_0_BITBAND << BB_ERRORTYPESTAT_CSFORMAT_ERROR_Pos))
#define CSFORMAT_ERROR_1                ((uint32_t)(CSFORMAT_ERROR_1_BITBAND << BB_ERRORTYPESTAT_CSFORMAT_ERROR_Pos))

#define TXDESC_EMPTY_ERROR_0_BITBAND    0x0
#define TXDESC_EMPTY_ERROR_1_BITBAND    0x1
#define TXDESC_EMPTY_ERROR_0            ((uint32_t)(TXDESC_EMPTY_ERROR_0_BITBAND << BB_ERRORTYPESTAT_TXDESC_EMPTY_ERROR_Pos))
#define TXDESC_EMPTY_ERROR_1            ((uint32_t)(TXDESC_EMPTY_ERROR_1_BITBAND << BB_ERRORTYPESTAT_TXDESC_EMPTY_ERROR_Pos))

#define RXDESC_EMPTY_ERROR_0_BITBAND    0x0
#define RXDESC_EMPTY_ERROR_1_BITBAND    0x1
#define RXDESC_EMPTY_ERROR_0            ((uint32_t)(RXDESC_EMPTY_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RXDESC_EMPTY_ERROR_Pos))
#define RXDESC_EMPTY_ERROR_1            ((uint32_t)(RXDESC_EMPTY_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RXDESC_EMPTY_ERROR_Pos))

#define TXDATA_PTR_ERROR_0_BITBAND      0x0
#define TXDATA_PTR_ERROR_1_BITBAND      0x1
#define TXDATA_PTR_ERROR_0              ((uint32_t)(TXDATA_PTR_ERROR_0_BITBAND << BB_ERRORTYPESTAT_TXDATA_PTR_ERROR_Pos))
#define TXDATA_PTR_ERROR_1              ((uint32_t)(TXDATA_PTR_ERROR_1_BITBAND << BB_ERRORTYPESTAT_TXDATA_PTR_ERROR_Pos))

#define RXDATA_PTR_ERROR_0_BITBAND      0x0
#define RXDATA_PTR_ERROR_1_BITBAND      0x1
#define RXDATA_PTR_ERROR_0              ((uint32_t)(RXDATA_PTR_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RXDATA_PTR_ERROR_Pos))
#define RXDATA_PTR_ERROR_1              ((uint32_t)(RXDATA_PTR_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RXDATA_PTR_ERROR_Pos))

#define CONCEVTIRQ_ERROR_0_BITBAND      0x0
#define CONCEVTIRQ_ERROR_1_BITBAND      0x1
#define CONCEVTIRQ_ERROR_0              ((uint32_t)(CONCEVTIRQ_ERROR_0_BITBAND << BB_ERRORTYPESTAT_CONCEVTIRQ_ERROR_Pos))
#define CONCEVTIRQ_ERROR_1              ((uint32_t)(CONCEVTIRQ_ERROR_1_BITBAND << BB_ERRORTYPESTAT_CONCEVTIRQ_ERROR_Pos))

#define RAL_ERROR_0_BITBAND             0x0
#define RAL_ERROR_1_BITBAND             0x1
#define RAL_ERROR_0                     ((uint32_t)(RAL_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RAL_ERROR_Pos))
#define RAL_ERROR_1                     ((uint32_t)(RAL_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RAL_ERROR_Pos))

#define RAL_UNDERRUN_0_BITBAND          0x0
#define RAL_UNDERRUN_1_BITBAND          0x1
#define RAL_UNDERRUN_0                  ((uint32_t)(RAL_UNDERRUN_0_BITBAND << BB_ERRORTYPESTAT_RAL_UNDERRUN_Pos))
#define RAL_UNDERRUN_1                  ((uint32_t)(RAL_UNDERRUN_1_BITBAND << BB_ERRORTYPESTAT_RAL_UNDERRUN_Pos))

/* BB_SWPROFILING bit positions */
#define BB_SWPROFILING_SWPROF_Pos       0
#define BB_SWPROFILING_SWPROF_Mask      ((uint32_t)(0xFFFFFFFFU << BB_SWPROFILING_SWPROF_Pos))

/* BB_SWPROFILING settings */
#define SWPROF_0                        ((uint32_t)(0x0U << BB_SWPROFILING_SWPROF_Pos))

/* BB_RADIOCNTL0 bit positions */
#define BB_RADIOCNTL0_SPIPTR_Pos        16
#define BB_RADIOCNTL0_SPIPTR_Mask       ((uint32_t)(0xFFFFU << BB_RADIOCNTL0_SPIPTR_Pos))
#define BB_RADIOCNTL0_SPIFREQ_Pos       4
#define BB_RADIOCNTL0_SPIFREQ_Mask      ((uint32_t)(0x3U << BB_RADIOCNTL0_SPIFREQ_Pos))
#define BB_RADIOCNTL0_SPICOMP_Pos       1
#define BB_RADIOCNTL0_SPIGO_Pos         0

/* BB_RADIOCNTL0 bit-band aliases */
typedef struct
{
    __IO uint32_t SPIGO_ALIAS;          /* Start SPI transfer when writing a 1 */
    __I  uint32_t SPICOMP_ALIAS;        /* SPI transfer status */
} BB_RADIOCNTL0_Type;

#define BB_RADIOCNTL0_BASE              SYS_CALC_BITBAND(0x40001570, 0)
#define BB_RADIOCNTL0                   ((BB_RADIOCNTL0_Type *) BB_RADIOCNTL0_BASE)

/* BB_RADIOCNTL0 settings */
#define SPIGO_0_BITBAND                 0x0
#define SPIGO_1_BITBAND                 0x1
#define SPIGO_0                         ((uint32_t)(SPIGO_0_BITBAND << BB_RADIOCNTL0_SPIGO_Pos))
#define SPIGO_1                         ((uint32_t)(SPIGO_1_BITBAND << BB_RADIOCNTL0_SPIGO_Pos))

#define SPICOMP_0_BITBAND               0x0
#define SPICOMP_1_BITBAND               0x1
#define SPICOMP_0                       ((uint32_t)(SPICOMP_0_BITBAND << BB_RADIOCNTL0_SPICOMP_Pos))
#define SPICOMP_1                       ((uint32_t)(SPICOMP_1_BITBAND << BB_RADIOCNTL0_SPICOMP_Pos))

#define SPIFREQ_0                       ((uint32_t)(0x0U << BB_RADIOCNTL0_SPIFREQ_Pos))
#define SPIFREQ_1                       ((uint32_t)(0x1U << BB_RADIOCNTL0_SPIFREQ_Pos))
#define SPIFREQ_2                       ((uint32_t)(0x2U << BB_RADIOCNTL0_SPIFREQ_Pos))
#define SPIFREQ_3                       ((uint32_t)(0x3U << BB_RADIOCNTL0_SPIFREQ_Pos))

#define SPIPTR_0                        ((uint32_t)(0x0U << BB_RADIOCNTL0_SPIPTR_Pos))

/* BB_RADIOCNTL1 bit positions */
#define BB_RADIOCNTL1_FORCEAGC_EN_Pos   31
#define BB_RADIOCNTL1_FORCEBLEIQ_Pos    30
#define BB_RADIOCNTL1_FORCEAGC_LENGTH_Pos 16
#define BB_RADIOCNTL1_FORCEAGC_LENGTH_Mask ((uint32_t)(0xFFFU << BB_RADIOCNTL1_FORCEAGC_LENGTH_Pos))
#define BB_RADIOCNTL1_SYNC_PULSE_MODE_Pos 15
#define BB_RADIOCNTL1_DPCORR_EN_Pos     13
#define BB_RADIOCNTL1_JEF_SELECT_Pos    12
#define BB_RADIOCNTL1_XRFSEL_Pos        4
#define BB_RADIOCNTL1_XRFSEL_Mask       ((uint32_t)(0x1FU << BB_RADIOCNTL1_XRFSEL_Pos))
#define BB_RADIOCNTL1_SUBVERSION_Pos    0
#define BB_RADIOCNTL1_SUBVERSION_Mask   ((uint32_t)(0xFU << BB_RADIOCNTL1_SUBVERSION_Pos))

/* BB_RADIOCNTL1 bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[12];
    __IO uint32_t JEF_SELECT_ALIAS;     /* Selects Jitter Elimination FIFO */
    __IO uint32_t DPCORR_EN_ALIAS;      /* Enable the use of delayed DC compensated data path in radio correlator block */
         uint32_t RESERVED1;
    __IO uint32_t SYNC_PULSE_MODE_ALIAS;/* Define whether the SYNC_P pulse is generated as pulse or level */
         uint32_t RESERVED2[14];
    __IO uint32_t FORCEBLEIQ_ALIAS;     /* Control Ripple modulation mode in between FM and I&Q */
    __IO uint32_t FORCEAGC_EN_ALIAS;    /* Control ATLAS/Ripple AGC force mode based on radioCNTL2-FORCEAGC_LENGTH value */
} BB_RADIOCNTL1_Type;

#define BB_RADIOCNTL1_BASE              SYS_CALC_BITBAND(0x40001574, 0)
#define BB_RADIOCNTL1                   ((BB_RADIOCNTL1_Type *) BB_RADIOCNTL1_BASE)

/* BB_RADIOCNTL1 settings */
#define SUBVERSION_0                    ((uint32_t)(0x0U << BB_RADIOCNTL1_SUBVERSION_Pos))
#define SUBVERSION_1                    ((uint32_t)(0x1U << BB_RADIOCNTL1_SUBVERSION_Pos))
#define SUBVERSION_2                    ((uint32_t)(0x2U << BB_RADIOCNTL1_SUBVERSION_Pos))
#define SUBVERSION_3                    ((uint32_t)(0x3U << BB_RADIOCNTL1_SUBVERSION_Pos))

#define XRFSEL_0                        ((uint32_t)(0x0U << BB_RADIOCNTL1_XRFSEL_Pos))
#define XRFSEL_1                        ((uint32_t)(0x1U << BB_RADIOCNTL1_XRFSEL_Pos))
#define XRFSEL_2                        ((uint32_t)(0x2U << BB_RADIOCNTL1_XRFSEL_Pos))
#define XRFSEL_3                        ((uint32_t)(0x3U << BB_RADIOCNTL1_XRFSEL_Pos))

#define JEF_SELECT_0_BITBAND            0x0
#define JEF_SELECT_1_BITBAND            0x1
#define JEF_SELECT_0                    ((uint32_t)(JEF_SELECT_0_BITBAND << BB_RADIOCNTL1_JEF_SELECT_Pos))
#define JEF_SELECT_1                    ((uint32_t)(JEF_SELECT_1_BITBAND << BB_RADIOCNTL1_JEF_SELECT_Pos))

#define DPCORR_EN_0_BITBAND             0x0
#define DPCORR_EN_1_BITBAND             0x1
#define DPCORR_EN_0                     ((uint32_t)(DPCORR_EN_0_BITBAND << BB_RADIOCNTL1_DPCORR_EN_Pos))
#define DPCORR_EN_1                     ((uint32_t)(DPCORR_EN_1_BITBAND << BB_RADIOCNTL1_DPCORR_EN_Pos))

#define SYNC_PULSE_MODE_0_BITBAND       0x0
#define SYNC_PULSE_MODE_1_BITBAND       0x1
#define SYNC_PULSE_MODE_0               ((uint32_t)(SYNC_PULSE_MODE_0_BITBAND << BB_RADIOCNTL1_SYNC_PULSE_MODE_Pos))
#define SYNC_PULSE_MODE_1               ((uint32_t)(SYNC_PULSE_MODE_1_BITBAND << BB_RADIOCNTL1_SYNC_PULSE_MODE_Pos))

#define FORCEAGC_LENGTH_0               ((uint32_t)(0x0U << BB_RADIOCNTL1_FORCEAGC_LENGTH_Pos))

#define FORCEBLEIQ_0_BITBAND            0x0
#define FORCEBLEIQ_1_BITBAND            0x1
#define FORCEBLEIQ_0                    ((uint32_t)(FORCEBLEIQ_0_BITBAND << BB_RADIOCNTL1_FORCEBLEIQ_Pos))
#define FORCEBLEIQ_1                    ((uint32_t)(FORCEBLEIQ_1_BITBAND << BB_RADIOCNTL1_FORCEBLEIQ_Pos))

#define FORCEAGC_EN_0_BITBAND           0x0
#define FORCEAGC_EN_1_BITBAND           0x1
#define FORCEAGC_EN_0                   ((uint32_t)(FORCEAGC_EN_0_BITBAND << BB_RADIOCNTL1_FORCEAGC_EN_Pos))
#define FORCEAGC_EN_1                   ((uint32_t)(FORCEAGC_EN_1_BITBAND << BB_RADIOCNTL1_FORCEAGC_EN_Pos))

/* BB_RADIOCNTL2 bit positions */
#define BB_RADIOCNTL2_FREQTABLE_PTR_Pos 0
#define BB_RADIOCNTL2_FREQTABLE_PTR_Mask ((uint32_t)(0xFFFFU << BB_RADIOCNTL2_FREQTABLE_PTR_Pos))

/* BB_RADIOCNTL2 settings */
#define FREQTABLE_PTR_64                ((uint32_t)(0x40U << BB_RADIOCNTL2_FREQTABLE_PTR_Pos))

/* BB_RADIOPWRUPDN0 bit positions */
#define BB_RADIOPWRUPDN0_RXPWRUP0_Pos   16
#define BB_RADIOPWRUPDN0_RXPWRUP0_Mask  ((uint32_t)(0xFFU << BB_RADIOPWRUPDN0_RXPWRUP0_Pos))
#define BB_RADIOPWRUPDN0_TXPWRDN0_Pos   8
#define BB_RADIOPWRUPDN0_TXPWRDN0_Mask  ((uint32_t)(0x1FU << BB_RADIOPWRUPDN0_TXPWRDN0_Pos))
#define BB_RADIOPWRUPDN0_TXPWRUP0_Pos   0
#define BB_RADIOPWRUPDN0_TXPWRUP0_Mask  ((uint32_t)(0xFFU << BB_RADIOPWRUPDN0_TXPWRUP0_Pos))

/* BB_RADIOPWRUPDN0 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TXPWRUP0_BYTE;        
    __IO uint8_t TXPWRDN0_BYTE;        
    __IO uint8_t RXPWRUP0_BYTE;        
         uint8_t RESERVED0[1];
} BB_RADIOPWRUPDN0_Type;

#define BB_RADIOPWRUPDN0_BASE           0x40001580
#define BB_RADIOPWRUPDN0                ((BB_RADIOPWRUPDN0_Type *) BB_RADIOPWRUPDN0_BASE)

/* BB_RADIOPWRUPDN0 settings */
#define TXPWRUP0_0                      ((uint32_t)(0x0U << BB_RADIOPWRUPDN0_TXPWRUP0_Pos))

#define TXPWRDN0_0                      ((uint32_t)(0x0U << BB_RADIOPWRUPDN0_TXPWRDN0_Pos))

#define RXPWRUP0_0                      ((uint32_t)(0x0U << BB_RADIOPWRUPDN0_RXPWRUP0_Pos))

/* BB_RADIOPWRUPDN0 sub-register bit positions */
#define BB_RADIOPWRUPDN0_TXPWRDN0_BYTE_Pos 0
#define BB_RADIOPWRUPDN0_TXPWRDN0_BYTE_Mask ((uint32_t)(0x1FU << BB_RADIOPWRUPDN0_TXPWRDN0_BYTE_Pos))

/* BB_RADIOPWRUPDN0 sub-register settings */
#define TXPWRUP0_0_BYTE                 ((uint8_t)0x0U)

#define TXPWRDN0_0_BYTE                 ((uint8_t)(0x0U << BB_RADIOPWRUPDN0_TXPWRDN0_BYTE_Pos))

#define RXPWRUP0_0_BYTE                 ((uint8_t)0x0U)

/* BB_RADIOPWRUPDN1 bit positions */
#define BB_RADIOPWRUPDN1_RXPWRUP1_Pos   16
#define BB_RADIOPWRUPDN1_RXPWRUP1_Mask  ((uint32_t)(0xFFU << BB_RADIOPWRUPDN1_RXPWRUP1_Pos))
#define BB_RADIOPWRUPDN1_TXPWRDN1_Pos   8
#define BB_RADIOPWRUPDN1_TXPWRDN1_Mask  ((uint32_t)(0x1FU << BB_RADIOPWRUPDN1_TXPWRDN1_Pos))
#define BB_RADIOPWRUPDN1_TXPWRUP1_Pos   0
#define BB_RADIOPWRUPDN1_TXPWRUP1_Mask  ((uint32_t)(0xFFU << BB_RADIOPWRUPDN1_TXPWRUP1_Pos))

/* BB_RADIOPWRUPDN1 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TXPWRUP1_BYTE;        
    __IO uint8_t TXPWRDN1_BYTE;        
    __IO uint8_t RXPWRUP1_BYTE;        
         uint8_t RESERVED0[1];
} BB_RADIOPWRUPDN1_Type;

#define BB_RADIOPWRUPDN1_BASE           0x40001584
#define BB_RADIOPWRUPDN1                ((BB_RADIOPWRUPDN1_Type *) BB_RADIOPWRUPDN1_BASE)

/* BB_RADIOPWRUPDN1 settings */
#define TXPWRUP1_0                      ((uint32_t)(0x0U << BB_RADIOPWRUPDN1_TXPWRUP1_Pos))

#define TXPWRDN1_0                      ((uint32_t)(0x0U << BB_RADIOPWRUPDN1_TXPWRDN1_Pos))

#define RXPWRUP1_0                      ((uint32_t)(0x0U << BB_RADIOPWRUPDN1_RXPWRUP1_Pos))

/* BB_RADIOPWRUPDN1 sub-register bit positions */
#define BB_RADIOPWRUPDN1_TXPWRDN1_BYTE_Pos 0
#define BB_RADIOPWRUPDN1_TXPWRDN1_BYTE_Mask ((uint32_t)(0x1FU << BB_RADIOPWRUPDN1_TXPWRDN1_BYTE_Pos))

/* BB_RADIOPWRUPDN1 sub-register settings */
#define TXPWRUP1_0_BYTE                 ((uint8_t)0x0U)

#define TXPWRDN1_0_BYTE                 ((uint8_t)(0x0U << BB_RADIOPWRUPDN1_TXPWRDN1_BYTE_Pos))

#define RXPWRUP1_0_BYTE                 ((uint8_t)0x0U)

/* BB_RADIOTXRXTIM0 bit positions */
#define BB_RADIOTXRXTIM0_TXPATHDLY0_Pos 24
#define BB_RADIOTXRXTIM0_TXPATHDLY0_Mask ((uint32_t)(0xFFU << BB_RADIOTXRXTIM0_TXPATHDLY0_Pos))
#define BB_RADIOTXRXTIM0_RXPATHDLY0_Pos 16
#define BB_RADIOTXRXTIM0_RXPATHDLY0_Mask ((uint32_t)(0x1FU << BB_RADIOTXRXTIM0_RXPATHDLY0_Pos))
#define BB_RADIOTXRXTIM0_RFRXTMDA0_Pos  8
#define BB_RADIOTXRXTIM0_RFRXTMDA0_Mask ((uint32_t)(0x7FU << BB_RADIOTXRXTIM0_RFRXTMDA0_Pos))
#define BB_RADIOTXRXTIM0_SYNC_POSITION0_Pos 0
#define BB_RADIOTXRXTIM0_SYNC_POSITION0_Mask ((uint32_t)(0x7FU << BB_RADIOTXRXTIM0_SYNC_POSITION0_Pos))

/* BB_RADIOTXRXTIM0 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t SYNC_POSITION0_BYTE;  
    __IO uint8_t RFRXTMDA0_BYTE;       
    __IO uint8_t RXPATHDLY0_BYTE;      
    __IO uint8_t TXPATHDLY0_BYTE;      
} BB_RADIOTXRXTIM0_Type;

#define BB_RADIOTXRXTIM0_BASE           0x40001590
#define BB_RADIOTXRXTIM0                ((BB_RADIOTXRXTIM0_Type *) BB_RADIOTXRXTIM0_BASE)

/* BB_RADIOTXRXTIM0 settings */
#define SYNC_POSITION0_0                ((uint32_t)(0x0U << BB_RADIOTXRXTIM0_SYNC_POSITION0_Pos))

#define RFRXTMDA0_0                     ((uint32_t)(0x0U << BB_RADIOTXRXTIM0_RFRXTMDA0_Pos))

#define RXPATHDLY0_0                    ((uint32_t)(0x0U << BB_RADIOTXRXTIM0_RXPATHDLY0_Pos))

#define TXPATHDLY0_0                    ((uint32_t)(0x0U << BB_RADIOTXRXTIM0_TXPATHDLY0_Pos))

/* BB_RADIOTXRXTIM0 sub-register bit positions */
#define BB_RADIOTXRXTIM0_SYNC_POSITION0_BYTE_Pos 0
#define BB_RADIOTXRXTIM0_SYNC_POSITION0_BYTE_Mask ((uint32_t)(0x7FU << BB_RADIOTXRXTIM0_SYNC_POSITION0_BYTE_Pos))
#define BB_RADIOTXRXTIM0_RFRXTMDA0_BYTE_Pos 0
#define BB_RADIOTXRXTIM0_RFRXTMDA0_BYTE_Mask ((uint32_t)(0x7FU << BB_RADIOTXRXTIM0_RFRXTMDA0_BYTE_Pos))
#define BB_RADIOTXRXTIM0_RXPATHDLY0_BYTE_Pos 0
#define BB_RADIOTXRXTIM0_RXPATHDLY0_BYTE_Mask ((uint32_t)(0x1FU << BB_RADIOTXRXTIM0_RXPATHDLY0_BYTE_Pos))

/* BB_RADIOTXRXTIM0 sub-register settings */
#define SYNC_POSITION0_0_BYTE           ((uint8_t)(0x0U << BB_RADIOTXRXTIM0_SYNC_POSITION0_BYTE_Pos))

#define RFRXTMDA0_0_BYTE                ((uint8_t)(0x0U << BB_RADIOTXRXTIM0_RFRXTMDA0_BYTE_Pos))

#define RXPATHDLY0_0_BYTE               ((uint8_t)(0x0U << BB_RADIOTXRXTIM0_RXPATHDLY0_BYTE_Pos))

#define TXPATHDLY0_0_BYTE               ((uint8_t)0x0U)

/* BB_RADIOTXRXTIM1 bit positions */
#define BB_RADIOTXRXTIM1_TXPATHDLY1_Pos 24
#define BB_RADIOTXRXTIM1_TXPATHDLY1_Mask ((uint32_t)(0xFFU << BB_RADIOTXRXTIM1_TXPATHDLY1_Pos))
#define BB_RADIOTXRXTIM1_RXPATHDLY1_Pos 16
#define BB_RADIOTXRXTIM1_RXPATHDLY1_Mask ((uint32_t)(0x1FU << BB_RADIOTXRXTIM1_RXPATHDLY1_Pos))
#define BB_RADIOTXRXTIM1_RFRXTMDA1_Pos  8
#define BB_RADIOTXRXTIM1_RFRXTMDA1_Mask ((uint32_t)(0x7FU << BB_RADIOTXRXTIM1_RFRXTMDA1_Pos))
#define BB_RADIOTXRXTIM1_SYNC_POSITION1_Pos 0
#define BB_RADIOTXRXTIM1_SYNC_POSITION1_Mask ((uint32_t)(0x7FU << BB_RADIOTXRXTIM1_SYNC_POSITION1_Pos))

/* BB_RADIOTXRXTIM1 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t SYNC_POSITION1_BYTE;  
    __IO uint8_t RFRXTMDA1_BYTE;       
    __IO uint8_t RXPATHDLY1_BYTE;      
    __IO uint8_t TXPATHDLY1_BYTE;      
} BB_RADIOTXRXTIM1_Type;

#define BB_RADIOTXRXTIM1_BASE           0x40001594
#define BB_RADIOTXRXTIM1                ((BB_RADIOTXRXTIM1_Type *) BB_RADIOTXRXTIM1_BASE)

/* BB_RADIOTXRXTIM1 settings */
#define SYNC_POSITION1_0                ((uint32_t)(0x0U << BB_RADIOTXRXTIM1_SYNC_POSITION1_Pos))

#define RFRXTMDA1_0                     ((uint32_t)(0x0U << BB_RADIOTXRXTIM1_RFRXTMDA1_Pos))

#define RXPATHDLY1_0                    ((uint32_t)(0x0U << BB_RADIOTXRXTIM1_RXPATHDLY1_Pos))

#define TXPATHDLY1_0                    ((uint32_t)(0x0U << BB_RADIOTXRXTIM1_TXPATHDLY1_Pos))

/* BB_RADIOTXRXTIM1 sub-register bit positions */
#define BB_RADIOTXRXTIM1_SYNC_POSITION1_BYTE_Pos 0
#define BB_RADIOTXRXTIM1_SYNC_POSITION1_BYTE_Mask ((uint32_t)(0x7FU << BB_RADIOTXRXTIM1_SYNC_POSITION1_BYTE_Pos))
#define BB_RADIOTXRXTIM1_RFRXTMDA1_BYTE_Pos 0
#define BB_RADIOTXRXTIM1_RFRXTMDA1_BYTE_Mask ((uint32_t)(0x7FU << BB_RADIOTXRXTIM1_RFRXTMDA1_BYTE_Pos))
#define BB_RADIOTXRXTIM1_RXPATHDLY1_BYTE_Pos 0
#define BB_RADIOTXRXTIM1_RXPATHDLY1_BYTE_Mask ((uint32_t)(0x1FU << BB_RADIOTXRXTIM1_RXPATHDLY1_BYTE_Pos))

/* BB_RADIOTXRXTIM1 sub-register settings */
#define SYNC_POSITION1_0_BYTE           ((uint8_t)(0x0U << BB_RADIOTXRXTIM1_SYNC_POSITION1_BYTE_Pos))

#define RFRXTMDA1_0_BYTE                ((uint8_t)(0x0U << BB_RADIOTXRXTIM1_RFRXTMDA1_BYTE_Pos))

#define RXPATHDLY1_0_BYTE               ((uint8_t)(0x0U << BB_RADIOTXRXTIM1_RXPATHDLY1_BYTE_Pos))

#define TXPATHDLY1_0_BYTE               ((uint8_t)0x0U)

/* BB_SPIPTRCNTL0 bit positions */
#define BB_SPIPTRCNTL0_TXOFFPTR_Pos     16
#define BB_SPIPTRCNTL0_TXOFFPTR_Mask    ((uint32_t)(0xFFFFU << BB_SPIPTRCNTL0_TXOFFPTR_Pos))
#define BB_SPIPTRCNTL0_TXONPTR_Pos      0
#define BB_SPIPTRCNTL0_TXONPTR_Mask     ((uint32_t)(0xFFFFU << BB_SPIPTRCNTL0_TXONPTR_Pos))

/* BB_SPIPTRCNTL0 settings */
#define TXONPTR_0                       ((uint32_t)(0x0U << BB_SPIPTRCNTL0_TXONPTR_Pos))

#define TXOFFPTR_0                      ((uint32_t)(0x0U << BB_SPIPTRCNTL0_TXOFFPTR_Pos))

/* BB_SPIPTRCNTL1 bit positions */
#define BB_SPIPTRCNTL1_RXOFFPTR_Pos     16
#define BB_SPIPTRCNTL1_RXOFFPTR_Mask    ((uint32_t)(0xFFFFU << BB_SPIPTRCNTL1_RXOFFPTR_Pos))
#define BB_SPIPTRCNTL1_RXONPTR_Pos      0
#define BB_SPIPTRCNTL1_RXONPTR_Mask     ((uint32_t)(0xFFFFU << BB_SPIPTRCNTL1_RXONPTR_Pos))

/* BB_SPIPTRCNTL1 settings */
#define RXONPTR_0                       ((uint32_t)(0x0U << BB_SPIPTRCNTL1_RXONPTR_Pos))

#define RXOFFPTR_0                      ((uint32_t)(0x0U << BB_SPIPTRCNTL1_RXOFFPTR_Pos))

/* BB_SPIPTRCNTL2 bit positions */
#define BB_SPIPTRCNTL2_RSSIPTR_Pos      0
#define BB_SPIPTRCNTL2_RSSIPTR_Mask     ((uint32_t)(0xFFFFU << BB_SPIPTRCNTL2_RSSIPTR_Pos))

/* BB_SPIPTRCNTL2 settings */
#define RSSIPTR_0                       ((uint32_t)(0x0U << BB_SPIPTRCNTL2_RSSIPTR_Pos))

/* BB_ADVCHMAP bit positions */
#define BB_ADVCHMAP_ADVCHMAP_Pos        0
#define BB_ADVCHMAP_ADVCHMAP_Mask       ((uint32_t)(0x7U << BB_ADVCHMAP_ADVCHMAP_Pos))

/* BB_ADVCHMAP settings */
#define ADVCHMAP_7                      ((uint32_t)(0x7U << BB_ADVCHMAP_ADVCHMAP_Pos))

/* BB_ADVTIM bit positions */
#define BB_ADVTIM_ADVINT_Pos            0
#define BB_ADVTIM_ADVINT_Mask           ((uint32_t)(0x3FFFU << BB_ADVTIM_ADVINT_Pos))

/* BB_ADVTIM settings */
#define ADVINT_0                        ((uint32_t)(0x0U << BB_ADVTIM_ADVINT_Pos))

/* BB_ACTSCANSTAT bit positions */
#define BB_ACTSCANSTAT_BACKOFF_Pos      16
#define BB_ACTSCANSTAT_BACKOFF_Mask     ((uint32_t)(0x1FFU << BB_ACTSCANSTAT_BACKOFF_Pos))
#define BB_ACTSCANSTAT_UPPERLIMIT_Pos   0
#define BB_ACTSCANSTAT_UPPERLIMIT_Mask  ((uint32_t)(0x1FFU << BB_ACTSCANSTAT_UPPERLIMIT_Pos))

/* BB_ACTSCANSTAT settings */
#define UPPERLIMIT_1                    ((uint32_t)(0x1U << BB_ACTSCANSTAT_UPPERLIMIT_Pos))

#define BACKOFF_1                       ((uint32_t)(0x1U << BB_ACTSCANSTAT_BACKOFF_Pos))

/* BB_WLPUBADDPTR bit positions */
#define BB_WLPUBADDPTR_WLPUBADDPTR_Pos  0
#define BB_WLPUBADDPTR_WLPUBADDPTR_Mask ((uint32_t)(0xFFFFU << BB_WLPUBADDPTR_WLPUBADDPTR_Pos))

/* BB_WLPUBADDPTR settings */
#define WLPUBADDPTR_0                   ((uint32_t)(0x0U << BB_WLPUBADDPTR_WLPUBADDPTR_Pos))

/* BB_WLPRIVADDPTR bit positions */
#define BB_WLPRIVADDPTR_WLPRIVADDPTR_Pos 0
#define BB_WLPRIVADDPTR_WLPRIVADDPTR_Mask ((uint32_t)(0xFFFFU << BB_WLPRIVADDPTR_WLPRIVADDPTR_Pos))

/* BB_WLPRIVADDPTR settings */
#define WLPRIVADDPTR_0                  ((uint32_t)(0x0U << BB_WLPRIVADDPTR_WLPRIVADDPTR_Pos))

/* BB_WLNBDEV bit positions */
#define BB_WLNBDEV_NBPRIVDEV_Pos        8
#define BB_WLNBDEV_NBPRIVDEV_Mask       ((uint32_t)(0xFFU << BB_WLNBDEV_NBPRIVDEV_Pos))
#define BB_WLNBDEV_NBPUBDEV_Pos         0
#define BB_WLNBDEV_NBPUBDEV_Mask        ((uint32_t)(0xFFU << BB_WLNBDEV_NBPUBDEV_Pos))

/* BB_WLNBDEV sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t NBPUBDEV_BYTE;        
    __IO uint8_t NBPRIVDEV_BYTE;       
         uint8_t RESERVED0[2];
} BB_WLNBDEV_Type;

#define BB_WLNBDEV_BASE                 0x400015D8
#define BB_WLNBDEV                      ((BB_WLNBDEV_Type *) BB_WLNBDEV_BASE)

/* BB_WLNBDEV settings */
#define NBPUBDEV_0                      ((uint32_t)(0x0U << BB_WLNBDEV_NBPUBDEV_Pos))

#define NBPRIVDEV_0                     ((uint32_t)(0x0U << BB_WLNBDEV_NBPRIVDEV_Pos))

/* BB_WLNBDEV sub-register settings */
#define NBPUBDEV_0_BYTE                 ((uint8_t)0x0U)

#define NBPRIVDEV_0_BYTE                ((uint8_t)0x0U)

/* BB_AESCNTL bit positions */
#define BB_AESCNTL_AES_MODE_Pos         1
#define BB_AESCNTL_AES_START_Pos        0

/* BB_AESCNTL bit-band aliases */
typedef struct
{
    __IO uint32_t AES_START_ALIAS;      /* Starts AES-128 ciphering/deciphering process */
    __IO uint32_t AES_MODE_ALIAS;       /* Cipher mode control */
} BB_AESCNTL_Type;

#define BB_AESCNTL_BASE                 SYS_CALC_BITBAND(0x400015E0, 0)
#define BB_AESCNTL                      ((BB_AESCNTL_Type *) BB_AESCNTL_BASE)

/* BB_AESCNTL settings */
#define AES_START_0_BITBAND             0x0
#define AES_START_1_BITBAND             0x1
#define AES_START_0                     ((uint32_t)(AES_START_0_BITBAND << BB_AESCNTL_AES_START_Pos))
#define AES_START_1                     ((uint32_t)(AES_START_1_BITBAND << BB_AESCNTL_AES_START_Pos))

#define AES_MODE_0_BITBAND              0x0
#define AES_MODE_1_BITBAND              0x1
#define AES_MODE_0                      ((uint32_t)(AES_MODE_0_BITBAND << BB_AESCNTL_AES_MODE_Pos))
#define AES_MODE_1                      ((uint32_t)(AES_MODE_1_BITBAND << BB_AESCNTL_AES_MODE_Pos))

/* BB_AESKEY31_0 bit positions */
#define BB_AESKEY31_0_AESKEY31_0_Pos    0
#define BB_AESKEY31_0_AESKEY31_0_Mask   ((uint32_t)(0xFFFFFFFFU << BB_AESKEY31_0_AESKEY31_0_Pos))

/* BB_AESKEY31_0 settings */
#define AESKEY31_0_0                    ((uint32_t)(0x0U << BB_AESKEY31_0_AESKEY31_0_Pos))

/* BB_AESKEY63_32 bit positions */
#define BB_AESKEY63_32_AESKEY63_32_Pos  0
#define BB_AESKEY63_32_AESKEY63_32_Mask ((uint32_t)(0xFFFFFFFFU << BB_AESKEY63_32_AESKEY63_32_Pos))

/* BB_AESKEY63_32 settings */
#define AESKEY63_32_0                   ((uint32_t)(0x0U << BB_AESKEY63_32_AESKEY63_32_Pos))

/* BB_AESKEY95_64 bit positions */
#define BB_AESKEY95_64_AESKEY95_64_Pos  0
#define BB_AESKEY95_64_AESKEY95_64_Mask ((uint32_t)(0xFFFFFFFFU << BB_AESKEY95_64_AESKEY95_64_Pos))

/* BB_AESKEY95_64 settings */
#define AESKEY95_64_0                   ((uint32_t)(0x0U << BB_AESKEY95_64_AESKEY95_64_Pos))

/* BB_AESKEY127_96 bit positions */
#define BB_AESKEY127_96_AESKEY127_96_Pos 0
#define BB_AESKEY127_96_AESKEY127_96_Mask ((uint32_t)(0xFFFFFFFFU << BB_AESKEY127_96_AESKEY127_96_Pos))

/* BB_AESKEY127_96 settings */
#define AESKEY127_96_0                  ((uint32_t)(0x0U << BB_AESKEY127_96_AESKEY127_96_Pos))

/* BB_AESPTR bit positions */
#define BB_AESPTR_AESPTR_Pos            0
#define BB_AESPTR_AESPTR_Mask           ((uint32_t)(0xFFFFU << BB_AESPTR_AESPTR_Pos))

/* BB_AESPTR settings */
#define AESPTR_0                        ((uint32_t)(0x0U << BB_AESPTR_AESPTR_Pos))

/* BB_TXMICVAL bit positions */
#define BB_TXMICVAL_TXMICVAL_Pos        0
#define BB_TXMICVAL_TXMICVAL_Mask       ((uint32_t)(0xFFFFFFFFU << BB_TXMICVAL_TXMICVAL_Pos))

/* BB_TXMICVAL settings */
#define TXMICVAL_0                      ((uint32_t)(0x0U << BB_TXMICVAL_TXMICVAL_Pos))

/* BB_RXMICVAL bit positions */
#define BB_RXMICVAL_RXMICVAL_Pos        0
#define BB_RXMICVAL_RXMICVAL_Mask       ((uint32_t)(0xFFFFFFFFU << BB_RXMICVAL_RXMICVAL_Pos))

/* BB_RXMICVAL settings */
#define RXMICVAL_0                      ((uint32_t)(0x0U << BB_RXMICVAL_RXMICVAL_Pos))

/* BB_RFTESTCNTL bit positions */
#define BB_RFTESTCNTL_INFINITERX_Pos    31
#define BB_RFTESTCNTL_RXPKTCNTEN_Pos    27
#define BB_RFTESTCNTL_INFINITETX_Pos    15
#define BB_RFTESTCNTL_TXLENGTHSRC_Pos   14
#define BB_RFTESTCNTL_PRBSTYPE_Pos      13
#define BB_RFTESTCNTL_TXPLDSRC_Pos      12
#define BB_RFTESTCNTL_TXPKTCNTEN_Pos    11
#define BB_RFTESTCNTL_TXLENGTH_Pos      0
#define BB_RFTESTCNTL_TXLENGTH_Mask     ((uint32_t)(0x1FFU << BB_RFTESTCNTL_TXLENGTH_Pos))

/* BB_RFTESTCNTL bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[11];
    __IO uint32_t TXPKTCNTEN_ALIAS;     /* Applicable in RF test mode only */
    __IO uint32_t TXPLDSRC_ALIAS;       /* Applicable only in Tx/Rx RF test mode */
    __IO uint32_t PRBSTYPE_ALIAS;       /* Applicable only in Tx/Rx RF test mode */
    __IO uint32_t TXLENGTHSRC_ALIAS;    /* Applicable only in Tx/Rx RF test mode */
    __IO uint32_t INFINITETX_ALIAS;     /* Applicable in RF test mode only */
         uint32_t RESERVED1[11];
    __IO uint32_t RXPKTCNTEN_ALIAS;     /* Applicable in RF test mode only */
         uint32_t RESERVED2[3];
    __IO uint32_t INFINITERX_ALIAS;     /* Applicable in RF test mode only */
} BB_RFTESTCNTL_Type;

#define BB_RFTESTCNTL_BASE              SYS_CALC_BITBAND(0x40001600, 0)
#define BB_RFTESTCNTL                   ((BB_RFTESTCNTL_Type *) BB_RFTESTCNTL_BASE)

/* BB_RFTESTCNTL settings */
#define TXLENGTH_0                      ((uint32_t)(0x0U << BB_RFTESTCNTL_TXLENGTH_Pos))

#define TXPKTCNTEN_0_BITBAND            0x0
#define TXPKTCNTEN_1_BITBAND            0x1
#define TXPKTCNTEN_0                    ((uint32_t)(TXPKTCNTEN_0_BITBAND << BB_RFTESTCNTL_TXPKTCNTEN_Pos))
#define TXPKTCNTEN_1                    ((uint32_t)(TXPKTCNTEN_1_BITBAND << BB_RFTESTCNTL_TXPKTCNTEN_Pos))

#define TXPLDSRC_0_BITBAND              0x0
#define TXPLDSRC_1_BITBAND              0x1
#define TXPLDSRC_0                      ((uint32_t)(TXPLDSRC_0_BITBAND << BB_RFTESTCNTL_TXPLDSRC_Pos))
#define TXPLDSRC_1                      ((uint32_t)(TXPLDSRC_1_BITBAND << BB_RFTESTCNTL_TXPLDSRC_Pos))

#define PRBSTYPE_0_BITBAND              0x0
#define PRBSTYPE_1_BITBAND              0x1
#define PRBSTYPE_0                      ((uint32_t)(PRBSTYPE_0_BITBAND << BB_RFTESTCNTL_PRBSTYPE_Pos))
#define PRBSTYPE_1                      ((uint32_t)(PRBSTYPE_1_BITBAND << BB_RFTESTCNTL_PRBSTYPE_Pos))

#define TXLENGTHSRC_0_BITBAND           0x0
#define TXLENGTHSRC_1_BITBAND           0x1
#define TXLENGTHSRC_0                   ((uint32_t)(TXLENGTHSRC_0_BITBAND << BB_RFTESTCNTL_TXLENGTHSRC_Pos))
#define TXLENGTHSRC_1                   ((uint32_t)(TXLENGTHSRC_1_BITBAND << BB_RFTESTCNTL_TXLENGTHSRC_Pos))

#define INFINITETX_0_BITBAND            0x0
#define INFINITETX_1_BITBAND            0x1
#define INFINITETX_0                    ((uint32_t)(INFINITETX_0_BITBAND << BB_RFTESTCNTL_INFINITETX_Pos))
#define INFINITETX_1                    ((uint32_t)(INFINITETX_1_BITBAND << BB_RFTESTCNTL_INFINITETX_Pos))

#define RXPKTCNTEN_0_BITBAND            0x0
#define RXPKTCNTEN_1_BITBAND            0x1
#define RXPKTCNTEN_0                    ((uint32_t)(RXPKTCNTEN_0_BITBAND << BB_RFTESTCNTL_RXPKTCNTEN_Pos))
#define RXPKTCNTEN_1                    ((uint32_t)(RXPKTCNTEN_1_BITBAND << BB_RFTESTCNTL_RXPKTCNTEN_Pos))

#define INFINITERX_0_BITBAND            0x0
#define INFINITERX_1_BITBAND            0x1
#define INFINITERX_0                    ((uint32_t)(INFINITERX_0_BITBAND << BB_RFTESTCNTL_INFINITERX_Pos))
#define INFINITERX_1                    ((uint32_t)(INFINITERX_1_BITBAND << BB_RFTESTCNTL_INFINITERX_Pos))

/* BB_RFTESTTXSTAT bit positions */
#define BB_RFTESTTXSTAT_TXPKTCNT_Pos    0
#define BB_RFTESTTXSTAT_TXPKTCNT_Mask   ((uint32_t)(0xFFFFFFFFU << BB_RFTESTTXSTAT_TXPKTCNT_Pos))

/* BB_RFTESTTXSTAT settings */
#define TXPKTCNT_00                     ((uint32_t)(0x0U << BB_RFTESTTXSTAT_TXPKTCNT_Pos))

/* BB_RFTESTRXSTAT bit positions */
#define BB_RFTESTRXSTAT_RXPKTCNT_Pos    0
#define BB_RFTESTRXSTAT_RXPKTCNT_Mask   ((uint32_t)(0xFFFFFFFFU << BB_RFTESTRXSTAT_RXPKTCNT_Pos))

/* BB_RFTESTRXSTAT settings */
#define RXPKTCNTX_0                     ((uint32_t)(0x0U << BB_RFTESTRXSTAT_RXPKTCNT_Pos))

/* BB_TIMGENCNTL bit positions */
#define BB_TIMGENCNTL_APFM_EN_Pos       31
#define BB_TIMGENCNTL_PREFETCHABORT_TIME_Pos 16
#define BB_TIMGENCNTL_PREFETCHABORT_TIME_Mask ((uint32_t)(0x3FFU << BB_TIMGENCNTL_PREFETCHABORT_TIME_Pos))
#define BB_TIMGENCNTL_PREFETCH_TIME_Pos 0
#define BB_TIMGENCNTL_PREFETCH_TIME_Mask ((uint32_t)(0x1FFU << BB_TIMGENCNTL_PREFETCH_TIME_Pos))

/* BB_TIMGENCNTL bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[31];
    __IO uint32_t APFM_EN_ALIAS;        /* Controls the anticipated pre-fetch abort mechanism */
} BB_TIMGENCNTL_Type;

#define BB_TIMGENCNTL_BASE              SYS_CALC_BITBAND(0x40001610, 0)
#define BB_TIMGENCNTL                   ((BB_TIMGENCNTL_Type *) BB_TIMGENCNTL_BASE)

/* BB_TIMGENCNTL settings */
#define PREFETCH_TIME_150               ((uint32_t)(0x96U << BB_TIMGENCNTL_PREFETCH_TIME_Pos))

#define PREFETCHABORT_TIME_254          ((uint32_t)(0x1FEU << BB_TIMGENCNTL_PREFETCHABORT_TIME_Pos))

#define APFM_EN_0_BITBAND               0x0
#define APFM_EN_1_BITBAND               0x1
#define APFM_EN_0                       ((uint32_t)(APFM_EN_0_BITBAND << BB_TIMGENCNTL_APFM_EN_Pos))
#define APFM_EN_1                       ((uint32_t)(APFM_EN_1_BITBAND << BB_TIMGENCNTL_APFM_EN_Pos))

/* BB_GROSSTIMTGT bit positions */
#define BB_GROSSTIMTGT_GROSSTARGET_Pos  0
#define BB_GROSSTIMTGT_GROSSTARGET_Mask ((uint32_t)(0x7FFFFFU << BB_GROSSTIMTGT_GROSSTARGET_Pos))

/* BB_GROSSTIMTGT settings */
#define GROSSTARGET_0                   ((uint32_t)(0x0U << BB_GROSSTIMTGT_GROSSTARGET_Pos))

/* BB_FINETIMTGT bit positions */
#define BB_FINETIMTGT_FINETARGET_Pos    0
#define BB_FINETIMTGT_FINETARGET_Mask   ((uint32_t)(0x7FFFFFFU << BB_FINETIMTGT_FINETARGET_Pos))

/* BB_FINETIMTGT settings */
#define FINETARGET_0                    ((uint32_t)(0x0U << BB_FINETIMTGT_FINETARGET_Pos))

/* BB_COEXIFCNTL0 bit positions */
#define BB_COEXIFCNTL0_MWSSCANFREQMSK_Pos 24
#define BB_COEXIFCNTL0_MWSSCANFREQMSK_Mask ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSSCANFREQMSK_Pos))
#define BB_COEXIFCNTL0_WLCRXPRIOMODE_Pos 20
#define BB_COEXIFCNTL0_WLCRXPRIOMODE_Mask ((uint32_t)(0x3U << BB_COEXIFCNTL0_WLCRXPRIOMODE_Pos))
#define BB_COEXIFCNTL0_WLCTXPRIOMODE_Pos 16
#define BB_COEXIFCNTL0_WLCTXPRIOMODE_Mask ((uint32_t)(0x3U << BB_COEXIFCNTL0_WLCTXPRIOMODE_Pos))
#define BB_COEXIFCNTL0_MWSTXFREQMSK_Pos 14
#define BB_COEXIFCNTL0_MWSTXFREQMSK_Mask ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSTXFREQMSK_Pos))
#define BB_COEXIFCNTL0_MWSRXFREQMSK_Pos 12
#define BB_COEXIFCNTL0_MWSRXFREQMSK_Mask ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSRXFREQMSK_Pos))
#define BB_COEXIFCNTL0_MWSTXMSK_Pos     10
#define BB_COEXIFCNTL0_MWSTXMSK_Mask    ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSTXMSK_Pos))
#define BB_COEXIFCNTL0_MWSRXMSK_Pos     8
#define BB_COEXIFCNTL0_MWSRXMSK_Mask    ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSRXMSK_Pos))
#define BB_COEXIFCNTL0_TXMSK_Pos        6
#define BB_COEXIFCNTL0_TXMSK_Mask       ((uint32_t)(0x3U << BB_COEXIFCNTL0_TXMSK_Pos))
#define BB_COEXIFCNTL0_RXMSK_Pos        4
#define BB_COEXIFCNTL0_RXMSK_Mask       ((uint32_t)(0x3U << BB_COEXIFCNTL0_RXMSK_Pos))
#define BB_COEXIFCNTL0_MWSWCI_EN_Pos    3
#define BB_COEXIFCNTL0_MWSCOEX_EN_Pos   2
#define BB_COEXIFCNTL0_SYNCGEN_EN_Pos   1
#define BB_COEXIFCNTL0_COEX_EN_Pos      0

/* BB_COEXIFCNTL0 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t MWSSCANFREQMSK_BYTE;  
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001620, 0) - (0x40001620 + 4)];
    __IO uint32_t COEX_EN_ALIAS;        /* Enable / disable control of the coexistence control */
    __IO uint32_t SYNCGEN_EN_ALIAS;     /* Determines whether ble_sync is generated or not */
    __IO uint32_t MWSCOEX_EN_ALIAS;     /* Enable / Disable control of the MWS Coexistence control / Valid in Dual Mode only */
    __IO uint32_t MWSWCI_EN_ALIAS;      /* Enable / Disable control of the WCI MWS Coexistence interface / Valid in Dual Mode only */
} BB_COEXIFCNTL0_Type;

#define BB_COEXIFCNTL0_BASE             0x40001620
#define BB_COEXIFCNTL0                  ((BB_COEXIFCNTL0_Type *) BB_COEXIFCNTL0_BASE)

/* BB_COEXIFCNTL0 settings */
#define COEX_EN_0_BITBAND               0x0
#define COEX_EN_1_BITBAND               0x1
#define COEX_EN_0                       ((uint32_t)(COEX_EN_0_BITBAND << BB_COEXIFCNTL0_COEX_EN_Pos))
#define COEX_EN_1                       ((uint32_t)(COEX_EN_1_BITBAND << BB_COEXIFCNTL0_COEX_EN_Pos))

#define SYNCGEN_EN_0_BITBAND            0x0
#define SYNCGEN_EN_1_BITBAND            0x1
#define SYNCGEN_EN_0                    ((uint32_t)(SYNCGEN_EN_0_BITBAND << BB_COEXIFCNTL0_SYNCGEN_EN_Pos))
#define SYNCGEN_EN_1                    ((uint32_t)(SYNCGEN_EN_1_BITBAND << BB_COEXIFCNTL0_SYNCGEN_EN_Pos))

#define MWSCOEX_EN_0_BITBAND            0x0
#define MWSCOEX_EN_1_BITBAND            0x1
#define MWSCOEX_EN_0                    ((uint32_t)(MWSCOEX_EN_0_BITBAND << BB_COEXIFCNTL0_MWSCOEX_EN_Pos))
#define MWSCOEX_EN_1                    ((uint32_t)(MWSCOEX_EN_1_BITBAND << BB_COEXIFCNTL0_MWSCOEX_EN_Pos))

#define MWSWCI_EN_0_BITBAND             0x0
#define MWSWCI_EN_1_BITBAND             0x1
#define MWSWCI_EN_0                     ((uint32_t)(MWSWCI_EN_0_BITBAND << BB_COEXIFCNTL0_MWSWCI_EN_Pos))
#define MWSWCI_EN_1                     ((uint32_t)(MWSWCI_EN_1_BITBAND << BB_COEXIFCNTL0_MWSWCI_EN_Pos))

#define RXMSK_0                         ((uint32_t)(0x0U << BB_COEXIFCNTL0_RXMSK_Pos))
#define RXMSK_1                         ((uint32_t)(0x1U << BB_COEXIFCNTL0_RXMSK_Pos))
#define RXMSK_2                         ((uint32_t)(0x2U << BB_COEXIFCNTL0_RXMSK_Pos))
#define RXMSK_3                         ((uint32_t)(0x3U << BB_COEXIFCNTL0_RXMSK_Pos))

#define TXMSK_0                         ((uint32_t)(0x0U << BB_COEXIFCNTL0_TXMSK_Pos))
#define TXMSK_1                         ((uint32_t)(0x1U << BB_COEXIFCNTL0_TXMSK_Pos))
#define TXMSK_2                         ((uint32_t)(0x2U << BB_COEXIFCNTL0_TXMSK_Pos))
#define TXMSK_3                         ((uint32_t)(0x3U << BB_COEXIFCNTL0_TXMSK_Pos))

#define MWSRXMSK_0                      ((uint32_t)(0x0U << BB_COEXIFCNTL0_MWSRXMSK_Pos))
#define MWSRXMSK_1                      ((uint32_t)(0x1U << BB_COEXIFCNTL0_MWSRXMSK_Pos))
#define MWSRXMSK_2                      ((uint32_t)(0x2U << BB_COEXIFCNTL0_MWSRXMSK_Pos))
#define MWSRXMSK_3                      ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSRXMSK_Pos))

#define MWSTXMSK_0                      ((uint32_t)(0x0U << BB_COEXIFCNTL0_MWSTXMSK_Pos))
#define MWSTXMSK_1                      ((uint32_t)(0x1U << BB_COEXIFCNTL0_MWSTXMSK_Pos))
#define MWSTXMSK_2                      ((uint32_t)(0x2U << BB_COEXIFCNTL0_MWSTXMSK_Pos))
#define MWSTXMSK_3                      ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSTXMSK_Pos))

#define MWSRXFREQMSK_0                  ((uint32_t)(0x0U << BB_COEXIFCNTL0_MWSRXFREQMSK_Pos))
#define MWSRXFREQMSK_1                  ((uint32_t)(0x1U << BB_COEXIFCNTL0_MWSRXFREQMSK_Pos))
#define MWSRXFREQMSK_2                  ((uint32_t)(0x2U << BB_COEXIFCNTL0_MWSRXFREQMSK_Pos))
#define MWSRXFREQMSK_3                  ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSRXFREQMSK_Pos))

#define MWSTXFREQMSK_0                  ((uint32_t)(0x0U << BB_COEXIFCNTL0_MWSTXFREQMSK_Pos))
#define MWSTXFREQMSK_1                  ((uint32_t)(0x1U << BB_COEXIFCNTL0_MWSTXFREQMSK_Pos))
#define MWSTXFREQMSK_2                  ((uint32_t)(0x2U << BB_COEXIFCNTL0_MWSTXFREQMSK_Pos))
#define MWSTXFREQMSK_3                  ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSTXFREQMSK_Pos))

#define WLCTXPRIOMODE_0                 ((uint32_t)(0x0U << BB_COEXIFCNTL0_WLCTXPRIOMODE_Pos))
#define WLCTXPRIOMODE_1                 ((uint32_t)(0x1U << BB_COEXIFCNTL0_WLCTXPRIOMODE_Pos))
#define WLCTXPRIOMODE_2                 ((uint32_t)(0x2U << BB_COEXIFCNTL0_WLCTXPRIOMODE_Pos))
#define WLCTXPRIOMODE_3                 ((uint32_t)(0x3U << BB_COEXIFCNTL0_WLCTXPRIOMODE_Pos))

#define WLCRXPRIOMODE_0                 ((uint32_t)(0x0U << BB_COEXIFCNTL0_WLCRXPRIOMODE_Pos))
#define WLCRXPRIOMODE_1                 ((uint32_t)(0x1U << BB_COEXIFCNTL0_WLCRXPRIOMODE_Pos))
#define WLCRXPRIOMODE_2                 ((uint32_t)(0x2U << BB_COEXIFCNTL0_WLCRXPRIOMODE_Pos))
#define WLCRXPRIOMODE_3                 ((uint32_t)(0x3U << BB_COEXIFCNTL0_WLCRXPRIOMODE_Pos))

#define MWSSCANFREQMSK_0                ((uint32_t)(0x0U << BB_COEXIFCNTL0_MWSSCANFREQMSK_Pos))
#define MWSSCANFREQMSK_1                ((uint32_t)(0x1U << BB_COEXIFCNTL0_MWSSCANFREQMSK_Pos))
#define MWSSCANFREQMSK_2                ((uint32_t)(0x2U << BB_COEXIFCNTL0_MWSSCANFREQMSK_Pos))
#define MWSSCANFREQMSK_3                ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSSCANFREQMSK_Pos))

/* BB_COEXIFCNTL0 sub-register bit positions */
#define BB_COEXIFCNTL0_MWSSCANFREQMSK_BYTE_Pos 0
#define BB_COEXIFCNTL0_MWSSCANFREQMSK_BYTE_Mask ((uint32_t)(0x3U << BB_COEXIFCNTL0_MWSSCANFREQMSK_BYTE_Pos))

/* BB_COEXIFCNTL0 sub-register settings */
#define MWSSCANFREQMSK_0_BYTE           ((uint8_t)(0x0U << BB_COEXIFCNTL0_MWSSCANFREQMSK_BYTE_Pos))
#define MWSSCANFREQMSK_1_BYTE           ((uint8_t)(0x1U << BB_COEXIFCNTL0_MWSSCANFREQMSK_BYTE_Pos))
#define MWSSCANFREQMSK_2_BYTE           ((uint8_t)(0x2U << BB_COEXIFCNTL0_MWSSCANFREQMSK_BYTE_Pos))
#define MWSSCANFREQMSK_3_BYTE           ((uint8_t)(0x3U << BB_COEXIFCNTL0_MWSSCANFREQMSK_BYTE_Pos))

/* BB_COEXIFCNTL1 bit positions */
#define BB_COEXIFCNTL1_WLCPRXTHR_Pos    24
#define BB_COEXIFCNTL1_WLCPRXTHR_Mask   ((uint32_t)(0x1FU << BB_COEXIFCNTL1_WLCPRXTHR_Pos))
#define BB_COEXIFCNTL1_WLCPTXTHR_Pos    16
#define BB_COEXIFCNTL1_WLCPTXTHR_Mask   ((uint32_t)(0x1FU << BB_COEXIFCNTL1_WLCPTXTHR_Pos))
#define BB_COEXIFCNTL1_WLCPDURATION_Pos 8
#define BB_COEXIFCNTL1_WLCPDURATION_Mask ((uint32_t)(0x7FU << BB_COEXIFCNTL1_WLCPDURATION_Pos))
#define BB_COEXIFCNTL1_WLCPDELAY_Pos    0
#define BB_COEXIFCNTL1_WLCPDELAY_Mask   ((uint32_t)(0x7FU << BB_COEXIFCNTL1_WLCPDELAY_Pos))

/* BB_COEXIFCNTL1 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t WLCPDELAY_BYTE;       
    __IO uint8_t WLCPDURATION_BYTE;    
    __IO uint8_t WLCPTXTHR_BYTE;       
    __IO uint8_t WLCPRXTHR_BYTE;       
} BB_COEXIFCNTL1_Type;

#define BB_COEXIFCNTL1_BASE             0x40001624
#define BB_COEXIFCNTL1                  ((BB_COEXIFCNTL1_Type *) BB_COEXIFCNTL1_BASE)

/* BB_COEXIFCNTL1 settings */
#define WLCPDELAY_0                     ((uint32_t)(0x0U << BB_COEXIFCNTL1_WLCPDELAY_Pos))

#define WLCPDURATION_0                  ((uint32_t)(0x0U << BB_COEXIFCNTL1_WLCPDURATION_Pos))

#define WLCPTXTHR_0                     ((uint32_t)(0x0U << BB_COEXIFCNTL1_WLCPTXTHR_Pos))

#define WLCPRXTHR_0                     ((uint32_t)(0x0U << BB_COEXIFCNTL1_WLCPRXTHR_Pos))

/* BB_COEXIFCNTL1 sub-register bit positions */
#define BB_COEXIFCNTL1_WLCPDELAY_BYTE_Pos 0
#define BB_COEXIFCNTL1_WLCPDELAY_BYTE_Mask ((uint32_t)(0x7FU << BB_COEXIFCNTL1_WLCPDELAY_BYTE_Pos))
#define BB_COEXIFCNTL1_WLCPDURATION_BYTE_Pos 0
#define BB_COEXIFCNTL1_WLCPDURATION_BYTE_Mask ((uint32_t)(0x7FU << BB_COEXIFCNTL1_WLCPDURATION_BYTE_Pos))
#define BB_COEXIFCNTL1_WLCPTXTHR_BYTE_Pos 0
#define BB_COEXIFCNTL1_WLCPTXTHR_BYTE_Mask ((uint32_t)(0x1FU << BB_COEXIFCNTL1_WLCPTXTHR_BYTE_Pos))
#define BB_COEXIFCNTL1_WLCPRXTHR_BYTE_Pos 0
#define BB_COEXIFCNTL1_WLCPRXTHR_BYTE_Mask ((uint32_t)(0x1FU << BB_COEXIFCNTL1_WLCPRXTHR_BYTE_Pos))

/* BB_COEXIFCNTL1 sub-register settings */
#define WLCPDELAY_0_BYTE                ((uint8_t)(0x0U << BB_COEXIFCNTL1_WLCPDELAY_BYTE_Pos))

#define WLCPDURATION_0_BYTE             ((uint8_t)(0x0U << BB_COEXIFCNTL1_WLCPDURATION_BYTE_Pos))

#define WLCPTXTHR_0_BYTE                ((uint8_t)(0x0U << BB_COEXIFCNTL1_WLCPTXTHR_BYTE_Pos))

#define WLCPRXTHR_0_BYTE                ((uint8_t)(0x0U << BB_COEXIFCNTL1_WLCPRXTHR_BYTE_Pos))

/* BB_COEXIFCNTL2 bit positions */
#define BB_COEXIFCNTL2_RX_ANT_DELAY_Pos 8
#define BB_COEXIFCNTL2_RX_ANT_DELAY_Mask ((uint32_t)(0xFU << BB_COEXIFCNTL2_RX_ANT_DELAY_Pos))
#define BB_COEXIFCNTL2_TX_ANT_DELAY_Pos 0
#define BB_COEXIFCNTL2_TX_ANT_DELAY_Mask ((uint32_t)(0xFU << BB_COEXIFCNTL2_TX_ANT_DELAY_Pos))

/* BB_COEXIFCNTL2 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TX_ANT_DELAY_BYTE;    
    __IO uint8_t RX_ANT_DELAY_BYTE;    
         uint8_t RESERVED0[2];
} BB_COEXIFCNTL2_Type;

#define BB_COEXIFCNTL2_BASE             0x40001628
#define BB_COEXIFCNTL2                  ((BB_COEXIFCNTL2_Type *) BB_COEXIFCNTL2_BASE)

/* BB_COEXIFCNTL2 settings */
#define TX_ANT_DELAY_0                  ((uint32_t)(0x0U << BB_COEXIFCNTL2_TX_ANT_DELAY_Pos))

#define RX_ANT_DELAY_0                  ((uint32_t)(0x0U << BB_COEXIFCNTL2_RX_ANT_DELAY_Pos))

/* BB_COEXIFCNTL2 sub-register bit positions */
#define BB_COEXIFCNTL2_TX_ANT_DELAY_BYTE_Pos 0
#define BB_COEXIFCNTL2_TX_ANT_DELAY_BYTE_Mask ((uint32_t)(0xFU << BB_COEXIFCNTL2_TX_ANT_DELAY_BYTE_Pos))
#define BB_COEXIFCNTL2_RX_ANT_DELAY_BYTE_Pos 0
#define BB_COEXIFCNTL2_RX_ANT_DELAY_BYTE_Mask ((uint32_t)(0xFU << BB_COEXIFCNTL2_RX_ANT_DELAY_BYTE_Pos))

/* BB_COEXIFCNTL2 sub-register settings */
#define TX_ANT_DELAY_0_BYTE             ((uint8_t)(0x0U << BB_COEXIFCNTL2_TX_ANT_DELAY_BYTE_Pos))

#define RX_ANT_DELAY_0_BYTE             ((uint8_t)(0x0U << BB_COEXIFCNTL2_RX_ANT_DELAY_BYTE_Pos))

/* BB_BBMPRIO0 bit positions */
#define BB_BBMPRIO0_BLEM7_Pos           28
#define BB_BBMPRIO0_BLEM7_Mask          ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM7_Pos))
#define BB_BBMPRIO0_BLEM6_Pos           24
#define BB_BBMPRIO0_BLEM6_Mask          ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM6_Pos))
#define BB_BBMPRIO0_BLEM5_Pos           20
#define BB_BBMPRIO0_BLEM5_Mask          ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM5_Pos))
#define BB_BBMPRIO0_BLEM4_Pos           16
#define BB_BBMPRIO0_BLEM4_Mask          ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM4_Pos))
#define BB_BBMPRIO0_BLEM3_Pos           12
#define BB_BBMPRIO0_BLEM3_Mask          ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM3_Pos))
#define BB_BBMPRIO0_BLEM2_Pos           8
#define BB_BBMPRIO0_BLEM2_Mask          ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM2_Pos))
#define BB_BBMPRIO0_BLEM1_Pos           4
#define BB_BBMPRIO0_BLEM1_Mask          ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM1_Pos))
#define BB_BBMPRIO0_BLEM0_Pos           0
#define BB_BBMPRIO0_BLEM0_Mask          ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM0_Pos))

/* BB_BBMPRIO0 settings */
#define BLEM0_15                        ((uint32_t)(0xFU << BB_BBMPRIO0_BLEM0_Pos))

#define BLEM1_14                        ((uint32_t)(0xEU << BB_BBMPRIO0_BLEM1_Pos))

#define BLEM2_13                        ((uint32_t)(0xDU << BB_BBMPRIO0_BLEM2_Pos))

#define BLEM3_10                        ((uint32_t)(0xAU << BB_BBMPRIO0_BLEM3_Pos))

#define BLEM4_9                         ((uint32_t)(0x9U << BB_BBMPRIO0_BLEM4_Pos))

#define BLEM5_8                         ((uint32_t)(0x8U << BB_BBMPRIO0_BLEM5_Pos))

#define BLEM6_4                         ((uint32_t)(0x4U << BB_BBMPRIO0_BLEM6_Pos))

#define BLEM7_3                         ((uint32_t)(0x3U << BB_BBMPRIO0_BLEM7_Pos))

/* BB_BBMPRIO1 bit positions */
#define BB_BBMPRIO1_BLEMDEFAULT_Pos     28
#define BB_BBMPRIO1_BLEMDEFAULT_Mask    ((uint32_t)(0xFU << BB_BBMPRIO1_BLEMDEFAULT_Pos))
#define BB_BBMPRIO1_BLEM9_Pos           4
#define BB_BBMPRIO1_BLEM9_Mask          ((uint32_t)(0xFU << BB_BBMPRIO1_BLEM9_Pos))
#define BB_BBMPRIO1_BLEM8_Pos           0
#define BB_BBMPRIO1_BLEM8_Mask          ((uint32_t)(0xFU << BB_BBMPRIO1_BLEM8_Pos))

/* BB_BBMPRIO1 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t BLEMDEFAULT_BYTE;     
} BB_BBMPRIO1_Type;

#define BB_BBMPRIO1_BASE                0x40001630
#define BB_BBMPRIO1                     ((BB_BBMPRIO1_Type *) BB_BBMPRIO1_BASE)

/* BB_BBMPRIO1 settings */
#define BLEM6_12                        ((uint32_t)(0xCU << BB_BBMPRIO1_BLEM8_Pos))

#define BLEM7_13                        ((uint32_t)(0xDU << BB_BBMPRIO1_BLEM9_Pos))

#define BLEMDEFAULT_3                   ((uint32_t)(0x3U << BB_BBMPRIO1_BLEMDEFAULT_Pos))

/* BB_BBMPRIO1 sub-register bit positions */
#define BB_BBMPRIO1_BLEMDEFAULT_BYTE_Pos 4
#define BB_BBMPRIO1_BLEMDEFAULT_BYTE_Mask ((uint32_t)(0xFU << BB_BBMPRIO1_BLEMDEFAULT_BYTE_Pos))

/* BB_BBMPRIO1 sub-register settings */
#define BLEMDEFAULT_3_BYTE              ((uint8_t)(0x3U << BB_BBMPRIO1_BLEMDEFAULT_BYTE_Pos))

/* BB_RALPTR bit positions */
#define BB_RALPTR_RALPTR_Pos            0
#define BB_RALPTR_RALPTR_Mask           ((uint32_t)(0xFFFFU << BB_RALPTR_RALPTR_Pos))

/* BB_RALPTR settings */
#define RALPTR_0                        ((uint32_t)(0x0U << BB_RALPTR_RALPTR_Pos))

/* BB_RALNBDEV bit positions */
#define BB_RALNBDEV_RALNBDEV_Pos        0
#define BB_RALNBDEV_RALNBDEV_Mask       ((uint32_t)(0xFFU << BB_RALNBDEV_RALNBDEV_Pos))

/* BB_RALNBDEV settings */
#define RALNBDEV_0                      ((uint32_t)(0x0U << BB_RALNBDEV_RALNBDEV_Pos))

/* BB_RAL_LOCAL_RND bit positions */
#define BB_RAL_LOCAL_RND_LRND_INIT_Pos  31
#define BB_RAL_LOCAL_RND_LRND_VAL_Pos   0
#define BB_RAL_LOCAL_RND_LRND_VAL_Mask  ((uint32_t)(0x3FFFFFU << BB_RAL_LOCAL_RND_LRND_VAL_Pos))

/* BB_RAL_LOCAL_RND bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[31];
    __IO uint32_t LRND_INIT_ALIAS;      /* Writing a 1 initializes of local RPA random number generation LFSR */
} BB_RAL_LOCAL_RND_Type;

#define BB_RAL_LOCAL_RND_BASE           SYS_CALC_BITBAND(0x40001648, 0)
#define BB_RAL_LOCAL_RND                ((BB_RAL_LOCAL_RND_Type *) BB_RAL_LOCAL_RND_BASE)

/* BB_RAL_LOCAL_RND settings */
#define LRND_VAL_4132623                ((uint32_t)(0x3F0F0FU << BB_RAL_LOCAL_RND_LRND_VAL_Pos))

#define LRND_INIT_0_BITBAND             0x0
#define LRND_INIT_0                     ((uint32_t)(LRND_INIT_0_BITBAND << BB_RAL_LOCAL_RND_LRND_INIT_Pos))

/* BB_RAL_PEER_RND bit positions */
#define BB_RAL_PEER_RND_PRND_INIT_Pos   31
#define BB_RAL_PEER_RND_PRND_VAL_Pos    0
#define BB_RAL_PEER_RND_PRND_VAL_Mask   ((uint32_t)(0x3FFFFFU << BB_RAL_PEER_RND_PRND_VAL_Pos))

/* BB_RAL_PEER_RND bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[31];
    __IO uint32_t PRND_INIT_ALIAS;      /* Writing a 1 initializes of peer RPA random number generation LFSR */
} BB_RAL_PEER_RND_Type;

#define BB_RAL_PEER_RND_BASE            SYS_CALC_BITBAND(0x4000164C, 0)
#define BB_RAL_PEER_RND                 ((BB_RAL_PEER_RND_Type *) BB_RAL_PEER_RND_BASE)

/* BB_RAL_PEER_RND settings */
#define PRND_VAL_3207408                ((uint32_t)(0x30F0F0U << BB_RAL_PEER_RND_PRND_VAL_Pos))

#define PRND_INIT_0_BITBAND             0x0
#define PRND_INIT_0                     ((uint32_t)(PRND_INIT_0_BITBAND << BB_RAL_PEER_RND_PRND_INIT_Pos))

/* BB_ISOCHANCNTL0 bit positions */
#define BB_ISOCHANCNTL0_RETXACKEN0_Pos  4
#define BB_ISOCHANCNTL0_SYNCGEN0_Pos    3
#define BB_ISOCHANCNTL0_ISOCHANEN0_Pos  2
#define BB_ISOCHANCNTL0_ISOTYPE0_Pos    0
#define BB_ISOCHANCNTL0_ISOTYPE0_Mask   ((uint32_t)(0x3U << BB_ISOCHANCNTL0_ISOTYPE0_Pos))

/* BB_ISOCHANCNTL0 bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[2];
    __IO uint32_t ISOCHANEN0_ALIAS;     /* Enable ISO channel */
    __IO uint32_t SYNCGEN0_ALIAS;       /* Enable audio syn_p generation */
    __IO uint32_t RETXACKEN0_ALIAS;     /* Generate Tx ACK */
} BB_ISOCHANCNTL0_Type;

#define BB_ISOCHANCNTL0_BASE            SYS_CALC_BITBAND(0x40001650, 0)
#define BB_ISOCHANCNTL0                 ((BB_ISOCHANCNTL0_Type *) BB_ISOCHANCNTL0_BASE)

/* BB_ISOCHANCNTL0 settings */
#define ISOTYPE0_0                      ((uint32_t)(0x0U << BB_ISOCHANCNTL0_ISOTYPE0_Pos))
#define ISOTYPE0_1                      ((uint32_t)(0x1U << BB_ISOCHANCNTL0_ISOTYPE0_Pos))
#define ISOTYPE0_2                      ((uint32_t)(0x2U << BB_ISOCHANCNTL0_ISOTYPE0_Pos))
#define ISOTYPE0_3                      ((uint32_t)(0x3U << BB_ISOCHANCNTL0_ISOTYPE0_Pos))

#define ISOCHANEN0_0_BITBAND            0x0
#define ISOCHANEN0_1_BITBAND            0x1
#define ISOCHANEN0_0                    ((uint32_t)(ISOCHANEN0_0_BITBAND << BB_ISOCHANCNTL0_ISOCHANEN0_Pos))
#define ISOCHANEN0_1                    ((uint32_t)(ISOCHANEN0_1_BITBAND << BB_ISOCHANCNTL0_ISOCHANEN0_Pos))

#define SYNCGEN0_0_BITBAND              0x0
#define SYNCGEN0_1_BITBAND              0x1
#define SYNCGEN0_0                      ((uint32_t)(SYNCGEN0_0_BITBAND << BB_ISOCHANCNTL0_SYNCGEN0_Pos))
#define SYNCGEN0_1                      ((uint32_t)(SYNCGEN0_1_BITBAND << BB_ISOCHANCNTL0_SYNCGEN0_Pos))

#define RETXACKEN0_0_BITBAND            0x0
#define RETXACKEN0_1_BITBAND            0x1
#define RETXACKEN0_0                    ((uint32_t)(RETXACKEN0_0_BITBAND << BB_ISOCHANCNTL0_RETXACKEN0_Pos))
#define RETXACKEN0_1                    ((uint32_t)(RETXACKEN0_1_BITBAND << BB_ISOCHANCNTL0_RETXACKEN0_Pos))

/* BB_ISOMUTECNTL0 bit positions */
#define BB_ISOMUTECNTL0_TOGO0_Pos       31
#define BB_ISOMUTECNTL0_MUTE_SINK0_Pos  19
#define BB_ISOMUTECNTL0_MUTE_SOURCE0_Pos 18
#define BB_ISOMUTECNTL0_INVL0_1_Pos     17
#define BB_ISOMUTECNTL0_INVL0_0_Pos     16
#define BB_ISOMUTECNTL0_MUTE_PATTERN0_Pos 0
#define BB_ISOMUTECNTL0_MUTE_PATTERN0_Mask ((uint32_t)(0xFFU << BB_ISOMUTECNTL0_MUTE_PATTERN0_Pos))

/* BB_ISOMUTECNTL0 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t MUTE_PATTERN0_BYTE;   
         uint8_t RESERVED0[2];
    __IO uint8_t TOGO0_BYTE;           
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001654, 0) - (0x40001654 + 4)];
         uint32_t RESERVED1[16];
    __IO uint32_t INVL0_0_ALIAS;        /* SW mute status for ISO buffer 0 (i.e updated when ET-ISOBUFSEL = 1) */
    __IO uint32_t INVL0_1_ALIAS;        /* SW mute status for ISO buffer 1 (i.e updated when ET-ISOBUFSEL = 0) */
    __IO uint32_t MUTE_SOURCE0_ALIAS;   /* HW mute control */
    __IO uint32_t MUTE_SINK0_ALIAS;     /* HW mute control */
         uint32_t RESERVED2[11];
    __IO uint32_t TOGO0_ALIAS;          /* Indicates which buffer is in use (direct copy of ET-ISOBUFSEL) */
} BB_ISOMUTECNTL0_Type;

#define BB_ISOMUTECNTL0_BASE            0x40001654
#define BB_ISOMUTECNTL0                 ((BB_ISOMUTECNTL0_Type *) BB_ISOMUTECNTL0_BASE)

/* BB_ISOMUTECNTL0 settings */
#define MUTE_PATTERN0_0                 ((uint32_t)(0x0U << BB_ISOMUTECNTL0_MUTE_PATTERN0_Pos))

#define INVL0_0_0_BITBAND               0x0
#define INVL0_0_1_BITBAND               0x1
#define INVL0_0_0                       ((uint32_t)(INVL0_0_0_BITBAND << BB_ISOMUTECNTL0_INVL0_0_Pos))
#define INVL0_0_1                       ((uint32_t)(INVL0_0_1_BITBAND << BB_ISOMUTECNTL0_INVL0_0_Pos))

#define INVL0_1_0_BITBAND               0x0
#define INVL0_1_1_BITBAND               0x1
#define INVL0_1_0                       ((uint32_t)(INVL0_1_0_BITBAND << BB_ISOMUTECNTL0_INVL0_1_Pos))
#define INVL0_1_1                       ((uint32_t)(INVL0_1_1_BITBAND << BB_ISOMUTECNTL0_INVL0_1_Pos))

#define MUTE_SOURCE0_0_BITBAND          0x0
#define MUTE_SOURCE0_1_BITBAND          0x1
#define MUTE_SOURCE0_0                  ((uint32_t)(MUTE_SOURCE0_0_BITBAND << BB_ISOMUTECNTL0_MUTE_SOURCE0_Pos))
#define MUTE_SOURCE0_1                  ((uint32_t)(MUTE_SOURCE0_1_BITBAND << BB_ISOMUTECNTL0_MUTE_SOURCE0_Pos))

#define MUTE_SINK0_0_BITBAND            0x0
#define MUTE_SINK0_1_BITBAND            0x1
#define MUTE_SINK0_0                    ((uint32_t)(MUTE_SINK0_0_BITBAND << BB_ISOMUTECNTL0_MUTE_SINK0_Pos))
#define MUTE_SINK0_1                    ((uint32_t)(MUTE_SINK0_1_BITBAND << BB_ISOMUTECNTL0_MUTE_SINK0_Pos))

#define TOGO0_0_BITBAND                 0x0
#define TOGO0_1_BITBAND                 0x1
#define TOGO0_0                         ((uint32_t)(TOGO0_0_BITBAND << BB_ISOMUTECNTL0_TOGO0_Pos))
#define TOGO0_1                         ((uint32_t)(TOGO0_1_BITBAND << BB_ISOMUTECNTL0_TOGO0_Pos))

/* BB_ISOMUTECNTL0 sub-register bit positions */
#define BB_ISOMUTECNTL0_TOGO0_BYTE_Pos  7

/* BB_ISOMUTECNTL0 sub-register settings */
#define MUTE_PATTERN0_0_BYTE            ((uint8_t)0x0U)

#define TOGO0_0_BYTE                    ((uint8_t)(TOGO0_0_BITBAND << BB_ISOMUTECNTL0_TOGO0_BYTE_Pos))
#define TOGO0_1_BYTE                    ((uint8_t)(TOGO0_1_BITBAND << BB_ISOMUTECNTL0_TOGO0_BYTE_Pos))

/* BB_ISOCURRENTTXPTR0 bit positions */
#define BB_ISOCURRENTTXPTR0_ISO0TXPTR0_Pos 16
#define BB_ISOCURRENTTXPTR0_ISO0TXPTR0_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTTXPTR0_ISO0TXPTR0_Pos))
#define BB_ISOCURRENTTXPTR0_ISO0TXPTR1_Pos 0
#define BB_ISOCURRENTTXPTR0_ISO0TXPTR1_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTTXPTR0_ISO0TXPTR1_Pos))

/* BB_ISOCURRENTTXPTR0 settings */
#define ISO0TXPTR1_0                    ((uint32_t)(0x0U << BB_ISOCURRENTTXPTR0_ISO0TXPTR1_Pos))

#define ISO0TXPTR0_0                    ((uint32_t)(0x0U << BB_ISOCURRENTTXPTR0_ISO0TXPTR0_Pos))

/* BB_ISOCURRENTRXPTR0 bit positions */
#define BB_ISOCURRENTRXPTR0_ISO0RXPTR0_Pos 16
#define BB_ISOCURRENTRXPTR0_ISO0RXPTR0_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTRXPTR0_ISO0RXPTR0_Pos))
#define BB_ISOCURRENTRXPTR0_ISO0RXPTR1_Pos 0
#define BB_ISOCURRENTRXPTR0_ISO0RXPTR1_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTRXPTR0_ISO0RXPTR1_Pos))

/* BB_ISOCURRENTRXPTR0 settings */
#define ISO0RXPTR1_0                    ((uint32_t)(0x0U << BB_ISOCURRENTRXPTR0_ISO0RXPTR1_Pos))

#define ISO0RXPTR0_0                    ((uint32_t)(0x0U << BB_ISOCURRENTRXPTR0_ISO0RXPTR0_Pos))

/* BB_ISOTRCNL0 bit positions */
#define BB_ISOTRCNL0_ISO0RXLEN_Pos      16
#define BB_ISOTRCNL0_ISO0RXLEN_Mask     ((uint32_t)(0xFFU << BB_ISOTRCNL0_ISO0RXLEN_Pos))
#define BB_ISOTRCNL0_ISO0TXLEN_Pos      0
#define BB_ISOTRCNL0_ISO0TXLEN_Mask     ((uint32_t)(0xFFU << BB_ISOTRCNL0_ISO0TXLEN_Pos))

/* BB_ISOTRCNL0 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t ISO0TXLEN_BYTE;       
         uint8_t RESERVED0[1];
    __IO uint8_t ISO0RXLEN_BYTE;       
         uint8_t RESERVED1[1];
} BB_ISOTRCNL0_Type;

#define BB_ISOTRCNL0_BASE               0x40001660
#define BB_ISOTRCNL0                    ((BB_ISOTRCNL0_Type *) BB_ISOTRCNL0_BASE)

/* BB_ISOTRCNL0 settings */
#define ISO0TXLEN_0                     ((uint32_t)(0x0U << BB_ISOTRCNL0_ISO0TXLEN_Pos))

#define ISO0RXLEN_0                     ((uint32_t)(0x0U << BB_ISOTRCNL0_ISO0RXLEN_Pos))

/* BB_ISOTRCNL0 sub-register settings */
#define ISO0TXLEN_0_BYTE                ((uint8_t)0x0U)

#define ISO0RXLEN_0_BYTE                ((uint8_t)0x0U)

/* BB_ISOEVTCNTLOFFSETL0 bit positions */
#define BB_ISOEVTCNTLOFFSETL0_EVT_CNT_OFFSETL0_Pos 0
#define BB_ISOEVTCNTLOFFSETL0_EVT_CNT_OFFSETL0_Mask ((uint32_t)(0xFFFFFFFFU << BB_ISOEVTCNTLOFFSETL0_EVT_CNT_OFFSETL0_Pos))

/* BB_ISOEVTCNTLOFFSETL0 settings */
#define EVT_CNT_OFFSETL0_0              ((uint32_t)(0x0U << BB_ISOEVTCNTLOFFSETL0_EVT_CNT_OFFSETL0_Pos))

/* BB_ISOEVTCNTLOFFSETU0 bit positions */
#define BB_ISOEVTCNTLOFFSETU0_EVT_CNT_OFFSETU0_Pos 0
#define BB_ISOEVTCNTLOFFSETU0_EVT_CNT_OFFSETU0_Mask ((uint32_t)(0x7FU << BB_ISOEVTCNTLOFFSETU0_EVT_CNT_OFFSETU0_Pos))

/* BB_ISOEVTCNTLOFFSETU0 settings */
#define EVT_CNT_OFFSETU0_0              ((uint32_t)(0x0U << BB_ISOEVTCNTLOFFSETU0_EVT_CNT_OFFSETU0_Pos))

/* BB_ISOCHANCNTL1 bit positions */
#define BB_ISOCHANCNTL1_RETXACKEN1_Pos  4
#define BB_ISOCHANCNTL1_SYNCGEN1_Pos    3
#define BB_ISOCHANCNTL1_ISOCHANEN1_Pos  2
#define BB_ISOCHANCNTL1_ISOTYPE1_Pos    0
#define BB_ISOCHANCNTL1_ISOTYPE1_Mask   ((uint32_t)(0x3U << BB_ISOCHANCNTL1_ISOTYPE1_Pos))

/* BB_ISOCHANCNTL1 bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[2];
    __IO uint32_t ISOCHANEN1_ALIAS;     /* Enable ISO channel */
    __IO uint32_t SYNCGEN1_ALIAS;       /* Enable audio syn_p generation */
    __IO uint32_t RETXACKEN1_ALIAS;     /* Generate Tx ACK */
} BB_ISOCHANCNTL1_Type;

#define BB_ISOCHANCNTL1_BASE            SYS_CALC_BITBAND(0x40001670, 0)
#define BB_ISOCHANCNTL1                 ((BB_ISOCHANCNTL1_Type *) BB_ISOCHANCNTL1_BASE)

/* BB_ISOCHANCNTL1 settings */
#define ISOTYPE1_0                      ((uint32_t)(0x0U << BB_ISOCHANCNTL1_ISOTYPE1_Pos))
#define ISOTYPE1_1                      ((uint32_t)(0x1U << BB_ISOCHANCNTL1_ISOTYPE1_Pos))
#define ISOTYPE1_2                      ((uint32_t)(0x2U << BB_ISOCHANCNTL1_ISOTYPE1_Pos))
#define ISOTYPE1_3                      ((uint32_t)(0x3U << BB_ISOCHANCNTL1_ISOTYPE1_Pos))

#define ISOCHANEN1_0_BITBAND            0x0
#define ISOCHANEN1_1_BITBAND            0x1
#define ISOCHANEN1_0                    ((uint32_t)(ISOCHANEN1_0_BITBAND << BB_ISOCHANCNTL1_ISOCHANEN1_Pos))
#define ISOCHANEN1_1                    ((uint32_t)(ISOCHANEN1_1_BITBAND << BB_ISOCHANCNTL1_ISOCHANEN1_Pos))

#define SYNCGEN1_0_BITBAND              0x0
#define SYNCGEN1_1_BITBAND              0x1
#define SYNCGEN1_0                      ((uint32_t)(SYNCGEN1_0_BITBAND << BB_ISOCHANCNTL1_SYNCGEN1_Pos))
#define SYNCGEN1_1                      ((uint32_t)(SYNCGEN1_1_BITBAND << BB_ISOCHANCNTL1_SYNCGEN1_Pos))

#define RETXACKEN1_0_BITBAND            0x0
#define RETXACKEN1_1_BITBAND            0x1
#define RETXACKEN1_0                    ((uint32_t)(RETXACKEN1_0_BITBAND << BB_ISOCHANCNTL1_RETXACKEN1_Pos))
#define RETXACKEN1_1                    ((uint32_t)(RETXACKEN1_1_BITBAND << BB_ISOCHANCNTL1_RETXACKEN1_Pos))

/* BB_ISOMUTECNTL1 bit positions */
#define BB_ISOMUTECNTL1_TOGO1_Pos       31
#define BB_ISOMUTECNTL1_MUTE_SINK1_Pos  19
#define BB_ISOMUTECNTL1_MUTE_SOURCE1_Pos 18
#define BB_ISOMUTECNTL1_INVL1_1_Pos     17
#define BB_ISOMUTECNTL1_INVL1_0_Pos     16
#define BB_ISOMUTECNTL1_MUTE_PATTERN1_Pos 0
#define BB_ISOMUTECNTL1_MUTE_PATTERN1_Mask ((uint32_t)(0xFFU << BB_ISOMUTECNTL1_MUTE_PATTERN1_Pos))

/* BB_ISOMUTECNTL1 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t MUTE_PATTERN1_BYTE;   
         uint8_t RESERVED0[2];
    __IO uint8_t TOGO1_BYTE;           
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001674, 0) - (0x40001674 + 4)];
         uint32_t RESERVED1[16];
    __IO uint32_t INVL1_0_ALIAS;        /* SW mute status for ISO buffer 0 (i.e updated when ET-ISOBUFSEL = 1) */
    __IO uint32_t INVL1_1_ALIAS;        /* SW mute status for ISO buffer 1 (i.e updated when ET-ISOBUFSEL = 0) */
    __IO uint32_t MUTE_SOURCE1_ALIAS;   /* HW mute control */
    __IO uint32_t MUTE_SINK1_ALIAS;     /* HW mute control */
         uint32_t RESERVED2[11];
    __IO uint32_t TOGO1_ALIAS;          /* Indicates which buffer is in use (direct copy of ET-ISOBUFSEL) */
} BB_ISOMUTECNTL1_Type;

#define BB_ISOMUTECNTL1_BASE            0x40001674
#define BB_ISOMUTECNTL1                 ((BB_ISOMUTECNTL1_Type *) BB_ISOMUTECNTL1_BASE)

/* BB_ISOMUTECNTL1 settings */
#define MUTE_PATTERN1_0                 ((uint32_t)(0x0U << BB_ISOMUTECNTL1_MUTE_PATTERN1_Pos))

#define INVL1_0_0_BITBAND               0x0
#define INVL1_0_1_BITBAND               0x1
#define INVL1_0_0                       ((uint32_t)(INVL1_0_0_BITBAND << BB_ISOMUTECNTL1_INVL1_0_Pos))
#define INVL1_0_1                       ((uint32_t)(INVL1_0_1_BITBAND << BB_ISOMUTECNTL1_INVL1_0_Pos))

#define INVL1_1_0_BITBAND               0x0
#define INVL1_1_1_BITBAND               0x1
#define INVL1_1_0                       ((uint32_t)(INVL1_1_0_BITBAND << BB_ISOMUTECNTL1_INVL1_1_Pos))
#define INVL1_1_1                       ((uint32_t)(INVL1_1_1_BITBAND << BB_ISOMUTECNTL1_INVL1_1_Pos))

#define MUTE_SOURCE1_0_BITBAND          0x0
#define MUTE_SOURCE1_1_BITBAND          0x1
#define MUTE_SOURCE1_0                  ((uint32_t)(MUTE_SOURCE1_0_BITBAND << BB_ISOMUTECNTL1_MUTE_SOURCE1_Pos))
#define MUTE_SOURCE1_1                  ((uint32_t)(MUTE_SOURCE1_1_BITBAND << BB_ISOMUTECNTL1_MUTE_SOURCE1_Pos))

#define MUTE_SINK1_0_BITBAND            0x0
#define MUTE_SINK1_1_BITBAND            0x1
#define MUTE_SINK1_0                    ((uint32_t)(MUTE_SINK1_0_BITBAND << BB_ISOMUTECNTL1_MUTE_SINK1_Pos))
#define MUTE_SINK1_1                    ((uint32_t)(MUTE_SINK1_1_BITBAND << BB_ISOMUTECNTL1_MUTE_SINK1_Pos))

#define TOGO1_0_BITBAND                 0x0
#define TOGO1_1_BITBAND                 0x1
#define TOGO1_0                         ((uint32_t)(TOGO1_0_BITBAND << BB_ISOMUTECNTL1_TOGO1_Pos))
#define TOGO1_1                         ((uint32_t)(TOGO1_1_BITBAND << BB_ISOMUTECNTL1_TOGO1_Pos))

/* BB_ISOMUTECNTL1 sub-register bit positions */
#define BB_ISOMUTECNTL1_TOGO1_BYTE_Pos  7

/* BB_ISOMUTECNTL1 sub-register settings */
#define MUTE_PATTERN1_0_BYTE            ((uint8_t)0x0U)

#define TOGO1_0_BYTE                    ((uint8_t)(TOGO1_0_BITBAND << BB_ISOMUTECNTL1_TOGO1_BYTE_Pos))
#define TOGO1_1_BYTE                    ((uint8_t)(TOGO1_1_BITBAND << BB_ISOMUTECNTL1_TOGO1_BYTE_Pos))

/* BB_ISOCURRENTTXPTR1 bit positions */
#define BB_ISOCURRENTTXPTR1_ISO1TXPTR0_Pos 16
#define BB_ISOCURRENTTXPTR1_ISO1TXPTR0_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTTXPTR1_ISO1TXPTR0_Pos))
#define BB_ISOCURRENTTXPTR1_ISO1TXPTR1_Pos 0
#define BB_ISOCURRENTTXPTR1_ISO1TXPTR1_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTTXPTR1_ISO1TXPTR1_Pos))

/* BB_ISOCURRENTTXPTR1 settings */
#define ISO1TXPTR1_0                    ((uint32_t)(0x0U << BB_ISOCURRENTTXPTR1_ISO1TXPTR1_Pos))

#define ISO1TXPTR0_0                    ((uint32_t)(0x0U << BB_ISOCURRENTTXPTR1_ISO1TXPTR0_Pos))

/* BB_ISOCURRENTRXPTR1 bit positions */
#define BB_ISOCURRENTRXPTR1_ISO1RXPTR0_Pos 16
#define BB_ISOCURRENTRXPTR1_ISO1RXPTR0_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTRXPTR1_ISO1RXPTR0_Pos))
#define BB_ISOCURRENTRXPTR1_ISO1RXPTR1_Pos 0
#define BB_ISOCURRENTRXPTR1_ISO1RXPTR1_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTRXPTR1_ISO1RXPTR1_Pos))

/* BB_ISOCURRENTRXPTR1 settings */
#define ISO1RXPTR1_0                    ((uint32_t)(0x0U << BB_ISOCURRENTRXPTR1_ISO1RXPTR1_Pos))

#define ISO1RXPTR0_0                    ((uint32_t)(0x0U << BB_ISOCURRENTRXPTR1_ISO1RXPTR0_Pos))

/* BB_ISOTRCNL1 bit positions */
#define BB_ISOTRCNL1_ISO1RXLEN_Pos      16
#define BB_ISOTRCNL1_ISO1RXLEN_Mask     ((uint32_t)(0xFFU << BB_ISOTRCNL1_ISO1RXLEN_Pos))
#define BB_ISOTRCNL1_ISO1TXLEN_Pos      0
#define BB_ISOTRCNL1_ISO1TXLEN_Mask     ((uint32_t)(0xFFU << BB_ISOTRCNL1_ISO1TXLEN_Pos))

/* BB_ISOTRCNL1 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t ISO1TXLEN_BYTE;       
         uint8_t RESERVED0[1];
    __IO uint8_t ISO1RXLEN_BYTE;       
         uint8_t RESERVED1[1];
} BB_ISOTRCNL1_Type;

#define BB_ISOTRCNL1_BASE               0x40001680
#define BB_ISOTRCNL1                    ((BB_ISOTRCNL1_Type *) BB_ISOTRCNL1_BASE)

/* BB_ISOTRCNL1 settings */
#define ISO1TXLEN_0                     ((uint32_t)(0x0U << BB_ISOTRCNL1_ISO1TXLEN_Pos))

#define ISO1RXLEN_0                     ((uint32_t)(0x0U << BB_ISOTRCNL1_ISO1RXLEN_Pos))

/* BB_ISOTRCNL1 sub-register settings */
#define ISO1TXLEN_0_BYTE                ((uint8_t)0x0U)

#define ISO1RXLEN_0_BYTE                ((uint8_t)0x0U)

/* BB_ISOEVTCNTLOFFSETL1 bit positions */
#define BB_ISOEVTCNTLOFFSETL1_EVT_CNT_OFFSETL1_Pos 0
#define BB_ISOEVTCNTLOFFSETL1_EVT_CNT_OFFSETL1_Mask ((uint32_t)(0xFFFFFFFFU << BB_ISOEVTCNTLOFFSETL1_EVT_CNT_OFFSETL1_Pos))

/* BB_ISOEVTCNTLOFFSETL1 settings */
#define EVT_CNT_OFFSETL1_0              ((uint32_t)(0x0U << BB_ISOEVTCNTLOFFSETL1_EVT_CNT_OFFSETL1_Pos))

/* BB_ISOEVTCNTLOFFSETU1 bit positions */
#define BB_ISOEVTCNTLOFFSETU1_EVT_CNT_OFFSETU1_Pos 0
#define BB_ISOEVTCNTLOFFSETU1_EVT_CNT_OFFSETU1_Mask ((uint32_t)(0x7FU << BB_ISOEVTCNTLOFFSETU1_EVT_CNT_OFFSETU1_Pos))

/* BB_ISOEVTCNTLOFFSETU1 settings */
#define EVT_CNT_OFFSETU1_0              ((uint32_t)(0x0U << BB_ISOEVTCNTLOFFSETU1_EVT_CNT_OFFSETU1_Pos))

/* BB_ISOCHANCNTL2 bit positions */
#define BB_ISOCHANCNTL2_RETXACKEN2_Pos  4
#define BB_ISOCHANCNTL2_SYNCGEN2_Pos    3
#define BB_ISOCHANCNTL2_ISOCHANEN2_Pos  2
#define BB_ISOCHANCNTL2_ISOTYPE2_Pos    0
#define BB_ISOCHANCNTL2_ISOTYPE2_Mask   ((uint32_t)(0x3U << BB_ISOCHANCNTL2_ISOTYPE2_Pos))

/* BB_ISOCHANCNTL2 bit-band aliases */
typedef struct
{
         uint32_t RESERVED0[2];
    __IO uint32_t ISOCHANEN2_ALIAS;     /* Enable ISO channel */
    __IO uint32_t SYNCGEN2_ALIAS;       /* Enable audio syn_p generation */
    __IO uint32_t RETXACKEN2_ALIAS;     /* Generate Tx ACK */
} BB_ISOCHANCNTL2_Type;

#define BB_ISOCHANCNTL2_BASE            SYS_CALC_BITBAND(0x40001690, 0)
#define BB_ISOCHANCNTL2                 ((BB_ISOCHANCNTL2_Type *) BB_ISOCHANCNTL2_BASE)

/* BB_ISOCHANCNTL2 settings */
#define ISOTYPE2_0                      ((uint32_t)(0x0U << BB_ISOCHANCNTL2_ISOTYPE2_Pos))
#define ISOTYPE2_1                      ((uint32_t)(0x1U << BB_ISOCHANCNTL2_ISOTYPE2_Pos))
#define ISOTYPE2_2                      ((uint32_t)(0x2U << BB_ISOCHANCNTL2_ISOTYPE2_Pos))
#define ISOTYPE2_3                      ((uint32_t)(0x3U << BB_ISOCHANCNTL2_ISOTYPE2_Pos))

#define ISOCHANEN2_0_BITBAND            0x0
#define ISOCHANEN2_1_BITBAND            0x1
#define ISOCHANEN2_0                    ((uint32_t)(ISOCHANEN2_0_BITBAND << BB_ISOCHANCNTL2_ISOCHANEN2_Pos))
#define ISOCHANEN2_1                    ((uint32_t)(ISOCHANEN2_1_BITBAND << BB_ISOCHANCNTL2_ISOCHANEN2_Pos))

#define SYNCGEN2_0_BITBAND              0x0
#define SYNCGEN2_1_BITBAND              0x1
#define SYNCGEN2_0                      ((uint32_t)(SYNCGEN2_0_BITBAND << BB_ISOCHANCNTL2_SYNCGEN2_Pos))
#define SYNCGEN2_1                      ((uint32_t)(SYNCGEN2_1_BITBAND << BB_ISOCHANCNTL2_SYNCGEN2_Pos))

#define RETXACKEN2_0_BITBAND            0x0
#define RETXACKEN2_1_BITBAND            0x1
#define RETXACKEN2_0                    ((uint32_t)(RETXACKEN2_0_BITBAND << BB_ISOCHANCNTL2_RETXACKEN2_Pos))
#define RETXACKEN2_1                    ((uint32_t)(RETXACKEN2_1_BITBAND << BB_ISOCHANCNTL2_RETXACKEN2_Pos))

/* BB_ISOMUTECNTL2 bit positions */
#define BB_ISOMUTECNTL2_TOGO2_Pos       31
#define BB_ISOMUTECNTL2_MUTE_SINK2_Pos  19
#define BB_ISOMUTECNTL2_MUTE_SOURCE2_Pos 18
#define BB_ISOMUTECNTL2_INVL2_1_Pos     17
#define BB_ISOMUTECNTL2_INVL2_0_Pos     16
#define BB_ISOMUTECNTL2_MUTE_PATTERN2_Pos 0
#define BB_ISOMUTECNTL2_MUTE_PATTERN2_Mask ((uint32_t)(0xFFU << BB_ISOMUTECNTL2_MUTE_PATTERN2_Pos))

/* BB_ISOMUTECNTL2 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t MUTE_PATTERN2_BYTE;   
         uint8_t RESERVED0[2];
    __IO uint8_t TOGO2_BYTE;           
         uint8_t RESERVED[SYS_CALC_BITBAND(0x40001694, 0) - (0x40001694 + 4)];
         uint32_t RESERVED1[16];
    __IO uint32_t INVL2_0_ALIAS;        /* SW mute status for ISO buffer 0 (i.e updated when ET-ISOBUFSEL = 1) */
    __IO uint32_t INVL2_1_ALIAS;        /* SW mute status for ISO buffer 1 (i.e updated when ET-ISOBUFSEL = 0) */
    __IO uint32_t MUTE_SOURCE2_ALIAS;   /* HW mute control */
    __IO uint32_t MUTE_SINK2_ALIAS;     /* HW mute control */
         uint32_t RESERVED2[11];
    __IO uint32_t TOGO2_ALIAS;          /* Indicates which buffer is in use (direct copy of ET-ISOBUFSEL) */
} BB_ISOMUTECNTL2_Type;

#define BB_ISOMUTECNTL2_BASE            0x40001694
#define BB_ISOMUTECNTL2                 ((BB_ISOMUTECNTL2_Type *) BB_ISOMUTECNTL2_BASE)

/* BB_ISOMUTECNTL2 settings */
#define MUTE_PATTERN2_0                 ((uint32_t)(0x0U << BB_ISOMUTECNTL2_MUTE_PATTERN2_Pos))

#define INVL2_0_0_BITBAND               0x0
#define INVL2_0_1_BITBAND               0x1
#define INVL2_0_0                       ((uint32_t)(INVL2_0_0_BITBAND << BB_ISOMUTECNTL2_INVL2_0_Pos))
#define INVL2_0_1                       ((uint32_t)(INVL2_0_1_BITBAND << BB_ISOMUTECNTL2_INVL2_0_Pos))

#define INVL2_1_0_BITBAND               0x0
#define INVL2_1_1_BITBAND               0x1
#define INVL2_1_0                       ((uint32_t)(INVL2_1_0_BITBAND << BB_ISOMUTECNTL2_INVL2_1_Pos))
#define INVL2_1_1                       ((uint32_t)(INVL2_1_1_BITBAND << BB_ISOMUTECNTL2_INVL2_1_Pos))

#define MUTE_SOURCE2_0_BITBAND          0x0
#define MUTE_SOURCE2_1_BITBAND          0x1
#define MUTE_SOURCE2_0                  ((uint32_t)(MUTE_SOURCE2_0_BITBAND << BB_ISOMUTECNTL2_MUTE_SOURCE2_Pos))
#define MUTE_SOURCE2_1                  ((uint32_t)(MUTE_SOURCE2_1_BITBAND << BB_ISOMUTECNTL2_MUTE_SOURCE2_Pos))

#define MUTE_SINK2_0_BITBAND            0x0
#define MUTE_SINK2_1_BITBAND            0x1
#define MUTE_SINK2_0                    ((uint32_t)(MUTE_SINK2_0_BITBAND << BB_ISOMUTECNTL2_MUTE_SINK2_Pos))
#define MUTE_SINK2_1                    ((uint32_t)(MUTE_SINK2_1_BITBAND << BB_ISOMUTECNTL2_MUTE_SINK2_Pos))

#define TOGO2_0_BITBAND                 0x0
#define TOGO2_1_BITBAND                 0x1
#define TOGO2_0                         ((uint32_t)(TOGO2_0_BITBAND << BB_ISOMUTECNTL2_TOGO2_Pos))
#define TOGO2_1                         ((uint32_t)(TOGO2_1_BITBAND << BB_ISOMUTECNTL2_TOGO2_Pos))

/* BB_ISOMUTECNTL2 sub-register bit positions */
#define BB_ISOMUTECNTL2_TOGO2_BYTE_Pos  7

/* BB_ISOMUTECNTL2 sub-register settings */
#define MUTE_PATTERN2_0_BYTE            ((uint8_t)0x0U)

#define TOGO2_0_BYTE                    ((uint8_t)(TOGO2_0_BITBAND << BB_ISOMUTECNTL2_TOGO2_BYTE_Pos))
#define TOGO2_1_BYTE                    ((uint8_t)(TOGO2_1_BITBAND << BB_ISOMUTECNTL2_TOGO2_BYTE_Pos))

/* BB_ISOCURRENTTXPTR2 bit positions */
#define BB_ISOCURRENTTXPTR2_ISO2TXPTR0_Pos 16
#define BB_ISOCURRENTTXPTR2_ISO2TXPTR0_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTTXPTR2_ISO2TXPTR0_Pos))
#define BB_ISOCURRENTTXPTR2_ISO2TXPTR1_Pos 0
#define BB_ISOCURRENTTXPTR2_ISO2TXPTR1_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTTXPTR2_ISO2TXPTR1_Pos))

/* BB_ISOCURRENTTXPTR2 settings */
#define ISO2TXPTR1_0                    ((uint32_t)(0x0U << BB_ISOCURRENTTXPTR2_ISO2TXPTR1_Pos))

#define ISO2TXPTR0_0                    ((uint32_t)(0x0U << BB_ISOCURRENTTXPTR2_ISO2TXPTR0_Pos))

/* BB_ISOCURRENTRXPTR2 bit positions */
#define BB_ISOCURRENTRXPTR2_ISO2RXPTR0_Pos 16
#define BB_ISOCURRENTRXPTR2_ISO2RXPTR0_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTRXPTR2_ISO2RXPTR0_Pos))
#define BB_ISOCURRENTRXPTR2_ISO2RXPTR1_Pos 0
#define BB_ISOCURRENTRXPTR2_ISO2RXPTR1_Mask ((uint32_t)(0xFFFFU << BB_ISOCURRENTRXPTR2_ISO2RXPTR1_Pos))

/* BB_ISOCURRENTRXPTR2 settings */
#define ISO2RXPTR1_0                    ((uint32_t)(0x0U << BB_ISOCURRENTRXPTR2_ISO2RXPTR1_Pos))

#define ISO2RXPTR0_0                    ((uint32_t)(0x0U << BB_ISOCURRENTRXPTR2_ISO2RXPTR0_Pos))

/* BB_ISOTRCNL2 bit positions */
#define BB_ISOTRCNL2_ISO2RXLEN_Pos      16
#define BB_ISOTRCNL2_ISO2RXLEN_Mask     ((uint32_t)(0xFFU << BB_ISOTRCNL2_ISO2RXLEN_Pos))
#define BB_ISOTRCNL2_ISO2TXLEN_Pos      0
#define BB_ISOTRCNL2_ISO2TXLEN_Mask     ((uint32_t)(0xFFU << BB_ISOTRCNL2_ISO2TXLEN_Pos))

/* BB_ISOTRCNL2 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t ISO2TXLEN_BYTE;       
         uint8_t RESERVED0[1];
    __IO uint8_t ISO2RXLEN_BYTE;       
         uint8_t RESERVED1[1];
} BB_ISOTRCNL2_Type;

#define BB_ISOTRCNL2_BASE               0x400016A0
#define BB_ISOTRCNL2                    ((BB_ISOTRCNL2_Type *) BB_ISOTRCNL2_BASE)

/* BB_ISOTRCNL2 settings */
#define ISO2TXLEN_0                     ((uint32_t)(0x0U << BB_ISOTRCNL2_ISO2TXLEN_Pos))

#define ISO2RXLEN_0                     ((uint32_t)(0x0U << BB_ISOTRCNL2_ISO2RXLEN_Pos))

/* BB_ISOTRCNL2 sub-register settings */
#define ISO2TXLEN_0_BYTE                ((uint8_t)0x0U)

#define ISO2RXLEN_0_BYTE                ((uint8_t)0x0U)

/* BB_ISOEVTCNTLOFFSETL2 bit positions */
#define BB_ISOEVTCNTLOFFSETL2_EVT_CNT_OFFSETL2_Pos 0
#define BB_ISOEVTCNTLOFFSETL2_EVT_CNT_OFFSETL2_Mask ((uint32_t)(0xFFFFFFFFU << BB_ISOEVTCNTLOFFSETL2_EVT_CNT_OFFSETL2_Pos))

/* BB_ISOEVTCNTLOFFSETL2 settings */
#define EVT_CNT_OFFSETL2_0              ((uint32_t)(0x0U << BB_ISOEVTCNTLOFFSETL2_EVT_CNT_OFFSETL2_Pos))

/* BB_ISOEVTCNTLOFFSETU2 bit positions */
#define BB_ISOEVTCNTLOFFSETU2_EVT_CNT_OFFSETU2_Pos 0
#define BB_ISOEVTCNTLOFFSETU2_EVT_CNT_OFFSETU2_Mask ((uint32_t)(0x7FU << BB_ISOEVTCNTLOFFSETU2_EVT_CNT_OFFSETU2_Pos))

/* BB_ISOEVTCNTLOFFSETU2 settings */
#define EVT_CNT_OFFSETU2_0              ((uint32_t)(0x0U << BB_ISOEVTCNTLOFFSETU2_EVT_CNT_OFFSETU2_Pos))

/* BB_BBPRIOSCHARB bit positions */
#define BB_BBPRIOSCHARB_BLEPRIOMODE_Pos 15
#define BB_BBPRIOSCHARB_BLEMARGIN_Pos   0
#define BB_BBPRIOSCHARB_BLEMARGIN_Mask  ((uint32_t)(0xFFU << BB_BBPRIOSCHARB_BLEMARGIN_Pos))

/* BB_BBPRIOSCHARB sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t BLEMARGIN_BYTE;       
    __IO uint8_t BLEPRIOMODE_BYTE;     
         uint8_t RESERVED0[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x400016B0, 0) - (0x400016B0 + 4)];
         uint32_t RESERVED1[15];
    __IO uint32_t BLEPRIOMODE_ALIAS;    /* Determine BLE priority scheduling arbitration mode */
} BB_BBPRIOSCHARB_Type;

#define BB_BBPRIOSCHARB_BASE            0x400016B0
#define BB_BBPRIOSCHARB                 ((BB_BBPRIOSCHARB_Type *) BB_BBPRIOSCHARB_BASE)

/* BB_BBPRIOSCHARB settings */
#define BLEMARGIN_0                     ((uint32_t)(0x0U << BB_BBPRIOSCHARB_BLEMARGIN_Pos))

#define BLEPRIOMODE_0_BITBAND           0x0
#define BLEPRIOMODE_1_BITBAND           0x1
#define BLEPRIOMODE_0                   ((uint32_t)(BLEPRIOMODE_0_BITBAND << BB_BBPRIOSCHARB_BLEPRIOMODE_Pos))
#define BLEPRIOMODE_1                   ((uint32_t)(BLEPRIOMODE_1_BITBAND << BB_BBPRIOSCHARB_BLEPRIOMODE_Pos))

/* BB_BBPRIOSCHARB sub-register bit positions */
#define BB_BBPRIOSCHARB_BLEPRIOMODE_BYTE_Pos 7

/* BB_BBPRIOSCHARB sub-register settings */
#define BLEMARGIN_0_BYTE                ((uint8_t)0x0U)

#define BLEPRIOMODE_0_BYTE              ((uint8_t)(BLEPRIOMODE_0_BITBAND << BB_BBPRIOSCHARB_BLEPRIOMODE_BYTE_Pos))
#define BLEPRIOMODE_1_BYTE              ((uint8_t)(BLEPRIOMODE_1_BITBAND << BB_BBPRIOSCHARB_BLEPRIOMODE_BYTE_Pos))

/* ----------------------------------------------------------------------------
 * RF Front-End 2.4 GHz
 * ------------------------------------------------------------------------- */

typedef struct
{
    __IO uint32_t REG00;                /* REG00 */
    __IO uint32_t REG01;                /* REG01 */
    __IO uint32_t REG02;                /* REG02 */
    __IO uint32_t REG03;                /* REG03 */
    __IO uint32_t REG04;                /* REG04 */
    __IO uint32_t REG05;                /* REG05 */
    __IO uint32_t CENTER_FREQ;          /* CENTER_FREQ */
    __IO uint32_t REG07;                /* REG07 */
    __IO uint32_t REG08;                /* REG08 */
    __IO uint32_t REG09;                /* REG09 */
    __IO uint32_t REG0A;                /* REG0A */
    __IO uint32_t SYNC_PATTERN;         /* SYNC_PATTERN */
    __IO uint32_t REG0C;                /* REG0C */
    __IO uint32_t CRC_POLYNOMIAL;       /* CRC_POLYNOMIAL */
    __IO uint32_t CRC_RST;              /* CRC_RST */
    __IO uint32_t REG0F;                /* REG0F */
    __IO uint32_t REG10;                /* REG10 */
    __IO uint32_t TX_PULSE0;            /* TX_PULSE0 */
    __IO uint32_t TX_PULSE1;            /* TX_PULSE1 */
    __IO uint32_t TX_PULSE2;            /* TX_PULSE2 */
    __IO uint32_t TX_PULSE3;            /* TX_PULSE3 */
    __IO uint32_t RX_PULSE;             /* RX_PULSE */
    __IO uint32_t REG16;                /* REG16 */
    __IO uint32_t REG17;                /* REG17 */
    __IO uint32_t REG18;                /* REG18 */
    __IO uint32_t REG19;                /* REG19 */
    __IO uint32_t REG1A;                /* REG1A */
    __IO uint32_t REG1B;                /* REG1B */
    __IO uint32_t AGC_LUT1;             /* AGC_LUT1 */
    __IO uint32_t AGC_LUT2;             /* AGC_LUT2 */
    __IO uint32_t AGC_LUT3;             /* AGC_LUT3 */
    __IO uint32_t AGC_LUT4;             /* AGC_LUT4 */
    __IO uint32_t REG20;                /* REG20 */
    __IO uint32_t AGC_ATT1;             /* AGC_ATT1 */
    __IO uint32_t REG22;                /* REG22 */
    __IO uint32_t REG23;                /* REG23 */
    __IO uint32_t REG24;                /* REG24 */
    __IO uint32_t REG25;                /* REG25 */
    __IO uint32_t REG26;                /* REG26 */
    __IO uint32_t REG27;                /* REG27 */
    __IO uint32_t REG28;                /* REG28 */
    __IO uint32_t PLL_CTRL;             /* PLL_CTRL */
    __IO uint32_t REG2A;                /* REG2A */
    __IO uint32_t XTAL_CTRL;            /* XTAL_CTRL */
    __IO uint32_t REG2C;                /* REG2C */
    __IO uint32_t REG2D;                /* REG2D */
    __IO uint32_t REG2E;                /* REG2E */
    __IO uint32_t REG2F;                /* REG2F */
    __IO uint32_t REG30;                /* REG30 */
    __I  uint32_t REG31;                /* REG31 */
    __I  uint32_t REG32;                /* REG32 */
    __O  uint32_t TXFIFO;               /* TXFIFO */
    __I  uint32_t RXFIFO;               /* RXFIFO */
    __I  uint32_t DESER_STATUS;         /* DESER_STATUS */
    __I  uint32_t IRQ_STATUS;           /* IRQ_STATUS */
    __I  uint32_t REG37;                /* REG37 */
    __I  uint32_t REG38;                /* REG38 */
    __I  uint32_t REG39;                /* REG39 */
         uint32_t RESERVED0[5];
    __I  uint32_t REVISION;             /* REVISION */
} RF_Type;

#define RF_BASE                         0x40010000
#define RF                              ((RF_Type *) RF_BASE)

/* RF_REG00 bit positions */
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_Pos 31
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_Pos 24
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_Mask ((uint32_t)(0x7FU << RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_Pos))
#define RF_REG00_FOURFSK_CODING_EN_FOURFSK_CODING_Pos 23
#define RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_Pos 20
#define RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_Mask ((uint32_t)(0x7U << RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_Pos))
#define RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_Pos 16
#define RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_Mask ((uint32_t)(0x7U << RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_Pos))
#define RF_REG00_MODE2_DIFF_CODING_Pos  14
#define RF_REG00_MODE2_PSK_NFSK_Pos     13
#define RF_REG00_MODE2_TESTMODE_Pos     8
#define RF_REG00_MODE2_TESTMODE_Mask    ((uint32_t)(0x1FU << RF_REG00_MODE2_TESTMODE_Pos))
#define RF_REG00_MODE_NOT_TO_IDLE_Pos   7
#define RF_REG00_MODE_EN_FSM_Pos        5
#define RF_REG00_MODE_EN_DESERIALIZER_Pos 4
#define RF_REG00_MODE_EN_SERIALIZER_Pos 3
#define RF_REG00_MODE_TX_NRX_Pos        2
#define RF_REG00_MODE_MODE_Pos          0
#define RF_REG00_MODE_MODE_Mask         ((uint32_t)(0x3U << RF_REG00_MODE_MODE_Pos))

/* RF_REG00 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t MODE2_BYTE;           
    __IO uint8_t FOURFSK_CODING_BYTE;  
    __IO uint8_t DATAWHITE_BTLE_DW_BYTE;
} RF_REG00_Type;

#define RF_REG00_BASE                   0x40010000
#define RF_REG00                        ((RF_REG00_Type *) RF_REG00_BASE)

/* RF_REG00 settings */
#define MODE_MODE_DEFAULT               ((uint32_t)(0x0U << RF_REG00_MODE_MODE_Pos))

#define MODE_TX_NRX_DEFAULT             ((uint32_t)(0x0U << RF_REG00_MODE_TX_NRX_Pos))

#define MODE_EN_SERIALIZER_DEFAULT      ((uint32_t)(0x0U << RF_REG00_MODE_EN_SERIALIZER_Pos))

#define MODE_EN_DESERIALIZER_DEFAULT    ((uint32_t)(0x0U << RF_REG00_MODE_EN_DESERIALIZER_Pos))

#define MODE_EN_FSM_DEFAULT             ((uint32_t)(0x0U << RF_REG00_MODE_EN_FSM_Pos))

#define MODE_NOT_TO_IDLE_DEFAULT        ((uint32_t)(0x0U << RF_REG00_MODE_NOT_TO_IDLE_Pos))

#define MODE2_TESTMODE_DEFAULT          ((uint32_t)(0x0U << RF_REG00_MODE2_TESTMODE_Pos))

#define MODE2_PSK_NFSK_DEFAULT          ((uint32_t)(0x0U << RF_REG00_MODE2_PSK_NFSK_Pos))

#define MODE2_DIFF_CODING_DEFAULT       ((uint32_t)(0x0U << RF_REG00_MODE2_DIFF_CODING_Pos))

#define FOURFSK_CODING_RX_FOURFSK_CODING_DEFAULT ((uint32_t)(0x0U << RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_Pos))

#define FOURFSK_CODING_TX_FOURFSK_CODING_DEFAULT ((uint32_t)(0x0U << RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_Pos))

#define FOURFSK_CODING_EN_FOURFSK_CODING_DEFAULT ((uint32_t)(0x0U << RF_REG00_FOURFSK_CODING_EN_FOURFSK_CODING_Pos))

#define DATAWHITE_BTLE_DW_BTLE_RST_DEFAULT ((uint32_t)(0x0U << RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_Pos))

#define DATAWHITE_BTLE_DW_BTLE_DEFAULT  ((uint32_t)(0x0U << RF_REG00_DATAWHITE_BTLE_DW_BTLE_Pos))

/* RF_REG00 sub-register bit positions */
#define RF_REG00_MODE2_TESTMODE_BYTE_Pos 0
#define RF_REG00_MODE2_TESTMODE_BYTE_Mask ((uint32_t)(0x1FU << RF_REG00_MODE2_TESTMODE_BYTE_Pos))
#define RF_REG00_MODE2_PSK_NFSK_BYTE_Pos 5
#define RF_REG00_MODE2_DIFF_CODING_BYTE_Pos 6
#define RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_BYTE_Pos 0
#define RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_BYTE_Mask ((uint32_t)(0x7U << RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_BYTE_Pos))
#define RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_BYTE_Pos 4
#define RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_BYTE_Mask ((uint32_t)(0x7U << RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_BYTE_Pos))
#define RF_REG00_FOURFSK_CODING_EN_FOURFSK_CODING_BYTE_Pos 7
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_BYTE_Pos 0
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_BYTE_Mask ((uint32_t)(0x7FU << RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_BYTE_Pos))
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_BYTE_Pos 7

/* RF_REG00 sub-register settings */
#define MODE2_TESTMODE_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG00_MODE2_TESTMODE_BYTE_Pos))

#define MODE2_PSK_NFSK_DEFAULT_BYTE     ((uint8_t)(MODE2_PSK_NFSK_DEFAULT_BITBAND << RF_REG00_MODE2_PSK_NFSK_BYTE_Pos))

#define MODE2_DIFF_CODING_DEFAULT_BYTE  ((uint8_t)(MODE2_DIFF_CODING_DEFAULT_BITBAND << RF_REG00_MODE2_DIFF_CODING_BYTE_Pos))

#define FOURFSK_CODING_RX_FOURFSK_CODING_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_BYTE_Pos))

#define FOURFSK_CODING_TX_FOURFSK_CODING_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_BYTE_Pos))

#define FOURFSK_CODING_EN_FOURFSK_CODING_DEFAULT_BYTE ((uint8_t)(FOURFSK_CODING_EN_FOURFSK_CODING_DEFAULT_BITBAND << RF_REG00_FOURFSK_CODING_EN_FOURFSK_CODING_BYTE_Pos))

#define DATAWHITE_BTLE_DW_BTLE_RST_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_BYTE_Pos))

#define DATAWHITE_BTLE_DW_BTLE_DEFAULT_BYTE ((uint8_t)(DATAWHITE_BTLE_DW_BTLE_DEFAULT_BITBAND << RF_REG00_DATAWHITE_BTLE_DW_BTLE_BYTE_Pos))

/* RF_REG01 bit positions */
#define RF_REG01_TAU_PHASE_RECOV_TAU_PHASE_RECOV_Pos 24
#define RF_REG01_TAU_PHASE_RECOV_TAU_PHASE_RECOV_Mask ((uint32_t)(0xFFU << RF_REG01_TAU_PHASE_RECOV_TAU_PHASE_RECOV_Pos))
#define RF_REG01_TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_Pos 16
#define RF_REG01_TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_Mask ((uint32_t)(0xFFU << RF_REG01_TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_Pos))
#define RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_Pos 15
#define RF_REG01_CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_Pos 14
#define RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_Pos 13
#define RF_REG01_CARRIER_RECOVERY_AFC_NEG_Pos 12
#define RF_REG01_CARRIER_RECOVERY_STARTER_MODE_Pos 11
#define RF_REG01_CARRIER_RECOVERY_EN_AFC_Pos 10
#define RF_REG01_CARRIER_RECOVERY_EN_FINE_RECOV_Pos 9
#define RF_REG01_CARRIER_RECOVERY_EN_ROUGH_RECOV_Pos 8
#define RF_REG01_MOD_TX_PULSE_NSYM_Pos  6
#define RF_REG01_MOD_TX_EN_INTERP_Pos   5
#define RF_REG01_MOD_TX_CK_TX_M_Pos     0
#define RF_REG01_MOD_TX_CK_TX_M_Mask    ((uint32_t)(0x1FU << RF_REG01_MOD_TX_CK_TX_M_Pos))

/* RF_REG01 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t MOD_TX_BYTE;          
    __IO uint8_t CARRIER_RECOVERY_BYTE;
    __IO uint8_t TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_BYTE;
    __IO uint8_t TAU_PHASE_RECOV_TAU_PHASE_RECOV_BYTE;
} RF_REG01_Type;

#define RF_REG01_BASE                   0x40010004
#define RF_REG01                        ((RF_REG01_Type *) RF_REG01_BASE)

/* RF_REG01 settings */
#define MOD_TX_CK_TX_M_DEFAULT          ((uint32_t)(0x0U << RF_REG01_MOD_TX_CK_TX_M_Pos))

#define MOD_TX_EN_INTERP_DEFAULT        ((uint32_t)(0x0U << RF_REG01_MOD_TX_EN_INTERP_Pos))

#define MOD_TX_PULSE_NSYM_DEFAULT       ((uint32_t)(0x0U << RF_REG01_MOD_TX_PULSE_NSYM_Pos))

#define CARRIER_RECOVERY_EN_ROUGH_RECOV_DEFAULT ((uint32_t)(0x0U << RF_REG01_CARRIER_RECOVERY_EN_ROUGH_RECOV_Pos))

#define CARRIER_RECOVERY_EN_FINE_RECOV_DEFAULT ((uint32_t)(0x0U << RF_REG01_CARRIER_RECOVERY_EN_FINE_RECOV_Pos))

#define CARRIER_RECOVERY_EN_AFC_DEFAULT ((uint32_t)(0x0U << RF_REG01_CARRIER_RECOVERY_EN_AFC_Pos))

#define CARRIER_RECOVERY_STARTER_MODE_DEFAULT ((uint32_t)(0x0U << RF_REG01_CARRIER_RECOVERY_STARTER_MODE_Pos))

#define CARRIER_RECOVERY_AFC_NEG_DEFAULT ((uint32_t)(0x0U << RF_REG01_CARRIER_RECOVERY_AFC_NEG_Pos))

#define CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_DEFAULT ((uint32_t)(0x0U << RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_Pos))

#define CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_DEFAULT ((uint32_t)(0x0U << RF_REG01_CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_Pos))

#define CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_DEFAULT ((uint32_t)(0x0U << RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_Pos))

#define TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_DEFAULT ((uint32_t)(0x0U << RF_REG01_TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_Pos))

#define TAU_PHASE_RECOV_TAU_PHASE_RECOV_DEFAULT ((uint32_t)(0x0U << RF_REG01_TAU_PHASE_RECOV_TAU_PHASE_RECOV_Pos))

/* RF_REG01 sub-register bit positions */
#define RF_REG01_MOD_TX_CK_TX_M_BYTE_Pos 0
#define RF_REG01_MOD_TX_CK_TX_M_BYTE_Mask ((uint32_t)(0x1FU << RF_REG01_MOD_TX_CK_TX_M_BYTE_Pos))
#define RF_REG01_MOD_TX_EN_INTERP_BYTE_Pos 5
#define RF_REG01_MOD_TX_PULSE_NSYM_BYTE_Pos 6
#define RF_REG01_CARRIER_RECOVERY_EN_ROUGH_RECOV_BYTE_Pos 0
#define RF_REG01_CARRIER_RECOVERY_EN_FINE_RECOV_BYTE_Pos 1
#define RF_REG01_CARRIER_RECOVERY_EN_AFC_BYTE_Pos 2
#define RF_REG01_CARRIER_RECOVERY_STARTER_MODE_BYTE_Pos 3
#define RF_REG01_CARRIER_RECOVERY_AFC_NEG_BYTE_Pos 4
#define RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_BYTE_Pos 5
#define RF_REG01_CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_BYTE_Pos 6
#define RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_BYTE_Pos 7

/* RF_REG01 sub-register settings */
#define MOD_TX_CK_TX_M_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG01_MOD_TX_CK_TX_M_BYTE_Pos))

#define MOD_TX_EN_INTERP_DEFAULT_BYTE   ((uint8_t)(MOD_TX_EN_INTERP_DEFAULT_BITBAND << RF_REG01_MOD_TX_EN_INTERP_BYTE_Pos))

#define MOD_TX_PULSE_NSYM_DEFAULT_BYTE  ((uint8_t)(MOD_TX_PULSE_NSYM_DEFAULT_BITBAND << RF_REG01_MOD_TX_PULSE_NSYM_BYTE_Pos))

#define CARRIER_RECOVERY_EN_ROUGH_RECOV_DEFAULT_BYTE ((uint8_t)(CARRIER_RECOVERY_EN_ROUGH_RECOV_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_ROUGH_RECOV_BYTE_Pos))

#define CARRIER_RECOVERY_EN_FINE_RECOV_DEFAULT_BYTE ((uint8_t)(CARRIER_RECOVERY_EN_FINE_RECOV_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_FINE_RECOV_BYTE_Pos))

#define CARRIER_RECOVERY_EN_AFC_DEFAULT_BYTE ((uint8_t)(CARRIER_RECOVERY_EN_AFC_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_AFC_BYTE_Pos))

#define CARRIER_RECOVERY_STARTER_MODE_DEFAULT_BYTE ((uint8_t)(CARRIER_RECOVERY_STARTER_MODE_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_STARTER_MODE_BYTE_Pos))

#define CARRIER_RECOVERY_AFC_NEG_DEFAULT_BYTE ((uint8_t)(CARRIER_RECOVERY_AFC_NEG_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_AFC_NEG_BYTE_Pos))

#define CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_DEFAULT_BYTE ((uint8_t)(CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_BYTE_Pos))

#define CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_DEFAULT_BYTE ((uint8_t)(CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_BYTE_Pos))

#define CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_DEFAULT_BYTE ((uint8_t)(CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_BYTE_Pos))

#define TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_DEFAULT_BYTE ((uint8_t)0x0U)

#define TAU_PHASE_RECOV_TAU_PHASE_RECOV_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_REG02 bit positions */
#define RF_REG02_FIFO_FIFO_FLUSH_ON_OVFLW_Pos 31
#define RF_REG02_FIFO_FIFO_FLUSH_ON_ADDR_ERR_Pos 30
#define RF_REG02_FIFO_FIFO_FLUSH_ON_PL_ERR_Pos 29
#define RF_REG02_FIFO_FIFO_FLUSH_ON_CRC_ERR_Pos 28
#define RF_REG02_FIFO_RX_FIFO_ACK_Pos   27
#define RF_REG02_FIFO_FIFO_THR_Pos      24
#define RF_REG02_FIFO_FIFO_THR_Mask     ((uint32_t)(0x7U << RF_REG02_FIFO_FIFO_THR_Pos))
#define RF_REG02_DATARATE_OFFSET_DATARATE_OFFSET_Pos 16
#define RF_REG02_DATARATE_OFFSET_DATARATE_OFFSET_Mask ((uint32_t)(0xFFU << RF_REG02_DATARATE_OFFSET_DATARATE_OFFSET_Pos))
#define RF_REG02_TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_Pos 8
#define RF_REG02_TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_Mask ((uint32_t)(0xFFU << RF_REG02_TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_Pos))
#define RF_REG02_TAU_CLK_RECOV_TAU_CLK_RECOV_Pos 0
#define RF_REG02_TAU_CLK_RECOV_TAU_CLK_RECOV_Mask ((uint32_t)(0xFFU << RF_REG02_TAU_CLK_RECOV_TAU_CLK_RECOV_Pos))

/* RF_REG02 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TAU_CLK_RECOV_TAU_CLK_RECOV_BYTE;
    __IO uint8_t TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_BYTE;
    __IO uint8_t DATARATE_OFFSET_DATARATE_OFFSET_BYTE;
    __IO uint8_t FIFO_BYTE;            
} RF_REG02_Type;

#define RF_REG02_BASE                   0x40010008
#define RF_REG02                        ((RF_REG02_Type *) RF_REG02_BASE)

/* RF_REG02 settings */
#define TAU_CLK_RECOV_TAU_CLK_RECOV_DEFAULT ((uint32_t)(0x0U << RF_REG02_TAU_CLK_RECOV_TAU_CLK_RECOV_Pos))

#define TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_DEFAULT ((uint32_t)(0x0U << RF_REG02_TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_Pos))

#define DATARATE_OFFSET_DATARATE_OFFSET_DEFAULT ((uint32_t)(0x0U << RF_REG02_DATARATE_OFFSET_DATARATE_OFFSET_Pos))

#define FIFO_FIFO_THR_DEFAULT           ((uint32_t)(0x0U << RF_REG02_FIFO_FIFO_THR_Pos))

#define FIFO_RX_FIFO_ACK_DEFAULT        ((uint32_t)(0x0U << RF_REG02_FIFO_RX_FIFO_ACK_Pos))

#define FIFO_FIFO_FLUSH_ON_CRC_ERR_DEFAULT ((uint32_t)(0x0U << RF_REG02_FIFO_FIFO_FLUSH_ON_CRC_ERR_Pos))

#define FIFO_FIFO_FLUSH_ON_PL_ERR_DEFAULT ((uint32_t)(0x0U << RF_REG02_FIFO_FIFO_FLUSH_ON_PL_ERR_Pos))

#define FIFO_FIFO_FLUSH_ON_ADDR_ERR_DEFAULT ((uint32_t)(0x0U << RF_REG02_FIFO_FIFO_FLUSH_ON_ADDR_ERR_Pos))

#define FIFO_FIFO_FLUSH_ON_OVFLW_DEFAULT ((uint32_t)(0x0U << RF_REG02_FIFO_FIFO_FLUSH_ON_OVFLW_Pos))

/* RF_REG02 sub-register bit positions */
#define RF_REG02_FIFO_FIFO_THR_BYTE_Pos 0
#define RF_REG02_FIFO_FIFO_THR_BYTE_Mask ((uint32_t)(0x7U << RF_REG02_FIFO_FIFO_THR_BYTE_Pos))
#define RF_REG02_FIFO_RX_FIFO_ACK_BYTE_Pos 3
#define RF_REG02_FIFO_FIFO_FLUSH_ON_CRC_ERR_BYTE_Pos 4
#define RF_REG02_FIFO_FIFO_FLUSH_ON_PL_ERR_BYTE_Pos 5
#define RF_REG02_FIFO_FIFO_FLUSH_ON_ADDR_ERR_BYTE_Pos 6
#define RF_REG02_FIFO_FIFO_FLUSH_ON_OVFLW_BYTE_Pos 7

/* RF_REG02 sub-register settings */
#define TAU_CLK_RECOV_TAU_CLK_RECOV_DEFAULT_BYTE ((uint8_t)0x0U)

#define TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_DEFAULT_BYTE ((uint8_t)0x0U)

#define DATARATE_OFFSET_DATARATE_OFFSET_DEFAULT_BYTE ((uint8_t)0x0U)

#define FIFO_FIFO_THR_DEFAULT_BYTE      ((uint8_t)(0x0U << RF_REG02_FIFO_FIFO_THR_BYTE_Pos))

#define FIFO_RX_FIFO_ACK_DEFAULT_BYTE   ((uint8_t)(FIFO_RX_FIFO_ACK_DEFAULT_BITBAND << RF_REG02_FIFO_RX_FIFO_ACK_BYTE_Pos))

#define FIFO_FIFO_FLUSH_ON_CRC_ERR_DEFAULT_BYTE ((uint8_t)(FIFO_FIFO_FLUSH_ON_CRC_ERR_DEFAULT_BITBAND << RF_REG02_FIFO_FIFO_FLUSH_ON_CRC_ERR_BYTE_Pos))

#define FIFO_FIFO_FLUSH_ON_PL_ERR_DEFAULT_BYTE ((uint8_t)(FIFO_FIFO_FLUSH_ON_PL_ERR_DEFAULT_BITBAND << RF_REG02_FIFO_FIFO_FLUSH_ON_PL_ERR_BYTE_Pos))

#define FIFO_FIFO_FLUSH_ON_ADDR_ERR_DEFAULT_BYTE ((uint8_t)(FIFO_FIFO_FLUSH_ON_ADDR_ERR_DEFAULT_BITBAND << RF_REG02_FIFO_FIFO_FLUSH_ON_ADDR_ERR_BYTE_Pos))

#define FIFO_FIFO_FLUSH_ON_OVFLW_DEFAULT_BYTE ((uint8_t)(FIFO_FIFO_FLUSH_ON_OVFLW_DEFAULT_BITBAND << RF_REG02_FIFO_FIFO_FLUSH_ON_OVFLW_BYTE_Pos))

/* RF_REG03 bit positions */
#define RF_REG03_PAD_CONF_2_PAD_3_CONF_Pos 28
#define RF_REG03_PAD_CONF_2_PAD_3_CONF_Mask ((uint32_t)(0xFU << RF_REG03_PAD_CONF_2_PAD_3_CONF_Pos))
#define RF_REG03_PAD_CONF_2_PAD_2_CONF_Pos 24
#define RF_REG03_PAD_CONF_2_PAD_2_CONF_Mask ((uint32_t)(0xFU << RF_REG03_PAD_CONF_2_PAD_2_CONF_Pos))
#define RF_REG03_PAD_CONF_1_PAD_1_CONF_Pos 20
#define RF_REG03_PAD_CONF_1_PAD_1_CONF_Mask ((uint32_t)(0xFU << RF_REG03_PAD_CONF_1_PAD_1_CONF_Pos))
#define RF_REG03_PAD_CONF_1_PAD_0_CONF_Pos 16
#define RF_REG03_PAD_CONF_1_PAD_0_CONF_Mask ((uint32_t)(0xFU << RF_REG03_PAD_CONF_1_PAD_0_CONF_Pos))
#define RF_REG03_IRQ_CONF_IRQ_HIGH_Z_Pos 15
#define RF_REG03_IRQ_CONF_IRQ_ACTIVE_LOW_Pos 14
#define RF_REG03_IRQ_CONF_IRQS_MASK_Pos 8
#define RF_REG03_IRQ_CONF_IRQS_MASK_Mask ((uint32_t)(0x3FU << RF_REG03_IRQ_CONF_IRQS_MASK_Pos))
#define RF_REG03_FIFO_2_FIFO_THR_TX_Pos 5
#define RF_REG03_FIFO_2_FIFO_THR_TX_Mask ((uint32_t)(0x7U << RF_REG03_FIFO_2_FIFO_THR_TX_Pos))
#define RF_REG03_FIFO_2_WAIT_TXFIFO_WR_Pos 4
#define RF_REG03_FIFO_2_STOP_ON_RXFF_OVFLW_Pos 3
#define RF_REG03_FIFO_2_STOP_ON_TXFF_UNFLW_Pos 2
#define RF_REG03_FIFO_2_RXFF_FLUSH_ON_START_Pos 1
#define RF_REG03_FIFO_2_TXFF_FLUSH_ON_STOP_Pos 0

/* RF_REG03 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t FIFO_2_BYTE;          
    __IO uint8_t IRQ_CONF_IRQ_BYTE;    
    __IO uint8_t PAD_CONF_1_PAD_BYTE;  
    __IO uint8_t PAD_CONF_2_PAD_BYTE;  
} RF_REG03_Type;

#define RF_REG03_BASE                   0x4001000C
#define RF_REG03                        ((RF_REG03_Type *) RF_REG03_BASE)

/* RF_REG03 settings */
#define FIFO_2_TXFF_FLUSH_ON_STOP_DEFAULT ((uint32_t)(0x0U << RF_REG03_FIFO_2_TXFF_FLUSH_ON_STOP_Pos))

#define FIFO_2_RXFF_FLUSH_ON_START_DEFAULT ((uint32_t)(0x0U << RF_REG03_FIFO_2_RXFF_FLUSH_ON_START_Pos))

#define FIFO_2_STOP_ON_TXFF_UNFLW_DEFAULT ((uint32_t)(0x0U << RF_REG03_FIFO_2_STOP_ON_TXFF_UNFLW_Pos))

#define FIFO_2_STOP_ON_RXFF_OVFLW_DEFAULT ((uint32_t)(0x0U << RF_REG03_FIFO_2_STOP_ON_RXFF_OVFLW_Pos))

#define FIFO_2_WAIT_TXFIFO_WR_DEFAULT   ((uint32_t)(0x0U << RF_REG03_FIFO_2_WAIT_TXFIFO_WR_Pos))

#define FIFO_2_FIFO_THR_TX_DEFAULT      ((uint32_t)(0x0U << RF_REG03_FIFO_2_FIFO_THR_TX_Pos))

#define IRQ_CONF_IRQS_MASK_DEFAULT      ((uint32_t)(0x0U << RF_REG03_IRQ_CONF_IRQS_MASK_Pos))

#define IRQ_CONF_IRQ_ACTIVE_LOW_DEFAULT ((uint32_t)(0x0U << RF_REG03_IRQ_CONF_IRQ_ACTIVE_LOW_Pos))

#define IRQ_CONF_IRQ_HIGH_Z_DEFAULT     ((uint32_t)(0x0U << RF_REG03_IRQ_CONF_IRQ_HIGH_Z_Pos))

#define PAD_CONF_1_PAD_0_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG03_PAD_CONF_1_PAD_0_CONF_Pos))

#define PAD_CONF_1_PAD_1_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG03_PAD_CONF_1_PAD_1_CONF_Pos))

#define PAD_CONF_2_PAD_2_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG03_PAD_CONF_2_PAD_2_CONF_Pos))

#define PAD_CONF_2_PAD_3_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG03_PAD_CONF_2_PAD_3_CONF_Pos))

/* RF_REG03 sub-register bit positions */
#define RF_REG03_FIFO_2_TXFF_FLUSH_ON_STOP_BYTE_Pos 0
#define RF_REG03_FIFO_2_RXFF_FLUSH_ON_START_BYTE_Pos 1
#define RF_REG03_FIFO_2_STOP_ON_TXFF_UNFLW_BYTE_Pos 2
#define RF_REG03_FIFO_2_STOP_ON_RXFF_OVFLW_BYTE_Pos 3
#define RF_REG03_FIFO_2_WAIT_TXFIFO_WR_BYTE_Pos 4
#define RF_REG03_FIFO_2_FIFO_THR_TX_BYTE_Pos 5
#define RF_REG03_FIFO_2_FIFO_THR_TX_BYTE_Mask ((uint32_t)(0x7U << RF_REG03_FIFO_2_FIFO_THR_TX_BYTE_Pos))
#define RF_REG03_IRQ_CONF_IRQS_MASK_BYTE_Pos 0
#define RF_REG03_IRQ_CONF_IRQS_MASK_BYTE_Mask ((uint32_t)(0x3FU << RF_REG03_IRQ_CONF_IRQS_MASK_BYTE_Pos))
#define RF_REG03_IRQ_CONF_IRQ_ACTIVE_LOW_BYTE_Pos 6
#define RF_REG03_IRQ_CONF_IRQ_HIGH_Z_BYTE_Pos 7
#define RF_REG03_PAD_CONF_1_PAD_0_CONF_BYTE_Pos 0
#define RF_REG03_PAD_CONF_1_PAD_0_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG03_PAD_CONF_1_PAD_0_CONF_BYTE_Pos))
#define RF_REG03_PAD_CONF_1_PAD_1_CONF_BYTE_Pos 4
#define RF_REG03_PAD_CONF_1_PAD_1_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG03_PAD_CONF_1_PAD_1_CONF_BYTE_Pos))
#define RF_REG03_PAD_CONF_2_PAD_2_CONF_BYTE_Pos 0
#define RF_REG03_PAD_CONF_2_PAD_2_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG03_PAD_CONF_2_PAD_2_CONF_BYTE_Pos))
#define RF_REG03_PAD_CONF_2_PAD_3_CONF_BYTE_Pos 4
#define RF_REG03_PAD_CONF_2_PAD_3_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG03_PAD_CONF_2_PAD_3_CONF_BYTE_Pos))

/* RF_REG03 sub-register settings */
#define FIFO_2_TXFF_FLUSH_ON_STOP_DEFAULT_BYTE ((uint8_t)(FIFO_2_TXFF_FLUSH_ON_STOP_DEFAULT_BITBAND << RF_REG03_FIFO_2_TXFF_FLUSH_ON_STOP_BYTE_Pos))

#define FIFO_2_RXFF_FLUSH_ON_START_DEFAULT_BYTE ((uint8_t)(FIFO_2_RXFF_FLUSH_ON_START_DEFAULT_BITBAND << RF_REG03_FIFO_2_RXFF_FLUSH_ON_START_BYTE_Pos))

#define FIFO_2_STOP_ON_TXFF_UNFLW_DEFAULT_BYTE ((uint8_t)(FIFO_2_STOP_ON_TXFF_UNFLW_DEFAULT_BITBAND << RF_REG03_FIFO_2_STOP_ON_TXFF_UNFLW_BYTE_Pos))

#define FIFO_2_STOP_ON_RXFF_OVFLW_DEFAULT_BYTE ((uint8_t)(FIFO_2_STOP_ON_RXFF_OVFLW_DEFAULT_BITBAND << RF_REG03_FIFO_2_STOP_ON_RXFF_OVFLW_BYTE_Pos))

#define FIFO_2_WAIT_TXFIFO_WR_DEFAULT_BYTE ((uint8_t)(FIFO_2_WAIT_TXFIFO_WR_DEFAULT_BITBAND << RF_REG03_FIFO_2_WAIT_TXFIFO_WR_BYTE_Pos))

#define FIFO_2_FIFO_THR_TX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG03_FIFO_2_FIFO_THR_TX_BYTE_Pos))

#define IRQ_CONF_IRQS_MASK_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG03_IRQ_CONF_IRQS_MASK_BYTE_Pos))

#define IRQ_CONF_IRQ_ACTIVE_LOW_DEFAULT_BYTE ((uint8_t)(IRQ_CONF_IRQ_ACTIVE_LOW_DEFAULT_BITBAND << RF_REG03_IRQ_CONF_IRQ_ACTIVE_LOW_BYTE_Pos))

#define IRQ_CONF_IRQ_HIGH_Z_DEFAULT_BYTE ((uint8_t)(IRQ_CONF_IRQ_HIGH_Z_DEFAULT_BITBAND << RF_REG03_IRQ_CONF_IRQ_HIGH_Z_BYTE_Pos))

#define PAD_CONF_1_PAD_0_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG03_PAD_CONF_1_PAD_0_CONF_BYTE_Pos))

#define PAD_CONF_1_PAD_1_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG03_PAD_CONF_1_PAD_1_CONF_BYTE_Pos))

#define PAD_CONF_2_PAD_2_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG03_PAD_CONF_2_PAD_2_CONF_BYTE_Pos))

#define PAD_CONF_2_PAD_3_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG03_PAD_CONF_2_PAD_3_CONF_BYTE_Pos))

/* RF_REG04 bit positions */
#define RF_REG04_MAC_CONF_MAC_TIMER_GR_Pos 30
#define RF_REG04_MAC_CONF_MAC_TIMER_GR_Mask ((uint32_t)(0x3U << RF_REG04_MAC_CONF_MAC_TIMER_GR_Pos))
#define RF_REG04_MAC_CONF_RX_MAC_ACT_Pos 29
#define RF_REG04_MAC_CONF_RX_MAC_TX_NRX_Pos 28
#define RF_REG04_MAC_CONF_RX_MAC_START_NSTOP_Pos 27
#define RF_REG04_MAC_CONF_TX_MAC_ACT_Pos 26
#define RF_REG04_MAC_CONF_TX_MAC_TX_NRX_Pos 25
#define RF_REG04_MAC_CONF_TX_MAC_START_NSTOP_Pos 24
#define RF_REG04_PAD_CONF_5_PAD_9_CONF_Pos 20
#define RF_REG04_PAD_CONF_5_PAD_9_CONF_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_5_PAD_9_CONF_Pos))
#define RF_REG04_PAD_CONF_5_PAD_8_CONF_Pos 16
#define RF_REG04_PAD_CONF_5_PAD_8_CONF_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_5_PAD_8_CONF_Pos))
#define RF_REG04_PAD_CONF_4_PAD_7_CONF_Pos 12
#define RF_REG04_PAD_CONF_4_PAD_7_CONF_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_4_PAD_7_CONF_Pos))
#define RF_REG04_PAD_CONF_4_PAD_6_CONF_Pos 8
#define RF_REG04_PAD_CONF_4_PAD_6_CONF_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_4_PAD_6_CONF_Pos))
#define RF_REG04_PAD_CONF_3_PAD_5_CONF_Pos 4
#define RF_REG04_PAD_CONF_3_PAD_5_CONF_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_3_PAD_5_CONF_Pos))
#define RF_REG04_PAD_CONF_3_PAD_4_CONF_Pos 0
#define RF_REG04_PAD_CONF_3_PAD_4_CONF_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_3_PAD_4_CONF_Pos))

/* RF_REG04 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t PAD_CONF_3_PAD_BYTE;  
    __IO uint8_t PAD_CONF_4_PAD_BYTE;  
    __IO uint8_t PAD_CONF_5_PAD_BYTE;  
    __IO uint8_t MAC_CONF_BYTE;        
} RF_REG04_Type;

#define RF_REG04_BASE                   0x40010010
#define RF_REG04                        ((RF_REG04_Type *) RF_REG04_BASE)

/* RF_REG04 settings */
#define PAD_CONF_3_PAD_4_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG04_PAD_CONF_3_PAD_4_CONF_Pos))

#define PAD_CONF_3_PAD_5_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG04_PAD_CONF_3_PAD_5_CONF_Pos))

#define PAD_CONF_4_PAD_6_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG04_PAD_CONF_4_PAD_6_CONF_Pos))

#define PAD_CONF_4_PAD_7_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG04_PAD_CONF_4_PAD_7_CONF_Pos))

#define PAD_CONF_5_PAD_8_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG04_PAD_CONF_5_PAD_8_CONF_Pos))

#define PAD_CONF_5_PAD_9_CONF_DEFAULT   ((uint32_t)(0x0U << RF_REG04_PAD_CONF_5_PAD_9_CONF_Pos))

#define MAC_CONF_TX_MAC_START_NSTOP_DEFAULT ((uint32_t)(0x0U << RF_REG04_MAC_CONF_TX_MAC_START_NSTOP_Pos))

#define MAC_CONF_TX_MAC_TX_NRX_DEFAULT  ((uint32_t)(0x0U << RF_REG04_MAC_CONF_TX_MAC_TX_NRX_Pos))

#define MAC_CONF_TX_MAC_ACT_DEFAULT     ((uint32_t)(0x0U << RF_REG04_MAC_CONF_TX_MAC_ACT_Pos))

#define MAC_CONF_RX_MAC_START_NSTOP_DEFAULT ((uint32_t)(0x0U << RF_REG04_MAC_CONF_RX_MAC_START_NSTOP_Pos))

#define MAC_CONF_RX_MAC_TX_NRX_DEFAULT  ((uint32_t)(0x0U << RF_REG04_MAC_CONF_RX_MAC_TX_NRX_Pos))

#define MAC_CONF_RX_MAC_ACT_DEFAULT     ((uint32_t)(0x0U << RF_REG04_MAC_CONF_RX_MAC_ACT_Pos))

#define MAC_CONF_MAC_TIMER_GR_DEFAULT   ((uint32_t)(0x0U << RF_REG04_MAC_CONF_MAC_TIMER_GR_Pos))

/* RF_REG04 sub-register bit positions */
#define RF_REG04_PAD_CONF_3_PAD_4_CONF_BYTE_Pos 0
#define RF_REG04_PAD_CONF_3_PAD_4_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_3_PAD_4_CONF_BYTE_Pos))
#define RF_REG04_PAD_CONF_3_PAD_5_CONF_BYTE_Pos 4
#define RF_REG04_PAD_CONF_3_PAD_5_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_3_PAD_5_CONF_BYTE_Pos))
#define RF_REG04_PAD_CONF_4_PAD_6_CONF_BYTE_Pos 0
#define RF_REG04_PAD_CONF_4_PAD_6_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_4_PAD_6_CONF_BYTE_Pos))
#define RF_REG04_PAD_CONF_4_PAD_7_CONF_BYTE_Pos 4
#define RF_REG04_PAD_CONF_4_PAD_7_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_4_PAD_7_CONF_BYTE_Pos))
#define RF_REG04_PAD_CONF_5_PAD_8_CONF_BYTE_Pos 0
#define RF_REG04_PAD_CONF_5_PAD_8_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_5_PAD_8_CONF_BYTE_Pos))
#define RF_REG04_PAD_CONF_5_PAD_9_CONF_BYTE_Pos 4
#define RF_REG04_PAD_CONF_5_PAD_9_CONF_BYTE_Mask ((uint32_t)(0xFU << RF_REG04_PAD_CONF_5_PAD_9_CONF_BYTE_Pos))
#define RF_REG04_MAC_CONF_TX_MAC_START_NSTOP_BYTE_Pos 0
#define RF_REG04_MAC_CONF_TX_MAC_TX_NRX_BYTE_Pos 1
#define RF_REG04_MAC_CONF_TX_MAC_ACT_BYTE_Pos 2
#define RF_REG04_MAC_CONF_RX_MAC_START_NSTOP_BYTE_Pos 3
#define RF_REG04_MAC_CONF_RX_MAC_TX_NRX_BYTE_Pos 4
#define RF_REG04_MAC_CONF_RX_MAC_ACT_BYTE_Pos 5
#define RF_REG04_MAC_CONF_MAC_TIMER_GR_BYTE_Pos 6
#define RF_REG04_MAC_CONF_MAC_TIMER_GR_BYTE_Mask ((uint32_t)(0x3U << RF_REG04_MAC_CONF_MAC_TIMER_GR_BYTE_Pos))

/* RF_REG04 sub-register settings */
#define PAD_CONF_3_PAD_4_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG04_PAD_CONF_3_PAD_4_CONF_BYTE_Pos))

#define PAD_CONF_3_PAD_5_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG04_PAD_CONF_3_PAD_5_CONF_BYTE_Pos))

#define PAD_CONF_4_PAD_6_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG04_PAD_CONF_4_PAD_6_CONF_BYTE_Pos))

#define PAD_CONF_4_PAD_7_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG04_PAD_CONF_4_PAD_7_CONF_BYTE_Pos))

#define PAD_CONF_5_PAD_8_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG04_PAD_CONF_5_PAD_8_CONF_BYTE_Pos))

#define PAD_CONF_5_PAD_9_CONF_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG04_PAD_CONF_5_PAD_9_CONF_BYTE_Pos))

#define MAC_CONF_TX_MAC_START_NSTOP_DEFAULT_BYTE ((uint8_t)(MAC_CONF_TX_MAC_START_NSTOP_DEFAULT_BITBAND << RF_REG04_MAC_CONF_TX_MAC_START_NSTOP_BYTE_Pos))

#define MAC_CONF_TX_MAC_TX_NRX_DEFAULT_BYTE ((uint8_t)(MAC_CONF_TX_MAC_TX_NRX_DEFAULT_BITBAND << RF_REG04_MAC_CONF_TX_MAC_TX_NRX_BYTE_Pos))

#define MAC_CONF_TX_MAC_ACT_DEFAULT_BYTE ((uint8_t)(MAC_CONF_TX_MAC_ACT_DEFAULT_BITBAND << RF_REG04_MAC_CONF_TX_MAC_ACT_BYTE_Pos))

#define MAC_CONF_RX_MAC_START_NSTOP_DEFAULT_BYTE ((uint8_t)(MAC_CONF_RX_MAC_START_NSTOP_DEFAULT_BITBAND << RF_REG04_MAC_CONF_RX_MAC_START_NSTOP_BYTE_Pos))

#define MAC_CONF_RX_MAC_TX_NRX_DEFAULT_BYTE ((uint8_t)(MAC_CONF_RX_MAC_TX_NRX_DEFAULT_BITBAND << RF_REG04_MAC_CONF_RX_MAC_TX_NRX_BYTE_Pos))

#define MAC_CONF_RX_MAC_ACT_DEFAULT_BYTE ((uint8_t)(MAC_CONF_RX_MAC_ACT_DEFAULT_BITBAND << RF_REG04_MAC_CONF_RX_MAC_ACT_BYTE_Pos))

#define MAC_CONF_MAC_TIMER_GR_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG04_MAC_CONF_MAC_TIMER_GR_BYTE_Pos))

/* RF_REG05 bit positions */
#define RF_REG05_CHANNEL_SWITCH_IQ_Pos  30
#define RF_REG05_CHANNEL_CHANNEL_Pos    24
#define RF_REG05_CHANNEL_CHANNEL_Mask   ((uint32_t)(0x3FU << RF_REG05_CHANNEL_CHANNEL_Pos))
#define RF_REG05_BANK_DATARATE_TX_NRX_Pos 18
#define RF_REG05_BANK_BANK_Pos          16
#define RF_REG05_BANK_BANK_Mask         ((uint32_t)(0x3U << RF_REG05_BANK_BANK_Pos))
#define RF_REG05_TX_MAC_TIMER_TX_MAC_TIMER_Pos 8
#define RF_REG05_TX_MAC_TIMER_TX_MAC_TIMER_Mask ((uint32_t)(0xFFU << RF_REG05_TX_MAC_TIMER_TX_MAC_TIMER_Pos))
#define RF_REG05_RX_MAC_TIMER_RX_MAC_TIMER_Pos 0
#define RF_REG05_RX_MAC_TIMER_RX_MAC_TIMER_Mask ((uint32_t)(0xFFU << RF_REG05_RX_MAC_TIMER_RX_MAC_TIMER_Pos))

/* RF_REG05 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t RX_MAC_TIMER_RX_MAC_TIMER_BYTE;
    __IO uint8_t TX_MAC_TIMER_TX_MAC_TIMER_BYTE;
    __IO uint8_t BANK_BYTE;            
    __IO uint8_t CHANNEL_BYTE;         
} RF_REG05_Type;

#define RF_REG05_BASE                   0x40010014
#define RF_REG05                        ((RF_REG05_Type *) RF_REG05_BASE)

/* RF_REG05 settings */
#define RX_MAC_TIMER_RX_MAC_TIMER_DEFAULT ((uint32_t)(0x0U << RF_REG05_RX_MAC_TIMER_RX_MAC_TIMER_Pos))

#define TX_MAC_TIMER_TX_MAC_TIMER_DEFAULT ((uint32_t)(0x0U << RF_REG05_TX_MAC_TIMER_TX_MAC_TIMER_Pos))

#define BANK_BANK_DEFAULT               ((uint32_t)(0x0U << RF_REG05_BANK_BANK_Pos))

#define BANK_DATARATE_TX_NRX_DEFAULT    ((uint32_t)(0x0U << RF_REG05_BANK_DATARATE_TX_NRX_Pos))

#define CHANNEL_CHANNEL_DEFAULT         ((uint32_t)(0x0U << RF_REG05_CHANNEL_CHANNEL_Pos))

#define CHANNEL_SWITCH_IQ_DEFAULT       ((uint32_t)(0x0U << RF_REG05_CHANNEL_SWITCH_IQ_Pos))

/* RF_REG05 sub-register bit positions */
#define RF_REG05_BANK_BANK_BYTE_Pos     0
#define RF_REG05_BANK_BANK_BYTE_Mask    ((uint32_t)(0x3U << RF_REG05_BANK_BANK_BYTE_Pos))
#define RF_REG05_BANK_DATARATE_TX_NRX_BYTE_Pos 2
#define RF_REG05_CHANNEL_CHANNEL_BYTE_Pos 0
#define RF_REG05_CHANNEL_CHANNEL_BYTE_Mask ((uint32_t)(0x3FU << RF_REG05_CHANNEL_CHANNEL_BYTE_Pos))
#define RF_REG05_CHANNEL_SWITCH_IQ_BYTE_Pos 6

/* RF_REG05 sub-register settings */
#define RX_MAC_TIMER_RX_MAC_TIMER_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_MAC_TIMER_TX_MAC_TIMER_DEFAULT_BYTE ((uint8_t)0x0U)

#define BANK_BANK_DEFAULT_BYTE          ((uint8_t)(0x0U << RF_REG05_BANK_BANK_BYTE_Pos))

#define BANK_DATARATE_TX_NRX_DEFAULT_BYTE ((uint8_t)(BANK_DATARATE_TX_NRX_DEFAULT_BITBAND << RF_REG05_BANK_DATARATE_TX_NRX_BYTE_Pos))

#define CHANNEL_CHANNEL_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG05_CHANNEL_CHANNEL_BYTE_Pos))

#define CHANNEL_SWITCH_IQ_DEFAULT_BYTE  ((uint8_t)(CHANNEL_SWITCH_IQ_DEFAULT_BITBAND << RF_REG05_CHANNEL_SWITCH_IQ_BYTE_Pos))

/* RF_CENTER_FREQ bit positions */
#define RF_CENTER_FREQ_CENTER_FREQ_ADAPT_CFREQ_Pos 31
#define RF_CENTER_FREQ_CENTER_FREQ_RX_DIV_5_N6_Pos 30
#define RF_CENTER_FREQ_CENTER_FREQ_CENTER_FREQUENCY_Pos 0
#define RF_CENTER_FREQ_CENTER_FREQ_CENTER_FREQUENCY_Mask ((uint32_t)(0x3FFFFFFFU << RF_CENTER_FREQ_CENTER_FREQ_CENTER_FREQUENCY_Pos))

/* RF_CENTER_FREQ settings */
#define CENTER_FREQ_CENTER_FREQUENCY_DEFAULT ((uint32_t)(0x0U << RF_CENTER_FREQ_CENTER_FREQ_CENTER_FREQUENCY_Pos))

#define CENTER_FREQ_RX_DIV_5_N6_DEFAULT ((uint32_t)(0x0U << RF_CENTER_FREQ_CENTER_FREQ_RX_DIV_5_N6_Pos))

#define CENTER_FREQ_ADAPT_CFREQ_DEFAULT ((uint32_t)(0x0U << RF_CENTER_FREQ_CENTER_FREQ_ADAPT_CFREQ_Pos))

/* RF_REG07 bit positions */
#define RF_REG07_CHANNELS_1_CHANNEL_SPACING_LO_Pos 16
#define RF_REG07_CHANNELS_1_CHANNEL_SPACING_LO_Mask ((uint32_t)(0xFFFFU << RF_REG07_CHANNELS_1_CHANNEL_SPACING_LO_Pos))
#define RF_REG07_MOD_INFO_RX_EN_DIV_2_N3_RX_Pos 14
#define RF_REG07_MOD_INFO_RX_SYMBOL_2BIT_RX_Pos 13
#define RF_REG07_MOD_INFO_RX_DR_M_RX_Pos 8
#define RF_REG07_MOD_INFO_RX_DR_M_RX_Mask ((uint32_t)(0x1FU << RF_REG07_MOD_INFO_RX_DR_M_RX_Pos))
#define RF_REG07_MOD_INFO_TX_EN_DIV_2_N3_TX_Pos 6
#define RF_REG07_MOD_INFO_TX_SYMBOL_2BIT_TX_Pos 5
#define RF_REG07_MOD_INFO_TX_DR_M_TX_Pos 0
#define RF_REG07_MOD_INFO_TX_DR_M_TX_Mask ((uint32_t)(0x1FU << RF_REG07_MOD_INFO_TX_DR_M_TX_Pos))

/* RF_REG07 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t MOD_INFO_TX_BYTE;     
    __IO uint8_t MOD_INFO_RX_BYTE;     
    __IO uint16_t CHANNELS_1_CHANNEL_SPACING_LO_SHORT;
} RF_REG07_Type;

#define RF_REG07_BASE                   0x4001001C
#define RF_REG07                        ((RF_REG07_Type *) RF_REG07_BASE)

/* RF_REG07 settings */
#define MOD_INFO_TX_DR_M_TX_DEFAULT     ((uint32_t)(0x0U << RF_REG07_MOD_INFO_TX_DR_M_TX_Pos))

#define MOD_INFO_TX_SYMBOL_2BIT_TX_DEFAULT ((uint32_t)(0x0U << RF_REG07_MOD_INFO_TX_SYMBOL_2BIT_TX_Pos))

#define MOD_INFO_TX_EN_DIV_2_N3_TX_DEFAULT ((uint32_t)(0x0U << RF_REG07_MOD_INFO_TX_EN_DIV_2_N3_TX_Pos))

#define MOD_INFO_RX_DR_M_RX_DEFAULT     ((uint32_t)(0x0U << RF_REG07_MOD_INFO_RX_DR_M_RX_Pos))

#define MOD_INFO_RX_SYMBOL_2BIT_RX_DEFAULT ((uint32_t)(0x0U << RF_REG07_MOD_INFO_RX_SYMBOL_2BIT_RX_Pos))

#define MOD_INFO_RX_EN_DIV_2_N3_RX_DEFAULT ((uint32_t)(0x0U << RF_REG07_MOD_INFO_RX_EN_DIV_2_N3_RX_Pos))

#define CHANNELS_1_CHANNEL_SPACING_LO_DEFAULT ((uint32_t)(0x0U << RF_REG07_CHANNELS_1_CHANNEL_SPACING_LO_Pos))

/* RF_REG07 sub-register bit positions */
#define RF_REG07_MOD_INFO_TX_DR_M_TX_BYTE_Pos 0
#define RF_REG07_MOD_INFO_TX_DR_M_TX_BYTE_Mask ((uint32_t)(0x1FU << RF_REG07_MOD_INFO_TX_DR_M_TX_BYTE_Pos))
#define RF_REG07_MOD_INFO_TX_SYMBOL_2BIT_TX_BYTE_Pos 5
#define RF_REG07_MOD_INFO_TX_EN_DIV_2_N3_TX_BYTE_Pos 6
#define RF_REG07_MOD_INFO_RX_DR_M_RX_BYTE_Pos 0
#define RF_REG07_MOD_INFO_RX_DR_M_RX_BYTE_Mask ((uint32_t)(0x1FU << RF_REG07_MOD_INFO_RX_DR_M_RX_BYTE_Pos))
#define RF_REG07_MOD_INFO_RX_SYMBOL_2BIT_RX_BYTE_Pos 5
#define RF_REG07_MOD_INFO_RX_EN_DIV_2_N3_RX_BYTE_Pos 6

/* RF_REG07 sub-register settings */
#define MOD_INFO_TX_DR_M_TX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG07_MOD_INFO_TX_DR_M_TX_BYTE_Pos))

#define MOD_INFO_TX_SYMBOL_2BIT_TX_DEFAULT_BYTE ((uint8_t)(MOD_INFO_TX_SYMBOL_2BIT_TX_DEFAULT_BITBAND << RF_REG07_MOD_INFO_TX_SYMBOL_2BIT_TX_BYTE_Pos))

#define MOD_INFO_TX_EN_DIV_2_N3_TX_DEFAULT_BYTE ((uint8_t)(MOD_INFO_TX_EN_DIV_2_N3_TX_DEFAULT_BITBAND << RF_REG07_MOD_INFO_TX_EN_DIV_2_N3_TX_BYTE_Pos))

#define MOD_INFO_RX_DR_M_RX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG07_MOD_INFO_RX_DR_M_RX_BYTE_Pos))

#define MOD_INFO_RX_SYMBOL_2BIT_RX_DEFAULT_BYTE ((uint8_t)(MOD_INFO_RX_SYMBOL_2BIT_RX_DEFAULT_BITBAND << RF_REG07_MOD_INFO_RX_SYMBOL_2BIT_RX_BYTE_Pos))

#define MOD_INFO_RX_EN_DIV_2_N3_RX_DEFAULT_BYTE ((uint8_t)(MOD_INFO_RX_EN_DIV_2_N3_RX_DEFAULT_BITBAND << RF_REG07_MOD_INFO_RX_EN_DIV_2_N3_RX_BYTE_Pos))

#define CHANNELS_1_CHANNEL_SPACING_LO_DEFAULT_SHORT ((uint16_t)0x0U)

/* RF_REG08 bit positions */
#define RF_REG08_PACKET_LENGTH_PACKET_LEN_Pos 24
#define RF_REG08_PACKET_LENGTH_PACKET_LEN_Mask ((uint32_t)(0xFFU << RF_REG08_PACKET_LENGTH_PACKET_LEN_Pos))
#define RF_REG08_PACKET_HANDLING_LSB_FIRST_Pos 23
#define RF_REG08_PACKET_HANDLING_EN_CRC_Pos 22
#define RF_REG08_PACKET_HANDLING_EN_CRC_ON_PKTLEN_Pos 21
#define RF_REG08_PACKET_HANDLING_EN_PREAMBLE_Pos 20
#define RF_REG08_PACKET_HANDLING_EN_MULTI_FRAME_Pos 19
#define RF_REG08_PACKET_HANDLING_ENB_DW_ON_CRC_Pos 18
#define RF_REG08_PACKET_HANDLING_EN_PATTERN_Pos 17
#define RF_REG08_PACKET_HANDLING_EN_PACKET_Pos 16
#define RF_REG08_CODING_EN_DATAWHITE_Pos 15
#define RF_REG08_CODING_I_NQ_DELAYED_Pos 14
#define RF_REG08_CODING_OFFSET_Pos      13
#define RF_REG08_CODING_BIT_INVERT_Pos  12
#define RF_REG08_CODING_EVEN_BEFORE_ODD_Pos 11
#define RF_REG08_CODING_EN_802154_L2F_Pos 10
#define RF_REG08_CODING_EN_802154_B2C_Pos 9
#define RF_REG08_CODING_EN_MANCHESTER_Pos 8
#define RF_REG08_CHANNELS_2_EN_CHANNEL_SEL_Pos 7
#define RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_Pos 0
#define RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_Mask ((uint32_t)(0xFU << RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_Pos))

/* RF_REG08 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t CHANNELS_2_BYTE;      
    __IO uint8_t CODING_BYTE;          
    __IO uint8_t PACKET_HANDLING_BYTE; 
    __IO uint8_t PACKET_LENGTH_PACKET_LEN_BYTE;
} RF_REG08_Type;

#define RF_REG08_BASE                   0x40010020
#define RF_REG08                        ((RF_REG08_Type *) RF_REG08_BASE)

/* RF_REG08 settings */
#define CHANNELS_2_CHANNEL_SPACING_HI_DEFAULT ((uint32_t)(0x0U << RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_Pos))

#define CHANNELS_2_EN_CHANNEL_SEL_DEFAULT ((uint32_t)(0x0U << RF_REG08_CHANNELS_2_EN_CHANNEL_SEL_Pos))

#define CODING_EN_MANCHESTER_DEFAULT    ((uint32_t)(0x0U << RF_REG08_CODING_EN_MANCHESTER_Pos))

#define CODING_EN_802154_B2C_DEFAULT    ((uint32_t)(0x0U << RF_REG08_CODING_EN_802154_B2C_Pos))

#define CODING_EN_802154_L2F_DEFAULT    ((uint32_t)(0x0U << RF_REG08_CODING_EN_802154_L2F_Pos))

#define CODING_EVEN_BEFORE_ODD_DEFAULT  ((uint32_t)(0x0U << RF_REG08_CODING_EVEN_BEFORE_ODD_Pos))

#define CODING_BIT_INVERT_DEFAULT       ((uint32_t)(0x0U << RF_REG08_CODING_BIT_INVERT_Pos))

#define CODING_OFFSET_DEFAULT           ((uint32_t)(0x0U << RF_REG08_CODING_OFFSET_Pos))

#define CODING_I_NQ_DELAYED_DEFAULT     ((uint32_t)(0x0U << RF_REG08_CODING_I_NQ_DELAYED_Pos))

#define CODING_EN_DATAWHITE_DEFAULT     ((uint32_t)(0x0U << RF_REG08_CODING_EN_DATAWHITE_Pos))

#define PACKET_HANDLING_EN_PACKET_DEFAULT ((uint32_t)(0x0U << RF_REG08_PACKET_HANDLING_EN_PACKET_Pos))

#define PACKET_HANDLING_EN_PATTERN_DEFAULT ((uint32_t)(0x0U << RF_REG08_PACKET_HANDLING_EN_PATTERN_Pos))

#define PACKET_HANDLING_ENB_DW_ON_CRC_DEFAULT ((uint32_t)(0x0U << RF_REG08_PACKET_HANDLING_ENB_DW_ON_CRC_Pos))

#define PACKET_HANDLING_EN_MULTI_FRAME_DEFAULT ((uint32_t)(0x0U << RF_REG08_PACKET_HANDLING_EN_MULTI_FRAME_Pos))

#define PACKET_HANDLING_EN_PREAMBLE_DEFAULT ((uint32_t)(0x0U << RF_REG08_PACKET_HANDLING_EN_PREAMBLE_Pos))

#define PACKET_HANDLING_EN_CRC_ON_PKTLEN_DEFAULT ((uint32_t)(0x0U << RF_REG08_PACKET_HANDLING_EN_CRC_ON_PKTLEN_Pos))

#define PACKET_HANDLING_EN_CRC_DEFAULT  ((uint32_t)(0x0U << RF_REG08_PACKET_HANDLING_EN_CRC_Pos))

#define PACKET_HANDLING_LSB_FIRST_DEFAULT ((uint32_t)(0x0U << RF_REG08_PACKET_HANDLING_LSB_FIRST_Pos))

#define PACKET_LENGTH_PACKET_LEN_DEFAULT ((uint32_t)(0xFFU << RF_REG08_PACKET_LENGTH_PACKET_LEN_Pos))

/* RF_REG08 sub-register bit positions */
#define RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_BYTE_Pos 0
#define RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_BYTE_Mask ((uint32_t)(0xFU << RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_BYTE_Pos))
#define RF_REG08_CHANNELS_2_EN_CHANNEL_SEL_BYTE_Pos 7
#define RF_REG08_CODING_EN_MANCHESTER_BYTE_Pos 0
#define RF_REG08_CODING_EN_802154_B2C_BYTE_Pos 1
#define RF_REG08_CODING_EN_802154_L2F_BYTE_Pos 2
#define RF_REG08_CODING_EVEN_BEFORE_ODD_BYTE_Pos 3
#define RF_REG08_CODING_BIT_INVERT_BYTE_Pos 4
#define RF_REG08_CODING_OFFSET_BYTE_Pos 5
#define RF_REG08_CODING_I_NQ_DELAYED_BYTE_Pos 6
#define RF_REG08_CODING_EN_DATAWHITE_BYTE_Pos 7
#define RF_REG08_PACKET_HANDLING_EN_PACKET_BYTE_Pos 0
#define RF_REG08_PACKET_HANDLING_EN_PATTERN_BYTE_Pos 1
#define RF_REG08_PACKET_HANDLING_ENB_DW_ON_CRC_BYTE_Pos 2
#define RF_REG08_PACKET_HANDLING_EN_MULTI_FRAME_BYTE_Pos 3
#define RF_REG08_PACKET_HANDLING_EN_PREAMBLE_BYTE_Pos 4
#define RF_REG08_PACKET_HANDLING_EN_CRC_ON_PKTLEN_BYTE_Pos 5
#define RF_REG08_PACKET_HANDLING_EN_CRC_BYTE_Pos 6
#define RF_REG08_PACKET_HANDLING_LSB_FIRST_BYTE_Pos 7

/* RF_REG08 sub-register settings */
#define CHANNELS_2_CHANNEL_SPACING_HI_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_BYTE_Pos))

#define CHANNELS_2_EN_CHANNEL_SEL_DEFAULT_BYTE ((uint8_t)(CHANNELS_2_EN_CHANNEL_SEL_DEFAULT_BITBAND << RF_REG08_CHANNELS_2_EN_CHANNEL_SEL_BYTE_Pos))

#define CODING_EN_MANCHESTER_DEFAULT_BYTE ((uint8_t)(CODING_EN_MANCHESTER_DEFAULT_BITBAND << RF_REG08_CODING_EN_MANCHESTER_BYTE_Pos))

#define CODING_EN_802154_B2C_DEFAULT_BYTE ((uint8_t)(CODING_EN_802154_B2C_DEFAULT_BITBAND << RF_REG08_CODING_EN_802154_B2C_BYTE_Pos))

#define CODING_EN_802154_L2F_DEFAULT_BYTE ((uint8_t)(CODING_EN_802154_L2F_DEFAULT_BITBAND << RF_REG08_CODING_EN_802154_L2F_BYTE_Pos))

#define CODING_EVEN_BEFORE_ODD_DEFAULT_BYTE ((uint8_t)(CODING_EVEN_BEFORE_ODD_DEFAULT_BITBAND << RF_REG08_CODING_EVEN_BEFORE_ODD_BYTE_Pos))

#define CODING_BIT_INVERT_DEFAULT_BYTE  ((uint8_t)(CODING_BIT_INVERT_DEFAULT_BITBAND << RF_REG08_CODING_BIT_INVERT_BYTE_Pos))

#define CODING_OFFSET_DEFAULT_BYTE      ((uint8_t)(CODING_OFFSET_DEFAULT_BITBAND << RF_REG08_CODING_OFFSET_BYTE_Pos))

#define CODING_I_NQ_DELAYED_DEFAULT_BYTE ((uint8_t)(CODING_I_NQ_DELAYED_DEFAULT_BITBAND << RF_REG08_CODING_I_NQ_DELAYED_BYTE_Pos))

#define CODING_EN_DATAWHITE_DEFAULT_BYTE ((uint8_t)(CODING_EN_DATAWHITE_DEFAULT_BITBAND << RF_REG08_CODING_EN_DATAWHITE_BYTE_Pos))

#define PACKET_HANDLING_EN_PACKET_DEFAULT_BYTE ((uint8_t)(PACKET_HANDLING_EN_PACKET_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_PACKET_BYTE_Pos))

#define PACKET_HANDLING_EN_PATTERN_DEFAULT_BYTE ((uint8_t)(PACKET_HANDLING_EN_PATTERN_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_PATTERN_BYTE_Pos))

#define PACKET_HANDLING_ENB_DW_ON_CRC_DEFAULT_BYTE ((uint8_t)(PACKET_HANDLING_ENB_DW_ON_CRC_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_ENB_DW_ON_CRC_BYTE_Pos))

#define PACKET_HANDLING_EN_MULTI_FRAME_DEFAULT_BYTE ((uint8_t)(PACKET_HANDLING_EN_MULTI_FRAME_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_MULTI_FRAME_BYTE_Pos))

#define PACKET_HANDLING_EN_PREAMBLE_DEFAULT_BYTE ((uint8_t)(PACKET_HANDLING_EN_PREAMBLE_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_PREAMBLE_BYTE_Pos))

#define PACKET_HANDLING_EN_CRC_ON_PKTLEN_DEFAULT_BYTE ((uint8_t)(PACKET_HANDLING_EN_CRC_ON_PKTLEN_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_CRC_ON_PKTLEN_BYTE_Pos))

#define PACKET_HANDLING_EN_CRC_DEFAULT_BYTE ((uint8_t)(PACKET_HANDLING_EN_CRC_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_CRC_BYTE_Pos))

#define PACKET_HANDLING_LSB_FIRST_DEFAULT_BYTE ((uint8_t)(PACKET_HANDLING_LSB_FIRST_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_LSB_FIRST_BYTE_Pos))

#define PACKET_LENGTH_PACKET_LEN_DEFAULT_BYTE ((uint8_t)0xFFU)

/* RF_REG09 bit positions */
#define RF_REG09_ADDRESS_CONF_ADDRESS_LEN_Pos 27
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BR_Pos 26
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_Pos 25
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_TX_Pos 24
#define RF_REG09_PREAMBLE_LENGTH_PREAMBLE_LEN_Pos 16
#define RF_REG09_PREAMBLE_LENGTH_PREAMBLE_LEN_Mask ((uint32_t)(0xFFU << RF_REG09_PREAMBLE_LENGTH_PREAMBLE_LEN_Pos))
#define RF_REG09_PREAMBLE_PREAMBLE_Pos  8
#define RF_REG09_PREAMBLE_PREAMBLE_Mask ((uint32_t)(0xFFU << RF_REG09_PREAMBLE_PREAMBLE_Pos))
#define RF_REG09_PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_Pos 6
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_Pos 2
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_Mask ((uint32_t)(0xFU << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_Pos))
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_Pos 0
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_Mask ((uint32_t)(0x3U << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_Pos))

/* RF_REG09 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t PACKET_LENGTH_OPTS_BYTE;
    __IO uint8_t PREAMBLE_PREAMBLE_BYTE;
    __IO uint8_t PREAMBLE_LENGTH_PREAMBLE_LEN_BYTE;
    __IO uint8_t ADDRESS_CONF_BYTE;    
} RF_REG09_Type;

#define RF_REG09_BASE                   0x40010024
#define RF_REG09                        ((RF_REG09_Type *) RF_REG09_BASE)

/* RF_REG09 settings */
#define PACKET_LENGTH_OPTS_PACKET_LEN_POS_DEFAULT ((uint32_t)(0x0U << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_Pos))

#define PACKET_LENGTH_OPTS_PACKET_LEN_CORR_DEFAULT ((uint32_t)(0x0U << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_Pos))

#define PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_DEFAULT ((uint32_t)(0x0U << RF_REG09_PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_Pos))

#define PREAMBLE_PREAMBLE_DEFAULT       ((uint32_t)(0x0U << RF_REG09_PREAMBLE_PREAMBLE_Pos))

#define PREAMBLE_LENGTH_PREAMBLE_LEN_DEFAULT ((uint32_t)(0x0U << RF_REG09_PREAMBLE_LENGTH_PREAMBLE_LEN_Pos))

#define ADDRESS_CONF_EN_ADDRESS_TX_DEFAULT ((uint32_t)(0x0U << RF_REG09_ADDRESS_CONF_EN_ADDRESS_TX_Pos))

#define ADDRESS_CONF_EN_ADDRESS_RX_DEFAULT ((uint32_t)(0x0U << RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_Pos))

#define ADDRESS_CONF_EN_ADDRESS_RX_BR_DEFAULT ((uint32_t)(0x0U << RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BR_Pos))

#define ADDRESS_CONF_ADDRESS_LEN_DEFAULT ((uint32_t)(0x0U << RF_REG09_ADDRESS_CONF_ADDRESS_LEN_Pos))

/* RF_REG09 sub-register bit positions */
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_BYTE_Pos 0
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_BYTE_Mask ((uint32_t)(0x3U << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_BYTE_Pos))
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_BYTE_Pos 2
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_BYTE_Mask ((uint32_t)(0xFU << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_BYTE_Pos))
#define RF_REG09_PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_BYTE_Pos 6
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_TX_BYTE_Pos 0
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BYTE_Pos 1
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BR_BYTE_Pos 2
#define RF_REG09_ADDRESS_CONF_ADDRESS_LEN_BYTE_Pos 3

/* RF_REG09 sub-register settings */
#define PACKET_LENGTH_OPTS_PACKET_LEN_POS_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_BYTE_Pos))

#define PACKET_LENGTH_OPTS_PACKET_LEN_CORR_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_BYTE_Pos))

#define PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_DEFAULT_BYTE ((uint8_t)(PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_DEFAULT_BITBAND << RF_REG09_PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_BYTE_Pos))

#define PREAMBLE_PREAMBLE_DEFAULT_BYTE  ((uint8_t)0x0U)

#define PREAMBLE_LENGTH_PREAMBLE_LEN_DEFAULT_BYTE ((uint8_t)0x0U)

#define ADDRESS_CONF_EN_ADDRESS_TX_DEFAULT_BYTE ((uint8_t)(ADDRESS_CONF_EN_ADDRESS_TX_DEFAULT_BITBAND << RF_REG09_ADDRESS_CONF_EN_ADDRESS_TX_BYTE_Pos))

#define ADDRESS_CONF_EN_ADDRESS_RX_DEFAULT_BYTE ((uint8_t)(ADDRESS_CONF_EN_ADDRESS_RX_DEFAULT_BITBAND << RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BYTE_Pos))

#define ADDRESS_CONF_EN_ADDRESS_RX_BR_DEFAULT_BYTE ((uint8_t)(ADDRESS_CONF_EN_ADDRESS_RX_BR_DEFAULT_BITBAND << RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BR_BYTE_Pos))

#define ADDRESS_CONF_ADDRESS_LEN_DEFAULT_BYTE ((uint8_t)(ADDRESS_CONF_ADDRESS_LEN_DEFAULT_BITBAND << RF_REG09_ADDRESS_CONF_ADDRESS_LEN_BYTE_Pos))

/* RF_REG0A bit positions */
#define RF_REG0A_ADDRESS_BROADCAST_ADDRESS_BR_Pos 16
#define RF_REG0A_ADDRESS_BROADCAST_ADDRESS_BR_Mask ((uint32_t)(0xFFFFU << RF_REG0A_ADDRESS_BROADCAST_ADDRESS_BR_Pos))
#define RF_REG0A_ADDRESS_ADDRESS_Pos    0
#define RF_REG0A_ADDRESS_ADDRESS_Mask   ((uint32_t)(0xFFFFU << RF_REG0A_ADDRESS_ADDRESS_Pos))

/* RF_REG0A sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t ADDRESS_ADDRESS_SHORT;
    __IO uint16_t ADDRESS_BROADCAST_ADDRESS_BR_SHORT;
} RF_REG0A_Type;

#define RF_REG0A_BASE                   0x40010028
#define RF_REG0A                        ((RF_REG0A_Type *) RF_REG0A_BASE)

/* RF_REG0A settings */
#define ADDRESS_ADDRESS_DEFAULT         ((uint32_t)(0x0U << RF_REG0A_ADDRESS_ADDRESS_Pos))

#define ADDRESS_BROADCAST_ADDRESS_BR_DEFAULT ((uint32_t)(0x0U << RF_REG0A_ADDRESS_BROADCAST_ADDRESS_BR_Pos))

/* RF_REG0A sub-register settings */
#define ADDRESS_ADDRESS_DEFAULT_SHORT   ((uint16_t)0x0U)

#define ADDRESS_BROADCAST_ADDRESS_BR_DEFAULT_SHORT ((uint16_t)0x0U)

/* RF_SYNC_PATTERN bit positions */
#define RF_SYNC_PATTERN_PATTERN_Pos     0
#define RF_SYNC_PATTERN_PATTERN_Mask    ((uint32_t)(0xFFFFFFFFU << RF_SYNC_PATTERN_PATTERN_Pos))

/* RF_SYNC_PATTERN settings */
#define PATTERN_DEFAULT                 ((uint32_t)(0x0U << RF_SYNC_PATTERN_PATTERN_Pos))

/* RF_REG0C bit positions */
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_2_Pos 26
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_2_Mask ((uint32_t)(0x1FU << RF_REG0C_CONV_CODES_POLY_CC_POLY_2_Pos))
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_1_Pos 21
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_1_Mask ((uint32_t)(0x1FU << RF_REG0C_CONV_CODES_POLY_CC_POLY_1_Pos))
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_0_Pos 16
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_0_Mask ((uint32_t)(0x1FU << RF_REG0C_CONV_CODES_POLY_CC_POLY_0_Pos))
#define RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_Pos 10
#define RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_Mask ((uint32_t)(0x3U << RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_Pos))
#define RF_REG0C_CONV_CODES_CONF_CC_EN_TX_STOP_Pos 9
#define RF_REG0C_CONV_CODES_CONF_EN_CONV_CODE_Pos 8
#define RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_Pos 6
#define RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_Mask ((uint32_t)(0x3U << RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_Pos))
#define RF_REG0C_PACKET_EXTRA_EN_STOP_WORD_Pos 5
#define RF_REG0C_PACKET_EXTRA_PKT_INFO_PRE_NPOST_Pos 4
#define RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_Pos 2
#define RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_Mask ((uint32_t)(0x3U << RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_Pos))
#define RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_Pos 0
#define RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_Mask ((uint32_t)(0x3U << RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_Pos))

/* RF_REG0C sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t PACKET_EXTRA_BYTE;    
    __IO uint8_t CONV_CODES_CONF_BYTE; 
    __IO uint16_t CONV_CODES_POLY_CC_POLY_SHORT;
} RF_REG0C_Type;

#define RF_REG0C_BASE                   0x40010030
#define RF_REG0C                        ((RF_REG0C_Type *) RF_REG0C_BASE)

/* RF_REG0C settings */
#define PACKET_EXTRA_PATTERN_WORD_LEN_DEFAULT ((uint32_t)(0x0U << RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_Pos))

#define PACKET_EXTRA_PATTERN_MAX_ERR_DEFAULT ((uint32_t)(0x0U << RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_Pos))

#define PACKET_EXTRA_PKT_INFO_PRE_NPOST_DEFAULT ((uint32_t)(0x0U << RF_REG0C_PACKET_EXTRA_PKT_INFO_PRE_NPOST_Pos))

#define PACKET_EXTRA_EN_STOP_WORD_DEFAULT ((uint32_t)(0x0U << RF_REG0C_PACKET_EXTRA_EN_STOP_WORD_Pos))

#define PACKET_EXTRA_STOP_WORD_LEN_DEFAULT ((uint32_t)(0x0U << RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_Pos))

#define CONV_CODES_CONF_EN_CONV_CODE_DEFAULT ((uint32_t)(0x0U << RF_REG0C_CONV_CODES_CONF_EN_CONV_CODE_Pos))

#define CONV_CODES_CONF_CC_EN_TX_STOP_DEFAULT ((uint32_t)(0x0U << RF_REG0C_CONV_CODES_CONF_CC_EN_TX_STOP_Pos))

#define CONV_CODES_CONF_CC_VITERBI_LEN_DEFAULT ((uint32_t)(0x0U << RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_Pos))

#define CONV_CODES_POLY_CC_POLY_0_DEFAULT ((uint32_t)(0x0U << RF_REG0C_CONV_CODES_POLY_CC_POLY_0_Pos))

#define CONV_CODES_POLY_CC_POLY_1_DEFAULT ((uint32_t)(0x0U << RF_REG0C_CONV_CODES_POLY_CC_POLY_1_Pos))

#define CONV_CODES_POLY_CC_POLY_2_DEFAULT ((uint32_t)(0x0U << RF_REG0C_CONV_CODES_POLY_CC_POLY_2_Pos))

/* RF_REG0C sub-register bit positions */
#define RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_BYTE_Pos 0
#define RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_BYTE_Mask ((uint32_t)(0x3U << RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_BYTE_Pos))
#define RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_BYTE_Pos 2
#define RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_BYTE_Mask ((uint32_t)(0x3U << RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_BYTE_Pos))
#define RF_REG0C_PACKET_EXTRA_PKT_INFO_PRE_NPOST_BYTE_Pos 4
#define RF_REG0C_PACKET_EXTRA_EN_STOP_WORD_BYTE_Pos 5
#define RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_BYTE_Pos 6
#define RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_BYTE_Mask ((uint32_t)(0x3U << RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_BYTE_Pos))
#define RF_REG0C_CONV_CODES_CONF_EN_CONV_CODE_BYTE_Pos 0
#define RF_REG0C_CONV_CODES_CONF_CC_EN_TX_STOP_BYTE_Pos 1
#define RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_BYTE_Pos 2
#define RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_BYTE_Mask ((uint32_t)(0x3U << RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_BYTE_Pos))
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_0_SHORT_Pos 0
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_0_SHORT_Mask ((uint32_t)(0x1FU << RF_REG0C_CONV_CODES_POLY_CC_POLY_0_SHORT_Pos))
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_1_SHORT_Pos 5
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_1_SHORT_Mask ((uint32_t)(0x1FU << RF_REG0C_CONV_CODES_POLY_CC_POLY_1_SHORT_Pos))
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_2_SHORT_Pos 10
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_2_SHORT_Mask ((uint32_t)(0x1FU << RF_REG0C_CONV_CODES_POLY_CC_POLY_2_SHORT_Pos))

/* RF_REG0C sub-register settings */
#define PACKET_EXTRA_PATTERN_WORD_LEN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_BYTE_Pos))

#define PACKET_EXTRA_PATTERN_MAX_ERR_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_BYTE_Pos))

#define PACKET_EXTRA_PKT_INFO_PRE_NPOST_DEFAULT_BYTE ((uint8_t)(PACKET_EXTRA_PKT_INFO_PRE_NPOST_DEFAULT_BITBAND << RF_REG0C_PACKET_EXTRA_PKT_INFO_PRE_NPOST_BYTE_Pos))

#define PACKET_EXTRA_EN_STOP_WORD_DEFAULT_BYTE ((uint8_t)(PACKET_EXTRA_EN_STOP_WORD_DEFAULT_BITBAND << RF_REG0C_PACKET_EXTRA_EN_STOP_WORD_BYTE_Pos))

#define PACKET_EXTRA_STOP_WORD_LEN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_BYTE_Pos))

#define CONV_CODES_CONF_EN_CONV_CODE_DEFAULT_BYTE ((uint8_t)(CONV_CODES_CONF_EN_CONV_CODE_DEFAULT_BITBAND << RF_REG0C_CONV_CODES_CONF_EN_CONV_CODE_BYTE_Pos))

#define CONV_CODES_CONF_CC_EN_TX_STOP_DEFAULT_BYTE ((uint8_t)(CONV_CODES_CONF_CC_EN_TX_STOP_DEFAULT_BITBAND << RF_REG0C_CONV_CODES_CONF_CC_EN_TX_STOP_BYTE_Pos))

#define CONV_CODES_CONF_CC_VITERBI_LEN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_BYTE_Pos))

#define CONV_CODES_POLY_CC_POLY_0_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG0C_CONV_CODES_POLY_CC_POLY_0_SHORT_Pos))

#define CONV_CODES_POLY_CC_POLY_1_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG0C_CONV_CODES_POLY_CC_POLY_1_SHORT_Pos))

#define CONV_CODES_POLY_CC_POLY_2_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG0C_CONV_CODES_POLY_CC_POLY_2_SHORT_Pos))

/* RF_CRC_POLYNOMIAL bit positions */
#define RF_CRC_POLYNOMIAL_CRC_POLYNOMIAL_CRC_POLY_Pos 0
#define RF_CRC_POLYNOMIAL_CRC_POLYNOMIAL_CRC_POLY_Mask ((uint32_t)(0xFFFFFFFFU << RF_CRC_POLYNOMIAL_CRC_POLYNOMIAL_CRC_POLY_Pos))

/* RF_CRC_POLYNOMIAL settings */
#define CRC_POLYNOMIAL_CRC_POLY_DEFAULT ((uint32_t)(0x0U << RF_CRC_POLYNOMIAL_CRC_POLYNOMIAL_CRC_POLY_Pos))

/* RF_CRC_RST bit positions */
#define RF_CRC_RST_CRC_RST_CRC_RST_Pos  0
#define RF_CRC_RST_CRC_RST_CRC_RST_Mask ((uint32_t)(0xFFFFFFFFU << RF_CRC_RST_CRC_RST_CRC_RST_Pos))

/* RF_CRC_RST settings */
#define CRC_RST_CRC_RST_DEFAULT         ((uint32_t)(0x0U << RF_CRC_RST_CRC_RST_CRC_RST_Pos))

/* RF_REG0F bit positions */
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_Pos 28
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_Mask ((uint32_t)(0xFU << RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_Pos))
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_Pos 24
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_Mask ((uint32_t)(0xFU << RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_Pos))
#define RF_REG0F_FRAC_CONF_TX_FRAC_GAIN_Pos 19
#define RF_REG0F_FRAC_CONF_RX_FRAC_GAIN_Pos 18
#define RF_REG0F_FRAC_CONF_TX_EN_FRAC_Pos 17
#define RF_REG0F_FRAC_CONF_RX_EN_FRAC_Pos 16
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_Pos 10
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_Mask ((uint32_t)(0x1FU << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_Pos))
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_Pos 5
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_Mask ((uint32_t)(0x1FU << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_Pos))
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_Pos 0
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_Mask ((uint32_t)(0x1FU << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_Pos))

/* RF_REG0F sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t CONV_CODES_PUNCT_CC_PUNCT_SHORT;
    __IO uint8_t FRAC_CONF_BYTE;       
    __IO uint8_t RX_FRAC_CONF_RX_FRAC_BYTE;
} RF_REG0F_Type;

#define RF_REG0F_BASE                   0x4001003C
#define RF_REG0F                        ((RF_REG0F_Type *) RF_REG0F_BASE)

/* RF_REG0F settings */
#define CONV_CODES_PUNCT_CC_PUNCT_0_DEFAULT ((uint32_t)(0x0U << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_Pos))

#define CONV_CODES_PUNCT_CC_PUNCT_1_DEFAULT ((uint32_t)(0x0U << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_Pos))

#define CONV_CODES_PUNCT_CC_PUNCT_2_DEFAULT ((uint32_t)(0x0U << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_Pos))

#define FRAC_CONF_RX_EN_FRAC_DEFAULT    ((uint32_t)(0x0U << RF_REG0F_FRAC_CONF_RX_EN_FRAC_Pos))

#define FRAC_CONF_TX_EN_FRAC_DEFAULT    ((uint32_t)(0x0U << RF_REG0F_FRAC_CONF_TX_EN_FRAC_Pos))

#define FRAC_CONF_RX_FRAC_GAIN_DEFAULT  ((uint32_t)(0x0U << RF_REG0F_FRAC_CONF_RX_FRAC_GAIN_Pos))

#define FRAC_CONF_TX_FRAC_GAIN_DEFAULT  ((uint32_t)(0x0U << RF_REG0F_FRAC_CONF_TX_FRAC_GAIN_Pos))

#define RX_FRAC_CONF_RX_FRAC_NUM_DEFAULT ((uint32_t)(0x0U << RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_Pos))

#define RX_FRAC_CONF_RX_FRAC_DEN_DEFAULT ((uint32_t)(0x0U << RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_Pos))

/* RF_REG0F sub-register bit positions */
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_SHORT_Pos 0
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_SHORT_Mask ((uint32_t)(0x1FU << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_SHORT_Pos))
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_SHORT_Pos 5
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_SHORT_Mask ((uint32_t)(0x1FU << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_SHORT_Pos))
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_SHORT_Pos 10
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_SHORT_Mask ((uint32_t)(0x1FU << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_SHORT_Pos))
#define RF_REG0F_FRAC_CONF_RX_EN_FRAC_BYTE_Pos 0
#define RF_REG0F_FRAC_CONF_TX_EN_FRAC_BYTE_Pos 1
#define RF_REG0F_FRAC_CONF_RX_FRAC_GAIN_BYTE_Pos 2
#define RF_REG0F_FRAC_CONF_TX_FRAC_GAIN_BYTE_Pos 3
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_BYTE_Pos 0
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_BYTE_Mask ((uint32_t)(0xFU << RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_BYTE_Pos))
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_BYTE_Pos 4
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_BYTE_Mask ((uint32_t)(0xFU << RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_BYTE_Pos))

/* RF_REG0F sub-register settings */
#define CONV_CODES_PUNCT_CC_PUNCT_0_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_SHORT_Pos))

#define CONV_CODES_PUNCT_CC_PUNCT_1_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_SHORT_Pos))

#define CONV_CODES_PUNCT_CC_PUNCT_2_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_SHORT_Pos))

#define FRAC_CONF_RX_EN_FRAC_DEFAULT_BYTE ((uint8_t)(FRAC_CONF_RX_EN_FRAC_DEFAULT_BITBAND << RF_REG0F_FRAC_CONF_RX_EN_FRAC_BYTE_Pos))

#define FRAC_CONF_TX_EN_FRAC_DEFAULT_BYTE ((uint8_t)(FRAC_CONF_TX_EN_FRAC_DEFAULT_BITBAND << RF_REG0F_FRAC_CONF_TX_EN_FRAC_BYTE_Pos))

#define FRAC_CONF_RX_FRAC_GAIN_DEFAULT_BYTE ((uint8_t)(FRAC_CONF_RX_FRAC_GAIN_DEFAULT_BITBAND << RF_REG0F_FRAC_CONF_RX_FRAC_GAIN_BYTE_Pos))

#define FRAC_CONF_TX_FRAC_GAIN_DEFAULT_BYTE ((uint8_t)(FRAC_CONF_TX_FRAC_GAIN_DEFAULT_BITBAND << RF_REG0F_FRAC_CONF_TX_FRAC_GAIN_BYTE_Pos))

#define RX_FRAC_CONF_RX_FRAC_NUM_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_BYTE_Pos))

#define RX_FRAC_CONF_RX_FRAC_DEN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_BYTE_Pos))

/* RF_REG10 bit positions */
#define RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_Pos 29
#define RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_Mask ((uint32_t)(0x7U << RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_Pos))
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_Pos 26
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_Mask ((uint32_t)(0x7U << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_Pos))
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_Pos 24
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_Mask ((uint32_t)(0x3U << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_Pos))
#define RF_REG10_FRONTEND_EN_PHADC_DEGLITCH_Pos 22
#define RF_REG10_FRONTEND_EN_RESAMPLE_RSSI_Pos 21
#define RF_REG10_FRONTEND_EN_RESAMPLE_PHADC_Pos 20
#define RF_REG10_FRONTEND_DIV_PHADC_Pos 16
#define RF_REG10_FRONTEND_DIV_PHADC_Mask ((uint32_t)(0xFU << RF_REG10_FRONTEND_DIV_PHADC_Pos))
#define RF_REG10_TX_MULT_TX_MULT_EXP_Pos 12
#define RF_REG10_TX_MULT_TX_MULT_EXP_Mask ((uint32_t)(0xFU << RF_REG10_TX_MULT_TX_MULT_EXP_Pos))
#define RF_REG10_TX_MULT_TX_MULT_MAN_Pos 8
#define RF_REG10_TX_MULT_TX_MULT_MAN_Mask ((uint32_t)(0xFU << RF_REG10_TX_MULT_TX_MULT_MAN_Pos))
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_Pos 4
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_Mask ((uint32_t)(0xFU << RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_Pos))
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_Pos 0
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_Mask ((uint32_t)(0xFU << RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_Pos))

/* RF_REG10 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TX_FRAC_CONF_TX_FRAC_BYTE;
    __IO uint8_t TX_MULT_TX_MULT_BYTE; 
         uint8_t RESERVED0[1];
    __IO uint8_t FRONTEND2_RESAMPLE_BYTE;
} RF_REG10_Type;

#define RF_REG10_BASE                   0x40010040
#define RF_REG10                        ((RF_REG10_Type *) RF_REG10_BASE)

/* RF_REG10 settings */
#define TX_FRAC_CONF_TX_FRAC_NUM_DEFAULT ((uint32_t)(0x0U << RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_Pos))

#define TX_FRAC_CONF_TX_FRAC_DEN_DEFAULT ((uint32_t)(0x0U << RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_Pos))

#define TX_MULT_TX_MULT_MAN_DEFAULT     ((uint32_t)(0x0U << RF_REG10_TX_MULT_TX_MULT_MAN_Pos))

#define TX_MULT_TX_MULT_EXP_DEFAULT     ((uint32_t)(0x0U << RF_REG10_TX_MULT_TX_MULT_EXP_Pos))

#define FRONTEND_DIV_PHADC_DEFAULT      ((uint32_t)(0x0U << RF_REG10_FRONTEND_DIV_PHADC_Pos))

#define FRONTEND_EN_RESAMPLE_PHADC_DEFAULT ((uint32_t)(0x0U << RF_REG10_FRONTEND_EN_RESAMPLE_PHADC_Pos))

#define FRONTEND_EN_RESAMPLE_RSSI_DEFAULT ((uint32_t)(0x0U << RF_REG10_FRONTEND_EN_RESAMPLE_RSSI_Pos))

#define FRONTEND_EN_PHADC_DEGLITCH_DEFAULT ((uint32_t)(0x0U << RF_REG10_FRONTEND_EN_PHADC_DEGLITCH_Pos))

#define FRONTEND2_RESAMPLE_RSSI_G1_DEFAULT ((uint32_t)(0x0U << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_Pos))

#define FRONTEND2_RESAMPLE_RSSI_G2_DEFAULT ((uint32_t)(0x0U << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_Pos))

#define FRONTEND2_RESAMPLE_PH_GAIN_DEFAULT ((uint32_t)(0x0U << RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_Pos))

/* RF_REG10 sub-register bit positions */
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_BYTE_Pos 0
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_BYTE_Mask ((uint32_t)(0xFU << RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_BYTE_Pos))
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_BYTE_Pos 4
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_BYTE_Mask ((uint32_t)(0xFU << RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_BYTE_Pos))
#define RF_REG10_TX_MULT_TX_MULT_MAN_BYTE_Pos 0
#define RF_REG10_TX_MULT_TX_MULT_MAN_BYTE_Mask ((uint32_t)(0xFU << RF_REG10_TX_MULT_TX_MULT_MAN_BYTE_Pos))
#define RF_REG10_TX_MULT_TX_MULT_EXP_BYTE_Pos 4
#define RF_REG10_TX_MULT_TX_MULT_EXP_BYTE_Mask ((uint32_t)(0xFU << RF_REG10_TX_MULT_TX_MULT_EXP_BYTE_Pos))
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_BYTE_Pos 0
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_BYTE_Mask ((uint32_t)(0x3U << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_BYTE_Pos))
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_BYTE_Pos 2
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_BYTE_Mask ((uint32_t)(0x7U << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_BYTE_Pos))
#define RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_BYTE_Pos 5
#define RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_BYTE_Mask ((uint32_t)(0x7U << RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_BYTE_Pos))

/* RF_REG10 sub-register settings */
#define TX_FRAC_CONF_TX_FRAC_NUM_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_BYTE_Pos))

#define TX_FRAC_CONF_TX_FRAC_DEN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_BYTE_Pos))

#define TX_MULT_TX_MULT_MAN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG10_TX_MULT_TX_MULT_MAN_BYTE_Pos))

#define TX_MULT_TX_MULT_EXP_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG10_TX_MULT_TX_MULT_EXP_BYTE_Pos))

#define FRONTEND2_RESAMPLE_RSSI_G1_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_BYTE_Pos))

#define FRONTEND2_RESAMPLE_RSSI_G2_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_BYTE_Pos))

#define FRONTEND2_RESAMPLE_PH_GAIN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_BYTE_Pos))

/* RF_TX_PULSE0 bit positions */
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF4_Pos 24
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF4_Mask ((uint32_t)(0xFFU << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF4_Pos))
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF3_Pos 16
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF3_Mask ((uint32_t)(0xFFU << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF3_Pos))
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF2_Pos 8
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF2_Mask ((uint32_t)(0xFFU << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF2_Pos))
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF1_Pos 0
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF1_Mask ((uint32_t)(0xFFU << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF1_Pos))

/* RF_TX_PULSE0 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TX_PULSE_SHAPE_1_TX_COEF1_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_1_TX_COEF2_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_1_TX_COEF3_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_1_TX_COEF4_BYTE;
} RF_TX_PULSE0_Type;

#define RF_TX_PULSE0_BASE               0x40010044
#define RF_TX_PULSE0                    ((RF_TX_PULSE0_Type *) RF_TX_PULSE0_BASE)

/* RF_TX_PULSE0 settings */
#define TX_PULSE_SHAPE_1_TX_COEF1_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF1_Pos))

#define TX_PULSE_SHAPE_1_TX_COEF2_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF2_Pos))

#define TX_PULSE_SHAPE_1_TX_COEF3_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF3_Pos))

#define TX_PULSE_SHAPE_1_TX_COEF4_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF4_Pos))

/* RF_TX_PULSE0 sub-register settings */
#define TX_PULSE_SHAPE_1_TX_COEF1_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_1_TX_COEF2_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_1_TX_COEF3_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_1_TX_COEF4_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_TX_PULSE1 bit positions */
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF8_Pos 24
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF8_Mask ((uint32_t)(0xFFU << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF8_Pos))
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF7_Pos 16
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF7_Mask ((uint32_t)(0xFFU << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF7_Pos))
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF6_Pos 8
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF6_Mask ((uint32_t)(0xFFU << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF6_Pos))
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF5_Pos 0
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF5_Mask ((uint32_t)(0xFFU << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF5_Pos))

/* RF_TX_PULSE1 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TX_PULSE_SHAPE_2_TX_COEF5_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_2_TX_COEF6_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_2_TX_COEF7_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_2_TX_COEF8_BYTE;
} RF_TX_PULSE1_Type;

#define RF_TX_PULSE1_BASE               0x40010048
#define RF_TX_PULSE1                    ((RF_TX_PULSE1_Type *) RF_TX_PULSE1_BASE)

/* RF_TX_PULSE1 settings */
#define TX_PULSE_SHAPE_2_TX_COEF5_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF5_Pos))

#define TX_PULSE_SHAPE_2_TX_COEF6_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF6_Pos))

#define TX_PULSE_SHAPE_2_TX_COEF7_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF7_Pos))

#define TX_PULSE_SHAPE_2_TX_COEF8_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF8_Pos))

/* RF_TX_PULSE1 sub-register settings */
#define TX_PULSE_SHAPE_2_TX_COEF5_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_2_TX_COEF6_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_2_TX_COEF7_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_2_TX_COEF8_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_TX_PULSE2 bit positions */
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF12_Pos 24
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF12_Mask ((uint32_t)(0xFFU << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF12_Pos))
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF11_Pos 16
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF11_Mask ((uint32_t)(0xFFU << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF11_Pos))
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF10_Pos 8
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF10_Mask ((uint32_t)(0xFFU << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF10_Pos))
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF9_Pos 0
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF9_Mask ((uint32_t)(0xFFU << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF9_Pos))

/* RF_TX_PULSE2 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TX_PULSE_SHAPE_3_TX_COEF9_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_3_TX_COEF10_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_3_TX_COEF11_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_3_TX_COEF12_BYTE;
} RF_TX_PULSE2_Type;

#define RF_TX_PULSE2_BASE               0x4001004C
#define RF_TX_PULSE2                    ((RF_TX_PULSE2_Type *) RF_TX_PULSE2_BASE)

/* RF_TX_PULSE2 settings */
#define TX_PULSE_SHAPE_3_TX_COEF9_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF9_Pos))

#define TX_PULSE_SHAPE_3_TX_COEF10_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF10_Pos))

#define TX_PULSE_SHAPE_3_TX_COEF11_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF11_Pos))

#define TX_PULSE_SHAPE_3_TX_COEF12_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF12_Pos))

/* RF_TX_PULSE2 sub-register settings */
#define TX_PULSE_SHAPE_3_TX_COEF9_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_3_TX_COEF10_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_3_TX_COEF11_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_3_TX_COEF12_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_TX_PULSE3 bit positions */
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF16_Pos 24
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF16_Mask ((uint32_t)(0xFFU << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF16_Pos))
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF15_Pos 16
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF15_Mask ((uint32_t)(0xFFU << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF15_Pos))
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF14_Pos 8
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF14_Mask ((uint32_t)(0xFFU << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF14_Pos))
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF13_Pos 0
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF13_Mask ((uint32_t)(0xFFU << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF13_Pos))

/* RF_TX_PULSE3 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TX_PULSE_SHAPE_4_TX_COEF13_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_4_TX_COEF14_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_4_TX_COEF15_BYTE;
    __IO uint8_t TX_PULSE_SHAPE_4_TX_COEF16_BYTE;
} RF_TX_PULSE3_Type;

#define RF_TX_PULSE3_BASE               0x40010050
#define RF_TX_PULSE3                    ((RF_TX_PULSE3_Type *) RF_TX_PULSE3_BASE)

/* RF_TX_PULSE3 settings */
#define TX_PULSE_SHAPE_4_TX_COEF13_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF13_Pos))

#define TX_PULSE_SHAPE_4_TX_COEF14_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF14_Pos))

#define TX_PULSE_SHAPE_4_TX_COEF15_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF15_Pos))

#define TX_PULSE_SHAPE_4_TX_COEF16_DEFAULT ((uint32_t)(0x0U << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF16_Pos))

/* RF_TX_PULSE3 sub-register settings */
#define TX_PULSE_SHAPE_4_TX_COEF13_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_4_TX_COEF14_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_4_TX_COEF15_DEFAULT_BYTE ((uint8_t)0x0U)

#define TX_PULSE_SHAPE_4_TX_COEF16_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_RX_PULSE bit positions */
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF8_Pos 28
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF8_Mask ((uint32_t)(0xFU << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF8_Pos))
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF7_Pos 24
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF7_Mask ((uint32_t)(0xFU << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF7_Pos))
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF6_Pos 20
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF6_Mask ((uint32_t)(0xFU << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF6_Pos))
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF5_Pos 16
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF5_Mask ((uint32_t)(0xFU << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF5_Pos))
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF4_Pos 12
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF4_Mask ((uint32_t)(0xFU << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF4_Pos))
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF3_Pos 8
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF3_Mask ((uint32_t)(0xFU << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF3_Pos))
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF2_Pos 4
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF2_Mask ((uint32_t)(0xFU << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF2_Pos))
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF1_Pos 0
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF1_Mask ((uint32_t)(0xFU << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF1_Pos))

/* RF_RX_PULSE settings */
#define RX_PULSE_SHAPE_RX_COEF1_DEFAULT ((uint32_t)(0x0U << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF1_Pos))

#define RX_PULSE_SHAPE_RX_COEF2_DEFAULT ((uint32_t)(0x0U << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF2_Pos))

#define RX_PULSE_SHAPE_RX_COEF3_DEFAULT ((uint32_t)(0x0U << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF3_Pos))

#define RX_PULSE_SHAPE_RX_COEF4_DEFAULT ((uint32_t)(0x0U << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF4_Pos))

#define RX_PULSE_SHAPE_RX_COEF5_DEFAULT ((uint32_t)(0x0U << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF5_Pos))

#define RX_PULSE_SHAPE_RX_COEF6_DEFAULT ((uint32_t)(0x0U << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF6_Pos))

#define RX_PULSE_SHAPE_RX_COEF7_DEFAULT ((uint32_t)(0x0U << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF7_Pos))

#define RX_PULSE_SHAPE_RX_COEF8_DEFAULT ((uint32_t)(0x0U << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF8_Pos))

/* RF_REG16 bit positions */
#define RF_REG16_RX_IF_RESAMPLE_PH_IF_Pos 25
#define RF_REG16_RX_IF_RESAMPLE_PH_IF_Mask ((uint32_t)(0xFU << RF_REG16_RX_IF_RESAMPLE_PH_IF_Pos))
#define RF_REG16_RX_IF_IF2_CLK_OS_Pos   16
#define RF_REG16_RX_IF_IF2_CLK_OS_Mask  ((uint32_t)(0x1FFU << RF_REG16_RX_IF_IF2_CLK_OS_Pos))
#define RF_REG16_FSK_FCR_AMP_1_FSK_FCR_AMP1_Pos 8
#define RF_REG16_FSK_FCR_AMP_1_FSK_FCR_AMP1_Mask ((uint32_t)(0xFFU << RF_REG16_FSK_FCR_AMP_1_FSK_FCR_AMP1_Pos))
#define RF_REG16_FILTER_GAIN_DR_LIMIT_Pos 6
#define RF_REG16_FILTER_GAIN_DR_LIMIT_Mask ((uint32_t)(0x3U << RF_REG16_FILTER_GAIN_DR_LIMIT_Pos))
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_M_Pos 3
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_M_Mask ((uint32_t)(0x7U << RF_REG16_FILTER_GAIN_FILTER_GAIN_M_Pos))
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_E_Pos 0
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_E_Mask ((uint32_t)(0x7U << RF_REG16_FILTER_GAIN_FILTER_GAIN_E_Pos))

/* RF_REG16 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t FILTER_GAIN_BYTE;     
    __IO uint8_t FSK_FCR_AMP_1_FSK_FCR_AMP1_BYTE;
    __IO uint16_t RX_IF_SHORT;          
} RF_REG16_Type;

#define RF_REG16_BASE                   0x40010058
#define RF_REG16                        ((RF_REG16_Type *) RF_REG16_BASE)

/* RF_REG16 settings */
#define FILTER_GAIN_FILTER_GAIN_E_DEFAULT ((uint32_t)(0x0U << RF_REG16_FILTER_GAIN_FILTER_GAIN_E_Pos))

#define FILTER_GAIN_FILTER_GAIN_M_DEFAULT ((uint32_t)(0x0U << RF_REG16_FILTER_GAIN_FILTER_GAIN_M_Pos))

#define FILTER_GAIN_DR_LIMIT_DEFAULT    ((uint32_t)(0x0U << RF_REG16_FILTER_GAIN_DR_LIMIT_Pos))

#define FSK_FCR_AMP_1_FSK_FCR_AMP1_DEFAULT ((uint32_t)(0x0U << RF_REG16_FSK_FCR_AMP_1_FSK_FCR_AMP1_Pos))

#define RX_IF_IF2_CLK_OS_DEFAULT        ((uint32_t)(0x0U << RF_REG16_RX_IF_IF2_CLK_OS_Pos))

#define RX_IF_RESAMPLE_PH_IF_DEFAULT    ((uint32_t)(0x0U << RF_REG16_RX_IF_RESAMPLE_PH_IF_Pos))

/* RF_REG16 sub-register bit positions */
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_E_BYTE_Pos 0
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_E_BYTE_Mask ((uint32_t)(0x7U << RF_REG16_FILTER_GAIN_FILTER_GAIN_E_BYTE_Pos))
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_M_BYTE_Pos 3
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_M_BYTE_Mask ((uint32_t)(0x7U << RF_REG16_FILTER_GAIN_FILTER_GAIN_M_BYTE_Pos))
#define RF_REG16_FILTER_GAIN_DR_LIMIT_BYTE_Pos 6
#define RF_REG16_FILTER_GAIN_DR_LIMIT_BYTE_Mask ((uint32_t)(0x3U << RF_REG16_FILTER_GAIN_DR_LIMIT_BYTE_Pos))
#define RF_REG16_RX_IF_IF2_CLK_OS_SHORT_Pos 0
#define RF_REG16_RX_IF_IF2_CLK_OS_SHORT_Mask ((uint32_t)(0x1FFU << RF_REG16_RX_IF_IF2_CLK_OS_SHORT_Pos))
#define RF_REG16_RX_IF_RESAMPLE_PH_IF_SHORT_Pos 9
#define RF_REG16_RX_IF_RESAMPLE_PH_IF_SHORT_Mask ((uint32_t)(0xFU << RF_REG16_RX_IF_RESAMPLE_PH_IF_SHORT_Pos))

/* RF_REG16 sub-register settings */
#define FILTER_GAIN_FILTER_GAIN_E_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG16_FILTER_GAIN_FILTER_GAIN_E_BYTE_Pos))

#define FILTER_GAIN_FILTER_GAIN_M_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG16_FILTER_GAIN_FILTER_GAIN_M_BYTE_Pos))

#define FILTER_GAIN_DR_LIMIT_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG16_FILTER_GAIN_DR_LIMIT_BYTE_Pos))

#define FSK_FCR_AMP_1_FSK_FCR_AMP1_DEFAULT_BYTE ((uint8_t)0x0U)

#define RX_IF_IF2_CLK_OS_DEFAULT_SHORT  ((uint16_t)(0x0U << RF_REG16_RX_IF_IF2_CLK_OS_SHORT_Pos))

#define RX_IF_RESAMPLE_PH_IF_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG16_RX_IF_RESAMPLE_PH_IF_SHORT_Pos))

/* RF_REG17 bit positions */
#define RF_REG17_FSK_FCR_AMP_3_FSK_FCR_AMP3_Pos 24
#define RF_REG17_FSK_FCR_AMP_3_FSK_FCR_AMP3_Mask ((uint32_t)(0xFFU << RF_REG17_FSK_FCR_AMP_3_FSK_FCR_AMP3_Pos))
#define RF_REG17_FSK_FCR_AMP_2_FSK_FCR_AMP2_Pos 16
#define RF_REG17_FSK_FCR_AMP_2_FSK_FCR_AMP2_Mask ((uint32_t)(0xFFU << RF_REG17_FSK_FCR_AMP_2_FSK_FCR_AMP2_Pos))
#define RF_REG17_CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_Pos 13
#define RF_REG17_CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_Mask ((uint32_t)(0x3U << RF_REG17_CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_Pos))
#define RF_REG17_CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_Pos 12
#define RF_REG17_CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_Pos 9
#define RF_REG17_CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_Mask ((uint32_t)(0x7U << RF_REG17_CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_Pos))
#define RF_REG17_CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_Pos 8
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_Pos 4
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_Mask ((uint32_t)(0x7U << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_Pos))
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_Pos 0
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_Mask ((uint32_t)(0x7U << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_Pos))

/* RF_REG17 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_BYTE;
         uint8_t RESERVED0[1];
    __IO uint8_t FSK_FCR_AMP_2_FSK_FCR_AMP2_BYTE;
    __IO uint8_t FSK_FCR_AMP_3_FSK_FCR_AMP3_BYTE;
} RF_REG17_Type;

#define RF_REG17_BASE                   0x4001005C
#define RF_REG17                        ((RF_REG17_Type *) RF_REG17_BASE)

/* RF_REG17 settings */
#define CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_DEFAULT ((uint32_t)(0x0U << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_Pos))

#define CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_DEFAULT ((uint32_t)(0x0U << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_Pos))

#define CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_DEFAULT ((uint32_t)(0x0U << RF_REG17_CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_Pos))

#define CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_DEFAULT ((uint32_t)(0x0U << RF_REG17_CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_Pos))

#define CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_DEFAULT ((uint32_t)(0x0U << RF_REG17_CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_Pos))

#define CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_DEFAULT ((uint32_t)(0x0U << RF_REG17_CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_Pos))

#define FSK_FCR_AMP_2_FSK_FCR_AMP2_DEFAULT ((uint32_t)(0x0U << RF_REG17_FSK_FCR_AMP_2_FSK_FCR_AMP2_Pos))

#define FSK_FCR_AMP_3_FSK_FCR_AMP3_DEFAULT ((uint32_t)(0x0U << RF_REG17_FSK_FCR_AMP_3_FSK_FCR_AMP3_Pos))

/* RF_REG17 sub-register bit positions */
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_BYTE_Pos 0
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_BYTE_Mask ((uint32_t)(0x7U << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_BYTE_Pos))
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_BYTE_Pos 4
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_BYTE_Mask ((uint32_t)(0x7U << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_BYTE_Pos))

/* RF_REG17 sub-register settings */
#define CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_BYTE_Pos))

#define CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_BYTE_Pos))

#define FSK_FCR_AMP_2_FSK_FCR_AMP2_DEFAULT_BYTE ((uint8_t)0x0U)

#define FSK_FCR_AMP_3_FSK_FCR_AMP3_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_REG18 bit positions */
#define RF_REG18_CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_Pos 16
#define RF_REG18_CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_Mask ((uint32_t)(0xFFFFU << RF_REG18_CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_Pos))
#define RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_Pos 14
#define RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_Mask ((uint32_t)(0x3U << RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_Pos))
#define RF_REG18_RSSI_BANK_FAST_RSSI_Pos 13
#define RF_REG18_RSSI_BANK_EN_FAST_PRE_SYNC_Pos 12
#define RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_Pos 8
#define RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_Mask ((uint32_t)(0xFU << RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_Pos))
#define RF_REG18_DECISION_USE_VIT_SOFT_Pos 4
#define RF_REG18_DECISION_VITERBI_LEN_Pos 2
#define RF_REG18_DECISION_VITERBI_LEN_Mask ((uint32_t)(0x3U << RF_REG18_DECISION_VITERBI_LEN_Pos))
#define RF_REG18_DECISION_VITERBI_POW_NLIN_Pos 1
#define RF_REG18_DECISION_EN_VITERBI_GFSK_Pos 0

/* RF_REG18 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DECISION_BYTE;        
    __IO uint8_t RSSI_BANK_BYTE;       
    __IO uint16_t CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_SHORT;
} RF_REG18_Type;

#define RF_REG18_BASE                   0x40010060
#define RF_REG18                        ((RF_REG18_Type *) RF_REG18_BASE)

/* RF_REG18 settings */
#define DECISION_EN_VITERBI_GFSK_DEFAULT ((uint32_t)(0x0U << RF_REG18_DECISION_EN_VITERBI_GFSK_Pos))

#define DECISION_VITERBI_POW_NLIN_DEFAULT ((uint32_t)(0x0U << RF_REG18_DECISION_VITERBI_POW_NLIN_Pos))

#define DECISION_VITERBI_LEN_DEFAULT    ((uint32_t)(0x0U << RF_REG18_DECISION_VITERBI_LEN_Pos))

#define DECISION_USE_VIT_SOFT_DEFAULT   ((uint32_t)(0x0U << RF_REG18_DECISION_USE_VIT_SOFT_Pos))

#define RSSI_BANK_TAU_RSSI_FILTERING_DEFAULT ((uint32_t)(0x0U << RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_Pos))

#define RSSI_BANK_EN_FAST_PRE_SYNC_DEFAULT ((uint32_t)(0x0U << RF_REG18_RSSI_BANK_EN_FAST_PRE_SYNC_Pos))

#define RSSI_BANK_FAST_RSSI_DEFAULT     ((uint32_t)(0x0U << RF_REG18_RSSI_BANK_FAST_RSSI_Pos))

#define RSSI_BANK_RSSI_TRI_CK_DIV_DEFAULT ((uint32_t)(0x0U << RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_Pos))

#define CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_DEFAULT ((uint32_t)(0x0U << RF_REG18_CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_Pos))

/* RF_REG18 sub-register bit positions */
#define RF_REG18_DECISION_EN_VITERBI_GFSK_BYTE_Pos 0
#define RF_REG18_DECISION_VITERBI_POW_NLIN_BYTE_Pos 1
#define RF_REG18_DECISION_VITERBI_LEN_BYTE_Pos 2
#define RF_REG18_DECISION_VITERBI_LEN_BYTE_Mask ((uint32_t)(0x3U << RF_REG18_DECISION_VITERBI_LEN_BYTE_Pos))
#define RF_REG18_DECISION_USE_VIT_SOFT_BYTE_Pos 4
#define RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_BYTE_Pos 0
#define RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_BYTE_Mask ((uint32_t)(0xFU << RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_BYTE_Pos))
#define RF_REG18_RSSI_BANK_EN_FAST_PRE_SYNC_BYTE_Pos 4
#define RF_REG18_RSSI_BANK_FAST_RSSI_BYTE_Pos 5
#define RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_BYTE_Pos 6
#define RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_BYTE_Mask ((uint32_t)(0x3U << RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_BYTE_Pos))

/* RF_REG18 sub-register settings */
#define DECISION_EN_VITERBI_GFSK_DEFAULT_BYTE ((uint8_t)(DECISION_EN_VITERBI_GFSK_DEFAULT_BITBAND << RF_REG18_DECISION_EN_VITERBI_GFSK_BYTE_Pos))

#define DECISION_VITERBI_POW_NLIN_DEFAULT_BYTE ((uint8_t)(DECISION_VITERBI_POW_NLIN_DEFAULT_BITBAND << RF_REG18_DECISION_VITERBI_POW_NLIN_BYTE_Pos))

#define DECISION_VITERBI_LEN_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG18_DECISION_VITERBI_LEN_BYTE_Pos))

#define DECISION_USE_VIT_SOFT_DEFAULT_BYTE ((uint8_t)(DECISION_USE_VIT_SOFT_DEFAULT_BITBAND << RF_REG18_DECISION_USE_VIT_SOFT_BYTE_Pos))

#define RSSI_BANK_TAU_RSSI_FILTERING_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_BYTE_Pos))

#define RSSI_BANK_EN_FAST_PRE_SYNC_DEFAULT_BYTE ((uint8_t)(RSSI_BANK_EN_FAST_PRE_SYNC_DEFAULT_BITBAND << RF_REG18_RSSI_BANK_EN_FAST_PRE_SYNC_BYTE_Pos))

#define RSSI_BANK_FAST_RSSI_DEFAULT_BYTE ((uint8_t)(RSSI_BANK_FAST_RSSI_DEFAULT_BITBAND << RF_REG18_RSSI_BANK_FAST_RSSI_BYTE_Pos))

#define RSSI_BANK_RSSI_TRI_CK_DIV_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_BYTE_Pos))

#define CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_DEFAULT_SHORT ((uint16_t)0x0U)

/* RF_REG19 bit positions */
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_Pos 28
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_Mask ((uint32_t)(0x3U << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_Pos))
#define RF_REG19_PLL_BANK_IQ_PLL_0_TX_Pos 24
#define RF_REG19_PLL_BANK_IQ_PLL_0_TX_Mask ((uint32_t)(0xFU << RF_REG19_PLL_BANK_IQ_PLL_0_TX_Pos))
#define RF_REG19_PLL_BANK_LOW_DR_TX_Pos 22
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_Pos 20
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_Mask ((uint32_t)(0x3U << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_Pos))
#define RF_REG19_PLL_BANK_IQ_PLL_0_Pos  16
#define RF_REG19_PLL_BANK_IQ_PLL_0_Mask ((uint32_t)(0xFU << RF_REG19_PLL_BANK_IQ_PLL_0_Pos))
#define RF_REG19_PA_PWR_MIN_PA_PWR_Pos  13
#define RF_REG19_PA_PWR_PA_PWR_Pos      8
#define RF_REG19_PA_PWR_PA_PWR_Mask     ((uint32_t)(0x1FU << RF_REG19_PA_PWR_PA_PWR_Pos))
#define RF_REG19_CLK_CH_FILTER_DIV_RSSI_Pos 4
#define RF_REG19_CLK_CH_FILTER_DIV_RSSI_Mask ((uint32_t)(0xFU << RF_REG19_CLK_CH_FILTER_DIV_RSSI_Pos))
#define RF_REG19_CLK_CH_FILTER_DIV_FILT_Pos 0
#define RF_REG19_CLK_CH_FILTER_DIV_FILT_Mask ((uint32_t)(0xFU << RF_REG19_CLK_CH_FILTER_DIV_FILT_Pos))

/* RF_REG19 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t CLK_CH_FILTER_DIV_BYTE;
    __IO uint8_t PA_PWR_BYTE;          
    __IO uint16_t PLL_BANK_SHORT;       
} RF_REG19_Type;

#define RF_REG19_BASE                   0x40010064
#define RF_REG19                        ((RF_REG19_Type *) RF_REG19_BASE)

/* RF_REG19 settings */
#define CLK_CH_FILTER_DIV_FILT_DEFAULT  ((uint32_t)(0x0U << RF_REG19_CLK_CH_FILTER_DIV_FILT_Pos))

#define CLK_CH_FILTER_DIV_RSSI_DEFAULT  ((uint32_t)(0x0U << RF_REG19_CLK_CH_FILTER_DIV_RSSI_Pos))

#define PA_PWR_PA_PWR_DEFAULT           ((uint32_t)(0x0U << RF_REG19_PA_PWR_PA_PWR_Pos))

#define PA_PWR_MIN_PA_PWR_DEFAULT       ((uint32_t)(0x0U << RF_REG19_PA_PWR_MIN_PA_PWR_Pos))

#define PLL_BANK_IQ_PLL_0_DEFAULT       ((uint32_t)(0x0U << RF_REG19_PLL_BANK_IQ_PLL_0_Pos))

#define PLL_BANK_PLL_FILTER_RES_TRIM_DEFAULT ((uint32_t)(0x0U << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_Pos))

#define PLL_BANK_LOW_DR_TX_DEFAULT      ((uint32_t)(0x0U << RF_REG19_PLL_BANK_LOW_DR_TX_Pos))

#define PLL_BANK_IQ_PLL_0_TX_DEFAULT    ((uint32_t)(0x0U << RF_REG19_PLL_BANK_IQ_PLL_0_TX_Pos))

#define PLL_BANK_PLL_FILTER_RES_TRIM_TX_DEFAULT ((uint32_t)(0x0U << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_Pos))

/* RF_REG19 sub-register bit positions */
#define RF_REG19_CLK_CH_FILTER_DIV_FILT_BYTE_Pos 0
#define RF_REG19_CLK_CH_FILTER_DIV_FILT_BYTE_Mask ((uint32_t)(0xFU << RF_REG19_CLK_CH_FILTER_DIV_FILT_BYTE_Pos))
#define RF_REG19_CLK_CH_FILTER_DIV_RSSI_BYTE_Pos 4
#define RF_REG19_CLK_CH_FILTER_DIV_RSSI_BYTE_Mask ((uint32_t)(0xFU << RF_REG19_CLK_CH_FILTER_DIV_RSSI_BYTE_Pos))
#define RF_REG19_PA_PWR_PA_PWR_BYTE_Pos 0
#define RF_REG19_PA_PWR_PA_PWR_BYTE_Mask ((uint32_t)(0x1FU << RF_REG19_PA_PWR_PA_PWR_BYTE_Pos))
#define RF_REG19_PA_PWR_MIN_PA_PWR_BYTE_Pos 5
#define RF_REG19_PLL_BANK_IQ_PLL_0_SHORT_Pos 0
#define RF_REG19_PLL_BANK_IQ_PLL_0_SHORT_Mask ((uint32_t)(0xFU << RF_REG19_PLL_BANK_IQ_PLL_0_SHORT_Pos))
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_SHORT_Pos 4
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_SHORT_Mask ((uint32_t)(0x3U << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_SHORT_Pos))
#define RF_REG19_PLL_BANK_LOW_DR_TX_SHORT_Pos 6
#define RF_REG19_PLL_BANK_IQ_PLL_0_TX_SHORT_Pos 8
#define RF_REG19_PLL_BANK_IQ_PLL_0_TX_SHORT_Mask ((uint32_t)(0xFU << RF_REG19_PLL_BANK_IQ_PLL_0_TX_SHORT_Pos))
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_SHORT_Pos 12
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_SHORT_Mask ((uint32_t)(0x3U << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_SHORT_Pos))

/* RF_REG19 sub-register settings */
#define CLK_CH_FILTER_DIV_FILT_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG19_CLK_CH_FILTER_DIV_FILT_BYTE_Pos))

#define CLK_CH_FILTER_DIV_RSSI_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG19_CLK_CH_FILTER_DIV_RSSI_BYTE_Pos))

#define PA_PWR_PA_PWR_DEFAULT_BYTE      ((uint8_t)(0x0U << RF_REG19_PA_PWR_PA_PWR_BYTE_Pos))

#define PA_PWR_MIN_PA_PWR_DEFAULT_BYTE  ((uint8_t)(PA_PWR_MIN_PA_PWR_DEFAULT_BITBAND << RF_REG19_PA_PWR_MIN_PA_PWR_BYTE_Pos))

#define PLL_BANK_IQ_PLL_0_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG19_PLL_BANK_IQ_PLL_0_SHORT_Pos))

#define PLL_BANK_PLL_FILTER_RES_TRIM_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_SHORT_Pos))

#define PLL_BANK_LOW_DR_TX_DEFAULT_SHORT ((uint16_t)(PLL_BANK_LOW_DR_TX_DEFAULT_BITBAND << RF_REG19_PLL_BANK_LOW_DR_TX_SHORT_Pos))

#define PLL_BANK_IQ_PLL_0_TX_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG19_PLL_BANK_IQ_PLL_0_TX_SHORT_Pos))

#define PLL_BANK_PLL_FILTER_RES_TRIM_TX_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_SHORT_Pos))

/* RF_REG1A bit positions */
#define RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_Pos 28
#define RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_Mask ((uint32_t)(0xFU << RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_Pos))
#define RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_Pos 24
#define RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_Mask ((uint32_t)(0xFU << RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_Pos))
#define RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_Pos 22
#define RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_Mask ((uint32_t)(0x3U << RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_Pos))
#define RF_REG1A_RSSI_CTRL_AGC_USE_LNA_Pos 21
#define RF_REG1A_RSSI_CTRL_AGC_MODE_Pos 20
#define RF_REG1A_RSSI_CTRL_AGC_WAIT_Pos 18
#define RF_REG1A_RSSI_CTRL_AGC_WAIT_Mask ((uint32_t)(0x3U << RF_REG1A_RSSI_CTRL_AGC_WAIT_Pos))
#define RF_REG1A_RSSI_CTRL_PAYLOAD_BLOCKS_AGC_Pos 17
#define RF_REG1A_RSSI_CTRL_BYPASS_AGC_Pos 16
#define RF_REG1A_FILTER_BIAS_IQ_FI_BW_Pos 8
#define RF_REG1A_FILTER_BIAS_IQ_FI_BW_Mask ((uint32_t)(0x1FU << RF_REG1A_FILTER_BIAS_IQ_FI_BW_Pos))
#define RF_REG1A_FILTER_BIAS_IQ_FI_FC_Pos 0
#define RF_REG1A_FILTER_BIAS_IQ_FI_FC_Mask ((uint32_t)(0x1FU << RF_REG1A_FILTER_BIAS_IQ_FI_FC_Pos))

/* RF_REG1A sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t FILTER_BIAS_IQ_FI_SHORT;
    __IO uint8_t RSSI_CTRL_BYTE;       
    __IO uint8_t ATT_CTRL_BYTE;        
} RF_REG1A_Type;

#define RF_REG1A_BASE                   0x40010068
#define RF_REG1A                        ((RF_REG1A_Type *) RF_REG1A_BASE)

/* RF_REG1A settings */
#define FILTER_BIAS_IQ_FI_FC_DEFAULT    ((uint32_t)(0x0U << RF_REG1A_FILTER_BIAS_IQ_FI_FC_Pos))

#define FILTER_BIAS_IQ_FI_BW_DEFAULT    ((uint32_t)(0x0U << RF_REG1A_FILTER_BIAS_IQ_FI_BW_Pos))

#define RSSI_CTRL_BYPASS_AGC_DEFAULT    ((uint32_t)(0x0U << RF_REG1A_RSSI_CTRL_BYPASS_AGC_Pos))

#define RSSI_CTRL_PAYLOAD_BLOCKS_AGC_DEFAULT ((uint32_t)(0x0U << RF_REG1A_RSSI_CTRL_PAYLOAD_BLOCKS_AGC_Pos))

#define RSSI_CTRL_AGC_WAIT_DEFAULT      ((uint32_t)(0x0U << RF_REG1A_RSSI_CTRL_AGC_WAIT_Pos))

#define RSSI_CTRL_AGC_MODE_DEFAULT      ((uint32_t)(0x0U << RF_REG1A_RSSI_CTRL_AGC_MODE_Pos))

#define RSSI_CTRL_AGC_USE_LNA_DEFAULT   ((uint32_t)(0x0U << RF_REG1A_RSSI_CTRL_AGC_USE_LNA_Pos))

#define RSSI_CTRL_AGC_DECAY_TAU_DEFAULT ((uint32_t)(0x0U << RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_Pos))

#define ATT_CTRL_SET_RX_ATT_CTRL_DEFAULT ((uint32_t)(0x0U << RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_Pos))

#define ATT_CTRL_ATT_CTRL_MAX_DEFAULT   ((uint32_t)(0x0U << RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_Pos))

/* RF_REG1A sub-register bit positions */
#define RF_REG1A_FILTER_BIAS_IQ_FI_FC_SHORT_Pos 0
#define RF_REG1A_FILTER_BIAS_IQ_FI_FC_SHORT_Mask ((uint32_t)(0x1FU << RF_REG1A_FILTER_BIAS_IQ_FI_FC_SHORT_Pos))
#define RF_REG1A_FILTER_BIAS_IQ_FI_BW_SHORT_Pos 8
#define RF_REG1A_FILTER_BIAS_IQ_FI_BW_SHORT_Mask ((uint32_t)(0x1FU << RF_REG1A_FILTER_BIAS_IQ_FI_BW_SHORT_Pos))
#define RF_REG1A_RSSI_CTRL_BYPASS_AGC_BYTE_Pos 0
#define RF_REG1A_RSSI_CTRL_PAYLOAD_BLOCKS_AGC_BYTE_Pos 1
#define RF_REG1A_RSSI_CTRL_AGC_WAIT_BYTE_Pos 2
#define RF_REG1A_RSSI_CTRL_AGC_WAIT_BYTE_Mask ((uint32_t)(0x3U << RF_REG1A_RSSI_CTRL_AGC_WAIT_BYTE_Pos))
#define RF_REG1A_RSSI_CTRL_AGC_MODE_BYTE_Pos 4
#define RF_REG1A_RSSI_CTRL_AGC_USE_LNA_BYTE_Pos 5
#define RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_BYTE_Pos 6
#define RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_BYTE_Mask ((uint32_t)(0x3U << RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_BYTE_Pos))
#define RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_BYTE_Pos 0
#define RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_BYTE_Mask ((uint32_t)(0xFU << RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_BYTE_Pos))
#define RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_BYTE_Pos 4
#define RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_BYTE_Mask ((uint32_t)(0xFU << RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_BYTE_Pos))

/* RF_REG1A sub-register settings */
#define FILTER_BIAS_IQ_FI_FC_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG1A_FILTER_BIAS_IQ_FI_FC_SHORT_Pos))

#define FILTER_BIAS_IQ_FI_BW_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG1A_FILTER_BIAS_IQ_FI_BW_SHORT_Pos))

#define RSSI_CTRL_BYPASS_AGC_DEFAULT_BYTE ((uint8_t)(RSSI_CTRL_BYPASS_AGC_DEFAULT_BITBAND << RF_REG1A_RSSI_CTRL_BYPASS_AGC_BYTE_Pos))

#define RSSI_CTRL_PAYLOAD_BLOCKS_AGC_DEFAULT_BYTE ((uint8_t)(RSSI_CTRL_PAYLOAD_BLOCKS_AGC_DEFAULT_BITBAND << RF_REG1A_RSSI_CTRL_PAYLOAD_BLOCKS_AGC_BYTE_Pos))

#define RSSI_CTRL_AGC_WAIT_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG1A_RSSI_CTRL_AGC_WAIT_BYTE_Pos))

#define RSSI_CTRL_AGC_MODE_DEFAULT_BYTE ((uint8_t)(RSSI_CTRL_AGC_MODE_DEFAULT_BITBAND << RF_REG1A_RSSI_CTRL_AGC_MODE_BYTE_Pos))

#define RSSI_CTRL_AGC_USE_LNA_DEFAULT_BYTE ((uint8_t)(RSSI_CTRL_AGC_USE_LNA_DEFAULT_BITBAND << RF_REG1A_RSSI_CTRL_AGC_USE_LNA_BYTE_Pos))

#define RSSI_CTRL_AGC_DECAY_TAU_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_BYTE_Pos))

#define ATT_CTRL_SET_RX_ATT_CTRL_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_BYTE_Pos))

#define ATT_CTRL_ATT_CTRL_MAX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_BYTE_Pos))

/* RF_REG1B bit positions */
#define RF_REG1B_IEEE802154_OPTS_EN_DW_TEST_Pos 31
#define RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_Pos 29
#define RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_Mask ((uint32_t)(0x3U << RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_Pos))
#define RF_REG1B_IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_Pos 28
#define RF_REG1B_IEEE802154_OPTS_EN_L2F_RX_Pos 27
#define RF_REG1B_IEEE802154_OPTS_C2B_THR_Pos 24
#define RF_REG1B_IEEE802154_OPTS_C2B_THR_Mask ((uint32_t)(0x7U << RF_REG1B_IEEE802154_OPTS_C2B_THR_Pos))
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_Pos 20
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_Mask ((uint32_t)(0xFU << RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_Pos))
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_Pos 18
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_Mask ((uint32_t)(0x3U << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_Pos))
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_HIGH_Pos 17
#define RF_REG1B_AGC_PEAK_DET_EN_AGC_PEAK_Pos 16
#define RF_REG1B_AGC_THR_HIGH_AGC_THR_HIGH_Pos 8
#define RF_REG1B_AGC_THR_HIGH_AGC_THR_HIGH_Mask ((uint32_t)(0xFFU << RF_REG1B_AGC_THR_HIGH_AGC_THR_HIGH_Pos))
#define RF_REG1B_AGC_THR_LOW_AGC_THR_LOW_Pos 0
#define RF_REG1B_AGC_THR_LOW_AGC_THR_LOW_Mask ((uint32_t)(0xFFU << RF_REG1B_AGC_THR_LOW_AGC_THR_LOW_Pos))

/* RF_REG1B sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t AGC_THR_LOW_AGC_THR_LOW_BYTE;
    __IO uint8_t AGC_THR_HIGH_AGC_THR_HIGH_BYTE;
    __IO uint8_t AGC_PEAK_DET_BYTE;    
    __IO uint8_t IEEE802154_OPTS_BYTE; 
} RF_REG1B_Type;

#define RF_REG1B_BASE                   0x4001006C
#define RF_REG1B                        ((RF_REG1B_Type *) RF_REG1B_BASE)

/* RF_REG1B settings */
#define AGC_THR_LOW_AGC_THR_LOW_DEFAULT ((uint32_t)(0x0U << RF_REG1B_AGC_THR_LOW_AGC_THR_LOW_Pos))

#define AGC_THR_HIGH_AGC_THR_HIGH_DEFAULT ((uint32_t)(0x0U << RF_REG1B_AGC_THR_HIGH_AGC_THR_HIGH_Pos))

#define AGC_PEAK_DET_EN_AGC_PEAK_DEFAULT ((uint32_t)(0x0U << RF_REG1B_AGC_PEAK_DET_EN_AGC_PEAK_Pos))

#define AGC_PEAK_DET_PEAK_DET_THR_HIGH_DEFAULT ((uint32_t)(0x0U << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_HIGH_Pos))

#define AGC_PEAK_DET_PEAK_DET_THR_LOW_DEFAULT ((uint32_t)(0x0U << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_Pos))

#define AGC_PEAK_DET_PEAK_DET_TAU_DEFAULT ((uint32_t)(0x0U << RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_Pos))

#define IEEE802154_OPTS_C2B_THR_DEFAULT ((uint32_t)(0x0U << RF_REG1B_IEEE802154_OPTS_C2B_THR_Pos))

#define IEEE802154_OPTS_EN_L2F_RX_DEFAULT ((uint32_t)(0x0U << RF_REG1B_IEEE802154_OPTS_EN_L2F_RX_Pos))

#define IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_DEFAULT ((uint32_t)(0x0U << RF_REG1B_IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_Pos))

#define IEEE802154_OPTS_BER_CLK_MODE_DEFAULT ((uint32_t)(0x0U << RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_Pos))

#define IEEE802154_OPTS_EN_DW_TEST_DEFAULT ((uint32_t)(0x0U << RF_REG1B_IEEE802154_OPTS_EN_DW_TEST_Pos))

/* RF_REG1B sub-register bit positions */
#define RF_REG1B_AGC_PEAK_DET_EN_AGC_PEAK_BYTE_Pos 0
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_HIGH_BYTE_Pos 1
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_BYTE_Pos 2
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_BYTE_Mask ((uint32_t)(0x3U << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_BYTE_Pos))
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_BYTE_Pos 4
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_BYTE_Mask ((uint32_t)(0xFU << RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_BYTE_Pos))
#define RF_REG1B_IEEE802154_OPTS_C2B_THR_BYTE_Pos 0
#define RF_REG1B_IEEE802154_OPTS_C2B_THR_BYTE_Mask ((uint32_t)(0x7U << RF_REG1B_IEEE802154_OPTS_C2B_THR_BYTE_Pos))
#define RF_REG1B_IEEE802154_OPTS_EN_L2F_RX_BYTE_Pos 3
#define RF_REG1B_IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_BYTE_Pos 4
#define RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_BYTE_Pos 5
#define RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_BYTE_Mask ((uint32_t)(0x3U << RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_BYTE_Pos))
#define RF_REG1B_IEEE802154_OPTS_EN_DW_TEST_BYTE_Pos 7

/* RF_REG1B sub-register settings */
#define AGC_THR_LOW_AGC_THR_LOW_DEFAULT_BYTE ((uint8_t)0x0U)

#define AGC_THR_HIGH_AGC_THR_HIGH_DEFAULT_BYTE ((uint8_t)0x0U)

#define AGC_PEAK_DET_EN_AGC_PEAK_DEFAULT_BYTE ((uint8_t)(AGC_PEAK_DET_EN_AGC_PEAK_DEFAULT_BITBAND << RF_REG1B_AGC_PEAK_DET_EN_AGC_PEAK_BYTE_Pos))

#define AGC_PEAK_DET_PEAK_DET_THR_HIGH_DEFAULT_BYTE ((uint8_t)(AGC_PEAK_DET_PEAK_DET_THR_HIGH_DEFAULT_BITBAND << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_HIGH_BYTE_Pos))

#define AGC_PEAK_DET_PEAK_DET_THR_LOW_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_BYTE_Pos))

#define AGC_PEAK_DET_PEAK_DET_TAU_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_BYTE_Pos))

#define IEEE802154_OPTS_C2B_THR_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG1B_IEEE802154_OPTS_C2B_THR_BYTE_Pos))

#define IEEE802154_OPTS_EN_L2F_RX_DEFAULT_BYTE ((uint8_t)(IEEE802154_OPTS_EN_L2F_RX_DEFAULT_BITBAND << RF_REG1B_IEEE802154_OPTS_EN_L2F_RX_BYTE_Pos))

#define IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_DEFAULT_BYTE ((uint8_t)(IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_DEFAULT_BITBAND << RF_REG1B_IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_BYTE_Pos))

#define IEEE802154_OPTS_BER_CLK_MODE_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_BYTE_Pos))

#define IEEE802154_OPTS_EN_DW_TEST_DEFAULT_BYTE ((uint8_t)(IEEE802154_OPTS_EN_DW_TEST_DEFAULT_BITBAND << RF_REG1B_IEEE802154_OPTS_EN_DW_TEST_BYTE_Pos))

/* RF_AGC_LUT1 bit positions */
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_2_LO_Pos 22
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_2_LO_Mask ((uint32_t)(0x3FFU << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_2_LO_Pos))
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_1_Pos 11
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_1_Mask ((uint32_t)(0x7FFU << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_1_Pos))
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_0_Pos 0
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_0_Mask ((uint32_t)(0x7FFU << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_0_Pos))

/* RF_AGC_LUT1 settings */
#define AGC_LUT_1_AGC_LEVEL_0_DEFAULT   ((uint32_t)(0x0U << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_0_Pos))

#define AGC_LUT_1_AGC_LEVEL_1_DEFAULT   ((uint32_t)(0x0U << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_1_Pos))

#define AGC_LUT_1_AGC_LEVEL_2_LO_DEFAULT ((uint32_t)(0x0U << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_2_LO_Pos))

/* RF_AGC_LUT2 bit positions */
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_5_LO_Pos 23
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_5_LO_Mask ((uint32_t)(0x1FFU << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_5_LO_Pos))
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_4_Pos 12
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_4_Mask ((uint32_t)(0x7FFU << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_4_Pos))
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_3_Pos 1
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_3_Mask ((uint32_t)(0x7FFU << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_3_Pos))
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_2_HI_Pos 0

/* RF_AGC_LUT2 settings */
#define AGC_LUT_2_AGC_LEVEL_2_HI_DEFAULT ((uint32_t)(0x0U << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_2_HI_Pos))

#define AGC_LUT_2_AGC_LEVEL_3_DEFAULT   ((uint32_t)(0x0U << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_3_Pos))

#define AGC_LUT_2_AGC_LEVEL_4_DEFAULT   ((uint32_t)(0x0U << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_4_Pos))

#define AGC_LUT_2_AGC_LEVEL_5_LO_DEFAULT ((uint32_t)(0x0U << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_5_LO_Pos))

/* RF_AGC_LUT3 bit positions */
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_8_LO_Pos 24
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_8_LO_Mask ((uint32_t)(0xFFU << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_8_LO_Pos))
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_7_Pos 13
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_7_Mask ((uint32_t)(0x7FFU << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_7_Pos))
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_6_Pos 2
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_6_Mask ((uint32_t)(0x7FFU << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_6_Pos))
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_5_HI_Pos 0
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_5_HI_Mask ((uint32_t)(0x3U << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_5_HI_Pos))

/* RF_AGC_LUT3 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t AGC_LUT_3_AGC_LEVEL_8_LO_BYTE;
} RF_AGC_LUT3_Type;

#define RF_AGC_LUT3_BASE                0x40010078
#define RF_AGC_LUT3                     ((RF_AGC_LUT3_Type *) RF_AGC_LUT3_BASE)

/* RF_AGC_LUT3 settings */
#define AGC_LUT_3_AGC_LEVEL_5_HI_DEFAULT ((uint32_t)(0x0U << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_5_HI_Pos))

#define AGC_LUT_3_AGC_LEVEL_6_DEFAULT   ((uint32_t)(0x0U << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_6_Pos))

#define AGC_LUT_3_AGC_LEVEL_7_DEFAULT   ((uint32_t)(0x0U << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_7_Pos))

#define AGC_LUT_3_AGC_LEVEL_8_LO_DEFAULT ((uint32_t)(0x0U << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_8_LO_Pos))

/* RF_AGC_LUT3 sub-register settings */
#define AGC_LUT_3_AGC_LEVEL_8_LO_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_AGC_LUT4 bit positions */
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_11_LO_Pos 25
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_11_LO_Mask ((uint32_t)(0x7FU << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_11_LO_Pos))
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_10_Pos 14
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_10_Mask ((uint32_t)(0x7FFU << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_10_Pos))
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_9_Pos 3
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_9_Mask ((uint32_t)(0x7FFU << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_9_Pos))
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_8_HI_Pos 0
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_8_HI_Mask ((uint32_t)(0x7U << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_8_HI_Pos))

/* RF_AGC_LUT4 settings */
#define AGC_LUT_4_AGC_LEVEL_8_HI_DEFAULT ((uint32_t)(0x0U << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_8_HI_Pos))

#define AGC_LUT_4_AGC_LEVEL_9_DEFAULT   ((uint32_t)(0x0U << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_9_Pos))

#define AGC_LUT_4_AGC_LEVEL_10_DEFAULT  ((uint32_t)(0x0U << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_10_Pos))

#define AGC_LUT_4_AGC_LEVEL_11_LO_DEFAULT ((uint32_t)(0x0U << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_11_LO_Pos))

/* RF_REG20 bit positions */
#define RF_REG20_TIMINGS_3_T_DLL_Pos    28
#define RF_REG20_TIMINGS_3_T_DLL_Mask   ((uint32_t)(0xFU << RF_REG20_TIMINGS_3_T_DLL_Pos))
#define RF_REG20_TIMINGS_3_T_PLL_TX_Pos 24
#define RF_REG20_TIMINGS_3_T_PLL_TX_Mask ((uint32_t)(0xFU << RF_REG20_TIMINGS_3_T_PLL_TX_Pos))
#define RF_REG20_TIMINGS_2_T_SUBBAND_TX_Pos 20
#define RF_REG20_TIMINGS_2_T_SUBBAND_TX_Mask ((uint32_t)(0xFU << RF_REG20_TIMINGS_2_T_SUBBAND_TX_Pos))
#define RF_REG20_TIMINGS_2_T_TX_RF_Pos  16
#define RF_REG20_TIMINGS_2_T_TX_RF_Mask ((uint32_t)(0xFU << RF_REG20_TIMINGS_2_T_TX_RF_Pos))
#define RF_REG20_TIMINGS_1_T_GRANULARITY_TX_Pos 12
#define RF_REG20_TIMINGS_1_T_GRANULARITY_TX_Mask ((uint32_t)(0x7U << RF_REG20_TIMINGS_1_T_GRANULARITY_TX_Pos))
#define RF_REG20_TIMINGS_1_T_GRANULARITY_RX_Pos 8
#define RF_REG20_TIMINGS_1_T_GRANULARITY_RX_Mask ((uint32_t)(0x7U << RF_REG20_TIMINGS_1_T_GRANULARITY_RX_Pos))
#define RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_Pos 0
#define RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_Mask ((uint32_t)(0xFU << RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_Pos))

/* RF_REG20 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t AGC_LUT_5_AGC_LEVEL_11_HI_BYTE;
    __IO uint8_t TIMINGS_1_T_GRANULARITY_BYTE;
    __IO uint8_t TIMINGS_2_T_BYTE;     
    __IO uint8_t TIMINGS_3_T_BYTE;     
} RF_REG20_Type;

#define RF_REG20_BASE                   0x40010080
#define RF_REG20                        ((RF_REG20_Type *) RF_REG20_BASE)

/* RF_REG20 settings */
#define AGC_LUT_5_AGC_LEVEL_11_HI_DEFAULT ((uint32_t)(0xFU << RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_Pos))

#define TIMINGS_1_T_GRANULARITY_RX_DEFAULT ((uint32_t)(0x1U << RF_REG20_TIMINGS_1_T_GRANULARITY_RX_Pos))

#define TIMINGS_1_T_GRANULARITY_TX_DEFAULT ((uint32_t)(0x0U << RF_REG20_TIMINGS_1_T_GRANULARITY_TX_Pos))

#define TIMINGS_2_T_TX_RF_DEFAULT       ((uint32_t)(0xFFU << RF_REG20_TIMINGS_2_T_TX_RF_Pos))

#define TIMINGS_2_T_SUBBAND_TX_DEFAULT  ((uint32_t)(0xFU << RF_REG20_TIMINGS_2_T_SUBBAND_TX_Pos))

#define TIMINGS_3_T_PLL_TX_DEFAULT      ((uint32_t)(0x1U << RF_REG20_TIMINGS_3_T_PLL_TX_Pos))

#define TIMINGS_3_T_DLL_DEFAULT         ((uint32_t)(0x0U << RF_REG20_TIMINGS_3_T_DLL_Pos))

/* RF_REG20 sub-register bit positions */
#define RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_BYTE_Pos 0
#define RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_BYTE_Mask ((uint32_t)(0xFU << RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_BYTE_Pos))
#define RF_REG20_TIMINGS_1_T_GRANULARITY_RX_BYTE_Pos 0
#define RF_REG20_TIMINGS_1_T_GRANULARITY_RX_BYTE_Mask ((uint32_t)(0x7U << RF_REG20_TIMINGS_1_T_GRANULARITY_RX_BYTE_Pos))
#define RF_REG20_TIMINGS_1_T_GRANULARITY_TX_BYTE_Pos 4
#define RF_REG20_TIMINGS_1_T_GRANULARITY_TX_BYTE_Mask ((uint32_t)(0x7U << RF_REG20_TIMINGS_1_T_GRANULARITY_TX_BYTE_Pos))
#define RF_REG20_TIMINGS_2_T_TX_RF_BYTE_Pos 0
#define RF_REG20_TIMINGS_2_T_TX_RF_BYTE_Mask ((uint32_t)(0xFU << RF_REG20_TIMINGS_2_T_TX_RF_BYTE_Pos))
#define RF_REG20_TIMINGS_2_T_SUBBAND_TX_BYTE_Pos 4
#define RF_REG20_TIMINGS_2_T_SUBBAND_TX_BYTE_Mask ((uint32_t)(0xFU << RF_REG20_TIMINGS_2_T_SUBBAND_TX_BYTE_Pos))
#define RF_REG20_TIMINGS_3_T_PLL_TX_BYTE_Pos 0
#define RF_REG20_TIMINGS_3_T_PLL_TX_BYTE_Mask ((uint32_t)(0xFU << RF_REG20_TIMINGS_3_T_PLL_TX_BYTE_Pos))
#define RF_REG20_TIMINGS_3_T_DLL_BYTE_Pos 4
#define RF_REG20_TIMINGS_3_T_DLL_BYTE_Mask ((uint32_t)(0xFU << RF_REG20_TIMINGS_3_T_DLL_BYTE_Pos))

/* RF_REG20 sub-register settings */
#define AGC_LUT_5_AGC_LEVEL_11_HI_DEFAULT_BYTE ((uint8_t)(0xFU << RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_BYTE_Pos))

#define TIMINGS_1_T_GRANULARITY_RX_DEFAULT_BYTE ((uint8_t)(0x1U << RF_REG20_TIMINGS_1_T_GRANULARITY_RX_BYTE_Pos))

#define TIMINGS_1_T_GRANULARITY_TX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG20_TIMINGS_1_T_GRANULARITY_TX_BYTE_Pos))

#define TIMINGS_2_T_TX_RF_DEFAULT_BYTE  ((uint8_t)(0xFFU << RF_REG20_TIMINGS_2_T_TX_RF_BYTE_Pos))

#define TIMINGS_2_T_SUBBAND_TX_DEFAULT_BYTE ((uint8_t)(0xFU << RF_REG20_TIMINGS_2_T_SUBBAND_TX_BYTE_Pos))

#define TIMINGS_3_T_PLL_TX_DEFAULT_BYTE ((uint8_t)(0x1U << RF_REG20_TIMINGS_3_T_PLL_TX_BYTE_Pos))

#define TIMINGS_3_T_DLL_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG20_TIMINGS_3_T_DLL_BYTE_Pos))

/* RF_AGC_ATT1 bit positions */
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_AB_LO_Pos 30
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_AB_LO_Mask ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_AB_LO_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_9A_Pos 27
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_9A_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_9A_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_89_Pos 24
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_89_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_89_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_78_Pos 21
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_78_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_78_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_67_Pos 18
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_67_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_67_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_56_Pos 15
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_56_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_56_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_45_Pos 12
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_45_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_45_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_34_Pos 9
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_34_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_34_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_23_Pos 6
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_23_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_23_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_12_Pos 3
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_12_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_12_Pos))
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_01_Pos 0
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_01_Mask ((uint32_t)(0x7U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_01_Pos))

/* RF_AGC_ATT1 settings */
#define AGC_ATT_1_AGC_ATT_01_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_01_Pos))

#define AGC_ATT_1_AGC_ATT_12_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_12_Pos))

#define AGC_ATT_1_AGC_ATT_23_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_23_Pos))

#define AGC_ATT_1_AGC_ATT_34_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_34_Pos))

#define AGC_ATT_1_AGC_ATT_45_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_45_Pos))

#define AGC_ATT_1_AGC_ATT_56_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_56_Pos))

#define AGC_ATT_1_AGC_ATT_67_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_67_Pos))

#define AGC_ATT_1_AGC_ATT_78_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_78_Pos))

#define AGC_ATT_1_AGC_ATT_89_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_89_Pos))

#define AGC_ATT_1_AGC_ATT_9A_DEFAULT    ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_9A_Pos))

#define AGC_ATT_1_AGC_ATT_AB_LO_DEFAULT ((uint32_t)(0x3U << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_AB_LO_Pos))

/* RF_REG22 bit positions */
#define RF_REG22_TIMING_FAST_RX_EN_FAST_RX_TXFILT_Pos 29
#define RF_REG22_TIMING_FAST_RX_EN_FAST_RX_Pos 28
#define RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_Pos 24
#define RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_Mask ((uint32_t)(0xFU << RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_Pos))
#define RF_REG22_TIMINGS_5_T_RX_RF_Pos  20
#define RF_REG22_TIMINGS_5_T_RX_RF_Mask ((uint32_t)(0xFU << RF_REG22_TIMINGS_5_T_RX_RF_Pos))
#define RF_REG22_TIMINGS_5_T_RX_BB_Pos  16
#define RF_REG22_TIMINGS_5_T_RX_BB_Mask ((uint32_t)(0xFU << RF_REG22_TIMINGS_5_T_RX_BB_Pos))
#define RF_REG22_TIMINGS_4_T_SUBBAND_RX_Pos 12
#define RF_REG22_TIMINGS_4_T_SUBBAND_RX_Mask ((uint32_t)(0xFU << RF_REG22_TIMINGS_4_T_SUBBAND_RX_Pos))
#define RF_REG22_TIMINGS_4_T_PLL_RX_Pos 8
#define RF_REG22_TIMINGS_4_T_PLL_RX_Mask ((uint32_t)(0xFU << RF_REG22_TIMINGS_4_T_PLL_RX_Pos))
#define RF_REG22_AGC_ATT_2_AGC_ATT_1DB_Pos 1
#define RF_REG22_AGC_ATT_2_AGC_ATT_AB_HI_Pos 0

/* RF_REG22 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t AGC_ATT_2_AGC_ATT_BYTE;
    __IO uint8_t TIMINGS_4_T_BYTE;     
    __IO uint8_t TIMINGS_5_T_RX_BYTE;  
    __IO uint8_t TIMING_FAST_RX_BYTE;  
} RF_REG22_Type;

#define RF_REG22_BASE                   0x40010088
#define RF_REG22                        ((RF_REG22_Type *) RF_REG22_BASE)

/* RF_REG22 settings */
#define AGC_ATT_2_AGC_ATT_AB_HI_DEFAULT ((uint32_t)(0x0U << RF_REG22_AGC_ATT_2_AGC_ATT_AB_HI_Pos))

#define AGC_ATT_2_AGC_ATT_1DB_DEFAULT   ((uint32_t)(0x0U << RF_REG22_AGC_ATT_2_AGC_ATT_1DB_Pos))

#define TIMINGS_4_T_PLL_RX_DEFAULT      ((uint32_t)(0x0U << RF_REG22_TIMINGS_4_T_PLL_RX_Pos))

#define TIMINGS_4_T_SUBBAND_RX_DEFAULT  ((uint32_t)(0x0U << RF_REG22_TIMINGS_4_T_SUBBAND_RX_Pos))

#define TIMINGS_5_T_RX_BB_DEFAULT       ((uint32_t)(0x0U << RF_REG22_TIMINGS_5_T_RX_BB_Pos))

#define TIMINGS_5_T_RX_RF_DEFAULT       ((uint32_t)(0x0U << RF_REG22_TIMINGS_5_T_RX_RF_Pos))

#define TIMING_FAST_RX_T_RX_FAST_CHP_DEFAULT ((uint32_t)(0x0U << RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_Pos))

#define TIMING_FAST_RX_EN_FAST_RX_DEFAULT ((uint32_t)(0x0U << RF_REG22_TIMING_FAST_RX_EN_FAST_RX_Pos))

#define TIMING_FAST_RX_EN_FAST_RX_TXFILT_DEFAULT ((uint32_t)(0x0U << RF_REG22_TIMING_FAST_RX_EN_FAST_RX_TXFILT_Pos))

/* RF_REG22 sub-register bit positions */
#define RF_REG22_AGC_ATT_2_AGC_ATT_AB_HI_BYTE_Pos 0
#define RF_REG22_AGC_ATT_2_AGC_ATT_1DB_BYTE_Pos 1
#define RF_REG22_TIMINGS_4_T_PLL_RX_BYTE_Pos 0
#define RF_REG22_TIMINGS_4_T_PLL_RX_BYTE_Mask ((uint32_t)(0xFU << RF_REG22_TIMINGS_4_T_PLL_RX_BYTE_Pos))
#define RF_REG22_TIMINGS_4_T_SUBBAND_RX_BYTE_Pos 4
#define RF_REG22_TIMINGS_4_T_SUBBAND_RX_BYTE_Mask ((uint32_t)(0xFU << RF_REG22_TIMINGS_4_T_SUBBAND_RX_BYTE_Pos))
#define RF_REG22_TIMINGS_5_T_RX_BB_BYTE_Pos 0
#define RF_REG22_TIMINGS_5_T_RX_BB_BYTE_Mask ((uint32_t)(0xFU << RF_REG22_TIMINGS_5_T_RX_BB_BYTE_Pos))
#define RF_REG22_TIMINGS_5_T_RX_RF_BYTE_Pos 4
#define RF_REG22_TIMINGS_5_T_RX_RF_BYTE_Mask ((uint32_t)(0xFU << RF_REG22_TIMINGS_5_T_RX_RF_BYTE_Pos))
#define RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_BYTE_Pos 0
#define RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_BYTE_Mask ((uint32_t)(0xFU << RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_BYTE_Pos))
#define RF_REG22_TIMING_FAST_RX_EN_FAST_RX_BYTE_Pos 4
#define RF_REG22_TIMING_FAST_RX_EN_FAST_RX_TXFILT_BYTE_Pos 5

/* RF_REG22 sub-register settings */
#define AGC_ATT_2_AGC_ATT_AB_HI_DEFAULT_BYTE ((uint8_t)(AGC_ATT_2_AGC_ATT_AB_HI_DEFAULT_BITBAND << RF_REG22_AGC_ATT_2_AGC_ATT_AB_HI_BYTE_Pos))

#define AGC_ATT_2_AGC_ATT_1DB_DEFAULT_BYTE ((uint8_t)(AGC_ATT_2_AGC_ATT_1DB_DEFAULT_BITBAND << RF_REG22_AGC_ATT_2_AGC_ATT_1DB_BYTE_Pos))

#define TIMINGS_4_T_PLL_RX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG22_TIMINGS_4_T_PLL_RX_BYTE_Pos))

#define TIMINGS_4_T_SUBBAND_RX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG22_TIMINGS_4_T_SUBBAND_RX_BYTE_Pos))

#define TIMINGS_5_T_RX_BB_DEFAULT_BYTE  ((uint8_t)(0x0U << RF_REG22_TIMINGS_5_T_RX_BB_BYTE_Pos))

#define TIMINGS_5_T_RX_RF_DEFAULT_BYTE  ((uint8_t)(0x0U << RF_REG22_TIMINGS_5_T_RX_RF_BYTE_Pos))

#define TIMING_FAST_RX_T_RX_FAST_CHP_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_BYTE_Pos))

#define TIMING_FAST_RX_EN_FAST_RX_DEFAULT_BYTE ((uint8_t)(TIMING_FAST_RX_EN_FAST_RX_DEFAULT_BITBAND << RF_REG22_TIMING_FAST_RX_EN_FAST_RX_BYTE_Pos))

#define TIMING_FAST_RX_EN_FAST_RX_TXFILT_DEFAULT_BYTE ((uint8_t)(TIMING_FAST_RX_EN_FAST_RX_TXFILT_DEFAULT_BITBAND << RF_REG22_TIMING_FAST_RX_EN_FAST_RX_TXFILT_BYTE_Pos))

/* RF_REG23 bit positions */
#define RF_REG23_BIAS_1_IQ_RXTX_3_Pos   28
#define RF_REG23_BIAS_1_IQ_RXTX_3_Mask  ((uint32_t)(0xFU << RF_REG23_BIAS_1_IQ_RXTX_3_Pos))
#define RF_REG23_BIAS_1_IQ_RXTX_2_Pos   24
#define RF_REG23_BIAS_1_IQ_RXTX_2_Mask  ((uint32_t)(0xFU << RF_REG23_BIAS_1_IQ_RXTX_2_Pos))
#define RF_REG23_BIAS_0_IQ_RXTX_1_Pos   20
#define RF_REG23_BIAS_0_IQ_RXTX_1_Mask  ((uint32_t)(0xFU << RF_REG23_BIAS_0_IQ_RXTX_1_Pos))
#define RF_REG23_BIAS_0_IQ_RXTX_0_Pos   16
#define RF_REG23_BIAS_0_IQ_RXTX_0_Mask  ((uint32_t)(0xFU << RF_REG23_BIAS_0_IQ_RXTX_0_Pos))
#define RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_Pos 12
#define RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_Mask ((uint32_t)(0x7U << RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_Pos))
#define RF_REG23_INTERFACE_CONF_SPI_SELECT_Pos 8
#define RF_REG23_INTERFACE_CONF_SPI_SELECT_Mask ((uint32_t)(0x3U << RF_REG23_INTERFACE_CONF_SPI_SELECT_Pos))
#define RF_REG23_TIMEOUT_EN_RX_TIMEOUT_Pos 7
#define RF_REG23_TIMEOUT_T_TIMEOUT_GR_Pos 4
#define RF_REG23_TIMEOUT_T_TIMEOUT_GR_Mask ((uint32_t)(0x7U << RF_REG23_TIMEOUT_T_TIMEOUT_GR_Pos))
#define RF_REG23_TIMEOUT_T_RX_TIMEOUT_Pos 0
#define RF_REG23_TIMEOUT_T_RX_TIMEOUT_Mask ((uint32_t)(0xFU << RF_REG23_TIMEOUT_T_RX_TIMEOUT_Pos))

/* RF_REG23 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TIMEOUT_BYTE;         
    __IO uint8_t INTERFACE_CONF_BYTE;  
    __IO uint8_t BIAS_0_IQ_RXTX_BYTE;  
    __IO uint8_t BIAS_1_IQ_RXTX_BYTE;  
} RF_REG23_Type;

#define RF_REG23_BASE                   0x4001008C
#define RF_REG23                        ((RF_REG23_Type *) RF_REG23_BASE)

/* RF_REG23 settings */
#define TIMEOUT_T_RX_TIMEOUT_DEFAULT    ((uint32_t)(0x0U << RF_REG23_TIMEOUT_T_RX_TIMEOUT_Pos))

#define TIMEOUT_T_TIMEOUT_GR_DEFAULT    ((uint32_t)(0x0U << RF_REG23_TIMEOUT_T_TIMEOUT_GR_Pos))

#define TIMEOUT_EN_RX_TIMEOUT_DEFAULT   ((uint32_t)(0x0U << RF_REG23_TIMEOUT_EN_RX_TIMEOUT_Pos))

#define INTERFACE_CONF_SPI_SELECT_DEFAULT ((uint32_t)(0x0U << RF_REG23_INTERFACE_CONF_SPI_SELECT_Pos))

#define INTERFACE_CONF_APB_WAIT_STATE_DEFAULT ((uint32_t)(0x0U << RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_Pos))

#define BIAS_0_IQ_RXTX_0_DEFAULT        ((uint32_t)(0x0U << RF_REG23_BIAS_0_IQ_RXTX_0_Pos))

#define BIAS_0_IQ_RXTX_1_DEFAULT        ((uint32_t)(0x0U << RF_REG23_BIAS_0_IQ_RXTX_1_Pos))

#define BIAS_1_IQ_RXTX_2_DEFAULT        ((uint32_t)(0x0U << RF_REG23_BIAS_1_IQ_RXTX_2_Pos))

#define BIAS_1_IQ_RXTX_3_DEFAULT        ((uint32_t)(0x0U << RF_REG23_BIAS_1_IQ_RXTX_3_Pos))

/* RF_REG23 sub-register bit positions */
#define RF_REG23_TIMEOUT_T_RX_TIMEOUT_BYTE_Pos 0
#define RF_REG23_TIMEOUT_T_RX_TIMEOUT_BYTE_Mask ((uint32_t)(0xFU << RF_REG23_TIMEOUT_T_RX_TIMEOUT_BYTE_Pos))
#define RF_REG23_TIMEOUT_T_TIMEOUT_GR_BYTE_Pos 4
#define RF_REG23_TIMEOUT_T_TIMEOUT_GR_BYTE_Mask ((uint32_t)(0x7U << RF_REG23_TIMEOUT_T_TIMEOUT_GR_BYTE_Pos))
#define RF_REG23_TIMEOUT_EN_RX_TIMEOUT_BYTE_Pos 7
#define RF_REG23_INTERFACE_CONF_SPI_SELECT_BYTE_Pos 0
#define RF_REG23_INTERFACE_CONF_SPI_SELECT_BYTE_Mask ((uint32_t)(0x3U << RF_REG23_INTERFACE_CONF_SPI_SELECT_BYTE_Pos))
#define RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_BYTE_Pos 4
#define RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_BYTE_Mask ((uint32_t)(0x7U << RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_BYTE_Pos))
#define RF_REG23_BIAS_0_IQ_RXTX_0_BYTE_Pos 0
#define RF_REG23_BIAS_0_IQ_RXTX_0_BYTE_Mask ((uint32_t)(0xFU << RF_REG23_BIAS_0_IQ_RXTX_0_BYTE_Pos))
#define RF_REG23_BIAS_0_IQ_RXTX_1_BYTE_Pos 4
#define RF_REG23_BIAS_0_IQ_RXTX_1_BYTE_Mask ((uint32_t)(0xFU << RF_REG23_BIAS_0_IQ_RXTX_1_BYTE_Pos))
#define RF_REG23_BIAS_1_IQ_RXTX_2_BYTE_Pos 0
#define RF_REG23_BIAS_1_IQ_RXTX_2_BYTE_Mask ((uint32_t)(0xFU << RF_REG23_BIAS_1_IQ_RXTX_2_BYTE_Pos))
#define RF_REG23_BIAS_1_IQ_RXTX_3_BYTE_Pos 4
#define RF_REG23_BIAS_1_IQ_RXTX_3_BYTE_Mask ((uint32_t)(0xFU << RF_REG23_BIAS_1_IQ_RXTX_3_BYTE_Pos))

/* RF_REG23 sub-register settings */
#define TIMEOUT_T_RX_TIMEOUT_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG23_TIMEOUT_T_RX_TIMEOUT_BYTE_Pos))

#define TIMEOUT_T_TIMEOUT_GR_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG23_TIMEOUT_T_TIMEOUT_GR_BYTE_Pos))

#define TIMEOUT_EN_RX_TIMEOUT_DEFAULT_BYTE ((uint8_t)(TIMEOUT_EN_RX_TIMEOUT_DEFAULT_BITBAND << RF_REG23_TIMEOUT_EN_RX_TIMEOUT_BYTE_Pos))

#define INTERFACE_CONF_SPI_SELECT_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG23_INTERFACE_CONF_SPI_SELECT_BYTE_Pos))

#define INTERFACE_CONF_APB_WAIT_STATE_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_BYTE_Pos))

#define BIAS_0_IQ_RXTX_0_DEFAULT_BYTE   ((uint8_t)(0x0U << RF_REG23_BIAS_0_IQ_RXTX_0_BYTE_Pos))

#define BIAS_0_IQ_RXTX_1_DEFAULT_BYTE   ((uint8_t)(0x0U << RF_REG23_BIAS_0_IQ_RXTX_1_BYTE_Pos))

#define BIAS_1_IQ_RXTX_2_DEFAULT_BYTE   ((uint8_t)(0x0U << RF_REG23_BIAS_1_IQ_RXTX_2_BYTE_Pos))

#define BIAS_1_IQ_RXTX_3_DEFAULT_BYTE   ((uint8_t)(0x0U << RF_REG23_BIAS_1_IQ_RXTX_3_BYTE_Pos))

/* RF_REG24 bit positions */
#define RF_REG24_BIAS_5_IQ_PLL_4_RX_Pos 28
#define RF_REG24_BIAS_5_IQ_PLL_4_RX_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_5_IQ_PLL_4_RX_Pos))
#define RF_REG24_BIAS_5_IQ_PLL_4_TX_Pos 24
#define RF_REG24_BIAS_5_IQ_PLL_4_TX_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_5_IQ_PLL_4_TX_Pos))
#define RF_REG24_BIAS_4_IQ_PLL_2_Pos    20
#define RF_REG24_BIAS_4_IQ_PLL_2_Mask   ((uint32_t)(0xFU << RF_REG24_BIAS_4_IQ_PLL_2_Pos))
#define RF_REG24_BIAS_4_IQ_PLL_1_Pos    16
#define RF_REG24_BIAS_4_IQ_PLL_1_Mask   ((uint32_t)(0xFU << RF_REG24_BIAS_4_IQ_PLL_1_Pos))
#define RF_REG24_BIAS_3_IQ_RXTX_8_Pos   12
#define RF_REG24_BIAS_3_IQ_RXTX_8_Mask  ((uint32_t)(0xFU << RF_REG24_BIAS_3_IQ_RXTX_8_Pos))
#define RF_REG24_BIAS_3_IQ_RXTX_7_Pos   8
#define RF_REG24_BIAS_3_IQ_RXTX_7_Mask  ((uint32_t)(0xFU << RF_REG24_BIAS_3_IQ_RXTX_7_Pos))
#define RF_REG24_BIAS_2_IQ_RXTX_6_Pos   4
#define RF_REG24_BIAS_2_IQ_RXTX_6_Mask  ((uint32_t)(0xFU << RF_REG24_BIAS_2_IQ_RXTX_6_Pos))
#define RF_REG24_BIAS_2_IQ_RXTX_5_Pos   0
#define RF_REG24_BIAS_2_IQ_RXTX_5_Mask  ((uint32_t)(0xFU << RF_REG24_BIAS_2_IQ_RXTX_5_Pos))

/* RF_REG24 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t BIAS_2_IQ_RXTX_BYTE;  
    __IO uint8_t BIAS_3_IQ_RXTX_BYTE;  
    __IO uint8_t BIAS_4_IQ_PLL_BYTE;   
    __IO uint8_t BIAS_5_IQ_PLL_4_BYTE; 
} RF_REG24_Type;

#define RF_REG24_BASE                   0x40010090
#define RF_REG24                        ((RF_REG24_Type *) RF_REG24_BASE)

/* RF_REG24 settings */
#define BIAS_2_IQ_RXTX_5_DEFAULT        ((uint32_t)(0x0U << RF_REG24_BIAS_2_IQ_RXTX_5_Pos))

#define BIAS_2_IQ_RXTX_6_DEFAULT        ((uint32_t)(0x0U << RF_REG24_BIAS_2_IQ_RXTX_6_Pos))

#define BIAS_3_IQ_RXTX_7_DEFAULT        ((uint32_t)(0x0U << RF_REG24_BIAS_3_IQ_RXTX_7_Pos))

#define BIAS_3_IQ_RXTX_8_DEFAULT        ((uint32_t)(0x0U << RF_REG24_BIAS_3_IQ_RXTX_8_Pos))

#define BIAS_4_IQ_PLL_1_DEFAULT         ((uint32_t)(0x0U << RF_REG24_BIAS_4_IQ_PLL_1_Pos))

#define BIAS_4_IQ_PLL_2_DEFAULT         ((uint32_t)(0x0U << RF_REG24_BIAS_4_IQ_PLL_2_Pos))

#define BIAS_5_IQ_PLL_4_TX_DEFAULT      ((uint32_t)(0x0U << RF_REG24_BIAS_5_IQ_PLL_4_TX_Pos))

#define BIAS_5_IQ_PLL_4_RX_DEFAULT      ((uint32_t)(0x0U << RF_REG24_BIAS_5_IQ_PLL_4_RX_Pos))

/* RF_REG24 sub-register bit positions */
#define RF_REG24_BIAS_2_IQ_RXTX_5_BYTE_Pos 0
#define RF_REG24_BIAS_2_IQ_RXTX_5_BYTE_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_2_IQ_RXTX_5_BYTE_Pos))
#define RF_REG24_BIAS_2_IQ_RXTX_6_BYTE_Pos 4
#define RF_REG24_BIAS_2_IQ_RXTX_6_BYTE_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_2_IQ_RXTX_6_BYTE_Pos))
#define RF_REG24_BIAS_3_IQ_RXTX_7_BYTE_Pos 0
#define RF_REG24_BIAS_3_IQ_RXTX_7_BYTE_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_3_IQ_RXTX_7_BYTE_Pos))
#define RF_REG24_BIAS_3_IQ_RXTX_8_BYTE_Pos 4
#define RF_REG24_BIAS_3_IQ_RXTX_8_BYTE_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_3_IQ_RXTX_8_BYTE_Pos))
#define RF_REG24_BIAS_4_IQ_PLL_1_BYTE_Pos 0
#define RF_REG24_BIAS_4_IQ_PLL_1_BYTE_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_4_IQ_PLL_1_BYTE_Pos))
#define RF_REG24_BIAS_4_IQ_PLL_2_BYTE_Pos 4
#define RF_REG24_BIAS_4_IQ_PLL_2_BYTE_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_4_IQ_PLL_2_BYTE_Pos))
#define RF_REG24_BIAS_5_IQ_PLL_4_TX_BYTE_Pos 0
#define RF_REG24_BIAS_5_IQ_PLL_4_TX_BYTE_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_5_IQ_PLL_4_TX_BYTE_Pos))
#define RF_REG24_BIAS_5_IQ_PLL_4_RX_BYTE_Pos 4
#define RF_REG24_BIAS_5_IQ_PLL_4_RX_BYTE_Mask ((uint32_t)(0xFU << RF_REG24_BIAS_5_IQ_PLL_4_RX_BYTE_Pos))

/* RF_REG24 sub-register settings */
#define BIAS_2_IQ_RXTX_5_DEFAULT_BYTE   ((uint8_t)(0x0U << RF_REG24_BIAS_2_IQ_RXTX_5_BYTE_Pos))

#define BIAS_2_IQ_RXTX_6_DEFAULT_BYTE   ((uint8_t)(0x0U << RF_REG24_BIAS_2_IQ_RXTX_6_BYTE_Pos))

#define BIAS_3_IQ_RXTX_7_DEFAULT_BYTE   ((uint8_t)(0x0U << RF_REG24_BIAS_3_IQ_RXTX_7_BYTE_Pos))

#define BIAS_3_IQ_RXTX_8_DEFAULT_BYTE   ((uint8_t)(0x0U << RF_REG24_BIAS_3_IQ_RXTX_8_BYTE_Pos))

#define BIAS_4_IQ_PLL_1_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG24_BIAS_4_IQ_PLL_1_BYTE_Pos))

#define BIAS_4_IQ_PLL_2_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG24_BIAS_4_IQ_PLL_2_BYTE_Pos))

#define BIAS_5_IQ_PLL_4_TX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG24_BIAS_5_IQ_PLL_4_TX_BYTE_Pos))

#define BIAS_5_IQ_PLL_4_RX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG24_BIAS_5_IQ_PLL_4_RX_BYTE_Pos))

/* RF_REG25 bit positions */
#define RF_REG25_BIAS_9_IQ_BB_6_Pos     28
#define RF_REG25_BIAS_9_IQ_BB_6_Mask    ((uint32_t)(0xFU << RF_REG25_BIAS_9_IQ_BB_6_Pos))
#define RF_REG25_BIAS_9_IQ_BB_5_Pos     24
#define RF_REG25_BIAS_9_IQ_BB_5_Mask    ((uint32_t)(0xFU << RF_REG25_BIAS_9_IQ_BB_5_Pos))
#define RF_REG25_BIAS_8_IQ_BB_4_Pos     20
#define RF_REG25_BIAS_8_IQ_BB_4_Mask    ((uint32_t)(0xFU << RF_REG25_BIAS_8_IQ_BB_4_Pos))
#define RF_REG25_BIAS_8_IQ_BB_3_Pos     16
#define RF_REG25_BIAS_8_IQ_BB_3_Mask    ((uint32_t)(0xFU << RF_REG25_BIAS_8_IQ_BB_3_Pos))
#define RF_REG25_BIAS_7_IQ_BB_2_Pos     12
#define RF_REG25_BIAS_7_IQ_BB_2_Mask    ((uint32_t)(0xFU << RF_REG25_BIAS_7_IQ_BB_2_Pos))
#define RF_REG25_BIAS_7_IQ_BB_1_Pos     8
#define RF_REG25_BIAS_7_IQ_BB_1_Mask    ((uint32_t)(0xFU << RF_REG25_BIAS_7_IQ_BB_1_Pos))
#define RF_REG25_BIAS_6_IQ_BB_0_Pos     4
#define RF_REG25_BIAS_6_IQ_BB_0_Mask    ((uint32_t)(0xFU << RF_REG25_BIAS_6_IQ_BB_0_Pos))
#define RF_REG25_BIAS_6_IQ_PLL_3_Pos    0
#define RF_REG25_BIAS_6_IQ_PLL_3_Mask   ((uint32_t)(0xFU << RF_REG25_BIAS_6_IQ_PLL_3_Pos))

/* RF_REG25 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t BIAS_6_IQ_BYTE;       
    __IO uint8_t BIAS_7_IQ_BB_BYTE;    
    __IO uint8_t BIAS_8_IQ_BB_BYTE;    
    __IO uint8_t BIAS_9_IQ_BB_BYTE;    
} RF_REG25_Type;

#define RF_REG25_BASE                   0x40010094
#define RF_REG25                        ((RF_REG25_Type *) RF_REG25_BASE)

/* RF_REG25 settings */
#define BIAS_6_IQ_PLL_3_DEFAULT         ((uint32_t)(0x0U << RF_REG25_BIAS_6_IQ_PLL_3_Pos))

#define BIAS_6_IQ_BB_0_DEFAULT          ((uint32_t)(0x0U << RF_REG25_BIAS_6_IQ_BB_0_Pos))

#define BIAS_7_IQ_BB_1_DEFAULT          ((uint32_t)(0x0U << RF_REG25_BIAS_7_IQ_BB_1_Pos))

#define BIAS_7_IQ_BB_2_DEFAULT          ((uint32_t)(0x0U << RF_REG25_BIAS_7_IQ_BB_2_Pos))

#define BIAS_8_IQ_BB_3_DEFAULT          ((uint32_t)(0x0U << RF_REG25_BIAS_8_IQ_BB_3_Pos))

#define BIAS_8_IQ_BB_4_DEFAULT          ((uint32_t)(0x0U << RF_REG25_BIAS_8_IQ_BB_4_Pos))

#define BIAS_9_IQ_BB_5_DEFAULT          ((uint32_t)(0x0U << RF_REG25_BIAS_9_IQ_BB_5_Pos))

#define BIAS_9_IQ_BB_6_DEFAULT          ((uint32_t)(0x0U << RF_REG25_BIAS_9_IQ_BB_6_Pos))

/* RF_REG25 sub-register bit positions */
#define RF_REG25_BIAS_6_IQ_PLL_3_BYTE_Pos 0
#define RF_REG25_BIAS_6_IQ_PLL_3_BYTE_Mask ((uint32_t)(0xFU << RF_REG25_BIAS_6_IQ_PLL_3_BYTE_Pos))
#define RF_REG25_BIAS_6_IQ_BB_0_BYTE_Pos 4
#define RF_REG25_BIAS_6_IQ_BB_0_BYTE_Mask ((uint32_t)(0xFU << RF_REG25_BIAS_6_IQ_BB_0_BYTE_Pos))
#define RF_REG25_BIAS_7_IQ_BB_1_BYTE_Pos 0
#define RF_REG25_BIAS_7_IQ_BB_1_BYTE_Mask ((uint32_t)(0xFU << RF_REG25_BIAS_7_IQ_BB_1_BYTE_Pos))
#define RF_REG25_BIAS_7_IQ_BB_2_BYTE_Pos 4
#define RF_REG25_BIAS_7_IQ_BB_2_BYTE_Mask ((uint32_t)(0xFU << RF_REG25_BIAS_7_IQ_BB_2_BYTE_Pos))
#define RF_REG25_BIAS_8_IQ_BB_3_BYTE_Pos 0
#define RF_REG25_BIAS_8_IQ_BB_3_BYTE_Mask ((uint32_t)(0xFU << RF_REG25_BIAS_8_IQ_BB_3_BYTE_Pos))
#define RF_REG25_BIAS_8_IQ_BB_4_BYTE_Pos 4
#define RF_REG25_BIAS_8_IQ_BB_4_BYTE_Mask ((uint32_t)(0xFU << RF_REG25_BIAS_8_IQ_BB_4_BYTE_Pos))
#define RF_REG25_BIAS_9_IQ_BB_5_BYTE_Pos 0
#define RF_REG25_BIAS_9_IQ_BB_5_BYTE_Mask ((uint32_t)(0xFU << RF_REG25_BIAS_9_IQ_BB_5_BYTE_Pos))
#define RF_REG25_BIAS_9_IQ_BB_6_BYTE_Pos 4
#define RF_REG25_BIAS_9_IQ_BB_6_BYTE_Mask ((uint32_t)(0xFU << RF_REG25_BIAS_9_IQ_BB_6_BYTE_Pos))

/* RF_REG25 sub-register settings */
#define BIAS_6_IQ_PLL_3_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG25_BIAS_6_IQ_PLL_3_BYTE_Pos))

#define BIAS_6_IQ_BB_0_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG25_BIAS_6_IQ_BB_0_BYTE_Pos))

#define BIAS_7_IQ_BB_1_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG25_BIAS_7_IQ_BB_1_BYTE_Pos))

#define BIAS_7_IQ_BB_2_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG25_BIAS_7_IQ_BB_2_BYTE_Pos))

#define BIAS_8_IQ_BB_3_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG25_BIAS_8_IQ_BB_3_BYTE_Pos))

#define BIAS_8_IQ_BB_4_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG25_BIAS_8_IQ_BB_4_BYTE_Pos))

#define BIAS_9_IQ_BB_5_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG25_BIAS_9_IQ_BB_5_BYTE_Pos))

#define BIAS_9_IQ_BB_6_DEFAULT_BYTE     ((uint8_t)(0x0U << RF_REG25_BIAS_9_IQ_BB_6_BYTE_Pos))

/* RF_REG26 bit positions */
#define RF_REG26_SD_MASH_MASH_ENABLE_Pos 28
#define RF_REG26_SD_MASH_MASH_DITHER_Pos 27
#define RF_REG26_SD_MASH_MASH_ORDER_Pos 25
#define RF_REG26_SD_MASH_MASH_ORDER_Mask ((uint32_t)(0x3U << RF_REG26_SD_MASH_MASH_ORDER_Pos))
#define RF_REG26_SD_MASH_MASH_RSTB_Pos  24
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_3_Pos 20
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_3_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_12_LNA_AGC_BIAS_3_Pos))
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_2_Pos 16
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_2_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_12_LNA_AGC_BIAS_2_Pos))
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_1_Pos 12
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_1_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_11_LNA_AGC_BIAS_1_Pos))
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_0_Pos 8
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_0_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_11_LNA_AGC_BIAS_0_Pos))
#define RF_REG26_BIAS_10_IQ_BB_8_Pos    4
#define RF_REG26_BIAS_10_IQ_BB_8_Mask   ((uint32_t)(0xFU << RF_REG26_BIAS_10_IQ_BB_8_Pos))
#define RF_REG26_BIAS_10_IQ_BB_7_Pos    0
#define RF_REG26_BIAS_10_IQ_BB_7_Mask   ((uint32_t)(0xFU << RF_REG26_BIAS_10_IQ_BB_7_Pos))

/* RF_REG26 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t BIAS_10_IQ_BB_BYTE;   
    __IO uint8_t BIAS_11_LNA_AGC_BIAS_BYTE;
    __IO uint8_t BIAS_12_LNA_AGC_BIAS_BYTE;
    __IO uint8_t SD_MASH_MASH_BYTE;    
} RF_REG26_Type;

#define RF_REG26_BASE                   0x40010098
#define RF_REG26                        ((RF_REG26_Type *) RF_REG26_BASE)

/* RF_REG26 settings */
#define BIAS_10_IQ_BB_7_DEFAULT         ((uint32_t)(0x0U << RF_REG26_BIAS_10_IQ_BB_7_Pos))

#define BIAS_10_IQ_BB_8_DEFAULT         ((uint32_t)(0x0U << RF_REG26_BIAS_10_IQ_BB_8_Pos))

#define BIAS_11_LNA_AGC_BIAS_0_DEFAULT  ((uint32_t)(0x0U << RF_REG26_BIAS_11_LNA_AGC_BIAS_0_Pos))

#define BIAS_11_LNA_AGC_BIAS_1_DEFAULT  ((uint32_t)(0x0U << RF_REG26_BIAS_11_LNA_AGC_BIAS_1_Pos))

#define BIAS_12_LNA_AGC_BIAS_2_DEFAULT  ((uint32_t)(0x0U << RF_REG26_BIAS_12_LNA_AGC_BIAS_2_Pos))

#define BIAS_12_LNA_AGC_BIAS_3_DEFAULT  ((uint32_t)(0x0U << RF_REG26_BIAS_12_LNA_AGC_BIAS_3_Pos))

#define SD_MASH_MASH_RSTB_DEFAULT       ((uint32_t)(0x0U << RF_REG26_SD_MASH_MASH_RSTB_Pos))

#define SD_MASH_MASH_ORDER_DEFAULT      ((uint32_t)(0x0U << RF_REG26_SD_MASH_MASH_ORDER_Pos))

#define SD_MASH_MASH_DITHER_DEFAULT     ((uint32_t)(0x0U << RF_REG26_SD_MASH_MASH_DITHER_Pos))

#define SD_MASH_MASH_ENABLE_DEFAULT     ((uint32_t)(0x0U << RF_REG26_SD_MASH_MASH_ENABLE_Pos))

/* RF_REG26 sub-register bit positions */
#define RF_REG26_BIAS_10_IQ_BB_7_BYTE_Pos 0
#define RF_REG26_BIAS_10_IQ_BB_7_BYTE_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_10_IQ_BB_7_BYTE_Pos))
#define RF_REG26_BIAS_10_IQ_BB_8_BYTE_Pos 4
#define RF_REG26_BIAS_10_IQ_BB_8_BYTE_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_10_IQ_BB_8_BYTE_Pos))
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_0_BYTE_Pos 0
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_0_BYTE_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_11_LNA_AGC_BIAS_0_BYTE_Pos))
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_1_BYTE_Pos 4
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_1_BYTE_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_11_LNA_AGC_BIAS_1_BYTE_Pos))
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_2_BYTE_Pos 0
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_2_BYTE_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_12_LNA_AGC_BIAS_2_BYTE_Pos))
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_3_BYTE_Pos 4
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_3_BYTE_Mask ((uint32_t)(0xFU << RF_REG26_BIAS_12_LNA_AGC_BIAS_3_BYTE_Pos))
#define RF_REG26_SD_MASH_MASH_RSTB_BYTE_Pos 0
#define RF_REG26_SD_MASH_MASH_ORDER_BYTE_Pos 1
#define RF_REG26_SD_MASH_MASH_ORDER_BYTE_Mask ((uint32_t)(0x3U << RF_REG26_SD_MASH_MASH_ORDER_BYTE_Pos))
#define RF_REG26_SD_MASH_MASH_DITHER_BYTE_Pos 3
#define RF_REG26_SD_MASH_MASH_ENABLE_BYTE_Pos 4

/* RF_REG26 sub-register settings */
#define BIAS_10_IQ_BB_7_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG26_BIAS_10_IQ_BB_7_BYTE_Pos))

#define BIAS_10_IQ_BB_8_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG26_BIAS_10_IQ_BB_8_BYTE_Pos))

#define BIAS_11_LNA_AGC_BIAS_0_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG26_BIAS_11_LNA_AGC_BIAS_0_BYTE_Pos))

#define BIAS_11_LNA_AGC_BIAS_1_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG26_BIAS_11_LNA_AGC_BIAS_1_BYTE_Pos))

#define BIAS_12_LNA_AGC_BIAS_2_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG26_BIAS_12_LNA_AGC_BIAS_2_BYTE_Pos))

#define BIAS_12_LNA_AGC_BIAS_3_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG26_BIAS_12_LNA_AGC_BIAS_3_BYTE_Pos))

#define SD_MASH_MASH_RSTB_DEFAULT_BYTE  ((uint8_t)(SD_MASH_MASH_RSTB_DEFAULT_BITBAND << RF_REG26_SD_MASH_MASH_RSTB_BYTE_Pos))

#define SD_MASH_MASH_ORDER_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG26_SD_MASH_MASH_ORDER_BYTE_Pos))

#define SD_MASH_MASH_DITHER_DEFAULT_BYTE ((uint8_t)(SD_MASH_MASH_DITHER_DEFAULT_BITBAND << RF_REG26_SD_MASH_MASH_DITHER_BYTE_Pos))

#define SD_MASH_MASH_ENABLE_DEFAULT_BYTE ((uint8_t)(SD_MASH_MASH_ENABLE_DEFAULT_BITBAND << RF_REG26_SD_MASH_MASH_ENABLE_BYTE_Pos))

/* RF_REG27 bit positions */
#define RF_REG27_CTRL_ADC_ONE_CK_RSSI_PHADC_Pos 31
#define RF_REG27_CTRL_ADC_PHADC_DELLATCH_Pos 29
#define RF_REG27_CTRL_ADC_PHADC_DELLATCH_Mask ((uint32_t)(0x3U << RF_REG27_CTRL_ADC_PHADC_DELLATCH_Pos))
#define RF_REG27_CTRL_ADC_CTRL_ADC_Pos  24
#define RF_REG27_CTRL_ADC_CTRL_ADC_Mask ((uint32_t)(0x1FU << RF_REG27_CTRL_ADC_CTRL_ADC_Pos))
#define RF_REG27_BIAS_EN_2_EN_PTAT_Pos  19
#define RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_Pos 16
#define RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_Mask ((uint32_t)(0x7U << RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_Pos))
#define RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_Pos 12
#define RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_Mask ((uint32_t)(0xFU << RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_Pos))
#define RF_REG27_BIAS_EN_1_EN_BIAS_PLL_Pos 7
#define RF_REG27_BIAS_EN_1_EN_BIAS_PLL_Mask ((uint32_t)(0x1FU << RF_REG27_BIAS_EN_1_EN_BIAS_PLL_Pos))
#define RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_Pos 0
#define RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_Mask ((uint32_t)(0x7FU << RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_Pos))

/* RF_REG27 sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t BIAS_EN_1_EN_BIAS_SHORT;
    __IO uint8_t BIAS_EN_2_EN_BYTE;    
    __IO uint8_t CTRL_ADC_BYTE;        
} RF_REG27_Type;

#define RF_REG27_BASE                   0x4001009C
#define RF_REG27                        ((RF_REG27_Type *) RF_REG27_BASE)

/* RF_REG27 settings */
#define BIAS_EN_1_EN_BIAS_RXTX_DEFAULT  ((uint32_t)(0x0U << RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_Pos))

#define BIAS_EN_1_EN_BIAS_PLL_DEFAULT   ((uint32_t)(0x0U << RF_REG27_BIAS_EN_1_EN_BIAS_PLL_Pos))

#define BIAS_EN_1_EN_BIAS_BB_LO_DEFAULT ((uint32_t)(0x0U << RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_Pos))

#define BIAS_EN_2_EN_BIAS_BB_HI_DEFAULT ((uint32_t)(0x0U << RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_Pos))

#define BIAS_EN_2_EN_PTAT_DEFAULT       ((uint32_t)(0x0U << RF_REG27_BIAS_EN_2_EN_PTAT_Pos))

#define CTRL_ADC_CTRL_ADC_DEFAULT       ((uint32_t)(0x0U << RF_REG27_CTRL_ADC_CTRL_ADC_Pos))

#define CTRL_ADC_PHADC_DELLATCH_DEFAULT ((uint32_t)(0x0U << RF_REG27_CTRL_ADC_PHADC_DELLATCH_Pos))

#define CTRL_ADC_ONE_CK_RSSI_PHADC_DEFAULT ((uint32_t)(0x0U << RF_REG27_CTRL_ADC_ONE_CK_RSSI_PHADC_Pos))

/* RF_REG27 sub-register bit positions */
#define RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_SHORT_Pos 0
#define RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_SHORT_Mask ((uint32_t)(0x7FU << RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_SHORT_Pos))
#define RF_REG27_BIAS_EN_1_EN_BIAS_PLL_SHORT_Pos 7
#define RF_REG27_BIAS_EN_1_EN_BIAS_PLL_SHORT_Mask ((uint32_t)(0x1FU << RF_REG27_BIAS_EN_1_EN_BIAS_PLL_SHORT_Pos))
#define RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_SHORT_Pos 12
#define RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_SHORT_Mask ((uint32_t)(0xFU << RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_SHORT_Pos))
#define RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_BYTE_Pos 0
#define RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_BYTE_Mask ((uint32_t)(0x7U << RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_BYTE_Pos))
#define RF_REG27_BIAS_EN_2_EN_PTAT_BYTE_Pos 3
#define RF_REG27_CTRL_ADC_CTRL_ADC_BYTE_Pos 0
#define RF_REG27_CTRL_ADC_CTRL_ADC_BYTE_Mask ((uint32_t)(0x1FU << RF_REG27_CTRL_ADC_CTRL_ADC_BYTE_Pos))
#define RF_REG27_CTRL_ADC_PHADC_DELLATCH_BYTE_Pos 5
#define RF_REG27_CTRL_ADC_PHADC_DELLATCH_BYTE_Mask ((uint32_t)(0x3U << RF_REG27_CTRL_ADC_PHADC_DELLATCH_BYTE_Pos))
#define RF_REG27_CTRL_ADC_ONE_CK_RSSI_PHADC_BYTE_Pos 7

/* RF_REG27 sub-register settings */
#define BIAS_EN_1_EN_BIAS_RXTX_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_SHORT_Pos))

#define BIAS_EN_1_EN_BIAS_PLL_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG27_BIAS_EN_1_EN_BIAS_PLL_SHORT_Pos))

#define BIAS_EN_1_EN_BIAS_BB_LO_DEFAULT_SHORT ((uint16_t)(0x0U << RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_SHORT_Pos))

#define BIAS_EN_2_EN_BIAS_BB_HI_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_BYTE_Pos))

#define BIAS_EN_2_EN_PTAT_DEFAULT_BYTE  ((uint8_t)(BIAS_EN_2_EN_PTAT_DEFAULT_BITBAND << RF_REG27_BIAS_EN_2_EN_PTAT_BYTE_Pos))

#define CTRL_ADC_CTRL_ADC_DEFAULT_BYTE  ((uint8_t)(0x0U << RF_REG27_CTRL_ADC_CTRL_ADC_BYTE_Pos))

#define CTRL_ADC_PHADC_DELLATCH_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG27_CTRL_ADC_PHADC_DELLATCH_BYTE_Pos))

#define CTRL_ADC_ONE_CK_RSSI_PHADC_DEFAULT_BYTE ((uint8_t)(CTRL_ADC_ONE_CK_RSSI_PHADC_DEFAULT_BITBAND << RF_REG27_CTRL_ADC_ONE_CK_RSSI_PHADC_BYTE_Pos))

/* RF_REG28 bit positions */
#define RF_REG28_CTRL_RX_SWITCH_LP_Pos  31
#define RF_REG28_CTRL_RX_USE_PEAK_DETECTOR_Pos 30
#define RF_REG28_CTRL_RX_START_MIX_ON_CAL_Pos 29
#define RF_REG28_CTRL_RX_CTRL_RX_Pos    24
#define RF_REG28_CTRL_RX_CTRL_RX_Mask   ((uint32_t)(0x1FU << RF_REG28_CTRL_RX_CTRL_RX_Pos))
#define RF_REG28_SWCAP_FSM_SB_CAP_RX_Pos 20
#define RF_REG28_SWCAP_FSM_SB_CAP_RX_Mask ((uint32_t)(0xFU << RF_REG28_SWCAP_FSM_SB_CAP_RX_Pos))
#define RF_REG28_SWCAP_FSM_SB_CAP_TX_Pos 16
#define RF_REG28_SWCAP_FSM_SB_CAP_TX_Mask ((uint32_t)(0xFU << RF_REG28_SWCAP_FSM_SB_CAP_TX_Pos))
#define RF_REG28_DLL_CTRL_CK_LAST_SEL_DELAY_Pos 10
#define RF_REG28_DLL_CTRL_CK_FIRST_SEL_DELAY_Pos 9
#define RF_REG28_DLL_CTRL_CK_EXT_SEL_Pos 8
#define RF_REG28_DLL_CTRL_CK_DIG_EN_Pos 7
#define RF_REG28_DLL_CTRL_CK_TEST_EN_Pos 6
#define RF_REG28_DLL_CTRL_TOO_FAST_ENB_Pos 5
#define RF_REG28_DLL_CTRL_LOCKED_DET_EN_Pos 4
#define RF_REG28_DLL_CTRL_LOCKED_AUTO_CHECK_EN_Pos 3
#define RF_REG28_DLL_CTRL_FAST_ENB_Pos  2
#define RF_REG28_DLL_CTRL_CK_SEL_Pos    0
#define RF_REG28_DLL_CTRL_CK_SEL_Mask   ((uint32_t)(0x3U << RF_REG28_DLL_CTRL_CK_SEL_Pos))

/* RF_REG28 sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t DLL_CTRL_SHORT;       
    __IO uint8_t SWCAP_FSM_SB_CAP_BYTE;
    __IO uint8_t CTRL_RX_BYTE;         
} RF_REG28_Type;

#define RF_REG28_BASE                   0x400100A0
#define RF_REG28                        ((RF_REG28_Type *) RF_REG28_BASE)

/* RF_REG28 settings */
#define DLL_CTRL_CK_SEL_DEFAULT         ((uint32_t)(0x2U << RF_REG28_DLL_CTRL_CK_SEL_Pos))

#define DLL_CTRL_FAST_ENB_DEFAULT       ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_FAST_ENB_Pos))

#define DLL_CTRL_LOCKED_AUTO_CHECK_EN_DEFAULT ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_LOCKED_AUTO_CHECK_EN_Pos))

#define DLL_CTRL_LOCKED_DET_EN_DEFAULT  ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_LOCKED_DET_EN_Pos))

#define DLL_CTRL_TOO_FAST_ENB_DEFAULT   ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_TOO_FAST_ENB_Pos))

#define DLL_CTRL_CK_TEST_EN_DEFAULT     ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_CK_TEST_EN_Pos))

#define DLL_CTRL_CK_DIG_EN_DEFAULT      ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_CK_DIG_EN_Pos))

#define DLL_CTRL_CK_EXT_SEL_DEFAULT     ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_CK_EXT_SEL_Pos))

#define DLL_CTRL_CK_FIRST_SEL_DELAY_DEFAULT ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_CK_FIRST_SEL_DELAY_Pos))

#define DLL_CTRL_CK_LAST_SEL_DELAY_DEFAULT ((uint32_t)(0x0U << RF_REG28_DLL_CTRL_CK_LAST_SEL_DELAY_Pos))

#define SWCAP_FSM_SB_CAP_TX_DEFAULT     ((uint32_t)(0x0U << RF_REG28_SWCAP_FSM_SB_CAP_TX_Pos))

#define SWCAP_FSM_SB_CAP_RX_DEFAULT     ((uint32_t)(0x0U << RF_REG28_SWCAP_FSM_SB_CAP_RX_Pos))

#define CTRL_RX_CTRL_RX_DEFAULT         ((uint32_t)(0x0U << RF_REG28_CTRL_RX_CTRL_RX_Pos))

#define CTRL_RX_START_MIX_ON_CAL_DEFAULT ((uint32_t)(0x0U << RF_REG28_CTRL_RX_START_MIX_ON_CAL_Pos))

#define CTRL_RX_USE_PEAK_DETECTOR_DEFAULT ((uint32_t)(0x0U << RF_REG28_CTRL_RX_USE_PEAK_DETECTOR_Pos))

#define CTRL_RX_SWITCH_LP_DEFAULT       ((uint32_t)(0x0U << RF_REG28_CTRL_RX_SWITCH_LP_Pos))

/* RF_REG28 sub-register bit positions */
#define RF_REG28_DLL_CTRL_CK_SEL_SHORT_Pos 0
#define RF_REG28_DLL_CTRL_CK_SEL_SHORT_Mask ((uint32_t)(0x3U << RF_REG28_DLL_CTRL_CK_SEL_SHORT_Pos))
#define RF_REG28_DLL_CTRL_FAST_ENB_SHORT_Pos 2
#define RF_REG28_DLL_CTRL_LOCKED_AUTO_CHECK_EN_SHORT_Pos 3
#define RF_REG28_DLL_CTRL_LOCKED_DET_EN_SHORT_Pos 4
#define RF_REG28_DLL_CTRL_TOO_FAST_ENB_SHORT_Pos 5
#define RF_REG28_DLL_CTRL_CK_TEST_EN_SHORT_Pos 6
#define RF_REG28_DLL_CTRL_CK_DIG_EN_SHORT_Pos 7
#define RF_REG28_DLL_CTRL_CK_EXT_SEL_SHORT_Pos 8
#define RF_REG28_DLL_CTRL_CK_FIRST_SEL_DELAY_SHORT_Pos 9
#define RF_REG28_DLL_CTRL_CK_LAST_SEL_DELAY_SHORT_Pos 10
#define RF_REG28_SWCAP_FSM_SB_CAP_TX_BYTE_Pos 0
#define RF_REG28_SWCAP_FSM_SB_CAP_TX_BYTE_Mask ((uint32_t)(0xFU << RF_REG28_SWCAP_FSM_SB_CAP_TX_BYTE_Pos))
#define RF_REG28_SWCAP_FSM_SB_CAP_RX_BYTE_Pos 4
#define RF_REG28_SWCAP_FSM_SB_CAP_RX_BYTE_Mask ((uint32_t)(0xFU << RF_REG28_SWCAP_FSM_SB_CAP_RX_BYTE_Pos))
#define RF_REG28_CTRL_RX_CTRL_RX_BYTE_Pos 0
#define RF_REG28_CTRL_RX_CTRL_RX_BYTE_Mask ((uint32_t)(0x1FU << RF_REG28_CTRL_RX_CTRL_RX_BYTE_Pos))
#define RF_REG28_CTRL_RX_START_MIX_ON_CAL_BYTE_Pos 5
#define RF_REG28_CTRL_RX_USE_PEAK_DETECTOR_BYTE_Pos 6
#define RF_REG28_CTRL_RX_SWITCH_LP_BYTE_Pos 7

/* RF_REG28 sub-register settings */
#define DLL_CTRL_CK_SEL_DEFAULT_SHORT   ((uint16_t)(0x2U << RF_REG28_DLL_CTRL_CK_SEL_SHORT_Pos))

#define DLL_CTRL_FAST_ENB_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_FAST_ENB_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_FAST_ENB_SHORT_Pos))

#define DLL_CTRL_LOCKED_AUTO_CHECK_EN_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_LOCKED_AUTO_CHECK_EN_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_LOCKED_AUTO_CHECK_EN_SHORT_Pos))

#define DLL_CTRL_LOCKED_DET_EN_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_LOCKED_DET_EN_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_LOCKED_DET_EN_SHORT_Pos))

#define DLL_CTRL_TOO_FAST_ENB_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_TOO_FAST_ENB_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_TOO_FAST_ENB_SHORT_Pos))

#define DLL_CTRL_CK_TEST_EN_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_CK_TEST_EN_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_TEST_EN_SHORT_Pos))

#define DLL_CTRL_CK_DIG_EN_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_CK_DIG_EN_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_DIG_EN_SHORT_Pos))

#define DLL_CTRL_CK_EXT_SEL_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_CK_EXT_SEL_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_EXT_SEL_SHORT_Pos))

#define DLL_CTRL_CK_FIRST_SEL_DELAY_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_CK_FIRST_SEL_DELAY_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_FIRST_SEL_DELAY_SHORT_Pos))

#define DLL_CTRL_CK_LAST_SEL_DELAY_DEFAULT_SHORT ((uint16_t)(DLL_CTRL_CK_LAST_SEL_DELAY_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_LAST_SEL_DELAY_SHORT_Pos))

#define SWCAP_FSM_SB_CAP_TX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG28_SWCAP_FSM_SB_CAP_TX_BYTE_Pos))

#define SWCAP_FSM_SB_CAP_RX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG28_SWCAP_FSM_SB_CAP_RX_BYTE_Pos))

#define CTRL_RX_CTRL_RX_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG28_CTRL_RX_CTRL_RX_BYTE_Pos))

#define CTRL_RX_START_MIX_ON_CAL_DEFAULT_BYTE ((uint8_t)(CTRL_RX_START_MIX_ON_CAL_DEFAULT_BITBAND << RF_REG28_CTRL_RX_START_MIX_ON_CAL_BYTE_Pos))

#define CTRL_RX_USE_PEAK_DETECTOR_DEFAULT_BYTE ((uint8_t)(CTRL_RX_USE_PEAK_DETECTOR_DEFAULT_BITBAND << RF_REG28_CTRL_RX_USE_PEAK_DETECTOR_BYTE_Pos))

#define CTRL_RX_SWITCH_LP_DEFAULT_BYTE  ((uint8_t)(CTRL_RX_SWITCH_LP_DEFAULT_BITBAND << RF_REG28_CTRL_RX_SWITCH_LP_BYTE_Pos))

/* RF_PLL_CTRL bit positions */
#define RF_PLL_CTRL_XTAL_TRIM_XTAL_TRIM_Pos 24
#define RF_PLL_CTRL_XTAL_TRIM_XTAL_TRIM_Mask ((uint32_t)(0xFFU << RF_PLL_CTRL_XTAL_TRIM_XTAL_TRIM_Pos))
#define RF_PLL_CTRL_PLL_CTRL_2_PLL_RX_48MEG_Pos 20
#define RF_PLL_CTRL_PLL_CTRL_2_SWCAP_TX_SAME_RX_Pos 19
#define RF_PLL_CTRL_PLL_CTRL_2_SWCAP_FSM_Pos 18
#define RF_PLL_CTRL_PLL_CTRL_2_DLL_RSTB_Pos 17
#define RF_PLL_CTRL_PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_Pos 16
#define RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_Pos 13
#define RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_Mask ((uint32_t)(0x7U << RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_Pos))
#define RF_PLL_CTRL_PLL_CTRL_1_SUB_SEL_OFFS_EN_Pos 12
#define RF_PLL_CTRL_PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_Pos 11
#define RF_PLL_CTRL_PLL_CTRL_1_VCODIV_CLK_TEST_EN_Pos 10
#define RF_PLL_CTRL_PLL_CTRL_1_EN_LOW_CHP_BIAS_Pos 9
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_DEAD_ZONE_EN_Pos 8
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_Pos 6
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_Mask ((uint32_t)(0x3U << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_Pos))
#define RF_PLL_CTRL_PLL_CTRL_1_HIGH_BW_FILTER_EN_Pos 5
#define RF_PLL_CTRL_PLL_CTRL_1_FAST_CHP_EN_Pos 4
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_Pos 2
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_Mask ((uint32_t)(0x3U << RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_Pos))
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CMC_EN_Pos 1
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_EN_Pos 0

/* RF_PLL_CTRL sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t PLL_CTRL_1_SHORT;     
    __IO uint8_t PLL_CTRL_2_BYTE;      
    __IO uint8_t XTAL_TRIM_XTAL_TRIM_BYTE;
} RF_PLL_CTRL_Type;

#define RF_PLL_CTRL_BASE                0x400100A4
#define RF_PLL_CTRL                     ((RF_PLL_CTRL_Type *) RF_PLL_CTRL_BASE)

/* RF_PLL_CTRL settings */
#define PLL_CTRL_1_CHP_CURR_OFFSET_EN_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_EN_Pos))

#define PLL_CTRL_1_CHP_CMC_EN_DEFAULT   ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_CHP_CMC_EN_Pos))

#define PLL_CTRL_1_CHP_MODE_TRIM_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_Pos))

#define PLL_CTRL_1_FAST_CHP_EN_DEFAULT  ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_FAST_CHP_EN_Pos))

#define PLL_CTRL_1_HIGH_BW_FILTER_EN_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_HIGH_BW_FILTER_EN_Pos))

#define PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_Pos))

#define PLL_CTRL_1_CHP_DEAD_ZONE_EN_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_CHP_DEAD_ZONE_EN_Pos))

#define PLL_CTRL_1_EN_LOW_CHP_BIAS_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_EN_LOW_CHP_BIAS_Pos))

#define PLL_CTRL_1_VCODIV_CLK_TEST_EN_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_VCODIV_CLK_TEST_EN_Pos))

#define PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_Pos))

#define PLL_CTRL_1_SUB_SEL_OFFS_EN_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_SUB_SEL_OFFS_EN_Pos))

#define PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_Pos))

#define PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_Pos))

#define PLL_CTRL_2_DLL_RSTB_DEFAULT     ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_2_DLL_RSTB_Pos))

#define PLL_CTRL_2_SWCAP_FSM_DEFAULT    ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_2_SWCAP_FSM_Pos))

#define PLL_CTRL_2_SWCAP_TX_SAME_RX_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_2_SWCAP_TX_SAME_RX_Pos))

#define PLL_CTRL_2_PLL_RX_48MEG_DEFAULT ((uint32_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_2_PLL_RX_48MEG_Pos))

#define XTAL_TRIM_XTAL_TRIM_DEFAULT     ((uint32_t)(0x80U << RF_PLL_CTRL_XTAL_TRIM_XTAL_TRIM_Pos))

/* RF_PLL_CTRL sub-register bit positions */
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_EN_SHORT_Pos 0
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CMC_EN_SHORT_Pos 1
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_SHORT_Pos 2
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_SHORT_Mask ((uint32_t)(0x3U << RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_SHORT_Pos))
#define RF_PLL_CTRL_PLL_CTRL_1_FAST_CHP_EN_SHORT_Pos 4
#define RF_PLL_CTRL_PLL_CTRL_1_HIGH_BW_FILTER_EN_SHORT_Pos 5
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_SHORT_Pos 6
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_SHORT_Mask ((uint32_t)(0x3U << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_SHORT_Pos))
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_DEAD_ZONE_EN_SHORT_Pos 8
#define RF_PLL_CTRL_PLL_CTRL_1_EN_LOW_CHP_BIAS_SHORT_Pos 9
#define RF_PLL_CTRL_PLL_CTRL_1_VCODIV_CLK_TEST_EN_SHORT_Pos 10
#define RF_PLL_CTRL_PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_SHORT_Pos 11
#define RF_PLL_CTRL_PLL_CTRL_1_SUB_SEL_OFFS_EN_SHORT_Pos 12
#define RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_SHORT_Pos 13
#define RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_SHORT_Mask ((uint32_t)(0x7U << RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_SHORT_Pos))
#define RF_PLL_CTRL_PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_BYTE_Pos 0
#define RF_PLL_CTRL_PLL_CTRL_2_DLL_RSTB_BYTE_Pos 1
#define RF_PLL_CTRL_PLL_CTRL_2_SWCAP_FSM_BYTE_Pos 2
#define RF_PLL_CTRL_PLL_CTRL_2_SWCAP_TX_SAME_RX_BYTE_Pos 3
#define RF_PLL_CTRL_PLL_CTRL_2_PLL_RX_48MEG_BYTE_Pos 4

/* RF_PLL_CTRL sub-register settings */
#define PLL_CTRL_1_CHP_CURR_OFFSET_EN_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_CHP_CURR_OFFSET_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_EN_SHORT_Pos))

#define PLL_CTRL_1_CHP_CMC_EN_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_CHP_CMC_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_CHP_CMC_EN_SHORT_Pos))

#define PLL_CTRL_1_CHP_MODE_TRIM_DEFAULT_SHORT ((uint16_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_SHORT_Pos))

#define PLL_CTRL_1_FAST_CHP_EN_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_FAST_CHP_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_FAST_CHP_EN_SHORT_Pos))

#define PLL_CTRL_1_HIGH_BW_FILTER_EN_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_HIGH_BW_FILTER_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_HIGH_BW_FILTER_EN_SHORT_Pos))

#define PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_DEFAULT_SHORT ((uint16_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_SHORT_Pos))

#define PLL_CTRL_1_CHP_DEAD_ZONE_EN_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_CHP_DEAD_ZONE_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_CHP_DEAD_ZONE_EN_SHORT_Pos))

#define PLL_CTRL_1_EN_LOW_CHP_BIAS_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_EN_LOW_CHP_BIAS_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_EN_LOW_CHP_BIAS_SHORT_Pos))

#define PLL_CTRL_1_VCODIV_CLK_TEST_EN_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_VCODIV_CLK_TEST_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_VCODIV_CLK_TEST_EN_SHORT_Pos))

#define PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_SHORT_Pos))

#define PLL_CTRL_1_SUB_SEL_OFFS_EN_DEFAULT_SHORT ((uint16_t)(PLL_CTRL_1_SUB_SEL_OFFS_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_SUB_SEL_OFFS_EN_SHORT_Pos))

#define PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_DEFAULT_SHORT ((uint16_t)(0x0U << RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_SHORT_Pos))

#define PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_DEFAULT_BYTE ((uint8_t)(PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_BYTE_Pos))

#define PLL_CTRL_2_DLL_RSTB_DEFAULT_BYTE ((uint8_t)(PLL_CTRL_2_DLL_RSTB_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_DLL_RSTB_BYTE_Pos))

#define PLL_CTRL_2_SWCAP_FSM_DEFAULT_BYTE ((uint8_t)(PLL_CTRL_2_SWCAP_FSM_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_SWCAP_FSM_BYTE_Pos))

#define PLL_CTRL_2_SWCAP_TX_SAME_RX_DEFAULT_BYTE ((uint8_t)(PLL_CTRL_2_SWCAP_TX_SAME_RX_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_SWCAP_TX_SAME_RX_BYTE_Pos))

#define PLL_CTRL_2_PLL_RX_48MEG_DEFAULT_BYTE ((uint8_t)(PLL_CTRL_2_PLL_RX_48MEG_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_PLL_RX_48MEG_BYTE_Pos))

#define XTAL_TRIM_XTAL_TRIM_DEFAULT_BYTE ((uint8_t)0x80U)

/* RF_REG2A bit positions */
#define RF_REG2A_ENABLES_SEPARATE_PPA_CASC_Pos 28
#define RF_REG2A_ENABLES_EN_RXTX_Pos    22
#define RF_REG2A_ENABLES_EN_RXTX_Mask   ((uint32_t)(0x3FU << RF_REG2A_ENABLES_EN_RXTX_Pos))
#define RF_REG2A_ENABLES_EN_BB_Pos      16
#define RF_REG2A_ENABLES_EN_BB_Mask     ((uint32_t)(0x3FU << RF_REG2A_ENABLES_EN_BB_Pos))
#define RF_REG2A_RSSI_TUN_RSSI_TUN_GAIN_Pos 13
#define RF_REG2A_RSSI_TUN_RSSI_TUN_GAIN_Mask ((uint32_t)(0x7U << RF_REG2A_RSSI_TUN_RSSI_TUN_GAIN_Pos))
#define RF_REG2A_RSSI_TUN_RSSI_ODD_OFFSET_Pos 8
#define RF_REG2A_RSSI_TUN_RSSI_ODD_OFFSET_Mask ((uint32_t)(0x1FU << RF_REG2A_RSSI_TUN_RSSI_ODD_OFFSET_Pos))
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_Pos 4
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_Mask ((uint32_t)(0xFU << RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_Pos))
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_Pos 0
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_Mask ((uint32_t)(0xFU << RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_Pos))

/* RF_REG2A sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t RSSI_TUN_RSSI_EVEN_BYTE;
         uint8_t RESERVED0[1];
    __IO uint16_t ENABLES_SHORT;        
} RF_REG2A_Type;

#define RF_REG2A_BASE                   0x400100A8
#define RF_REG2A                        ((RF_REG2A_Type *) RF_REG2A_BASE)

/* RF_REG2A settings */
#define RSSI_TUN_RSSI_EVEN_MIN_DEFAULT  ((uint32_t)(0x7U << RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_Pos))

#define RSSI_TUN_RSSI_EVEN_MAX_DEFAULT  ((uint32_t)(0x7U << RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_Pos))

#define RSSI_TUN_RSSI_ODD_OFFSET_DEFAULT ((uint32_t)(0x0U << RF_REG2A_RSSI_TUN_RSSI_ODD_OFFSET_Pos))

#define RSSI_TUN_RSSI_TUN_GAIN_DEFAULT  ((uint32_t)(0x3U << RF_REG2A_RSSI_TUN_RSSI_TUN_GAIN_Pos))

#define ENABLES_EN_BB_DEFAULT           ((uint32_t)(0x0U << RF_REG2A_ENABLES_EN_BB_Pos))

#define ENABLES_EN_RXTX_DEFAULT         ((uint32_t)(0x0U << RF_REG2A_ENABLES_EN_RXTX_Pos))

#define ENABLES_SEPARATE_PPA_CASC_DEFAULT ((uint32_t)(0x0U << RF_REG2A_ENABLES_SEPARATE_PPA_CASC_Pos))

/* RF_REG2A sub-register bit positions */
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_BYTE_Pos 0
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_BYTE_Mask ((uint32_t)(0xFU << RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_BYTE_Pos))
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_BYTE_Pos 4
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_BYTE_Mask ((uint32_t)(0xFU << RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_BYTE_Pos))
#define RF_REG2A_ENABLES_EN_BB_SHORT_Pos 0
#define RF_REG2A_ENABLES_EN_BB_SHORT_Mask ((uint32_t)(0x3FU << RF_REG2A_ENABLES_EN_BB_SHORT_Pos))
#define RF_REG2A_ENABLES_EN_RXTX_SHORT_Pos 6
#define RF_REG2A_ENABLES_EN_RXTX_SHORT_Mask ((uint32_t)(0x3FU << RF_REG2A_ENABLES_EN_RXTX_SHORT_Pos))
#define RF_REG2A_ENABLES_SEPARATE_PPA_CASC_SHORT_Pos 12

/* RF_REG2A sub-register settings */
#define RSSI_TUN_RSSI_EVEN_MIN_DEFAULT_BYTE ((uint8_t)(0x7U << RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_BYTE_Pos))

#define RSSI_TUN_RSSI_EVEN_MAX_DEFAULT_BYTE ((uint8_t)(0x7U << RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_BYTE_Pos))

#define ENABLES_EN_BB_DEFAULT_SHORT     ((uint16_t)(0x0U << RF_REG2A_ENABLES_EN_BB_SHORT_Pos))

#define ENABLES_EN_RXTX_DEFAULT_SHORT   ((uint16_t)(0x0U << RF_REG2A_ENABLES_EN_RXTX_SHORT_Pos))

#define ENABLES_SEPARATE_PPA_CASC_DEFAULT_SHORT ((uint16_t)(ENABLES_SEPARATE_PPA_CASC_DEFAULT_BITBAND << RF_REG2A_ENABLES_SEPARATE_PPA_CASC_SHORT_Pos))

/* RF_XTAL_CTRL bit positions */
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_Pos 28
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_Mask ((uint32_t)(0xFU << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_Pos))
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_Pos 24
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_Mask ((uint32_t)(0xFU << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_Pos))
#define RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_Pos 22
#define RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_Mask ((uint32_t)(0x3U << RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_Pos))
#define RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_LOW_Pos 20
#define RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_LOW_Mask ((uint32_t)(0x3U << RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_LOW_Pos))
#define RF_XTAL_CTRL_XTAL_CTRL_XTAL_CTRL_BYPASS_Pos 18
#define RF_XTAL_CTRL_XTAL_CTRL_DIG_CLK_IN_SEL_Pos 17
#define RF_XTAL_CTRL_XTAL_CTRL_XO_EN_B_REG_Pos 16
#define RF_XTAL_CTRL_XTAL_CTRL_XTAL_CKDIV_Pos 14
#define RF_XTAL_CTRL_XTAL_CTRL_XTAL_CKDIV_Mask ((uint32_t)(0x3U << RF_XTAL_CTRL_XTAL_CTRL_XTAL_CKDIV_Pos))
#define RF_XTAL_CTRL_XTAL_CTRL_CLK_OUT_EN_B_Pos 13
#define RF_XTAL_CTRL_XTAL_CTRL_REG_VALUE_SEL_Pos 12
#define RF_XTAL_CTRL_XTAL_CTRL_AFTERSTARTUP_CURR_SEL_Pos 10
#define RF_XTAL_CTRL_XTAL_CTRL_AFTERSTARTUP_CURR_SEL_Mask ((uint32_t)(0x3U << RF_XTAL_CTRL_XTAL_CTRL_AFTERSTARTUP_CURR_SEL_Pos))
#define RF_XTAL_CTRL_XTAL_CTRL_STARTUP_CURR_SEL_Pos 8
#define RF_XTAL_CTRL_XTAL_CTRL_STARTUP_CURR_SEL_Mask ((uint32_t)(0x3U << RF_XTAL_CTRL_XTAL_CTRL_STARTUP_CURR_SEL_Pos))
#define RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_DIG_Pos 7
#define RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_PLL_Pos 6
#define RF_XTAL_CTRL_XTAL_CTRL_FORCE_CLK_READY_Pos 5
#define RF_XTAL_CTRL_XTAL_CTRL_CLK_DIG_EN_B_Pos 4
#define RF_XTAL_CTRL_XTAL_CTRL_BUFF_EN_B_Pos 3
#define RF_XTAL_CTRL_XTAL_CTRL_HP_MODE_Pos 2
#define RF_XTAL_CTRL_XTAL_CTRL_LP_MODE_Pos 1
#define RF_XTAL_CTRL_XTAL_CTRL_EXT_CLK_MODE_Pos 0

/* RF_XTAL_CTRL sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t XTAL_CTRL_XO_THR_BYTE;
} RF_XTAL_CTRL_Type;

#define RF_XTAL_CTRL_BASE               0x400100AC
#define RF_XTAL_CTRL                    ((RF_XTAL_CTRL_Type *) RF_XTAL_CTRL_BASE)

/* RF_XTAL_CTRL settings */
#define XTAL_CTRL_EXT_CLK_MODE_DEFAULT  ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_EXT_CLK_MODE_Pos))

#define XTAL_CTRL_LP_MODE_DEFAULT       ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_LP_MODE_Pos))

#define XTAL_CTRL_HP_MODE_DEFAULT       ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_HP_MODE_Pos))

#define XTAL_CTRL_BUFF_EN_B_DEFAULT     ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_BUFF_EN_B_Pos))

#define XTAL_CTRL_CLK_DIG_EN_B_DEFAULT  ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_CLK_DIG_EN_B_Pos))

#define XTAL_CTRL_FORCE_CLK_READY_DEFAULT ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_FORCE_CLK_READY_Pos))

#define XTAL_CTRL_INV_CLK_PLL_DEFAULT   ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_PLL_Pos))

#define XTAL_CTRL_INV_CLK_DIG_DEFAULT   ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_DIG_Pos))

#define XTAL_CTRL_STARTUP_CURR_SEL_DEFAULT ((uint32_t)(0x1U << RF_XTAL_CTRL_XTAL_CTRL_STARTUP_CURR_SEL_Pos))

#define XTAL_CTRL_AFTERSTARTUP_CURR_SEL_DEFAULT ((uint32_t)(0x1U << RF_XTAL_CTRL_XTAL_CTRL_AFTERSTARTUP_CURR_SEL_Pos))

#define XTAL_CTRL_REG_VALUE_SEL_EXTERNAL ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_REG_VALUE_SEL_Pos))
#define XTAL_CTRL_REG_VALUE_SEL_INTERNAL ((uint32_t)(0x1U << RF_XTAL_CTRL_XTAL_CTRL_REG_VALUE_SEL_Pos))

#define XTAL_CTRL_CLK_OUT_EN_B_DEFAULT  ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_CLK_OUT_EN_B_Pos))

#define XTAL_CTRL_XTAL_CKDIV_DEFAULT    ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_XTAL_CKDIV_Pos))

#define XTAL_CTRL_ENABLE_OSCILLATOR     ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_XO_EN_B_REG_Pos))
#define XTAL_CTRL_DISABLE_OSCILLATOR    ((uint32_t)(0x1U << RF_XTAL_CTRL_XTAL_CTRL_XO_EN_B_REG_Pos))

#define XTAL_CTRL_DIG_CLK_IN_SEL_DEFAULT ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_DIG_CLK_IN_SEL_Pos))

#define XTAL_CTRL_XTAL_CTRL_BYPASS_DEFAULT ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_XTAL_CTRL_BYPASS_Pos))

#define XTAL_CTRL_XO_A_S_CURR_SEL_LOW_DEFAULT ((uint32_t)(0x0U << RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_LOW_Pos))

#define XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_DEFAULT ((uint32_t)(0x2U << RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_Pos))

#define XTAL_CTRL_XO_THR_LOW_DEFAULT    ((uint32_t)(0x3U << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_Pos))

#define XTAL_CTRL_XO_THR_HIGH_DEFAULT   ((uint32_t)(0xCU << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_Pos))

/* RF_XTAL_CTRL sub-register bit positions */
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_BYTE_Pos 0
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_BYTE_Mask ((uint32_t)(0xFU << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_BYTE_Pos))
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_BYTE_Pos 4
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_BYTE_Mask ((uint32_t)(0xFU << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_BYTE_Pos))

/* RF_XTAL_CTRL sub-register settings */
#define XTAL_CTRL_XO_THR_LOW_DEFAULT_BYTE ((uint8_t)(0x3U << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_BYTE_Pos))

#define XTAL_CTRL_XO_THR_HIGH_DEFAULT_BYTE ((uint8_t)(0xCU << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_BYTE_Pos))

/* RF_REG2C bit positions */
#define RF_REG2C_SUBBAND_OFFSET_SB_OFFSET_Pos 24
#define RF_REG2C_SUBBAND_OFFSET_SB_OFFSET_Mask ((uint32_t)(0xFFU << RF_REG2C_SUBBAND_OFFSET_SB_OFFSET_Pos))
#define RF_REG2C_SWCAP_LIM_SB_MAX_VAL_Pos 20
#define RF_REG2C_SWCAP_LIM_SB_MAX_VAL_Mask ((uint32_t)(0xFU << RF_REG2C_SWCAP_LIM_SB_MAX_VAL_Pos))
#define RF_REG2C_SWCAP_LIM_SB_MIN_VAL_Pos 16
#define RF_REG2C_SWCAP_LIM_SB_MIN_VAL_Mask ((uint32_t)(0xFU << RF_REG2C_SWCAP_LIM_SB_MIN_VAL_Pos))
#define RF_REG2C_SUBBAND_CONF_SB_FLL_MODE_Pos 15
#define RF_REG2C_SUBBAND_CONF_SB_INV_BAND_Pos 14
#define RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_Pos 12
#define RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_Mask ((uint32_t)(0x3U << RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_Pos))
#define RF_REG2C_SUBBAND_CONF_SB_WAIT_T_Pos 10
#define RF_REG2C_SUBBAND_CONF_SB_WAIT_T_Mask ((uint32_t)(0x3U << RF_REG2C_SUBBAND_CONF_SB_WAIT_T_Pos))
#define RF_REG2C_SUBBAND_CONF_SB_MODE_Pos 8
#define RF_REG2C_SUBBAND_CONF_SB_MODE_Mask ((uint32_t)(0x3U << RF_REG2C_SUBBAND_CONF_SB_MODE_Pos))
#define RF_REG2C_PA_CONF_SW_CN_Pos      4
#define RF_REG2C_PA_CONF_SW_CN_Mask     ((uint32_t)(0x3U << RF_REG2C_PA_CONF_SW_CN_Pos))
#define RF_REG2C_PA_CONF_TX_SWITCHPA_Pos 3
#define RF_REG2C_PA_CONF_TX_0DBM_Pos    2
#define RF_REG2C_PA_CONF_CTRL_PA_Pos    0
#define RF_REG2C_PA_CONF_CTRL_PA_Mask   ((uint32_t)(0x3U << RF_REG2C_PA_CONF_CTRL_PA_Pos))

/* RF_REG2C sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t PA_CONF_BYTE;         
    __IO uint8_t SUBBAND_CONF_SB_BYTE; 
    __IO uint8_t SWCAP_LIM_SB_BYTE;    
    __IO uint8_t SUBBAND_OFFSET_SB_OFFSET_BYTE;
} RF_REG2C_Type;

#define RF_REG2C_BASE                   0x400100B0
#define RF_REG2C                        ((RF_REG2C_Type *) RF_REG2C_BASE)

/* RF_REG2C settings */
#define PA_CONF_CTRL_PA_DEFAULT         ((uint32_t)(0x0U << RF_REG2C_PA_CONF_CTRL_PA_Pos))

#define PA_CONF_TX_0DBM_DEFAULT         ((uint32_t)(0x0U << RF_REG2C_PA_CONF_TX_0DBM_Pos))

#define PA_CONF_TX_SWITCHPA_DEFAULT     ((uint32_t)(0x0U << RF_REG2C_PA_CONF_TX_SWITCHPA_Pos))

#define PA_CONF_SW_CN_DEFAULT           ((uint32_t)(0x0U << RF_REG2C_PA_CONF_SW_CN_Pos))

#define SUBBAND_CONF_SB_MODE_DEFAULT    ((uint32_t)(0x0U << RF_REG2C_SUBBAND_CONF_SB_MODE_Pos))

#define SUBBAND_CONF_SB_WAIT_T_DEFAULT  ((uint32_t)(0x0U << RF_REG2C_SUBBAND_CONF_SB_WAIT_T_Pos))

#define SUBBAND_CONF_SB_FREQ_CNT_DEFAULT ((uint32_t)(0x0U << RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_Pos))

#define SUBBAND_CONF_SB_INV_BAND_DEFAULT ((uint32_t)(0x0U << RF_REG2C_SUBBAND_CONF_SB_INV_BAND_Pos))

#define SUBBAND_CONF_SB_FLL_MODE_DEFAULT ((uint32_t)(0x0U << RF_REG2C_SUBBAND_CONF_SB_FLL_MODE_Pos))

#define SWCAP_LIM_SB_MIN_VAL_DEFAULT    ((uint32_t)(0x0U << RF_REG2C_SWCAP_LIM_SB_MIN_VAL_Pos))

#define SWCAP_LIM_SB_MAX_VAL_DEFAULT    ((uint32_t)(0x0U << RF_REG2C_SWCAP_LIM_SB_MAX_VAL_Pos))

#define SUBBAND_OFFSET_SB_OFFSET_DEFAULT ((uint32_t)(0x0U << RF_REG2C_SUBBAND_OFFSET_SB_OFFSET_Pos))

/* RF_REG2C sub-register bit positions */
#define RF_REG2C_PA_CONF_CTRL_PA_BYTE_Pos 0
#define RF_REG2C_PA_CONF_CTRL_PA_BYTE_Mask ((uint32_t)(0x3U << RF_REG2C_PA_CONF_CTRL_PA_BYTE_Pos))
#define RF_REG2C_PA_CONF_TX_0DBM_BYTE_Pos 2
#define RF_REG2C_PA_CONF_TX_SWITCHPA_BYTE_Pos 3
#define RF_REG2C_PA_CONF_SW_CN_BYTE_Pos 4
#define RF_REG2C_PA_CONF_SW_CN_BYTE_Mask ((uint32_t)(0x3U << RF_REG2C_PA_CONF_SW_CN_BYTE_Pos))
#define RF_REG2C_SUBBAND_CONF_SB_MODE_BYTE_Pos 0
#define RF_REG2C_SUBBAND_CONF_SB_MODE_BYTE_Mask ((uint32_t)(0x3U << RF_REG2C_SUBBAND_CONF_SB_MODE_BYTE_Pos))
#define RF_REG2C_SUBBAND_CONF_SB_WAIT_T_BYTE_Pos 2
#define RF_REG2C_SUBBAND_CONF_SB_WAIT_T_BYTE_Mask ((uint32_t)(0x3U << RF_REG2C_SUBBAND_CONF_SB_WAIT_T_BYTE_Pos))
#define RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_BYTE_Pos 4
#define RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_BYTE_Mask ((uint32_t)(0x3U << RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_BYTE_Pos))
#define RF_REG2C_SUBBAND_CONF_SB_INV_BAND_BYTE_Pos 6
#define RF_REG2C_SUBBAND_CONF_SB_FLL_MODE_BYTE_Pos 7
#define RF_REG2C_SWCAP_LIM_SB_MIN_VAL_BYTE_Pos 0
#define RF_REG2C_SWCAP_LIM_SB_MIN_VAL_BYTE_Mask ((uint32_t)(0xFU << RF_REG2C_SWCAP_LIM_SB_MIN_VAL_BYTE_Pos))
#define RF_REG2C_SWCAP_LIM_SB_MAX_VAL_BYTE_Pos 4
#define RF_REG2C_SWCAP_LIM_SB_MAX_VAL_BYTE_Mask ((uint32_t)(0xFU << RF_REG2C_SWCAP_LIM_SB_MAX_VAL_BYTE_Pos))

/* RF_REG2C sub-register settings */
#define PA_CONF_CTRL_PA_DEFAULT_BYTE    ((uint8_t)(0x0U << RF_REG2C_PA_CONF_CTRL_PA_BYTE_Pos))

#define PA_CONF_TX_0DBM_DEFAULT_BYTE    ((uint8_t)(PA_CONF_TX_0DBM_DEFAULT_BITBAND << RF_REG2C_PA_CONF_TX_0DBM_BYTE_Pos))

#define PA_CONF_TX_SWITCHPA_DEFAULT_BYTE ((uint8_t)(PA_CONF_TX_SWITCHPA_DEFAULT_BITBAND << RF_REG2C_PA_CONF_TX_SWITCHPA_BYTE_Pos))

#define PA_CONF_SW_CN_DEFAULT_BYTE      ((uint8_t)(0x0U << RF_REG2C_PA_CONF_SW_CN_BYTE_Pos))

#define SUBBAND_CONF_SB_MODE_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2C_SUBBAND_CONF_SB_MODE_BYTE_Pos))

#define SUBBAND_CONF_SB_WAIT_T_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2C_SUBBAND_CONF_SB_WAIT_T_BYTE_Pos))

#define SUBBAND_CONF_SB_FREQ_CNT_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_BYTE_Pos))

#define SUBBAND_CONF_SB_INV_BAND_DEFAULT_BYTE ((uint8_t)(SUBBAND_CONF_SB_INV_BAND_DEFAULT_BITBAND << RF_REG2C_SUBBAND_CONF_SB_INV_BAND_BYTE_Pos))

#define SUBBAND_CONF_SB_FLL_MODE_DEFAULT_BYTE ((uint8_t)(SUBBAND_CONF_SB_FLL_MODE_DEFAULT_BITBAND << RF_REG2C_SUBBAND_CONF_SB_FLL_MODE_BYTE_Pos))

#define SWCAP_LIM_SB_MIN_VAL_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2C_SWCAP_LIM_SB_MIN_VAL_BYTE_Pos))

#define SWCAP_LIM_SB_MAX_VAL_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2C_SWCAP_LIM_SB_MAX_VAL_BYTE_Pos))

#define SUBBAND_OFFSET_SB_OFFSET_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_REG2D bit positions */
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_EN_Pos 31
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_Pos 28
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_Mask ((uint32_t)(0x7U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_Pos))
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_Pos 24
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_Mask ((uint32_t)(0x7U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_Pos))
#define RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_PLL_TX_Pos 23
#define RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_DIG_TX_Pos 22
#define RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_Pos 20
#define RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_Mask ((uint32_t)(0x3U << RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_Pos))
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_Pos 17
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_Mask ((uint32_t)(0x3U << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_Pos))
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_Pos 16
#define RF_REG2D_PA_RAMPUP_FULL_PA_RAMPUP_Pos 15
#define RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_Pos 12
#define RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_Mask ((uint32_t)(0x7U << RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_Pos))
#define RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_Pos 10
#define RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_Mask ((uint32_t)(0x3U << RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_Pos))
#define RF_REG2D_PA_RAMPUP_EN_PA_RAMPDOWN_Pos 9
#define RF_REG2D_PA_RAMPUP_EN_PA_RAMPUP_Pos 8
#define RF_REG2D_MISC_SPARES_Pos        3
#define RF_REG2D_MISC_SPARES_Mask       ((uint32_t)(0x1FU << RF_REG2D_MISC_SPARES_Pos))
#define RF_REG2D_MISC_RSSI_PRE_ATT_Pos  1
#define RF_REG2D_MISC_RSSI_PRE_ATT_Mask ((uint32_t)(0x3U << RF_REG2D_MISC_RSSI_PRE_ATT_Pos))
#define RF_REG2D_MISC_XTAL_LOW_CLK_READY_TH_EN_Pos 0

/* RF_REG2D sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t MISC_BYTE;            
    __IO uint8_t PA_RAMPUP_BYTE;       
    __IO uint8_t PLL_CONF_TX_NRX_BYTE; 
    __IO uint8_t SUBBAND_CORR_SUBBAND_CORR_BYTE;
} RF_REG2D_Type;

#define RF_REG2D_BASE                   0x400100B4
#define RF_REG2D                        ((RF_REG2D_Type *) RF_REG2D_BASE)

/* RF_REG2D settings */
#define MISC_XTAL_LOW_CLK_READY_TH_EN_DEFAULT ((uint32_t)(0x0U << RF_REG2D_MISC_XTAL_LOW_CLK_READY_TH_EN_Pos))

#define MISC_RSSI_PRE_ATT_DEFAULT       ((uint32_t)(0x0U << RF_REG2D_MISC_RSSI_PRE_ATT_Pos))

#define MISC_SPARES_DEFAULT             ((uint32_t)(0x0U << RF_REG2D_MISC_SPARES_Pos))

#define PA_RAMPUP_EN_PA_RAMPUP_DEFAULT  ((uint32_t)(0x0U << RF_REG2D_PA_RAMPUP_EN_PA_RAMPUP_Pos))

#define PA_RAMPUP_EN_PA_RAMPDOWN_DEFAULT ((uint32_t)(0x0U << RF_REG2D_PA_RAMPUP_EN_PA_RAMPDOWN_Pos))

#define PA_RAMPUP_TAU_PA_RAMPUP_DEFAULT ((uint32_t)(0x0U << RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_Pos))

#define PA_RAMPUP_DEL_PA_RAMPUP_DEFAULT ((uint32_t)(0x0U << RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_Pos))

#define PA_RAMPUP_FULL_PA_RAMPUP_DEFAULT ((uint32_t)(0x0U << RF_REG2D_PA_RAMPUP_FULL_PA_RAMPUP_Pos))

#define PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_DEFAULT ((uint32_t)(0x0U << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_Pos))

#define PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_DEFAULT ((uint32_t)(0x0U << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_Pos))

#define PLL_CONF_TX_NRX_CK_SEL_TX_DEFAULT ((uint32_t)(0x3U << RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_Pos))

#define PLL_CONF_TX_NRX_INV_CLK_DIG_TX_DEFAULT ((uint32_t)(0x0U << RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_DIG_TX_Pos))

#define PLL_CONF_TX_NRX_INV_CLK_PLL_TX_DEFAULT ((uint32_t)(0x0U << RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_PLL_TX_Pos))

#define SUBBAND_CORR_SUBBAND_CORR_TX_DEFAULT ((uint32_t)(0x0U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_Pos))

#define SUBBAND_CORR_SUBBAND_CORR_RX_DEFAULT ((uint32_t)(0x0U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_Pos))

#define SUBBAND_CORR_SUBBAND_CORR_EN_DEFAULT ((uint32_t)(0x0U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_EN_Pos))

/* RF_REG2D sub-register bit positions */
#define RF_REG2D_MISC_XTAL_LOW_CLK_READY_TH_EN_BYTE_Pos 0
#define RF_REG2D_MISC_RSSI_PRE_ATT_BYTE_Pos 1
#define RF_REG2D_MISC_RSSI_PRE_ATT_BYTE_Mask ((uint32_t)(0x3U << RF_REG2D_MISC_RSSI_PRE_ATT_BYTE_Pos))
#define RF_REG2D_MISC_SPARES_BYTE_Pos   3
#define RF_REG2D_MISC_SPARES_BYTE_Mask  ((uint32_t)(0x1FU << RF_REG2D_MISC_SPARES_BYTE_Pos))
#define RF_REG2D_PA_RAMPUP_EN_PA_RAMPUP_BYTE_Pos 0
#define RF_REG2D_PA_RAMPUP_EN_PA_RAMPDOWN_BYTE_Pos 1
#define RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_BYTE_Pos 2
#define RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_BYTE_Mask ((uint32_t)(0x3U << RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_BYTE_Pos))
#define RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_BYTE_Pos 4
#define RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_BYTE_Mask ((uint32_t)(0x7U << RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_BYTE_Pos))
#define RF_REG2D_PA_RAMPUP_FULL_PA_RAMPUP_BYTE_Pos 7
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_BYTE_Pos 0
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_BYTE_Pos 1
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_BYTE_Mask ((uint32_t)(0x3U << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_BYTE_Pos))
#define RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_BYTE_Pos 4
#define RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_BYTE_Mask ((uint32_t)(0x3U << RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_BYTE_Pos))
#define RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_DIG_TX_BYTE_Pos 6
#define RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_PLL_TX_BYTE_Pos 7
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_BYTE_Pos 0
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_BYTE_Mask ((uint32_t)(0x7U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_BYTE_Pos))
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_BYTE_Pos 4
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_BYTE_Mask ((uint32_t)(0x7U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_BYTE_Pos))
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_EN_BYTE_Pos 7

/* RF_REG2D sub-register settings */
#define MISC_XTAL_LOW_CLK_READY_TH_EN_DEFAULT_BYTE ((uint8_t)(MISC_XTAL_LOW_CLK_READY_TH_EN_DEFAULT_BITBAND << RF_REG2D_MISC_XTAL_LOW_CLK_READY_TH_EN_BYTE_Pos))

#define MISC_RSSI_PRE_ATT_DEFAULT_BYTE  ((uint8_t)(0x0U << RF_REG2D_MISC_RSSI_PRE_ATT_BYTE_Pos))

#define MISC_SPARES_DEFAULT_BYTE        ((uint8_t)(0x0U << RF_REG2D_MISC_SPARES_BYTE_Pos))

#define PA_RAMPUP_EN_PA_RAMPUP_DEFAULT_BYTE ((uint8_t)(PA_RAMPUP_EN_PA_RAMPUP_DEFAULT_BITBAND << RF_REG2D_PA_RAMPUP_EN_PA_RAMPUP_BYTE_Pos))

#define PA_RAMPUP_EN_PA_RAMPDOWN_DEFAULT_BYTE ((uint8_t)(PA_RAMPUP_EN_PA_RAMPDOWN_DEFAULT_BITBAND << RF_REG2D_PA_RAMPUP_EN_PA_RAMPDOWN_BYTE_Pos))

#define PA_RAMPUP_TAU_PA_RAMPUP_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_BYTE_Pos))

#define PA_RAMPUP_DEL_PA_RAMPUP_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_BYTE_Pos))

#define PA_RAMPUP_FULL_PA_RAMPUP_DEFAULT_BYTE ((uint8_t)(PA_RAMPUP_FULL_PA_RAMPUP_DEFAULT_BITBAND << RF_REG2D_PA_RAMPUP_FULL_PA_RAMPUP_BYTE_Pos))

#define PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_DEFAULT_BYTE ((uint8_t)(PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_DEFAULT_BITBAND << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_BYTE_Pos))

#define PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_BYTE_Pos))

#define PLL_CONF_TX_NRX_CK_SEL_TX_DEFAULT_BYTE ((uint8_t)(0x3U << RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_BYTE_Pos))

#define PLL_CONF_TX_NRX_INV_CLK_DIG_TX_DEFAULT_BYTE ((uint8_t)(PLL_CONF_TX_NRX_INV_CLK_DIG_TX_DEFAULT_BITBAND << RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_DIG_TX_BYTE_Pos))

#define PLL_CONF_TX_NRX_INV_CLK_PLL_TX_DEFAULT_BYTE ((uint8_t)(PLL_CONF_TX_NRX_INV_CLK_PLL_TX_DEFAULT_BITBAND << RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_PLL_TX_BYTE_Pos))

#define SUBBAND_CORR_SUBBAND_CORR_TX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_BYTE_Pos))

#define SUBBAND_CORR_SUBBAND_CORR_RX_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_BYTE_Pos))

#define SUBBAND_CORR_SUBBAND_CORR_EN_DEFAULT_BYTE ((uint8_t)(SUBBAND_CORR_SUBBAND_CORR_EN_DEFAULT_BITBAND << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_EN_BYTE_Pos))

/* RF_REG2E bit positions */
#define RF_REG2E_RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_Pos 24
#define RF_REG2E_RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_Mask ((uint32_t)(0xFFU << RF_REG2E_RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_Pos))
#define RF_REG2E_RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_Pos 16
#define RF_REG2E_RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_Mask ((uint32_t)(0xFFU << RF_REG2E_RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_Pos))
#define RF_REG2E_DEMOD_CTRL_EN_DELLINE_SYNC_DET_Pos 14
#define RF_REG2E_DEMOD_CTRL_RSSI_DET_FILT_Pos 13
#define RF_REG2E_DEMOD_CTRL_EN_FAST_CLK_RECOV_Pos 12
#define RF_REG2E_DEMOD_CTRL_EN_MIN_MAX_MF_Pos 11
#define RF_REG2E_DEMOD_CTRL_EN_PRE_SYNC_Pos 10
#define RF_REG2E_DEMOD_CTRL_BLOCK_RSSI_DET_Pos 9
#define RF_REG2E_DEMOD_CTRL_EARLY_FINE_RECOV_Pos 8
#define RF_REG2E_RSSI_DETECT_RSSI_DET_CR_LEN_Pos 6
#define RF_REG2E_RSSI_DETECT_RSSI_DET_CR_LEN_Mask ((uint32_t)(0x3U << RF_REG2E_RSSI_DETECT_RSSI_DET_CR_LEN_Pos))
#define RF_REG2E_RSSI_DETECT_RSSI_DET_WAIT_Pos 4
#define RF_REG2E_RSSI_DETECT_RSSI_DET_WAIT_Mask ((uint32_t)(0x3U << RF_REG2E_RSSI_DETECT_RSSI_DET_WAIT_Pos))
#define RF_REG2E_RSSI_DETECT_RSSI_DET_DIFF_LL_Pos 2
#define RF_REG2E_RSSI_DETECT_RSSI_DET_DIFF_LL_Mask ((uint32_t)(0x3U << RF_REG2E_RSSI_DETECT_RSSI_DET_DIFF_LL_Pos))
#define RF_REG2E_RSSI_DETECT_EN_ABS_RSSI_DETECT_Pos 1
#define RF_REG2E_RSSI_DETECT_EN_DIFF_RSSI_DETECT_Pos 0

/* RF_REG2E sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t DEMOD_CTRL_BYTE;      
    __IO uint8_t RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_BYTE;
    __IO uint8_t RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_BYTE;
} RF_REG2E_Type;

#define RF_REG2E_BASE                   0x400100B8
#define RF_REG2E                        ((RF_REG2E_Type *) RF_REG2E_BASE)

/* RF_REG2E settings */
#define RSSI_DETECT_EN_DIFF_RSSI_DETECT_DEFAULT ((uint32_t)(0x0U << RF_REG2E_RSSI_DETECT_EN_DIFF_RSSI_DETECT_Pos))

#define RSSI_DETECT_EN_ABS_RSSI_DETECT_DEFAULT ((uint32_t)(0x0U << RF_REG2E_RSSI_DETECT_EN_ABS_RSSI_DETECT_Pos))

#define RSSI_DETECT_RSSI_DET_DIFF_LL_DEFAULT ((uint32_t)(0x0U << RF_REG2E_RSSI_DETECT_RSSI_DET_DIFF_LL_Pos))

#define RSSI_DETECT_RSSI_DET_WAIT_DEFAULT ((uint32_t)(0x0U << RF_REG2E_RSSI_DETECT_RSSI_DET_WAIT_Pos))

#define RSSI_DETECT_RSSI_DET_CR_LEN_DEFAULT ((uint32_t)(0x0U << RF_REG2E_RSSI_DETECT_RSSI_DET_CR_LEN_Pos))

#define DEMOD_CTRL_EARLY_FINE_RECOV_DEFAULT ((uint32_t)(0x0U << RF_REG2E_DEMOD_CTRL_EARLY_FINE_RECOV_Pos))

#define DEMOD_CTRL_BLOCK_RSSI_DET_DEFAULT ((uint32_t)(0x0U << RF_REG2E_DEMOD_CTRL_BLOCK_RSSI_DET_Pos))

#define DEMOD_CTRL_EN_PRE_SYNC_DEFAULT  ((uint32_t)(0x0U << RF_REG2E_DEMOD_CTRL_EN_PRE_SYNC_Pos))

#define DEMOD_CTRL_EN_MIN_MAX_MF_DEFAULT ((uint32_t)(0x0U << RF_REG2E_DEMOD_CTRL_EN_MIN_MAX_MF_Pos))

#define DEMOD_CTRL_EN_FAST_CLK_RECOV_DEFAULT ((uint32_t)(0x0U << RF_REG2E_DEMOD_CTRL_EN_FAST_CLK_RECOV_Pos))

#define DEMOD_CTRL_RSSI_DET_FILT_DEFAULT ((uint32_t)(0x0U << RF_REG2E_DEMOD_CTRL_RSSI_DET_FILT_Pos))

#define DEMOD_CTRL_EN_DELLINE_SYNC_DET_DEFAULT ((uint32_t)(0x0U << RF_REG2E_DEMOD_CTRL_EN_DELLINE_SYNC_DET_Pos))

#define RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_DEFAULT ((uint32_t)(0x0U << RF_REG2E_RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_Pos))

#define RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_DEFAULT ((uint32_t)(0x0U << RF_REG2E_RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_Pos))

/* RF_REG2E sub-register bit positions */
#define RF_REG2E_DEMOD_CTRL_EARLY_FINE_RECOV_BYTE_Pos 0
#define RF_REG2E_DEMOD_CTRL_BLOCK_RSSI_DET_BYTE_Pos 1
#define RF_REG2E_DEMOD_CTRL_EN_PRE_SYNC_BYTE_Pos 2
#define RF_REG2E_DEMOD_CTRL_EN_MIN_MAX_MF_BYTE_Pos 3
#define RF_REG2E_DEMOD_CTRL_EN_FAST_CLK_RECOV_BYTE_Pos 4
#define RF_REG2E_DEMOD_CTRL_RSSI_DET_FILT_BYTE_Pos 5
#define RF_REG2E_DEMOD_CTRL_EN_DELLINE_SYNC_DET_BYTE_Pos 6

/* RF_REG2E sub-register settings */
#define DEMOD_CTRL_EARLY_FINE_RECOV_DEFAULT_BYTE ((uint8_t)(DEMOD_CTRL_EARLY_FINE_RECOV_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EARLY_FINE_RECOV_BYTE_Pos))

#define DEMOD_CTRL_BLOCK_RSSI_DET_DEFAULT_BYTE ((uint8_t)(DEMOD_CTRL_BLOCK_RSSI_DET_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_BLOCK_RSSI_DET_BYTE_Pos))

#define DEMOD_CTRL_EN_PRE_SYNC_DEFAULT_BYTE ((uint8_t)(DEMOD_CTRL_EN_PRE_SYNC_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EN_PRE_SYNC_BYTE_Pos))

#define DEMOD_CTRL_EN_MIN_MAX_MF_DEFAULT_BYTE ((uint8_t)(DEMOD_CTRL_EN_MIN_MAX_MF_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EN_MIN_MAX_MF_BYTE_Pos))

#define DEMOD_CTRL_EN_FAST_CLK_RECOV_DEFAULT_BYTE ((uint8_t)(DEMOD_CTRL_EN_FAST_CLK_RECOV_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EN_FAST_CLK_RECOV_BYTE_Pos))

#define DEMOD_CTRL_RSSI_DET_FILT_DEFAULT_BYTE ((uint8_t)(DEMOD_CTRL_RSSI_DET_FILT_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_RSSI_DET_FILT_BYTE_Pos))

#define DEMOD_CTRL_EN_DELLINE_SYNC_DET_DEFAULT_BYTE ((uint8_t)(DEMOD_CTRL_EN_DELLINE_SYNC_DET_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EN_DELLINE_SYNC_DET_BYTE_Pos))

#define RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_DEFAULT_BYTE ((uint8_t)0x0U)

#define RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_DEFAULT_BYTE ((uint8_t)0x0U)

/* RF_REG2F bit positions */
#define RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos 24
#define RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Mask ((uint32_t)(0x7U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))
#define RF_REG2F_PADS_PE_DS_GPIO_DS_Pos 22
#define RF_REG2F_PADS_PE_DS_GPIO_PE_Pos 21
#define RF_REG2F_PADS_PE_DS_NRESET_PE_Pos 20
#define RF_REG2F_PADS_PE_DS_SPI_MISO_PE_Pos 19
#define RF_REG2F_PADS_PE_DS_SPI_MOSI_PE_Pos 18
#define RF_REG2F_PADS_PE_DS_SPI_SCLK_PE_Pos 17
#define RF_REG2F_PADS_PE_DS_SPI_CS_N_PE_Pos 16
#define RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_Pos 14
#define RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_Mask ((uint32_t)(0x3U << RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_Pos))
#define RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_Pos 12
#define RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_Mask ((uint32_t)(0x3U << RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_Pos))
#define RF_REG2F_SUBBAND_FLL_SB_FLL_PH_4_N8_Pos 11
#define RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_Pos 8
#define RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_Mask ((uint32_t)(0x7U << RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_Pos))
#define RF_REG2F_SYNC_WORD_CORR_EN_SYNC_WORD_CORR_Pos 7
#define RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_Pos 0
#define RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_Mask ((uint32_t)(0x3FU << RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_Pos))

/* RF_REG2F sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t SYNC_WORD_CORR_BYTE;  
    __IO uint8_t SUBBAND_FLL_SB_FLL_BYTE;
    __IO uint8_t PADS_PE_DS_BYTE;      
    __IO uint8_t CK_DIV_1_6_CK_DIV_1_6_BYTE;
} RF_REG2F_Type;

#define RF_REG2F_BASE                   0x400100BC
#define RF_REG2F                        ((RF_REG2F_Type *) RF_REG2F_BASE)

/* RF_REG2F settings */
#define SYNC_WORD_CORR_SYNC_WORD_BIAS_DEFAULT ((uint32_t)(0x0U << RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_Pos))

#define SYNC_WORD_CORR_EN_SYNC_WORD_CORR_DEFAULT ((uint32_t)(0x0U << RF_REG2F_SYNC_WORD_CORR_EN_SYNC_WORD_CORR_Pos))

#define SUBBAND_FLL_SB_FLL_WAIT_DEFAULT ((uint32_t)(0x0U << RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_Pos))

#define SUBBAND_FLL_SB_FLL_PH_4_N8_DEFAULT ((uint32_t)(0x0U << RF_REG2F_SUBBAND_FLL_SB_FLL_PH_4_N8_Pos))

#define SUBBAND_FLL_SB_FLL_CIC_TAU_DEFAULT ((uint32_t)(0x0U << RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_Pos))

#define SUBBAND_FLL_SB_FLL_DITHER_DEFAULT ((uint32_t)(0x0U << RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_Pos))

#define PADS_PE_DS_SPI_CS_N_PE_DEFAULT  ((uint32_t)(0x0U << RF_REG2F_PADS_PE_DS_SPI_CS_N_PE_Pos))

#define PADS_PE_DS_SPI_SCLK_PE_DEFAULT  ((uint32_t)(0x0U << RF_REG2F_PADS_PE_DS_SPI_SCLK_PE_Pos))

#define PADS_PE_DS_SPI_MOSI_PE_DEFAULT  ((uint32_t)(0x0U << RF_REG2F_PADS_PE_DS_SPI_MOSI_PE_Pos))

#define PADS_PE_DS_SPI_MISO_PE_DEFAULT  ((uint32_t)(0x0U << RF_REG2F_PADS_PE_DS_SPI_MISO_PE_Pos))

#define PADS_PE_DS_NRESET_PE_DEFAULT    ((uint32_t)(0x0U << RF_REG2F_PADS_PE_DS_NRESET_PE_Pos))

#define PADS_PE_DS_GPIO_PE_DEFAULT      ((uint32_t)(0x0U << RF_REG2F_PADS_PE_DS_GPIO_PE_Pos))

#define PADS_PE_DS_GPIO_DS_DEFAULT      ((uint32_t)(0x0U << RF_REG2F_PADS_PE_DS_GPIO_DS_Pos))

#define CK_DIV_1_6_NO_CLOCK             ((uint32_t)(0x0U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))
#define CK_DIV_1_6_PRESCALE_1           ((uint32_t)(0x1U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))
#define CK_DIV_1_6_PRESCALE_2           ((uint32_t)(0x2U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))
#define CK_DIV_1_6_PRESCALE_3           ((uint32_t)(0x3U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))
#define CK_DIV_1_6_PRESCALE_4           ((uint32_t)(0x4U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))
#define CK_DIV_1_6_PRESCALE_5           ((uint32_t)(0x5U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))
#define CK_DIV_1_6_PRESCALE_6           ((uint32_t)(0x6U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))
#define CK_DIV_1_6_PRESCALE_7           ((uint32_t)(0x7U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_Pos))

/* RF_REG2F sub-register bit positions */
#define RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_BYTE_Pos 0
#define RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_BYTE_Mask ((uint32_t)(0x3FU << RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_BYTE_Pos))
#define RF_REG2F_SYNC_WORD_CORR_EN_SYNC_WORD_CORR_BYTE_Pos 7
#define RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_BYTE_Pos 0
#define RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_BYTE_Mask ((uint32_t)(0x7U << RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_BYTE_Pos))
#define RF_REG2F_SUBBAND_FLL_SB_FLL_PH_4_N8_BYTE_Pos 3
#define RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_BYTE_Pos 4
#define RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_BYTE_Mask ((uint32_t)(0x3U << RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_BYTE_Pos))
#define RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_BYTE_Pos 6
#define RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_BYTE_Mask ((uint32_t)(0x3U << RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_BYTE_Pos))
#define RF_REG2F_PADS_PE_DS_SPI_CS_N_PE_BYTE_Pos 0
#define RF_REG2F_PADS_PE_DS_SPI_SCLK_PE_BYTE_Pos 1
#define RF_REG2F_PADS_PE_DS_SPI_MOSI_PE_BYTE_Pos 2
#define RF_REG2F_PADS_PE_DS_SPI_MISO_PE_BYTE_Pos 3
#define RF_REG2F_PADS_PE_DS_NRESET_PE_BYTE_Pos 4
#define RF_REG2F_PADS_PE_DS_GPIO_PE_BYTE_Pos 5
#define RF_REG2F_PADS_PE_DS_GPIO_DS_BYTE_Pos 6
#define RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos 0
#define RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Mask ((uint32_t)(0x7U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))

/* RF_REG2F sub-register settings */
#define SYNC_WORD_CORR_SYNC_WORD_BIAS_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_BYTE_Pos))

#define SYNC_WORD_CORR_EN_SYNC_WORD_CORR_DEFAULT_BYTE ((uint8_t)(SYNC_WORD_CORR_EN_SYNC_WORD_CORR_DEFAULT_BITBAND << RF_REG2F_SYNC_WORD_CORR_EN_SYNC_WORD_CORR_BYTE_Pos))

#define SUBBAND_FLL_SB_FLL_WAIT_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_BYTE_Pos))

#define SUBBAND_FLL_SB_FLL_PH_4_N8_DEFAULT_BYTE ((uint8_t)(SUBBAND_FLL_SB_FLL_PH_4_N8_DEFAULT_BITBAND << RF_REG2F_SUBBAND_FLL_SB_FLL_PH_4_N8_BYTE_Pos))

#define SUBBAND_FLL_SB_FLL_CIC_TAU_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_BYTE_Pos))

#define SUBBAND_FLL_SB_FLL_DITHER_DEFAULT_BYTE ((uint8_t)(0x0U << RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_BYTE_Pos))

#define PADS_PE_DS_SPI_CS_N_PE_DEFAULT_BYTE ((uint8_t)(PADS_PE_DS_SPI_CS_N_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_SPI_CS_N_PE_BYTE_Pos))

#define PADS_PE_DS_SPI_SCLK_PE_DEFAULT_BYTE ((uint8_t)(PADS_PE_DS_SPI_SCLK_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_SPI_SCLK_PE_BYTE_Pos))

#define PADS_PE_DS_SPI_MOSI_PE_DEFAULT_BYTE ((uint8_t)(PADS_PE_DS_SPI_MOSI_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_SPI_MOSI_PE_BYTE_Pos))

#define PADS_PE_DS_SPI_MISO_PE_DEFAULT_BYTE ((uint8_t)(PADS_PE_DS_SPI_MISO_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_SPI_MISO_PE_BYTE_Pos))

#define PADS_PE_DS_NRESET_PE_DEFAULT_BYTE ((uint8_t)(PADS_PE_DS_NRESET_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_NRESET_PE_BYTE_Pos))

#define PADS_PE_DS_GPIO_PE_DEFAULT_BYTE ((uint8_t)(PADS_PE_DS_GPIO_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_GPIO_PE_BYTE_Pos))

#define PADS_PE_DS_GPIO_DS_DEFAULT_BYTE ((uint8_t)(PADS_PE_DS_GPIO_DS_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_GPIO_DS_BYTE_Pos))

#define CK_DIV_1_6_NO_CLOCK_BYTE        ((uint8_t)(0x0U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))
#define CK_DIV_1_6_PRESCALE_1_BYTE      ((uint8_t)(0x1U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))
#define CK_DIV_1_6_PRESCALE_2_BYTE      ((uint8_t)(0x2U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))
#define CK_DIV_1_6_PRESCALE_3_BYTE      ((uint8_t)(0x3U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))
#define CK_DIV_1_6_PRESCALE_4_BYTE      ((uint8_t)(0x4U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))
#define CK_DIV_1_6_PRESCALE_5_BYTE      ((uint8_t)(0x5U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))
#define CK_DIV_1_6_PRESCALE_6_BYTE      ((uint8_t)(0x6U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))
#define CK_DIV_1_6_PRESCALE_7_BYTE      ((uint8_t)(0x7U << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_BYTE_Pos))

/* RF_REG30 bit positions */
#define RF_REG30_RXFIFO_STATUS_BIST_Pos 25
#define RF_REG30_RXFIFO_STATUS_BIST_Mask ((uint32_t)(0x7FU << RF_REG30_RXFIFO_STATUS_BIST_Pos))
#define RF_REG30_RXFIFO_STATUS_BIST_ERRORS_Pos 30
#define RF_REG30_RXFIFO_STATUS_BIST_ERRORS_Mask ((uint32_t)(0x3U << RF_REG30_RXFIFO_STATUS_BIST_ERRORS_Pos))
#define RF_REG30_RXFIFO_STATUS_NEAR_UNDERFLOW_Pos 29
#define RF_REG30_RXFIFO_STATUS_NEAR_OVERFLOW_Pos 28
#define RF_REG30_RXFIFO_STATUS_UNDERFLOW_Pos 27
#define RF_REG30_RXFIFO_STATUS_OVERFLOW_Pos 26
#define RF_REG30_RXFIFO_STATUS_FULL_Pos 25
#define RF_REG30_RXFIFO_STATUS_FLUSH_Pos 24
#define RF_REG30_RXFIFO_STATUS_EMPTY_Pos 24
#define RF_REG30_TXFIFO_STATUS_BIST_Pos 17
#define RF_REG30_TXFIFO_STATUS_BIST_Mask ((uint32_t)(0x7FU << RF_REG30_TXFIFO_STATUS_BIST_Pos))
#define RF_REG30_TXFIFO_STATUS_BIST_ERRORS_Pos 22
#define RF_REG30_TXFIFO_STATUS_BIST_ERRORS_Mask ((uint32_t)(0x3U << RF_REG30_TXFIFO_STATUS_BIST_ERRORS_Pos))
#define RF_REG30_TXFIFO_STATUS_NEAR_UNDERFLOW_Pos 21
#define RF_REG30_TXFIFO_STATUS_NEAR_OVERFLOW_Pos 20
#define RF_REG30_TXFIFO_STATUS_UNDERFLOW_Pos 19
#define RF_REG30_TXFIFO_STATUS_OVERFLOW_Pos 18
#define RF_REG30_TXFIFO_STATUS_FULL_Pos 17
#define RF_REG30_TXFIFO_STATUS_FLUSH_Pos 16
#define RF_REG30_TXFIFO_STATUS_EMPTY_Pos 16
#define RF_REG30_FSM_STATUS_TX_NRX_Pos  10
#define RF_REG30_FSM_STATUS_STATUS_Pos  8
#define RF_REG30_FSM_STATUS_STATUS_Mask ((uint32_t)(0x3U << RF_REG30_FSM_STATUS_STATUS_Pos))
#define RF_REG30_FSM_MODE_RESET_Pos     3
#define RF_REG30_FSM_MODE_TX_NRX_Pos    2
#define RF_REG30_FSM_MODE_RX_MODE_Pos   2
#define RF_REG30_FSM_MODE_MODE_Pos      0
#define RF_REG30_FSM_MODE_MODE_Mask     ((uint32_t)(0x3U << RF_REG30_FSM_MODE_MODE_Pos))
#define RF_REG30_FSM_MODE_TX_MODE_Pos   1
#define RF_REG30_FSM_MODE_N_IDLE_Pos    0

/* RF_REG30 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t FSM_MODE_BYTE;        
    __I  uint8_t FSM_STATUS_BYTE;      
    __IO uint8_t TXFIFO_STATUS_BYTE;   
    __IO uint8_t RXFIFO_STATUS_BYTE;   
         uint8_t RESERVED[SYS_CALC_BITBAND(0x400100C0, 0) - (0x400100C0 + 4)];
    __I  uint32_t FSM_MODE_N_IDLE_ALIAS;/* The field is set to 1 if the FSM is not in the Idle mode. */
    __I  uint32_t FSM_MODE_TX_MODE_ALIAS;/* The field keep the value 1 as long as the transmission isn't over */
    __I  uint32_t FSM_MODE_RX_MODE_ALIAS;/* The field stay with value 1 as long as the reception isn't over */
         uint32_t RESERVED0[7];
    __I  uint32_t FSM_STATUS_TX_NRX_ALIAS;/* Is set to 0 if the radio is in Rx mode, to 1 if in Tx mode */
         uint32_t RESERVED1[5];
    __I  uint32_t TXFIFO_STATUS_EMPTY_ALIAS;/* Is set to 1 if the Tx FIFO is empty */
    __I  uint32_t TXFIFO_STATUS_FULL_ALIAS;/* Is set to 1 if the Tx FIFO is full */
    __I  uint32_t TXFIFO_STATUS_OVERFLOW_ALIAS;/* Is set to 1 if there has been an overflow */
    __I  uint32_t TXFIFO_STATUS_UNDERFLOW_ALIAS;/* Is set to 1 if there has been an underflow */
    __I  uint32_t TXFIFO_STATUS_NEAR_OVERFLOW_ALIAS;/* Is set to 1 if the Tx FIFO is close to the overflow */
    __I  uint32_t TXFIFO_STATUS_NEAR_UNDERFLOW_ALIAS;/* Is set to 1 if the Tx FIFO is close to the underflow */
         uint32_t RESERVED2[2];
    __I  uint32_t RXFIFO_STATUS_EMPTY_ALIAS;/* Is set to 1 if the Rx FIFO is empty */
    __I  uint32_t RXFIFO_STATUS_FULL_ALIAS;/* Is set to 1 if the Rx FIFO is full */
    __I  uint32_t RXFIFO_STATUS_OVERFLOW_ALIAS;/* Is set to 1 if there has been an overflow */
    __I  uint32_t RXFIFO_STATUS_UNDERFLOW_ALIAS;/* Is set to 1 if there has been an underflow */
    __I  uint32_t RXFIFO_STATUS_NEAR_OVERFLOW_ALIAS;/* Is set to 1 if the Rx FIFO is close to the overflow */
    __I  uint32_t RXFIFO_STATUS_NEAR_UNDERFLOW_ALIAS;/* Is set to 1 if the Rx FIFO is close to the underflow */
} RF_REG30_Type;

#define RF_REG30_BASE                   0x400100C0
#define RF_REG30                        ((RF_REG30_Type *) RF_REG30_BASE)

/* RF_REG30 sub-register bit positions */
#define RF_REG30_FSM_MODE_N_IDLE_BYTE_Pos 0
#define RF_REG30_FSM_MODE_MODE_BYTE_Pos 0
#define RF_REG30_FSM_MODE_MODE_BYTE_Mask ((uint32_t)(0x3U << RF_REG30_FSM_MODE_MODE_BYTE_Pos))
#define RF_REG30_FSM_MODE_TX_MODE_BYTE_Pos 1
#define RF_REG30_FSM_MODE_TX_NRX_BYTE_Pos 2
#define RF_REG30_FSM_MODE_RX_MODE_BYTE_Pos 2
#define RF_REG30_FSM_MODE_RESET_BYTE_Pos 3
#define RF_REG30_FSM_STATUS_STATUS_BYTE_Pos 0
#define RF_REG30_FSM_STATUS_STATUS_BYTE_Mask ((uint32_t)(0x3U << RF_REG30_FSM_STATUS_STATUS_BYTE_Pos))
#define RF_REG30_FSM_STATUS_TX_NRX_BYTE_Pos 2
#define RF_REG30_TXFIFO_STATUS_FLUSH_BYTE_Pos 0
#define RF_REG30_TXFIFO_STATUS_EMPTY_BYTE_Pos 0
#define RF_REG30_TXFIFO_STATUS_FULL_BYTE_Pos 1
#define RF_REG30_TXFIFO_STATUS_OVERFLOW_BYTE_Pos 2
#define RF_REG30_TXFIFO_STATUS_UNDERFLOW_BYTE_Pos 3
#define RF_REG30_TXFIFO_STATUS_NEAR_OVERFLOW_BYTE_Pos 4
#define RF_REG30_TXFIFO_STATUS_NEAR_UNDERFLOW_BYTE_Pos 5
#define RF_REG30_TXFIFO_STATUS_BIST_BYTE_Pos 1
#define RF_REG30_TXFIFO_STATUS_BIST_BYTE_Mask ((uint32_t)(0x7FU << RF_REG30_TXFIFO_STATUS_BIST_BYTE_Pos))
#define RF_REG30_TXFIFO_STATUS_BIST_ERRORS_BYTE_Pos 6
#define RF_REG30_TXFIFO_STATUS_BIST_ERRORS_BYTE_Mask ((uint32_t)(0x3U << RF_REG30_TXFIFO_STATUS_BIST_ERRORS_BYTE_Pos))
#define RF_REG30_RXFIFO_STATUS_FLUSH_BYTE_Pos 0
#define RF_REG30_RXFIFO_STATUS_EMPTY_BYTE_Pos 0
#define RF_REG30_RXFIFO_STATUS_FULL_BYTE_Pos 1
#define RF_REG30_RXFIFO_STATUS_OVERFLOW_BYTE_Pos 2
#define RF_REG30_RXFIFO_STATUS_UNDERFLOW_BYTE_Pos 3
#define RF_REG30_RXFIFO_STATUS_NEAR_OVERFLOW_BYTE_Pos 4
#define RF_REG30_RXFIFO_STATUS_NEAR_UNDERFLOW_BYTE_Pos 5
#define RF_REG30_RXFIFO_STATUS_BIST_BYTE_Pos 1
#define RF_REG30_RXFIFO_STATUS_BIST_BYTE_Mask ((uint32_t)(0x7FU << RF_REG30_RXFIFO_STATUS_BIST_BYTE_Pos))
#define RF_REG30_RXFIFO_STATUS_BIST_ERRORS_BYTE_Pos 6
#define RF_REG30_RXFIFO_STATUS_BIST_ERRORS_BYTE_Mask ((uint32_t)(0x3U << RF_REG30_RXFIFO_STATUS_BIST_ERRORS_BYTE_Pos))

/* RF_REG31 bit positions */
#define RF_REG31_RSSI_MAX_RSSI_MAX_Pos  24
#define RF_REG31_RSSI_MAX_RSSI_MAX_Mask ((uint32_t)(0xFFU << RF_REG31_RSSI_MAX_RSSI_MAX_Pos))
#define RF_REG31_RSSI_MIN_RSSI_MIN_Pos  16
#define RF_REG31_RSSI_MIN_RSSI_MIN_Mask ((uint32_t)(0xFFU << RF_REG31_RSSI_MIN_RSSI_MIN_Pos))
#define RF_REG31_RXFIFO_COUNT_RX_COUNT_Pos 8
#define RF_REG31_RXFIFO_COUNT_RX_COUNT_Mask ((uint32_t)(0xFFU << RF_REG31_RXFIFO_COUNT_RX_COUNT_Pos))
#define RF_REG31_TXFIFO_COUNT_TX_COUNT_Pos 0
#define RF_REG31_TXFIFO_COUNT_TX_COUNT_Mask ((uint32_t)(0xFFU << RF_REG31_TXFIFO_COUNT_TX_COUNT_Pos))

/* RF_REG31 sub-register and bit-band aliases */
typedef struct
{
    __I  uint8_t TXFIFO_COUNT_TX_COUNT_BYTE;
    __I  uint8_t RXFIFO_COUNT_RX_COUNT_BYTE;
    __I  uint8_t RSSI_MIN_RSSI_MIN_BYTE;
    __I  uint8_t RSSI_MAX_RSSI_MAX_BYTE;
} RF_REG31_Type;

#define RF_REG31_BASE                   0x400100C4
#define RF_REG31                        ((RF_REG31_Type *) RF_REG31_BASE)

/* RF_REG32 bit positions */
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_Pos 28
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_Mask ((uint32_t)(0x7U << RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_Pos))
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_Pos 24
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_Mask ((uint32_t)(0x7U << RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_Pos))
#define RF_REG32_RSSI_AVG_RSSI_AVG_Pos  16
#define RF_REG32_RSSI_AVG_RSSI_AVG_Mask ((uint32_t)(0xFFU << RF_REG32_RSSI_AVG_RSSI_AVG_Pos))
#define RF_REG32_DR_ERR_IND_DR_ERR_IND_Pos 8
#define RF_REG32_DR_ERR_IND_DR_ERR_IND_Mask ((uint32_t)(0xFFU << RF_REG32_DR_ERR_IND_DR_ERR_IND_Pos))
#define RF_REG32_RSSI_PKT_RSSI_PKT_Pos  0
#define RF_REG32_RSSI_PKT_RSSI_PKT_Mask ((uint32_t)(0xFFU << RF_REG32_RSSI_PKT_RSSI_PKT_Pos))

/* RF_REG32 sub-register and bit-band aliases */
typedef struct
{
    __I  uint8_t RSSI_PKT_RSSI_PKT_BYTE;
    __I  uint8_t DR_ERR_IND_DR_ERR_IND_BYTE;
    __I  uint8_t RSSI_AVG_RSSI_AVG_BYTE;
    __I  uint8_t RX_ATT_LEVEL_RX_ATT_LEVEL_BYTE;
} RF_REG32_Type;

#define RF_REG32_BASE                   0x400100C8
#define RF_REG32                        ((RF_REG32_Type *) RF_REG32_BASE)

/* RF_REG32 sub-register bit positions */
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_BYTE_Pos 0
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_BYTE_Mask ((uint32_t)(0x7U << RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_BYTE_Pos))
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_BYTE_Pos 4
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_BYTE_Mask ((uint32_t)(0x7U << RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_BYTE_Pos))

/* RF_TXFIFO bit positions */
#define RF_TXFIFO_TXFIFO_TX_DATA_Pos    0
#define RF_TXFIFO_TXFIFO_TX_DATA_Mask   ((uint32_t)(0xFFU << RF_TXFIFO_TXFIFO_TX_DATA_Pos))

/* RF_RXFIFO bit positions */
#define RF_RXFIFO_RXFIFO_RX_DATA_Pos    0
#define RF_RXFIFO_RXFIFO_RX_DATA_Mask   ((uint32_t)(0xFFU << RF_RXFIFO_RXFIFO_RX_DATA_Pos))

/* RF_DESER_STATUS bit positions */
#define RF_DESER_STATUS_DESER_STATUS_SIGNAL_RECEIVING_Pos 7
#define RF_DESER_STATUS_DESER_STATUS_SYNC_DETECTED_Pos 6
#define RF_DESER_STATUS_DESER_STATUS_WAIT_SYNC_Pos 5
#define RF_DESER_STATUS_DESER_STATUS_IS_ADDRESS_BR_Pos 4
#define RF_DESER_STATUS_DESER_STATUS_PKT_LEN_ERR_Pos 3
#define RF_DESER_STATUS_DESER_STATUS_ADDRESS_ERR_Pos 2
#define RF_DESER_STATUS_DESER_STATUS_CRC_ERR_Pos 1
#define RF_DESER_STATUS_DESER_STATUS_DESER_FINISH_Pos 0

/* RF_DESER_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t DESER_STATUS_DESER_FINISH_ALIAS;/* Is set to 1 when the deserializer has finished */
    __I  uint32_t DESER_STATUS_CRC_ERR_ALIAS;/* Is set to 1 in case of a CRC error */
    __I  uint32_t DESER_STATUS_ADDRESS_ERR_ALIAS;/* Is set to 1 in case of an address error */
    __I  uint32_t DESER_STATUS_PKT_LEN_ERR_ALIAS;/* Is set to 1 in case of the packet length is longer than the maximum acceptable packet length */
    __I  uint32_t DESER_STATUS_IS_ADDRESS_BR_ALIAS;/* Is set to 1 if the received address is the broadcast address. */
    __I  uint32_t DESER_STATUS_WAIT_SYNC_ALIAS;/* Is set to 1 if the deserializer is waiting the sync word */
    __I  uint32_t DESER_STATUS_SYNC_DETECTED_ALIAS;/* Is set to 1 is the sync word (pattern) has been detected */
    __I  uint32_t DESER_STATUS_SIGNAL_RECEIVING_ALIAS;/* Is set to 1 if the deserializer is on */
} RF_DESER_STATUS_Type;

#define RF_DESER_STATUS_BASE            SYS_CALC_BITBAND(0x400100D4, 0)
#define RF_DESER_STATUS                 ((RF_DESER_STATUS_Type *) RF_DESER_STATUS_BASE)

/* RF_IRQ_STATUS bit positions */
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RXFIFO_Pos 5
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_TXFIFO_Pos 4
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_SYNC_Pos 3
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RECEIVED_Pos 2
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RXSTOP_Pos 1
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_TX_Pos 0

/* RF_IRQ_STATUS bit-band aliases */
typedef struct
{
    __I  uint32_t IRQ_STATUS_FLAG_TX_ALIAS;/* Is set to 1 when the IRQ TX is active */
    __I  uint32_t IRQ_STATUS_FLAG_RXSTOP_ALIAS;/* Is set to 1 when the IRQ RXSTOP is active */
    __I  uint32_t IRQ_STATUS_FLAG_RECEIVED_ALIAS;/* Is set to 1 when the IRQ RECEIVED is active */
    __I  uint32_t IRQ_STATUS_FLAG_SYNC_ALIAS;/* Is set to 1 when the IRQ SYNC is active */
    __I  uint32_t IRQ_STATUS_FLAG_TXFIFO_ALIAS;/* Is set to 1 when the IRQ TXFIFO is active */
    __I  uint32_t IRQ_STATUS_FLAG_RXFIFO_ALIAS;/* Is set to 1 when the IRQ RXFIFO is active */
} RF_IRQ_STATUS_Type;

#define RF_IRQ_STATUS_BASE              SYS_CALC_BITBAND(0x400100D8, 0)
#define RF_IRQ_STATUS                   ((RF_IRQ_STATUS_Type *) RF_IRQ_STATUS_BASE)

/* RF_REG37 bit positions */
#define RF_REG37_FEI_PKT_FEI_PKT_Pos    16
#define RF_REG37_FEI_PKT_FEI_PKT_Mask   ((uint32_t)(0xFFFFU << RF_REG37_FEI_PKT_FEI_PKT_Pos))
#define RF_REG37_FEI_FEI_OUT_Pos        0
#define RF_REG37_FEI_FEI_OUT_Mask       ((uint32_t)(0xFFFFU << RF_REG37_FEI_FEI_OUT_Pos))

/* RF_REG37 sub-register and bit-band aliases */
typedef struct
{
    __I  uint16_t FEI_FEI_OUT_SHORT;    
    __I  uint16_t FEI_PKT_FEI_PKT_SHORT;
} RF_REG37_Type;

#define RF_REG37_BASE                   0x400100DC
#define RF_REG37                        ((RF_REG37_Type *) RF_REG37_BASE)

/* RF_REG38 bit positions */
#define RF_REG38_LINK_QUAL_PKT_LINK_QUALITY_PKT_Pos 24
#define RF_REG38_LINK_QUAL_PKT_LINK_QUALITY_PKT_Mask ((uint32_t)(0xFFU << RF_REG38_LINK_QUAL_PKT_LINK_QUALITY_PKT_Pos))
#define RF_REG38_LINK_QUAL_LINK_QUALITY_Pos 16
#define RF_REG38_LINK_QUAL_LINK_QUALITY_Mask ((uint32_t)(0xFFU << RF_REG38_LINK_QUAL_LINK_QUALITY_Pos))
#define RF_REG38_FEI_AFC_FEI_AFC_Pos    0
#define RF_REG38_FEI_AFC_FEI_AFC_Mask   ((uint32_t)(0xFFFFU << RF_REG38_FEI_AFC_FEI_AFC_Pos))

/* RF_REG38 sub-register and bit-band aliases */
typedef struct
{
    __I  uint16_t FEI_AFC_FEI_AFC_SHORT;
    __I  uint8_t LINK_QUAL_LINK_QUALITY_BYTE;
    __I  uint8_t LINK_QUAL_PKT_LINK_QUALITY_PKT_BYTE;
} RF_REG38_Type;

#define RF_REG38_BASE                   0x400100E0
#define RF_REG38                        ((RF_REG38_Type *) RF_REG38_BASE)

/* RF_REG39 bit positions */
#define RF_REG39_ANALOG_INFO_XTAL_FINISH_Pos 13
#define RF_REG39_ANALOG_INFO_DLL_LOCKED_Pos 12
#define RF_REG39_ANALOG_INFO_CLK_DIG_READY_Pos 11
#define RF_REG39_ANALOG_INFO_CLK_PLL_READY_Pos 10
#define RF_REG39_ANALOG_INFO_SUBBAND_HI_Pos 9
#define RF_REG39_ANALOG_INFO_SUBBAND_LO_Pos 8
#define RF_REG39_SUBBAND_ERR_SB_FLL_ERR_Pos 0
#define RF_REG39_SUBBAND_ERR_SB_FLL_ERR_Mask ((uint32_t)(0xFFU << RF_REG39_SUBBAND_ERR_SB_FLL_ERR_Pos))

/* RF_REG39 sub-register and bit-band aliases */
typedef struct
{
    __I  uint8_t SUBBAND_ERR_SB_FLL_ERR_BYTE;
    __I  uint8_t ANALOG_INFO_BYTE;     
         uint8_t RESERVED0[2];
         uint8_t RESERVED[SYS_CALC_BITBAND(0x400100E4, 0) - (0x400100E4 + 4)];
         uint32_t RESERVED1[8];
    __I  uint32_t ANALOG_INFO_SUBBAND_LO_ALIAS;/* Status of the subband comparator Lo */
    __I  uint32_t ANALOG_INFO_SUBBAND_HI_ALIAS;/* Status of the subband comparator Hi */
    __I  uint32_t ANALOG_INFO_CLK_PLL_READY_ALIAS;/* Ready signal of the PLL clock */
    __I  uint32_t ANALOG_INFO_CLK_DIG_READY_ALIAS;/* Ready signal of the digital clock */
    __I  uint32_t ANALOG_INFO_DLL_LOCKED_ALIAS;/* DLL locked signal */
    __I  uint32_t ANALOG_INFO_XTAL_FINISH_ALIAS;/* If set to 1, the Xtal algorithm has finished */
} RF_REG39_Type;

#define RF_REG39_BASE                   0x400100E4
#define RF_REG39                        ((RF_REG39_Type *) RF_REG39_BASE)

/* RF_REG39 settings */
#define ANALOG_INFO_CLK_DIG_NOT_READY_BITBAND 0x0
#define ANALOG_INFO_CLK_DIG_READY_BITBAND 0x1
#define ANALOG_INFO_CLK_DIG_NOT_READY   ((uint32_t)(ANALOG_INFO_CLK_DIG_NOT_READY_BITBAND << RF_REG39_ANALOG_INFO_CLK_DIG_READY_Pos))
#define ANALOG_INFO_CLK_DIG_READY       ((uint32_t)(ANALOG_INFO_CLK_DIG_READY_BITBAND << RF_REG39_ANALOG_INFO_CLK_DIG_READY_Pos))

#define ANALOG_INFO_DLL_UNLOCKED_BITBAND 0x0
#define ANALOG_INFO_DLL_LOCKED_BITBAND  0x1
#define ANALOG_INFO_DLL_UNLOCKED        ((uint32_t)(ANALOG_INFO_DLL_UNLOCKED_BITBAND << RF_REG39_ANALOG_INFO_DLL_LOCKED_Pos))
#define ANALOG_INFO_DLL_LOCKED          ((uint32_t)(ANALOG_INFO_DLL_LOCKED_BITBAND << RF_REG39_ANALOG_INFO_DLL_LOCKED_Pos))

#define ANALOG_INFO_XTAL_TRIM_RUNNING_BITBAND 0x0
#define ANALOG_INFO_XTAL_TRIM_FINISHED_BITBAND 0x1
#define ANALOG_INFO_XTAL_TRIM_RUNNING   ((uint32_t)(ANALOG_INFO_XTAL_TRIM_RUNNING_BITBAND << RF_REG39_ANALOG_INFO_XTAL_FINISH_Pos))
#define ANALOG_INFO_XTAL_TRIM_FINISHED  ((uint32_t)(ANALOG_INFO_XTAL_TRIM_FINISHED_BITBAND << RF_REG39_ANALOG_INFO_XTAL_FINISH_Pos))

/* RF_REG39 sub-register bit positions */
#define RF_REG39_ANALOG_INFO_SUBBAND_LO_BYTE_Pos 0
#define RF_REG39_ANALOG_INFO_SUBBAND_HI_BYTE_Pos 1
#define RF_REG39_ANALOG_INFO_CLK_PLL_READY_BYTE_Pos 2
#define RF_REG39_ANALOG_INFO_CLK_DIG_READY_BYTE_Pos 3
#define RF_REG39_ANALOG_INFO_DLL_LOCKED_BYTE_Pos 4
#define RF_REG39_ANALOG_INFO_XTAL_FINISH_BYTE_Pos 5

/* RF_REG39 sub-register settings */
#define ANALOG_INFO_CLK_DIG_NOT_READY_BYTE ((uint8_t)(ANALOG_INFO_CLK_DIG_NOT_READY_BITBAND << RF_REG39_ANALOG_INFO_CLK_DIG_READY_BYTE_Pos))
#define ANALOG_INFO_CLK_DIG_READY_BYTE  ((uint8_t)(ANALOG_INFO_CLK_DIG_READY_BITBAND << RF_REG39_ANALOG_INFO_CLK_DIG_READY_BYTE_Pos))

#define ANALOG_INFO_DLL_UNLOCKED_BYTE   ((uint8_t)(ANALOG_INFO_DLL_UNLOCKED_BITBAND << RF_REG39_ANALOG_INFO_DLL_LOCKED_BYTE_Pos))
#define ANALOG_INFO_DLL_LOCKED_BYTE     ((uint8_t)(ANALOG_INFO_DLL_LOCKED_BITBAND << RF_REG39_ANALOG_INFO_DLL_LOCKED_BYTE_Pos))

#define ANALOG_INFO_XTAL_TRIM_RUNNING_BYTE ((uint8_t)(ANALOG_INFO_XTAL_TRIM_RUNNING_BITBAND << RF_REG39_ANALOG_INFO_XTAL_FINISH_BYTE_Pos))
#define ANALOG_INFO_XTAL_TRIM_FINISHED_BYTE ((uint8_t)(ANALOG_INFO_XTAL_TRIM_FINISHED_BITBAND << RF_REG39_ANALOG_INFO_XTAL_FINISH_BYTE_Pos))

/* RF_REVISION bit positions */
#define RF_REVISION_CHIP_ID_Pos         24
#define RF_REVISION_CHIP_ID_Mask        ((uint32_t)(0x3FU << RF_REVISION_CHIP_ID_Pos))

/* ----------------------------------------------------------------------------
 * SYSTICK Timer
 * ------------------------------------------------------------------------- */

/* SysTick_CTRL sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[2];
    __I  uint8_t COUNTFLAG_BYTE;       
         uint8_t RESERVED1[1];
} SysTick_CTRL_Type;

#define SysTick_CTRL_BASE               0xE000E010
#define SysTick_CTRL                    ((SysTick_CTRL_Type *) SysTick_CTRL_BASE)

/* SysTick_CTRL settings */
#define SYSTICK_DISABLE                 ((uint32_t)(0x0U << SysTick_CTRL_ENABLE_Pos))
#define SYSTICK_ENABLE                  ((uint32_t)(0x1U << SysTick_CTRL_ENABLE_Pos))

#define SYSTICK_TICKINT_DISABLE         ((uint32_t)(0x0U << SysTick_CTRL_TICKINT_Pos))
#define SYSTICK_TICKINT_ENABLE          ((uint32_t)(0x1U << SysTick_CTRL_TICKINT_Pos))

#define SYSTICK_CLKSOURCE_EXTREF_CLK    ((uint32_t)(0x0U << SysTick_CTRL_CLKSOURCE_Pos))
#define SYSTICK_CLKSOURCE_CORE_CLK      ((uint32_t)(0x1U << SysTick_CTRL_CLKSOURCE_Pos))

#define SYSTICK_COUNTFLAG_NOT_ZERO      ((uint32_t)(0x0U << SysTick_CTRL_COUNTFLAG_Pos))
#define SYSTICK_COUNTFLAG_ZERO          ((uint32_t)(0x1U << SysTick_CTRL_COUNTFLAG_Pos))

/* SysTick_CTRL sub-register bit positions */
#define SysTick_CTRL_COUNTFLAG_BYTE_Pos 0

/* SysTick_CTRL sub-register settings */
#define SYSTICK_COUNTFLAG_NOT_ZERO_BYTE ((uint8_t)(0x0U << SysTick_CTRL_COUNTFLAG_BYTE_Pos))
#define SYSTICK_COUNTFLAG_ZERO_BYTE     ((uint8_t)(0x1U << SysTick_CTRL_COUNTFLAG_BYTE_Pos))

/* SysTick_CALIB settings */
#define SYSTICK_SKEW                    ((uint32_t)(0x1U << SysTick_CALIB_SKEW_Pos))
#define SYSTICK_NOSKEW                  ((uint32_t)(0x0U << SysTick_CALIB_SKEW_Pos))

#define SYSTICK_NOREF                   ((uint32_t)(0x1U << SysTick_CALIB_NOREF_Pos))
#define SYSTICK_REF                     ((uint32_t)(0x0U << SysTick_CALIB_NOREF_Pos))

/* ----------------------------------------------------------------------------
 * System Control and ID register not in the SCB
 * ------------------------------------------------------------------------- */

/* SCnSCB_ICTR settings */
#define NVIC_INTLINESNUM_1_32           ((uint32_t)(0x0U << SCnSCB_ICTR_INTLINESNUM_Pos))
#define NVIC_INTLINESNUM_33_64          ((uint32_t)(0x1U << SCnSCB_ICTR_INTLINESNUM_Pos))
#define NVIC_INTLINESNUM_65_96          ((uint32_t)(0x2U << SCnSCB_ICTR_INTLINESNUM_Pos))

/* ----------------------------------------------------------------------------
 * Nested Vector Interrupt Controller
 * ------------------------------------------------------------------------- */

/* NVIC_ISER0 bit positions */
#define NVIC_ISER0_DMIC_OUT_OD_IN_Pos   31
#define NVIC_ISER0_UART_ERROR_Pos       30
#define NVIC_ISER0_UART_TX_Pos          29
#define NVIC_ISER0_UART_RX_Pos          28
#define NVIC_ISER0_I2C_Pos              27
#define NVIC_ISER0_SPI1_ERROR_Pos       26
#define NVIC_ISER0_SPI1_TX_Pos          25
#define NVIC_ISER0_SPI1_RX_Pos          24
#define NVIC_ISER0_SPI0_ERROR_Pos       23
#define NVIC_ISER0_SPI0_TX_Pos          22
#define NVIC_ISER0_SPI0_RX_Pos          21
#define NVIC_ISER0_WATCHDOG_Pos         20
#define NVIC_ISER0_DIO3_Pos             19
#define NVIC_ISER0_DIO2_Pos             18
#define NVIC_ISER0_DIO1_Pos             17
#define NVIC_ISER0_DIO0_Pos             16
#define NVIC_ISER0_DMA7_Pos             15
#define NVIC_ISER0_DMA6_Pos             14
#define NVIC_ISER0_DMA5_Pos             13
#define NVIC_ISER0_DMA4_Pos             12
#define NVIC_ISER0_DMA3_Pos             11
#define NVIC_ISER0_DMA2_Pos             10
#define NVIC_ISER0_DMA1_Pos             9
#define NVIC_ISER0_DMA0_Pos             8
#define NVIC_ISER0_TIMER3_Pos           7
#define NVIC_ISER0_TIMER2_Pos           6
#define NVIC_ISER0_TIMER1_Pos           5
#define NVIC_ISER0_TIMER0_Pos           4
#define NVIC_ISER0_ADC_BATMON_Pos       3
#define NVIC_ISER0_RTC_CLOCK_Pos        2
#define NVIC_ISER0_RTC_ALARM_Pos        1
#define NVIC_ISER0_WAKEUP_Pos           0

/* NVIC_ISER0 settings */
#define NVIC_WAKEUP_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER0_WAKEUP_Pos))

#define NVIC_RTC_ALARM_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER0_RTC_ALARM_Pos))

#define NVIC_RTC_CLOCK_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER0_RTC_CLOCK_Pos))

#define NVIC_ADC_BATMON_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER0_ADC_BATMON_Pos))

#define NVIC_TIMER0_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER0_TIMER0_Pos))

#define NVIC_TIMER1_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER0_TIMER1_Pos))

#define NVIC_TIMER2_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER0_TIMER2_Pos))

#define NVIC_TIMER3_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER0_TIMER3_Pos))

#define NVIC_DMA0_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DMA0_Pos))

#define NVIC_DMA1_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DMA1_Pos))

#define NVIC_DMA2_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DMA2_Pos))

#define NVIC_DMA3_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DMA3_Pos))

#define NVIC_DMA4_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DMA4_Pos))

#define NVIC_DMA5_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DMA5_Pos))

#define NVIC_DMA6_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DMA6_Pos))

#define NVIC_DMA7_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DMA7_Pos))

#define NVIC_DIO0_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DIO0_Pos))

#define NVIC_DIO1_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DIO1_Pos))

#define NVIC_DIO2_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DIO2_Pos))

#define NVIC_DIO3_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER0_DIO3_Pos))

#define NVIC_WATCHDOG_INT_ENABLE        ((uint32_t)(0x1U << NVIC_ISER0_WATCHDOG_Pos))

#define NVIC_SPI0_RX_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER0_SPI0_RX_Pos))

#define NVIC_SPI0_TX_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER0_SPI0_TX_Pos))

#define NVIC_SPI0_ERROR_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER0_SPI0_ERROR_Pos))

#define NVIC_SPI1_RX_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER0_SPI1_RX_Pos))

#define NVIC_SPI1_TX_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER0_SPI1_TX_Pos))

#define NVIC_SPI1_ERROR_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER0_SPI1_ERROR_Pos))

#define NVIC_I2C_INT_ENABLE             ((uint32_t)(0x1U << NVIC_ISER0_I2C_Pos))

#define NVIC_UART_RX_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER0_UART_RX_Pos))

#define NVIC_UART_TX_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER0_UART_TX_Pos))

#define NVIC_UART_ERROR_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER0_UART_ERROR_Pos))

#define NVIC_DMIC_OUT_OD_IN_INT_ENABLE  ((uint32_t)(0x1U << NVIC_ISER0_DMIC_OUT_OD_IN_Pos))

/* NVIC_ISER1 bit positions */
#define NVIC_ISER1_ASRC_OUT_Pos         31
#define NVIC_ISER1_ASRC_IN_Pos          30
#define NVIC_ISER1_ASRC_ERROR_Pos       29
#define NVIC_ISER1_RF_RXFIFO_Pos        28
#define NVIC_ISER1_RF_TXFIFO_Pos        27
#define NVIC_ISER1_RF_SYNC_Pos          26
#define NVIC_ISER1_RF_RECEIVED_Pos      25
#define NVIC_ISER1_RF_RXSTOP_Pos        24
#define NVIC_ISER1_RF_TX_Pos            23
#define NVIC_ISER1_BLE_COEX_IN_PROCESS_Pos 22
#define NVIC_ISER1_BLE_COEX_RX_TX_Pos   21
#define NVIC_ISER1_BLE_SW_Pos           20
#define NVIC_ISER1_BLE_FINETGTIM_Pos    19
#define NVIC_ISER1_BLE_GROSSTGTIM_Pos   18
#define NVIC_ISER1_BLE_ERROR_Pos        17
#define NVIC_ISER1_BLE_CRYPT_Pos        16
#define NVIC_ISER1_BLE_EVENT_Pos        15
#define NVIC_ISER1_BLE_RX_Pos           14
#define NVIC_ISER1_BLE_SLP_Pos          13
#define NVIC_ISER1_BLE_CSCNT_Pos        12
#define NVIC_ISER1_DSS7_Pos             11
#define NVIC_ISER1_DSS6_Pos             10
#define NVIC_ISER1_DSS5_Pos             9
#define NVIC_ISER1_DSS4_Pos             8
#define NVIC_ISER1_DSS3_Pos             7
#define NVIC_ISER1_DSS2_Pos             6
#define NVIC_ISER1_DSS1_Pos             5
#define NVIC_ISER1_DSS0_Pos             4
#define NVIC_ISER1_PCM_ERROR_Pos        3
#define NVIC_ISER1_PCM_TX_Pos           2
#define NVIC_ISER1_PCM_RX_Pos           1
#define NVIC_ISER1_DMIC_OD_ERROR_Pos    0

/* NVIC_ISER1 settings */
#define NVIC_DMIC_OD_ERROR_INT_ENABLE   ((uint32_t)(0x1U << NVIC_ISER1_DMIC_OD_ERROR_Pos))

#define NVIC_PCM_RX_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER1_PCM_RX_Pos))

#define NVIC_PCM_TX_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER1_PCM_TX_Pos))

#define NVIC_PCM_ERROR_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER1_PCM_ERROR_Pos))

#define NVIC_DSS0_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER1_DSS0_Pos))

#define NVIC_DSS1_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER1_DSS1_Pos))

#define NVIC_DSS2_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER1_DSS2_Pos))

#define NVIC_DSS3_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER1_DSS3_Pos))

#define NVIC_DSS4_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER1_DSS4_Pos))

#define NVIC_DSS5_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER1_DSS5_Pos))

#define NVIC_DSS6_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER1_DSS6_Pos))

#define NVIC_DSS7_INT_ENABLE            ((uint32_t)(0x1U << NVIC_ISER1_DSS7_Pos))

#define NVIC_BLE_CSCNT_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER1_BLE_CSCNT_Pos))

#define NVIC_BLE_SLP_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER1_BLE_SLP_Pos))

#define NVIC_BLE_RX_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER1_BLE_RX_Pos))

#define NVIC_BLE_EVENT_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER1_BLE_EVENT_Pos))

#define NVIC_BLE_CRYPT_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER1_BLE_CRYPT_Pos))

#define NVIC_BLE_ERROR_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER1_BLE_ERROR_Pos))

#define NVIC_BLE_GROSSTGTIM_INT_ENABLE  ((uint32_t)(0x1U << NVIC_ISER1_BLE_GROSSTGTIM_Pos))

#define NVIC_BLE_FINETGTIM_INT_ENABLE   ((uint32_t)(0x1U << NVIC_ISER1_BLE_FINETGTIM_Pos))

#define NVIC_BLE_SW_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER1_BLE_SW_Pos))

#define NVIC_BLE_COEX_RX_TX_INT_ENABLE  ((uint32_t)(0x1U << NVIC_ISER1_BLE_COEX_RX_TX_Pos))

#define NVIC_BLE_COEX_IN_PROCESS_INT_ENABLE ((uint32_t)(0x1U << NVIC_ISER1_BLE_COEX_IN_PROCESS_Pos))

#define NVIC_RF_TX_INT_ENABLE           ((uint32_t)(0x1U << NVIC_ISER1_RF_TX_Pos))

#define NVIC_RF_RXSTOP_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER1_RF_RXSTOP_Pos))

#define NVIC_RF_RECEIVED_INT_ENABLE     ((uint32_t)(0x1U << NVIC_ISER1_RF_RECEIVED_Pos))

#define NVIC_RF_SYNC_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER1_RF_SYNC_Pos))

#define NVIC_RF_TXFIFO_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER1_RF_TXFIFO_Pos))

#define NVIC_RF_RXFIFO_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER1_RF_RXFIFO_Pos))

#define NVIC_ASRC_ERROR_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER1_ASRC_ERROR_Pos))

#define NVIC_ASRC_IN_INT_ENABLE         ((uint32_t)(0x1U << NVIC_ISER1_ASRC_IN_Pos))

#define NVIC_ASRC_OUT_INT_ENABLE        ((uint32_t)(0x1U << NVIC_ISER1_ASRC_OUT_Pos))

/* NVIC_ISER2 bit positions */
#define NVIC_ISER2_BLE_AUDIO2_Pos       8
#define NVIC_ISER2_BLE_AUDIO1_Pos       7
#define NVIC_ISER2_BLE_AUDIO0_Pos       6
#define NVIC_ISER2_MEM_ERROR_Pos        5
#define NVIC_ISER2_FLASH_ECC_Pos        4
#define NVIC_ISER2_FLASH_COPY_Pos       3
#define NVIC_ISER2_CLKDET_Pos           2
#define NVIC_ISER2_AUDIO_SINK_PERIOD_Pos 1
#define NVIC_ISER2_AUDIO_SINK_DELAY_Pos 0

/* NVIC_ISER2 settings */
#define NVIC_AUDIO_SINK_DELAY_INT_ENABLE ((uint32_t)(0x1U << NVIC_ISER2_AUDIO_SINK_DELAY_Pos))

#define NVIC_AUDIO_SINK_PERIOD_INT_ENABLE ((uint32_t)(0x1U << NVIC_ISER2_AUDIO_SINK_PERIOD_Pos))

#define NVIC_CLKDET_INT_ENABLE          ((uint32_t)(0x1U << NVIC_ISER2_CLKDET_Pos))

#define NVIC_FLASH_COPY_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER2_FLASH_COPY_Pos))

#define NVIC_FLASH_ECC_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER2_FLASH_ECC_Pos))

#define NVIC_MEM_ERROR_INT_ENABLE       ((uint32_t)(0x1U << NVIC_ISER2_MEM_ERROR_Pos))

#define NVIC_BLE_AUDIO0_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER2_BLE_AUDIO0_Pos))

#define NVIC_BLE_AUDIO1_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER2_BLE_AUDIO1_Pos))

#define NVIC_BLE_AUDIO2_INT_ENABLE      ((uint32_t)(0x1U << NVIC_ISER2_BLE_AUDIO2_Pos))

/* NVIC_ICER0 bit positions */
#define NVIC_ICER0_DMIC_OUT_OD_IN_Pos   31
#define NVIC_ICER0_UART_ERROR_Pos       30
#define NVIC_ICER0_UART_TX_Pos          29
#define NVIC_ICER0_UART_RX_Pos          28
#define NVIC_ICER0_I2C_Pos              27
#define NVIC_ICER0_SPI1_ERROR_Pos       26
#define NVIC_ICER0_SPI1_TX_Pos          25
#define NVIC_ICER0_SPI1_RX_Pos          24
#define NVIC_ICER0_SPI0_ERROR_Pos       23
#define NVIC_ICER0_SPI0_TX_Pos          22
#define NVIC_ICER0_SPI0_RX_Pos          21
#define NVIC_ICER0_WATCHDOG_Pos         20
#define NVIC_ICER0_DIO3_Pos             19
#define NVIC_ICER0_DIO2_Pos             18
#define NVIC_ICER0_DIO1_Pos             17
#define NVIC_ICER0_DIO0_Pos             16
#define NVIC_ICER0_DMA7_Pos             15
#define NVIC_ICER0_DMA6_Pos             14
#define NVIC_ICER0_DMA5_Pos             13
#define NVIC_ICER0_DMA4_Pos             12
#define NVIC_ICER0_DMA3_Pos             11
#define NVIC_ICER0_DMA2_Pos             10
#define NVIC_ICER0_DMA1_Pos             9
#define NVIC_ICER0_DMA0_Pos             8
#define NVIC_ICER0_TIMER3_Pos           7
#define NVIC_ICER0_TIMER2_Pos           6
#define NVIC_ICER0_TIMER1_Pos           5
#define NVIC_ICER0_TIMER0_Pos           4
#define NVIC_ICER0_ADC_BATMON_Pos       3
#define NVIC_ICER0_RTC_CLOCK_Pos        2
#define NVIC_ICER0_RTC_ALARM_Pos        1
#define NVIC_ICER0_WAKEUP_Pos           0

/* NVIC_ICER0 settings */
#define NVIC_WAKEUP_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER0_WAKEUP_Pos))

#define NVIC_RTC_ALARM_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER0_RTC_ALARM_Pos))

#define NVIC_RTC_CLOCK_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER0_RTC_CLOCK_Pos))

#define NVIC_ADC_BATMON_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER0_ADC_BATMON_Pos))

#define NVIC_TIMER0_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER0_TIMER0_Pos))

#define NVIC_TIMER1_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER0_TIMER1_Pos))

#define NVIC_TIMER2_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER0_TIMER2_Pos))

#define NVIC_TIMER3_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER0_TIMER3_Pos))

#define NVIC_DMA0_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DMA0_Pos))

#define NVIC_DMA1_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DMA1_Pos))

#define NVIC_DMA2_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DMA2_Pos))

#define NVIC_DMA3_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DMA3_Pos))

#define NVIC_DMA4_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DMA4_Pos))

#define NVIC_DMA5_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DMA5_Pos))

#define NVIC_DMA6_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DMA6_Pos))

#define NVIC_DMA7_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DMA7_Pos))

#define NVIC_DIO0_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DIO0_Pos))

#define NVIC_DIO1_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DIO1_Pos))

#define NVIC_DIO2_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DIO2_Pos))

#define NVIC_DIO3_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER0_DIO3_Pos))

#define NVIC_WATCHDOG_INT_DISABLE       ((uint32_t)(0x1U << NVIC_ICER0_WATCHDOG_Pos))

#define NVIC_SPI0_RX_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER0_SPI0_RX_Pos))

#define NVIC_SPI0_TX_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER0_SPI0_TX_Pos))

#define NVIC_SPI0_ERROR_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER0_SPI0_ERROR_Pos))

#define NVIC_SPI1_RX_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER0_SPI1_RX_Pos))

#define NVIC_SPI1_TX_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER0_SPI1_TX_Pos))

#define NVIC_SPI1_ERROR_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER0_SPI1_ERROR_Pos))

#define NVIC_I2C_INT_DISABLE            ((uint32_t)(0x1U << NVIC_ICER0_I2C_Pos))

#define NVIC_UART_RX_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER0_UART_RX_Pos))

#define NVIC_UART_TX_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER0_UART_TX_Pos))

#define NVIC_UART_ERROR_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER0_UART_ERROR_Pos))

#define NVIC_DMIC_OUT_OD_IN_INT_DISABLE ((uint32_t)(0x1U << NVIC_ICER0_DMIC_OUT_OD_IN_Pos))

/* NVIC_ICER1 bit positions */
#define NVIC_ICER1_ASRC_OUT_Pos         31
#define NVIC_ICER1_ASRC_IN_Pos          30
#define NVIC_ICER1_ASRC_ERROR_Pos       29
#define NVIC_ICER1_RF_RXFIFO_Pos        28
#define NVIC_ICER1_RF_TXFIFO_Pos        27
#define NVIC_ICER1_RF_SYNC_Pos          26
#define NVIC_ICER1_RF_RECEIVED_Pos      25
#define NVIC_ICER1_RF_RXSTOP_Pos        24
#define NVIC_ICER1_RF_TX_Pos            23
#define NVIC_ICER1_BLE_COEX_IN_PROCESS_Pos 22
#define NVIC_ICER1_BLE_COEX_RX_TX_Pos   21
#define NVIC_ICER1_BLE_SW_Pos           20
#define NVIC_ICER1_BLE_FINETGTIM_Pos    19
#define NVIC_ICER1_BLE_GROSSTGTIM_Pos   18
#define NVIC_ICER1_BLE_ERROR_Pos        17
#define NVIC_ICER1_BLE_CRYPT_Pos        16
#define NVIC_ICER1_BLE_EVENT_Pos        15
#define NVIC_ICER1_BLE_RX_Pos           14
#define NVIC_ICER1_BLE_SLP_Pos          13
#define NVIC_ICER1_BLE_CSCNT_Pos        12
#define NVIC_ICER1_DSS7_Pos             11
#define NVIC_ICER1_DSS6_Pos             10
#define NVIC_ICER1_DSS5_Pos             9
#define NVIC_ICER1_DSS4_Pos             8
#define NVIC_ICER1_DSS3_Pos             7
#define NVIC_ICER1_DSS2_Pos             6
#define NVIC_ICER1_DSS1_Pos             5
#define NVIC_ICER1_DSS0_Pos             4
#define NVIC_ICER1_PCM_ERROR_Pos        3
#define NVIC_ICER1_PCM_TX_Pos           2
#define NVIC_ICER1_PCM_RX_Pos           1
#define NVIC_ICER1_DMIC_OD_ERROR_Pos    0

/* NVIC_ICER1 settings */
#define NVIC_DMIC_OD_ERROR_INT_DISABLE  ((uint32_t)(0x1U << NVIC_ICER1_DMIC_OD_ERROR_Pos))

#define NVIC_PCM_RX_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER1_PCM_RX_Pos))

#define NVIC_PCM_TX_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER1_PCM_TX_Pos))

#define NVIC_PCM_ERROR_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER1_PCM_ERROR_Pos))

#define NVIC_DSS0_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER1_DSS0_Pos))

#define NVIC_DSS1_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER1_DSS1_Pos))

#define NVIC_DSS2_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER1_DSS2_Pos))

#define NVIC_DSS3_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER1_DSS3_Pos))

#define NVIC_DSS4_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER1_DSS4_Pos))

#define NVIC_DSS5_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER1_DSS5_Pos))

#define NVIC_DSS6_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER1_DSS6_Pos))

#define NVIC_DSS7_INT_DISABLE           ((uint32_t)(0x1U << NVIC_ICER1_DSS7_Pos))

#define NVIC_BLE_CSCNT_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER1_BLE_CSCNT_Pos))

#define NVIC_BLE_SLP_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER1_BLE_SLP_Pos))

#define NVIC_BLE_RX_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER1_BLE_RX_Pos))

#define NVIC_BLE_EVENT_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER1_BLE_EVENT_Pos))

#define NVIC_BLE_CRYPT_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER1_BLE_CRYPT_Pos))

#define NVIC_BLE_ERROR_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER1_BLE_ERROR_Pos))

#define NVIC_BLE_GROSSTGTIM_INT_DISABLE ((uint32_t)(0x1U << NVIC_ICER1_BLE_GROSSTGTIM_Pos))

#define NVIC_BLE_FINETGTIM_INT_DISABLE  ((uint32_t)(0x1U << NVIC_ICER1_BLE_FINETGTIM_Pos))

#define NVIC_BLE_SW_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER1_BLE_SW_Pos))

#define NVIC_BLE_COEX_RX_TX_INT_DISABLE ((uint32_t)(0x1U << NVIC_ICER1_BLE_COEX_RX_TX_Pos))

#define NVIC_BLE_COEX_IN_PROCESS_INT_DISABLE ((uint32_t)(0x1U << NVIC_ICER1_BLE_COEX_IN_PROCESS_Pos))

#define NVIC_RF_TX_INT_DISABLE          ((uint32_t)(0x1U << NVIC_ICER1_RF_TX_Pos))

#define NVIC_RF_RXSTOP_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER1_RF_RXSTOP_Pos))

#define NVIC_RF_RECEIVED_INT_DISABLE    ((uint32_t)(0x1U << NVIC_ICER1_RF_RECEIVED_Pos))

#define NVIC_RF_SYNC_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER1_RF_SYNC_Pos))

#define NVIC_RF_TXFIFO_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER1_RF_TXFIFO_Pos))

#define NVIC_RF_RXFIFO_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER1_RF_RXFIFO_Pos))

#define NVIC_ASRC_ERROR_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER1_ASRC_ERROR_Pos))

#define NVIC_ASRC_IN_INT_DISABLE        ((uint32_t)(0x1U << NVIC_ICER1_ASRC_IN_Pos))

#define NVIC_ASRC_OUT_INT_DISABLE       ((uint32_t)(0x1U << NVIC_ICER1_ASRC_OUT_Pos))

/* NVIC_ICER2 bit positions */
#define NVIC_ICER2_BLE_AUDIO2_Pos       8
#define NVIC_ICER2_BLE_AUDIO1_Pos       7
#define NVIC_ICER2_BLE_AUDIO0_Pos       6
#define NVIC_ICER2_MEM_ERROR_Pos        5
#define NVIC_ICER2_FLASH_ECC_Pos        4
#define NVIC_ICER2_FLASH_COPY_Pos       3
#define NVIC_ICER2_CLKDET_Pos           2
#define NVIC_ICER2_AUDIO_SINK_PERIOD_Pos 1
#define NVIC_ICER2_AUDIO_SINK_DELAY_Pos 0

/* NVIC_ICER2 settings */
#define NVIC_AUDIO_SINK_DELAY_INT_DISABLE ((uint32_t)(0x1U << NVIC_ICER2_AUDIO_SINK_DELAY_Pos))

#define NVIC_AUDIO_SINK_PERIOD_INT_DISABLE ((uint32_t)(0x1U << NVIC_ICER2_AUDIO_SINK_PERIOD_Pos))

#define NVIC_CLKDET_INT_DISABLE         ((uint32_t)(0x1U << NVIC_ICER2_CLKDET_Pos))

#define NVIC_FLASH_COPY_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER2_FLASH_COPY_Pos))

#define NVIC_FLASH_ECC_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER2_FLASH_ECC_Pos))

#define NVIC_MEM_ERROR_INT_DISABLE      ((uint32_t)(0x1U << NVIC_ICER2_MEM_ERROR_Pos))

#define NVIC_BLE_AUDIO0_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER2_BLE_AUDIO0_Pos))

#define NVIC_BLE_AUDIO1_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER2_BLE_AUDIO1_Pos))

#define NVIC_BLE_AUDIO2_INT_DISABLE     ((uint32_t)(0x1U << NVIC_ICER2_BLE_AUDIO2_Pos))

/* NVIC_ISPR0 bit positions */
#define NVIC_ISPR0_DMIC_OUT_OD_IN_Pos   31
#define NVIC_ISPR0_UART_ERROR_Pos       30
#define NVIC_ISPR0_UART_TX_Pos          29
#define NVIC_ISPR0_UART_RX_Pos          28
#define NVIC_ISPR0_I2C_Pos              27
#define NVIC_ISPR0_SPI1_ERROR_Pos       26
#define NVIC_ISPR0_SPI1_TX_Pos          25
#define NVIC_ISPR0_SPI1_RX_Pos          24
#define NVIC_ISPR0_SPI0_ERROR_Pos       23
#define NVIC_ISPR0_SPI0_TX_Pos          22
#define NVIC_ISPR0_SPI0_RX_Pos          21
#define NVIC_ISPR0_WATCHDOG_Pos         20
#define NVIC_ISPR0_DIO3_Pos             19
#define NVIC_ISPR0_DIO2_Pos             18
#define NVIC_ISPR0_DIO1_Pos             17
#define NVIC_ISPR0_DIO0_Pos             16
#define NVIC_ISPR0_DMA7_Pos             15
#define NVIC_ISPR0_DMA6_Pos             14
#define NVIC_ISPR0_DMA5_Pos             13
#define NVIC_ISPR0_DMA4_Pos             12
#define NVIC_ISPR0_DMA3_Pos             11
#define NVIC_ISPR0_DMA2_Pos             10
#define NVIC_ISPR0_DMA1_Pos             9
#define NVIC_ISPR0_DMA0_Pos             8
#define NVIC_ISPR0_TIMER3_Pos           7
#define NVIC_ISPR0_TIMER2_Pos           6
#define NVIC_ISPR0_TIMER1_Pos           5
#define NVIC_ISPR0_TIMER0_Pos           4
#define NVIC_ISPR0_ADC_BATMON_Pos       3
#define NVIC_ISPR0_RTC_CLOCK_Pos        2
#define NVIC_ISPR0_RTC_ALARM_Pos        1
#define NVIC_ISPR0_WAKEUP_Pos           0

/* NVIC_ISPR0 settings */
#define NVIC_WAKEUP_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR0_WAKEUP_Pos))

#define NVIC_RTC_ALARM_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR0_RTC_ALARM_Pos))

#define NVIC_RTC_CLOCK_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR0_RTC_CLOCK_Pos))

#define NVIC_ADC_BATMON_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR0_ADC_BATMON_Pos))

#define NVIC_TIMER0_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR0_TIMER0_Pos))

#define NVIC_TIMER1_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR0_TIMER1_Pos))

#define NVIC_TIMER2_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR0_TIMER2_Pos))

#define NVIC_TIMER3_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR0_TIMER3_Pos))

#define NVIC_DMA0_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DMA0_Pos))

#define NVIC_DMA1_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DMA1_Pos))

#define NVIC_DMA2_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DMA2_Pos))

#define NVIC_DMA3_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DMA3_Pos))

#define NVIC_DMA4_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DMA4_Pos))

#define NVIC_DMA5_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DMA5_Pos))

#define NVIC_DMA6_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DMA6_Pos))

#define NVIC_DMA7_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DMA7_Pos))

#define NVIC_DIO0_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DIO0_Pos))

#define NVIC_DIO1_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DIO1_Pos))

#define NVIC_DIO2_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DIO2_Pos))

#define NVIC_DIO3_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR0_DIO3_Pos))

#define NVIC_WATCHDOG_INT_SETPEND       ((uint32_t)(0x1U << NVIC_ISPR0_WATCHDOG_Pos))

#define NVIC_SPI0_RX_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR0_SPI0_RX_Pos))

#define NVIC_SPI0_TX_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR0_SPI0_TX_Pos))

#define NVIC_SPI0_ERROR_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR0_SPI0_ERROR_Pos))

#define NVIC_SPI1_RX_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR0_SPI1_RX_Pos))

#define NVIC_SPI1_TX_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR0_SPI1_TX_Pos))

#define NVIC_SPI1_ERROR_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR0_SPI1_ERROR_Pos))

#define NVIC_I2C_INT_SETPEND            ((uint32_t)(0x1U << NVIC_ISPR0_I2C_Pos))

#define NVIC_UART_RX_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR0_UART_RX_Pos))

#define NVIC_UART_TX_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR0_UART_TX_Pos))

#define NVIC_UART_ERROR_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR0_UART_ERROR_Pos))

#define NVIC_DMIC_OUT_OD_IN_INT_SETPEND ((uint32_t)(0x1U << NVIC_ISPR0_DMIC_OUT_OD_IN_Pos))

/* NVIC_ISPR1 bit positions */
#define NVIC_ISPR1_ASRC_OUT_Pos         31
#define NVIC_ISPR1_ASRC_IN_Pos          30
#define NVIC_ISPR1_ASRC_ERROR_Pos       29
#define NVIC_ISPR1_RF_RXFIFO_Pos        28
#define NVIC_ISPR1_RF_TXFIFO_Pos        27
#define NVIC_ISPR1_RF_SYNC_Pos          26
#define NVIC_ISPR1_RF_RECEIVED_Pos      25
#define NVIC_ISPR1_RF_RXSTOP_Pos        24
#define NVIC_ISPR1_RF_TX_Pos            23
#define NVIC_ISPR1_BLE_COEX_IN_PROCESS_Pos 22
#define NVIC_ISPR1_BLE_COEX_RX_TX_Pos   21
#define NVIC_ISPR1_BLE_SW_Pos           20
#define NVIC_ISPR1_BLE_FINETGTIM_Pos    19
#define NVIC_ISPR1_BLE_GROSSTGTIM_Pos   18
#define NVIC_ISPR1_BLE_ERROR_Pos        17
#define NVIC_ISPR1_BLE_CRYPT_Pos        16
#define NVIC_ISPR1_BLE_EVENT_Pos        15
#define NVIC_ISPR1_BLE_RX_Pos           14
#define NVIC_ISPR1_BLE_SLP_Pos          13
#define NVIC_ISPR1_BLE_CSCNT_Pos        12
#define NVIC_ISPR1_DSS7_Pos             11
#define NVIC_ISPR1_DSS6_Pos             10
#define NVIC_ISPR1_DSS5_Pos             9
#define NVIC_ISPR1_DSS4_Pos             8
#define NVIC_ISPR1_DSS3_Pos             7
#define NVIC_ISPR1_DSS2_Pos             6
#define NVIC_ISPR1_DSS1_Pos             5
#define NVIC_ISPR1_DSS0_Pos             4
#define NVIC_ISPR1_PCM_ERROR_Pos        3
#define NVIC_ISPR1_PCM_TX_Pos           2
#define NVIC_ISPR1_PCM_RX_Pos           1
#define NVIC_ISPR1_DMIC_OD_ERROR_Pos    0

/* NVIC_ISPR1 settings */
#define NVIC_DMIC_OD_ERROR_INT_SETPEND  ((uint32_t)(0x1U << NVIC_ISPR1_DMIC_OD_ERROR_Pos))

#define NVIC_PCM_RX_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR1_PCM_RX_Pos))

#define NVIC_PCM_TX_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR1_PCM_TX_Pos))

#define NVIC_PCM_ERROR_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR1_PCM_ERROR_Pos))

#define NVIC_DSS0_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR1_DSS0_Pos))

#define NVIC_DSS1_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR1_DSS1_Pos))

#define NVIC_DSS2_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR1_DSS2_Pos))

#define NVIC_DSS3_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR1_DSS3_Pos))

#define NVIC_DSS4_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR1_DSS4_Pos))

#define NVIC_DSS5_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR1_DSS5_Pos))

#define NVIC_DSS6_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR1_DSS6_Pos))

#define NVIC_DSS7_INT_SETPEND           ((uint32_t)(0x1U << NVIC_ISPR1_DSS7_Pos))

#define NVIC_BLE_CSCNT_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR1_BLE_CSCNT_Pos))

#define NVIC_BLE_SLP_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR1_BLE_SLP_Pos))

#define NVIC_BLE_RX_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR1_BLE_RX_Pos))

#define NVIC_BLE_EVENT_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR1_BLE_EVENT_Pos))

#define NVIC_BLE_CRYPT_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR1_BLE_CRYPT_Pos))

#define NVIC_BLE_ERROR_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR1_BLE_ERROR_Pos))

#define NVIC_BLE_GROSSTGTIM_INT_SETPEND ((uint32_t)(0x1U << NVIC_ISPR1_BLE_GROSSTGTIM_Pos))

#define NVIC_BLE_FINETGTIM_INT_SETPEND  ((uint32_t)(0x1U << NVIC_ISPR1_BLE_FINETGTIM_Pos))

#define NVIC_BLE_SW_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR1_BLE_SW_Pos))

#define NVIC_BLE_COEX_RX_TX_INT_SETPEND ((uint32_t)(0x1U << NVIC_ISPR1_BLE_COEX_RX_TX_Pos))

#define NVIC_BLE_COEX_IN_PROCESS_INT_SETPEND ((uint32_t)(0x1U << NVIC_ISPR1_BLE_COEX_IN_PROCESS_Pos))

#define NVIC_RF_TX_INT_SETPEND          ((uint32_t)(0x1U << NVIC_ISPR1_RF_TX_Pos))

#define NVIC_RF_RXSTOP_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR1_RF_RXSTOP_Pos))

#define NVIC_RF_RECEIVED_INT_SETPEND    ((uint32_t)(0x1U << NVIC_ISPR1_RF_RECEIVED_Pos))

#define NVIC_RF_SYNC_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR1_RF_SYNC_Pos))

#define NVIC_RF_TXFIFO_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR1_RF_TXFIFO_Pos))

#define NVIC_RF_RXFIFO_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR1_RF_RXFIFO_Pos))

#define NVIC_ASRC_ERROR_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR1_ASRC_ERROR_Pos))

#define NVIC_ASRC_IN_INT_SETPEND        ((uint32_t)(0x1U << NVIC_ISPR1_ASRC_IN_Pos))

#define NVIC_ASRC_OUT_INT_SETPEND       ((uint32_t)(0x1U << NVIC_ISPR1_ASRC_OUT_Pos))

/* NVIC_ISPR2 bit positions */
#define NVIC_ISPR2_BLE_AUDIO2_Pos       8
#define NVIC_ISPR2_BLE_AUDIO1_Pos       7
#define NVIC_ISPR2_BLE_AUDIO0_Pos       6
#define NVIC_ISPR2_MEM_ERROR_Pos        5
#define NVIC_ISPR2_FLASH_ECC_Pos        4
#define NVIC_ISPR2_FLASH_COPY_Pos       3
#define NVIC_ISPR2_CLKDET_Pos           2
#define NVIC_ISPR2_AUDIO_SINK_PERIOD_Pos 1
#define NVIC_ISPR2_AUDIO_SINK_DELAY_Pos 0

/* NVIC_ISPR2 settings */
#define NVIC_AUDIO_SINK_DELAY_INT_SETPEND ((uint32_t)(0x1U << NVIC_ISPR2_AUDIO_SINK_DELAY_Pos))

#define NVIC_AUDIO_SINK_PERIOD_INT_SETPEND ((uint32_t)(0x1U << NVIC_ISPR2_AUDIO_SINK_PERIOD_Pos))

#define NVIC_CLKDET_INT_SETPEND         ((uint32_t)(0x1U << NVIC_ISPR2_CLKDET_Pos))

#define NVIC_FLASH_COPY_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR2_FLASH_COPY_Pos))

#define NVIC_FLASH_ECC_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR2_FLASH_ECC_Pos))

#define NVIC_MEM_ERROR_INT_SETPEND      ((uint32_t)(0x1U << NVIC_ISPR2_MEM_ERROR_Pos))

#define NVIC_BLE_AUDIO0_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR2_BLE_AUDIO0_Pos))

#define NVIC_BLE_AUDIO1_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR2_BLE_AUDIO1_Pos))

#define NVIC_BLE_AUDIO2_INT_SETPEND     ((uint32_t)(0x1U << NVIC_ISPR2_BLE_AUDIO2_Pos))

/* NVIC_ICPR0 bit positions */
#define NVIC_ICPR0_DMIC_OUT_OD_IN_Pos   31
#define NVIC_ICPR0_UART_ERROR_Pos       30
#define NVIC_ICPR0_UART_TX_Pos          29
#define NVIC_ICPR0_UART_RX_Pos          28
#define NVIC_ICPR0_I2C_Pos              27
#define NVIC_ICPR0_SPI1_ERROR_Pos       26
#define NVIC_ICPR0_SPI1_TX_Pos          25
#define NVIC_ICPR0_SPI1_RX_Pos          24
#define NVIC_ICPR0_SPI0_ERROR_Pos       23
#define NVIC_ICPR0_SPI0_TX_Pos          22
#define NVIC_ICPR0_SPI0_RX_Pos          21
#define NVIC_ICPR0_WATCHDOG_Pos         20
#define NVIC_ICPR0_DIO3_Pos             19
#define NVIC_ICPR0_DIO2_Pos             18
#define NVIC_ICPR0_DIO1_Pos             17
#define NVIC_ICPR0_DIO0_Pos             16
#define NVIC_ICPR0_DMA7_Pos             15
#define NVIC_ICPR0_DMA6_Pos             14
#define NVIC_ICPR0_DMA5_Pos             13
#define NVIC_ICPR0_DMA4_Pos             12
#define NVIC_ICPR0_DMA3_Pos             11
#define NVIC_ICPR0_DMA2_Pos             10
#define NVIC_ICPR0_DMA1_Pos             9
#define NVIC_ICPR0_DMA0_Pos             8
#define NVIC_ICPR0_TIMER3_Pos           7
#define NVIC_ICPR0_TIMER2_Pos           6
#define NVIC_ICPR0_TIMER1_Pos           5
#define NVIC_ICPR0_TIMER0_Pos           4
#define NVIC_ICPR0_ADC_BATMON_Pos       3
#define NVIC_ICPR0_RTC_CLOCK_Pos        2
#define NVIC_ICPR0_RTC_ALARM_Pos        1
#define NVIC_ICPR0_WAKEUP_Pos           0

/* NVIC_ICPR0 settings */
#define NVIC_WAKEUP_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR0_WAKEUP_Pos))

#define NVIC_RTC_ALARM_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR0_RTC_ALARM_Pos))

#define NVIC_RTC_CLOCK_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR0_RTC_CLOCK_Pos))

#define NVIC_ADC_BATMON_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR0_ADC_BATMON_Pos))

#define NVIC_TIMER0_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR0_TIMER0_Pos))

#define NVIC_TIMER1_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR0_TIMER1_Pos))

#define NVIC_TIMER2_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR0_TIMER2_Pos))

#define NVIC_TIMER3_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR0_TIMER3_Pos))

#define NVIC_DMA0_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DMA0_Pos))

#define NVIC_DMA1_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DMA1_Pos))

#define NVIC_DMA2_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DMA2_Pos))

#define NVIC_DMA3_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DMA3_Pos))

#define NVIC_DMA4_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DMA4_Pos))

#define NVIC_DMA5_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DMA5_Pos))

#define NVIC_DMA6_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DMA6_Pos))

#define NVIC_DMA7_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DMA7_Pos))

#define NVIC_DIO0_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DIO0_Pos))

#define NVIC_DIO1_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DIO1_Pos))

#define NVIC_DIO2_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DIO2_Pos))

#define NVIC_DIO3_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR0_DIO3_Pos))

#define NVIC_WATCHDOG_INT_CLRPEND       ((uint32_t)(0x1U << NVIC_ICPR0_WATCHDOG_Pos))

#define NVIC_SPI0_RX_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR0_SPI0_RX_Pos))

#define NVIC_SPI0_TX_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR0_SPI0_TX_Pos))

#define NVIC_SPI0_ERROR_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR0_SPI0_ERROR_Pos))

#define NVIC_SPI1_RX_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR0_SPI1_RX_Pos))

#define NVIC_SPI1_TX_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR0_SPI1_TX_Pos))

#define NVIC_SPI1_ERROR_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR0_SPI1_ERROR_Pos))

#define NVIC_I2C_INT_CLRPEND            ((uint32_t)(0x1U << NVIC_ICPR0_I2C_Pos))

#define NVIC_UART_RX_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR0_UART_RX_Pos))

#define NVIC_UART_TX_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR0_UART_TX_Pos))

#define NVIC_UART_ERROR_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR0_UART_ERROR_Pos))

#define NVIC_DMIC_OUT_OD_IN_INT_CLRPEND ((uint32_t)(0x1U << NVIC_ICPR0_DMIC_OUT_OD_IN_Pos))

/* NVIC_ICPR1 bit positions */
#define NVIC_ICPR1_ASRC_OUT_Pos         31
#define NVIC_ICPR1_ASRC_IN_Pos          30
#define NVIC_ICPR1_ASRC_ERROR_Pos       29
#define NVIC_ICPR1_RF_RXFIFO_Pos        28
#define NVIC_ICPR1_RF_TXFIFO_Pos        27
#define NVIC_ICPR1_RF_SYNC_Pos          26
#define NVIC_ICPR1_RF_RECEIVED_Pos      25
#define NVIC_ICPR1_RF_RXSTOP_Pos        24
#define NVIC_ICPR1_RF_TX_Pos            23
#define NVIC_ICPR1_BLE_COEX_IN_PROCESS_Pos 22
#define NVIC_ICPR1_BLE_COEX_RX_TX_Pos   21
#define NVIC_ICPR1_BLE_SW_Pos           20
#define NVIC_ICPR1_BLE_FINETGTIM_Pos    19
#define NVIC_ICPR1_BLE_GROSSTGTIM_Pos   18
#define NVIC_ICPR1_BLE_ERROR_Pos        17
#define NVIC_ICPR1_BLE_CRYPT_Pos        16
#define NVIC_ICPR1_BLE_EVENT_Pos        15
#define NVIC_ICPR1_BLE_RX_Pos           14
#define NVIC_ICPR1_BLE_SLP_Pos          13
#define NVIC_ICPR1_BLE_CSCNT_Pos        12
#define NVIC_ICPR1_DSS7_Pos             11
#define NVIC_ICPR1_DSS6_Pos             10
#define NVIC_ICPR1_DSS5_Pos             9
#define NVIC_ICPR1_DSS4_Pos             8
#define NVIC_ICPR1_DSS3_Pos             7
#define NVIC_ICPR1_DSS2_Pos             6
#define NVIC_ICPR1_DSS1_Pos             5
#define NVIC_ICPR1_DSS0_Pos             4
#define NVIC_ICPR1_PCM_ERROR_Pos        3
#define NVIC_ICPR1_PCM_TX_Pos           2
#define NVIC_ICPR1_PCM_RX_Pos           1
#define NVIC_ICPR1_DMIC_OD_ERROR_Pos    0

/* NVIC_ICPR1 settings */
#define NVIC_DMIC_OD_ERROR_INT_CLRPEND  ((uint32_t)(0x1U << NVIC_ICPR1_DMIC_OD_ERROR_Pos))

#define NVIC_PCM_RX_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR1_PCM_RX_Pos))

#define NVIC_PCM_TX_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR1_PCM_TX_Pos))

#define NVIC_PCM_ERROR_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR1_PCM_ERROR_Pos))

#define NVIC_DSS0_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR1_DSS0_Pos))

#define NVIC_DSS1_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR1_DSS1_Pos))

#define NVIC_DSS2_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR1_DSS2_Pos))

#define NVIC_DSS3_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR1_DSS3_Pos))

#define NVIC_DSS4_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR1_DSS4_Pos))

#define NVIC_DSS5_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR1_DSS5_Pos))

#define NVIC_DSS6_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR1_DSS6_Pos))

#define NVIC_DSS7_INT_CLRPEND           ((uint32_t)(0x1U << NVIC_ICPR1_DSS7_Pos))

#define NVIC_BLE_CSCNT_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR1_BLE_CSCNT_Pos))

#define NVIC_BLE_SLP_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR1_BLE_SLP_Pos))

#define NVIC_BLE_RX_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR1_BLE_RX_Pos))

#define NVIC_BLE_EVENT_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR1_BLE_EVENT_Pos))

#define NVIC_BLE_CRYPT_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR1_BLE_CRYPT_Pos))

#define NVIC_BLE_ERROR_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR1_BLE_ERROR_Pos))

#define NVIC_BLE_GROSSTGTIM_INT_CLRPEND ((uint32_t)(0x1U << NVIC_ICPR1_BLE_GROSSTGTIM_Pos))

#define NVIC_BLE_FINETGTIM_INT_CLRPEND  ((uint32_t)(0x1U << NVIC_ICPR1_BLE_FINETGTIM_Pos))

#define NVIC_BLE_SW_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR1_BLE_SW_Pos))

#define NVIC_BLE_COEX_RX_TX_INT_CLRPEND ((uint32_t)(0x1U << NVIC_ICPR1_BLE_COEX_RX_TX_Pos))

#define NVIC_BLE_COEX_IN_PROCESS_INT_CLRPEND ((uint32_t)(0x1U << NVIC_ICPR1_BLE_COEX_IN_PROCESS_Pos))

#define NVIC_RF_TX_INT_CLRPEND          ((uint32_t)(0x1U << NVIC_ICPR1_RF_TX_Pos))

#define NVIC_RF_RXSTOP_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR1_RF_RXSTOP_Pos))

#define NVIC_RF_RECEIVED_INT_CLRPEND    ((uint32_t)(0x1U << NVIC_ICPR1_RF_RECEIVED_Pos))

#define NVIC_RF_SYNC_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR1_RF_SYNC_Pos))

#define NVIC_RF_TXFIFO_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR1_RF_TXFIFO_Pos))

#define NVIC_RF_RXFIFO_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR1_RF_RXFIFO_Pos))

#define NVIC_ASRC_ERROR_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR1_ASRC_ERROR_Pos))

#define NVIC_ASRC_IN_INT_CLRPEND        ((uint32_t)(0x1U << NVIC_ICPR1_ASRC_IN_Pos))

#define NVIC_ASRC_OUT_INT_CLRPEND       ((uint32_t)(0x1U << NVIC_ICPR1_ASRC_OUT_Pos))

/* NVIC_ICPR2 bit positions */
#define NVIC_ICPR2_BLE_AUDIO2_Pos       8
#define NVIC_ICPR2_BLE_AUDIO1_Pos       7
#define NVIC_ICPR2_BLE_AUDIO0_Pos       6
#define NVIC_ICPR2_MEM_ERROR_Pos        5
#define NVIC_ICPR2_FLASH_ECC_Pos        4
#define NVIC_ICPR2_FLASH_COPY_Pos       3
#define NVIC_ICPR2_CLKDET_Pos           2
#define NVIC_ICPR2_AUDIO_SINK_PERIOD_Pos 1
#define NVIC_ICPR2_AUDIO_SINK_DELAY_Pos 0

/* NVIC_ICPR2 settings */
#define NVIC_AUDIO_SINK_DELAY_INT_CLRPEND ((uint32_t)(0x1U << NVIC_ICPR2_AUDIO_SINK_DELAY_Pos))

#define NVIC_AUDIO_SINK_PERIOD_INT_CLRPEND ((uint32_t)(0x1U << NVIC_ICPR2_AUDIO_SINK_PERIOD_Pos))

#define NVIC_CLKDET_INT_CLRPEND         ((uint32_t)(0x1U << NVIC_ICPR2_CLKDET_Pos))

#define NVIC_FLASH_COPY_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR2_FLASH_COPY_Pos))

#define NVIC_FLASH_ECC_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR2_FLASH_ECC_Pos))

#define NVIC_MEM_ERROR_INT_CLRPEND      ((uint32_t)(0x1U << NVIC_ICPR2_MEM_ERROR_Pos))

#define NVIC_BLE_AUDIO0_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR2_BLE_AUDIO0_Pos))

#define NVIC_BLE_AUDIO1_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR2_BLE_AUDIO1_Pos))

#define NVIC_BLE_AUDIO2_INT_CLRPEND     ((uint32_t)(0x1U << NVIC_ICPR2_BLE_AUDIO2_Pos))

/* NVIC_IABR0 bit positions */
#define NVIC_IABR0_DMIC_OUT_OD_IN_Pos   31
#define NVIC_IABR0_UART_ERROR_Pos       30
#define NVIC_IABR0_UART_TX_Pos          29
#define NVIC_IABR0_UART_RX_Pos          28
#define NVIC_IABR0_I2C_Pos              27
#define NVIC_IABR0_SPI1_ERROR_Pos       26
#define NVIC_IABR0_SPI1_TX_Pos          25
#define NVIC_IABR0_SPI1_RX_Pos          24
#define NVIC_IABR0_SPI0_ERROR_Pos       23
#define NVIC_IABR0_SPI0_TX_Pos          22
#define NVIC_IABR0_SPI0_RX_Pos          21
#define NVIC_IABR0_WATCHDOG_Pos         20
#define NVIC_IABR0_DIO3_Pos             19
#define NVIC_IABR0_DIO2_Pos             18
#define NVIC_IABR0_DIO1_Pos             17
#define NVIC_IABR0_DIO0_Pos             16
#define NVIC_IABR0_DMA7_Pos             15
#define NVIC_IABR0_DMA6_Pos             14
#define NVIC_IABR0_DMA5_Pos             13
#define NVIC_IABR0_DMA4_Pos             12
#define NVIC_IABR0_DMA3_Pos             11
#define NVIC_IABR0_DMA2_Pos             10
#define NVIC_IABR0_DMA1_Pos             9
#define NVIC_IABR0_DMA0_Pos             8
#define NVIC_IABR0_TIMER3_Pos           7
#define NVIC_IABR0_TIMER2_Pos           6
#define NVIC_IABR0_TIMER1_Pos           5
#define NVIC_IABR0_TIMER0_Pos           4
#define NVIC_IABR0_ADC_BATMON_Pos       3
#define NVIC_IABR0_RTC_CLOCK_Pos        2
#define NVIC_IABR0_RTC_ALARM_Pos        1
#define NVIC_IABR0_WAKEUP_Pos           0

/* NVIC_IABR0 settings */
#define NVIC_WAKEUP_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR0_WAKEUP_Pos))

#define NVIC_RTC_ALARM_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR0_RTC_ALARM_Pos))

#define NVIC_RTC_CLOCK_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR0_RTC_CLOCK_Pos))

#define NVIC_ADC_BATMON_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR0_ADC_BATMON_Pos))

#define NVIC_TIMER0_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR0_TIMER0_Pos))

#define NVIC_TIMER1_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR0_TIMER1_Pos))

#define NVIC_TIMER2_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR0_TIMER2_Pos))

#define NVIC_TIMER3_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR0_TIMER3_Pos))

#define NVIC_DMA0_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DMA0_Pos))

#define NVIC_DMA1_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DMA1_Pos))

#define NVIC_DMA2_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DMA2_Pos))

#define NVIC_DMA3_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DMA3_Pos))

#define NVIC_DMA4_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DMA4_Pos))

#define NVIC_DMA5_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DMA5_Pos))

#define NVIC_DMA6_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DMA6_Pos))

#define NVIC_DMA7_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DMA7_Pos))

#define NVIC_DIO0_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DIO0_Pos))

#define NVIC_DIO1_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DIO1_Pos))

#define NVIC_DIO2_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DIO2_Pos))

#define NVIC_DIO3_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR0_DIO3_Pos))

#define NVIC_WATCHDOG_INT_ACTIVE        ((uint32_t)(0x1U << NVIC_IABR0_WATCHDOG_Pos))

#define NVIC_SPI0_RX_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR0_SPI0_RX_Pos))

#define NVIC_SPI0_TX_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR0_SPI0_TX_Pos))

#define NVIC_SPI0_ERROR_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR0_SPI0_ERROR_Pos))

#define NVIC_SPI1_RX_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR0_SPI1_RX_Pos))

#define NVIC_SPI1_TX_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR0_SPI1_TX_Pos))

#define NVIC_SPI1_ERROR_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR0_SPI1_ERROR_Pos))

#define NVIC_I2C_INT_ACTIVE             ((uint32_t)(0x1U << NVIC_IABR0_I2C_Pos))

#define NVIC_UART_RX_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR0_UART_RX_Pos))

#define NVIC_UART_TX_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR0_UART_TX_Pos))

#define NVIC_UART_ERROR_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR0_UART_ERROR_Pos))

#define NVIC_DMIC_OUT_OD_IN_INT_ACTIVE  ((uint32_t)(0x1U << NVIC_IABR0_DMIC_OUT_OD_IN_Pos))

/* NVIC_IABR1 bit positions */
#define NVIC_IABR1_ASRC_OUT_Pos         31
#define NVIC_IABR1_ASRC_IN_Pos          30
#define NVIC_IABR1_ASRC_ERROR_Pos       29
#define NVIC_IABR1_RF_RXFIFO_Pos        28
#define NVIC_IABR1_RF_TXFIFO_Pos        27
#define NVIC_IABR1_RF_SYNC_Pos          26
#define NVIC_IABR1_RF_RECEIVED_Pos      25
#define NVIC_IABR1_RF_RXSTOP_Pos        24
#define NVIC_IABR1_RF_TX_Pos            23
#define NVIC_IABR1_BLE_COEX_IN_PROCESS_Pos 22
#define NVIC_IABR1_BLE_COEX_RX_TX_Pos   21
#define NVIC_IABR1_BLE_SW_Pos           20
#define NVIC_IABR1_BLE_FINETGTIM_Pos    19
#define NVIC_IABR1_BLE_GROSSTGTIM_Pos   18
#define NVIC_IABR1_BLE_ERROR_Pos        17
#define NVIC_IABR1_BLE_CRYPT_Pos        16
#define NVIC_IABR1_BLE_EVENT_Pos        15
#define NVIC_IABR1_BLE_RX_Pos           14
#define NVIC_IABR1_BLE_SLP_Pos          13
#define NVIC_IABR1_BLE_CSCNT_Pos        12
#define NVIC_IABR1_DSS7_Pos             11
#define NVIC_IABR1_DSS6_Pos             10
#define NVIC_IABR1_DSS5_Pos             9
#define NVIC_IABR1_DSS4_Pos             8
#define NVIC_IABR1_DSS3_Pos             7
#define NVIC_IABR1_DSS2_Pos             6
#define NVIC_IABR1_DSS1_Pos             5
#define NVIC_IABR1_DSS0_Pos             4
#define NVIC_IABR1_PCM_ERROR_Pos        3
#define NVIC_IABR1_PCM_TX_Pos           2
#define NVIC_IABR1_PCM_RX_Pos           1
#define NVIC_IABR1_DMIC_OD_ERROR_Pos    0

/* NVIC_IABR1 settings */
#define NVIC_DMIC_OD_ERROR_INT_ACTIVE   ((uint32_t)(0x1U << NVIC_IABR1_DMIC_OD_ERROR_Pos))

#define NVIC_PCM_RX_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR1_PCM_RX_Pos))

#define NVIC_PCM_TX_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR1_PCM_TX_Pos))

#define NVIC_PCM_ERROR_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR1_PCM_ERROR_Pos))

#define NVIC_DSS0_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR1_DSS0_Pos))

#define NVIC_DSS1_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR1_DSS1_Pos))

#define NVIC_DSS2_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR1_DSS2_Pos))

#define NVIC_DSS3_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR1_DSS3_Pos))

#define NVIC_DSS4_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR1_DSS4_Pos))

#define NVIC_DSS5_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR1_DSS5_Pos))

#define NVIC_DSS6_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR1_DSS6_Pos))

#define NVIC_DSS7_INT_ACTIVE            ((uint32_t)(0x1U << NVIC_IABR1_DSS7_Pos))

#define NVIC_BLE_CSCNT_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR1_BLE_CSCNT_Pos))

#define NVIC_BLE_SLP_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR1_BLE_SLP_Pos))

#define NVIC_BLE_RX_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR1_BLE_RX_Pos))

#define NVIC_BLE_EVENT_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR1_BLE_EVENT_Pos))

#define NVIC_BLE_CRYPT_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR1_BLE_CRYPT_Pos))

#define NVIC_BLE_ERROR_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR1_BLE_ERROR_Pos))

#define NVIC_BLE_GROSSTGTIM_INT_ACTIVE  ((uint32_t)(0x1U << NVIC_IABR1_BLE_GROSSTGTIM_Pos))

#define NVIC_BLE_FINETGTIM_INT_ACTIVE   ((uint32_t)(0x1U << NVIC_IABR1_BLE_FINETGTIM_Pos))

#define NVIC_BLE_SW_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR1_BLE_SW_Pos))

#define NVIC_BLE_COEX_RX_TX_INT_ACTIVE  ((uint32_t)(0x1U << NVIC_IABR1_BLE_COEX_RX_TX_Pos))

#define NVIC_BLE_COEX_IN_PROCESS_INT_ACTIVE ((uint32_t)(0x1U << NVIC_IABR1_BLE_COEX_IN_PROCESS_Pos))

#define NVIC_RF_TX_INT_ACTIVE           ((uint32_t)(0x1U << NVIC_IABR1_RF_TX_Pos))

#define NVIC_RF_RXSTOP_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR1_RF_RXSTOP_Pos))

#define NVIC_RF_RECEIVED_INT_ACTIVE     ((uint32_t)(0x1U << NVIC_IABR1_RF_RECEIVED_Pos))

#define NVIC_RF_SYNC_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR1_RF_SYNC_Pos))

#define NVIC_RF_TXFIFO_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR1_RF_TXFIFO_Pos))

#define NVIC_RF_RXFIFO_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR1_RF_RXFIFO_Pos))

#define NVIC_ASRC_ERROR_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR1_ASRC_ERROR_Pos))

#define NVIC_ASRC_IN_INT_ACTIVE         ((uint32_t)(0x1U << NVIC_IABR1_ASRC_IN_Pos))

#define NVIC_ASRC_OUT_INT_ACTIVE        ((uint32_t)(0x1U << NVIC_IABR1_ASRC_OUT_Pos))

/* NVIC_IABR2 bit positions */
#define NVIC_IABR2_BLE_AUDIO2_Pos       8
#define NVIC_IABR2_BLE_AUDIO1_Pos       7
#define NVIC_IABR2_BLE_AUDIO0_Pos       6
#define NVIC_IABR2_MEM_ERROR_Pos        5
#define NVIC_IABR2_FLASH_ECC_Pos        4
#define NVIC_IABR2_FLASH_COPY_Pos       3
#define NVIC_IABR2_CLKDET_Pos           2
#define NVIC_IABR2_AUDIO_SINK_PERIOD_Pos 1
#define NVIC_IABR2_AUDIO_SINK_DELAY_Pos 0

/* NVIC_IABR2 settings */
#define NVIC_AUDIO_SINK_DELAY_INT_ACTIVE ((uint32_t)(0x1U << NVIC_IABR2_AUDIO_SINK_DELAY_Pos))

#define NVIC_AUDIO_SINK_PERIOD_INT_ACTIVE ((uint32_t)(0x1U << NVIC_IABR2_AUDIO_SINK_PERIOD_Pos))

#define NVIC_CLKDET_INT_ACTIVE          ((uint32_t)(0x1U << NVIC_IABR2_CLKDET_Pos))

#define NVIC_FLASH_COPY_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR2_FLASH_COPY_Pos))

#define NVIC_FLASH_ECC_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR2_FLASH_ECC_Pos))

#define NVIC_MEM_ERROR_INT_ACTIVE       ((uint32_t)(0x1U << NVIC_IABR2_MEM_ERROR_Pos))

#define NVIC_BLE_AUDIO0_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR2_BLE_AUDIO0_Pos))

#define NVIC_BLE_AUDIO1_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR2_BLE_AUDIO1_Pos))

#define NVIC_BLE_AUDIO2_INT_ACTIVE      ((uint32_t)(0x1U << NVIC_IABR2_BLE_AUDIO2_Pos))

/* NVIC_IP0 bit positions */
#define NVIC_IP0_ADC_BATMON_Pos         29
#define NVIC_IP0_ADC_BATMON_Mask        ((uint32_t)(0x7U << NVIC_IP0_ADC_BATMON_Pos))
#define NVIC_IP0_RTC_CLOCK_Pos          21
#define NVIC_IP0_RTC_CLOCK_Mask         ((uint32_t)(0x7U << NVIC_IP0_RTC_CLOCK_Pos))
#define NVIC_IP0_RTC_ALARM_Pos          13
#define NVIC_IP0_RTC_ALARM_Mask         ((uint32_t)(0x7U << NVIC_IP0_RTC_ALARM_Pos))
#define NVIC_IP0_WAKEUP_Pos             5
#define NVIC_IP0_WAKEUP_Mask            ((uint32_t)(0x7U << NVIC_IP0_WAKEUP_Pos))

/* NVIC_IP0 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t WAKEUP_BYTE;          
         uint8_t RESERVED0[2];
    __IO uint8_t ADC_BATMON_BYTE;      
} NVIC_IP0_Type;

#define NVIC_IP0_BASE                   0xE000E400
#define NVIC_IP0                        ((NVIC_IP0_Type *) NVIC_IP0_BASE)

/* NVIC_IP0 sub-register bit positions */
#define NVIC_IP0_WAKEUP_BYTE_Pos        5
#define NVIC_IP0_WAKEUP_BYTE_Mask       ((uint32_t)(0x7U << NVIC_IP0_WAKEUP_BYTE_Pos))
#define NVIC_IP0_ADC_BATMON_BYTE_Pos    5
#define NVIC_IP0_ADC_BATMON_BYTE_Mask   ((uint32_t)(0x7U << NVIC_IP0_ADC_BATMON_BYTE_Pos))

/* NVIC_IP1 bit positions */
#define NVIC_IP1_TIMER3_Pos             29
#define NVIC_IP1_TIMER3_Mask            ((uint32_t)(0x7U << NVIC_IP1_TIMER3_Pos))
#define NVIC_IP1_TIMER2_Pos             21
#define NVIC_IP1_TIMER2_Mask            ((uint32_t)(0x7U << NVIC_IP1_TIMER2_Pos))
#define NVIC_IP1_TIMER1_Pos             13
#define NVIC_IP1_TIMER1_Mask            ((uint32_t)(0x7U << NVIC_IP1_TIMER1_Pos))
#define NVIC_IP1_TIMER0_Pos             5
#define NVIC_IP1_TIMER0_Mask            ((uint32_t)(0x7U << NVIC_IP1_TIMER0_Pos))

/* NVIC_IP1 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t TIMER0_BYTE;          
    __IO uint8_t TIMER1_BYTE;          
    __IO uint8_t TIMER2_BYTE;          
    __IO uint8_t TIMER3_BYTE;          
} NVIC_IP1_Type;

#define NVIC_IP1_BASE                   0xE000E404
#define NVIC_IP1                        ((NVIC_IP1_Type *) NVIC_IP1_BASE)

/* NVIC_IP1 sub-register bit positions */
#define NVIC_IP1_TIMER0_BYTE_Pos        5
#define NVIC_IP1_TIMER0_BYTE_Mask       ((uint32_t)(0x7U << NVIC_IP1_TIMER0_BYTE_Pos))
#define NVIC_IP1_TIMER1_BYTE_Pos        5
#define NVIC_IP1_TIMER1_BYTE_Mask       ((uint32_t)(0x7U << NVIC_IP1_TIMER1_BYTE_Pos))
#define NVIC_IP1_TIMER2_BYTE_Pos        5
#define NVIC_IP1_TIMER2_BYTE_Mask       ((uint32_t)(0x7U << NVIC_IP1_TIMER2_BYTE_Pos))
#define NVIC_IP1_TIMER3_BYTE_Pos        5
#define NVIC_IP1_TIMER3_BYTE_Mask       ((uint32_t)(0x7U << NVIC_IP1_TIMER3_BYTE_Pos))

/* NVIC_IP2 bit positions */
#define NVIC_IP2_DMA3_Pos               29
#define NVIC_IP2_DMA3_Mask              ((uint32_t)(0x7U << NVIC_IP2_DMA3_Pos))
#define NVIC_IP2_DMA2_Pos               21
#define NVIC_IP2_DMA2_Mask              ((uint32_t)(0x7U << NVIC_IP2_DMA2_Pos))
#define NVIC_IP2_DMA1_Pos               13
#define NVIC_IP2_DMA1_Mask              ((uint32_t)(0x7U << NVIC_IP2_DMA1_Pos))
#define NVIC_IP2_DMA0_Pos               5
#define NVIC_IP2_DMA0_Mask              ((uint32_t)(0x7U << NVIC_IP2_DMA0_Pos))

/* NVIC_IP2 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DMA0_BYTE;            
    __IO uint8_t DMA1_BYTE;            
    __IO uint8_t DMA2_BYTE;            
    __IO uint8_t DMA3_BYTE;            
} NVIC_IP2_Type;

#define NVIC_IP2_BASE                   0xE000E408
#define NVIC_IP2                        ((NVIC_IP2_Type *) NVIC_IP2_BASE)

/* NVIC_IP2 sub-register bit positions */
#define NVIC_IP2_DMA0_BYTE_Pos          5
#define NVIC_IP2_DMA0_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP2_DMA0_BYTE_Pos))
#define NVIC_IP2_DMA1_BYTE_Pos          5
#define NVIC_IP2_DMA1_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP2_DMA1_BYTE_Pos))
#define NVIC_IP2_DMA2_BYTE_Pos          5
#define NVIC_IP2_DMA2_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP2_DMA2_BYTE_Pos))
#define NVIC_IP2_DMA3_BYTE_Pos          5
#define NVIC_IP2_DMA3_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP2_DMA3_BYTE_Pos))

/* NVIC_IP3 bit positions */
#define NVIC_IP3_DMA7_Pos               29
#define NVIC_IP3_DMA7_Mask              ((uint32_t)(0x7U << NVIC_IP3_DMA7_Pos))
#define NVIC_IP3_DMA6_Pos               21
#define NVIC_IP3_DMA6_Mask              ((uint32_t)(0x7U << NVIC_IP3_DMA6_Pos))
#define NVIC_IP3_DMA5_Pos               13
#define NVIC_IP3_DMA5_Mask              ((uint32_t)(0x7U << NVIC_IP3_DMA5_Pos))
#define NVIC_IP3_DMA4_Pos               5
#define NVIC_IP3_DMA4_Mask              ((uint32_t)(0x7U << NVIC_IP3_DMA4_Pos))

/* NVIC_IP3 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DMA4_BYTE;            
    __IO uint8_t DMA5_BYTE;            
    __IO uint8_t DMA6_BYTE;            
    __IO uint8_t DMA7_BYTE;            
} NVIC_IP3_Type;

#define NVIC_IP3_BASE                   0xE000E40C
#define NVIC_IP3                        ((NVIC_IP3_Type *) NVIC_IP3_BASE)

/* NVIC_IP3 sub-register bit positions */
#define NVIC_IP3_DMA4_BYTE_Pos          5
#define NVIC_IP3_DMA4_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP3_DMA4_BYTE_Pos))
#define NVIC_IP3_DMA5_BYTE_Pos          5
#define NVIC_IP3_DMA5_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP3_DMA5_BYTE_Pos))
#define NVIC_IP3_DMA6_BYTE_Pos          5
#define NVIC_IP3_DMA6_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP3_DMA6_BYTE_Pos))
#define NVIC_IP3_DMA7_BYTE_Pos          5
#define NVIC_IP3_DMA7_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP3_DMA7_BYTE_Pos))

/* NVIC_IP4 bit positions */
#define NVIC_IP4_DIO3_Pos               29
#define NVIC_IP4_DIO3_Mask              ((uint32_t)(0x7U << NVIC_IP4_DIO3_Pos))
#define NVIC_IP4_DIO2_Pos               21
#define NVIC_IP4_DIO2_Mask              ((uint32_t)(0x7U << NVIC_IP4_DIO2_Pos))
#define NVIC_IP4_DIO1_Pos               13
#define NVIC_IP4_DIO1_Mask              ((uint32_t)(0x7U << NVIC_IP4_DIO1_Pos))
#define NVIC_IP4_DIO0_Pos               5
#define NVIC_IP4_DIO0_Mask              ((uint32_t)(0x7U << NVIC_IP4_DIO0_Pos))

/* NVIC_IP4 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DIO0_BYTE;            
    __IO uint8_t DIO1_BYTE;            
    __IO uint8_t DIO2_BYTE;            
    __IO uint8_t DIO3_BYTE;            
} NVIC_IP4_Type;

#define NVIC_IP4_BASE                   0xE000E410
#define NVIC_IP4                        ((NVIC_IP4_Type *) NVIC_IP4_BASE)

/* NVIC_IP4 sub-register bit positions */
#define NVIC_IP4_DIO0_BYTE_Pos          5
#define NVIC_IP4_DIO0_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP4_DIO0_BYTE_Pos))
#define NVIC_IP4_DIO1_BYTE_Pos          5
#define NVIC_IP4_DIO1_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP4_DIO1_BYTE_Pos))
#define NVIC_IP4_DIO2_BYTE_Pos          5
#define NVIC_IP4_DIO2_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP4_DIO2_BYTE_Pos))
#define NVIC_IP4_DIO3_BYTE_Pos          5
#define NVIC_IP4_DIO3_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP4_DIO3_BYTE_Pos))

/* NVIC_IP5 bit positions */
#define NVIC_IP5_SPI0_ERROR_Pos         29
#define NVIC_IP5_SPI0_ERROR_Mask        ((uint32_t)(0x7U << NVIC_IP5_SPI0_ERROR_Pos))
#define NVIC_IP5_SPI0_TX_Pos            21
#define NVIC_IP5_SPI0_TX_Mask           ((uint32_t)(0x7U << NVIC_IP5_SPI0_TX_Pos))
#define NVIC_IP5_SPI0_RX_Pos            13
#define NVIC_IP5_SPI0_RX_Mask           ((uint32_t)(0x7U << NVIC_IP5_SPI0_RX_Pos))
#define NVIC_IP5_WATCHDOG_Pos           5
#define NVIC_IP5_WATCHDOG_Mask          ((uint32_t)(0x7U << NVIC_IP5_WATCHDOG_Pos))

/* NVIC_IP5 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t WATCHDOG_BYTE;        
         uint8_t RESERVED0[3];
} NVIC_IP5_Type;

#define NVIC_IP5_BASE                   0xE000E414
#define NVIC_IP5                        ((NVIC_IP5_Type *) NVIC_IP5_BASE)

/* NVIC_IP5 sub-register bit positions */
#define NVIC_IP5_WATCHDOG_BYTE_Pos      5
#define NVIC_IP5_WATCHDOG_BYTE_Mask     ((uint32_t)(0x7U << NVIC_IP5_WATCHDOG_BYTE_Pos))

/* NVIC_IP6 bit positions */
#define NVIC_IP6_I2C_Pos                29
#define NVIC_IP6_I2C_Mask               ((uint32_t)(0x7U << NVIC_IP6_I2C_Pos))
#define NVIC_IP6_SPI1_ERROR_Pos         21
#define NVIC_IP6_SPI1_ERROR_Mask        ((uint32_t)(0x7U << NVIC_IP6_SPI1_ERROR_Pos))
#define NVIC_IP6_SPI1_TX_Pos            13
#define NVIC_IP6_SPI1_TX_Mask           ((uint32_t)(0x7U << NVIC_IP6_SPI1_TX_Pos))
#define NVIC_IP6_SPI1_RX_Pos            5
#define NVIC_IP6_SPI1_RX_Mask           ((uint32_t)(0x7U << NVIC_IP6_SPI1_RX_Pos))

/* NVIC_IP6 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t I2C_BYTE;             
} NVIC_IP6_Type;

#define NVIC_IP6_BASE                   0xE000E418
#define NVIC_IP6                        ((NVIC_IP6_Type *) NVIC_IP6_BASE)

/* NVIC_IP6 sub-register bit positions */
#define NVIC_IP6_I2C_BYTE_Pos           5
#define NVIC_IP6_I2C_BYTE_Mask          ((uint32_t)(0x7U << NVIC_IP6_I2C_BYTE_Pos))

/* NVIC_IP7 bit positions */
#define NVIC_IP7_DMIC_OUT_OD_IN_Pos     29
#define NVIC_IP7_DMIC_OUT_OD_IN_Mask    ((uint32_t)(0x7U << NVIC_IP7_DMIC_OUT_OD_IN_Pos))
#define NVIC_IP7_UART_ERROR_Pos         21
#define NVIC_IP7_UART_ERROR_Mask        ((uint32_t)(0x7U << NVIC_IP7_UART_ERROR_Pos))
#define NVIC_IP7_UART_TX_Pos            13
#define NVIC_IP7_UART_TX_Mask           ((uint32_t)(0x7U << NVIC_IP7_UART_TX_Pos))
#define NVIC_IP7_UART_RX_Pos            5
#define NVIC_IP7_UART_RX_Mask           ((uint32_t)(0x7U << NVIC_IP7_UART_RX_Pos))

/* NVIC_IP7 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __IO uint8_t DMIC_OUT_OD_IN_BYTE;  
} NVIC_IP7_Type;

#define NVIC_IP7_BASE                   0xE000E41C
#define NVIC_IP7                        ((NVIC_IP7_Type *) NVIC_IP7_BASE)

/* NVIC_IP7 sub-register bit positions */
#define NVIC_IP7_DMIC_OUT_OD_IN_BYTE_Pos 5
#define NVIC_IP7_DMIC_OUT_OD_IN_BYTE_Mask ((uint32_t)(0x7U << NVIC_IP7_DMIC_OUT_OD_IN_BYTE_Pos))

/* NVIC_IP8 bit positions */
#define NVIC_IP8_PCM_ERROR_Pos          29
#define NVIC_IP8_PCM_ERROR_Mask         ((uint32_t)(0x7U << NVIC_IP8_PCM_ERROR_Pos))
#define NVIC_IP8_PCM_TX_Pos             21
#define NVIC_IP8_PCM_TX_Mask            ((uint32_t)(0x7U << NVIC_IP8_PCM_TX_Pos))
#define NVIC_IP8_PCM_RX_Pos             13
#define NVIC_IP8_PCM_RX_Mask            ((uint32_t)(0x7U << NVIC_IP8_PCM_RX_Pos))
#define NVIC_IP8_DMIC_OD_ERROR_Pos      5
#define NVIC_IP8_DMIC_OD_ERROR_Mask     ((uint32_t)(0x7U << NVIC_IP8_DMIC_OD_ERROR_Pos))

/* NVIC_IP8 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DMIC_OD_ERROR_BYTE;   
         uint8_t RESERVED0[3];
} NVIC_IP8_Type;

#define NVIC_IP8_BASE                   0xE000E420
#define NVIC_IP8                        ((NVIC_IP8_Type *) NVIC_IP8_BASE)

/* NVIC_IP8 sub-register bit positions */
#define NVIC_IP8_DMIC_OD_ERROR_BYTE_Pos 5
#define NVIC_IP8_DMIC_OD_ERROR_BYTE_Mask ((uint32_t)(0x7U << NVIC_IP8_DMIC_OD_ERROR_BYTE_Pos))

/* NVIC_IP9 bit positions */
#define NVIC_IP9_DSS3_Pos               29
#define NVIC_IP9_DSS3_Mask              ((uint32_t)(0x7U << NVIC_IP9_DSS3_Pos))
#define NVIC_IP9_DSS2_Pos               21
#define NVIC_IP9_DSS2_Mask              ((uint32_t)(0x7U << NVIC_IP9_DSS2_Pos))
#define NVIC_IP9_DSS1_Pos               13
#define NVIC_IP9_DSS1_Mask              ((uint32_t)(0x7U << NVIC_IP9_DSS1_Pos))
#define NVIC_IP9_DSS0_Pos               5
#define NVIC_IP9_DSS0_Mask              ((uint32_t)(0x7U << NVIC_IP9_DSS0_Pos))

/* NVIC_IP9 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DSS0_BYTE;            
    __IO uint8_t DSS1_BYTE;            
    __IO uint8_t DSS2_BYTE;            
    __IO uint8_t DSS3_BYTE;            
} NVIC_IP9_Type;

#define NVIC_IP9_BASE                   0xE000E424
#define NVIC_IP9                        ((NVIC_IP9_Type *) NVIC_IP9_BASE)

/* NVIC_IP9 sub-register bit positions */
#define NVIC_IP9_DSS0_BYTE_Pos          5
#define NVIC_IP9_DSS0_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP9_DSS0_BYTE_Pos))
#define NVIC_IP9_DSS1_BYTE_Pos          5
#define NVIC_IP9_DSS1_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP9_DSS1_BYTE_Pos))
#define NVIC_IP9_DSS2_BYTE_Pos          5
#define NVIC_IP9_DSS2_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP9_DSS2_BYTE_Pos))
#define NVIC_IP9_DSS3_BYTE_Pos          5
#define NVIC_IP9_DSS3_BYTE_Mask         ((uint32_t)(0x7U << NVIC_IP9_DSS3_BYTE_Pos))

/* NVIC_IP10 bit positions */
#define NVIC_IP10_DSS7_Pos              29
#define NVIC_IP10_DSS7_Mask             ((uint32_t)(0x7U << NVIC_IP10_DSS7_Pos))
#define NVIC_IP10_DSS6_Pos              21
#define NVIC_IP10_DSS6_Mask             ((uint32_t)(0x7U << NVIC_IP10_DSS6_Pos))
#define NVIC_IP10_DSS5_Pos              13
#define NVIC_IP10_DSS5_Mask             ((uint32_t)(0x7U << NVIC_IP10_DSS5_Pos))
#define NVIC_IP10_DSS4_Pos              5
#define NVIC_IP10_DSS4_Mask             ((uint32_t)(0x7U << NVIC_IP10_DSS4_Pos))

/* NVIC_IP10 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t DSS4_BYTE;            
    __IO uint8_t DSS5_BYTE;            
    __IO uint8_t DSS6_BYTE;            
    __IO uint8_t DSS7_BYTE;            
} NVIC_IP10_Type;

#define NVIC_IP10_BASE                  0xE000E428
#define NVIC_IP10                       ((NVIC_IP10_Type *) NVIC_IP10_BASE)

/* NVIC_IP10 sub-register bit positions */
#define NVIC_IP10_DSS4_BYTE_Pos         5
#define NVIC_IP10_DSS4_BYTE_Mask        ((uint32_t)(0x7U << NVIC_IP10_DSS4_BYTE_Pos))
#define NVIC_IP10_DSS5_BYTE_Pos         5
#define NVIC_IP10_DSS5_BYTE_Mask        ((uint32_t)(0x7U << NVIC_IP10_DSS5_BYTE_Pos))
#define NVIC_IP10_DSS6_BYTE_Pos         5
#define NVIC_IP10_DSS6_BYTE_Mask        ((uint32_t)(0x7U << NVIC_IP10_DSS6_BYTE_Pos))
#define NVIC_IP10_DSS7_BYTE_Pos         5
#define NVIC_IP10_DSS7_BYTE_Mask        ((uint32_t)(0x7U << NVIC_IP10_DSS7_BYTE_Pos))

/* NVIC_IP11 bit positions */
#define NVIC_IP11_BLE_EVENT_Pos         29
#define NVIC_IP11_BLE_EVENT_Mask        ((uint32_t)(0x7U << NVIC_IP11_BLE_EVENT_Pos))
#define NVIC_IP11_BLE_RX_Pos            21
#define NVIC_IP11_BLE_RX_Mask           ((uint32_t)(0x7U << NVIC_IP11_BLE_RX_Pos))
#define NVIC_IP11_BLE_SLP_Pos           13
#define NVIC_IP11_BLE_SLP_Mask          ((uint32_t)(0x7U << NVIC_IP11_BLE_SLP_Pos))
#define NVIC_IP11_BLE_CSCNT_Pos         5
#define NVIC_IP11_BLE_CSCNT_Mask        ((uint32_t)(0x7U << NVIC_IP11_BLE_CSCNT_Pos))

/* NVIC_IP12 bit positions */
#define NVIC_IP12_BLE_FINETGTIM_Pos     29
#define NVIC_IP12_BLE_FINETGTIM_Mask    ((uint32_t)(0x7U << NVIC_IP12_BLE_FINETGTIM_Pos))
#define NVIC_IP12_BLE_GROSSTGTIM_Pos    21
#define NVIC_IP12_BLE_GROSSTGTIM_Mask   ((uint32_t)(0x7U << NVIC_IP12_BLE_GROSSTGTIM_Pos))
#define NVIC_IP12_BLE_ERROR_Pos         13
#define NVIC_IP12_BLE_ERROR_Mask        ((uint32_t)(0x7U << NVIC_IP12_BLE_ERROR_Pos))
#define NVIC_IP12_BLE_CRYPT_Pos         5
#define NVIC_IP12_BLE_CRYPT_Mask        ((uint32_t)(0x7U << NVIC_IP12_BLE_CRYPT_Pos))

/* NVIC_IP13 bit positions */
#define NVIC_IP13_RF_TX_Pos             29
#define NVIC_IP13_RF_TX_Mask            ((uint32_t)(0x7U << NVIC_IP13_RF_TX_Pos))
#define NVIC_IP13_BLE_COEX_IN_PROCESS_Pos 21
#define NVIC_IP13_BLE_COEX_IN_PROCESS_Mask ((uint32_t)(0x7U << NVIC_IP13_BLE_COEX_IN_PROCESS_Pos))
#define NVIC_IP13_BLE_COEX_RX_TX_Pos    13
#define NVIC_IP13_BLE_COEX_RX_TX_Mask   ((uint32_t)(0x7U << NVIC_IP13_BLE_COEX_RX_TX_Pos))
#define NVIC_IP13_BLE_SW_Pos            5
#define NVIC_IP13_BLE_SW_Mask           ((uint32_t)(0x7U << NVIC_IP13_BLE_SW_Pos))

/* NVIC_IP13 sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[1];
    __IO uint8_t BLE_COEX_RX_TX_BYTE;  
    __IO uint8_t BLE_COEX_IN_PROCESS_BYTE;
    __IO uint8_t RF_TX_BYTE;           
} NVIC_IP13_Type;

#define NVIC_IP13_BASE                  0xE000E434
#define NVIC_IP13                       ((NVIC_IP13_Type *) NVIC_IP13_BASE)

/* NVIC_IP13 sub-register bit positions */
#define NVIC_IP13_BLE_COEX_RX_TX_BYTE_Pos 5
#define NVIC_IP13_BLE_COEX_RX_TX_BYTE_Mask ((uint32_t)(0x7U << NVIC_IP13_BLE_COEX_RX_TX_BYTE_Pos))
#define NVIC_IP13_BLE_COEX_IN_PROCESS_BYTE_Pos 5
#define NVIC_IP13_BLE_COEX_IN_PROCESS_BYTE_Mask ((uint32_t)(0x7U << NVIC_IP13_BLE_COEX_IN_PROCESS_BYTE_Pos))
#define NVIC_IP13_RF_TX_BYTE_Pos        5
#define NVIC_IP13_RF_TX_BYTE_Mask       ((uint32_t)(0x7U << NVIC_IP13_RF_TX_BYTE_Pos))

/* NVIC_IP14 bit positions */
#define NVIC_IP14_RF_TXFIFO_Pos         29
#define NVIC_IP14_RF_TXFIFO_Mask        ((uint32_t)(0x7U << NVIC_IP14_RF_TXFIFO_Pos))
#define NVIC_IP14_RF_SYNC_Pos           21
#define NVIC_IP14_RF_SYNC_Mask          ((uint32_t)(0x7U << NVIC_IP14_RF_SYNC_Pos))
#define NVIC_IP14_RF_RECEIVED_Pos       13
#define NVIC_IP14_RF_RECEIVED_Mask      ((uint32_t)(0x7U << NVIC_IP14_RF_RECEIVED_Pos))
#define NVIC_IP14_RF_RXSTOP_Pos         5
#define NVIC_IP14_RF_RXSTOP_Mask        ((uint32_t)(0x7U << NVIC_IP14_RF_RXSTOP_Pos))

/* NVIC_IP15 bit positions */
#define NVIC_IP15_ASRC_OUT_Pos          29
#define NVIC_IP15_ASRC_OUT_Mask         ((uint32_t)(0x7U << NVIC_IP15_ASRC_OUT_Pos))
#define NVIC_IP15_ASRC_IN_Pos           21
#define NVIC_IP15_ASRC_IN_Mask          ((uint32_t)(0x7U << NVIC_IP15_ASRC_IN_Pos))
#define NVIC_IP15_ASRC_ERROR_Pos        13
#define NVIC_IP15_ASRC_ERROR_Mask       ((uint32_t)(0x7U << NVIC_IP15_ASRC_ERROR_Pos))
#define NVIC_IP15_RF_RXFIFO_Pos         5
#define NVIC_IP15_RF_RXFIFO_Mask        ((uint32_t)(0x7U << NVIC_IP15_RF_RXFIFO_Pos))

/* NVIC_IP15 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t RF_RXFIFO_BYTE;       
         uint8_t RESERVED0[3];
} NVIC_IP15_Type;

#define NVIC_IP15_BASE                  0xE000E43C
#define NVIC_IP15                       ((NVIC_IP15_Type *) NVIC_IP15_BASE)

/* NVIC_IP15 sub-register bit positions */
#define NVIC_IP15_RF_RXFIFO_BYTE_Pos    5
#define NVIC_IP15_RF_RXFIFO_BYTE_Mask   ((uint32_t)(0x7U << NVIC_IP15_RF_RXFIFO_BYTE_Pos))

/* NVIC_IP16 bit positions */
#define NVIC_IP16_FLASH_COPY_Pos        29
#define NVIC_IP16_FLASH_COPY_Mask       ((uint32_t)(0x7U << NVIC_IP16_FLASH_COPY_Pos))
#define NVIC_IP16_CLKDET_Pos            21
#define NVIC_IP16_CLKDET_Mask           ((uint32_t)(0x7U << NVIC_IP16_CLKDET_Pos))
#define NVIC_IP16_AUDIO_SINK_PERIOD_Pos 13
#define NVIC_IP16_AUDIO_SINK_PERIOD_Mask ((uint32_t)(0x7U << NVIC_IP16_AUDIO_SINK_PERIOD_Pos))
#define NVIC_IP16_AUDIO_SINK_DELAY_Pos  5
#define NVIC_IP16_AUDIO_SINK_DELAY_Mask ((uint32_t)(0x7U << NVIC_IP16_AUDIO_SINK_DELAY_Pos))

/* NVIC_IP16 sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t AUDIO_SINK_SHORT;     
    __IO uint8_t CLKDET_BYTE;          
    __IO uint8_t FLASH_COPY_BYTE;      
} NVIC_IP16_Type;

#define NVIC_IP16_BASE                  0xE000E440
#define NVIC_IP16                       ((NVIC_IP16_Type *) NVIC_IP16_BASE)

/* NVIC_IP16 sub-register bit positions */
#define NVIC_IP16_AUDIO_SINK_DELAY_SHORT_Pos 5
#define NVIC_IP16_AUDIO_SINK_DELAY_SHORT_Mask ((uint32_t)(0x7U << NVIC_IP16_AUDIO_SINK_DELAY_SHORT_Pos))
#define NVIC_IP16_AUDIO_SINK_PERIOD_SHORT_Pos 13
#define NVIC_IP16_AUDIO_SINK_PERIOD_SHORT_Mask ((uint32_t)(0x7U << NVIC_IP16_AUDIO_SINK_PERIOD_SHORT_Pos))
#define NVIC_IP16_CLKDET_BYTE_Pos       5
#define NVIC_IP16_CLKDET_BYTE_Mask      ((uint32_t)(0x7U << NVIC_IP16_CLKDET_BYTE_Pos))
#define NVIC_IP16_FLASH_COPY_BYTE_Pos   5
#define NVIC_IP16_FLASH_COPY_BYTE_Mask  ((uint32_t)(0x7U << NVIC_IP16_FLASH_COPY_BYTE_Pos))

/* NVIC_IP17 bit positions */
#define NVIC_IP17_BLE_AUDIO1_Pos        29
#define NVIC_IP17_BLE_AUDIO1_Mask       ((uint32_t)(0x7U << NVIC_IP17_BLE_AUDIO1_Pos))
#define NVIC_IP17_BLE_AUDIO0_Pos        21
#define NVIC_IP17_BLE_AUDIO0_Mask       ((uint32_t)(0x7U << NVIC_IP17_BLE_AUDIO0_Pos))
#define NVIC_IP17_MEM_ERROR_Pos         13
#define NVIC_IP17_MEM_ERROR_Mask        ((uint32_t)(0x7U << NVIC_IP17_MEM_ERROR_Pos))
#define NVIC_IP17_FLASH_ECC_Pos         5
#define NVIC_IP17_FLASH_ECC_Mask        ((uint32_t)(0x7U << NVIC_IP17_FLASH_ECC_Pos))

/* NVIC_IP17 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t FLASH_ECC_BYTE;       
    __IO uint8_t MEM_ERROR_BYTE;       
    __IO uint16_t BLE_SHORT;            
} NVIC_IP17_Type;

#define NVIC_IP17_BASE                  0xE000E444
#define NVIC_IP17                       ((NVIC_IP17_Type *) NVIC_IP17_BASE)

/* NVIC_IP17 sub-register bit positions */
#define NVIC_IP17_FLASH_ECC_BYTE_Pos    5
#define NVIC_IP17_FLASH_ECC_BYTE_Mask   ((uint32_t)(0x7U << NVIC_IP17_FLASH_ECC_BYTE_Pos))
#define NVIC_IP17_MEM_ERROR_BYTE_Pos    5
#define NVIC_IP17_MEM_ERROR_BYTE_Mask   ((uint32_t)(0x7U << NVIC_IP17_MEM_ERROR_BYTE_Pos))
#define NVIC_IP17_BLE_AUDIO0_SHORT_Pos  5
#define NVIC_IP17_BLE_AUDIO0_SHORT_Mask ((uint32_t)(0x7U << NVIC_IP17_BLE_AUDIO0_SHORT_Pos))
#define NVIC_IP17_BLE_AUDIO1_SHORT_Pos  13
#define NVIC_IP17_BLE_AUDIO1_SHORT_Mask ((uint32_t)(0x7U << NVIC_IP17_BLE_AUDIO1_SHORT_Pos))

/* NVIC_IP18 bit positions */
#define NVIC_IP18_BLE_AUDIO2_Pos        5
#define NVIC_IP18_BLE_AUDIO2_Mask       ((uint32_t)(0x7U << NVIC_IP18_BLE_AUDIO2_Pos))

/* ----------------------------------------------------------------------------
 * System Control Block
 * ------------------------------------------------------------------------- */

/* SCB_CPUID sub-register and bit-band aliases */
typedef struct
{
         uint8_t RESERVED0[3];
    __I  uint8_t IMPLEMENTER_BYTE;     
} SCB_CPUID_Type;

#define SCB_CPUID_BASE                  0xE000ED00
#define SCB_CPUID                       ((SCB_CPUID_Type *) SCB_CPUID_BASE)

/* SCB_ICSR settings */
#define NVIC_SYSTICK_CLRPEND            ((uint32_t)(0x1U << SCB_ICSR_PENDSTCLR_Pos))

#define NVIC_SYSTICK_SETPEND            ((uint32_t)(0x1U << SCB_ICSR_PENDSTSET_Pos))

#define NVIC_PENDSV_CLRPEND             ((uint32_t)(0x1U << SCB_ICSR_PENDSVCLR_Pos))

#define NVIC_PENDSV_SETPEND             ((uint32_t)(0x1U << SCB_ICSR_PENDSVSET_Pos))

#define NVIC_NMI_SETPEND                ((uint32_t)(0x1U << SCB_ICSR_NMIPENDSET_Pos))

/* SCB_AIRCR settings */
#define NVIC_VECTRESET                  ((uint32_t)(0x1U << SCB_AIRCR_VECTRESET_Pos))

#define NVIC_CLEAR_ACTIVE_VECTORS       ((uint32_t)(0x1U << SCB_AIRCR_VECTCLRACTIVE_Pos))

#define NVIC_SYSRESET                   ((uint32_t)(0x1U << SCB_AIRCR_SYSRESETREQ_Pos))

#define NVIC_PRIGROUP_4_0               ((uint32_t)(0x3U << SCB_AIRCR_PRIGROUP_Pos))
#define NVIC_PRIGROUP_3_1               ((uint32_t)(0x4U << SCB_AIRCR_PRIGROUP_Pos))
#define NVIC_PRIGROUP_2_2               ((uint32_t)(0x5U << SCB_AIRCR_PRIGROUP_Pos))
#define NVIC_PRIGROUP_1_3               ((uint32_t)(0x6U << SCB_AIRCR_PRIGROUP_Pos))
#define NVIC_PRIGROUP_0_4               ((uint32_t)(0x7U << SCB_AIRCR_PRIGROUP_Pos))

#define NVIC_LITTLE_ENDIAN              ((uint32_t)(0x0U << SCB_AIRCR_ENDIANESS_Pos))

#define NVIC_VECTKEY_UNLOCK             ((uint32_t)(0x5FAU << SCB_AIRCR_VECTKEY_Pos))
#define NVIC_VECTKEY_UNLOCK_READBACK    ((uint32_t)(0xFA05U << SCB_AIRCR_VECTKEY_Pos))

/* SCB_SCR settings */
#define NVIC_SLEEPONEXIT_DISABLE        ((uint32_t)(0x0U << SCB_SCR_SLEEPONEXIT_Pos))
#define NVIC_SLEEPONEXIT_ENABLE         ((uint32_t)(0x1U << SCB_SCR_SLEEPONEXIT_Pos))

#define NVIC_SLEEPDEEP_DISABLE          ((uint32_t)(0x0U << SCB_SCR_SLEEPDEEP_Pos))
#define NVIC_SLEEPDEEP_ENABLE           ((uint32_t)(0x1U << SCB_SCR_SLEEPDEEP_Pos))

/* SCB_CCR settings */
#define NVIC_NONBASETHRDENA_DISABLE     ((uint32_t)(0x0U << SCB_CCR_NONBASETHRDENA_Pos))
#define NVIC_NONBASETHRDENA_ENABLE      ((uint32_t)(0x1U << SCB_CCR_NONBASETHRDENA_Pos))

#define NVIC_USERSETMPEND_DISABLE       ((uint32_t)(0x0U << SCB_CCR_USERSETMPEND_Pos))
#define NVIC_USERSETMPEND_ENABLE        ((uint32_t)(0x1U << SCB_CCR_USERSETMPEND_Pos))

#define NVIC_UNALIGN_TRP_DISABLE        ((uint32_t)(0x0U << SCB_CCR_UNALIGN_TRP_Pos))
#define NVIC_UNALIGN_TRP_ENABLE         ((uint32_t)(0x1U << SCB_CCR_UNALIGN_TRP_Pos))

#define NVIC_DIV_0_TRP_DISABLE          ((uint32_t)(0x0U << SCB_CCR_DIV_0_TRP_Pos))
#define NVIC_DIV_0_TRP_ENABLE           ((uint32_t)(0x1U << SCB_CCR_DIV_0_TRP_Pos))

#define NVIC_BFHFNMIGN_DISABLE          ((uint32_t)(0x0U << SCB_CCR_BFHFNMIGN_Pos))
#define NVIC_BFHFNMIGN_ENABLE           ((uint32_t)(0x1U << SCB_CCR_BFHFNMIGN_Pos))

#define NVIC_STKALIGN_DISABLE           ((uint32_t)(0x0U << SCB_CCR_STKALIGN_Pos))
#define NVIC_STKALIGN_ENABLE            ((uint32_t)(0x1U << SCB_CCR_STKALIGN_Pos))

/* SCB_SHP0 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t NVIC_MEM_FAULT_PRIORITY_BYTE;
    __IO uint8_t NVIC_BUS_FAULT_PRIORITY_BYTE;
    __IO uint8_t NVIC_USAGE_FAULT_PRIORITY_BYTE;
         uint8_t RESERVED0[1];
} SCB_SHP0_Type;

#define SCB_SHP0_BASE                   0xE000ED18
#define SCB_SHP0                        ((SCB_SHP0_Type *) SCB_SHP0_BASE)

/* SCB_SHP2 sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t NVIC_MONITOR_PRIORITY_BYTE;
         uint8_t RESERVED0[1];
    __IO uint8_t NVIC_PENDSV_PRIORITY_BYTE;
    __IO uint8_t NVIC_SYSTICK_PRIORITY_BYTE;
} SCB_SHP2_Type;

#define SCB_SHP2_BASE                   0xE000ED20
#define SCB_SHP2                        ((SCB_SHP2_Type *) SCB_SHP2_BASE)

/* SCB_SHCSR settings */
#define NVIC_MEMFAULT_ACTIVE            ((uint32_t)(0x1U << SCB_SHCSR_MEMFAULTACT_Pos))

#define NVIC_BUSFAULT_ACTIVE            ((uint32_t)(0x1U << SCB_SHCSR_BUSFAULTACT_Pos))

#define NVIC_USGFAULT_ACTIVE            ((uint32_t)(0x1U << SCB_SHCSR_USGFAULTACT_Pos))

#define NVIC_SVCALL_ACTIVE              ((uint32_t)(0x1U << SCB_SHCSR_SVCALLACT_Pos))

#define NVIC_MONITOR_ACTIVE             ((uint32_t)(0x1U << SCB_SHCSR_MONITORACT_Pos))

#define NVIC_PENDSV_ACTIVE              ((uint32_t)(0x1U << SCB_SHCSR_PENDSVACT_Pos))

#define NVIC_SYSTICK_ACTIVE             ((uint32_t)(0x1U << SCB_SHCSR_SYSTICKACT_Pos))

#define NVIC_USGFAULT_CLRPEND           ((uint32_t)(0x0U << SCB_SHCSR_USGFAULTPENDED_Pos))
#define NVIC_USGFAULT_SETPEND           ((uint32_t)(0x1U << SCB_SHCSR_USGFAULTPENDED_Pos))

#define NVIC_MEMFAULT_CLRPEND           ((uint32_t)(0x0U << SCB_SHCSR_MEMFAULTPENDED_Pos))
#define NVIC_MEMFAULT_SETPEND           ((uint32_t)(0x1U << SCB_SHCSR_MEMFAULTPENDED_Pos))

#define NVIC_BUSFAULT_CLRPEND           ((uint32_t)(0x0U << SCB_SHCSR_BUSFAULTPENDED_Pos))
#define NVIC_BUSFAULT_SETPEND           ((uint32_t)(0x1U << SCB_SHCSR_BUSFAULTPENDED_Pos))

#define NVIC_SVCALL_CLRPEND             ((uint32_t)(0x0U << SCB_SHCSR_SVCALLPENDED_Pos))
#define NVIC_SVCALL_SETPEND             ((uint32_t)(0x1U << SCB_SHCSR_SVCALLPENDED_Pos))

#define NVIC_MEMFAULT_DISABLE           ((uint32_t)(0x0U << SCB_SHCSR_MEMFAULTENA_Pos))
#define NVIC_MEMFAULT_ENABLE            ((uint32_t)(0x1U << SCB_SHCSR_MEMFAULTENA_Pos))

#define NVIC_BUSFAULT_DISABLE           ((uint32_t)(0x0U << SCB_SHCSR_BUSFAULTENA_Pos))
#define NVIC_BUSFAULT_ENABLE            ((uint32_t)(0x1U << SCB_SHCSR_BUSFAULTENA_Pos))

#define NVIC_USGFAULT_DISABLE           ((uint32_t)(0x0U << SCB_SHCSR_USGFAULTENA_Pos))
#define NVIC_USGFAULT_ENABLE            ((uint32_t)(0x1U << SCB_SHCSR_USGFAULTENA_Pos))

/* SCB_HFSR sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t VECTBL_BYTE;          
         uint8_t RESERVED0[3];
} SCB_HFSR_Type;

#define SCB_HFSR_BASE                   0xE000ED2C
#define SCB_HFSR                        ((SCB_HFSR_Type *) SCB_HFSR_BASE)

/* SCB_HFSR sub-register bit positions */
#define SCB_HFSR_VECTBL_BYTE_Pos        1

/* ----------------------------------------------------------------------------
 * Debug Controller
 * ------------------------------------------------------------------------- */

/* DEBUG_DHCSR settings */
#define DEBUG_HALT_KEY                  ((uint32_t)(0xA05FU << DEBUG_DHCSR_DBGKEY_Pos))

/* DEBUG_DCRSR sub-register and bit-band aliases */
typedef struct
{
    __IO uint8_t REGSEL_BYTE;          
         uint8_t RESERVED0[1];
    __IO uint8_t REGWnR_BYTE;          
         uint8_t RESERVED1[1];
} DEBUG_DCRSR_Type;

#define DEBUG_DCRSR_BASE                0xE000EDF4
#define DEBUG_DCRSR                     ((DEBUG_DCRSR_Type *) DEBUG_DCRSR_BASE)

/* DEBUG_DCRSR settings */
#define REGSEL_R0                       ((uint32_t)(0x0U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R1                       ((uint32_t)(0x1U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R2                       ((uint32_t)(0x2U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R3                       ((uint32_t)(0x3U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R4                       ((uint32_t)(0x4U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R5                       ((uint32_t)(0x5U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R6                       ((uint32_t)(0x6U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R7                       ((uint32_t)(0x7U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R8                       ((uint32_t)(0x8U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R9                       ((uint32_t)(0x9U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R10                      ((uint32_t)(0xAU << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R11                      ((uint32_t)(0xBU << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R12                      ((uint32_t)(0xCU << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R13                      ((uint32_t)(0xDU << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R14                      ((uint32_t)(0xEU << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_R15                      ((uint32_t)(0xFU << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_PSRFLGS                  ((uint32_t)(0x10U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_MSP                      ((uint32_t)(0x11U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_PSP                      ((uint32_t)(0x12U << DEBUG_DCRSR_REGSEL_Pos))
#define REGSEL_SPECREG                  ((uint32_t)(0x14U << DEBUG_DCRSR_REGSEL_Pos))

#define REGWNR_READ                     ((uint32_t)(0x0U << DEBUG_DCRSR_REGWnR_Pos))
#define REGWNR_WRITE                    ((uint32_t)(0x1U << DEBUG_DCRSR_REGWnR_Pos))

/* DEBUG_DCRSR sub-register bit positions */
#define DEBUG_DCRSR_REGSEL_BYTE_Pos     0
#define DEBUG_DCRSR_REGSEL_BYTE_Mask    ((uint32_t)(0x1FU << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define DEBUG_DCRSR_REGWnR_BYTE_Pos     0

/* DEBUG_DCRSR sub-register settings */
#define REGSEL_R0_BYTE                  ((uint8_t)(0x0U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R1_BYTE                  ((uint8_t)(0x1U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R2_BYTE                  ((uint8_t)(0x2U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R3_BYTE                  ((uint8_t)(0x3U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R4_BYTE                  ((uint8_t)(0x4U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R5_BYTE                  ((uint8_t)(0x5U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R6_BYTE                  ((uint8_t)(0x6U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R7_BYTE                  ((uint8_t)(0x7U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R8_BYTE                  ((uint8_t)(0x8U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R9_BYTE                  ((uint8_t)(0x9U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R10_BYTE                 ((uint8_t)(0xAU << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R11_BYTE                 ((uint8_t)(0xBU << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R12_BYTE                 ((uint8_t)(0xCU << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R13_BYTE                 ((uint8_t)(0xDU << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R14_BYTE                 ((uint8_t)(0xEU << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_R15_BYTE                 ((uint8_t)(0xFU << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_PSRFLGS_BYTE             ((uint8_t)(0x10U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_MSP_BYTE                 ((uint8_t)(0x11U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_PSP_BYTE                 ((uint8_t)(0x12U << DEBUG_DCRSR_REGSEL_BYTE_Pos))
#define REGSEL_SPECREG_BYTE             ((uint8_t)(0x14U << DEBUG_DCRSR_REGSEL_BYTE_Pos))

#define REGWNR_READ_BYTE                ((uint8_t)(0x0U << DEBUG_DCRSR_REGWnR_BYTE_Pos))
#define REGWNR_WRITE_BYTE               ((uint8_t)(0x1U << DEBUG_DCRSR_REGWnR_BYTE_Pos))

/* DEBUG_DEMCR sub-register and bit-band aliases */
typedef struct
{
    __IO uint16_t VC_SHORT;             
    __IO uint8_t MON_BYTE;             
    __IO uint8_t TRCENA_BYTE;          
} DEBUG_DEMCR_Type;

#define DEBUG_DEMCR_BASE                0xE000EDFC
#define DEBUG_DEMCR                     ((DEBUG_DEMCR_Type *) DEBUG_DEMCR_BASE)

/* DEBUG_DEMCR settings */
#define DEBUG_MON_DISABLE               ((uint32_t)(0x0U << DEBUG_DEMCR_MON_EN_Pos))
#define DEBUG_MON_ENABLE                ((uint32_t)(0x1U << DEBUG_DEMCR_MON_EN_Pos))

#define DEBUG_MON_CLRPEND               ((uint32_t)(0x0U << DEBUG_DEMCR_MON_PEND_Pos))
#define DEBUG_MON_SETPEND               ((uint32_t)(0x1U << DEBUG_DEMCR_MON_PEND_Pos))

/* DEBUG_DEMCR sub-register bit positions */
#define DEBUG_DEMCR_VC_CORERESET_SHORT_Pos 0
#define DEBUG_DEMCR_VC_MMERR_SHORT_Pos  4
#define DEBUG_DEMCR_VC_NOCPERR_SHORT_Pos 5
#define DEBUG_DEMCR_VC_CHKERR_SHORT_Pos 6
#define DEBUG_DEMCR_VC_STATERR_SHORT_Pos 7
#define DEBUG_DEMCR_VC_BUSERR_SHORT_Pos 8
#define DEBUG_DEMCR_VC_INTERR_SHORT_Pos 9
#define DEBUG_DEMCR_VC_HARDERR_SHORT_Pos 10
#define DEBUG_DEMCR_MON_EN_BYTE_Pos     0
#define DEBUG_DEMCR_MON_PEND_BYTE_Pos   1
#define DEBUG_DEMCR_MON_STEP_BYTE_Pos   2
#define DEBUG_DEMCR_MON_REQ_BYTE_Pos    3
#define DEBUG_DEMCR_TRCENA_BYTE_Pos     0

/* DEBUG_DEMCR sub-register settings */
#define DEBUG_MON_DISABLE_BYTE          ((uint8_t)(0x0U << DEBUG_DEMCR_MON_EN_BYTE_Pos))
#define DEBUG_MON_ENABLE_BYTE           ((uint8_t)(0x1U << DEBUG_DEMCR_MON_EN_BYTE_Pos))

#define DEBUG_MON_CLRPEND_BYTE          ((uint8_t)(0x0U << DEBUG_DEMCR_MON_PEND_BYTE_Pos))
#define DEBUG_MON_SETPEND_BYTE          ((uint8_t)(0x1U << DEBUG_DEMCR_MON_PEND_BYTE_Pos))


#endif /* RSL10_HW_H */
