/* ----------------------------------------------------------------------------
 * Copyright (c) 2015-2017 Semiconductor Components Industries, LLC (d/b/a ON
 * Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ----------------------------------------------------------------------------
 * rsl10_hw_nonCMSIS.h
 * - A set of non-CMSIS compliant definittions for the ARM Cortex-M3 accessible
 *   hardware registers and bit-field definitions. This file should included
 *   instead of the regular rsl10_hw.h to provide register definitions in
 *   assembly files, and other places where the structures, fixed width units, 
 *   and input/output defines used by CMSIS are not available.
 * ----------------------------------------------------------------------------
 * $Revision: 1.4 $
 * $Date: 2017/06/07 19:54:21 $
 * ------------------------------------------------------------------------- */

#ifndef RSL10_HW_H
#define RSL10_HW_H

/* Include the system library register header */
#include <rsl10_reg.h>

/* ----------------------------------------------------------------------------
 * Chip Identification
 * ------------------------------------------------------------------------- */

/* Chip ID number */
#define AHBREGS_CHIP_ID_NUM_BASE        0x1FFFFFFC
#define AHBREGS_CHIP_ID_NUM             READONLY_REG32_POINTER(AHBREGS_CHIP_ID_NUM_BASE)

/* AHBREGS_CHIP_ID_NUM bit positions */
#define AHBREGS_CHIP_ID_NUM_CHIP_FAMILY_POS 24
#define AHBREGS_CHIP_ID_NUM_CHIP_FAMILY_MASK (0xFF << AHBREGS_CHIP_ID_NUM_CHIP_FAMILY_POS)
#define AHBREGS_CHIP_ID_NUM_CHIP_VERSION_POS 16
#define AHBREGS_CHIP_ID_NUM_CHIP_VERSION_MASK (0xFF << AHBREGS_CHIP_ID_NUM_CHIP_VERSION_POS)
#define AHBREGS_CHIP_ID_NUM_CHIP_MAJOR_REVISION_POS 8
#define AHBREGS_CHIP_ID_NUM_CHIP_MAJOR_REVISION_MASK (0xFF << AHBREGS_CHIP_ID_NUM_CHIP_MAJOR_REVISION_POS)
#define AHBREGS_CHIP_ID_NUM_CHIP_MINOR_REVISION_POS 3
#define AHBREGS_CHIP_ID_NUM_CHIP_MINOR_REVISION_MASK (0x1F << AHBREGS_CHIP_ID_NUM_CHIP_MINOR_REVISION_POS)
#define AHBREGS_CHIP_ID_NUM_OD_FEATURE_POS 2
#define AHBREGS_CHIP_ID_NUM_LPDSP32_FEATURE_POS 1
#define AHBREGS_CHIP_ID_NUM_AOBLE_FEATURE_POS 0

/* AHBREGS_CHIP_ID_NUM bit-band aliases */
#define AHBREGS_CHIP_ID_NUM_OD_FEATURE_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AHBREGS_CHIP_ID_NUM_BASE, AHBREGS_CHIP_ID_NUM_OD_FEATURE_POS))
#define AHBREGS_CHIP_ID_NUM_LPDSP32_FEATURE_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AHBREGS_CHIP_ID_NUM_BASE, AHBREGS_CHIP_ID_NUM_LPDSP32_FEATURE_POS))
#define AHBREGS_CHIP_ID_NUM_AOBLE_FEATURE_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AHBREGS_CHIP_ID_NUM_BASE, AHBREGS_CHIP_ID_NUM_AOBLE_FEATURE_POS))

/* AHBREGS_CHIP_ID_NUM settings */
#define CHIP_ID_OD_NOT_PRESENT_BITBAND  0x0
#define CHIP_ID_OD_PRESENT_BITBAND      0x1
#define CHIP_ID_OD_NOT_PRESENT          (CHIP_ID_OD_NOT_PRESENT_BITBAND << AHBREGS_CHIP_ID_NUM_OD_FEATURE_POS)
#define CHIP_ID_OD_PRESENT              (CHIP_ID_OD_PRESENT_BITBAND << AHBREGS_CHIP_ID_NUM_OD_FEATURE_POS)

#define CHIP_ID_LPDSP32_NOT_PRESENT_BITBAND 0x0
#define CHIP_ID_LPDSP32_PRESENT_BITBAND 0x1
#define CHIP_ID_LPDSP32_NOT_PRESENT     (CHIP_ID_LPDSP32_NOT_PRESENT_BITBAND << AHBREGS_CHIP_ID_NUM_LPDSP32_FEATURE_POS)
#define CHIP_ID_LPDSP32_PRESENT         (CHIP_ID_LPDSP32_PRESENT_BITBAND << AHBREGS_CHIP_ID_NUM_LPDSP32_FEATURE_POS)

#define CHIP_ID_AOBLE_NOT_PRESENT_BITBAND 0x0
#define CHIP_ID_AOBLE_PRESENT_BITBAND   0x1
#define CHIP_ID_AOBLE_NOT_PRESENT       (CHIP_ID_AOBLE_NOT_PRESENT_BITBAND << AHBREGS_CHIP_ID_NUM_AOBLE_FEATURE_POS)
#define CHIP_ID_AOBLE_PRESENT           (CHIP_ID_AOBLE_PRESENT_BITBAND << AHBREGS_CHIP_ID_NUM_AOBLE_FEATURE_POS)

/* ----------------------------------------------------------------------------
 * System Control
 * ------------------------------------------------------------------------- */

/* DSS Control */
#define SYSCTRL_DSS_CTRL_BASE           0x40000000
#define SYSCTRL_DSS_CTRL                REG32_POINTER(SYSCTRL_DSS_CTRL_BASE)

/* SYSCTRL_DSS_CTRL bit positions */
#define SYSCTRL_DSS_CTRL_LPDSP32_STATUS_POS 8
#define SYSCTRL_DSS_CTRL_DSS_CSS_INT_RESET_POS 5
#define SYSCTRL_DSS_CTRL_DSS_DMA_INT_RESET_POS 4
#define SYSCTRL_DSS_CTRL_DSS_RESET_POS  3
#define SYSCTRL_DSS_CTRL_LPDSP32_PAUSE_POS 2
#define SYSCTRL_DSS_CTRL_LPDSP32_RESUME_POS 1
#define SYSCTRL_DSS_CTRL_LPDSP32_RUNNING_POS 0

/* SYSCTRL_DSS_CTRL bit-band aliases */
#define SYSCTRL_DSS_CTRL_LPDSP32_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CTRL_BASE, SYSCTRL_DSS_CTRL_LPDSP32_STATUS_POS))
#define SYSCTRL_DSS_CTRL_DSS_CSS_INT_RESET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CTRL_BASE, SYSCTRL_DSS_CTRL_DSS_CSS_INT_RESET_POS))
#define SYSCTRL_DSS_CTRL_DSS_DMA_INT_RESET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CTRL_BASE, SYSCTRL_DSS_CTRL_DSS_DMA_INT_RESET_POS))
#define SYSCTRL_DSS_CTRL_DSS_RESET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CTRL_BASE, SYSCTRL_DSS_CTRL_DSS_RESET_POS))
#define SYSCTRL_DSS_CTRL_LPDSP32_PAUSE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CTRL_BASE, SYSCTRL_DSS_CTRL_LPDSP32_PAUSE_POS))
#define SYSCTRL_DSS_CTRL_LPDSP32_RESUME_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CTRL_BASE, SYSCTRL_DSS_CTRL_LPDSP32_RESUME_POS))
#define SYSCTRL_DSS_CTRL_LPDSP32_RUNNING_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CTRL_BASE, SYSCTRL_DSS_CTRL_LPDSP32_RUNNING_POS))

/* SYSCTRL_DSS_CTRL settings */
#define LPDSP32_DISABLED_DEVICE_BITBAND 0x0
#define LPDSP32_ENABLED_DEVICE_BITBAND  0x1
#define LPDSP32_DISABLED_DEVICE         (LPDSP32_DISABLED_DEVICE_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_STATUS_POS)
#define LPDSP32_ENABLED_DEVICE          (LPDSP32_ENABLED_DEVICE_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_STATUS_POS)

#define DSS_CSS_INT_RESET_BITBAND       0x1
#define DSS_CSS_INT_RESET               (DSS_CSS_INT_RESET_BITBAND << SYSCTRL_DSS_CTRL_DSS_CSS_INT_RESET_POS)

#define DSS_DMA_INT_RESET_BITBAND       0x1
#define DSS_DMA_INT_RESET               (DSS_DMA_INT_RESET_BITBAND << SYSCTRL_DSS_CTRL_DSS_DMA_INT_RESET_POS)

#define DSS_RESET_BITBAND               0x1
#define DSS_RESET                       (DSS_RESET_BITBAND << SYSCTRL_DSS_CTRL_DSS_RESET_POS)

#define DSS_LPDSP32_PAUSE_BITBAND       0x1
#define DSS_LPDSP32_PAUSE               (DSS_LPDSP32_PAUSE_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_PAUSE_POS)

#define DSS_LPDSP32_RESUME_BITBAND      0x1
#define DSS_LPDSP32_RESUME              (DSS_LPDSP32_RESUME_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_RESUME_POS)

#define DSS_LPDSP32_STATE_PAUSE_BITBAND 0x0
#define DSS_LPDSP32_STATE_RUN_BITBAND   0x1
#define DSS_LPDSP32_STATE_PAUSE         (DSS_LPDSP32_STATE_PAUSE_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_RUNNING_POS)
#define DSS_LPDSP32_STATE_RUN           (DSS_LPDSP32_STATE_RUN_BITBAND << SYSCTRL_DSS_CTRL_LPDSP32_RUNNING_POS)

/* DSS Commands */
#define SYSCTRL_DSS_CMD_BASE            0x40000004
#define SYSCTRL_DSS_CMD                 REG32_POINTER(SYSCTRL_DSS_CMD_BASE)

/* SYSCTRL_DSS_CMD bit positions */
#define SYSCTRL_DSS_CMD_DSS_CMD_6_POS   6
#define SYSCTRL_DSS_CMD_DSS_CMD_5_POS   5
#define SYSCTRL_DSS_CMD_DSS_CMD_4_POS   4
#define SYSCTRL_DSS_CMD_DSS_CMD_3_POS   3
#define SYSCTRL_DSS_CMD_DSS_CMD_2_POS   2
#define SYSCTRL_DSS_CMD_DSS_CMD_1_POS   1
#define SYSCTRL_DSS_CMD_DSS_CMD_0_POS   0

/* SYSCTRL_DSS_CMD bit-band aliases */
#define SYSCTRL_DSS_CMD_DSS_CMD_6_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CMD_BASE, SYSCTRL_DSS_CMD_DSS_CMD_6_POS))
#define SYSCTRL_DSS_CMD_DSS_CMD_5_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CMD_BASE, SYSCTRL_DSS_CMD_DSS_CMD_5_POS))
#define SYSCTRL_DSS_CMD_DSS_CMD_4_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CMD_BASE, SYSCTRL_DSS_CMD_DSS_CMD_4_POS))
#define SYSCTRL_DSS_CMD_DSS_CMD_3_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CMD_BASE, SYSCTRL_DSS_CMD_DSS_CMD_3_POS))
#define SYSCTRL_DSS_CMD_DSS_CMD_2_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CMD_BASE, SYSCTRL_DSS_CMD_DSS_CMD_2_POS))
#define SYSCTRL_DSS_CMD_DSS_CMD_1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CMD_BASE, SYSCTRL_DSS_CMD_DSS_CMD_1_POS))
#define SYSCTRL_DSS_CMD_DSS_CMD_0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_CMD_BASE, SYSCTRL_DSS_CMD_DSS_CMD_0_POS))

/* SYSCTRL_DSS_CMD settings */
#define DSS_CMD_6_BITBAND               0x1
#define DSS_CMD_6                       (DSS_CMD_6_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_6_POS)

#define DSS_CMD_5_BITBAND               0x1
#define DSS_CMD_5                       (DSS_CMD_5_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_5_POS)

#define DSS_CMD_4_BITBAND               0x1
#define DSS_CMD_4                       (DSS_CMD_4_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_4_POS)

#define DSS_CMD_3_BITBAND               0x1
#define DSS_CMD_3                       (DSS_CMD_3_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_3_POS)

#define DSS_CMD_2_BITBAND               0x1
#define DSS_CMD_2                       (DSS_CMD_2_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_2_POS)

#define DSS_CMD_1_BITBAND               0x1
#define DSS_CMD_1                       (DSS_CMD_1_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_1_POS)

#define DSS_CMD_0_BITBAND               0x1
#define DSS_CMD_0                       (DSS_CMD_0_BITBAND << SYSCTRL_DSS_CMD_DSS_CMD_0_POS)

/* Flash Overlay Configuration */
#define SYSCTRL_FLASH_OVERLAY_CFG_BASE  0x40000008
#define SYSCTRL_FLASH_OVERLAY_CFG       REG32_POINTER(SYSCTRL_FLASH_OVERLAY_CFG_BASE)

/* SYSCTRL_FLASH_OVERLAY_CFG bit positions */
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM0_OVERLAY_CFG_POS 7
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM1_OVERLAY_CFG_POS 6
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM2_OVERLAY_CFG_POS 5
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM3_OVERLAY_CFG_POS 4
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM3_OVERLAY_CFG_POS 3
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM2_OVERLAY_CFG_POS 2
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM1_OVERLAY_CFG_POS 1
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM0_OVERLAY_CFG_POS 0

/* SYSCTRL_FLASH_OVERLAY_CFG bit-band aliases */
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM0_OVERLAY_CFG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_FLASH_OVERLAY_CFG_BASE, SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM0_OVERLAY_CFG_POS))
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM1_OVERLAY_CFG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_FLASH_OVERLAY_CFG_BASE, SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM1_OVERLAY_CFG_POS))
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM2_OVERLAY_CFG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_FLASH_OVERLAY_CFG_BASE, SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM2_OVERLAY_CFG_POS))
#define SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM3_OVERLAY_CFG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_FLASH_OVERLAY_CFG_BASE, SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM3_OVERLAY_CFG_POS))
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM3_OVERLAY_CFG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_FLASH_OVERLAY_CFG_BASE, SYSCTRL_FLASH_OVERLAY_CFG_PRAM3_OVERLAY_CFG_POS))
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM2_OVERLAY_CFG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_FLASH_OVERLAY_CFG_BASE, SYSCTRL_FLASH_OVERLAY_CFG_PRAM2_OVERLAY_CFG_POS))
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM1_OVERLAY_CFG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_FLASH_OVERLAY_CFG_BASE, SYSCTRL_FLASH_OVERLAY_CFG_PRAM1_OVERLAY_CFG_POS))
#define SYSCTRL_FLASH_OVERLAY_CFG_PRAM0_OVERLAY_CFG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_FLASH_OVERLAY_CFG_BASE, SYSCTRL_FLASH_OVERLAY_CFG_PRAM0_OVERLAY_CFG_POS))

/* SYSCTRL_FLASH_OVERLAY_CFG settings */
#define DSP_PRAM0_OVERLAY_DISABLE_BITBAND 0x0
#define DSP_PRAM0_OVERLAY_ENABLE_BITBAND 0x1
#define DSP_PRAM0_OVERLAY_DISABLE       (DSP_PRAM0_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM0_OVERLAY_CFG_POS)
#define DSP_PRAM0_OVERLAY_ENABLE        (DSP_PRAM0_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM0_OVERLAY_CFG_POS)

#define DSP_PRAM1_OVERLAY_DISABLE_BITBAND 0x0
#define DSP_PRAM1_OVERLAY_ENABLE_BITBAND 0x1
#define DSP_PRAM1_OVERLAY_DISABLE       (DSP_PRAM1_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM1_OVERLAY_CFG_POS)
#define DSP_PRAM1_OVERLAY_ENABLE        (DSP_PRAM1_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM1_OVERLAY_CFG_POS)

#define DSP_PRAM2_OVERLAY_DISABLE_BITBAND 0x0
#define DSP_PRAM2_OVERLAY_ENABLE_BITBAND 0x1
#define DSP_PRAM2_OVERLAY_DISABLE       (DSP_PRAM2_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM2_OVERLAY_CFG_POS)
#define DSP_PRAM2_OVERLAY_ENABLE        (DSP_PRAM2_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM2_OVERLAY_CFG_POS)

#define DSP_PRAM3_OVERLAY_DISABLE_BITBAND 0x0
#define DSP_PRAM3_OVERLAY_ENABLE_BITBAND 0x1
#define DSP_PRAM3_OVERLAY_DISABLE       (DSP_PRAM3_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM3_OVERLAY_CFG_POS)
#define DSP_PRAM3_OVERLAY_ENABLE        (DSP_PRAM3_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_DSP_PRAM3_OVERLAY_CFG_POS)

#define PRAM3_OVERLAY_DISABLE_BITBAND   0x0
#define PRAM3_OVERLAY_ENABLE_BITBAND    0x1
#define PRAM3_OVERLAY_DISABLE           (PRAM3_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM3_OVERLAY_CFG_POS)
#define PRAM3_OVERLAY_ENABLE            (PRAM3_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM3_OVERLAY_CFG_POS)

#define PRAM2_OVERLAY_DISABLE_BITBAND   0x0
#define PRAM2_OVERLAY_ENABLE_BITBAND    0x1
#define PRAM2_OVERLAY_DISABLE           (PRAM2_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM2_OVERLAY_CFG_POS)
#define PRAM2_OVERLAY_ENABLE            (PRAM2_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM2_OVERLAY_CFG_POS)

#define PRAM1_OVERLAY_DISABLE_BITBAND   0x0
#define PRAM1_OVERLAY_ENABLE_BITBAND    0x1
#define PRAM1_OVERLAY_DISABLE           (PRAM1_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM1_OVERLAY_CFG_POS)
#define PRAM1_OVERLAY_ENABLE            (PRAM1_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM1_OVERLAY_CFG_POS)

#define PRAM0_OVERLAY_DISABLE_BITBAND   0x0
#define PRAM0_OVERLAY_ENABLE_BITBAND    0x1
#define PRAM0_OVERLAY_DISABLE           (PRAM0_OVERLAY_DISABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM0_OVERLAY_CFG_POS)
#define PRAM0_OVERLAY_ENABLE            (PRAM0_OVERLAY_ENABLE_BITBAND << SYSCTRL_FLASH_OVERLAY_CFG_PRAM0_OVERLAY_CFG_POS)

/* CSS Loop Cache Configuration */
#define SYSCTRL_CSS_LOOP_CACHE_CFG_BASE 0x4000000C
#define SYSCTRL_CSS_LOOP_CACHE_CFG      REG32_POINTER(SYSCTRL_CSS_LOOP_CACHE_CFG_BASE)

/* SYSCTRL_CSS_LOOP_CACHE_CFG bit positions */
#define SYSCTRL_CSS_LOOP_CACHE_CFG_CSS_LOOP_CACHE_ENABLE_POS 0

/* SYSCTRL_CSS_LOOP_CACHE_CFG bit-band aliases */
#define SYSCTRL_CSS_LOOP_CACHE_CFG_CSS_LOOP_CACHE_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_CSS_LOOP_CACHE_CFG_BASE, SYSCTRL_CSS_LOOP_CACHE_CFG_CSS_LOOP_CACHE_ENABLE_POS))

/* SYSCTRL_CSS_LOOP_CACHE_CFG settings */
#define CSS_LOOP_CACHE_DISABLE_BITBAND  0x0
#define CSS_LOOP_CACHE_ENABLE_BITBAND   0x1
#define CSS_LOOP_CACHE_DISABLE          (CSS_LOOP_CACHE_DISABLE_BITBAND << SYSCTRL_CSS_LOOP_CACHE_CFG_CSS_LOOP_CACHE_ENABLE_POS)
#define CSS_LOOP_CACHE_ENABLE           (CSS_LOOP_CACHE_ENABLE_BITBAND << SYSCTRL_CSS_LOOP_CACHE_CFG_CSS_LOOP_CACHE_ENABLE_POS)

/* DSS Loop Cache Configuration */
#define SYSCTRL_DSS_LOOP_CACHE_CFG_BASE 0x40000010
#define SYSCTRL_DSS_LOOP_CACHE_CFG      REG32_POINTER(SYSCTRL_DSS_LOOP_CACHE_CFG_BASE)

/* SYSCTRL_DSS_LOOP_CACHE_CFG bit positions */
#define SYSCTRL_DSS_LOOP_CACHE_CFG_DSS_LOOP_CACHE_ENABLE_POS 0

/* SYSCTRL_DSS_LOOP_CACHE_CFG bit-band aliases */
#define SYSCTRL_DSS_LOOP_CACHE_CFG_DSS_LOOP_CACHE_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DSS_LOOP_CACHE_CFG_BASE, SYSCTRL_DSS_LOOP_CACHE_CFG_DSS_LOOP_CACHE_ENABLE_POS))

/* SYSCTRL_DSS_LOOP_CACHE_CFG settings */
#define DSS_LOOP_CACHE_DISABLE_BITBAND  0x0
#define DSS_LOOP_CACHE_ENABLE_BITBAND   0x1
#define DSS_LOOP_CACHE_DISABLE          (DSS_LOOP_CACHE_DISABLE_BITBAND << SYSCTRL_DSS_LOOP_CACHE_CFG_DSS_LOOP_CACHE_ENABLE_POS)
#define DSS_LOOP_CACHE_ENABLE           (DSS_LOOP_CACHE_ENABLE_BITBAND << SYSCTRL_DSS_LOOP_CACHE_CFG_DSS_LOOP_CACHE_ENABLE_POS)

/* Memory Access Error Flags */
#define SYSCTRL_MEM_ERROR_BASE          0x40000014
#define SYSCTRL_MEM_ERROR               REG32_POINTER(SYSCTRL_MEM_ERROR_BASE)

/* SYSCTRL_MEM_ERROR bit positions */
#define SYSCTRL_MEM_ERROR_MEM_ERROR_CLEAR_POS 5
#define SYSCTRL_MEM_ERROR_BB_MEM_ERROR_POS 4
#define SYSCTRL_MEM_ERROR_FLASH_COPIER_MEM_ERROR_POS 3
#define SYSCTRL_MEM_ERROR_DMA_MEM_ERROR_POS 2
#define SYSCTRL_MEM_ERROR_LPDSP32_DMEM_ERROR_POS 1
#define SYSCTRL_MEM_ERROR_LPDSP32_PMEM_ERROR_POS 0

/* SYSCTRL_MEM_ERROR bit-band aliases */
#define SYSCTRL_MEM_ERROR_MEM_ERROR_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ERROR_BASE, SYSCTRL_MEM_ERROR_MEM_ERROR_CLEAR_POS))
#define SYSCTRL_MEM_ERROR_BB_MEM_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ERROR_BASE, SYSCTRL_MEM_ERROR_BB_MEM_ERROR_POS))
#define SYSCTRL_MEM_ERROR_FLASH_COPIER_MEM_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ERROR_BASE, SYSCTRL_MEM_ERROR_FLASH_COPIER_MEM_ERROR_POS))
#define SYSCTRL_MEM_ERROR_DMA_MEM_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ERROR_BASE, SYSCTRL_MEM_ERROR_DMA_MEM_ERROR_POS))
#define SYSCTRL_MEM_ERROR_LPDSP32_DMEM_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ERROR_BASE, SYSCTRL_MEM_ERROR_LPDSP32_DMEM_ERROR_POS))
#define SYSCTRL_MEM_ERROR_LPDSP32_PMEM_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ERROR_BASE, SYSCTRL_MEM_ERROR_LPDSP32_PMEM_ERROR_POS))

/* SYSCTRL_MEM_ERROR settings */
#define MEM_ERROR_CLEAR_BITBAND         0x1
#define MEM_ERROR_CLEAR                 (MEM_ERROR_CLEAR_BITBAND << SYSCTRL_MEM_ERROR_MEM_ERROR_CLEAR_POS)

#define BB_MEM_NO_ERROR_DETECTED_BITBAND 0x0
#define BB_MEM_ERROR_DETECTED_BITBAND   0x1
#define BB_MEM_NO_ERROR_DETECTED        (BB_MEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_BB_MEM_ERROR_POS)
#define BB_MEM_ERROR_DETECTED           (BB_MEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_BB_MEM_ERROR_POS)

#define FLASH_COPIER_MEM_NO_ERROR_DETECTED_BITBAND 0x0
#define FLASH_COPIER_MEM_ERROR_DETECTED_BITBAND 0x1
#define FLASH_COPIER_MEM_NO_ERROR_DETECTED (FLASH_COPIER_MEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_FLASH_COPIER_MEM_ERROR_POS)
#define FLASH_COPIER_MEM_ERROR_DETECTED (FLASH_COPIER_MEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_FLASH_COPIER_MEM_ERROR_POS)

#define DMA_MEM_NO_ERROR_DETECTED_BITBAND 0x0
#define DMA_MEM_ERROR_DETECTED_BITBAND  0x1
#define DMA_MEM_NO_ERROR_DETECTED       (DMA_MEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_DMA_MEM_ERROR_POS)
#define DMA_MEM_ERROR_DETECTED          (DMA_MEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_DMA_MEM_ERROR_POS)

#define LPDSP32_DMEM_NO_ERROR_DETECTED_BITBAND 0x0
#define LPDSP32_DMEM_ERROR_DETECTED_BITBAND 0x1
#define LPDSP32_DMEM_NO_ERROR_DETECTED  (LPDSP32_DMEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_LPDSP32_DMEM_ERROR_POS)
#define LPDSP32_DMEM_ERROR_DETECTED     (LPDSP32_DMEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_LPDSP32_DMEM_ERROR_POS)

#define LPDSP32_PMEM_NO_ERROR_DETECTED_BITBAND 0x0
#define LPDSP32_PMEM_ERROR_DETECTED_BITBAND 0x1
#define LPDSP32_PMEM_NO_ERROR_DETECTED  (LPDSP32_PMEM_NO_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_LPDSP32_PMEM_ERROR_POS)
#define LPDSP32_PMEM_ERROR_DETECTED     (LPDSP32_PMEM_ERROR_DETECTED_BITBAND << SYSCTRL_MEM_ERROR_LPDSP32_PMEM_ERROR_POS)

/* Memory Power Configuration */
#define SYSCTRL_MEM_POWER_CFG_BASE      0x40000018
#define SYSCTRL_MEM_POWER_CFG           REG32_POINTER(SYSCTRL_MEM_POWER_CFG_BASE)

/* SYSCTRL_MEM_POWER_CFG bit positions */
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM5_POWER_POS 21
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM4_POWER_POS 20
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM3_POWER_POS 19
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM2_POWER_POS 18
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM1_POWER_POS 17
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM0_POWER_POS 16
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM3_POWER_POS 15
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM2_POWER_POS 14
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM1_POWER_POS 13
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM0_POWER_POS 12
#define SYSCTRL_MEM_POWER_CFG_BB_DRAM1_POWER_POS 11
#define SYSCTRL_MEM_POWER_CFG_BB_DRAM0_POWER_POS 10
#define SYSCTRL_MEM_POWER_CFG_DRAM2_POWER_POS 8
#define SYSCTRL_MEM_POWER_CFG_DRAM1_POWER_POS 7
#define SYSCTRL_MEM_POWER_CFG_DRAM0_POWER_POS 6
#define SYSCTRL_MEM_POWER_CFG_PRAM3_POWER_POS 5
#define SYSCTRL_MEM_POWER_CFG_PRAM2_POWER_POS 4
#define SYSCTRL_MEM_POWER_CFG_PRAM1_POWER_POS 3
#define SYSCTRL_MEM_POWER_CFG_PRAM0_POWER_POS 2
#define SYSCTRL_MEM_POWER_CFG_FLASH_POWER_POS 1
#define SYSCTRL_MEM_POWER_CFG_PROM_POWER_POS 0

/* SYSCTRL_MEM_POWER_CFG bit-band aliases */
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM5_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_DRAM5_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM4_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_DRAM4_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM3_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_DRAM3_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM2_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_DRAM2_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM1_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_DRAM1_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_DRAM0_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_DRAM0_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM3_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_PRAM3_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM2_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_PRAM2_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM1_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_PRAM1_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DSP_PRAM0_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DSP_PRAM0_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_BB_DRAM1_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_BB_DRAM1_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_BB_DRAM0_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_BB_DRAM0_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DRAM2_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DRAM2_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DRAM1_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DRAM1_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_DRAM0_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_DRAM0_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_PRAM3_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_PRAM3_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_PRAM2_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_PRAM2_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_PRAM1_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_PRAM1_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_PRAM0_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_PRAM0_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_FLASH_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_FLASH_POWER_POS))
#define SYSCTRL_MEM_POWER_CFG_PROM_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_POWER_CFG_BASE, SYSCTRL_MEM_POWER_CFG_PROM_POWER_POS))

/* SYSCTRL_MEM_POWER_CFG settings */
#define DSP_DRAM5_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM5_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM5_POWER_DISABLE         (DSP_DRAM5_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM5_POWER_POS)
#define DSP_DRAM5_POWER_ENABLE          (DSP_DRAM5_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM5_POWER_POS)

#define DSP_DRAM4_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM4_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM4_POWER_DISABLE         (DSP_DRAM4_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM4_POWER_POS)
#define DSP_DRAM4_POWER_ENABLE          (DSP_DRAM4_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM4_POWER_POS)

#define DSP_DRAM3_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM3_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM3_POWER_DISABLE         (DSP_DRAM3_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM3_POWER_POS)
#define DSP_DRAM3_POWER_ENABLE          (DSP_DRAM3_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM3_POWER_POS)

#define DSP_DRAM2_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM2_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM2_POWER_DISABLE         (DSP_DRAM2_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM2_POWER_POS)
#define DSP_DRAM2_POWER_ENABLE          (DSP_DRAM2_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM2_POWER_POS)

#define DSP_DRAM1_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM1_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM1_POWER_DISABLE         (DSP_DRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM1_POWER_POS)
#define DSP_DRAM1_POWER_ENABLE          (DSP_DRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM1_POWER_POS)

#define DSP_DRAM0_POWER_DISABLE_BITBAND 0x0
#define DSP_DRAM0_POWER_ENABLE_BITBAND  0x1
#define DSP_DRAM0_POWER_DISABLE         (DSP_DRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM0_POWER_POS)
#define DSP_DRAM0_POWER_ENABLE          (DSP_DRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_DRAM0_POWER_POS)

#define DSP_PRAM3_POWER_DISABLE_BITBAND 0x0
#define DSP_PRAM3_POWER_ENABLE_BITBAND  0x1
#define DSP_PRAM3_POWER_DISABLE         (DSP_PRAM3_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM3_POWER_POS)
#define DSP_PRAM3_POWER_ENABLE          (DSP_PRAM3_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM3_POWER_POS)

#define DSP_PRAM2_POWER_DISABLE_BITBAND 0x0
#define DSP_PRAM2_POWER_ENABLE_BITBAND  0x1
#define DSP_PRAM2_POWER_DISABLE         (DSP_PRAM2_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM2_POWER_POS)
#define DSP_PRAM2_POWER_ENABLE          (DSP_PRAM2_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM2_POWER_POS)

#define DSP_PRAM1_POWER_DISABLE_BITBAND 0x0
#define DSP_PRAM1_POWER_ENABLE_BITBAND  0x1
#define DSP_PRAM1_POWER_DISABLE         (DSP_PRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM1_POWER_POS)
#define DSP_PRAM1_POWER_ENABLE          (DSP_PRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM1_POWER_POS)

#define DSP_PRAM0_POWER_DISABLE_BITBAND 0x0
#define DSP_PRAM0_POWER_ENABLE_BITBAND  0x1
#define DSP_PRAM0_POWER_DISABLE         (DSP_PRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM0_POWER_POS)
#define DSP_PRAM0_POWER_ENABLE          (DSP_PRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DSP_PRAM0_POWER_POS)

#define BB_DRAM1_POWER_DISABLE_BITBAND  0x0
#define BB_DRAM1_POWER_ENABLE_BITBAND   0x1
#define BB_DRAM1_POWER_DISABLE          (BB_DRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_BB_DRAM1_POWER_POS)
#define BB_DRAM1_POWER_ENABLE           (BB_DRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_BB_DRAM1_POWER_POS)

#define BB_DRAM0_POWER_DISABLE_BITBAND  0x0
#define BB_DRAM0_POWER_ENABLE_BITBAND   0x1
#define BB_DRAM0_POWER_DISABLE          (BB_DRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_BB_DRAM0_POWER_POS)
#define BB_DRAM0_POWER_ENABLE           (BB_DRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_BB_DRAM0_POWER_POS)

#define DRAM2_POWER_DISABLE_BITBAND     0x0
#define DRAM2_POWER_ENABLE_BITBAND      0x1
#define DRAM2_POWER_DISABLE             (DRAM2_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM2_POWER_POS)
#define DRAM2_POWER_ENABLE              (DRAM2_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM2_POWER_POS)

#define DRAM1_POWER_DISABLE_BITBAND     0x0
#define DRAM1_POWER_ENABLE_BITBAND      0x1
#define DRAM1_POWER_DISABLE             (DRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM1_POWER_POS)
#define DRAM1_POWER_ENABLE              (DRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM1_POWER_POS)

#define DRAM0_POWER_DISABLE_BITBAND     0x0
#define DRAM0_POWER_ENABLE_BITBAND      0x1
#define DRAM0_POWER_DISABLE             (DRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM0_POWER_POS)
#define DRAM0_POWER_ENABLE              (DRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_DRAM0_POWER_POS)

#define PRAM3_POWER_DISABLE_BITBAND     0x0
#define PRAM3_POWER_ENABLE_BITBAND      0x1
#define PRAM3_POWER_DISABLE             (PRAM3_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM3_POWER_POS)
#define PRAM3_POWER_ENABLE              (PRAM3_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM3_POWER_POS)

#define PRAM2_POWER_DISABLE_BITBAND     0x0
#define PRAM2_POWER_ENABLE_BITBAND      0x1
#define PRAM2_POWER_DISABLE             (PRAM2_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM2_POWER_POS)
#define PRAM2_POWER_ENABLE              (PRAM2_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM2_POWER_POS)

#define PRAM1_POWER_DISABLE_BITBAND     0x0
#define PRAM1_POWER_ENABLE_BITBAND      0x1
#define PRAM1_POWER_DISABLE             (PRAM1_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM1_POWER_POS)
#define PRAM1_POWER_ENABLE              (PRAM1_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM1_POWER_POS)

#define PRAM0_POWER_DISABLE_BITBAND     0x0
#define PRAM0_POWER_ENABLE_BITBAND      0x1
#define PRAM0_POWER_DISABLE             (PRAM0_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM0_POWER_POS)
#define PRAM0_POWER_ENABLE              (PRAM0_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PRAM0_POWER_POS)

#define FLASH_POWER_DISABLE_BITBAND     0x0
#define FLASH_POWER_ENABLE_BITBAND      0x1
#define FLASH_POWER_DISABLE             (FLASH_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_FLASH_POWER_POS)
#define FLASH_POWER_ENABLE              (FLASH_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_FLASH_POWER_POS)

#define PROM_POWER_DISABLE_BITBAND      0x0
#define PROM_POWER_ENABLE_BITBAND       0x1
#define PROM_POWER_DISABLE              (PROM_POWER_DISABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PROM_POWER_POS)
#define PROM_POWER_ENABLE               (PROM_POWER_ENABLE_BITBAND << SYSCTRL_MEM_POWER_CFG_PROM_POWER_POS)

/* Memory Access Configuration and Wakeup Restore Address in packed 7-bit format */
#define SYSCTRL_MEM_ACCESS_CFG_BASE     0x4000001C
#define SYSCTRL_MEM_ACCESS_CFG          REG32_POINTER(SYSCTRL_MEM_ACCESS_CFG_BASE)

/* SYSCTRL_MEM_ACCESS_CFG bit positions */
#define SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_POS 24
#define SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_MASK (0x7F << SYSCTRL_MEM_ACCESS_CFG_WAKEUP_ADDR_PACKED_POS)
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM5_ACCESS_POS 21
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM4_ACCESS_POS 20
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM3_ACCESS_POS 19
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM2_ACCESS_POS 18
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM1_ACCESS_POS 17
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM0_ACCESS_POS 16
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM3_ACCESS_POS 15
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM2_ACCESS_POS 14
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM1_ACCESS_POS 13
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM0_ACCESS_POS 12
#define SYSCTRL_MEM_ACCESS_CFG_BB_DRAM1_ACCESS_POS 11
#define SYSCTRL_MEM_ACCESS_CFG_BB_DRAM0_ACCESS_POS 10
#define SYSCTRL_MEM_ACCESS_CFG_DRAM2_ACCESS_POS 8
#define SYSCTRL_MEM_ACCESS_CFG_DRAM1_ACCESS_POS 7
#define SYSCTRL_MEM_ACCESS_CFG_DRAM0_ACCESS_POS 6
#define SYSCTRL_MEM_ACCESS_CFG_PRAM3_ACCESS_POS 5
#define SYSCTRL_MEM_ACCESS_CFG_PRAM2_ACCESS_POS 4
#define SYSCTRL_MEM_ACCESS_CFG_PRAM1_ACCESS_POS 3
#define SYSCTRL_MEM_ACCESS_CFG_PRAM0_ACCESS_POS 2
#define SYSCTRL_MEM_ACCESS_CFG_FLASH_ACCESS_POS 1
#define SYSCTRL_MEM_ACCESS_CFG_PROM_ACCESS_POS 0

/* SYSCTRL_MEM_ACCESS_CFG bit-band aliases */
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM5_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM5_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM4_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM4_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM3_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM3_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM2_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM2_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM1_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM1_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM0_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM0_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM3_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM3_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM2_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM2_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM1_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM1_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM0_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM0_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_BB_DRAM1_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_BB_DRAM1_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_BB_DRAM0_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_BB_DRAM0_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DRAM2_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DRAM2_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DRAM1_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DRAM1_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_DRAM0_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_DRAM0_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_PRAM3_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_PRAM3_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_PRAM2_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_PRAM2_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_PRAM1_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_PRAM1_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_PRAM0_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_PRAM0_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_FLASH_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_FLASH_ACCESS_POS))
#define SYSCTRL_MEM_ACCESS_CFG_PROM_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ACCESS_CFG_BASE, SYSCTRL_MEM_ACCESS_CFG_PROM_ACCESS_POS))

/* SYSCTRL_MEM_ACCESS_CFG settings */
#define DSP_DRAM5_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM5_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM5_ACCESS_DISABLE        (DSP_DRAM5_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM5_ACCESS_POS)
#define DSP_DRAM5_ACCESS_ENABLE         (DSP_DRAM5_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM5_ACCESS_POS)

#define DSP_DRAM4_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM4_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM4_ACCESS_DISABLE        (DSP_DRAM4_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM4_ACCESS_POS)
#define DSP_DRAM4_ACCESS_ENABLE         (DSP_DRAM4_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM4_ACCESS_POS)

#define DSP_DRAM3_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM3_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM3_ACCESS_DISABLE        (DSP_DRAM3_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM3_ACCESS_POS)
#define DSP_DRAM3_ACCESS_ENABLE         (DSP_DRAM3_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM3_ACCESS_POS)

#define DSP_DRAM2_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM2_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM2_ACCESS_DISABLE        (DSP_DRAM2_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM2_ACCESS_POS)
#define DSP_DRAM2_ACCESS_ENABLE         (DSP_DRAM2_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM2_ACCESS_POS)

#define DSP_DRAM1_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM1_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM1_ACCESS_DISABLE        (DSP_DRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM1_ACCESS_POS)
#define DSP_DRAM1_ACCESS_ENABLE         (DSP_DRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM1_ACCESS_POS)

#define DSP_DRAM0_ACCESS_DISABLE_BITBAND 0x0
#define DSP_DRAM0_ACCESS_ENABLE_BITBAND 0x1
#define DSP_DRAM0_ACCESS_DISABLE        (DSP_DRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM0_ACCESS_POS)
#define DSP_DRAM0_ACCESS_ENABLE         (DSP_DRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_DRAM0_ACCESS_POS)

#define DSP_PRAM3_ACCESS_DISABLE_BITBAND 0x0
#define DSP_PRAM3_ACCESS_ENABLE_BITBAND 0x1
#define DSP_PRAM3_ACCESS_DISABLE        (DSP_PRAM3_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM3_ACCESS_POS)
#define DSP_PRAM3_ACCESS_ENABLE         (DSP_PRAM3_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM3_ACCESS_POS)

#define DSP_PRAM2_ACCESS_DISABLE_BITBAND 0x0
#define DSP_PRAM2_ACCESS_ENABLE_BITBAND 0x1
#define DSP_PRAM2_ACCESS_DISABLE        (DSP_PRAM2_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM2_ACCESS_POS)
#define DSP_PRAM2_ACCESS_ENABLE         (DSP_PRAM2_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM2_ACCESS_POS)

#define DSP_PRAM1_ACCESS_DISABLE_BITBAND 0x0
#define DSP_PRAM1_ACCESS_ENABLE_BITBAND 0x1
#define DSP_PRAM1_ACCESS_DISABLE        (DSP_PRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM1_ACCESS_POS)
#define DSP_PRAM1_ACCESS_ENABLE         (DSP_PRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM1_ACCESS_POS)

#define DSP_PRAM0_ACCESS_DISABLE_BITBAND 0x0
#define DSP_PRAM0_ACCESS_ENABLE_BITBAND 0x1
#define DSP_PRAM0_ACCESS_DISABLE        (DSP_PRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM0_ACCESS_POS)
#define DSP_PRAM0_ACCESS_ENABLE         (DSP_PRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DSP_PRAM0_ACCESS_POS)

#define BB_DRAM1_ACCESS_DISABLE_BITBAND 0x0
#define BB_DRAM1_ACCESS_ENABLE_BITBAND  0x1
#define BB_DRAM1_ACCESS_DISABLE         (BB_DRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_BB_DRAM1_ACCESS_POS)
#define BB_DRAM1_ACCESS_ENABLE          (BB_DRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_BB_DRAM1_ACCESS_POS)

#define BB_DRAM0_ACCESS_DISABLE_BITBAND 0x0
#define BB_DRAM0_ACCESS_ENABLE_BITBAND  0x1
#define BB_DRAM0_ACCESS_DISABLE         (BB_DRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_BB_DRAM0_ACCESS_POS)
#define BB_DRAM0_ACCESS_ENABLE          (BB_DRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_BB_DRAM0_ACCESS_POS)

#define DRAM2_ACCESS_DISABLE_BITBAND    0x0
#define DRAM2_ACCESS_ENABLE_BITBAND     0x1
#define DRAM2_ACCESS_DISABLE            (DRAM2_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM2_ACCESS_POS)
#define DRAM2_ACCESS_ENABLE             (DRAM2_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM2_ACCESS_POS)

#define DRAM1_ACCESS_DISABLE_BITBAND    0x0
#define DRAM1_ACCESS_ENABLE_BITBAND     0x1
#define DRAM1_ACCESS_DISABLE            (DRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM1_ACCESS_POS)
#define DRAM1_ACCESS_ENABLE             (DRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM1_ACCESS_POS)

#define DRAM0_ACCESS_DISABLE_BITBAND    0x0
#define DRAM0_ACCESS_ENABLE_BITBAND     0x1
#define DRAM0_ACCESS_DISABLE            (DRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM0_ACCESS_POS)
#define DRAM0_ACCESS_ENABLE             (DRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_DRAM0_ACCESS_POS)

#define PRAM3_ACCESS_DISABLE_BITBAND    0x0
#define PRAM3_ACCESS_ENABLE_BITBAND     0x1
#define PRAM3_ACCESS_DISABLE            (PRAM3_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM3_ACCESS_POS)
#define PRAM3_ACCESS_ENABLE             (PRAM3_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM3_ACCESS_POS)

#define PRAM2_ACCESS_DISABLE_BITBAND    0x0
#define PRAM2_ACCESS_ENABLE_BITBAND     0x1
#define PRAM2_ACCESS_DISABLE            (PRAM2_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM2_ACCESS_POS)
#define PRAM2_ACCESS_ENABLE             (PRAM2_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM2_ACCESS_POS)

#define PRAM1_ACCESS_DISABLE_BITBAND    0x0
#define PRAM1_ACCESS_ENABLE_BITBAND     0x1
#define PRAM1_ACCESS_DISABLE            (PRAM1_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM1_ACCESS_POS)
#define PRAM1_ACCESS_ENABLE             (PRAM1_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM1_ACCESS_POS)

#define PRAM0_ACCESS_DISABLE_BITBAND    0x0
#define PRAM0_ACCESS_ENABLE_BITBAND     0x1
#define PRAM0_ACCESS_DISABLE            (PRAM0_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM0_ACCESS_POS)
#define PRAM0_ACCESS_ENABLE             (PRAM0_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PRAM0_ACCESS_POS)

#define FLASH_ACCESS_DISABLE_BITBAND    0x0
#define FLASH_ACCESS_ENABLE_BITBAND     0x1
#define FLASH_ACCESS_DISABLE            (FLASH_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_FLASH_ACCESS_POS)
#define FLASH_ACCESS_ENABLE             (FLASH_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_FLASH_ACCESS_POS)

#define PROM_ACCESS_DISABLE_BITBAND     0x0
#define PROM_ACCESS_ENABLE_BITBAND      0x1
#define PROM_ACCESS_DISABLE             (PROM_ACCESS_DISABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PROM_ACCESS_POS)
#define PROM_ACCESS_ENABLE              (PROM_ACCESS_ENABLE_BITBAND << SYSCTRL_MEM_ACCESS_CFG_PROM_ACCESS_POS)

/* Wakeup Restore Address in Unpacked 32-bit Format */
#define SYSCTRL_WAKEUP_ADDR_BASE        0x40000020
#define SYSCTRL_WAKEUP_ADDR             REG32_POINTER(SYSCTRL_WAKEUP_ADDR_BASE)

/* Memory Retention Configuration */
#define SYSCTRL_MEM_RETENTION_CFG_BASE  0x40000024
#define SYSCTRL_MEM_RETENTION_CFG       REG32_POINTER(SYSCTRL_MEM_RETENTION_CFG_BASE)

/* SYSCTRL_MEM_RETENTION_CFG bit positions */
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM5_RETENTION_POS 21
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM4_RETENTION_POS 20
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM3_RETENTION_POS 19
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM2_RETENTION_POS 18
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM1_RETENTION_POS 17
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM0_RETENTION_POS 16
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM3_RETENTION_POS 15
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM2_RETENTION_POS 14
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM1_RETENTION_POS 13
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM0_RETENTION_POS 12
#define SYSCTRL_MEM_RETENTION_CFG_BB_DRAM1_RETENTION_POS 11
#define SYSCTRL_MEM_RETENTION_CFG_BB_DRAM0_RETENTION_POS 10
#define SYSCTRL_MEM_RETENTION_CFG_DRAM2_RETENTION_POS 8
#define SYSCTRL_MEM_RETENTION_CFG_DRAM1_RETENTION_POS 7
#define SYSCTRL_MEM_RETENTION_CFG_DRAM0_RETENTION_POS 6
#define SYSCTRL_MEM_RETENTION_CFG_PRAM3_RETENTION_POS 5
#define SYSCTRL_MEM_RETENTION_CFG_PRAM2_RETENTION_POS 4
#define SYSCTRL_MEM_RETENTION_CFG_PRAM1_RETENTION_POS 3
#define SYSCTRL_MEM_RETENTION_CFG_PRAM0_RETENTION_POS 2

/* SYSCTRL_MEM_RETENTION_CFG bit-band aliases */
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM5_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM5_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM4_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM4_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM3_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM3_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM2_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM2_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM1_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM1_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM0_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM0_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM3_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM3_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM2_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM2_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM1_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM1_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM0_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM0_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_BB_DRAM1_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_BB_DRAM1_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_BB_DRAM0_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_BB_DRAM0_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DRAM2_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DRAM2_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DRAM1_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DRAM1_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_DRAM0_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_DRAM0_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_PRAM3_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_PRAM3_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_PRAM2_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_PRAM2_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_PRAM1_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_PRAM1_RETENTION_POS))
#define SYSCTRL_MEM_RETENTION_CFG_PRAM0_RETENTION_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_RETENTION_CFG_BASE, SYSCTRL_MEM_RETENTION_CFG_PRAM0_RETENTION_POS))

/* SYSCTRL_MEM_RETENTION_CFG settings */
#define DSP_DRAM5_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM5_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM5_NORMAL_MODE           (DSP_DRAM5_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM5_RETENTION_POS)
#define DSP_DRAM5_RETENTION_MODE        (DSP_DRAM5_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM5_RETENTION_POS)

#define DSP_DRAM4_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM4_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM4_NORMAL_MODE           (DSP_DRAM4_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM4_RETENTION_POS)
#define DSP_DRAM4_RETENTION_MODE        (DSP_DRAM4_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM4_RETENTION_POS)

#define DSP_DRAM3_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM3_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM3_NORMAL_MODE           (DSP_DRAM3_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM3_RETENTION_POS)
#define DSP_DRAM3_RETENTION_MODE        (DSP_DRAM3_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM3_RETENTION_POS)

#define DSP_DRAM2_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM2_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM2_NORMAL_MODE           (DSP_DRAM2_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM2_RETENTION_POS)
#define DSP_DRAM2_RETENTION_MODE        (DSP_DRAM2_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM2_RETENTION_POS)

#define DSP_DRAM1_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM1_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM1_NORMAL_MODE           (DSP_DRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM1_RETENTION_POS)
#define DSP_DRAM1_RETENTION_MODE        (DSP_DRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM1_RETENTION_POS)

#define DSP_DRAM0_NORMAL_MODE_BITBAND   0x0
#define DSP_DRAM0_RETENTION_MODE_BITBAND 0x1
#define DSP_DRAM0_NORMAL_MODE           (DSP_DRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM0_RETENTION_POS)
#define DSP_DRAM0_RETENTION_MODE        (DSP_DRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_DRAM0_RETENTION_POS)

#define DSP_PRAM3_NORMAL_MODE_BITBAND   0x0
#define DSP_PRAM3_RETENTION_MODE_BITBAND 0x1
#define DSP_PRAM3_NORMAL_MODE           (DSP_PRAM3_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM3_RETENTION_POS)
#define DSP_PRAM3_RETENTION_MODE        (DSP_PRAM3_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM3_RETENTION_POS)

#define DSP_PRAM2_NORMAL_MODE_BITBAND   0x0
#define DSP_PRAM2_RETENTION_MODE_BITBAND 0x1
#define DSP_PRAM2_NORMAL_MODE           (DSP_PRAM2_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM2_RETENTION_POS)
#define DSP_PRAM2_RETENTION_MODE        (DSP_PRAM2_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM2_RETENTION_POS)

#define DSP_PRAM1_NORMAL_MODE_BITBAND   0x0
#define DSP_PRAM1_RETENTION_MODE_BITBAND 0x1
#define DSP_PRAM1_NORMAL_MODE           (DSP_PRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM1_RETENTION_POS)
#define DSP_PRAM1_RETENTION_MODE        (DSP_PRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM1_RETENTION_POS)

#define DSP_PRAM0_NORMAL_MODE_BITBAND   0x0
#define DSP_PRAM0_RETENTION_MODE_BITBAND 0x1
#define DSP_PRAM0_NORMAL_MODE           (DSP_PRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM0_RETENTION_POS)
#define DSP_PRAM0_RETENTION_MODE        (DSP_PRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DSP_PRAM0_RETENTION_POS)

#define BB_DRAM1_NORMAL_MODE_BITBAND    0x0
#define BB_DRAM1_RETENTION_MODE_BITBAND 0x1
#define BB_DRAM1_NORMAL_MODE            (BB_DRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_BB_DRAM1_RETENTION_POS)
#define BB_DRAM1_RETENTION_MODE         (BB_DRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_BB_DRAM1_RETENTION_POS)

#define BB_DRAM0_NORMAL_MODE_BITBAND    0x0
#define BB_DRAM0_RETENTION_MODE_BITBAND 0x1
#define BB_DRAM0_NORMAL_MODE            (BB_DRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_BB_DRAM0_RETENTION_POS)
#define BB_DRAM0_RETENTION_MODE         (BB_DRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_BB_DRAM0_RETENTION_POS)

#define DRAM2_NORMAL_MODE_BITBAND       0x0
#define DRAM2_RETENTION_MODE_BITBAND    0x1
#define DRAM2_NORMAL_MODE               (DRAM2_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM2_RETENTION_POS)
#define DRAM2_RETENTION_MODE            (DRAM2_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM2_RETENTION_POS)

#define DRAM1_NORMAL_MODE_BITBAND       0x0
#define DRAM1_RETENTION_MODE_BITBAND    0x1
#define DRAM1_NORMAL_MODE               (DRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM1_RETENTION_POS)
#define DRAM1_RETENTION_MODE            (DRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM1_RETENTION_POS)

#define DRAM0_NORMAL_MODE_BITBAND       0x0
#define DRAM0_RETENTION_MODE_BITBAND    0x1
#define DRAM0_NORMAL_MODE               (DRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM0_RETENTION_POS)
#define DRAM0_RETENTION_MODE            (DRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_DRAM0_RETENTION_POS)

#define PRAM3_NORMAL_MODE_BITBAND       0x0
#define PRAM3_RETENTION_MODE_BITBAND    0x1
#define PRAM3_NORMAL_MODE               (PRAM3_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM3_RETENTION_POS)
#define PRAM3_RETENTION_MODE            (PRAM3_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM3_RETENTION_POS)

#define PRAM2_NORMAL_MODE_BITBAND       0x0
#define PRAM2_RETENTION_MODE_BITBAND    0x1
#define PRAM2_NORMAL_MODE               (PRAM2_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM2_RETENTION_POS)
#define PRAM2_RETENTION_MODE            (PRAM2_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM2_RETENTION_POS)

#define PRAM1_NORMAL_MODE_BITBAND       0x0
#define PRAM1_RETENTION_MODE_BITBAND    0x1
#define PRAM1_NORMAL_MODE               (PRAM1_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM1_RETENTION_POS)
#define PRAM1_RETENTION_MODE            (PRAM1_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM1_RETENTION_POS)

#define PRAM0_NORMAL_MODE_BITBAND       0x0
#define PRAM0_RETENTION_MODE_BITBAND    0x1
#define PRAM0_NORMAL_MODE               (PRAM0_NORMAL_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM0_RETENTION_POS)
#define PRAM0_RETENTION_MODE            (PRAM0_RETENTION_MODE_BITBAND << SYSCTRL_MEM_RETENTION_CFG_PRAM0_RETENTION_POS)

/* Memory Arbiter Configuration */
#define SYSCTRL_MEM_ARBITER_CFG_BASE    0x40000028
#define SYSCTRL_MEM_ARBITER_CFG         REG32_POINTER(SYSCTRL_MEM_ARBITER_CFG_BASE)

/* SYSCTRL_MEM_ARBITER_CFG bit positions */
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_POS 28
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_POS 26
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_POS 24
#define SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_POS 22
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_POS 20
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_POS 18
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_POS 16
#define SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_POS 10
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_POS 8
#define SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_POS 4
#define SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_MASK (0x3 << SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_POS)
#define SYSCTRL_MEM_ARBITER_CFG_DRAM0_ARBITER_POS 2
#define SYSCTRL_MEM_ARBITER_CFG_PRAM_ARBITER_POS 1
#define SYSCTRL_MEM_ARBITER_CFG_ROUND_ROBIN_TOKEN_POS 0

/* SYSCTRL_MEM_ARBITER_CFG bit-band aliases */
#define SYSCTRL_MEM_ARBITER_CFG_DRAM0_ARBITER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ARBITER_CFG_BASE, SYSCTRL_MEM_ARBITER_CFG_DRAM0_ARBITER_POS))
#define SYSCTRL_MEM_ARBITER_CFG_PRAM_ARBITER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ARBITER_CFG_BASE, SYSCTRL_MEM_ARBITER_CFG_PRAM_ARBITER_POS))
#define SYSCTRL_MEM_ARBITER_CFG_ROUND_ROBIN_TOKEN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_ARBITER_CFG_BASE, SYSCTRL_MEM_ARBITER_CFG_ROUND_ROBIN_TOKEN_POS))

/* SYSCTRL_MEM_ARBITER_CFG settings */
#define DSP_DRAM45_DSP_PRIORITY         (0x0 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_POS)
#define DSP_DRAM45_CM3_PRIORITY         (0x1 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_POS)
#define DSP_DRAM45_ROUND_ROBIN_PRIORITY (0x2 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM45_ARBITER_POS)

#define DSP_DRAM23_DSP_PRIORITY         (0x0 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_POS)
#define DSP_DRAM23_CM3_PRIORITY         (0x1 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_POS)
#define DSP_DRAM23_ROUND_ROBIN_PRIORITY (0x2 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM23_ARBITER_POS)

#define DSP_DRAM01_DSP_PRIORITY         (0x0 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_POS)
#define DSP_DRAM01_CM3_PRIORITY         (0x1 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_POS)
#define DSP_DRAM01_ROUND_ROBIN_PRIORITY (0x2 << SYSCTRL_MEM_ARBITER_CFG_DSP_DRAM01_ARBITER_POS)

#define DSP_PRAM3_DSP_PRIORITY          (0x0 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_POS)
#define DSP_PRAM3_CM3_PRIORITY          (0x1 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_POS)
#define DSP_PRAM3_ROUND_ROBIN_PRIORITY  (0x2 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM3_ARBITER_POS)

#define DSP_PRAM2_DSP_PRIORITY          (0x0 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_POS)
#define DSP_PRAM2_CM3_PRIORITY          (0x1 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_POS)
#define DSP_PRAM2_ROUND_ROBIN_PRIORITY  (0x2 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM2_ARBITER_POS)

#define DSP_PRAM1_DSP_PRIORITY          (0x0 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_POS)
#define DSP_PRAM1_CM3_PRIORITY          (0x1 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_POS)
#define DSP_PRAM1_ROUND_ROBIN_PRIORITY  (0x2 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM1_ARBITER_POS)

#define DSP_PRAM0_DSP_PRIORITY          (0x0 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_POS)
#define DSP_PRAM0_CM3_PRIORITY          (0x1 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_POS)
#define DSP_PRAM0_ROUND_ROBIN_PRIORITY  (0x2 << SYSCTRL_MEM_ARBITER_CFG_DSP_PRAM0_ARBITER_POS)

#define BB_DRAM1_BB_PRIORITY            (0x0 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_POS)
#define BB_DRAM1_CM3_PRIORITY           (0x1 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_POS)
#define BB_DRAM1_ROUND_ROBIN_PRIORITY   (0x2 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_POS)
#define BB_DRAM1_SMART_PRIORITY         (0x3 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM1_ARBITER_POS)

#define BB_DRAM0_BB_PRIORITY            (0x0 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_POS)
#define BB_DRAM0_CM3_PRIORITY           (0x1 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_POS)
#define BB_DRAM0_ROUND_ROBIN_PRIORITY   (0x2 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_POS)
#define BB_DRAM0_SMART_PRIORITY         (0x3 << SYSCTRL_MEM_ARBITER_CFG_BB_DRAM0_ARBITER_POS)

#define DRAM12_CM3_PRIORITY             (0x0 << SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_POS)
#define DRAM12_DSP_PRIORITY             (0x1 << SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_POS)
#define DRAM12_ROUND_ROBIN_PRIORITY     (0x2 << SYSCTRL_MEM_ARBITER_CFG_DRAM12_ARBITER_POS)

#define DRAM0_CM3_PRIORITY_BITBAND      0x0
#define DRAM0_ROUND_ROBIN_PRIORITY_BITBAND 0x1
#define DRAM0_CM3_PRIORITY              (DRAM0_CM3_PRIORITY_BITBAND << SYSCTRL_MEM_ARBITER_CFG_DRAM0_ARBITER_POS)
#define DRAM0_ROUND_ROBIN_PRIORITY      (DRAM0_ROUND_ROBIN_PRIORITY_BITBAND << SYSCTRL_MEM_ARBITER_CFG_DRAM0_ARBITER_POS)

#define PRAM_CM3_PRIORITY_BITBAND       0x0
#define PRAM_ROUND_ROBIN_PRIORITY_BITBAND 0x1
#define PRAM_CM3_PRIORITY               (PRAM_CM3_PRIORITY_BITBAND << SYSCTRL_MEM_ARBITER_CFG_PRAM_ARBITER_POS)
#define PRAM_ROUND_ROBIN_PRIORITY       (PRAM_ROUND_ROBIN_PRIORITY_BITBAND << SYSCTRL_MEM_ARBITER_CFG_PRAM_ARBITER_POS)

#define REALTIME_DMA_MODE_BITBAND       0x0
#define ROUND_ROBIN_MODE_BITBAND        0x1
#define REALTIME_DMA_MODE               (REALTIME_DMA_MODE_BITBAND << SYSCTRL_MEM_ARBITER_CFG_ROUND_ROBIN_TOKEN_POS)
#define ROUND_ROBIN_MODE                (ROUND_ROBIN_MODE_BITBAND << SYSCTRL_MEM_ARBITER_CFG_ROUND_ROBIN_TOKEN_POS)

/* Memory Timing Configuration */
#define SYSCTRL_MEM_TIMING_CFG_BASE     0x4000002C
#define SYSCTRL_MEM_TIMING_CFG          REG32_POINTER(SYSCTRL_MEM_TIMING_CFG_BASE)

/* SYSCTRL_MEM_TIMING_CFG bit positions */
#define SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_POS 8
#define SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_MASK (0x3 << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_POS)
#define SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_POS 4
#define SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_MASK (0x7 << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_POS)
#define SYSCTRL_MEM_TIMING_CFG_PROM_KEN_POS 3
#define SYSCTRL_MEM_TIMING_CFG_PROM_EMA_POS 0
#define SYSCTRL_MEM_TIMING_CFG_PROM_EMA_MASK (0x7 << SYSCTRL_MEM_TIMING_CFG_PROM_EMA_POS)

/* SYSCTRL_MEM_TIMING_CFG bit-band aliases */
#define SYSCTRL_MEM_TIMING_CFG_PROM_KEN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_MEM_TIMING_CFG_BASE, SYSCTRL_MEM_TIMING_CFG_PROM_KEN_POS))

/* SYSCTRL_MEM_TIMING_CFG settings */
#define DSP_PRAM_EMAW_DEFAULT           (0x0 << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_POS)
#define DSP_PRAM_EMAW_MAX               (0x3 << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMAW_POS)

#define DSP_PRAM_EMA_MIN                (0x0 << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_POS)
#define DSP_PRAM_EMA_DEFAULT            (0x2 << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_POS)
#define DSP_PRAM_EMA_MAX                (0x7 << SYSCTRL_MEM_TIMING_CFG_DSP_PRAM_EMA_POS)

#define PROM_KEN_ENABLED_BITBAND        0x0
#define PROM_KEN_DISABLED_BITBAND       0x1
#define PROM_KEN_ENABLED                (PROM_KEN_ENABLED_BITBAND << SYSCTRL_MEM_TIMING_CFG_PROM_KEN_POS)
#define PROM_KEN_DISABLED               (PROM_KEN_DISABLED_BITBAND << SYSCTRL_MEM_TIMING_CFG_PROM_KEN_POS)

#define PROM_EMA_MIN                    (0x0 << SYSCTRL_MEM_TIMING_CFG_PROM_EMA_POS)
#define PROM_EMA_DEFAULT                (0x5 << SYSCTRL_MEM_TIMING_CFG_PROM_EMA_POS)
#define PROM_EMA_MAX                    (0x7 << SYSCTRL_MEM_TIMING_CFG_PROM_EMA_POS)

/* Activity Counters Control */
#define SYSCTRL_CNT_CTRL_BASE           0x40000030
#define SYSCTRL_CNT_CTRL                REG32_POINTER(SYSCTRL_CNT_CTRL_BASE)

/* SYSCTRL_CNT_CTRL bit positions */
#define SYSCTRL_CNT_CTRL_CNT_STATUS_POS 3
#define SYSCTRL_CNT_CTRL_CNT_CLEAR_POS  2
#define SYSCTRL_CNT_CTRL_CNT_STOP_POS   1
#define SYSCTRL_CNT_CTRL_CNT_START_POS  0

/* SYSCTRL_CNT_CTRL bit-band aliases */
#define SYSCTRL_CNT_CTRL_CNT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_CNT_CTRL_BASE, SYSCTRL_CNT_CTRL_CNT_STATUS_POS))
#define SYSCTRL_CNT_CTRL_CNT_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_CNT_CTRL_BASE, SYSCTRL_CNT_CTRL_CNT_CLEAR_POS))
#define SYSCTRL_CNT_CTRL_CNT_STOP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_CNT_CTRL_BASE, SYSCTRL_CNT_CTRL_CNT_STOP_POS))
#define SYSCTRL_CNT_CTRL_CNT_START_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_CNT_CTRL_BASE, SYSCTRL_CNT_CTRL_CNT_START_POS))

/* SYSCTRL_CNT_CTRL settings */
#define CNT_STOPPED_BITBAND             0x0
#define CNT_RUNNING_BITBAND             0x1
#define CNT_STOPPED                     (CNT_STOPPED_BITBAND << SYSCTRL_CNT_CTRL_CNT_STATUS_POS)
#define CNT_RUNNING                     (CNT_RUNNING_BITBAND << SYSCTRL_CNT_CTRL_CNT_STATUS_POS)

#define CNT_CLEAR_BITBAND               0x1
#define CNT_CLEAR                       (CNT_CLEAR_BITBAND << SYSCTRL_CNT_CTRL_CNT_CLEAR_POS)

#define CNT_STOP_BITBAND                0x1
#define CNT_STOP                        (CNT_STOP_BITBAND << SYSCTRL_CNT_CTRL_CNT_STOP_POS)

#define CNT_START_BITBAND               0x1
#define CNT_START                       (CNT_START_BITBAND << SYSCTRL_CNT_CTRL_CNT_START_POS)

/* System Clock Counter Value */
#define SYSCTRL_SYSCLK_CNT_BASE         0x40000034
#define SYSCTRL_SYSCLK_CNT              REG32_POINTER(SYSCTRL_SYSCLK_CNT_BASE)

/* CM3 Activity Counter Value */
#define SYSCTRL_CM3_CNT_BASE            0x40000038
#define SYSCTRL_CM3_CNT                 REG32_POINTER(SYSCTRL_CM3_CNT_BASE)

/* LPDSP32 Activity Counter Value */
#define SYSCTRL_LPDSP32_CNT_BASE        0x4000003C
#define SYSCTRL_LPDSP32_CNT             REG32_POINTER(SYSCTRL_LPDSP32_CNT_BASE)

/* Flash Read Access Counter Value */
#define SYSCTRL_FLASH_READ_CNT_BASE     0x40000040
#define SYSCTRL_FLASH_READ_CNT          REG32_POINTER(SYSCTRL_FLASH_READ_CNT_BASE)

/* Critical Path Speed Measurement */
#define SYSCTRL_SPEED_MEASURE_BASE      0x40000048
#define SYSCTRL_SPEED_MEASURE           REG32_POINTER(SYSCTRL_SPEED_MEASURE_BASE)

/* SYSCTRL_SPEED_MEASURE bit positions */
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_START_POS 4
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_STATUS_POS 3
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_RESULT_POS 0
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_RESULT_MASK (0x7 << SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_RESULT_POS)

/* SYSCTRL_SPEED_MEASURE bit-band aliases */
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_START_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_SPEED_MEASURE_BASE, SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_START_POS))
#define SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_SPEED_MEASURE_BASE, SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_STATUS_POS))

/* SYSCTRL_SPEED_MEASURE settings */
#define SPEED_MEASURE_START_BITBAND     0x1
#define SPEED_MEASURE_START             (SPEED_MEASURE_START_BITBAND << SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_START_POS)

#define SPEED_MEASURE_IDLE_BITBAND      0x0
#define SPEED_MEASURE_BUSY_BITBAND      0x1
#define SPEED_MEASURE_IDLE              (SPEED_MEASURE_IDLE_BITBAND << SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_STATUS_POS)
#define SPEED_MEASURE_BUSY              (SPEED_MEASURE_BUSY_BITBAND << SYSCTRL_SPEED_MEASURE_SPEED_MEASURE_STATUS_POS)

/* LPDSP32 Debug Port Configuration */
#define SYSCTRL_LPDSP32_DEBUG_CFG_BASE  0x4000004C
#define SYSCTRL_LPDSP32_DEBUG_CFG       REG32_POINTER(SYSCTRL_LPDSP32_DEBUG_CFG_BASE)

/* SYSCTRL_LPDSP32_DEBUG_CFG bit positions */
#define SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_POS 1
#define SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_DEBUG_ENABLE_POS 0

/* SYSCTRL_LPDSP32_DEBUG_CFG bit-band aliases */
#define SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_LPDSP32_DEBUG_CFG_BASE, SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_POS))
#define SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_DEBUG_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_LPDSP32_DEBUG_CFG_BASE, SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_DEBUG_ENABLE_POS))

/* SYSCTRL_LPDSP32_DEBUG_CFG settings */
#define LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_DISABLED_BITBAND 0x0
#define LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_ENABLED_BITBAND 0x1
#define LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_DISABLED (LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_DISABLED_BITBAND << SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_POS)
#define LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_ENABLED (LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_ENABLED_BITBAND << SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_EXIT_POWERDOWN_WHEN_HALTED_POS)

#define LPDSP32_DEBUG_DISABLED_BITBAND  0x0
#define LPDSP32_DEBUG_ENABLED_BITBAND   0x1
#define LPDSP32_DEBUG_DISABLED          (LPDSP32_DEBUG_DISABLED_BITBAND << SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_DEBUG_ENABLE_POS)
#define LPDSP32_DEBUG_ENABLED           (LPDSP32_DEBUG_ENABLED_BITBAND << SYSCTRL_LPDSP32_DEBUG_CFG_LPDSP32_DEBUG_ENABLE_POS)

/* RF Power Configuration */
#define SYSCTRL_RF_POWER_CFG_BASE       0x40000050
#define SYSCTRL_RF_POWER_CFG            REG32_POINTER(SYSCTRL_RF_POWER_CFG_BASE)

/* SYSCTRL_RF_POWER_CFG bit positions */
#define SYSCTRL_RF_POWER_CFG_RF_POWER_POS 0

/* SYSCTRL_RF_POWER_CFG bit-band aliases */
#define SYSCTRL_RF_POWER_CFG_RF_POWER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_RF_POWER_CFG_BASE, SYSCTRL_RF_POWER_CFG_RF_POWER_POS))

/* SYSCTRL_RF_POWER_CFG settings */
#define RF_POWER_DISABLE_BITBAND        0x0
#define RF_POWER_ENABLE_BITBAND         0x1
#define RF_POWER_DISABLE                (RF_POWER_DISABLE_BITBAND << SYSCTRL_RF_POWER_CFG_RF_POWER_POS)
#define RF_POWER_ENABLE                 (RF_POWER_ENABLE_BITBAND << SYSCTRL_RF_POWER_CFG_RF_POWER_POS)

/* RF Access Configuration */
#define SYSCTRL_RF_ACCESS_CFG_BASE      0x40000054
#define SYSCTRL_RF_ACCESS_CFG           REG32_POINTER(SYSCTRL_RF_ACCESS_CFG_BASE)

/* SYSCTRL_RF_ACCESS_CFG bit positions */
#define SYSCTRL_RF_ACCESS_CFG_RF_IRQ_ACCESS_POS 1
#define SYSCTRL_RF_ACCESS_CFG_RF_ACCESS_POS 0

/* SYSCTRL_RF_ACCESS_CFG bit-band aliases */
#define SYSCTRL_RF_ACCESS_CFG_RF_IRQ_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_RF_ACCESS_CFG_BASE, SYSCTRL_RF_ACCESS_CFG_RF_IRQ_ACCESS_POS))
#define SYSCTRL_RF_ACCESS_CFG_RF_ACCESS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_RF_ACCESS_CFG_BASE, SYSCTRL_RF_ACCESS_CFG_RF_ACCESS_POS))

/* SYSCTRL_RF_ACCESS_CFG settings */
#define RF_IRQ_ACCESS_DISABLE_BITBAND   0x0
#define RF_IRQ_ACCESS_ENABLE_BITBAND    0x1
#define RF_IRQ_ACCESS_DISABLE           (RF_IRQ_ACCESS_DISABLE_BITBAND << SYSCTRL_RF_ACCESS_CFG_RF_IRQ_ACCESS_POS)
#define RF_IRQ_ACCESS_ENABLE            (RF_IRQ_ACCESS_ENABLE_BITBAND << SYSCTRL_RF_ACCESS_CFG_RF_IRQ_ACCESS_POS)

#define RF_ACCESS_DISABLE_BITBAND       0x0
#define RF_ACCESS_ENABLE_BITBAND        0x1
#define RF_ACCESS_DISABLE               (RF_ACCESS_DISABLE_BITBAND << SYSCTRL_RF_ACCESS_CFG_RF_ACCESS_POS)
#define RF_ACCESS_ENABLE                (RF_ACCESS_ENABLE_BITBAND << SYSCTRL_RF_ACCESS_CFG_RF_ACCESS_POS)

/* WAKEUP Pad Value */
#define SYSCTRL_WAKEUP_PAD_BASE         0x40000058
#define SYSCTRL_WAKEUP_PAD              READONLY_REG32_POINTER(SYSCTRL_WAKEUP_PAD_BASE)

/* SYSCTRL_WAKEUP_PAD bit positions */
#define SYSCTRL_WAKEUP_PAD_WAKEUP_PAD_VALUE_POS 0

/* SYSCTRL_WAKEUP_PAD bit-band aliases */
#define SYSCTRL_WAKEUP_PAD_WAKEUP_PAD_VALUE_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_WAKEUP_PAD_BASE, SYSCTRL_WAKEUP_PAD_WAKEUP_PAD_VALUE_POS))

/* SYSCTRL_WAKEUP_PAD settings */
#define WAKEUP_PAD_LOW_BITBAND          0x0
#define WAKEUP_PAD_HIGH_BITBAND         0x1
#define WAKEUP_PAD_LOW                  (WAKEUP_PAD_LOW_BITBAND << SYSCTRL_WAKEUP_PAD_WAKEUP_PAD_VALUE_POS)
#define WAKEUP_PAD_HIGH                 (WAKEUP_PAD_HIGH_BITBAND << SYSCTRL_WAKEUP_PAD_WAKEUP_PAD_VALUE_POS)

/* Debug Port Access Configuration */
#define SYSCTRL_DBG_LOCK_BASE           0x400000DC
#define SYSCTRL_DBG_LOCK                REG32_POINTER(SYSCTRL_DBG_LOCK_BASE)

/* SYSCTRL_DBG_LOCK bit positions */
#define SYSCTRL_DBG_LOCK_DBG_LOCK_WR_POS 0
#define SYSCTRL_DBG_LOCK_DBG_LOCK_WR_MASK (0xFFFFFFFF << SYSCTRL_DBG_LOCK_DBG_LOCK_WR_POS)
#define SYSCTRL_DBG_LOCK_DBG_LOCK_RD_POS 0

/* SYSCTRL_DBG_LOCK bit-band aliases */
#define SYSCTRL_DBG_LOCK_DBG_LOCK_RD_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SYSCTRL_DBG_LOCK_BASE, SYSCTRL_DBG_LOCK_DBG_LOCK_RD_POS))

/* SYSCTRL_DBG_LOCK settings */
#define DBG_ACCESS_UNLOCK               (0xB3909CB5 << SYSCTRL_DBG_LOCK_DBG_LOCK_WR_POS)
#define DBG_ACCESS_LOCK                 (0x4C6F634B << SYSCTRL_DBG_LOCK_DBG_LOCK_WR_POS)

#define DBG_ACCESS_UNLOCKED_BITBAND     0x0
#define DBG_ACCESS_LOCKED_BITBAND       0x1
#define DBG_ACCESS_UNLOCKED             (DBG_ACCESS_UNLOCKED_BITBAND << SYSCTRL_DBG_LOCK_DBG_LOCK_RD_POS)
#define DBG_ACCESS_LOCKED               (DBG_ACCESS_LOCKED_BITBAND << SYSCTRL_DBG_LOCK_DBG_LOCK_RD_POS)

/* Debug Port Lock Key Part 0 to 3 */
#define SYSCTRL_DBG_LOCK_KEY_BASE       0x400000E0
#define SYSCTRL_DBG_LOCK_KEY            REG32_POINTER(SYSCTRL_DBG_LOCK_KEY_BASE)

/* ----------------------------------------------------------------------------
 * Clock Generation
 * ------------------------------------------------------------------------- */

/* System Clock Configuration Register */
#define CLK_SYS_CFG_BASE                0x40000100
#define CLK_SYS_CFG                     REG32_POINTER(CLK_SYS_CFG_BASE)

/* CLK_SYS_CFG bit positions */
#define CLK_SYS_CFG_JTCK_PRESCALE_POS   16
#define CLK_SYS_CFG_JTCK_PRESCALE_MASK  (0xF << CLK_SYS_CFG_JTCK_PRESCALE_POS)
#define CLK_SYS_CFG_EXTCLK_PRESCALE_POS 8
#define CLK_SYS_CFG_EXTCLK_PRESCALE_MASK (0xF << CLK_SYS_CFG_EXTCLK_PRESCALE_POS)
#define CLK_SYS_CFG_SYSCLK_SRC_SEL_POS  0
#define CLK_SYS_CFG_SYSCLK_SRC_SEL_MASK (0x7 << CLK_SYS_CFG_SYSCLK_SRC_SEL_POS)

/* CLK_SYS_CFG settings */
#define JTCK_PRESCALE_1                 (0x0 << CLK_SYS_CFG_JTCK_PRESCALE_POS)
#define JTCK_PRESCALE_2                 (0x1 << CLK_SYS_CFG_JTCK_PRESCALE_POS)
#define JTCK_PRESCALE_15                (0xE << CLK_SYS_CFG_JTCK_PRESCALE_POS)
#define JTCK_PRESCALE_16                (0xF << CLK_SYS_CFG_JTCK_PRESCALE_POS)

#define EXTCLK_PRESCALE_1               (0x0 << CLK_SYS_CFG_EXTCLK_PRESCALE_POS)
#define EXTCLK_PRESCALE_2               (0x1 << CLK_SYS_CFG_EXTCLK_PRESCALE_POS)
#define EXTCLK_PRESCALE_15              (0xE << CLK_SYS_CFG_EXTCLK_PRESCALE_POS)
#define EXTCLK_PRESCALE_16              (0xF << CLK_SYS_CFG_EXTCLK_PRESCALE_POS)

#define SYSCLK_CLKSRC_RCCLK             (0x0 << CLK_SYS_CFG_SYSCLK_SRC_SEL_POS)
#define SYSCLK_CLKSRC_STANDBYCLK        (0x1 << CLK_SYS_CFG_SYSCLK_SRC_SEL_POS)
#define SYSCLK_CLKSRC_RFCLK             (0x2 << CLK_SYS_CFG_SYSCLK_SRC_SEL_POS)
#define SYSCLK_CLKSRC_EXTCLK            (0x3 << CLK_SYS_CFG_SYSCLK_SRC_SEL_POS)
#define SYSCLK_CLKSRC_JTCK              (0x4 << CLK_SYS_CFG_SYSCLK_SRC_SEL_POS)

/* Prescale register for SLOWCLK, BBCLK and USRCLK clocks */
#define CLK_DIV_CFG0_BASE               0x40000104
#define CLK_DIV_CFG0                    REG32_POINTER(CLK_DIV_CFG0_BASE)

/* CLK_DIV_CFG0 bit positions */
#define CLK_DIV_CFG0_USRCLK_PRESCALE_POS 16
#define CLK_DIV_CFG0_USRCLK_PRESCALE_MASK (0xFFF << CLK_DIV_CFG0_USRCLK_PRESCALE_POS)
#define CLK_DIV_CFG0_BBCLK_PRESCALE_POS 8
#define CLK_DIV_CFG0_BBCLK_PRESCALE_MASK (0x7 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)
#define CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS 0
#define CLK_DIV_CFG0_SLOWCLK_PRESCALE_MASK (0x3F << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)

/* CLK_DIV_CFG0 settings */
#define USRCLK_PRESCALE_1               (0x0 << CLK_DIV_CFG0_USRCLK_PRESCALE_POS)
#define USRCLK_PRESCALE_2               (0x1 << CLK_DIV_CFG0_USRCLK_PRESCALE_POS)
#define USRCLK_PRESCALE_3               (0x2 << CLK_DIV_CFG0_USRCLK_PRESCALE_POS)
#define USRCLK_PRESCALE_4095            (0xFFE << CLK_DIV_CFG0_USRCLK_PRESCALE_POS)
#define USRCLK_PRESCALE_4096            (0xFFF << CLK_DIV_CFG0_USRCLK_PRESCALE_POS)

#define BBCLK_PRESCALE_1                (0x0 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)
#define BBCLK_PRESCALE_2                (0x1 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)
#define BBCLK_PRESCALE_3                (0x2 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)
#define BBCLK_PRESCALE_4                (0x3 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)
#define BBCLK_PRESCALE_5                (0x4 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)
#define BBCLK_PRESCALE_6                (0x5 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)
#define BBCLK_PRESCALE_7                (0x6 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)
#define BBCLK_PRESCALE_8                (0x7 << CLK_DIV_CFG0_BBCLK_PRESCALE_POS)

#define SLOWCLK_PRESCALE_1              (0x0 << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_2              (0x1 << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_3              (0x2 << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_4              (0x3 << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_8              (0x7 << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_10             (0x9 << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_12             (0xB << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_16             (0xF << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_24             (0x17 << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_48             (0x2F << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_63             (0x3E << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)
#define SLOWCLK_PRESCALE_64             (0x3F << CLK_DIV_CFG0_SLOWCLK_PRESCALE_POS)

/* Prescale register for PWM clock, UART and DMIC clocks */
#define CLK_DIV_CFG1_BASE               0x40000108
#define CLK_DIV_CFG1                    REG32_POINTER(CLK_DIV_CFG1_BASE)

/* CLK_DIV_CFG1 bit positions */
#define CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_POS 30
#define CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_MASK (0x3 << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_POS)
#define CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS 24
#define CLK_DIV_CFG1_AUDIOCLK_PRESCALE_MASK (0x3F << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define CLK_DIV_CFG1_UARTCLK_PRESCALE_POS 16
#define CLK_DIV_CFG1_UARTCLK_PRESCALE_MASK (0x1F << CLK_DIV_CFG1_UARTCLK_PRESCALE_POS)
#define CLK_DIV_CFG1_PWM1CLK_PRESCALE_POS 8
#define CLK_DIV_CFG1_PWM1CLK_PRESCALE_MASK (0x3F << CLK_DIV_CFG1_PWM1CLK_PRESCALE_POS)
#define CLK_DIV_CFG1_PWM0CLK_PRESCALE_POS 0
#define CLK_DIV_CFG1_PWM0CLK_PRESCALE_MASK (0x3F << CLK_DIV_CFG1_PWM0CLK_PRESCALE_POS)

/* CLK_DIV_CFG1 settings */
#define AUDIOSLOWCLK_PRESCALE_1         (0x0 << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_POS)
#define AUDIOSLOWCLK_PRESCALE_2         (0x1 << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_POS)
#define AUDIOSLOWCLK_PRESCALE_3         (0x2 << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_POS)
#define AUDIOSLOWCLK_PRESCALE_4         (0x3 << CLK_DIV_CFG1_AUDIOSLOWCLK_PRESCALE_POS)

#define AUDIOCLK_PRESCALE_1             (0x0 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_2             (0x1 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_3             (0x2 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_4             (0x3 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_5             (0x4 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_6             (0x5 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_7             (0x6 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_8             (0x7 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_9             (0x8 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_10            (0x9 << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_11            (0xA << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_12            (0xB << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_13            (0xC << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_14            (0xD << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_15            (0xE << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_16            (0xF << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_63            (0x3E << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)
#define AUDIOCLK_PRESCALE_64            (0x3F << CLK_DIV_CFG1_AUDIOCLK_PRESCALE_POS)

#define UARTCLK_PRESCALE_1              (0x0 << CLK_DIV_CFG1_UARTCLK_PRESCALE_POS)
#define UARTCLK_PRESCALE_2              (0x1 << CLK_DIV_CFG1_UARTCLK_PRESCALE_POS)
#define UARTCLK_PRESCALE_31             (0x1E << CLK_DIV_CFG1_UARTCLK_PRESCALE_POS)
#define UARTCLK_PRESCALE_32             (0x1F << CLK_DIV_CFG1_UARTCLK_PRESCALE_POS)

#define PWM1CLK_PRESCALE_1              (0x0 << CLK_DIV_CFG1_PWM1CLK_PRESCALE_POS)
#define PWM1CLK_PRESCALE_2              (0x1 << CLK_DIV_CFG1_PWM1CLK_PRESCALE_POS)
#define PWM1CLK_PRESCALE_63             (0x3E << CLK_DIV_CFG1_PWM1CLK_PRESCALE_POS)
#define PWM1CLK_PRESCALE_64             (0x3F << CLK_DIV_CFG1_PWM1CLK_PRESCALE_POS)

#define PWM0CLK_PRESCALE_1              (0x0 << CLK_DIV_CFG1_PWM0CLK_PRESCALE_POS)
#define PWM0CLK_PRESCALE_2              (0x1 << CLK_DIV_CFG1_PWM0CLK_PRESCALE_POS)
#define PWM0CLK_PRESCALE_63             (0x3E << CLK_DIV_CFG1_PWM0CLK_PRESCALE_POS)
#define PWM0CLK_PRESCALE_64             (0x3F << CLK_DIV_CFG1_PWM0CLK_PRESCALE_POS)

/* Prescale register for DC-DC converter and charge pump clocks */
#define CLK_DIV_CFG2_BASE               0x4000010C
#define CLK_DIV_CFG2                    REG32_POINTER(CLK_DIV_CFG2_BASE)

/* CLK_DIV_CFG2 bit positions */
#define CLK_DIV_CFG2_CPCLK_DISABLE_POS  15
#define CLK_DIV_CFG2_CPCLK_PRESCALE_POS 8
#define CLK_DIV_CFG2_CPCLK_PRESCALE_MASK (0x3F << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CLK_DIV_CFG2_DCCLK_DISABLE_POS  7
#define CLK_DIV_CFG2_DCCLK_PRESCALE_POS 0
#define CLK_DIV_CFG2_DCCLK_PRESCALE_MASK (0x3F << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)

/* CLK_DIV_CFG2 bit-band aliases */
#define CLK_DIV_CFG2_CPCLK_DISABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(CLK_DIV_CFG2_BASE, CLK_DIV_CFG2_CPCLK_DISABLE_POS))
#define CLK_DIV_CFG2_DCCLK_DISABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(CLK_DIV_CFG2_BASE, CLK_DIV_CFG2_DCCLK_DISABLE_POS))

/* CLK_DIV_CFG2 settings */
#define CPCLK_ENABLE_BITBAND            0x0
#define CPCLK_DISABLE_BITBAND           0x1
#define CPCLK_ENABLE                    (CPCLK_ENABLE_BITBAND << CLK_DIV_CFG2_CPCLK_DISABLE_POS)
#define CPCLK_DISABLE                   (CPCLK_DISABLE_BITBAND << CLK_DIV_CFG2_CPCLK_DISABLE_POS)

#define CPCLK_PRESCALE_1                (0x0 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_2                (0x1 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_3                (0x2 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_4                (0x3 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_5                (0x4 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_6                (0x5 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_7                (0x6 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_8                (0x7 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_9                (0x8 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_10               (0x9 << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_63               (0x3E << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)
#define CPCLK_PRESCALE_64               (0x3F << CLK_DIV_CFG2_CPCLK_PRESCALE_POS)

#define DCCLK_ENABLE_BITBAND            0x0
#define DCCLK_DISABLE_BITBAND           0x1
#define DCCLK_ENABLE                    (DCCLK_ENABLE_BITBAND << CLK_DIV_CFG2_DCCLK_DISABLE_POS)
#define DCCLK_DISABLE                   (DCCLK_DISABLE_BITBAND << CLK_DIV_CFG2_DCCLK_DISABLE_POS)

#define DCCLK_PRESCALE_1                (0x0 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_2                (0x1 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_3                (0x2 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_4                (0x3 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_5                (0x4 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_6                (0x5 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_7                (0x6 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_8                (0x7 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_9                (0x8 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_10               (0x9 << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_12               (0xB << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_63               (0x3E << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)
#define DCCLK_PRESCALE_64               (0x3F << CLK_DIV_CFG2_DCCLK_PRESCALE_POS)

/* External clock detector configuration register (including interrupt) */
#define CLK_DET_CFG_BASE                0x40000110
#define CLK_DET_CFG                     REG32_POINTER(CLK_DET_CFG_BASE)

/* CLK_DET_CFG bit positions */
#define CLK_DET_CFG_CLK_DET_SEL_POS     5
#define CLK_DET_CFG_CLK_DET_INT_SEL_POS 3
#define CLK_DET_CFG_CLK_DET_INT_SEL_MASK (0x3 << CLK_DET_CFG_CLK_DET_INT_SEL_POS)
#define CLK_DET_CFG_CLK_DET_DIV_POS     1
#define CLK_DET_CFG_CLK_DET_DIV_MASK    (0x3 << CLK_DET_CFG_CLK_DET_DIV_POS)
#define CLK_DET_CFG_CLK_DET_ENABLE_POS  0

/* CLK_DET_CFG bit-band aliases */
#define CLK_DET_CFG_CLK_DET_SEL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(CLK_DET_CFG_BASE, CLK_DET_CFG_CLK_DET_SEL_POS))
#define CLK_DET_CFG_CLK_DET_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(CLK_DET_CFG_BASE, CLK_DET_CFG_CLK_DET_ENABLE_POS))

/* CLK_DET_CFG settings */
#define CLK_DET_SEL_EXTCLK_BITBAND      0x0
#define CLK_DET_SEL_JTCK_BITBAND        0x1
#define CLK_DET_SEL_EXTCLK              (CLK_DET_SEL_EXTCLK_BITBAND << CLK_DET_CFG_CLK_DET_SEL_POS)
#define CLK_DET_SEL_JTCK                (CLK_DET_SEL_JTCK_BITBAND << CLK_DET_CFG_CLK_DET_SEL_POS)

#define CLK_DET_INT_DISABLE             (0x0 << CLK_DET_CFG_CLK_DET_INT_SEL_POS)
#define CLK_DET_INT_ACTIVATED           (0x1 << CLK_DET_CFG_CLK_DET_INT_SEL_POS)
#define CLK_DET_INT_DEACTIVATED         (0x2 << CLK_DET_CFG_CLK_DET_INT_SEL_POS)
#define CLK_DET_INT_ACTIVITY_CHANGE     (0x3 << CLK_DET_CFG_CLK_DET_INT_SEL_POS)

#define CLK_DET_SLOWCLK_DIV32           (0x0 << CLK_DET_CFG_CLK_DET_DIV_POS)
#define CLK_DET_SLOWCLK_DIV64           (0x1 << CLK_DET_CFG_CLK_DET_DIV_POS)
#define CLK_DET_SLOWCLK_DIV96           (0x2 << CLK_DET_CFG_CLK_DET_DIV_POS)
#define CLK_DET_SLOWCLK_DIV128          (0x3 << CLK_DET_CFG_CLK_DET_DIV_POS)

#define CLK_DET_DISABLE_BITBAND         0x0
#define CLK_DET_ENABLE_BITBAND          0x1
#define CLK_DET_DISABLE                 (CLK_DET_DISABLE_BITBAND << CLK_DET_CFG_CLK_DET_ENABLE_POS)
#define CLK_DET_ENABLE                  (CLK_DET_ENABLE_BITBAND << CLK_DET_CFG_CLK_DET_ENABLE_POS)

/* External clock detector status register */
#define CLK_DET_STATUS_BASE             0x40000114
#define CLK_DET_STATUS                  READONLY_REG32_POINTER(CLK_DET_STATUS_BASE)

/* CLK_DET_STATUS bit positions */
#define CLK_DET_STATUS_CLK_DET_INT_STATUS_POS 1
#define CLK_DET_STATUS_CLK_DET_STATUS_POS 0

/* CLK_DET_STATUS bit-band aliases */
#define CLK_DET_STATUS_CLK_DET_INT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(CLK_DET_STATUS_BASE, CLK_DET_STATUS_CLK_DET_INT_STATUS_POS))
#define CLK_DET_STATUS_CLK_DET_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(CLK_DET_STATUS_BASE, CLK_DET_STATUS_CLK_DET_STATUS_POS))

/* CLK_DET_STATUS settings */
#define CLK_DET_INT_FALSE_BITBAND       0x0
#define CLK_DET_INT_TRUE_BITBAND        0x1
#define CLK_DET_INT_FALSE               (CLK_DET_INT_FALSE_BITBAND << CLK_DET_STATUS_CLK_DET_INT_STATUS_POS)
#define CLK_DET_INT_TRUE                (CLK_DET_INT_TRUE_BITBAND << CLK_DET_STATUS_CLK_DET_INT_STATUS_POS)

#define CLK_NOT_ACTIVE_BITBAND          0x0
#define CLK_ACTIVE_BITBAND              0x1
#define CLK_NOT_ACTIVE                  (CLK_NOT_ACTIVE_BITBAND << CLK_DET_STATUS_CLK_DET_STATUS_POS)
#define CLK_ACTIVE                      (CLK_ACTIVE_BITBAND << CLK_DET_STATUS_CLK_DET_STATUS_POS)

/* ----------------------------------------------------------------------------
 * Reset
 * ------------------------------------------------------------------------- */

/* Reset status register */
#define DIG_RESET_STATUS_BASE           0x40000200
#define DIG_RESET_STATUS                REG32_POINTER(DIG_RESET_STATUS_BASE)

/* DIG_RESET_STATUS bit positions */
#define DIG_RESET_STATUS_LOCKUP_RESET_FLAG_CLEAR_POS 7
#define DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_CLEAR_POS 6
#define DIG_RESET_STATUS_CM3_SW_RESET_FLAG_CLEAR_POS 5
#define DIG_RESET_STATUS_ACS_RESET_FLAG_CLEAR_POS 4
#define DIG_RESET_STATUS_LOCKUP_FLAG_POS 3
#define DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_POS 2
#define DIG_RESET_STATUS_CM3_SW_RESET_FLAG_POS 1
#define DIG_RESET_STATUS_ACS_RESET_FLAG_POS 0

/* DIG_RESET_STATUS bit-band aliases */
#define DIG_RESET_STATUS_LOCKUP_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIG_RESET_STATUS_BASE, DIG_RESET_STATUS_LOCKUP_RESET_FLAG_CLEAR_POS))
#define DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIG_RESET_STATUS_BASE, DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_CLEAR_POS))
#define DIG_RESET_STATUS_CM3_SW_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIG_RESET_STATUS_BASE, DIG_RESET_STATUS_CM3_SW_RESET_FLAG_CLEAR_POS))
#define DIG_RESET_STATUS_ACS_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIG_RESET_STATUS_BASE, DIG_RESET_STATUS_ACS_RESET_FLAG_CLEAR_POS))
#define DIG_RESET_STATUS_LOCKUP_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DIG_RESET_STATUS_BASE, DIG_RESET_STATUS_LOCKUP_FLAG_POS))
#define DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DIG_RESET_STATUS_BASE, DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_POS))
#define DIG_RESET_STATUS_CM3_SW_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DIG_RESET_STATUS_BASE, DIG_RESET_STATUS_CM3_SW_RESET_FLAG_POS))
#define DIG_RESET_STATUS_ACS_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DIG_RESET_STATUS_BASE, DIG_RESET_STATUS_ACS_RESET_FLAG_POS))

/* DIG_RESET_STATUS settings */
#define LOCKUP_FLAG_CLEAR_BITBAND       0x1
#define LOCKUP_FLAG_CLEAR               (LOCKUP_FLAG_CLEAR_BITBAND << DIG_RESET_STATUS_LOCKUP_RESET_FLAG_CLEAR_POS)

#define WATCHDOG_RESET_FLAG_CLEAR_BITBAND 0x1
#define WATCHDOG_RESET_FLAG_CLEAR       (WATCHDOG_RESET_FLAG_CLEAR_BITBAND << DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_CLEAR_POS)

#define CM3_SW_RESET_FLAG_CLEAR_BITBAND 0x1
#define CM3_SW_RESET_FLAG_CLEAR         (CM3_SW_RESET_FLAG_CLEAR_BITBAND << DIG_RESET_STATUS_CM3_SW_RESET_FLAG_CLEAR_POS)

#define ACS_RESET_FLAG_CLEAR_BITBAND    0x1
#define ACS_RESET_FLAG_CLEAR            (ACS_RESET_FLAG_CLEAR_BITBAND << DIG_RESET_STATUS_ACS_RESET_FLAG_CLEAR_POS)

#define LOCKUP_NOT_SET_BITBAND          0x0
#define LOCKUP_SET_BITBAND              0x1
#define LOCKUP_NOT_SET                  (LOCKUP_NOT_SET_BITBAND << DIG_RESET_STATUS_LOCKUP_FLAG_POS)
#define LOCKUP_SET                      (LOCKUP_SET_BITBAND << DIG_RESET_STATUS_LOCKUP_FLAG_POS)

#define WATCHDOG_RESET_NOT_SET_BITBAND  0x0
#define WATCHDOG_RESET_SET_BITBAND      0x1
#define WATCHDOG_RESET_NOT_SET          (WATCHDOG_RESET_NOT_SET_BITBAND << DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_POS)
#define WATCHDOG_RESET_SET              (WATCHDOG_RESET_SET_BITBAND << DIG_RESET_STATUS_WATCHDOG_RESET_FLAG_POS)

#define CM3_SW_RESET_NOT_SET_BITBAND    0x0
#define CM3_SW_RESET_SET_BITBAND        0x1
#define CM3_SW_RESET_NOT_SET            (CM3_SW_RESET_NOT_SET_BITBAND << DIG_RESET_STATUS_CM3_SW_RESET_FLAG_POS)
#define CM3_SW_RESET_SET                (CM3_SW_RESET_SET_BITBAND << DIG_RESET_STATUS_CM3_SW_RESET_FLAG_POS)

#define ACS_RESET_NOT_SET_BITBAND       0x0
#define ACS_RESET_SET_BITBAND           0x1
#define ACS_RESET_NOT_SET               (ACS_RESET_NOT_SET_BITBAND << DIG_RESET_STATUS_ACS_RESET_FLAG_POS)
#define ACS_RESET_SET                   (ACS_RESET_SET_BITBAND << DIG_RESET_STATUS_ACS_RESET_FLAG_POS)

/* ----------------------------------------------------------------------------
 * Watchdog Timer
 * ------------------------------------------------------------------------- */

/* Watchdog timer Configuration Register */
#define WATCHDOG_CFG_BASE               0x40000300
#define WATCHDOG_CFG                    REG32_POINTER(WATCHDOG_CFG_BASE)

/* WATCHDOG_CFG bit positions */
#define WATCHDOG_CFG_TIMEOUT_VALUE_POS  0
#define WATCHDOG_CFG_TIMEOUT_VALUE_MASK (0xF << WATCHDOG_CFG_TIMEOUT_VALUE_POS)

/* WATCHDOG_CFG settings */
#define WATCHDOG_TIMEOUT_2M048          (0x0 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_4M096          (0x1 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_8M2            (0x2 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_16M4           (0x3 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_32M8           (0x4 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_65M5           (0x5 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_131M1          (0x6 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_262M1          (0x7 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_524M3          (0x8 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_1048M6         (0x9 << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_2097M1         (0xA << WATCHDOG_CFG_TIMEOUT_VALUE_POS)
#define WATCHDOG_TIMEOUT_4194M3         (0xB << WATCHDOG_CFG_TIMEOUT_VALUE_POS)

/* Watchdog Refresh Control Register */
#define WATCHDOG_CTRL_BASE              0x40000304
#define WATCHDOG_CTRL                   REG32_POINTER(WATCHDOG_CTRL_BASE)

/* ----------------------------------------------------------------------------
 * General-Purpose Timers 0, 1, 2 and 3
 * ------------------------------------------------------------------------- */

/* Timer Configuration Registers */
#define TIMER_CFG_BASE                  0x40000400
#define TIMER_CFG                       REG32_POINTER(TIMER_CFG_BASE)

/* TIMER_CFG bit positions */
#define TIMER_CFG_MULTI_COUNT_POS       29
#define TIMER_CFG_MULTI_COUNT_MASK      (0x7 << TIMER_CFG_MULTI_COUNT_POS)
#define TIMER_CFG_MODE_POS              28
#define TIMER_CFG_CLK_SRC_POS           27
#define TIMER_CFG_PRESCALE_POS          24
#define TIMER_CFG_PRESCALE_MASK         (0x7 << TIMER_CFG_PRESCALE_POS)
#define TIMER_CFG_TIMEOUT_VALUE_POS     0
#define TIMER_CFG_TIMEOUT_VALUE_MASK    (0xFFFFFF << TIMER_CFG_TIMEOUT_VALUE_POS)

/* TIMER_CFG bit-band aliases */
#define TIMER_CFG_MODE_BITBAND          REG32_POINTER(SYS_CALC_BITBAND(TIMER_CFG_BASE, TIMER_CFG_MODE_POS))
#define TIMER_CFG_CLK_SRC_BITBAND       REG32_POINTER(SYS_CALC_BITBAND(TIMER_CFG_BASE, TIMER_CFG_CLK_SRC_POS))

/* TIMER_CFG settings */
#define TIMER_MULTI_COUNT_1             (0x0 << TIMER_CFG_MULTI_COUNT_POS)
#define TIMER_MULTI_COUNT_2             (0x1 << TIMER_CFG_MULTI_COUNT_POS)
#define TIMER_MULTI_COUNT_3             (0x2 << TIMER_CFG_MULTI_COUNT_POS)
#define TIMER_MULTI_COUNT_4             (0x3 << TIMER_CFG_MULTI_COUNT_POS)
#define TIMER_MULTI_COUNT_5             (0x4 << TIMER_CFG_MULTI_COUNT_POS)
#define TIMER_MULTI_COUNT_6             (0x5 << TIMER_CFG_MULTI_COUNT_POS)
#define TIMER_MULTI_COUNT_7             (0x6 << TIMER_CFG_MULTI_COUNT_POS)
#define TIMER_MULTI_COUNT_8             (0x7 << TIMER_CFG_MULTI_COUNT_POS)

#define TIMER_SHOT_MODE_BITBAND         0x0
#define TIMER_FREE_RUN_BITBAND          0x1
#define TIMER_SHOT_MODE                 (TIMER_SHOT_MODE_BITBAND << TIMER_CFG_MODE_POS)
#define TIMER_FREE_RUN                  (TIMER_FREE_RUN_BITBAND << TIMER_CFG_MODE_POS)

#define TIMER_SLOWCLK_DIV32_BITBAND     0x0
#define TIMER_SLOWCLK_DIV2_BITBAND      0x1
#define TIMER_SLOWCLK_DIV32             (TIMER_SLOWCLK_DIV32_BITBAND << TIMER_CFG_CLK_SRC_POS)
#define TIMER_SLOWCLK_DIV2              (TIMER_SLOWCLK_DIV2_BITBAND << TIMER_CFG_CLK_SRC_POS)

#define TIMER_PRESCALE_1                (0x0 << TIMER_CFG_PRESCALE_POS)
#define TIMER_PRESCALE_2                (0x1 << TIMER_CFG_PRESCALE_POS)
#define TIMER_PRESCALE_4                (0x2 << TIMER_CFG_PRESCALE_POS)
#define TIMER_PRESCALE_8                (0x3 << TIMER_CFG_PRESCALE_POS)
#define TIMER_PRESCALE_16               (0x4 << TIMER_CFG_PRESCALE_POS)
#define TIMER_PRESCALE_32               (0x5 << TIMER_CFG_PRESCALE_POS)
#define TIMER_PRESCALE_64               (0x6 << TIMER_CFG_PRESCALE_POS)
#define TIMER_PRESCALE_128              (0x7 << TIMER_CFG_PRESCALE_POS)

/* General-Purpose timer Control / Status Register */
#define TIMER_CTRL_BASE                 0x40000410
#define TIMER_CTRL                      REG32_POINTER(TIMER_CTRL_BASE)

/* TIMER_CTRL bit positions */
#define TIMER_CTRL_TIMER_STATUS_POS     2
#define TIMER_CTRL_TIMER_START_POS      1
#define TIMER_CTRL_TIMER_STOP_POS       0

/* TIMER_CTRL bit-band aliases */
#define TIMER_CTRL_TIMER_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(TIMER_CTRL_BASE, TIMER_CTRL_TIMER_STATUS_POS))
#define TIMER_CTRL_TIMER_START_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(TIMER_CTRL_BASE, TIMER_CTRL_TIMER_START_POS))
#define TIMER_CTRL_TIMER_STOP_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(TIMER_CTRL_BASE, TIMER_CTRL_TIMER_STOP_POS))

/* TIMER_CTRL settings */
#define TIMER_INACTIVE_BITBAND          0x0
#define TIMER_ACTIVE_BITBAND            0x1
#define TIMER_INACTIVE                  (TIMER_INACTIVE_BITBAND << TIMER_CTRL_TIMER_STATUS_POS)
#define TIMER_ACTIVE                    (TIMER_ACTIVE_BITBAND << TIMER_CTRL_TIMER_STATUS_POS)

#define TIMER_START_BITBAND             0x1
#define TIMER_START                     (TIMER_START_BITBAND << TIMER_CTRL_TIMER_START_POS)

#define TIMER_STOP_BITBAND              0x1
#define TIMER_STOP                      (TIMER_STOP_BITBAND << TIMER_CTRL_TIMER_STOP_POS)

/* Timer Current Value Register */
#define TIMER_VAL_BASE                  0x40000420
#define TIMER_VAL                       READONLY_REG32_POINTER(TIMER_VAL_BASE)

/* TIMER_VAL bit positions */
#define TIMER_VAL_TIMER_VALUE_POS       0
#define TIMER_VAL_TIMER_VALUE_MASK      (0xFFFFFF << TIMER_VAL_TIMER_VALUE_POS)

/* ----------------------------------------------------------------------------
 * Flash Interface Configuration and Control
 * ------------------------------------------------------------------------- */

/* Flash Interface Control Register */
#define FLASH_IF_CTRL_BASE              0x40000500
#define FLASH_IF_CTRL                   REG32_POINTER(FLASH_IF_CTRL_BASE)

/* FLASH_IF_CTRL bit positions */
#define FLASH_IF_CTRL_PREFETCH_DBUS_POS 18
#define FLASH_IF_CTRL_PREFETCH_IBUS_POS 17
#define FLASH_IF_CTRL_NOT_LOAD_AUTO_POS 16
#define FLASH_IF_CTRL_VREAD1_MODE_POS   12
#define FLASH_IF_CTRL_VREAD0_MODE_POS   11
#define FLASH_IF_CTRL_RECALL_POS        10
#define FLASH_IF_CTRL_RETRY_POS         8
#define FLASH_IF_CTRL_RETRY_MASK        (0x3 << FLASH_IF_CTRL_RETRY_POS)
#define FLASH_IF_CTRL_LP_MODE_POS       0

/* FLASH_IF_CTRL bit-band aliases */
#define FLASH_IF_CTRL_PREFETCH_DBUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_CTRL_BASE, FLASH_IF_CTRL_PREFETCH_DBUS_POS))
#define FLASH_IF_CTRL_PREFETCH_IBUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_CTRL_BASE, FLASH_IF_CTRL_PREFETCH_IBUS_POS))
#define FLASH_IF_CTRL_NOT_LOAD_AUTO_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_CTRL_BASE, FLASH_IF_CTRL_NOT_LOAD_AUTO_POS))
#define FLASH_IF_CTRL_VREAD1_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_CTRL_BASE, FLASH_IF_CTRL_VREAD1_MODE_POS))
#define FLASH_IF_CTRL_VREAD0_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_CTRL_BASE, FLASH_IF_CTRL_VREAD0_MODE_POS))
#define FLASH_IF_CTRL_RECALL_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_CTRL_BASE, FLASH_IF_CTRL_RECALL_POS))
#define FLASH_IF_CTRL_LP_MODE_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_CTRL_BASE, FLASH_IF_CTRL_LP_MODE_POS))

/* FLASH_IF_CTRL settings */
#define FLASH_PREFETCH_DBUS_DISABLE_BITBAND 0x0
#define FLASH_PREFETCH_DBUS_ENABLE_BITBAND 0x1
#define FLASH_PREFETCH_DBUS_DISABLE     (FLASH_PREFETCH_DBUS_DISABLE_BITBAND << FLASH_IF_CTRL_PREFETCH_DBUS_POS)
#define FLASH_PREFETCH_DBUS_ENABLE      (FLASH_PREFETCH_DBUS_ENABLE_BITBAND << FLASH_IF_CTRL_PREFETCH_DBUS_POS)

#define FLASH_PREFETCH_IBUS_DISABLE_BITBAND 0x0
#define FLASH_PREFETCH_IBUS_ENABLE_BITBAND 0x1
#define FLASH_PREFETCH_IBUS_DISABLE     (FLASH_PREFETCH_IBUS_DISABLE_BITBAND << FLASH_IF_CTRL_PREFETCH_IBUS_POS)
#define FLASH_PREFETCH_IBUS_ENABLE      (FLASH_PREFETCH_IBUS_ENABLE_BITBAND << FLASH_IF_CTRL_PREFETCH_IBUS_POS)

#define FLASH_LOAD_AUTO_ENABLE_BITBAND  0x0
#define FLASH_LOAD_AUTO_DISABLE_BITBAND 0x1
#define FLASH_LOAD_AUTO_ENABLE          (FLASH_LOAD_AUTO_ENABLE_BITBAND << FLASH_IF_CTRL_NOT_LOAD_AUTO_POS)
#define FLASH_LOAD_AUTO_DISABLE         (FLASH_LOAD_AUTO_DISABLE_BITBAND << FLASH_IF_CTRL_NOT_LOAD_AUTO_POS)

#define FLASH_VREAD1_DISABLE_BITBAND    0x0
#define FLASH_VREAD1_ENABLE_BITBAND     0x1
#define FLASH_VREAD1_DISABLE            (FLASH_VREAD1_DISABLE_BITBAND << FLASH_IF_CTRL_VREAD1_MODE_POS)
#define FLASH_VREAD1_ENABLE             (FLASH_VREAD1_ENABLE_BITBAND << FLASH_IF_CTRL_VREAD1_MODE_POS)

#define FLASH_VREAD0_DISABLE_BITBAND    0x0
#define FLASH_VREAD0_ENABLE_BITBAND     0x1
#define FLASH_VREAD0_DISABLE            (FLASH_VREAD0_DISABLE_BITBAND << FLASH_IF_CTRL_VREAD0_MODE_POS)
#define FLASH_VREAD0_ENABLE             (FLASH_VREAD0_ENABLE_BITBAND << FLASH_IF_CTRL_VREAD0_MODE_POS)

#define FLASH_RECALL_DISABLE_BITBAND    0x0
#define FLASH_RECALL_ENABLE_BITBAND     0x1
#define FLASH_RECALL_DISABLE            (FLASH_RECALL_DISABLE_BITBAND << FLASH_IF_CTRL_RECALL_POS)
#define FLASH_RECALL_ENABLE             (FLASH_RECALL_ENABLE_BITBAND << FLASH_IF_CTRL_RECALL_POS)

#define FLASH_RETRY_1                   (0x0 << FLASH_IF_CTRL_RETRY_POS)
#define FLASH_RETRY_2                   (0x1 << FLASH_IF_CTRL_RETRY_POS)
#define FLASH_RETRY_3                   (0x2 << FLASH_IF_CTRL_RETRY_POS)
#define FLASH_RETRY_4                   (0x3 << FLASH_IF_CTRL_RETRY_POS)

#define FLASH_LOW_POWER_DISABLE_BITBAND 0x0
#define FLASH_LOW_POWER_ENABLE_BITBAND  0x1
#define FLASH_LOW_POWER_DISABLE         (FLASH_LOW_POWER_DISABLE_BITBAND << FLASH_IF_CTRL_LP_MODE_POS)
#define FLASH_LOW_POWER_ENABLE          (FLASH_LOW_POWER_ENABLE_BITBAND << FLASH_IF_CTRL_LP_MODE_POS)

/* Flash Main Write Unlock Register */
#define FLASH_MAIN_WRITE_UNLOCK_BASE    0x40000504
#define FLASH_MAIN_WRITE_UNLOCK         REG32_POINTER(FLASH_MAIN_WRITE_UNLOCK_BASE)

/* Flash Main Write Control Register */
#define FLASH_MAIN_CTRL_BASE            0x40000508
#define FLASH_MAIN_CTRL                 REG32_POINTER(FLASH_MAIN_CTRL_BASE)

/* FLASH_MAIN_CTRL bit positions */
#define FLASH_MAIN_CTRL_MAIN_HIGH_W_EN_POS 2
#define FLASH_MAIN_CTRL_MAIN_MIDDLE_W_EN_POS 1
#define FLASH_MAIN_CTRL_MAIN_LOW_W_EN_POS 0

/* FLASH_MAIN_CTRL bit-band aliases */
#define FLASH_MAIN_CTRL_MAIN_HIGH_W_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_MAIN_CTRL_BASE, FLASH_MAIN_CTRL_MAIN_HIGH_W_EN_POS))
#define FLASH_MAIN_CTRL_MAIN_MIDDLE_W_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_MAIN_CTRL_BASE, FLASH_MAIN_CTRL_MAIN_MIDDLE_W_EN_POS))
#define FLASH_MAIN_CTRL_MAIN_LOW_W_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_MAIN_CTRL_BASE, FLASH_MAIN_CTRL_MAIN_LOW_W_EN_POS))

/* FLASH_MAIN_CTRL settings */
#define MAIN_HIGH_W_DISABLE_BITBAND     0x0
#define MAIN_HIGH_W_ENABLE_BITBAND      0x1
#define MAIN_HIGH_W_DISABLE             (MAIN_HIGH_W_DISABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_HIGH_W_EN_POS)
#define MAIN_HIGH_W_ENABLE              (MAIN_HIGH_W_ENABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_HIGH_W_EN_POS)

#define MAIN_MIDDLE_W_DISABLE_BITBAND   0x0
#define MAIN_MIDDLE_W_ENABLE_BITBAND    0x1
#define MAIN_MIDDLE_W_DISABLE           (MAIN_MIDDLE_W_DISABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_MIDDLE_W_EN_POS)
#define MAIN_MIDDLE_W_ENABLE            (MAIN_MIDDLE_W_ENABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_MIDDLE_W_EN_POS)

#define MAIN_LOW_W_DISABLE_BITBAND      0x0
#define MAIN_LOW_W_ENABLE_BITBAND       0x1
#define MAIN_LOW_W_DISABLE              (MAIN_LOW_W_DISABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_LOW_W_EN_POS)
#define MAIN_LOW_W_ENABLE               (MAIN_LOW_W_ENABLE_BITBAND << FLASH_MAIN_CTRL_MAIN_LOW_W_EN_POS)

/* Flash, Memory and RF Power-Up Delay Configuration */
#define FLASH_DELAY_CTRL_BASE           0x40000510
#define FLASH_DELAY_CTRL                REG32_POINTER(FLASH_DELAY_CTRL_BASE)

/* FLASH_DELAY_CTRL bit positions */
#define FLASH_DELAY_CTRL_READ_MARGIN_POS 7
#define FLASH_DELAY_CTRL_SYSCLK_FREQ_POS 0
#define FLASH_DELAY_CTRL_SYSCLK_FREQ_MASK (0xF << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)

/* FLASH_DELAY_CTRL bit-band aliases */
#define FLASH_DELAY_CTRL_READ_MARGIN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_DELAY_CTRL_BASE, FLASH_DELAY_CTRL_READ_MARGIN_POS))

/* FLASH_DELAY_CTRL settings */
#define DEFAULT_READ_MARGIN_BITBAND     0x0
#define FAST_READ_MARGIN_BITBAND        0x1
#define DEFAULT_READ_MARGIN             (DEFAULT_READ_MARGIN_BITBAND << FLASH_DELAY_CTRL_READ_MARGIN_POS)
#define FAST_READ_MARGIN                (FAST_READ_MARGIN_BITBAND << FLASH_DELAY_CTRL_READ_MARGIN_POS)

#define FLASH_DELAY_FOR_SYSCLK_3MHZ     (0x0 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_4MHZ     (0x1 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_5MHZ     (0x2 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_8MHZ     (0x3 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_10MHZ    (0x4 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_12MHZ    (0x5 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_16MHZ    (0x6 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_20MHZ    (0x7 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_24MHZ    (0x8 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)
#define FLASH_DELAY_FOR_SYSCLK_48MHZ    (0x9 << FLASH_DELAY_CTRL_SYSCLK_FREQ_POS)

/* Flash Command Control Register */
#define FLASH_CMD_CTRL_BASE             0x40000534
#define FLASH_CMD_CTRL                  REG32_POINTER(FLASH_CMD_CTRL_BASE)

/* FLASH_CMD_CTRL bit positions */
#define FLASH_CMD_CTRL_CMD_END_POS      5
#define FLASH_CMD_CTRL_CMD_END_MASK     (0x3 << FLASH_CMD_CTRL_CMD_END_POS)
#define FLASH_CMD_CTRL_COMMAND_POS      0
#define FLASH_CMD_CTRL_COMMAND_MASK     (0x1F << FLASH_CMD_CTRL_COMMAND_POS)

/* FLASH_CMD_CTRL settings */
#define CMD_END                         (0x3 << FLASH_CMD_CTRL_CMD_END_POS)

#define CMD_IDLE                        (0x0 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_WAKE_UP                     (0x1 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_LOAD_TRIM                   (0x2 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_READ                        (0x5 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_PROGRAM_NOSEQ               (0x6 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_PROGRAM_SEQ                 (0x7 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_SECTOR_ERASE                (0x8 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_MASS_ERASE                  (0x9 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_SET_LOW_POWER               (0xA << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_UNSET_LOW_POWER             (0xB << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_SET_RECALL                  (0xC << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_UNSET_RECALL                (0xD << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_SET_VREAD1                  (0xE << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_UNSET_VREAD1                (0xF << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_SET_VREAD0                  (0x10 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_UNSET_VREAD0                (0x11 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_WRITE_USER_RED1             (0x12 << FLASH_CMD_CTRL_COMMAND_POS)
#define CMD_WRITE_USER_RED2             (0x13 << FLASH_CMD_CTRL_COMMAND_POS)

/* Flash Interface Status Register */
#define FLASH_IF_STATUS_BASE            0x40000538
#define FLASH_IF_STATUS                 READONLY_REG32_POINTER(FLASH_IF_STATUS_BASE)

/* FLASH_IF_STATUS bit positions */
#define FLASH_IF_STATUS_TRIMMED_STATUS_POS 13
#define FLASH_IF_STATUS_ISOLATE_STATUS_POS 12
#define FLASH_IF_STATUS_PROG_SEQ_DATA_REQ_POS 11
#define FLASH_IF_STATUS_BUSY_POS        10
#define FLASH_IF_STATUS_RED2_W_UNLOCK_POS 9
#define FLASH_IF_STATUS_RED1_W_UNLOCK_POS 8
#define FLASH_IF_STATUS_NVR3_W_UNLOCK_POS 6
#define FLASH_IF_STATUS_NVR2_W_UNLOCK_POS 5
#define FLASH_IF_STATUS_NVR1_W_UNLOCK_POS 4
#define FLASH_IF_STATUS_MAIN_HIGH_W_UNLOCK_POS 2
#define FLASH_IF_STATUS_MAIN_MIDDLE_W_UNLOCK_POS 1
#define FLASH_IF_STATUS_MAIN_LOW_W_UNLOCK_POS 0

/* FLASH_IF_STATUS bit-band aliases */
#define FLASH_IF_STATUS_TRIMMED_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_TRIMMED_STATUS_POS))
#define FLASH_IF_STATUS_ISOLATE_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_ISOLATE_STATUS_POS))
#define FLASH_IF_STATUS_PROG_SEQ_DATA_REQ_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_PROG_SEQ_DATA_REQ_POS))
#define FLASH_IF_STATUS_BUSY_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_BUSY_POS))
#define FLASH_IF_STATUS_RED2_W_UNLOCK_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_RED2_W_UNLOCK_POS))
#define FLASH_IF_STATUS_RED1_W_UNLOCK_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_RED1_W_UNLOCK_POS))
#define FLASH_IF_STATUS_NVR3_W_UNLOCK_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_NVR3_W_UNLOCK_POS))
#define FLASH_IF_STATUS_NVR2_W_UNLOCK_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_NVR2_W_UNLOCK_POS))
#define FLASH_IF_STATUS_NVR1_W_UNLOCK_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_NVR1_W_UNLOCK_POS))
#define FLASH_IF_STATUS_MAIN_HIGH_W_UNLOCK_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_MAIN_HIGH_W_UNLOCK_POS))
#define FLASH_IF_STATUS_MAIN_MIDDLE_W_UNLOCK_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_MAIN_MIDDLE_W_UNLOCK_POS))
#define FLASH_IF_STATUS_MAIN_LOW_W_UNLOCK_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_IF_STATUS_BASE, FLASH_IF_STATUS_MAIN_LOW_W_UNLOCK_POS))

/* FLASH_IF_STATUS settings */
#define FLASH_UNTRIMMED_BITBAND         0x0
#define FLASH_TRIMMED_BITBAND           0x1
#define FLASH_UNTRIMMED                 (FLASH_UNTRIMMED_BITBAND << FLASH_IF_STATUS_TRIMMED_STATUS_POS)
#define FLASH_TRIMMED                   (FLASH_TRIMMED_BITBAND << FLASH_IF_STATUS_TRIMMED_STATUS_POS)

#define FLASH_ACCESSIBLE_BITBAND        0x0
#define FLASH_ISOLATE_BITBAND           0x1
#define FLASH_ACCESSIBLE                (FLASH_ACCESSIBLE_BITBAND << FLASH_IF_STATUS_ISOLATE_STATUS_POS)
#define FLASH_ISOLATE                   (FLASH_ISOLATE_BITBAND << FLASH_IF_STATUS_ISOLATE_STATUS_POS)

#define FLASH_PROG_SEQ_IDLE_BITBAND     0x0
#define FLASH_PROG_SEQ_REQ_NEW_DATA_BITBAND 0x1
#define FLASH_PROG_SEQ_IDLE             (FLASH_PROG_SEQ_IDLE_BITBAND << FLASH_IF_STATUS_PROG_SEQ_DATA_REQ_POS)
#define FLASH_PROG_SEQ_REQ_NEW_DATA     (FLASH_PROG_SEQ_REQ_NEW_DATA_BITBAND << FLASH_IF_STATUS_PROG_SEQ_DATA_REQ_POS)

#define FLASH_IF_IDLE_BITBAND           0x0
#define FLASH_IF_BUSY_BITBAND           0x1
#define FLASH_IF_IDLE                   (FLASH_IF_IDLE_BITBAND << FLASH_IF_STATUS_BUSY_POS)
#define FLASH_IF_BUSY                   (FLASH_IF_BUSY_BITBAND << FLASH_IF_STATUS_BUSY_POS)

#define FLASH_RED2_W_LOCKED_BITBAND     0x0
#define FLASH_RED2_W_UNLOCKED_BITBAND   0x1
#define FLASH_RED2_W_LOCKED             (FLASH_RED2_W_LOCKED_BITBAND << FLASH_IF_STATUS_RED2_W_UNLOCK_POS)
#define FLASH_RED2_W_UNLOCKED           (FLASH_RED2_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_RED2_W_UNLOCK_POS)

#define FLASH_RED1_W_LOCKED_BITBAND     0x0
#define FLASH_RED1_W_UNLOCKED_BITBAND   0x1
#define FLASH_RED1_W_LOCKED             (FLASH_RED1_W_LOCKED_BITBAND << FLASH_IF_STATUS_RED1_W_UNLOCK_POS)
#define FLASH_RED1_W_UNLOCKED           (FLASH_RED1_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_RED1_W_UNLOCK_POS)

#define FLASH_NVR3_W_LOCKED_BITBAND     0x0
#define FLASH_NVR3_W_UNLOCKED_BITBAND   0x1
#define FLASH_NVR3_W_LOCKED             (FLASH_NVR3_W_LOCKED_BITBAND << FLASH_IF_STATUS_NVR3_W_UNLOCK_POS)
#define FLASH_NVR3_W_UNLOCKED           (FLASH_NVR3_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_NVR3_W_UNLOCK_POS)

#define FLASH_NVR2_W_LOCKED_BITBAND     0x0
#define FLASH_NVR2_W_UNLOCKED_BITBAND   0x1
#define FLASH_NVR2_W_LOCKED             (FLASH_NVR2_W_LOCKED_BITBAND << FLASH_IF_STATUS_NVR2_W_UNLOCK_POS)
#define FLASH_NVR2_W_UNLOCKED           (FLASH_NVR2_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_NVR2_W_UNLOCK_POS)

#define FLASH_NVR1_W_LOCKED_BITBAND     0x0
#define FLASH_NVR1_W_UNLOCKED_BITBAND   0x1
#define FLASH_NVR1_W_LOCKED             (FLASH_NVR1_W_LOCKED_BITBAND << FLASH_IF_STATUS_NVR1_W_UNLOCK_POS)
#define FLASH_NVR1_W_UNLOCKED           (FLASH_NVR1_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_NVR1_W_UNLOCK_POS)

#define FLASH_MAIN_HIGH_W_LOCKED_BITBAND 0x0
#define FLASH_MAIN_HIGH_W_UNLOCKED_BITBAND 0x1
#define FLASH_MAIN_HIGH_W_LOCKED        (FLASH_MAIN_HIGH_W_LOCKED_BITBAND << FLASH_IF_STATUS_MAIN_HIGH_W_UNLOCK_POS)
#define FLASH_MAIN_HIGH_W_UNLOCKED      (FLASH_MAIN_HIGH_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_MAIN_HIGH_W_UNLOCK_POS)

#define FLASH_MAIN_MIDDLE_W_LOCKED_BITBAND 0x0
#define FLASH_MAIN_MIDDLE_W_UNLOCKED_BITBAND 0x1
#define FLASH_MAIN_MIDDLE_W_LOCKED      (FLASH_MAIN_MIDDLE_W_LOCKED_BITBAND << FLASH_IF_STATUS_MAIN_MIDDLE_W_UNLOCK_POS)
#define FLASH_MAIN_MIDDLE_W_UNLOCKED    (FLASH_MAIN_MIDDLE_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_MAIN_MIDDLE_W_UNLOCK_POS)

#define FLASH_MAIN_LOW_W_LOCKED_BITBAND 0x0
#define FLASH_MAIN_LOW_W_UNLOCKED_BITBAND 0x1
#define FLASH_MAIN_LOW_W_LOCKED         (FLASH_MAIN_LOW_W_LOCKED_BITBAND << FLASH_IF_STATUS_MAIN_LOW_W_UNLOCK_POS)
#define FLASH_MAIN_LOW_W_UNLOCKED       (FLASH_MAIN_LOW_W_UNLOCKED_BITBAND << FLASH_IF_STATUS_MAIN_LOW_W_UNLOCK_POS)

/* Flash Address Register */
#define FLASH_ADDR_BASE                 0x4000053C
#define FLASH_ADDR                      REG32_POINTER(FLASH_ADDR_BASE)

/* Flash Read/Write Data Register */
#define FLASH_DATA_BASE                 0x40000540
#define FLASH_DATA                      REG32_POINTER(FLASH_DATA_BASE)

/* Flash NVR Write Unlock Register */
#define FLASH_NVR_WRITE_UNLOCK_BASE     0x40000548
#define FLASH_NVR_WRITE_UNLOCK          REG32_POINTER(FLASH_NVR_WRITE_UNLOCK_BASE)

/* Flash NVR Control Register */
#define FLASH_NVR_CTRL_BASE             0x4000054C
#define FLASH_NVR_CTRL                  REG32_POINTER(FLASH_NVR_CTRL_BASE)

/* FLASH_NVR_CTRL bit positions */
#define FLASH_NVR_CTRL_NVR3_W_EN_POS    3
#define FLASH_NVR_CTRL_NVR2_W_EN_POS    2
#define FLASH_NVR_CTRL_NVR1_W_EN_POS    1

/* FLASH_NVR_CTRL bit-band aliases */
#define FLASH_NVR_CTRL_NVR3_W_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_NVR_CTRL_BASE, FLASH_NVR_CTRL_NVR3_W_EN_POS))
#define FLASH_NVR_CTRL_NVR2_W_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_NVR_CTRL_BASE, FLASH_NVR_CTRL_NVR2_W_EN_POS))
#define FLASH_NVR_CTRL_NVR1_W_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_NVR_CTRL_BASE, FLASH_NVR_CTRL_NVR1_W_EN_POS))

/* FLASH_NVR_CTRL settings */
#define NVR3_WRITE_DISABLE_BITBAND      0x0
#define NVR3_WRITE_ENABLE_BITBAND       0x1
#define NVR3_WRITE_DISABLE              (NVR3_WRITE_DISABLE_BITBAND << FLASH_NVR_CTRL_NVR3_W_EN_POS)
#define NVR3_WRITE_ENABLE               (NVR3_WRITE_ENABLE_BITBAND << FLASH_NVR_CTRL_NVR3_W_EN_POS)

#define NVR2_WRITE_DISABLE_BITBAND      0x0
#define NVR2_WRITE_ENABLE_BITBAND       0x1
#define NVR2_WRITE_DISABLE              (NVR2_WRITE_DISABLE_BITBAND << FLASH_NVR_CTRL_NVR2_W_EN_POS)
#define NVR2_WRITE_ENABLE               (NVR2_WRITE_ENABLE_BITBAND << FLASH_NVR_CTRL_NVR2_W_EN_POS)

#define NVR1_WRITE_DISABLE_BITBAND      0x0
#define NVR1_WRITE_ENABLE_BITBAND       0x1
#define NVR1_WRITE_DISABLE              (NVR1_WRITE_DISABLE_BITBAND << FLASH_NVR_CTRL_NVR1_W_EN_POS)
#define NVR1_WRITE_ENABLE               (NVR1_WRITE_ENABLE_BITBAND << FLASH_NVR_CTRL_NVR1_W_EN_POS)

/* Flash Patch Address Register */
#define FLASH_PATCH_ADDR_BASE           0x40000568
#define FLASH_PATCH_ADDR                REG32_POINTER(FLASH_PATCH_ADDR_BASE)

/* FLASH_PATCH_ADDR bit positions */
#define FLASH_PATCH_ADDR_PATCH_ADDR_POS 11
#define FLASH_PATCH_ADDR_PATCH_ADDR_MASK (0x3FF << FLASH_PATCH_ADDR_PATCH_ADDR_POS)

/* Flash Copier Config Register */
#define FLASH_COPY_CFG_BASE             0x40000580
#define FLASH_COPY_CFG                  REG32_POINTER(FLASH_COPY_CFG_BASE)

/* FLASH_COPY_CFG bit positions */
#define FLASH_COPY_CFG_COMP_ADDR_STEP_POS 18
#define FLASH_COPY_CFG_COMP_ADDR_DIR_POS 17
#define FLASH_COPY_CFG_COMP_MODE_POS    16
#define FLASH_COPY_CFG_COPY_DEST_POS    9
#define FLASH_COPY_CFG_COPY_MODE_POS    8
#define FLASH_COPY_CFG_MODE_POS         0

/* FLASH_COPY_CFG bit-band aliases */
#define FLASH_COPY_CFG_COMP_ADDR_STEP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CFG_BASE, FLASH_COPY_CFG_COMP_ADDR_STEP_POS))
#define FLASH_COPY_CFG_COMP_ADDR_DIR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CFG_BASE, FLASH_COPY_CFG_COMP_ADDR_DIR_POS))
#define FLASH_COPY_CFG_COMP_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CFG_BASE, FLASH_COPY_CFG_COMP_MODE_POS))
#define FLASH_COPY_CFG_COPY_DEST_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CFG_BASE, FLASH_COPY_CFG_COPY_DEST_POS))
#define FLASH_COPY_CFG_COPY_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CFG_BASE, FLASH_COPY_CFG_COPY_MODE_POS))
#define FLASH_COPY_CFG_MODE_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CFG_BASE, FLASH_COPY_CFG_MODE_POS))

/* FLASH_COPY_CFG settings */
#define COMP_ADDR_STEP_1_BITBAND        0x0
#define COMP_ADDR_STEP_2_BITBAND        0x1
#define COMP_ADDR_STEP_1                (COMP_ADDR_STEP_1_BITBAND << FLASH_COPY_CFG_COMP_ADDR_STEP_POS)
#define COMP_ADDR_STEP_2                (COMP_ADDR_STEP_2_BITBAND << FLASH_COPY_CFG_COMP_ADDR_STEP_POS)

#define COMP_ADDR_DOWN_BITBAND          0x0
#define COMP_ADDR_UP_BITBAND            0x1
#define COMP_ADDR_DOWN                  (COMP_ADDR_DOWN_BITBAND << FLASH_COPY_CFG_COMP_ADDR_DIR_POS)
#define COMP_ADDR_UP                    (COMP_ADDR_UP_BITBAND << FLASH_COPY_CFG_COMP_ADDR_DIR_POS)

#define COMP_MODE_CONSTANT_BITBAND      0x0
#define COMP_MODE_CHBK_BITBAND          0x1
#define COMP_MODE_CONSTANT              (COMP_MODE_CONSTANT_BITBAND << FLASH_COPY_CFG_COMP_MODE_POS)
#define COMP_MODE_CHBK                  (COMP_MODE_CHBK_BITBAND << FLASH_COPY_CFG_COMP_MODE_POS)

#define COPY_TO_MEM_BITBAND             0x0
#define COPY_TO_CRC_BITBAND             0x1
#define COPY_TO_MEM                     (COPY_TO_MEM_BITBAND << FLASH_COPY_CFG_COPY_DEST_POS)
#define COPY_TO_CRC                     (COPY_TO_CRC_BITBAND << FLASH_COPY_CFG_COPY_DEST_POS)

#define COPY_TO_32BIT_BITBAND           0x0
#define COPY_TO_40BIT_BITBAND           0x1
#define COPY_TO_32BIT                   (COPY_TO_32BIT_BITBAND << FLASH_COPY_CFG_COPY_MODE_POS)
#define COPY_TO_40BIT                   (COPY_TO_40BIT_BITBAND << FLASH_COPY_CFG_COPY_MODE_POS)

#define COPY_MODE_BITBAND               0x0
#define COMPARATOR_MODE_BITBAND         0x1
#define COPY_MODE                       (COPY_MODE_BITBAND << FLASH_COPY_CFG_MODE_POS)
#define COMPARATOR_MODE                 (COMPARATOR_MODE_BITBAND << FLASH_COPY_CFG_MODE_POS)

/* Flash-to-Memory Copier Control and Status */
#define FLASH_COPY_CTRL_BASE            0x400005C8
#define FLASH_COPY_CTRL                 REG32_POINTER(FLASH_COPY_CTRL_BASE)

/* FLASH_COPY_CTRL bit positions */
#define FLASH_COPY_CTRL_ERROR_POS       3
#define FLASH_COPY_CTRL_STOP_POS        2
#define FLASH_COPY_CTRL_START_POS       1
#define FLASH_COPY_CTRL_BUSY_POS        0

/* FLASH_COPY_CTRL bit-band aliases */
#define FLASH_COPY_CTRL_ERROR_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CTRL_BASE, FLASH_COPY_CTRL_ERROR_POS))
#define FLASH_COPY_CTRL_STOP_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CTRL_BASE, FLASH_COPY_CTRL_STOP_POS))
#define FLASH_COPY_CTRL_START_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CTRL_BASE, FLASH_COPY_CTRL_START_POS))
#define FLASH_COPY_CTRL_BUSY_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_COPY_CTRL_BASE, FLASH_COPY_CTRL_BUSY_POS))

/* FLASH_COPY_CTRL settings */
#define COPY_NO_ERROR_BITBAND           0x0
#define COPY_ERROR_BITBAND              0x1
#define COPY_NO_ERROR                   (COPY_NO_ERROR_BITBAND << FLASH_COPY_CTRL_ERROR_POS)
#define COPY_ERROR                      (COPY_ERROR_BITBAND << FLASH_COPY_CTRL_ERROR_POS)

#define COPY_STOP_BITBAND               0x1
#define COPY_STOP                       (COPY_STOP_BITBAND << FLASH_COPY_CTRL_STOP_POS)

#define COPY_START_BITBAND              0x1
#define COPY_START                      (COPY_START_BITBAND << FLASH_COPY_CTRL_START_POS)

#define COPY_IDLE_BITBAND               0x0
#define COPY_BUSY_BITBAND               0x1
#define COPY_IDLE                       (COPY_IDLE_BITBAND << FLASH_COPY_CTRL_BUSY_POS)
#define COPY_BUSY                       (COPY_BUSY_BITBAND << FLASH_COPY_CTRL_BUSY_POS)

/* Flash-to-Memory Copier Source Address Pointer */
#define FLASH_COPY_SRC_ADDR_PTR_BASE    0x400005D0
#define FLASH_COPY_SRC_ADDR_PTR         REG32_POINTER(FLASH_COPY_SRC_ADDR_PTR_BASE)

/* FLASH_COPY_SRC_ADDR_PTR bit positions */
#define FLASH_COPY_SRC_ADDR_PTR_COPY_SRC_ADDR_PTR_POS 0
#define FLASH_COPY_SRC_ADDR_PTR_COPY_SRC_ADDR_PTR_MASK (0x1FFFFF << FLASH_COPY_SRC_ADDR_PTR_COPY_SRC_ADDR_PTR_POS)

/* Flash-to-Memory Copier Destination Address Pointer */
#define FLASH_COPY_DST_ADDR_PTR_BASE    0x400005D4
#define FLASH_COPY_DST_ADDR_PTR         REG32_POINTER(FLASH_COPY_DST_ADDR_PTR_BASE)

/* FLASH_COPY_DST_ADDR_PTR bit positions */
#define FLASH_COPY_DST_ADDR_PTR_COPY_DST_ADDR_PTR_POS 2
#define FLASH_COPY_DST_ADDR_PTR_COPY_DST_ADDR_PTR_MASK (0x3FFFFFFF << FLASH_COPY_DST_ADDR_PTR_COPY_DST_ADDR_PTR_POS)

/* Flash-to-Memory Copier Word Count */
#define FLASH_COPY_WORD_CNT_BASE        0x400005D8
#define FLASH_COPY_WORD_CNT             REG32_POINTER(FLASH_COPY_WORD_CNT_BASE)

/* FLASH_COPY_WORD_CNT bit positions */
#define FLASH_COPY_WORD_CNT_COPY_WORD_CNT_POS 0
#define FLASH_COPY_WORD_CNT_COPY_WORD_CNT_MASK (0x1FFFF << FLASH_COPY_WORD_CNT_COPY_WORD_CNT_POS)

/* Flash ECC Control Register */
#define FLASH_ECC_CTRL_BASE             0x400005DC
#define FLASH_ECC_CTRL                  REG32_POINTER(FLASH_ECC_CTRL_BASE)

/* FLASH_ECC_CTRL bit positions */
#define FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_POS 8
#define FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_MASK (0xFF << FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_POS)
#define FLASH_ECC_CTRL_COPIER_ECC_CTRL_POS 3
#define FLASH_ECC_CTRL_CMD_ECC_CTRL_POS 2
#define FLASH_ECC_CTRL_IDBUS_ECC_CTRL_POS 0

/* FLASH_ECC_CTRL bit-band aliases */
#define FLASH_ECC_CTRL_COPIER_ECC_CTRL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_ECC_CTRL_BASE, FLASH_ECC_CTRL_COPIER_ECC_CTRL_POS))
#define FLASH_ECC_CTRL_CMD_ECC_CTRL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_ECC_CTRL_BASE, FLASH_ECC_CTRL_CMD_ECC_CTRL_POS))
#define FLASH_ECC_CTRL_IDBUS_ECC_CTRL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_ECC_CTRL_BASE, FLASH_ECC_CTRL_IDBUS_ECC_CTRL_POS))

/* FLASH_ECC_CTRL settings */
#define FLASH_ECC_COR_INT_THRESHOLD_DISABLED (0x0 << FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_POS)
#define FLASH_ECC_COR_INT_THRESHOLD_1   (0x1 << FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_POS)
#define FLASH_ECC_COR_INT_THRESHOLD_255 (0xFF << FLASH_ECC_CTRL_ECC_COR_CNT_INT_THRESHOLD_POS)

#define FLASH_COPIER_ECC_DISABLE_BITBAND 0x0
#define FLASH_COPIER_ECC_ENABLE_BITBAND 0x1
#define FLASH_COPIER_ECC_DISABLE        (FLASH_COPIER_ECC_DISABLE_BITBAND << FLASH_ECC_CTRL_COPIER_ECC_CTRL_POS)
#define FLASH_COPIER_ECC_ENABLE         (FLASH_COPIER_ECC_ENABLE_BITBAND << FLASH_ECC_CTRL_COPIER_ECC_CTRL_POS)

#define FLASH_CMD_ECC_DISABLE_BITBAND   0x0
#define FLASH_CMD_ECC_ENABLE_BITBAND    0x1
#define FLASH_CMD_ECC_DISABLE           (FLASH_CMD_ECC_DISABLE_BITBAND << FLASH_ECC_CTRL_CMD_ECC_CTRL_POS)
#define FLASH_CMD_ECC_ENABLE            (FLASH_CMD_ECC_ENABLE_BITBAND << FLASH_ECC_CTRL_CMD_ECC_CTRL_POS)

#define FLASH_IDBUS_ECC_DISABLE_BITBAND 0x0
#define FLASH_IDBUS_ECC_ENABLE_BITBAND  0x1
#define FLASH_IDBUS_ECC_DISABLE         (FLASH_IDBUS_ECC_DISABLE_BITBAND << FLASH_ECC_CTRL_IDBUS_ECC_CTRL_POS)
#define FLASH_IDBUS_ECC_ENABLE          (FLASH_IDBUS_ECC_ENABLE_BITBAND << FLASH_ECC_CTRL_IDBUS_ECC_CTRL_POS)

/* Flash ECC Status Register */
#define FLASH_ECC_STATUS_BASE           0x400005E0
#define FLASH_ECC_STATUS                REG32_POINTER(FLASH_ECC_STATUS_BASE)

/* FLASH_ECC_STATUS bit positions */
#define FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_CLEAR_POS 6
#define FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_CLEAR_POS 5
#define FLASH_ECC_STATUS_ECC_ERROR_ADDR_CLEAR_POS 4
#define FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_STATUS_POS 1
#define FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_STATUS_POS 0

/* FLASH_ECC_STATUS bit-band aliases */
#define FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_ECC_STATUS_BASE, FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_CLEAR_POS))
#define FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_ECC_STATUS_BASE, FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_CLEAR_POS))
#define FLASH_ECC_STATUS_ECC_ERROR_ADDR_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(FLASH_ECC_STATUS_BASE, FLASH_ECC_STATUS_ECC_ERROR_ADDR_CLEAR_POS))
#define FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_ECC_STATUS_BASE, FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_STATUS_POS))
#define FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(FLASH_ECC_STATUS_BASE, FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_STATUS_POS))

/* FLASH_ECC_STATUS settings */
#define FLASH_ECC_COR_ERROR_CNT_CLEAR_BITBAND 0x1
#define FLASH_ECC_COR_ERROR_CNT_CLEAR   (FLASH_ECC_COR_ERROR_CNT_CLEAR_BITBAND << FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_CLEAR_POS)

#define FLASH_ECC_UNCOR_ERROR_CNT_CLEAR_BITBAND 0x1
#define FLASH_ECC_UNCOR_ERROR_CNT_CLEAR (FLASH_ECC_UNCOR_ERROR_CNT_CLEAR_BITBAND << FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_CLEAR_POS)

#define FLASH_ECC_ERROR_ADDR_CLEAR_BITBAND 0x1
#define FLASH_ECC_ERROR_ADDR_CLEAR      (FLASH_ECC_ERROR_ADDR_CLEAR_BITBAND << FLASH_ECC_STATUS_ECC_ERROR_ADDR_CLEAR_POS)

#define FLASH_ECC_NO_CORRECTED_ERROR_BITBAND 0x0
#define FLASH_ECC_CORRECTED_ERROR_BITBAND 0x1
#define FLASH_ECC_NO_CORRECTED_ERROR    (FLASH_ECC_NO_CORRECTED_ERROR_BITBAND << FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_STATUS_POS)
#define FLASH_ECC_CORRECTED_ERROR       (FLASH_ECC_CORRECTED_ERROR_BITBAND << FLASH_ECC_STATUS_ECC_COR_ERROR_CNT_STATUS_POS)

#define FLASH_ECC_NO_UNCORRECTED_ERROR_BITBAND 0x0
#define FLASH_ECC_UNCORRECTED_ERROR_BITBAND 0x1
#define FLASH_ECC_NO_UNCORRECTED_ERROR  (FLASH_ECC_NO_UNCORRECTED_ERROR_BITBAND << FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_STATUS_POS)
#define FLASH_ECC_UNCORRECTED_ERROR     (FLASH_ECC_UNCORRECTED_ERROR_BITBAND << FLASH_ECC_STATUS_ECC_UNCOR_ERROR_CNT_STATUS_POS)

/* Flash Address of the Latest Detected Error */
#define FLASH_ECC_ERROR_ADDR_BASE       0x400005E4
#define FLASH_ECC_ERROR_ADDR            READONLY_REG32_POINTER(FLASH_ECC_ERROR_ADDR_BASE)

/* FLASH_ECC_ERROR_ADDR bit positions */
#define FLASH_ECC_ERROR_ADDR_ECC_ERROR_ADDR_POS 2
#define FLASH_ECC_ERROR_ADDR_ECC_ERROR_ADDR_MASK (0x7FFFF << FLASH_ECC_ERROR_ADDR_ECC_ERROR_ADDR_POS)

/* Flash ECC Uncorrected Error Counter */
#define FLASH_ECC_UNCOR_ERROR_CNT_BASE  0x400005E8
#define FLASH_ECC_UNCOR_ERROR_CNT       READONLY_REG32_POINTER(FLASH_ECC_UNCOR_ERROR_CNT_BASE)

/* FLASH_ECC_UNCOR_ERROR_CNT bit positions */
#define FLASH_ECC_UNCOR_ERROR_CNT_ECC_UNCOR_ERROR_CNT_POS 0
#define FLASH_ECC_UNCOR_ERROR_CNT_ECC_UNCOR_ERROR_CNT_MASK (0xFF << FLASH_ECC_UNCOR_ERROR_CNT_ECC_UNCOR_ERROR_CNT_POS)

/* Flash ECC Corrected Error Counter */
#define FLASH_ECC_COR_ERROR_CNT_BASE    0x400005EC
#define FLASH_ECC_COR_ERROR_CNT         READONLY_REG32_POINTER(FLASH_ECC_COR_ERROR_CNT_BASE)

/* FLASH_ECC_COR_ERROR_CNT bit positions */
#define FLASH_ECC_COR_ERROR_CNT_ECC_COR_ERROR_CNT_POS 0
#define FLASH_ECC_COR_ERROR_CNT_ECC_COR_ERROR_CNT_MASK (0xFF << FLASH_ECC_COR_ERROR_CNT_ECC_COR_ERROR_CNT_POS)

/* ----------------------------------------------------------------------------
 * DMA Controller Configuration and Control
 * ------------------------------------------------------------------------- */

/* DMA Channel Control and Configuration */
#define DMA_CTRL0_BASE                  0x40000600
#define DMA_CTRL0                       REG32_POINTER(DMA_CTRL0_BASE)

/* DMA_CTRL0 bit positions */
#define DMA_CTRL0_DEST_ADDR_STEP_SIZE_POS 30
#define DMA_CTRL0_DEST_ADDR_STEP_SIZE_MASK (0x3 << DMA_CTRL0_DEST_ADDR_STEP_SIZE_POS)
#define DMA_CTRL0_SRC_ADDR_STEP_SIZE_POS 28
#define DMA_CTRL0_SRC_ADDR_STEP_SIZE_MASK (0x3 << DMA_CTRL0_SRC_ADDR_STEP_SIZE_POS)
#define DMA_CTRL0_DEST_ADDR_STEP_MODE_POS 27
#define DMA_CTRL0_SRC_ADDR_STEP_MODE_POS 26
#define DMA_CTRL0_BYTE_ORDER_POS        25
#define DMA_CTRL0_DISABLE_INT_ENABLE_POS 24
#define DMA_CTRL0_ERROR_INT_ENABLE_POS  23
#define DMA_CTRL0_COMPLETE_INT_ENABLE_POS 22
#define DMA_CTRL0_COUNTER_INT_ENABLE_POS 21
#define DMA_CTRL0_START_INT_ENABLE_POS  20
#define DMA_CTRL0_DEST_WORD_SIZE_POS    18
#define DMA_CTRL0_DEST_WORD_SIZE_MASK   (0x3 << DMA_CTRL0_DEST_WORD_SIZE_POS)
#define DMA_CTRL0_SRC_WORD_SIZE_POS     16
#define DMA_CTRL0_SRC_WORD_SIZE_MASK    (0x3 << DMA_CTRL0_SRC_WORD_SIZE_POS)
#define DMA_CTRL0_DEST_SELECT_POS       12
#define DMA_CTRL0_DEST_SELECT_MASK      (0x7 << DMA_CTRL0_DEST_SELECT_POS)
#define DMA_CTRL0_SRC_SELECT_POS        9
#define DMA_CTRL0_SRC_SELECT_MASK       (0x7 << DMA_CTRL0_SRC_SELECT_POS)
#define DMA_CTRL0_CHANNEL_PRIORITY_POS  6
#define DMA_CTRL0_CHANNEL_PRIORITY_MASK (0x3 << DMA_CTRL0_CHANNEL_PRIORITY_POS)
#define DMA_CTRL0_TRANSFER_TYPE_POS     4
#define DMA_CTRL0_TRANSFER_TYPE_MASK    (0x3 << DMA_CTRL0_TRANSFER_TYPE_POS)
#define DMA_CTRL0_DEST_ADDR_INC_POS     3
#define DMA_CTRL0_SRC_ADDR_INC_POS      2
#define DMA_CTRL0_ADDR_MODE_POS         1
#define DMA_CTRL0_ENABLE_POS            0

/* DMA_CTRL0 bit-band aliases */
#define DMA_CTRL0_DEST_ADDR_STEP_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_DEST_ADDR_STEP_MODE_POS))
#define DMA_CTRL0_SRC_ADDR_STEP_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_SRC_ADDR_STEP_MODE_POS))
#define DMA_CTRL0_BYTE_ORDER_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_BYTE_ORDER_POS))
#define DMA_CTRL0_DISABLE_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_DISABLE_INT_ENABLE_POS))
#define DMA_CTRL0_ERROR_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_ERROR_INT_ENABLE_POS))
#define DMA_CTRL0_COMPLETE_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_COMPLETE_INT_ENABLE_POS))
#define DMA_CTRL0_COUNTER_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_COUNTER_INT_ENABLE_POS))
#define DMA_CTRL0_START_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_START_INT_ENABLE_POS))
#define DMA_CTRL0_DEST_ADDR_INC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_DEST_ADDR_INC_POS))
#define DMA_CTRL0_SRC_ADDR_INC_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_SRC_ADDR_INC_POS))
#define DMA_CTRL0_ADDR_MODE_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_ADDR_MODE_POS))
#define DMA_CTRL0_ENABLE_BITBAND        REG32_POINTER(SYS_CALC_BITBAND(DMA_CTRL0_BASE, DMA_CTRL0_ENABLE_POS))

/* DMA_CTRL0 settings */
#define DMA_DEST_ADDR_STEP_SIZE_1       (0x0 << DMA_CTRL0_DEST_ADDR_STEP_SIZE_POS)
#define DMA_DEST_ADDR_STEP_SIZE_2       (0x1 << DMA_CTRL0_DEST_ADDR_STEP_SIZE_POS)
#define DMA_DEST_ADDR_STEP_SIZE_3       (0x2 << DMA_CTRL0_DEST_ADDR_STEP_SIZE_POS)
#define DMA_DEST_ADDR_STEP_SIZE_4       (0x3 << DMA_CTRL0_DEST_ADDR_STEP_SIZE_POS)

#define DMA_SRC_ADDR_STEP_SIZE_1        (0x0 << DMA_CTRL0_SRC_ADDR_STEP_SIZE_POS)
#define DMA_SRC_ADDR_STEP_SIZE_2        (0x1 << DMA_CTRL0_SRC_ADDR_STEP_SIZE_POS)
#define DMA_SRC_ADDR_STEP_SIZE_3        (0x2 << DMA_CTRL0_SRC_ADDR_STEP_SIZE_POS)
#define DMA_SRC_ADDR_STEP_SIZE_4        (0x3 << DMA_CTRL0_SRC_ADDR_STEP_SIZE_POS)

#define DMA_DEST_ADDR_POS_BITBAND       0x0
#define DMA_DEST_ADDR_NEG_BITBAND       0x1
#define DMA_DEST_ADDR_POS               (DMA_DEST_ADDR_POS_BITBAND << DMA_CTRL0_DEST_ADDR_STEP_MODE_POS)
#define DMA_DEST_ADDR_NEG               (DMA_DEST_ADDR_NEG_BITBAND << DMA_CTRL0_DEST_ADDR_STEP_MODE_POS)

#define DMA_SRC_ADDR_POS_BITBAND        0x0
#define DMA_SRC_ADDR_NEG_BITBAND        0x1
#define DMA_SRC_ADDR_POS                (DMA_SRC_ADDR_POS_BITBAND << DMA_CTRL0_SRC_ADDR_STEP_MODE_POS)
#define DMA_SRC_ADDR_NEG                (DMA_SRC_ADDR_NEG_BITBAND << DMA_CTRL0_SRC_ADDR_STEP_MODE_POS)

#define DMA_LITTLE_ENDIAN_BITBAND       0x0
#define DMA_BIG_ENDIAN_BITBAND          0x1
#define DMA_LITTLE_ENDIAN               (DMA_LITTLE_ENDIAN_BITBAND << DMA_CTRL0_BYTE_ORDER_POS)
#define DMA_BIG_ENDIAN                  (DMA_BIG_ENDIAN_BITBAND << DMA_CTRL0_BYTE_ORDER_POS)

#define DMA_DISABLE_INT_DISABLE_BITBAND 0x0
#define DMA_DISABLE_INT_ENABLE_BITBAND  0x1
#define DMA_DISABLE_INT_DISABLE         (DMA_DISABLE_INT_DISABLE_BITBAND << DMA_CTRL0_DISABLE_INT_ENABLE_POS)
#define DMA_DISABLE_INT_ENABLE          (DMA_DISABLE_INT_ENABLE_BITBAND << DMA_CTRL0_DISABLE_INT_ENABLE_POS)

#define DMA_ERROR_INT_DISABLE_BITBAND   0x0
#define DMA_ERROR_INT_ENABLE_BITBAND    0x1
#define DMA_ERROR_INT_DISABLE           (DMA_ERROR_INT_DISABLE_BITBAND << DMA_CTRL0_ERROR_INT_ENABLE_POS)
#define DMA_ERROR_INT_ENABLE            (DMA_ERROR_INT_ENABLE_BITBAND << DMA_CTRL0_ERROR_INT_ENABLE_POS)

#define DMA_COMPLETE_INT_DISABLE_BITBAND 0x0
#define DMA_COMPLETE_INT_ENABLE_BITBAND 0x1
#define DMA_COMPLETE_INT_DISABLE        (DMA_COMPLETE_INT_DISABLE_BITBAND << DMA_CTRL0_COMPLETE_INT_ENABLE_POS)
#define DMA_COMPLETE_INT_ENABLE         (DMA_COMPLETE_INT_ENABLE_BITBAND << DMA_CTRL0_COMPLETE_INT_ENABLE_POS)

#define DMA_COUNTER_INT_DISABLE_BITBAND 0x0
#define DMA_COUNTER_INT_ENABLE_BITBAND  0x1
#define DMA_COUNTER_INT_DISABLE         (DMA_COUNTER_INT_DISABLE_BITBAND << DMA_CTRL0_COUNTER_INT_ENABLE_POS)
#define DMA_COUNTER_INT_ENABLE          (DMA_COUNTER_INT_ENABLE_BITBAND << DMA_CTRL0_COUNTER_INT_ENABLE_POS)

#define DMA_START_INT_DISABLE_BITBAND   0x0
#define DMA_START_INT_ENABLE_BITBAND    0x1
#define DMA_START_INT_DISABLE           (DMA_START_INT_DISABLE_BITBAND << DMA_CTRL0_START_INT_ENABLE_POS)
#define DMA_START_INT_ENABLE            (DMA_START_INT_ENABLE_BITBAND << DMA_CTRL0_START_INT_ENABLE_POS)

#define DMA_DEST_WORD_SIZE_8            (0x0 << DMA_CTRL0_DEST_WORD_SIZE_POS)
#define DMA_DEST_WORD_SIZE_16           (0x1 << DMA_CTRL0_DEST_WORD_SIZE_POS)
#define DMA_DEST_WORD_SIZE_32           (0x2 << DMA_CTRL0_DEST_WORD_SIZE_POS)
#define DMA_DEST_WORD_SIZE_4            (0x3 << DMA_CTRL0_DEST_WORD_SIZE_POS)

#define DMA_SRC_WORD_SIZE_8             (0x0 << DMA_CTRL0_SRC_WORD_SIZE_POS)
#define DMA_SRC_WORD_SIZE_16            (0x1 << DMA_CTRL0_SRC_WORD_SIZE_POS)
#define DMA_SRC_WORD_SIZE_32            (0x2 << DMA_CTRL0_SRC_WORD_SIZE_POS)
#define DMA_SRC_WORD_SIZE_4             (0x3 << DMA_CTRL0_SRC_WORD_SIZE_POS)

#define DMA_DEST_I2C                    (0x0 << DMA_CTRL0_DEST_SELECT_POS)
#define DMA_DEST_SPI0                   (0x1 << DMA_CTRL0_DEST_SELECT_POS)
#define DMA_DEST_SPI1                   (0x2 << DMA_CTRL0_DEST_SELECT_POS)
#define DMA_DEST_PCM                    (0x3 << DMA_CTRL0_DEST_SELECT_POS)
#define DMA_DEST_UART                   (0x4 << DMA_CTRL0_DEST_SELECT_POS)
#define DMA_DEST_ASRC                   (0x5 << DMA_CTRL0_DEST_SELECT_POS)
#define DMA_DEST_PBUS                   (0x6 << DMA_CTRL0_DEST_SELECT_POS)
#define DMA_DEST_OD                     (0x7 << DMA_CTRL0_DEST_SELECT_POS)

#define DMA_SRC_I2C                     (0x0 << DMA_CTRL0_SRC_SELECT_POS)
#define DMA_SRC_SPI0                    (0x1 << DMA_CTRL0_SRC_SELECT_POS)
#define DMA_SRC_SPI1                    (0x2 << DMA_CTRL0_SRC_SELECT_POS)
#define DMA_SRC_PCM                     (0x3 << DMA_CTRL0_SRC_SELECT_POS)
#define DMA_SRC_UART                    (0x4 << DMA_CTRL0_SRC_SELECT_POS)
#define DMA_SRC_ASRC                    (0x5 << DMA_CTRL0_SRC_SELECT_POS)
#define DMA_SRC_PBUS                    (0x6 << DMA_CTRL0_SRC_SELECT_POS)
#define DMA_SRC_DMIC                    (0x7 << DMA_CTRL0_SRC_SELECT_POS)

#define DMA_PRIORITY_0                  (0x0 << DMA_CTRL0_CHANNEL_PRIORITY_POS)
#define DMA_PRIORITY_1                  (0x1 << DMA_CTRL0_CHANNEL_PRIORITY_POS)
#define DMA_PRIORITY_2                  (0x2 << DMA_CTRL0_CHANNEL_PRIORITY_POS)
#define DMA_PRIORITY_3                  (0x3 << DMA_CTRL0_CHANNEL_PRIORITY_POS)

#define DMA_TRANSFER_M_TO_M             (0x0 << DMA_CTRL0_TRANSFER_TYPE_POS)
#define DMA_TRANSFER_M_TO_P             (0x1 << DMA_CTRL0_TRANSFER_TYPE_POS)
#define DMA_TRANSFER_P_TO_M             (0x2 << DMA_CTRL0_TRANSFER_TYPE_POS)
#define DMA_TRANSFER_P_TO_P             (0x3 << DMA_CTRL0_TRANSFER_TYPE_POS)

#define DMA_DEST_ADDR_STATIC_BITBAND    0x0
#define DMA_DEST_ADDR_INC_BITBAND       0x1
#define DMA_DEST_ADDR_STATIC            (DMA_DEST_ADDR_STATIC_BITBAND << DMA_CTRL0_DEST_ADDR_INC_POS)
#define DMA_DEST_ADDR_INC               (DMA_DEST_ADDR_INC_BITBAND << DMA_CTRL0_DEST_ADDR_INC_POS)

#define DMA_SRC_ADDR_STATIC_BITBAND     0x0
#define DMA_SRC_ADDR_INC_BITBAND        0x1
#define DMA_SRC_ADDR_STATIC             (DMA_SRC_ADDR_STATIC_BITBAND << DMA_CTRL0_SRC_ADDR_INC_POS)
#define DMA_SRC_ADDR_INC                (DMA_SRC_ADDR_INC_BITBAND << DMA_CTRL0_SRC_ADDR_INC_POS)

#define DMA_ADDR_CIRC_BITBAND           0x0
#define DMA_ADDR_LIN_BITBAND            0x1
#define DMA_ADDR_CIRC                   (DMA_ADDR_CIRC_BITBAND << DMA_CTRL0_ADDR_MODE_POS)
#define DMA_ADDR_LIN                    (DMA_ADDR_LIN_BITBAND << DMA_CTRL0_ADDR_MODE_POS)

#define DMA_DISABLE_BITBAND             0x0
#define DMA_ENABLE_BITBAND              0x1
#define DMA_DISABLE                     (DMA_DISABLE_BITBAND << DMA_CTRL0_ENABLE_POS)
#define DMA_ENABLE                      (DMA_ENABLE_BITBAND << DMA_CTRL0_ENABLE_POS)

/* DMA Channel Source Base Address */
#define DMA_SRC_BASE_ADDR_BASE          0x40000620
#define DMA_SRC_BASE_ADDR               REG32_POINTER(DMA_SRC_BASE_ADDR_BASE)

/* DMA Channel Destination Base Address */
#define DMA_DEST_BASE_ADDR_BASE         0x40000640
#define DMA_DEST_BASE_ADDR              REG32_POINTER(DMA_DEST_BASE_ADDR_BASE)

/* DMA Channel Transfer Control */
#define DMA_CTRL1_BASE                  0x40000660
#define DMA_CTRL1                       REG32_POINTER(DMA_CTRL1_BASE)

/* DMA_CTRL1 bit positions */
#define DMA_CTRL1_COUNTER_INT_VALUE_POS 16
#define DMA_CTRL1_COUNTER_INT_VALUE_MASK (0xFFFF << DMA_CTRL1_COUNTER_INT_VALUE_POS)
#define DMA_CTRL1_TRANSFER_LENGTH_POS   0
#define DMA_CTRL1_TRANSFER_LENGTH_MASK  (0xFFFF << DMA_CTRL1_TRANSFER_LENGTH_POS)

/* DMA Channel Next Source Address */
#define DMA_NEXT_SRC_ADDR_BASE          0x40000680
#define DMA_NEXT_SRC_ADDR               READONLY_REG32_POINTER(DMA_NEXT_SRC_ADDR_BASE)

/* DMA Channel Next Destination Address */
#define DMA_NEXT_DEST_ADDR_BASE         0x400006A0
#define DMA_NEXT_DEST_ADDR              READONLY_REG32_POINTER(DMA_NEXT_DEST_ADDR_BASE)

/* DMA Channel Word Count */
#define DMA_WORD_CNT_BASE               0x400006C0
#define DMA_WORD_CNT                    READONLY_REG32_POINTER(DMA_WORD_CNT_BASE)

/* DMA Status channel */
#define DMA_STATUS_BASE                 0x400006E0
#define DMA_STATUS                      REG32_POINTER(DMA_STATUS_BASE)

/* DMA_STATUS bit positions */
#define DMA_STATUS_ERROR_INT_CLEAR_POS  12
#define DMA_STATUS_COMPLETE_INT_CLEAR_POS 11
#define DMA_STATUS_COUNTER_INT_CLEAR_POS 10
#define DMA_STATUS_START_INT_CLEAR_POS  9
#define DMA_STATUS_DISABLE_INT_CLEAR_POS 8
#define DMA_STATUS_STATE_POS            5
#define DMA_STATUS_STATE_MASK           (0x7 << DMA_STATUS_STATE_POS)
#define DMA_STATUS_ERROR_INT_STATUS_POS 4
#define DMA_STATUS_COMPLETE_INT_STATUS_POS 3
#define DMA_STATUS_COUNTER_INT_STATUS_POS 2
#define DMA_STATUS_START_INT_STATUS_POS 1
#define DMA_STATUS_DISABLE_INT_STATUS_POS 0

/* DMA_STATUS bit-band aliases */
#define DMA_STATUS_ERROR_INT_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_ERROR_INT_CLEAR_POS))
#define DMA_STATUS_COMPLETE_INT_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_COMPLETE_INT_CLEAR_POS))
#define DMA_STATUS_COUNTER_INT_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_COUNTER_INT_CLEAR_POS))
#define DMA_STATUS_START_INT_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_START_INT_CLEAR_POS))
#define DMA_STATUS_DISABLE_INT_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_DISABLE_INT_CLEAR_POS))
#define DMA_STATUS_ERROR_INT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_ERROR_INT_STATUS_POS))
#define DMA_STATUS_COMPLETE_INT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_COMPLETE_INT_STATUS_POS))
#define DMA_STATUS_COUNTER_INT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_COUNTER_INT_STATUS_POS))
#define DMA_STATUS_START_INT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_START_INT_STATUS_POS))
#define DMA_STATUS_DISABLE_INT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(DMA_STATUS_BASE, DMA_STATUS_DISABLE_INT_STATUS_POS))

/* DMA_STATUS settings */
#define DMA_ERROR_INT_CLEAR_BITBAND     0x1
#define DMA_ERROR_INT_CLEAR             (DMA_ERROR_INT_CLEAR_BITBAND << DMA_STATUS_ERROR_INT_CLEAR_POS)

#define DMA_COMPLETE_INT_CLEAR_BITBAND  0x1
#define DMA_COMPLETE_INT_CLEAR          (DMA_COMPLETE_INT_CLEAR_BITBAND << DMA_STATUS_COMPLETE_INT_CLEAR_POS)

#define DMA_COUNTER_INT_CLEAR_BITBAND   0x1
#define DMA_COUNTER_INT_CLEAR           (DMA_COUNTER_INT_CLEAR_BITBAND << DMA_STATUS_COUNTER_INT_CLEAR_POS)

#define DMA_START_INT_CLEAR_BITBAND     0x1
#define DMA_START_INT_CLEAR             (DMA_START_INT_CLEAR_BITBAND << DMA_STATUS_START_INT_CLEAR_POS)

#define DMA_DISABLE_INT_CLEAR_BITBAND   0x1
#define DMA_DISABLE_INT_CLEAR           (DMA_DISABLE_INT_CLEAR_BITBAND << DMA_STATUS_DISABLE_INT_CLEAR_POS)

#define DMA_IDLE                        (0x0 << DMA_STATUS_STATE_POS)
#define DMA_WAIT_SRC_DST_RDY            (0x1 << DMA_STATUS_STATE_POS)
#define DMA_WAIT_SRC_RDY                (0x2 << DMA_STATUS_STATE_POS)
#define DMA_WAIT_RD_COMP                (0x3 << DMA_STATUS_STATE_POS)
#define DMA_CHK_DST                     (0x4 << DMA_STATUS_STATE_POS)
#define DMA_DIS_CHAN                    (0x5 << DMA_STATUS_STATE_POS)
#define DMA_WAIT_DST_RDY                (0x6 << DMA_STATUS_STATE_POS)
#define DMA_WAIT_WR_COMP                (0x7 << DMA_STATUS_STATE_POS)

#define DMA_ERROR_INT_STATUS_BITBAND    0x1
#define DMA_ERROR_INT_STATUS            (DMA_ERROR_INT_STATUS_BITBAND << DMA_STATUS_ERROR_INT_STATUS_POS)

#define DMA_COMPLETE_INT_STATUS_BITBAND 0x1
#define DMA_COMPLETE_INT_STATUS         (DMA_COMPLETE_INT_STATUS_BITBAND << DMA_STATUS_COMPLETE_INT_STATUS_POS)

#define DMA_COUNTER_INT_STATUS_BITBAND  0x1
#define DMA_COUNTER_INT_STATUS          (DMA_COUNTER_INT_STATUS_BITBAND << DMA_STATUS_COUNTER_INT_STATUS_POS)

#define DMA_START_INT_STATUS_BITBAND    0x1
#define DMA_START_INT_STATUS            (DMA_START_INT_STATUS_BITBAND << DMA_STATUS_START_INT_STATUS_POS)

#define DMA_DISABLE_INT_STATUS_BITBAND  0x1
#define DMA_DISABLE_INT_STATUS          (DMA_DISABLE_INT_STATUS_BITBAND << DMA_STATUS_DISABLE_INT_STATUS_POS)

/* ----------------------------------------------------------------------------
 * DIO Interface and Digital Pad control
 * ------------------------------------------------------------------------- */

/* Digital IO Configuration (ADC function only for pads 0 to 3) */
#define DIO_CFG_BASE                    0x40000700
#define DIO_CFG                         REG32_POINTER(DIO_CFG_BASE)

/* DIO_CFG bit positions */
#define DIO_CFG_DRIVE_POS               12
#define DIO_CFG_DRIVE_MASK              (0x3 << DIO_CFG_DRIVE_POS)
#define DIO_CFG_LPF_POS                 10
#define DIO_CFG_PULL_CTRL_POS           8
#define DIO_CFG_PULL_CTRL_MASK          (0x3 << DIO_CFG_PULL_CTRL_POS)
#define DIO_CFG_IO_MODE_POS             0
#define DIO_CFG_IO_MODE_MASK            (0x3F << DIO_CFG_IO_MODE_POS)

/* DIO_CFG bit-band aliases */
#define DIO_CFG_LPF_BITBAND             REG32_POINTER(SYS_CALC_BITBAND(DIO_CFG_BASE, DIO_CFG_LPF_POS))

/* DIO_CFG settings */
#define DIO_2X_DRIVE                    (0x0 << DIO_CFG_DRIVE_POS)
#define DIO_3X_DRIVE                    (0x1 << DIO_CFG_DRIVE_POS)
#define DIO_5X_DRIVE                    (0x2 << DIO_CFG_DRIVE_POS)
#define DIO_6X_DRIVE                    (0x3 << DIO_CFG_DRIVE_POS)

#define DIO_LPF_DISABLE_BITBAND         0x0
#define DIO_LPF_ENABLE_BITBAND          0x1
#define DIO_LPF_DISABLE                 (DIO_LPF_DISABLE_BITBAND << DIO_CFG_LPF_POS)
#define DIO_LPF_ENABLE                  (DIO_LPF_ENABLE_BITBAND << DIO_CFG_LPF_POS)

#define DIO_NO_PULL                     (0x0 << DIO_CFG_PULL_CTRL_POS)
#define DIO_WEAK_PULL_UP                (0x1 << DIO_CFG_PULL_CTRL_POS)
#define DIO_WEAK_PULL_DOWN              (0x2 << DIO_CFG_PULL_CTRL_POS)
#define DIO_STRONG_PULL_UP              (0x3 << DIO_CFG_PULL_CTRL_POS)

#define DIO_MODE_GPIO_IN_0              (0x0 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_GPIO_IN_1              (0x1 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_GPIO_OUT_0             (0x2 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_GPIO_OUT_1             (0x3 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_USRCLK                 (0x4 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SLOWCLK                (0x5 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SYSCLK                 (0x6 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_PCM_SERO               (0x7 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_PCM_FRAME              (0x8 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SPI0_SERO              (0x9 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SPI0_CS                (0xA << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SPI0_CLK               (0xB << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SPI1_SERO              (0xC << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SPI1_CS                (0xD << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SPI1_CLK               (0xE << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_UART_TX                (0xF << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SCL                    (0x10 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_SDA                    (0x11 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_PWM0                   (0x12 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_PWM0_INV               (0x13 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_PWM1                   (0x14 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_PWM1_INV               (0x15 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_LPDSP32_TDO            (0x16 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RFCLK                  (0x17 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RCCLK                  (0x18 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_JTCK_DIV               (0x19 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_EXTCLK_DIV             (0x1A << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_STANDBYCLK             (0x1B << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_TX_DATA             (0x1C << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_TX_DATA_VALID       (0x1D << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_SYNC_P              (0x1E << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_AUDIO0_SYNC_P       (0x1F << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_AUDIO1_SYNC_P       (0x20 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_AUDIO2_SYNC_P       (0x21 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_SPI_CSN             (0x22 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_SPI_CLK             (0x23 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_SPI_MOSI            (0x24 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_DBG0_0              (0x25 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_DBG0_1              (0x26 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_DBG0_2              (0x27 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_DBG0_3              (0x28 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_DBG0_4              (0x29 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_DBG0_5              (0x2A << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_DBG0_6              (0x2B << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_BB_DBG0_7              (0x2C << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_SPI_MISO            (0x2D << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO0               (0x2E << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO1               (0x2F << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO2               (0x30 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO3               (0x31 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO4               (0x32 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO5               (0x33 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO6               (0x34 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO7               (0x35 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO8               (0x36 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_RF_GPIO9               (0x37 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_AUDIOCLK               (0x38 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_AUDIOSLOWCLK           (0x39 << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_OD_P                   (0x3A << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_OD_N                   (0x3B << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_AUDIO_SYNC_PULSE       (0x3C << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_AUDIO_SYNC_MISSED      (0x3D << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_INPUT                  (0x3E << DIO_CFG_IO_MODE_POS)
#define DIO_MODE_DISABLE                (0x3F << DIO_CFG_IO_MODE_POS)

/* Digital IOs Data Access Register */
#define DIO_DATA_BASE                   0x40000740
#define DIO_DATA                        REG32_POINTER(DIO_DATA_BASE)

/* DIO_DATA bit positions */
#define DIO_DATA_DIO_POS                0
#define DIO_DATA_DIO_MASK               (0xFFFF << DIO_DATA_DIO_POS)
#define DIO_DATA_GPIO_POS               0
#define DIO_DATA_GPIO_MASK              (0xFFFF << DIO_DATA_GPIO_POS)

/* DIO_DATA settings */
#define DIO0_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO1_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO2_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO3_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO4_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO5_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO6_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO7_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO8_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO9_LOW                        (0x0 << DIO_DATA_DIO_POS)
#define DIO10_LOW                       (0x0 << DIO_DATA_DIO_POS)
#define DIO11_LOW                       (0x0 << DIO_DATA_DIO_POS)
#define DIO12_LOW                       (0x0 << DIO_DATA_DIO_POS)
#define DIO13_LOW                       (0x0 << DIO_DATA_DIO_POS)
#define DIO14_LOW                       (0x0 << DIO_DATA_DIO_POS)
#define DIO15_LOW                       (0x0 << DIO_DATA_DIO_POS)
#define DIO0_HIGH                       (0x1 << DIO_DATA_DIO_POS)
#define DIO1_HIGH                       (0x2 << DIO_DATA_DIO_POS)
#define DIO2_HIGH                       (0x4 << DIO_DATA_DIO_POS)
#define DIO3_HIGH                       (0x8 << DIO_DATA_DIO_POS)
#define DIO4_HIGH                       (0x10 << DIO_DATA_DIO_POS)
#define DIO5_HIGH                       (0x20 << DIO_DATA_DIO_POS)
#define DIO6_HIGH                       (0x40 << DIO_DATA_DIO_POS)
#define DIO7_HIGH                       (0x80 << DIO_DATA_DIO_POS)
#define DIO8_HIGH                       (0x100 << DIO_DATA_DIO_POS)
#define DIO9_HIGH                       (0x200 << DIO_DATA_DIO_POS)
#define DIO10_HIGH                      (0x400 << DIO_DATA_DIO_POS)
#define DIO11_HIGH                      (0x800 << DIO_DATA_DIO_POS)
#define DIO12_HIGH                      (0x1000 << DIO_DATA_DIO_POS)
#define DIO13_HIGH                      (0x2000 << DIO_DATA_DIO_POS)
#define DIO14_HIGH                      (0x4000 << DIO_DATA_DIO_POS)
#define DIO15_HIGH                      (0x8000 << DIO_DATA_DIO_POS)

#define GPIO0_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO1_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO2_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO3_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO4_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO5_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO6_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO7_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO8_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO9_LOW                       (0x0 << DIO_DATA_GPIO_POS)
#define GPIO10_LOW                      (0x0 << DIO_DATA_GPIO_POS)
#define GPIO11_LOW                      (0x0 << DIO_DATA_GPIO_POS)
#define GPIO12_LOW                      (0x0 << DIO_DATA_GPIO_POS)
#define GPIO13_LOW                      (0x0 << DIO_DATA_GPIO_POS)
#define GPIO14_LOW                      (0x0 << DIO_DATA_GPIO_POS)
#define GPIO15_LOW                      (0x0 << DIO_DATA_GPIO_POS)
#define GPIO0_HIGH                      (0x1 << DIO_DATA_GPIO_POS)
#define GPIO1_HIGH                      (0x2 << DIO_DATA_GPIO_POS)
#define GPIO2_HIGH                      (0x4 << DIO_DATA_GPIO_POS)
#define GPIO3_HIGH                      (0x8 << DIO_DATA_GPIO_POS)
#define GPIO4_HIGH                      (0x10 << DIO_DATA_GPIO_POS)
#define GPIO5_HIGH                      (0x20 << DIO_DATA_GPIO_POS)
#define GPIO6_HIGH                      (0x40 << DIO_DATA_GPIO_POS)
#define GPIO7_HIGH                      (0x80 << DIO_DATA_GPIO_POS)
#define GPIO8_HIGH                      (0x100 << DIO_DATA_GPIO_POS)
#define GPIO9_HIGH                      (0x200 << DIO_DATA_GPIO_POS)
#define GPIO10_HIGH                     (0x400 << DIO_DATA_GPIO_POS)
#define GPIO11_HIGH                     (0x800 << DIO_DATA_GPIO_POS)
#define GPIO12_HIGH                     (0x1000 << DIO_DATA_GPIO_POS)
#define GPIO13_HIGH                     (0x2000 << DIO_DATA_GPIO_POS)
#define GPIO14_HIGH                     (0x4000 << DIO_DATA_GPIO_POS)
#define GPIO15_HIGH                     (0x8000 << DIO_DATA_GPIO_POS)

/* Digital IOs Direction State */
#define DIO_DIR_BASE                    0x40000744
#define DIO_DIR                         REG32_POINTER(DIO_DIR_BASE)

/* DIO_DIR bit positions */
#define DIO_DIR_DIO_POS                 0
#define DIO_DIR_DIO_MASK                (0xFFFF << DIO_DIR_DIO_POS)
#define DIO_DIR_GPIO_POS                0
#define DIO_DIR_GPIO_MASK               (0xFFFF << DIO_DIR_GPIO_POS)

/* DIO_DIR settings */
#define DIO0_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO1_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO2_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO3_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO4_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO5_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO6_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO7_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO8_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO9_INPUT                      (0x0 << DIO_DIR_DIO_POS)
#define DIO10_INPUT                     (0x0 << DIO_DIR_DIO_POS)
#define DIO11_INPUT                     (0x0 << DIO_DIR_DIO_POS)
#define DIO12_INPUT                     (0x0 << DIO_DIR_DIO_POS)
#define DIO13_INPUT                     (0x0 << DIO_DIR_DIO_POS)
#define DIO14_INPUT                     (0x0 << DIO_DIR_DIO_POS)
#define DIO15_INPUT                     (0x0 << DIO_DIR_DIO_POS)
#define DIO0_OUTPUT                     (0x1 << DIO_DIR_DIO_POS)
#define DIO1_OUTPUT                     (0x2 << DIO_DIR_DIO_POS)
#define DIO2_OUTPUT                     (0x4 << DIO_DIR_DIO_POS)
#define DIO3_OUTPUT                     (0x8 << DIO_DIR_DIO_POS)
#define DIO4_OUTPUT                     (0x10 << DIO_DIR_DIO_POS)
#define DIO5_OUTPUT                     (0x20 << DIO_DIR_DIO_POS)
#define DIO6_OUTPUT                     (0x40 << DIO_DIR_DIO_POS)
#define DIO7_OUTPUT                     (0x80 << DIO_DIR_DIO_POS)
#define DIO8_OUTPUT                     (0x100 << DIO_DIR_DIO_POS)
#define DIO9_OUTPUT                     (0x200 << DIO_DIR_DIO_POS)
#define DIO10_OUTPUT                    (0x400 << DIO_DIR_DIO_POS)
#define DIO11_OUTPUT                    (0x800 << DIO_DIR_DIO_POS)
#define DIO12_OUTPUT                    (0x1000 << DIO_DIR_DIO_POS)
#define DIO13_OUTPUT                    (0x2000 << DIO_DIR_DIO_POS)
#define DIO14_OUTPUT                    (0x4000 << DIO_DIR_DIO_POS)
#define DIO15_OUTPUT                    (0x8000 << DIO_DIR_DIO_POS)

#define GPIO0_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO1_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO2_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO3_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO4_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO5_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO6_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO7_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO8_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO9_INPUT                     (0x0 << DIO_DIR_GPIO_POS)
#define GPIO10_INPUT                    (0x0 << DIO_DIR_GPIO_POS)
#define GPIO11_INPUT                    (0x0 << DIO_DIR_GPIO_POS)
#define GPIO12_INPUT                    (0x0 << DIO_DIR_GPIO_POS)
#define GPIO13_INPUT                    (0x0 << DIO_DIR_GPIO_POS)
#define GPIO14_INPUT                    (0x0 << DIO_DIR_GPIO_POS)
#define GPIO15_INPUT                    (0x0 << DIO_DIR_GPIO_POS)
#define GPIO0_OUTPUT                    (0x1 << DIO_DIR_GPIO_POS)
#define GPIO1_OUTPUT                    (0x2 << DIO_DIR_GPIO_POS)
#define GPIO2_OUTPUT                    (0x4 << DIO_DIR_GPIO_POS)
#define GPIO3_OUTPUT                    (0x8 << DIO_DIR_GPIO_POS)
#define GPIO4_OUTPUT                    (0x10 << DIO_DIR_GPIO_POS)
#define GPIO5_OUTPUT                    (0x20 << DIO_DIR_GPIO_POS)
#define GPIO6_OUTPUT                    (0x40 << DIO_DIR_GPIO_POS)
#define GPIO7_OUTPUT                    (0x80 << DIO_DIR_GPIO_POS)
#define GPIO8_OUTPUT                    (0x100 << DIO_DIR_GPIO_POS)
#define GPIO9_OUTPUT                    (0x200 << DIO_DIR_GPIO_POS)
#define GPIO10_OUTPUT                   (0x400 << DIO_DIR_GPIO_POS)
#define GPIO11_OUTPUT                   (0x800 << DIO_DIR_GPIO_POS)
#define GPIO12_OUTPUT                   (0x1000 << DIO_DIR_GPIO_POS)
#define GPIO13_OUTPUT                   (0x2000 << DIO_DIR_GPIO_POS)
#define GPIO14_OUTPUT                   (0x4000 << DIO_DIR_GPIO_POS)
#define GPIO15_OUTPUT                   (0x8000 << DIO_DIR_GPIO_POS)

/* Digital IOs Mode State */
#define DIO_MODE_BASE                   0x40000748
#define DIO_MODE                        READONLY_REG32_POINTER(DIO_MODE_BASE)

/* DIO_MODE bit positions */
#define DIO_MODE_GPIO_POS               0
#define DIO_MODE_GPIO_MASK              (0xFFFF << DIO_MODE_GPIO_POS)

/* DIO_MODE settings */
#define DIO0_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO1_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO2_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO3_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO4_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO5_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO6_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO7_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO8_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO9_IS_NOT_GPIO                (0x0 << DIO_MODE_GPIO_POS)
#define DIO10_IS_NOT_GPIO               (0x0 << DIO_MODE_GPIO_POS)
#define DIO11_IS_NOT_GPIO               (0x0 << DIO_MODE_GPIO_POS)
#define DIO12_IS_NOT_GPIO               (0x0 << DIO_MODE_GPIO_POS)
#define DIO13_IS_NOT_GPIO               (0x0 << DIO_MODE_GPIO_POS)
#define DIO14_IS_NOT_GPIO               (0x0 << DIO_MODE_GPIO_POS)
#define DIO15_IS_NOT_GPIO               (0x0 << DIO_MODE_GPIO_POS)
#define DIO0_IS_GPIO                    (0x1 << DIO_MODE_GPIO_POS)
#define DIO1_IS_GPIO                    (0x2 << DIO_MODE_GPIO_POS)
#define DIO2_IS_GPIO                    (0x4 << DIO_MODE_GPIO_POS)
#define DIO3_IS_GPIO                    (0x8 << DIO_MODE_GPIO_POS)
#define DIO4_IS_GPIO                    (0x10 << DIO_MODE_GPIO_POS)
#define DIO5_IS_GPIO                    (0x20 << DIO_MODE_GPIO_POS)
#define DIO6_IS_GPIO                    (0x40 << DIO_MODE_GPIO_POS)
#define DIO7_IS_GPIO                    (0x80 << DIO_MODE_GPIO_POS)
#define DIO8_IS_GPIO                    (0x100 << DIO_MODE_GPIO_POS)
#define DIO9_IS_GPIO                    (0x200 << DIO_MODE_GPIO_POS)
#define DIO10_IS_GPIO                   (0x400 << DIO_MODE_GPIO_POS)
#define DIO11_IS_GPIO                   (0x800 << DIO_MODE_GPIO_POS)
#define DIO12_IS_GPIO                   (0x1000 << DIO_MODE_GPIO_POS)
#define DIO13_IS_GPIO                   (0x2000 << DIO_MODE_GPIO_POS)
#define DIO14_IS_GPIO                   (0x4000 << DIO_MODE_GPIO_POS)
#define DIO15_IS_GPIO                   (0x8000 << DIO_MODE_GPIO_POS)

/* DIO Interrupt Configuration */
#define DIO_INT_CFG_BASE                0x4000074C
#define DIO_INT_CFG                     REG32_POINTER(DIO_INT_CFG_BASE)

/* DIO_INT_CFG bit positions */
#define DIO_INT_CFG_DEBOUNCE_ENABLE_POS 11
#define DIO_INT_CFG_EVENT_POS           8
#define DIO_INT_CFG_EVENT_MASK          (0x7 << DIO_INT_CFG_EVENT_POS)
#define DIO_INT_CFG_SRC_POS             0
#define DIO_INT_CFG_SRC_MASK            (0x1F << DIO_INT_CFG_SRC_POS)

/* DIO_INT_CFG bit-band aliases */
#define DIO_INT_CFG_DEBOUNCE_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIO_INT_CFG_BASE, DIO_INT_CFG_DEBOUNCE_ENABLE_POS))

/* DIO_INT_CFG settings */
#define DIO_DEBOUNCE_DISABLE_BITBAND    0x0
#define DIO_DEBOUNCE_ENABLE_BITBAND     0x1
#define DIO_DEBOUNCE_DISABLE            (DIO_DEBOUNCE_DISABLE_BITBAND << DIO_INT_CFG_DEBOUNCE_ENABLE_POS)
#define DIO_DEBOUNCE_ENABLE             (DIO_DEBOUNCE_ENABLE_BITBAND << DIO_INT_CFG_DEBOUNCE_ENABLE_POS)

#define DIO_EVENT_NONE                  (0x0 << DIO_INT_CFG_EVENT_POS)
#define DIO_EVENT_HIGH_LEVEL            (0x1 << DIO_INT_CFG_EVENT_POS)
#define DIO_EVENT_LOW_LEVEL             (0x2 << DIO_INT_CFG_EVENT_POS)
#define DIO_EVENT_RISING_EDGE           (0x3 << DIO_INT_CFG_EVENT_POS)
#define DIO_EVENT_FALLING_EDGE          (0x4 << DIO_INT_CFG_EVENT_POS)
#define DIO_EVENT_TRANSITION            (0x5 << DIO_INT_CFG_EVENT_POS)

#define DIO_SRC_DIO_0                   (0x0 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_1                   (0x1 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_2                   (0x2 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_3                   (0x3 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_4                   (0x4 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_5                   (0x5 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_6                   (0x6 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_7                   (0x7 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_8                   (0x8 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_9                   (0x9 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_10                  (0xA << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_11                  (0xB << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_12                  (0xC << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_13                  (0xD << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_14                  (0xE << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_DIO_15                  (0xF << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_EXTCLK                  (0x10 << DIO_INT_CFG_SRC_POS)
#define DIO_SRC_WAKEUP                  (0x11 << DIO_INT_CFG_SRC_POS)

/* DIO Interrupt Button Debounce Filter Time Configuration */
#define DIO_INT_DEBOUNCE_BASE           0x4000075C
#define DIO_INT_DEBOUNCE                REG32_POINTER(DIO_INT_DEBOUNCE_BASE)

/* DIO_INT_DEBOUNCE bit positions */
#define DIO_INT_DEBOUNCE_DEBOUNCE_CLK_POS 8
#define DIO_INT_DEBOUNCE_DEBOUNCE_COUNT_POS 0
#define DIO_INT_DEBOUNCE_DEBOUNCE_COUNT_MASK (0xFF << DIO_INT_DEBOUNCE_DEBOUNCE_COUNT_POS)

/* DIO_INT_DEBOUNCE bit-band aliases */
#define DIO_INT_DEBOUNCE_DEBOUNCE_CLK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIO_INT_DEBOUNCE_BASE, DIO_INT_DEBOUNCE_DEBOUNCE_CLK_POS))

/* DIO_INT_DEBOUNCE settings */
#define DIO_DEBOUNCE_SLOWCLK_DIV32_BITBAND 0x0
#define DIO_DEBOUNCE_SLOWCLK_DIV1024_BITBAND 0x1
#define DIO_DEBOUNCE_SLOWCLK_DIV32      (DIO_DEBOUNCE_SLOWCLK_DIV32_BITBAND << DIO_INT_DEBOUNCE_DEBOUNCE_CLK_POS)
#define DIO_DEBOUNCE_SLOWCLK_DIV1024    (DIO_DEBOUNCE_SLOWCLK_DIV1024_BITBAND << DIO_INT_DEBOUNCE_DEBOUNCE_CLK_POS)

/* PCM Input Selection */
#define DIO_PCM_SRC_BASE                0x40000760
#define DIO_PCM_SRC                     REG32_POINTER(DIO_PCM_SRC_BASE)

/* DIO_PCM_SRC bit positions */
#define DIO_PCM_SRC_SERI_POS            16
#define DIO_PCM_SRC_SERI_MASK           (0x1F << DIO_PCM_SRC_SERI_POS)
#define DIO_PCM_SRC_FRAME_POS           8
#define DIO_PCM_SRC_FRAME_MASK          (0x1F << DIO_PCM_SRC_FRAME_POS)
#define DIO_PCM_SRC_CLK_POS             0
#define DIO_PCM_SRC_CLK_MASK            (0x1F << DIO_PCM_SRC_CLK_POS)

/* DIO_PCM_SRC settings */
#define PCM_SERI_SRC_DIO_0              (0x0 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_1              (0x1 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_2              (0x2 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_3              (0x3 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_4              (0x4 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_5              (0x5 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_6              (0x6 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_7              (0x7 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_8              (0x8 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_9              (0x9 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_10             (0xA << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_11             (0xB << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_12             (0xC << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_13             (0xD << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_14             (0xE << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_DIO_15             (0xF << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_CONST_LOW          (0x10 << DIO_PCM_SRC_SERI_POS)
#define PCM_SERI_SRC_CONST_HIGH         (0x11 << DIO_PCM_SRC_SERI_POS)

#define PCM_FRAME_SRC_DIO_0             (0x0 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_1             (0x1 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_2             (0x2 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_3             (0x3 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_4             (0x4 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_5             (0x5 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_6             (0x6 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_7             (0x7 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_8             (0x8 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_9             (0x9 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_10            (0xA << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_11            (0xB << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_12            (0xC << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_13            (0xD << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_14            (0xE << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_DIO_15            (0xF << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_CONST_LOW         (0x10 << DIO_PCM_SRC_FRAME_POS)
#define PCM_FRAME_SRC_CONST_HIGH        (0x11 << DIO_PCM_SRC_FRAME_POS)

#define PCM_CLK_SRC_DIO_0               (0x0 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_1               (0x1 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_2               (0x2 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_3               (0x3 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_4               (0x4 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_5               (0x5 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_6               (0x6 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_7               (0x7 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_8               (0x8 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_9               (0x9 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_10              (0xA << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_11              (0xB << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_12              (0xC << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_13              (0xD << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_14              (0xE << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_DIO_15              (0xF << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_CONST_LOW           (0x10 << DIO_PCM_SRC_CLK_POS)
#define PCM_CLK_SRC_CONST_HIGH          (0x11 << DIO_PCM_SRC_CLK_POS)

/* SPI[1:0] Interface Input Selection */
#define DIO_SPI_SRC_BASE                0x40000764
#define DIO_SPI_SRC                     REG32_POINTER(DIO_SPI_SRC_BASE)

/* DIO_SPI_SRC bit positions */
#define DIO_SPI_SRC_SERI_POS            16
#define DIO_SPI_SRC_SERI_MASK           (0x1F << DIO_SPI_SRC_SERI_POS)
#define DIO_SPI_SRC_CS_POS              8
#define DIO_SPI_SRC_CS_MASK             (0x1F << DIO_SPI_SRC_CS_POS)
#define DIO_SPI_SRC_CLK_POS             0
#define DIO_SPI_SRC_CLK_MASK            (0x1F << DIO_SPI_SRC_CLK_POS)

/* DIO_SPI_SRC settings */
#define SPI_SERI_SRC_DIO_0              (0x0 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_1              (0x1 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_2              (0x2 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_3              (0x3 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_4              (0x4 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_5              (0x5 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_6              (0x6 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_7              (0x7 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_8              (0x8 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_9              (0x9 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_10             (0xA << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_11             (0xB << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_12             (0xC << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_13             (0xD << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_14             (0xE << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_DIO_15             (0xF << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_CONST_LOW          (0x10 << DIO_SPI_SRC_SERI_POS)
#define SPI_SERI_SRC_CONST_HIGH         (0x11 << DIO_SPI_SRC_SERI_POS)

#define SPI_CS_SRC_DIO_0                (0x0 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_1                (0x1 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_2                (0x2 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_3                (0x3 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_4                (0x4 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_5                (0x5 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_6                (0x6 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_7                (0x7 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_8                (0x8 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_9                (0x9 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_10               (0xA << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_11               (0xB << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_12               (0xC << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_13               (0xD << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_14               (0xE << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_DIO_15               (0xF << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_CONST_LOW            (0x10 << DIO_SPI_SRC_CS_POS)
#define SPI_CS_SRC_CONST_HIGH           (0x11 << DIO_SPI_SRC_CS_POS)

#define SPI_CLK_SRC_DIO_0               (0x0 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_1               (0x1 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_2               (0x2 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_3               (0x3 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_4               (0x4 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_5               (0x5 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_6               (0x6 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_7               (0x7 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_8               (0x8 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_9               (0x9 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_10              (0xA << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_11              (0xB << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_12              (0xC << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_13              (0xD << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_14              (0xE << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_DIO_15              (0xF << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_CONST_LOW           (0x10 << DIO_SPI_SRC_CLK_POS)
#define SPI_CLK_SRC_CONST_HIGH          (0x11 << DIO_SPI_SRC_CLK_POS)

/* UART Interface Input Selection */
#define DIO_UART_SRC_BASE               0x4000076C
#define DIO_UART_SRC                    REG32_POINTER(DIO_UART_SRC_BASE)

/* DIO_UART_SRC bit positions */
#define DIO_UART_SRC_RX_POS             0
#define DIO_UART_SRC_RX_MASK            (0x1F << DIO_UART_SRC_RX_POS)

/* DIO_UART_SRC settings */
#define UART_RX_SRC_DIO_0               (0x0 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_1               (0x1 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_2               (0x2 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_3               (0x3 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_4               (0x4 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_5               (0x5 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_6               (0x6 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_7               (0x7 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_8               (0x8 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_9               (0x9 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_10              (0xA << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_11              (0xB << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_12              (0xC << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_13              (0xD << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_14              (0xE << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_DIO_15              (0xF << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_CONST_LOW           (0x10 << DIO_UART_SRC_RX_POS)
#define UART_RX_SRC_CONST_HIGH          (0x11 << DIO_UART_SRC_RX_POS)

/* I2C Input Selection */
#define DIO_I2C_SRC_BASE                0x40000770
#define DIO_I2C_SRC                     REG32_POINTER(DIO_I2C_SRC_BASE)

/* DIO_I2C_SRC bit positions */
#define DIO_I2C_SRC_SDA_POS             8
#define DIO_I2C_SRC_SDA_MASK            (0x1F << DIO_I2C_SRC_SDA_POS)
#define DIO_I2C_SRC_SCL_POS             0
#define DIO_I2C_SRC_SCL_MASK            (0x1F << DIO_I2C_SRC_SCL_POS)

/* DIO_I2C_SRC settings */
#define SDA_SRC_DIO_0                   (0x0 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_1                   (0x1 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_2                   (0x2 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_3                   (0x3 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_4                   (0x4 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_5                   (0x5 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_6                   (0x6 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_7                   (0x7 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_8                   (0x8 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_9                   (0x9 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_10                  (0xA << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_11                  (0xB << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_12                  (0xC << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_13                  (0xD << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_14                  (0xE << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_DIO_15                  (0xF << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_CONST_LOW               (0x10 << DIO_I2C_SRC_SDA_POS)
#define SDA_SRC_CONST_HIGH              (0x11 << DIO_I2C_SRC_SDA_POS)

#define SCL_SRC_DIO_0                   (0x0 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_1                   (0x1 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_2                   (0x2 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_3                   (0x3 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_4                   (0x4 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_5                   (0x5 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_6                   (0x6 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_7                   (0x7 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_8                   (0x8 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_9                   (0x9 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_10                  (0xA << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_11                  (0xB << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_12                  (0xC << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_13                  (0xD << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_14                  (0xE << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_DIO_15                  (0xF << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_CONST_LOW               (0x10 << DIO_I2C_SRC_SCL_POS)
#define SCL_SRC_CONST_HIGH              (0x11 << DIO_I2C_SRC_SCL_POS)

/* Audio Sink Input Selection */
#define DIO_AUDIOSINK_SRC_BASE          0x40000774
#define DIO_AUDIOSINK_SRC               REG32_POINTER(DIO_AUDIOSINK_SRC_BASE)

/* DIO_AUDIOSINK_SRC bit positions */
#define DIO_AUDIOSINK_SRC_CLK_POS       0
#define DIO_AUDIOSINK_SRC_CLK_MASK      (0x1F << DIO_AUDIOSINK_SRC_CLK_POS)

/* DIO_AUDIOSINK_SRC settings */
#define AUDIOSINK_CLK_SRC_DIO_0         (0x0 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_1         (0x1 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_2         (0x2 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_3         (0x3 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_4         (0x4 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_5         (0x5 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_6         (0x6 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_7         (0x7 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_8         (0x8 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_9         (0x9 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_10        (0xA << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_11        (0xB << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_12        (0xC << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_13        (0xD << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_14        (0xE << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DIO_15        (0xF << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_CONST_LOW     (0x10 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_CONST_HIGH    (0x11 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_STANDBYCLK    (0x12 << DIO_AUDIOSINK_SRC_CLK_POS)
#define AUDIOSINK_CLK_SRC_DMIC_OD       (0x13 << DIO_AUDIOSINK_SRC_CLK_POS)

/* NMI Input Selection */
#define DIO_NMI_SRC_BASE                0x40000778
#define DIO_NMI_SRC                     REG32_POINTER(DIO_NMI_SRC_BASE)

/* DIO_NMI_SRC bit positions */
#define DIO_NMI_SRC_NMI_POLARITY_POS    5
#define DIO_NMI_SRC_NMI_POS             0
#define DIO_NMI_SRC_NMI_MASK            (0x1F << DIO_NMI_SRC_NMI_POS)

/* DIO_NMI_SRC bit-band aliases */
#define DIO_NMI_SRC_NMI_POLARITY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIO_NMI_SRC_BASE, DIO_NMI_SRC_NMI_POLARITY_POS))

/* DIO_NMI_SRC settings */
#define NMI_ACTIVE_LOW_BITBAND          0x0
#define NMI_ACTIVE_HIGH_BITBAND         0x1
#define NMI_ACTIVE_LOW                  (NMI_ACTIVE_LOW_BITBAND << DIO_NMI_SRC_NMI_POLARITY_POS)
#define NMI_ACTIVE_HIGH                 (NMI_ACTIVE_HIGH_BITBAND << DIO_NMI_SRC_NMI_POLARITY_POS)

#define NMI_SRC_DIO_0                   (0x0 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_1                   (0x1 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_2                   (0x2 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_3                   (0x3 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_4                   (0x4 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_5                   (0x5 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_6                   (0x6 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_7                   (0x7 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_8                   (0x8 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_9                   (0x9 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_10                  (0xA << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_11                  (0xB << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_12                  (0xC << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_13                  (0xD << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_14                  (0xE << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_DIO_15                  (0xF << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_CONST_LOW               (0x10 << DIO_NMI_SRC_NMI_POS)
#define NMI_SRC_CONST_HIGH              (0x11 << DIO_NMI_SRC_NMI_POS)

/* Baseband Controller RX Data And Clock Input Selection */
#define DIO_BB_RX_SRC_BASE              0x4000077C
#define DIO_BB_RX_SRC                   REG32_POINTER(DIO_BB_RX_SRC_BASE)

/* DIO_BB_RX_SRC bit positions */
#define DIO_BB_RX_SRC_RF_SYNC_P_POS     16
#define DIO_BB_RX_SRC_RF_SYNC_P_MASK    (0x1F << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define DIO_BB_RX_SRC_CLK_POS           8
#define DIO_BB_RX_SRC_CLK_MASK          (0x1F << DIO_BB_RX_SRC_CLK_POS)
#define DIO_BB_RX_SRC_DATA_POS          0
#define DIO_BB_RX_SRC_DATA_MASK         (0x1F << DIO_BB_RX_SRC_DATA_POS)

/* DIO_BB_RX_SRC settings */
#define BB_RF_SYNC_P_SRC_DIO_0          (0x0 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_1          (0x1 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_2          (0x2 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_3          (0x3 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_4          (0x4 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_5          (0x5 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_6          (0x6 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_7          (0x7 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_8          (0x8 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_9          (0x9 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_10         (0xA << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_11         (0xB << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_12         (0xC << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_13         (0xD << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_14         (0xE << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_DIO_15         (0xF << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_CONST_LOW      (0x10 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_CONST_HIGH     (0x11 << DIO_BB_RX_SRC_RF_SYNC_P_POS)
#define BB_RF_SYNC_P_SRC_RF             (0x12 << DIO_BB_RX_SRC_RF_SYNC_P_POS)

#define BB_RX_CLK_SRC_DIO_0             (0x0 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_1             (0x1 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_2             (0x2 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_3             (0x3 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_4             (0x4 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_5             (0x5 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_6             (0x6 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_7             (0x7 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_8             (0x8 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_9             (0x9 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_10            (0xA << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_11            (0xB << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_12            (0xC << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_13            (0xD << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_14            (0xE << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_DIO_15            (0xF << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_CONST_LOW         (0x10 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_CONST_HIGH        (0x11 << DIO_BB_RX_SRC_CLK_POS)
#define BB_RX_CLK_SRC_RF_GPIO1          (0x12 << DIO_BB_RX_SRC_CLK_POS)

#define BB_RX_DATA_SRC_DIO_0            (0x0 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_1            (0x1 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_2            (0x2 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_3            (0x3 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_4            (0x4 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_5            (0x5 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_6            (0x6 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_7            (0x7 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_8            (0x8 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_9            (0x9 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_10           (0xA << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_11           (0xB << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_12           (0xC << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_13           (0xD << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_14           (0xE << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_DIO_15           (0xF << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_CONST_LOW        (0x10 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_CONST_HIGH       (0x11 << DIO_BB_RX_SRC_DATA_POS)
#define BB_RX_DATA_SRC_RF_GPIO0         (0x12 << DIO_BB_RX_SRC_DATA_POS)

/* Baseband Controller SPI Input Selection */
#define DIO_BB_SPI_SRC_BASE             0x40000780
#define DIO_BB_SPI_SRC                  REG32_POINTER(DIO_BB_SPI_SRC_BASE)

/* DIO_BB_SPI_SRC bit positions */
#define DIO_BB_SPI_SRC_MISO_POS         0
#define DIO_BB_SPI_SRC_MISO_MASK        (0x1F << DIO_BB_SPI_SRC_MISO_POS)

/* DIO_BB_SPI_SRC settings */
#define BB_SPI_MISO_SRC_DIO_0           (0x0 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_1           (0x1 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_2           (0x2 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_3           (0x3 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_4           (0x4 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_5           (0x5 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_6           (0x6 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_7           (0x7 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_8           (0x8 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_9           (0x9 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_10          (0xA << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_11          (0xB << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_12          (0xC << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_13          (0xD << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_14          (0xE << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_DIO_15          (0xF << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_CONST_LOW       (0x10 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_CONST_HIGH      (0x11 << DIO_BB_SPI_SRC_MISO_POS)
#define BB_SPI_MISO_SRC_RF_SPI_MISO     (0x12 << DIO_BB_SPI_SRC_MISO_POS)

/* RF Front-End SPI Input Selection */
#define DIO_RF_SPI_SRC_BASE             0x40000784
#define DIO_RF_SPI_SRC                  REG32_POINTER(DIO_RF_SPI_SRC_BASE)

/* DIO_RF_SPI_SRC bit positions */
#define DIO_RF_SPI_SRC_MOSI_POS         16
#define DIO_RF_SPI_SRC_MOSI_MASK        (0x1F << DIO_RF_SPI_SRC_MOSI_POS)
#define DIO_RF_SPI_SRC_CSN_POS          8
#define DIO_RF_SPI_SRC_CSN_MASK         (0x1F << DIO_RF_SPI_SRC_CSN_POS)
#define DIO_RF_SPI_SRC_CLK_POS          0
#define DIO_RF_SPI_SRC_CLK_MASK         (0x1F << DIO_RF_SPI_SRC_CLK_POS)

/* DIO_RF_SPI_SRC settings */
#define RF_SPI_MOSI_SRC_DIO_0           (0x0 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_1           (0x1 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_2           (0x2 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_3           (0x3 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_4           (0x4 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_5           (0x5 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_6           (0x6 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_7           (0x7 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_8           (0x8 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_9           (0x9 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_10          (0xA << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_11          (0xB << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_12          (0xC << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_13          (0xD << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_14          (0xE << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_DIO_15          (0xF << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_CONST_LOW       (0x10 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_CONST_HIGH      (0x11 << DIO_RF_SPI_SRC_MOSI_POS)
#define RF_SPI_MOSI_SRC_BB_SPI_MOSI     (0x12 << DIO_RF_SPI_SRC_MOSI_POS)

#define RF_SPI_CSN_SRC_DIO_0            (0x0 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_1            (0x1 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_2            (0x2 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_3            (0x3 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_4            (0x4 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_5            (0x5 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_6            (0x6 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_7            (0x7 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_8            (0x8 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_9            (0x9 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_10           (0xA << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_11           (0xB << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_12           (0xC << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_13           (0xD << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_14           (0xE << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_DIO_15           (0xF << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_CONST_LOW        (0x10 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_CONST_HIGH       (0x11 << DIO_RF_SPI_SRC_CSN_POS)
#define RF_SPI_CSN_SRC_BB_SPI_CSN       (0x12 << DIO_RF_SPI_SRC_CSN_POS)

#define RF_SPI_CLK_SRC_DIO_0            (0x0 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_1            (0x1 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_2            (0x2 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_3            (0x3 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_4            (0x4 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_5            (0x5 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_6            (0x6 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_7            (0x7 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_8            (0x8 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_9            (0x9 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_10           (0xA << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_11           (0xB << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_12           (0xC << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_13           (0xD << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_14           (0xE << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_DIO_15           (0xF << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_CONST_LOW        (0x10 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_CONST_HIGH       (0x11 << DIO_RF_SPI_SRC_CLK_POS)
#define RF_SPI_CLK_SRC_BB_SPI_CLK       (0x12 << DIO_RF_SPI_SRC_CLK_POS)

/* RF Front-End GPIOs 0-3 Input Selection */
#define DIO_RF_GPIO03_SRC_BASE          0x40000788
#define DIO_RF_GPIO03_SRC               REG32_POINTER(DIO_RF_GPIO03_SRC_BASE)

/* DIO_RF_GPIO03_SRC bit positions */
#define DIO_RF_GPIO03_SRC_GPIO3_POS     24
#define DIO_RF_GPIO03_SRC_GPIO3_MASK    (0x1F << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define DIO_RF_GPIO03_SRC_GPIO2_POS     16
#define DIO_RF_GPIO03_SRC_GPIO2_MASK    (0x1F << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define DIO_RF_GPIO03_SRC_GPIO1_POS     8
#define DIO_RF_GPIO03_SRC_GPIO1_MASK    (0x1F << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define DIO_RF_GPIO03_SRC_GPIO0_POS     0
#define DIO_RF_GPIO03_SRC_GPIO0_MASK    (0x1F << DIO_RF_GPIO03_SRC_GPIO0_POS)

/* DIO_RF_GPIO03_SRC settings */
#define RF_GPIO3_SRC_DIO_0              (0x0 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_1              (0x1 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_2              (0x2 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_3              (0x3 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_4              (0x4 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_5              (0x5 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_6              (0x6 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_7              (0x7 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_8              (0x8 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_9              (0x9 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_10             (0xA << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_11             (0xB << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_12             (0xC << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_13             (0xD << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_14             (0xE << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_DIO_15             (0xF << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO03_SRC_GPIO3_POS)
#define RF_GPIO3_SRC_BB_TX_DATA         (0x12 << DIO_RF_GPIO03_SRC_GPIO3_POS)

#define RF_GPIO2_SRC_DIO_0              (0x0 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_1              (0x1 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_2              (0x2 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_3              (0x3 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_4              (0x4 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_5              (0x5 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_6              (0x6 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_7              (0x7 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_8              (0x8 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_9              (0x9 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_10             (0xA << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_11             (0xB << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_12             (0xC << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_13             (0xD << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_14             (0xE << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_DIO_15             (0xF << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO03_SRC_GPIO2_POS)
#define RF_GPIO2_SRC_BB_SYNC_P          (0x12 << DIO_RF_GPIO03_SRC_GPIO2_POS)

#define RF_GPIO1_SRC_DIO_0              (0x0 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_1              (0x1 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_2              (0x2 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_3              (0x3 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_4              (0x4 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_5              (0x5 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_6              (0x6 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_7              (0x7 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_8              (0x8 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_9              (0x9 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_10             (0xA << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_11             (0xB << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_12             (0xC << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_13             (0xD << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_14             (0xE << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_DIO_15             (0xF << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO03_SRC_GPIO1_POS)
#define RF_GPIO1_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO03_SRC_GPIO1_POS)

#define RF_GPIO0_SRC_DIO_0              (0x0 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_1              (0x1 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_2              (0x2 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_3              (0x3 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_4              (0x4 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_5              (0x5 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_6              (0x6 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_7              (0x7 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_8              (0x8 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_9              (0x9 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_10             (0xA << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_11             (0xB << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_12             (0xC << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_13             (0xD << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_14             (0xE << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_DIO_15             (0xF << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO03_SRC_GPIO0_POS)
#define RF_GPIO0_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO03_SRC_GPIO0_POS)

/* RF Front-End GPIOs 4-7 Input Selection */
#define DIO_RF_GPIO47_SRC_BASE          0x4000078C
#define DIO_RF_GPIO47_SRC               REG32_POINTER(DIO_RF_GPIO47_SRC_BASE)

/* DIO_RF_GPIO47_SRC bit positions */
#define DIO_RF_GPIO47_SRC_GPIO7_POS     24
#define DIO_RF_GPIO47_SRC_GPIO7_MASK    (0x1F << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define DIO_RF_GPIO47_SRC_GPIO6_POS     16
#define DIO_RF_GPIO47_SRC_GPIO6_MASK    (0x1F << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define DIO_RF_GPIO47_SRC_GPIO5_POS     8
#define DIO_RF_GPIO47_SRC_GPIO5_MASK    (0x1F << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define DIO_RF_GPIO47_SRC_GPIO4_POS     0
#define DIO_RF_GPIO47_SRC_GPIO4_MASK    (0x1F << DIO_RF_GPIO47_SRC_GPIO4_POS)

/* DIO_RF_GPIO47_SRC settings */
#define RF_GPIO7_SRC_DIO_0              (0x0 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_1              (0x1 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_2              (0x2 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_3              (0x3 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_4              (0x4 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_5              (0x5 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_6              (0x6 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_7              (0x7 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_8              (0x8 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_9              (0x9 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_10             (0xA << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_11             (0xB << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_12             (0xC << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_13             (0xD << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_14             (0xE << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_DIO_15             (0xF << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO47_SRC_GPIO7_POS)
#define RF_GPIO7_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO47_SRC_GPIO7_POS)

#define RF_GPIO6_SRC_DIO_0              (0x0 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_1              (0x1 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_2              (0x2 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_3              (0x3 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_4              (0x4 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_5              (0x5 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_6              (0x6 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_7              (0x7 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_8              (0x8 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_9              (0x9 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_10             (0xA << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_11             (0xB << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_12             (0xC << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_13             (0xD << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_14             (0xE << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_DIO_15             (0xF << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO47_SRC_GPIO6_POS)
#define RF_GPIO6_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO47_SRC_GPIO6_POS)

#define RF_GPIO5_SRC_DIO_0              (0x0 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_1              (0x1 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_2              (0x2 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_3              (0x3 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_4              (0x4 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_5              (0x5 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_6              (0x6 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_7              (0x7 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_8              (0x8 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_9              (0x9 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_10             (0xA << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_11             (0xB << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_12             (0xC << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_13             (0xD << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_14             (0xE << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_DIO_15             (0xF << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO47_SRC_GPIO5_POS)
#define RF_GPIO5_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO47_SRC_GPIO5_POS)

#define RF_GPIO4_SRC_DIO_0              (0x0 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_1              (0x1 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_2              (0x2 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_3              (0x3 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_4              (0x4 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_5              (0x5 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_6              (0x6 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_7              (0x7 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_8              (0x8 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_9              (0x9 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_10             (0xA << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_11             (0xB << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_12             (0xC << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_13             (0xD << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_14             (0xE << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_DIO_15             (0xF << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO47_SRC_GPIO4_POS)
#define RF_GPIO4_SRC_BB_TX_DATA_VALID   (0x12 << DIO_RF_GPIO47_SRC_GPIO4_POS)

/* RF Front-End GPIOs 8-9 Input Selection */
#define DIO_RF_GPIO89_SRC_BASE          0x40000790
#define DIO_RF_GPIO89_SRC               REG32_POINTER(DIO_RF_GPIO89_SRC_BASE)

/* DIO_RF_GPIO89_SRC bit positions */
#define DIO_RF_GPIO89_SRC_GPIO9_POS     8
#define DIO_RF_GPIO89_SRC_GPIO9_MASK    (0x1F << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define DIO_RF_GPIO89_SRC_GPIO8_POS     0
#define DIO_RF_GPIO89_SRC_GPIO8_MASK    (0x1F << DIO_RF_GPIO89_SRC_GPIO8_POS)

/* DIO_RF_GPIO89_SRC settings */
#define RF_GPIO9_SRC_DIO_0              (0x0 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_1              (0x1 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_2              (0x2 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_3              (0x3 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_4              (0x4 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_5              (0x5 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_6              (0x6 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_7              (0x7 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_8              (0x8 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_9              (0x9 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_10             (0xA << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_11             (0xB << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_12             (0xC << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_13             (0xD << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_14             (0xE << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_DIO_15             (0xF << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO89_SRC_GPIO9_POS)
#define RF_GPIO9_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO89_SRC_GPIO9_POS)

#define RF_GPIO8_SRC_DIO_0              (0x0 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_1              (0x1 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_2              (0x2 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_3              (0x3 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_4              (0x4 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_5              (0x5 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_6              (0x6 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_7              (0x7 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_8              (0x8 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_9              (0x9 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_10             (0xA << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_11             (0xB << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_12             (0xC << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_13             (0xD << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_14             (0xE << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_DIO_15             (0xF << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_CONST_LOW          (0x10 << DIO_RF_GPIO89_SRC_GPIO8_POS)
#define RF_GPIO8_SRC_CONST_HIGH         (0x11 << DIO_RF_GPIO89_SRC_GPIO8_POS)

/* DMIC Data Input Selection Register */
#define DIO_DMIC_SRC_BASE               0x40000794
#define DIO_DMIC_SRC                    REG32_POINTER(DIO_DMIC_SRC_BASE)

/* DIO_DMIC_SRC bit positions */
#define DIO_DMIC_SRC_CLK_POS            8
#define DIO_DMIC_SRC_CLK_MASK           (0x1F << DIO_DMIC_SRC_CLK_POS)
#define DIO_DMIC_SRC_DATA_POS           0
#define DIO_DMIC_SRC_DATA_MASK          (0x1F << DIO_DMIC_SRC_DATA_POS)

/* DIO_DMIC_SRC settings */
#define DMIC_CLK_SRC_DIO_0              (0x0 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_1              (0x1 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_2              (0x2 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_3              (0x3 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_4              (0x4 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_5              (0x5 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_6              (0x6 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_7              (0x7 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_8              (0x8 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_9              (0x9 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_10             (0xA << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_11             (0xB << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_12             (0xC << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_13             (0xD << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_14             (0xE << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_DIO_15             (0xF << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_CONST_LOW          (0x10 << DIO_DMIC_SRC_CLK_POS)
#define DMIC_CLK_SRC_CONST_HIGH         (0x11 << DIO_DMIC_SRC_CLK_POS)

#define DMIC_DATA_SRC_DIO_0             (0x0 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_1             (0x1 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_2             (0x2 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_3             (0x3 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_4             (0x4 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_5             (0x5 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_6             (0x6 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_7             (0x7 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_8             (0x8 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_9             (0x9 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_10            (0xA << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_11            (0xB << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_12            (0xC << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_13            (0xD << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_14            (0xE << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_DIO_15            (0xF << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_CONST_LOW         (0x10 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_CONST_HIGH        (0x11 << DIO_DMIC_SRC_DATA_POS)
#define DMIC_DATA_SRC_EXTCLK            (0x12 << DIO_DMIC_SRC_DATA_POS)

/* LPDSP32 JTAG Configuration Register */
#define DIO_LPDSP32_JTAG_SRC_BASE       0x40000798
#define DIO_LPDSP32_JTAG_SRC            REG32_POINTER(DIO_LPDSP32_JTAG_SRC_BASE)

/* DIO_LPDSP32_JTAG_SRC bit positions */
#define DIO_LPDSP32_JTAG_SRC_TDI_POS    16
#define DIO_LPDSP32_JTAG_SRC_TDI_MASK   (0x1F << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define DIO_LPDSP32_JTAG_SRC_TMS_POS    8
#define DIO_LPDSP32_JTAG_SRC_TMS_MASK   (0x1F << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define DIO_LPDSP32_JTAG_SRC_TCK_POS    0
#define DIO_LPDSP32_JTAG_SRC_TCK_MASK   (0x1F << DIO_LPDSP32_JTAG_SRC_TCK_POS)

/* DIO_LPDSP32_JTAG_SRC settings */
#define LPDSP32_TDI_SRC_DIO_0           (0x0 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_1           (0x1 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_2           (0x2 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_3           (0x3 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_4           (0x4 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_5           (0x5 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_6           (0x6 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_7           (0x7 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_8           (0x8 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_9           (0x9 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_10          (0xA << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_11          (0xB << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_12          (0xC << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_13          (0xD << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_14          (0xE << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_DIO_15          (0xF << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_CONST_LOW       (0x10 << DIO_LPDSP32_JTAG_SRC_TDI_POS)
#define LPDSP32_TDI_SRC_CONST_HIGH      (0x11 << DIO_LPDSP32_JTAG_SRC_TDI_POS)

#define LPDSP32_TMS_SRC_DIO_0           (0x0 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_1           (0x1 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_2           (0x2 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_3           (0x3 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_4           (0x4 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_5           (0x5 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_6           (0x6 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_7           (0x7 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_8           (0x8 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_9           (0x9 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_10          (0xA << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_11          (0xB << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_12          (0xC << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_13          (0xD << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_14          (0xE << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_DIO_15          (0xF << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_CONST_LOW       (0x10 << DIO_LPDSP32_JTAG_SRC_TMS_POS)
#define LPDSP32_TMS_SRC_CONST_HIGH      (0x11 << DIO_LPDSP32_JTAG_SRC_TMS_POS)

#define LPDSP32_TCK_SRC_DIO_0           (0x0 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_1           (0x1 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_2           (0x2 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_3           (0x3 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_4           (0x4 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_5           (0x5 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_6           (0x6 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_7           (0x7 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_8           (0x8 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_9           (0x9 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_10          (0xA << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_11          (0xB << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_12          (0xC << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_13          (0xD << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_14          (0xE << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_DIO_15          (0xF << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_CONST_LOW       (0x10 << DIO_LPDSP32_JTAG_SRC_TCK_POS)
#define LPDSP32_TCK_SRC_CONST_HIGH      (0x11 << DIO_LPDSP32_JTAG_SRC_TCK_POS)

/* JTAG / SW Pad Configuration Register */
#define DIO_JTAG_SW_PAD_CFG_BASE        0x4000079C
#define DIO_JTAG_SW_PAD_CFG             REG32_POINTER(DIO_JTAG_SW_PAD_CFG_BASE)

/* DIO_JTAG_SW_PAD_CFG bit positions */
#define DIO_JTAG_SW_PAD_CFG_JTCK_LPF_POS 9
#define DIO_JTAG_SW_PAD_CFG_JTMS_LPF_POS 8
#define DIO_JTAG_SW_PAD_CFG_CM3_JTAG_DATA_EN_POS 7
#define DIO_JTAG_SW_PAD_CFG_CM3_JTAG_TRST_EN_POS 6
#define DIO_JTAG_SW_PAD_CFG_JTCK_PULL_POS 4
#define DIO_JTAG_SW_PAD_CFG_JTCK_PULL_MASK (0x3 << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_POS)
#define DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_POS 2
#define DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_MASK (0x3 << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_POS)
#define DIO_JTAG_SW_PAD_CFG_JTMS_PULL_POS 0
#define DIO_JTAG_SW_PAD_CFG_JTMS_PULL_MASK (0x3 << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_POS)

/* DIO_JTAG_SW_PAD_CFG bit-band aliases */
#define DIO_JTAG_SW_PAD_CFG_JTCK_LPF_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIO_JTAG_SW_PAD_CFG_BASE, DIO_JTAG_SW_PAD_CFG_JTCK_LPF_POS))
#define DIO_JTAG_SW_PAD_CFG_JTMS_LPF_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIO_JTAG_SW_PAD_CFG_BASE, DIO_JTAG_SW_PAD_CFG_JTMS_LPF_POS))
#define DIO_JTAG_SW_PAD_CFG_CM3_JTAG_DATA_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIO_JTAG_SW_PAD_CFG_BASE, DIO_JTAG_SW_PAD_CFG_CM3_JTAG_DATA_EN_POS))
#define DIO_JTAG_SW_PAD_CFG_CM3_JTAG_TRST_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(DIO_JTAG_SW_PAD_CFG_BASE, DIO_JTAG_SW_PAD_CFG_CM3_JTAG_TRST_EN_POS))

/* DIO_JTAG_SW_PAD_CFG settings */
#define JTCK_LPF_DISABLED_BITBAND       0x0
#define JTCK_LPF_ENABLED_BITBAND        0x1
#define JTCK_LPF_DISABLED               (JTCK_LPF_DISABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_JTCK_LPF_POS)
#define JTCK_LPF_ENABLED                (JTCK_LPF_ENABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_JTCK_LPF_POS)

#define JTMS_LPF_DISABLED_BITBAND       0x0
#define JTMS_LPF_ENABLED_BITBAND        0x1
#define JTMS_LPF_DISABLED               (JTMS_LPF_DISABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_JTMS_LPF_POS)
#define JTMS_LPF_ENABLED                (JTMS_LPF_ENABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_JTMS_LPF_POS)

#define CM3_JTAG_DATA_DISABLED_BITBAND  0x0
#define CM3_JTAG_DATA_ENABLED_BITBAND   0x1
#define CM3_JTAG_DATA_DISABLED          (CM3_JTAG_DATA_DISABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_CM3_JTAG_DATA_EN_POS)
#define CM3_JTAG_DATA_ENABLED           (CM3_JTAG_DATA_ENABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_CM3_JTAG_DATA_EN_POS)

#define CM3_JTAG_TRST_DISABLED_BITBAND  0x0
#define CM3_JTAG_TRST_ENABLED_BITBAND   0x1
#define CM3_JTAG_TRST_DISABLED          (CM3_JTAG_TRST_DISABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_CM3_JTAG_TRST_EN_POS)
#define CM3_JTAG_TRST_ENABLED           (CM3_JTAG_TRST_ENABLED_BITBAND << DIO_JTAG_SW_PAD_CFG_CM3_JTAG_TRST_EN_POS)

#define JTCK_NO_PULL                    (0x0 << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_POS)
#define JTCK_WEAK_PULL_UP               (0x1 << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_POS)
#define JTCK_WEAK_PULL_DOWN             (0x2 << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_POS)
#define JTCK_STRONG_PULL_UP             (0x3 << DIO_JTAG_SW_PAD_CFG_JTCK_PULL_POS)

#define JTMS_2X_DRIVE                   (0x0 << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_POS)
#define JTMS_3X_DRIVE                   (0x1 << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_POS)
#define JTMS_5X_DRIVE                   (0x2 << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_POS)
#define JTMS_6X_DRIVE                   (0x3 << DIO_JTAG_SW_PAD_CFG_JTMS_DRIVE_POS)

#define JTMS_NO_PULL                    (0x0 << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_POS)
#define JTMS_WEAK_PULL_UP               (0x1 << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_POS)
#define JTMS_WEAK_PULL_DOWN             (0x2 << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_POS)
#define JTMS_STRONG_PULL_UP             (0x3 << DIO_JTAG_SW_PAD_CFG_JTMS_PULL_POS)

/* External Clock Pad Configuration Register */
#define DIO_EXTCLK_CFG_BASE             0x400007A0
#define DIO_EXTCLK_CFG                  REG32_POINTER(DIO_EXTCLK_CFG_BASE)

/* DIO_EXTCLK_CFG bit positions */
#define DIO_EXTCLK_CFG_LPF_POS          2
#define DIO_EXTCLK_CFG_PULL_CTRL_POS    0
#define DIO_EXTCLK_CFG_PULL_CTRL_MASK   (0x3 << DIO_EXTCLK_CFG_PULL_CTRL_POS)

/* DIO_EXTCLK_CFG bit-band aliases */
#define DIO_EXTCLK_CFG_LPF_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(DIO_EXTCLK_CFG_BASE, DIO_EXTCLK_CFG_LPF_POS))

/* DIO_EXTCLK_CFG settings */
#define EXTCLK_LPF_DISABLE_BITBAND      0x0
#define EXTCLK_LPF_ENABLE_BITBAND       0x1
#define EXTCLK_LPF_DISABLE              (EXTCLK_LPF_DISABLE_BITBAND << DIO_EXTCLK_CFG_LPF_POS)
#define EXTCLK_LPF_ENABLE               (EXTCLK_LPF_ENABLE_BITBAND << DIO_EXTCLK_CFG_LPF_POS)

#define EXTCLK_NO_PULL                  (0x0 << DIO_EXTCLK_CFG_PULL_CTRL_POS)
#define EXTCLK_WEAK_PULL_UP             (0x1 << DIO_EXTCLK_CFG_PULL_CTRL_POS)
#define EXTCLK_WEAK_PULL_DOWN           (0x2 << DIO_EXTCLK_CFG_PULL_CTRL_POS)
#define EXTCLK_STRONG_PULL_UP           (0x3 << DIO_EXTCLK_CFG_PULL_CTRL_POS)

/* Global Pads Configuration Register */
#define DIO_PAD_CFG_BASE                0x400007A4
#define DIO_PAD_CFG                     REG32_POINTER(DIO_PAD_CFG_BASE)

/* DIO_PAD_CFG bit positions */
#define DIO_PAD_CFG_DRIVE_POS           0

/* DIO_PAD_CFG bit-band aliases */
#define DIO_PAD_CFG_DRIVE_BITBAND       REG32_POINTER(SYS_CALC_BITBAND(DIO_PAD_CFG_BASE, DIO_PAD_CFG_DRIVE_POS))

/* DIO_PAD_CFG settings */
#define PAD_LOW_DRIVE_BITBAND           0x0
#define PAD_HIGH_DRIVE_BITBAND          0x1
#define PAD_LOW_DRIVE                   (PAD_LOW_DRIVE_BITBAND << DIO_PAD_CFG_DRIVE_POS)
#define PAD_HIGH_DRIVE                  (PAD_HIGH_DRIVE_BITBAND << DIO_PAD_CFG_DRIVE_POS)

/* ----------------------------------------------------------------------------
 * SPI Interface Configuration and Control
 * ------------------------------------------------------------------------- */

/* SPI0 Control and Configuration Register */
#define SPI0_CTRL0_BASE                 0x40000800
#define SPI0_CTRL0                      REG32_POINTER(SPI0_CTRL0_BASE)

/* SPI0_CTRL0 bit positions */
#define SPI0_CTRL0_SPI0_OVERRUN_INT_ENABLE_POS 10
#define SPI0_CTRL0_SPI0_UNDERRUN_INT_ENABLE_POS 9
#define SPI0_CTRL0_SPI0_CONTROLLER_POS  8
#define SPI0_CTRL0_SPI0_SLAVE_POS       7
#define SPI0_CTRL0_SPI0_CLK_POLARITY_POS 6
#define SPI0_CTRL0_SPI0_MODE_SELECT_POS 5
#define SPI0_CTRL0_SPI0_ENABLE_POS      4
#define SPI0_CTRL0_SPI0_PRESCALE_POS    0
#define SPI0_CTRL0_SPI0_PRESCALE_MASK   (0xF << SPI0_CTRL0_SPI0_PRESCALE_POS)

/* SPI0_CTRL0 bit-band aliases */
#define SPI0_CTRL0_SPI0_OVERRUN_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL0_BASE, SPI0_CTRL0_SPI0_OVERRUN_INT_ENABLE_POS))
#define SPI0_CTRL0_SPI0_UNDERRUN_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL0_BASE, SPI0_CTRL0_SPI0_UNDERRUN_INT_ENABLE_POS))
#define SPI0_CTRL0_SPI0_CONTROLLER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL0_BASE, SPI0_CTRL0_SPI0_CONTROLLER_POS))
#define SPI0_CTRL0_SPI0_SLAVE_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL0_BASE, SPI0_CTRL0_SPI0_SLAVE_POS))
#define SPI0_CTRL0_SPI0_CLK_POLARITY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL0_BASE, SPI0_CTRL0_SPI0_CLK_POLARITY_POS))
#define SPI0_CTRL0_SPI0_MODE_SELECT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL0_BASE, SPI0_CTRL0_SPI0_MODE_SELECT_POS))
#define SPI0_CTRL0_SPI0_ENABLE_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL0_BASE, SPI0_CTRL0_SPI0_ENABLE_POS))

/* SPI0_CTRL0 settings */
#define SPI0_OVERRUN_INT_DISABLE_BITBAND 0x0
#define SPI0_OVERRUN_INT_ENABLE_BITBAND 0x1
#define SPI0_OVERRUN_INT_DISABLE        (SPI0_OVERRUN_INT_DISABLE_BITBAND << SPI0_CTRL0_SPI0_OVERRUN_INT_ENABLE_POS)
#define SPI0_OVERRUN_INT_ENABLE         (SPI0_OVERRUN_INT_ENABLE_BITBAND << SPI0_CTRL0_SPI0_OVERRUN_INT_ENABLE_POS)

#define SPI0_UNDERRUN_INT_DISABLE_BITBAND 0x0
#define SPI0_UNDERRUN_INT_ENABLE_BITBAND 0x1
#define SPI0_UNDERRUN_INT_DISABLE       (SPI0_UNDERRUN_INT_DISABLE_BITBAND << SPI0_CTRL0_SPI0_UNDERRUN_INT_ENABLE_POS)
#define SPI0_UNDERRUN_INT_ENABLE        (SPI0_UNDERRUN_INT_ENABLE_BITBAND << SPI0_CTRL0_SPI0_UNDERRUN_INT_ENABLE_POS)

#define SPI0_CONTROLLER_CM3_BITBAND     0x0
#define SPI0_CONTROLLER_DMA_BITBAND     0x1
#define SPI0_CONTROLLER_CM3             (SPI0_CONTROLLER_CM3_BITBAND << SPI0_CTRL0_SPI0_CONTROLLER_POS)
#define SPI0_CONTROLLER_DMA             (SPI0_CONTROLLER_DMA_BITBAND << SPI0_CTRL0_SPI0_CONTROLLER_POS)

#define SPI0_SELECT_MASTER_BITBAND      0x0
#define SPI0_SELECT_SLAVE_BITBAND       0x1
#define SPI0_SELECT_MASTER              (SPI0_SELECT_MASTER_BITBAND << SPI0_CTRL0_SPI0_SLAVE_POS)
#define SPI0_SELECT_SLAVE               (SPI0_SELECT_SLAVE_BITBAND << SPI0_CTRL0_SPI0_SLAVE_POS)

#define SPI0_CLK_POLARITY_NORMAL_BITBAND 0x0
#define SPI0_CLK_POLARITY_INVERSE_BITBAND 0x1
#define SPI0_CLK_POLARITY_NORMAL        (SPI0_CLK_POLARITY_NORMAL_BITBAND << SPI0_CTRL0_SPI0_CLK_POLARITY_POS)
#define SPI0_CLK_POLARITY_INVERSE       (SPI0_CLK_POLARITY_INVERSE_BITBAND << SPI0_CTRL0_SPI0_CLK_POLARITY_POS)

#define SPI0_MODE_SELECT_MANUAL_BITBAND 0x0
#define SPI0_MODE_SELECT_AUTO_BITBAND   0x1
#define SPI0_MODE_SELECT_MANUAL         (SPI0_MODE_SELECT_MANUAL_BITBAND << SPI0_CTRL0_SPI0_MODE_SELECT_POS)
#define SPI0_MODE_SELECT_AUTO           (SPI0_MODE_SELECT_AUTO_BITBAND << SPI0_CTRL0_SPI0_MODE_SELECT_POS)

#define SPI0_DISABLE_BITBAND            0x0
#define SPI0_ENABLE_BITBAND             0x1
#define SPI0_DISABLE                    (SPI0_DISABLE_BITBAND << SPI0_CTRL0_SPI0_ENABLE_POS)
#define SPI0_ENABLE                     (SPI0_ENABLE_BITBAND << SPI0_CTRL0_SPI0_ENABLE_POS)

#define SPI0_PRESCALE_2                 (0x0 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_4                 (0x1 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_8                 (0x2 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_16                (0x3 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_32                (0x4 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_64                (0x5 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_128               (0x6 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_256               (0x7 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_512               (0x8 << SPI0_CTRL0_SPI0_PRESCALE_POS)
#define SPI0_PRESCALE_1024              (0x9 << SPI0_CTRL0_SPI0_PRESCALE_POS)

/* SPI0 Transaction Control Register */
#define SPI0_CTRL1_BASE                 0x40000804
#define SPI0_CTRL1                      REG32_POINTER(SPI0_CTRL1_BASE)

/* SPI0_CTRL1 bit positions */
#define SPI0_CTRL1_SPI0_START_BUSY_POS  8
#define SPI0_CTRL1_SPI0_BUSY_STATUS_POS 8
#define SPI0_CTRL1_SPI0_RW_CMD_POS      6
#define SPI0_CTRL1_SPI0_RW_CMD_MASK     (0x3 << SPI0_CTRL1_SPI0_RW_CMD_POS)
#define SPI0_CTRL1_SPI0_CS_POS          5
#define SPI0_CTRL1_SPI0_WORD_SIZE_POS   0
#define SPI0_CTRL1_SPI0_WORD_SIZE_MASK  (0x1F << SPI0_CTRL1_SPI0_WORD_SIZE_POS)

/* SPI0_CTRL1 bit-band aliases */
#define SPI0_CTRL1_SPI0_START_BUSY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL1_BASE, SPI0_CTRL1_SPI0_START_BUSY_POS))
#define SPI0_CTRL1_SPI0_BUSY_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL1_BASE, SPI0_CTRL1_SPI0_BUSY_STATUS_POS))
#define SPI0_CTRL1_SPI0_CS_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(SPI0_CTRL1_BASE, SPI0_CTRL1_SPI0_CS_POS))

/* SPI0_CTRL1 settings */
#define SPI0_IDLE_BITBAND               0x0
#define SPI0_START_BITBAND              0x1
#define SPI0_IDLE                       (SPI0_IDLE_BITBAND << SPI0_CTRL1_SPI0_START_BUSY_POS)
#define SPI0_START                      (SPI0_START_BITBAND << SPI0_CTRL1_SPI0_START_BUSY_POS)

#define SPI0_BUSY_BITBAND               0x1
#define SPI0_BUSY                       (SPI0_BUSY_BITBAND << SPI0_CTRL1_SPI0_BUSY_STATUS_POS)

#define SPI0_NOP                        (0x0 << SPI0_CTRL1_SPI0_RW_CMD_POS)
#define SPI0_WRITE_DATA                 (0x1 << SPI0_CTRL1_SPI0_RW_CMD_POS)
#define SPI0_READ_DATA                  (0x2 << SPI0_CTRL1_SPI0_RW_CMD_POS)
#define SPI0_RW_DATA                    (0x3 << SPI0_CTRL1_SPI0_RW_CMD_POS)

#define SPI0_CS_0_BITBAND               0x0
#define SPI0_CS_1_BITBAND               0x1
#define SPI0_CS_0                       (SPI0_CS_0_BITBAND << SPI0_CTRL1_SPI0_CS_POS)
#define SPI0_CS_1                       (SPI0_CS_1_BITBAND << SPI0_CTRL1_SPI0_CS_POS)

#define SPI0_WORD_SIZE_1                (0x0 << SPI0_CTRL1_SPI0_WORD_SIZE_POS)
#define SPI0_WORD_SIZE_8                (0x7 << SPI0_CTRL1_SPI0_WORD_SIZE_POS)
#define SPI0_WORD_SIZE_16               (0xF << SPI0_CTRL1_SPI0_WORD_SIZE_POS)
#define SPI0_WORD_SIZE_24               (0x17 << SPI0_CTRL1_SPI0_WORD_SIZE_POS)
#define SPI0_WORD_SIZE_32               (0x1F << SPI0_CTRL1_SPI0_WORD_SIZE_POS)

/* SPI0 Transmit Data */
#define SPI0_TX_DATA_BASE               0x40000808
#define SPI0_TX_DATA                    REG32_POINTER(SPI0_TX_DATA_BASE)

/* SPI0 Received Data */
#define SPI0_RX_DATA_BASE               0x4000080C
#define SPI0_RX_DATA                    READONLY_REG32_POINTER(SPI0_RX_DATA_BASE)

/* SPI0 Status */
#define SPI0_STATUS_BASE                0x40000810
#define SPI0_STATUS                     REG32_POINTER(SPI0_STATUS_BASE)

/* SPI0_STATUS bit positions */
#define SPI0_STATUS_SPI0_TRANSMIT_STATUS_POS 3
#define SPI0_STATUS_SPI0_RECEIVE_STATUS_POS 2
#define SPI0_STATUS_SPI0_OVERRUN_STATUS_POS 1
#define SPI0_STATUS_SPI0_UNDERRUN_STATUS_POS 0

/* SPI0_STATUS bit-band aliases */
#define SPI0_STATUS_SPI0_TRANSMIT_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_STATUS_BASE, SPI0_STATUS_SPI0_TRANSMIT_STATUS_POS))
#define SPI0_STATUS_SPI0_RECEIVE_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_STATUS_BASE, SPI0_STATUS_SPI0_RECEIVE_STATUS_POS))
#define SPI0_STATUS_SPI0_OVERRUN_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_STATUS_BASE, SPI0_STATUS_SPI0_OVERRUN_STATUS_POS))
#define SPI0_STATUS_SPI0_UNDERRUN_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI0_STATUS_BASE, SPI0_STATUS_SPI0_UNDERRUN_STATUS_POS))

/* SPI0_STATUS settings */
#define SPI0_TRANSMIT_FALSE_BITBAND     0x0
#define SPI0_TRANSMIT_TRUE_BITBAND      0x1
#define SPI0_TRANSMIT_CLEAR_BITBAND     0x1
#define SPI0_TRANSMIT_FALSE             (SPI0_TRANSMIT_FALSE_BITBAND << SPI0_STATUS_SPI0_TRANSMIT_STATUS_POS)
#define SPI0_TRANSMIT_TRUE              (SPI0_TRANSMIT_TRUE_BITBAND << SPI0_STATUS_SPI0_TRANSMIT_STATUS_POS)
#define SPI0_TRANSMIT_CLEAR             (SPI0_TRANSMIT_CLEAR_BITBAND << SPI0_STATUS_SPI0_TRANSMIT_STATUS_POS)

#define SPI0_RECEIVE_FALSE_BITBAND      0x0
#define SPI0_RECEIVE_TRUE_BITBAND       0x1
#define SPI0_RECEIVE_CLEAR_BITBAND      0x1
#define SPI0_RECEIVE_FALSE              (SPI0_RECEIVE_FALSE_BITBAND << SPI0_STATUS_SPI0_RECEIVE_STATUS_POS)
#define SPI0_RECEIVE_TRUE               (SPI0_RECEIVE_TRUE_BITBAND << SPI0_STATUS_SPI0_RECEIVE_STATUS_POS)
#define SPI0_RECEIVE_CLEAR              (SPI0_RECEIVE_CLEAR_BITBAND << SPI0_STATUS_SPI0_RECEIVE_STATUS_POS)

#define SPI0_OVERRUN_FALSE_BITBAND      0x0
#define SPI0_OVERRUN_TRUE_BITBAND       0x1
#define SPI0_OVERRUN_CLEAR_BITBAND      0x1
#define SPI0_OVERRUN_FALSE              (SPI0_OVERRUN_FALSE_BITBAND << SPI0_STATUS_SPI0_OVERRUN_STATUS_POS)
#define SPI0_OVERRUN_TRUE               (SPI0_OVERRUN_TRUE_BITBAND << SPI0_STATUS_SPI0_OVERRUN_STATUS_POS)
#define SPI0_OVERRUN_CLEAR              (SPI0_OVERRUN_CLEAR_BITBAND << SPI0_STATUS_SPI0_OVERRUN_STATUS_POS)

#define SPI0_UNDERRUN_FALSE_BITBAND     0x0
#define SPI0_UNDERRUN_TRUE_BITBAND      0x1
#define SPI0_UNDERRUN_CLEAR_BITBAND     0x1
#define SPI0_UNDERRUN_FALSE             (SPI0_UNDERRUN_FALSE_BITBAND << SPI0_STATUS_SPI0_UNDERRUN_STATUS_POS)
#define SPI0_UNDERRUN_TRUE              (SPI0_UNDERRUN_TRUE_BITBAND << SPI0_STATUS_SPI0_UNDERRUN_STATUS_POS)
#define SPI0_UNDERRUN_CLEAR             (SPI0_UNDERRUN_CLEAR_BITBAND << SPI0_STATUS_SPI0_UNDERRUN_STATUS_POS)

/* ----------------------------------------------------------------------------
 * SPI Interface Configuration and Control
 * ------------------------------------------------------------------------- */

/* SPI1 Control and Configuration Register */
#define SPI1_CTRL0_BASE                 0x40000900
#define SPI1_CTRL0                      REG32_POINTER(SPI1_CTRL0_BASE)

/* SPI1_CTRL0 bit positions */
#define SPI1_CTRL0_SPI1_OVERRUN_INT_ENABLE_POS 10
#define SPI1_CTRL0_SPI1_UNDERRUN_INT_ENABLE_POS 9
#define SPI1_CTRL0_SPI1_CONTROLLER_POS  8
#define SPI1_CTRL0_SPI1_SLAVE_POS       7
#define SPI1_CTRL0_SPI1_CLK_POLARITY_POS 6
#define SPI1_CTRL0_SPI1_MODE_SELECT_POS 5
#define SPI1_CTRL0_SPI1_ENABLE_POS      4
#define SPI1_CTRL0_SPI1_PRESCALE_POS    0
#define SPI1_CTRL0_SPI1_PRESCALE_MASK   (0xF << SPI1_CTRL0_SPI1_PRESCALE_POS)

/* SPI1_CTRL0 bit-band aliases */
#define SPI1_CTRL0_SPI1_OVERRUN_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL0_BASE, SPI1_CTRL0_SPI1_OVERRUN_INT_ENABLE_POS))
#define SPI1_CTRL0_SPI1_UNDERRUN_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL0_BASE, SPI1_CTRL0_SPI1_UNDERRUN_INT_ENABLE_POS))
#define SPI1_CTRL0_SPI1_CONTROLLER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL0_BASE, SPI1_CTRL0_SPI1_CONTROLLER_POS))
#define SPI1_CTRL0_SPI1_SLAVE_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL0_BASE, SPI1_CTRL0_SPI1_SLAVE_POS))
#define SPI1_CTRL0_SPI1_CLK_POLARITY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL0_BASE, SPI1_CTRL0_SPI1_CLK_POLARITY_POS))
#define SPI1_CTRL0_SPI1_MODE_SELECT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL0_BASE, SPI1_CTRL0_SPI1_MODE_SELECT_POS))
#define SPI1_CTRL0_SPI1_ENABLE_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL0_BASE, SPI1_CTRL0_SPI1_ENABLE_POS))

/* SPI1_CTRL0 settings */
#define SPI1_OVERRUN_INT_DISABLE_BITBAND 0x0
#define SPI1_OVERRUN_INT_ENABLE_BITBAND 0x1
#define SPI1_OVERRUN_INT_DISABLE        (SPI1_OVERRUN_INT_DISABLE_BITBAND << SPI1_CTRL0_SPI1_OVERRUN_INT_ENABLE_POS)
#define SPI1_OVERRUN_INT_ENABLE         (SPI1_OVERRUN_INT_ENABLE_BITBAND << SPI1_CTRL0_SPI1_OVERRUN_INT_ENABLE_POS)

#define SPI1_UNDERRUN_INT_DISABLE_BITBAND 0x0
#define SPI1_UNDERRUN_INT_ENABLE_BITBAND 0x1
#define SPI1_UNDERRUN_INT_DISABLE       (SPI1_UNDERRUN_INT_DISABLE_BITBAND << SPI1_CTRL0_SPI1_UNDERRUN_INT_ENABLE_POS)
#define SPI1_UNDERRUN_INT_ENABLE        (SPI1_UNDERRUN_INT_ENABLE_BITBAND << SPI1_CTRL0_SPI1_UNDERRUN_INT_ENABLE_POS)

#define SPI1_CONTROLLER_CM3_BITBAND     0x0
#define SPI1_CONTROLLER_DMA_BITBAND     0x1
#define SPI1_CONTROLLER_CM3             (SPI1_CONTROLLER_CM3_BITBAND << SPI1_CTRL0_SPI1_CONTROLLER_POS)
#define SPI1_CONTROLLER_DMA             (SPI1_CONTROLLER_DMA_BITBAND << SPI1_CTRL0_SPI1_CONTROLLER_POS)

#define SPI1_SELECT_MASTER_BITBAND      0x0
#define SPI1_SELECT_SLAVE_BITBAND       0x1
#define SPI1_SELECT_MASTER              (SPI1_SELECT_MASTER_BITBAND << SPI1_CTRL0_SPI1_SLAVE_POS)
#define SPI1_SELECT_SLAVE               (SPI1_SELECT_SLAVE_BITBAND << SPI1_CTRL0_SPI1_SLAVE_POS)

#define SPI1_CLK_POLARITY_NORMAL_BITBAND 0x0
#define SPI1_CLK_POLARITY_INVERSE_BITBAND 0x1
#define SPI1_CLK_POLARITY_NORMAL        (SPI1_CLK_POLARITY_NORMAL_BITBAND << SPI1_CTRL0_SPI1_CLK_POLARITY_POS)
#define SPI1_CLK_POLARITY_INVERSE       (SPI1_CLK_POLARITY_INVERSE_BITBAND << SPI1_CTRL0_SPI1_CLK_POLARITY_POS)

#define SPI1_MODE_SELECT_MANUAL_BITBAND 0x0
#define SPI1_MODE_SELECT_AUTO_BITBAND   0x1
#define SPI1_MODE_SELECT_MANUAL         (SPI1_MODE_SELECT_MANUAL_BITBAND << SPI1_CTRL0_SPI1_MODE_SELECT_POS)
#define SPI1_MODE_SELECT_AUTO           (SPI1_MODE_SELECT_AUTO_BITBAND << SPI1_CTRL0_SPI1_MODE_SELECT_POS)

#define SPI1_DISABLE_BITBAND            0x0
#define SPI1_ENABLE_BITBAND             0x1
#define SPI1_DISABLE                    (SPI1_DISABLE_BITBAND << SPI1_CTRL0_SPI1_ENABLE_POS)
#define SPI1_ENABLE                     (SPI1_ENABLE_BITBAND << SPI1_CTRL0_SPI1_ENABLE_POS)

#define SPI1_PRESCALE_2                 (0x0 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_4                 (0x1 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_8                 (0x2 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_16                (0x3 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_32                (0x4 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_64                (0x5 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_128               (0x6 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_256               (0x7 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_512               (0x8 << SPI1_CTRL0_SPI1_PRESCALE_POS)
#define SPI1_PRESCALE_1024              (0x9 << SPI1_CTRL0_SPI1_PRESCALE_POS)

/* SPI1 Transaction Control Register */
#define SPI1_CTRL1_BASE                 0x40000904
#define SPI1_CTRL1                      REG32_POINTER(SPI1_CTRL1_BASE)

/* SPI1_CTRL1 bit positions */
#define SPI1_CTRL1_SPI1_START_BUSY_POS  8
#define SPI1_CTRL1_SPI1_RW_CMD_POS      6
#define SPI1_CTRL1_SPI1_RW_CMD_MASK     (0x3 << SPI1_CTRL1_SPI1_RW_CMD_POS)
#define SPI1_CTRL1_SPI1_CS_POS          5
#define SPI1_CTRL1_SPI1_WORD_SIZE_POS   0
#define SPI1_CTRL1_SPI1_WORD_SIZE_MASK  (0x1F << SPI1_CTRL1_SPI1_WORD_SIZE_POS)

/* SPI1_CTRL1 bit-band aliases */
#define SPI1_CTRL1_SPI1_START_BUSY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL1_BASE, SPI1_CTRL1_SPI1_START_BUSY_POS))
#define SPI1_CTRL1_SPI1_CS_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(SPI1_CTRL1_BASE, SPI1_CTRL1_SPI1_CS_POS))

/* SPI1_CTRL1 settings */
#define SPI1_IDLE_BITBAND               0x0
#define SPI1_START_BITBAND              0x1
#define SPI1_BUSY_BITBAND               0x1
#define SPI1_IDLE                       (SPI1_IDLE_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_POS)
#define SPI1_START                      (SPI1_START_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_POS)
#define SPI1_BUSY                       (SPI1_BUSY_BITBAND << SPI1_CTRL1_SPI1_START_BUSY_POS)

#define SPI1_NOP                        (0x0 << SPI1_CTRL1_SPI1_RW_CMD_POS)
#define SPI1_WRITE_DATA                 (0x1 << SPI1_CTRL1_SPI1_RW_CMD_POS)
#define SPI1_READ_DATA                  (0x2 << SPI1_CTRL1_SPI1_RW_CMD_POS)
#define SPI1_RW_DATA                    (0x3 << SPI1_CTRL1_SPI1_RW_CMD_POS)

#define SPI1_CS_0_BITBAND               0x0
#define SPI1_CS_1_BITBAND               0x1
#define SPI1_CS_0                       (SPI1_CS_0_BITBAND << SPI1_CTRL1_SPI1_CS_POS)
#define SPI1_CS_1                       (SPI1_CS_1_BITBAND << SPI1_CTRL1_SPI1_CS_POS)

#define SPI1_WORD_SIZE_1                (0x0 << SPI1_CTRL1_SPI1_WORD_SIZE_POS)
#define SPI1_WORD_SIZE_8                (0x7 << SPI1_CTRL1_SPI1_WORD_SIZE_POS)
#define SPI1_WORD_SIZE_16               (0xF << SPI1_CTRL1_SPI1_WORD_SIZE_POS)
#define SPI1_WORD_SIZE_24               (0x17 << SPI1_CTRL1_SPI1_WORD_SIZE_POS)
#define SPI1_WORD_SIZE_32               (0x1F << SPI1_CTRL1_SPI1_WORD_SIZE_POS)

/* SPI1 Transmit Data */
#define SPI1_TX_DATA_BASE               0x40000908
#define SPI1_TX_DATA                    REG32_POINTER(SPI1_TX_DATA_BASE)

/* SPI1 Received Data */
#define SPI1_RX_DATA_BASE               0x4000090C
#define SPI1_RX_DATA                    READONLY_REG32_POINTER(SPI1_RX_DATA_BASE)

/* SPI1 Status */
#define SPI1_STATUS_BASE                0x40000910
#define SPI1_STATUS                     REG32_POINTER(SPI1_STATUS_BASE)

/* SPI1_STATUS bit positions */
#define SPI1_STATUS_SPI1_TRANSMIT_STATUS_POS 3
#define SPI1_STATUS_SPI1_RECEIVE_STATUS_POS 2
#define SPI1_STATUS_SPI1_OVERRUN_STATUS_POS 1
#define SPI1_STATUS_SPI1_UNDERRUN_STATUS_POS 0

/* SPI1_STATUS bit-band aliases */
#define SPI1_STATUS_SPI1_TRANSMIT_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_STATUS_BASE, SPI1_STATUS_SPI1_TRANSMIT_STATUS_POS))
#define SPI1_STATUS_SPI1_RECEIVE_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_STATUS_BASE, SPI1_STATUS_SPI1_RECEIVE_STATUS_POS))
#define SPI1_STATUS_SPI1_OVERRUN_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_STATUS_BASE, SPI1_STATUS_SPI1_OVERRUN_STATUS_POS))
#define SPI1_STATUS_SPI1_UNDERRUN_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(SPI1_STATUS_BASE, SPI1_STATUS_SPI1_UNDERRUN_STATUS_POS))

/* SPI1_STATUS settings */
#define SPI1_TRANSMIT_FALSE_BITBAND     0x0
#define SPI1_TRANSMIT_TRUE_BITBAND      0x1
#define SPI1_TRANSMIT_CLEAR_BITBAND     0x1
#define SPI1_TRANSMIT_FALSE             (SPI1_TRANSMIT_FALSE_BITBAND << SPI1_STATUS_SPI1_TRANSMIT_STATUS_POS)
#define SPI1_TRANSMIT_TRUE              (SPI1_TRANSMIT_TRUE_BITBAND << SPI1_STATUS_SPI1_TRANSMIT_STATUS_POS)
#define SPI1_TRANSMIT_CLEAR             (SPI1_TRANSMIT_CLEAR_BITBAND << SPI1_STATUS_SPI1_TRANSMIT_STATUS_POS)

#define SPI1_RECEIVE_FALSE_BITBAND      0x0
#define SPI1_RECEIVE_TRUE_BITBAND       0x1
#define SPI1_RECEIVE_CLEAR_BITBAND      0x1
#define SPI1_RECEIVE_FALSE              (SPI1_RECEIVE_FALSE_BITBAND << SPI1_STATUS_SPI1_RECEIVE_STATUS_POS)
#define SPI1_RECEIVE_TRUE               (SPI1_RECEIVE_TRUE_BITBAND << SPI1_STATUS_SPI1_RECEIVE_STATUS_POS)
#define SPI1_RECEIVE_CLEAR              (SPI1_RECEIVE_CLEAR_BITBAND << SPI1_STATUS_SPI1_RECEIVE_STATUS_POS)

#define SPI1_OVERRUN_FALSE_BITBAND      0x0
#define SPI1_OVERRUN_TRUE_BITBAND       0x1
#define SPI1_OVERRUN_CLEAR_BITBAND      0x1
#define SPI1_OVERRUN_FALSE              (SPI1_OVERRUN_FALSE_BITBAND << SPI1_STATUS_SPI1_OVERRUN_STATUS_POS)
#define SPI1_OVERRUN_TRUE               (SPI1_OVERRUN_TRUE_BITBAND << SPI1_STATUS_SPI1_OVERRUN_STATUS_POS)
#define SPI1_OVERRUN_CLEAR              (SPI1_OVERRUN_CLEAR_BITBAND << SPI1_STATUS_SPI1_OVERRUN_STATUS_POS)

#define SPI1_UNDERRUN_FALSE_BITBAND     0x0
#define SPI1_UNDERRUN_TRUE_BITBAND      0x1
#define SPI1_UNDERRUN_CLEAR_BITBAND     0x1
#define SPI1_UNDERRUN_FALSE             (SPI1_UNDERRUN_FALSE_BITBAND << SPI1_STATUS_SPI1_UNDERRUN_STATUS_POS)
#define SPI1_UNDERRUN_TRUE              (SPI1_UNDERRUN_TRUE_BITBAND << SPI1_STATUS_SPI1_UNDERRUN_STATUS_POS)
#define SPI1_UNDERRUN_CLEAR             (SPI1_UNDERRUN_CLEAR_BITBAND << SPI1_STATUS_SPI1_UNDERRUN_STATUS_POS)

/* ----------------------------------------------------------------------------
 * PCM Interface Configuration and Control
 * ------------------------------------------------------------------------- */

/* PCM Control */
#define PCM_CTRL_BASE                   0x40000A00
#define PCM_CTRL                        REG32_POINTER(PCM_CTRL_BASE)

/* PCM_CTRL bit positions */
#define PCM_CTRL_PCM_CLK_POL_POS        14
#define PCM_CTRL_BIT_ORDER_POS          12
#define PCM_CTRL_TX_ALIGN_POS           11
#define PCM_CTRL_WORD_SIZE_POS          9
#define PCM_CTRL_WORD_SIZE_MASK         (0x3 << PCM_CTRL_WORD_SIZE_POS)
#define PCM_CTRL_FRAME_ALIGN_POS        8
#define PCM_CTRL_FRAME_WIDTH_POS        7
#define PCM_CTRL_FRAME_LENGTH_POS       4
#define PCM_CTRL_FRAME_LENGTH_MASK      (0x7 << PCM_CTRL_FRAME_LENGTH_POS)
#define PCM_CTRL_FRAME_SUBFRAMES_POS    3
#define PCM_CTRL_CONTROLLER_POS         2
#define PCM_CTRL_ENABLE_POS             1
#define PCM_CTRL_SLAVE_POS              0

/* PCM_CTRL bit-band aliases */
#define PCM_CTRL_PCM_CLK_POL_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_PCM_CLK_POL_POS))
#define PCM_CTRL_BIT_ORDER_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_BIT_ORDER_POS))
#define PCM_CTRL_TX_ALIGN_BITBAND       REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_TX_ALIGN_POS))
#define PCM_CTRL_FRAME_ALIGN_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_FRAME_ALIGN_POS))
#define PCM_CTRL_FRAME_WIDTH_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_FRAME_WIDTH_POS))
#define PCM_CTRL_FRAME_SUBFRAMES_BITBAND REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_FRAME_SUBFRAMES_POS))
#define PCM_CTRL_CONTROLLER_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_CONTROLLER_POS))
#define PCM_CTRL_ENABLE_BITBAND         REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_ENABLE_POS))
#define PCM_CTRL_SLAVE_BITBAND          REG32_POINTER(SYS_CALC_BITBAND(PCM_CTRL_BASE, PCM_CTRL_SLAVE_POS))

/* PCM_CTRL settings */
#define PCM_SAMPLE_FALLING_EDGE_BITBAND 0x0
#define PCM_SAMPLE_RISING_EDGE_BITBAND  0x1
#define PCM_SAMPLE_FALLING_EDGE         (PCM_SAMPLE_FALLING_EDGE_BITBAND << PCM_CTRL_PCM_CLK_POL_POS)
#define PCM_SAMPLE_RISING_EDGE          (PCM_SAMPLE_RISING_EDGE_BITBAND << PCM_CTRL_PCM_CLK_POL_POS)

#define PCM_BIT_ORDER_MSB_FIRST_BITBAND 0x0
#define PCM_BIT_ORDER_LSB_FIRST_BITBAND 0x1
#define PCM_BIT_ORDER_MSB_FIRST         (PCM_BIT_ORDER_MSB_FIRST_BITBAND << PCM_CTRL_BIT_ORDER_POS)
#define PCM_BIT_ORDER_LSB_FIRST         (PCM_BIT_ORDER_LSB_FIRST_BITBAND << PCM_CTRL_BIT_ORDER_POS)

#define PCM_TX_ALIGN_MSB_BITBAND        0x0
#define PCM_TX_ALIGN_LSB_BITBAND        0x1
#define PCM_TX_ALIGN_MSB                (PCM_TX_ALIGN_MSB_BITBAND << PCM_CTRL_TX_ALIGN_POS)
#define PCM_TX_ALIGN_LSB                (PCM_TX_ALIGN_LSB_BITBAND << PCM_CTRL_TX_ALIGN_POS)

#define PCM_WORD_SIZE_8                 (0x0 << PCM_CTRL_WORD_SIZE_POS)
#define PCM_WORD_SIZE_16                (0x1 << PCM_CTRL_WORD_SIZE_POS)
#define PCM_WORD_SIZE_24                (0x2 << PCM_CTRL_WORD_SIZE_POS)
#define PCM_WORD_SIZE_32                (0x3 << PCM_CTRL_WORD_SIZE_POS)

#define PCM_FRAME_ALIGN_LAST_BITBAND    0x0
#define PCM_FRAME_ALIGN_FIRST_BITBAND   0x1
#define PCM_FRAME_ALIGN_LAST            (PCM_FRAME_ALIGN_LAST_BITBAND << PCM_CTRL_FRAME_ALIGN_POS)
#define PCM_FRAME_ALIGN_FIRST           (PCM_FRAME_ALIGN_FIRST_BITBAND << PCM_CTRL_FRAME_ALIGN_POS)

#define PCM_FRAME_WIDTH_SHORT_BITBAND   0x0
#define PCM_FRAME_WIDTH_LONG_BITBAND    0x1
#define PCM_FRAME_WIDTH_SHORT           (PCM_FRAME_WIDTH_SHORT_BITBAND << PCM_CTRL_FRAME_WIDTH_POS)
#define PCM_FRAME_WIDTH_LONG            (PCM_FRAME_WIDTH_LONG_BITBAND << PCM_CTRL_FRAME_WIDTH_POS)

#define PCM_MULTIWORD_2                 (0x0 << PCM_CTRL_FRAME_LENGTH_POS)
#define PCM_MULTIWORD_4                 (0x1 << PCM_CTRL_FRAME_LENGTH_POS)
#define PCM_MULTIWORD_6                 (0x2 << PCM_CTRL_FRAME_LENGTH_POS)
#define PCM_MULTIWORD_8                 (0x3 << PCM_CTRL_FRAME_LENGTH_POS)
#define PCM_MULTIWORD_10                (0x4 << PCM_CTRL_FRAME_LENGTH_POS)
#define PCM_MULTIWORD_12                (0x5 << PCM_CTRL_FRAME_LENGTH_POS)
#define PCM_MULTIWORD_14                (0x6 << PCM_CTRL_FRAME_LENGTH_POS)
#define PCM_MULTIWORD_16                (0x7 << PCM_CTRL_FRAME_LENGTH_POS)

#define PCM_SUBFRAME_DISABLE_BITBAND    0x0
#define PCM_SUBFRAME_ENABLE_BITBAND     0x1
#define PCM_SUBFRAME_DISABLE            (PCM_SUBFRAME_DISABLE_BITBAND << PCM_CTRL_FRAME_SUBFRAMES_POS)
#define PCM_SUBFRAME_ENABLE             (PCM_SUBFRAME_ENABLE_BITBAND << PCM_CTRL_FRAME_SUBFRAMES_POS)

#define PCM_CONTROLLER_CM3_BITBAND      0x0
#define PCM_CONTROLLER_DMA_BITBAND      0x1
#define PCM_CONTROLLER_CM3              (PCM_CONTROLLER_CM3_BITBAND << PCM_CTRL_CONTROLLER_POS)
#define PCM_CONTROLLER_DMA              (PCM_CONTROLLER_DMA_BITBAND << PCM_CTRL_CONTROLLER_POS)

#define PCM_DISABLE_BITBAND             0x0
#define PCM_ENABLE_BITBAND              0x1
#define PCM_DISABLE                     (PCM_DISABLE_BITBAND << PCM_CTRL_ENABLE_POS)
#define PCM_ENABLE                      (PCM_ENABLE_BITBAND << PCM_CTRL_ENABLE_POS)

#define PCM_SELECT_MASTER_BITBAND       0x0
#define PCM_SELECT_SLAVE_BITBAND        0x1
#define PCM_SELECT_MASTER               (PCM_SELECT_MASTER_BITBAND << PCM_CTRL_SLAVE_POS)
#define PCM_SELECT_SLAVE                (PCM_SELECT_SLAVE_BITBAND << PCM_CTRL_SLAVE_POS)

/* PCM Transmit Data */
#define PCM_TX_DATA_BASE                0x40000A04
#define PCM_TX_DATA                     REG32_POINTER(PCM_TX_DATA_BASE)

/* PCM Receive Data */
#define PCM_RX_DATA_BASE                0x40000A08
#define PCM_RX_DATA                     READONLY_REG32_POINTER(PCM_RX_DATA_BASE)

/* PCM Status */
#define PCM_STATUS_BASE                 0x40000A0C
#define PCM_STATUS                      REG32_POINTER(PCM_STATUS_BASE)

/* PCM_STATUS bit positions */
#define PCM_STATUS_TRANSMIT_STATUS_POS  3
#define PCM_STATUS_RECEIVE_STATUS_POS   2
#define PCM_STATUS_OVERRUN_STATUS_POS   1
#define PCM_STATUS_UNDERRUN_STATUS_POS  0

/* PCM_STATUS bit-band aliases */
#define PCM_STATUS_TRANSMIT_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(PCM_STATUS_BASE, PCM_STATUS_TRANSMIT_STATUS_POS))
#define PCM_STATUS_RECEIVE_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(PCM_STATUS_BASE, PCM_STATUS_RECEIVE_STATUS_POS))
#define PCM_STATUS_OVERRUN_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(PCM_STATUS_BASE, PCM_STATUS_OVERRUN_STATUS_POS))
#define PCM_STATUS_UNDERRUN_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(PCM_STATUS_BASE, PCM_STATUS_UNDERRUN_STATUS_POS))

/* PCM_STATUS settings */
#define PCM_TRANSMIT_FALSE_BITBAND      0x0
#define PCM_TRANSMIT_TRUE_BITBAND       0x1
#define PCM_TRANSMIT_CLEAR_BITBAND      0x1
#define PCM_TRANSMIT_FALSE              (PCM_TRANSMIT_FALSE_BITBAND << PCM_STATUS_TRANSMIT_STATUS_POS)
#define PCM_TRANSMIT_TRUE               (PCM_TRANSMIT_TRUE_BITBAND << PCM_STATUS_TRANSMIT_STATUS_POS)
#define PCM_TRANSMIT_CLEAR              (PCM_TRANSMIT_CLEAR_BITBAND << PCM_STATUS_TRANSMIT_STATUS_POS)

#define PCM_RECEIVE_FALSE_BITBAND       0x0
#define PCM_RECEIVE_TRUE_BITBAND        0x1
#define PCM_RECEIVE_CLEAR_BITBAND       0x1
#define PCM_RECEIVE_FALSE               (PCM_RECEIVE_FALSE_BITBAND << PCM_STATUS_RECEIVE_STATUS_POS)
#define PCM_RECEIVE_TRUE                (PCM_RECEIVE_TRUE_BITBAND << PCM_STATUS_RECEIVE_STATUS_POS)
#define PCM_RECEIVE_CLEAR               (PCM_RECEIVE_CLEAR_BITBAND << PCM_STATUS_RECEIVE_STATUS_POS)

#define PCM_OVERRUN_FALSE_BITBAND       0x0
#define PCM_OVERRUN_TRUE_BITBAND        0x1
#define PCM_OVERRUN_CLEAR_BITBAND       0x1
#define PCM_OVERRUN_FALSE               (PCM_OVERRUN_FALSE_BITBAND << PCM_STATUS_OVERRUN_STATUS_POS)
#define PCM_OVERRUN_TRUE                (PCM_OVERRUN_TRUE_BITBAND << PCM_STATUS_OVERRUN_STATUS_POS)
#define PCM_OVERRUN_CLEAR               (PCM_OVERRUN_CLEAR_BITBAND << PCM_STATUS_OVERRUN_STATUS_POS)

#define PCM_UNDERRUN_FALSE_BITBAND      0x0
#define PCM_UNDERRUN_TRUE_BITBAND       0x1
#define PCM_UNDERRUN_CLEAR_BITBAND      0x1
#define PCM_UNDERRUN_FALSE              (PCM_UNDERRUN_FALSE_BITBAND << PCM_STATUS_UNDERRUN_STATUS_POS)
#define PCM_UNDERRUN_TRUE               (PCM_UNDERRUN_TRUE_BITBAND << PCM_STATUS_UNDERRUN_STATUS_POS)
#define PCM_UNDERRUN_CLEAR              (PCM_UNDERRUN_CLEAR_BITBAND << PCM_STATUS_UNDERRUN_STATUS_POS)

/* ----------------------------------------------------------------------------
 * I2C Interface Configuration and Control
 * ------------------------------------------------------------------------- */

/* I2C Interface Configuration and Control */
#define I2C_CTRL0_BASE                  0x40000B00
#define I2C_CTRL0                       REG32_POINTER(I2C_CTRL0_BASE)

/* I2C_CTRL0 bit positions */
#define I2C_CTRL0_SPEED_POS             16
#define I2C_CTRL0_SPEED_MASK            (0xFF << I2C_CTRL0_SPEED_POS)
#define I2C_CTRL0_SLAVE_ADDRESS_POS     8
#define I2C_CTRL0_SLAVE_ADDRESS_MASK    (0x7F << I2C_CTRL0_SLAVE_ADDRESS_POS)
#define I2C_CTRL0_CONTROLLER_POS        4
#define I2C_CTRL0_STOP_INT_ENABLE_POS   3
#define I2C_CTRL0_AUTO_ACK_ENABLE_POS   2
#define I2C_CTRL0_I2C_SAMPLE_CLK_ENABLE_POS 1
#define I2C_CTRL0_SLAVE_ENABLE_POS      0

/* I2C_CTRL0 bit-band aliases */
#define I2C_CTRL0_CONTROLLER_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL0_BASE, I2C_CTRL0_CONTROLLER_POS))
#define I2C_CTRL0_STOP_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL0_BASE, I2C_CTRL0_STOP_INT_ENABLE_POS))
#define I2C_CTRL0_AUTO_ACK_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL0_BASE, I2C_CTRL0_AUTO_ACK_ENABLE_POS))
#define I2C_CTRL0_I2C_SAMPLE_CLK_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL0_BASE, I2C_CTRL0_I2C_SAMPLE_CLK_ENABLE_POS))
#define I2C_CTRL0_SLAVE_ENABLE_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL0_BASE, I2C_CTRL0_SLAVE_ENABLE_POS))

/* I2C_CTRL0 settings */
#define I2C_MASTER_SPEED_3              (0x0 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_6              (0x1 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_9              (0x2 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_12             (0x3 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_15             (0x4 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_18             (0x5 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_21             (0x6 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_24             (0x7 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_27             (0x8 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_30             (0x9 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_33             (0xA << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_36             (0xB << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_39             (0xC << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_42             (0xD << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_45             (0xE << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_48             (0xF << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_51             (0x10 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_54             (0x11 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_57             (0x12 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_60             (0x13 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_120            (0x27 << I2C_CTRL0_SPEED_POS)
#define I2C_MASTER_SPEED_768            (0xFF << I2C_CTRL0_SPEED_POS)
#define I2C_SLAVE_SPEED_1               (0x0 << I2C_CTRL0_SPEED_POS)
#define I2C_SLAVE_SPEED_2               (0x1 << I2C_CTRL0_SPEED_POS)
#define I2C_SLAVE_SPEED_3               (0x2 << I2C_CTRL0_SPEED_POS)
#define I2C_SLAVE_SPEED_4               (0x3 << I2C_CTRL0_SPEED_POS)
#define I2C_SLAVE_SPEED_5               (0x4 << I2C_CTRL0_SPEED_POS)
#define I2C_SLAVE_SPEED_6               (0x5 << I2C_CTRL0_SPEED_POS)
#define I2C_SLAVE_SPEED_7               (0x6 << I2C_CTRL0_SPEED_POS)
#define I2C_SLAVE_SPEED_14              (0xD << I2C_CTRL0_SPEED_POS)

#define I2C_CONTROLLER_CM3_BITBAND      0x0
#define I2C_CONTROLLER_DMA_BITBAND      0x1
#define I2C_CONTROLLER_CM3              (I2C_CONTROLLER_CM3_BITBAND << I2C_CTRL0_CONTROLLER_POS)
#define I2C_CONTROLLER_DMA              (I2C_CONTROLLER_DMA_BITBAND << I2C_CTRL0_CONTROLLER_POS)

#define I2C_STOP_INT_DISABLE_BITBAND    0x0
#define I2C_STOP_INT_ENABLE_BITBAND     0x1
#define I2C_STOP_INT_DISABLE            (I2C_STOP_INT_DISABLE_BITBAND << I2C_CTRL0_STOP_INT_ENABLE_POS)
#define I2C_STOP_INT_ENABLE             (I2C_STOP_INT_ENABLE_BITBAND << I2C_CTRL0_STOP_INT_ENABLE_POS)

#define I2C_AUTO_ACK_DISABLE_BITBAND    0x0
#define I2C_AUTO_ACK_ENABLE_BITBAND     0x1
#define I2C_AUTO_ACK_DISABLE            (I2C_AUTO_ACK_DISABLE_BITBAND << I2C_CTRL0_AUTO_ACK_ENABLE_POS)
#define I2C_AUTO_ACK_ENABLE             (I2C_AUTO_ACK_ENABLE_BITBAND << I2C_CTRL0_AUTO_ACK_ENABLE_POS)

#define I2C_SAMPLE_CLK_DISABLE_BITBAND  0x0
#define I2C_SAMPLE_CLK_ENABLE_BITBAND   0x1
#define I2C_SAMPLE_CLK_DISABLE          (I2C_SAMPLE_CLK_DISABLE_BITBAND << I2C_CTRL0_I2C_SAMPLE_CLK_ENABLE_POS)
#define I2C_SAMPLE_CLK_ENABLE           (I2C_SAMPLE_CLK_ENABLE_BITBAND << I2C_CTRL0_I2C_SAMPLE_CLK_ENABLE_POS)

#define I2C_SLAVE_DISABLE_BITBAND       0x0
#define I2C_SLAVE_ENABLE_BITBAND        0x1
#define I2C_SLAVE_DISABLE               (I2C_SLAVE_DISABLE_BITBAND << I2C_CTRL0_SLAVE_ENABLE_POS)
#define I2C_SLAVE_ENABLE                (I2C_SLAVE_ENABLE_BITBAND << I2C_CTRL0_SLAVE_ENABLE_POS)

/* I2C Interface Status and Control */
#define I2C_CTRL1_BASE                  0x40000B04
#define I2C_CTRL1                       REG32_POINTER(I2C_CTRL1_BASE)

/* I2C_CTRL1 bit positions */
#define I2C_CTRL1_RESET_POS             5
#define I2C_CTRL1_LAST_DATA_POS         4
#define I2C_CTRL1_STOP_POS              3
#define I2C_CTRL1_NACK_POS              1
#define I2C_CTRL1_ACK_POS               0

/* I2C_CTRL1 bit-band aliases */
#define I2C_CTRL1_RESET_BITBAND         REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL1_BASE, I2C_CTRL1_RESET_POS))
#define I2C_CTRL1_LAST_DATA_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL1_BASE, I2C_CTRL1_LAST_DATA_POS))
#define I2C_CTRL1_STOP_BITBAND          REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL1_BASE, I2C_CTRL1_STOP_POS))
#define I2C_CTRL1_NACK_BITBAND          REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL1_BASE, I2C_CTRL1_NACK_POS))
#define I2C_CTRL1_ACK_BITBAND           REG32_POINTER(SYS_CALC_BITBAND(I2C_CTRL1_BASE, I2C_CTRL1_ACK_POS))

/* I2C_CTRL1 settings */
#define I2C_RESET_BITBAND               0x1
#define I2C_RESET                       (I2C_RESET_BITBAND << I2C_CTRL1_RESET_POS)

#define I2C_LAST_DATA_BITBAND           0x1
#define I2C_LAST_DATA                   (I2C_LAST_DATA_BITBAND << I2C_CTRL1_LAST_DATA_POS)

#define I2C_STOP_BITBAND                0x1
#define I2C_STOP                        (I2C_STOP_BITBAND << I2C_CTRL1_STOP_POS)

#define I2C_NACK_BITBAND                0x1
#define I2C_NACK                        (I2C_NACK_BITBAND << I2C_CTRL1_NACK_POS)

#define I2C_ACK_BITBAND                 0x1
#define I2C_ACK                         (I2C_ACK_BITBAND << I2C_CTRL1_ACK_POS)

/* I2C Interface Data */
#define I2C_DATA_BASE                   0x40000B08
#define I2C_DATA                        REG32_POINTER(I2C_DATA_BASE)

/* I2C Interface Data (Mirror) */
#define I2C_DATA_M_BASE                 0x40000B0C
#define I2C_DATA_M                      REG32_POINTER(I2C_DATA_M_BASE)

/* I2C Master Address and Start */
#define I2C_ADDR_START_BASE             0x40000B10
#define I2C_ADDR_START                  REG32_POINTER(I2C_ADDR_START_BASE)

/* I2C_ADDR_START bit positions */
#define I2C_ADDR_START_ADDRESS_POS      1
#define I2C_ADDR_START_ADDRESS_MASK     (0x7F << I2C_ADDR_START_ADDRESS_POS)
#define I2C_ADDR_START_READ_WRITE_POS   0

/* I2C_ADDR_START bit-band aliases */
#define I2C_ADDR_START_READ_WRITE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(I2C_ADDR_START_BASE, I2C_ADDR_START_READ_WRITE_POS))

/* I2C_ADDR_START settings */
#define I2C_START_WRITE_BITBAND         0x0
#define I2C_START_READ_BITBAND          0x1
#define I2C_START_WRITE                 (I2C_START_WRITE_BITBAND << I2C_ADDR_START_READ_WRITE_POS)
#define I2C_START_READ                  (I2C_START_READ_BITBAND << I2C_ADDR_START_READ_WRITE_POS)

/* I2C Status */
#define I2C_STATUS_BASE                 0x40000B14
#define I2C_STATUS                      READONLY_REG32_POINTER(I2C_STATUS_BASE)

/* I2C_STATUS bit positions */
#define I2C_STATUS_ERROR_S_POS          15
#define I2C_STATUS_BUS_ERROR_S_POS      14
#define I2C_STATUS_START_PENDING_POS    13
#define I2C_STATUS_MASTER_MODE_POS      12
#define I2C_STATUS_DMA_REQ_POS          11
#define I2C_STATUS_STOP_DETECT_POS      10
#define I2C_STATUS_DATA_EVENT_POS       9
#define I2C_STATUS_ERROR_POS            8
#define I2C_STATUS_BUS_ERROR_POS        7
#define I2C_STATUS_BUFFER_FULL_POS      6
#define I2C_STATUS_CLK_STRETCH_POS      5
#define I2C_STATUS_BUS_FREE_POS         4
#define I2C_STATUS_ADDR_DATA_POS        3
#define I2C_STATUS_READ_WRITE_POS       2
#define I2C_STATUS_GEN_CALL_POS         1
#define I2C_STATUS_ACK_STATUS_POS       0

/* I2C_STATUS bit-band aliases */
#define I2C_STATUS_ERROR_S_BITBAND      READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_ERROR_S_POS))
#define I2C_STATUS_BUS_ERROR_S_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_BUS_ERROR_S_POS))
#define I2C_STATUS_START_PENDING_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_START_PENDING_POS))
#define I2C_STATUS_MASTER_MODE_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_MASTER_MODE_POS))
#define I2C_STATUS_DMA_REQ_BITBAND      READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_DMA_REQ_POS))
#define I2C_STATUS_STOP_DETECT_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_STOP_DETECT_POS))
#define I2C_STATUS_DATA_EVENT_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_DATA_EVENT_POS))
#define I2C_STATUS_ERROR_BITBAND        READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_ERROR_POS))
#define I2C_STATUS_BUS_ERROR_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_BUS_ERROR_POS))
#define I2C_STATUS_BUFFER_FULL_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_BUFFER_FULL_POS))
#define I2C_STATUS_CLK_STRETCH_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_CLK_STRETCH_POS))
#define I2C_STATUS_BUS_FREE_BITBAND     READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_BUS_FREE_POS))
#define I2C_STATUS_ADDR_DATA_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_ADDR_DATA_POS))
#define I2C_STATUS_READ_WRITE_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_READ_WRITE_POS))
#define I2C_STATUS_GEN_CALL_BITBAND     READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_GEN_CALL_POS))
#define I2C_STATUS_ACK_STATUS_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(I2C_STATUS_BASE, I2C_STATUS_ACK_STATUS_POS))

/* I2C_STATUS settings */
#define I2C_NO_ERROR_S_BITBAND          0x0
#define I2C_ERROR_S_BITBAND             0x1
#define I2C_NO_ERROR_S                  (I2C_NO_ERROR_S_BITBAND << I2C_STATUS_ERROR_S_POS)
#define I2C_ERROR_S                     (I2C_ERROR_S_BITBAND << I2C_STATUS_ERROR_S_POS)

#define I2C_NO_BUS_ERROR_S_BITBAND      0x0
#define I2C_BUS_ERROR_S_BITBAND         0x1
#define I2C_NO_BUS_ERROR_S              (I2C_NO_BUS_ERROR_S_BITBAND << I2C_STATUS_BUS_ERROR_S_POS)
#define I2C_BUS_ERROR_S                 (I2C_BUS_ERROR_S_BITBAND << I2C_STATUS_BUS_ERROR_S_POS)

#define I2C_START_NOT_PENDING_BITBAND   0x0
#define I2C_START_PENDING_BITBAND       0x1
#define I2C_START_NOT_PENDING           (I2C_START_NOT_PENDING_BITBAND << I2C_STATUS_START_PENDING_POS)
#define I2C_START_PENDING               (I2C_START_PENDING_BITBAND << I2C_STATUS_START_PENDING_POS)

#define I2C_MASTER_INACTIVE_BITBAND     0x0
#define I2C_MASTER_ACTIVE_BITBAND       0x1
#define I2C_MASTER_INACTIVE             (I2C_MASTER_INACTIVE_BITBAND << I2C_STATUS_MASTER_MODE_POS)
#define I2C_MASTER_ACTIVE               (I2C_MASTER_ACTIVE_BITBAND << I2C_STATUS_MASTER_MODE_POS)

#define I2C_NO_DMA_REQUEST_BITBAND      0x0
#define I2C_DMA_REQUEST_BITBAND         0x1
#define I2C_NO_DMA_REQUEST              (I2C_NO_DMA_REQUEST_BITBAND << I2C_STATUS_DMA_REQ_POS)
#define I2C_DMA_REQUEST                 (I2C_DMA_REQUEST_BITBAND << I2C_STATUS_DMA_REQ_POS)

#define I2C_NO_STOP_DETECTED_BITBAND    0x0
#define I2C_STOP_DETECTED_BITBAND       0x1
#define I2C_NO_STOP_DETECTED            (I2C_NO_STOP_DETECTED_BITBAND << I2C_STATUS_STOP_DETECT_POS)
#define I2C_STOP_DETECTED               (I2C_STOP_DETECTED_BITBAND << I2C_STATUS_STOP_DETECT_POS)

#define I2C_NON_DATA_EVENT_BITBAND      0x0
#define I2C_DATA_EVENT_BITBAND          0x1
#define I2C_NON_DATA_EVENT              (I2C_NON_DATA_EVENT_BITBAND << I2C_STATUS_DATA_EVENT_POS)
#define I2C_DATA_EVENT                  (I2C_DATA_EVENT_BITBAND << I2C_STATUS_DATA_EVENT_POS)

#define I2C_NO_ERROR_BITBAND            0x0
#define I2C_ERROR_BITBAND               0x1
#define I2C_NO_ERROR                    (I2C_NO_ERROR_BITBAND << I2C_STATUS_ERROR_POS)
#define I2C_ERROR                       (I2C_ERROR_BITBAND << I2C_STATUS_ERROR_POS)

#define I2C_NO_BUS_ERROR_BITBAND        0x0
#define I2C_BUS_ERROR_BITBAND           0x1
#define I2C_NO_BUS_ERROR                (I2C_NO_BUS_ERROR_BITBAND << I2C_STATUS_BUS_ERROR_POS)
#define I2C_BUS_ERROR                   (I2C_BUS_ERROR_BITBAND << I2C_STATUS_BUS_ERROR_POS)

#define I2C_BUFFER_EMPTY_BITBAND        0x0
#define I2C_BUFFER_FULL_BITBAND         0x1
#define I2C_BUFFER_EMPTY                (I2C_BUFFER_EMPTY_BITBAND << I2C_STATUS_BUFFER_FULL_POS)
#define I2C_BUFFER_FULL                 (I2C_BUFFER_FULL_BITBAND << I2C_STATUS_BUFFER_FULL_POS)

#define I2C_CLK_NOT_STRETCHED_BITBAND   0x0
#define I2C_CLK_STRETCHED_BITBAND       0x1
#define I2C_CLK_NOT_STRETCHED           (I2C_CLK_NOT_STRETCHED_BITBAND << I2C_STATUS_CLK_STRETCH_POS)
#define I2C_CLK_STRETCHED               (I2C_CLK_STRETCHED_BITBAND << I2C_STATUS_CLK_STRETCH_POS)

#define I2C_BUS_BUSY_BITBAND            0x0
#define I2C_BUS_FREE_BITBAND            0x1
#define I2C_BUS_BUSY                    (I2C_BUS_BUSY_BITBAND << I2C_STATUS_BUS_FREE_POS)
#define I2C_BUS_FREE                    (I2C_BUS_FREE_BITBAND << I2C_STATUS_BUS_FREE_POS)

#define I2C_DATA_IS_DATA_BITBAND        0x0
#define I2C_DATA_IS_ADDR_BITBAND        0x1
#define I2C_DATA_IS_DATA                (I2C_DATA_IS_DATA_BITBAND << I2C_STATUS_ADDR_DATA_POS)
#define I2C_DATA_IS_ADDR                (I2C_DATA_IS_ADDR_BITBAND << I2C_STATUS_ADDR_DATA_POS)

#define I2C_IS_WRITE_BITBAND            0x0
#define I2C_IS_READ_BITBAND             0x1
#define I2C_IS_WRITE                    (I2C_IS_WRITE_BITBAND << I2C_STATUS_READ_WRITE_POS)
#define I2C_IS_READ                     (I2C_IS_READ_BITBAND << I2C_STATUS_READ_WRITE_POS)

#define I2C_ADDR_OTHER_BITBAND          0x0
#define I2C_ADDR_GEN_CALL_BITBAND       0x1
#define I2C_ADDR_OTHER                  (I2C_ADDR_OTHER_BITBAND << I2C_STATUS_GEN_CALL_POS)
#define I2C_ADDR_GEN_CALL               (I2C_ADDR_GEN_CALL_BITBAND << I2C_STATUS_GEN_CALL_POS)

#define I2C_HAS_ACK_BITBAND             0x0
#define I2C_HAS_NACK_BITBAND            0x1
#define I2C_HAS_ACK                     (I2C_HAS_ACK_BITBAND << I2C_STATUS_ACK_STATUS_POS)
#define I2C_HAS_NACK                    (I2C_HAS_NACK_BITBAND << I2C_STATUS_ACK_STATUS_POS)

/* ----------------------------------------------------------------------------
 * UART Interface Configuration and Control
 * ------------------------------------------------------------------------- */

/* UART Configuration Register */
#define UART_CFG_BASE                   0x40000C00
#define UART_CFG                        REG32_POINTER(UART_CFG_BASE)

/* UART_CFG bit positions */
#define UART_CFG_PRESCALE_POS           8
#define UART_CFG_PRESCALE_MASK          (0xFFFF << UART_CFG_PRESCALE_POS)
#define UART_CFG_PRESCALE_ENABLE_POS    4
#define UART_CFG_DMA_ENABLE_POS         1
#define UART_CFG_ENABLE_POS             0

/* UART_CFG bit-band aliases */
#define UART_CFG_PRESCALE_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(UART_CFG_BASE, UART_CFG_PRESCALE_ENABLE_POS))
#define UART_CFG_DMA_ENABLE_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(UART_CFG_BASE, UART_CFG_DMA_ENABLE_POS))
#define UART_CFG_ENABLE_BITBAND         REG32_POINTER(SYS_CALC_BITBAND(UART_CFG_BASE, UART_CFG_ENABLE_POS))

/* UART_CFG settings */
#define UART_PRESCALE_DISABLE_BITBAND   0x0
#define UART_PRESCALE_ENABLE_BITBAND    0x1
#define UART_PRESCALE_DISABLE           (UART_PRESCALE_DISABLE_BITBAND << UART_CFG_PRESCALE_ENABLE_POS)
#define UART_PRESCALE_ENABLE            (UART_PRESCALE_ENABLE_BITBAND << UART_CFG_PRESCALE_ENABLE_POS)

#define UART_DMA_MODE_DISABLE_BITBAND   0x0
#define UART_DMA_MODE_ENABLE_BITBAND    0x1
#define UART_DMA_MODE_DISABLE           (UART_DMA_MODE_DISABLE_BITBAND << UART_CFG_DMA_ENABLE_POS)
#define UART_DMA_MODE_ENABLE            (UART_DMA_MODE_ENABLE_BITBAND << UART_CFG_DMA_ENABLE_POS)

#define UART_DISABLE_BITBAND            0x0
#define UART_ENABLE_BITBAND             0x1
#define UART_DISABLE                    (UART_DISABLE_BITBAND << UART_CFG_ENABLE_POS)
#define UART_ENABLE                     (UART_ENABLE_BITBAND << UART_CFG_ENABLE_POS)

/* UART Transmit Data Register */
#define UART_TX_DATA_BASE               0x40000C04
#define UART_TX_DATA                    REG32_POINTER(UART_TX_DATA_BASE)

/* UART Receive Data Register */
#define UART_RX_DATA_BASE               0x40000C08
#define UART_RX_DATA                    READONLY_REG32_POINTER(UART_RX_DATA_BASE)

/* UART Status Register */
#define UART_STATUS_BASE                0x40000C0C
#define UART_STATUS                     REG32_POINTER(UART_STATUS_BASE)

/* UART_STATUS bit positions */
#define UART_STATUS_UART_RX_OVERRUN_STATUS_POS 0

/* UART_STATUS bit-band aliases */
#define UART_STATUS_UART_RX_OVERRUN_STATUS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(UART_STATUS_BASE, UART_STATUS_UART_RX_OVERRUN_STATUS_POS))

/* UART_STATUS settings */
#define UART_RX_OVERRUN_FALSE_BITBAND   0x0
#define UART_RX_OVERRUN_TRUE_BITBAND    0x1
#define UART_RX_OVERRUN_CLEAR_BITBAND   0x1
#define UART_RX_OVERRUN_FALSE           (UART_RX_OVERRUN_FALSE_BITBAND << UART_STATUS_UART_RX_OVERRUN_STATUS_POS)
#define UART_RX_OVERRUN_TRUE            (UART_RX_OVERRUN_TRUE_BITBAND << UART_STATUS_UART_RX_OVERRUN_STATUS_POS)
#define UART_RX_OVERRUN_CLEAR           (UART_RX_OVERRUN_CLEAR_BITBAND << UART_STATUS_UART_RX_OVERRUN_STATUS_POS)

/* ----------------------------------------------------------------------------
 * PWM 0 and 1 Configuration and Control
 * ------------------------------------------------------------------------- */

/* PWM Configuration Register */
#define PWM_CFG_BASE                    0x40000D00
#define PWM_CFG                         REG32_POINTER(PWM_CFG_BASE)

/* PWM_CFG bit positions */
#define PWM_CFG_PWM_HIGH_POS            8
#define PWM_CFG_PWM_HIGH_MASK           (0xFF << PWM_CFG_PWM_HIGH_POS)
#define PWM_CFG_PWM_PERIOD_POS          0
#define PWM_CFG_PWM_PERIOD_MASK         (0xFF << PWM_CFG_PWM_PERIOD_POS)

/* PWM 0 and 1 Control Register */
#define PWM_CTRL_BASE                   0x40000D08
#define PWM_CTRL                        REG32_POINTER(PWM_CTRL_BASE)

/* PWM_CTRL bit positions */
#define PWM_CTRL_PWM_OFFSET_ENABLE_POS  16
#define PWM_CTRL_PWM_OFFSET_POS         8
#define PWM_CTRL_PWM_OFFSET_MASK        (0xFF << PWM_CTRL_PWM_OFFSET_POS)
#define PWM_CTRL_PWM1_ENABLE_POS        4
#define PWM_CTRL_PWM0_ENABLE_POS        0

/* PWM_CTRL bit-band aliases */
#define PWM_CTRL_PWM_OFFSET_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(PWM_CTRL_BASE, PWM_CTRL_PWM_OFFSET_ENABLE_POS))
#define PWM_CTRL_PWM1_ENABLE_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(PWM_CTRL_BASE, PWM_CTRL_PWM1_ENABLE_POS))
#define PWM_CTRL_PWM0_ENABLE_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(PWM_CTRL_BASE, PWM_CTRL_PWM0_ENABLE_POS))

/* PWM_CTRL settings */
#define PWM_OFFSET_DISABLE_BITBAND      0x0
#define PWM_OFFSET_ENABLE_BITBAND       0x1
#define PWM_OFFSET_DISABLE              (PWM_OFFSET_DISABLE_BITBAND << PWM_CTRL_PWM_OFFSET_ENABLE_POS)
#define PWM_OFFSET_ENABLE               (PWM_OFFSET_ENABLE_BITBAND << PWM_CTRL_PWM_OFFSET_ENABLE_POS)

#define PWM1_DISABLE_BITBAND            0x0
#define PWM1_ENABLE_BITBAND             0x1
#define PWM1_DISABLE                    (PWM1_DISABLE_BITBAND << PWM_CTRL_PWM1_ENABLE_POS)
#define PWM1_ENABLE                     (PWM1_ENABLE_BITBAND << PWM_CTRL_PWM1_ENABLE_POS)

#define PWM0_DISABLE_BITBAND            0x0
#define PWM0_ENABLE_BITBAND             0x1
#define PWM0_DISABLE                    (PWM0_DISABLE_BITBAND << PWM_CTRL_PWM0_ENABLE_POS)
#define PWM0_ENABLE                     (PWM0_ENABLE_BITBAND << PWM_CTRL_PWM0_ENABLE_POS)

/* ----------------------------------------------------------------------------
 * DMIC Input and Output Driver Configuration and Control
 * ------------------------------------------------------------------------- */

/* DMIC and OD Configuration Register */
#define AUDIO_CFG_BASE                  0x40000E00
#define AUDIO_CFG                       REG32_POINTER(AUDIO_CFG_BASE)

/* AUDIO_CFG bit positions */
#define AUDIO_CFG_OD_CLK_SRC_POS        25
#define AUDIO_CFG_DMIC_CLK_SRC_POS      24
#define AUDIO_CFG_DEC_RATE_POS          16
#define AUDIO_CFG_DEC_RATE_MASK         (0x1F << AUDIO_CFG_DEC_RATE_POS)
#define AUDIO_CFG_OD_UNDERRUN_PROTECT_POS 12
#define AUDIO_CFG_OD_DMA_REQ_EN_POS     11
#define AUDIO_CFG_OD_INT_GEN_EN_POS     10
#define AUDIO_CFG_OD_DATA_ALIGN_POS     9
#define AUDIO_CFG_OD_ENABLE_POS         8
#define AUDIO_CFG_DMIC1_DMA_REQ_EN_POS  7
#define AUDIO_CFG_DMIC1_INT_GEN_EN_POS  6
#define AUDIO_CFG_DMIC1_DATA_ALIGN_POS  5
#define AUDIO_CFG_DMIC1_ENABLE_POS      4
#define AUDIO_CFG_DMIC0_DMA_REQ_EN_POS  3
#define AUDIO_CFG_DMIC0_INT_GEN_EN_POS  2
#define AUDIO_CFG_DMIC0_DATA_ALIGN_POS  1
#define AUDIO_CFG_DMIC0_ENABLE_POS      0

/* AUDIO_CFG bit-band aliases */
#define AUDIO_CFG_OD_CLK_SRC_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_OD_CLK_SRC_POS))
#define AUDIO_CFG_DMIC_CLK_SRC_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC_CLK_SRC_POS))
#define AUDIO_CFG_OD_UNDERRUN_PROTECT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_OD_UNDERRUN_PROTECT_POS))
#define AUDIO_CFG_OD_DMA_REQ_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_OD_DMA_REQ_EN_POS))
#define AUDIO_CFG_OD_INT_GEN_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_OD_INT_GEN_EN_POS))
#define AUDIO_CFG_OD_DATA_ALIGN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_OD_DATA_ALIGN_POS))
#define AUDIO_CFG_OD_ENABLE_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_OD_ENABLE_POS))
#define AUDIO_CFG_DMIC1_DMA_REQ_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC1_DMA_REQ_EN_POS))
#define AUDIO_CFG_DMIC1_INT_GEN_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC1_INT_GEN_EN_POS))
#define AUDIO_CFG_DMIC1_DATA_ALIGN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC1_DATA_ALIGN_POS))
#define AUDIO_CFG_DMIC1_ENABLE_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC1_ENABLE_POS))
#define AUDIO_CFG_DMIC0_DMA_REQ_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC0_DMA_REQ_EN_POS))
#define AUDIO_CFG_DMIC0_INT_GEN_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC0_INT_GEN_EN_POS))
#define AUDIO_CFG_DMIC0_DATA_ALIGN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC0_DATA_ALIGN_POS))
#define AUDIO_CFG_DMIC0_ENABLE_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(AUDIO_CFG_BASE, AUDIO_CFG_DMIC0_ENABLE_POS))

/* AUDIO_CFG settings */
#define OD_AUDIOCLK_BITBAND             0x0
#define OD_AUDIOSLOWCLK_BITBAND         0x1
#define OD_AUDIOCLK                     (OD_AUDIOCLK_BITBAND << AUDIO_CFG_OD_CLK_SRC_POS)
#define OD_AUDIOSLOWCLK                 (OD_AUDIOSLOWCLK_BITBAND << AUDIO_CFG_OD_CLK_SRC_POS)

#define DMIC_AUDIOCLK_BITBAND           0x0
#define DMIC_AUDIOSLOWCLK_BITBAND       0x1
#define DMIC_AUDIOCLK                   (DMIC_AUDIOCLK_BITBAND << AUDIO_CFG_DMIC_CLK_SRC_POS)
#define DMIC_AUDIOSLOWCLK               (DMIC_AUDIOSLOWCLK_BITBAND << AUDIO_CFG_DMIC_CLK_SRC_POS)

#define DECIMATE_BY_64                  (0x0 << AUDIO_CFG_DEC_RATE_POS)
#define DECIMATE_BY_72                  (0x1 << AUDIO_CFG_DEC_RATE_POS)
#define DECIMATE_BY_80                  (0x2 << AUDIO_CFG_DEC_RATE_POS)
#define DECIMATE_BY_128                 (0x8 << AUDIO_CFG_DEC_RATE_POS)
#define DECIMATE_BY_136                 (0x9 << AUDIO_CFG_DEC_RATE_POS)
#define DECIMATE_BY_256                 (0x16 << AUDIO_CFG_DEC_RATE_POS)
#define DECIMATE_BY_288                 (0x1C << AUDIO_CFG_DEC_RATE_POS)

#define OD_UNDERRUN_PROTECT_DISABLE_BITBAND 0x0
#define OD_UNDERRUN_PROTECT_ENABLE_BITBAND 0x1
#define OD_UNDERRUN_PROTECT_DISABLE     (OD_UNDERRUN_PROTECT_DISABLE_BITBAND << AUDIO_CFG_OD_UNDERRUN_PROTECT_POS)
#define OD_UNDERRUN_PROTECT_ENABLE      (OD_UNDERRUN_PROTECT_ENABLE_BITBAND << AUDIO_CFG_OD_UNDERRUN_PROTECT_POS)

#define OD_DMA_REQ_DISABLE_BITBAND      0x0
#define OD_DMA_REQ_ENABLE_BITBAND       0x1
#define OD_DMA_REQ_DISABLE              (OD_DMA_REQ_DISABLE_BITBAND << AUDIO_CFG_OD_DMA_REQ_EN_POS)
#define OD_DMA_REQ_ENABLE               (OD_DMA_REQ_ENABLE_BITBAND << AUDIO_CFG_OD_DMA_REQ_EN_POS)

#define OD_INT_GEN_DISABLE_BITBAND      0x0
#define OD_INT_GEN_ENABLE_BITBAND       0x1
#define OD_INT_GEN_DISABLE              (OD_INT_GEN_DISABLE_BITBAND << AUDIO_CFG_OD_INT_GEN_EN_POS)
#define OD_INT_GEN_ENABLE               (OD_INT_GEN_ENABLE_BITBAND << AUDIO_CFG_OD_INT_GEN_EN_POS)

#define OD_DATA_LSB_ALIGNED_BITBAND     0x0
#define OD_DATA_MSB_ALIGNED_BITBAND     0x1
#define OD_DATA_LSB_ALIGNED             (OD_DATA_LSB_ALIGNED_BITBAND << AUDIO_CFG_OD_DATA_ALIGN_POS)
#define OD_DATA_MSB_ALIGNED             (OD_DATA_MSB_ALIGNED_BITBAND << AUDIO_CFG_OD_DATA_ALIGN_POS)

#define OD_DISABLE_BITBAND              0x0
#define OD_ENABLE_BITBAND               0x1
#define OD_DISABLE                      (OD_DISABLE_BITBAND << AUDIO_CFG_OD_ENABLE_POS)
#define OD_ENABLE                       (OD_ENABLE_BITBAND << AUDIO_CFG_OD_ENABLE_POS)

#define DMIC1_DMA_REQ_DISABLE_BITBAND   0x0
#define DMIC1_DMA_REQ_ENABLE_BITBAND    0x1
#define DMIC1_DMA_REQ_DISABLE           (DMIC1_DMA_REQ_DISABLE_BITBAND << AUDIO_CFG_DMIC1_DMA_REQ_EN_POS)
#define DMIC1_DMA_REQ_ENABLE            (DMIC1_DMA_REQ_ENABLE_BITBAND << AUDIO_CFG_DMIC1_DMA_REQ_EN_POS)

#define DMIC1_INT_GEN_DISABLE_BITBAND   0x0
#define DMIC1_INT_GEN_ENABLE_BITBAND    0x1
#define DMIC1_INT_GEN_DISABLE           (DMIC1_INT_GEN_DISABLE_BITBAND << AUDIO_CFG_DMIC1_INT_GEN_EN_POS)
#define DMIC1_INT_GEN_ENABLE            (DMIC1_INT_GEN_ENABLE_BITBAND << AUDIO_CFG_DMIC1_INT_GEN_EN_POS)

#define DMIC1_DATA_LSB_ALIGNED_BITBAND  0x0
#define DMIC1_DATA_MSB_ALIGNED_BITBAND  0x1
#define DMIC1_DATA_LSB_ALIGNED          (DMIC1_DATA_LSB_ALIGNED_BITBAND << AUDIO_CFG_DMIC1_DATA_ALIGN_POS)
#define DMIC1_DATA_MSB_ALIGNED          (DMIC1_DATA_MSB_ALIGNED_BITBAND << AUDIO_CFG_DMIC1_DATA_ALIGN_POS)

#define DMIC1_DISABLE_BITBAND           0x0
#define DMIC1_ENABLE_BITBAND            0x1
#define DMIC1_DISABLE                   (DMIC1_DISABLE_BITBAND << AUDIO_CFG_DMIC1_ENABLE_POS)
#define DMIC1_ENABLE                    (DMIC1_ENABLE_BITBAND << AUDIO_CFG_DMIC1_ENABLE_POS)

#define DMIC0_DMA_REQ_DISABLE_BITBAND   0x0
#define DMIC0_DMA_REQ_ENABLE_BITBAND    0x1
#define DMIC0_DMA_REQ_DISABLE           (DMIC0_DMA_REQ_DISABLE_BITBAND << AUDIO_CFG_DMIC0_DMA_REQ_EN_POS)
#define DMIC0_DMA_REQ_ENABLE            (DMIC0_DMA_REQ_ENABLE_BITBAND << AUDIO_CFG_DMIC0_DMA_REQ_EN_POS)

#define DMIC0_INT_GEN_DISABLE_BITBAND   0x0
#define DMIC0_INT_GEN_ENABLE_BITBAND    0x1
#define DMIC0_INT_GEN_DISABLE           (DMIC0_INT_GEN_DISABLE_BITBAND << AUDIO_CFG_DMIC0_INT_GEN_EN_POS)
#define DMIC0_INT_GEN_ENABLE            (DMIC0_INT_GEN_ENABLE_BITBAND << AUDIO_CFG_DMIC0_INT_GEN_EN_POS)

#define DMIC0_DATA_LSB_ALIGNED_BITBAND  0x0
#define DMIC0_DATA_MSB_ALIGNED_BITBAND  0x1
#define DMIC0_DATA_LSB_ALIGNED          (DMIC0_DATA_LSB_ALIGNED_BITBAND << AUDIO_CFG_DMIC0_DATA_ALIGN_POS)
#define DMIC0_DATA_MSB_ALIGNED          (DMIC0_DATA_MSB_ALIGNED_BITBAND << AUDIO_CFG_DMIC0_DATA_ALIGN_POS)

#define DMIC0_DISABLE_BITBAND           0x0
#define DMIC0_ENABLE_BITBAND            0x1
#define DMIC0_DISABLE                   (DMIC0_DISABLE_BITBAND << AUDIO_CFG_DMIC0_ENABLE_POS)
#define DMIC0_ENABLE                    (DMIC0_ENABLE_BITBAND << AUDIO_CFG_DMIC0_ENABLE_POS)

/* DMIC and OD Status Register */
#define AUDIO_STATUS_BASE               0x40000E04
#define AUDIO_STATUS                    REG32_POINTER(AUDIO_STATUS_BASE)

/* AUDIO_STATUS bit positions */
#define AUDIO_STATUS_OD_STATUS_POS      11
#define AUDIO_STATUS_OD_UNDERRUN_FLAG_CLEAR_POS 10
#define AUDIO_STATUS_OD_UNDERRUN_FLAG_POS 9
#define AUDIO_STATUS_OD_DATA_REQ_FLAG_POS 8
#define AUDIO_STATUS_DMIC1_OVERRUN_FLAG_CLEAR_POS 6
#define AUDIO_STATUS_DMIC1_OVERRUN_FLAG_POS 5
#define AUDIO_STATUS_DMIC1_DATA_RDY_FLAG_POS 4
#define AUDIO_STATUS_DMIC0_OVERRUN_FLAG_CLEAR_POS 2
#define AUDIO_STATUS_DMIC0_OVERRUN_FLAG_POS 1
#define AUDIO_STATUS_DMIC0_DATA_RDY_FLAG_POS 0

/* AUDIO_STATUS bit-band aliases */
#define AUDIO_STATUS_OD_STATUS_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_OD_STATUS_POS))
#define AUDIO_STATUS_OD_UNDERRUN_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_OD_UNDERRUN_FLAG_CLEAR_POS))
#define AUDIO_STATUS_OD_UNDERRUN_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_OD_UNDERRUN_FLAG_POS))
#define AUDIO_STATUS_OD_DATA_REQ_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_OD_DATA_REQ_FLAG_POS))
#define AUDIO_STATUS_DMIC1_OVERRUN_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_DMIC1_OVERRUN_FLAG_CLEAR_POS))
#define AUDIO_STATUS_DMIC1_OVERRUN_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_DMIC1_OVERRUN_FLAG_POS))
#define AUDIO_STATUS_DMIC1_DATA_RDY_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_DMIC1_DATA_RDY_FLAG_POS))
#define AUDIO_STATUS_DMIC0_OVERRUN_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_DMIC0_OVERRUN_FLAG_CLEAR_POS))
#define AUDIO_STATUS_DMIC0_OVERRUN_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_DMIC0_OVERRUN_FLAG_POS))
#define AUDIO_STATUS_DMIC0_DATA_RDY_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIO_STATUS_BASE, AUDIO_STATUS_DMIC0_DATA_RDY_FLAG_POS))

/* AUDIO_STATUS settings */
#define OD_DISABLED_DEVICE_BITBAND      0x0
#define OD_ENABLED_DEVICE_BITBAND       0x1
#define OD_DISABLED_DEVICE              (OD_DISABLED_DEVICE_BITBAND << AUDIO_STATUS_OD_STATUS_POS)
#define OD_ENABLED_DEVICE               (OD_ENABLED_DEVICE_BITBAND << AUDIO_STATUS_OD_STATUS_POS)

#define OD_UNDERRUN_FLAG_CLEAR_BITBAND  0x1
#define OD_UNDERRUN_FLAG_CLEAR          (OD_UNDERRUN_FLAG_CLEAR_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_CLEAR_POS)

#define OD_UNDERRUN_NOT_DETECTED_BITBAND 0x0
#define OD_UNDERRUN_DETECTED_BITBAND    0x1
#define OD_UNDERRUN_NOT_DETECTED        (OD_UNDERRUN_NOT_DETECTED_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_POS)
#define OD_UNDERRUN_DETECTED            (OD_UNDERRUN_DETECTED_BITBAND << AUDIO_STATUS_OD_UNDERRUN_FLAG_POS)

#define OD_DATA_NOT_REQUIRED_BITBAND    0x0
#define OD_DATA_REQUIRED_BITBAND        0x1
#define OD_DATA_NOT_REQUIRED            (OD_DATA_NOT_REQUIRED_BITBAND << AUDIO_STATUS_OD_DATA_REQ_FLAG_POS)
#define OD_DATA_REQUIRED                (OD_DATA_REQUIRED_BITBAND << AUDIO_STATUS_OD_DATA_REQ_FLAG_POS)

#define DMIC1_OVERRUN_FLAG_CLEAR_BITBAND 0x1
#define DMIC1_OVERRUN_FLAG_CLEAR        (DMIC1_OVERRUN_FLAG_CLEAR_BITBAND << AUDIO_STATUS_DMIC1_OVERRUN_FLAG_CLEAR_POS)

#define DMIC1_OVERRUN_NOT_DETECTED_BITBAND 0x0
#define DMIC1_OVERRUN_DETECTED_BITBAND  0x1
#define DMIC1_OVERRUN_NOT_DETECTED      (DMIC1_OVERRUN_NOT_DETECTED_BITBAND << AUDIO_STATUS_DMIC1_OVERRUN_FLAG_POS)
#define DMIC1_OVERRUN_DETECTED          (DMIC1_OVERRUN_DETECTED_BITBAND << AUDIO_STATUS_DMIC1_OVERRUN_FLAG_POS)

#define DMIC1_DATA_NOT_READY_BITBAND    0x0
#define DMIC1_DATA_READY_BITBAND        0x1
#define DMIC1_DATA_NOT_READY            (DMIC1_DATA_NOT_READY_BITBAND << AUDIO_STATUS_DMIC1_DATA_RDY_FLAG_POS)
#define DMIC1_DATA_READY                (DMIC1_DATA_READY_BITBAND << AUDIO_STATUS_DMIC1_DATA_RDY_FLAG_POS)

#define DMIC0_OVERRUN_FLAG_CLEAR_BITBAND 0x1
#define DMIC0_OVERRUN_FLAG_CLEAR        (DMIC0_OVERRUN_FLAG_CLEAR_BITBAND << AUDIO_STATUS_DMIC0_OVERRUN_FLAG_CLEAR_POS)

#define DMIC0_OVERRUN_NOT_DETECTED_BITBAND 0x0
#define DMIC0_OVERRUN_DETECTED_BITBAND  0x1
#define DMIC0_OVERRUN_NOT_DETECTED      (DMIC0_OVERRUN_NOT_DETECTED_BITBAND << AUDIO_STATUS_DMIC0_OVERRUN_FLAG_POS)
#define DMIC0_OVERRUN_DETECTED          (DMIC0_OVERRUN_DETECTED_BITBAND << AUDIO_STATUS_DMIC0_OVERRUN_FLAG_POS)

#define DMIC0_DATA_NOT_READY_BITBAND    0x0
#define DMIC0_DATA_READY_BITBAND        0x1
#define DMIC0_DATA_NOT_READY            (DMIC0_DATA_NOT_READY_BITBAND << AUDIO_STATUS_DMIC0_DATA_RDY_FLAG_POS)
#define DMIC0_DATA_READY                (DMIC0_DATA_READY_BITBAND << AUDIO_STATUS_DMIC0_DATA_RDY_FLAG_POS)

/* DMIC Configuration Register */
#define AUDIO_DMIC_CFG_BASE             0x40000E08
#define AUDIO_DMIC_CFG                  REG32_POINTER(AUDIO_DMIC_CFG_BASE)

/* AUDIO_DMIC_CFG bit positions */
#define AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_POS 24
#define AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_MASK (0x1F << AUDIO_DMIC_CFG_DMIC1_FRAC_DELAY_POS)
#define AUDIO_DMIC_CFG_DMIC1_DELAY_POS  16
#define AUDIO_DMIC_CFG_DMIC1_DELAY_MASK (0xF << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define AUDIO_DMIC_CFG_DMIC1_DCRM_POS   12
#define AUDIO_DMIC_CFG_DMIC1_DCRM_MASK  (0x7 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)
#define AUDIO_DMIC_CFG_DMIC0_DCRM_POS   8
#define AUDIO_DMIC_CFG_DMIC0_DCRM_MASK  (0x7 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)
#define AUDIO_DMIC_CFG_DMIC1_CLK_EDGE_POS 1
#define AUDIO_DMIC_CFG_DMIC0_CLK_EDGE_POS 0

/* AUDIO_DMIC_CFG bit-band aliases */
#define AUDIO_DMIC_CFG_DMIC1_CLK_EDGE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_DMIC_CFG_BASE, AUDIO_DMIC_CFG_DMIC1_CLK_EDGE_POS))
#define AUDIO_DMIC_CFG_DMIC0_CLK_EDGE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIO_DMIC_CFG_BASE, AUDIO_DMIC_CFG_DMIC0_CLK_EDGE_POS))

/* AUDIO_DMIC_CFG settings */
#define DMIC1_DELAY_DISABLE             (0x0 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_0P125               (0x1 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_0P25                (0x2 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_0P375               (0x3 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_0P5                 (0x4 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_0P625               (0x5 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_0P75                (0x6 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_0P875               (0x7 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_1P0                 (0x8 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_1P125               (0x9 << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_1P25                (0xA << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_1P375               (0xB << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_1P5                 (0xC << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_1P625               (0xD << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_1P75                (0xE << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)
#define DMIC1_DELAY_1P875               (0xF << AUDIO_DMIC_CFG_DMIC1_DELAY_POS)

#define DMIC1_DCRM_CUTOFF_5HZ           (0x0 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)
#define DMIC1_DCRM_CUTOFF_10HZ          (0x1 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)
#define DMIC1_DCRM_CUTOFF_20HZ          (0x2 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)
#define DMIC1_DCRM_CUTOFF_40HZ          (0x3 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)
#define DMIC1_DCRM_CUTOFF_80HZ          (0x4 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)
#define DMIC1_DCRM_CUTOFF_160HZ         (0x5 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)
#define DMIC1_DCRM_CUTOFF_320HZ         (0x6 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)
#define DMIC1_DCRM_DISABLE              (0x7 << AUDIO_DMIC_CFG_DMIC1_DCRM_POS)

#define DMIC0_DCRM_CUTOFF_5HZ           (0x0 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)
#define DMIC0_DCRM_CUTOFF_10HZ          (0x1 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)
#define DMIC0_DCRM_CUTOFF_20HZ          (0x2 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)
#define DMIC0_DCRM_CUTOFF_40HZ          (0x3 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)
#define DMIC0_DCRM_CUTOFF_80HZ          (0x4 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)
#define DMIC0_DCRM_CUTOFF_160HZ         (0x5 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)
#define DMIC0_DCRM_CUTOFF_320HZ         (0x6 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)
#define DMIC0_DCRM_DISABLE              (0x7 << AUDIO_DMIC_CFG_DMIC0_DCRM_POS)

#define DMIC1_FALLING_EDGE_BITBAND      0x0
#define DMIC1_RISING_EDGE_BITBAND       0x1
#define DMIC1_FALLING_EDGE              (DMIC1_FALLING_EDGE_BITBAND << AUDIO_DMIC_CFG_DMIC1_CLK_EDGE_POS)
#define DMIC1_RISING_EDGE               (DMIC1_RISING_EDGE_BITBAND << AUDIO_DMIC_CFG_DMIC1_CLK_EDGE_POS)

#define DMIC0_FALLING_EDGE_BITBAND      0x0
#define DMIC0_RISING_EDGE_BITBAND       0x1
#define DMIC0_FALLING_EDGE              (DMIC0_FALLING_EDGE_BITBAND << AUDIO_DMIC_CFG_DMIC0_CLK_EDGE_POS)
#define DMIC0_RISING_EDGE               (DMIC0_RISING_EDGE_BITBAND << AUDIO_DMIC_CFG_DMIC0_CLK_EDGE_POS)

/* DMIC0 Gain Configuration Register */
#define AUDIO_DMIC0_GAIN_BASE           0x40000E0C
#define AUDIO_DMIC0_GAIN                REG32_POINTER(AUDIO_DMIC0_GAIN_BASE)

/* AUDIO_DMIC0_GAIN bit positions */
#define AUDIO_DMIC0_GAIN_GAIN_POS       0
#define AUDIO_DMIC0_GAIN_GAIN_MASK      (0xFFF << AUDIO_DMIC0_GAIN_GAIN_POS)

/* AUDIO_DMIC0_GAIN settings */
#define DMIC1_NOMINAL_GAIN              (0x800 << AUDIO_DMIC0_GAIN_GAIN_POS)

/* DMIC1 Gain Configuration Register */
#define AUDIO_DMIC1_GAIN_BASE           0x40000E10
#define AUDIO_DMIC1_GAIN                REG32_POINTER(AUDIO_DMIC1_GAIN_BASE)

/* AUDIO_DMIC1_GAIN bit positions */
#define AUDIO_DMIC1_GAIN_GAIN_POS       0
#define AUDIO_DMIC1_GAIN_GAIN_MASK      (0xFFF << AUDIO_DMIC1_GAIN_GAIN_POS)

/* AUDIO_DMIC1_GAIN settings */
#define DMIC0_NOMINAL_GAIN              (0x800 << AUDIO_DMIC1_GAIN_GAIN_POS)

/* DMIC0 and DMIC1 Data Register */
#define AUDIO_DMIC_DATA_BASE            0x40000E14
#define AUDIO_DMIC_DATA                 READONLY_REG32_POINTER(AUDIO_DMIC_DATA_BASE)

/* AUDIO_DMIC_DATA bit positions */
#define AUDIO_DMIC_DATA_DMIC1_DATA_POS  16
#define AUDIO_DMIC_DATA_DMIC1_DATA_MASK (0xFFFF << AUDIO_DMIC_DATA_DMIC1_DATA_POS)
#define AUDIO_DMIC_DATA_DMIC0_DATA_POS  0
#define AUDIO_DMIC_DATA_DMIC0_DATA_MASK (0xFFFF << AUDIO_DMIC_DATA_DMIC0_DATA_POS)

/* DMIC0 Data Register */
#define AUDIO_DMIC0_DATA_BASE           0x40000E18
#define AUDIO_DMIC0_DATA                READONLY_REG32_POINTER(AUDIO_DMIC0_DATA_BASE)

/* DMIC1 Data Register */
#define AUDIO_DMIC1_DATA_BASE           0x40000E1C
#define AUDIO_DMIC1_DATA                READONLY_REG32_POINTER(AUDIO_DMIC1_DATA_BASE)

/* Output Driver Configuration Register */
#define AUDIO_OD_CFG_BASE               0x40000E20
#define AUDIO_OD_CFG                    REG32_POINTER(AUDIO_OD_CFG_BASE)

/* AUDIO_OD_CFG bit positions */
#define AUDIO_OD_CFG_DCRM_POS           16
#define AUDIO_OD_CFG_DCRM_MASK          (0xF << AUDIO_OD_CFG_DCRM_POS)
#define AUDIO_OD_CFG_DITHER_POS         10
#define AUDIO_OD_CFG_CLK_EDGE_POS       0

/* AUDIO_OD_CFG bit-band aliases */
#define AUDIO_OD_CFG_DITHER_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(AUDIO_OD_CFG_BASE, AUDIO_OD_CFG_DITHER_POS))
#define AUDIO_OD_CFG_CLK_EDGE_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(AUDIO_OD_CFG_BASE, AUDIO_OD_CFG_CLK_EDGE_POS))

/* AUDIO_OD_CFG settings */
#define DCRM_DISABLE                    (0x0 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_1P25HZ              (0x1 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_2P5HZ               (0x2 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_3P75HZ              (0x3 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_5HZ                 (0x4 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_7P5HZ               (0x5 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_10HZ                (0x6 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_15HZ                (0x7 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_20HZ                (0x8 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_30HZ                (0x9 << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_40HZ                (0xA << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_60HZ                (0xB << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_80HZ                (0xC << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_120HZ               (0xD << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_160HZ               (0xE << AUDIO_OD_CFG_DCRM_POS)
#define DCRM_CUTOFF_240HZ               (0xF << AUDIO_OD_CFG_DCRM_POS)

#define DITHER_DISABLE_BITBAND          0x0
#define DITHER_ENABLE_BITBAND           0x1
#define DITHER_DISABLE                  (DITHER_DISABLE_BITBAND << AUDIO_OD_CFG_DITHER_POS)
#define DITHER_ENABLE                   (DITHER_ENABLE_BITBAND << AUDIO_OD_CFG_DITHER_POS)

#define OD_FALLING_EDGE_BITBAND         0x0
#define OD_RISING_EDGE_BITBAND          0x1
#define OD_FALLING_EDGE                 (OD_FALLING_EDGE_BITBAND << AUDIO_OD_CFG_CLK_EDGE_POS)
#define OD_RISING_EDGE                  (OD_RISING_EDGE_BITBAND << AUDIO_OD_CFG_CLK_EDGE_POS)

/* Output Driver Gain Configuration Register */
#define AUDIO_OD_GAIN_BASE              0x40000E24
#define AUDIO_OD_GAIN                   REG32_POINTER(AUDIO_OD_GAIN_BASE)

/* AUDIO_OD_GAIN bit positions */
#define AUDIO_OD_GAIN_GAIN_POS          0
#define AUDIO_OD_GAIN_GAIN_MASK         (0xFFF << AUDIO_OD_GAIN_GAIN_POS)

/* AUDIO_OD_GAIN settings */
#define OD_NOMINAL_GAIN                 (0x800 << AUDIO_OD_GAIN_GAIN_POS)

/* Output Driver Data Register */
#define AUDIO_OD_DATA_BASE              0x40000E28
#define AUDIO_OD_DATA                   REG32_POINTER(AUDIO_OD_DATA_BASE)

/* AUDIO_OD_DATA bit positions */
#define AUDIO_OD_DATA_DATA_POS          0
#define AUDIO_OD_DATA_DATA_MASK         (0xFFFFFFFF << AUDIO_OD_DATA_DATA_POS)
#define AUDIO_OD_DATA_DATA_RD_POS       0
#define AUDIO_OD_DATA_DATA_RD_MASK      (0xFFFFFFFF << AUDIO_OD_DATA_DATA_RD_POS)

/* Output Driver Sigma-delta Modulator Configuration Register */
#define AUDIO_SDM_CFG_BASE              0x40000E30
#define AUDIO_SDM_CFG                   REG32_POINTER(AUDIO_SDM_CFG_BASE)

/* AUDIO_SDM_CFG bit positions */
#define AUDIO_SDM_CFG_SDM_CFG_POS       0
#define AUDIO_SDM_CFG_SDM_CFG_MASK      (0xFFFFFFFF << AUDIO_SDM_CFG_SDM_CFG_POS)

/* AUDIO_SDM_CFG settings */
#define SDM_CFG_NORMAL                  (0x50012 << AUDIO_SDM_CFG_SDM_CFG_POS)

/* ----------------------------------------------------------------------------
 * CRC Generator Control
 * ------------------------------------------------------------------------- */

/* CRC Generator Control */
#define CRC_CTRL_BASE                   0x40000F00
#define CRC_CTRL                        REG32_POINTER(CRC_CTRL_BASE)

/* CRC_CTRL bit positions */
#define CRC_CTRL_FINAL_CRC_XOR_POS      4
#define CRC_CTRL_FINAL_CRC_REVERSE_POS  3
#define CRC_CTRL_BIT_ORDER_POS          2
#define CRC_CTRL_CRC_TYPE_POS           1
#define CRC_CTRL_BYTE_ORDER_POS         0

/* CRC_CTRL bit-band aliases */
#define CRC_CTRL_FINAL_CRC_XOR_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(CRC_CTRL_BASE, CRC_CTRL_FINAL_CRC_XOR_POS))
#define CRC_CTRL_FINAL_CRC_REVERSE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(CRC_CTRL_BASE, CRC_CTRL_FINAL_CRC_REVERSE_POS))
#define CRC_CTRL_BIT_ORDER_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(CRC_CTRL_BASE, CRC_CTRL_BIT_ORDER_POS))
#define CRC_CTRL_CRC_TYPE_BITBAND       REG32_POINTER(SYS_CALC_BITBAND(CRC_CTRL_BASE, CRC_CTRL_CRC_TYPE_POS))
#define CRC_CTRL_BYTE_ORDER_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(CRC_CTRL_BASE, CRC_CTRL_BYTE_ORDER_POS))

/* CRC_CTRL settings */
#define CRC_FINAL_XOR_STANDARD_BITBAND  0x0
#define CRC_FINAL_XOR_NON_STANDARD_BITBAND 0x1
#define CRC_FINAL_XOR_STANDARD          (CRC_FINAL_XOR_STANDARD_BITBAND << CRC_CTRL_FINAL_CRC_XOR_POS)
#define CRC_FINAL_XOR_NON_STANDARD      (CRC_FINAL_XOR_NON_STANDARD_BITBAND << CRC_CTRL_FINAL_CRC_XOR_POS)

#define CRC_FINAL_REVERSE_STANDARD_BITBAND 0x0
#define CRC_FINAL_REVERSE_NON_STANDARD_BITBAND 0x1
#define CRC_FINAL_REVERSE_STANDARD      (CRC_FINAL_REVERSE_STANDARD_BITBAND << CRC_CTRL_FINAL_CRC_REVERSE_POS)
#define CRC_FINAL_REVERSE_NON_STANDARD  (CRC_FINAL_REVERSE_NON_STANDARD_BITBAND << CRC_CTRL_FINAL_CRC_REVERSE_POS)

#define CRC_BIT_ORDER_STANDARD_BITBAND  0x0
#define CRC_BIT_ORDER_NON_STANDARD_BITBAND 0x1
#define CRC_BIT_ORDER_STANDARD          (CRC_BIT_ORDER_STANDARD_BITBAND << CRC_CTRL_BIT_ORDER_POS)
#define CRC_BIT_ORDER_NON_STANDARD      (CRC_BIT_ORDER_NON_STANDARD_BITBAND << CRC_CTRL_BIT_ORDER_POS)

#define CRC_CCITT_BITBAND               0x0
#define CRC_32_BITBAND                  0x1
#define CRC_CCITT                       (CRC_CCITT_BITBAND << CRC_CTRL_CRC_TYPE_POS)
#define CRC_32                          (CRC_32_BITBAND << CRC_CTRL_CRC_TYPE_POS)

#define CRC_BIG_ENDIAN_BITBAND          0x0
#define CRC_LITTLE_ENDIAN_BITBAND       0x1
#define CRC_BIG_ENDIAN                  (CRC_BIG_ENDIAN_BITBAND << CRC_CTRL_BYTE_ORDER_POS)
#define CRC_LITTLE_ENDIAN               (CRC_LITTLE_ENDIAN_BITBAND << CRC_CTRL_BYTE_ORDER_POS)

/* CRC Generator Current Value */
#define CRC_VALUE_BASE                  0x40000F04
#define CRC_VALUE                       REG32_POINTER(CRC_VALUE_BASE)

/* CRC Generator - Add 1 Bit */
#define CRC_ADD_1_BASE                  0x40000F08
#define CRC_ADD_1                       REG32_POINTER(CRC_ADD_1_BASE)

/* CRC_ADD_1 bit positions */
#define CRC_ADD_1_CRC_ADD_1_BIT_POS     0

/* CRC_ADD_1 bit-band aliases */
#define CRC_ADD_1_CRC_ADD_1_BIT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(CRC_ADD_1_BASE, CRC_ADD_1_CRC_ADD_1_BIT_POS))

/* CRC Generator - Add 1 Byte */
#define CRC_ADD_8_BASE                  0x40000F0C
#define CRC_ADD_8                       REG32_POINTER(CRC_ADD_8_BASE)

/* CRC Generator - Add 1 Half-word */
#define CRC_ADD_16_BASE                 0x40000F10
#define CRC_ADD_16                      REG32_POINTER(CRC_ADD_16_BASE)

/* CRC Generator - Add 3 Bytes */
#define CRC_ADD_24_BASE                 0x40000F14
#define CRC_ADD_24                      REG32_POINTER(CRC_ADD_24_BASE)

/* CRC_ADD_24 bit positions */
#define CRC_ADD_24_CRC_ADD_24_BITS_POS  0
#define CRC_ADD_24_CRC_ADD_24_BITS_MASK (0xFFFFFF << CRC_ADD_24_CRC_ADD_24_BITS_POS)

/* CRC Generator - Add 1 Word */
#define CRC_ADD_32_BASE                 0x40000F18
#define CRC_ADD_32                      REG32_POINTER(CRC_ADD_32_BASE)

/* CRC Generator Final Value */
#define CRC_FINAL_BASE                  0x40000F1C
#define CRC_FINAL                       READONLY_REG32_POINTER(CRC_FINAL_BASE)

/* ----------------------------------------------------------------------------
 * Audio Sink Clock Counters
 * ------------------------------------------------------------------------- */

/* Audio Sink Clock Control Register */
#define AUDIOSINK_CTRL_BASE             0x40001000
#define AUDIOSINK_CTRL                  REG32_POINTER(AUDIOSINK_CTRL_BASE)

/* AUDIOSINK_CTRL bit positions */
#define AUDIOSINK_CTRL_PHASE_CNT_START_NO_WAIT_POS 8
#define AUDIOSINK_CTRL_PERIOD_CNT_STATUS_POS 7
#define AUDIOSINK_CTRL_PERIOD_CNT_STOP_POS 6
#define AUDIOSINK_CTRL_PERIOD_CNT_START_POS 5
#define AUDIOSINK_CTRL_PHASE_CNT_MISSED_STATUS_POS 4
#define AUDIOSINK_CTRL_PHASE_CNT_STATUS_POS 3
#define AUDIOSINK_CTRL_PHASE_CNT_STOP_POS 2
#define AUDIOSINK_CTRL_PHASE_CNT_START_POS 1
#define AUDIOSINK_CTRL_CNT_RESET_POS    0

/* AUDIOSINK_CTRL bit-band aliases */
#define AUDIOSINK_CTRL_PHASE_CNT_START_NO_WAIT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_PHASE_CNT_START_NO_WAIT_POS))
#define AUDIOSINK_CTRL_PERIOD_CNT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_PERIOD_CNT_STATUS_POS))
#define AUDIOSINK_CTRL_PERIOD_CNT_STOP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_PERIOD_CNT_STOP_POS))
#define AUDIOSINK_CTRL_PERIOD_CNT_START_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_PERIOD_CNT_START_POS))
#define AUDIOSINK_CTRL_PHASE_CNT_MISSED_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_PHASE_CNT_MISSED_STATUS_POS))
#define AUDIOSINK_CTRL_PHASE_CNT_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_PHASE_CNT_STATUS_POS))
#define AUDIOSINK_CTRL_PHASE_CNT_STOP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_PHASE_CNT_STOP_POS))
#define AUDIOSINK_CTRL_PHASE_CNT_START_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_PHASE_CNT_START_POS))
#define AUDIOSINK_CTRL_CNT_RESET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(AUDIOSINK_CTRL_BASE, AUDIOSINK_CTRL_CNT_RESET_POS))

/* AUDIOSINK_CTRL settings */
#define PHASE_CNT_START_NO_WAIT_BITBAND 0x1
#define PHASE_CNT_START_NO_WAIT         (PHASE_CNT_START_NO_WAIT_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_START_NO_WAIT_POS)

#define PERIOD_CNT_IDLE_BITBAND         0x0
#define PERIOD_CNT_BUSY_BITBAND         0x1
#define PERIOD_CNT_IDLE                 (PERIOD_CNT_IDLE_BITBAND << AUDIOSINK_CTRL_PERIOD_CNT_STATUS_POS)
#define PERIOD_CNT_BUSY                 (PERIOD_CNT_BUSY_BITBAND << AUDIOSINK_CTRL_PERIOD_CNT_STATUS_POS)

#define PERIOD_CNT_STOP_BITBAND         0x1
#define PERIOD_CNT_STOP                 (PERIOD_CNT_STOP_BITBAND << AUDIOSINK_CTRL_PERIOD_CNT_STOP_POS)

#define PERIOD_CNT_START_BITBAND        0x1
#define PERIOD_CNT_START                (PERIOD_CNT_START_BITBAND << AUDIOSINK_CTRL_PERIOD_CNT_START_POS)

#define PHASE_CNT_NORMAL_BITBAND        0x0
#define PHASE_CNT_MISSED_BITBAND        0x1
#define PHASE_CNT_NORMAL                (PHASE_CNT_NORMAL_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_MISSED_STATUS_POS)
#define PHASE_CNT_MISSED                (PHASE_CNT_MISSED_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_MISSED_STATUS_POS)

#define PHASE_CNT_IDLE_BITBAND          0x0
#define PHASE_CNT_BUSY_BITBAND          0x1
#define PHASE_CNT_IDLE                  (PHASE_CNT_IDLE_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_STATUS_POS)
#define PHASE_CNT_BUSY                  (PHASE_CNT_BUSY_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_STATUS_POS)

#define PHASE_CNT_STOP_BITBAND          0x1
#define PHASE_CNT_STOP                  (PHASE_CNT_STOP_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_STOP_POS)

#define PHASE_CNT_START_BITBAND         0x1
#define PHASE_CNT_START                 (PHASE_CNT_START_BITBAND << AUDIOSINK_CTRL_PHASE_CNT_START_POS)

#define CNT_RESET_BITBAND               0x1
#define CNT_RESET                       (CNT_RESET_BITBAND << AUDIOSINK_CTRL_CNT_RESET_POS)

/* Audio Sink Clock Configuration Register */
#define AUDIOSINK_CFG_BASE              0x40001004
#define AUDIOSINK_CFG                   REG32_POINTER(AUDIOSINK_CFG_BASE)

/* AUDIOSINK_CFG bit positions */
#define AUDIOSINK_CFG_PERIODS_CFG_POS   0
#define AUDIOSINK_CFG_PERIODS_CFG_MASK  (0xF << AUDIOSINK_CFG_PERIODS_CFG_POS)

/* AUDIOSINK_CFG settings */
#define AUDIO_SINK_PERIODS_1            (0x0 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_2            (0x1 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_3            (0x2 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_4            (0x3 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_5            (0x4 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_6            (0x5 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_7            (0x6 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_8            (0x7 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_9            (0x8 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_10           (0x9 << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_11           (0xA << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_12           (0xB << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_13           (0xC << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_14           (0xD << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_15           (0xE << AUDIOSINK_CFG_PERIODS_CFG_POS)
#define AUDIO_SINK_PERIODS_16           (0xF << AUDIOSINK_CFG_PERIODS_CFG_POS)

/* Audio Sink Clock Counter Register */
#define AUDIOSINK_CNT_BASE              0x40001008
#define AUDIOSINK_CNT                   READONLY_REG32_POINTER(AUDIOSINK_CNT_BASE)

/* AUDIOSINK_CNT bit positions */
#define AUDIOSINK_CNT_CNT_POS           0
#define AUDIOSINK_CNT_CNT_MASK          (0xFFF << AUDIOSINK_CNT_CNT_POS)

/* Audio Sink Clock Phase Counter Register */
#define AUDIOSINK_PHASE_CNT_BASE        0x4000100C
#define AUDIOSINK_PHASE_CNT             REG32_POINTER(AUDIOSINK_PHASE_CNT_BASE)

/* AUDIOSINK_PHASE_CNT bit positions */
#define AUDIOSINK_PHASE_CNT_PHASE_CNT_POS 0
#define AUDIOSINK_PHASE_CNT_PHASE_CNT_MASK (0xFFFF << AUDIOSINK_PHASE_CNT_PHASE_CNT_POS)

/* Audio Sink Clock Period Counter Register */
#define AUDIOSINK_PERIOD_CNT_BASE       0x40001010
#define AUDIOSINK_PERIOD_CNT            REG32_POINTER(AUDIOSINK_PERIOD_CNT_BASE)

/* AUDIOSINK_PERIOD_CNT bit positions */
#define AUDIOSINK_PERIOD_CNT_PERIOD_CNT_POS 0
#define AUDIOSINK_PERIOD_CNT_PERIOD_CNT_MASK (0xFFFF << AUDIOSINK_PERIOD_CNT_PERIOD_CNT_POS)

/* ----------------------------------------------------------------------------
 * ASRC Configuration and Control
 * ------------------------------------------------------------------------- */

/* ASRC Control Register */
#define ASRC_CTRL_BASE                  0x40001100
#define ASRC_CTRL                       REG32_POINTER(ASRC_CTRL_BASE)

/* ASRC_CTRL bit positions */
#define ASRC_CTRL_ASRC_PROC_STATUS_POS  14
#define ASRC_CTRL_ASRC_OUT_REQ_POS      13
#define ASRC_CTRL_ASRC_IN_REQ_POS       12
#define ASRC_CTRL_ASRC_UPDATE_ERR_POS   11
#define ASRC_CTRL_ASRC_IN_ERR_POS       10
#define ASRC_CTRL_ASRC_UPDATE_ERR_CLR_POS 9
#define ASRC_CTRL_ASRC_IN_ERR_CLR_POS   8
#define ASRC_CTRL_ASRC_RESET_POS        3
#define ASRC_CTRL_ASRC_EN_STATUS_POS    2
#define ASRC_CTRL_ASRC_DISABLE_POS      1
#define ASRC_CTRL_ASRC_ENABLE_POS       0

/* ASRC_CTRL bit-band aliases */
#define ASRC_CTRL_ASRC_PROC_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_PROC_STATUS_POS))
#define ASRC_CTRL_ASRC_OUT_REQ_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_OUT_REQ_POS))
#define ASRC_CTRL_ASRC_IN_REQ_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_IN_REQ_POS))
#define ASRC_CTRL_ASRC_UPDATE_ERR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_UPDATE_ERR_POS))
#define ASRC_CTRL_ASRC_IN_ERR_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_IN_ERR_POS))
#define ASRC_CTRL_ASRC_UPDATE_ERR_CLR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_UPDATE_ERR_CLR_POS))
#define ASRC_CTRL_ASRC_IN_ERR_CLR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_IN_ERR_CLR_POS))
#define ASRC_CTRL_ASRC_RESET_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_RESET_POS))
#define ASRC_CTRL_ASRC_EN_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_EN_STATUS_POS))
#define ASRC_CTRL_ASRC_DISABLE_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_DISABLE_POS))
#define ASRC_CTRL_ASRC_ENABLE_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(ASRC_CTRL_BASE, ASRC_CTRL_ASRC_ENABLE_POS))

/* ASRC_CTRL settings */
#define ASRC_IDLE_BITBAND               0x0
#define ASRC_BUSY_BITBAND               0x1
#define ASRC_IDLE                       (ASRC_IDLE_BITBAND << ASRC_CTRL_ASRC_PROC_STATUS_POS)
#define ASRC_BUSY                       (ASRC_BUSY_BITBAND << ASRC_CTRL_ASRC_PROC_STATUS_POS)

#define INT_IDLE_ASRC_OUT_BITBAND       0x0
#define INT_RDY_ASRC_OUT_BITBAND        0x1
#define INT_IDLE_ASRC_OUT               (INT_IDLE_ASRC_OUT_BITBAND << ASRC_CTRL_ASRC_OUT_REQ_POS)
#define INT_RDY_ASRC_OUT                (INT_RDY_ASRC_OUT_BITBAND << ASRC_CTRL_ASRC_OUT_REQ_POS)

#define INT_IDLE_ASRC_IN_BITBAND        0x0
#define INT_RDY_ASRC_IN_BITBAND         0x1
#define INT_IDLE_ASRC_IN                (INT_IDLE_ASRC_IN_BITBAND << ASRC_CTRL_ASRC_IN_REQ_POS)
#define INT_RDY_ASRC_IN                 (INT_RDY_ASRC_IN_BITBAND << ASRC_CTRL_ASRC_IN_REQ_POS)

#define INT_IDLE_ASRC_PH_CNT_ERR_BITBAND 0x0
#define INT_PENDING_ASRC_PH_CNT_ERR_BITBAND 0x1
#define INT_IDLE_ASRC_PH_CNT_ERR        (INT_IDLE_ASRC_PH_CNT_ERR_BITBAND << ASRC_CTRL_ASRC_UPDATE_ERR_POS)
#define INT_PENDING_ASRC_PH_CNT_ERR     (INT_PENDING_ASRC_PH_CNT_ERR_BITBAND << ASRC_CTRL_ASRC_UPDATE_ERR_POS)

#define INT_IDLE_ASRC_IN_ERR_BITBAND    0x0
#define INT_PENDING_ASRC_IN_ERR_BITBAND 0x1
#define INT_IDLE_ASRC_IN_ERR            (INT_IDLE_ASRC_IN_ERR_BITBAND << ASRC_CTRL_ASRC_IN_ERR_POS)
#define INT_PENDING_ASRC_IN_ERR         (INT_PENDING_ASRC_IN_ERR_BITBAND << ASRC_CTRL_ASRC_IN_ERR_POS)

#define CLR_ASRC_UPDATE_ERR_BITBAND     0x1
#define CLR_ASRC_UPDATE_ERR             (CLR_ASRC_UPDATE_ERR_BITBAND << ASRC_CTRL_ASRC_UPDATE_ERR_CLR_POS)

#define CLR_ASRC_IN_ERR_BITBAND         0x1
#define CLR_ASRC_IN_ERR                 (CLR_ASRC_IN_ERR_BITBAND << ASRC_CTRL_ASRC_IN_ERR_CLR_POS)

#define ASRC_RESET_BITBAND              0x1
#define ASRC_RESET                      (ASRC_RESET_BITBAND << ASRC_CTRL_ASRC_RESET_POS)

#define ASRC_DISABLED_BITBAND           0x0
#define ASRC_ENABLED_BITBAND            0x1
#define ASRC_DISABLED                   (ASRC_DISABLED_BITBAND << ASRC_CTRL_ASRC_EN_STATUS_POS)
#define ASRC_ENABLED                    (ASRC_ENABLED_BITBAND << ASRC_CTRL_ASRC_EN_STATUS_POS)

#define ASRC_DISABLE_BITBAND            0x1
#define ASRC_DISABLE                    (ASRC_DISABLE_BITBAND << ASRC_CTRL_ASRC_DISABLE_POS)

#define ASRC_ENABLE_BITBAND             0x1
#define ASRC_ENABLE                     (ASRC_ENABLE_BITBAND << ASRC_CTRL_ASRC_ENABLE_POS)

/* ASRC Interrupt Mask Register */
#define ASRC_INT_ENABLE_BASE            0x40001104
#define ASRC_INT_ENABLE                 REG32_POINTER(ASRC_INT_ENABLE_BASE)

/* ASRC_INT_ENABLE bit positions */
#define ASRC_INT_ENABLE_ASRC_UPDATE_ERR_POS 3
#define ASRC_INT_ENABLE_ASRC_IN_ERR_POS 2
#define ASRC_INT_ENABLE_ASRC_OUT_REQ_POS 1
#define ASRC_INT_ENABLE_ASRC_IN_REQ_POS 0

/* ASRC_INT_ENABLE bit-band aliases */
#define ASRC_INT_ENABLE_ASRC_UPDATE_ERR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ASRC_INT_ENABLE_BASE, ASRC_INT_ENABLE_ASRC_UPDATE_ERR_POS))
#define ASRC_INT_ENABLE_ASRC_IN_ERR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ASRC_INT_ENABLE_BASE, ASRC_INT_ENABLE_ASRC_IN_ERR_POS))
#define ASRC_INT_ENABLE_ASRC_OUT_REQ_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ASRC_INT_ENABLE_BASE, ASRC_INT_ENABLE_ASRC_OUT_REQ_POS))
#define ASRC_INT_ENABLE_ASRC_IN_REQ_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ASRC_INT_ENABLE_BASE, ASRC_INT_ENABLE_ASRC_IN_REQ_POS))

/* ASRC_INT_ENABLE settings */
#define INT_DIS_ASRC_UPDATE_ERR_BITBAND 0x0
#define INT_EBL_ASRC_UPDATE_ERR_BITBAND 0x1
#define INT_DIS_ASRC_UPDATE_ERR         (INT_DIS_ASRC_UPDATE_ERR_BITBAND << ASRC_INT_ENABLE_ASRC_UPDATE_ERR_POS)
#define INT_EBL_ASRC_UPDATE_ERR         (INT_EBL_ASRC_UPDATE_ERR_BITBAND << ASRC_INT_ENABLE_ASRC_UPDATE_ERR_POS)

#define INT_DIS_ASRC_IN_ERR_BITBAND     0x0
#define INT_EBL_ASRC_IN_ERR_BITBAND     0x1
#define INT_DIS_ASRC_IN_ERR             (INT_DIS_ASRC_IN_ERR_BITBAND << ASRC_INT_ENABLE_ASRC_IN_ERR_POS)
#define INT_EBL_ASRC_IN_ERR             (INT_EBL_ASRC_IN_ERR_BITBAND << ASRC_INT_ENABLE_ASRC_IN_ERR_POS)

#define INT_DIS_ASRC_OUT_BITBAND        0x0
#define INT_EBL_ASRC_OUT_BITBAND        0x1
#define INT_DIS_ASRC_OUT                (INT_DIS_ASRC_OUT_BITBAND << ASRC_INT_ENABLE_ASRC_OUT_REQ_POS)
#define INT_EBL_ASRC_OUT                (INT_EBL_ASRC_OUT_BITBAND << ASRC_INT_ENABLE_ASRC_OUT_REQ_POS)

#define INT_DIS_ASRC_IN_BITBAND         0x0
#define INT_EBL_ASRC_IN_BITBAND         0x1
#define INT_DIS_ASRC_IN                 (INT_DIS_ASRC_IN_BITBAND << ASRC_INT_ENABLE_ASRC_IN_REQ_POS)
#define INT_EBL_ASRC_IN                 (INT_EBL_ASRC_IN_BITBAND << ASRC_INT_ENABLE_ASRC_IN_REQ_POS)

/* ASRC Output Data Register */
#define ASRC_OUT_BASE                   0x40001108
#define ASRC_OUT                        REG32_POINTER(ASRC_OUT_BASE)

/* ASRC Input Data Register */
#define ASRC_IN_BASE                    0x4000110C
#define ASRC_IN                         REG32_POINTER(ASRC_IN_BASE)

/* ASRC Configuration Register */
#define ASRC_CFG_BASE                   0x40001110
#define ASRC_CFG                        REG32_POINTER(ASRC_CFG_BASE)

/* ASRC_CFG bit positions */
#define ASRC_CFG_WDF_TYPE_POS           2
#define ASRC_CFG_ASRC_MODE_POS          0
#define ASRC_CFG_ASRC_MODE_MASK         (0x3 << ASRC_CFG_ASRC_MODE_POS)

/* ASRC_CFG bit-band aliases */
#define ASRC_CFG_WDF_TYPE_BITBAND       REG32_POINTER(SYS_CALC_BITBAND(ASRC_CFG_BASE, ASRC_CFG_WDF_TYPE_POS))

/* ASRC_CFG settings */
#define LOW_DELAY_BITBAND               0x0
#define WIDE_BAND_BITBAND               0x1
#define LOW_DELAY                       (LOW_DELAY_BITBAND << ASRC_CFG_WDF_TYPE_POS)
#define WIDE_BAND                       (WIDE_BAND_BITBAND << ASRC_CFG_WDF_TYPE_POS)

#define ASRC_INT_MODE                   (0x0 << ASRC_CFG_ASRC_MODE_POS)
#define ASRC_DEC_MODE1                  (0x1 << ASRC_CFG_ASRC_MODE_POS)
#define ASRC_DEC_MODE2                  (0x2 << ASRC_CFG_ASRC_MODE_POS)
#define ASRC_DEC_MODE3                  (0x3 << ASRC_CFG_ASRC_MODE_POS)

/* ASRC output sample counter */
#define ASRC_OUTPUT_CNT_BASE            0x40001114
#define ASRC_OUTPUT_CNT                 REG32_POINTER(ASRC_OUTPUT_CNT_BASE)

/* ASRC phase counter increment value */
#define ASRC_PHASE_INC_BASE             0x40001118
#define ASRC_PHASE_INC                  REG32_POINTER(ASRC_PHASE_INC_BASE)

/* ASRC phase counter */
#define ASRC_PHASE_CNT_BASE             0x4000111C
#define ASRC_PHASE_CNT                  REG32_POINTER(ASRC_PHASE_CNT_BASE)

/* ASRC State Memory 0 to 29 (32 bit) */
#define ASRC_STATE_MEM_BASE             0x40001120
#define ASRC_STATE_MEM                  REG32_POINTER(ASRC_STATE_MEM_BASE)

/* ----------------------------------------------------------------------------
 * Analog-to-Digital Converter and Battery Monitoring
 * ------------------------------------------------------------------------- */

/* ADC conversion result for channel 0 to 7 in trimmer mode */
#define ADC_DATA_TRIM_CH_BASE           0x40001200
#define ADC_DATA_TRIM_CH                READONLY_REG32_POINTER(ADC_DATA_TRIM_CH_BASE)

/* ADC_DATA_TRIM_CH bit positions */
#define ADC_DATA_TRIM_CH_DATA_POS       0
#define ADC_DATA_TRIM_CH_DATA_MASK      (0x3FFF << ADC_DATA_TRIM_CH_DATA_POS)

/* ADC conversion result for channel 0 to 7 in audio mode (signed) */
#define ADC_DATA_AUDIO_CH_BASE          0x40001220
#define ADC_DATA_AUDIO_CH               READONLY_REG32_POINTER(ADC_DATA_AUDIO_CH_BASE)

/* ADC input selection for channel 0 to 7 */
#define ADC_INPUT_SEL_BASE              0x40001240
#define ADC_INPUT_SEL                   REG32_POINTER(ADC_INPUT_SEL_BASE)

/* ADC_INPUT_SEL bit positions */
#define ADC_INPUT_SEL_POS_INPUT_SEL_POS 4
#define ADC_INPUT_SEL_POS_INPUT_SEL_MASK (0x7 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)
#define ADC_INPUT_SEL_NEG_INPUT_SEL_POS 0
#define ADC_INPUT_SEL_NEG_INPUT_SEL_MASK (0x7 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)

/* ADC_INPUT_SEL settings */
#define ADC_POS_INPUT_DIO0              (0x0 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)
#define ADC_POS_INPUT_DIO1              (0x1 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)
#define ADC_POS_INPUT_DIO2              (0x2 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)
#define ADC_POS_INPUT_DIO3              (0x3 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)
#define ADC_POS_INPUT_AOUT              (0x4 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)
#define ADC_POS_INPUT_VDDC              (0x5 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)
#define ADC_POS_INPUT_VBAT_DIV2         (0x6 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)
#define ADC_POS_INPUT_GND               (0x7 << ADC_INPUT_SEL_POS_INPUT_SEL_POS)

#define ADC_NEG_INPUT_DIO0              (0x0 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)
#define ADC_NEG_INPUT_DIO1              (0x1 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)
#define ADC_NEG_INPUT_DIO2              (0x2 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)
#define ADC_NEG_INPUT_DIO3              (0x3 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)
#define ADC_NEG_INPUT_AOUT              (0x4 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)
#define ADC_NEG_INPUT_VDDC              (0x5 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)
#define ADC_NEG_INPUT_VBAT_DIV2         (0x6 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)
#define ADC_NEG_INPUT_GND               (0x7 << ADC_INPUT_SEL_NEG_INPUT_SEL_POS)

/* ADC Configuration Register */
#define ADC_CFG_BASE                    0x40001260
#define ADC_CFG                         REG32_POINTER(ADC_CFG_BASE)

/* ADC_CFG bit positions */
#define ADC_CFG_DUTY_DIVIDER_POS        5
#define ADC_CFG_CONTINUOUS_MODE_POS     4
#define ADC_CFG_FREQ_POS                0
#define ADC_CFG_FREQ_MASK               (0xF << ADC_CFG_FREQ_POS)

/* ADC_CFG bit-band aliases */
#define ADC_CFG_DUTY_DIVIDER_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(ADC_CFG_BASE, ADC_CFG_DUTY_DIVIDER_POS))
#define ADC_CFG_CONTINUOUS_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ADC_CFG_BASE, ADC_CFG_CONTINUOUS_MODE_POS))

/* ADC_CFG settings */
#define ADC_VBAT_DIV2_NORMAL_BITBAND    0x0
#define ADC_VBAT_DIV2_DUTY_BITBAND      0x1
#define ADC_VBAT_DIV2_NORMAL            (ADC_VBAT_DIV2_NORMAL_BITBAND << ADC_CFG_DUTY_DIVIDER_POS)
#define ADC_VBAT_DIV2_DUTY              (ADC_VBAT_DIV2_DUTY_BITBAND << ADC_CFG_DUTY_DIVIDER_POS)

#define ADC_NORMAL_BITBAND              0x0
#define ADC_CONTINUOUS_BITBAND          0x1
#define ADC_NORMAL                      (ADC_NORMAL_BITBAND << ADC_CFG_CONTINUOUS_MODE_POS)
#define ADC_CONTINUOUS                  (ADC_CONTINUOUS_BITBAND << ADC_CFG_CONTINUOUS_MODE_POS)

#define ADC_DISABLE                     (0x0 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_200                (0x1 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_400                (0x2 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_640                (0x3 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_800                (0x4 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_1600               (0x5 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_3200               (0x6 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_6400               (0x7 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_20H                (0x8 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_40H                (0x9 << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_80H                (0xA << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_128H               (0xB << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_160H               (0xC << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_320H               (0xD << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_640H               (0xE << ADC_CFG_FREQ_POS)
#define ADC_PRESCALE_1280H              (0xF << ADC_CFG_FREQ_POS)

/* ADC conversion result for ADC GND */
#define ADC_OFFSET_BASE                 0x40001264
#define ADC_OFFSET                      REG32_POINTER(ADC_OFFSET_BASE)

/* ADC_OFFSET bit positions */
#define ADC_OFFSET_DATA_POS             0
#define ADC_OFFSET_DATA_MASK            (0x7FFF << ADC_OFFSET_DATA_POS)

/* Battery Monitoring Configuration Register */
#define ADC_BATMON_CFG_BASE             0x40001270
#define ADC_BATMON_CFG                  REG32_POINTER(ADC_BATMON_CFG_BASE)

/* ADC_BATMON_CFG bit positions */
#define ADC_BATMON_CFG_ALARM_COUNT_VALUE_POS 16
#define ADC_BATMON_CFG_ALARM_COUNT_VALUE_MASK (0xFF << ADC_BATMON_CFG_ALARM_COUNT_VALUE_POS)
#define ADC_BATMON_CFG_SUPPLY_THRESHOLD_POS 8
#define ADC_BATMON_CFG_SUPPLY_THRESHOLD_MASK (0xFF << ADC_BATMON_CFG_SUPPLY_THRESHOLD_POS)
#define ADC_BATMON_CFG_SUPPLY_SRC_POS   0

/* ADC_BATMON_CFG bit-band aliases */
#define ADC_BATMON_CFG_SUPPLY_SRC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_CFG_BASE, ADC_BATMON_CFG_SUPPLY_SRC_POS))

/* ADC_BATMON_CFG settings */
#define BATMON_ALARM_NONE               (0x0 << ADC_BATMON_CFG_ALARM_COUNT_VALUE_POS)
#define BATMON_ALARM_COUNT1             (0x1 << ADC_BATMON_CFG_ALARM_COUNT_VALUE_POS)
#define BATMON_ALARM_COUNT255           (0xFF << ADC_BATMON_CFG_ALARM_COUNT_VALUE_POS)

#define SUPPLY_THRESHOLD_LOW            (0x0 << ADC_BATMON_CFG_SUPPLY_THRESHOLD_POS)
#define SUPPLY_THRESHOLD_MID            (0x80 << ADC_BATMON_CFG_SUPPLY_THRESHOLD_POS)
#define SUPPLY_THRESHOLD_HIGH           (0xFF << ADC_BATMON_CFG_SUPPLY_THRESHOLD_POS)

#define BATMON_CH6_BITBAND              0x0
#define BATMON_CH7_BITBAND              0x1
#define BATMON_CH6                      (BATMON_CH6_BITBAND << ADC_BATMON_CFG_SUPPLY_SRC_POS)
#define BATMON_CH7                      (BATMON_CH7_BITBAND << ADC_BATMON_CFG_SUPPLY_SRC_POS)

/* ADC / BATMON Interrupt Mask Register */
#define ADC_BATMON_INT_ENABLE_BASE      0x40001274
#define ADC_BATMON_INT_ENABLE           REG32_POINTER(ADC_BATMON_INT_ENABLE_BASE)

/* ADC_BATMON_INT_ENABLE bit positions */
#define ADC_BATMON_INT_ENABLE_BATMON_ALARM_INT_ENABLE_POS 4
#define ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS 1
#define ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_MASK (0x7 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)
#define ADC_BATMON_INT_ENABLE_ADC_INT_ENABLE_POS 0

/* ADC_BATMON_INT_ENABLE bit-band aliases */
#define ADC_BATMON_INT_ENABLE_BATMON_ALARM_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_INT_ENABLE_BASE, ADC_BATMON_INT_ENABLE_BATMON_ALARM_INT_ENABLE_POS))
#define ADC_BATMON_INT_ENABLE_ADC_INT_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_INT_ENABLE_BASE, ADC_BATMON_INT_ENABLE_ADC_INT_ENABLE_POS))

/* ADC_BATMON_INT_ENABLE settings */
#define INT_DIS_BATMON_ALARM_BITBAND    0x0
#define INT_EBL_BATMON_ALARM_BITBAND    0x1
#define INT_DIS_BATMON_ALARM            (INT_DIS_BATMON_ALARM_BITBAND << ADC_BATMON_INT_ENABLE_BATMON_ALARM_INT_ENABLE_POS)
#define INT_EBL_BATMON_ALARM            (INT_EBL_BATMON_ALARM_BITBAND << ADC_BATMON_INT_ENABLE_BATMON_ALARM_INT_ENABLE_POS)

#define ADC_INT_CH0                     (0x0 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)
#define ADC_INT_CH1                     (0x1 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)
#define ADC_INT_CH2                     (0x2 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)
#define ADC_INT_CH3                     (0x3 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)
#define ADC_INT_CH4                     (0x4 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)
#define ADC_INT_CH5                     (0x5 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)
#define ADC_INT_CH6                     (0x6 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)
#define ADC_INT_CH7                     (0x7 << ADC_BATMON_INT_ENABLE_ADC_INT_CH_NUM_POS)

#define INT_DIS_ADC_BITBAND             0x0
#define INT_EBL_ADC_BITBAND             0x1
#define INT_DIS_ADC                     (INT_DIS_ADC_BITBAND << ADC_BATMON_INT_ENABLE_ADC_INT_ENABLE_POS)
#define INT_EBL_ADC                     (INT_EBL_ADC_BITBAND << ADC_BATMON_INT_ENABLE_ADC_INT_ENABLE_POS)

/* Battery Monitoring Status Register */
#define ADC_BATMON_COUNT_VAL_BASE       0x40001278
#define ADC_BATMON_COUNT_VAL            READONLY_REG32_POINTER(ADC_BATMON_COUNT_VAL_BASE)

/* ADC_BATMON_COUNT_VAL bit positions */
#define ADC_BATMON_COUNT_VAL_SUPPLY_COUNT_VALUE_POS 0
#define ADC_BATMON_COUNT_VAL_SUPPLY_COUNT_VALUE_MASK (0xFF << ADC_BATMON_COUNT_VAL_SUPPLY_COUNT_VALUE_POS)

/* ADC / BATMON Status Register */
#define ADC_BATMON_STATUS_BASE          0x4000127C
#define ADC_BATMON_STATUS               REG32_POINTER(ADC_BATMON_STATUS_BASE)

/* ADC_BATMON_STATUS bit positions */
#define ADC_BATMON_STATUS_BATMON_ALARM_CLEAR_POS 12
#define ADC_BATMON_STATUS_ADC_OVERRUN_CLEAR_POS 9
#define ADC_BATMON_STATUS_ADC_READY_CLEAR_POS 8
#define ADC_BATMON_STATUS_BATMON_ALARM_STAT_POS 4
#define ADC_BATMON_STATUS_ADC_OVERRUN_STAT_POS 1
#define ADC_BATMON_STATUS_ADC_READY_STAT_POS 0

/* ADC_BATMON_STATUS bit-band aliases */
#define ADC_BATMON_STATUS_BATMON_ALARM_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_STATUS_BASE, ADC_BATMON_STATUS_BATMON_ALARM_CLEAR_POS))
#define ADC_BATMON_STATUS_ADC_OVERRUN_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_STATUS_BASE, ADC_BATMON_STATUS_ADC_OVERRUN_CLEAR_POS))
#define ADC_BATMON_STATUS_ADC_READY_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_STATUS_BASE, ADC_BATMON_STATUS_ADC_READY_CLEAR_POS))
#define ADC_BATMON_STATUS_BATMON_ALARM_STAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_STATUS_BASE, ADC_BATMON_STATUS_BATMON_ALARM_STAT_POS))
#define ADC_BATMON_STATUS_ADC_OVERRUN_STAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_STATUS_BASE, ADC_BATMON_STATUS_ADC_OVERRUN_STAT_POS))
#define ADC_BATMON_STATUS_ADC_READY_STAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ADC_BATMON_STATUS_BASE, ADC_BATMON_STATUS_ADC_READY_STAT_POS))

/* ADC_BATMON_STATUS settings */
#define BATMON_ALARM_CLEAR_BITBAND      0x1
#define BATMON_ALARM_CLEAR              (BATMON_ALARM_CLEAR_BITBAND << ADC_BATMON_STATUS_BATMON_ALARM_CLEAR_POS)

#define ADC_OVERRUN_CLEAR_BITBAND       0x1
#define ADC_OVERRUN_CLEAR               (ADC_OVERRUN_CLEAR_BITBAND << ADC_BATMON_STATUS_ADC_OVERRUN_CLEAR_POS)

#define ADC_READY_CLEAR_BITBAND         0x1
#define ADC_READY_CLEAR                 (ADC_READY_CLEAR_BITBAND << ADC_BATMON_STATUS_ADC_READY_CLEAR_POS)

#define BATMON_ALARM_FALSE_BITBAND      0x0
#define BATMON_ALARM_TRUE_BITBAND       0x1
#define BATMON_ALARM_FALSE              (BATMON_ALARM_FALSE_BITBAND << ADC_BATMON_STATUS_BATMON_ALARM_STAT_POS)
#define BATMON_ALARM_TRUE               (BATMON_ALARM_TRUE_BITBAND << ADC_BATMON_STATUS_BATMON_ALARM_STAT_POS)

#define ADC_OVERRUN_FALSE_BITBAND       0x0
#define ADC_OVERRUN_TRUE_BITBAND        0x1
#define ADC_OVERRUN_FALSE               (ADC_OVERRUN_FALSE_BITBAND << ADC_BATMON_STATUS_ADC_OVERRUN_STAT_POS)
#define ADC_OVERRUN_TRUE                (ADC_OVERRUN_TRUE_BITBAND << ADC_BATMON_STATUS_ADC_OVERRUN_STAT_POS)

#define ADC_READY_FALSE_BITBAND         0x0
#define ADC_READY_TRUE_BITBAND          0x1
#define ADC_READY_FALSE                 (ADC_READY_FALSE_BITBAND << ADC_BATMON_STATUS_ADC_READY_STAT_POS)
#define ADC_READY_TRUE                  (ADC_READY_TRUE_BITBAND << ADC_BATMON_STATUS_ADC_READY_STAT_POS)

/* ----------------------------------------------------------------------------
 * ACS domain (Analog Bridge Access)
 * ------------------------------------------------------------------------- */

/* Bandgap Configuration / Control register */
#define ACS_BG_CTRL_BASE                0x40001300
#define ACS_BG_CTRL                     REG32_POINTER(ACS_BG_CTRL_BASE)

/* ACS_BG_CTRL bit positions */
#define ACS_BG_CTRL_SLOPE_TRIM_POS      8
#define ACS_BG_CTRL_SLOPE_TRIM_MASK     (0x1F << ACS_BG_CTRL_SLOPE_TRIM_POS)
#define ACS_BG_CTRL_VTRIM_POS           0
#define ACS_BG_CTRL_VTRIM_MASK          (0x3F << ACS_BG_CTRL_VTRIM_POS)

/* ACS_BG_CTRL settings */
#define BG_SLOPE_TRIM_VALUE             (0xB << ACS_BG_CTRL_SLOPE_TRIM_POS)

#define BG_TRIM_0P675V                  (0x0 << ACS_BG_CTRL_VTRIM_POS)
#define BG_TRIM_0P678V                  (0x1 << ACS_BG_CTRL_VTRIM_POS)
#define BG_TRIM_0P748V                  (0x1D << ACS_BG_CTRL_VTRIM_POS)
#define BG_TRIM_0P750V                  (0x1E << ACS_BG_CTRL_VTRIM_POS)
#define BG_TRIM_0P753V                  (0x1F << ACS_BG_CTRL_VTRIM_POS)
#define BG_TRIM_0P830V                  (0x3E << ACS_BG_CTRL_VTRIM_POS)
#define BG_TRIM_0P833V                  (0x3F << ACS_BG_CTRL_VTRIM_POS)

/* DC-DC / LDO Supply Configuration / Control register */
#define ACS_VCC_CTRL_BASE               0x40001304
#define ACS_VCC_CTRL                    REG32_POINTER(ACS_VCC_CTRL_BASE)

/* ACS_VCC_CTRL bit positions */
#define ACS_VCC_CTRL_ICH_TRIM_POS       16
#define ACS_VCC_CTRL_ICH_TRIM_MASK      (0xF << ACS_VCC_CTRL_ICH_TRIM_POS)
#define ACS_VCC_CTRL_CCM_ENABLE_POS     11
#define ACS_VCC_CTRL_PULSE_CTRL_POS     10
#define ACS_VCC_CTRL_CHARGE_CTRL_POS    9
#define ACS_VCC_CTRL_BUCK_ENABLE_POS    8
#define ACS_VCC_CTRL_VTRIM_POS          0
#define ACS_VCC_CTRL_VTRIM_MASK         (0x1F << ACS_VCC_CTRL_VTRIM_POS)

/* ACS_VCC_CTRL bit-band aliases */
#define ACS_VCC_CTRL_CCM_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VCC_CTRL_BASE, ACS_VCC_CTRL_CCM_ENABLE_POS))
#define ACS_VCC_CTRL_PULSE_CTRL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VCC_CTRL_BASE, ACS_VCC_CTRL_PULSE_CTRL_POS))
#define ACS_VCC_CTRL_CHARGE_CTRL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VCC_CTRL_BASE, ACS_VCC_CTRL_CHARGE_CTRL_POS))
#define ACS_VCC_CTRL_BUCK_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VCC_CTRL_BASE, ACS_VCC_CTRL_BUCK_ENABLE_POS))

/* ACS_VCC_CTRL settings */
#define VCC_ICHTRIM_16MA                (0x0 << ACS_VCC_CTRL_ICH_TRIM_POS)
#define VCC_ICHTRIM_32MA                (0x1 << ACS_VCC_CTRL_ICH_TRIM_POS)
#define VCC_ICHTRIM_64MA                (0x3 << ACS_VCC_CTRL_ICH_TRIM_POS)
#define VCC_ICHTRIM_80MA                (0x4 << ACS_VCC_CTRL_ICH_TRIM_POS)
#define VCC_ICHTRIM_256MA               (0xF << ACS_VCC_CTRL_ICH_TRIM_POS)

#define VCC_DCM_MODE_BITBAND            0x0
#define VCC_CCM_MODE_BITBAND            0x1
#define VCC_DCM_MODE                    (VCC_DCM_MODE_BITBAND << ACS_VCC_CTRL_CCM_ENABLE_POS)
#define VCC_CCM_MODE                    (VCC_CCM_MODE_BITBAND << ACS_VCC_CTRL_CCM_ENABLE_POS)

#define VCC_SINGLE_PULSE_BITBAND        0x0
#define VCC_MULTI_PULSE_BITBAND         0x1
#define VCC_SINGLE_PULSE                (VCC_SINGLE_PULSE_BITBAND << ACS_VCC_CTRL_PULSE_CTRL_POS)
#define VCC_MULTI_PULSE                 (VCC_MULTI_PULSE_BITBAND << ACS_VCC_CTRL_PULSE_CTRL_POS)

#define VCC_CONSTANT_CHARGE_BITBAND     0x0
#define VCC_CONSTANT_IMAX_BITBAND       0x1
#define VCC_CONSTANT_CHARGE             (VCC_CONSTANT_CHARGE_BITBAND << ACS_VCC_CTRL_CHARGE_CTRL_POS)
#define VCC_CONSTANT_IMAX               (VCC_CONSTANT_IMAX_BITBAND << ACS_VCC_CTRL_CHARGE_CTRL_POS)

#define VCC_LDO_BITBAND                 0x0
#define VCC_BUCK_BITBAND                0x1
#define VCC_LDO                         (VCC_LDO_BITBAND << ACS_VCC_CTRL_BUCK_ENABLE_POS)
#define VCC_BUCK                        (VCC_BUCK_BITBAND << ACS_VCC_CTRL_BUCK_ENABLE_POS)

#define VCC_TRIM_1P00V                  (0x0 << ACS_VCC_CTRL_VTRIM_POS)
#define VCC_TRIM_1P05V                  (0x5 << ACS_VCC_CTRL_VTRIM_POS)
#define VCC_TRIM_1P10V                  (0xA << ACS_VCC_CTRL_VTRIM_POS)
#define VCC_TRIM_1P15V                  (0xF << ACS_VCC_CTRL_VTRIM_POS)
#define VCC_TRIM_1P20V                  (0x14 << ACS_VCC_CTRL_VTRIM_POS)
#define VCC_TRIM_1P25V                  (0x19 << ACS_VCC_CTRL_VTRIM_POS)
#define VCC_TRIM_1P31V                  (0x1F << ACS_VCC_CTRL_VTRIM_POS)

/* Analog Voltage and Flash Charge Pump Configuration / Control register */
#define ACS_VDDA_CP_CTRL_BASE           0x40001308
#define ACS_VDDA_CP_CTRL                REG32_POINTER(ACS_VDDA_CP_CTRL_BASE)

/* ACS_VDDA_CP_CTRL bit positions */
#define ACS_VDDA_CP_CTRL_PTRIM_POS      0
#define ACS_VDDA_CP_CTRL_PTRIM_MASK     (0x3 << ACS_VDDA_CP_CTRL_PTRIM_POS)

/* ACS_VDDA_CP_CTRL settings */
#define VDDA_PTRIM_4MA                  (0x0 << ACS_VDDA_CP_CTRL_PTRIM_POS)
#define VDDA_PTRIM_8MA                  (0x1 << ACS_VDDA_CP_CTRL_PTRIM_POS)
#define VDDA_PTRIM_12MA                 (0x2 << ACS_VDDA_CP_CTRL_PTRIM_POS)
#define VDDA_PTRIM_16MA                 (0x3 << ACS_VDDA_CP_CTRL_PTRIM_POS)

/* Digital Core Voltage Regulator Configuration / Control register */
#define ACS_VDDC_CTRL_BASE              0x4000130C
#define ACS_VDDC_CTRL                   REG32_POINTER(ACS_VDDC_CTRL_BASE)

/* ACS_VDDC_CTRL bit positions */
#define ACS_VDDC_CTRL_STANDBY_VTRIM_POS 16
#define ACS_VDDC_CTRL_STANDBY_VTRIM_MASK (0x3F << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define ACS_VDDC_CTRL_ENABLE_LOW_BIAS_POS 13
#define ACS_VDDC_CTRL_SLEEP_CLAMP_POS   12
#define ACS_VDDC_CTRL_VTRIM_POS         0
#define ACS_VDDC_CTRL_VTRIM_MASK        (0x3F << ACS_VDDC_CTRL_VTRIM_POS)

/* ACS_VDDC_CTRL bit-band aliases */
#define ACS_VDDC_CTRL_ENABLE_LOW_BIAS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDC_CTRL_BASE, ACS_VDDC_CTRL_ENABLE_LOW_BIAS_POS))
#define ACS_VDDC_CTRL_SLEEP_CLAMP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDC_CTRL_BASE, ACS_VDDC_CTRL_SLEEP_CLAMP_POS))

/* ACS_VDDC_CTRL settings */
#define VDDC_STANDBY_TRIM_0P75V         (0x0 << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_0P76V         (0x1 << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_1P00V         (0x19 << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_1P08V         (0x21 << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_1P10V         (0x23 << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_1P15V         (0x28 << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_1P20V         (0x2D << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_1P25V         (0x32 << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_1P32V         (0x39 << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)
#define VDDC_STANDBY_TRIM_1P38V         (0x3F << ACS_VDDC_CTRL_STANDBY_VTRIM_POS)

#define VDDC_NOMINAL_BIAS_BITBAND       0x0
#define VDDC_LOW_BIAS_BITBAND           0x1
#define VDDC_NOMINAL_BIAS               (VDDC_NOMINAL_BIAS_BITBAND << ACS_VDDC_CTRL_ENABLE_LOW_BIAS_POS)
#define VDDC_LOW_BIAS                   (VDDC_LOW_BIAS_BITBAND << ACS_VDDC_CTRL_ENABLE_LOW_BIAS_POS)

#define VDDC_SLEEP_HIZ_BITBAND          0x0
#define VDDC_SLEEP_GND_BITBAND          0x1
#define VDDC_SLEEP_HIZ                  (VDDC_SLEEP_HIZ_BITBAND << ACS_VDDC_CTRL_SLEEP_CLAMP_POS)
#define VDDC_SLEEP_GND                  (VDDC_SLEEP_GND_BITBAND << ACS_VDDC_CTRL_SLEEP_CLAMP_POS)

#define VDDC_TRIM_0P75V                 (0x0 << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_0P76V                 (0x1 << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_1P00V                 (0x19 << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_1P08V                 (0x21 << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_1P10V                 (0x23 << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_1P15V                 (0x28 << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_1P20V                 (0x2D << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_1P25V                 (0x32 << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_1P32V                 (0x39 << ACS_VDDC_CTRL_VTRIM_POS)
#define VDDC_TRIM_1P38V                 (0x3F << ACS_VDDC_CTRL_VTRIM_POS)

/* Memories Voltage Regulator Configuration / Control register */
#define ACS_VDDM_CTRL_BASE              0x40001310
#define ACS_VDDM_CTRL                   REG32_POINTER(ACS_VDDM_CTRL_BASE)

/* ACS_VDDM_CTRL bit positions */
#define ACS_VDDM_CTRL_STANDBY_VTRIM_POS 16
#define ACS_VDDM_CTRL_STANDBY_VTRIM_MASK (0x3F << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define ACS_VDDM_CTRL_ENABLE_LOW_BIAS_POS 13
#define ACS_VDDM_CTRL_SLEEP_CLAMP_POS   12
#define ACS_VDDM_CTRL_VTRIM_POS         0
#define ACS_VDDM_CTRL_VTRIM_MASK        (0x3F << ACS_VDDM_CTRL_VTRIM_POS)

/* ACS_VDDM_CTRL bit-band aliases */
#define ACS_VDDM_CTRL_ENABLE_LOW_BIAS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDM_CTRL_BASE, ACS_VDDM_CTRL_ENABLE_LOW_BIAS_POS))
#define ACS_VDDM_CTRL_SLEEP_CLAMP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDM_CTRL_BASE, ACS_VDDM_CTRL_SLEEP_CLAMP_POS))

/* ACS_VDDM_CTRL settings */
#define VDDM_STANDBY_TRIM_0P75V         (0x0 << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_0P76V         (0x1 << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_1P00V         (0x19 << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_1P08V         (0x21 << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_1P10V         (0x23 << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_1P15V         (0x28 << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_1P20V         (0x2D << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_1P25V         (0x32 << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_1P32V         (0x39 << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)
#define VDDM_STANDBY_TRIM_1P38V         (0x3F << ACS_VDDM_CTRL_STANDBY_VTRIM_POS)

#define VDDM_NOMINAL_BIAS_BITBAND       0x0
#define VDDM_LOW_BIAS_BITBAND           0x1
#define VDDM_NOMINAL_BIAS               (VDDM_NOMINAL_BIAS_BITBAND << ACS_VDDM_CTRL_ENABLE_LOW_BIAS_POS)
#define VDDM_LOW_BIAS                   (VDDM_LOW_BIAS_BITBAND << ACS_VDDM_CTRL_ENABLE_LOW_BIAS_POS)

#define VDDM_SLEEP_HIZ_BITBAND          0x0
#define VDDM_SLEEP_GND_BITBAND          0x1
#define VDDM_SLEEP_HIZ                  (VDDM_SLEEP_HIZ_BITBAND << ACS_VDDM_CTRL_SLEEP_CLAMP_POS)
#define VDDM_SLEEP_GND                  (VDDM_SLEEP_GND_BITBAND << ACS_VDDM_CTRL_SLEEP_CLAMP_POS)

#define VDDM_TRIM_0P75V                 (0x0 << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_0P76V                 (0x1 << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_1P00V                 (0x19 << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_1P08V                 (0x21 << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_1P10V                 (0x23 << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_1P15V                 (0x28 << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_1P20V                 (0x2D << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_1P25V                 (0x32 << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_1P32V                 (0x39 << ACS_VDDM_CTRL_VTRIM_POS)
#define VDDM_TRIM_1P38V                 (0x3F << ACS_VDDM_CTRL_VTRIM_POS)

/* RF Block Regulator Configuration / Control register */
#define ACS_VDDRF_CTRL_BASE             0x40001314
#define ACS_VDDRF_CTRL                  REG32_POINTER(ACS_VDDRF_CTRL_BASE)

/* ACS_VDDRF_CTRL bit positions */
#define ACS_VDDRF_CTRL_READY_POS        24
#define ACS_VDDRF_CTRL_CLAMP_POS        12
#define ACS_VDDRF_CTRL_ENABLE_POS       8
#define ACS_VDDRF_CTRL_VTRIM_POS        0
#define ACS_VDDRF_CTRL_VTRIM_MASK       (0x3F << ACS_VDDRF_CTRL_VTRIM_POS)

/* ACS_VDDRF_CTRL bit-band aliases */
#define ACS_VDDRF_CTRL_READY_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDRF_CTRL_BASE, ACS_VDDRF_CTRL_READY_POS))
#define ACS_VDDRF_CTRL_CLAMP_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDRF_CTRL_BASE, ACS_VDDRF_CTRL_CLAMP_POS))
#define ACS_VDDRF_CTRL_ENABLE_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDRF_CTRL_BASE, ACS_VDDRF_CTRL_ENABLE_POS))

/* ACS_VDDRF_CTRL settings */
#define VDDRF_NOT_READY_BITBAND         0x0
#define VDDRF_READY_BITBAND             0x1
#define VDDRF_NOT_READY                 (VDDRF_NOT_READY_BITBAND << ACS_VDDRF_CTRL_READY_POS)
#define VDDRF_READY                     (VDDRF_READY_BITBAND << ACS_VDDRF_CTRL_READY_POS)

#define VDDRF_DISABLE_HIZ_BITBAND       0x0
#define VDDRF_DISABLE_GND_BITBAND       0x1
#define VDDRF_DISABLE_HIZ               (VDDRF_DISABLE_HIZ_BITBAND << ACS_VDDRF_CTRL_CLAMP_POS)
#define VDDRF_DISABLE_GND               (VDDRF_DISABLE_GND_BITBAND << ACS_VDDRF_CTRL_CLAMP_POS)

#define VDDRF_DISABLE_BITBAND           0x0
#define VDDRF_ENABLE_BITBAND            0x1
#define VDDRF_DISABLE                   (VDDRF_DISABLE_BITBAND << ACS_VDDRF_CTRL_ENABLE_POS)
#define VDDRF_ENABLE                    (VDDRF_ENABLE_BITBAND << ACS_VDDRF_CTRL_ENABLE_POS)

#define VDDRF_TRIM_0P75V                (0x0 << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_0P76V                (0x1 << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_1P00V                (0x19 << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_1P08V                (0x21 << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_1P10V                (0x23 << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_1P15V                (0x28 << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_1P20V                (0x2D << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_1P25V                (0x32 << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_1P32V                (0x39 << ACS_VDDRF_CTRL_VTRIM_POS)
#define VDDRF_TRIM_1P38V                (0x3F << ACS_VDDRF_CTRL_VTRIM_POS)

/* RF Block Regulator Configuration / Control register */
#define ACS_VDDPA_CTRL_BASE             0x40001318
#define ACS_VDDPA_CTRL                  REG32_POINTER(ACS_VDDPA_CTRL_BASE)

/* ACS_VDDPA_CTRL bit positions */
#define ACS_VDDPA_CTRL_VDDPA_SW_CTRL_POS 12
#define ACS_VDDPA_CTRL_ENABLE_ISENSE_POS 9
#define ACS_VDDPA_CTRL_ENABLE_POS       8
#define ACS_VDDPA_CTRL_VTRIM_POS        0
#define ACS_VDDPA_CTRL_VTRIM_MASK       (0x3F << ACS_VDDPA_CTRL_VTRIM_POS)

/* ACS_VDDPA_CTRL bit-band aliases */
#define ACS_VDDPA_CTRL_VDDPA_SW_CTRL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDPA_CTRL_BASE, ACS_VDDPA_CTRL_VDDPA_SW_CTRL_POS))
#define ACS_VDDPA_CTRL_ENABLE_ISENSE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDPA_CTRL_BASE, ACS_VDDPA_CTRL_ENABLE_ISENSE_POS))
#define ACS_VDDPA_CTRL_ENABLE_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDPA_CTRL_BASE, ACS_VDDPA_CTRL_ENABLE_POS))

/* ACS_VDDPA_CTRL settings */
#define VDDPA_SW_HIZ_BITBAND            0x0
#define VDDPA_SW_VDDRF_BITBAND          0x1
#define VDDPA_SW_HIZ                    (VDDPA_SW_HIZ_BITBAND << ACS_VDDPA_CTRL_VDDPA_SW_CTRL_POS)
#define VDDPA_SW_VDDRF                  (VDDPA_SW_VDDRF_BITBAND << ACS_VDDPA_CTRL_VDDPA_SW_CTRL_POS)

#define VDDPA_ISENSE_DISABLE_BITBAND    0x0
#define VDDPA_ISENSE_ENABLE_BITBAND     0x1
#define VDDPA_ISENSE_DISABLE            (VDDPA_ISENSE_DISABLE_BITBAND << ACS_VDDPA_CTRL_ENABLE_ISENSE_POS)
#define VDDPA_ISENSE_ENABLE             (VDDPA_ISENSE_ENABLE_BITBAND << ACS_VDDPA_CTRL_ENABLE_ISENSE_POS)

#define VDDPA_DISABLE_BITBAND           0x0
#define VDDPA_ENABLE_BITBAND            0x1
#define VDDPA_DISABLE                   (VDDPA_DISABLE_BITBAND << ACS_VDDPA_CTRL_ENABLE_POS)
#define VDDPA_ENABLE                    (VDDPA_ENABLE_BITBAND << ACS_VDDPA_CTRL_ENABLE_POS)

#define VDDPA_TRIM_1P05V                (0x0 << ACS_VDDPA_CTRL_VTRIM_POS)
#define VDDPA_TRIM_1P06V                (0x1 << ACS_VDDPA_CTRL_VTRIM_POS)
#define VDDPA_TRIM_1P59V                (0x36 << ACS_VDDPA_CTRL_VTRIM_POS)
#define VDDPA_TRIM_1P60V                (0x37 << ACS_VDDPA_CTRL_VTRIM_POS)
#define VDDPA_TRIM_1P61V                (0x38 << ACS_VDDPA_CTRL_VTRIM_POS)
#define VDDPA_TRIM_1P68V                (0x3F << ACS_VDDPA_CTRL_VTRIM_POS)

/* Retention Regulator Configuration / Control register */
#define ACS_VDDRET_CTRL_BASE            0x4000131C
#define ACS_VDDRET_CTRL                 REG32_POINTER(ACS_VDDRET_CTRL_BASE)

/* ACS_VDDRET_CTRL bit positions */
#define ACS_VDDRET_CTRL_VDDMRET_VTRIM_POS 17
#define ACS_VDDRET_CTRL_VDDMRET_VTRIM_MASK (0x3 << ACS_VDDRET_CTRL_VDDMRET_VTRIM_POS)
#define ACS_VDDRET_CTRL_VDDMRET_ENABLE_POS 16
#define ACS_VDDRET_CTRL_VDDTRET_VTRIM_POS 9
#define ACS_VDDRET_CTRL_VDDTRET_VTRIM_MASK (0x3 << ACS_VDDRET_CTRL_VDDTRET_VTRIM_POS)
#define ACS_VDDRET_CTRL_VDDTRET_ENABLE_POS 8
#define ACS_VDDRET_CTRL_VDDCRET_VTRIM_POS 1
#define ACS_VDDRET_CTRL_VDDCRET_VTRIM_MASK (0x3 << ACS_VDDRET_CTRL_VDDCRET_VTRIM_POS)
#define ACS_VDDRET_CTRL_VDDCRET_ENABLE_POS 0

/* ACS_VDDRET_CTRL bit-band aliases */
#define ACS_VDDRET_CTRL_VDDMRET_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDRET_CTRL_BASE, ACS_VDDRET_CTRL_VDDMRET_ENABLE_POS))
#define ACS_VDDRET_CTRL_VDDTRET_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDRET_CTRL_BASE, ACS_VDDRET_CTRL_VDDTRET_ENABLE_POS))
#define ACS_VDDRET_CTRL_VDDCRET_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_VDDRET_CTRL_BASE, ACS_VDDRET_CTRL_VDDCRET_ENABLE_POS))

/* ACS_VDDRET_CTRL settings */
#define VDDMRET_TRIM_VALUE              (0x3 << ACS_VDDRET_CTRL_VDDMRET_VTRIM_POS)

#define VDDMRET_DISABLE_BITBAND         0x0
#define VDDMRET_ENABLE_BITBAND          0x1
#define VDDMRET_DISABLE                 (VDDMRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDMRET_ENABLE_POS)
#define VDDMRET_ENABLE                  (VDDMRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDMRET_ENABLE_POS)

#define VDDTRET_TRIM_VALUE              (0x3 << ACS_VDDRET_CTRL_VDDTRET_VTRIM_POS)

#define VDDTRET_DISABLE_BITBAND         0x0
#define VDDTRET_ENABLE_BITBAND          0x1
#define VDDTRET_DISABLE                 (VDDTRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDTRET_ENABLE_POS)
#define VDDTRET_ENABLE                  (VDDTRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDTRET_ENABLE_POS)

#define VDDCRET_TRIM_VALUE              (0x3 << ACS_VDDRET_CTRL_VDDCRET_VTRIM_POS)

#define VDDCRET_DISABLE_BITBAND         0x0
#define VDDCRET_ENABLE_BITBAND          0x1
#define VDDCRET_DISABLE                 (VDDCRET_DISABLE_BITBAND << ACS_VDDRET_CTRL_VDDCRET_ENABLE_POS)
#define VDDCRET_ENABLE                  (VDDCRET_ENABLE_BITBAND << ACS_VDDRET_CTRL_VDDCRET_ENABLE_POS)

/* RC Oscillator Configuration / Control register */
#define ACS_RCOSC_CTRL_BASE             0x40001320
#define ACS_RCOSC_CTRL                  REG32_POINTER(ACS_RCOSC_CTRL_BASE)

/* ACS_RCOSC_CTRL bit positions */
#define ACS_RCOSC_CTRL_CLOCK_MULT_POS   18
#define ACS_RCOSC_CTRL_RC_OSC_EN_POS    16
#define ACS_RCOSC_CTRL_FTRIM_FLAG_POS   15
#define ACS_RCOSC_CTRL_FTRIM_START_POS  8
#define ACS_RCOSC_CTRL_FTRIM_START_MASK (0x3F << ACS_RCOSC_CTRL_FTRIM_START_POS)
#define ACS_RCOSC_CTRL_FTRIM_32K_ADJ_POS 6
#define ACS_RCOSC_CTRL_FTRIM_32K_POS    0
#define ACS_RCOSC_CTRL_FTRIM_32K_MASK   (0x3F << ACS_RCOSC_CTRL_FTRIM_32K_POS)

/* ACS_RCOSC_CTRL bit-band aliases */
#define ACS_RCOSC_CTRL_CLOCK_MULT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RCOSC_CTRL_BASE, ACS_RCOSC_CTRL_CLOCK_MULT_POS))
#define ACS_RCOSC_CTRL_RC_OSC_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RCOSC_CTRL_BASE, ACS_RCOSC_CTRL_RC_OSC_EN_POS))
#define ACS_RCOSC_CTRL_FTRIM_FLAG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RCOSC_CTRL_BASE, ACS_RCOSC_CTRL_FTRIM_FLAG_POS))
#define ACS_RCOSC_CTRL_FTRIM_32K_ADJ_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RCOSC_CTRL_BASE, ACS_RCOSC_CTRL_FTRIM_32K_ADJ_POS))

/* ACS_RCOSC_CTRL settings */
#define RC_START_OSC_3MHZ_BITBAND       0x0
#define RC_START_OSC_12MHZ_BITBAND      0x1
#define RC_START_OSC_3MHZ               (RC_START_OSC_3MHZ_BITBAND << ACS_RCOSC_CTRL_CLOCK_MULT_POS)
#define RC_START_OSC_12MHZ              (RC_START_OSC_12MHZ_BITBAND << ACS_RCOSC_CTRL_CLOCK_MULT_POS)

#define RC_OSC_DISABLE_BITBAND          0x0
#define RC_OSC_ENABLE_BITBAND           0x1
#define RC_OSC_DISABLE                  (RC_OSC_DISABLE_BITBAND << ACS_RCOSC_CTRL_RC_OSC_EN_POS)
#define RC_OSC_ENABLE                   (RC_OSC_ENABLE_BITBAND << ACS_RCOSC_CTRL_RC_OSC_EN_POS)

#define RC_OSC_UNCALIBRATED_BITBAND     0x0
#define RC_OSC_CALIBRATED_BITBAND       0x1
#define RC_OSC_UNCALIBRATED             (RC_OSC_UNCALIBRATED_BITBAND << ACS_RCOSC_CTRL_FTRIM_FLAG_POS)
#define RC_OSC_CALIBRATED               (RC_OSC_CALIBRATED_BITBAND << ACS_RCOSC_CTRL_FTRIM_FLAG_POS)

#define RC_START_OSC_M48                (0x0 << ACS_RCOSC_CTRL_FTRIM_START_POS)
#define RC_START_OSC_M46P5              (0x1 << ACS_RCOSC_CTRL_FTRIM_START_POS)
#define RC_START_OSC_NOM                (0x20 << ACS_RCOSC_CTRL_FTRIM_START_POS)
#define RC_START_OSC_P46P5              (0x3F << ACS_RCOSC_CTRL_FTRIM_START_POS)

#define RC_OSC_RANGE_NOM_BITBAND        0x0
#define RC_OSC_RANGE_M25_BITBAND        0x1
#define RC_OSC_RANGE_NOM                (RC_OSC_RANGE_NOM_BITBAND << ACS_RCOSC_CTRL_FTRIM_32K_ADJ_POS)
#define RC_OSC_RANGE_M25                (RC_OSC_RANGE_M25_BITBAND << ACS_RCOSC_CTRL_FTRIM_32K_ADJ_POS)

#define RC_OSC_M48                      (0x0 << ACS_RCOSC_CTRL_FTRIM_32K_POS)
#define RC_OSC_M46P5                    (0x1 << ACS_RCOSC_CTRL_FTRIM_32K_POS)
#define RC_OSC_NOM                      (0x20 << ACS_RCOSC_CTRL_FTRIM_32K_POS)
#define RC_OSC_P46P5                    (0x3F << ACS_RCOSC_CTRL_FTRIM_32K_POS)

/* XTAL 32 kHz configuration register */
#define ACS_XTAL32K_CTRL_BASE           0x40001324
#define ACS_XTAL32K_CTRL                REG32_POINTER(ACS_XTAL32K_CTRL_BASE)

/* ACS_XTAL32K_CTRL bit positions */
#define ACS_XTAL32K_CTRL_READY_POS      24
#define ACS_XTAL32K_CTRL_XIN_CAP_BYPASS_EN_POS 18
#define ACS_XTAL32K_CTRL_EN_AMPL_CTRL_POS 17
#define ACS_XTAL32K_CTRL_FORCE_READY_POS 16
#define ACS_XTAL32K_CTRL_CLOAD_TRIM_POS 8
#define ACS_XTAL32K_CTRL_CLOAD_TRIM_MASK (0x3F << ACS_XTAL32K_CTRL_CLOAD_TRIM_POS)
#define ACS_XTAL32K_CTRL_ITRIM_POS      4
#define ACS_XTAL32K_CTRL_ITRIM_MASK     (0xF << ACS_XTAL32K_CTRL_ITRIM_POS)
#define ACS_XTAL32K_CTRL_IBOOST_POS     1
#define ACS_XTAL32K_CTRL_ENABLE_POS     0

/* ACS_XTAL32K_CTRL bit-band aliases */
#define ACS_XTAL32K_CTRL_READY_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_XTAL32K_CTRL_BASE, ACS_XTAL32K_CTRL_READY_POS))
#define ACS_XTAL32K_CTRL_XIN_CAP_BYPASS_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_XTAL32K_CTRL_BASE, ACS_XTAL32K_CTRL_XIN_CAP_BYPASS_EN_POS))
#define ACS_XTAL32K_CTRL_EN_AMPL_CTRL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_XTAL32K_CTRL_BASE, ACS_XTAL32K_CTRL_EN_AMPL_CTRL_POS))
#define ACS_XTAL32K_CTRL_FORCE_READY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_XTAL32K_CTRL_BASE, ACS_XTAL32K_CTRL_FORCE_READY_POS))
#define ACS_XTAL32K_CTRL_IBOOST_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_XTAL32K_CTRL_BASE, ACS_XTAL32K_CTRL_IBOOST_POS))
#define ACS_XTAL32K_CTRL_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_XTAL32K_CTRL_BASE, ACS_XTAL32K_CTRL_ENABLE_POS))

/* ACS_XTAL32K_CTRL settings */
#define XTAL32K_NOT_OK_BITBAND          0x0
#define XTAL32K_OK_BITBAND              0x1
#define XTAL32K_NOT_OK                  (XTAL32K_NOT_OK_BITBAND << ACS_XTAL32K_CTRL_READY_POS)
#define XTAL32K_OK                      (XTAL32K_OK_BITBAND << ACS_XTAL32K_CTRL_READY_POS)

#define XTAL32K_XIN_CAP_BYPASS_DISABLE_BITBAND 0x0
#define XTAL32K_XIN_CAP_BYPASS_ENABLE_BITBAND 0x1
#define XTAL32K_XIN_CAP_BYPASS_DISABLE  (XTAL32K_XIN_CAP_BYPASS_DISABLE_BITBAND << ACS_XTAL32K_CTRL_XIN_CAP_BYPASS_EN_POS)
#define XTAL32K_XIN_CAP_BYPASS_ENABLE   (XTAL32K_XIN_CAP_BYPASS_ENABLE_BITBAND << ACS_XTAL32K_CTRL_XIN_CAP_BYPASS_EN_POS)

#define XTAL32K_AMPL_CTRL_DISABLE_BITBAND 0x0
#define XTAL32K_AMPL_CTRL_ENABLE_BITBAND 0x1
#define XTAL32K_AMPL_CTRL_DISABLE       (XTAL32K_AMPL_CTRL_DISABLE_BITBAND << ACS_XTAL32K_CTRL_EN_AMPL_CTRL_POS)
#define XTAL32K_AMPL_CTRL_ENABLE        (XTAL32K_AMPL_CTRL_ENABLE_BITBAND << ACS_XTAL32K_CTRL_EN_AMPL_CTRL_POS)

#define XTAL32K_NOT_FORCE_READY_BITBAND 0x0
#define XTAL32K_FORCE_READY_BITBAND     0x1
#define XTAL32K_NOT_FORCE_READY         (XTAL32K_NOT_FORCE_READY_BITBAND << ACS_XTAL32K_CTRL_FORCE_READY_POS)
#define XTAL32K_FORCE_READY             (XTAL32K_FORCE_READY_BITBAND << ACS_XTAL32K_CTRL_FORCE_READY_POS)

#define XTAL32K_CTRIM_0P0PF             (0x0 << ACS_XTAL32K_CTRL_CLOAD_TRIM_POS)
#define XTAL32K_CTRIM_0P4PF             (0x1 << ACS_XTAL32K_CTRL_CLOAD_TRIM_POS)
#define XTAL32K_CTRIM_3P6PF             (0x9 << ACS_XTAL32K_CTRL_CLOAD_TRIM_POS)
#define XTAL32K_CTRIM_25P2PF            (0x3F << ACS_XTAL32K_CTRL_CLOAD_TRIM_POS)

#define XTAL32K_ITRIM_20NA              (0x0 << ACS_XTAL32K_CTRL_ITRIM_POS)
#define XTAL32K_ITRIM_80NA              (0x3 << ACS_XTAL32K_CTRL_ITRIM_POS)
#define XTAL32K_ITRIM_160NA             (0x7 << ACS_XTAL32K_CTRL_ITRIM_POS)
#define XTAL32K_ITRIM_320NA             (0xF << ACS_XTAL32K_CTRL_ITRIM_POS)

#define XTAL32K_IBOOST_DISABLE_BITBAND  0x0
#define XTAL32K_IBOOST_ENABLE_BITBAND   0x1
#define XTAL32K_IBOOST_DISABLE          (XTAL32K_IBOOST_DISABLE_BITBAND << ACS_XTAL32K_CTRL_IBOOST_POS)
#define XTAL32K_IBOOST_ENABLE           (XTAL32K_IBOOST_ENABLE_BITBAND << ACS_XTAL32K_CTRL_IBOOST_POS)

#define XTAL32K_DISABLE_BITBAND         0x0
#define XTAL32K_ENABLE_BITBAND          0x1
#define XTAL32K_DISABLE                 (XTAL32K_DISABLE_BITBAND << ACS_XTAL32K_CTRL_ENABLE_POS)
#define XTAL32K_ENABLE                  (XTAL32K_ENABLE_BITBAND << ACS_XTAL32K_CTRL_ENABLE_POS)

/* Baseband timer and standby clock configuration register */
#define ACS_BB_TIMER_CTRL_BASE          0x40001328
#define ACS_BB_TIMER_CTRL               REG32_POINTER(ACS_BB_TIMER_CTRL_BASE)

/* ACS_BB_TIMER_CTRL bit positions */
#define ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_POS 8
#define ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_MASK (0x3 << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_POS)
#define ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_POS 0

/* ACS_BB_TIMER_CTRL bit-band aliases */
#define ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_BB_TIMER_CTRL_BASE, ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_POS))

/* ACS_BB_TIMER_CTRL settings */
#define BB_CLK_PRESCALE_1               (0x0 << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_POS)
#define BB_CLK_PRESCALE_2               (0x1 << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_POS)
#define BB_CLK_PRESCALE_4               (0x2 << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_POS)
#define BB_CLK_PRESCALE_8               (0x3 << ACS_BB_TIMER_CTRL_BB_CLK_PRESCALE_POS)

#define BB_TIMER_RESET_BITBAND          0x0
#define BB_TIMER_NRESET_BITBAND         0x1
#define BB_TIMER_RESET                  (BB_TIMER_RESET_BITBAND << ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_POS)
#define BB_TIMER_NRESET                 (BB_TIMER_NRESET_BITBAND << ACS_BB_TIMER_CTRL_BB_TIMER_NRESET_POS)

/* Clock Detector configuration register */
#define ACS_CLK_DET_CTRL_BASE           0x4000132C
#define ACS_CLK_DET_CTRL                REG32_POINTER(ACS_CLK_DET_CTRL_BASE)

/* ACS_CLK_DET_CTRL bit positions */
#define ACS_CLK_DET_CTRL_CLOCK_PRESENT_POS 8
#define ACS_CLK_DET_CTRL_RESET_IGNORE_POS 1
#define ACS_CLK_DET_CTRL_ENABLE_POS     0

/* ACS_CLK_DET_CTRL bit-band aliases */
#define ACS_CLK_DET_CTRL_CLOCK_PRESENT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_CLK_DET_CTRL_BASE, ACS_CLK_DET_CTRL_CLOCK_PRESENT_POS))
#define ACS_CLK_DET_CTRL_RESET_IGNORE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_CLK_DET_CTRL_BASE, ACS_CLK_DET_CTRL_RESET_IGNORE_POS))
#define ACS_CLK_DET_CTRL_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_CLK_DET_CTRL_BASE, ACS_CLK_DET_CTRL_ENABLE_POS))

/* ACS_CLK_DET_CTRL settings */
#define ACS_CLK_DET_NO_CLOCK_BITBAND    0x0
#define ACS_CLK_DET_CLOCK_PRESENT_BITBAND 0x1
#define ACS_CLK_DET_NO_CLOCK            (ACS_CLK_DET_NO_CLOCK_BITBAND << ACS_CLK_DET_CTRL_CLOCK_PRESENT_POS)
#define ACS_CLK_DET_CLOCK_PRESENT       (ACS_CLK_DET_CLOCK_PRESENT_BITBAND << ACS_CLK_DET_CTRL_CLOCK_PRESENT_POS)

#define ACS_CLK_DET_RESET_DISABLE_BITBAND 0x1
#define ACS_CLK_DET_RESET_ENABLE_BITBAND 0x0
#define ACS_CLK_DET_RESET_DISABLE       (ACS_CLK_DET_RESET_DISABLE_BITBAND << ACS_CLK_DET_CTRL_RESET_IGNORE_POS)
#define ACS_CLK_DET_RESET_ENABLE        (ACS_CLK_DET_RESET_ENABLE_BITBAND << ACS_CLK_DET_CTRL_RESET_IGNORE_POS)

#define ACS_CLK_DET_ENABLE_BITBAND      0x1
#define ACS_CLK_DET_DISABLE_BITBAND     0x0
#define ACS_CLK_DET_ENABLE              (ACS_CLK_DET_ENABLE_BITBAND << ACS_CLK_DET_CTRL_ENABLE_POS)
#define ACS_CLK_DET_DISABLE             (ACS_CLK_DET_DISABLE_BITBAND << ACS_CLK_DET_CTRL_ENABLE_POS)

/* RTC Timer Counter Preload */
#define ACS_RTC_CFG_BASE                0x40001330
#define ACS_RTC_CFG                     REG32_POINTER(ACS_RTC_CFG_BASE)

/* RTC Timer Counter Current Value (only reset by pmu reset or by writting 1 at ACS_RTC_CTRL.RESET) */
#define ACS_RTC_COUNT_BASE              0x40001334
#define ACS_RTC_COUNT                   READONLY_REG32_POINTER(ACS_RTC_COUNT_BASE)

/* RTC Control Register */
#define ACS_RTC_CTRL_BASE               0x40001338
#define ACS_RTC_CTRL                    REG32_POINTER(ACS_RTC_CTRL_BASE)

/* ACS_RTC_CTRL bit positions */
#define ACS_RTC_CTRL_FORCE_CLOCK_POS    25
#define ACS_RTC_CTRL_RESET_POS          24
#define ACS_RTC_CTRL_ALARM_CFG_POS      4
#define ACS_RTC_CTRL_ALARM_CFG_MASK     (0xF << ACS_RTC_CTRL_ALARM_CFG_POS)
#define ACS_RTC_CTRL_CLK_SRC_SEL_POS    1
#define ACS_RTC_CTRL_CLK_SRC_SEL_MASK   (0x7 << ACS_RTC_CTRL_CLK_SRC_SEL_POS)
#define ACS_RTC_CTRL_ENABLE_POS         0

/* ACS_RTC_CTRL bit-band aliases */
#define ACS_RTC_CTRL_FORCE_CLOCK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RTC_CTRL_BASE, ACS_RTC_CTRL_FORCE_CLOCK_POS))
#define ACS_RTC_CTRL_RESET_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(ACS_RTC_CTRL_BASE, ACS_RTC_CTRL_RESET_POS))
#define ACS_RTC_CTRL_ENABLE_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(ACS_RTC_CTRL_BASE, ACS_RTC_CTRL_ENABLE_POS))

/* ACS_RTC_CTRL settings */
#define RTC_FORCE_CLOCK_BITBAND         0x1
#define RTC_FORCE_CLOCK                 (RTC_FORCE_CLOCK_BITBAND << ACS_RTC_CTRL_FORCE_CLOCK_POS)

#define RTC_RESET_BITBAND               0x1
#define RTC_RESET                       (RTC_RESET_BITBAND << ACS_RTC_CTRL_RESET_POS)

#define RTC_ALARM_DISABLE               (0x0 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_7P8125MS              (0x1 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_15P625MS              (0x2 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_31P25MS               (0x3 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_62P5MS                (0x4 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_125MS                 (0x5 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_250MS                 (0x6 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_500MS                 (0x7 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_1S                    (0x8 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_2S                    (0x9 << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_4S                    (0xA << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_8S                    (0xB << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_16S                   (0xC << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_32S                   (0xD << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_64S                   (0xE << ACS_RTC_CTRL_ALARM_CFG_POS)
#define RTC_ALARM_ZERO                  (0xF << ACS_RTC_CTRL_ALARM_CFG_POS)

#define RTC_CLK_SRC_RC_OSC              (0x0 << ACS_RTC_CTRL_CLK_SRC_SEL_POS)
#define RTC_CLK_SRC_XTAL32K             (0x1 << ACS_RTC_CTRL_CLK_SRC_SEL_POS)
#define RTC_CLK_SRC_DIO0                (0x4 << ACS_RTC_CTRL_CLK_SRC_SEL_POS)
#define RTC_CLK_SRC_DIO1                (0x5 << ACS_RTC_CTRL_CLK_SRC_SEL_POS)
#define RTC_CLK_SRC_DIO2                (0x6 << ACS_RTC_CTRL_CLK_SRC_SEL_POS)
#define RTC_CLK_SRC_DIO3                (0x7 << ACS_RTC_CTRL_CLK_SRC_SEL_POS)

#define RTC_DISABLE_BITBAND             0x0
#define RTC_ENABLE_BITBAND              0x1
#define RTC_DISABLE                     (RTC_DISABLE_BITBAND << ACS_RTC_CTRL_ENABLE_POS)
#define RTC_ENABLE                      (RTC_ENABLE_BITBAND << ACS_RTC_CTRL_ENABLE_POS)

/* Power Modes Control Register */
#define ACS_PWR_MODES_CTRL_BASE         0x40001340
#define ACS_PWR_MODES_CTRL              REG32_POINTER(ACS_PWR_MODES_CTRL_BASE)

/* Wake-Up Control / Status Register */
#define ACS_WAKEUP_CTRL_BASE            0x40001344
#define ACS_WAKEUP_CTRL                 REG32_POINTER(ACS_WAKEUP_CTRL_BASE)

/* ACS_WAKEUP_CTRL bit positions */
#define ACS_WAKEUP_CTRL_PADS_RETENTION_EN_POS 24
#define ACS_WAKEUP_CTRL_BOOT_FLASH_APP_REBOOT_POS 20
#define ACS_WAKEUP_CTRL_RC_CLOCK_MULT_POS 19
#define ACS_WAKEUP_CTRL_RC_FTRIM_FLAG_POS 18
#define ACS_WAKEUP_CTRL_BOOT_SELECT_POS 16
#define ACS_WAKEUP_CTRL_BOOT_SELECT_MASK (0x3 << ACS_WAKEUP_CTRL_BOOT_SELECT_POS)
#define ACS_WAKEUP_CTRL_DCDC_OVERLOAD_WAKEUP_POS 15
#define ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_POS 14
#define ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_POS 13
#define ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_POS 12
#define ACS_WAKEUP_CTRL_DIO3_WAKEUP_POS 11
#define ACS_WAKEUP_CTRL_DIO2_WAKEUP_POS 10
#define ACS_WAKEUP_CTRL_DIO1_WAKEUP_POS 9
#define ACS_WAKEUP_CTRL_DIO0_WAKEUP_POS 8
#define ACS_WAKEUP_CTRL_DCDC_OVERLOAD_CLEAR_POS 7
#define ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_CLEAR_POS 6
#define ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_CLEAR_POS 5
#define ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_CLEAR_POS 4
#define ACS_WAKEUP_CTRL_DIO3_WAKEUP_CLEAR_POS 3
#define ACS_WAKEUP_CTRL_DIO2_WAKEUP_CLEAR_POS 2
#define ACS_WAKEUP_CTRL_DIO1_WAKEUP_CLEAR_POS 1
#define ACS_WAKEUP_CTRL_DIO0_WAKEUP_CLEAR_POS 0

/* ACS_WAKEUP_CTRL bit-band aliases */
#define ACS_WAKEUP_CTRL_PADS_RETENTION_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_PADS_RETENTION_EN_POS))
#define ACS_WAKEUP_CTRL_BOOT_FLASH_APP_REBOOT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_BOOT_FLASH_APP_REBOOT_POS))
#define ACS_WAKEUP_CTRL_RC_CLOCK_MULT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_RC_CLOCK_MULT_POS))
#define ACS_WAKEUP_CTRL_RC_FTRIM_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_RC_FTRIM_FLAG_POS))
#define ACS_WAKEUP_CTRL_DCDC_OVERLOAD_WAKEUP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DCDC_OVERLOAD_WAKEUP_POS))
#define ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_POS))
#define ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_POS))
#define ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_POS))
#define ACS_WAKEUP_CTRL_DIO3_WAKEUP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DIO3_WAKEUP_POS))
#define ACS_WAKEUP_CTRL_DIO2_WAKEUP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DIO2_WAKEUP_POS))
#define ACS_WAKEUP_CTRL_DIO1_WAKEUP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DIO1_WAKEUP_POS))
#define ACS_WAKEUP_CTRL_DIO0_WAKEUP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DIO0_WAKEUP_POS))
#define ACS_WAKEUP_CTRL_DCDC_OVERLOAD_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DCDC_OVERLOAD_CLEAR_POS))
#define ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_CLEAR_POS))
#define ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_CLEAR_POS))
#define ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_CLEAR_POS))
#define ACS_WAKEUP_CTRL_DIO3_WAKEUP_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DIO3_WAKEUP_CLEAR_POS))
#define ACS_WAKEUP_CTRL_DIO2_WAKEUP_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DIO2_WAKEUP_CLEAR_POS))
#define ACS_WAKEUP_CTRL_DIO1_WAKEUP_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DIO1_WAKEUP_CLEAR_POS))
#define ACS_WAKEUP_CTRL_DIO0_WAKEUP_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CTRL_BASE, ACS_WAKEUP_CTRL_DIO0_WAKEUP_CLEAR_POS))

/* ACS_WAKEUP_CTRL settings */
#define PADS_RETENTION_DISABLE_BITBAND  0x0
#define PADS_RETENTION_ENABLE_BITBAND   0x1
#define PADS_RETENTION_DISABLE          (PADS_RETENTION_DISABLE_BITBAND << ACS_WAKEUP_CTRL_PADS_RETENTION_EN_POS)
#define PADS_RETENTION_ENABLE           (PADS_RETENTION_ENABLE_BITBAND << ACS_WAKEUP_CTRL_PADS_RETENTION_EN_POS)

#define BOOT_FLASH_APP_REBOOT_DISABLE_BITBAND 0x0
#define BOOT_FLASH_APP_REBOOT_ENABLE_BITBAND 0x1
#define BOOT_FLASH_APP_REBOOT_DISABLE   (BOOT_FLASH_APP_REBOOT_DISABLE_BITBAND << ACS_WAKEUP_CTRL_BOOT_FLASH_APP_REBOOT_POS)
#define BOOT_FLASH_APP_REBOOT_ENABLE    (BOOT_FLASH_APP_REBOOT_ENABLE_BITBAND << ACS_WAKEUP_CTRL_BOOT_FLASH_APP_REBOOT_POS)

#define RC_START_OSC_STATUS_3MHZ_BITBAND 0x0
#define RC_START_OSC_STATUS_12MHZ_BITBAND 0x1
#define RC_START_OSC_STATUS_3MHZ        (RC_START_OSC_STATUS_3MHZ_BITBAND << ACS_WAKEUP_CTRL_RC_CLOCK_MULT_POS)
#define RC_START_OSC_STATUS_12MHZ       (RC_START_OSC_STATUS_12MHZ_BITBAND << ACS_WAKEUP_CTRL_RC_CLOCK_MULT_POS)

#define RC_OSC_STATUS_UNCALIBRATED_BITBAND 0x0
#define RC_OSC_STATUS_CALIBRATED_BITBAND 0x1
#define RC_OSC_STATUS_UNCALIBRATED      (RC_OSC_STATUS_UNCALIBRATED_BITBAND << ACS_WAKEUP_CTRL_RC_FTRIM_FLAG_POS)
#define RC_OSC_STATUS_CALIBRATED        (RC_OSC_STATUS_CALIBRATED_BITBAND << ACS_WAKEUP_CTRL_RC_FTRIM_FLAG_POS)

#define BOOT_FLASH_XTAL_DISABLE         (0x0 << ACS_WAKEUP_CTRL_BOOT_SELECT_POS)
#define BOOT_CUSTOM                     (0x1 << ACS_WAKEUP_CTRL_BOOT_SELECT_POS)
#define BOOT_FLASH_XTAL_DEFAULT_TRIM    (0x2 << ACS_WAKEUP_CTRL_BOOT_SELECT_POS)
#define BOOT_FLASH_XTAL_CUSTOM_TRIM     (0x3 << ACS_WAKEUP_CTRL_BOOT_SELECT_POS)

#define WAKEUP_DCDC_OVERLOAD_NOT_SET_BITBAND 0x0
#define WAKEUP_DCDC_OVERLOAD_SET_BITBAND 0x1
#define WAKEUP_DCDC_OVERLOAD_NOT_SET    (WAKEUP_DCDC_OVERLOAD_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DCDC_OVERLOAD_WAKEUP_POS)
#define WAKEUP_DCDC_OVERLOAD_SET        (WAKEUP_DCDC_OVERLOAD_SET_BITBAND << ACS_WAKEUP_CTRL_DCDC_OVERLOAD_WAKEUP_POS)

#define WAKEUP_PAD_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_PAD_EVENT_SET_BITBAND    0x1
#define WAKEUP_PAD_EVENT_NOT_SET        (WAKEUP_PAD_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_POS)
#define WAKEUP_PAD_EVENT_SET            (WAKEUP_PAD_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_POS)

#define WAKEUP_RTC_ALARM_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_RTC_ALARM_EVENT_SET_BITBAND 0x1
#define WAKEUP_RTC_ALARM_EVENT_NOT_SET  (WAKEUP_RTC_ALARM_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_POS)
#define WAKEUP_RTC_ALARM_EVENT_SET      (WAKEUP_RTC_ALARM_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_POS)

#define WAKEUP_BB_TIMER_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_BB_TIMER_EVENT_SET_BITBAND 0x1
#define WAKEUP_BB_TIMER_EVENT_NOT_SET   (WAKEUP_BB_TIMER_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_POS)
#define WAKEUP_BB_TIMER_EVENT_SET       (WAKEUP_BB_TIMER_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_POS)

#define WAKEUP_DIO3_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_DIO3_EVENT_SET_BITBAND   0x1
#define WAKEUP_DIO3_EVENT_NOT_SET       (WAKEUP_DIO3_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO3_WAKEUP_POS)
#define WAKEUP_DIO3_EVENT_SET           (WAKEUP_DIO3_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO3_WAKEUP_POS)

#define WAKEUP_DIO2_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_DIO2_EVENT_SET_BITBAND   0x1
#define WAKEUP_DIO2_EVENT_NOT_SET       (WAKEUP_DIO2_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO2_WAKEUP_POS)
#define WAKEUP_DIO2_EVENT_SET           (WAKEUP_DIO2_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO2_WAKEUP_POS)

#define WAKEUP_DIO1_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_DIO1_EVENT_SET_BITBAND   0x1
#define WAKEUP_DIO1_EVENT_NOT_SET       (WAKEUP_DIO1_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO1_WAKEUP_POS)
#define WAKEUP_DIO1_EVENT_SET           (WAKEUP_DIO1_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO1_WAKEUP_POS)

#define WAKEUP_DIO0_EVENT_NOT_SET_BITBAND 0x0
#define WAKEUP_DIO0_EVENT_SET_BITBAND   0x1
#define WAKEUP_DIO0_EVENT_NOT_SET       (WAKEUP_DIO0_EVENT_NOT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO0_WAKEUP_POS)
#define WAKEUP_DIO0_EVENT_SET           (WAKEUP_DIO0_EVENT_SET_BITBAND << ACS_WAKEUP_CTRL_DIO0_WAKEUP_POS)

#define WAKEUP_DCDC_OVERLOAD_CLEAR_BITBAND 0x1
#define WAKEUP_DCDC_OVERLOAD_CLEAR      (WAKEUP_DCDC_OVERLOAD_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DCDC_OVERLOAD_CLEAR_POS)

#define WAKEUP_PAD_EVENT_CLEAR_BITBAND  0x1
#define WAKEUP_PAD_EVENT_CLEAR          (WAKEUP_PAD_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_WAKEUP_PAD_WAKEUP_CLEAR_POS)

#define WAKEUP_RTC_ALARM_CLEAR_BITBAND  0x1
#define WAKEUP_RTC_ALARM_CLEAR          (WAKEUP_RTC_ALARM_CLEAR_BITBAND << ACS_WAKEUP_CTRL_RTC_ALARM_WAKEUP_CLEAR_POS)

#define WAKEUP_BB_TIMER_CLEAR_BITBAND   0x1
#define WAKEUP_BB_TIMER_CLEAR           (WAKEUP_BB_TIMER_CLEAR_BITBAND << ACS_WAKEUP_CTRL_BB_TIMER_WAKEUP_CLEAR_POS)

#define WAKEUP_DIO3_EVENT_CLEAR_BITBAND 0x1
#define WAKEUP_DIO3_EVENT_CLEAR         (WAKEUP_DIO3_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DIO3_WAKEUP_CLEAR_POS)

#define WAKEUP_DIO2_EVENT_CLEAR_BITBAND 0x1
#define WAKEUP_DIO2_EVENT_CLEAR         (WAKEUP_DIO2_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DIO2_WAKEUP_CLEAR_POS)

#define WAKEUP_DIO1_EVENT_CLEAR_BITBAND 0x1
#define WAKEUP_DIO1_EVENT_CLEAR         (WAKEUP_DIO1_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DIO1_WAKEUP_CLEAR_POS)

#define WAKEUP_DIO0_EVENT_CLEAR_BITBAND 0x1
#define WAKEUP_DIO0_EVENT_CLEAR         (WAKEUP_DIO0_EVENT_CLEAR_BITBAND << ACS_WAKEUP_CTRL_DIO0_WAKEUP_CLEAR_POS)

/* Wakeup configuration */
#define ACS_WAKEUP_CFG_BASE             0x40001348
#define ACS_WAKEUP_CFG                  REG32_POINTER(ACS_WAKEUP_CFG_BASE)

/* ACS_WAKEUP_CFG bit positions */
#define ACS_WAKEUP_CFG_DELAY_POS        16
#define ACS_WAKEUP_CFG_DELAY_MASK       (0x7 << ACS_WAKEUP_CFG_DELAY_POS)
#define ACS_WAKEUP_CFG_DCDC_OVERLOAD_EN_POS 9
#define ACS_WAKEUP_CFG_WAKEUP_PAD_POL_POS 8
#define ACS_WAKEUP_CFG_DIO3_POL_POS     7
#define ACS_WAKEUP_CFG_DIO2_POL_POS     6
#define ACS_WAKEUP_CFG_DIO1_POL_POS     5
#define ACS_WAKEUP_CFG_DIO0_POL_POS     4
#define ACS_WAKEUP_CFG_DIO3_EN_POS      3
#define ACS_WAKEUP_CFG_DIO2_EN_POS      2
#define ACS_WAKEUP_CFG_DIO1_EN_POS      1
#define ACS_WAKEUP_CFG_DIO0_EN_POS      0

/* ACS_WAKEUP_CFG bit-band aliases */
#define ACS_WAKEUP_CFG_DCDC_OVERLOAD_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DCDC_OVERLOAD_EN_POS))
#define ACS_WAKEUP_CFG_WAKEUP_PAD_POL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_WAKEUP_PAD_POL_POS))
#define ACS_WAKEUP_CFG_DIO3_POL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DIO3_POL_POS))
#define ACS_WAKEUP_CFG_DIO2_POL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DIO2_POL_POS))
#define ACS_WAKEUP_CFG_DIO1_POL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DIO1_POL_POS))
#define ACS_WAKEUP_CFG_DIO0_POL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DIO0_POL_POS))
#define ACS_WAKEUP_CFG_DIO3_EN_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DIO3_EN_POS))
#define ACS_WAKEUP_CFG_DIO2_EN_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DIO2_EN_POS))
#define ACS_WAKEUP_CFG_DIO1_EN_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DIO1_EN_POS))
#define ACS_WAKEUP_CFG_DIO0_EN_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(ACS_WAKEUP_CFG_BASE, ACS_WAKEUP_CFG_DIO0_EN_POS))

/* ACS_WAKEUP_CFG settings */
#define WAKEUP_DELAY_1                  (0x0 << ACS_WAKEUP_CFG_DELAY_POS)
#define WAKEUP_DELAY_2                  (0x1 << ACS_WAKEUP_CFG_DELAY_POS)
#define WAKEUP_DELAY_4                  (0x2 << ACS_WAKEUP_CFG_DELAY_POS)
#define WAKEUP_DELAY_8                  (0x3 << ACS_WAKEUP_CFG_DELAY_POS)
#define WAKEUP_DELAY_16                 (0x4 << ACS_WAKEUP_CFG_DELAY_POS)
#define WAKEUP_DELAY_32                 (0x5 << ACS_WAKEUP_CFG_DELAY_POS)
#define WAKEUP_DELAY_64                 (0x6 << ACS_WAKEUP_CFG_DELAY_POS)
#define WAKEUP_DELAY_128                (0x7 << ACS_WAKEUP_CFG_DELAY_POS)

#define WAKEUP_DCDC_OVERLOAD_DISABLE_BITBAND 0x0
#define WAKEUP_DCDC_OVERLOAD_ENABLE_BITBAND 0x1
#define WAKEUP_DCDC_OVERLOAD_DISABLE    (WAKEUP_DCDC_OVERLOAD_DISABLE_BITBAND << ACS_WAKEUP_CFG_DCDC_OVERLOAD_EN_POS)
#define WAKEUP_DCDC_OVERLOAD_ENABLE     (WAKEUP_DCDC_OVERLOAD_ENABLE_BITBAND << ACS_WAKEUP_CFG_DCDC_OVERLOAD_EN_POS)

#define WAKEUP_WAKEUP_PAD_RISING_BITBAND 0x0
#define WAKEUP_WAKEUP_PAD_FALLING_BITBAND 0x1
#define WAKEUP_WAKEUP_PAD_RISING        (WAKEUP_WAKEUP_PAD_RISING_BITBAND << ACS_WAKEUP_CFG_WAKEUP_PAD_POL_POS)
#define WAKEUP_WAKEUP_PAD_FALLING       (WAKEUP_WAKEUP_PAD_FALLING_BITBAND << ACS_WAKEUP_CFG_WAKEUP_PAD_POL_POS)

#define WAKEUP_DIO3_RISING_BITBAND      0x0
#define WAKEUP_DIO3_FALLING_BITBAND     0x1
#define WAKEUP_DIO3_RISING              (WAKEUP_DIO3_RISING_BITBAND << ACS_WAKEUP_CFG_DIO3_POL_POS)
#define WAKEUP_DIO3_FALLING             (WAKEUP_DIO3_FALLING_BITBAND << ACS_WAKEUP_CFG_DIO3_POL_POS)

#define WAKEUP_DIO2_RISING_BITBAND      0x0
#define WAKEUP_DIO2_FALLING_BITBAND     0x1
#define WAKEUP_DIO2_RISING              (WAKEUP_DIO2_RISING_BITBAND << ACS_WAKEUP_CFG_DIO2_POL_POS)
#define WAKEUP_DIO2_FALLING             (WAKEUP_DIO2_FALLING_BITBAND << ACS_WAKEUP_CFG_DIO2_POL_POS)

#define WAKEUP_DIO1_RISING_BITBAND      0x0
#define WAKEUP_DIO1_FALLING_BITBAND     0x1
#define WAKEUP_DIO1_RISING              (WAKEUP_DIO1_RISING_BITBAND << ACS_WAKEUP_CFG_DIO1_POL_POS)
#define WAKEUP_DIO1_FALLING             (WAKEUP_DIO1_FALLING_BITBAND << ACS_WAKEUP_CFG_DIO1_POL_POS)

#define WAKEUP_DIO0_RISING_BITBAND      0x0
#define WAKEUP_DIO0_FALLING_BITBAND     0x1
#define WAKEUP_DIO0_RISING              (WAKEUP_DIO0_RISING_BITBAND << ACS_WAKEUP_CFG_DIO0_POL_POS)
#define WAKEUP_DIO0_FALLING             (WAKEUP_DIO0_FALLING_BITBAND << ACS_WAKEUP_CFG_DIO0_POL_POS)

#define WAKEUP_DIO3_DISABLE_BITBAND     0x0
#define WAKEUP_DIO3_ENABLE_BITBAND      0x1
#define WAKEUP_DIO3_DISABLE             (WAKEUP_DIO3_DISABLE_BITBAND << ACS_WAKEUP_CFG_DIO3_EN_POS)
#define WAKEUP_DIO3_ENABLE              (WAKEUP_DIO3_ENABLE_BITBAND << ACS_WAKEUP_CFG_DIO3_EN_POS)

#define WAKEUP_DIO2_DISABLE_BITBAND     0x0
#define WAKEUP_DIO2_ENABLE_BITBAND      0x1
#define WAKEUP_DIO2_DISABLE             (WAKEUP_DIO2_DISABLE_BITBAND << ACS_WAKEUP_CFG_DIO2_EN_POS)
#define WAKEUP_DIO2_ENABLE              (WAKEUP_DIO2_ENABLE_BITBAND << ACS_WAKEUP_CFG_DIO2_EN_POS)

#define WAKEUP_DIO1_DISABLE_BITBAND     0x0
#define WAKEUP_DIO1_ENABLE_BITBAND      0x1
#define WAKEUP_DIO1_DISABLE             (WAKEUP_DIO1_DISABLE_BITBAND << ACS_WAKEUP_CFG_DIO1_EN_POS)
#define WAKEUP_DIO1_ENABLE              (WAKEUP_DIO1_ENABLE_BITBAND << ACS_WAKEUP_CFG_DIO1_EN_POS)

#define WAKEUP_DIO0_DISABLE_BITBAND     0x0
#define WAKEUP_DIO0_ENABLE_BITBAND      0x1
#define WAKEUP_DIO0_DISABLE             (WAKEUP_DIO0_DISABLE_BITBAND << ACS_WAKEUP_CFG_DIO0_EN_POS)
#define WAKEUP_DIO0_ENABLE              (WAKEUP_DIO0_ENABLE_BITBAND << ACS_WAKEUP_CFG_DIO0_EN_POS)

/* RTC Timer wakeup value and wakeup source */
#define ACS_WAKEUP_STATE_BASE           0x4000134C
#define ACS_WAKEUP_STATE                READONLY_REG32_POINTER(ACS_WAKEUP_STATE_BASE)

/* ACS_WAKEUP_STATE bit positions */
#define ACS_WAKEUP_STATE_WAKEUP_SRC_POS 16
#define ACS_WAKEUP_STATE_WAKEUP_SRC_MASK (0x7 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)
#define ACS_WAKEUP_STATE_RTC_VALUE_POS  0
#define ACS_WAKEUP_STATE_RTC_VALUE_MASK (0xFF << ACS_WAKEUP_STATE_RTC_VALUE_POS)

/* ACS_WAKEUP_STATE settings */
#define WAKEUP_DUE_TO_DCDC_OVERLOAD     (0x7 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)
#define WAKEUP_DUE_TO_WAKEUP_PAD        (0x6 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)
#define WAKEUP_DUE_TO_RTC_ALARM         (0x5 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)
#define WAKEUP_DUE_TO_BB_TIMER          (0x4 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)
#define WAKEUP_DUE_TO_DIO3              (0x3 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)
#define WAKEUP_DUE_TO_DIO2              (0x2 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)
#define WAKEUP_DUE_TO_DIO1              (0x1 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)
#define WAKEUP_DUE_TO_DIO0              (0x0 << ACS_WAKEUP_STATE_WAKEUP_SRC_POS)

/* Wake-Up Control Register / JIC RW registers */
#define ACS_WAKEUP_GP_DATA_BASE         0x40001350
#define ACS_WAKEUP_GP_DATA              REG32_POINTER(ACS_WAKEUP_GP_DATA_BASE)

/* ACS reset source status registers */
#define ACS_RESET_STATUS_BASE           0x40001354
#define ACS_RESET_STATUS                REG32_POINTER(ACS_RESET_STATUS_BASE)

/* ACS_RESET_STATUS bit positions */
#define ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_POS 14
#define ACS_RESET_STATUS_CLK_DET_RESET_FLAG_POS 13
#define ACS_RESET_STATUS_VDDA_RESET_FLAG_POS 12
#define ACS_RESET_STATUS_VDDM_RESET_FLAG_POS 11
#define ACS_RESET_STATUS_VDDC_RESET_FLAG_POS 10
#define ACS_RESET_STATUS_PAD_RESET_FLAG_POS 9
#define ACS_RESET_STATUS_POR_RESET_FLAG_POS 8
#define ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_CLEAR_POS 6
#define ACS_RESET_STATUS_CLK_DET_RESET_FLAG_CLEAR_POS 5
#define ACS_RESET_STATUS_VDDA_RESET_FLAG_CLEAR_POS 4
#define ACS_RESET_STATUS_VDDM_RESET_FLAG_CLEAR_POS 3
#define ACS_RESET_STATUS_VDDC_RESET_FLAG_CLEAR_POS 2
#define ACS_RESET_STATUS_PAD_RESET_FLAG_CLEAR_POS 1
#define ACS_RESET_STATUS_POR_RESET_FLAG_CLEAR_POS 0

/* ACS_RESET_STATUS bit-band aliases */
#define ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_POS))
#define ACS_RESET_STATUS_CLK_DET_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_CLK_DET_RESET_FLAG_POS))
#define ACS_RESET_STATUS_VDDA_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_VDDA_RESET_FLAG_POS))
#define ACS_RESET_STATUS_VDDM_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_VDDM_RESET_FLAG_POS))
#define ACS_RESET_STATUS_VDDC_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_VDDC_RESET_FLAG_POS))
#define ACS_RESET_STATUS_PAD_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_PAD_RESET_FLAG_POS))
#define ACS_RESET_STATUS_POR_RESET_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_POR_RESET_FLAG_POS))
#define ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_CLEAR_POS))
#define ACS_RESET_STATUS_CLK_DET_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_CLK_DET_RESET_FLAG_CLEAR_POS))
#define ACS_RESET_STATUS_VDDA_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_VDDA_RESET_FLAG_CLEAR_POS))
#define ACS_RESET_STATUS_VDDM_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_VDDM_RESET_FLAG_CLEAR_POS))
#define ACS_RESET_STATUS_VDDC_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_VDDC_RESET_FLAG_CLEAR_POS))
#define ACS_RESET_STATUS_PAD_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_PAD_RESET_FLAG_CLEAR_POS))
#define ACS_RESET_STATUS_POR_RESET_FLAG_CLEAR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_RESET_STATUS_BASE, ACS_RESET_STATUS_POR_RESET_FLAG_CLEAR_POS))

/* ACS_RESET_STATUS settings */
#define TIMEOUT_RESET_FLAG_NOT_SET_BITBAND 0x0
#define TIMEOUT_RESET_FLAG_SET_BITBAND  0x1
#define TIMEOUT_RESET_FLAG_NOT_SET      (TIMEOUT_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_POS)
#define TIMEOUT_RESET_FLAG_SET          (TIMEOUT_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_POS)

#define CLK_DET_RESET_FLAG_NOT_SET_BITBAND 0x0
#define CLK_DET_RESET_FLAG_SET_BITBAND  0x1
#define CLK_DET_RESET_FLAG_NOT_SET      (CLK_DET_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_CLK_DET_RESET_FLAG_POS)
#define CLK_DET_RESET_FLAG_SET          (CLK_DET_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_CLK_DET_RESET_FLAG_POS)

#define VDDA_RESET_FLAG_NOT_SET_BITBAND 0x0
#define VDDA_RESET_FLAG_SET_BITBAND     0x1
#define VDDA_RESET_FLAG_NOT_SET         (VDDA_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_VDDA_RESET_FLAG_POS)
#define VDDA_RESET_FLAG_SET             (VDDA_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_VDDA_RESET_FLAG_POS)

#define VDDM_RESET_FLAG_NOT_SET_BITBAND 0x0
#define VDDM_RESET_FLAG_SET_BITBAND     0x1
#define VDDM_RESET_FLAG_NOT_SET         (VDDM_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_VDDM_RESET_FLAG_POS)
#define VDDM_RESET_FLAG_SET             (VDDM_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_VDDM_RESET_FLAG_POS)

#define VDDC_RESET_FLAG_NOT_SET_BITBAND 0x0
#define VDDC_RESET_FLAG_SET_BITBAND     0x1
#define VDDC_RESET_FLAG_NOT_SET         (VDDC_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_VDDC_RESET_FLAG_POS)
#define VDDC_RESET_FLAG_SET             (VDDC_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_VDDC_RESET_FLAG_POS)

#define PAD_RESET_FLAG_NOT_SET_BITBAND  0x0
#define PAD_RESET_FLAG_SET_BITBAND      0x1
#define PAD_RESET_FLAG_NOT_SET          (PAD_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_PAD_RESET_FLAG_POS)
#define PAD_RESET_FLAG_SET              (PAD_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_PAD_RESET_FLAG_POS)

#define POR_RESET_FLAG_NOT_SET_BITBAND  0x0
#define POR_RESET_FLAG_SET_BITBAND      0x1
#define POR_RESET_FLAG_NOT_SET          (POR_RESET_FLAG_NOT_SET_BITBAND << ACS_RESET_STATUS_POR_RESET_FLAG_POS)
#define POR_RESET_FLAG_SET              (POR_RESET_FLAG_SET_BITBAND << ACS_RESET_STATUS_POR_RESET_FLAG_POS)

#define TIMEOUT_RESET_FLAG_CLEAR_BITBAND 0x1
#define TIMEOUT_RESET_FLAG_CLEAR        (TIMEOUT_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_TIMEOUT_RESET_FLAG_CLEAR_POS)

#define CLK_DET_RESET_FLAG_CLEAR_BITBAND 0x1
#define CLK_DET_RESET_FLAG_CLEAR        (CLK_DET_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_CLK_DET_RESET_FLAG_CLEAR_POS)

#define VDDA_RESET_FLAG_CLEAR_BITBAND   0x1
#define VDDA_RESET_FLAG_CLEAR           (VDDA_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_VDDA_RESET_FLAG_CLEAR_POS)

#define VDDM_RESET_FLAG_CLEAR_BITBAND   0x1
#define VDDM_RESET_FLAG_CLEAR           (VDDM_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_VDDM_RESET_FLAG_CLEAR_POS)

#define VDDC_RESET_FLAG_CLEAR_BITBAND   0x1
#define VDDC_RESET_FLAG_CLEAR           (VDDC_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_VDDC_RESET_FLAG_CLEAR_POS)

#define PAD_RESET_FLAG_CLEAR_BITBAND    0x1
#define PAD_RESET_FLAG_CLEAR            (PAD_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_PAD_RESET_FLAG_CLEAR_POS)

#define POR_RESET_FLAG_CLEAR_BITBAND    0x1
#define POR_RESET_FLAG_CLEAR            (POR_RESET_FLAG_CLEAR_BITBAND << ACS_RESET_STATUS_POR_RESET_FLAG_CLEAR_POS)

/* Analog output configuration register */
#define ACS_AOUT_CTRL_BASE              0x40001358
#define ACS_AOUT_CTRL                   REG32_POINTER(ACS_AOUT_CTRL_BASE)

/* ACS_AOUT_CTRL bit positions */
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_POS 13
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_POS 11
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_MASK (0x3 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_POS)
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS 8
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_MASK (0x7 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)
#define ACS_AOUT_CTRL_TEST_AOUT_POS     0
#define ACS_AOUT_CTRL_TEST_AOUT_MASK    (0x1F << ACS_AOUT_CTRL_TEST_AOUT_POS)

/* ACS_AOUT_CTRL bit-band aliases */
#define ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(ACS_AOUT_CTRL_BASE, ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_POS))

/* ACS_AOUT_CTRL settings */
#define DIO0_RTC_CLK_STOP_RISING_BITBAND 0x0
#define DIO0_RTC_CLK_STOP_FALLING_BITBAND 0x1
#define DIO0_RTC_CLK_STOP_RISING        (DIO0_RTC_CLK_STOP_RISING_BITBAND << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_POS)
#define DIO0_RTC_CLK_STOP_FALLING       (DIO0_RTC_CLK_STOP_FALLING_BITBAND << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_EDGE_POS)

#define DIO0_RTC_CLK_STOP_DIO0          (0x0 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_POS)
#define DIO0_RTC_CLK_STOP_DIO1          (0x1 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_POS)
#define DIO0_RTC_CLK_STOP_DIO2          (0x2 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_POS)
#define DIO0_RTC_CLK_STOP_DIO3          (0x3 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_STOP_SRC_POS)

#define DIO0_RTC_CLK_DISABLE            (0x0 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)
#define DIO0_RTC_CLK_125MS              (0x1 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)
#define DIO0_RTC_CLK_250MS              (0x2 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)
#define DIO0_RTC_CLK_500MS              (0x3 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)
#define DIO0_RTC_CLK_1S                 (0x4 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)
#define DIO0_RTC_CLK_2S                 (0x5 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)
#define DIO0_RTC_CLK_4S                 (0x6 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)
#define DIO0_RTC_CLK_8S                 (0x7 << ACS_AOUT_CTRL_RTC_CLOCK_DIO0_START_POS)

#define AOUT_VSSA                       (0x0 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VCC_SENSE                  (0x1 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VREF_0P75V_OUTPUT          (0x2 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VREF_0P67V_OUTPUT          (0x3 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_IREF_50N_OUTPUT            (0x4 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_IREF_1N_OUTPUT             (0x5 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDACS_OUTPUT              (0x6 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VREF_0P75V_BUF_OUTPUT      (0x7 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VREG_BG                    (0x8 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDRF_SW                   (0x9 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDRF_SENSE                (0xA << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDT                       (0xB << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDC_SENSE                 (0xC << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDA_SW                    (0xD << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDA_SENSE                 (0xE << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDM_SENSE                 (0xF << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_NC                         (0x10 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDPA_SENSE                (0x11 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDPA_ISENSE               (0x12 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_TM0                        (0x13 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_BG_READY                   (0x14 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VCC_READY                  (0x15 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_DCDC_OVERLOAD              (0x16 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_DCDC_ACTIVATED             (0x17 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDRF_READY                (0x18 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDC_READY                 (0x19 << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDM_READY                 (0x1A << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_VDDA_READY                 (0x1B << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_CLK_PRESENT                (0x1C << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_XTAL_OK                    (0x1D << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_XTAL_CLK                   (0x1E << ACS_AOUT_CTRL_TEST_AOUT_POS)
#define AOUT_CLK_32K                    (0x1F << ACS_AOUT_CTRL_TEST_AOUT_POS)

/* Just In Case register in the ACS block */
#define ACS_JIC_READ_BASE               0x4000135C
#define ACS_JIC_READ                    READONLY_REG32_POINTER(ACS_JIC_READ_BASE)

/* ACS_JIC_READ bit positions */
#define ACS_JIC_READ_BYTE0_RO_POS       0
#define ACS_JIC_READ_BYTE0_RO_MASK      (0xFF << ACS_JIC_READ_BYTE0_RO_POS)

/* ----------------------------------------------------------------------------
 * Baseband Controller Interface
 * ------------------------------------------------------------------------- */

/* Baseband controller control register */
#define BBIF_CTRL_BASE                  0x40001400
#define BBIF_CTRL                       REG32_POINTER(BBIF_CTRL_BASE)

/* BBIF_CTRL bit positions */
#define BBIF_CTRL_WAKEUP_REQ_POS        16
#define BBIF_CTRL_CLK_SEL_POS           4
#define BBIF_CTRL_CLK_SEL_MASK          (0x3F << BBIF_CTRL_CLK_SEL_POS)
#define BBIF_CTRL_CLK_ENABLE_POS        0

/* BBIF_CTRL bit-band aliases */
#define BBIF_CTRL_WAKEUP_REQ_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BBIF_CTRL_BASE, BBIF_CTRL_WAKEUP_REQ_POS))
#define BBIF_CTRL_CLK_ENABLE_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BBIF_CTRL_BASE, BBIF_CTRL_CLK_ENABLE_POS))

/* BBIF_CTRL settings */
#define BB_DEEP_SLEEP_BITBAND           0x0
#define BB_WAKEUP_BITBAND               0x1
#define BB_DEEP_SLEEP                   (BB_DEEP_SLEEP_BITBAND << BBIF_CTRL_WAKEUP_REQ_POS)
#define BB_WAKEUP                       (BB_WAKEUP_BITBAND << BBIF_CTRL_WAKEUP_REQ_POS)

#define BBCLK_DIVIDER_6                 (0x6 << BBIF_CTRL_CLK_SEL_POS)
#define BBCLK_DIVIDER_8                 (0x8 << BBIF_CTRL_CLK_SEL_POS)
#define BBCLK_DIVIDER_12                (0xC << BBIF_CTRL_CLK_SEL_POS)
#define BBCLK_DIVIDER_16                (0x10 << BBIF_CTRL_CLK_SEL_POS)
#define BBCLK_DIVIDER_24                (0x18 << BBIF_CTRL_CLK_SEL_POS)

#define BB_CLK_DISABLE_BITBAND          0x0
#define BB_CLK_ENABLE_BITBAND           0x1
#define BB_CLK_DISABLE                  (BB_CLK_DISABLE_BITBAND << BBIF_CTRL_CLK_ENABLE_POS)
#define BB_CLK_ENABLE                   (BB_CLK_ENABLE_BITBAND << BBIF_CTRL_CLK_ENABLE_POS)

/* Baseband controller status register */
#define BBIF_STATUS_BASE                0x40001404
#define BBIF_STATUS                     READONLY_REG32_POINTER(BBIF_STATUS_BASE)

/* BBIF_STATUS bit positions */
#define BBIF_STATUS_LINK_FORMAT_POS     11
#define BBIF_STATUS_LINK_FORMAT_MASK    (0x1F << BBIF_STATUS_LINK_FORMAT_POS)
#define BBIF_STATUS_LINK_LABEL_POS      4
#define BBIF_STATUS_LINK_LABEL_MASK     (0x1F << BBIF_STATUS_LINK_LABEL_POS)
#define BBIF_STATUS_AOBLE_STATUS_POS    3
#define BBIF_STATUS_CLK_STATUS_POS      2
#define BBIF_STATUS_OSC_EN_POS          1
#define BBIF_STATUS_RADIO_EN_POS        0

/* BBIF_STATUS bit-band aliases */
#define BBIF_STATUS_AOBLE_STATUS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_STATUS_BASE, BBIF_STATUS_AOBLE_STATUS_POS))
#define BBIF_STATUS_CLK_STATUS_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_STATUS_BASE, BBIF_STATUS_CLK_STATUS_POS))
#define BBIF_STATUS_OSC_EN_BITBAND      READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_STATUS_BASE, BBIF_STATUS_OSC_EN_POS))
#define BBIF_STATUS_RADIO_EN_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_STATUS_BASE, BBIF_STATUS_RADIO_EN_POS))

/* BBIF_STATUS settings */
#define LINK_FORMAT_RESET               (0x0 << BBIF_STATUS_LINK_FORMAT_POS)
#define MASTER_CONNECT                  (0x2 << BBIF_STATUS_LINK_FORMAT_POS)
#define SLAVE_CONNECT                   (0x3 << BBIF_STATUS_LINK_FORMAT_POS)
#define LOW_DUTY_ADVERTISER             (0x4 << BBIF_STATUS_LINK_FORMAT_POS)
#define HIGH_DUTY_ADVERTISER            (0x5 << BBIF_STATUS_LINK_FORMAT_POS)
#define PASSIVE_SCANNER                 (0x8 << BBIF_STATUS_LINK_FORMAT_POS)
#define ACTIVE_SCANNER                  (0x9 << BBIF_STATUS_LINK_FORMAT_POS)
#define INITIATOR                       (0xF << BBIF_STATUS_LINK_FORMAT_POS)
#define TX_TEST_MODE                    (0x1C << BBIF_STATUS_LINK_FORMAT_POS)
#define RX_TEST_MODE                    (0x1D << BBIF_STATUS_LINK_FORMAT_POS)
#define TX_RX_TEST_MODE                 (0x1E << BBIF_STATUS_LINK_FORMAT_POS)

#define AOBLE_DISABLED_DEVICE_BITBAND   0x0
#define AOBLE_ENABLED_DEVICE_BITBAND    0x1
#define AOBLE_DISABLED_DEVICE           (AOBLE_DISABLED_DEVICE_BITBAND << BBIF_STATUS_AOBLE_STATUS_POS)
#define AOBLE_ENABLED_DEVICE            (AOBLE_ENABLED_DEVICE_BITBAND << BBIF_STATUS_AOBLE_STATUS_POS)

#define MASTER_CLK_BITBAND              0x0
#define LOW_POWER_CLK_BITBAND           0x1
#define MASTER_CLK                      (MASTER_CLK_BITBAND << BBIF_STATUS_CLK_STATUS_POS)
#define LOW_POWER_CLK                   (LOW_POWER_CLK_BITBAND << BBIF_STATUS_CLK_STATUS_POS)

#define OSC_DISABLED_BITBAND            0x0
#define OSC_ENABLED_BITBAND             0x1
#define OSC_DISABLED                    (OSC_DISABLED_BITBAND << BBIF_STATUS_OSC_EN_POS)
#define OSC_ENABLED                     (OSC_ENABLED_BITBAND << BBIF_STATUS_OSC_EN_POS)

#define RF_DISABLED_BITBAND             0x0
#define RF_ENABLED_BITBAND              0x1
#define RF_DISABLED                     (RF_DISABLED_BITBAND << BBIF_STATUS_RADIO_EN_POS)
#define RF_ENABLED                      (RF_ENABLED_BITBAND << BBIF_STATUS_RADIO_EN_POS)

/* Coexistence control register */
#define BBIF_COEX_CTRL_BASE             0x40001408
#define BBIF_COEX_CTRL                  REG32_POINTER(BBIF_COEX_CTRL_BASE)

/* BBIF_COEX_CTRL bit positions */
#define BBIF_COEX_CTRL_TX_POS           4
#define BBIF_COEX_CTRL_RX_POS           0

/* BBIF_COEX_CTRL bit-band aliases */
#define BBIF_COEX_CTRL_TX_BITBAND       REG32_POINTER(SYS_CALC_BITBAND(BBIF_COEX_CTRL_BASE, BBIF_COEX_CTRL_TX_POS))
#define BBIF_COEX_CTRL_RX_BITBAND       REG32_POINTER(SYS_CALC_BITBAND(BBIF_COEX_CTRL_BASE, BBIF_COEX_CTRL_RX_POS))

/* BBIF_COEX_CTRL settings */
#define COEX_TX_IDLE_BITBAND            0x0
#define COEX_TX_BUSY_BITBAND            0x1
#define COEX_TX_IDLE                    (COEX_TX_IDLE_BITBAND << BBIF_COEX_CTRL_TX_POS)
#define COEX_TX_BUSY                    (COEX_TX_BUSY_BITBAND << BBIF_COEX_CTRL_TX_POS)

#define COEX_RX_IDLE_BITBAND            0x0
#define COEX_RX_BUSY_BITBAND            0x1
#define COEX_RX_IDLE                    (COEX_RX_IDLE_BITBAND << BBIF_COEX_CTRL_RX_POS)
#define COEX_RX_BUSY                    (COEX_RX_BUSY_BITBAND << BBIF_COEX_CTRL_RX_POS)

/* Coexistence status register */
#define BBIF_COEX_STATUS_BASE           0x4000140C
#define BBIF_COEX_STATUS                READONLY_REG32_POINTER(BBIF_COEX_STATUS_BASE)

/* BBIF_COEX_STATUS bit positions */
#define BBIF_COEX_STATUS_BLE_PTI_POS    12
#define BBIF_COEX_STATUS_BLE_PTI_MASK   (0xF << BBIF_COEX_STATUS_BLE_PTI_POS)
#define BBIF_COEX_STATUS_BLE_IN_PROCESS_POS 8
#define BBIF_COEX_STATUS_BLE_TX_POS     4
#define BBIF_COEX_STATUS_BLE_RX_POS     0

/* BBIF_COEX_STATUS bit-band aliases */
#define BBIF_COEX_STATUS_BLE_IN_PROCESS_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_COEX_STATUS_BASE, BBIF_COEX_STATUS_BLE_IN_PROCESS_POS))
#define BBIF_COEX_STATUS_BLE_TX_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_COEX_STATUS_BASE, BBIF_COEX_STATUS_BLE_TX_POS))
#define BBIF_COEX_STATUS_BLE_RX_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_COEX_STATUS_BASE, BBIF_COEX_STATUS_BLE_RX_POS))

/* BBIF_COEX_STATUS settings */
#define BLE_PTI_PRIORITY_0              (0x0 << BBIF_COEX_STATUS_BLE_PTI_POS)
#define BLE_PTI_PRIORITY_15             (0xF << BBIF_COEX_STATUS_BLE_PTI_POS)

#define BLE_IDLE_BITBAND                0x0
#define BLE_IN_PROCESS_BITBAND          0x1
#define BLE_IDLE                        (BLE_IDLE_BITBAND << BBIF_COEX_STATUS_BLE_IN_PROCESS_POS)
#define BLE_IN_PROCESS                  (BLE_IN_PROCESS_BITBAND << BBIF_COEX_STATUS_BLE_IN_PROCESS_POS)

#define BLE_TX_IDLE_BITBAND             0x0
#define BLE_TX_BUSY_BITBAND             0x1
#define BLE_TX_IDLE                     (BLE_TX_IDLE_BITBAND << BBIF_COEX_STATUS_BLE_TX_POS)
#define BLE_TX_BUSY                     (BLE_TX_BUSY_BITBAND << BBIF_COEX_STATUS_BLE_TX_POS)

#define BLE_RX_IDLE_BITBAND             0x0
#define BLE_RX_BUSY_BITBAND             0x1
#define BLE_RX_IDLE                     (BLE_RX_IDLE_BITBAND << BBIF_COEX_STATUS_BLE_RX_POS)
#define BLE_RX_BUSY                     (BLE_RX_BUSY_BITBAND << BBIF_COEX_STATUS_BLE_RX_POS)

/* Coexistence interrupts configuration register */
#define BBIF_COEX_INT_CFG_BASE          0x40001410
#define BBIF_COEX_INT_CFG               REG32_POINTER(BBIF_COEX_INT_CFG_BASE)

/* BBIF_COEX_INT_CFG bit positions */
#define BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_POS 8
#define BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_MASK (0x3 << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_POS)
#define BBIF_COEX_INT_CFG_BLE_TX_EVENT_POS 4
#define BBIF_COEX_INT_CFG_BLE_TX_EVENT_MASK (0x3 << BBIF_COEX_INT_CFG_BLE_TX_EVENT_POS)
#define BBIF_COEX_INT_CFG_BLE_RX_EVENT_POS 0
#define BBIF_COEX_INT_CFG_BLE_RX_EVENT_MASK (0x3 << BBIF_COEX_INT_CFG_BLE_RX_EVENT_POS)

/* BBIF_COEX_INT_CFG settings */
#define BLE_IN_PROCESS_EVENT_NONE       (0x0 << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_POS)
#define BLE_IN_PROCESS_EVENT_RISING_EDGE (0x1 << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_POS)
#define BLE_IN_PROCESS_EVENT_FALLING_EDGE (0x2 << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_POS)
#define BLE_IN_PROCESS_EVENT_TRANSITION (0x3 << BBIF_COEX_INT_CFG_BLE_IN_PROCESS_EVENT_POS)

#define BLE_TX_EVENT_NONE               (0x0 << BBIF_COEX_INT_CFG_BLE_TX_EVENT_POS)
#define BLE_TX_EVENT_RISING_EDGE        (0x1 << BBIF_COEX_INT_CFG_BLE_TX_EVENT_POS)
#define BLE_TX_EVENT_FALLING_EDGE       (0x2 << BBIF_COEX_INT_CFG_BLE_TX_EVENT_POS)
#define BLE_TX_EVENT_TRANSITION         (0x3 << BBIF_COEX_INT_CFG_BLE_TX_EVENT_POS)

#define BLE_RX_EVENT_NONE               (0x0 << BBIF_COEX_INT_CFG_BLE_RX_EVENT_POS)
#define BLE_RX_EVENT_RISING_EDGE        (0x1 << BBIF_COEX_INT_CFG_BLE_RX_EVENT_POS)
#define BLE_RX_EVENT_FALLING_EDGE       (0x2 << BBIF_COEX_INT_CFG_BLE_RX_EVENT_POS)
#define BLE_RX_EVENT_TRANSITION         (0x3 << BBIF_COEX_INT_CFG_BLE_RX_EVENT_POS)

/* Coexistence interrupt status register */
#define BBIF_COEX_INT_STATUS_BASE       0x40001414
#define BBIF_COEX_INT_STATUS            READONLY_REG32_POINTER(BBIF_COEX_INT_STATUS_BASE)

/* BBIF_COEX_INT_STATUS bit positions */
#define BBIF_COEX_INT_STATUS_BLE_TX_EVENT_FLAG_POS 4
#define BBIF_COEX_INT_STATUS_BLE_RX_EVENT_FLAG_POS 0

/* BBIF_COEX_INT_STATUS bit-band aliases */
#define BBIF_COEX_INT_STATUS_BLE_TX_EVENT_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_COEX_INT_STATUS_BASE, BBIF_COEX_INT_STATUS_BLE_TX_EVENT_FLAG_POS))
#define BBIF_COEX_INT_STATUS_BLE_RX_EVENT_FLAG_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BBIF_COEX_INT_STATUS_BASE, BBIF_COEX_INT_STATUS_BLE_RX_EVENT_FLAG_POS))

/* BBIF_COEX_INT_STATUS settings */
#define BLE_TX_EVENT_NO_INT_BITBAND     0x0
#define BLE_TX_EVENT_INT_BITBAND        0x1
#define BLE_TX_EVENT_NO_INT             (BLE_TX_EVENT_NO_INT_BITBAND << BBIF_COEX_INT_STATUS_BLE_TX_EVENT_FLAG_POS)
#define BLE_TX_EVENT_INT                (BLE_TX_EVENT_INT_BITBAND << BBIF_COEX_INT_STATUS_BLE_TX_EVENT_FLAG_POS)

#define BLE_RX_EVENT_NO_INT_BITBAND     0x0
#define BLE_RX_EVENT_INT_BITBAND        0x1
#define BLE_RX_EVENT_NO_INT             (BLE_RX_EVENT_NO_INT_BITBAND << BBIF_COEX_INT_STATUS_BLE_RX_EVENT_FLAG_POS)
#define BLE_RX_EVENT_INT                (BLE_RX_EVENT_INT_BITBAND << BBIF_COEX_INT_STATUS_BLE_RX_EVENT_FLAG_POS)

/* BLE and RF link synchronization configuration register */
#define BBIF_SYNC_CFG_BASE              0x40001418
#define BBIF_SYNC_CFG                   REG32_POINTER(BBIF_SYNC_CFG_BASE)

/* BBIF_SYNC_CFG bit positions */
#define BBIF_SYNC_CFG_RF_RX_POS         17
#define BBIF_SYNC_CFG_RF_ACTIVE_POS     16
#define BBIF_SYNC_CFG_LINK_FORMAT_POS   11
#define BBIF_SYNC_CFG_LINK_FORMAT_MASK  (0x1F << BBIF_SYNC_CFG_LINK_FORMAT_POS)
#define BBIF_SYNC_CFG_LINK_LABEL_POS    4
#define BBIF_SYNC_CFG_LINK_LABEL_MASK   (0x1F << BBIF_SYNC_CFG_LINK_LABEL_POS)
#define BBIF_SYNC_CFG_SOURCE_POS        1
#define BBIF_SYNC_CFG_SOURCE_MASK       (0x7 << BBIF_SYNC_CFG_SOURCE_POS)
#define BBIF_SYNC_CFG_ENABLE_POS        0

/* BBIF_SYNC_CFG bit-band aliases */
#define BBIF_SYNC_CFG_RF_RX_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(BBIF_SYNC_CFG_BASE, BBIF_SYNC_CFG_RF_RX_POS))
#define BBIF_SYNC_CFG_RF_ACTIVE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BBIF_SYNC_CFG_BASE, BBIF_SYNC_CFG_RF_ACTIVE_POS))
#define BBIF_SYNC_CFG_ENABLE_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BBIF_SYNC_CFG_BASE, BBIF_SYNC_CFG_ENABLE_POS))

/* BBIF_SYNC_CFG settings */
#define RX_IDLE_BITBAND                 0x0
#define RX_ACTIVE_BITBAND               0x1
#define RX_IDLE                         (RX_IDLE_BITBAND << BBIF_SYNC_CFG_RF_RX_POS)
#define RX_ACTIVE                       (RX_ACTIVE_BITBAND << BBIF_SYNC_CFG_RF_RX_POS)

#define IDLE_BITBAND                    0x0
#define ACTIVE_BITBAND                  0x1
#define IDLE                            (IDLE_BITBAND << BBIF_SYNC_CFG_RF_ACTIVE_POS)
#define ACTIVE                          (ACTIVE_BITBAND << BBIF_SYNC_CFG_RF_ACTIVE_POS)

#define SYNC_SOURCE_BLE_RX              (0x0 << BBIF_SYNC_CFG_SOURCE_POS)
#define SYNC_SOURCE_BLE_RX_AUDIO0       (0x1 << BBIF_SYNC_CFG_SOURCE_POS)
#define SYNC_SOURCE_BLE_RX_AUDIO1       (0x2 << BBIF_SYNC_CFG_SOURCE_POS)
#define SYNC_SOURCE_BLE_RX_AUDIO2       (0x3 << BBIF_SYNC_CFG_SOURCE_POS)
#define SYNC_SOURCE_RF_RX               (0x4 << BBIF_SYNC_CFG_SOURCE_POS)
#define SYNC_SOURCE_BLE_TX              (0x5 << BBIF_SYNC_CFG_SOURCE_POS)

#define SYNC_DISABLE_BITBAND            0x0
#define SYNC_ENABLE_BITBAND             0x1
#define SYNC_DISABLE                    (SYNC_DISABLE_BITBAND << BBIF_SYNC_CFG_ENABLE_POS)
#define SYNC_ENABLE                     (SYNC_ENABLE_BITBAND << BBIF_SYNC_CFG_ENABLE_POS)

/* ----------------------------------------------------------------------------
 * Baseband Controller
 * ------------------------------------------------------------------------- */

/* Baseband control register */
#define BB_RWBBCNTL_BASE                0x40001500
#define BB_RWBBCNTL                     REG32_POINTER(BB_RWBBCNTL_BASE)

/* BB_RWBBCNTL bit positions */
#define BB_RWBBCNTL_MASTER_SOFT_RST_POS 31
#define BB_RWBBCNTL_MASTER_TGSOFT_RST_POS 30
#define BB_RWBBCNTL_REG_SOFT_RST_POS    29
#define BB_RWBBCNTL_SWINT_REQ_POS       28
#define BB_RWBBCNTL_RFTEST_ABORT_POS    26
#define BB_RWBBCNTL_ADVERT_ABORT_POS    25
#define BB_RWBBCNTL_SCAN_ABORT_POS      24
#define BB_RWBBCNTL_MD_DSB_POS          22
#define BB_RWBBCNTL_SN_DSB_POS          21
#define BB_RWBBCNTL_NESN_DSB_POS        20
#define BB_RWBBCNTL_CRYPT_DSB_POS       19
#define BB_RWBBCNTL_WHIT_DSB_POS        18
#define BB_RWBBCNTL_CRC_DSB_POS         17
#define BB_RWBBCNTL_HOP_REMAP_DSB_POS   16
#define BB_RWBBCNTL_ADVERTFILT_EN_POS   9
#define BB_RWBBCNTL_RWBLE_EN_POS        8
#define BB_RWBBCNTL_RXWINSZDEF_POS      4
#define BB_RWBBCNTL_RXWINSZDEF_MASK     (0xF << BB_RWBBCNTL_RXWINSZDEF_POS)
#define BB_RWBBCNTL_SYNCERR_POS         0
#define BB_RWBBCNTL_SYNCERR_MASK        (0x7 << BB_RWBBCNTL_SYNCERR_POS)

/* BB_RWBBCNTL bit-band aliases */
#define BB_RWBBCNTL_MASTER_SOFT_RST_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_MASTER_SOFT_RST_POS))
#define BB_RWBBCNTL_MASTER_TGSOFT_RST_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_MASTER_TGSOFT_RST_POS))
#define BB_RWBBCNTL_REG_SOFT_RST_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_REG_SOFT_RST_POS))
#define BB_RWBBCNTL_SWINT_REQ_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_SWINT_REQ_POS))
#define BB_RWBBCNTL_RFTEST_ABORT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_RFTEST_ABORT_POS))
#define BB_RWBBCNTL_ADVERT_ABORT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_ADVERT_ABORT_POS))
#define BB_RWBBCNTL_SCAN_ABORT_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_SCAN_ABORT_POS))
#define BB_RWBBCNTL_MD_DSB_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_MD_DSB_POS))
#define BB_RWBBCNTL_SN_DSB_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_SN_DSB_POS))
#define BB_RWBBCNTL_NESN_DSB_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_NESN_DSB_POS))
#define BB_RWBBCNTL_CRYPT_DSB_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_CRYPT_DSB_POS))
#define BB_RWBBCNTL_WHIT_DSB_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_WHIT_DSB_POS))
#define BB_RWBBCNTL_CRC_DSB_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_CRC_DSB_POS))
#define BB_RWBBCNTL_HOP_REMAP_DSB_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_HOP_REMAP_DSB_POS))
#define BB_RWBBCNTL_ADVERTFILT_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_ADVERTFILT_EN_POS))
#define BB_RWBBCNTL_RWBLE_EN_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_RWBBCNTL_BASE, BB_RWBBCNTL_RWBLE_EN_POS))

/* BB_RWBBCNTL settings */
#define MASTER_SOFT_RST_0_BITBAND       0x0
#define MASTER_SOFT_RST_1_BITBAND       0x1
#define MASTER_SOFT_RST_0               (MASTER_SOFT_RST_0_BITBAND << BB_RWBBCNTL_MASTER_SOFT_RST_POS)
#define MASTER_SOFT_RST_1               (MASTER_SOFT_RST_1_BITBAND << BB_RWBBCNTL_MASTER_SOFT_RST_POS)

#define MASTER_TGSOFT_RST_0_BITBAND     0x0
#define MASTER_TGSOFT_RST_1_BITBAND     0x1
#define MASTER_TGSOFT_RST_0             (MASTER_TGSOFT_RST_0_BITBAND << BB_RWBBCNTL_MASTER_TGSOFT_RST_POS)
#define MASTER_TGSOFT_RST_1             (MASTER_TGSOFT_RST_1_BITBAND << BB_RWBBCNTL_MASTER_TGSOFT_RST_POS)

#define REG_SOFT_RST_0_BITBAND          0x0
#define REG_SOFT_RST_1_BITBAND          0x1
#define REG_SOFT_RST_0                  (REG_SOFT_RST_0_BITBAND << BB_RWBBCNTL_REG_SOFT_RST_POS)
#define REG_SOFT_RST_1                  (REG_SOFT_RST_1_BITBAND << BB_RWBBCNTL_REG_SOFT_RST_POS)

#define SWINT_REQ_0_BITBAND             0x0
#define SWINT_REQ_1_BITBAND             0x1
#define SWINT_REQ_0                     (SWINT_REQ_0_BITBAND << BB_RWBBCNTL_SWINT_REQ_POS)
#define SWINT_REQ_1                     (SWINT_REQ_1_BITBAND << BB_RWBBCNTL_SWINT_REQ_POS)

#define RFTEST_ABORT_0_BITBAND          0x0
#define RFTEST_ABORT_1_BITBAND          0x1
#define RFTEST_ABORT_0                  (RFTEST_ABORT_0_BITBAND << BB_RWBBCNTL_RFTEST_ABORT_POS)
#define RFTEST_ABORT_1                  (RFTEST_ABORT_1_BITBAND << BB_RWBBCNTL_RFTEST_ABORT_POS)

#define ADVERT_ABORT_0_BITBAND          0x0
#define ADVERT_ABORT_1_BITBAND          0x1
#define ADVERT_ABORT_0                  (ADVERT_ABORT_0_BITBAND << BB_RWBBCNTL_ADVERT_ABORT_POS)
#define ADVERT_ABORT_1                  (ADVERT_ABORT_1_BITBAND << BB_RWBBCNTL_ADVERT_ABORT_POS)

#define SCAN_ABORT_0_BITBAND            0x0
#define SCAN_ABORT_1_BITBAND            0x1
#define SCAN_ABORT_0                    (SCAN_ABORT_0_BITBAND << BB_RWBBCNTL_SCAN_ABORT_POS)
#define SCAN_ABORT_1                    (SCAN_ABORT_1_BITBAND << BB_RWBBCNTL_SCAN_ABORT_POS)

#define MD_DSB_0_BITBAND                0x0
#define MD_DSB_1_BITBAND                0x1
#define MD_DSB_0                        (MD_DSB_0_BITBAND << BB_RWBBCNTL_MD_DSB_POS)
#define MD_DSB_1                        (MD_DSB_1_BITBAND << BB_RWBBCNTL_MD_DSB_POS)

#define SN_DSB_0_BITBAND                0x0
#define SN_DSB_1_BITBAND                0x1
#define SN_DSB_0                        (SN_DSB_0_BITBAND << BB_RWBBCNTL_SN_DSB_POS)
#define SN_DSB_1                        (SN_DSB_1_BITBAND << BB_RWBBCNTL_SN_DSB_POS)

#define NESN_DSB_0_BITBAND              0x0
#define NESN_DSB_1_BITBAND              0x1
#define NESN_DSB_0                      (NESN_DSB_0_BITBAND << BB_RWBBCNTL_NESN_DSB_POS)
#define NESN_DSB_1                      (NESN_DSB_1_BITBAND << BB_RWBBCNTL_NESN_DSB_POS)

#define CRYPT_DSB_0_BITBAND             0x0
#define CRYPT_DSB_1_BITBAND             0x1
#define CRYPT_DSB_0                     (CRYPT_DSB_0_BITBAND << BB_RWBBCNTL_CRYPT_DSB_POS)
#define CRYPT_DSB_1                     (CRYPT_DSB_1_BITBAND << BB_RWBBCNTL_CRYPT_DSB_POS)

#define WHIT_DSB_0_BITBAND              0x0
#define WHIT_DSB_1_BITBAND              0x1
#define WHIT_DSB_0                      (WHIT_DSB_0_BITBAND << BB_RWBBCNTL_WHIT_DSB_POS)
#define WHIT_DSB_1                      (WHIT_DSB_1_BITBAND << BB_RWBBCNTL_WHIT_DSB_POS)

#define CRC_DSB_0_BITBAND               0x0
#define CRC_DSB_1_BITBAND               0x1
#define CRC_DSB_0                       (CRC_DSB_0_BITBAND << BB_RWBBCNTL_CRC_DSB_POS)
#define CRC_DSB_1                       (CRC_DSB_1_BITBAND << BB_RWBBCNTL_CRC_DSB_POS)

#define HOP_REMAP_DSB_0_BITBAND         0x0
#define HOP_REMAP_DSB_1_BITBAND         0x1
#define HOP_REMAP_DSB_0                 (HOP_REMAP_DSB_0_BITBAND << BB_RWBBCNTL_HOP_REMAP_DSB_POS)
#define HOP_REMAP_DSB_1                 (HOP_REMAP_DSB_1_BITBAND << BB_RWBBCNTL_HOP_REMAP_DSB_POS)

#define ADVERTFILT_EN_0_BITBAND         0x0
#define ADVERTFILT_EN_1_BITBAND         0x1
#define ADVERTFILT_EN_0                 (ADVERTFILT_EN_0_BITBAND << BB_RWBBCNTL_ADVERTFILT_EN_POS)
#define ADVERTFILT_EN_1                 (ADVERTFILT_EN_1_BITBAND << BB_RWBBCNTL_ADVERTFILT_EN_POS)

#define RWBLE_EN_0_BITBAND              0x0
#define RWBLE_EN_1_BITBAND              0x1
#define RWBLE_EN_0                      (RWBLE_EN_0_BITBAND << BB_RWBBCNTL_RWBLE_EN_POS)
#define RWBLE_EN_1                      (RWBLE_EN_1_BITBAND << BB_RWBBCNTL_RWBLE_EN_POS)

#define RXWINSZDEF_0                    (0x0 << BB_RWBBCNTL_RXWINSZDEF_POS)

#define SYNCERR_0                       (0x0 << BB_RWBBCNTL_SYNCERR_POS)

/* BLE revision register */
#define BB_VERSION_BASE                 0x40001504
#define BB_VERSION                      READONLY_REG32_POINTER(BB_VERSION_BASE)

/* BB_VERSION bit positions */
#define BB_VERSION_TYP_POS              24
#define BB_VERSION_TYP_MASK             (0xFF << BB_VERSION_TYP_POS)
#define BB_VERSION_REL_POS              16
#define BB_VERSION_REL_MASK             (0xFF << BB_VERSION_REL_POS)
#define BB_VERSION_UPG_POS              8
#define BB_VERSION_UPG_MASK             (0xFF << BB_VERSION_UPG_POS)
#define BB_VERSION_BUILD_POS            0
#define BB_VERSION_BUILD_MASK           (0xFF << BB_VERSION_BUILD_POS)

/* BB_VERSION settings */
#define TYP_8                           (0x8 << BB_VERSION_TYP_POS)

#define REL_0                           (0x0 << BB_VERSION_REL_POS)

#define UPG_09                          (0x9 << BB_VERSION_UPG_POS)

#define BUILD_1                         (0x1 << BB_VERSION_BUILD_POS)

/* Baseband configuration register (compilation options dependant) */
#define BB_RWBLEBCONF_BASE              0x40001508
#define BB_RWBLEBCONF                   READONLY_REG32_POINTER(BB_RWBLEBCONF_BASE)

/* BB_RWBLEBCONF bit positions */
#define BB_RWBLEBCONF_DMMODE_POS        31
#define BB_RWBLEBCONF_ISOPORTNB_POS     24
#define BB_RWBLEBCONF_ISOPORTNB_MASK    (0x3 << BB_RWBLEBCONF_ISOPORTNB_POS)
#define BB_RWBLEBCONF_DECIPHER_POS      23
#define BB_RWBLEBCONF_COEX_POS          21
#define BB_RWBLEBCONF_RFIF_POS          16
#define BB_RWBLEBCONF_RFIF_MASK         (0x1F << BB_RWBLEBCONF_RFIF_POS)
#define BB_RWBLEBCONF_USEDBG_POS        15
#define BB_RWBLEBCONF_USECRYPT_POS      14
#define BB_RWBLEBCONF_CLK_SEL_POS       8
#define BB_RWBLEBCONF_CLK_SEL_MASK      (0x3F << BB_RWBLEBCONF_CLK_SEL_POS)
#define BB_RWBLEBCONF_INTMODE_POS       7
#define BB_RWBLEBCONF_BUSTYPE_POS       6
#define BB_RWBLEBCONF_DATA_WIDTH_POS    5
#define BB_RWBLEBCONF_ADD_WIDTH_POS     0
#define BB_RWBLEBCONF_ADD_WIDTH_MASK    (0x1F << BB_RWBLEBCONF_ADD_WIDTH_POS)

/* BB_RWBLEBCONF bit-band aliases */
#define BB_RWBLEBCONF_DMMODE_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RWBLEBCONF_BASE, BB_RWBLEBCONF_DMMODE_POS))
#define BB_RWBLEBCONF_DECIPHER_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RWBLEBCONF_BASE, BB_RWBLEBCONF_DECIPHER_POS))
#define BB_RWBLEBCONF_COEX_BITBAND      READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RWBLEBCONF_BASE, BB_RWBLEBCONF_COEX_POS))
#define BB_RWBLEBCONF_USEDBG_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RWBLEBCONF_BASE, BB_RWBLEBCONF_USEDBG_POS))
#define BB_RWBLEBCONF_USECRYPT_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RWBLEBCONF_BASE, BB_RWBLEBCONF_USECRYPT_POS))
#define BB_RWBLEBCONF_INTMODE_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RWBLEBCONF_BASE, BB_RWBLEBCONF_INTMODE_POS))
#define BB_RWBLEBCONF_BUSTYPE_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RWBLEBCONF_BASE, BB_RWBLEBCONF_BUSTYPE_POS))
#define BB_RWBLEBCONF_DATA_WIDTH_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RWBLEBCONF_BASE, BB_RWBLEBCONF_DATA_WIDTH_POS))

/* BB_RWBLEBCONF settings */
#define DMMODE_0_BITBAND                0x0
#define DMMODE_1_BITBAND                0x1
#define DMMODE_0                        (DMMODE_0_BITBAND << BB_RWBLEBCONF_DMMODE_POS)
#define DMMODE_1                        (DMMODE_1_BITBAND << BB_RWBLEBCONF_DMMODE_POS)

#define NO_ISO_CH                       (0x0 << BB_RWBLEBCONF_ISOPORTNB_POS)
#define ONE_ISO_CH                      (0x1 << BB_RWBLEBCONF_ISOPORTNB_POS)
#define TWO_ISO_CH                      (0x2 << BB_RWBLEBCONF_ISOPORTNB_POS)
#define THREE_ISO_CH                    (0x3 << BB_RWBLEBCONF_ISOPORTNB_POS)

#define DECIPHER_0_BITBAND              0x0
#define DECIPHER_1_BITBAND              0x1
#define DECIPHER_0                      (DECIPHER_0_BITBAND << BB_RWBLEBCONF_DECIPHER_POS)
#define DECIPHER_1                      (DECIPHER_1_BITBAND << BB_RWBLEBCONF_DECIPHER_POS)

#define COEX_0_BITBAND                  0x0
#define COEX_1_BITBAND                  0x1
#define COEX_0                          (COEX_0_BITBAND << BB_RWBLEBCONF_COEX_POS)
#define COEX_1                          (COEX_1_BITBAND << BB_RWBLEBCONF_COEX_POS)

#define RFIF_RIPPLE                     (0x0 << BB_RWBLEBCONF_RFIF_POS)
#define RFIF_EXT                        (0x1 << BB_RWBLEBCONF_RFIF_POS)
#define RFIF_ATLAS                      (0x2 << BB_RWBLEBCONF_RFIF_POS)
#define RFIF_ICYTRX                     (0x3 << BB_RWBLEBCONF_RFIF_POS)

#define USEDBG_0_BITBAND                0x0
#define USEDBG_1_BITBAND                0x1
#define USEDBG_0                        (USEDBG_0_BITBAND << BB_RWBLEBCONF_USEDBG_POS)
#define USEDBG_1                        (USEDBG_1_BITBAND << BB_RWBLEBCONF_USEDBG_POS)

#define USECRYPT_0_BITBAND              0x0
#define USECRYPT_1_BITBAND              0x1
#define USECRYPT_0                      (USECRYPT_0_BITBAND << BB_RWBLEBCONF_USECRYPT_POS)
#define USECRYPT_1                      (USECRYPT_1_BITBAND << BB_RWBLEBCONF_USECRYPT_POS)

#define CLK_SEL_8                       (0x8 << BB_RWBLEBCONF_CLK_SEL_POS)

#define INTMODE_0_BITBAND               0x0
#define INTMODE_1_BITBAND               0x1
#define INTMODE_0                       (INTMODE_0_BITBAND << BB_RWBLEBCONF_INTMODE_POS)
#define INTMODE_1                       (INTMODE_1_BITBAND << BB_RWBLEBCONF_INTMODE_POS)

#define BUSTYPE_0_BITBAND               0x0
#define BUSTYPE_1_BITBAND               0x1
#define BUSTYPE_0                       (BUSTYPE_0_BITBAND << BB_RWBLEBCONF_BUSTYPE_POS)
#define BUSTYPE_1                       (BUSTYPE_1_BITBAND << BB_RWBLEBCONF_BUSTYPE_POS)

#define DATA_WIDTH_0_BITBAND            0x0
#define DATA_WIDTH_1_BITBAND            0x1
#define DATA_WIDTH_0                    (DATA_WIDTH_0_BITBAND << BB_RWBLEBCONF_DATA_WIDTH_POS)
#define DATA_WIDTH_1                    (DATA_WIDTH_1_BITBAND << BB_RWBLEBCONF_DATA_WIDTH_POS)

#define ADD_WIDTH_14                    (0xE << BB_RWBLEBCONF_ADD_WIDTH_POS)

/* Interrupts control register */
#define BB_INTCNTL_BASE                 0x4000150C
#define BB_INTCNTL                      REG32_POINTER(BB_INTCNTL_BASE)

/* BB_INTCNTL bit positions */
#define BB_INTCNTL_CSCNTDEVMSK_POS      15
#define BB_INTCNTL_AUDIOINT2MSK_POS     12
#define BB_INTCNTL_AUDIOINT1MSK_POS     11
#define BB_INTCNTL_AUDIOINT0MSK_POS     10
#define BB_INTCNTL_SWINTMSK_POS         9
#define BB_INTCNTL_EVENTAPFAINTMSK_POS  8
#define BB_INTCNTL_FINETGTIMINTMSK_POS  7
#define BB_INTCNTL_GROSSTGTIMINTMSK_POS 6
#define BB_INTCNTL_ERRORINTMSK_POS      5
#define BB_INTCNTL_CRYPTINTMSK_POS      4
#define BB_INTCNTL_EVENTINTMSK_POS      3
#define BB_INTCNTL_SLPINTMSK_POS        2
#define BB_INTCNTL_RXINTMSK_POS         1
#define BB_INTCNTL_CSCNTINTMSK_POS      0

/* BB_INTCNTL bit-band aliases */
#define BB_INTCNTL_CSCNTDEVMSK_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_CSCNTDEVMSK_POS))
#define BB_INTCNTL_AUDIOINT2MSK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_AUDIOINT2MSK_POS))
#define BB_INTCNTL_AUDIOINT1MSK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_AUDIOINT1MSK_POS))
#define BB_INTCNTL_AUDIOINT0MSK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_AUDIOINT0MSK_POS))
#define BB_INTCNTL_SWINTMSK_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_SWINTMSK_POS))
#define BB_INTCNTL_EVENTAPFAINTMSK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_EVENTAPFAINTMSK_POS))
#define BB_INTCNTL_FINETGTIMINTMSK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_FINETGTIMINTMSK_POS))
#define BB_INTCNTL_GROSSTGTIMINTMSK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_GROSSTGTIMINTMSK_POS))
#define BB_INTCNTL_ERRORINTMSK_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_ERRORINTMSK_POS))
#define BB_INTCNTL_CRYPTINTMSK_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_CRYPTINTMSK_POS))
#define BB_INTCNTL_EVENTINTMSK_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_EVENTINTMSK_POS))
#define BB_INTCNTL_SLPINTMSK_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_SLPINTMSK_POS))
#define BB_INTCNTL_RXINTMSK_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_RXINTMSK_POS))
#define BB_INTCNTL_CSCNTINTMSK_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_INTCNTL_BASE, BB_INTCNTL_CSCNTINTMSK_POS))

/* BB_INTCNTL settings */
#define CSCNTDEVMSK_0_BITBAND           0x0
#define CSCNTDEVMSK_1_BITBAND           0x1
#define CSCNTDEVMSK_0                   (CSCNTDEVMSK_0_BITBAND << BB_INTCNTL_CSCNTDEVMSK_POS)
#define CSCNTDEVMSK_1                   (CSCNTDEVMSK_1_BITBAND << BB_INTCNTL_CSCNTDEVMSK_POS)

#define AUDIOINT2MSK_0_BITBAND          0x0
#define AUDIOINT2MSK_1_BITBAND          0x1
#define AUDIOINT2MSK_0                  (AUDIOINT2MSK_0_BITBAND << BB_INTCNTL_AUDIOINT2MSK_POS)
#define AUDIOINT2MSK_1                  (AUDIOINT2MSK_1_BITBAND << BB_INTCNTL_AUDIOINT2MSK_POS)

#define AUDIOINT1MSK_0_BITBAND          0x0
#define AUDIOINT1MSK_1_BITBAND          0x1
#define AUDIOINT1MSK_0                  (AUDIOINT1MSK_0_BITBAND << BB_INTCNTL_AUDIOINT1MSK_POS)
#define AUDIOINT1MSK_1                  (AUDIOINT1MSK_1_BITBAND << BB_INTCNTL_AUDIOINT1MSK_POS)

#define AUDIOINT0MSK_0_BITBAND          0x0
#define AUDIOINT0MSK_1_BITBAND          0x1
#define AUDIOINT0MSK_0                  (AUDIOINT0MSK_0_BITBAND << BB_INTCNTL_AUDIOINT0MSK_POS)
#define AUDIOINT0MSK_1                  (AUDIOINT0MSK_1_BITBAND << BB_INTCNTL_AUDIOINT0MSK_POS)

#define SWINTMSK_0_BITBAND              0x0
#define SWINTMSK_1_BITBAND              0x1
#define SWINTMSK_0                      (SWINTMSK_0_BITBAND << BB_INTCNTL_SWINTMSK_POS)
#define SWINTMSK_1                      (SWINTMSK_1_BITBAND << BB_INTCNTL_SWINTMSK_POS)

#define EVENTAPFAINTMSK_0_BITBAND       0x0
#define EVENTAPFAINTMSK_1_BITBAND       0x1
#define EVENTAPFAINTMSK_0               (EVENTAPFAINTMSK_0_BITBAND << BB_INTCNTL_EVENTAPFAINTMSK_POS)
#define EVENTAPFAINTMSK_1               (EVENTAPFAINTMSK_1_BITBAND << BB_INTCNTL_EVENTAPFAINTMSK_POS)

#define FINETGTIMINTMSK_0_BITBAND       0x0
#define FINETGTIMINTMSK_1_BITBAND       0x1
#define FINETGTIMINTMSK_0               (FINETGTIMINTMSK_0_BITBAND << BB_INTCNTL_FINETGTIMINTMSK_POS)
#define FINETGTIMINTMSK_1               (FINETGTIMINTMSK_1_BITBAND << BB_INTCNTL_FINETGTIMINTMSK_POS)

#define GROSSTGTIMINTMSK_0_BITBAND      0x0
#define GROSSTGTIMINTMSK_1_BITBAND      0x1
#define GROSSTGTIMINTMSK_0              (GROSSTGTIMINTMSK_0_BITBAND << BB_INTCNTL_GROSSTGTIMINTMSK_POS)
#define GROSSTGTIMINTMSK_1              (GROSSTGTIMINTMSK_1_BITBAND << BB_INTCNTL_GROSSTGTIMINTMSK_POS)

#define ERRORINTMSK_0_BITBAND           0x0
#define ERRORINTMSK_1_BITBAND           0x1
#define ERRORINTMSK_0                   (ERRORINTMSK_0_BITBAND << BB_INTCNTL_ERRORINTMSK_POS)
#define ERRORINTMSK_1                   (ERRORINTMSK_1_BITBAND << BB_INTCNTL_ERRORINTMSK_POS)

#define CRYPTINTMSK_0_BITBAND           0x0
#define CRYPTINTMSK_1_BITBAND           0x1
#define CRYPTINTMSK_0                   (CRYPTINTMSK_0_BITBAND << BB_INTCNTL_CRYPTINTMSK_POS)
#define CRYPTINTMSK_1                   (CRYPTINTMSK_1_BITBAND << BB_INTCNTL_CRYPTINTMSK_POS)

#define EVENTINTMSK_0_BITBAND           0x0
#define EVENTINTMSK_1_BITBAND           0x1
#define EVENTINTMSK_0                   (EVENTINTMSK_0_BITBAND << BB_INTCNTL_EVENTINTMSK_POS)
#define EVENTINTMSK_1                   (EVENTINTMSK_1_BITBAND << BB_INTCNTL_EVENTINTMSK_POS)

#define SLPINTMSK_0_BITBAND             0x0
#define SLPINTMSK_1_BITBAND             0x1
#define SLPINTMSK_0                     (SLPINTMSK_0_BITBAND << BB_INTCNTL_SLPINTMSK_POS)
#define SLPINTMSK_1                     (SLPINTMSK_1_BITBAND << BB_INTCNTL_SLPINTMSK_POS)

#define RXINTMSK_0_BITBAND              0x0
#define RXINTMSK_1_BITBAND              0x1
#define RXINTMSK_0                      (RXINTMSK_0_BITBAND << BB_INTCNTL_RXINTMSK_POS)
#define RXINTMSK_1                      (RXINTMSK_1_BITBAND << BB_INTCNTL_RXINTMSK_POS)

#define CSCNTINTMSK_0_BITBAND           0x0
#define CSCNTINTMSK_1_BITBAND           0x1
#define CSCNTINTMSK_0                   (CSCNTINTMSK_0_BITBAND << BB_INTCNTL_CSCNTINTMSK_POS)
#define CSCNTINTMSK_1                   (CSCNTINTMSK_1_BITBAND << BB_INTCNTL_CSCNTINTMSK_POS)

/* Interrupts status register */
#define BB_INTSTAT_BASE                 0x40001510
#define BB_INTSTAT                      READONLY_REG32_POINTER(BB_INTSTAT_BASE)

/* BB_INTSTAT bit positions */
#define BB_INTSTAT_AUDIOINT2STAT_POS    12
#define BB_INTSTAT_AUDIOINT1STAT_POS    11
#define BB_INTSTAT_AUDIOINT0STAT_POS    10
#define BB_INTSTAT_SWINTSTAT_POS        9
#define BB_INTSTAT_EVENTAPFAINTSTAT_POS 8
#define BB_INTSTAT_FINETGTIMINTSTAT_POS 7
#define BB_INTSTAT_GROSSTGTIMINTSTAT_POS 6
#define BB_INTSTAT_ERRORINTSTAT_POS     5
#define BB_INTSTAT_CRYPTINTSTAT_POS     4
#define BB_INTSTAT_EVENTINTSTAT_POS     3
#define BB_INTSTAT_SLPINTSTAT_POS       2
#define BB_INTSTAT_RXINTSTAT_POS        1
#define BB_INTSTAT_CSCNTINTSTAT_POS     0

/* BB_INTSTAT bit-band aliases */
#define BB_INTSTAT_AUDIOINT2STAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_AUDIOINT2STAT_POS))
#define BB_INTSTAT_AUDIOINT1STAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_AUDIOINT1STAT_POS))
#define BB_INTSTAT_AUDIOINT0STAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_AUDIOINT0STAT_POS))
#define BB_INTSTAT_SWINTSTAT_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_SWINTSTAT_POS))
#define BB_INTSTAT_EVENTAPFAINTSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_EVENTAPFAINTSTAT_POS))
#define BB_INTSTAT_FINETGTIMINTSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_FINETGTIMINTSTAT_POS))
#define BB_INTSTAT_GROSSTGTIMINTSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_GROSSTGTIMINTSTAT_POS))
#define BB_INTSTAT_ERRORINTSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_ERRORINTSTAT_POS))
#define BB_INTSTAT_CRYPTINTSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_CRYPTINTSTAT_POS))
#define BB_INTSTAT_EVENTINTSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_EVENTINTSTAT_POS))
#define BB_INTSTAT_SLPINTSTAT_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_SLPINTSTAT_POS))
#define BB_INTSTAT_RXINTSTAT_BITBAND    READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_RXINTSTAT_POS))
#define BB_INTSTAT_CSCNTINTSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTSTAT_BASE, BB_INTSTAT_CSCNTINTSTAT_POS))

/* BB_INTSTAT settings */
#define AUDIOINT2STAT_0_BITBAND         0x0
#define AUDIOINT2STAT_1_BITBAND         0x1
#define AUDIOINT2STAT_0                 (AUDIOINT2STAT_0_BITBAND << BB_INTSTAT_AUDIOINT2STAT_POS)
#define AUDIOINT2STAT_1                 (AUDIOINT2STAT_1_BITBAND << BB_INTSTAT_AUDIOINT2STAT_POS)

#define AUDIOINT1STAT_0_BITBAND         0x0
#define AUDIOINT1STAT_1_BITBAND         0x1
#define AUDIOINT1STAT_0                 (AUDIOINT1STAT_0_BITBAND << BB_INTSTAT_AUDIOINT1STAT_POS)
#define AUDIOINT1STAT_1                 (AUDIOINT1STAT_1_BITBAND << BB_INTSTAT_AUDIOINT1STAT_POS)

#define AUDIOINT0STAT_0_BITBAND         0x0
#define AUDIOINT0STAT_1_BITBAND         0x1
#define AUDIOINT0STAT_0                 (AUDIOINT0STAT_0_BITBAND << BB_INTSTAT_AUDIOINT0STAT_POS)
#define AUDIOINT0STAT_1                 (AUDIOINT0STAT_1_BITBAND << BB_INTSTAT_AUDIOINT0STAT_POS)

#define SWINTSTAT_0_BITBAND             0x0
#define SWINTSTAT_1_BITBAND             0x1
#define SWINTSTAT_0                     (SWINTSTAT_0_BITBAND << BB_INTSTAT_SWINTSTAT_POS)
#define SWINTSTAT_1                     (SWINTSTAT_1_BITBAND << BB_INTSTAT_SWINTSTAT_POS)

#define EVENTAPFAINTSTAT_0_BITBAND      0x0
#define EVENTAPFAINTSTAT_1_BITBAND      0x1
#define EVENTAPFAINTSTAT_0              (EVENTAPFAINTSTAT_0_BITBAND << BB_INTSTAT_EVENTAPFAINTSTAT_POS)
#define EVENTAPFAINTSTAT_1              (EVENTAPFAINTSTAT_1_BITBAND << BB_INTSTAT_EVENTAPFAINTSTAT_POS)

#define FINETGTIMINTSTAT_0_BITBAND      0x0
#define FINETGTIMINTSTAT_1_BITBAND      0x1
#define FINETGTIMINTSTAT_0              (FINETGTIMINTSTAT_0_BITBAND << BB_INTSTAT_FINETGTIMINTSTAT_POS)
#define FINETGTIMINTSTAT_1              (FINETGTIMINTSTAT_1_BITBAND << BB_INTSTAT_FINETGTIMINTSTAT_POS)

#define GROSSTGTIMINTSTAT_0_BITBAND     0x0
#define GROSSTGTIMINTSTAT_1_BITBAND     0x1
#define GROSSTGTIMINTSTAT_0             (GROSSTGTIMINTSTAT_0_BITBAND << BB_INTSTAT_GROSSTGTIMINTSTAT_POS)
#define GROSSTGTIMINTSTAT_1             (GROSSTGTIMINTSTAT_1_BITBAND << BB_INTSTAT_GROSSTGTIMINTSTAT_POS)

#define ERRORINTSTAT_0_BITBAND          0x0
#define ERRORINTSTAT_1_BITBAND          0x1
#define ERRORINTSTAT_0                  (ERRORINTSTAT_0_BITBAND << BB_INTSTAT_ERRORINTSTAT_POS)
#define ERRORINTSTAT_1                  (ERRORINTSTAT_1_BITBAND << BB_INTSTAT_ERRORINTSTAT_POS)

#define CRYPTINTSTAT_0_BITBAND          0x0
#define CRYPTINTSTAT_1_BITBAND          0x1
#define CRYPTINTSTAT_0                  (CRYPTINTSTAT_0_BITBAND << BB_INTSTAT_CRYPTINTSTAT_POS)
#define CRYPTINTSTAT_1                  (CRYPTINTSTAT_1_BITBAND << BB_INTSTAT_CRYPTINTSTAT_POS)

#define EVENTINTSTAT_0_BITBAND          0x0
#define EVENTINTSTAT_1_BITBAND          0x1
#define EVENTINTSTAT_0                  (EVENTINTSTAT_0_BITBAND << BB_INTSTAT_EVENTINTSTAT_POS)
#define EVENTINTSTAT_1                  (EVENTINTSTAT_1_BITBAND << BB_INTSTAT_EVENTINTSTAT_POS)

#define SLPINTSTAT_0_BITBAND            0x0
#define SLPINTSTAT_1_BITBAND            0x1
#define SLPINTSTAT_0                    (SLPINTSTAT_0_BITBAND << BB_INTSTAT_SLPINTSTAT_POS)
#define SLPINTSTAT_1                    (SLPINTSTAT_1_BITBAND << BB_INTSTAT_SLPINTSTAT_POS)

#define RXINTSTAT_0_BITBAND             0x0
#define RXINTSTAT_1_BITBAND             0x1
#define RXINTSTAT_0                     (RXINTSTAT_0_BITBAND << BB_INTSTAT_RXINTSTAT_POS)
#define RXINTSTAT_1                     (RXINTSTAT_1_BITBAND << BB_INTSTAT_RXINTSTAT_POS)

#define CSCNTINTSTAT_0_BITBAND          0x0
#define CSCNTINTSTAT_1_BITBAND          0x1
#define CSCNTINTSTAT_0                  (CSCNTINTSTAT_0_BITBAND << BB_INTSTAT_CSCNTINTSTAT_POS)
#define CSCNTINTSTAT_1                  (CSCNTINTSTAT_1_BITBAND << BB_INTSTAT_CSCNTINTSTAT_POS)

/* Interrupts raw status register */
#define BB_INTRAWSTAT_BASE              0x40001514
#define BB_INTRAWSTAT                   READONLY_REG32_POINTER(BB_INTRAWSTAT_BASE)

/* BB_INTRAWSTAT bit positions */
#define BB_INTRAWSTAT_AUDIOINT2RAWSTAT_POS 12
#define BB_INTRAWSTAT_AUDIOINT1RAWSTAT_POS 11
#define BB_INTRAWSTAT_AUDIOINT0RAWSTAT_POS 10
#define BB_INTRAWSTAT_SWINTRAWSTAT_POS  9
#define BB_INTRAWSTAT_EVENTAPFAINTRAWSTAT_POS 8
#define BB_INTRAWSTAT_FINETGTIMINTRAWSTAT_POS 7
#define BB_INTRAWSTAT_GROSSTGTIMINTRAWSTAT_POS 6
#define BB_INTRAWSTAT_ERRORINTRAWSTAT_POS 5
#define BB_INTRAWSTAT_CRYPTINTRAWSTAT_POS 4
#define BB_INTRAWSTAT_EVENTINTRAWSTAT_POS 3
#define BB_INTRAWSTAT_SLPINTRAWSTAT_POS 2
#define BB_INTRAWSTAT_RXINTRAWSTAT_POS  1
#define BB_INTRAWSTAT_CSCNTINTRAWSTAT_POS 0

/* BB_INTRAWSTAT bit-band aliases */
#define BB_INTRAWSTAT_AUDIOINT2RAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_AUDIOINT2RAWSTAT_POS))
#define BB_INTRAWSTAT_AUDIOINT1RAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_AUDIOINT1RAWSTAT_POS))
#define BB_INTRAWSTAT_AUDIOINT0RAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_AUDIOINT0RAWSTAT_POS))
#define BB_INTRAWSTAT_SWINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_SWINTRAWSTAT_POS))
#define BB_INTRAWSTAT_EVENTAPFAINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_EVENTAPFAINTRAWSTAT_POS))
#define BB_INTRAWSTAT_FINETGTIMINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_FINETGTIMINTRAWSTAT_POS))
#define BB_INTRAWSTAT_GROSSTGTIMINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_GROSSTGTIMINTRAWSTAT_POS))
#define BB_INTRAWSTAT_ERRORINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_ERRORINTRAWSTAT_POS))
#define BB_INTRAWSTAT_CRYPTINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_CRYPTINTRAWSTAT_POS))
#define BB_INTRAWSTAT_EVENTINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_EVENTINTRAWSTAT_POS))
#define BB_INTRAWSTAT_SLPINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_SLPINTRAWSTAT_POS))
#define BB_INTRAWSTAT_RXINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_RXINTRAWSTAT_POS))
#define BB_INTRAWSTAT_CSCNTINTRAWSTAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTRAWSTAT_BASE, BB_INTRAWSTAT_CSCNTINTRAWSTAT_POS))

/* BB_INTRAWSTAT settings */
#define AUDIOINT2RAWSTAT_0_BITBAND      0x0
#define AUDIOINT2RAWSTAT_1_BITBAND      0x1
#define AUDIOINT2RAWSTAT_0              (AUDIOINT2RAWSTAT_0_BITBAND << BB_INTRAWSTAT_AUDIOINT2RAWSTAT_POS)
#define AUDIOINT2RAWSTAT_1              (AUDIOINT2RAWSTAT_1_BITBAND << BB_INTRAWSTAT_AUDIOINT2RAWSTAT_POS)

#define AUDIOINT1RAWSTAT_0_BITBAND      0x0
#define AUDIOINT1RAWSTAT_1_BITBAND      0x1
#define AUDIOINT1RAWSTAT_0              (AUDIOINT1RAWSTAT_0_BITBAND << BB_INTRAWSTAT_AUDIOINT1RAWSTAT_POS)
#define AUDIOINT1RAWSTAT_1              (AUDIOINT1RAWSTAT_1_BITBAND << BB_INTRAWSTAT_AUDIOINT1RAWSTAT_POS)

#define AUDIOINT0RAWSTAT_0_BITBAND      0x0
#define AUDIOINT0RAWSTAT_1_BITBAND      0x1
#define AUDIOINT0RAWSTAT_0              (AUDIOINT0RAWSTAT_0_BITBAND << BB_INTRAWSTAT_AUDIOINT0RAWSTAT_POS)
#define AUDIOINT0RAWSTAT_1              (AUDIOINT0RAWSTAT_1_BITBAND << BB_INTRAWSTAT_AUDIOINT0RAWSTAT_POS)

#define SWINTRAWSTAT_0_BITBAND          0x0
#define SWINTRAWSTAT_1_BITBAND          0x1
#define SWINTRAWSTAT_0                  (SWINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_SWINTRAWSTAT_POS)
#define SWINTRAWSTAT_1                  (SWINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_SWINTRAWSTAT_POS)

#define EVENTAPFAINTRAWSTAT_0_BITBAND   0x0
#define EVENTAPFAINTRAWSTAT_1_BITBAND   0x1
#define EVENTAPFAINTRAWSTAT_0           (EVENTAPFAINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_EVENTAPFAINTRAWSTAT_POS)
#define EVENTAPFAINTRAWSTAT_1           (EVENTAPFAINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_EVENTAPFAINTRAWSTAT_POS)

#define FINETGTIMINTRAWSTAT_0_BITBAND   0x0
#define FINETGTIMINTRAWSTAT_1_BITBAND   0x1
#define FINETGTIMINTRAWSTAT_0           (FINETGTIMINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_FINETGTIMINTRAWSTAT_POS)
#define FINETGTIMINTRAWSTAT_1           (FINETGTIMINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_FINETGTIMINTRAWSTAT_POS)

#define GROSSTGTIMINTRAWSTAT_0_BITBAND  0x0
#define GROSSTGTIMINTRAWSTAT_1_BITBAND  0x1
#define GROSSTGTIMINTRAWSTAT_0          (GROSSTGTIMINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_GROSSTGTIMINTRAWSTAT_POS)
#define GROSSTGTIMINTRAWSTAT_1          (GROSSTGTIMINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_GROSSTGTIMINTRAWSTAT_POS)

#define ERRORINTRAWSTAT_0_BITBAND       0x0
#define ERRORINTRAWSTAT_1_BITBAND       0x1
#define ERRORINTRAWSTAT_0               (ERRORINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_ERRORINTRAWSTAT_POS)
#define ERRORINTRAWSTAT_1               (ERRORINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_ERRORINTRAWSTAT_POS)

#define CRYPTINTRAWSTAT_0_BITBAND       0x0
#define CRYPTINTRAWSTAT_1_BITBAND       0x1
#define CRYPTINTRAWSTAT_0               (CRYPTINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_CRYPTINTRAWSTAT_POS)
#define CRYPTINTRAWSTAT_1               (CRYPTINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_CRYPTINTRAWSTAT_POS)

#define EVENTINTRAWSTAT_0_BITBAND       0x0
#define EVENTINTRAWSTAT_1_BITBAND       0x1
#define EVENTINTRAWSTAT_0               (EVENTINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_EVENTINTRAWSTAT_POS)
#define EVENTINTRAWSTAT_1               (EVENTINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_EVENTINTRAWSTAT_POS)

#define SLPINTRAWSTAT_0_BITBAND         0x0
#define SLPINTRAWSTAT_1_BITBAND         0x1
#define SLPINTRAWSTAT_0                 (SLPINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_SLPINTRAWSTAT_POS)
#define SLPINTRAWSTAT_1                 (SLPINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_SLPINTRAWSTAT_POS)

#define RXINTRAWSTAT_0_BITBAND          0x0
#define RXINTRAWSTAT_1_BITBAND          0x1
#define RXINTRAWSTAT_0                  (RXINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_RXINTRAWSTAT_POS)
#define RXINTRAWSTAT_1                  (RXINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_RXINTRAWSTAT_POS)

#define CSCNTINTRAWSTAT_0_BITBAND       0x0
#define CSCNTINTRAWSTAT_1_BITBAND       0x1
#define CSCNTINTRAWSTAT_0               (CSCNTINTRAWSTAT_0_BITBAND << BB_INTRAWSTAT_CSCNTINTRAWSTAT_POS)
#define CSCNTINTRAWSTAT_1               (CSCNTINTRAWSTAT_1_BITBAND << BB_INTRAWSTAT_CSCNTINTRAWSTAT_POS)

/* Interrupts acknowledgement register */
#define BB_INTACK_BASE                  0x40001518
#define BB_INTACK                       REG32_POINTER(BB_INTACK_BASE)

/* BB_INTACK bit positions */
#define BB_INTACK_AUDIOINT2ACK_POS      12
#define BB_INTACK_AUDIOINT1ACK_POS      11
#define BB_INTACK_AUDIOINT0ACK_POS      10
#define BB_INTACK_SWINTACK_POS          9
#define BB_INTACK_EVENTAPFAINTACK_POS   8
#define BB_INTACK_FINETGTIMINTACK_POS   7
#define BB_INTACK_GROSSTGTIMINTACK_POS  6
#define BB_INTACK_ERRORINTACK_POS       5
#define BB_INTACK_CRYPTINTACK_POS       4
#define BB_INTACK_EVENTINTACK_POS       3
#define BB_INTACK_SLPINTACK_POS         2
#define BB_INTACK_RXINTACK_POS          1
#define BB_INTACK_CSCNTINTACK_POS       0

/* BB_INTACK bit-band aliases */
#define BB_INTACK_AUDIOINT2ACK_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_AUDIOINT2ACK_POS))
#define BB_INTACK_AUDIOINT1ACK_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_AUDIOINT1ACK_POS))
#define BB_INTACK_AUDIOINT0ACK_BITBAND  READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_AUDIOINT0ACK_POS))
#define BB_INTACK_SWINTACK_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_SWINTACK_POS))
#define BB_INTACK_EVENTAPFAINTACK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_EVENTAPFAINTACK_POS))
#define BB_INTACK_FINETGTIMINTACK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_FINETGTIMINTACK_POS))
#define BB_INTACK_GROSSTGTIMINTACK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_GROSSTGTIMINTACK_POS))
#define BB_INTACK_ERRORINTACK_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_ERRORINTACK_POS))
#define BB_INTACK_CRYPTINTACK_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_CRYPTINTACK_POS))
#define BB_INTACK_EVENTINTACK_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_EVENTINTACK_POS))
#define BB_INTACK_SLPINTACK_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_SLPINTACK_POS))
#define BB_INTACK_RXINTACK_BITBAND      REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_RXINTACK_POS))
#define BB_INTACK_CSCNTINTACK_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_INTACK_BASE, BB_INTACK_CSCNTINTACK_POS))

/* BB_INTACK settings */
#define AUDIOINT2ACK_0_BITBAND          0x0
#define AUDIOINT2ACK_1_BITBAND          0x1
#define AUDIOINT2ACK_0                  (AUDIOINT2ACK_0_BITBAND << BB_INTACK_AUDIOINT2ACK_POS)
#define AUDIOINT2ACK_1                  (AUDIOINT2ACK_1_BITBAND << BB_INTACK_AUDIOINT2ACK_POS)

#define AUDIOINT1ACK_0_BITBAND          0x0
#define AUDIOINT1ACK_1_BITBAND          0x1
#define AUDIOINT1ACK_0                  (AUDIOINT1ACK_0_BITBAND << BB_INTACK_AUDIOINT1ACK_POS)
#define AUDIOINT1ACK_1                  (AUDIOINT1ACK_1_BITBAND << BB_INTACK_AUDIOINT1ACK_POS)

#define AUDIOINT0ACK_0_BITBAND          0x0
#define AUDIOINT0ACK_1_BITBAND          0x1
#define AUDIOINT0ACK_0                  (AUDIOINT0ACK_0_BITBAND << BB_INTACK_AUDIOINT0ACK_POS)
#define AUDIOINT0ACK_1                  (AUDIOINT0ACK_1_BITBAND << BB_INTACK_AUDIOINT0ACK_POS)

#define SWINTACK_0_BITBAND              0x0
#define SWINTACK_1_BITBAND              0x1
#define SWINTACK_0                      (SWINTACK_0_BITBAND << BB_INTACK_SWINTACK_POS)
#define SWINTACK_1                      (SWINTACK_1_BITBAND << BB_INTACK_SWINTACK_POS)

#define EVENTAPFAINTACK_0_BITBAND       0x0
#define EVENTAPFAINTACK_1_BITBAND       0x1
#define EVENTAPFAINTACK_0               (EVENTAPFAINTACK_0_BITBAND << BB_INTACK_EVENTAPFAINTACK_POS)
#define EVENTAPFAINTACK_1               (EVENTAPFAINTACK_1_BITBAND << BB_INTACK_EVENTAPFAINTACK_POS)

#define FINETGTIMINTACK_0_BITBAND       0x0
#define FINETGTIMINTACK_1_BITBAND       0x1
#define FINETGTIMINTACK_0               (FINETGTIMINTACK_0_BITBAND << BB_INTACK_FINETGTIMINTACK_POS)
#define FINETGTIMINTACK_1               (FINETGTIMINTACK_1_BITBAND << BB_INTACK_FINETGTIMINTACK_POS)

#define GROSSTGTIMINTACK_0_BITBAND      0x0
#define GROSSTGTIMINTACK_1_BITBAND      0x1
#define GROSSTGTIMINTACK_0              (GROSSTGTIMINTACK_0_BITBAND << BB_INTACK_GROSSTGTIMINTACK_POS)
#define GROSSTGTIMINTACK_1              (GROSSTGTIMINTACK_1_BITBAND << BB_INTACK_GROSSTGTIMINTACK_POS)

#define ERRORINTACK_0_BITBAND           0x0
#define ERRORINTACK_1_BITBAND           0x1
#define ERRORINTACK_0                   (ERRORINTACK_0_BITBAND << BB_INTACK_ERRORINTACK_POS)
#define ERRORINTACK_1                   (ERRORINTACK_1_BITBAND << BB_INTACK_ERRORINTACK_POS)

#define CRYPTINTACK_0_BITBAND           0x0
#define CRYPTINTACK_1_BITBAND           0x1
#define CRYPTINTACK_0                   (CRYPTINTACK_0_BITBAND << BB_INTACK_CRYPTINTACK_POS)
#define CRYPTINTACK_1                   (CRYPTINTACK_1_BITBAND << BB_INTACK_CRYPTINTACK_POS)

#define EVENTINTACK_0_BITBAND           0x0
#define EVENTINTACK_1_BITBAND           0x1
#define EVENTINTACK_0                   (EVENTINTACK_0_BITBAND << BB_INTACK_EVENTINTACK_POS)
#define EVENTINTACK_1                   (EVENTINTACK_1_BITBAND << BB_INTACK_EVENTINTACK_POS)

#define SLPINTACK_0_BITBAND             0x0
#define SLPINTACK_1_BITBAND             0x1
#define SLPINTACK_0                     (SLPINTACK_0_BITBAND << BB_INTACK_SLPINTACK_POS)
#define SLPINTACK_1                     (SLPINTACK_1_BITBAND << BB_INTACK_SLPINTACK_POS)

#define RXINTACK_0_BITBAND              0x0
#define RXINTACK_1_BITBAND              0x1
#define RXINTACK_0                      (RXINTACK_0_BITBAND << BB_INTACK_RXINTACK_POS)
#define RXINTACK_1                      (RXINTACK_1_BITBAND << BB_INTACK_RXINTACK_POS)

#define CSCNTINTACK_0_BITBAND           0x0
#define CSCNTINTACK_1_BITBAND           0x1
#define CSCNTINTACK_0                   (CSCNTINTACK_0_BITBAND << BB_INTACK_CSCNTINTACK_POS)
#define CSCNTINTACK_1                   (CSCNTINTACK_1_BITBAND << BB_INTACK_CSCNTINTACK_POS)

/* base timer configuration register */
#define BB_BASETIMECNT_BASE             0x4000151C
#define BB_BASETIMECNT                  REG32_POINTER(BB_BASETIMECNT_BASE)

/* BB_BASETIMECNT bit positions */
#define BB_BASETIMECNT_SAMP_POS         31
#define BB_BASETIMECNT_BASETIMECNT_POS  0
#define BB_BASETIMECNT_BASETIMECNT_MASK (0x7FFFFFF << BB_BASETIMECNT_BASETIMECNT_POS)

/* BB_BASETIMECNT bit-band aliases */
#define BB_BASETIMECNT_SAMP_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(BB_BASETIMECNT_BASE, BB_BASETIMECNT_SAMP_POS))

/* BB_BASETIMECNT settings */
#define SAMP_0_BITBAND                  0x0
#define SAMP_1_BITBAND                  0x1
#define SAMP_0                          (SAMP_0_BITBAND << BB_BASETIMECNT_SAMP_POS)
#define SAMP_1                          (SAMP_1_BITBAND << BB_BASETIMECNT_SAMP_POS)

#define BASETIMECNT_0                   (0x0 << BB_BASETIMECNT_BASETIMECNT_POS)

/* Fine timer configuration register */
#define BB_FINETIMECNT_BASE             0x40001520
#define BB_FINETIMECNT                  READONLY_REG32_POINTER(BB_FINETIMECNT_BASE)

/* BB_FINETIMECNT bit positions */
#define BB_FINETIMECNT_FINECNT_POS      0
#define BB_FINETIMECNT_FINECNT_MASK     (0x3FF << BB_FINETIMECNT_FINECNT_POS)

/* BB_FINETIMECNT settings */
#define FINECNT_0                       (0x0 << BB_FINETIMECNT_FINECNT_POS)

/* BLE device address (LSB part) register */
#define BB_BDADDRL_BASE                 0x40001524
#define BB_BDADDRL                      REG32_POINTER(BB_BDADDRL_BASE)

/* BLE device address (MSB part) register */
#define BB_BDADDRU_BASE                 0x40001528
#define BB_BDADDRU                      REG32_POINTER(BB_BDADDRU_BASE)

/* BB_BDADDRU bit positions */
#define BB_BDADDRU_PRIV_NPUB_POS        16
#define BB_BDADDRU_BDADDRU_POS          0
#define BB_BDADDRU_BDADDRU_MASK         (0xFFFF << BB_BDADDRU_BDADDRU_POS)

/* BB_BDADDRU bit-band aliases */
#define BB_BDADDRU_PRIV_NPUB_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_BDADDRU_BASE, BB_BDADDRU_PRIV_NPUB_POS))

/* BB_BDADDRU settings */
#define PRIV_NPUB_0_BITBAND             0x0
#define PRIV_NPUB_1_BITBAND             0x1
#define PRIV_NPUB_0                     (PRIV_NPUB_0_BITBAND << BB_BDADDRU_PRIV_NPUB_POS)
#define PRIV_NPUB_1                     (PRIV_NPUB_1_BITBAND << BB_BDADDRU_PRIV_NPUB_POS)

#define BDADDRU_0                       (0x0 << BB_BDADDRU_BDADDRU_POS)

/* Rx descriptor pointer register */
#define BB_ET_CURRENTRXDESCPTR_BASE     0x4000152C
#define BB_ET_CURRENTRXDESCPTR          REG32_POINTER(BB_ET_CURRENTRXDESCPTR_BASE)

/* BB_ET_CURRENTRXDESCPTR bit positions */
#define BB_ET_CURRENTRXDESCPTR_ETPTR_POS 16
#define BB_ET_CURRENTRXDESCPTR_ETPTR_MASK (0xFFFF << BB_ET_CURRENTRXDESCPTR_ETPTR_POS)
#define BB_ET_CURRENTRXDESCPTR_CURRENTRXDESCPTR_POS 0
#define BB_ET_CURRENTRXDESCPTR_CURRENTRXDESCPTR_MASK (0x7FFF << BB_ET_CURRENTRXDESCPTR_CURRENTRXDESCPTR_POS)

/* BB_ET_CURRENTRXDESCPTR settings */
#define ETPTR_0                         (0x0 << BB_ET_CURRENTRXDESCPTR_ETPTR_POS)

#define CURRENTRXDESCPTR_0              (0x0 << BB_ET_CURRENTRXDESCPTR_CURRENTRXDESCPTR_POS)

/* Deep sleep control register */
#define BB_DEEPSLCNTL_BASE              0x40001530
#define BB_DEEPSLCNTL                   REG32_POINTER(BB_DEEPSLCNTL_BASE)

/* BB_DEEPSLCNTL bit positions */
#define BB_DEEPSLCNTL_EXTWKUPDSB_POS    31
#define BB_DEEPSLCNTL_DEEP_SLEEP_STAT_POS 15
#define BB_DEEPSLCNTL_SOFT_WAKEUP_REQ_POS 4
#define BB_DEEPSLCNTL_DEEP_SLEEP_CORR_EN_POS 3
#define BB_DEEPSLCNTL_DEEP_SLEEP_ON_POS 2
#define BB_DEEPSLCNTL_RADIO_SLEEP_EN_POS 1
#define BB_DEEPSLCNTL_OSC_SLEEP_EN_POS  0

/* BB_DEEPSLCNTL bit-band aliases */
#define BB_DEEPSLCNTL_EXTWKUPDSB_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_DEEPSLCNTL_BASE, BB_DEEPSLCNTL_EXTWKUPDSB_POS))
#define BB_DEEPSLCNTL_DEEP_SLEEP_STAT_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_DEEPSLCNTL_BASE, BB_DEEPSLCNTL_DEEP_SLEEP_STAT_POS))
#define BB_DEEPSLCNTL_SOFT_WAKEUP_REQ_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_DEEPSLCNTL_BASE, BB_DEEPSLCNTL_SOFT_WAKEUP_REQ_POS))
#define BB_DEEPSLCNTL_DEEP_SLEEP_CORR_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_DEEPSLCNTL_BASE, BB_DEEPSLCNTL_DEEP_SLEEP_CORR_EN_POS))
#define BB_DEEPSLCNTL_DEEP_SLEEP_ON_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_DEEPSLCNTL_BASE, BB_DEEPSLCNTL_DEEP_SLEEP_ON_POS))
#define BB_DEEPSLCNTL_RADIO_SLEEP_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_DEEPSLCNTL_BASE, BB_DEEPSLCNTL_RADIO_SLEEP_EN_POS))
#define BB_DEEPSLCNTL_OSC_SLEEP_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_DEEPSLCNTL_BASE, BB_DEEPSLCNTL_OSC_SLEEP_EN_POS))

/* BB_DEEPSLCNTL settings */
#define EXTWKUPDSB_0_BITBAND            0x0
#define EXTWKUPDSB_1_BITBAND            0x1
#define EXTWKUPDSB_0                    (EXTWKUPDSB_0_BITBAND << BB_DEEPSLCNTL_EXTWKUPDSB_POS)
#define EXTWKUPDSB_1                    (EXTWKUPDSB_1_BITBAND << BB_DEEPSLCNTL_EXTWKUPDSB_POS)

#define DEEP_SLEEP_STAT_0_BITBAND       0x0
#define DEEP_SLEEP_STAT_1_BITBAND       0x1
#define DEEP_SLEEP_STAT_0               (DEEP_SLEEP_STAT_0_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_STAT_POS)
#define DEEP_SLEEP_STAT_1               (DEEP_SLEEP_STAT_1_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_STAT_POS)

#define SOFT_WAKEUP_REQ_0_BITBAND       0x0
#define SOFT_WAKEUP_REQ_1_BITBAND       0x1
#define SOFT_WAKEUP_REQ_0               (SOFT_WAKEUP_REQ_0_BITBAND << BB_DEEPSLCNTL_SOFT_WAKEUP_REQ_POS)
#define SOFT_WAKEUP_REQ_1               (SOFT_WAKEUP_REQ_1_BITBAND << BB_DEEPSLCNTL_SOFT_WAKEUP_REQ_POS)

#define DEEP_SLEEP_CORR_EN_0_BITBAND    0x0
#define DEEP_SLEEP_CORR_EN_1_BITBAND    0x1
#define DEEP_SLEEP_CORR_EN_0            (DEEP_SLEEP_CORR_EN_0_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_CORR_EN_POS)
#define DEEP_SLEEP_CORR_EN_1            (DEEP_SLEEP_CORR_EN_1_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_CORR_EN_POS)

#define DEEP_SLEEP_ON_0_BITBAND         0x0
#define DEEP_SLEEP_ON_1_BITBAND         0x1
#define DEEP_SLEEP_ON_0                 (DEEP_SLEEP_ON_0_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_ON_POS)
#define DEEP_SLEEP_ON_1                 (DEEP_SLEEP_ON_1_BITBAND << BB_DEEPSLCNTL_DEEP_SLEEP_ON_POS)

#define RADIO_SLEEP_EN_0_BITBAND        0x0
#define RADIO_SLEEP_EN_1_BITBAND        0x1
#define RADIO_SLEEP_EN_0                (RADIO_SLEEP_EN_0_BITBAND << BB_DEEPSLCNTL_RADIO_SLEEP_EN_POS)
#define RADIO_SLEEP_EN_1                (RADIO_SLEEP_EN_1_BITBAND << BB_DEEPSLCNTL_RADIO_SLEEP_EN_POS)

#define OSC_SLEEP_EN_0_BITBAND          0x0
#define OSC_SLEEP_EN_1_BITBAND          0x1
#define OSC_SLEEP_EN_0                  (OSC_SLEEP_EN_0_BITBAND << BB_DEEPSLCNTL_OSC_SLEEP_EN_POS)
#define OSC_SLEEP_EN_1                  (OSC_SLEEP_EN_1_BITBAND << BB_DEEPSLCNTL_OSC_SLEEP_EN_POS)

/* Deep sleep wakeup register */
#define BB_DEEPSLWKUP_BASE              0x40001534
#define BB_DEEPSLWKUP                   REG32_POINTER(BB_DEEPSLWKUP_BASE)

/* Deep sleep status register */
#define BB_DEEPSLSTAT_BASE              0x40001538
#define BB_DEEPSLSTAT                   READONLY_REG32_POINTER(BB_DEEPSLSTAT_BASE)

/* Stabilization times */
#define BB_ENBPRESET_BASE               0x4000153C
#define BB_ENBPRESET                    REG32_POINTER(BB_ENBPRESET_BASE)

/* BB_ENBPRESET bit positions */
#define BB_ENBPRESET_TWOSC_POS          10
#define BB_ENBPRESET_TWOSC_MASK         (0x7FF << BB_ENBPRESET_TWOSC_POS)

/* BB_ENBPRESET settings */
#define TWOSC_0                         (0x0 << BB_ENBPRESET_TWOSC_POS)

/* Fine timer correction register */
#define BB_FINECNTCORR_BASE             0x40001540
#define BB_FINECNTCORR                  REG32_POINTER(BB_FINECNTCORR_BASE)

/* BB_FINECNTCORR bit positions */
#define BB_FINECNTCORR_FINECNTCORR_POS  0
#define BB_FINECNTCORR_FINECNTCORR_MASK (0x3FF << BB_FINECNTCORR_FINECNTCORR_POS)

/* BB_FINECNTCORR settings */
#define FINECNTCORR_0                   (0x0 << BB_FINECNTCORR_FINECNTCORR_POS)

/* Base timer correction register */
#define BB_BASETIMECNTCORR_BASE         0x40001544
#define BB_BASETIMECNTCORR              REG32_POINTER(BB_BASETIMECNTCORR_BASE)

/* BB_BASETIMECNTCORR bit positions */
#define BB_BASETIMECNTCORR_BASETIMECNTCORR_POS 0
#define BB_BASETIMECNTCORR_BASETIMECNTCORR_MASK (0x7FFFFFF << BB_BASETIMECNTCORR_BASETIMECNTCORR_POS)

/* BB_BASETIMECNTCORR settings */
#define BASETIMECNTCORR_0               (0x0 << BB_BASETIMECNTCORR_BASETIMECNTCORR_POS)

/* Diagnostic ports control register */
#define BB_DIAGCNTL_BASE                0x40001550
#define BB_DIAGCNTL                     REG32_POINTER(BB_DIAGCNTL_BASE)

/* BB_DIAGCNTL bit positions */
#define BB_DIAGCNTL_DIAG3_EN_POS        31
#define BB_DIAGCNTL_DIAG3_POS           24
#define BB_DIAGCNTL_DIAG3_MASK          (0x3F << BB_DIAGCNTL_DIAG3_POS)
#define BB_DIAGCNTL_DIAG2_EN_POS        23
#define BB_DIAGCNTL_DIAG2_POS           16
#define BB_DIAGCNTL_DIAG2_MASK          (0x3F << BB_DIAGCNTL_DIAG2_POS)
#define BB_DIAGCNTL_DIAG1_EN_POS        15
#define BB_DIAGCNTL_DIAG1_POS           8
#define BB_DIAGCNTL_DIAG1_MASK          (0x3F << BB_DIAGCNTL_DIAG1_POS)
#define BB_DIAGCNTL_DIAG0_EN_POS        7
#define BB_DIAGCNTL_DIAG0_POS           0
#define BB_DIAGCNTL_DIAG0_MASK          (0x3F << BB_DIAGCNTL_DIAG0_POS)

/* BB_DIAGCNTL bit-band aliases */
#define BB_DIAGCNTL_DIAG3_EN_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_DIAGCNTL_BASE, BB_DIAGCNTL_DIAG3_EN_POS))
#define BB_DIAGCNTL_DIAG2_EN_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_DIAGCNTL_BASE, BB_DIAGCNTL_DIAG2_EN_POS))
#define BB_DIAGCNTL_DIAG1_EN_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_DIAGCNTL_BASE, BB_DIAGCNTL_DIAG1_EN_POS))
#define BB_DIAGCNTL_DIAG0_EN_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_DIAGCNTL_BASE, BB_DIAGCNTL_DIAG0_EN_POS))

/* BB_DIAGCNTL settings */
#define DIAG3_EN_0_BITBAND              0x0
#define DIAG3_EN_1_BITBAND              0x1
#define DIAG3_EN_0                      (DIAG3_EN_0_BITBAND << BB_DIAGCNTL_DIAG3_EN_POS)
#define DIAG3_EN_1                      (DIAG3_EN_1_BITBAND << BB_DIAGCNTL_DIAG3_EN_POS)

#define DIAG3_0                         (0x0 << BB_DIAGCNTL_DIAG3_POS)

#define DIAG2_EN_0_BITBAND              0x0
#define DIAG2_EN_1_BITBAND              0x1
#define DIAG2_EN_0                      (DIAG2_EN_0_BITBAND << BB_DIAGCNTL_DIAG2_EN_POS)
#define DIAG2_EN_1                      (DIAG2_EN_1_BITBAND << BB_DIAGCNTL_DIAG2_EN_POS)

#define DIAG2_0                         (0x0 << BB_DIAGCNTL_DIAG2_POS)

#define DIAG1_EN_0_BITBAND              0x0
#define DIAG1_EN_1_BITBAND              0x1
#define DIAG1_EN_0                      (DIAG1_EN_0_BITBAND << BB_DIAGCNTL_DIAG1_EN_POS)
#define DIAG1_EN_1                      (DIAG1_EN_1_BITBAND << BB_DIAGCNTL_DIAG1_EN_POS)

#define DIAG1_0                         (0x0 << BB_DIAGCNTL_DIAG1_POS)

#define DIAG0_EN_0_BITBAND              0x0
#define DIAG0_EN_1_BITBAND              0x1
#define DIAG0_EN_0                      (DIAG0_EN_0_BITBAND << BB_DIAGCNTL_DIAG0_EN_POS)
#define DIAG0_EN_1                      (DIAG0_EN_1_BITBAND << BB_DIAGCNTL_DIAG0_EN_POS)

#define DIAG0_0                         (0x0 << BB_DIAGCNTL_DIAG0_POS)

/* Diagnostic ports status register */
#define BB_DIAGSTAT_BASE                0x40001554
#define BB_DIAGSTAT                     READONLY_REG32_POINTER(BB_DIAGSTAT_BASE)

/* BB_DIAGSTAT bit positions */
#define BB_DIAGSTAT_DIAG3STAT_POS       24
#define BB_DIAGSTAT_DIAG3STAT_MASK      (0xFF << BB_DIAGSTAT_DIAG3STAT_POS)
#define BB_DIAGSTAT_DIAG2STAT_POS       16
#define BB_DIAGSTAT_DIAG2STAT_MASK      (0xFF << BB_DIAGSTAT_DIAG2STAT_POS)
#define BB_DIAGSTAT_DIAG1STAT_POS       8
#define BB_DIAGSTAT_DIAG1STAT_MASK      (0xFF << BB_DIAGSTAT_DIAG1STAT_POS)
#define BB_DIAGSTAT_DIAG0STAT_POS       0
#define BB_DIAGSTAT_DIAG0STAT_MASK      (0xFF << BB_DIAGSTAT_DIAG0STAT_POS)

/* BB_DIAGSTAT settings */
#define DIAG3STAT_0                     (0x0 << BB_DIAGSTAT_DIAG3STAT_POS)

#define DIAG2STAT_0                     (0x0 << BB_DIAGSTAT_DIAG2STAT_POS)

#define DIAG1STAT_0                     (0x0 << BB_DIAGSTAT_DIAG1STAT_POS)

#define DIAG0STAT_0                     (0x0 << BB_DIAGSTAT_DIAG0STAT_POS)

/* Diagnostic ports upper limit */
#define BB_DEBUGADDMAX_BASE             0x40001558
#define BB_DEBUGADDMAX                  REG32_POINTER(BB_DEBUGADDMAX_BASE)

/* BB_DEBUGADDMAX bit positions */
#define BB_DEBUGADDMAX_REG_ADDMAX_POS   16
#define BB_DEBUGADDMAX_REG_ADDMAX_MASK  (0xFFFF << BB_DEBUGADDMAX_REG_ADDMAX_POS)
#define BB_DEBUGADDMAX_EM_ADDMAX_POS    0
#define BB_DEBUGADDMAX_EM_ADDMAX_MASK   (0xFFFF << BB_DEBUGADDMAX_EM_ADDMAX_POS)

/* BB_DEBUGADDMAX settings */
#define REG_ADDMAX_0                    (0x0 << BB_DEBUGADDMAX_REG_ADDMAX_POS)

#define EM_ADDMAX_0                     (0x0 << BB_DEBUGADDMAX_EM_ADDMAX_POS)

/* Diagnostic ports lower limit */
#define BB_DEBUGADDMIN_BASE             0x4000155C
#define BB_DEBUGADDMIN                  REG32_POINTER(BB_DEBUGADDMIN_BASE)

/* BB_DEBUGADDMIN bit positions */
#define BB_DEBUGADDMIN_REG_ADDMIN_POS   16
#define BB_DEBUGADDMIN_REG_ADDMIN_MASK  (0xFFFF << BB_DEBUGADDMIN_REG_ADDMIN_POS)
#define BB_DEBUGADDMIN_EM_ADDMIN_POS    0
#define BB_DEBUGADDMIN_EM_ADDMIN_MASK   (0xFFFF << BB_DEBUGADDMIN_EM_ADDMIN_POS)

/* BB_DEBUGADDMIN settings */
#define REG_ADDMIN_0                    (0x0 << BB_DEBUGADDMIN_REG_ADDMIN_POS)

#define EM_ADDMIN_0                     (0x0 << BB_DEBUGADDMIN_EM_ADDMIN_POS)

/* Diagnostic ports errors register */
#define BB_ERRORTYPESTAT_BASE           0x40001560
#define BB_ERRORTYPESTAT                READONLY_REG32_POINTER(BB_ERRORTYPESTAT_BASE)

/* BB_ERRORTYPESTAT bit positions */
#define BB_ERRORTYPESTAT_RAL_UNDERRUN_POS 19
#define BB_ERRORTYPESTAT_RAL_ERROR_POS  18
#define BB_ERRORTYPESTAT_CONCEVTIRQ_ERROR_POS 17
#define BB_ERRORTYPESTAT_RXDATA_PTR_ERROR_POS 16
#define BB_ERRORTYPESTAT_TXDATA_PTR_ERROR_POS 15
#define BB_ERRORTYPESTAT_RXDESC_EMPTY_ERROR_POS 14
#define BB_ERRORTYPESTAT_TXDESC_EMPTY_ERROR_POS 13
#define BB_ERRORTYPESTAT_CSFORMAT_ERROR_POS 12
#define BB_ERRORTYPESTAT_LLCHMAP_ERROR_POS 11
#define BB_ERRORTYPESTAT_ADV_UNDERRUN_POS 10
#define BB_ERRORTYPESTAT_IFS_UNDERRUN_POS 9
#define BB_ERRORTYPESTAT_WHITELIST_ERROR_POS 8
#define BB_ERRORTYPESTAT_EVT_CNTL_APFM_ERROR_POS 7
#define BB_ERRORTYPESTAT_EVT_SCHDL_APFM_ERROR_POS 6
#define BB_ERRORTYPESTAT_EVT_SCHDL_ENTRY_ERROR_POS 5
#define BB_ERRORTYPESTAT_EVT_SCHDL_EMACC_ERROR_POS 4
#define BB_ERRORTYPESTAT_RADIO_EMACC_ERROR_POS 3
#define BB_ERRORTYPESTAT_PKTCNTL_EMACC_ERROR_POS 2
#define BB_ERRORTYPESTAT_RXCRYPT_ERROR_POS 1
#define BB_ERRORTYPESTAT_TXCRYPT_ERROR_POS 0

/* BB_ERRORTYPESTAT bit-band aliases */
#define BB_ERRORTYPESTAT_RAL_UNDERRUN_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_RAL_UNDERRUN_POS))
#define BB_ERRORTYPESTAT_RAL_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_RAL_ERROR_POS))
#define BB_ERRORTYPESTAT_CONCEVTIRQ_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_CONCEVTIRQ_ERROR_POS))
#define BB_ERRORTYPESTAT_RXDATA_PTR_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_RXDATA_PTR_ERROR_POS))
#define BB_ERRORTYPESTAT_TXDATA_PTR_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_TXDATA_PTR_ERROR_POS))
#define BB_ERRORTYPESTAT_RXDESC_EMPTY_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_RXDESC_EMPTY_ERROR_POS))
#define BB_ERRORTYPESTAT_TXDESC_EMPTY_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_TXDESC_EMPTY_ERROR_POS))
#define BB_ERRORTYPESTAT_CSFORMAT_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_CSFORMAT_ERROR_POS))
#define BB_ERRORTYPESTAT_LLCHMAP_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_LLCHMAP_ERROR_POS))
#define BB_ERRORTYPESTAT_ADV_UNDERRUN_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_ADV_UNDERRUN_POS))
#define BB_ERRORTYPESTAT_IFS_UNDERRUN_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_IFS_UNDERRUN_POS))
#define BB_ERRORTYPESTAT_WHITELIST_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_WHITELIST_ERROR_POS))
#define BB_ERRORTYPESTAT_EVT_CNTL_APFM_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_EVT_CNTL_APFM_ERROR_POS))
#define BB_ERRORTYPESTAT_EVT_SCHDL_APFM_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_EVT_SCHDL_APFM_ERROR_POS))
#define BB_ERRORTYPESTAT_EVT_SCHDL_ENTRY_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_EVT_SCHDL_ENTRY_ERROR_POS))
#define BB_ERRORTYPESTAT_EVT_SCHDL_EMACC_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_EVT_SCHDL_EMACC_ERROR_POS))
#define BB_ERRORTYPESTAT_RADIO_EMACC_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_RADIO_EMACC_ERROR_POS))
#define BB_ERRORTYPESTAT_PKTCNTL_EMACC_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_PKTCNTL_EMACC_ERROR_POS))
#define BB_ERRORTYPESTAT_RXCRYPT_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_RXCRYPT_ERROR_POS))
#define BB_ERRORTYPESTAT_TXCRYPT_ERROR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_ERRORTYPESTAT_BASE, BB_ERRORTYPESTAT_TXCRYPT_ERROR_POS))

/* BB_ERRORTYPESTAT settings */
#define RAL_UNDERRUN_0_BITBAND          0x0
#define RAL_UNDERRUN_1_BITBAND          0x1
#define RAL_UNDERRUN_0                  (RAL_UNDERRUN_0_BITBAND << BB_ERRORTYPESTAT_RAL_UNDERRUN_POS)
#define RAL_UNDERRUN_1                  (RAL_UNDERRUN_1_BITBAND << BB_ERRORTYPESTAT_RAL_UNDERRUN_POS)

#define RAL_ERROR_0_BITBAND             0x0
#define RAL_ERROR_1_BITBAND             0x1
#define RAL_ERROR_0                     (RAL_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RAL_ERROR_POS)
#define RAL_ERROR_1                     (RAL_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RAL_ERROR_POS)

#define CONCEVTIRQ_ERROR_0_BITBAND      0x0
#define CONCEVTIRQ_ERROR_1_BITBAND      0x1
#define CONCEVTIRQ_ERROR_0              (CONCEVTIRQ_ERROR_0_BITBAND << BB_ERRORTYPESTAT_CONCEVTIRQ_ERROR_POS)
#define CONCEVTIRQ_ERROR_1              (CONCEVTIRQ_ERROR_1_BITBAND << BB_ERRORTYPESTAT_CONCEVTIRQ_ERROR_POS)

#define RXDATA_PTR_ERROR_0_BITBAND      0x0
#define RXDATA_PTR_ERROR_1_BITBAND      0x1
#define RXDATA_PTR_ERROR_0              (RXDATA_PTR_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RXDATA_PTR_ERROR_POS)
#define RXDATA_PTR_ERROR_1              (RXDATA_PTR_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RXDATA_PTR_ERROR_POS)

#define TXDATA_PTR_ERROR_0_BITBAND      0x0
#define TXDATA_PTR_ERROR_1_BITBAND      0x1
#define TXDATA_PTR_ERROR_0              (TXDATA_PTR_ERROR_0_BITBAND << BB_ERRORTYPESTAT_TXDATA_PTR_ERROR_POS)
#define TXDATA_PTR_ERROR_1              (TXDATA_PTR_ERROR_1_BITBAND << BB_ERRORTYPESTAT_TXDATA_PTR_ERROR_POS)

#define RXDESC_EMPTY_ERROR_0_BITBAND    0x0
#define RXDESC_EMPTY_ERROR_1_BITBAND    0x1
#define RXDESC_EMPTY_ERROR_0            (RXDESC_EMPTY_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RXDESC_EMPTY_ERROR_POS)
#define RXDESC_EMPTY_ERROR_1            (RXDESC_EMPTY_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RXDESC_EMPTY_ERROR_POS)

#define TXDESC_EMPTY_ERROR_0_BITBAND    0x0
#define TXDESC_EMPTY_ERROR_1_BITBAND    0x1
#define TXDESC_EMPTY_ERROR_0            (TXDESC_EMPTY_ERROR_0_BITBAND << BB_ERRORTYPESTAT_TXDESC_EMPTY_ERROR_POS)
#define TXDESC_EMPTY_ERROR_1            (TXDESC_EMPTY_ERROR_1_BITBAND << BB_ERRORTYPESTAT_TXDESC_EMPTY_ERROR_POS)

#define CSFORMAT_ERROR_0_BITBAND        0x0
#define CSFORMAT_ERROR_1_BITBAND        0x1
#define CSFORMAT_ERROR_0                (CSFORMAT_ERROR_0_BITBAND << BB_ERRORTYPESTAT_CSFORMAT_ERROR_POS)
#define CSFORMAT_ERROR_1                (CSFORMAT_ERROR_1_BITBAND << BB_ERRORTYPESTAT_CSFORMAT_ERROR_POS)

#define LLCHMAP_ERROR_0_BITBAND         0x0
#define LLCHMAP_ERROR_1_BITBAND         0x1
#define LLCHMAP_ERROR_0                 (LLCHMAP_ERROR_0_BITBAND << BB_ERRORTYPESTAT_LLCHMAP_ERROR_POS)
#define LLCHMAP_ERROR_1                 (LLCHMAP_ERROR_1_BITBAND << BB_ERRORTYPESTAT_LLCHMAP_ERROR_POS)

#define ADV_UNDERRUN_0_BITBAND          0x0
#define ADV_UNDERRUN_1_BITBAND          0x1
#define ADV_UNDERRUN_0                  (ADV_UNDERRUN_0_BITBAND << BB_ERRORTYPESTAT_ADV_UNDERRUN_POS)
#define ADV_UNDERRUN_1                  (ADV_UNDERRUN_1_BITBAND << BB_ERRORTYPESTAT_ADV_UNDERRUN_POS)

#define IFS_UNDERRUN_0_BITBAND          0x0
#define IFS_UNDERRUN_1_BITBAND          0x1
#define IFS_UNDERRUN_0                  (IFS_UNDERRUN_0_BITBAND << BB_ERRORTYPESTAT_IFS_UNDERRUN_POS)
#define IFS_UNDERRUN_1                  (IFS_UNDERRUN_1_BITBAND << BB_ERRORTYPESTAT_IFS_UNDERRUN_POS)

#define WHITELIST_ERROR_0_BITBAND       0x0
#define WHITELIST_ERROR_1_BITBAND       0x1
#define WHITELIST_ERROR_0               (WHITELIST_ERROR_0_BITBAND << BB_ERRORTYPESTAT_WHITELIST_ERROR_POS)
#define WHITELIST_ERROR_1               (WHITELIST_ERROR_1_BITBAND << BB_ERRORTYPESTAT_WHITELIST_ERROR_POS)

#define EVT_CNTL_APFM_ERROR_0_BITBAND   0x0
#define EVT_CNTL_APFM_ERROR_1_BITBAND   0x1
#define EVT_CNTL_APFM_ERROR_0           (EVT_CNTL_APFM_ERROR_0_BITBAND << BB_ERRORTYPESTAT_EVT_CNTL_APFM_ERROR_POS)
#define EVT_CNTL_APFM_ERROR_1           (EVT_CNTL_APFM_ERROR_1_BITBAND << BB_ERRORTYPESTAT_EVT_CNTL_APFM_ERROR_POS)

#define EVT_SCHDL_APFM_ERROR_0_BITBAND  0x0
#define EVT_SCHDL_APFM_ERROR_1_BITBAND  0x1
#define EVT_SCHDL_APFM_ERROR_0          (EVT_SCHDL_APFM_ERROR_0_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_APFM_ERROR_POS)
#define EVT_SCHDL_APFM_ERROR_1          (EVT_SCHDL_APFM_ERROR_1_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_APFM_ERROR_POS)

#define EVT_SCHDL_ENTRY_ERROR_0_BITBAND 0x0
#define EVT_SCHDL_ENTRY_ERROR_1_BITBAND 0x1
#define EVT_SCHDL_ENTRY_ERROR_0         (EVT_SCHDL_ENTRY_ERROR_0_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_ENTRY_ERROR_POS)
#define EVT_SCHDL_ENTRY_ERROR_1         (EVT_SCHDL_ENTRY_ERROR_1_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_ENTRY_ERROR_POS)

#define EVT_SCHDL_EMACC_ERROR_0_BITBAND 0x0
#define EVT_SCHDL_EMACC_ERROR_1_BITBAND 0x1
#define EVT_SCHDL_EMACC_ERROR_0         (EVT_SCHDL_EMACC_ERROR_0_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_EMACC_ERROR_POS)
#define EVT_SCHDL_EMACC_ERROR_1         (EVT_SCHDL_EMACC_ERROR_1_BITBAND << BB_ERRORTYPESTAT_EVT_SCHDL_EMACC_ERROR_POS)

#define RADIO_EMACC_ERROR_0_BITBAND     0x0
#define RADIO_EMACC_ERROR_1_BITBAND     0x1
#define RADIO_EMACC_ERROR_0             (RADIO_EMACC_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RADIO_EMACC_ERROR_POS)
#define RADIO_EMACC_ERROR_1             (RADIO_EMACC_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RADIO_EMACC_ERROR_POS)

#define PKTCNTL_EMACC_ERROR_0_BITBAND   0x0
#define PKTCNTL_EMACC_ERROR_1_BITBAND   0x1
#define PKTCNTL_EMACC_ERROR_0           (PKTCNTL_EMACC_ERROR_0_BITBAND << BB_ERRORTYPESTAT_PKTCNTL_EMACC_ERROR_POS)
#define PKTCNTL_EMACC_ERROR_1           (PKTCNTL_EMACC_ERROR_1_BITBAND << BB_ERRORTYPESTAT_PKTCNTL_EMACC_ERROR_POS)

#define RXCRYPT_ERROR_0_BITBAND         0x0
#define RXCRYPT_ERROR_1_BITBAND         0x1
#define RXCRYPT_ERROR_0                 (RXCRYPT_ERROR_0_BITBAND << BB_ERRORTYPESTAT_RXCRYPT_ERROR_POS)
#define RXCRYPT_ERROR_1                 (RXCRYPT_ERROR_1_BITBAND << BB_ERRORTYPESTAT_RXCRYPT_ERROR_POS)

#define TXCRYPT_ERROR_0_BITBAND         0x0
#define TXCRYPT_ERROR_1_BITBAND         0x1
#define TXCRYPT_ERROR_0                 (TXCRYPT_ERROR_0_BITBAND << BB_ERRORTYPESTAT_TXCRYPT_ERROR_POS)
#define TXCRYPT_ERROR_1                 (TXCRYPT_ERROR_1_BITBAND << BB_ERRORTYPESTAT_TXCRYPT_ERROR_POS)

/* Software profiling register */
#define BB_SWPROFILING_BASE             0x40001564
#define BB_SWPROFILING                  REG32_POINTER(BB_SWPROFILING_BASE)

/* Principal control register for the radio interface */
#define BB_RADIOCNTL0_BASE              0x40001570
#define BB_RADIOCNTL0                   REG32_POINTER(BB_RADIOCNTL0_BASE)

/* BB_RADIOCNTL0 bit positions */
#define BB_RADIOCNTL0_SPIPTR_POS        16
#define BB_RADIOCNTL0_SPIPTR_MASK       (0xFFFF << BB_RADIOCNTL0_SPIPTR_POS)
#define BB_RADIOCNTL0_SPIFREQ_POS       4
#define BB_RADIOCNTL0_SPIFREQ_MASK      (0x3 << BB_RADIOCNTL0_SPIFREQ_POS)
#define BB_RADIOCNTL0_SPICOMP_POS       1
#define BB_RADIOCNTL0_SPIGO_POS         0

/* BB_RADIOCNTL0 bit-band aliases */
#define BB_RADIOCNTL0_SPICOMP_BITBAND   READONLY_REG32_POINTER(SYS_CALC_BITBAND(BB_RADIOCNTL0_BASE, BB_RADIOCNTL0_SPICOMP_POS))
#define BB_RADIOCNTL0_SPIGO_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(BB_RADIOCNTL0_BASE, BB_RADIOCNTL0_SPIGO_POS))

/* BB_RADIOCNTL0 settings */
#define SPIPTR_0                        (0x0 << BB_RADIOCNTL0_SPIPTR_POS)

#define SPIFREQ_0                       (0x0 << BB_RADIOCNTL0_SPIFREQ_POS)
#define SPIFREQ_1                       (0x1 << BB_RADIOCNTL0_SPIFREQ_POS)
#define SPIFREQ_2                       (0x2 << BB_RADIOCNTL0_SPIFREQ_POS)
#define SPIFREQ_3                       (0x3 << BB_RADIOCNTL0_SPIFREQ_POS)

#define SPICOMP_0_BITBAND               0x0
#define SPICOMP_1_BITBAND               0x1
#define SPICOMP_0                       (SPICOMP_0_BITBAND << BB_RADIOCNTL0_SPICOMP_POS)
#define SPICOMP_1                       (SPICOMP_1_BITBAND << BB_RADIOCNTL0_SPICOMP_POS)

#define SPIGO_0_BITBAND                 0x0
#define SPIGO_1_BITBAND                 0x1
#define SPIGO_0                         (SPIGO_0_BITBAND << BB_RADIOCNTL0_SPIGO_POS)
#define SPIGO_1                         (SPIGO_1_BITBAND << BB_RADIOCNTL0_SPIGO_POS)

/* Second control register for the radio interface */
#define BB_RADIOCNTL1_BASE              0x40001574
#define BB_RADIOCNTL1                   REG32_POINTER(BB_RADIOCNTL1_BASE)

/* BB_RADIOCNTL1 bit positions */
#define BB_RADIOCNTL1_FORCEAGC_EN_POS   31
#define BB_RADIOCNTL1_FORCEBLEIQ_POS    30
#define BB_RADIOCNTL1_FORCEAGC_LENGTH_POS 16
#define BB_RADIOCNTL1_FORCEAGC_LENGTH_MASK (0xFFF << BB_RADIOCNTL1_FORCEAGC_LENGTH_POS)
#define BB_RADIOCNTL1_SYNC_PULSE_MODE_POS 15
#define BB_RADIOCNTL1_DPCORR_EN_POS     13
#define BB_RADIOCNTL1_JEF_SELECT_POS    12
#define BB_RADIOCNTL1_XRFSEL_POS        4
#define BB_RADIOCNTL1_XRFSEL_MASK       (0x1F << BB_RADIOCNTL1_XRFSEL_POS)
#define BB_RADIOCNTL1_SUBVERSION_POS    0
#define BB_RADIOCNTL1_SUBVERSION_MASK   (0xF << BB_RADIOCNTL1_SUBVERSION_POS)

/* BB_RADIOCNTL1 bit-band aliases */
#define BB_RADIOCNTL1_FORCEAGC_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RADIOCNTL1_BASE, BB_RADIOCNTL1_FORCEAGC_EN_POS))
#define BB_RADIOCNTL1_FORCEBLEIQ_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RADIOCNTL1_BASE, BB_RADIOCNTL1_FORCEBLEIQ_POS))
#define BB_RADIOCNTL1_SYNC_PULSE_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RADIOCNTL1_BASE, BB_RADIOCNTL1_SYNC_PULSE_MODE_POS))
#define BB_RADIOCNTL1_DPCORR_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RADIOCNTL1_BASE, BB_RADIOCNTL1_DPCORR_EN_POS))
#define BB_RADIOCNTL1_JEF_SELECT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RADIOCNTL1_BASE, BB_RADIOCNTL1_JEF_SELECT_POS))

/* BB_RADIOCNTL1 settings */
#define FORCEAGC_EN_0_BITBAND           0x0
#define FORCEAGC_EN_1_BITBAND           0x1
#define FORCEAGC_EN_0                   (FORCEAGC_EN_0_BITBAND << BB_RADIOCNTL1_FORCEAGC_EN_POS)
#define FORCEAGC_EN_1                   (FORCEAGC_EN_1_BITBAND << BB_RADIOCNTL1_FORCEAGC_EN_POS)

#define FORCEBLEIQ_0_BITBAND            0x0
#define FORCEBLEIQ_1_BITBAND            0x1
#define FORCEBLEIQ_0                    (FORCEBLEIQ_0_BITBAND << BB_RADIOCNTL1_FORCEBLEIQ_POS)
#define FORCEBLEIQ_1                    (FORCEBLEIQ_1_BITBAND << BB_RADIOCNTL1_FORCEBLEIQ_POS)

#define FORCEAGC_LENGTH_0               (0x0 << BB_RADIOCNTL1_FORCEAGC_LENGTH_POS)

#define SYNC_PULSE_MODE_0_BITBAND       0x0
#define SYNC_PULSE_MODE_1_BITBAND       0x1
#define SYNC_PULSE_MODE_0               (SYNC_PULSE_MODE_0_BITBAND << BB_RADIOCNTL1_SYNC_PULSE_MODE_POS)
#define SYNC_PULSE_MODE_1               (SYNC_PULSE_MODE_1_BITBAND << BB_RADIOCNTL1_SYNC_PULSE_MODE_POS)

#define DPCORR_EN_0_BITBAND             0x0
#define DPCORR_EN_1_BITBAND             0x1
#define DPCORR_EN_0                     (DPCORR_EN_0_BITBAND << BB_RADIOCNTL1_DPCORR_EN_POS)
#define DPCORR_EN_1                     (DPCORR_EN_1_BITBAND << BB_RADIOCNTL1_DPCORR_EN_POS)

#define JEF_SELECT_0_BITBAND            0x0
#define JEF_SELECT_1_BITBAND            0x1
#define JEF_SELECT_0                    (JEF_SELECT_0_BITBAND << BB_RADIOCNTL1_JEF_SELECT_POS)
#define JEF_SELECT_1                    (JEF_SELECT_1_BITBAND << BB_RADIOCNTL1_JEF_SELECT_POS)

#define XRFSEL_0                        (0x0 << BB_RADIOCNTL1_XRFSEL_POS)
#define XRFSEL_1                        (0x1 << BB_RADIOCNTL1_XRFSEL_POS)
#define XRFSEL_2                        (0x2 << BB_RADIOCNTL1_XRFSEL_POS)
#define XRFSEL_3                        (0x3 << BB_RADIOCNTL1_XRFSEL_POS)

#define SUBVERSION_0                    (0x0 << BB_RADIOCNTL1_SUBVERSION_POS)
#define SUBVERSION_1                    (0x1 << BB_RADIOCNTL1_SUBVERSION_POS)
#define SUBVERSION_2                    (0x2 << BB_RADIOCNTL1_SUBVERSION_POS)
#define SUBVERSION_3                    (0x3 << BB_RADIOCNTL1_SUBVERSION_POS)

/* Third control register for the radio interface */
#define BB_RADIOCNTL2_BASE              0x40001578
#define BB_RADIOCNTL2                   REG32_POINTER(BB_RADIOCNTL2_BASE)

/* BB_RADIOCNTL2 bit positions */
#define BB_RADIOCNTL2_FREQTABLE_PTR_POS 0
#define BB_RADIOCNTL2_FREQTABLE_PTR_MASK (0xFFFF << BB_RADIOCNTL2_FREQTABLE_PTR_POS)

/* BB_RADIOCNTL2 settings */
#define FREQTABLE_PTR_64                (0x40 << BB_RADIOCNTL2_FREQTABLE_PTR_POS)

/* Principal control register for the radio interface power up/down delays */
#define BB_RADIOPWRUPDN0_BASE           0x40001580
#define BB_RADIOPWRUPDN0                REG32_POINTER(BB_RADIOPWRUPDN0_BASE)

/* BB_RADIOPWRUPDN0 bit positions */
#define BB_RADIOPWRUPDN0_RXPWRUP0_POS   16
#define BB_RADIOPWRUPDN0_RXPWRUP0_MASK  (0xFF << BB_RADIOPWRUPDN0_RXPWRUP0_POS)
#define BB_RADIOPWRUPDN0_TXPWRDN0_POS   8
#define BB_RADIOPWRUPDN0_TXPWRDN0_MASK  (0x1F << BB_RADIOPWRUPDN0_TXPWRDN0_POS)
#define BB_RADIOPWRUPDN0_TXPWRUP0_POS   0
#define BB_RADIOPWRUPDN0_TXPWRUP0_MASK  (0xFF << BB_RADIOPWRUPDN0_TXPWRUP0_POS)

/* BB_RADIOPWRUPDN0 settings */
#define RXPWRUP0_0                      (0x0 << BB_RADIOPWRUPDN0_RXPWRUP0_POS)

#define TXPWRDN0_0                      (0x0 << BB_RADIOPWRUPDN0_TXPWRDN0_POS)

#define TXPWRUP0_0                      (0x0 << BB_RADIOPWRUPDN0_TXPWRUP0_POS)

/* Second control register for the radio interface power up/down delays */
#define BB_RADIOPWRUPDN1_BASE           0x40001584
#define BB_RADIOPWRUPDN1                REG32_POINTER(BB_RADIOPWRUPDN1_BASE)

/* BB_RADIOPWRUPDN1 bit positions */
#define BB_RADIOPWRUPDN1_RXPWRUP1_POS   16
#define BB_RADIOPWRUPDN1_RXPWRUP1_MASK  (0xFF << BB_RADIOPWRUPDN1_RXPWRUP1_POS)
#define BB_RADIOPWRUPDN1_TXPWRDN1_POS   8
#define BB_RADIOPWRUPDN1_TXPWRDN1_MASK  (0x1F << BB_RADIOPWRUPDN1_TXPWRDN1_POS)
#define BB_RADIOPWRUPDN1_TXPWRUP1_POS   0
#define BB_RADIOPWRUPDN1_TXPWRUP1_MASK  (0xFF << BB_RADIOPWRUPDN1_TXPWRUP1_POS)

/* BB_RADIOPWRUPDN1 settings */
#define RXPWRUP1_0                      (0x0 << BB_RADIOPWRUPDN1_RXPWRUP1_POS)

#define TXPWRDN1_0                      (0x0 << BB_RADIOPWRUPDN1_TXPWRDN1_POS)

#define TXPWRUP1_0                      (0x0 << BB_RADIOPWRUPDN1_TXPWRUP1_POS)

/* Principal control register for the radio interface timing compensation delays */
#define BB_RADIOTXRXTIM0_BASE           0x40001590
#define BB_RADIOTXRXTIM0                REG32_POINTER(BB_RADIOTXRXTIM0_BASE)

/* BB_RADIOTXRXTIM0 bit positions */
#define BB_RADIOTXRXTIM0_TXPATHDLY0_POS 24
#define BB_RADIOTXRXTIM0_TXPATHDLY0_MASK (0xFF << BB_RADIOTXRXTIM0_TXPATHDLY0_POS)
#define BB_RADIOTXRXTIM0_RXPATHDLY0_POS 16
#define BB_RADIOTXRXTIM0_RXPATHDLY0_MASK (0x1F << BB_RADIOTXRXTIM0_RXPATHDLY0_POS)
#define BB_RADIOTXRXTIM0_RFRXTMDA0_POS  8
#define BB_RADIOTXRXTIM0_RFRXTMDA0_MASK (0x7F << BB_RADIOTXRXTIM0_RFRXTMDA0_POS)
#define BB_RADIOTXRXTIM0_SYNC_POSITION0_POS 0
#define BB_RADIOTXRXTIM0_SYNC_POSITION0_MASK (0x7F << BB_RADIOTXRXTIM0_SYNC_POSITION0_POS)

/* BB_RADIOTXRXTIM0 settings */
#define TXPATHDLY0_0                    (0x0 << BB_RADIOTXRXTIM0_TXPATHDLY0_POS)

#define RXPATHDLY0_0                    (0x0 << BB_RADIOTXRXTIM0_RXPATHDLY0_POS)

#define RFRXTMDA0_0                     (0x0 << BB_RADIOTXRXTIM0_RFRXTMDA0_POS)

#define SYNC_POSITION0_0                (0x0 << BB_RADIOTXRXTIM0_SYNC_POSITION0_POS)

/* Second control register for the radio interface timing compensation delays */
#define BB_RADIOTXRXTIM1_BASE           0x40001594
#define BB_RADIOTXRXTIM1                REG32_POINTER(BB_RADIOTXRXTIM1_BASE)

/* BB_RADIOTXRXTIM1 bit positions */
#define BB_RADIOTXRXTIM1_TXPATHDLY1_POS 24
#define BB_RADIOTXRXTIM1_TXPATHDLY1_MASK (0xFF << BB_RADIOTXRXTIM1_TXPATHDLY1_POS)
#define BB_RADIOTXRXTIM1_RXPATHDLY1_POS 16
#define BB_RADIOTXRXTIM1_RXPATHDLY1_MASK (0x1F << BB_RADIOTXRXTIM1_RXPATHDLY1_POS)
#define BB_RADIOTXRXTIM1_RFRXTMDA1_POS  8
#define BB_RADIOTXRXTIM1_RFRXTMDA1_MASK (0x7F << BB_RADIOTXRXTIM1_RFRXTMDA1_POS)
#define BB_RADIOTXRXTIM1_SYNC_POSITION1_POS 0
#define BB_RADIOTXRXTIM1_SYNC_POSITION1_MASK (0x7F << BB_RADIOTXRXTIM1_SYNC_POSITION1_POS)

/* BB_RADIOTXRXTIM1 settings */
#define TXPATHDLY1_0                    (0x0 << BB_RADIOTXRXTIM1_TXPATHDLY1_POS)

#define RXPATHDLY1_0                    (0x0 << BB_RADIOTXRXTIM1_RXPATHDLY1_POS)

#define RFRXTMDA1_0                     (0x0 << BB_RADIOTXRXTIM1_RFRXTMDA1_POS)

#define SYNC_POSITION1_0                (0x0 << BB_RADIOTXRXTIM1_SYNC_POSITION1_POS)

/* First control register for the radio interface SPI pointers */
#define BB_SPIPTRCNTL0_BASE             0x400015A0
#define BB_SPIPTRCNTL0                  REG32_POINTER(BB_SPIPTRCNTL0_BASE)

/* BB_SPIPTRCNTL0 bit positions */
#define BB_SPIPTRCNTL0_TXOFFPTR_POS     16
#define BB_SPIPTRCNTL0_TXOFFPTR_MASK    (0xFFFF << BB_SPIPTRCNTL0_TXOFFPTR_POS)
#define BB_SPIPTRCNTL0_TXONPTR_POS      0
#define BB_SPIPTRCNTL0_TXONPTR_MASK     (0xFFFF << BB_SPIPTRCNTL0_TXONPTR_POS)

/* BB_SPIPTRCNTL0 settings */
#define TXOFFPTR_0                      (0x0 << BB_SPIPTRCNTL0_TXOFFPTR_POS)

#define TXONPTR_0                       (0x0 << BB_SPIPTRCNTL0_TXONPTR_POS)

/* Second control register for the radio interface SPI pointers */
#define BB_SPIPTRCNTL1_BASE             0x400015A4
#define BB_SPIPTRCNTL1                  REG32_POINTER(BB_SPIPTRCNTL1_BASE)

/* BB_SPIPTRCNTL1 bit positions */
#define BB_SPIPTRCNTL1_RXOFFPTR_POS     16
#define BB_SPIPTRCNTL1_RXOFFPTR_MASK    (0xFFFF << BB_SPIPTRCNTL1_RXOFFPTR_POS)
#define BB_SPIPTRCNTL1_RXONPTR_POS      0
#define BB_SPIPTRCNTL1_RXONPTR_MASK     (0xFFFF << BB_SPIPTRCNTL1_RXONPTR_POS)

/* BB_SPIPTRCNTL1 settings */
#define RXOFFPTR_0                      (0x0 << BB_SPIPTRCNTL1_RXOFFPTR_POS)

#define RXONPTR_0                       (0x0 << BB_SPIPTRCNTL1_RXONPTR_POS)

/* Third control register for the radio interface SPI pointers */
#define BB_SPIPTRCNTL2_BASE             0x400015A8
#define BB_SPIPTRCNTL2                  REG32_POINTER(BB_SPIPTRCNTL2_BASE)

/* BB_SPIPTRCNTL2 bit positions */
#define BB_SPIPTRCNTL2_RSSIPTR_POS      0
#define BB_SPIPTRCNTL2_RSSIPTR_MASK     (0xFFFF << BB_SPIPTRCNTL2_RSSIPTR_POS)

/* BB_SPIPTRCNTL2 settings */
#define RSSIPTR_0                       (0x0 << BB_SPIPTRCNTL2_RSSIPTR_POS)

/* Advertising channel map register */
#define BB_ADVCHMAP_BASE                0x400015B0
#define BB_ADVCHMAP                     REG32_POINTER(BB_ADVCHMAP_BASE)

/* BB_ADVCHMAP bit positions */
#define BB_ADVCHMAP_ADVCHMAP_POS        0
#define BB_ADVCHMAP_ADVCHMAP_MASK       (0x7 << BB_ADVCHMAP_ADVCHMAP_POS)

/* BB_ADVCHMAP settings */
#define ADVCHMAP_7                      (0x7 << BB_ADVCHMAP_ADVCHMAP_POS)

/* Delay information register handling advertising event timers */
#define BB_ADVTIM_BASE                  0x400015C0
#define BB_ADVTIM                       REG32_POINTER(BB_ADVTIM_BASE)

/* BB_ADVTIM bit positions */
#define BB_ADVTIM_ADVINT_POS            0
#define BB_ADVTIM_ADVINT_MASK           (0x3FFF << BB_ADVTIM_ADVINT_POS)

/* BB_ADVTIM settings */
#define ADVINT_0                        (0x0 << BB_ADVTIM_ADVINT_POS)

/* Active scan mode control register */
#define BB_ACTSCANSTAT_BASE             0x400015C4
#define BB_ACTSCANSTAT                  READONLY_REG32_POINTER(BB_ACTSCANSTAT_BASE)

/* BB_ACTSCANSTAT bit positions */
#define BB_ACTSCANSTAT_BACKOFF_POS      16
#define BB_ACTSCANSTAT_BACKOFF_MASK     (0x1FF << BB_ACTSCANSTAT_BACKOFF_POS)
#define BB_ACTSCANSTAT_UPPERLIMIT_POS   0
#define BB_ACTSCANSTAT_UPPERLIMIT_MASK  (0x1FF << BB_ACTSCANSTAT_UPPERLIMIT_POS)

/* BB_ACTSCANSTAT settings */
#define BACKOFF_1                       (0x1 << BB_ACTSCANSTAT_BACKOFF_POS)

#define UPPERLIMIT_1                    (0x1 << BB_ACTSCANSTAT_UPPERLIMIT_POS)

/* Address pointer of public devices */
#define BB_WLPUBADDPTR_BASE             0x400015D0
#define BB_WLPUBADDPTR                  REG32_POINTER(BB_WLPUBADDPTR_BASE)

/* BB_WLPUBADDPTR bit positions */
#define BB_WLPUBADDPTR_WLPUBADDPTR_POS  0
#define BB_WLPUBADDPTR_WLPUBADDPTR_MASK (0xFFFF << BB_WLPUBADDPTR_WLPUBADDPTR_POS)

/* BB_WLPUBADDPTR settings */
#define WLPUBADDPTR_0                   (0x0 << BB_WLPUBADDPTR_WLPUBADDPTR_POS)

/* Address pointer of private devices */
#define BB_WLPRIVADDPTR_BASE            0x400015D4
#define BB_WLPRIVADDPTR                 REG32_POINTER(BB_WLPRIVADDPTR_BASE)

/* BB_WLPRIVADDPTR bit positions */
#define BB_WLPRIVADDPTR_WLPRIVADDPTR_POS 0
#define BB_WLPRIVADDPTR_WLPRIVADDPTR_MASK (0xFFFF << BB_WLPRIVADDPTR_WLPRIVADDPTR_POS)

/* BB_WLPRIVADDPTR settings */
#define WLPRIVADDPTR_0                  (0x0 << BB_WLPRIVADDPTR_WLPRIVADDPTR_POS)

/* Devices in white list */
#define BB_WLNBDEV_BASE                 0x400015D8
#define BB_WLNBDEV                      REG32_POINTER(BB_WLNBDEV_BASE)

/* BB_WLNBDEV bit positions */
#define BB_WLNBDEV_NBPRIVDEV_POS        8
#define BB_WLNBDEV_NBPRIVDEV_MASK       (0xFF << BB_WLNBDEV_NBPRIVDEV_POS)
#define BB_WLNBDEV_NBPUBDEV_POS         0
#define BB_WLNBDEV_NBPUBDEV_MASK        (0xFF << BB_WLNBDEV_NBPUBDEV_POS)

/* BB_WLNBDEV settings */
#define NBPRIVDEV_0                     (0x0 << BB_WLNBDEV_NBPRIVDEV_POS)

#define NBPUBDEV_0                      (0x0 << BB_WLNBDEV_NBPUBDEV_POS)

/* AES-128 ciphering control register */
#define BB_AESCNTL_BASE                 0x400015E0
#define BB_AESCNTL                      REG32_POINTER(BB_AESCNTL_BASE)

/* BB_AESCNTL bit positions */
#define BB_AESCNTL_AES_MODE_POS         1
#define BB_AESCNTL_AES_START_POS        0

/* BB_AESCNTL bit-band aliases */
#define BB_AESCNTL_AES_MODE_BITBAND     REG32_POINTER(SYS_CALC_BITBAND(BB_AESCNTL_BASE, BB_AESCNTL_AES_MODE_POS))
#define BB_AESCNTL_AES_START_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(BB_AESCNTL_BASE, BB_AESCNTL_AES_START_POS))

/* BB_AESCNTL settings */
#define AES_MODE_0_BITBAND              0x0
#define AES_MODE_1_BITBAND              0x1
#define AES_MODE_0                      (AES_MODE_0_BITBAND << BB_AESCNTL_AES_MODE_POS)
#define AES_MODE_1                      (AES_MODE_1_BITBAND << BB_AESCNTL_AES_MODE_POS)

#define AES_START_0_BITBAND             0x0
#define AES_START_1_BITBAND             0x1
#define AES_START_0                     (AES_START_0_BITBAND << BB_AESCNTL_AES_START_POS)
#define AES_START_1                     (AES_START_1_BITBAND << BB_AESCNTL_AES_START_POS)

/* AES encryption 128-bit key register (bits 31:0) */
#define BB_AESKEY31_0_BASE              0x400015E4
#define BB_AESKEY31_0                   REG32_POINTER(BB_AESKEY31_0_BASE)

/* AES encryption 128-bit key register (bits 63:32) */
#define BB_AESKEY63_32_BASE             0x400015E8
#define BB_AESKEY63_32                  REG32_POINTER(BB_AESKEY63_32_BASE)

/* AES encryption 128-bit key register (bits 95:64) */
#define BB_AESKEY95_64_BASE             0x400015EC
#define BB_AESKEY95_64                  REG32_POINTER(BB_AESKEY95_64_BASE)

/* AES encryption 128-bit key register (bits 127:96) */
#define BB_AESKEY127_96_BASE            0x400015F0
#define BB_AESKEY127_96                 REG32_POINTER(BB_AESKEY127_96_BASE)

/* AES memory zone pointer */
#define BB_AESPTR_BASE                  0x400015F4
#define BB_AESPTR                       REG32_POINTER(BB_AESPTR_BASE)

/* BB_AESPTR bit positions */
#define BB_AESPTR_AESPTR_POS            0
#define BB_AESPTR_AESPTR_MASK           (0xFFFF << BB_AESPTR_AESPTR_POS)

/* BB_AESPTR settings */
#define AESPTR_0                        (0x0 << BB_AESPTR_AESPTR_POS)

/* AES-CCM plain MIC value register in Tx */
#define BB_TXMICVAL_BASE                0x400015F8
#define BB_TXMICVAL                     READONLY_REG32_POINTER(BB_TXMICVAL_BASE)

/* AES-CCM plain MIC value register in Rx */
#define BB_RXMICVAL_BASE                0x400015FC
#define BB_RXMICVAL                     READONLY_REG32_POINTER(BB_RXMICVAL_BASE)

/* RF testing and regulatory body support register */
#define BB_RFTESTCNTL_BASE              0x40001600
#define BB_RFTESTCNTL                   REG32_POINTER(BB_RFTESTCNTL_BASE)

/* BB_RFTESTCNTL bit positions */
#define BB_RFTESTCNTL_INFINITERX_POS    31
#define BB_RFTESTCNTL_RXPKTCNTEN_POS    27
#define BB_RFTESTCNTL_INFINITETX_POS    15
#define BB_RFTESTCNTL_TXLENGTHSRC_POS   14
#define BB_RFTESTCNTL_PRBSTYPE_POS      13
#define BB_RFTESTCNTL_TXPLDSRC_POS      12
#define BB_RFTESTCNTL_TXPKTCNTEN_POS    11
#define BB_RFTESTCNTL_TXLENGTH_POS      0
#define BB_RFTESTCNTL_TXLENGTH_MASK     (0x1FF << BB_RFTESTCNTL_TXLENGTH_POS)

/* BB_RFTESTCNTL bit-band aliases */
#define BB_RFTESTCNTL_INFINITERX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RFTESTCNTL_BASE, BB_RFTESTCNTL_INFINITERX_POS))
#define BB_RFTESTCNTL_RXPKTCNTEN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RFTESTCNTL_BASE, BB_RFTESTCNTL_RXPKTCNTEN_POS))
#define BB_RFTESTCNTL_INFINITETX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RFTESTCNTL_BASE, BB_RFTESTCNTL_INFINITETX_POS))
#define BB_RFTESTCNTL_TXLENGTHSRC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RFTESTCNTL_BASE, BB_RFTESTCNTL_TXLENGTHSRC_POS))
#define BB_RFTESTCNTL_PRBSTYPE_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_RFTESTCNTL_BASE, BB_RFTESTCNTL_PRBSTYPE_POS))
#define BB_RFTESTCNTL_TXPLDSRC_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_RFTESTCNTL_BASE, BB_RFTESTCNTL_TXPLDSRC_POS))
#define BB_RFTESTCNTL_TXPKTCNTEN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RFTESTCNTL_BASE, BB_RFTESTCNTL_TXPKTCNTEN_POS))

/* BB_RFTESTCNTL settings */
#define INFINITERX_0_BITBAND            0x0
#define INFINITERX_1_BITBAND            0x1
#define INFINITERX_0                    (INFINITERX_0_BITBAND << BB_RFTESTCNTL_INFINITERX_POS)
#define INFINITERX_1                    (INFINITERX_1_BITBAND << BB_RFTESTCNTL_INFINITERX_POS)

#define RXPKTCNTEN_0_BITBAND            0x0
#define RXPKTCNTEN_1_BITBAND            0x1
#define RXPKTCNTEN_0                    (RXPKTCNTEN_0_BITBAND << BB_RFTESTCNTL_RXPKTCNTEN_POS)
#define RXPKTCNTEN_1                    (RXPKTCNTEN_1_BITBAND << BB_RFTESTCNTL_RXPKTCNTEN_POS)

#define INFINITETX_0_BITBAND            0x0
#define INFINITETX_1_BITBAND            0x1
#define INFINITETX_0                    (INFINITETX_0_BITBAND << BB_RFTESTCNTL_INFINITETX_POS)
#define INFINITETX_1                    (INFINITETX_1_BITBAND << BB_RFTESTCNTL_INFINITETX_POS)

#define TXLENGTHSRC_0_BITBAND           0x0
#define TXLENGTHSRC_1_BITBAND           0x1
#define TXLENGTHSRC_0                   (TXLENGTHSRC_0_BITBAND << BB_RFTESTCNTL_TXLENGTHSRC_POS)
#define TXLENGTHSRC_1                   (TXLENGTHSRC_1_BITBAND << BB_RFTESTCNTL_TXLENGTHSRC_POS)

#define PRBSTYPE_0_BITBAND              0x0
#define PRBSTYPE_1_BITBAND              0x1
#define PRBSTYPE_0                      (PRBSTYPE_0_BITBAND << BB_RFTESTCNTL_PRBSTYPE_POS)
#define PRBSTYPE_1                      (PRBSTYPE_1_BITBAND << BB_RFTESTCNTL_PRBSTYPE_POS)

#define TXPLDSRC_0_BITBAND              0x0
#define TXPLDSRC_1_BITBAND              0x1
#define TXPLDSRC_0                      (TXPLDSRC_0_BITBAND << BB_RFTESTCNTL_TXPLDSRC_POS)
#define TXPLDSRC_1                      (TXPLDSRC_1_BITBAND << BB_RFTESTCNTL_TXPLDSRC_POS)

#define TXPKTCNTEN_0_BITBAND            0x0
#define TXPKTCNTEN_1_BITBAND            0x1
#define TXPKTCNTEN_0                    (TXPKTCNTEN_0_BITBAND << BB_RFTESTCNTL_TXPKTCNTEN_POS)
#define TXPKTCNTEN_1                    (TXPKTCNTEN_1_BITBAND << BB_RFTESTCNTL_TXPKTCNTEN_POS)

#define TXLENGTH_0                      (0x0 << BB_RFTESTCNTL_TXLENGTH_POS)

/* Number of transmitted packet during test modes */
#define BB_RFTESTTXSTAT_BASE            0x40001604
#define BB_RFTESTTXSTAT                 REG32_POINTER(BB_RFTESTTXSTAT_BASE)

/* Number of correctly received packet during test modes */
#define BB_RFTESTRXSTAT_BASE            0x40001608
#define BB_RFTESTRXSTAT                 REG32_POINTER(BB_RFTESTRXSTAT_BASE)

/* Timing generator control register */
#define BB_TIMGENCNTL_BASE              0x40001610
#define BB_TIMGENCNTL                   REG32_POINTER(BB_TIMGENCNTL_BASE)

/* BB_TIMGENCNTL bit positions */
#define BB_TIMGENCNTL_APFM_EN_POS       31
#define BB_TIMGENCNTL_PREFETCHABORT_TIME_POS 16
#define BB_TIMGENCNTL_PREFETCHABORT_TIME_MASK (0x3FF << BB_TIMGENCNTL_PREFETCHABORT_TIME_POS)
#define BB_TIMGENCNTL_PREFETCH_TIME_POS 0
#define BB_TIMGENCNTL_PREFETCH_TIME_MASK (0x1FF << BB_TIMGENCNTL_PREFETCH_TIME_POS)

/* BB_TIMGENCNTL bit-band aliases */
#define BB_TIMGENCNTL_APFM_EN_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_TIMGENCNTL_BASE, BB_TIMGENCNTL_APFM_EN_POS))

/* BB_TIMGENCNTL settings */
#define APFM_EN_0_BITBAND               0x0
#define APFM_EN_1_BITBAND               0x1
#define APFM_EN_0                       (APFM_EN_0_BITBAND << BB_TIMGENCNTL_APFM_EN_POS)
#define APFM_EN_1                       (APFM_EN_1_BITBAND << BB_TIMGENCNTL_APFM_EN_POS)

#define PREFETCHABORT_TIME_254          (0x1FE << BB_TIMGENCNTL_PREFETCHABORT_TIME_POS)

#define PREFETCH_TIME_150               (0x96 << BB_TIMGENCNTL_PREFETCH_TIME_POS)

/* Gross timer control register */
#define BB_GROSSTIMTGT_BASE             0x40001614
#define BB_GROSSTIMTGT                  REG32_POINTER(BB_GROSSTIMTGT_BASE)

/* BB_GROSSTIMTGT bit positions */
#define BB_GROSSTIMTGT_GROSSTARGET_POS  0
#define BB_GROSSTIMTGT_GROSSTARGET_MASK (0x7FFFFF << BB_GROSSTIMTGT_GROSSTARGET_POS)

/* BB_GROSSTIMTGT settings */
#define GROSSTARGET_0                   (0x0 << BB_GROSSTIMTGT_GROSSTARGET_POS)

/* Fine timer control register */
#define BB_FINETIMTGT_BASE              0x40001618
#define BB_FINETIMTGT                   REG32_POINTER(BB_FINETIMTGT_BASE)

/* BB_FINETIMTGT bit positions */
#define BB_FINETIMTGT_FINETARGET_POS    0
#define BB_FINETIMTGT_FINETARGET_MASK   (0x7FFFFFF << BB_FINETIMTGT_FINETARGET_POS)

/* BB_FINETIMTGT settings */
#define FINETARGET_0                    (0x0 << BB_FINETIMTGT_FINETARGET_POS)

/* Coexistence control register 0 */
#define BB_COEXIFCNTL0_BASE             0x40001620
#define BB_COEXIFCNTL0                  REG32_POINTER(BB_COEXIFCNTL0_BASE)

/* BB_COEXIFCNTL0 bit positions */
#define BB_COEXIFCNTL0_MWSSCANFREQMSK_POS 24
#define BB_COEXIFCNTL0_MWSSCANFREQMSK_MASK (0x3 << BB_COEXIFCNTL0_MWSSCANFREQMSK_POS)
#define BB_COEXIFCNTL0_WLCRXPRIOMODE_POS 20
#define BB_COEXIFCNTL0_WLCRXPRIOMODE_MASK (0x3 << BB_COEXIFCNTL0_WLCRXPRIOMODE_POS)
#define BB_COEXIFCNTL0_WLCTXPRIOMODE_POS 16
#define BB_COEXIFCNTL0_WLCTXPRIOMODE_MASK (0x3 << BB_COEXIFCNTL0_WLCTXPRIOMODE_POS)
#define BB_COEXIFCNTL0_MWSTXFREQMSK_POS 14
#define BB_COEXIFCNTL0_MWSTXFREQMSK_MASK (0x3 << BB_COEXIFCNTL0_MWSTXFREQMSK_POS)
#define BB_COEXIFCNTL0_MWSRXFREQMSK_POS 12
#define BB_COEXIFCNTL0_MWSRXFREQMSK_MASK (0x3 << BB_COEXIFCNTL0_MWSRXFREQMSK_POS)
#define BB_COEXIFCNTL0_MWSTXMSK_POS     10
#define BB_COEXIFCNTL0_MWSTXMSK_MASK    (0x3 << BB_COEXIFCNTL0_MWSTXMSK_POS)
#define BB_COEXIFCNTL0_MWSRXMSK_POS     8
#define BB_COEXIFCNTL0_MWSRXMSK_MASK    (0x3 << BB_COEXIFCNTL0_MWSRXMSK_POS)
#define BB_COEXIFCNTL0_TXMSK_POS        6
#define BB_COEXIFCNTL0_TXMSK_MASK       (0x3 << BB_COEXIFCNTL0_TXMSK_POS)
#define BB_COEXIFCNTL0_RXMSK_POS        4
#define BB_COEXIFCNTL0_RXMSK_MASK       (0x3 << BB_COEXIFCNTL0_RXMSK_POS)
#define BB_COEXIFCNTL0_MWSWCI_EN_POS    3
#define BB_COEXIFCNTL0_MWSCOEX_EN_POS   2
#define BB_COEXIFCNTL0_SYNCGEN_EN_POS   1
#define BB_COEXIFCNTL0_COEX_EN_POS      0

/* BB_COEXIFCNTL0 bit-band aliases */
#define BB_COEXIFCNTL0_MWSWCI_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_COEXIFCNTL0_BASE, BB_COEXIFCNTL0_MWSWCI_EN_POS))
#define BB_COEXIFCNTL0_MWSCOEX_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_COEXIFCNTL0_BASE, BB_COEXIFCNTL0_MWSCOEX_EN_POS))
#define BB_COEXIFCNTL0_SYNCGEN_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_COEXIFCNTL0_BASE, BB_COEXIFCNTL0_SYNCGEN_EN_POS))
#define BB_COEXIFCNTL0_COEX_EN_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(BB_COEXIFCNTL0_BASE, BB_COEXIFCNTL0_COEX_EN_POS))

/* BB_COEXIFCNTL0 settings */
#define MWSSCANFREQMSK_0                (0x0 << BB_COEXIFCNTL0_MWSSCANFREQMSK_POS)
#define MWSSCANFREQMSK_1                (0x1 << BB_COEXIFCNTL0_MWSSCANFREQMSK_POS)
#define MWSSCANFREQMSK_2                (0x2 << BB_COEXIFCNTL0_MWSSCANFREQMSK_POS)
#define MWSSCANFREQMSK_3                (0x3 << BB_COEXIFCNTL0_MWSSCANFREQMSK_POS)

#define WLCRXPRIOMODE_0                 (0x0 << BB_COEXIFCNTL0_WLCRXPRIOMODE_POS)
#define WLCRXPRIOMODE_1                 (0x1 << BB_COEXIFCNTL0_WLCRXPRIOMODE_POS)
#define WLCRXPRIOMODE_2                 (0x2 << BB_COEXIFCNTL0_WLCRXPRIOMODE_POS)
#define WLCRXPRIOMODE_3                 (0x3 << BB_COEXIFCNTL0_WLCRXPRIOMODE_POS)

#define WLCTXPRIOMODE_0                 (0x0 << BB_COEXIFCNTL0_WLCTXPRIOMODE_POS)
#define WLCTXPRIOMODE_1                 (0x1 << BB_COEXIFCNTL0_WLCTXPRIOMODE_POS)
#define WLCTXPRIOMODE_2                 (0x2 << BB_COEXIFCNTL0_WLCTXPRIOMODE_POS)
#define WLCTXPRIOMODE_3                 (0x3 << BB_COEXIFCNTL0_WLCTXPRIOMODE_POS)

#define MWSTXFREQMSK_0                  (0x0 << BB_COEXIFCNTL0_MWSTXFREQMSK_POS)
#define MWSTXFREQMSK_1                  (0x1 << BB_COEXIFCNTL0_MWSTXFREQMSK_POS)
#define MWSTXFREQMSK_2                  (0x2 << BB_COEXIFCNTL0_MWSTXFREQMSK_POS)
#define MWSTXFREQMSK_3                  (0x3 << BB_COEXIFCNTL0_MWSTXFREQMSK_POS)

#define MWSRXFREQMSK_0                  (0x0 << BB_COEXIFCNTL0_MWSRXFREQMSK_POS)
#define MWSRXFREQMSK_1                  (0x1 << BB_COEXIFCNTL0_MWSRXFREQMSK_POS)
#define MWSRXFREQMSK_2                  (0x2 << BB_COEXIFCNTL0_MWSRXFREQMSK_POS)
#define MWSRXFREQMSK_3                  (0x3 << BB_COEXIFCNTL0_MWSRXFREQMSK_POS)

#define MWSTXMSK_0                      (0x0 << BB_COEXIFCNTL0_MWSTXMSK_POS)
#define MWSTXMSK_1                      (0x1 << BB_COEXIFCNTL0_MWSTXMSK_POS)
#define MWSTXMSK_2                      (0x2 << BB_COEXIFCNTL0_MWSTXMSK_POS)
#define MWSTXMSK_3                      (0x3 << BB_COEXIFCNTL0_MWSTXMSK_POS)

#define MWSRXMSK_0                      (0x0 << BB_COEXIFCNTL0_MWSRXMSK_POS)
#define MWSRXMSK_1                      (0x1 << BB_COEXIFCNTL0_MWSRXMSK_POS)
#define MWSRXMSK_2                      (0x2 << BB_COEXIFCNTL0_MWSRXMSK_POS)
#define MWSRXMSK_3                      (0x3 << BB_COEXIFCNTL0_MWSRXMSK_POS)

#define TXMSK_0                         (0x0 << BB_COEXIFCNTL0_TXMSK_POS)
#define TXMSK_1                         (0x1 << BB_COEXIFCNTL0_TXMSK_POS)
#define TXMSK_2                         (0x2 << BB_COEXIFCNTL0_TXMSK_POS)
#define TXMSK_3                         (0x3 << BB_COEXIFCNTL0_TXMSK_POS)

#define RXMSK_0                         (0x0 << BB_COEXIFCNTL0_RXMSK_POS)
#define RXMSK_1                         (0x1 << BB_COEXIFCNTL0_RXMSK_POS)
#define RXMSK_2                         (0x2 << BB_COEXIFCNTL0_RXMSK_POS)
#define RXMSK_3                         (0x3 << BB_COEXIFCNTL0_RXMSK_POS)

#define MWSWCI_EN_0_BITBAND             0x0
#define MWSWCI_EN_1_BITBAND             0x1
#define MWSWCI_EN_0                     (MWSWCI_EN_0_BITBAND << BB_COEXIFCNTL0_MWSWCI_EN_POS)
#define MWSWCI_EN_1                     (MWSWCI_EN_1_BITBAND << BB_COEXIFCNTL0_MWSWCI_EN_POS)

#define MWSCOEX_EN_0_BITBAND            0x0
#define MWSCOEX_EN_1_BITBAND            0x1
#define MWSCOEX_EN_0                    (MWSCOEX_EN_0_BITBAND << BB_COEXIFCNTL0_MWSCOEX_EN_POS)
#define MWSCOEX_EN_1                    (MWSCOEX_EN_1_BITBAND << BB_COEXIFCNTL0_MWSCOEX_EN_POS)

#define SYNCGEN_EN_0_BITBAND            0x0
#define SYNCGEN_EN_1_BITBAND            0x1
#define SYNCGEN_EN_0                    (SYNCGEN_EN_0_BITBAND << BB_COEXIFCNTL0_SYNCGEN_EN_POS)
#define SYNCGEN_EN_1                    (SYNCGEN_EN_1_BITBAND << BB_COEXIFCNTL0_SYNCGEN_EN_POS)

#define COEX_EN_0_BITBAND               0x0
#define COEX_EN_1_BITBAND               0x1
#define COEX_EN_0                       (COEX_EN_0_BITBAND << BB_COEXIFCNTL0_COEX_EN_POS)
#define COEX_EN_1                       (COEX_EN_1_BITBAND << BB_COEXIFCNTL0_COEX_EN_POS)

/* Coexistence control register 1 */
#define BB_COEXIFCNTL1_BASE             0x40001624
#define BB_COEXIFCNTL1                  REG32_POINTER(BB_COEXIFCNTL1_BASE)

/* BB_COEXIFCNTL1 bit positions */
#define BB_COEXIFCNTL1_WLCPRXTHR_POS    24
#define BB_COEXIFCNTL1_WLCPRXTHR_MASK   (0x1F << BB_COEXIFCNTL1_WLCPRXTHR_POS)
#define BB_COEXIFCNTL1_WLCPTXTHR_POS    16
#define BB_COEXIFCNTL1_WLCPTXTHR_MASK   (0x1F << BB_COEXIFCNTL1_WLCPTXTHR_POS)
#define BB_COEXIFCNTL1_WLCPDURATION_POS 8
#define BB_COEXIFCNTL1_WLCPDURATION_MASK (0x7F << BB_COEXIFCNTL1_WLCPDURATION_POS)
#define BB_COEXIFCNTL1_WLCPDELAY_POS    0
#define BB_COEXIFCNTL1_WLCPDELAY_MASK   (0x7F << BB_COEXIFCNTL1_WLCPDELAY_POS)

/* BB_COEXIFCNTL1 settings */
#define WLCPRXTHR_0                     (0x0 << BB_COEXIFCNTL1_WLCPRXTHR_POS)

#define WLCPTXTHR_0                     (0x0 << BB_COEXIFCNTL1_WLCPTXTHR_POS)

#define WLCPDURATION_0                  (0x0 << BB_COEXIFCNTL1_WLCPDURATION_POS)

#define WLCPDELAY_0                     (0x0 << BB_COEXIFCNTL1_WLCPDELAY_POS)

/* Coexistence control register 2 */
#define BB_COEXIFCNTL2_BASE             0x40001628
#define BB_COEXIFCNTL2                  REG32_POINTER(BB_COEXIFCNTL2_BASE)

/* BB_COEXIFCNTL2 bit positions */
#define BB_COEXIFCNTL2_RX_ANT_DELAY_POS 8
#define BB_COEXIFCNTL2_RX_ANT_DELAY_MASK (0xF << BB_COEXIFCNTL2_RX_ANT_DELAY_POS)
#define BB_COEXIFCNTL2_TX_ANT_DELAY_POS 0
#define BB_COEXIFCNTL2_TX_ANT_DELAY_MASK (0xF << BB_COEXIFCNTL2_TX_ANT_DELAY_POS)

/* BB_COEXIFCNTL2 settings */
#define RX_ANT_DELAY_0                  (0x0 << BB_COEXIFCNTL2_RX_ANT_DELAY_POS)

#define TX_ANT_DELAY_0                  (0x0 << BB_COEXIFCNTL2_TX_ANT_DELAY_POS)

/* Priority control register 0 */
#define BB_BBMPRIO0_BASE                0x4000162C
#define BB_BBMPRIO0                     REG32_POINTER(BB_BBMPRIO0_BASE)

/* BB_BBMPRIO0 bit positions */
#define BB_BBMPRIO0_BLEM7_POS           28
#define BB_BBMPRIO0_BLEM7_MASK          (0xF << BB_BBMPRIO0_BLEM7_POS)
#define BB_BBMPRIO0_BLEM6_POS           24
#define BB_BBMPRIO0_BLEM6_MASK          (0xF << BB_BBMPRIO0_BLEM6_POS)
#define BB_BBMPRIO0_BLEM5_POS           20
#define BB_BBMPRIO0_BLEM5_MASK          (0xF << BB_BBMPRIO0_BLEM5_POS)
#define BB_BBMPRIO0_BLEM4_POS           16
#define BB_BBMPRIO0_BLEM4_MASK          (0xF << BB_BBMPRIO0_BLEM4_POS)
#define BB_BBMPRIO0_BLEM3_POS           12
#define BB_BBMPRIO0_BLEM3_MASK          (0xF << BB_BBMPRIO0_BLEM3_POS)
#define BB_BBMPRIO0_BLEM2_POS           8
#define BB_BBMPRIO0_BLEM2_MASK          (0xF << BB_BBMPRIO0_BLEM2_POS)
#define BB_BBMPRIO0_BLEM1_POS           4
#define BB_BBMPRIO0_BLEM1_MASK          (0xF << BB_BBMPRIO0_BLEM1_POS)
#define BB_BBMPRIO0_BLEM0_POS           0
#define BB_BBMPRIO0_BLEM0_MASK          (0xF << BB_BBMPRIO0_BLEM0_POS)

/* BB_BBMPRIO0 settings */
#define BLEM7_3                         (0x3 << BB_BBMPRIO0_BLEM7_POS)

#define BLEM6_4                         (0x4 << BB_BBMPRIO0_BLEM6_POS)

#define BLEM5_8                         (0x8 << BB_BBMPRIO0_BLEM5_POS)

#define BLEM4_9                         (0x9 << BB_BBMPRIO0_BLEM4_POS)

#define BLEM3_10                        (0xA << BB_BBMPRIO0_BLEM3_POS)

#define BLEM2_13                        (0xD << BB_BBMPRIO0_BLEM2_POS)

#define BLEM1_14                        (0xE << BB_BBMPRIO0_BLEM1_POS)

#define BLEM0_15                        (0xF << BB_BBMPRIO0_BLEM0_POS)

/* Priority control register 1 */
#define BB_BBMPRIO1_BASE                0x40001630
#define BB_BBMPRIO1                     REG32_POINTER(BB_BBMPRIO1_BASE)

/* BB_BBMPRIO1 bit positions */
#define BB_BBMPRIO1_BLEMDEFAULT_POS     28
#define BB_BBMPRIO1_BLEMDEFAULT_MASK    (0xF << BB_BBMPRIO1_BLEMDEFAULT_POS)
#define BB_BBMPRIO1_BLEM9_POS           4
#define BB_BBMPRIO1_BLEM9_MASK          (0xF << BB_BBMPRIO1_BLEM9_POS)
#define BB_BBMPRIO1_BLEM8_POS           0
#define BB_BBMPRIO1_BLEM8_MASK          (0xF << BB_BBMPRIO1_BLEM8_POS)

/* BB_BBMPRIO1 settings */
#define BLEMDEFAULT_3                   (0x3 << BB_BBMPRIO1_BLEMDEFAULT_POS)

#define BLEM7_13                        (0xD << BB_BBMPRIO1_BLEM9_POS)

#define BLEM6_12                        (0xC << BB_BBMPRIO1_BLEM8_POS)

/* Register used by the Resolving Address List engine */
#define BB_RALPTR_BASE                  0x40001640
#define BB_RALPTR                       REG32_POINTER(BB_RALPTR_BASE)

/* BB_RALPTR bit positions */
#define BB_RALPTR_RALPTR_POS            0
#define BB_RALPTR_RALPTR_MASK           (0xFFFF << BB_RALPTR_RALPTR_POS)

/* BB_RALPTR settings */
#define RALPTR_0                        (0x0 << BB_RALPTR_RALPTR_POS)

/* Register used by the Resolving Address List engine */
#define BB_RALNBDEV_BASE                0x40001644
#define BB_RALNBDEV                     REG32_POINTER(BB_RALNBDEV_BASE)

/* BB_RALNBDEV bit positions */
#define BB_RALNBDEV_RALNBDEV_POS        0
#define BB_RALNBDEV_RALNBDEV_MASK       (0xFF << BB_RALNBDEV_RALNBDEV_POS)

/* BB_RALNBDEV settings */
#define RALNBDEV_0                      (0x0 << BB_RALNBDEV_RALNBDEV_POS)

/* Register used by the Resolving Address List engine */
#define BB_RAL_LOCAL_RND_BASE           0x40001648
#define BB_RAL_LOCAL_RND                REG32_POINTER(BB_RAL_LOCAL_RND_BASE)

/* BB_RAL_LOCAL_RND bit positions */
#define BB_RAL_LOCAL_RND_LRND_INIT_POS  31
#define BB_RAL_LOCAL_RND_LRND_VAL_POS   0
#define BB_RAL_LOCAL_RND_LRND_VAL_MASK  (0x3FFFFF << BB_RAL_LOCAL_RND_LRND_VAL_POS)

/* BB_RAL_LOCAL_RND bit-band aliases */
#define BB_RAL_LOCAL_RND_LRND_INIT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RAL_LOCAL_RND_BASE, BB_RAL_LOCAL_RND_LRND_INIT_POS))

/* BB_RAL_LOCAL_RND settings */
#define LRND_INIT_0_BITBAND             0x0
#define LRND_INIT_0                     (LRND_INIT_0_BITBAND << BB_RAL_LOCAL_RND_LRND_INIT_POS)

#define LRND_VAL_4132623                (0x3F0F0F << BB_RAL_LOCAL_RND_LRND_VAL_POS)

/* Register used by the Resolving Address List engine */
#define BB_RAL_PEER_RND_BASE            0x4000164C
#define BB_RAL_PEER_RND                 REG32_POINTER(BB_RAL_PEER_RND_BASE)

/* BB_RAL_PEER_RND bit positions */
#define BB_RAL_PEER_RND_PRND_INIT_POS   31
#define BB_RAL_PEER_RND_PRND_VAL_POS    0
#define BB_RAL_PEER_RND_PRND_VAL_MASK   (0x3FFFFF << BB_RAL_PEER_RND_PRND_VAL_POS)

/* BB_RAL_PEER_RND bit-band aliases */
#define BB_RAL_PEER_RND_PRND_INIT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_RAL_PEER_RND_BASE, BB_RAL_PEER_RND_PRND_INIT_POS))

/* BB_RAL_PEER_RND settings */
#define PRND_INIT_0_BITBAND             0x0
#define PRND_INIT_0                     (PRND_INIT_0_BITBAND << BB_RAL_PEER_RND_PRND_INIT_POS)

#define PRND_VAL_3207408                (0x30F0F0 << BB_RAL_PEER_RND_PRND_VAL_POS)

/* ISO Channel 0 control */
#define BB_ISOCHANCNTL0_BASE            0x40001650
#define BB_ISOCHANCNTL0                 REG32_POINTER(BB_ISOCHANCNTL0_BASE)

/* BB_ISOCHANCNTL0 bit positions */
#define BB_ISOCHANCNTL0_RETXACKEN0_POS  4
#define BB_ISOCHANCNTL0_SYNCGEN0_POS    3
#define BB_ISOCHANCNTL0_ISOCHANEN0_POS  2
#define BB_ISOCHANCNTL0_ISOTYPE0_POS    0
#define BB_ISOCHANCNTL0_ISOTYPE0_MASK   (0x3 << BB_ISOCHANCNTL0_ISOTYPE0_POS)

/* BB_ISOCHANCNTL0 bit-band aliases */
#define BB_ISOCHANCNTL0_RETXACKEN0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL0_BASE, BB_ISOCHANCNTL0_RETXACKEN0_POS))
#define BB_ISOCHANCNTL0_SYNCGEN0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL0_BASE, BB_ISOCHANCNTL0_SYNCGEN0_POS))
#define BB_ISOCHANCNTL0_ISOCHANEN0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL0_BASE, BB_ISOCHANCNTL0_ISOCHANEN0_POS))

/* BB_ISOCHANCNTL0 settings */
#define RETXACKEN0_0_BITBAND            0x0
#define RETXACKEN0_1_BITBAND            0x1
#define RETXACKEN0_0                    (RETXACKEN0_0_BITBAND << BB_ISOCHANCNTL0_RETXACKEN0_POS)
#define RETXACKEN0_1                    (RETXACKEN0_1_BITBAND << BB_ISOCHANCNTL0_RETXACKEN0_POS)

#define SYNCGEN0_0_BITBAND              0x0
#define SYNCGEN0_1_BITBAND              0x1
#define SYNCGEN0_0                      (SYNCGEN0_0_BITBAND << BB_ISOCHANCNTL0_SYNCGEN0_POS)
#define SYNCGEN0_1                      (SYNCGEN0_1_BITBAND << BB_ISOCHANCNTL0_SYNCGEN0_POS)

#define ISOCHANEN0_0_BITBAND            0x0
#define ISOCHANEN0_1_BITBAND            0x1
#define ISOCHANEN0_0                    (ISOCHANEN0_0_BITBAND << BB_ISOCHANCNTL0_ISOCHANEN0_POS)
#define ISOCHANEN0_1                    (ISOCHANEN0_1_BITBAND << BB_ISOCHANCNTL0_ISOCHANEN0_POS)

#define ISOTYPE0_0                      (0x0 << BB_ISOCHANCNTL0_ISOTYPE0_POS)
#define ISOTYPE0_1                      (0x1 << BB_ISOCHANCNTL0_ISOTYPE0_POS)
#define ISOTYPE0_2                      (0x2 << BB_ISOCHANCNTL0_ISOTYPE0_POS)
#define ISOTYPE0_3                      (0x3 << BB_ISOCHANCNTL0_ISOTYPE0_POS)

/* ISO Channel 0 mute control */
#define BB_ISOMUTECNTL0_BASE            0x40001654
#define BB_ISOMUTECNTL0                 REG32_POINTER(BB_ISOMUTECNTL0_BASE)

/* BB_ISOMUTECNTL0 bit positions */
#define BB_ISOMUTECNTL0_TOGO0_POS       31
#define BB_ISOMUTECNTL0_MUTE_SINK0_POS  19
#define BB_ISOMUTECNTL0_MUTE_SOURCE0_POS 18
#define BB_ISOMUTECNTL0_INVL0_1_POS     17
#define BB_ISOMUTECNTL0_INVL0_0_POS     16
#define BB_ISOMUTECNTL0_MUTE_PATTERN0_POS 0
#define BB_ISOMUTECNTL0_MUTE_PATTERN0_MASK (0xFF << BB_ISOMUTECNTL0_MUTE_PATTERN0_POS)

/* BB_ISOMUTECNTL0 bit-band aliases */
#define BB_ISOMUTECNTL0_TOGO0_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL0_BASE, BB_ISOMUTECNTL0_TOGO0_POS))
#define BB_ISOMUTECNTL0_MUTE_SINK0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL0_BASE, BB_ISOMUTECNTL0_MUTE_SINK0_POS))
#define BB_ISOMUTECNTL0_MUTE_SOURCE0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL0_BASE, BB_ISOMUTECNTL0_MUTE_SOURCE0_POS))
#define BB_ISOMUTECNTL0_INVL0_1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL0_BASE, BB_ISOMUTECNTL0_INVL0_1_POS))
#define BB_ISOMUTECNTL0_INVL0_0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL0_BASE, BB_ISOMUTECNTL0_INVL0_0_POS))

/* BB_ISOMUTECNTL0 settings */
#define TOGO0_0_BITBAND                 0x0
#define TOGO0_1_BITBAND                 0x1
#define TOGO0_0                         (TOGO0_0_BITBAND << BB_ISOMUTECNTL0_TOGO0_POS)
#define TOGO0_1                         (TOGO0_1_BITBAND << BB_ISOMUTECNTL0_TOGO0_POS)

#define MUTE_SINK0_0_BITBAND            0x0
#define MUTE_SINK0_1_BITBAND            0x1
#define MUTE_SINK0_0                    (MUTE_SINK0_0_BITBAND << BB_ISOMUTECNTL0_MUTE_SINK0_POS)
#define MUTE_SINK0_1                    (MUTE_SINK0_1_BITBAND << BB_ISOMUTECNTL0_MUTE_SINK0_POS)

#define MUTE_SOURCE0_0_BITBAND          0x0
#define MUTE_SOURCE0_1_BITBAND          0x1
#define MUTE_SOURCE0_0                  (MUTE_SOURCE0_0_BITBAND << BB_ISOMUTECNTL0_MUTE_SOURCE0_POS)
#define MUTE_SOURCE0_1                  (MUTE_SOURCE0_1_BITBAND << BB_ISOMUTECNTL0_MUTE_SOURCE0_POS)

#define INVL0_1_0_BITBAND               0x0
#define INVL0_1_1_BITBAND               0x1
#define INVL0_1_0                       (INVL0_1_0_BITBAND << BB_ISOMUTECNTL0_INVL0_1_POS)
#define INVL0_1_1                       (INVL0_1_1_BITBAND << BB_ISOMUTECNTL0_INVL0_1_POS)

#define INVL0_0_0_BITBAND               0x0
#define INVL0_0_1_BITBAND               0x1
#define INVL0_0_0                       (INVL0_0_0_BITBAND << BB_ISOMUTECNTL0_INVL0_0_POS)
#define INVL0_0_1                       (INVL0_0_1_BITBAND << BB_ISOMUTECNTL0_INVL0_0_POS)

#define MUTE_PATTERN0_0                 (0x0 << BB_ISOMUTECNTL0_MUTE_PATTERN0_POS)

/* ISO Channel 0 current Tx pointer */
#define BB_ISOCURRENTTXPTR0_BASE        0x40001658
#define BB_ISOCURRENTTXPTR0             REG32_POINTER(BB_ISOCURRENTTXPTR0_BASE)

/* BB_ISOCURRENTTXPTR0 bit positions */
#define BB_ISOCURRENTTXPTR0_ISO0TXPTR0_POS 16
#define BB_ISOCURRENTTXPTR0_ISO0TXPTR0_MASK (0xFFFF << BB_ISOCURRENTTXPTR0_ISO0TXPTR0_POS)
#define BB_ISOCURRENTTXPTR0_ISO0TXPTR1_POS 0
#define BB_ISOCURRENTTXPTR0_ISO0TXPTR1_MASK (0xFFFF << BB_ISOCURRENTTXPTR0_ISO0TXPTR1_POS)

/* BB_ISOCURRENTTXPTR0 settings */
#define ISO0TXPTR0_0                    (0x0 << BB_ISOCURRENTTXPTR0_ISO0TXPTR0_POS)

#define ISO0TXPTR1_0                    (0x0 << BB_ISOCURRENTTXPTR0_ISO0TXPTR1_POS)

/* ISO Channel 0 current Rx pointer */
#define BB_ISOCURRENTRXPTR0_BASE        0x4000165C
#define BB_ISOCURRENTRXPTR0             REG32_POINTER(BB_ISOCURRENTRXPTR0_BASE)

/* BB_ISOCURRENTRXPTR0 bit positions */
#define BB_ISOCURRENTRXPTR0_ISO0RXPTR0_POS 16
#define BB_ISOCURRENTRXPTR0_ISO0RXPTR0_MASK (0xFFFF << BB_ISOCURRENTRXPTR0_ISO0RXPTR0_POS)
#define BB_ISOCURRENTRXPTR0_ISO0RXPTR1_POS 0
#define BB_ISOCURRENTRXPTR0_ISO0RXPTR1_MASK (0xFFFF << BB_ISOCURRENTRXPTR0_ISO0RXPTR1_POS)

/* BB_ISOCURRENTRXPTR0 settings */
#define ISO0RXPTR0_0                    (0x0 << BB_ISOCURRENTRXPTR0_ISO0RXPTR0_POS)

#define ISO0RXPTR1_0                    (0x0 << BB_ISOCURRENTRXPTR0_ISO0RXPTR1_POS)

/* ISO Channel 0 payloads */
#define BB_ISOTRCNL0_BASE               0x40001660
#define BB_ISOTRCNL0                    REG32_POINTER(BB_ISOTRCNL0_BASE)

/* BB_ISOTRCNL0 bit positions */
#define BB_ISOTRCNL0_ISO0RXLEN_POS      16
#define BB_ISOTRCNL0_ISO0RXLEN_MASK     (0xFF << BB_ISOTRCNL0_ISO0RXLEN_POS)
#define BB_ISOTRCNL0_ISO0TXLEN_POS      0
#define BB_ISOTRCNL0_ISO0TXLEN_MASK     (0xFF << BB_ISOTRCNL0_ISO0TXLEN_POS)

/* BB_ISOTRCNL0 settings */
#define ISO0RXLEN_0                     (0x0 << BB_ISOTRCNL0_ISO0RXLEN_POS)

#define ISO0TXLEN_0                     (0x0 << BB_ISOTRCNL0_ISO0TXLEN_POS)

/* ISO Channel 0 mute control */
#define BB_ISOEVTCNTLOFFSETL0_BASE      0x40001664
#define BB_ISOEVTCNTLOFFSETL0           REG32_POINTER(BB_ISOEVTCNTLOFFSETL0_BASE)

/* ISO Channel 0 mute control */
#define BB_ISOEVTCNTLOFFSETU0_BASE      0x40001668
#define BB_ISOEVTCNTLOFFSETU0           REG32_POINTER(BB_ISOEVTCNTLOFFSETU0_BASE)

/* BB_ISOEVTCNTLOFFSETU0 bit positions */
#define BB_ISOEVTCNTLOFFSETU0_EVT_CNT_OFFSETU0_POS 0
#define BB_ISOEVTCNTLOFFSETU0_EVT_CNT_OFFSETU0_MASK (0x7F << BB_ISOEVTCNTLOFFSETU0_EVT_CNT_OFFSETU0_POS)

/* BB_ISOEVTCNTLOFFSETU0 settings */
#define EVT_CNT_OFFSETU0_0              (0x0 << BB_ISOEVTCNTLOFFSETU0_EVT_CNT_OFFSETU0_POS)

/* ISO Channel 1 control */
#define BB_ISOCHANCNTL1_BASE            0x40001670
#define BB_ISOCHANCNTL1                 REG32_POINTER(BB_ISOCHANCNTL1_BASE)

/* BB_ISOCHANCNTL1 bit positions */
#define BB_ISOCHANCNTL1_RETXACKEN1_POS  4
#define BB_ISOCHANCNTL1_SYNCGEN1_POS    3
#define BB_ISOCHANCNTL1_ISOCHANEN1_POS  2
#define BB_ISOCHANCNTL1_ISOTYPE1_POS    0
#define BB_ISOCHANCNTL1_ISOTYPE1_MASK   (0x3 << BB_ISOCHANCNTL1_ISOTYPE1_POS)

/* BB_ISOCHANCNTL1 bit-band aliases */
#define BB_ISOCHANCNTL1_RETXACKEN1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL1_BASE, BB_ISOCHANCNTL1_RETXACKEN1_POS))
#define BB_ISOCHANCNTL1_SYNCGEN1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL1_BASE, BB_ISOCHANCNTL1_SYNCGEN1_POS))
#define BB_ISOCHANCNTL1_ISOCHANEN1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL1_BASE, BB_ISOCHANCNTL1_ISOCHANEN1_POS))

/* BB_ISOCHANCNTL1 settings */
#define RETXACKEN1_0_BITBAND            0x0
#define RETXACKEN1_1_BITBAND            0x1
#define RETXACKEN1_0                    (RETXACKEN1_0_BITBAND << BB_ISOCHANCNTL1_RETXACKEN1_POS)
#define RETXACKEN1_1                    (RETXACKEN1_1_BITBAND << BB_ISOCHANCNTL1_RETXACKEN1_POS)

#define SYNCGEN1_0_BITBAND              0x0
#define SYNCGEN1_1_BITBAND              0x1
#define SYNCGEN1_0                      (SYNCGEN1_0_BITBAND << BB_ISOCHANCNTL1_SYNCGEN1_POS)
#define SYNCGEN1_1                      (SYNCGEN1_1_BITBAND << BB_ISOCHANCNTL1_SYNCGEN1_POS)

#define ISOCHANEN1_0_BITBAND            0x0
#define ISOCHANEN1_1_BITBAND            0x1
#define ISOCHANEN1_0                    (ISOCHANEN1_0_BITBAND << BB_ISOCHANCNTL1_ISOCHANEN1_POS)
#define ISOCHANEN1_1                    (ISOCHANEN1_1_BITBAND << BB_ISOCHANCNTL1_ISOCHANEN1_POS)

#define ISOTYPE1_0                      (0x0 << BB_ISOCHANCNTL1_ISOTYPE1_POS)
#define ISOTYPE1_1                      (0x1 << BB_ISOCHANCNTL1_ISOTYPE1_POS)
#define ISOTYPE1_2                      (0x2 << BB_ISOCHANCNTL1_ISOTYPE1_POS)
#define ISOTYPE1_3                      (0x3 << BB_ISOCHANCNTL1_ISOTYPE1_POS)

/* ISO Channel 1 mute control */
#define BB_ISOMUTECNTL1_BASE            0x40001674
#define BB_ISOMUTECNTL1                 REG32_POINTER(BB_ISOMUTECNTL1_BASE)

/* BB_ISOMUTECNTL1 bit positions */
#define BB_ISOMUTECNTL1_TOGO1_POS       31
#define BB_ISOMUTECNTL1_MUTE_SINK1_POS  19
#define BB_ISOMUTECNTL1_MUTE_SOURCE1_POS 18
#define BB_ISOMUTECNTL1_INVL1_1_POS     17
#define BB_ISOMUTECNTL1_INVL1_0_POS     16
#define BB_ISOMUTECNTL1_MUTE_PATTERN1_POS 0
#define BB_ISOMUTECNTL1_MUTE_PATTERN1_MASK (0xFF << BB_ISOMUTECNTL1_MUTE_PATTERN1_POS)

/* BB_ISOMUTECNTL1 bit-band aliases */
#define BB_ISOMUTECNTL1_TOGO1_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL1_BASE, BB_ISOMUTECNTL1_TOGO1_POS))
#define BB_ISOMUTECNTL1_MUTE_SINK1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL1_BASE, BB_ISOMUTECNTL1_MUTE_SINK1_POS))
#define BB_ISOMUTECNTL1_MUTE_SOURCE1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL1_BASE, BB_ISOMUTECNTL1_MUTE_SOURCE1_POS))
#define BB_ISOMUTECNTL1_INVL1_1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL1_BASE, BB_ISOMUTECNTL1_INVL1_1_POS))
#define BB_ISOMUTECNTL1_INVL1_0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL1_BASE, BB_ISOMUTECNTL1_INVL1_0_POS))

/* BB_ISOMUTECNTL1 settings */
#define TOGO1_0_BITBAND                 0x0
#define TOGO1_1_BITBAND                 0x1
#define TOGO1_0                         (TOGO1_0_BITBAND << BB_ISOMUTECNTL1_TOGO1_POS)
#define TOGO1_1                         (TOGO1_1_BITBAND << BB_ISOMUTECNTL1_TOGO1_POS)

#define MUTE_SINK1_0_BITBAND            0x0
#define MUTE_SINK1_1_BITBAND            0x1
#define MUTE_SINK1_0                    (MUTE_SINK1_0_BITBAND << BB_ISOMUTECNTL1_MUTE_SINK1_POS)
#define MUTE_SINK1_1                    (MUTE_SINK1_1_BITBAND << BB_ISOMUTECNTL1_MUTE_SINK1_POS)

#define MUTE_SOURCE1_0_BITBAND          0x0
#define MUTE_SOURCE1_1_BITBAND          0x1
#define MUTE_SOURCE1_0                  (MUTE_SOURCE1_0_BITBAND << BB_ISOMUTECNTL1_MUTE_SOURCE1_POS)
#define MUTE_SOURCE1_1                  (MUTE_SOURCE1_1_BITBAND << BB_ISOMUTECNTL1_MUTE_SOURCE1_POS)

#define INVL1_1_0_BITBAND               0x0
#define INVL1_1_1_BITBAND               0x1
#define INVL1_1_0                       (INVL1_1_0_BITBAND << BB_ISOMUTECNTL1_INVL1_1_POS)
#define INVL1_1_1                       (INVL1_1_1_BITBAND << BB_ISOMUTECNTL1_INVL1_1_POS)

#define INVL1_0_0_BITBAND               0x0
#define INVL1_0_1_BITBAND               0x1
#define INVL1_0_0                       (INVL1_0_0_BITBAND << BB_ISOMUTECNTL1_INVL1_0_POS)
#define INVL1_0_1                       (INVL1_0_1_BITBAND << BB_ISOMUTECNTL1_INVL1_0_POS)

#define MUTE_PATTERN1_0                 (0x0 << BB_ISOMUTECNTL1_MUTE_PATTERN1_POS)

/* ISO Channel 1 current Tx pointer */
#define BB_ISOCURRENTTXPTR1_BASE        0x40001678
#define BB_ISOCURRENTTXPTR1             REG32_POINTER(BB_ISOCURRENTTXPTR1_BASE)

/* BB_ISOCURRENTTXPTR1 bit positions */
#define BB_ISOCURRENTTXPTR1_ISO1TXPTR0_POS 16
#define BB_ISOCURRENTTXPTR1_ISO1TXPTR0_MASK (0xFFFF << BB_ISOCURRENTTXPTR1_ISO1TXPTR0_POS)
#define BB_ISOCURRENTTXPTR1_ISO1TXPTR1_POS 0
#define BB_ISOCURRENTTXPTR1_ISO1TXPTR1_MASK (0xFFFF << BB_ISOCURRENTTXPTR1_ISO1TXPTR1_POS)

/* BB_ISOCURRENTTXPTR1 settings */
#define ISO1TXPTR0_0                    (0x0 << BB_ISOCURRENTTXPTR1_ISO1TXPTR0_POS)

#define ISO1TXPTR1_0                    (0x0 << BB_ISOCURRENTTXPTR1_ISO1TXPTR1_POS)

/* ISO Channel 1 current Rx pointer */
#define BB_ISOCURRENTRXPTR1_BASE        0x4000167C
#define BB_ISOCURRENTRXPTR1             REG32_POINTER(BB_ISOCURRENTRXPTR1_BASE)

/* BB_ISOCURRENTRXPTR1 bit positions */
#define BB_ISOCURRENTRXPTR1_ISO1RXPTR0_POS 16
#define BB_ISOCURRENTRXPTR1_ISO1RXPTR0_MASK (0xFFFF << BB_ISOCURRENTRXPTR1_ISO1RXPTR0_POS)
#define BB_ISOCURRENTRXPTR1_ISO1RXPTR1_POS 0
#define BB_ISOCURRENTRXPTR1_ISO1RXPTR1_MASK (0xFFFF << BB_ISOCURRENTRXPTR1_ISO1RXPTR1_POS)

/* BB_ISOCURRENTRXPTR1 settings */
#define ISO1RXPTR0_0                    (0x0 << BB_ISOCURRENTRXPTR1_ISO1RXPTR0_POS)

#define ISO1RXPTR1_0                    (0x0 << BB_ISOCURRENTRXPTR1_ISO1RXPTR1_POS)

/* ISO Channel 1 payloads */
#define BB_ISOTRCNL1_BASE               0x40001680
#define BB_ISOTRCNL1                    REG32_POINTER(BB_ISOTRCNL1_BASE)

/* BB_ISOTRCNL1 bit positions */
#define BB_ISOTRCNL1_ISO1RXLEN_POS      16
#define BB_ISOTRCNL1_ISO1RXLEN_MASK     (0xFF << BB_ISOTRCNL1_ISO1RXLEN_POS)
#define BB_ISOTRCNL1_ISO1TXLEN_POS      0
#define BB_ISOTRCNL1_ISO1TXLEN_MASK     (0xFF << BB_ISOTRCNL1_ISO1TXLEN_POS)

/* BB_ISOTRCNL1 settings */
#define ISO1RXLEN_0                     (0x0 << BB_ISOTRCNL1_ISO1RXLEN_POS)

#define ISO1TXLEN_0                     (0x0 << BB_ISOTRCNL1_ISO1TXLEN_POS)

/* ISO Channel 1 mute control */
#define BB_ISOEVTCNTLOFFSETL1_BASE      0x40001684
#define BB_ISOEVTCNTLOFFSETL1           REG32_POINTER(BB_ISOEVTCNTLOFFSETL1_BASE)

/* ISO Channel 1 mute control */
#define BB_ISOEVTCNTLOFFSETU1_BASE      0x40001688
#define BB_ISOEVTCNTLOFFSETU1           REG32_POINTER(BB_ISOEVTCNTLOFFSETU1_BASE)

/* BB_ISOEVTCNTLOFFSETU1 bit positions */
#define BB_ISOEVTCNTLOFFSETU1_EVT_CNT_OFFSETU1_POS 0
#define BB_ISOEVTCNTLOFFSETU1_EVT_CNT_OFFSETU1_MASK (0x7F << BB_ISOEVTCNTLOFFSETU1_EVT_CNT_OFFSETU1_POS)

/* BB_ISOEVTCNTLOFFSETU1 settings */
#define EVT_CNT_OFFSETU1_0              (0x0 << BB_ISOEVTCNTLOFFSETU1_EVT_CNT_OFFSETU1_POS)

/* ISO Channel 2 control */
#define BB_ISOCHANCNTL2_BASE            0x40001690
#define BB_ISOCHANCNTL2                 REG32_POINTER(BB_ISOCHANCNTL2_BASE)

/* BB_ISOCHANCNTL2 bit positions */
#define BB_ISOCHANCNTL2_RETXACKEN2_POS  4
#define BB_ISOCHANCNTL2_SYNCGEN2_POS    3
#define BB_ISOCHANCNTL2_ISOCHANEN2_POS  2
#define BB_ISOCHANCNTL2_ISOTYPE2_POS    0
#define BB_ISOCHANCNTL2_ISOTYPE2_MASK   (0x3 << BB_ISOCHANCNTL2_ISOTYPE2_POS)

/* BB_ISOCHANCNTL2 bit-band aliases */
#define BB_ISOCHANCNTL2_RETXACKEN2_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL2_BASE, BB_ISOCHANCNTL2_RETXACKEN2_POS))
#define BB_ISOCHANCNTL2_SYNCGEN2_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL2_BASE, BB_ISOCHANCNTL2_SYNCGEN2_POS))
#define BB_ISOCHANCNTL2_ISOCHANEN2_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOCHANCNTL2_BASE, BB_ISOCHANCNTL2_ISOCHANEN2_POS))

/* BB_ISOCHANCNTL2 settings */
#define RETXACKEN2_0_BITBAND            0x0
#define RETXACKEN2_1_BITBAND            0x1
#define RETXACKEN2_0                    (RETXACKEN2_0_BITBAND << BB_ISOCHANCNTL2_RETXACKEN2_POS)
#define RETXACKEN2_1                    (RETXACKEN2_1_BITBAND << BB_ISOCHANCNTL2_RETXACKEN2_POS)

#define SYNCGEN2_0_BITBAND              0x0
#define SYNCGEN2_1_BITBAND              0x1
#define SYNCGEN2_0                      (SYNCGEN2_0_BITBAND << BB_ISOCHANCNTL2_SYNCGEN2_POS)
#define SYNCGEN2_1                      (SYNCGEN2_1_BITBAND << BB_ISOCHANCNTL2_SYNCGEN2_POS)

#define ISOCHANEN2_0_BITBAND            0x0
#define ISOCHANEN2_1_BITBAND            0x1
#define ISOCHANEN2_0                    (ISOCHANEN2_0_BITBAND << BB_ISOCHANCNTL2_ISOCHANEN2_POS)
#define ISOCHANEN2_1                    (ISOCHANEN2_1_BITBAND << BB_ISOCHANCNTL2_ISOCHANEN2_POS)

#define ISOTYPE2_0                      (0x0 << BB_ISOCHANCNTL2_ISOTYPE2_POS)
#define ISOTYPE2_1                      (0x1 << BB_ISOCHANCNTL2_ISOTYPE2_POS)
#define ISOTYPE2_2                      (0x2 << BB_ISOCHANCNTL2_ISOTYPE2_POS)
#define ISOTYPE2_3                      (0x3 << BB_ISOCHANCNTL2_ISOTYPE2_POS)

/* ISO Channel 2 mute control */
#define BB_ISOMUTECNTL2_BASE            0x40001694
#define BB_ISOMUTECNTL2                 REG32_POINTER(BB_ISOMUTECNTL2_BASE)

/* BB_ISOMUTECNTL2 bit positions */
#define BB_ISOMUTECNTL2_TOGO2_POS       31
#define BB_ISOMUTECNTL2_MUTE_SINK2_POS  19
#define BB_ISOMUTECNTL2_MUTE_SOURCE2_POS 18
#define BB_ISOMUTECNTL2_INVL2_1_POS     17
#define BB_ISOMUTECNTL2_INVL2_0_POS     16
#define BB_ISOMUTECNTL2_MUTE_PATTERN2_POS 0
#define BB_ISOMUTECNTL2_MUTE_PATTERN2_MASK (0xFF << BB_ISOMUTECNTL2_MUTE_PATTERN2_POS)

/* BB_ISOMUTECNTL2 bit-band aliases */
#define BB_ISOMUTECNTL2_TOGO2_BITBAND   REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL2_BASE, BB_ISOMUTECNTL2_TOGO2_POS))
#define BB_ISOMUTECNTL2_MUTE_SINK2_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL2_BASE, BB_ISOMUTECNTL2_MUTE_SINK2_POS))
#define BB_ISOMUTECNTL2_MUTE_SOURCE2_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL2_BASE, BB_ISOMUTECNTL2_MUTE_SOURCE2_POS))
#define BB_ISOMUTECNTL2_INVL2_1_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL2_BASE, BB_ISOMUTECNTL2_INVL2_1_POS))
#define BB_ISOMUTECNTL2_INVL2_0_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_ISOMUTECNTL2_BASE, BB_ISOMUTECNTL2_INVL2_0_POS))

/* BB_ISOMUTECNTL2 settings */
#define TOGO2_0_BITBAND                 0x0
#define TOGO2_1_BITBAND                 0x1
#define TOGO2_0                         (TOGO2_0_BITBAND << BB_ISOMUTECNTL2_TOGO2_POS)
#define TOGO2_1                         (TOGO2_1_BITBAND << BB_ISOMUTECNTL2_TOGO2_POS)

#define MUTE_SINK2_0_BITBAND            0x0
#define MUTE_SINK2_1_BITBAND            0x1
#define MUTE_SINK2_0                    (MUTE_SINK2_0_BITBAND << BB_ISOMUTECNTL2_MUTE_SINK2_POS)
#define MUTE_SINK2_1                    (MUTE_SINK2_1_BITBAND << BB_ISOMUTECNTL2_MUTE_SINK2_POS)

#define MUTE_SOURCE2_0_BITBAND          0x0
#define MUTE_SOURCE2_1_BITBAND          0x1
#define MUTE_SOURCE2_0                  (MUTE_SOURCE2_0_BITBAND << BB_ISOMUTECNTL2_MUTE_SOURCE2_POS)
#define MUTE_SOURCE2_1                  (MUTE_SOURCE2_1_BITBAND << BB_ISOMUTECNTL2_MUTE_SOURCE2_POS)

#define INVL2_1_0_BITBAND               0x0
#define INVL2_1_1_BITBAND               0x1
#define INVL2_1_0                       (INVL2_1_0_BITBAND << BB_ISOMUTECNTL2_INVL2_1_POS)
#define INVL2_1_1                       (INVL2_1_1_BITBAND << BB_ISOMUTECNTL2_INVL2_1_POS)

#define INVL2_0_0_BITBAND               0x0
#define INVL2_0_1_BITBAND               0x1
#define INVL2_0_0                       (INVL2_0_0_BITBAND << BB_ISOMUTECNTL2_INVL2_0_POS)
#define INVL2_0_1                       (INVL2_0_1_BITBAND << BB_ISOMUTECNTL2_INVL2_0_POS)

#define MUTE_PATTERN2_0                 (0x0 << BB_ISOMUTECNTL2_MUTE_PATTERN2_POS)

/* ISO Channel 2 current Tx pointer */
#define BB_ISOCURRENTTXPTR2_BASE        0x40001698
#define BB_ISOCURRENTTXPTR2             REG32_POINTER(BB_ISOCURRENTTXPTR2_BASE)

/* BB_ISOCURRENTTXPTR2 bit positions */
#define BB_ISOCURRENTTXPTR2_ISO2TXPTR0_POS 16
#define BB_ISOCURRENTTXPTR2_ISO2TXPTR0_MASK (0xFFFF << BB_ISOCURRENTTXPTR2_ISO2TXPTR0_POS)
#define BB_ISOCURRENTTXPTR2_ISO2TXPTR1_POS 0
#define BB_ISOCURRENTTXPTR2_ISO2TXPTR1_MASK (0xFFFF << BB_ISOCURRENTTXPTR2_ISO2TXPTR1_POS)

/* BB_ISOCURRENTTXPTR2 settings */
#define ISO2TXPTR0_0                    (0x0 << BB_ISOCURRENTTXPTR2_ISO2TXPTR0_POS)

#define ISO2TXPTR1_0                    (0x0 << BB_ISOCURRENTTXPTR2_ISO2TXPTR1_POS)

/* ISO Channel 2 current Rx pointer */
#define BB_ISOCURRENTRXPTR2_BASE        0x4000169C
#define BB_ISOCURRENTRXPTR2             REG32_POINTER(BB_ISOCURRENTRXPTR2_BASE)

/* BB_ISOCURRENTRXPTR2 bit positions */
#define BB_ISOCURRENTRXPTR2_ISO2RXPTR0_POS 16
#define BB_ISOCURRENTRXPTR2_ISO2RXPTR0_MASK (0xFFFF << BB_ISOCURRENTRXPTR2_ISO2RXPTR0_POS)
#define BB_ISOCURRENTRXPTR2_ISO2RXPTR1_POS 0
#define BB_ISOCURRENTRXPTR2_ISO2RXPTR1_MASK (0xFFFF << BB_ISOCURRENTRXPTR2_ISO2RXPTR1_POS)

/* BB_ISOCURRENTRXPTR2 settings */
#define ISO2RXPTR0_0                    (0x0 << BB_ISOCURRENTRXPTR2_ISO2RXPTR0_POS)

#define ISO2RXPTR1_0                    (0x0 << BB_ISOCURRENTRXPTR2_ISO2RXPTR1_POS)

/* ISO Channel 2 payloads */
#define BB_ISOTRCNL2_BASE               0x400016A0
#define BB_ISOTRCNL2                    REG32_POINTER(BB_ISOTRCNL2_BASE)

/* BB_ISOTRCNL2 bit positions */
#define BB_ISOTRCNL2_ISO2RXLEN_POS      16
#define BB_ISOTRCNL2_ISO2RXLEN_MASK     (0xFF << BB_ISOTRCNL2_ISO2RXLEN_POS)
#define BB_ISOTRCNL2_ISO2TXLEN_POS      0
#define BB_ISOTRCNL2_ISO2TXLEN_MASK     (0xFF << BB_ISOTRCNL2_ISO2TXLEN_POS)

/* BB_ISOTRCNL2 settings */
#define ISO2RXLEN_0                     (0x0 << BB_ISOTRCNL2_ISO2RXLEN_POS)

#define ISO2TXLEN_0                     (0x0 << BB_ISOTRCNL2_ISO2TXLEN_POS)

/* ISO Channel 2 mute control */
#define BB_ISOEVTCNTLOFFSETL2_BASE      0x400016A4
#define BB_ISOEVTCNTLOFFSETL2           REG32_POINTER(BB_ISOEVTCNTLOFFSETL2_BASE)

/* ISO Channel 2 mute control */
#define BB_ISOEVTCNTLOFFSETU2_BASE      0x400016A8
#define BB_ISOEVTCNTLOFFSETU2           REG32_POINTER(BB_ISOEVTCNTLOFFSETU2_BASE)

/* BB_ISOEVTCNTLOFFSETU2 bit positions */
#define BB_ISOEVTCNTLOFFSETU2_EVT_CNT_OFFSETU2_POS 0
#define BB_ISOEVTCNTLOFFSETU2_EVT_CNT_OFFSETU2_MASK (0x7F << BB_ISOEVTCNTLOFFSETU2_EVT_CNT_OFFSETU2_POS)

/* BB_ISOEVTCNTLOFFSETU2 settings */
#define EVT_CNT_OFFSETU2_0              (0x0 << BB_ISOEVTCNTLOFFSETU2_EVT_CNT_OFFSETU2_POS)

/* Register controlling the decision instant for priority scheduling arbitration */
#define BB_BBPRIOSCHARB_BASE            0x400016B0
#define BB_BBPRIOSCHARB                 REG32_POINTER(BB_BBPRIOSCHARB_BASE)

/* BB_BBPRIOSCHARB bit positions */
#define BB_BBPRIOSCHARB_BLEPRIOMODE_POS 15
#define BB_BBPRIOSCHARB_BLEMARGIN_POS   0
#define BB_BBPRIOSCHARB_BLEMARGIN_MASK  (0xFF << BB_BBPRIOSCHARB_BLEMARGIN_POS)

/* BB_BBPRIOSCHARB bit-band aliases */
#define BB_BBPRIOSCHARB_BLEPRIOMODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(BB_BBPRIOSCHARB_BASE, BB_BBPRIOSCHARB_BLEPRIOMODE_POS))

/* BB_BBPRIOSCHARB settings */
#define BLEPRIOMODE_0_BITBAND           0x0
#define BLEPRIOMODE_1_BITBAND           0x1
#define BLEPRIOMODE_0                   (BLEPRIOMODE_0_BITBAND << BB_BBPRIOSCHARB_BLEPRIOMODE_POS)
#define BLEPRIOMODE_1                   (BLEPRIOMODE_1_BITBAND << BB_BBPRIOSCHARB_BLEPRIOMODE_POS)

#define BLEMARGIN_0                     (0x0 << BB_BBPRIOSCHARB_BLEMARGIN_POS)

/* ----------------------------------------------------------------------------
 * RF Front-End 2.4 GHz
 * ------------------------------------------------------------------------- */

/* REG00 */
#define RF_REG00_BASE                   0x40010000
#define RF_REG00                        REG32_POINTER(RF_REG00_BASE)

/* RF_REG00 bit positions */
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_POS 31
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_POS 24
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_MASK (0x7F << RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_POS)
#define RF_REG00_FOURFSK_CODING_EN_FOURFSK_CODING_POS 23
#define RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_POS 20
#define RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_MASK (0x7 << RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_POS)
#define RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_POS 16
#define RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_MASK (0x7 << RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_POS)
#define RF_REG00_MODE2_DIFF_CODING_POS  14
#define RF_REG00_MODE2_PSK_NFSK_POS     13
#define RF_REG00_MODE2_TESTMODE_POS     8
#define RF_REG00_MODE2_TESTMODE_MASK    (0x1F << RF_REG00_MODE2_TESTMODE_POS)
#define RF_REG00_MODE_NOT_TO_IDLE_POS   7
#define RF_REG00_MODE_EN_FSM_POS        5
#define RF_REG00_MODE_EN_DESERIALIZER_POS 4
#define RF_REG00_MODE_EN_SERIALIZER_POS 3
#define RF_REG00_MODE_TX_NRX_POS        2
#define RF_REG00_MODE_MODE_POS          0
#define RF_REG00_MODE_MODE_MASK         (0x3 << RF_REG00_MODE_MODE_POS)

/* RF_REG00 bit-band aliases */
#define RF_REG00_DATAWHITE_BTLE_DW_BTLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_DATAWHITE_BTLE_DW_BTLE_POS))
#define RF_REG00_FOURFSK_CODING_EN_FOURFSK_CODING_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_FOURFSK_CODING_EN_FOURFSK_CODING_POS))
#define RF_REG00_MODE2_DIFF_CODING_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_MODE2_DIFF_CODING_POS))
#define RF_REG00_MODE2_PSK_NFSK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_MODE2_PSK_NFSK_POS))
#define RF_REG00_MODE_NOT_TO_IDLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_MODE_NOT_TO_IDLE_POS))
#define RF_REG00_MODE_EN_FSM_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_MODE_EN_FSM_POS))
#define RF_REG00_MODE_EN_DESERIALIZER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_MODE_EN_DESERIALIZER_POS))
#define RF_REG00_MODE_EN_SERIALIZER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_MODE_EN_SERIALIZER_POS))
#define RF_REG00_MODE_TX_NRX_BITBAND    REG32_POINTER(SYS_CALC_BITBAND(RF_REG00_BASE, RF_REG00_MODE_TX_NRX_POS))

/* RF_REG00 settings */
#define DATAWHITE_BTLE_DW_BTLE_DEFAULT_BITBAND 0x0
#define DATAWHITE_BTLE_DW_BTLE_DEFAULT  (DATAWHITE_BTLE_DW_BTLE_DEFAULT_BITBAND << RF_REG00_DATAWHITE_BTLE_DW_BTLE_POS)

#define DATAWHITE_BTLE_DW_BTLE_RST_DEFAULT (0x0 << RF_REG00_DATAWHITE_BTLE_DW_BTLE_RST_POS)

#define FOURFSK_CODING_EN_FOURFSK_CODING_DEFAULT_BITBAND 0x0
#define FOURFSK_CODING_EN_FOURFSK_CODING_DEFAULT (FOURFSK_CODING_EN_FOURFSK_CODING_DEFAULT_BITBAND << RF_REG00_FOURFSK_CODING_EN_FOURFSK_CODING_POS)

#define FOURFSK_CODING_TX_FOURFSK_CODING_DEFAULT (0x0 << RF_REG00_FOURFSK_CODING_TX_FOURFSK_CODING_POS)

#define FOURFSK_CODING_RX_FOURFSK_CODING_DEFAULT (0x0 << RF_REG00_FOURFSK_CODING_RX_FOURFSK_CODING_POS)

#define MODE2_DIFF_CODING_DEFAULT_BITBAND 0x0
#define MODE2_DIFF_CODING_DEFAULT       (MODE2_DIFF_CODING_DEFAULT_BITBAND << RF_REG00_MODE2_DIFF_CODING_POS)

#define MODE2_PSK_NFSK_DEFAULT_BITBAND  0x0
#define MODE2_PSK_NFSK_DEFAULT          (MODE2_PSK_NFSK_DEFAULT_BITBAND << RF_REG00_MODE2_PSK_NFSK_POS)

#define MODE2_TESTMODE_DEFAULT          (0x0 << RF_REG00_MODE2_TESTMODE_POS)

#define MODE_NOT_TO_IDLE_DEFAULT_BITBAND 0x0
#define MODE_NOT_TO_IDLE_DEFAULT        (MODE_NOT_TO_IDLE_DEFAULT_BITBAND << RF_REG00_MODE_NOT_TO_IDLE_POS)

#define MODE_EN_FSM_DEFAULT_BITBAND     0x0
#define MODE_EN_FSM_DEFAULT             (MODE_EN_FSM_DEFAULT_BITBAND << RF_REG00_MODE_EN_FSM_POS)

#define MODE_EN_DESERIALIZER_DEFAULT_BITBAND 0x0
#define MODE_EN_DESERIALIZER_DEFAULT    (MODE_EN_DESERIALIZER_DEFAULT_BITBAND << RF_REG00_MODE_EN_DESERIALIZER_POS)

#define MODE_EN_SERIALIZER_DEFAULT_BITBAND 0x0
#define MODE_EN_SERIALIZER_DEFAULT      (MODE_EN_SERIALIZER_DEFAULT_BITBAND << RF_REG00_MODE_EN_SERIALIZER_POS)

#define MODE_TX_NRX_DEFAULT_BITBAND     0x0
#define MODE_TX_NRX_DEFAULT             (MODE_TX_NRX_DEFAULT_BITBAND << RF_REG00_MODE_TX_NRX_POS)

#define MODE_MODE_DEFAULT               (0x0 << RF_REG00_MODE_MODE_POS)

/* REG01 */
#define RF_REG01_BASE                   0x40010004
#define RF_REG01                        REG32_POINTER(RF_REG01_BASE)

/* RF_REG01 bit positions */
#define RF_REG01_TAU_PHASE_RECOV_TAU_PHASE_RECOV_POS 24
#define RF_REG01_TAU_PHASE_RECOV_TAU_PHASE_RECOV_MASK (0xFF << RF_REG01_TAU_PHASE_RECOV_TAU_PHASE_RECOV_POS)
#define RF_REG01_TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_POS 16
#define RF_REG01_TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_MASK (0xFF << RF_REG01_TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_POS)
#define RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_POS 15
#define RF_REG01_CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_POS 14
#define RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_POS 13
#define RF_REG01_CARRIER_RECOVERY_AFC_NEG_POS 12
#define RF_REG01_CARRIER_RECOVERY_STARTER_MODE_POS 11
#define RF_REG01_CARRIER_RECOVERY_EN_AFC_POS 10
#define RF_REG01_CARRIER_RECOVERY_EN_FINE_RECOV_POS 9
#define RF_REG01_CARRIER_RECOVERY_EN_ROUGH_RECOV_POS 8
#define RF_REG01_MOD_TX_PULSE_NSYM_POS  6
#define RF_REG01_MOD_TX_EN_INTERP_POS   5
#define RF_REG01_MOD_TX_CK_TX_M_POS     0
#define RF_REG01_MOD_TX_CK_TX_M_MASK    (0x1F << RF_REG01_MOD_TX_CK_TX_M_POS)

/* RF_REG01 bit-band aliases */
#define RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_POS))
#define RF_REG01_CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_POS))
#define RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_POS))
#define RF_REG01_CARRIER_RECOVERY_AFC_NEG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_CARRIER_RECOVERY_AFC_NEG_POS))
#define RF_REG01_CARRIER_RECOVERY_STARTER_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_CARRIER_RECOVERY_STARTER_MODE_POS))
#define RF_REG01_CARRIER_RECOVERY_EN_AFC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_CARRIER_RECOVERY_EN_AFC_POS))
#define RF_REG01_CARRIER_RECOVERY_EN_FINE_RECOV_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_CARRIER_RECOVERY_EN_FINE_RECOV_POS))
#define RF_REG01_CARRIER_RECOVERY_EN_ROUGH_RECOV_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_CARRIER_RECOVERY_EN_ROUGH_RECOV_POS))
#define RF_REG01_MOD_TX_PULSE_NSYM_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_MOD_TX_PULSE_NSYM_POS))
#define RF_REG01_MOD_TX_EN_INTERP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG01_BASE, RF_REG01_MOD_TX_EN_INTERP_POS))

/* RF_REG01 settings */
#define TAU_PHASE_RECOV_TAU_PHASE_RECOV_DEFAULT (0x0 << RF_REG01_TAU_PHASE_RECOV_TAU_PHASE_RECOV_POS)

#define TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_DEFAULT (0x0 << RF_REG01_TAU_ROUGH_RECOV_TAU_ROUGH_RECOV_POS)

#define CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_DEFAULT (CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_AFC_POS)

#define CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_DEFAULT (CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_CORRECT_CFREQ_IF_NEG_POS)

#define CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_DEFAULT (CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_CORRECT_CFREQ_IF_POS)

#define CARRIER_RECOVERY_AFC_NEG_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_AFC_NEG_DEFAULT (CARRIER_RECOVERY_AFC_NEG_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_AFC_NEG_POS)

#define CARRIER_RECOVERY_STARTER_MODE_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_STARTER_MODE_DEFAULT (CARRIER_RECOVERY_STARTER_MODE_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_STARTER_MODE_POS)

#define CARRIER_RECOVERY_EN_AFC_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_EN_AFC_DEFAULT (CARRIER_RECOVERY_EN_AFC_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_AFC_POS)

#define CARRIER_RECOVERY_EN_FINE_RECOV_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_EN_FINE_RECOV_DEFAULT (CARRIER_RECOVERY_EN_FINE_RECOV_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_FINE_RECOV_POS)

#define CARRIER_RECOVERY_EN_ROUGH_RECOV_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_EN_ROUGH_RECOV_DEFAULT (CARRIER_RECOVERY_EN_ROUGH_RECOV_DEFAULT_BITBAND << RF_REG01_CARRIER_RECOVERY_EN_ROUGH_RECOV_POS)

#define MOD_TX_PULSE_NSYM_DEFAULT_BITBAND 0x0
#define MOD_TX_PULSE_NSYM_DEFAULT       (MOD_TX_PULSE_NSYM_DEFAULT_BITBAND << RF_REG01_MOD_TX_PULSE_NSYM_POS)

#define MOD_TX_EN_INTERP_DEFAULT_BITBAND 0x0
#define MOD_TX_EN_INTERP_DEFAULT        (MOD_TX_EN_INTERP_DEFAULT_BITBAND << RF_REG01_MOD_TX_EN_INTERP_POS)

#define MOD_TX_CK_TX_M_DEFAULT          (0x0 << RF_REG01_MOD_TX_CK_TX_M_POS)

/* REG02 */
#define RF_REG02_BASE                   0x40010008
#define RF_REG02                        REG32_POINTER(RF_REG02_BASE)

/* RF_REG02 bit positions */
#define RF_REG02_FIFO_FIFO_FLUSH_ON_OVFLW_POS 31
#define RF_REG02_FIFO_FIFO_FLUSH_ON_ADDR_ERR_POS 30
#define RF_REG02_FIFO_FIFO_FLUSH_ON_PL_ERR_POS 29
#define RF_REG02_FIFO_FIFO_FLUSH_ON_CRC_ERR_POS 28
#define RF_REG02_FIFO_RX_FIFO_ACK_POS   27
#define RF_REG02_FIFO_FIFO_THR_POS      24
#define RF_REG02_FIFO_FIFO_THR_MASK     (0x7 << RF_REG02_FIFO_FIFO_THR_POS)
#define RF_REG02_DATARATE_OFFSET_DATARATE_OFFSET_POS 16
#define RF_REG02_DATARATE_OFFSET_DATARATE_OFFSET_MASK (0xFF << RF_REG02_DATARATE_OFFSET_DATARATE_OFFSET_POS)
#define RF_REG02_TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_POS 8
#define RF_REG02_TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_MASK (0xFF << RF_REG02_TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_POS)
#define RF_REG02_TAU_CLK_RECOV_TAU_CLK_RECOV_POS 0
#define RF_REG02_TAU_CLK_RECOV_TAU_CLK_RECOV_MASK (0xFF << RF_REG02_TAU_CLK_RECOV_TAU_CLK_RECOV_POS)

/* RF_REG02 bit-band aliases */
#define RF_REG02_FIFO_FIFO_FLUSH_ON_OVFLW_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG02_BASE, RF_REG02_FIFO_FIFO_FLUSH_ON_OVFLW_POS))
#define RF_REG02_FIFO_FIFO_FLUSH_ON_ADDR_ERR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG02_BASE, RF_REG02_FIFO_FIFO_FLUSH_ON_ADDR_ERR_POS))
#define RF_REG02_FIFO_FIFO_FLUSH_ON_PL_ERR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG02_BASE, RF_REG02_FIFO_FIFO_FLUSH_ON_PL_ERR_POS))
#define RF_REG02_FIFO_FIFO_FLUSH_ON_CRC_ERR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG02_BASE, RF_REG02_FIFO_FIFO_FLUSH_ON_CRC_ERR_POS))
#define RF_REG02_FIFO_RX_FIFO_ACK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG02_BASE, RF_REG02_FIFO_RX_FIFO_ACK_POS))

/* RF_REG02 settings */
#define FIFO_FIFO_FLUSH_ON_OVFLW_DEFAULT_BITBAND 0x0
#define FIFO_FIFO_FLUSH_ON_OVFLW_DEFAULT (FIFO_FIFO_FLUSH_ON_OVFLW_DEFAULT_BITBAND << RF_REG02_FIFO_FIFO_FLUSH_ON_OVFLW_POS)

#define FIFO_FIFO_FLUSH_ON_ADDR_ERR_DEFAULT_BITBAND 0x0
#define FIFO_FIFO_FLUSH_ON_ADDR_ERR_DEFAULT (FIFO_FIFO_FLUSH_ON_ADDR_ERR_DEFAULT_BITBAND << RF_REG02_FIFO_FIFO_FLUSH_ON_ADDR_ERR_POS)

#define FIFO_FIFO_FLUSH_ON_PL_ERR_DEFAULT_BITBAND 0x0
#define FIFO_FIFO_FLUSH_ON_PL_ERR_DEFAULT (FIFO_FIFO_FLUSH_ON_PL_ERR_DEFAULT_BITBAND << RF_REG02_FIFO_FIFO_FLUSH_ON_PL_ERR_POS)

#define FIFO_FIFO_FLUSH_ON_CRC_ERR_DEFAULT_BITBAND 0x0
#define FIFO_FIFO_FLUSH_ON_CRC_ERR_DEFAULT (FIFO_FIFO_FLUSH_ON_CRC_ERR_DEFAULT_BITBAND << RF_REG02_FIFO_FIFO_FLUSH_ON_CRC_ERR_POS)

#define FIFO_RX_FIFO_ACK_DEFAULT_BITBAND 0x0
#define FIFO_RX_FIFO_ACK_DEFAULT        (FIFO_RX_FIFO_ACK_DEFAULT_BITBAND << RF_REG02_FIFO_RX_FIFO_ACK_POS)

#define FIFO_FIFO_THR_DEFAULT           (0x0 << RF_REG02_FIFO_FIFO_THR_POS)

#define DATARATE_OFFSET_DATARATE_OFFSET_DEFAULT (0x0 << RF_REG02_DATARATE_OFFSET_DATARATE_OFFSET_POS)

#define TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_DEFAULT (0x0 << RF_REG02_TAU_DATARATE_RECOV_TAU_DATARATE_RECOV_POS)

#define TAU_CLK_RECOV_TAU_CLK_RECOV_DEFAULT (0x0 << RF_REG02_TAU_CLK_RECOV_TAU_CLK_RECOV_POS)

/* REG03 */
#define RF_REG03_BASE                   0x4001000C
#define RF_REG03                        REG32_POINTER(RF_REG03_BASE)

/* RF_REG03 bit positions */
#define RF_REG03_PAD_CONF_2_PAD_3_CONF_POS 28
#define RF_REG03_PAD_CONF_2_PAD_3_CONF_MASK (0xF << RF_REG03_PAD_CONF_2_PAD_3_CONF_POS)
#define RF_REG03_PAD_CONF_2_PAD_2_CONF_POS 24
#define RF_REG03_PAD_CONF_2_PAD_2_CONF_MASK (0xF << RF_REG03_PAD_CONF_2_PAD_2_CONF_POS)
#define RF_REG03_PAD_CONF_1_PAD_1_CONF_POS 20
#define RF_REG03_PAD_CONF_1_PAD_1_CONF_MASK (0xF << RF_REG03_PAD_CONF_1_PAD_1_CONF_POS)
#define RF_REG03_PAD_CONF_1_PAD_0_CONF_POS 16
#define RF_REG03_PAD_CONF_1_PAD_0_CONF_MASK (0xF << RF_REG03_PAD_CONF_1_PAD_0_CONF_POS)
#define RF_REG03_IRQ_CONF_IRQ_HIGH_Z_POS 15
#define RF_REG03_IRQ_CONF_IRQ_ACTIVE_LOW_POS 14
#define RF_REG03_IRQ_CONF_IRQS_MASK_POS 8
#define RF_REG03_IRQ_CONF_IRQS_MASK_MASK (0x3F << RF_REG03_IRQ_CONF_IRQS_MASK_POS)
#define RF_REG03_FIFO_2_FIFO_THR_TX_POS 5
#define RF_REG03_FIFO_2_FIFO_THR_TX_MASK (0x7 << RF_REG03_FIFO_2_FIFO_THR_TX_POS)
#define RF_REG03_FIFO_2_WAIT_TXFIFO_WR_POS 4
#define RF_REG03_FIFO_2_STOP_ON_RXFF_OVFLW_POS 3
#define RF_REG03_FIFO_2_STOP_ON_TXFF_UNFLW_POS 2
#define RF_REG03_FIFO_2_RXFF_FLUSH_ON_START_POS 1
#define RF_REG03_FIFO_2_TXFF_FLUSH_ON_STOP_POS 0

/* RF_REG03 bit-band aliases */
#define RF_REG03_IRQ_CONF_IRQ_HIGH_Z_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG03_BASE, RF_REG03_IRQ_CONF_IRQ_HIGH_Z_POS))
#define RF_REG03_IRQ_CONF_IRQ_ACTIVE_LOW_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG03_BASE, RF_REG03_IRQ_CONF_IRQ_ACTIVE_LOW_POS))
#define RF_REG03_FIFO_2_WAIT_TXFIFO_WR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG03_BASE, RF_REG03_FIFO_2_WAIT_TXFIFO_WR_POS))
#define RF_REG03_FIFO_2_STOP_ON_RXFF_OVFLW_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG03_BASE, RF_REG03_FIFO_2_STOP_ON_RXFF_OVFLW_POS))
#define RF_REG03_FIFO_2_STOP_ON_TXFF_UNFLW_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG03_BASE, RF_REG03_FIFO_2_STOP_ON_TXFF_UNFLW_POS))
#define RF_REG03_FIFO_2_RXFF_FLUSH_ON_START_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG03_BASE, RF_REG03_FIFO_2_RXFF_FLUSH_ON_START_POS))
#define RF_REG03_FIFO_2_TXFF_FLUSH_ON_STOP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG03_BASE, RF_REG03_FIFO_2_TXFF_FLUSH_ON_STOP_POS))

/* RF_REG03 settings */
#define PAD_CONF_2_PAD_3_CONF_DEFAULT   (0x0 << RF_REG03_PAD_CONF_2_PAD_3_CONF_POS)

#define PAD_CONF_2_PAD_2_CONF_DEFAULT   (0x0 << RF_REG03_PAD_CONF_2_PAD_2_CONF_POS)

#define PAD_CONF_1_PAD_1_CONF_DEFAULT   (0x0 << RF_REG03_PAD_CONF_1_PAD_1_CONF_POS)

#define PAD_CONF_1_PAD_0_CONF_DEFAULT   (0x0 << RF_REG03_PAD_CONF_1_PAD_0_CONF_POS)

#define IRQ_CONF_IRQ_HIGH_Z_DEFAULT_BITBAND 0x0
#define IRQ_CONF_IRQ_HIGH_Z_DEFAULT     (IRQ_CONF_IRQ_HIGH_Z_DEFAULT_BITBAND << RF_REG03_IRQ_CONF_IRQ_HIGH_Z_POS)

#define IRQ_CONF_IRQ_ACTIVE_LOW_DEFAULT_BITBAND 0x0
#define IRQ_CONF_IRQ_ACTIVE_LOW_DEFAULT (IRQ_CONF_IRQ_ACTIVE_LOW_DEFAULT_BITBAND << RF_REG03_IRQ_CONF_IRQ_ACTIVE_LOW_POS)

#define IRQ_CONF_IRQS_MASK_DEFAULT      (0x0 << RF_REG03_IRQ_CONF_IRQS_MASK_POS)

#define FIFO_2_FIFO_THR_TX_DEFAULT      (0x0 << RF_REG03_FIFO_2_FIFO_THR_TX_POS)

#define FIFO_2_WAIT_TXFIFO_WR_DEFAULT_BITBAND 0x0
#define FIFO_2_WAIT_TXFIFO_WR_DEFAULT   (FIFO_2_WAIT_TXFIFO_WR_DEFAULT_BITBAND << RF_REG03_FIFO_2_WAIT_TXFIFO_WR_POS)

#define FIFO_2_STOP_ON_RXFF_OVFLW_DEFAULT_BITBAND 0x0
#define FIFO_2_STOP_ON_RXFF_OVFLW_DEFAULT (FIFO_2_STOP_ON_RXFF_OVFLW_DEFAULT_BITBAND << RF_REG03_FIFO_2_STOP_ON_RXFF_OVFLW_POS)

#define FIFO_2_STOP_ON_TXFF_UNFLW_DEFAULT_BITBAND 0x0
#define FIFO_2_STOP_ON_TXFF_UNFLW_DEFAULT (FIFO_2_STOP_ON_TXFF_UNFLW_DEFAULT_BITBAND << RF_REG03_FIFO_2_STOP_ON_TXFF_UNFLW_POS)

#define FIFO_2_RXFF_FLUSH_ON_START_DEFAULT_BITBAND 0x0
#define FIFO_2_RXFF_FLUSH_ON_START_DEFAULT (FIFO_2_RXFF_FLUSH_ON_START_DEFAULT_BITBAND << RF_REG03_FIFO_2_RXFF_FLUSH_ON_START_POS)

#define FIFO_2_TXFF_FLUSH_ON_STOP_DEFAULT_BITBAND 0x0
#define FIFO_2_TXFF_FLUSH_ON_STOP_DEFAULT (FIFO_2_TXFF_FLUSH_ON_STOP_DEFAULT_BITBAND << RF_REG03_FIFO_2_TXFF_FLUSH_ON_STOP_POS)

/* REG04 */
#define RF_REG04_BASE                   0x40010010
#define RF_REG04                        REG32_POINTER(RF_REG04_BASE)

/* RF_REG04 bit positions */
#define RF_REG04_MAC_CONF_MAC_TIMER_GR_POS 30
#define RF_REG04_MAC_CONF_MAC_TIMER_GR_MASK (0x3 << RF_REG04_MAC_CONF_MAC_TIMER_GR_POS)
#define RF_REG04_MAC_CONF_RX_MAC_ACT_POS 29
#define RF_REG04_MAC_CONF_RX_MAC_TX_NRX_POS 28
#define RF_REG04_MAC_CONF_RX_MAC_START_NSTOP_POS 27
#define RF_REG04_MAC_CONF_TX_MAC_ACT_POS 26
#define RF_REG04_MAC_CONF_TX_MAC_TX_NRX_POS 25
#define RF_REG04_MAC_CONF_TX_MAC_START_NSTOP_POS 24
#define RF_REG04_PAD_CONF_5_PAD_9_CONF_POS 20
#define RF_REG04_PAD_CONF_5_PAD_9_CONF_MASK (0xF << RF_REG04_PAD_CONF_5_PAD_9_CONF_POS)
#define RF_REG04_PAD_CONF_5_PAD_8_CONF_POS 16
#define RF_REG04_PAD_CONF_5_PAD_8_CONF_MASK (0xF << RF_REG04_PAD_CONF_5_PAD_8_CONF_POS)
#define RF_REG04_PAD_CONF_4_PAD_7_CONF_POS 12
#define RF_REG04_PAD_CONF_4_PAD_7_CONF_MASK (0xF << RF_REG04_PAD_CONF_4_PAD_7_CONF_POS)
#define RF_REG04_PAD_CONF_4_PAD_6_CONF_POS 8
#define RF_REG04_PAD_CONF_4_PAD_6_CONF_MASK (0xF << RF_REG04_PAD_CONF_4_PAD_6_CONF_POS)
#define RF_REG04_PAD_CONF_3_PAD_5_CONF_POS 4
#define RF_REG04_PAD_CONF_3_PAD_5_CONF_MASK (0xF << RF_REG04_PAD_CONF_3_PAD_5_CONF_POS)
#define RF_REG04_PAD_CONF_3_PAD_4_CONF_POS 0
#define RF_REG04_PAD_CONF_3_PAD_4_CONF_MASK (0xF << RF_REG04_PAD_CONF_3_PAD_4_CONF_POS)

/* RF_REG04 bit-band aliases */
#define RF_REG04_MAC_CONF_RX_MAC_ACT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG04_BASE, RF_REG04_MAC_CONF_RX_MAC_ACT_POS))
#define RF_REG04_MAC_CONF_RX_MAC_TX_NRX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG04_BASE, RF_REG04_MAC_CONF_RX_MAC_TX_NRX_POS))
#define RF_REG04_MAC_CONF_RX_MAC_START_NSTOP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG04_BASE, RF_REG04_MAC_CONF_RX_MAC_START_NSTOP_POS))
#define RF_REG04_MAC_CONF_TX_MAC_ACT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG04_BASE, RF_REG04_MAC_CONF_TX_MAC_ACT_POS))
#define RF_REG04_MAC_CONF_TX_MAC_TX_NRX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG04_BASE, RF_REG04_MAC_CONF_TX_MAC_TX_NRX_POS))
#define RF_REG04_MAC_CONF_TX_MAC_START_NSTOP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG04_BASE, RF_REG04_MAC_CONF_TX_MAC_START_NSTOP_POS))

/* RF_REG04 settings */
#define MAC_CONF_MAC_TIMER_GR_DEFAULT   (0x0 << RF_REG04_MAC_CONF_MAC_TIMER_GR_POS)

#define MAC_CONF_RX_MAC_ACT_DEFAULT_BITBAND 0x0
#define MAC_CONF_RX_MAC_ACT_DEFAULT     (MAC_CONF_RX_MAC_ACT_DEFAULT_BITBAND << RF_REG04_MAC_CONF_RX_MAC_ACT_POS)

#define MAC_CONF_RX_MAC_TX_NRX_DEFAULT_BITBAND 0x0
#define MAC_CONF_RX_MAC_TX_NRX_DEFAULT  (MAC_CONF_RX_MAC_TX_NRX_DEFAULT_BITBAND << RF_REG04_MAC_CONF_RX_MAC_TX_NRX_POS)

#define MAC_CONF_RX_MAC_START_NSTOP_DEFAULT_BITBAND 0x0
#define MAC_CONF_RX_MAC_START_NSTOP_DEFAULT (MAC_CONF_RX_MAC_START_NSTOP_DEFAULT_BITBAND << RF_REG04_MAC_CONF_RX_MAC_START_NSTOP_POS)

#define MAC_CONF_TX_MAC_ACT_DEFAULT_BITBAND 0x0
#define MAC_CONF_TX_MAC_ACT_DEFAULT     (MAC_CONF_TX_MAC_ACT_DEFAULT_BITBAND << RF_REG04_MAC_CONF_TX_MAC_ACT_POS)

#define MAC_CONF_TX_MAC_TX_NRX_DEFAULT_BITBAND 0x0
#define MAC_CONF_TX_MAC_TX_NRX_DEFAULT  (MAC_CONF_TX_MAC_TX_NRX_DEFAULT_BITBAND << RF_REG04_MAC_CONF_TX_MAC_TX_NRX_POS)

#define MAC_CONF_TX_MAC_START_NSTOP_DEFAULT_BITBAND 0x0
#define MAC_CONF_TX_MAC_START_NSTOP_DEFAULT (MAC_CONF_TX_MAC_START_NSTOP_DEFAULT_BITBAND << RF_REG04_MAC_CONF_TX_MAC_START_NSTOP_POS)

#define PAD_CONF_5_PAD_9_CONF_DEFAULT   (0x0 << RF_REG04_PAD_CONF_5_PAD_9_CONF_POS)

#define PAD_CONF_5_PAD_8_CONF_DEFAULT   (0x0 << RF_REG04_PAD_CONF_5_PAD_8_CONF_POS)

#define PAD_CONF_4_PAD_7_CONF_DEFAULT   (0x0 << RF_REG04_PAD_CONF_4_PAD_7_CONF_POS)

#define PAD_CONF_4_PAD_6_CONF_DEFAULT   (0x0 << RF_REG04_PAD_CONF_4_PAD_6_CONF_POS)

#define PAD_CONF_3_PAD_5_CONF_DEFAULT   (0x0 << RF_REG04_PAD_CONF_3_PAD_5_CONF_POS)

#define PAD_CONF_3_PAD_4_CONF_DEFAULT   (0x0 << RF_REG04_PAD_CONF_3_PAD_4_CONF_POS)

/* REG05 */
#define RF_REG05_BASE                   0x40010014
#define RF_REG05                        REG32_POINTER(RF_REG05_BASE)

/* RF_REG05 bit positions */
#define RF_REG05_CHANNEL_SWITCH_IQ_POS  30
#define RF_REG05_CHANNEL_CHANNEL_POS    24
#define RF_REG05_CHANNEL_CHANNEL_MASK   (0x3F << RF_REG05_CHANNEL_CHANNEL_POS)
#define RF_REG05_BANK_DATARATE_TX_NRX_POS 18
#define RF_REG05_BANK_BANK_POS          16
#define RF_REG05_BANK_BANK_MASK         (0x3 << RF_REG05_BANK_BANK_POS)
#define RF_REG05_TX_MAC_TIMER_TX_MAC_TIMER_POS 8
#define RF_REG05_TX_MAC_TIMER_TX_MAC_TIMER_MASK (0xFF << RF_REG05_TX_MAC_TIMER_TX_MAC_TIMER_POS)
#define RF_REG05_RX_MAC_TIMER_RX_MAC_TIMER_POS 0
#define RF_REG05_RX_MAC_TIMER_RX_MAC_TIMER_MASK (0xFF << RF_REG05_RX_MAC_TIMER_RX_MAC_TIMER_POS)

/* RF_REG05 bit-band aliases */
#define RF_REG05_CHANNEL_SWITCH_IQ_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG05_BASE, RF_REG05_CHANNEL_SWITCH_IQ_POS))
#define RF_REG05_BANK_DATARATE_TX_NRX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG05_BASE, RF_REG05_BANK_DATARATE_TX_NRX_POS))

/* RF_REG05 settings */
#define CHANNEL_SWITCH_IQ_DEFAULT_BITBAND 0x0
#define CHANNEL_SWITCH_IQ_DEFAULT       (CHANNEL_SWITCH_IQ_DEFAULT_BITBAND << RF_REG05_CHANNEL_SWITCH_IQ_POS)

#define CHANNEL_CHANNEL_DEFAULT         (0x0 << RF_REG05_CHANNEL_CHANNEL_POS)

#define BANK_DATARATE_TX_NRX_DEFAULT_BITBAND 0x0
#define BANK_DATARATE_TX_NRX_DEFAULT    (BANK_DATARATE_TX_NRX_DEFAULT_BITBAND << RF_REG05_BANK_DATARATE_TX_NRX_POS)

#define BANK_BANK_DEFAULT               (0x0 << RF_REG05_BANK_BANK_POS)

#define TX_MAC_TIMER_TX_MAC_TIMER_DEFAULT (0x0 << RF_REG05_TX_MAC_TIMER_TX_MAC_TIMER_POS)

#define RX_MAC_TIMER_RX_MAC_TIMER_DEFAULT (0x0 << RF_REG05_RX_MAC_TIMER_RX_MAC_TIMER_POS)

/* CENTER_FREQ */
#define RF_CENTER_FREQ_BASE             0x40010018
#define RF_CENTER_FREQ                  REG32_POINTER(RF_CENTER_FREQ_BASE)

/* RF_CENTER_FREQ bit positions */
#define RF_CENTER_FREQ_CENTER_FREQ_ADAPT_CFREQ_POS 31
#define RF_CENTER_FREQ_CENTER_FREQ_RX_DIV_5_N6_POS 30
#define RF_CENTER_FREQ_CENTER_FREQ_CENTER_FREQUENCY_POS 0
#define RF_CENTER_FREQ_CENTER_FREQ_CENTER_FREQUENCY_MASK (0x3FFFFFFF << RF_CENTER_FREQ_CENTER_FREQ_CENTER_FREQUENCY_POS)

/* RF_CENTER_FREQ bit-band aliases */
#define RF_CENTER_FREQ_CENTER_FREQ_ADAPT_CFREQ_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_CENTER_FREQ_BASE, RF_CENTER_FREQ_CENTER_FREQ_ADAPT_CFREQ_POS))
#define RF_CENTER_FREQ_CENTER_FREQ_RX_DIV_5_N6_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_CENTER_FREQ_BASE, RF_CENTER_FREQ_CENTER_FREQ_RX_DIV_5_N6_POS))

/* RF_CENTER_FREQ settings */
#define CENTER_FREQ_ADAPT_CFREQ_DEFAULT_BITBAND 0x0
#define CENTER_FREQ_ADAPT_CFREQ_DEFAULT (CENTER_FREQ_ADAPT_CFREQ_DEFAULT_BITBAND << RF_CENTER_FREQ_CENTER_FREQ_ADAPT_CFREQ_POS)

#define CENTER_FREQ_RX_DIV_5_N6_DEFAULT_BITBAND 0x0
#define CENTER_FREQ_RX_DIV_5_N6_DEFAULT (CENTER_FREQ_RX_DIV_5_N6_DEFAULT_BITBAND << RF_CENTER_FREQ_CENTER_FREQ_RX_DIV_5_N6_POS)

#define CENTER_FREQ_CENTER_FREQUENCY_DEFAULT (0x0 << RF_CENTER_FREQ_CENTER_FREQ_CENTER_FREQUENCY_POS)

/* REG07 */
#define RF_REG07_BASE                   0x4001001C
#define RF_REG07                        REG32_POINTER(RF_REG07_BASE)

/* RF_REG07 bit positions */
#define RF_REG07_CHANNELS_1_CHANNEL_SPACING_LO_POS 16
#define RF_REG07_CHANNELS_1_CHANNEL_SPACING_LO_MASK (0xFFFF << RF_REG07_CHANNELS_1_CHANNEL_SPACING_LO_POS)
#define RF_REG07_MOD_INFO_RX_EN_DIV_2_N3_RX_POS 14
#define RF_REG07_MOD_INFO_RX_SYMBOL_2BIT_RX_POS 13
#define RF_REG07_MOD_INFO_RX_DR_M_RX_POS 8
#define RF_REG07_MOD_INFO_RX_DR_M_RX_MASK (0x1F << RF_REG07_MOD_INFO_RX_DR_M_RX_POS)
#define RF_REG07_MOD_INFO_TX_EN_DIV_2_N3_TX_POS 6
#define RF_REG07_MOD_INFO_TX_SYMBOL_2BIT_TX_POS 5
#define RF_REG07_MOD_INFO_TX_DR_M_TX_POS 0
#define RF_REG07_MOD_INFO_TX_DR_M_TX_MASK (0x1F << RF_REG07_MOD_INFO_TX_DR_M_TX_POS)

/* RF_REG07 bit-band aliases */
#define RF_REG07_MOD_INFO_RX_EN_DIV_2_N3_RX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG07_BASE, RF_REG07_MOD_INFO_RX_EN_DIV_2_N3_RX_POS))
#define RF_REG07_MOD_INFO_RX_SYMBOL_2BIT_RX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG07_BASE, RF_REG07_MOD_INFO_RX_SYMBOL_2BIT_RX_POS))
#define RF_REG07_MOD_INFO_TX_EN_DIV_2_N3_TX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG07_BASE, RF_REG07_MOD_INFO_TX_EN_DIV_2_N3_TX_POS))
#define RF_REG07_MOD_INFO_TX_SYMBOL_2BIT_TX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG07_BASE, RF_REG07_MOD_INFO_TX_SYMBOL_2BIT_TX_POS))

/* RF_REG07 settings */
#define CHANNELS_1_CHANNEL_SPACING_LO_DEFAULT (0x0 << RF_REG07_CHANNELS_1_CHANNEL_SPACING_LO_POS)

#define MOD_INFO_RX_EN_DIV_2_N3_RX_DEFAULT_BITBAND 0x0
#define MOD_INFO_RX_EN_DIV_2_N3_RX_DEFAULT (MOD_INFO_RX_EN_DIV_2_N3_RX_DEFAULT_BITBAND << RF_REG07_MOD_INFO_RX_EN_DIV_2_N3_RX_POS)

#define MOD_INFO_RX_SYMBOL_2BIT_RX_DEFAULT_BITBAND 0x0
#define MOD_INFO_RX_SYMBOL_2BIT_RX_DEFAULT (MOD_INFO_RX_SYMBOL_2BIT_RX_DEFAULT_BITBAND << RF_REG07_MOD_INFO_RX_SYMBOL_2BIT_RX_POS)

#define MOD_INFO_RX_DR_M_RX_DEFAULT     (0x0 << RF_REG07_MOD_INFO_RX_DR_M_RX_POS)

#define MOD_INFO_TX_EN_DIV_2_N3_TX_DEFAULT_BITBAND 0x0
#define MOD_INFO_TX_EN_DIV_2_N3_TX_DEFAULT (MOD_INFO_TX_EN_DIV_2_N3_TX_DEFAULT_BITBAND << RF_REG07_MOD_INFO_TX_EN_DIV_2_N3_TX_POS)

#define MOD_INFO_TX_SYMBOL_2BIT_TX_DEFAULT_BITBAND 0x0
#define MOD_INFO_TX_SYMBOL_2BIT_TX_DEFAULT (MOD_INFO_TX_SYMBOL_2BIT_TX_DEFAULT_BITBAND << RF_REG07_MOD_INFO_TX_SYMBOL_2BIT_TX_POS)

#define MOD_INFO_TX_DR_M_TX_DEFAULT     (0x0 << RF_REG07_MOD_INFO_TX_DR_M_TX_POS)

/* REG08 */
#define RF_REG08_BASE                   0x40010020
#define RF_REG08                        REG32_POINTER(RF_REG08_BASE)

/* RF_REG08 bit positions */
#define RF_REG08_PACKET_LENGTH_PACKET_LEN_POS 24
#define RF_REG08_PACKET_LENGTH_PACKET_LEN_MASK (0xFF << RF_REG08_PACKET_LENGTH_PACKET_LEN_POS)
#define RF_REG08_PACKET_HANDLING_LSB_FIRST_POS 23
#define RF_REG08_PACKET_HANDLING_EN_CRC_POS 22
#define RF_REG08_PACKET_HANDLING_EN_CRC_ON_PKTLEN_POS 21
#define RF_REG08_PACKET_HANDLING_EN_PREAMBLE_POS 20
#define RF_REG08_PACKET_HANDLING_EN_MULTI_FRAME_POS 19
#define RF_REG08_PACKET_HANDLING_ENB_DW_ON_CRC_POS 18
#define RF_REG08_PACKET_HANDLING_EN_PATTERN_POS 17
#define RF_REG08_PACKET_HANDLING_EN_PACKET_POS 16
#define RF_REG08_CODING_EN_DATAWHITE_POS 15
#define RF_REG08_CODING_I_NQ_DELAYED_POS 14
#define RF_REG08_CODING_OFFSET_POS      13
#define RF_REG08_CODING_BIT_INVERT_POS  12
#define RF_REG08_CODING_EVEN_BEFORE_ODD_POS 11
#define RF_REG08_CODING_EN_802154_L2F_POS 10
#define RF_REG08_CODING_EN_802154_B2C_POS 9
#define RF_REG08_CODING_EN_MANCHESTER_POS 8
#define RF_REG08_CHANNELS_2_EN_CHANNEL_SEL_POS 7
#define RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_POS 0
#define RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_MASK (0xF << RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_POS)

/* RF_REG08 bit-band aliases */
#define RF_REG08_PACKET_HANDLING_LSB_FIRST_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_PACKET_HANDLING_LSB_FIRST_POS))
#define RF_REG08_PACKET_HANDLING_EN_CRC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_PACKET_HANDLING_EN_CRC_POS))
#define RF_REG08_PACKET_HANDLING_EN_CRC_ON_PKTLEN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_PACKET_HANDLING_EN_CRC_ON_PKTLEN_POS))
#define RF_REG08_PACKET_HANDLING_EN_PREAMBLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_PACKET_HANDLING_EN_PREAMBLE_POS))
#define RF_REG08_PACKET_HANDLING_EN_MULTI_FRAME_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_PACKET_HANDLING_EN_MULTI_FRAME_POS))
#define RF_REG08_PACKET_HANDLING_ENB_DW_ON_CRC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_PACKET_HANDLING_ENB_DW_ON_CRC_POS))
#define RF_REG08_PACKET_HANDLING_EN_PATTERN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_PACKET_HANDLING_EN_PATTERN_POS))
#define RF_REG08_PACKET_HANDLING_EN_PACKET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_PACKET_HANDLING_EN_PACKET_POS))
#define RF_REG08_CODING_EN_DATAWHITE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CODING_EN_DATAWHITE_POS))
#define RF_REG08_CODING_I_NQ_DELAYED_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CODING_I_NQ_DELAYED_POS))
#define RF_REG08_CODING_OFFSET_BITBAND  REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CODING_OFFSET_POS))
#define RF_REG08_CODING_BIT_INVERT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CODING_BIT_INVERT_POS))
#define RF_REG08_CODING_EVEN_BEFORE_ODD_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CODING_EVEN_BEFORE_ODD_POS))
#define RF_REG08_CODING_EN_802154_L2F_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CODING_EN_802154_L2F_POS))
#define RF_REG08_CODING_EN_802154_B2C_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CODING_EN_802154_B2C_POS))
#define RF_REG08_CODING_EN_MANCHESTER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CODING_EN_MANCHESTER_POS))
#define RF_REG08_CHANNELS_2_EN_CHANNEL_SEL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG08_BASE, RF_REG08_CHANNELS_2_EN_CHANNEL_SEL_POS))

/* RF_REG08 settings */
#define PACKET_LENGTH_PACKET_LEN_DEFAULT (0xFF << RF_REG08_PACKET_LENGTH_PACKET_LEN_POS)

#define PACKET_HANDLING_LSB_FIRST_DEFAULT_BITBAND 0x0
#define PACKET_HANDLING_LSB_FIRST_DEFAULT (PACKET_HANDLING_LSB_FIRST_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_LSB_FIRST_POS)

#define PACKET_HANDLING_EN_CRC_DEFAULT_BITBAND 0x0
#define PACKET_HANDLING_EN_CRC_DEFAULT  (PACKET_HANDLING_EN_CRC_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_CRC_POS)

#define PACKET_HANDLING_EN_CRC_ON_PKTLEN_DEFAULT_BITBAND 0x0
#define PACKET_HANDLING_EN_CRC_ON_PKTLEN_DEFAULT (PACKET_HANDLING_EN_CRC_ON_PKTLEN_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_CRC_ON_PKTLEN_POS)

#define PACKET_HANDLING_EN_PREAMBLE_DEFAULT_BITBAND 0x0
#define PACKET_HANDLING_EN_PREAMBLE_DEFAULT (PACKET_HANDLING_EN_PREAMBLE_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_PREAMBLE_POS)

#define PACKET_HANDLING_EN_MULTI_FRAME_DEFAULT_BITBAND 0x0
#define PACKET_HANDLING_EN_MULTI_FRAME_DEFAULT (PACKET_HANDLING_EN_MULTI_FRAME_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_MULTI_FRAME_POS)

#define PACKET_HANDLING_ENB_DW_ON_CRC_DEFAULT_BITBAND 0x0
#define PACKET_HANDLING_ENB_DW_ON_CRC_DEFAULT (PACKET_HANDLING_ENB_DW_ON_CRC_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_ENB_DW_ON_CRC_POS)

#define PACKET_HANDLING_EN_PATTERN_DEFAULT_BITBAND 0x0
#define PACKET_HANDLING_EN_PATTERN_DEFAULT (PACKET_HANDLING_EN_PATTERN_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_PATTERN_POS)

#define PACKET_HANDLING_EN_PACKET_DEFAULT_BITBAND 0x0
#define PACKET_HANDLING_EN_PACKET_DEFAULT (PACKET_HANDLING_EN_PACKET_DEFAULT_BITBAND << RF_REG08_PACKET_HANDLING_EN_PACKET_POS)

#define CODING_EN_DATAWHITE_DEFAULT_BITBAND 0x0
#define CODING_EN_DATAWHITE_DEFAULT     (CODING_EN_DATAWHITE_DEFAULT_BITBAND << RF_REG08_CODING_EN_DATAWHITE_POS)

#define CODING_I_NQ_DELAYED_DEFAULT_BITBAND 0x0
#define CODING_I_NQ_DELAYED_DEFAULT     (CODING_I_NQ_DELAYED_DEFAULT_BITBAND << RF_REG08_CODING_I_NQ_DELAYED_POS)

#define CODING_OFFSET_DEFAULT_BITBAND   0x0
#define CODING_OFFSET_DEFAULT           (CODING_OFFSET_DEFAULT_BITBAND << RF_REG08_CODING_OFFSET_POS)

#define CODING_BIT_INVERT_DEFAULT_BITBAND 0x0
#define CODING_BIT_INVERT_DEFAULT       (CODING_BIT_INVERT_DEFAULT_BITBAND << RF_REG08_CODING_BIT_INVERT_POS)

#define CODING_EVEN_BEFORE_ODD_DEFAULT_BITBAND 0x0
#define CODING_EVEN_BEFORE_ODD_DEFAULT  (CODING_EVEN_BEFORE_ODD_DEFAULT_BITBAND << RF_REG08_CODING_EVEN_BEFORE_ODD_POS)

#define CODING_EN_802154_L2F_DEFAULT_BITBAND 0x0
#define CODING_EN_802154_L2F_DEFAULT    (CODING_EN_802154_L2F_DEFAULT_BITBAND << RF_REG08_CODING_EN_802154_L2F_POS)

#define CODING_EN_802154_B2C_DEFAULT_BITBAND 0x0
#define CODING_EN_802154_B2C_DEFAULT    (CODING_EN_802154_B2C_DEFAULT_BITBAND << RF_REG08_CODING_EN_802154_B2C_POS)

#define CODING_EN_MANCHESTER_DEFAULT_BITBAND 0x0
#define CODING_EN_MANCHESTER_DEFAULT    (CODING_EN_MANCHESTER_DEFAULT_BITBAND << RF_REG08_CODING_EN_MANCHESTER_POS)

#define CHANNELS_2_EN_CHANNEL_SEL_DEFAULT_BITBAND 0x0
#define CHANNELS_2_EN_CHANNEL_SEL_DEFAULT (CHANNELS_2_EN_CHANNEL_SEL_DEFAULT_BITBAND << RF_REG08_CHANNELS_2_EN_CHANNEL_SEL_POS)

#define CHANNELS_2_CHANNEL_SPACING_HI_DEFAULT (0x0 << RF_REG08_CHANNELS_2_CHANNEL_SPACING_HI_POS)

/* REG09 */
#define RF_REG09_BASE                   0x40010024
#define RF_REG09                        REG32_POINTER(RF_REG09_BASE)

/* RF_REG09 bit positions */
#define RF_REG09_ADDRESS_CONF_ADDRESS_LEN_POS 27
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BR_POS 26
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_POS 25
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_TX_POS 24
#define RF_REG09_PREAMBLE_LENGTH_PREAMBLE_LEN_POS 16
#define RF_REG09_PREAMBLE_LENGTH_PREAMBLE_LEN_MASK (0xFF << RF_REG09_PREAMBLE_LENGTH_PREAMBLE_LEN_POS)
#define RF_REG09_PREAMBLE_PREAMBLE_POS  8
#define RF_REG09_PREAMBLE_PREAMBLE_MASK (0xFF << RF_REG09_PREAMBLE_PREAMBLE_POS)
#define RF_REG09_PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_POS 6
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_POS 2
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_MASK (0xF << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_POS)
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_POS 0
#define RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_MASK (0x3 << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_POS)

/* RF_REG09 bit-band aliases */
#define RF_REG09_ADDRESS_CONF_ADDRESS_LEN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG09_BASE, RF_REG09_ADDRESS_CONF_ADDRESS_LEN_POS))
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG09_BASE, RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BR_POS))
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG09_BASE, RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_POS))
#define RF_REG09_ADDRESS_CONF_EN_ADDRESS_TX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG09_BASE, RF_REG09_ADDRESS_CONF_EN_ADDRESS_TX_POS))
#define RF_REG09_PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG09_BASE, RF_REG09_PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_POS))

/* RF_REG09 settings */
#define ADDRESS_CONF_ADDRESS_LEN_DEFAULT_BITBAND 0x0
#define ADDRESS_CONF_ADDRESS_LEN_DEFAULT (ADDRESS_CONF_ADDRESS_LEN_DEFAULT_BITBAND << RF_REG09_ADDRESS_CONF_ADDRESS_LEN_POS)

#define ADDRESS_CONF_EN_ADDRESS_RX_BR_DEFAULT_BITBAND 0x0
#define ADDRESS_CONF_EN_ADDRESS_RX_BR_DEFAULT (ADDRESS_CONF_EN_ADDRESS_RX_BR_DEFAULT_BITBAND << RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_BR_POS)

#define ADDRESS_CONF_EN_ADDRESS_RX_DEFAULT_BITBAND 0x0
#define ADDRESS_CONF_EN_ADDRESS_RX_DEFAULT (ADDRESS_CONF_EN_ADDRESS_RX_DEFAULT_BITBAND << RF_REG09_ADDRESS_CONF_EN_ADDRESS_RX_POS)

#define ADDRESS_CONF_EN_ADDRESS_TX_DEFAULT_BITBAND 0x0
#define ADDRESS_CONF_EN_ADDRESS_TX_DEFAULT (ADDRESS_CONF_EN_ADDRESS_TX_DEFAULT_BITBAND << RF_REG09_ADDRESS_CONF_EN_ADDRESS_TX_POS)

#define PREAMBLE_LENGTH_PREAMBLE_LEN_DEFAULT (0x0 << RF_REG09_PREAMBLE_LENGTH_PREAMBLE_LEN_POS)

#define PREAMBLE_PREAMBLE_DEFAULT       (0x0 << RF_REG09_PREAMBLE_PREAMBLE_POS)

#define PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_DEFAULT_BITBAND 0x0
#define PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_DEFAULT (PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_DEFAULT_BITBAND << RF_REG09_PACKET_LENGTH_OPTS_EN_PACKET_LEN_FIX_POS)

#define PACKET_LENGTH_OPTS_PACKET_LEN_CORR_DEFAULT (0x0 << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_CORR_POS)

#define PACKET_LENGTH_OPTS_PACKET_LEN_POS_DEFAULT (0x0 << RF_REG09_PACKET_LENGTH_OPTS_PACKET_LEN_POS_POS)

/* REG0A */
#define RF_REG0A_BASE                   0x40010028
#define RF_REG0A                        REG32_POINTER(RF_REG0A_BASE)

/* RF_REG0A bit positions */
#define RF_REG0A_ADDRESS_BROADCAST_ADDRESS_BR_POS 16
#define RF_REG0A_ADDRESS_BROADCAST_ADDRESS_BR_MASK (0xFFFF << RF_REG0A_ADDRESS_BROADCAST_ADDRESS_BR_POS)
#define RF_REG0A_ADDRESS_ADDRESS_POS    0
#define RF_REG0A_ADDRESS_ADDRESS_MASK   (0xFFFF << RF_REG0A_ADDRESS_ADDRESS_POS)

/* RF_REG0A settings */
#define ADDRESS_BROADCAST_ADDRESS_BR_DEFAULT (0x0 << RF_REG0A_ADDRESS_BROADCAST_ADDRESS_BR_POS)

#define ADDRESS_ADDRESS_DEFAULT         (0x0 << RF_REG0A_ADDRESS_ADDRESS_POS)

/* SYNC_PATTERN */
#define RF_SYNC_PATTERN_BASE            0x4001002C
#define RF_SYNC_PATTERN                 REG32_POINTER(RF_SYNC_PATTERN_BASE)

/* REG0C */
#define RF_REG0C_BASE                   0x40010030
#define RF_REG0C                        REG32_POINTER(RF_REG0C_BASE)

/* RF_REG0C bit positions */
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_2_POS 26
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_2_MASK (0x1F << RF_REG0C_CONV_CODES_POLY_CC_POLY_2_POS)
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_1_POS 21
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_1_MASK (0x1F << RF_REG0C_CONV_CODES_POLY_CC_POLY_1_POS)
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_0_POS 16
#define RF_REG0C_CONV_CODES_POLY_CC_POLY_0_MASK (0x1F << RF_REG0C_CONV_CODES_POLY_CC_POLY_0_POS)
#define RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_POS 10
#define RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_MASK (0x3 << RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_POS)
#define RF_REG0C_CONV_CODES_CONF_CC_EN_TX_STOP_POS 9
#define RF_REG0C_CONV_CODES_CONF_EN_CONV_CODE_POS 8
#define RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_POS 6
#define RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_MASK (0x3 << RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_POS)
#define RF_REG0C_PACKET_EXTRA_EN_STOP_WORD_POS 5
#define RF_REG0C_PACKET_EXTRA_PKT_INFO_PRE_NPOST_POS 4
#define RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_POS 2
#define RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_MASK (0x3 << RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_POS)
#define RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_POS 0
#define RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_MASK (0x3 << RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_POS)

/* RF_REG0C bit-band aliases */
#define RF_REG0C_CONV_CODES_CONF_CC_EN_TX_STOP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG0C_BASE, RF_REG0C_CONV_CODES_CONF_CC_EN_TX_STOP_POS))
#define RF_REG0C_CONV_CODES_CONF_EN_CONV_CODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG0C_BASE, RF_REG0C_CONV_CODES_CONF_EN_CONV_CODE_POS))
#define RF_REG0C_PACKET_EXTRA_EN_STOP_WORD_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG0C_BASE, RF_REG0C_PACKET_EXTRA_EN_STOP_WORD_POS))
#define RF_REG0C_PACKET_EXTRA_PKT_INFO_PRE_NPOST_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG0C_BASE, RF_REG0C_PACKET_EXTRA_PKT_INFO_PRE_NPOST_POS))

/* RF_REG0C settings */
#define CONV_CODES_POLY_CC_POLY_2_DEFAULT (0x0 << RF_REG0C_CONV_CODES_POLY_CC_POLY_2_POS)

#define CONV_CODES_POLY_CC_POLY_1_DEFAULT (0x0 << RF_REG0C_CONV_CODES_POLY_CC_POLY_1_POS)

#define CONV_CODES_POLY_CC_POLY_0_DEFAULT (0x0 << RF_REG0C_CONV_CODES_POLY_CC_POLY_0_POS)

#define CONV_CODES_CONF_CC_VITERBI_LEN_DEFAULT (0x0 << RF_REG0C_CONV_CODES_CONF_CC_VITERBI_LEN_POS)

#define CONV_CODES_CONF_CC_EN_TX_STOP_DEFAULT_BITBAND 0x0
#define CONV_CODES_CONF_CC_EN_TX_STOP_DEFAULT (CONV_CODES_CONF_CC_EN_TX_STOP_DEFAULT_BITBAND << RF_REG0C_CONV_CODES_CONF_CC_EN_TX_STOP_POS)

#define CONV_CODES_CONF_EN_CONV_CODE_DEFAULT_BITBAND 0x0
#define CONV_CODES_CONF_EN_CONV_CODE_DEFAULT (CONV_CODES_CONF_EN_CONV_CODE_DEFAULT_BITBAND << RF_REG0C_CONV_CODES_CONF_EN_CONV_CODE_POS)

#define PACKET_EXTRA_STOP_WORD_LEN_DEFAULT (0x0 << RF_REG0C_PACKET_EXTRA_STOP_WORD_LEN_POS)

#define PACKET_EXTRA_EN_STOP_WORD_DEFAULT_BITBAND 0x0
#define PACKET_EXTRA_EN_STOP_WORD_DEFAULT (PACKET_EXTRA_EN_STOP_WORD_DEFAULT_BITBAND << RF_REG0C_PACKET_EXTRA_EN_STOP_WORD_POS)

#define PACKET_EXTRA_PKT_INFO_PRE_NPOST_DEFAULT_BITBAND 0x0
#define PACKET_EXTRA_PKT_INFO_PRE_NPOST_DEFAULT (PACKET_EXTRA_PKT_INFO_PRE_NPOST_DEFAULT_BITBAND << RF_REG0C_PACKET_EXTRA_PKT_INFO_PRE_NPOST_POS)

#define PACKET_EXTRA_PATTERN_MAX_ERR_DEFAULT (0x0 << RF_REG0C_PACKET_EXTRA_PATTERN_MAX_ERR_POS)

#define PACKET_EXTRA_PATTERN_WORD_LEN_DEFAULT (0x0 << RF_REG0C_PACKET_EXTRA_PATTERN_WORD_LEN_POS)

/* CRC_POLYNOMIAL */
#define RF_CRC_POLYNOMIAL_BASE          0x40010034
#define RF_CRC_POLYNOMIAL               REG32_POINTER(RF_CRC_POLYNOMIAL_BASE)

/* CRC_RST */
#define RF_CRC_RST_BASE                 0x40010038
#define RF_CRC_RST                      REG32_POINTER(RF_CRC_RST_BASE)

/* REG0F */
#define RF_REG0F_BASE                   0x4001003C
#define RF_REG0F                        REG32_POINTER(RF_REG0F_BASE)

/* RF_REG0F bit positions */
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_POS 28
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_MASK (0xF << RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_POS)
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_POS 24
#define RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_MASK (0xF << RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_POS)
#define RF_REG0F_FRAC_CONF_TX_FRAC_GAIN_POS 19
#define RF_REG0F_FRAC_CONF_RX_FRAC_GAIN_POS 18
#define RF_REG0F_FRAC_CONF_TX_EN_FRAC_POS 17
#define RF_REG0F_FRAC_CONF_RX_EN_FRAC_POS 16
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_POS 10
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_MASK (0x1F << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_POS)
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_POS 5
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_MASK (0x1F << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_POS)
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_POS 0
#define RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_MASK (0x1F << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_POS)

/* RF_REG0F bit-band aliases */
#define RF_REG0F_FRAC_CONF_TX_FRAC_GAIN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG0F_BASE, RF_REG0F_FRAC_CONF_TX_FRAC_GAIN_POS))
#define RF_REG0F_FRAC_CONF_RX_FRAC_GAIN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG0F_BASE, RF_REG0F_FRAC_CONF_RX_FRAC_GAIN_POS))
#define RF_REG0F_FRAC_CONF_TX_EN_FRAC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG0F_BASE, RF_REG0F_FRAC_CONF_TX_EN_FRAC_POS))
#define RF_REG0F_FRAC_CONF_RX_EN_FRAC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG0F_BASE, RF_REG0F_FRAC_CONF_RX_EN_FRAC_POS))

/* RF_REG0F settings */
#define RX_FRAC_CONF_RX_FRAC_DEN_DEFAULT (0x0 << RF_REG0F_RX_FRAC_CONF_RX_FRAC_DEN_POS)

#define RX_FRAC_CONF_RX_FRAC_NUM_DEFAULT (0x0 << RF_REG0F_RX_FRAC_CONF_RX_FRAC_NUM_POS)

#define FRAC_CONF_TX_FRAC_GAIN_DEFAULT_BITBAND 0x0
#define FRAC_CONF_TX_FRAC_GAIN_DEFAULT  (FRAC_CONF_TX_FRAC_GAIN_DEFAULT_BITBAND << RF_REG0F_FRAC_CONF_TX_FRAC_GAIN_POS)

#define FRAC_CONF_RX_FRAC_GAIN_DEFAULT_BITBAND 0x0
#define FRAC_CONF_RX_FRAC_GAIN_DEFAULT  (FRAC_CONF_RX_FRAC_GAIN_DEFAULT_BITBAND << RF_REG0F_FRAC_CONF_RX_FRAC_GAIN_POS)

#define FRAC_CONF_TX_EN_FRAC_DEFAULT_BITBAND 0x0
#define FRAC_CONF_TX_EN_FRAC_DEFAULT    (FRAC_CONF_TX_EN_FRAC_DEFAULT_BITBAND << RF_REG0F_FRAC_CONF_TX_EN_FRAC_POS)

#define FRAC_CONF_RX_EN_FRAC_DEFAULT_BITBAND 0x0
#define FRAC_CONF_RX_EN_FRAC_DEFAULT    (FRAC_CONF_RX_EN_FRAC_DEFAULT_BITBAND << RF_REG0F_FRAC_CONF_RX_EN_FRAC_POS)

#define CONV_CODES_PUNCT_CC_PUNCT_2_DEFAULT (0x0 << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_2_POS)

#define CONV_CODES_PUNCT_CC_PUNCT_1_DEFAULT (0x0 << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_1_POS)

#define CONV_CODES_PUNCT_CC_PUNCT_0_DEFAULT (0x0 << RF_REG0F_CONV_CODES_PUNCT_CC_PUNCT_0_POS)

/* REG10 */
#define RF_REG10_BASE                   0x40010040
#define RF_REG10                        REG32_POINTER(RF_REG10_BASE)

/* RF_REG10 bit positions */
#define RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_POS 29
#define RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_MASK (0x7 << RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_POS)
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_POS 26
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_MASK (0x7 << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_POS)
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_POS 24
#define RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_MASK (0x3 << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_POS)
#define RF_REG10_FRONTEND_EN_PHADC_DEGLITCH_POS 22
#define RF_REG10_FRONTEND_EN_RESAMPLE_RSSI_POS 21
#define RF_REG10_FRONTEND_EN_RESAMPLE_PHADC_POS 20
#define RF_REG10_FRONTEND_DIV_PHADC_POS 16
#define RF_REG10_FRONTEND_DIV_PHADC_MASK (0xF << RF_REG10_FRONTEND_DIV_PHADC_POS)
#define RF_REG10_TX_MULT_TX_MULT_EXP_POS 12
#define RF_REG10_TX_MULT_TX_MULT_EXP_MASK (0xF << RF_REG10_TX_MULT_TX_MULT_EXP_POS)
#define RF_REG10_TX_MULT_TX_MULT_MAN_POS 8
#define RF_REG10_TX_MULT_TX_MULT_MAN_MASK (0xF << RF_REG10_TX_MULT_TX_MULT_MAN_POS)
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_POS 4
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_MASK (0xF << RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_POS)
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_POS 0
#define RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_MASK (0xF << RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_POS)

/* RF_REG10 bit-band aliases */
#define RF_REG10_FRONTEND_EN_PHADC_DEGLITCH_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG10_BASE, RF_REG10_FRONTEND_EN_PHADC_DEGLITCH_POS))
#define RF_REG10_FRONTEND_EN_RESAMPLE_RSSI_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG10_BASE, RF_REG10_FRONTEND_EN_RESAMPLE_RSSI_POS))
#define RF_REG10_FRONTEND_EN_RESAMPLE_PHADC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG10_BASE, RF_REG10_FRONTEND_EN_RESAMPLE_PHADC_POS))

/* RF_REG10 settings */
#define FRONTEND2_RESAMPLE_PH_GAIN_DEFAULT (0x0 << RF_REG10_FRONTEND2_RESAMPLE_PH_GAIN_POS)

#define FRONTEND2_RESAMPLE_RSSI_G2_DEFAULT (0x0 << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G2_POS)

#define FRONTEND2_RESAMPLE_RSSI_G1_DEFAULT (0x0 << RF_REG10_FRONTEND2_RESAMPLE_RSSI_G1_POS)

#define FRONTEND_EN_PHADC_DEGLITCH_DEFAULT_BITBAND 0x0
#define FRONTEND_EN_PHADC_DEGLITCH_DEFAULT (FRONTEND_EN_PHADC_DEGLITCH_DEFAULT_BITBAND << RF_REG10_FRONTEND_EN_PHADC_DEGLITCH_POS)

#define FRONTEND_EN_RESAMPLE_RSSI_DEFAULT_BITBAND 0x0
#define FRONTEND_EN_RESAMPLE_RSSI_DEFAULT (FRONTEND_EN_RESAMPLE_RSSI_DEFAULT_BITBAND << RF_REG10_FRONTEND_EN_RESAMPLE_RSSI_POS)

#define FRONTEND_EN_RESAMPLE_PHADC_DEFAULT_BITBAND 0x0
#define FRONTEND_EN_RESAMPLE_PHADC_DEFAULT (FRONTEND_EN_RESAMPLE_PHADC_DEFAULT_BITBAND << RF_REG10_FRONTEND_EN_RESAMPLE_PHADC_POS)

#define FRONTEND_DIV_PHADC_DEFAULT      (0x0 << RF_REG10_FRONTEND_DIV_PHADC_POS)

#define TX_MULT_TX_MULT_EXP_DEFAULT     (0x0 << RF_REG10_TX_MULT_TX_MULT_EXP_POS)

#define TX_MULT_TX_MULT_MAN_DEFAULT     (0x0 << RF_REG10_TX_MULT_TX_MULT_MAN_POS)

#define TX_FRAC_CONF_TX_FRAC_DEN_DEFAULT (0x0 << RF_REG10_TX_FRAC_CONF_TX_FRAC_DEN_POS)

#define TX_FRAC_CONF_TX_FRAC_NUM_DEFAULT (0x0 << RF_REG10_TX_FRAC_CONF_TX_FRAC_NUM_POS)

/* TX_PULSE0 */
#define RF_TX_PULSE0_BASE               0x40010044
#define RF_TX_PULSE0                    REG32_POINTER(RF_TX_PULSE0_BASE)

/* RF_TX_PULSE0 bit positions */
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF4_POS 24
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF4_MASK (0xFF << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF4_POS)
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF3_POS 16
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF3_MASK (0xFF << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF3_POS)
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF2_POS 8
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF2_MASK (0xFF << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF2_POS)
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF1_POS 0
#define RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF1_MASK (0xFF << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF1_POS)

/* RF_TX_PULSE0 settings */
#define TX_PULSE_SHAPE_1_TX_COEF4_DEFAULT (0x0 << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF4_POS)

#define TX_PULSE_SHAPE_1_TX_COEF3_DEFAULT (0x0 << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF3_POS)

#define TX_PULSE_SHAPE_1_TX_COEF2_DEFAULT (0x0 << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF2_POS)

#define TX_PULSE_SHAPE_1_TX_COEF1_DEFAULT (0x0 << RF_TX_PULSE0_TX_PULSE_SHAPE_1_TX_COEF1_POS)

/* TX_PULSE1 */
#define RF_TX_PULSE1_BASE               0x40010048
#define RF_TX_PULSE1                    REG32_POINTER(RF_TX_PULSE1_BASE)

/* RF_TX_PULSE1 bit positions */
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF8_POS 24
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF8_MASK (0xFF << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF8_POS)
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF7_POS 16
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF7_MASK (0xFF << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF7_POS)
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF6_POS 8
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF6_MASK (0xFF << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF6_POS)
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF5_POS 0
#define RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF5_MASK (0xFF << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF5_POS)

/* RF_TX_PULSE1 settings */
#define TX_PULSE_SHAPE_2_TX_COEF8_DEFAULT (0x0 << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF8_POS)

#define TX_PULSE_SHAPE_2_TX_COEF7_DEFAULT (0x0 << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF7_POS)

#define TX_PULSE_SHAPE_2_TX_COEF6_DEFAULT (0x0 << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF6_POS)

#define TX_PULSE_SHAPE_2_TX_COEF5_DEFAULT (0x0 << RF_TX_PULSE1_TX_PULSE_SHAPE_2_TX_COEF5_POS)

/* TX_PULSE2 */
#define RF_TX_PULSE2_BASE               0x4001004C
#define RF_TX_PULSE2                    REG32_POINTER(RF_TX_PULSE2_BASE)

/* RF_TX_PULSE2 bit positions */
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF12_POS 24
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF12_MASK (0xFF << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF12_POS)
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF11_POS 16
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF11_MASK (0xFF << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF11_POS)
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF10_POS 8
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF10_MASK (0xFF << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF10_POS)
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF9_POS 0
#define RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF9_MASK (0xFF << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF9_POS)

/* RF_TX_PULSE2 settings */
#define TX_PULSE_SHAPE_3_TX_COEF12_DEFAULT (0x0 << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF12_POS)

#define TX_PULSE_SHAPE_3_TX_COEF11_DEFAULT (0x0 << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF11_POS)

#define TX_PULSE_SHAPE_3_TX_COEF10_DEFAULT (0x0 << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF10_POS)

#define TX_PULSE_SHAPE_3_TX_COEF9_DEFAULT (0x0 << RF_TX_PULSE2_TX_PULSE_SHAPE_3_TX_COEF9_POS)

/* TX_PULSE3 */
#define RF_TX_PULSE3_BASE               0x40010050
#define RF_TX_PULSE3                    REG32_POINTER(RF_TX_PULSE3_BASE)

/* RF_TX_PULSE3 bit positions */
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF16_POS 24
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF16_MASK (0xFF << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF16_POS)
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF15_POS 16
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF15_MASK (0xFF << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF15_POS)
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF14_POS 8
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF14_MASK (0xFF << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF14_POS)
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF13_POS 0
#define RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF13_MASK (0xFF << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF13_POS)

/* RF_TX_PULSE3 settings */
#define TX_PULSE_SHAPE_4_TX_COEF16_DEFAULT (0x0 << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF16_POS)

#define TX_PULSE_SHAPE_4_TX_COEF15_DEFAULT (0x0 << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF15_POS)

#define TX_PULSE_SHAPE_4_TX_COEF14_DEFAULT (0x0 << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF14_POS)

#define TX_PULSE_SHAPE_4_TX_COEF13_DEFAULT (0x0 << RF_TX_PULSE3_TX_PULSE_SHAPE_4_TX_COEF13_POS)

/* RX_PULSE */
#define RF_RX_PULSE_BASE                0x40010054
#define RF_RX_PULSE                     REG32_POINTER(RF_RX_PULSE_BASE)

/* RF_RX_PULSE bit positions */
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF8_POS 28
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF8_MASK (0xF << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF8_POS)
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF7_POS 24
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF7_MASK (0xF << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF7_POS)
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF6_POS 20
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF6_MASK (0xF << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF6_POS)
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF5_POS 16
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF5_MASK (0xF << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF5_POS)
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF4_POS 12
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF4_MASK (0xF << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF4_POS)
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF3_POS 8
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF3_MASK (0xF << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF3_POS)
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF2_POS 4
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF2_MASK (0xF << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF2_POS)
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF1_POS 0
#define RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF1_MASK (0xF << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF1_POS)

/* RF_RX_PULSE settings */
#define RX_PULSE_SHAPE_RX_COEF8_DEFAULT (0x0 << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF8_POS)

#define RX_PULSE_SHAPE_RX_COEF7_DEFAULT (0x0 << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF7_POS)

#define RX_PULSE_SHAPE_RX_COEF6_DEFAULT (0x0 << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF6_POS)

#define RX_PULSE_SHAPE_RX_COEF5_DEFAULT (0x0 << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF5_POS)

#define RX_PULSE_SHAPE_RX_COEF4_DEFAULT (0x0 << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF4_POS)

#define RX_PULSE_SHAPE_RX_COEF3_DEFAULT (0x0 << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF3_POS)

#define RX_PULSE_SHAPE_RX_COEF2_DEFAULT (0x0 << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF2_POS)

#define RX_PULSE_SHAPE_RX_COEF1_DEFAULT (0x0 << RF_RX_PULSE_RX_PULSE_SHAPE_RX_COEF1_POS)

/* REG16 */
#define RF_REG16_BASE                   0x40010058
#define RF_REG16                        REG32_POINTER(RF_REG16_BASE)

/* RF_REG16 bit positions */
#define RF_REG16_RX_IF_RESAMPLE_PH_IF_POS 25
#define RF_REG16_RX_IF_RESAMPLE_PH_IF_MASK (0xF << RF_REG16_RX_IF_RESAMPLE_PH_IF_POS)
#define RF_REG16_RX_IF_IF2_CLK_OS_POS   16
#define RF_REG16_RX_IF_IF2_CLK_OS_MASK  (0x1FF << RF_REG16_RX_IF_IF2_CLK_OS_POS)
#define RF_REG16_FSK_FCR_AMP_1_FSK_FCR_AMP1_POS 8
#define RF_REG16_FSK_FCR_AMP_1_FSK_FCR_AMP1_MASK (0xFF << RF_REG16_FSK_FCR_AMP_1_FSK_FCR_AMP1_POS)
#define RF_REG16_FILTER_GAIN_DR_LIMIT_POS 6
#define RF_REG16_FILTER_GAIN_DR_LIMIT_MASK (0x3 << RF_REG16_FILTER_GAIN_DR_LIMIT_POS)
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_M_POS 3
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_M_MASK (0x7 << RF_REG16_FILTER_GAIN_FILTER_GAIN_M_POS)
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_E_POS 0
#define RF_REG16_FILTER_GAIN_FILTER_GAIN_E_MASK (0x7 << RF_REG16_FILTER_GAIN_FILTER_GAIN_E_POS)

/* RF_REG16 settings */
#define RX_IF_RESAMPLE_PH_IF_DEFAULT    (0x0 << RF_REG16_RX_IF_RESAMPLE_PH_IF_POS)

#define RX_IF_IF2_CLK_OS_DEFAULT        (0x0 << RF_REG16_RX_IF_IF2_CLK_OS_POS)

#define FSK_FCR_AMP_1_FSK_FCR_AMP1_DEFAULT (0x0 << RF_REG16_FSK_FCR_AMP_1_FSK_FCR_AMP1_POS)

#define FILTER_GAIN_DR_LIMIT_DEFAULT    (0x0 << RF_REG16_FILTER_GAIN_DR_LIMIT_POS)

#define FILTER_GAIN_FILTER_GAIN_M_DEFAULT (0x0 << RF_REG16_FILTER_GAIN_FILTER_GAIN_M_POS)

#define FILTER_GAIN_FILTER_GAIN_E_DEFAULT (0x0 << RF_REG16_FILTER_GAIN_FILTER_GAIN_E_POS)

/* REG17 */
#define RF_REG17_BASE                   0x4001005C
#define RF_REG17                        REG32_POINTER(RF_REG17_BASE)

/* RF_REG17 bit positions */
#define RF_REG17_FSK_FCR_AMP_3_FSK_FCR_AMP3_POS 24
#define RF_REG17_FSK_FCR_AMP_3_FSK_FCR_AMP3_MASK (0xFF << RF_REG17_FSK_FCR_AMP_3_FSK_FCR_AMP3_POS)
#define RF_REG17_FSK_FCR_AMP_2_FSK_FCR_AMP2_POS 16
#define RF_REG17_FSK_FCR_AMP_2_FSK_FCR_AMP2_MASK (0xFF << RF_REG17_FSK_FCR_AMP_2_FSK_FCR_AMP2_POS)
#define RF_REG17_CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_POS 13
#define RF_REG17_CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_MASK (0x3 << RF_REG17_CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_POS)
#define RF_REG17_CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_POS 12
#define RF_REG17_CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_POS 9
#define RF_REG17_CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_MASK (0x7 << RF_REG17_CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_POS)
#define RF_REG17_CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_POS 8
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_POS 4
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_MASK (0x7 << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_POS)
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_POS 0
#define RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_MASK (0x7 << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_POS)

/* RF_REG17 bit-band aliases */
#define RF_REG17_CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG17_BASE, RF_REG17_CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_POS))
#define RF_REG17_CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG17_BASE, RF_REG17_CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_POS))

/* RF_REG17 settings */
#define FSK_FCR_AMP_3_FSK_FCR_AMP3_DEFAULT (0x0 << RF_REG17_FSK_FCR_AMP_3_FSK_FCR_AMP3_POS)

#define FSK_FCR_AMP_2_FSK_FCR_AMP2_DEFAULT (0x0 << RF_REG17_FSK_FCR_AMP_2_FSK_FCR_AMP2_POS)

#define CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_DEFAULT (0x0 << RF_REG17_CARRIER_RECOVERY_EXTRA_MAX_ERR_IN_DL_SYNC_POS)

#define CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_DEFAULT (CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_DEFAULT_BITBAND << RF_REG17_CARRIER_RECOVERY_EXTRA_EN_SYNC_OK_DELAY_LINE_POS)

#define CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_DEFAULT (0x0 << RF_REG17_CARRIER_RECOVERY_EXTRA_NC_SEL_OUT_POS)

#define CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_DEFAULT_BITBAND 0x0
#define CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_DEFAULT (CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_DEFAULT_BITBAND << RF_REG17_CARRIER_RECOVERY_EXTRA_EN_NOT_CAUSAL_POS)

#define CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_DEFAULT (0x0 << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_MAN_POS)

#define CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_DEFAULT (0x0 << RF_REG17_CARRIER_RECOVERY_EXTRA_FREQ_LIMIT_EXP_POS)

/* REG18 */
#define RF_REG18_BASE                   0x40010060
#define RF_REG18                        REG32_POINTER(RF_REG18_BASE)

/* RF_REG18 bit positions */
#define RF_REG18_CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_POS 16
#define RF_REG18_CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_MASK (0xFFFF << RF_REG18_CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_POS)
#define RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_POS 14
#define RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_MASK (0x3 << RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_POS)
#define RF_REG18_RSSI_BANK_FAST_RSSI_POS 13
#define RF_REG18_RSSI_BANK_EN_FAST_PRE_SYNC_POS 12
#define RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_POS 8
#define RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_MASK (0xF << RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_POS)
#define RF_REG18_DECISION_USE_VIT_SOFT_POS 4
#define RF_REG18_DECISION_VITERBI_LEN_POS 2
#define RF_REG18_DECISION_VITERBI_LEN_MASK (0x3 << RF_REG18_DECISION_VITERBI_LEN_POS)
#define RF_REG18_DECISION_VITERBI_POW_NLIN_POS 1
#define RF_REG18_DECISION_EN_VITERBI_GFSK_POS 0

/* RF_REG18 bit-band aliases */
#define RF_REG18_RSSI_BANK_FAST_RSSI_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG18_BASE, RF_REG18_RSSI_BANK_FAST_RSSI_POS))
#define RF_REG18_RSSI_BANK_EN_FAST_PRE_SYNC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG18_BASE, RF_REG18_RSSI_BANK_EN_FAST_PRE_SYNC_POS))
#define RF_REG18_DECISION_USE_VIT_SOFT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG18_BASE, RF_REG18_DECISION_USE_VIT_SOFT_POS))
#define RF_REG18_DECISION_VITERBI_POW_NLIN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG18_BASE, RF_REG18_DECISION_VITERBI_POW_NLIN_POS))
#define RF_REG18_DECISION_EN_VITERBI_GFSK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG18_BASE, RF_REG18_DECISION_EN_VITERBI_GFSK_POS))

/* RF_REG18 settings */
#define CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_DEFAULT (0x0 << RF_REG18_CORRECT_CFREQ_IF_CORRECT_CFREQ_IF_POS)

#define RSSI_BANK_RSSI_TRI_CK_DIV_DEFAULT (0x0 << RF_REG18_RSSI_BANK_RSSI_TRI_CK_DIV_POS)

#define RSSI_BANK_FAST_RSSI_DEFAULT_BITBAND 0x0
#define RSSI_BANK_FAST_RSSI_DEFAULT     (RSSI_BANK_FAST_RSSI_DEFAULT_BITBAND << RF_REG18_RSSI_BANK_FAST_RSSI_POS)

#define RSSI_BANK_EN_FAST_PRE_SYNC_DEFAULT_BITBAND 0x0
#define RSSI_BANK_EN_FAST_PRE_SYNC_DEFAULT (RSSI_BANK_EN_FAST_PRE_SYNC_DEFAULT_BITBAND << RF_REG18_RSSI_BANK_EN_FAST_PRE_SYNC_POS)

#define RSSI_BANK_TAU_RSSI_FILTERING_DEFAULT (0x0 << RF_REG18_RSSI_BANK_TAU_RSSI_FILTERING_POS)

#define DECISION_USE_VIT_SOFT_DEFAULT_BITBAND 0x0
#define DECISION_USE_VIT_SOFT_DEFAULT   (DECISION_USE_VIT_SOFT_DEFAULT_BITBAND << RF_REG18_DECISION_USE_VIT_SOFT_POS)

#define DECISION_VITERBI_LEN_DEFAULT    (0x0 << RF_REG18_DECISION_VITERBI_LEN_POS)

#define DECISION_VITERBI_POW_NLIN_DEFAULT_BITBAND 0x0
#define DECISION_VITERBI_POW_NLIN_DEFAULT (DECISION_VITERBI_POW_NLIN_DEFAULT_BITBAND << RF_REG18_DECISION_VITERBI_POW_NLIN_POS)

#define DECISION_EN_VITERBI_GFSK_DEFAULT_BITBAND 0x0
#define DECISION_EN_VITERBI_GFSK_DEFAULT (DECISION_EN_VITERBI_GFSK_DEFAULT_BITBAND << RF_REG18_DECISION_EN_VITERBI_GFSK_POS)

/* REG19 */
#define RF_REG19_BASE                   0x40010064
#define RF_REG19                        REG32_POINTER(RF_REG19_BASE)

/* RF_REG19 bit positions */
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_POS 28
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_MASK (0x3 << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_POS)
#define RF_REG19_PLL_BANK_IQ_PLL_0_TX_POS 24
#define RF_REG19_PLL_BANK_IQ_PLL_0_TX_MASK (0xF << RF_REG19_PLL_BANK_IQ_PLL_0_TX_POS)
#define RF_REG19_PLL_BANK_LOW_DR_TX_POS 22
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_POS 20
#define RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_MASK (0x3 << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_POS)
#define RF_REG19_PLL_BANK_IQ_PLL_0_POS  16
#define RF_REG19_PLL_BANK_IQ_PLL_0_MASK (0xF << RF_REG19_PLL_BANK_IQ_PLL_0_POS)
#define RF_REG19_PA_PWR_MIN_PA_PWR_POS  13
#define RF_REG19_PA_PWR_PA_PWR_POS      8
#define RF_REG19_PA_PWR_PA_PWR_MASK     (0x1F << RF_REG19_PA_PWR_PA_PWR_POS)
#define RF_REG19_CLK_CH_FILTER_DIV_RSSI_POS 4
#define RF_REG19_CLK_CH_FILTER_DIV_RSSI_MASK (0xF << RF_REG19_CLK_CH_FILTER_DIV_RSSI_POS)
#define RF_REG19_CLK_CH_FILTER_DIV_FILT_POS 0
#define RF_REG19_CLK_CH_FILTER_DIV_FILT_MASK (0xF << RF_REG19_CLK_CH_FILTER_DIV_FILT_POS)

/* RF_REG19 bit-band aliases */
#define RF_REG19_PLL_BANK_LOW_DR_TX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG19_BASE, RF_REG19_PLL_BANK_LOW_DR_TX_POS))
#define RF_REG19_PA_PWR_MIN_PA_PWR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG19_BASE, RF_REG19_PA_PWR_MIN_PA_PWR_POS))

/* RF_REG19 settings */
#define PLL_BANK_PLL_FILTER_RES_TRIM_TX_DEFAULT (0x0 << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_TX_POS)

#define PLL_BANK_IQ_PLL_0_TX_DEFAULT    (0x0 << RF_REG19_PLL_BANK_IQ_PLL_0_TX_POS)

#define PLL_BANK_LOW_DR_TX_DEFAULT_BITBAND 0x0
#define PLL_BANK_LOW_DR_TX_DEFAULT      (PLL_BANK_LOW_DR_TX_DEFAULT_BITBAND << RF_REG19_PLL_BANK_LOW_DR_TX_POS)

#define PLL_BANK_PLL_FILTER_RES_TRIM_DEFAULT (0x0 << RF_REG19_PLL_BANK_PLL_FILTER_RES_TRIM_POS)

#define PLL_BANK_IQ_PLL_0_DEFAULT       (0x0 << RF_REG19_PLL_BANK_IQ_PLL_0_POS)

#define PA_PWR_MIN_PA_PWR_DEFAULT_BITBAND 0x0
#define PA_PWR_MIN_PA_PWR_DEFAULT       (PA_PWR_MIN_PA_PWR_DEFAULT_BITBAND << RF_REG19_PA_PWR_MIN_PA_PWR_POS)

#define PA_PWR_PA_PWR_DEFAULT           (0x0 << RF_REG19_PA_PWR_PA_PWR_POS)

#define CLK_CH_FILTER_DIV_RSSI_DEFAULT  (0x0 << RF_REG19_CLK_CH_FILTER_DIV_RSSI_POS)

#define CLK_CH_FILTER_DIV_FILT_DEFAULT  (0x0 << RF_REG19_CLK_CH_FILTER_DIV_FILT_POS)

/* REG1A */
#define RF_REG1A_BASE                   0x40010068
#define RF_REG1A                        REG32_POINTER(RF_REG1A_BASE)

/* RF_REG1A bit positions */
#define RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_POS 28
#define RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_MASK (0xF << RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_POS)
#define RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_POS 24
#define RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_MASK (0xF << RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_POS)
#define RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_POS 22
#define RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_MASK (0x3 << RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_POS)
#define RF_REG1A_RSSI_CTRL_AGC_USE_LNA_POS 21
#define RF_REG1A_RSSI_CTRL_AGC_MODE_POS 20
#define RF_REG1A_RSSI_CTRL_AGC_WAIT_POS 18
#define RF_REG1A_RSSI_CTRL_AGC_WAIT_MASK (0x3 << RF_REG1A_RSSI_CTRL_AGC_WAIT_POS)
#define RF_REG1A_RSSI_CTRL_PAYLOAD_BLOCKS_AGC_POS 17
#define RF_REG1A_RSSI_CTRL_BYPASS_AGC_POS 16
#define RF_REG1A_FILTER_BIAS_IQ_FI_BW_POS 8
#define RF_REG1A_FILTER_BIAS_IQ_FI_BW_MASK (0x1F << RF_REG1A_FILTER_BIAS_IQ_FI_BW_POS)
#define RF_REG1A_FILTER_BIAS_IQ_FI_FC_POS 0
#define RF_REG1A_FILTER_BIAS_IQ_FI_FC_MASK (0x1F << RF_REG1A_FILTER_BIAS_IQ_FI_FC_POS)

/* RF_REG1A bit-band aliases */
#define RF_REG1A_RSSI_CTRL_AGC_USE_LNA_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1A_BASE, RF_REG1A_RSSI_CTRL_AGC_USE_LNA_POS))
#define RF_REG1A_RSSI_CTRL_AGC_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1A_BASE, RF_REG1A_RSSI_CTRL_AGC_MODE_POS))
#define RF_REG1A_RSSI_CTRL_PAYLOAD_BLOCKS_AGC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1A_BASE, RF_REG1A_RSSI_CTRL_PAYLOAD_BLOCKS_AGC_POS))
#define RF_REG1A_RSSI_CTRL_BYPASS_AGC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1A_BASE, RF_REG1A_RSSI_CTRL_BYPASS_AGC_POS))

/* RF_REG1A settings */
#define ATT_CTRL_ATT_CTRL_MAX_DEFAULT   (0x0 << RF_REG1A_ATT_CTRL_ATT_CTRL_MAX_POS)

#define ATT_CTRL_SET_RX_ATT_CTRL_DEFAULT (0x0 << RF_REG1A_ATT_CTRL_SET_RX_ATT_CTRL_POS)

#define RSSI_CTRL_AGC_DECAY_TAU_DEFAULT (0x0 << RF_REG1A_RSSI_CTRL_AGC_DECAY_TAU_POS)

#define RSSI_CTRL_AGC_USE_LNA_DEFAULT_BITBAND 0x0
#define RSSI_CTRL_AGC_USE_LNA_DEFAULT   (RSSI_CTRL_AGC_USE_LNA_DEFAULT_BITBAND << RF_REG1A_RSSI_CTRL_AGC_USE_LNA_POS)

#define RSSI_CTRL_AGC_MODE_DEFAULT_BITBAND 0x0
#define RSSI_CTRL_AGC_MODE_DEFAULT      (RSSI_CTRL_AGC_MODE_DEFAULT_BITBAND << RF_REG1A_RSSI_CTRL_AGC_MODE_POS)

#define RSSI_CTRL_AGC_WAIT_DEFAULT      (0x0 << RF_REG1A_RSSI_CTRL_AGC_WAIT_POS)

#define RSSI_CTRL_PAYLOAD_BLOCKS_AGC_DEFAULT_BITBAND 0x0
#define RSSI_CTRL_PAYLOAD_BLOCKS_AGC_DEFAULT (RSSI_CTRL_PAYLOAD_BLOCKS_AGC_DEFAULT_BITBAND << RF_REG1A_RSSI_CTRL_PAYLOAD_BLOCKS_AGC_POS)

#define RSSI_CTRL_BYPASS_AGC_DEFAULT_BITBAND 0x0
#define RSSI_CTRL_BYPASS_AGC_DEFAULT    (RSSI_CTRL_BYPASS_AGC_DEFAULT_BITBAND << RF_REG1A_RSSI_CTRL_BYPASS_AGC_POS)

#define FILTER_BIAS_IQ_FI_BW_DEFAULT    (0x0 << RF_REG1A_FILTER_BIAS_IQ_FI_BW_POS)

#define FILTER_BIAS_IQ_FI_FC_DEFAULT    (0x0 << RF_REG1A_FILTER_BIAS_IQ_FI_FC_POS)

/* REG1B */
#define RF_REG1B_BASE                   0x4001006C
#define RF_REG1B                        REG32_POINTER(RF_REG1B_BASE)

/* RF_REG1B bit positions */
#define RF_REG1B_IEEE802154_OPTS_EN_DW_TEST_POS 31
#define RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_POS 29
#define RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_MASK (0x3 << RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_POS)
#define RF_REG1B_IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_POS 28
#define RF_REG1B_IEEE802154_OPTS_EN_L2F_RX_POS 27
#define RF_REG1B_IEEE802154_OPTS_C2B_THR_POS 24
#define RF_REG1B_IEEE802154_OPTS_C2B_THR_MASK (0x7 << RF_REG1B_IEEE802154_OPTS_C2B_THR_POS)
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_POS 20
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_MASK (0xF << RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_POS)
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_POS 18
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_MASK (0x3 << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_POS)
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_HIGH_POS 17
#define RF_REG1B_AGC_PEAK_DET_EN_AGC_PEAK_POS 16
#define RF_REG1B_AGC_THR_HIGH_AGC_THR_HIGH_POS 8
#define RF_REG1B_AGC_THR_HIGH_AGC_THR_HIGH_MASK (0xFF << RF_REG1B_AGC_THR_HIGH_AGC_THR_HIGH_POS)
#define RF_REG1B_AGC_THR_LOW_AGC_THR_LOW_POS 0
#define RF_REG1B_AGC_THR_LOW_AGC_THR_LOW_MASK (0xFF << RF_REG1B_AGC_THR_LOW_AGC_THR_LOW_POS)

/* RF_REG1B bit-band aliases */
#define RF_REG1B_IEEE802154_OPTS_EN_DW_TEST_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1B_BASE, RF_REG1B_IEEE802154_OPTS_EN_DW_TEST_POS))
#define RF_REG1B_IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1B_BASE, RF_REG1B_IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_POS))
#define RF_REG1B_IEEE802154_OPTS_EN_L2F_RX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1B_BASE, RF_REG1B_IEEE802154_OPTS_EN_L2F_RX_POS))
#define RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_HIGH_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1B_BASE, RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_HIGH_POS))
#define RF_REG1B_AGC_PEAK_DET_EN_AGC_PEAK_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG1B_BASE, RF_REG1B_AGC_PEAK_DET_EN_AGC_PEAK_POS))

/* RF_REG1B settings */
#define IEEE802154_OPTS_EN_DW_TEST_DEFAULT_BITBAND 0x0
#define IEEE802154_OPTS_EN_DW_TEST_DEFAULT (IEEE802154_OPTS_EN_DW_TEST_DEFAULT_BITBAND << RF_REG1B_IEEE802154_OPTS_EN_DW_TEST_POS)

#define IEEE802154_OPTS_BER_CLK_MODE_DEFAULT (0x0 << RF_REG1B_IEEE802154_OPTS_BER_CLK_MODE_POS)

#define IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_DEFAULT_BITBAND 0x0
#define IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_DEFAULT (IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_DEFAULT_BITBAND << RF_REG1B_IEEE802154_OPTS_RX_DATA_NOT_SAMPLED_POS)

#define IEEE802154_OPTS_EN_L2F_RX_DEFAULT_BITBAND 0x0
#define IEEE802154_OPTS_EN_L2F_RX_DEFAULT (IEEE802154_OPTS_EN_L2F_RX_DEFAULT_BITBAND << RF_REG1B_IEEE802154_OPTS_EN_L2F_RX_POS)

#define IEEE802154_OPTS_C2B_THR_DEFAULT (0x0 << RF_REG1B_IEEE802154_OPTS_C2B_THR_POS)

#define AGC_PEAK_DET_PEAK_DET_TAU_DEFAULT (0x0 << RF_REG1B_AGC_PEAK_DET_PEAK_DET_TAU_POS)

#define AGC_PEAK_DET_PEAK_DET_THR_LOW_DEFAULT (0x0 << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_LOW_POS)

#define AGC_PEAK_DET_PEAK_DET_THR_HIGH_DEFAULT_BITBAND 0x0
#define AGC_PEAK_DET_PEAK_DET_THR_HIGH_DEFAULT (AGC_PEAK_DET_PEAK_DET_THR_HIGH_DEFAULT_BITBAND << RF_REG1B_AGC_PEAK_DET_PEAK_DET_THR_HIGH_POS)

#define AGC_PEAK_DET_EN_AGC_PEAK_DEFAULT_BITBAND 0x0
#define AGC_PEAK_DET_EN_AGC_PEAK_DEFAULT (AGC_PEAK_DET_EN_AGC_PEAK_DEFAULT_BITBAND << RF_REG1B_AGC_PEAK_DET_EN_AGC_PEAK_POS)

#define AGC_THR_HIGH_AGC_THR_HIGH_DEFAULT (0x0 << RF_REG1B_AGC_THR_HIGH_AGC_THR_HIGH_POS)

#define AGC_THR_LOW_AGC_THR_LOW_DEFAULT (0x0 << RF_REG1B_AGC_THR_LOW_AGC_THR_LOW_POS)

/* AGC_LUT1 */
#define RF_AGC_LUT1_BASE                0x40010070
#define RF_AGC_LUT1                     REG32_POINTER(RF_AGC_LUT1_BASE)

/* RF_AGC_LUT1 bit positions */
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_2_LO_POS 22
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_2_LO_MASK (0x3FF << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_2_LO_POS)
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_1_POS 11
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_1_MASK (0x7FF << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_1_POS)
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_0_POS 0
#define RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_0_MASK (0x7FF << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_0_POS)

/* RF_AGC_LUT1 settings */
#define AGC_LUT_1_AGC_LEVEL_2_LO_DEFAULT (0x0 << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_2_LO_POS)

#define AGC_LUT_1_AGC_LEVEL_1_DEFAULT   (0x0 << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_1_POS)

#define AGC_LUT_1_AGC_LEVEL_0_DEFAULT   (0x0 << RF_AGC_LUT1_AGC_LUT_1_AGC_LEVEL_0_POS)

/* AGC_LUT2 */
#define RF_AGC_LUT2_BASE                0x40010074
#define RF_AGC_LUT2                     REG32_POINTER(RF_AGC_LUT2_BASE)

/* RF_AGC_LUT2 bit positions */
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_5_LO_POS 23
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_5_LO_MASK (0x1FF << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_5_LO_POS)
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_4_POS 12
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_4_MASK (0x7FF << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_4_POS)
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_3_POS 1
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_3_MASK (0x7FF << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_3_POS)
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_2_HI_POS 0

/* RF_AGC_LUT2 bit-band aliases */
#define RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_2_HI_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_AGC_LUT2_BASE, RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_2_HI_POS))

/* RF_AGC_LUT2 settings */
#define AGC_LUT_2_AGC_LEVEL_5_LO_DEFAULT (0x0 << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_5_LO_POS)

#define AGC_LUT_2_AGC_LEVEL_4_DEFAULT   (0x0 << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_4_POS)

#define AGC_LUT_2_AGC_LEVEL_3_DEFAULT   (0x0 << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_3_POS)

#define AGC_LUT_2_AGC_LEVEL_2_HI_DEFAULT_BITBAND 0x0
#define AGC_LUT_2_AGC_LEVEL_2_HI_DEFAULT (AGC_LUT_2_AGC_LEVEL_2_HI_DEFAULT_BITBAND << RF_AGC_LUT2_AGC_LUT_2_AGC_LEVEL_2_HI_POS)

/* AGC_LUT3 */
#define RF_AGC_LUT3_BASE                0x40010078
#define RF_AGC_LUT3                     REG32_POINTER(RF_AGC_LUT3_BASE)

/* RF_AGC_LUT3 bit positions */
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_8_LO_POS 24
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_8_LO_MASK (0xFF << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_8_LO_POS)
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_7_POS 13
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_7_MASK (0x7FF << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_7_POS)
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_6_POS 2
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_6_MASK (0x7FF << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_6_POS)
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_5_HI_POS 0
#define RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_5_HI_MASK (0x3 << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_5_HI_POS)

/* RF_AGC_LUT3 settings */
#define AGC_LUT_3_AGC_LEVEL_8_LO_DEFAULT (0x0 << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_8_LO_POS)

#define AGC_LUT_3_AGC_LEVEL_7_DEFAULT   (0x0 << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_7_POS)

#define AGC_LUT_3_AGC_LEVEL_6_DEFAULT   (0x0 << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_6_POS)

#define AGC_LUT_3_AGC_LEVEL_5_HI_DEFAULT (0x0 << RF_AGC_LUT3_AGC_LUT_3_AGC_LEVEL_5_HI_POS)

/* AGC_LUT4 */
#define RF_AGC_LUT4_BASE                0x4001007C
#define RF_AGC_LUT4                     REG32_POINTER(RF_AGC_LUT4_BASE)

/* RF_AGC_LUT4 bit positions */
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_11_LO_POS 25
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_11_LO_MASK (0x7F << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_11_LO_POS)
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_10_POS 14
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_10_MASK (0x7FF << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_10_POS)
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_9_POS 3
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_9_MASK (0x7FF << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_9_POS)
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_8_HI_POS 0
#define RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_8_HI_MASK (0x7 << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_8_HI_POS)

/* RF_AGC_LUT4 settings */
#define AGC_LUT_4_AGC_LEVEL_11_LO_DEFAULT (0x0 << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_11_LO_POS)

#define AGC_LUT_4_AGC_LEVEL_10_DEFAULT  (0x0 << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_10_POS)

#define AGC_LUT_4_AGC_LEVEL_9_DEFAULT   (0x0 << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_9_POS)

#define AGC_LUT_4_AGC_LEVEL_8_HI_DEFAULT (0x0 << RF_AGC_LUT4_AGC_LUT_4_AGC_LEVEL_8_HI_POS)

/* REG20 */
#define RF_REG20_BASE                   0x40010080
#define RF_REG20                        REG32_POINTER(RF_REG20_BASE)

/* RF_REG20 bit positions */
#define RF_REG20_TIMINGS_3_T_DLL_POS    28
#define RF_REG20_TIMINGS_3_T_DLL_MASK   (0xF << RF_REG20_TIMINGS_3_T_DLL_POS)
#define RF_REG20_TIMINGS_3_T_PLL_TX_POS 24
#define RF_REG20_TIMINGS_3_T_PLL_TX_MASK (0xF << RF_REG20_TIMINGS_3_T_PLL_TX_POS)
#define RF_REG20_TIMINGS_2_T_SUBBAND_TX_POS 20
#define RF_REG20_TIMINGS_2_T_SUBBAND_TX_MASK (0xF << RF_REG20_TIMINGS_2_T_SUBBAND_TX_POS)
#define RF_REG20_TIMINGS_2_T_TX_RF_POS  16
#define RF_REG20_TIMINGS_2_T_TX_RF_MASK (0xF << RF_REG20_TIMINGS_2_T_TX_RF_POS)
#define RF_REG20_TIMINGS_1_T_GRANULARITY_TX_POS 12
#define RF_REG20_TIMINGS_1_T_GRANULARITY_TX_MASK (0x7 << RF_REG20_TIMINGS_1_T_GRANULARITY_TX_POS)
#define RF_REG20_TIMINGS_1_T_GRANULARITY_RX_POS 8
#define RF_REG20_TIMINGS_1_T_GRANULARITY_RX_MASK (0x7 << RF_REG20_TIMINGS_1_T_GRANULARITY_RX_POS)
#define RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_POS 0
#define RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_MASK (0xF << RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_POS)

/* RF_REG20 settings */
#define TIMINGS_3_T_DLL_DEFAULT         (0x0 << RF_REG20_TIMINGS_3_T_DLL_POS)

#define TIMINGS_3_T_PLL_TX_DEFAULT      (0x1 << RF_REG20_TIMINGS_3_T_PLL_TX_POS)

#define TIMINGS_2_T_SUBBAND_TX_DEFAULT  (0xF << RF_REG20_TIMINGS_2_T_SUBBAND_TX_POS)

#define TIMINGS_2_T_TX_RF_DEFAULT       (0xFF << RF_REG20_TIMINGS_2_T_TX_RF_POS)

#define TIMINGS_1_T_GRANULARITY_TX_DEFAULT (0x0 << RF_REG20_TIMINGS_1_T_GRANULARITY_TX_POS)

#define TIMINGS_1_T_GRANULARITY_RX_DEFAULT (0x1 << RF_REG20_TIMINGS_1_T_GRANULARITY_RX_POS)

#define AGC_LUT_5_AGC_LEVEL_11_HI_DEFAULT (0xF << RF_REG20_AGC_LUT_5_AGC_LEVEL_11_HI_POS)

/* AGC_ATT1 */
#define RF_AGC_ATT1_BASE                0x40010084
#define RF_AGC_ATT1                     REG32_POINTER(RF_AGC_ATT1_BASE)

/* RF_AGC_ATT1 bit positions */
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_AB_LO_POS 30
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_AB_LO_MASK (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_AB_LO_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_9A_POS 27
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_9A_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_9A_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_89_POS 24
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_89_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_89_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_78_POS 21
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_78_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_78_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_67_POS 18
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_67_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_67_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_56_POS 15
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_56_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_56_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_45_POS 12
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_45_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_45_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_34_POS 9
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_34_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_34_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_23_POS 6
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_23_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_23_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_12_POS 3
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_12_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_12_POS)
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_01_POS 0
#define RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_01_MASK (0x7 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_01_POS)

/* RF_AGC_ATT1 settings */
#define AGC_ATT_1_AGC_ATT_AB_LO_DEFAULT (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_AB_LO_POS)

#define AGC_ATT_1_AGC_ATT_9A_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_9A_POS)

#define AGC_ATT_1_AGC_ATT_89_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_89_POS)

#define AGC_ATT_1_AGC_ATT_78_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_78_POS)

#define AGC_ATT_1_AGC_ATT_67_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_67_POS)

#define AGC_ATT_1_AGC_ATT_56_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_56_POS)

#define AGC_ATT_1_AGC_ATT_45_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_45_POS)

#define AGC_ATT_1_AGC_ATT_34_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_34_POS)

#define AGC_ATT_1_AGC_ATT_23_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_23_POS)

#define AGC_ATT_1_AGC_ATT_12_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_12_POS)

#define AGC_ATT_1_AGC_ATT_01_DEFAULT    (0x3 << RF_AGC_ATT1_AGC_ATT_1_AGC_ATT_01_POS)

/* REG22 */
#define RF_REG22_BASE                   0x40010088
#define RF_REG22                        REG32_POINTER(RF_REG22_BASE)

/* RF_REG22 bit positions */
#define RF_REG22_TIMING_FAST_RX_EN_FAST_RX_TXFILT_POS 29
#define RF_REG22_TIMING_FAST_RX_EN_FAST_RX_POS 28
#define RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_POS 24
#define RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_MASK (0xF << RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_POS)
#define RF_REG22_TIMINGS_5_T_RX_RF_POS  20
#define RF_REG22_TIMINGS_5_T_RX_RF_MASK (0xF << RF_REG22_TIMINGS_5_T_RX_RF_POS)
#define RF_REG22_TIMINGS_5_T_RX_BB_POS  16
#define RF_REG22_TIMINGS_5_T_RX_BB_MASK (0xF << RF_REG22_TIMINGS_5_T_RX_BB_POS)
#define RF_REG22_TIMINGS_4_T_SUBBAND_RX_POS 12
#define RF_REG22_TIMINGS_4_T_SUBBAND_RX_MASK (0xF << RF_REG22_TIMINGS_4_T_SUBBAND_RX_POS)
#define RF_REG22_TIMINGS_4_T_PLL_RX_POS 8
#define RF_REG22_TIMINGS_4_T_PLL_RX_MASK (0xF << RF_REG22_TIMINGS_4_T_PLL_RX_POS)
#define RF_REG22_AGC_ATT_2_AGC_ATT_1DB_POS 1
#define RF_REG22_AGC_ATT_2_AGC_ATT_AB_HI_POS 0

/* RF_REG22 bit-band aliases */
#define RF_REG22_TIMING_FAST_RX_EN_FAST_RX_TXFILT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG22_BASE, RF_REG22_TIMING_FAST_RX_EN_FAST_RX_TXFILT_POS))
#define RF_REG22_TIMING_FAST_RX_EN_FAST_RX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG22_BASE, RF_REG22_TIMING_FAST_RX_EN_FAST_RX_POS))
#define RF_REG22_AGC_ATT_2_AGC_ATT_1DB_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG22_BASE, RF_REG22_AGC_ATT_2_AGC_ATT_1DB_POS))
#define RF_REG22_AGC_ATT_2_AGC_ATT_AB_HI_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG22_BASE, RF_REG22_AGC_ATT_2_AGC_ATT_AB_HI_POS))

/* RF_REG22 settings */
#define TIMING_FAST_RX_EN_FAST_RX_TXFILT_DEFAULT_BITBAND 0x0
#define TIMING_FAST_RX_EN_FAST_RX_TXFILT_DEFAULT (TIMING_FAST_RX_EN_FAST_RX_TXFILT_DEFAULT_BITBAND << RF_REG22_TIMING_FAST_RX_EN_FAST_RX_TXFILT_POS)

#define TIMING_FAST_RX_EN_FAST_RX_DEFAULT_BITBAND 0x0
#define TIMING_FAST_RX_EN_FAST_RX_DEFAULT (TIMING_FAST_RX_EN_FAST_RX_DEFAULT_BITBAND << RF_REG22_TIMING_FAST_RX_EN_FAST_RX_POS)

#define TIMING_FAST_RX_T_RX_FAST_CHP_DEFAULT (0x0 << RF_REG22_TIMING_FAST_RX_T_RX_FAST_CHP_POS)

#define TIMINGS_5_T_RX_RF_DEFAULT       (0x0 << RF_REG22_TIMINGS_5_T_RX_RF_POS)

#define TIMINGS_5_T_RX_BB_DEFAULT       (0x0 << RF_REG22_TIMINGS_5_T_RX_BB_POS)

#define TIMINGS_4_T_SUBBAND_RX_DEFAULT  (0x0 << RF_REG22_TIMINGS_4_T_SUBBAND_RX_POS)

#define TIMINGS_4_T_PLL_RX_DEFAULT      (0x0 << RF_REG22_TIMINGS_4_T_PLL_RX_POS)

#define AGC_ATT_2_AGC_ATT_1DB_DEFAULT_BITBAND 0x0
#define AGC_ATT_2_AGC_ATT_1DB_DEFAULT   (AGC_ATT_2_AGC_ATT_1DB_DEFAULT_BITBAND << RF_REG22_AGC_ATT_2_AGC_ATT_1DB_POS)

#define AGC_ATT_2_AGC_ATT_AB_HI_DEFAULT_BITBAND 0x0
#define AGC_ATT_2_AGC_ATT_AB_HI_DEFAULT (AGC_ATT_2_AGC_ATT_AB_HI_DEFAULT_BITBAND << RF_REG22_AGC_ATT_2_AGC_ATT_AB_HI_POS)

/* REG23 */
#define RF_REG23_BASE                   0x4001008C
#define RF_REG23                        REG32_POINTER(RF_REG23_BASE)

/* RF_REG23 bit positions */
#define RF_REG23_BIAS_1_IQ_RXTX_3_POS   28
#define RF_REG23_BIAS_1_IQ_RXTX_3_MASK  (0xF << RF_REG23_BIAS_1_IQ_RXTX_3_POS)
#define RF_REG23_BIAS_1_IQ_RXTX_2_POS   24
#define RF_REG23_BIAS_1_IQ_RXTX_2_MASK  (0xF << RF_REG23_BIAS_1_IQ_RXTX_2_POS)
#define RF_REG23_BIAS_0_IQ_RXTX_1_POS   20
#define RF_REG23_BIAS_0_IQ_RXTX_1_MASK  (0xF << RF_REG23_BIAS_0_IQ_RXTX_1_POS)
#define RF_REG23_BIAS_0_IQ_RXTX_0_POS   16
#define RF_REG23_BIAS_0_IQ_RXTX_0_MASK  (0xF << RF_REG23_BIAS_0_IQ_RXTX_0_POS)
#define RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_POS 12
#define RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_MASK (0x7 << RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_POS)
#define RF_REG23_INTERFACE_CONF_SPI_SELECT_POS 8
#define RF_REG23_INTERFACE_CONF_SPI_SELECT_MASK (0x3 << RF_REG23_INTERFACE_CONF_SPI_SELECT_POS)
#define RF_REG23_TIMEOUT_EN_RX_TIMEOUT_POS 7
#define RF_REG23_TIMEOUT_T_TIMEOUT_GR_POS 4
#define RF_REG23_TIMEOUT_T_TIMEOUT_GR_MASK (0x7 << RF_REG23_TIMEOUT_T_TIMEOUT_GR_POS)
#define RF_REG23_TIMEOUT_T_RX_TIMEOUT_POS 0
#define RF_REG23_TIMEOUT_T_RX_TIMEOUT_MASK (0xF << RF_REG23_TIMEOUT_T_RX_TIMEOUT_POS)

/* RF_REG23 bit-band aliases */
#define RF_REG23_TIMEOUT_EN_RX_TIMEOUT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG23_BASE, RF_REG23_TIMEOUT_EN_RX_TIMEOUT_POS))

/* RF_REG23 settings */
#define BIAS_1_IQ_RXTX_3_DEFAULT        (0x0 << RF_REG23_BIAS_1_IQ_RXTX_3_POS)

#define BIAS_1_IQ_RXTX_2_DEFAULT        (0x0 << RF_REG23_BIAS_1_IQ_RXTX_2_POS)

#define BIAS_0_IQ_RXTX_1_DEFAULT        (0x0 << RF_REG23_BIAS_0_IQ_RXTX_1_POS)

#define BIAS_0_IQ_RXTX_0_DEFAULT        (0x0 << RF_REG23_BIAS_0_IQ_RXTX_0_POS)

#define INTERFACE_CONF_APB_WAIT_STATE_DEFAULT (0x0 << RF_REG23_INTERFACE_CONF_APB_WAIT_STATE_POS)

#define INTERFACE_CONF_SPI_SELECT_DEFAULT (0x0 << RF_REG23_INTERFACE_CONF_SPI_SELECT_POS)

#define TIMEOUT_EN_RX_TIMEOUT_DEFAULT_BITBAND 0x0
#define TIMEOUT_EN_RX_TIMEOUT_DEFAULT   (TIMEOUT_EN_RX_TIMEOUT_DEFAULT_BITBAND << RF_REG23_TIMEOUT_EN_RX_TIMEOUT_POS)

#define TIMEOUT_T_TIMEOUT_GR_DEFAULT    (0x0 << RF_REG23_TIMEOUT_T_TIMEOUT_GR_POS)

#define TIMEOUT_T_RX_TIMEOUT_DEFAULT    (0x0 << RF_REG23_TIMEOUT_T_RX_TIMEOUT_POS)

/* REG24 */
#define RF_REG24_BASE                   0x40010090
#define RF_REG24                        REG32_POINTER(RF_REG24_BASE)

/* RF_REG24 bit positions */
#define RF_REG24_BIAS_5_IQ_PLL_4_RX_POS 28
#define RF_REG24_BIAS_5_IQ_PLL_4_RX_MASK (0xF << RF_REG24_BIAS_5_IQ_PLL_4_RX_POS)
#define RF_REG24_BIAS_5_IQ_PLL_4_TX_POS 24
#define RF_REG24_BIAS_5_IQ_PLL_4_TX_MASK (0xF << RF_REG24_BIAS_5_IQ_PLL_4_TX_POS)
#define RF_REG24_BIAS_4_IQ_PLL_2_POS    20
#define RF_REG24_BIAS_4_IQ_PLL_2_MASK   (0xF << RF_REG24_BIAS_4_IQ_PLL_2_POS)
#define RF_REG24_BIAS_4_IQ_PLL_1_POS    16
#define RF_REG24_BIAS_4_IQ_PLL_1_MASK   (0xF << RF_REG24_BIAS_4_IQ_PLL_1_POS)
#define RF_REG24_BIAS_3_IQ_RXTX_8_POS   12
#define RF_REG24_BIAS_3_IQ_RXTX_8_MASK  (0xF << RF_REG24_BIAS_3_IQ_RXTX_8_POS)
#define RF_REG24_BIAS_3_IQ_RXTX_7_POS   8
#define RF_REG24_BIAS_3_IQ_RXTX_7_MASK  (0xF << RF_REG24_BIAS_3_IQ_RXTX_7_POS)
#define RF_REG24_BIAS_2_IQ_RXTX_6_POS   4
#define RF_REG24_BIAS_2_IQ_RXTX_6_MASK  (0xF << RF_REG24_BIAS_2_IQ_RXTX_6_POS)
#define RF_REG24_BIAS_2_IQ_RXTX_5_POS   0
#define RF_REG24_BIAS_2_IQ_RXTX_5_MASK  (0xF << RF_REG24_BIAS_2_IQ_RXTX_5_POS)

/* RF_REG24 settings */
#define BIAS_5_IQ_PLL_4_RX_DEFAULT      (0x0 << RF_REG24_BIAS_5_IQ_PLL_4_RX_POS)

#define BIAS_5_IQ_PLL_4_TX_DEFAULT      (0x0 << RF_REG24_BIAS_5_IQ_PLL_4_TX_POS)

#define BIAS_4_IQ_PLL_2_DEFAULT         (0x0 << RF_REG24_BIAS_4_IQ_PLL_2_POS)

#define BIAS_4_IQ_PLL_1_DEFAULT         (0x0 << RF_REG24_BIAS_4_IQ_PLL_1_POS)

#define BIAS_3_IQ_RXTX_8_DEFAULT        (0x0 << RF_REG24_BIAS_3_IQ_RXTX_8_POS)

#define BIAS_3_IQ_RXTX_7_DEFAULT        (0x0 << RF_REG24_BIAS_3_IQ_RXTX_7_POS)

#define BIAS_2_IQ_RXTX_6_DEFAULT        (0x0 << RF_REG24_BIAS_2_IQ_RXTX_6_POS)

#define BIAS_2_IQ_RXTX_5_DEFAULT        (0x0 << RF_REG24_BIAS_2_IQ_RXTX_5_POS)

/* REG25 */
#define RF_REG25_BASE                   0x40010094
#define RF_REG25                        REG32_POINTER(RF_REG25_BASE)

/* RF_REG25 bit positions */
#define RF_REG25_BIAS_9_IQ_BB_6_POS     28
#define RF_REG25_BIAS_9_IQ_BB_6_MASK    (0xF << RF_REG25_BIAS_9_IQ_BB_6_POS)
#define RF_REG25_BIAS_9_IQ_BB_5_POS     24
#define RF_REG25_BIAS_9_IQ_BB_5_MASK    (0xF << RF_REG25_BIAS_9_IQ_BB_5_POS)
#define RF_REG25_BIAS_8_IQ_BB_4_POS     20
#define RF_REG25_BIAS_8_IQ_BB_4_MASK    (0xF << RF_REG25_BIAS_8_IQ_BB_4_POS)
#define RF_REG25_BIAS_8_IQ_BB_3_POS     16
#define RF_REG25_BIAS_8_IQ_BB_3_MASK    (0xF << RF_REG25_BIAS_8_IQ_BB_3_POS)
#define RF_REG25_BIAS_7_IQ_BB_2_POS     12
#define RF_REG25_BIAS_7_IQ_BB_2_MASK    (0xF << RF_REG25_BIAS_7_IQ_BB_2_POS)
#define RF_REG25_BIAS_7_IQ_BB_1_POS     8
#define RF_REG25_BIAS_7_IQ_BB_1_MASK    (0xF << RF_REG25_BIAS_7_IQ_BB_1_POS)
#define RF_REG25_BIAS_6_IQ_BB_0_POS     4
#define RF_REG25_BIAS_6_IQ_BB_0_MASK    (0xF << RF_REG25_BIAS_6_IQ_BB_0_POS)
#define RF_REG25_BIAS_6_IQ_PLL_3_POS    0
#define RF_REG25_BIAS_6_IQ_PLL_3_MASK   (0xF << RF_REG25_BIAS_6_IQ_PLL_3_POS)

/* RF_REG25 settings */
#define BIAS_9_IQ_BB_6_DEFAULT          (0x0 << RF_REG25_BIAS_9_IQ_BB_6_POS)

#define BIAS_9_IQ_BB_5_DEFAULT          (0x0 << RF_REG25_BIAS_9_IQ_BB_5_POS)

#define BIAS_8_IQ_BB_4_DEFAULT          (0x0 << RF_REG25_BIAS_8_IQ_BB_4_POS)

#define BIAS_8_IQ_BB_3_DEFAULT          (0x0 << RF_REG25_BIAS_8_IQ_BB_3_POS)

#define BIAS_7_IQ_BB_2_DEFAULT          (0x0 << RF_REG25_BIAS_7_IQ_BB_2_POS)

#define BIAS_7_IQ_BB_1_DEFAULT          (0x0 << RF_REG25_BIAS_7_IQ_BB_1_POS)

#define BIAS_6_IQ_BB_0_DEFAULT          (0x0 << RF_REG25_BIAS_6_IQ_BB_0_POS)

#define BIAS_6_IQ_PLL_3_DEFAULT         (0x0 << RF_REG25_BIAS_6_IQ_PLL_3_POS)

/* REG26 */
#define RF_REG26_BASE                   0x40010098
#define RF_REG26                        REG32_POINTER(RF_REG26_BASE)

/* RF_REG26 bit positions */
#define RF_REG26_SD_MASH_MASH_ENABLE_POS 28
#define RF_REG26_SD_MASH_MASH_DITHER_POS 27
#define RF_REG26_SD_MASH_MASH_ORDER_POS 25
#define RF_REG26_SD_MASH_MASH_ORDER_MASK (0x3 << RF_REG26_SD_MASH_MASH_ORDER_POS)
#define RF_REG26_SD_MASH_MASH_RSTB_POS  24
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_3_POS 20
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_3_MASK (0xF << RF_REG26_BIAS_12_LNA_AGC_BIAS_3_POS)
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_2_POS 16
#define RF_REG26_BIAS_12_LNA_AGC_BIAS_2_MASK (0xF << RF_REG26_BIAS_12_LNA_AGC_BIAS_2_POS)
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_1_POS 12
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_1_MASK (0xF << RF_REG26_BIAS_11_LNA_AGC_BIAS_1_POS)
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_0_POS 8
#define RF_REG26_BIAS_11_LNA_AGC_BIAS_0_MASK (0xF << RF_REG26_BIAS_11_LNA_AGC_BIAS_0_POS)
#define RF_REG26_BIAS_10_IQ_BB_8_POS    4
#define RF_REG26_BIAS_10_IQ_BB_8_MASK   (0xF << RF_REG26_BIAS_10_IQ_BB_8_POS)
#define RF_REG26_BIAS_10_IQ_BB_7_POS    0
#define RF_REG26_BIAS_10_IQ_BB_7_MASK   (0xF << RF_REG26_BIAS_10_IQ_BB_7_POS)

/* RF_REG26 bit-band aliases */
#define RF_REG26_SD_MASH_MASH_ENABLE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG26_BASE, RF_REG26_SD_MASH_MASH_ENABLE_POS))
#define RF_REG26_SD_MASH_MASH_DITHER_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG26_BASE, RF_REG26_SD_MASH_MASH_DITHER_POS))
#define RF_REG26_SD_MASH_MASH_RSTB_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG26_BASE, RF_REG26_SD_MASH_MASH_RSTB_POS))

/* RF_REG26 settings */
#define SD_MASH_MASH_ENABLE_DEFAULT_BITBAND 0x0
#define SD_MASH_MASH_ENABLE_DEFAULT     (SD_MASH_MASH_ENABLE_DEFAULT_BITBAND << RF_REG26_SD_MASH_MASH_ENABLE_POS)

#define SD_MASH_MASH_DITHER_DEFAULT_BITBAND 0x0
#define SD_MASH_MASH_DITHER_DEFAULT     (SD_MASH_MASH_DITHER_DEFAULT_BITBAND << RF_REG26_SD_MASH_MASH_DITHER_POS)

#define SD_MASH_MASH_ORDER_DEFAULT      (0x0 << RF_REG26_SD_MASH_MASH_ORDER_POS)

#define SD_MASH_MASH_RSTB_DEFAULT_BITBAND 0x0
#define SD_MASH_MASH_RSTB_DEFAULT       (SD_MASH_MASH_RSTB_DEFAULT_BITBAND << RF_REG26_SD_MASH_MASH_RSTB_POS)

#define BIAS_12_LNA_AGC_BIAS_3_DEFAULT  (0x0 << RF_REG26_BIAS_12_LNA_AGC_BIAS_3_POS)

#define BIAS_12_LNA_AGC_BIAS_2_DEFAULT  (0x0 << RF_REG26_BIAS_12_LNA_AGC_BIAS_2_POS)

#define BIAS_11_LNA_AGC_BIAS_1_DEFAULT  (0x0 << RF_REG26_BIAS_11_LNA_AGC_BIAS_1_POS)

#define BIAS_11_LNA_AGC_BIAS_0_DEFAULT  (0x0 << RF_REG26_BIAS_11_LNA_AGC_BIAS_0_POS)

#define BIAS_10_IQ_BB_8_DEFAULT         (0x0 << RF_REG26_BIAS_10_IQ_BB_8_POS)

#define BIAS_10_IQ_BB_7_DEFAULT         (0x0 << RF_REG26_BIAS_10_IQ_BB_7_POS)

/* REG27 */
#define RF_REG27_BASE                   0x4001009C
#define RF_REG27                        REG32_POINTER(RF_REG27_BASE)

/* RF_REG27 bit positions */
#define RF_REG27_CTRL_ADC_ONE_CK_RSSI_PHADC_POS 31
#define RF_REG27_CTRL_ADC_PHADC_DELLATCH_POS 29
#define RF_REG27_CTRL_ADC_PHADC_DELLATCH_MASK (0x3 << RF_REG27_CTRL_ADC_PHADC_DELLATCH_POS)
#define RF_REG27_CTRL_ADC_CTRL_ADC_POS  24
#define RF_REG27_CTRL_ADC_CTRL_ADC_MASK (0x1F << RF_REG27_CTRL_ADC_CTRL_ADC_POS)
#define RF_REG27_BIAS_EN_2_EN_PTAT_POS  19
#define RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_POS 16
#define RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_MASK (0x7 << RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_POS)
#define RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_POS 12
#define RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_MASK (0xF << RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_POS)
#define RF_REG27_BIAS_EN_1_EN_BIAS_PLL_POS 7
#define RF_REG27_BIAS_EN_1_EN_BIAS_PLL_MASK (0x1F << RF_REG27_BIAS_EN_1_EN_BIAS_PLL_POS)
#define RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_POS 0
#define RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_MASK (0x7F << RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_POS)

/* RF_REG27 bit-band aliases */
#define RF_REG27_CTRL_ADC_ONE_CK_RSSI_PHADC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG27_BASE, RF_REG27_CTRL_ADC_ONE_CK_RSSI_PHADC_POS))
#define RF_REG27_BIAS_EN_2_EN_PTAT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG27_BASE, RF_REG27_BIAS_EN_2_EN_PTAT_POS))

/* RF_REG27 settings */
#define CTRL_ADC_ONE_CK_RSSI_PHADC_DEFAULT_BITBAND 0x0
#define CTRL_ADC_ONE_CK_RSSI_PHADC_DEFAULT (CTRL_ADC_ONE_CK_RSSI_PHADC_DEFAULT_BITBAND << RF_REG27_CTRL_ADC_ONE_CK_RSSI_PHADC_POS)

#define CTRL_ADC_PHADC_DELLATCH_DEFAULT (0x0 << RF_REG27_CTRL_ADC_PHADC_DELLATCH_POS)

#define CTRL_ADC_CTRL_ADC_DEFAULT       (0x0 << RF_REG27_CTRL_ADC_CTRL_ADC_POS)

#define BIAS_EN_2_EN_PTAT_DEFAULT_BITBAND 0x0
#define BIAS_EN_2_EN_PTAT_DEFAULT       (BIAS_EN_2_EN_PTAT_DEFAULT_BITBAND << RF_REG27_BIAS_EN_2_EN_PTAT_POS)

#define BIAS_EN_2_EN_BIAS_BB_HI_DEFAULT (0x0 << RF_REG27_BIAS_EN_2_EN_BIAS_BB_HI_POS)

#define BIAS_EN_1_EN_BIAS_BB_LO_DEFAULT (0x0 << RF_REG27_BIAS_EN_1_EN_BIAS_BB_LO_POS)

#define BIAS_EN_1_EN_BIAS_PLL_DEFAULT   (0x0 << RF_REG27_BIAS_EN_1_EN_BIAS_PLL_POS)

#define BIAS_EN_1_EN_BIAS_RXTX_DEFAULT  (0x0 << RF_REG27_BIAS_EN_1_EN_BIAS_RXTX_POS)

/* REG28 */
#define RF_REG28_BASE                   0x400100A0
#define RF_REG28                        REG32_POINTER(RF_REG28_BASE)

/* RF_REG28 bit positions */
#define RF_REG28_CTRL_RX_SWITCH_LP_POS  31
#define RF_REG28_CTRL_RX_USE_PEAK_DETECTOR_POS 30
#define RF_REG28_CTRL_RX_START_MIX_ON_CAL_POS 29
#define RF_REG28_CTRL_RX_CTRL_RX_POS    24
#define RF_REG28_CTRL_RX_CTRL_RX_MASK   (0x1F << RF_REG28_CTRL_RX_CTRL_RX_POS)
#define RF_REG28_SWCAP_FSM_SB_CAP_RX_POS 20
#define RF_REG28_SWCAP_FSM_SB_CAP_RX_MASK (0xF << RF_REG28_SWCAP_FSM_SB_CAP_RX_POS)
#define RF_REG28_SWCAP_FSM_SB_CAP_TX_POS 16
#define RF_REG28_SWCAP_FSM_SB_CAP_TX_MASK (0xF << RF_REG28_SWCAP_FSM_SB_CAP_TX_POS)
#define RF_REG28_DLL_CTRL_CK_LAST_SEL_DELAY_POS 10
#define RF_REG28_DLL_CTRL_CK_FIRST_SEL_DELAY_POS 9
#define RF_REG28_DLL_CTRL_CK_EXT_SEL_POS 8
#define RF_REG28_DLL_CTRL_CK_DIG_EN_POS 7
#define RF_REG28_DLL_CTRL_CK_TEST_EN_POS 6
#define RF_REG28_DLL_CTRL_TOO_FAST_ENB_POS 5
#define RF_REG28_DLL_CTRL_LOCKED_DET_EN_POS 4
#define RF_REG28_DLL_CTRL_LOCKED_AUTO_CHECK_EN_POS 3
#define RF_REG28_DLL_CTRL_FAST_ENB_POS  2
#define RF_REG28_DLL_CTRL_CK_SEL_POS    0
#define RF_REG28_DLL_CTRL_CK_SEL_MASK   (0x3 << RF_REG28_DLL_CTRL_CK_SEL_POS)

/* RF_REG28 bit-band aliases */
#define RF_REG28_CTRL_RX_SWITCH_LP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_CTRL_RX_SWITCH_LP_POS))
#define RF_REG28_CTRL_RX_USE_PEAK_DETECTOR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_CTRL_RX_USE_PEAK_DETECTOR_POS))
#define RF_REG28_CTRL_RX_START_MIX_ON_CAL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_CTRL_RX_START_MIX_ON_CAL_POS))
#define RF_REG28_DLL_CTRL_CK_LAST_SEL_DELAY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_CK_LAST_SEL_DELAY_POS))
#define RF_REG28_DLL_CTRL_CK_FIRST_SEL_DELAY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_CK_FIRST_SEL_DELAY_POS))
#define RF_REG28_DLL_CTRL_CK_EXT_SEL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_CK_EXT_SEL_POS))
#define RF_REG28_DLL_CTRL_CK_DIG_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_CK_DIG_EN_POS))
#define RF_REG28_DLL_CTRL_CK_TEST_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_CK_TEST_EN_POS))
#define RF_REG28_DLL_CTRL_TOO_FAST_ENB_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_TOO_FAST_ENB_POS))
#define RF_REG28_DLL_CTRL_LOCKED_DET_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_LOCKED_DET_EN_POS))
#define RF_REG28_DLL_CTRL_LOCKED_AUTO_CHECK_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_LOCKED_AUTO_CHECK_EN_POS))
#define RF_REG28_DLL_CTRL_FAST_ENB_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG28_BASE, RF_REG28_DLL_CTRL_FAST_ENB_POS))

/* RF_REG28 settings */
#define CTRL_RX_SWITCH_LP_DEFAULT_BITBAND 0x0
#define CTRL_RX_SWITCH_LP_DEFAULT       (CTRL_RX_SWITCH_LP_DEFAULT_BITBAND << RF_REG28_CTRL_RX_SWITCH_LP_POS)

#define CTRL_RX_USE_PEAK_DETECTOR_DEFAULT_BITBAND 0x0
#define CTRL_RX_USE_PEAK_DETECTOR_DEFAULT (CTRL_RX_USE_PEAK_DETECTOR_DEFAULT_BITBAND << RF_REG28_CTRL_RX_USE_PEAK_DETECTOR_POS)

#define CTRL_RX_START_MIX_ON_CAL_DEFAULT_BITBAND 0x0
#define CTRL_RX_START_MIX_ON_CAL_DEFAULT (CTRL_RX_START_MIX_ON_CAL_DEFAULT_BITBAND << RF_REG28_CTRL_RX_START_MIX_ON_CAL_POS)

#define CTRL_RX_CTRL_RX_DEFAULT         (0x0 << RF_REG28_CTRL_RX_CTRL_RX_POS)

#define SWCAP_FSM_SB_CAP_RX_DEFAULT     (0x0 << RF_REG28_SWCAP_FSM_SB_CAP_RX_POS)

#define SWCAP_FSM_SB_CAP_TX_DEFAULT     (0x0 << RF_REG28_SWCAP_FSM_SB_CAP_TX_POS)

#define DLL_CTRL_CK_LAST_SEL_DELAY_DEFAULT_BITBAND 0x0
#define DLL_CTRL_CK_LAST_SEL_DELAY_DEFAULT (DLL_CTRL_CK_LAST_SEL_DELAY_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_LAST_SEL_DELAY_POS)

#define DLL_CTRL_CK_FIRST_SEL_DELAY_DEFAULT_BITBAND 0x0
#define DLL_CTRL_CK_FIRST_SEL_DELAY_DEFAULT (DLL_CTRL_CK_FIRST_SEL_DELAY_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_FIRST_SEL_DELAY_POS)

#define DLL_CTRL_CK_EXT_SEL_DEFAULT_BITBAND 0x0
#define DLL_CTRL_CK_EXT_SEL_DEFAULT     (DLL_CTRL_CK_EXT_SEL_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_EXT_SEL_POS)

#define DLL_CTRL_CK_DIG_EN_DEFAULT_BITBAND 0x0
#define DLL_CTRL_CK_DIG_EN_DEFAULT      (DLL_CTRL_CK_DIG_EN_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_DIG_EN_POS)

#define DLL_CTRL_CK_TEST_EN_DEFAULT_BITBAND 0x0
#define DLL_CTRL_CK_TEST_EN_DEFAULT     (DLL_CTRL_CK_TEST_EN_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_CK_TEST_EN_POS)

#define DLL_CTRL_TOO_FAST_ENB_DEFAULT_BITBAND 0x0
#define DLL_CTRL_TOO_FAST_ENB_DEFAULT   (DLL_CTRL_TOO_FAST_ENB_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_TOO_FAST_ENB_POS)

#define DLL_CTRL_LOCKED_DET_EN_DEFAULT_BITBAND 0x0
#define DLL_CTRL_LOCKED_DET_EN_DEFAULT  (DLL_CTRL_LOCKED_DET_EN_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_LOCKED_DET_EN_POS)

#define DLL_CTRL_LOCKED_AUTO_CHECK_EN_DEFAULT_BITBAND 0x0
#define DLL_CTRL_LOCKED_AUTO_CHECK_EN_DEFAULT (DLL_CTRL_LOCKED_AUTO_CHECK_EN_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_LOCKED_AUTO_CHECK_EN_POS)

#define DLL_CTRL_FAST_ENB_DEFAULT_BITBAND 0x0
#define DLL_CTRL_FAST_ENB_DEFAULT       (DLL_CTRL_FAST_ENB_DEFAULT_BITBAND << RF_REG28_DLL_CTRL_FAST_ENB_POS)

#define DLL_CTRL_CK_SEL_DEFAULT         (0x2 << RF_REG28_DLL_CTRL_CK_SEL_POS)

/* PLL_CTRL */
#define RF_PLL_CTRL_BASE                0x400100A4
#define RF_PLL_CTRL                     REG32_POINTER(RF_PLL_CTRL_BASE)

/* RF_PLL_CTRL bit positions */
#define RF_PLL_CTRL_XTAL_TRIM_XTAL_TRIM_POS 24
#define RF_PLL_CTRL_XTAL_TRIM_XTAL_TRIM_MASK (0xFF << RF_PLL_CTRL_XTAL_TRIM_XTAL_TRIM_POS)
#define RF_PLL_CTRL_PLL_CTRL_2_PLL_RX_48MEG_POS 20
#define RF_PLL_CTRL_PLL_CTRL_2_SWCAP_TX_SAME_RX_POS 19
#define RF_PLL_CTRL_PLL_CTRL_2_SWCAP_FSM_POS 18
#define RF_PLL_CTRL_PLL_CTRL_2_DLL_RSTB_POS 17
#define RF_PLL_CTRL_PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_POS 16
#define RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_POS 13
#define RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_MASK (0x7 << RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_POS)
#define RF_PLL_CTRL_PLL_CTRL_1_SUB_SEL_OFFS_EN_POS 12
#define RF_PLL_CTRL_PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_POS 11
#define RF_PLL_CTRL_PLL_CTRL_1_VCODIV_CLK_TEST_EN_POS 10
#define RF_PLL_CTRL_PLL_CTRL_1_EN_LOW_CHP_BIAS_POS 9
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_DEAD_ZONE_EN_POS 8
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_POS 6
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_MASK (0x3 << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_POS)
#define RF_PLL_CTRL_PLL_CTRL_1_HIGH_BW_FILTER_EN_POS 5
#define RF_PLL_CTRL_PLL_CTRL_1_FAST_CHP_EN_POS 4
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_POS 2
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_MASK (0x3 << RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_POS)
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CMC_EN_POS 1
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_EN_POS 0

/* RF_PLL_CTRL bit-band aliases */
#define RF_PLL_CTRL_PLL_CTRL_2_PLL_RX_48MEG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_2_PLL_RX_48MEG_POS))
#define RF_PLL_CTRL_PLL_CTRL_2_SWCAP_TX_SAME_RX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_2_SWCAP_TX_SAME_RX_POS))
#define RF_PLL_CTRL_PLL_CTRL_2_SWCAP_FSM_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_2_SWCAP_FSM_POS))
#define RF_PLL_CTRL_PLL_CTRL_2_DLL_RSTB_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_2_DLL_RSTB_POS))
#define RF_PLL_CTRL_PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_SUB_SEL_OFFS_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_SUB_SEL_OFFS_EN_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_VCODIV_CLK_TEST_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_VCODIV_CLK_TEST_EN_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_EN_LOW_CHP_BIAS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_EN_LOW_CHP_BIAS_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_DEAD_ZONE_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_CHP_DEAD_ZONE_EN_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_HIGH_BW_FILTER_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_HIGH_BW_FILTER_EN_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_FAST_CHP_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_FAST_CHP_EN_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CMC_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_CHP_CMC_EN_POS))
#define RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_PLL_CTRL_BASE, RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_EN_POS))

/* RF_PLL_CTRL settings */
#define XTAL_TRIM_XTAL_TRIM_DEFAULT     (0x80 << RF_PLL_CTRL_XTAL_TRIM_XTAL_TRIM_POS)

#define PLL_CTRL_2_PLL_RX_48MEG_DEFAULT_BITBAND 0x0
#define PLL_CTRL_2_PLL_RX_48MEG_DEFAULT (PLL_CTRL_2_PLL_RX_48MEG_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_PLL_RX_48MEG_POS)

#define PLL_CTRL_2_SWCAP_TX_SAME_RX_DEFAULT_BITBAND 0x0
#define PLL_CTRL_2_SWCAP_TX_SAME_RX_DEFAULT (PLL_CTRL_2_SWCAP_TX_SAME_RX_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_SWCAP_TX_SAME_RX_POS)

#define PLL_CTRL_2_SWCAP_FSM_DEFAULT_BITBAND 0x0
#define PLL_CTRL_2_SWCAP_FSM_DEFAULT    (PLL_CTRL_2_SWCAP_FSM_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_SWCAP_FSM_POS)

#define PLL_CTRL_2_DLL_RSTB_DEFAULT_BITBAND 0x0
#define PLL_CTRL_2_DLL_RSTB_DEFAULT     (PLL_CTRL_2_DLL_RSTB_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_DLL_RSTB_POS)

#define PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_DEFAULT_BITBAND 0x0
#define PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_DEFAULT (PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_2_VCO_SUBBAND_TRIM_HI_POS)

#define PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_DEFAULT (0x0 << RF_PLL_CTRL_PLL_CTRL_1_VCO_SUBBAND_TRIM_LO_POS)

#define PLL_CTRL_1_SUB_SEL_OFFS_EN_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_SUB_SEL_OFFS_EN_DEFAULT (PLL_CTRL_1_SUB_SEL_OFFS_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_SUB_SEL_OFFS_EN_POS)

#define PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_DEFAULT (PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_DIV2_CLKVCO_TEST_EN_POS)

#define PLL_CTRL_1_VCODIV_CLK_TEST_EN_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_VCODIV_CLK_TEST_EN_DEFAULT (PLL_CTRL_1_VCODIV_CLK_TEST_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_VCODIV_CLK_TEST_EN_POS)

#define PLL_CTRL_1_EN_LOW_CHP_BIAS_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_EN_LOW_CHP_BIAS_DEFAULT (PLL_CTRL_1_EN_LOW_CHP_BIAS_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_EN_LOW_CHP_BIAS_POS)

#define PLL_CTRL_1_CHP_DEAD_ZONE_EN_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_CHP_DEAD_ZONE_EN_DEFAULT (PLL_CTRL_1_CHP_DEAD_ZONE_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_CHP_DEAD_ZONE_EN_POS)

#define PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_DEFAULT (0x0 << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_TRIM_POS)

#define PLL_CTRL_1_HIGH_BW_FILTER_EN_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_HIGH_BW_FILTER_EN_DEFAULT (PLL_CTRL_1_HIGH_BW_FILTER_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_HIGH_BW_FILTER_EN_POS)

#define PLL_CTRL_1_FAST_CHP_EN_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_FAST_CHP_EN_DEFAULT  (PLL_CTRL_1_FAST_CHP_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_FAST_CHP_EN_POS)

#define PLL_CTRL_1_CHP_MODE_TRIM_DEFAULT (0x0 << RF_PLL_CTRL_PLL_CTRL_1_CHP_MODE_TRIM_POS)

#define PLL_CTRL_1_CHP_CMC_EN_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_CHP_CMC_EN_DEFAULT   (PLL_CTRL_1_CHP_CMC_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_CHP_CMC_EN_POS)

#define PLL_CTRL_1_CHP_CURR_OFFSET_EN_DEFAULT_BITBAND 0x0
#define PLL_CTRL_1_CHP_CURR_OFFSET_EN_DEFAULT (PLL_CTRL_1_CHP_CURR_OFFSET_EN_DEFAULT_BITBAND << RF_PLL_CTRL_PLL_CTRL_1_CHP_CURR_OFFSET_EN_POS)

/* REG2A */
#define RF_REG2A_BASE                   0x400100A8
#define RF_REG2A                        REG32_POINTER(RF_REG2A_BASE)

/* RF_REG2A bit positions */
#define RF_REG2A_ENABLES_SEPARATE_PPA_CASC_POS 28
#define RF_REG2A_ENABLES_EN_RXTX_POS    22
#define RF_REG2A_ENABLES_EN_RXTX_MASK   (0x3F << RF_REG2A_ENABLES_EN_RXTX_POS)
#define RF_REG2A_ENABLES_EN_BB_POS      16
#define RF_REG2A_ENABLES_EN_BB_MASK     (0x3F << RF_REG2A_ENABLES_EN_BB_POS)
#define RF_REG2A_RSSI_TUN_RSSI_TUN_GAIN_POS 13
#define RF_REG2A_RSSI_TUN_RSSI_TUN_GAIN_MASK (0x7 << RF_REG2A_RSSI_TUN_RSSI_TUN_GAIN_POS)
#define RF_REG2A_RSSI_TUN_RSSI_ODD_OFFSET_POS 8
#define RF_REG2A_RSSI_TUN_RSSI_ODD_OFFSET_MASK (0x1F << RF_REG2A_RSSI_TUN_RSSI_ODD_OFFSET_POS)
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_POS 4
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_MASK (0xF << RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_POS)
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_POS 0
#define RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_MASK (0xF << RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_POS)

/* RF_REG2A bit-band aliases */
#define RF_REG2A_ENABLES_SEPARATE_PPA_CASC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2A_BASE, RF_REG2A_ENABLES_SEPARATE_PPA_CASC_POS))

/* RF_REG2A settings */
#define ENABLES_SEPARATE_PPA_CASC_DEFAULT_BITBAND 0x0
#define ENABLES_SEPARATE_PPA_CASC_DEFAULT (ENABLES_SEPARATE_PPA_CASC_DEFAULT_BITBAND << RF_REG2A_ENABLES_SEPARATE_PPA_CASC_POS)

#define ENABLES_EN_RXTX_DEFAULT         (0x0 << RF_REG2A_ENABLES_EN_RXTX_POS)

#define ENABLES_EN_BB_DEFAULT           (0x0 << RF_REG2A_ENABLES_EN_BB_POS)

#define RSSI_TUN_RSSI_TUN_GAIN_DEFAULT  (0x3 << RF_REG2A_RSSI_TUN_RSSI_TUN_GAIN_POS)

#define RSSI_TUN_RSSI_ODD_OFFSET_DEFAULT (0x0 << RF_REG2A_RSSI_TUN_RSSI_ODD_OFFSET_POS)

#define RSSI_TUN_RSSI_EVEN_MAX_DEFAULT  (0x7 << RF_REG2A_RSSI_TUN_RSSI_EVEN_MAX_POS)

#define RSSI_TUN_RSSI_EVEN_MIN_DEFAULT  (0x7 << RF_REG2A_RSSI_TUN_RSSI_EVEN_MIN_POS)

/* XTAL_CTRL */
#define RF_XTAL_CTRL_BASE               0x400100AC
#define RF_XTAL_CTRL                    REG32_POINTER(RF_XTAL_CTRL_BASE)

/* RF_XTAL_CTRL bit positions */
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_POS 28
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_MASK (0xF << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_POS)
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_POS 24
#define RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_MASK (0xF << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_POS)
#define RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_POS 22
#define RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_MASK (0x3 << RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_POS)
#define RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_LOW_POS 20
#define RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_LOW_MASK (0x3 << RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_LOW_POS)
#define RF_XTAL_CTRL_XTAL_CTRL_XTAL_CTRL_BYPASS_POS 18
#define RF_XTAL_CTRL_XTAL_CTRL_DIG_CLK_IN_SEL_POS 17
#define RF_XTAL_CTRL_XTAL_CTRL_XO_EN_B_REG_POS 16
#define RF_XTAL_CTRL_XTAL_CTRL_XTAL_CKDIV_POS 14
#define RF_XTAL_CTRL_XTAL_CTRL_XTAL_CKDIV_MASK (0x3 << RF_XTAL_CTRL_XTAL_CTRL_XTAL_CKDIV_POS)
#define RF_XTAL_CTRL_XTAL_CTRL_CLK_OUT_EN_B_POS 13
#define RF_XTAL_CTRL_XTAL_CTRL_REG_VALUE_SEL_POS 12
#define RF_XTAL_CTRL_XTAL_CTRL_AFTERSTARTUP_CURR_SEL_POS 10
#define RF_XTAL_CTRL_XTAL_CTRL_AFTERSTARTUP_CURR_SEL_MASK (0x3 << RF_XTAL_CTRL_XTAL_CTRL_AFTERSTARTUP_CURR_SEL_POS)
#define RF_XTAL_CTRL_XTAL_CTRL_STARTUP_CURR_SEL_POS 8
#define RF_XTAL_CTRL_XTAL_CTRL_STARTUP_CURR_SEL_MASK (0x3 << RF_XTAL_CTRL_XTAL_CTRL_STARTUP_CURR_SEL_POS)
#define RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_DIG_POS 7
#define RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_PLL_POS 6
#define RF_XTAL_CTRL_XTAL_CTRL_FORCE_CLK_READY_POS 5
#define RF_XTAL_CTRL_XTAL_CTRL_CLK_DIG_EN_B_POS 4
#define RF_XTAL_CTRL_XTAL_CTRL_BUFF_EN_B_POS 3
#define RF_XTAL_CTRL_XTAL_CTRL_HP_MODE_POS 2
#define RF_XTAL_CTRL_XTAL_CTRL_LP_MODE_POS 1
#define RF_XTAL_CTRL_XTAL_CTRL_EXT_CLK_MODE_POS 0

/* RF_XTAL_CTRL bit-band aliases */
#define RF_XTAL_CTRL_XTAL_CTRL_XTAL_CTRL_BYPASS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_XTAL_CTRL_BYPASS_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_DIG_CLK_IN_SEL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_DIG_CLK_IN_SEL_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_XO_EN_B_REG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_XO_EN_B_REG_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_CLK_OUT_EN_B_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_CLK_OUT_EN_B_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_REG_VALUE_SEL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_REG_VALUE_SEL_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_DIG_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_DIG_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_PLL_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_PLL_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_FORCE_CLK_READY_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_FORCE_CLK_READY_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_CLK_DIG_EN_B_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_CLK_DIG_EN_B_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_BUFF_EN_B_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_BUFF_EN_B_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_HP_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_HP_MODE_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_LP_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_LP_MODE_POS))
#define RF_XTAL_CTRL_XTAL_CTRL_EXT_CLK_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_XTAL_CTRL_BASE, RF_XTAL_CTRL_XTAL_CTRL_EXT_CLK_MODE_POS))

/* RF_XTAL_CTRL settings */
#define XTAL_CTRL_XO_THR_HIGH_DEFAULT   (0xC << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_HIGH_POS)

#define XTAL_CTRL_XO_THR_LOW_DEFAULT    (0x3 << RF_XTAL_CTRL_XTAL_CTRL_XO_THR_LOW_POS)

#define XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_DEFAULT (0x2 << RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_HIGH_POS)

#define XTAL_CTRL_XO_A_S_CURR_SEL_LOW_DEFAULT (0x0 << RF_XTAL_CTRL_XTAL_CTRL_XO_A_S_CURR_SEL_LOW_POS)

#define XTAL_CTRL_XTAL_CTRL_BYPASS_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_XTAL_CTRL_BYPASS_DEFAULT (XTAL_CTRL_XTAL_CTRL_BYPASS_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_XTAL_CTRL_BYPASS_POS)

#define XTAL_CTRL_DIG_CLK_IN_SEL_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_DIG_CLK_IN_SEL_DEFAULT (XTAL_CTRL_DIG_CLK_IN_SEL_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_DIG_CLK_IN_SEL_POS)

#define XTAL_CTRL_ENABLE_OSCILLATOR_BITBAND 0x0
#define XTAL_CTRL_DISABLE_OSCILLATOR_BITBAND 0x1
#define XTAL_CTRL_ENABLE_OSCILLATOR     (XTAL_CTRL_ENABLE_OSCILLATOR_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_XO_EN_B_REG_POS)
#define XTAL_CTRL_DISABLE_OSCILLATOR    (XTAL_CTRL_DISABLE_OSCILLATOR_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_XO_EN_B_REG_POS)

#define XTAL_CTRL_XTAL_CKDIV_DEFAULT    (0x0 << RF_XTAL_CTRL_XTAL_CTRL_XTAL_CKDIV_POS)

#define XTAL_CTRL_CLK_OUT_EN_B_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_CLK_OUT_EN_B_DEFAULT  (XTAL_CTRL_CLK_OUT_EN_B_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_CLK_OUT_EN_B_POS)

#define XTAL_CTRL_REG_VALUE_SEL_EXTERNAL_BITBAND 0x0
#define XTAL_CTRL_REG_VALUE_SEL_INTERNAL_BITBAND 0x1
#define XTAL_CTRL_REG_VALUE_SEL_EXTERNAL (XTAL_CTRL_REG_VALUE_SEL_EXTERNAL_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_REG_VALUE_SEL_POS)
#define XTAL_CTRL_REG_VALUE_SEL_INTERNAL (XTAL_CTRL_REG_VALUE_SEL_INTERNAL_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_REG_VALUE_SEL_POS)

#define XTAL_CTRL_AFTERSTARTUP_CURR_SEL_DEFAULT (0x1 << RF_XTAL_CTRL_XTAL_CTRL_AFTERSTARTUP_CURR_SEL_POS)

#define XTAL_CTRL_STARTUP_CURR_SEL_DEFAULT (0x1 << RF_XTAL_CTRL_XTAL_CTRL_STARTUP_CURR_SEL_POS)

#define XTAL_CTRL_INV_CLK_DIG_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_INV_CLK_DIG_DEFAULT   (XTAL_CTRL_INV_CLK_DIG_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_DIG_POS)

#define XTAL_CTRL_INV_CLK_PLL_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_INV_CLK_PLL_DEFAULT   (XTAL_CTRL_INV_CLK_PLL_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_INV_CLK_PLL_POS)

#define XTAL_CTRL_FORCE_CLK_READY_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_FORCE_CLK_READY_DEFAULT (XTAL_CTRL_FORCE_CLK_READY_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_FORCE_CLK_READY_POS)

#define XTAL_CTRL_CLK_DIG_EN_B_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_CLK_DIG_EN_B_DEFAULT  (XTAL_CTRL_CLK_DIG_EN_B_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_CLK_DIG_EN_B_POS)

#define XTAL_CTRL_BUFF_EN_B_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_BUFF_EN_B_DEFAULT     (XTAL_CTRL_BUFF_EN_B_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_BUFF_EN_B_POS)

#define XTAL_CTRL_HP_MODE_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_HP_MODE_DEFAULT       (XTAL_CTRL_HP_MODE_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_HP_MODE_POS)

#define XTAL_CTRL_LP_MODE_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_LP_MODE_DEFAULT       (XTAL_CTRL_LP_MODE_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_LP_MODE_POS)

#define XTAL_CTRL_EXT_CLK_MODE_DEFAULT_BITBAND 0x0
#define XTAL_CTRL_EXT_CLK_MODE_DEFAULT  (XTAL_CTRL_EXT_CLK_MODE_DEFAULT_BITBAND << RF_XTAL_CTRL_XTAL_CTRL_EXT_CLK_MODE_POS)

/* REG2C */
#define RF_REG2C_BASE                   0x400100B0
#define RF_REG2C                        REG32_POINTER(RF_REG2C_BASE)

/* RF_REG2C bit positions */
#define RF_REG2C_SUBBAND_OFFSET_SB_OFFSET_POS 24
#define RF_REG2C_SUBBAND_OFFSET_SB_OFFSET_MASK (0xFF << RF_REG2C_SUBBAND_OFFSET_SB_OFFSET_POS)
#define RF_REG2C_SWCAP_LIM_SB_MAX_VAL_POS 20
#define RF_REG2C_SWCAP_LIM_SB_MAX_VAL_MASK (0xF << RF_REG2C_SWCAP_LIM_SB_MAX_VAL_POS)
#define RF_REG2C_SWCAP_LIM_SB_MIN_VAL_POS 16
#define RF_REG2C_SWCAP_LIM_SB_MIN_VAL_MASK (0xF << RF_REG2C_SWCAP_LIM_SB_MIN_VAL_POS)
#define RF_REG2C_SUBBAND_CONF_SB_FLL_MODE_POS 15
#define RF_REG2C_SUBBAND_CONF_SB_INV_BAND_POS 14
#define RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_POS 12
#define RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_MASK (0x3 << RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_POS)
#define RF_REG2C_SUBBAND_CONF_SB_WAIT_T_POS 10
#define RF_REG2C_SUBBAND_CONF_SB_WAIT_T_MASK (0x3 << RF_REG2C_SUBBAND_CONF_SB_WAIT_T_POS)
#define RF_REG2C_SUBBAND_CONF_SB_MODE_POS 8
#define RF_REG2C_SUBBAND_CONF_SB_MODE_MASK (0x3 << RF_REG2C_SUBBAND_CONF_SB_MODE_POS)
#define RF_REG2C_PA_CONF_SW_CN_POS      4
#define RF_REG2C_PA_CONF_SW_CN_MASK     (0x3 << RF_REG2C_PA_CONF_SW_CN_POS)
#define RF_REG2C_PA_CONF_TX_SWITCHPA_POS 3
#define RF_REG2C_PA_CONF_TX_0DBM_POS    2
#define RF_REG2C_PA_CONF_CTRL_PA_POS    0
#define RF_REG2C_PA_CONF_CTRL_PA_MASK   (0x3 << RF_REG2C_PA_CONF_CTRL_PA_POS)

/* RF_REG2C bit-band aliases */
#define RF_REG2C_SUBBAND_CONF_SB_FLL_MODE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2C_BASE, RF_REG2C_SUBBAND_CONF_SB_FLL_MODE_POS))
#define RF_REG2C_SUBBAND_CONF_SB_INV_BAND_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2C_BASE, RF_REG2C_SUBBAND_CONF_SB_INV_BAND_POS))
#define RF_REG2C_PA_CONF_TX_SWITCHPA_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2C_BASE, RF_REG2C_PA_CONF_TX_SWITCHPA_POS))
#define RF_REG2C_PA_CONF_TX_0DBM_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2C_BASE, RF_REG2C_PA_CONF_TX_0DBM_POS))

/* RF_REG2C settings */
#define SUBBAND_OFFSET_SB_OFFSET_DEFAULT (0x0 << RF_REG2C_SUBBAND_OFFSET_SB_OFFSET_POS)

#define SWCAP_LIM_SB_MAX_VAL_DEFAULT    (0x0 << RF_REG2C_SWCAP_LIM_SB_MAX_VAL_POS)

#define SWCAP_LIM_SB_MIN_VAL_DEFAULT    (0x0 << RF_REG2C_SWCAP_LIM_SB_MIN_VAL_POS)

#define SUBBAND_CONF_SB_FLL_MODE_DEFAULT_BITBAND 0x0
#define SUBBAND_CONF_SB_FLL_MODE_DEFAULT (SUBBAND_CONF_SB_FLL_MODE_DEFAULT_BITBAND << RF_REG2C_SUBBAND_CONF_SB_FLL_MODE_POS)

#define SUBBAND_CONF_SB_INV_BAND_DEFAULT_BITBAND 0x0
#define SUBBAND_CONF_SB_INV_BAND_DEFAULT (SUBBAND_CONF_SB_INV_BAND_DEFAULT_BITBAND << RF_REG2C_SUBBAND_CONF_SB_INV_BAND_POS)

#define SUBBAND_CONF_SB_FREQ_CNT_DEFAULT (0x0 << RF_REG2C_SUBBAND_CONF_SB_FREQ_CNT_POS)

#define SUBBAND_CONF_SB_WAIT_T_DEFAULT  (0x0 << RF_REG2C_SUBBAND_CONF_SB_WAIT_T_POS)

#define SUBBAND_CONF_SB_MODE_DEFAULT    (0x0 << RF_REG2C_SUBBAND_CONF_SB_MODE_POS)

#define PA_CONF_SW_CN_DEFAULT           (0x0 << RF_REG2C_PA_CONF_SW_CN_POS)

#define PA_CONF_TX_SWITCHPA_DEFAULT_BITBAND 0x0
#define PA_CONF_TX_SWITCHPA_DEFAULT     (PA_CONF_TX_SWITCHPA_DEFAULT_BITBAND << RF_REG2C_PA_CONF_TX_SWITCHPA_POS)

#define PA_CONF_TX_0DBM_DEFAULT_BITBAND 0x0
#define PA_CONF_TX_0DBM_DEFAULT         (PA_CONF_TX_0DBM_DEFAULT_BITBAND << RF_REG2C_PA_CONF_TX_0DBM_POS)

#define PA_CONF_CTRL_PA_DEFAULT         (0x0 << RF_REG2C_PA_CONF_CTRL_PA_POS)

/* REG2D */
#define RF_REG2D_BASE                   0x400100B4
#define RF_REG2D                        REG32_POINTER(RF_REG2D_BASE)

/* RF_REG2D bit positions */
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_EN_POS 31
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_POS 28
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_MASK (0x7 << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_POS)
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_POS 24
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_MASK (0x7 << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_POS)
#define RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_PLL_TX_POS 23
#define RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_DIG_TX_POS 22
#define RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_POS 20
#define RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_MASK (0x3 << RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_POS)
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_POS 17
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_MASK (0x3 << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_POS)
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_POS 16
#define RF_REG2D_PA_RAMPUP_FULL_PA_RAMPUP_POS 15
#define RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_POS 12
#define RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_MASK (0x7 << RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_POS)
#define RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_POS 10
#define RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_MASK (0x3 << RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_POS)
#define RF_REG2D_PA_RAMPUP_EN_PA_RAMPDOWN_POS 9
#define RF_REG2D_PA_RAMPUP_EN_PA_RAMPUP_POS 8
#define RF_REG2D_MISC_SPARES_POS        3
#define RF_REG2D_MISC_SPARES_MASK       (0x1F << RF_REG2D_MISC_SPARES_POS)
#define RF_REG2D_MISC_RSSI_PRE_ATT_POS  1
#define RF_REG2D_MISC_RSSI_PRE_ATT_MASK (0x3 << RF_REG2D_MISC_RSSI_PRE_ATT_POS)
#define RF_REG2D_MISC_XTAL_LOW_CLK_READY_TH_EN_POS 0

/* RF_REG2D bit-band aliases */
#define RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2D_BASE, RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_EN_POS))
#define RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_PLL_TX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2D_BASE, RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_PLL_TX_POS))
#define RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_DIG_TX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2D_BASE, RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_DIG_TX_POS))
#define RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2D_BASE, RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_POS))
#define RF_REG2D_PA_RAMPUP_FULL_PA_RAMPUP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2D_BASE, RF_REG2D_PA_RAMPUP_FULL_PA_RAMPUP_POS))
#define RF_REG2D_PA_RAMPUP_EN_PA_RAMPDOWN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2D_BASE, RF_REG2D_PA_RAMPUP_EN_PA_RAMPDOWN_POS))
#define RF_REG2D_PA_RAMPUP_EN_PA_RAMPUP_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2D_BASE, RF_REG2D_PA_RAMPUP_EN_PA_RAMPUP_POS))
#define RF_REG2D_MISC_XTAL_LOW_CLK_READY_TH_EN_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2D_BASE, RF_REG2D_MISC_XTAL_LOW_CLK_READY_TH_EN_POS))

/* RF_REG2D settings */
#define SUBBAND_CORR_SUBBAND_CORR_EN_DEFAULT_BITBAND 0x0
#define SUBBAND_CORR_SUBBAND_CORR_EN_DEFAULT (SUBBAND_CORR_SUBBAND_CORR_EN_DEFAULT_BITBAND << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_EN_POS)

#define SUBBAND_CORR_SUBBAND_CORR_RX_DEFAULT (0x0 << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_RX_POS)

#define SUBBAND_CORR_SUBBAND_CORR_TX_DEFAULT (0x0 << RF_REG2D_SUBBAND_CORR_SUBBAND_CORR_TX_POS)

#define PLL_CONF_TX_NRX_INV_CLK_PLL_TX_DEFAULT_BITBAND 0x0
#define PLL_CONF_TX_NRX_INV_CLK_PLL_TX_DEFAULT (PLL_CONF_TX_NRX_INV_CLK_PLL_TX_DEFAULT_BITBAND << RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_PLL_TX_POS)

#define PLL_CONF_TX_NRX_INV_CLK_DIG_TX_DEFAULT_BITBAND 0x0
#define PLL_CONF_TX_NRX_INV_CLK_DIG_TX_DEFAULT (PLL_CONF_TX_NRX_INV_CLK_DIG_TX_DEFAULT_BITBAND << RF_REG2D_PLL_CONF_TX_NRX_INV_CLK_DIG_TX_POS)

#define PLL_CONF_TX_NRX_CK_SEL_TX_DEFAULT (0x3 << RF_REG2D_PLL_CONF_TX_NRX_CK_SEL_TX_POS)

#define PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_DEFAULT (0x0 << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_TRIM_TX_POS)

#define PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_DEFAULT_BITBAND 0x0
#define PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_DEFAULT (PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_DEFAULT_BITBAND << RF_REG2D_PLL_CONF_TX_NRX_CHP_CURR_OFF_EN_TX_POS)

#define PA_RAMPUP_FULL_PA_RAMPUP_DEFAULT_BITBAND 0x0
#define PA_RAMPUP_FULL_PA_RAMPUP_DEFAULT (PA_RAMPUP_FULL_PA_RAMPUP_DEFAULT_BITBAND << RF_REG2D_PA_RAMPUP_FULL_PA_RAMPUP_POS)

#define PA_RAMPUP_DEL_PA_RAMPUP_DEFAULT (0x0 << RF_REG2D_PA_RAMPUP_DEL_PA_RAMPUP_POS)

#define PA_RAMPUP_TAU_PA_RAMPUP_DEFAULT (0x0 << RF_REG2D_PA_RAMPUP_TAU_PA_RAMPUP_POS)

#define PA_RAMPUP_EN_PA_RAMPDOWN_DEFAULT_BITBAND 0x0
#define PA_RAMPUP_EN_PA_RAMPDOWN_DEFAULT (PA_RAMPUP_EN_PA_RAMPDOWN_DEFAULT_BITBAND << RF_REG2D_PA_RAMPUP_EN_PA_RAMPDOWN_POS)

#define PA_RAMPUP_EN_PA_RAMPUP_DEFAULT_BITBAND 0x0
#define PA_RAMPUP_EN_PA_RAMPUP_DEFAULT  (PA_RAMPUP_EN_PA_RAMPUP_DEFAULT_BITBAND << RF_REG2D_PA_RAMPUP_EN_PA_RAMPUP_POS)

#define MISC_SPARES_DEFAULT             (0x0 << RF_REG2D_MISC_SPARES_POS)

#define MISC_RSSI_PRE_ATT_DEFAULT       (0x0 << RF_REG2D_MISC_RSSI_PRE_ATT_POS)

#define MISC_XTAL_LOW_CLK_READY_TH_EN_DEFAULT_BITBAND 0x0
#define MISC_XTAL_LOW_CLK_READY_TH_EN_DEFAULT (MISC_XTAL_LOW_CLK_READY_TH_EN_DEFAULT_BITBAND << RF_REG2D_MISC_XTAL_LOW_CLK_READY_TH_EN_POS)

/* REG2E */
#define RF_REG2E_BASE                   0x400100B8
#define RF_REG2E                        REG32_POINTER(RF_REG2E_BASE)

/* RF_REG2E bit positions */
#define RF_REG2E_RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_POS 24
#define RF_REG2E_RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_MASK (0xFF << RF_REG2E_RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_POS)
#define RF_REG2E_RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_POS 16
#define RF_REG2E_RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_MASK (0xFF << RF_REG2E_RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_POS)
#define RF_REG2E_DEMOD_CTRL_EN_DELLINE_SYNC_DET_POS 14
#define RF_REG2E_DEMOD_CTRL_RSSI_DET_FILT_POS 13
#define RF_REG2E_DEMOD_CTRL_EN_FAST_CLK_RECOV_POS 12
#define RF_REG2E_DEMOD_CTRL_EN_MIN_MAX_MF_POS 11
#define RF_REG2E_DEMOD_CTRL_EN_PRE_SYNC_POS 10
#define RF_REG2E_DEMOD_CTRL_BLOCK_RSSI_DET_POS 9
#define RF_REG2E_DEMOD_CTRL_EARLY_FINE_RECOV_POS 8
#define RF_REG2E_RSSI_DETECT_RSSI_DET_CR_LEN_POS 6
#define RF_REG2E_RSSI_DETECT_RSSI_DET_CR_LEN_MASK (0x3 << RF_REG2E_RSSI_DETECT_RSSI_DET_CR_LEN_POS)
#define RF_REG2E_RSSI_DETECT_RSSI_DET_WAIT_POS 4
#define RF_REG2E_RSSI_DETECT_RSSI_DET_WAIT_MASK (0x3 << RF_REG2E_RSSI_DETECT_RSSI_DET_WAIT_POS)
#define RF_REG2E_RSSI_DETECT_RSSI_DET_DIFF_LL_POS 2
#define RF_REG2E_RSSI_DETECT_RSSI_DET_DIFF_LL_MASK (0x3 << RF_REG2E_RSSI_DETECT_RSSI_DET_DIFF_LL_POS)
#define RF_REG2E_RSSI_DETECT_EN_ABS_RSSI_DETECT_POS 1
#define RF_REG2E_RSSI_DETECT_EN_DIFF_RSSI_DETECT_POS 0

/* RF_REG2E bit-band aliases */
#define RF_REG2E_DEMOD_CTRL_EN_DELLINE_SYNC_DET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_DEMOD_CTRL_EN_DELLINE_SYNC_DET_POS))
#define RF_REG2E_DEMOD_CTRL_RSSI_DET_FILT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_DEMOD_CTRL_RSSI_DET_FILT_POS))
#define RF_REG2E_DEMOD_CTRL_EN_FAST_CLK_RECOV_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_DEMOD_CTRL_EN_FAST_CLK_RECOV_POS))
#define RF_REG2E_DEMOD_CTRL_EN_MIN_MAX_MF_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_DEMOD_CTRL_EN_MIN_MAX_MF_POS))
#define RF_REG2E_DEMOD_CTRL_EN_PRE_SYNC_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_DEMOD_CTRL_EN_PRE_SYNC_POS))
#define RF_REG2E_DEMOD_CTRL_BLOCK_RSSI_DET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_DEMOD_CTRL_BLOCK_RSSI_DET_POS))
#define RF_REG2E_DEMOD_CTRL_EARLY_FINE_RECOV_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_DEMOD_CTRL_EARLY_FINE_RECOV_POS))
#define RF_REG2E_RSSI_DETECT_EN_ABS_RSSI_DETECT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_RSSI_DETECT_EN_ABS_RSSI_DETECT_POS))
#define RF_REG2E_RSSI_DETECT_EN_DIFF_RSSI_DETECT_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2E_BASE, RF_REG2E_RSSI_DETECT_EN_DIFF_RSSI_DETECT_POS))

/* RF_REG2E settings */
#define RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_DEFAULT (0x0 << RF_REG2E_RSSI_DETECT_ABS_THR_RSSI_DET_ABS_THR_POS)

#define RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_DEFAULT (0x0 << RF_REG2E_RSSI_DETECT_DIFF_THR_RSSI_DET_DIFF_THR_POS)

#define DEMOD_CTRL_EN_DELLINE_SYNC_DET_DEFAULT_BITBAND 0x0
#define DEMOD_CTRL_EN_DELLINE_SYNC_DET_DEFAULT (DEMOD_CTRL_EN_DELLINE_SYNC_DET_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EN_DELLINE_SYNC_DET_POS)

#define DEMOD_CTRL_RSSI_DET_FILT_DEFAULT_BITBAND 0x0
#define DEMOD_CTRL_RSSI_DET_FILT_DEFAULT (DEMOD_CTRL_RSSI_DET_FILT_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_RSSI_DET_FILT_POS)

#define DEMOD_CTRL_EN_FAST_CLK_RECOV_DEFAULT_BITBAND 0x0
#define DEMOD_CTRL_EN_FAST_CLK_RECOV_DEFAULT (DEMOD_CTRL_EN_FAST_CLK_RECOV_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EN_FAST_CLK_RECOV_POS)

#define DEMOD_CTRL_EN_MIN_MAX_MF_DEFAULT_BITBAND 0x0
#define DEMOD_CTRL_EN_MIN_MAX_MF_DEFAULT (DEMOD_CTRL_EN_MIN_MAX_MF_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EN_MIN_MAX_MF_POS)

#define DEMOD_CTRL_EN_PRE_SYNC_DEFAULT_BITBAND 0x0
#define DEMOD_CTRL_EN_PRE_SYNC_DEFAULT  (DEMOD_CTRL_EN_PRE_SYNC_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EN_PRE_SYNC_POS)

#define DEMOD_CTRL_BLOCK_RSSI_DET_DEFAULT_BITBAND 0x0
#define DEMOD_CTRL_BLOCK_RSSI_DET_DEFAULT (DEMOD_CTRL_BLOCK_RSSI_DET_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_BLOCK_RSSI_DET_POS)

#define DEMOD_CTRL_EARLY_FINE_RECOV_DEFAULT_BITBAND 0x0
#define DEMOD_CTRL_EARLY_FINE_RECOV_DEFAULT (DEMOD_CTRL_EARLY_FINE_RECOV_DEFAULT_BITBAND << RF_REG2E_DEMOD_CTRL_EARLY_FINE_RECOV_POS)

#define RSSI_DETECT_RSSI_DET_CR_LEN_DEFAULT (0x0 << RF_REG2E_RSSI_DETECT_RSSI_DET_CR_LEN_POS)

#define RSSI_DETECT_RSSI_DET_WAIT_DEFAULT (0x0 << RF_REG2E_RSSI_DETECT_RSSI_DET_WAIT_POS)

#define RSSI_DETECT_RSSI_DET_DIFF_LL_DEFAULT (0x0 << RF_REG2E_RSSI_DETECT_RSSI_DET_DIFF_LL_POS)

#define RSSI_DETECT_EN_ABS_RSSI_DETECT_DEFAULT_BITBAND 0x0
#define RSSI_DETECT_EN_ABS_RSSI_DETECT_DEFAULT (RSSI_DETECT_EN_ABS_RSSI_DETECT_DEFAULT_BITBAND << RF_REG2E_RSSI_DETECT_EN_ABS_RSSI_DETECT_POS)

#define RSSI_DETECT_EN_DIFF_RSSI_DETECT_DEFAULT_BITBAND 0x0
#define RSSI_DETECT_EN_DIFF_RSSI_DETECT_DEFAULT (RSSI_DETECT_EN_DIFF_RSSI_DETECT_DEFAULT_BITBAND << RF_REG2E_RSSI_DETECT_EN_DIFF_RSSI_DETECT_POS)

/* REG2F */
#define RF_REG2F_BASE                   0x400100BC
#define RF_REG2F                        REG32_POINTER(RF_REG2F_BASE)

/* RF_REG2F bit positions */
#define RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS 24
#define RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_MASK (0x7 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)
#define RF_REG2F_PADS_PE_DS_GPIO_DS_POS 22
#define RF_REG2F_PADS_PE_DS_GPIO_PE_POS 21
#define RF_REG2F_PADS_PE_DS_NRESET_PE_POS 20
#define RF_REG2F_PADS_PE_DS_SPI_MISO_PE_POS 19
#define RF_REG2F_PADS_PE_DS_SPI_MOSI_PE_POS 18
#define RF_REG2F_PADS_PE_DS_SPI_SCLK_PE_POS 17
#define RF_REG2F_PADS_PE_DS_SPI_CS_N_PE_POS 16
#define RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_POS 14
#define RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_MASK (0x3 << RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_POS)
#define RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_POS 12
#define RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_MASK (0x3 << RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_POS)
#define RF_REG2F_SUBBAND_FLL_SB_FLL_PH_4_N8_POS 11
#define RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_POS 8
#define RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_MASK (0x7 << RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_POS)
#define RF_REG2F_SYNC_WORD_CORR_EN_SYNC_WORD_CORR_POS 7
#define RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_POS 0
#define RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_MASK (0x3F << RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_POS)

/* RF_REG2F bit-band aliases */
#define RF_REG2F_PADS_PE_DS_GPIO_DS_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_PADS_PE_DS_GPIO_DS_POS))
#define RF_REG2F_PADS_PE_DS_GPIO_PE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_PADS_PE_DS_GPIO_PE_POS))
#define RF_REG2F_PADS_PE_DS_NRESET_PE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_PADS_PE_DS_NRESET_PE_POS))
#define RF_REG2F_PADS_PE_DS_SPI_MISO_PE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_PADS_PE_DS_SPI_MISO_PE_POS))
#define RF_REG2F_PADS_PE_DS_SPI_MOSI_PE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_PADS_PE_DS_SPI_MOSI_PE_POS))
#define RF_REG2F_PADS_PE_DS_SPI_SCLK_PE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_PADS_PE_DS_SPI_SCLK_PE_POS))
#define RF_REG2F_PADS_PE_DS_SPI_CS_N_PE_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_PADS_PE_DS_SPI_CS_N_PE_POS))
#define RF_REG2F_SUBBAND_FLL_SB_FLL_PH_4_N8_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_SUBBAND_FLL_SB_FLL_PH_4_N8_POS))
#define RF_REG2F_SYNC_WORD_CORR_EN_SYNC_WORD_CORR_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG2F_BASE, RF_REG2F_SYNC_WORD_CORR_EN_SYNC_WORD_CORR_POS))

/* RF_REG2F settings */
#define CK_DIV_1_6_NO_CLOCK             (0x0 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)
#define CK_DIV_1_6_PRESCALE_1           (0x1 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)
#define CK_DIV_1_6_PRESCALE_2           (0x2 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)
#define CK_DIV_1_6_PRESCALE_3           (0x3 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)
#define CK_DIV_1_6_PRESCALE_4           (0x4 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)
#define CK_DIV_1_6_PRESCALE_5           (0x5 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)
#define CK_DIV_1_6_PRESCALE_6           (0x6 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)
#define CK_DIV_1_6_PRESCALE_7           (0x7 << RF_REG2F_CK_DIV_1_6_CK_DIV_1_6_POS)

#define PADS_PE_DS_GPIO_DS_DEFAULT_BITBAND 0x0
#define PADS_PE_DS_GPIO_DS_DEFAULT      (PADS_PE_DS_GPIO_DS_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_GPIO_DS_POS)

#define PADS_PE_DS_GPIO_PE_DEFAULT_BITBAND 0x0
#define PADS_PE_DS_GPIO_PE_DEFAULT      (PADS_PE_DS_GPIO_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_GPIO_PE_POS)

#define PADS_PE_DS_NRESET_PE_DEFAULT_BITBAND 0x0
#define PADS_PE_DS_NRESET_PE_DEFAULT    (PADS_PE_DS_NRESET_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_NRESET_PE_POS)

#define PADS_PE_DS_SPI_MISO_PE_DEFAULT_BITBAND 0x0
#define PADS_PE_DS_SPI_MISO_PE_DEFAULT  (PADS_PE_DS_SPI_MISO_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_SPI_MISO_PE_POS)

#define PADS_PE_DS_SPI_MOSI_PE_DEFAULT_BITBAND 0x0
#define PADS_PE_DS_SPI_MOSI_PE_DEFAULT  (PADS_PE_DS_SPI_MOSI_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_SPI_MOSI_PE_POS)

#define PADS_PE_DS_SPI_SCLK_PE_DEFAULT_BITBAND 0x0
#define PADS_PE_DS_SPI_SCLK_PE_DEFAULT  (PADS_PE_DS_SPI_SCLK_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_SPI_SCLK_PE_POS)

#define PADS_PE_DS_SPI_CS_N_PE_DEFAULT_BITBAND 0x0
#define PADS_PE_DS_SPI_CS_N_PE_DEFAULT  (PADS_PE_DS_SPI_CS_N_PE_DEFAULT_BITBAND << RF_REG2F_PADS_PE_DS_SPI_CS_N_PE_POS)

#define SUBBAND_FLL_SB_FLL_DITHER_DEFAULT (0x0 << RF_REG2F_SUBBAND_FLL_SB_FLL_DITHER_POS)

#define SUBBAND_FLL_SB_FLL_CIC_TAU_DEFAULT (0x0 << RF_REG2F_SUBBAND_FLL_SB_FLL_CIC_TAU_POS)

#define SUBBAND_FLL_SB_FLL_PH_4_N8_DEFAULT_BITBAND 0x0
#define SUBBAND_FLL_SB_FLL_PH_4_N8_DEFAULT (SUBBAND_FLL_SB_FLL_PH_4_N8_DEFAULT_BITBAND << RF_REG2F_SUBBAND_FLL_SB_FLL_PH_4_N8_POS)

#define SUBBAND_FLL_SB_FLL_WAIT_DEFAULT (0x0 << RF_REG2F_SUBBAND_FLL_SB_FLL_WAIT_POS)

#define SYNC_WORD_CORR_EN_SYNC_WORD_CORR_DEFAULT_BITBAND 0x0
#define SYNC_WORD_CORR_EN_SYNC_WORD_CORR_DEFAULT (SYNC_WORD_CORR_EN_SYNC_WORD_CORR_DEFAULT_BITBAND << RF_REG2F_SYNC_WORD_CORR_EN_SYNC_WORD_CORR_POS)

#define SYNC_WORD_CORR_SYNC_WORD_BIAS_DEFAULT (0x0 << RF_REG2F_SYNC_WORD_CORR_SYNC_WORD_BIAS_POS)

/* REG30 */
#define RF_REG30_BASE                   0x400100C0
#define RF_REG30                        REG32_POINTER(RF_REG30_BASE)

/* RF_REG30 bit positions */
#define RF_REG30_RXFIFO_STATUS_BIST_POS 25
#define RF_REG30_RXFIFO_STATUS_BIST_MASK (0x7F << RF_REG30_RXFIFO_STATUS_BIST_POS)
#define RF_REG30_RXFIFO_STATUS_FLUSH_POS 24
#define RF_REG30_RXFIFO_STATUS_BIST_ERRORS_POS 30
#define RF_REG30_RXFIFO_STATUS_BIST_ERRORS_MASK (0x3 << RF_REG30_RXFIFO_STATUS_BIST_ERRORS_POS)
#define RF_REG30_RXFIFO_STATUS_NEAR_UNDERFLOW_POS 29
#define RF_REG30_RXFIFO_STATUS_NEAR_OVERFLOW_POS 28
#define RF_REG30_RXFIFO_STATUS_UNDERFLOW_POS 27
#define RF_REG30_RXFIFO_STATUS_OVERFLOW_POS 26
#define RF_REG30_RXFIFO_STATUS_FULL_POS 25
#define RF_REG30_RXFIFO_STATUS_EMPTY_POS 24
#define RF_REG30_TXFIFO_STATUS_BIST_POS 17
#define RF_REG30_TXFIFO_STATUS_BIST_MASK (0x7F << RF_REG30_TXFIFO_STATUS_BIST_POS)
#define RF_REG30_TXFIFO_STATUS_FLUSH_POS 16
#define RF_REG30_TXFIFO_STATUS_BIST_ERRORS_POS 22
#define RF_REG30_TXFIFO_STATUS_BIST_ERRORS_MASK (0x3 << RF_REG30_TXFIFO_STATUS_BIST_ERRORS_POS)
#define RF_REG30_TXFIFO_STATUS_NEAR_UNDERFLOW_POS 21
#define RF_REG30_TXFIFO_STATUS_NEAR_OVERFLOW_POS 20
#define RF_REG30_TXFIFO_STATUS_UNDERFLOW_POS 19
#define RF_REG30_TXFIFO_STATUS_OVERFLOW_POS 18
#define RF_REG30_TXFIFO_STATUS_FULL_POS 17
#define RF_REG30_TXFIFO_STATUS_EMPTY_POS 16
#define RF_REG30_FSM_STATUS_TX_NRX_POS  10
#define RF_REG30_FSM_STATUS_STATUS_POS  8
#define RF_REG30_FSM_STATUS_STATUS_MASK (0x3 << RF_REG30_FSM_STATUS_STATUS_POS)
#define RF_REG30_FSM_MODE_RESET_POS     3
#define RF_REG30_FSM_MODE_TX_NRX_POS    2
#define RF_REG30_FSM_MODE_MODE_POS      0
#define RF_REG30_FSM_MODE_MODE_MASK     (0x3 << RF_REG30_FSM_MODE_MODE_POS)
#define RF_REG30_FSM_MODE_RX_MODE_POS   2
#define RF_REG30_FSM_MODE_TX_MODE_POS   1
#define RF_REG30_FSM_MODE_N_IDLE_POS    0

/* RF_REG30 bit-band aliases */
#define RF_REG30_RXFIFO_STATUS_FLUSH_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_RXFIFO_STATUS_FLUSH_POS))
#define RF_REG30_RXFIFO_STATUS_NEAR_UNDERFLOW_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_RXFIFO_STATUS_NEAR_UNDERFLOW_POS))
#define RF_REG30_RXFIFO_STATUS_NEAR_OVERFLOW_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_RXFIFO_STATUS_NEAR_OVERFLOW_POS))
#define RF_REG30_RXFIFO_STATUS_UNDERFLOW_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_RXFIFO_STATUS_UNDERFLOW_POS))
#define RF_REG30_RXFIFO_STATUS_OVERFLOW_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_RXFIFO_STATUS_OVERFLOW_POS))
#define RF_REG30_RXFIFO_STATUS_FULL_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_RXFIFO_STATUS_FULL_POS))
#define RF_REG30_RXFIFO_STATUS_EMPTY_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_RXFIFO_STATUS_EMPTY_POS))
#define RF_REG30_TXFIFO_STATUS_FLUSH_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_TXFIFO_STATUS_FLUSH_POS))
#define RF_REG30_TXFIFO_STATUS_NEAR_UNDERFLOW_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_TXFIFO_STATUS_NEAR_UNDERFLOW_POS))
#define RF_REG30_TXFIFO_STATUS_NEAR_OVERFLOW_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_TXFIFO_STATUS_NEAR_OVERFLOW_POS))
#define RF_REG30_TXFIFO_STATUS_UNDERFLOW_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_TXFIFO_STATUS_UNDERFLOW_POS))
#define RF_REG30_TXFIFO_STATUS_OVERFLOW_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_TXFIFO_STATUS_OVERFLOW_POS))
#define RF_REG30_TXFIFO_STATUS_FULL_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_TXFIFO_STATUS_FULL_POS))
#define RF_REG30_TXFIFO_STATUS_EMPTY_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_TXFIFO_STATUS_EMPTY_POS))
#define RF_REG30_FSM_STATUS_TX_NRX_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_FSM_STATUS_TX_NRX_POS))
#define RF_REG30_FSM_MODE_RESET_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_FSM_MODE_RESET_POS))
#define RF_REG30_FSM_MODE_TX_NRX_BITBAND REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_FSM_MODE_TX_NRX_POS))
#define RF_REG30_FSM_MODE_RX_MODE_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_FSM_MODE_RX_MODE_POS))
#define RF_REG30_FSM_MODE_TX_MODE_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_FSM_MODE_TX_MODE_POS))
#define RF_REG30_FSM_MODE_N_IDLE_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG30_BASE, RF_REG30_FSM_MODE_N_IDLE_POS))

/* REG31 */
#define RF_REG31_BASE                   0x400100C4
#define RF_REG31                        READONLY_REG32_POINTER(RF_REG31_BASE)

/* RF_REG31 bit positions */
#define RF_REG31_RSSI_MAX_RSSI_MAX_POS  24
#define RF_REG31_RSSI_MAX_RSSI_MAX_MASK (0xFF << RF_REG31_RSSI_MAX_RSSI_MAX_POS)
#define RF_REG31_RSSI_MIN_RSSI_MIN_POS  16
#define RF_REG31_RSSI_MIN_RSSI_MIN_MASK (0xFF << RF_REG31_RSSI_MIN_RSSI_MIN_POS)
#define RF_REG31_RXFIFO_COUNT_RX_COUNT_POS 8
#define RF_REG31_RXFIFO_COUNT_RX_COUNT_MASK (0xFF << RF_REG31_RXFIFO_COUNT_RX_COUNT_POS)
#define RF_REG31_TXFIFO_COUNT_TX_COUNT_POS 0
#define RF_REG31_TXFIFO_COUNT_TX_COUNT_MASK (0xFF << RF_REG31_TXFIFO_COUNT_TX_COUNT_POS)

/* REG32 */
#define RF_REG32_BASE                   0x400100C8
#define RF_REG32                        READONLY_REG32_POINTER(RF_REG32_BASE)

/* RF_REG32 bit positions */
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_POS 28
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_MASK (0x7 << RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_PKT_LVL_POS)
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_POS 24
#define RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_MASK (0x7 << RF_REG32_RX_ATT_LEVEL_RX_ATT_LEVEL_POS)
#define RF_REG32_RSSI_AVG_RSSI_AVG_POS  16
#define RF_REG32_RSSI_AVG_RSSI_AVG_MASK (0xFF << RF_REG32_RSSI_AVG_RSSI_AVG_POS)
#define RF_REG32_DR_ERR_IND_DR_ERR_IND_POS 8
#define RF_REG32_DR_ERR_IND_DR_ERR_IND_MASK (0xFF << RF_REG32_DR_ERR_IND_DR_ERR_IND_POS)
#define RF_REG32_RSSI_PKT_RSSI_PKT_POS  0
#define RF_REG32_RSSI_PKT_RSSI_PKT_MASK (0xFF << RF_REG32_RSSI_PKT_RSSI_PKT_POS)

/* TXFIFO */
#define RF_TXFIFO_BASE                  0x400100CC
#define RF_TXFIFO                       REG32_POINTER(RF_TXFIFO_BASE)

/* RF_TXFIFO bit positions */
#define RF_TXFIFO_TXFIFO_TX_DATA_POS    0
#define RF_TXFIFO_TXFIFO_TX_DATA_MASK   (0xFF << RF_TXFIFO_TXFIFO_TX_DATA_POS)

/* RXFIFO */
#define RF_RXFIFO_BASE                  0x400100D0
#define RF_RXFIFO                       READONLY_REG32_POINTER(RF_RXFIFO_BASE)

/* RF_RXFIFO bit positions */
#define RF_RXFIFO_RXFIFO_RX_DATA_POS    0
#define RF_RXFIFO_RXFIFO_RX_DATA_MASK   (0xFF << RF_RXFIFO_RXFIFO_RX_DATA_POS)

/* DESER_STATUS */
#define RF_DESER_STATUS_BASE            0x400100D4
#define RF_DESER_STATUS                 READONLY_REG32_POINTER(RF_DESER_STATUS_BASE)

/* RF_DESER_STATUS bit positions */
#define RF_DESER_STATUS_DESER_STATUS_SIGNAL_RECEIVING_POS 7
#define RF_DESER_STATUS_DESER_STATUS_SYNC_DETECTED_POS 6
#define RF_DESER_STATUS_DESER_STATUS_WAIT_SYNC_POS 5
#define RF_DESER_STATUS_DESER_STATUS_IS_ADDRESS_BR_POS 4
#define RF_DESER_STATUS_DESER_STATUS_PKT_LEN_ERR_POS 3
#define RF_DESER_STATUS_DESER_STATUS_ADDRESS_ERR_POS 2
#define RF_DESER_STATUS_DESER_STATUS_CRC_ERR_POS 1
#define RF_DESER_STATUS_DESER_STATUS_DESER_FINISH_POS 0

/* RF_DESER_STATUS bit-band aliases */
#define RF_DESER_STATUS_DESER_STATUS_SIGNAL_RECEIVING_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_DESER_STATUS_BASE, RF_DESER_STATUS_DESER_STATUS_SIGNAL_RECEIVING_POS))
#define RF_DESER_STATUS_DESER_STATUS_SYNC_DETECTED_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_DESER_STATUS_BASE, RF_DESER_STATUS_DESER_STATUS_SYNC_DETECTED_POS))
#define RF_DESER_STATUS_DESER_STATUS_WAIT_SYNC_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_DESER_STATUS_BASE, RF_DESER_STATUS_DESER_STATUS_WAIT_SYNC_POS))
#define RF_DESER_STATUS_DESER_STATUS_IS_ADDRESS_BR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_DESER_STATUS_BASE, RF_DESER_STATUS_DESER_STATUS_IS_ADDRESS_BR_POS))
#define RF_DESER_STATUS_DESER_STATUS_PKT_LEN_ERR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_DESER_STATUS_BASE, RF_DESER_STATUS_DESER_STATUS_PKT_LEN_ERR_POS))
#define RF_DESER_STATUS_DESER_STATUS_ADDRESS_ERR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_DESER_STATUS_BASE, RF_DESER_STATUS_DESER_STATUS_ADDRESS_ERR_POS))
#define RF_DESER_STATUS_DESER_STATUS_CRC_ERR_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_DESER_STATUS_BASE, RF_DESER_STATUS_DESER_STATUS_CRC_ERR_POS))
#define RF_DESER_STATUS_DESER_STATUS_DESER_FINISH_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_DESER_STATUS_BASE, RF_DESER_STATUS_DESER_STATUS_DESER_FINISH_POS))

/* IRQ_STATUS */
#define RF_IRQ_STATUS_BASE              0x400100D8
#define RF_IRQ_STATUS                   READONLY_REG32_POINTER(RF_IRQ_STATUS_BASE)

/* RF_IRQ_STATUS bit positions */
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RXFIFO_POS 5
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_TXFIFO_POS 4
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_SYNC_POS 3
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RECEIVED_POS 2
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RXSTOP_POS 1
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_TX_POS 0

/* RF_IRQ_STATUS bit-band aliases */
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RXFIFO_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_IRQ_STATUS_BASE, RF_IRQ_STATUS_IRQ_STATUS_FLAG_RXFIFO_POS))
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_TXFIFO_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_IRQ_STATUS_BASE, RF_IRQ_STATUS_IRQ_STATUS_FLAG_TXFIFO_POS))
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_SYNC_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_IRQ_STATUS_BASE, RF_IRQ_STATUS_IRQ_STATUS_FLAG_SYNC_POS))
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RECEIVED_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_IRQ_STATUS_BASE, RF_IRQ_STATUS_IRQ_STATUS_FLAG_RECEIVED_POS))
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_RXSTOP_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_IRQ_STATUS_BASE, RF_IRQ_STATUS_IRQ_STATUS_FLAG_RXSTOP_POS))
#define RF_IRQ_STATUS_IRQ_STATUS_FLAG_TX_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_IRQ_STATUS_BASE, RF_IRQ_STATUS_IRQ_STATUS_FLAG_TX_POS))

/* REG37 */
#define RF_REG37_BASE                   0x400100DC
#define RF_REG37                        READONLY_REG32_POINTER(RF_REG37_BASE)

/* RF_REG37 bit positions */
#define RF_REG37_FEI_PKT_FEI_PKT_POS    16
#define RF_REG37_FEI_PKT_FEI_PKT_MASK   (0xFFFF << RF_REG37_FEI_PKT_FEI_PKT_POS)
#define RF_REG37_FEI_FEI_OUT_POS        0
#define RF_REG37_FEI_FEI_OUT_MASK       (0xFFFF << RF_REG37_FEI_FEI_OUT_POS)

/* REG38 */
#define RF_REG38_BASE                   0x400100E0
#define RF_REG38                        READONLY_REG32_POINTER(RF_REG38_BASE)

/* RF_REG38 bit positions */
#define RF_REG38_LINK_QUAL_PKT_LINK_QUALITY_PKT_POS 24
#define RF_REG38_LINK_QUAL_PKT_LINK_QUALITY_PKT_MASK (0xFF << RF_REG38_LINK_QUAL_PKT_LINK_QUALITY_PKT_POS)
#define RF_REG38_LINK_QUAL_LINK_QUALITY_POS 16
#define RF_REG38_LINK_QUAL_LINK_QUALITY_MASK (0xFF << RF_REG38_LINK_QUAL_LINK_QUALITY_POS)
#define RF_REG38_FEI_AFC_FEI_AFC_POS    0
#define RF_REG38_FEI_AFC_FEI_AFC_MASK   (0xFFFF << RF_REG38_FEI_AFC_FEI_AFC_POS)

/* REG39 */
#define RF_REG39_BASE                   0x400100E4
#define RF_REG39                        READONLY_REG32_POINTER(RF_REG39_BASE)

/* RF_REG39 bit positions */
#define RF_REG39_ANALOG_INFO_XTAL_FINISH_POS 13
#define RF_REG39_ANALOG_INFO_DLL_LOCKED_POS 12
#define RF_REG39_ANALOG_INFO_CLK_DIG_READY_POS 11
#define RF_REG39_ANALOG_INFO_CLK_PLL_READY_POS 10
#define RF_REG39_ANALOG_INFO_SUBBAND_HI_POS 9
#define RF_REG39_ANALOG_INFO_SUBBAND_LO_POS 8
#define RF_REG39_SUBBAND_ERR_SB_FLL_ERR_POS 0
#define RF_REG39_SUBBAND_ERR_SB_FLL_ERR_MASK (0xFF << RF_REG39_SUBBAND_ERR_SB_FLL_ERR_POS)

/* RF_REG39 bit-band aliases */
#define RF_REG39_ANALOG_INFO_XTAL_FINISH_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG39_BASE, RF_REG39_ANALOG_INFO_XTAL_FINISH_POS))
#define RF_REG39_ANALOG_INFO_DLL_LOCKED_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG39_BASE, RF_REG39_ANALOG_INFO_DLL_LOCKED_POS))
#define RF_REG39_ANALOG_INFO_CLK_DIG_READY_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG39_BASE, RF_REG39_ANALOG_INFO_CLK_DIG_READY_POS))
#define RF_REG39_ANALOG_INFO_CLK_PLL_READY_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG39_BASE, RF_REG39_ANALOG_INFO_CLK_PLL_READY_POS))
#define RF_REG39_ANALOG_INFO_SUBBAND_HI_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG39_BASE, RF_REG39_ANALOG_INFO_SUBBAND_HI_POS))
#define RF_REG39_ANALOG_INFO_SUBBAND_LO_BITBAND READONLY_REG32_POINTER(SYS_CALC_BITBAND(RF_REG39_BASE, RF_REG39_ANALOG_INFO_SUBBAND_LO_POS))

/* RF_REG39 settings */
#define ANALOG_INFO_XTAL_TRIM_RUNNING_BITBAND 0x0
#define ANALOG_INFO_XTAL_TRIM_FINISHED_BITBAND 0x1
#define ANALOG_INFO_XTAL_TRIM_RUNNING   (ANALOG_INFO_XTAL_TRIM_RUNNING_BITBAND << RF_REG39_ANALOG_INFO_XTAL_FINISH_POS)
#define ANALOG_INFO_XTAL_TRIM_FINISHED  (ANALOG_INFO_XTAL_TRIM_FINISHED_BITBAND << RF_REG39_ANALOG_INFO_XTAL_FINISH_POS)

#define ANALOG_INFO_DLL_UNLOCKED_BITBAND 0x0
#define ANALOG_INFO_DLL_LOCKED_BITBAND  0x1
#define ANALOG_INFO_DLL_UNLOCKED        (ANALOG_INFO_DLL_UNLOCKED_BITBAND << RF_REG39_ANALOG_INFO_DLL_LOCKED_POS)
#define ANALOG_INFO_DLL_LOCKED          (ANALOG_INFO_DLL_LOCKED_BITBAND << RF_REG39_ANALOG_INFO_DLL_LOCKED_POS)

#define ANALOG_INFO_CLK_DIG_NOT_READY_BITBAND 0x0
#define ANALOG_INFO_CLK_DIG_READY_BITBAND 0x1
#define ANALOG_INFO_CLK_DIG_NOT_READY   (ANALOG_INFO_CLK_DIG_NOT_READY_BITBAND << RF_REG39_ANALOG_INFO_CLK_DIG_READY_POS)
#define ANALOG_INFO_CLK_DIG_READY       (ANALOG_INFO_CLK_DIG_READY_BITBAND << RF_REG39_ANALOG_INFO_CLK_DIG_READY_POS)

/* REVISION */
#define RF_REVISION_BASE                0x400100FC
#define RF_REVISION                     READONLY_REG32_POINTER(RF_REVISION_BASE)

/* RF_REVISION bit positions */
#define RF_REVISION_CHIP_ID_POS         24
#define RF_REVISION_CHIP_ID_MASK        (0x3F << RF_REVISION_CHIP_ID_POS)

/* ----------------------------------------------------------------------------
 * SYSTICK Timer
 * ------------------------------------------------------------------------- */

/* SYSTICK Control and Status Register */
#define SysTick_CTRL_BASE               0xE000E010
#define SysTick_CTRL                    REG32_POINTER(SysTick_CTRL_BASE)

/* SysTick_CTRL bit positions */
#define SysTick_CTRL_COUNTFLAG_POS      16
#define SysTick_CTRL_CLKSOURCE_POS      2
#define SysTick_CTRL_TICKINT_POS        1
#define SysTick_CTRL_ENABLE_POS         0

/* SysTick_CTRL settings */
#define SYSTICK_COUNTFLAG_NOT_ZERO      (0x0 << SysTick_CTRL_COUNTFLAG_POS)
#define SYSTICK_COUNTFLAG_ZERO          (0x1 << SysTick_CTRL_COUNTFLAG_POS)

#define SYSTICK_CLKSOURCE_EXTREF_CLK    (0x0 << SysTick_CTRL_CLKSOURCE_POS)
#define SYSTICK_CLKSOURCE_CORE_CLK      (0x1 << SysTick_CTRL_CLKSOURCE_POS)

#define SYSTICK_TICKINT_DISABLE         (0x0 << SysTick_CTRL_TICKINT_POS)
#define SYSTICK_TICKINT_ENABLE          (0x1 << SysTick_CTRL_TICKINT_POS)

#define SYSTICK_DISABLE                 (0x0 << SysTick_CTRL_ENABLE_POS)
#define SYSTICK_ENABLE                  (0x1 << SysTick_CTRL_ENABLE_POS)

/* SYSTICK Reload Value Register */
#define SysTick_LOAD_BASE               0xE000E014
#define SysTick_LOAD                    REG32_POINTER(SysTick_LOAD_BASE)

/* SysTick_LOAD bit positions */
#define SysTick_LOAD_RELOAD_POS         0
#define SysTick_LOAD_RELOAD_MASK        (0xFFFFFF << SysTick_LOAD_RELOAD_POS)

/* SYSTICK Current Value Register */
#define SysTick_VAL_BASE                0xE000E018
#define SysTick_VAL                     REG32_POINTER(SysTick_VAL_BASE)

/* SysTick_VAL bit positions */
#define SysTick_VAL_CURRENT_POS         0
#define SysTick_VAL_CURRENT_MASK        (0xFFFFFF << SysTick_VAL_CURRENT_POS)

/* SYSTICK Calibration Register */
#define SysTick_CALIB_BASE              0xE000E01C
#define SysTick_CALIB                   READONLY_REG32_POINTER(SysTick_CALIB_BASE)

/* SysTick_CALIB bit positions */
#define SysTick_CALIB_NOREF_POS         31
#define SysTick_CALIB_SKEW_POS          30
#define SysTick_CALIB_TENMS_POS         0
#define SysTick_CALIB_TENMS_MASK        (0xFFFFFF << SysTick_CALIB_TENMS_POS)

/* SysTick_CALIB settings */
#define SYSTICK_NOREF                   (0x1 << SysTick_CALIB_NOREF_POS)
#define SYSTICK_REF                     (0x0 << SysTick_CALIB_NOREF_POS)

#define SYSTICK_SKEW                    (0x1 << SysTick_CALIB_SKEW_POS)
#define SYSTICK_NOSKEW                  (0x0 << SysTick_CALIB_SKEW_POS)

/* ----------------------------------------------------------------------------
 * System Control and ID register not in the SCB
 * ------------------------------------------------------------------------- */

/* Interrupt Controller Type Register */
#define SCnSCB_ICTR_BASE                0xE000E004
#define SCnSCB_ICTR                     READONLY_REG32_POINTER(SCnSCB_ICTR_BASE)

/* SCnSCB_ICTR bit positions */
#define SCnSCB_ICTR_INTLINESNUM_POS     0
#define SCnSCB_ICTR_INTLINESNUM_MASK    (0x1F << SCnSCB_ICTR_INTLINESNUM_POS)

/* SCnSCB_ICTR settings */
#define NVIC_INTLINESNUM_1_32           (0x0 << SCnSCB_ICTR_INTLINESNUM_POS)
#define NVIC_INTLINESNUM_33_64          (0x1 << SCnSCB_ICTR_INTLINESNUM_POS)
#define NVIC_INTLINESNUM_65_96          (0x2 << SCnSCB_ICTR_INTLINESNUM_POS)

/* ----------------------------------------------------------------------------
 * Nested Vector Interrupt Controller
 * ------------------------------------------------------------------------- */

/* NVIC External Interrupt Set Enable Register 0 */
#define NVIC_ISER0_BASE                 0xE000E100
#define NVIC_ISER0                      REG32_POINTER(NVIC_ISER0_BASE)

/* NVIC_ISER0 bit positions */
#define NVIC_ISER0_DMIC_OUT_OD_IN_POS   31
#define NVIC_ISER0_UART_ERROR_POS       30
#define NVIC_ISER0_UART_TX_POS          29
#define NVIC_ISER0_UART_RX_POS          28
#define NVIC_ISER0_I2C_POS              27
#define NVIC_ISER0_SPI1_ERROR_POS       26
#define NVIC_ISER0_SPI1_TX_POS          25
#define NVIC_ISER0_SPI1_RX_POS          24
#define NVIC_ISER0_SPI0_ERROR_POS       23
#define NVIC_ISER0_SPI0_TX_POS          22
#define NVIC_ISER0_SPI0_RX_POS          21
#define NVIC_ISER0_WATCHDOG_POS         20
#define NVIC_ISER0_DIO3_POS             19
#define NVIC_ISER0_DIO2_POS             18
#define NVIC_ISER0_DIO1_POS             17
#define NVIC_ISER0_DIO0_POS             16
#define NVIC_ISER0_DMA7_POS             15
#define NVIC_ISER0_DMA6_POS             14
#define NVIC_ISER0_DMA5_POS             13
#define NVIC_ISER0_DMA4_POS             12
#define NVIC_ISER0_DMA3_POS             11
#define NVIC_ISER0_DMA2_POS             10
#define NVIC_ISER0_DMA1_POS             9
#define NVIC_ISER0_DMA0_POS             8
#define NVIC_ISER0_TIMER3_POS           7
#define NVIC_ISER0_TIMER2_POS           6
#define NVIC_ISER0_TIMER1_POS           5
#define NVIC_ISER0_TIMER0_POS           4
#define NVIC_ISER0_ADC_BATMON_POS       3
#define NVIC_ISER0_RTC_CLOCK_POS        2
#define NVIC_ISER0_RTC_ALARM_POS        1
#define NVIC_ISER0_WAKEUP_POS           0

/* NVIC_ISER0 settings */
#define NVIC_DMIC_OUT_OD_IN_INT_ENABLE  (0x1 << NVIC_ISER0_DMIC_OUT_OD_IN_POS)

#define NVIC_UART_ERROR_INT_ENABLE      (0x1 << NVIC_ISER0_UART_ERROR_POS)

#define NVIC_UART_TX_INT_ENABLE         (0x1 << NVIC_ISER0_UART_TX_POS)

#define NVIC_UART_RX_INT_ENABLE         (0x1 << NVIC_ISER0_UART_RX_POS)

#define NVIC_I2C_INT_ENABLE             (0x1 << NVIC_ISER0_I2C_POS)

#define NVIC_SPI1_ERROR_INT_ENABLE      (0x1 << NVIC_ISER0_SPI1_ERROR_POS)

#define NVIC_SPI1_TX_INT_ENABLE         (0x1 << NVIC_ISER0_SPI1_TX_POS)

#define NVIC_SPI1_RX_INT_ENABLE         (0x1 << NVIC_ISER0_SPI1_RX_POS)

#define NVIC_SPI0_ERROR_INT_ENABLE      (0x1 << NVIC_ISER0_SPI0_ERROR_POS)

#define NVIC_SPI0_TX_INT_ENABLE         (0x1 << NVIC_ISER0_SPI0_TX_POS)

#define NVIC_SPI0_RX_INT_ENABLE         (0x1 << NVIC_ISER0_SPI0_RX_POS)

#define NVIC_WATCHDOG_INT_ENABLE        (0x1 << NVIC_ISER0_WATCHDOG_POS)

#define NVIC_DIO3_INT_ENABLE            (0x1 << NVIC_ISER0_DIO3_POS)

#define NVIC_DIO2_INT_ENABLE            (0x1 << NVIC_ISER0_DIO2_POS)

#define NVIC_DIO1_INT_ENABLE            (0x1 << NVIC_ISER0_DIO1_POS)

#define NVIC_DIO0_INT_ENABLE            (0x1 << NVIC_ISER0_DIO0_POS)

#define NVIC_DMA7_INT_ENABLE            (0x1 << NVIC_ISER0_DMA7_POS)

#define NVIC_DMA6_INT_ENABLE            (0x1 << NVIC_ISER0_DMA6_POS)

#define NVIC_DMA5_INT_ENABLE            (0x1 << NVIC_ISER0_DMA5_POS)

#define NVIC_DMA4_INT_ENABLE            (0x1 << NVIC_ISER0_DMA4_POS)

#define NVIC_DMA3_INT_ENABLE            (0x1 << NVIC_ISER0_DMA3_POS)

#define NVIC_DMA2_INT_ENABLE            (0x1 << NVIC_ISER0_DMA2_POS)

#define NVIC_DMA1_INT_ENABLE            (0x1 << NVIC_ISER0_DMA1_POS)

#define NVIC_DMA0_INT_ENABLE            (0x1 << NVIC_ISER0_DMA0_POS)

#define NVIC_TIMER3_INT_ENABLE          (0x1 << NVIC_ISER0_TIMER3_POS)

#define NVIC_TIMER2_INT_ENABLE          (0x1 << NVIC_ISER0_TIMER2_POS)

#define NVIC_TIMER1_INT_ENABLE          (0x1 << NVIC_ISER0_TIMER1_POS)

#define NVIC_TIMER0_INT_ENABLE          (0x1 << NVIC_ISER0_TIMER0_POS)

#define NVIC_ADC_BATMON_INT_ENABLE      (0x1 << NVIC_ISER0_ADC_BATMON_POS)

#define NVIC_RTC_CLOCK_INT_ENABLE       (0x1 << NVIC_ISER0_RTC_CLOCK_POS)

#define NVIC_RTC_ALARM_INT_ENABLE       (0x1 << NVIC_ISER0_RTC_ALARM_POS)

#define NVIC_WAKEUP_INT_ENABLE          (0x1 << NVIC_ISER0_WAKEUP_POS)

/* NVIC External Interrupt Set Enable Register 1 */
#define NVIC_ISER1_BASE                 0xE000E104
#define NVIC_ISER1                      REG32_POINTER(NVIC_ISER1_BASE)

/* NVIC_ISER1 bit positions */
#define NVIC_ISER1_ASRC_OUT_POS         31
#define NVIC_ISER1_ASRC_IN_POS          30
#define NVIC_ISER1_ASRC_ERROR_POS       29
#define NVIC_ISER1_RF_RXFIFO_POS        28
#define NVIC_ISER1_RF_TXFIFO_POS        27
#define NVIC_ISER1_RF_SYNC_POS          26
#define NVIC_ISER1_RF_RECEIVED_POS      25
#define NVIC_ISER1_RF_RXSTOP_POS        24
#define NVIC_ISER1_RF_TX_POS            23
#define NVIC_ISER1_BLE_COEX_IN_PROCESS_POS 22
#define NVIC_ISER1_BLE_COEX_RX_TX_POS   21
#define NVIC_ISER1_BLE_SW_POS           20
#define NVIC_ISER1_BLE_FINETGTIM_POS    19
#define NVIC_ISER1_BLE_GROSSTGTIM_POS   18
#define NVIC_ISER1_BLE_ERROR_POS        17
#define NVIC_ISER1_BLE_CRYPT_POS        16
#define NVIC_ISER1_BLE_EVENT_POS        15
#define NVIC_ISER1_BLE_RX_POS           14
#define NVIC_ISER1_BLE_SLP_POS          13
#define NVIC_ISER1_BLE_CSCNT_POS        12
#define NVIC_ISER1_DSS7_POS             11
#define NVIC_ISER1_DSS6_POS             10
#define NVIC_ISER1_DSS5_POS             9
#define NVIC_ISER1_DSS4_POS             8
#define NVIC_ISER1_DSS3_POS             7
#define NVIC_ISER1_DSS2_POS             6
#define NVIC_ISER1_DSS1_POS             5
#define NVIC_ISER1_DSS0_POS             4
#define NVIC_ISER1_PCM_ERROR_POS        3
#define NVIC_ISER1_PCM_TX_POS           2
#define NVIC_ISER1_PCM_RX_POS           1
#define NVIC_ISER1_DMIC_OD_ERROR_POS    0

/* NVIC_ISER1 settings */
#define NVIC_ASRC_OUT_INT_ENABLE        (0x1 << NVIC_ISER1_ASRC_OUT_POS)

#define NVIC_ASRC_IN_INT_ENABLE         (0x1 << NVIC_ISER1_ASRC_IN_POS)

#define NVIC_ASRC_ERROR_INT_ENABLE      (0x1 << NVIC_ISER1_ASRC_ERROR_POS)

#define NVIC_RF_RXFIFO_INT_ENABLE       (0x1 << NVIC_ISER1_RF_RXFIFO_POS)

#define NVIC_RF_TXFIFO_INT_ENABLE       (0x1 << NVIC_ISER1_RF_TXFIFO_POS)

#define NVIC_RF_SYNC_INT_ENABLE         (0x1 << NVIC_ISER1_RF_SYNC_POS)

#define NVIC_RF_RECEIVED_INT_ENABLE     (0x1 << NVIC_ISER1_RF_RECEIVED_POS)

#define NVIC_RF_RXSTOP_INT_ENABLE       (0x1 << NVIC_ISER1_RF_RXSTOP_POS)

#define NVIC_RF_TX_INT_ENABLE           (0x1 << NVIC_ISER1_RF_TX_POS)

#define NVIC_BLE_COEX_IN_PROCESS_INT_ENABLE (0x1 << NVIC_ISER1_BLE_COEX_IN_PROCESS_POS)

#define NVIC_BLE_COEX_RX_TX_INT_ENABLE  (0x1 << NVIC_ISER1_BLE_COEX_RX_TX_POS)

#define NVIC_BLE_SW_INT_ENABLE          (0x1 << NVIC_ISER1_BLE_SW_POS)

#define NVIC_BLE_FINETGTIM_INT_ENABLE   (0x1 << NVIC_ISER1_BLE_FINETGTIM_POS)

#define NVIC_BLE_GROSSTGTIM_INT_ENABLE  (0x1 << NVIC_ISER1_BLE_GROSSTGTIM_POS)

#define NVIC_BLE_ERROR_INT_ENABLE       (0x1 << NVIC_ISER1_BLE_ERROR_POS)

#define NVIC_BLE_CRYPT_INT_ENABLE       (0x1 << NVIC_ISER1_BLE_CRYPT_POS)

#define NVIC_BLE_EVENT_INT_ENABLE       (0x1 << NVIC_ISER1_BLE_EVENT_POS)

#define NVIC_BLE_RX_INT_ENABLE          (0x1 << NVIC_ISER1_BLE_RX_POS)

#define NVIC_BLE_SLP_INT_ENABLE         (0x1 << NVIC_ISER1_BLE_SLP_POS)

#define NVIC_BLE_CSCNT_INT_ENABLE       (0x1 << NVIC_ISER1_BLE_CSCNT_POS)

#define NVIC_DSS7_INT_ENABLE            (0x1 << NVIC_ISER1_DSS7_POS)

#define NVIC_DSS6_INT_ENABLE            (0x1 << NVIC_ISER1_DSS6_POS)

#define NVIC_DSS5_INT_ENABLE            (0x1 << NVIC_ISER1_DSS5_POS)

#define NVIC_DSS4_INT_ENABLE            (0x1 << NVIC_ISER1_DSS4_POS)

#define NVIC_DSS3_INT_ENABLE            (0x1 << NVIC_ISER1_DSS3_POS)

#define NVIC_DSS2_INT_ENABLE            (0x1 << NVIC_ISER1_DSS2_POS)

#define NVIC_DSS1_INT_ENABLE            (0x1 << NVIC_ISER1_DSS1_POS)

#define NVIC_DSS0_INT_ENABLE            (0x1 << NVIC_ISER1_DSS0_POS)

#define NVIC_PCM_ERROR_INT_ENABLE       (0x1 << NVIC_ISER1_PCM_ERROR_POS)

#define NVIC_PCM_TX_INT_ENABLE          (0x1 << NVIC_ISER1_PCM_TX_POS)

#define NVIC_PCM_RX_INT_ENABLE          (0x1 << NVIC_ISER1_PCM_RX_POS)

#define NVIC_DMIC_OD_ERROR_INT_ENABLE   (0x1 << NVIC_ISER1_DMIC_OD_ERROR_POS)

/* NVIC External Interrupt Set Enable Register 2 */
#define NVIC_ISER2_BASE                 0xE000E108
#define NVIC_ISER2                      REG32_POINTER(NVIC_ISER2_BASE)

/* NVIC_ISER2 bit positions */
#define NVIC_ISER2_BLE_AUDIO2_POS       8
#define NVIC_ISER2_BLE_AUDIO1_POS       7
#define NVIC_ISER2_BLE_AUDIO0_POS       6
#define NVIC_ISER2_MEM_ERROR_POS        5
#define NVIC_ISER2_FLASH_ECC_POS        4
#define NVIC_ISER2_FLASH_COPY_POS       3
#define NVIC_ISER2_CLKDET_POS           2
#define NVIC_ISER2_AUDIO_SINK_PERIOD_POS 1
#define NVIC_ISER2_AUDIO_SINK_DELAY_POS 0

/* NVIC_ISER2 settings */
#define NVIC_BLE_AUDIO2_INT_ENABLE      (0x1 << NVIC_ISER2_BLE_AUDIO2_POS)

#define NVIC_BLE_AUDIO1_INT_ENABLE      (0x1 << NVIC_ISER2_BLE_AUDIO1_POS)

#define NVIC_BLE_AUDIO0_INT_ENABLE      (0x1 << NVIC_ISER2_BLE_AUDIO0_POS)

#define NVIC_MEM_ERROR_INT_ENABLE       (0x1 << NVIC_ISER2_MEM_ERROR_POS)

#define NVIC_FLASH_ECC_INT_ENABLE       (0x1 << NVIC_ISER2_FLASH_ECC_POS)

#define NVIC_FLASH_COPY_INT_ENABLE      (0x1 << NVIC_ISER2_FLASH_COPY_POS)

#define NVIC_CLKDET_INT_ENABLE          (0x1 << NVIC_ISER2_CLKDET_POS)

#define NVIC_AUDIO_SINK_PERIOD_INT_ENABLE (0x1 << NVIC_ISER2_AUDIO_SINK_PERIOD_POS)

#define NVIC_AUDIO_SINK_DELAY_INT_ENABLE (0x1 << NVIC_ISER2_AUDIO_SINK_DELAY_POS)

/* NVIC External Interrupt Clear Enable Register 0 */
#define NVIC_ICER0_BASE                 0xE000E180
#define NVIC_ICER0                      REG32_POINTER(NVIC_ICER0_BASE)

/* NVIC_ICER0 bit positions */
#define NVIC_ICER0_DMIC_OUT_OD_IN_POS   31
#define NVIC_ICER0_UART_ERROR_POS       30
#define NVIC_ICER0_UART_TX_POS          29
#define NVIC_ICER0_UART_RX_POS          28
#define NVIC_ICER0_I2C_POS              27
#define NVIC_ICER0_SPI1_ERROR_POS       26
#define NVIC_ICER0_SPI1_TX_POS          25
#define NVIC_ICER0_SPI1_RX_POS          24
#define NVIC_ICER0_SPI0_ERROR_POS       23
#define NVIC_ICER0_SPI0_TX_POS          22
#define NVIC_ICER0_SPI0_RX_POS          21
#define NVIC_ICER0_WATCHDOG_POS         20
#define NVIC_ICER0_DIO3_POS             19
#define NVIC_ICER0_DIO2_POS             18
#define NVIC_ICER0_DIO1_POS             17
#define NVIC_ICER0_DIO0_POS             16
#define NVIC_ICER0_DMA7_POS             15
#define NVIC_ICER0_DMA6_POS             14
#define NVIC_ICER0_DMA5_POS             13
#define NVIC_ICER0_DMA4_POS             12
#define NVIC_ICER0_DMA3_POS             11
#define NVIC_ICER0_DMA2_POS             10
#define NVIC_ICER0_DMA1_POS             9
#define NVIC_ICER0_DMA0_POS             8
#define NVIC_ICER0_TIMER3_POS           7
#define NVIC_ICER0_TIMER2_POS           6
#define NVIC_ICER0_TIMER1_POS           5
#define NVIC_ICER0_TIMER0_POS           4
#define NVIC_ICER0_ADC_BATMON_POS       3
#define NVIC_ICER0_RTC_CLOCK_POS        2
#define NVIC_ICER0_RTC_ALARM_POS        1
#define NVIC_ICER0_WAKEUP_POS           0

/* NVIC_ICER0 settings */
#define NVIC_DMIC_OUT_OD_IN_INT_DISABLE (0x1 << NVIC_ICER0_DMIC_OUT_OD_IN_POS)

#define NVIC_UART_ERROR_INT_DISABLE     (0x1 << NVIC_ICER0_UART_ERROR_POS)

#define NVIC_UART_TX_INT_DISABLE        (0x1 << NVIC_ICER0_UART_TX_POS)

#define NVIC_UART_RX_INT_DISABLE        (0x1 << NVIC_ICER0_UART_RX_POS)

#define NVIC_I2C_INT_DISABLE            (0x1 << NVIC_ICER0_I2C_POS)

#define NVIC_SPI1_ERROR_INT_DISABLE     (0x1 << NVIC_ICER0_SPI1_ERROR_POS)

#define NVIC_SPI1_TX_INT_DISABLE        (0x1 << NVIC_ICER0_SPI1_TX_POS)

#define NVIC_SPI1_RX_INT_DISABLE        (0x1 << NVIC_ICER0_SPI1_RX_POS)

#define NVIC_SPI0_ERROR_INT_DISABLE     (0x1 << NVIC_ICER0_SPI0_ERROR_POS)

#define NVIC_SPI0_TX_INT_DISABLE        (0x1 << NVIC_ICER0_SPI0_TX_POS)

#define NVIC_SPI0_RX_INT_DISABLE        (0x1 << NVIC_ICER0_SPI0_RX_POS)

#define NVIC_WATCHDOG_INT_DISABLE       (0x1 << NVIC_ICER0_WATCHDOG_POS)

#define NVIC_DIO3_INT_DISABLE           (0x1 << NVIC_ICER0_DIO3_POS)

#define NVIC_DIO2_INT_DISABLE           (0x1 << NVIC_ICER0_DIO2_POS)

#define NVIC_DIO1_INT_DISABLE           (0x1 << NVIC_ICER0_DIO1_POS)

#define NVIC_DIO0_INT_DISABLE           (0x1 << NVIC_ICER0_DIO0_POS)

#define NVIC_DMA7_INT_DISABLE           (0x1 << NVIC_ICER0_DMA7_POS)

#define NVIC_DMA6_INT_DISABLE           (0x1 << NVIC_ICER0_DMA6_POS)

#define NVIC_DMA5_INT_DISABLE           (0x1 << NVIC_ICER0_DMA5_POS)

#define NVIC_DMA4_INT_DISABLE           (0x1 << NVIC_ICER0_DMA4_POS)

#define NVIC_DMA3_INT_DISABLE           (0x1 << NVIC_ICER0_DMA3_POS)

#define NVIC_DMA2_INT_DISABLE           (0x1 << NVIC_ICER0_DMA2_POS)

#define NVIC_DMA1_INT_DISABLE           (0x1 << NVIC_ICER0_DMA1_POS)

#define NVIC_DMA0_INT_DISABLE           (0x1 << NVIC_ICER0_DMA0_POS)

#define NVIC_TIMER3_INT_DISABLE         (0x1 << NVIC_ICER0_TIMER3_POS)

#define NVIC_TIMER2_INT_DISABLE         (0x1 << NVIC_ICER0_TIMER2_POS)

#define NVIC_TIMER1_INT_DISABLE         (0x1 << NVIC_ICER0_TIMER1_POS)

#define NVIC_TIMER0_INT_DISABLE         (0x1 << NVIC_ICER0_TIMER0_POS)

#define NVIC_ADC_BATMON_INT_DISABLE     (0x1 << NVIC_ICER0_ADC_BATMON_POS)

#define NVIC_RTC_CLOCK_INT_DISABLE      (0x1 << NVIC_ICER0_RTC_CLOCK_POS)

#define NVIC_RTC_ALARM_INT_DISABLE      (0x1 << NVIC_ICER0_RTC_ALARM_POS)

#define NVIC_WAKEUP_INT_DISABLE         (0x1 << NVIC_ICER0_WAKEUP_POS)

/* NVIC External Interrupt Clear Enable Register 1 */
#define NVIC_ICER1_BASE                 0xE000E184
#define NVIC_ICER1                      REG32_POINTER(NVIC_ICER1_BASE)

/* NVIC_ICER1 bit positions */
#define NVIC_ICER1_ASRC_OUT_POS         31
#define NVIC_ICER1_ASRC_IN_POS          30
#define NVIC_ICER1_ASRC_ERROR_POS       29
#define NVIC_ICER1_RF_RXFIFO_POS        28
#define NVIC_ICER1_RF_TXFIFO_POS        27
#define NVIC_ICER1_RF_SYNC_POS          26
#define NVIC_ICER1_RF_RECEIVED_POS      25
#define NVIC_ICER1_RF_RXSTOP_POS        24
#define NVIC_ICER1_RF_TX_POS            23
#define NVIC_ICER1_BLE_COEX_IN_PROCESS_POS 22
#define NVIC_ICER1_BLE_COEX_RX_TX_POS   21
#define NVIC_ICER1_BLE_SW_POS           20
#define NVIC_ICER1_BLE_FINETGTIM_POS    19
#define NVIC_ICER1_BLE_GROSSTGTIM_POS   18
#define NVIC_ICER1_BLE_ERROR_POS        17
#define NVIC_ICER1_BLE_CRYPT_POS        16
#define NVIC_ICER1_BLE_EVENT_POS        15
#define NVIC_ICER1_BLE_RX_POS           14
#define NVIC_ICER1_BLE_SLP_POS          13
#define NVIC_ICER1_BLE_CSCNT_POS        12
#define NVIC_ICER1_DSS7_POS             11
#define NVIC_ICER1_DSS6_POS             10
#define NVIC_ICER1_DSS5_POS             9
#define NVIC_ICER1_DSS4_POS             8
#define NVIC_ICER1_DSS3_POS             7
#define NVIC_ICER1_DSS2_POS             6
#define NVIC_ICER1_DSS1_POS             5
#define NVIC_ICER1_DSS0_POS             4
#define NVIC_ICER1_PCM_ERROR_POS        3
#define NVIC_ICER1_PCM_TX_POS           2
#define NVIC_ICER1_PCM_RX_POS           1
#define NVIC_ICER1_DMIC_OD_ERROR_POS    0

/* NVIC_ICER1 settings */
#define NVIC_ASRC_OUT_INT_DISABLE       (0x1 << NVIC_ICER1_ASRC_OUT_POS)

#define NVIC_ASRC_IN_INT_DISABLE        (0x1 << NVIC_ICER1_ASRC_IN_POS)

#define NVIC_ASRC_ERROR_INT_DISABLE     (0x1 << NVIC_ICER1_ASRC_ERROR_POS)

#define NVIC_RF_RXFIFO_INT_DISABLE      (0x1 << NVIC_ICER1_RF_RXFIFO_POS)

#define NVIC_RF_TXFIFO_INT_DISABLE      (0x1 << NVIC_ICER1_RF_TXFIFO_POS)

#define NVIC_RF_SYNC_INT_DISABLE        (0x1 << NVIC_ICER1_RF_SYNC_POS)

#define NVIC_RF_RECEIVED_INT_DISABLE    (0x1 << NVIC_ICER1_RF_RECEIVED_POS)

#define NVIC_RF_RXSTOP_INT_DISABLE      (0x1 << NVIC_ICER1_RF_RXSTOP_POS)

#define NVIC_RF_TX_INT_DISABLE          (0x1 << NVIC_ICER1_RF_TX_POS)

#define NVIC_BLE_COEX_IN_PROCESS_INT_DISABLE (0x1 << NVIC_ICER1_BLE_COEX_IN_PROCESS_POS)

#define NVIC_BLE_COEX_RX_TX_INT_DISABLE (0x1 << NVIC_ICER1_BLE_COEX_RX_TX_POS)

#define NVIC_BLE_SW_INT_DISABLE         (0x1 << NVIC_ICER1_BLE_SW_POS)

#define NVIC_BLE_FINETGTIM_INT_DISABLE  (0x1 << NVIC_ICER1_BLE_FINETGTIM_POS)

#define NVIC_BLE_GROSSTGTIM_INT_DISABLE (0x1 << NVIC_ICER1_BLE_GROSSTGTIM_POS)

#define NVIC_BLE_ERROR_INT_DISABLE      (0x1 << NVIC_ICER1_BLE_ERROR_POS)

#define NVIC_BLE_CRYPT_INT_DISABLE      (0x1 << NVIC_ICER1_BLE_CRYPT_POS)

#define NVIC_BLE_EVENT_INT_DISABLE      (0x1 << NVIC_ICER1_BLE_EVENT_POS)

#define NVIC_BLE_RX_INT_DISABLE         (0x1 << NVIC_ICER1_BLE_RX_POS)

#define NVIC_BLE_SLP_INT_DISABLE        (0x1 << NVIC_ICER1_BLE_SLP_POS)

#define NVIC_BLE_CSCNT_INT_DISABLE      (0x1 << NVIC_ICER1_BLE_CSCNT_POS)

#define NVIC_DSS7_INT_DISABLE           (0x1 << NVIC_ICER1_DSS7_POS)

#define NVIC_DSS6_INT_DISABLE           (0x1 << NVIC_ICER1_DSS6_POS)

#define NVIC_DSS5_INT_DISABLE           (0x1 << NVIC_ICER1_DSS5_POS)

#define NVIC_DSS4_INT_DISABLE           (0x1 << NVIC_ICER1_DSS4_POS)

#define NVIC_DSS3_INT_DISABLE           (0x1 << NVIC_ICER1_DSS3_POS)

#define NVIC_DSS2_INT_DISABLE           (0x1 << NVIC_ICER1_DSS2_POS)

#define NVIC_DSS1_INT_DISABLE           (0x1 << NVIC_ICER1_DSS1_POS)

#define NVIC_DSS0_INT_DISABLE           (0x1 << NVIC_ICER1_DSS0_POS)

#define NVIC_PCM_ERROR_INT_DISABLE      (0x1 << NVIC_ICER1_PCM_ERROR_POS)

#define NVIC_PCM_TX_INT_DISABLE         (0x1 << NVIC_ICER1_PCM_TX_POS)

#define NVIC_PCM_RX_INT_DISABLE         (0x1 << NVIC_ICER1_PCM_RX_POS)

#define NVIC_DMIC_OD_ERROR_INT_DISABLE  (0x1 << NVIC_ICER1_DMIC_OD_ERROR_POS)

/* NVIC External Interrupt Clear Enable Register 2 */
#define NVIC_ICER2_BASE                 0xE000E188
#define NVIC_ICER2                      REG32_POINTER(NVIC_ICER2_BASE)

/* NVIC_ICER2 bit positions */
#define NVIC_ICER2_BLE_AUDIO2_POS       8
#define NVIC_ICER2_BLE_AUDIO1_POS       7
#define NVIC_ICER2_BLE_AUDIO0_POS       6
#define NVIC_ICER2_MEM_ERROR_POS        5
#define NVIC_ICER2_FLASH_ECC_POS        4
#define NVIC_ICER2_FLASH_COPY_POS       3
#define NVIC_ICER2_CLKDET_POS           2
#define NVIC_ICER2_AUDIO_SINK_PERIOD_POS 1
#define NVIC_ICER2_AUDIO_SINK_DELAY_POS 0

/* NVIC_ICER2 settings */
#define NVIC_BLE_AUDIO2_INT_DISABLE     (0x1 << NVIC_ICER2_BLE_AUDIO2_POS)

#define NVIC_BLE_AUDIO1_INT_DISABLE     (0x1 << NVIC_ICER2_BLE_AUDIO1_POS)

#define NVIC_BLE_AUDIO0_INT_DISABLE     (0x1 << NVIC_ICER2_BLE_AUDIO0_POS)

#define NVIC_MEM_ERROR_INT_DISABLE      (0x1 << NVIC_ICER2_MEM_ERROR_POS)

#define NVIC_FLASH_ECC_INT_DISABLE      (0x1 << NVIC_ICER2_FLASH_ECC_POS)

#define NVIC_FLASH_COPY_INT_DISABLE     (0x1 << NVIC_ICER2_FLASH_COPY_POS)

#define NVIC_CLKDET_INT_DISABLE         (0x1 << NVIC_ICER2_CLKDET_POS)

#define NVIC_AUDIO_SINK_PERIOD_INT_DISABLE (0x1 << NVIC_ICER2_AUDIO_SINK_PERIOD_POS)

#define NVIC_AUDIO_SINK_DELAY_INT_DISABLE (0x1 << NVIC_ICER2_AUDIO_SINK_DELAY_POS)

/* NVIC External Interrupt Set Pending Register 0 */
#define NVIC_ISPR0_BASE                 0xE000E200
#define NVIC_ISPR0                      REG32_POINTER(NVIC_ISPR0_BASE)

/* NVIC_ISPR0 bit positions */
#define NVIC_ISPR0_DMIC_OUT_OD_IN_POS   31
#define NVIC_ISPR0_UART_ERROR_POS       30
#define NVIC_ISPR0_UART_TX_POS          29
#define NVIC_ISPR0_UART_RX_POS          28
#define NVIC_ISPR0_I2C_POS              27
#define NVIC_ISPR0_SPI1_ERROR_POS       26
#define NVIC_ISPR0_SPI1_TX_POS          25
#define NVIC_ISPR0_SPI1_RX_POS          24
#define NVIC_ISPR0_SPI0_ERROR_POS       23
#define NVIC_ISPR0_SPI0_TX_POS          22
#define NVIC_ISPR0_SPI0_RX_POS          21
#define NVIC_ISPR0_WATCHDOG_POS         20
#define NVIC_ISPR0_DIO3_POS             19
#define NVIC_ISPR0_DIO2_POS             18
#define NVIC_ISPR0_DIO1_POS             17
#define NVIC_ISPR0_DIO0_POS             16
#define NVIC_ISPR0_DMA7_POS             15
#define NVIC_ISPR0_DMA6_POS             14
#define NVIC_ISPR0_DMA5_POS             13
#define NVIC_ISPR0_DMA4_POS             12
#define NVIC_ISPR0_DMA3_POS             11
#define NVIC_ISPR0_DMA2_POS             10
#define NVIC_ISPR0_DMA1_POS             9
#define NVIC_ISPR0_DMA0_POS             8
#define NVIC_ISPR0_TIMER3_POS           7
#define NVIC_ISPR0_TIMER2_POS           6
#define NVIC_ISPR0_TIMER1_POS           5
#define NVIC_ISPR0_TIMER0_POS           4
#define NVIC_ISPR0_ADC_BATMON_POS       3
#define NVIC_ISPR0_RTC_CLOCK_POS        2
#define NVIC_ISPR0_RTC_ALARM_POS        1
#define NVIC_ISPR0_WAKEUP_POS           0

/* NVIC_ISPR0 settings */
#define NVIC_DMIC_OUT_OD_IN_INT_SETPEND (0x1 << NVIC_ISPR0_DMIC_OUT_OD_IN_POS)

#define NVIC_UART_ERROR_INT_SETPEND     (0x1 << NVIC_ISPR0_UART_ERROR_POS)

#define NVIC_UART_TX_INT_SETPEND        (0x1 << NVIC_ISPR0_UART_TX_POS)

#define NVIC_UART_RX_INT_SETPEND        (0x1 << NVIC_ISPR0_UART_RX_POS)

#define NVIC_I2C_INT_SETPEND            (0x1 << NVIC_ISPR0_I2C_POS)

#define NVIC_SPI1_ERROR_INT_SETPEND     (0x1 << NVIC_ISPR0_SPI1_ERROR_POS)

#define NVIC_SPI1_TX_INT_SETPEND        (0x1 << NVIC_ISPR0_SPI1_TX_POS)

#define NVIC_SPI1_RX_INT_SETPEND        (0x1 << NVIC_ISPR0_SPI1_RX_POS)

#define NVIC_SPI0_ERROR_INT_SETPEND     (0x1 << NVIC_ISPR0_SPI0_ERROR_POS)

#define NVIC_SPI0_TX_INT_SETPEND        (0x1 << NVIC_ISPR0_SPI0_TX_POS)

#define NVIC_SPI0_RX_INT_SETPEND        (0x1 << NVIC_ISPR0_SPI0_RX_POS)

#define NVIC_WATCHDOG_INT_SETPEND       (0x1 << NVIC_ISPR0_WATCHDOG_POS)

#define NVIC_DIO3_INT_SETPEND           (0x1 << NVIC_ISPR0_DIO3_POS)

#define NVIC_DIO2_INT_SETPEND           (0x1 << NVIC_ISPR0_DIO2_POS)

#define NVIC_DIO1_INT_SETPEND           (0x1 << NVIC_ISPR0_DIO1_POS)

#define NVIC_DIO0_INT_SETPEND           (0x1 << NVIC_ISPR0_DIO0_POS)

#define NVIC_DMA7_INT_SETPEND           (0x1 << NVIC_ISPR0_DMA7_POS)

#define NVIC_DMA6_INT_SETPEND           (0x1 << NVIC_ISPR0_DMA6_POS)

#define NVIC_DMA5_INT_SETPEND           (0x1 << NVIC_ISPR0_DMA5_POS)

#define NVIC_DMA4_INT_SETPEND           (0x1 << NVIC_ISPR0_DMA4_POS)

#define NVIC_DMA3_INT_SETPEND           (0x1 << NVIC_ISPR0_DMA3_POS)

#define NVIC_DMA2_INT_SETPEND           (0x1 << NVIC_ISPR0_DMA2_POS)

#define NVIC_DMA1_INT_SETPEND           (0x1 << NVIC_ISPR0_DMA1_POS)

#define NVIC_DMA0_INT_SETPEND           (0x1 << NVIC_ISPR0_DMA0_POS)

#define NVIC_TIMER3_INT_SETPEND         (0x1 << NVIC_ISPR0_TIMER3_POS)

#define NVIC_TIMER2_INT_SETPEND         (0x1 << NVIC_ISPR0_TIMER2_POS)

#define NVIC_TIMER1_INT_SETPEND         (0x1 << NVIC_ISPR0_TIMER1_POS)

#define NVIC_TIMER0_INT_SETPEND         (0x1 << NVIC_ISPR0_TIMER0_POS)

#define NVIC_ADC_BATMON_INT_SETPEND     (0x1 << NVIC_ISPR0_ADC_BATMON_POS)

#define NVIC_RTC_CLOCK_INT_SETPEND      (0x1 << NVIC_ISPR0_RTC_CLOCK_POS)

#define NVIC_RTC_ALARM_INT_SETPEND      (0x1 << NVIC_ISPR0_RTC_ALARM_POS)

#define NVIC_WAKEUP_INT_SETPEND         (0x1 << NVIC_ISPR0_WAKEUP_POS)

/* NVIC External Interrupt Set Pending Register 1 */
#define NVIC_ISPR1_BASE                 0xE000E204
#define NVIC_ISPR1                      REG32_POINTER(NVIC_ISPR1_BASE)

/* NVIC_ISPR1 bit positions */
#define NVIC_ISPR1_ASRC_OUT_POS         31
#define NVIC_ISPR1_ASRC_IN_POS          30
#define NVIC_ISPR1_ASRC_ERROR_POS       29
#define NVIC_ISPR1_RF_RXFIFO_POS        28
#define NVIC_ISPR1_RF_TXFIFO_POS        27
#define NVIC_ISPR1_RF_SYNC_POS          26
#define NVIC_ISPR1_RF_RECEIVED_POS      25
#define NVIC_ISPR1_RF_RXSTOP_POS        24
#define NVIC_ISPR1_RF_TX_POS            23
#define NVIC_ISPR1_BLE_COEX_IN_PROCESS_POS 22
#define NVIC_ISPR1_BLE_COEX_RX_TX_POS   21
#define NVIC_ISPR1_BLE_SW_POS           20
#define NVIC_ISPR1_BLE_FINETGTIM_POS    19
#define NVIC_ISPR1_BLE_GROSSTGTIM_POS   18
#define NVIC_ISPR1_BLE_ERROR_POS        17
#define NVIC_ISPR1_BLE_CRYPT_POS        16
#define NVIC_ISPR1_BLE_EVENT_POS        15
#define NVIC_ISPR1_BLE_RX_POS           14
#define NVIC_ISPR1_BLE_SLP_POS          13
#define NVIC_ISPR1_BLE_CSCNT_POS        12
#define NVIC_ISPR1_DSS7_POS             11
#define NVIC_ISPR1_DSS6_POS             10
#define NVIC_ISPR1_DSS5_POS             9
#define NVIC_ISPR1_DSS4_POS             8
#define NVIC_ISPR1_DSS3_POS             7
#define NVIC_ISPR1_DSS2_POS             6
#define NVIC_ISPR1_DSS1_POS             5
#define NVIC_ISPR1_DSS0_POS             4
#define NVIC_ISPR1_PCM_ERROR_POS        3
#define NVIC_ISPR1_PCM_TX_POS           2
#define NVIC_ISPR1_PCM_RX_POS           1
#define NVIC_ISPR1_DMIC_OD_ERROR_POS    0

/* NVIC_ISPR1 settings */
#define NVIC_ASRC_OUT_INT_SETPEND       (0x1 << NVIC_ISPR1_ASRC_OUT_POS)

#define NVIC_ASRC_IN_INT_SETPEND        (0x1 << NVIC_ISPR1_ASRC_IN_POS)

#define NVIC_ASRC_ERROR_INT_SETPEND     (0x1 << NVIC_ISPR1_ASRC_ERROR_POS)

#define NVIC_RF_RXFIFO_INT_SETPEND      (0x1 << NVIC_ISPR1_RF_RXFIFO_POS)

#define NVIC_RF_TXFIFO_INT_SETPEND      (0x1 << NVIC_ISPR1_RF_TXFIFO_POS)

#define NVIC_RF_SYNC_INT_SETPEND        (0x1 << NVIC_ISPR1_RF_SYNC_POS)

#define NVIC_RF_RECEIVED_INT_SETPEND    (0x1 << NVIC_ISPR1_RF_RECEIVED_POS)

#define NVIC_RF_RXSTOP_INT_SETPEND      (0x1 << NVIC_ISPR1_RF_RXSTOP_POS)

#define NVIC_RF_TX_INT_SETPEND          (0x1 << NVIC_ISPR1_RF_TX_POS)

#define NVIC_BLE_COEX_IN_PROCESS_INT_SETPEND (0x1 << NVIC_ISPR1_BLE_COEX_IN_PROCESS_POS)

#define NVIC_BLE_COEX_RX_TX_INT_SETPEND (0x1 << NVIC_ISPR1_BLE_COEX_RX_TX_POS)

#define NVIC_BLE_SW_INT_SETPEND         (0x1 << NVIC_ISPR1_BLE_SW_POS)

#define NVIC_BLE_FINETGTIM_INT_SETPEND  (0x1 << NVIC_ISPR1_BLE_FINETGTIM_POS)

#define NVIC_BLE_GROSSTGTIM_INT_SETPEND (0x1 << NVIC_ISPR1_BLE_GROSSTGTIM_POS)

#define NVIC_BLE_ERROR_INT_SETPEND      (0x1 << NVIC_ISPR1_BLE_ERROR_POS)

#define NVIC_BLE_CRYPT_INT_SETPEND      (0x1 << NVIC_ISPR1_BLE_CRYPT_POS)

#define NVIC_BLE_EVENT_INT_SETPEND      (0x1 << NVIC_ISPR1_BLE_EVENT_POS)

#define NVIC_BLE_RX_INT_SETPEND         (0x1 << NVIC_ISPR1_BLE_RX_POS)

#define NVIC_BLE_SLP_INT_SETPEND        (0x1 << NVIC_ISPR1_BLE_SLP_POS)

#define NVIC_BLE_CSCNT_INT_SETPEND      (0x1 << NVIC_ISPR1_BLE_CSCNT_POS)

#define NVIC_DSS7_INT_SETPEND           (0x1 << NVIC_ISPR1_DSS7_POS)

#define NVIC_DSS6_INT_SETPEND           (0x1 << NVIC_ISPR1_DSS6_POS)

#define NVIC_DSS5_INT_SETPEND           (0x1 << NVIC_ISPR1_DSS5_POS)

#define NVIC_DSS4_INT_SETPEND           (0x1 << NVIC_ISPR1_DSS4_POS)

#define NVIC_DSS3_INT_SETPEND           (0x1 << NVIC_ISPR1_DSS3_POS)

#define NVIC_DSS2_INT_SETPEND           (0x1 << NVIC_ISPR1_DSS2_POS)

#define NVIC_DSS1_INT_SETPEND           (0x1 << NVIC_ISPR1_DSS1_POS)

#define NVIC_DSS0_INT_SETPEND           (0x1 << NVIC_ISPR1_DSS0_POS)

#define NVIC_PCM_ERROR_INT_SETPEND      (0x1 << NVIC_ISPR1_PCM_ERROR_POS)

#define NVIC_PCM_TX_INT_SETPEND         (0x1 << NVIC_ISPR1_PCM_TX_POS)

#define NVIC_PCM_RX_INT_SETPEND         (0x1 << NVIC_ISPR1_PCM_RX_POS)

#define NVIC_DMIC_OD_ERROR_INT_SETPEND  (0x1 << NVIC_ISPR1_DMIC_OD_ERROR_POS)

/* NVIC External Interrupt Set Pending Register 2 */
#define NVIC_ISPR2_BASE                 0xE000E208
#define NVIC_ISPR2                      REG32_POINTER(NVIC_ISPR2_BASE)

/* NVIC_ISPR2 bit positions */
#define NVIC_ISPR2_BLE_AUDIO2_POS       8
#define NVIC_ISPR2_BLE_AUDIO1_POS       7
#define NVIC_ISPR2_BLE_AUDIO0_POS       6
#define NVIC_ISPR2_MEM_ERROR_POS        5
#define NVIC_ISPR2_FLASH_ECC_POS        4
#define NVIC_ISPR2_FLASH_COPY_POS       3
#define NVIC_ISPR2_CLKDET_POS           2
#define NVIC_ISPR2_AUDIO_SINK_PERIOD_POS 1
#define NVIC_ISPR2_AUDIO_SINK_DELAY_POS 0

/* NVIC_ISPR2 settings */
#define NVIC_BLE_AUDIO2_INT_SETPEND     (0x1 << NVIC_ISPR2_BLE_AUDIO2_POS)

#define NVIC_BLE_AUDIO1_INT_SETPEND     (0x1 << NVIC_ISPR2_BLE_AUDIO1_POS)

#define NVIC_BLE_AUDIO0_INT_SETPEND     (0x1 << NVIC_ISPR2_BLE_AUDIO0_POS)

#define NVIC_MEM_ERROR_INT_SETPEND      (0x1 << NVIC_ISPR2_MEM_ERROR_POS)

#define NVIC_FLASH_ECC_INT_SETPEND      (0x1 << NVIC_ISPR2_FLASH_ECC_POS)

#define NVIC_FLASH_COPY_INT_SETPEND     (0x1 << NVIC_ISPR2_FLASH_COPY_POS)

#define NVIC_CLKDET_INT_SETPEND         (0x1 << NVIC_ISPR2_CLKDET_POS)

#define NVIC_AUDIO_SINK_PERIOD_INT_SETPEND (0x1 << NVIC_ISPR2_AUDIO_SINK_PERIOD_POS)

#define NVIC_AUDIO_SINK_DELAY_INT_SETPEND (0x1 << NVIC_ISPR2_AUDIO_SINK_DELAY_POS)

/* NVIC External Interrupt Clear Pending Register 0 */
#define NVIC_ICPR0_BASE                 0xE000E280
#define NVIC_ICPR0                      REG32_POINTER(NVIC_ICPR0_BASE)

/* NVIC_ICPR0 bit positions */
#define NVIC_ICPR0_DMIC_OUT_OD_IN_POS   31
#define NVIC_ICPR0_UART_ERROR_POS       30
#define NVIC_ICPR0_UART_TX_POS          29
#define NVIC_ICPR0_UART_RX_POS          28
#define NVIC_ICPR0_I2C_POS              27
#define NVIC_ICPR0_SPI1_ERROR_POS       26
#define NVIC_ICPR0_SPI1_TX_POS          25
#define NVIC_ICPR0_SPI1_RX_POS          24
#define NVIC_ICPR0_SPI0_ERROR_POS       23
#define NVIC_ICPR0_SPI0_TX_POS          22
#define NVIC_ICPR0_SPI0_RX_POS          21
#define NVIC_ICPR0_WATCHDOG_POS         20
#define NVIC_ICPR0_DIO3_POS             19
#define NVIC_ICPR0_DIO2_POS             18
#define NVIC_ICPR0_DIO1_POS             17
#define NVIC_ICPR0_DIO0_POS             16
#define NVIC_ICPR0_DMA7_POS             15
#define NVIC_ICPR0_DMA6_POS             14
#define NVIC_ICPR0_DMA5_POS             13
#define NVIC_ICPR0_DMA4_POS             12
#define NVIC_ICPR0_DMA3_POS             11
#define NVIC_ICPR0_DMA2_POS             10
#define NVIC_ICPR0_DMA1_POS             9
#define NVIC_ICPR0_DMA0_POS             8
#define NVIC_ICPR0_TIMER3_POS           7
#define NVIC_ICPR0_TIMER2_POS           6
#define NVIC_ICPR0_TIMER1_POS           5
#define NVIC_ICPR0_TIMER0_POS           4
#define NVIC_ICPR0_ADC_BATMON_POS       3
#define NVIC_ICPR0_RTC_CLOCK_POS        2
#define NVIC_ICPR0_RTC_ALARM_POS        1
#define NVIC_ICPR0_WAKEUP_POS           0

/* NVIC_ICPR0 settings */
#define NVIC_DMIC_OUT_OD_IN_INT_CLRPEND (0x1 << NVIC_ICPR0_DMIC_OUT_OD_IN_POS)

#define NVIC_UART_ERROR_INT_CLRPEND     (0x1 << NVIC_ICPR0_UART_ERROR_POS)

#define NVIC_UART_TX_INT_CLRPEND        (0x1 << NVIC_ICPR0_UART_TX_POS)

#define NVIC_UART_RX_INT_CLRPEND        (0x1 << NVIC_ICPR0_UART_RX_POS)

#define NVIC_I2C_INT_CLRPEND            (0x1 << NVIC_ICPR0_I2C_POS)

#define NVIC_SPI1_ERROR_INT_CLRPEND     (0x1 << NVIC_ICPR0_SPI1_ERROR_POS)

#define NVIC_SPI1_TX_INT_CLRPEND        (0x1 << NVIC_ICPR0_SPI1_TX_POS)

#define NVIC_SPI1_RX_INT_CLRPEND        (0x1 << NVIC_ICPR0_SPI1_RX_POS)

#define NVIC_SPI0_ERROR_INT_CLRPEND     (0x1 << NVIC_ICPR0_SPI0_ERROR_POS)

#define NVIC_SPI0_TX_INT_CLRPEND        (0x1 << NVIC_ICPR0_SPI0_TX_POS)

#define NVIC_SPI0_RX_INT_CLRPEND        (0x1 << NVIC_ICPR0_SPI0_RX_POS)

#define NVIC_WATCHDOG_INT_CLRPEND       (0x1 << NVIC_ICPR0_WATCHDOG_POS)

#define NVIC_DIO3_INT_CLRPEND           (0x1 << NVIC_ICPR0_DIO3_POS)

#define NVIC_DIO2_INT_CLRPEND           (0x1 << NVIC_ICPR0_DIO2_POS)

#define NVIC_DIO1_INT_CLRPEND           (0x1 << NVIC_ICPR0_DIO1_POS)

#define NVIC_DIO0_INT_CLRPEND           (0x1 << NVIC_ICPR0_DIO0_POS)

#define NVIC_DMA7_INT_CLRPEND           (0x1 << NVIC_ICPR0_DMA7_POS)

#define NVIC_DMA6_INT_CLRPEND           (0x1 << NVIC_ICPR0_DMA6_POS)

#define NVIC_DMA5_INT_CLRPEND           (0x1 << NVIC_ICPR0_DMA5_POS)

#define NVIC_DMA4_INT_CLRPEND           (0x1 << NVIC_ICPR0_DMA4_POS)

#define NVIC_DMA3_INT_CLRPEND           (0x1 << NVIC_ICPR0_DMA3_POS)

#define NVIC_DMA2_INT_CLRPEND           (0x1 << NVIC_ICPR0_DMA2_POS)

#define NVIC_DMA1_INT_CLRPEND           (0x1 << NVIC_ICPR0_DMA1_POS)

#define NVIC_DMA0_INT_CLRPEND           (0x1 << NVIC_ICPR0_DMA0_POS)

#define NVIC_TIMER3_INT_CLRPEND         (0x1 << NVIC_ICPR0_TIMER3_POS)

#define NVIC_TIMER2_INT_CLRPEND         (0x1 << NVIC_ICPR0_TIMER2_POS)

#define NVIC_TIMER1_INT_CLRPEND         (0x1 << NVIC_ICPR0_TIMER1_POS)

#define NVIC_TIMER0_INT_CLRPEND         (0x1 << NVIC_ICPR0_TIMER0_POS)

#define NVIC_ADC_BATMON_INT_CLRPEND     (0x1 << NVIC_ICPR0_ADC_BATMON_POS)

#define NVIC_RTC_CLOCK_INT_CLRPEND      (0x1 << NVIC_ICPR0_RTC_CLOCK_POS)

#define NVIC_RTC_ALARM_INT_CLRPEND      (0x1 << NVIC_ICPR0_RTC_ALARM_POS)

#define NVIC_WAKEUP_INT_CLRPEND         (0x1 << NVIC_ICPR0_WAKEUP_POS)

/* NVIC External Interrupt Clear Pending Register 1 */
#define NVIC_ICPR1_BASE                 0xE000E284
#define NVIC_ICPR1                      REG32_POINTER(NVIC_ICPR1_BASE)

/* NVIC_ICPR1 bit positions */
#define NVIC_ICPR1_ASRC_OUT_POS         31
#define NVIC_ICPR1_ASRC_IN_POS          30
#define NVIC_ICPR1_ASRC_ERROR_POS       29
#define NVIC_ICPR1_RF_RXFIFO_POS        28
#define NVIC_ICPR1_RF_TXFIFO_POS        27
#define NVIC_ICPR1_RF_SYNC_POS          26
#define NVIC_ICPR1_RF_RECEIVED_POS      25
#define NVIC_ICPR1_RF_RXSTOP_POS        24
#define NVIC_ICPR1_RF_TX_POS            23
#define NVIC_ICPR1_BLE_COEX_IN_PROCESS_POS 22
#define NVIC_ICPR1_BLE_COEX_RX_TX_POS   21
#define NVIC_ICPR1_BLE_SW_POS           20
#define NVIC_ICPR1_BLE_FINETGTIM_POS    19
#define NVIC_ICPR1_BLE_GROSSTGTIM_POS   18
#define NVIC_ICPR1_BLE_ERROR_POS        17
#define NVIC_ICPR1_BLE_CRYPT_POS        16
#define NVIC_ICPR1_BLE_EVENT_POS        15
#define NVIC_ICPR1_BLE_RX_POS           14
#define NVIC_ICPR1_BLE_SLP_POS          13
#define NVIC_ICPR1_BLE_CSCNT_POS        12
#define NVIC_ICPR1_DSS7_POS             11
#define NVIC_ICPR1_DSS6_POS             10
#define NVIC_ICPR1_DSS5_POS             9
#define NVIC_ICPR1_DSS4_POS             8
#define NVIC_ICPR1_DSS3_POS             7
#define NVIC_ICPR1_DSS2_POS             6
#define NVIC_ICPR1_DSS1_POS             5
#define NVIC_ICPR1_DSS0_POS             4
#define NVIC_ICPR1_PCM_ERROR_POS        3
#define NVIC_ICPR1_PCM_TX_POS           2
#define NVIC_ICPR1_PCM_RX_POS           1
#define NVIC_ICPR1_DMIC_OD_ERROR_POS    0

/* NVIC_ICPR1 settings */
#define NVIC_ASRC_OUT_INT_CLRPEND       (0x1 << NVIC_ICPR1_ASRC_OUT_POS)

#define NVIC_ASRC_IN_INT_CLRPEND        (0x1 << NVIC_ICPR1_ASRC_IN_POS)

#define NVIC_ASRC_ERROR_INT_CLRPEND     (0x1 << NVIC_ICPR1_ASRC_ERROR_POS)

#define NVIC_RF_RXFIFO_INT_CLRPEND      (0x1 << NVIC_ICPR1_RF_RXFIFO_POS)

#define NVIC_RF_TXFIFO_INT_CLRPEND      (0x1 << NVIC_ICPR1_RF_TXFIFO_POS)

#define NVIC_RF_SYNC_INT_CLRPEND        (0x1 << NVIC_ICPR1_RF_SYNC_POS)

#define NVIC_RF_RECEIVED_INT_CLRPEND    (0x1 << NVIC_ICPR1_RF_RECEIVED_POS)

#define NVIC_RF_RXSTOP_INT_CLRPEND      (0x1 << NVIC_ICPR1_RF_RXSTOP_POS)

#define NVIC_RF_TX_INT_CLRPEND          (0x1 << NVIC_ICPR1_RF_TX_POS)

#define NVIC_BLE_COEX_IN_PROCESS_INT_CLRPEND (0x1 << NVIC_ICPR1_BLE_COEX_IN_PROCESS_POS)

#define NVIC_BLE_COEX_RX_TX_INT_CLRPEND (0x1 << NVIC_ICPR1_BLE_COEX_RX_TX_POS)

#define NVIC_BLE_SW_INT_CLRPEND         (0x1 << NVIC_ICPR1_BLE_SW_POS)

#define NVIC_BLE_FINETGTIM_INT_CLRPEND  (0x1 << NVIC_ICPR1_BLE_FINETGTIM_POS)

#define NVIC_BLE_GROSSTGTIM_INT_CLRPEND (0x1 << NVIC_ICPR1_BLE_GROSSTGTIM_POS)

#define NVIC_BLE_ERROR_INT_CLRPEND      (0x1 << NVIC_ICPR1_BLE_ERROR_POS)

#define NVIC_BLE_CRYPT_INT_CLRPEND      (0x1 << NVIC_ICPR1_BLE_CRYPT_POS)

#define NVIC_BLE_EVENT_INT_CLRPEND      (0x1 << NVIC_ICPR1_BLE_EVENT_POS)

#define NVIC_BLE_RX_INT_CLRPEND         (0x1 << NVIC_ICPR1_BLE_RX_POS)

#define NVIC_BLE_SLP_INT_CLRPEND        (0x1 << NVIC_ICPR1_BLE_SLP_POS)

#define NVIC_BLE_CSCNT_INT_CLRPEND      (0x1 << NVIC_ICPR1_BLE_CSCNT_POS)

#define NVIC_DSS7_INT_CLRPEND           (0x1 << NVIC_ICPR1_DSS7_POS)

#define NVIC_DSS6_INT_CLRPEND           (0x1 << NVIC_ICPR1_DSS6_POS)

#define NVIC_DSS5_INT_CLRPEND           (0x1 << NVIC_ICPR1_DSS5_POS)

#define NVIC_DSS4_INT_CLRPEND           (0x1 << NVIC_ICPR1_DSS4_POS)

#define NVIC_DSS3_INT_CLRPEND           (0x1 << NVIC_ICPR1_DSS3_POS)

#define NVIC_DSS2_INT_CLRPEND           (0x1 << NVIC_ICPR1_DSS2_POS)

#define NVIC_DSS1_INT_CLRPEND           (0x1 << NVIC_ICPR1_DSS1_POS)

#define NVIC_DSS0_INT_CLRPEND           (0x1 << NVIC_ICPR1_DSS0_POS)

#define NVIC_PCM_ERROR_INT_CLRPEND      (0x1 << NVIC_ICPR1_PCM_ERROR_POS)

#define NVIC_PCM_TX_INT_CLRPEND         (0x1 << NVIC_ICPR1_PCM_TX_POS)

#define NVIC_PCM_RX_INT_CLRPEND         (0x1 << NVIC_ICPR1_PCM_RX_POS)

#define NVIC_DMIC_OD_ERROR_INT_CLRPEND  (0x1 << NVIC_ICPR1_DMIC_OD_ERROR_POS)

/* NVIC External Interrupt Clear Pending Register 2 */
#define NVIC_ICPR2_BASE                 0xE000E288
#define NVIC_ICPR2                      REG32_POINTER(NVIC_ICPR2_BASE)

/* NVIC_ICPR2 bit positions */
#define NVIC_ICPR2_BLE_AUDIO2_POS       8
#define NVIC_ICPR2_BLE_AUDIO1_POS       7
#define NVIC_ICPR2_BLE_AUDIO0_POS       6
#define NVIC_ICPR2_MEM_ERROR_POS        5
#define NVIC_ICPR2_FLASH_ECC_POS        4
#define NVIC_ICPR2_FLASH_COPY_POS       3
#define NVIC_ICPR2_CLKDET_POS           2
#define NVIC_ICPR2_AUDIO_SINK_PERIOD_POS 1
#define NVIC_ICPR2_AUDIO_SINK_DELAY_POS 0

/* NVIC_ICPR2 settings */
#define NVIC_BLE_AUDIO2_INT_CLRPEND     (0x1 << NVIC_ICPR2_BLE_AUDIO2_POS)

#define NVIC_BLE_AUDIO1_INT_CLRPEND     (0x1 << NVIC_ICPR2_BLE_AUDIO1_POS)

#define NVIC_BLE_AUDIO0_INT_CLRPEND     (0x1 << NVIC_ICPR2_BLE_AUDIO0_POS)

#define NVIC_MEM_ERROR_INT_CLRPEND      (0x1 << NVIC_ICPR2_MEM_ERROR_POS)

#define NVIC_FLASH_ECC_INT_CLRPEND      (0x1 << NVIC_ICPR2_FLASH_ECC_POS)

#define NVIC_FLASH_COPY_INT_CLRPEND     (0x1 << NVIC_ICPR2_FLASH_COPY_POS)

#define NVIC_CLKDET_INT_CLRPEND         (0x1 << NVIC_ICPR2_CLKDET_POS)

#define NVIC_AUDIO_SINK_PERIOD_INT_CLRPEND (0x1 << NVIC_ICPR2_AUDIO_SINK_PERIOD_POS)

#define NVIC_AUDIO_SINK_DELAY_INT_CLRPEND (0x1 << NVIC_ICPR2_AUDIO_SINK_DELAY_POS)

/* NVIC External Interrupt Active Register 0 */
#define NVIC_IABR0_BASE                 0xE000E300
#define NVIC_IABR0                      READONLY_REG32_POINTER(NVIC_IABR0_BASE)

/* NVIC_IABR0 bit positions */
#define NVIC_IABR0_DMIC_OUT_OD_IN_POS   31
#define NVIC_IABR0_UART_ERROR_POS       30
#define NVIC_IABR0_UART_TX_POS          29
#define NVIC_IABR0_UART_RX_POS          28
#define NVIC_IABR0_I2C_POS              27
#define NVIC_IABR0_SPI1_ERROR_POS       26
#define NVIC_IABR0_SPI1_TX_POS          25
#define NVIC_IABR0_SPI1_RX_POS          24
#define NVIC_IABR0_SPI0_ERROR_POS       23
#define NVIC_IABR0_SPI0_TX_POS          22
#define NVIC_IABR0_SPI0_RX_POS          21
#define NVIC_IABR0_WATCHDOG_POS         20
#define NVIC_IABR0_DIO3_POS             19
#define NVIC_IABR0_DIO2_POS             18
#define NVIC_IABR0_DIO1_POS             17
#define NVIC_IABR0_DIO0_POS             16
#define NVIC_IABR0_DMA7_POS             15
#define NVIC_IABR0_DMA6_POS             14
#define NVIC_IABR0_DMA5_POS             13
#define NVIC_IABR0_DMA4_POS             12
#define NVIC_IABR0_DMA3_POS             11
#define NVIC_IABR0_DMA2_POS             10
#define NVIC_IABR0_DMA1_POS             9
#define NVIC_IABR0_DMA0_POS             8
#define NVIC_IABR0_TIMER3_POS           7
#define NVIC_IABR0_TIMER2_POS           6
#define NVIC_IABR0_TIMER1_POS           5
#define NVIC_IABR0_TIMER0_POS           4
#define NVIC_IABR0_ADC_BATMON_POS       3
#define NVIC_IABR0_RTC_CLOCK_POS        2
#define NVIC_IABR0_RTC_ALARM_POS        1
#define NVIC_IABR0_WAKEUP_POS           0

/* NVIC_IABR0 settings */
#define NVIC_DMIC_OUT_OD_IN_INT_ACTIVE  (0x1 << NVIC_IABR0_DMIC_OUT_OD_IN_POS)

#define NVIC_UART_ERROR_INT_ACTIVE      (0x1 << NVIC_IABR0_UART_ERROR_POS)

#define NVIC_UART_TX_INT_ACTIVE         (0x1 << NVIC_IABR0_UART_TX_POS)

#define NVIC_UART_RX_INT_ACTIVE         (0x1 << NVIC_IABR0_UART_RX_POS)

#define NVIC_I2C_INT_ACTIVE             (0x1 << NVIC_IABR0_I2C_POS)

#define NVIC_SPI1_ERROR_INT_ACTIVE      (0x1 << NVIC_IABR0_SPI1_ERROR_POS)

#define NVIC_SPI1_TX_INT_ACTIVE         (0x1 << NVIC_IABR0_SPI1_TX_POS)

#define NVIC_SPI1_RX_INT_ACTIVE         (0x1 << NVIC_IABR0_SPI1_RX_POS)

#define NVIC_SPI0_ERROR_INT_ACTIVE      (0x1 << NVIC_IABR0_SPI0_ERROR_POS)

#define NVIC_SPI0_TX_INT_ACTIVE         (0x1 << NVIC_IABR0_SPI0_TX_POS)

#define NVIC_SPI0_RX_INT_ACTIVE         (0x1 << NVIC_IABR0_SPI0_RX_POS)

#define NVIC_WATCHDOG_INT_ACTIVE        (0x1 << NVIC_IABR0_WATCHDOG_POS)

#define NVIC_DIO3_INT_ACTIVE            (0x1 << NVIC_IABR0_DIO3_POS)

#define NVIC_DIO2_INT_ACTIVE            (0x1 << NVIC_IABR0_DIO2_POS)

#define NVIC_DIO1_INT_ACTIVE            (0x1 << NVIC_IABR0_DIO1_POS)

#define NVIC_DIO0_INT_ACTIVE            (0x1 << NVIC_IABR0_DIO0_POS)

#define NVIC_DMA7_INT_ACTIVE            (0x1 << NVIC_IABR0_DMA7_POS)

#define NVIC_DMA6_INT_ACTIVE            (0x1 << NVIC_IABR0_DMA6_POS)

#define NVIC_DMA5_INT_ACTIVE            (0x1 << NVIC_IABR0_DMA5_POS)

#define NVIC_DMA4_INT_ACTIVE            (0x1 << NVIC_IABR0_DMA4_POS)

#define NVIC_DMA3_INT_ACTIVE            (0x1 << NVIC_IABR0_DMA3_POS)

#define NVIC_DMA2_INT_ACTIVE            (0x1 << NVIC_IABR0_DMA2_POS)

#define NVIC_DMA1_INT_ACTIVE            (0x1 << NVIC_IABR0_DMA1_POS)

#define NVIC_DMA0_INT_ACTIVE            (0x1 << NVIC_IABR0_DMA0_POS)

#define NVIC_TIMER3_INT_ACTIVE          (0x1 << NVIC_IABR0_TIMER3_POS)

#define NVIC_TIMER2_INT_ACTIVE          (0x1 << NVIC_IABR0_TIMER2_POS)

#define NVIC_TIMER1_INT_ACTIVE          (0x1 << NVIC_IABR0_TIMER1_POS)

#define NVIC_TIMER0_INT_ACTIVE          (0x1 << NVIC_IABR0_TIMER0_POS)

#define NVIC_ADC_BATMON_INT_ACTIVE      (0x1 << NVIC_IABR0_ADC_BATMON_POS)

#define NVIC_RTC_CLOCK_INT_ACTIVE       (0x1 << NVIC_IABR0_RTC_CLOCK_POS)

#define NVIC_RTC_ALARM_INT_ACTIVE       (0x1 << NVIC_IABR0_RTC_ALARM_POS)

#define NVIC_WAKEUP_INT_ACTIVE          (0x1 << NVIC_IABR0_WAKEUP_POS)

/* NVIC External Interrupt Active Register 1 */
#define NVIC_IABR1_BASE                 0xE000E304
#define NVIC_IABR1                      READONLY_REG32_POINTER(NVIC_IABR1_BASE)

/* NVIC_IABR1 bit positions */
#define NVIC_IABR1_ASRC_OUT_POS         31
#define NVIC_IABR1_ASRC_IN_POS          30
#define NVIC_IABR1_ASRC_ERROR_POS       29
#define NVIC_IABR1_RF_RXFIFO_POS        28
#define NVIC_IABR1_RF_TXFIFO_POS        27
#define NVIC_IABR1_RF_SYNC_POS          26
#define NVIC_IABR1_RF_RECEIVED_POS      25
#define NVIC_IABR1_RF_RXSTOP_POS        24
#define NVIC_IABR1_RF_TX_POS            23
#define NVIC_IABR1_BLE_COEX_IN_PROCESS_POS 22
#define NVIC_IABR1_BLE_COEX_RX_TX_POS   21
#define NVIC_IABR1_BLE_SW_POS           20
#define NVIC_IABR1_BLE_FINETGTIM_POS    19
#define NVIC_IABR1_BLE_GROSSTGTIM_POS   18
#define NVIC_IABR1_BLE_ERROR_POS        17
#define NVIC_IABR1_BLE_CRYPT_POS        16
#define NVIC_IABR1_BLE_EVENT_POS        15
#define NVIC_IABR1_BLE_RX_POS           14
#define NVIC_IABR1_BLE_SLP_POS          13
#define NVIC_IABR1_BLE_CSCNT_POS        12
#define NVIC_IABR1_DSS7_POS             11
#define NVIC_IABR1_DSS6_POS             10
#define NVIC_IABR1_DSS5_POS             9
#define NVIC_IABR1_DSS4_POS             8
#define NVIC_IABR1_DSS3_POS             7
#define NVIC_IABR1_DSS2_POS             6
#define NVIC_IABR1_DSS1_POS             5
#define NVIC_IABR1_DSS0_POS             4
#define NVIC_IABR1_PCM_ERROR_POS        3
#define NVIC_IABR1_PCM_TX_POS           2
#define NVIC_IABR1_PCM_RX_POS           1
#define NVIC_IABR1_DMIC_OD_ERROR_POS    0

/* NVIC_IABR1 settings */
#define NVIC_ASRC_OUT_INT_ACTIVE        (0x1 << NVIC_IABR1_ASRC_OUT_POS)

#define NVIC_ASRC_IN_INT_ACTIVE         (0x1 << NVIC_IABR1_ASRC_IN_POS)

#define NVIC_ASRC_ERROR_INT_ACTIVE      (0x1 << NVIC_IABR1_ASRC_ERROR_POS)

#define NVIC_RF_RXFIFO_INT_ACTIVE       (0x1 << NVIC_IABR1_RF_RXFIFO_POS)

#define NVIC_RF_TXFIFO_INT_ACTIVE       (0x1 << NVIC_IABR1_RF_TXFIFO_POS)

#define NVIC_RF_SYNC_INT_ACTIVE         (0x1 << NVIC_IABR1_RF_SYNC_POS)

#define NVIC_RF_RECEIVED_INT_ACTIVE     (0x1 << NVIC_IABR1_RF_RECEIVED_POS)

#define NVIC_RF_RXSTOP_INT_ACTIVE       (0x1 << NVIC_IABR1_RF_RXSTOP_POS)

#define NVIC_RF_TX_INT_ACTIVE           (0x1 << NVIC_IABR1_RF_TX_POS)

#define NVIC_BLE_COEX_IN_PROCESS_INT_ACTIVE (0x1 << NVIC_IABR1_BLE_COEX_IN_PROCESS_POS)

#define NVIC_BLE_COEX_RX_TX_INT_ACTIVE  (0x1 << NVIC_IABR1_BLE_COEX_RX_TX_POS)

#define NVIC_BLE_SW_INT_ACTIVE          (0x1 << NVIC_IABR1_BLE_SW_POS)

#define NVIC_BLE_FINETGTIM_INT_ACTIVE   (0x1 << NVIC_IABR1_BLE_FINETGTIM_POS)

#define NVIC_BLE_GROSSTGTIM_INT_ACTIVE  (0x1 << NVIC_IABR1_BLE_GROSSTGTIM_POS)

#define NVIC_BLE_ERROR_INT_ACTIVE       (0x1 << NVIC_IABR1_BLE_ERROR_POS)

#define NVIC_BLE_CRYPT_INT_ACTIVE       (0x1 << NVIC_IABR1_BLE_CRYPT_POS)

#define NVIC_BLE_EVENT_INT_ACTIVE       (0x1 << NVIC_IABR1_BLE_EVENT_POS)

#define NVIC_BLE_RX_INT_ACTIVE          (0x1 << NVIC_IABR1_BLE_RX_POS)

#define NVIC_BLE_SLP_INT_ACTIVE         (0x1 << NVIC_IABR1_BLE_SLP_POS)

#define NVIC_BLE_CSCNT_INT_ACTIVE       (0x1 << NVIC_IABR1_BLE_CSCNT_POS)

#define NVIC_DSS7_INT_ACTIVE            (0x1 << NVIC_IABR1_DSS7_POS)

#define NVIC_DSS6_INT_ACTIVE            (0x1 << NVIC_IABR1_DSS6_POS)

#define NVIC_DSS5_INT_ACTIVE            (0x1 << NVIC_IABR1_DSS5_POS)

#define NVIC_DSS4_INT_ACTIVE            (0x1 << NVIC_IABR1_DSS4_POS)

#define NVIC_DSS3_INT_ACTIVE            (0x1 << NVIC_IABR1_DSS3_POS)

#define NVIC_DSS2_INT_ACTIVE            (0x1 << NVIC_IABR1_DSS2_POS)

#define NVIC_DSS1_INT_ACTIVE            (0x1 << NVIC_IABR1_DSS1_POS)

#define NVIC_DSS0_INT_ACTIVE            (0x1 << NVIC_IABR1_DSS0_POS)

#define NVIC_PCM_ERROR_INT_ACTIVE       (0x1 << NVIC_IABR1_PCM_ERROR_POS)

#define NVIC_PCM_TX_INT_ACTIVE          (0x1 << NVIC_IABR1_PCM_TX_POS)

#define NVIC_PCM_RX_INT_ACTIVE          (0x1 << NVIC_IABR1_PCM_RX_POS)

#define NVIC_DMIC_OD_ERROR_INT_ACTIVE   (0x1 << NVIC_IABR1_DMIC_OD_ERROR_POS)

/* NVIC External Interrupt Active Register 2 */
#define NVIC_IABR2_BASE                 0xE000E308
#define NVIC_IABR2                      READONLY_REG32_POINTER(NVIC_IABR2_BASE)

/* NVIC_IABR2 bit positions */
#define NVIC_IABR2_BLE_AUDIO2_POS       8
#define NVIC_IABR2_BLE_AUDIO1_POS       7
#define NVIC_IABR2_BLE_AUDIO0_POS       6
#define NVIC_IABR2_MEM_ERROR_POS        5
#define NVIC_IABR2_FLASH_ECC_POS        4
#define NVIC_IABR2_FLASH_COPY_POS       3
#define NVIC_IABR2_CLKDET_POS           2
#define NVIC_IABR2_AUDIO_SINK_PERIOD_POS 1
#define NVIC_IABR2_AUDIO_SINK_DELAY_POS 0

/* NVIC_IABR2 settings */
#define NVIC_BLE_AUDIO2_INT_ACTIVE      (0x1 << NVIC_IABR2_BLE_AUDIO2_POS)

#define NVIC_BLE_AUDIO1_INT_ACTIVE      (0x1 << NVIC_IABR2_BLE_AUDIO1_POS)

#define NVIC_BLE_AUDIO0_INT_ACTIVE      (0x1 << NVIC_IABR2_BLE_AUDIO0_POS)

#define NVIC_MEM_ERROR_INT_ACTIVE       (0x1 << NVIC_IABR2_MEM_ERROR_POS)

#define NVIC_FLASH_ECC_INT_ACTIVE       (0x1 << NVIC_IABR2_FLASH_ECC_POS)

#define NVIC_FLASH_COPY_INT_ACTIVE      (0x1 << NVIC_IABR2_FLASH_COPY_POS)

#define NVIC_CLKDET_INT_ACTIVE          (0x1 << NVIC_IABR2_CLKDET_POS)

#define NVIC_AUDIO_SINK_PERIOD_INT_ACTIVE (0x1 << NVIC_IABR2_AUDIO_SINK_PERIOD_POS)

#define NVIC_AUDIO_SINK_DELAY_INT_ACTIVE (0x1 << NVIC_IABR2_AUDIO_SINK_DELAY_POS)

/* NVIC External Interrupt Priority Register 0 */
#define NVIC_IP0_BASE                   0xE000E400
#define NVIC_IP0                        REG32_POINTER(NVIC_IP0_BASE)

/* NVIC_IP0 bit positions */
#define NVIC_IP0_ADC_BATMON_POS         29
#define NVIC_IP0_ADC_BATMON_MASK        (0x7 << NVIC_IP0_ADC_BATMON_POS)
#define NVIC_IP0_RTC_CLOCK_POS          21
#define NVIC_IP0_RTC_CLOCK_MASK         (0x7 << NVIC_IP0_RTC_CLOCK_POS)
#define NVIC_IP0_RTC_ALARM_POS          13
#define NVIC_IP0_RTC_ALARM_MASK         (0x7 << NVIC_IP0_RTC_ALARM_POS)
#define NVIC_IP0_WAKEUP_POS             5
#define NVIC_IP0_WAKEUP_MASK            (0x7 << NVIC_IP0_WAKEUP_POS)

/* NVIC External Interrupt Priority Register 1 */
#define NVIC_IP1_BASE                   0xE000E404
#define NVIC_IP1                        REG32_POINTER(NVIC_IP1_BASE)

/* NVIC_IP1 bit positions */
#define NVIC_IP1_TIMER3_POS             29
#define NVIC_IP1_TIMER3_MASK            (0x7 << NVIC_IP1_TIMER3_POS)
#define NVIC_IP1_TIMER2_POS             21
#define NVIC_IP1_TIMER2_MASK            (0x7 << NVIC_IP1_TIMER2_POS)
#define NVIC_IP1_TIMER1_POS             13
#define NVIC_IP1_TIMER1_MASK            (0x7 << NVIC_IP1_TIMER1_POS)
#define NVIC_IP1_TIMER0_POS             5
#define NVIC_IP1_TIMER0_MASK            (0x7 << NVIC_IP1_TIMER0_POS)

/* NVIC External Interrupt Priority Register 2 */
#define NVIC_IP2_BASE                   0xE000E408
#define NVIC_IP2                        REG32_POINTER(NVIC_IP2_BASE)

/* NVIC_IP2 bit positions */
#define NVIC_IP2_DMA3_POS               29
#define NVIC_IP2_DMA3_MASK              (0x7 << NVIC_IP2_DMA3_POS)
#define NVIC_IP2_DMA2_POS               21
#define NVIC_IP2_DMA2_MASK              (0x7 << NVIC_IP2_DMA2_POS)
#define NVIC_IP2_DMA1_POS               13
#define NVIC_IP2_DMA1_MASK              (0x7 << NVIC_IP2_DMA1_POS)
#define NVIC_IP2_DMA0_POS               5
#define NVIC_IP2_DMA0_MASK              (0x7 << NVIC_IP2_DMA0_POS)

/* NVIC External Interrupt Priority Register 3 */
#define NVIC_IP3_BASE                   0xE000E40C
#define NVIC_IP3                        REG32_POINTER(NVIC_IP3_BASE)

/* NVIC_IP3 bit positions */
#define NVIC_IP3_DMA7_POS               29
#define NVIC_IP3_DMA7_MASK              (0x7 << NVIC_IP3_DMA7_POS)
#define NVIC_IP3_DMA6_POS               21
#define NVIC_IP3_DMA6_MASK              (0x7 << NVIC_IP3_DMA6_POS)
#define NVIC_IP3_DMA5_POS               13
#define NVIC_IP3_DMA5_MASK              (0x7 << NVIC_IP3_DMA5_POS)
#define NVIC_IP3_DMA4_POS               5
#define NVIC_IP3_DMA4_MASK              (0x7 << NVIC_IP3_DMA4_POS)

/* NVIC External Interrupt Priority Register 4 */
#define NVIC_IP4_BASE                   0xE000E410
#define NVIC_IP4                        REG32_POINTER(NVIC_IP4_BASE)

/* NVIC_IP4 bit positions */
#define NVIC_IP4_DIO3_POS               29
#define NVIC_IP4_DIO3_MASK              (0x7 << NVIC_IP4_DIO3_POS)
#define NVIC_IP4_DIO2_POS               21
#define NVIC_IP4_DIO2_MASK              (0x7 << NVIC_IP4_DIO2_POS)
#define NVIC_IP4_DIO1_POS               13
#define NVIC_IP4_DIO1_MASK              (0x7 << NVIC_IP4_DIO1_POS)
#define NVIC_IP4_DIO0_POS               5
#define NVIC_IP4_DIO0_MASK              (0x7 << NVIC_IP4_DIO0_POS)

/* NVIC External Interrupt Priority Register 5 */
#define NVIC_IP5_BASE                   0xE000E414
#define NVIC_IP5                        REG32_POINTER(NVIC_IP5_BASE)

/* NVIC_IP5 bit positions */
#define NVIC_IP5_SPI0_ERROR_POS         29
#define NVIC_IP5_SPI0_ERROR_MASK        (0x7 << NVIC_IP5_SPI0_ERROR_POS)
#define NVIC_IP5_SPI0_TX_POS            21
#define NVIC_IP5_SPI0_TX_MASK           (0x7 << NVIC_IP5_SPI0_TX_POS)
#define NVIC_IP5_SPI0_RX_POS            13
#define NVIC_IP5_SPI0_RX_MASK           (0x7 << NVIC_IP5_SPI0_RX_POS)
#define NVIC_IP5_WATCHDOG_POS           5
#define NVIC_IP5_WATCHDOG_MASK          (0x7 << NVIC_IP5_WATCHDOG_POS)

/* NVIC External Interrupt Priority Register 6 */
#define NVIC_IP6_BASE                   0xE000E418
#define NVIC_IP6                        REG32_POINTER(NVIC_IP6_BASE)

/* NVIC_IP6 bit positions */
#define NVIC_IP6_I2C_POS                29
#define NVIC_IP6_I2C_MASK               (0x7 << NVIC_IP6_I2C_POS)
#define NVIC_IP6_SPI1_ERROR_POS         21
#define NVIC_IP6_SPI1_ERROR_MASK        (0x7 << NVIC_IP6_SPI1_ERROR_POS)
#define NVIC_IP6_SPI1_TX_POS            13
#define NVIC_IP6_SPI1_TX_MASK           (0x7 << NVIC_IP6_SPI1_TX_POS)
#define NVIC_IP6_SPI1_RX_POS            5
#define NVIC_IP6_SPI1_RX_MASK           (0x7 << NVIC_IP6_SPI1_RX_POS)

/* NVIC External Interrupt Priority Register 7 */
#define NVIC_IP7_BASE                   0xE000E41C
#define NVIC_IP7                        REG32_POINTER(NVIC_IP7_BASE)

/* NVIC_IP7 bit positions */
#define NVIC_IP7_DMIC_OUT_OD_IN_POS     29
#define NVIC_IP7_DMIC_OUT_OD_IN_MASK    (0x7 << NVIC_IP7_DMIC_OUT_OD_IN_POS)
#define NVIC_IP7_UART_ERROR_POS         21
#define NVIC_IP7_UART_ERROR_MASK        (0x7 << NVIC_IP7_UART_ERROR_POS)
#define NVIC_IP7_UART_TX_POS            13
#define NVIC_IP7_UART_TX_MASK           (0x7 << NVIC_IP7_UART_TX_POS)
#define NVIC_IP7_UART_RX_POS            5
#define NVIC_IP7_UART_RX_MASK           (0x7 << NVIC_IP7_UART_RX_POS)

/* NVIC External Interrupt Priority Register 8 */
#define NVIC_IP8_BASE                   0xE000E420
#define NVIC_IP8                        REG32_POINTER(NVIC_IP8_BASE)

/* NVIC_IP8 bit positions */
#define NVIC_IP8_PCM_ERROR_POS          29
#define NVIC_IP8_PCM_ERROR_MASK         (0x7 << NVIC_IP8_PCM_ERROR_POS)
#define NVIC_IP8_PCM_TX_POS             21
#define NVIC_IP8_PCM_TX_MASK            (0x7 << NVIC_IP8_PCM_TX_POS)
#define NVIC_IP8_PCM_RX_POS             13
#define NVIC_IP8_PCM_RX_MASK            (0x7 << NVIC_IP8_PCM_RX_POS)
#define NVIC_IP8_DMIC_OD_ERROR_POS      5
#define NVIC_IP8_DMIC_OD_ERROR_MASK     (0x7 << NVIC_IP8_DMIC_OD_ERROR_POS)

/* NVIC External Interrupt Priority Register 9 */
#define NVIC_IP9_BASE                   0xE000E424
#define NVIC_IP9                        REG32_POINTER(NVIC_IP9_BASE)

/* NVIC_IP9 bit positions */
#define NVIC_IP9_DSS3_POS               29
#define NVIC_IP9_DSS3_MASK              (0x7 << NVIC_IP9_DSS3_POS)
#define NVIC_IP9_DSS2_POS               21
#define NVIC_IP9_DSS2_MASK              (0x7 << NVIC_IP9_DSS2_POS)
#define NVIC_IP9_DSS1_POS               13
#define NVIC_IP9_DSS1_MASK              (0x7 << NVIC_IP9_DSS1_POS)
#define NVIC_IP9_DSS0_POS               5
#define NVIC_IP9_DSS0_MASK              (0x7 << NVIC_IP9_DSS0_POS)

/* NVIC External Interrupt Priority Register 10 */
#define NVIC_IP10_BASE                  0xE000E428
#define NVIC_IP10                       REG32_POINTER(NVIC_IP10_BASE)

/* NVIC_IP10 bit positions */
#define NVIC_IP10_DSS7_POS              29
#define NVIC_IP10_DSS7_MASK             (0x7 << NVIC_IP10_DSS7_POS)
#define NVIC_IP10_DSS6_POS              21
#define NVIC_IP10_DSS6_MASK             (0x7 << NVIC_IP10_DSS6_POS)
#define NVIC_IP10_DSS5_POS              13
#define NVIC_IP10_DSS5_MASK             (0x7 << NVIC_IP10_DSS5_POS)
#define NVIC_IP10_DSS4_POS              5
#define NVIC_IP10_DSS4_MASK             (0x7 << NVIC_IP10_DSS4_POS)

/* NVIC External Interrupt Priority Register 11 */
#define NVIC_IP11_BASE                  0xE000E42C
#define NVIC_IP11                       REG32_POINTER(NVIC_IP11_BASE)

/* NVIC_IP11 bit positions */
#define NVIC_IP11_BLE_EVENT_POS         29
#define NVIC_IP11_BLE_EVENT_MASK        (0x7 << NVIC_IP11_BLE_EVENT_POS)
#define NVIC_IP11_BLE_RX_POS            21
#define NVIC_IP11_BLE_RX_MASK           (0x7 << NVIC_IP11_BLE_RX_POS)
#define NVIC_IP11_BLE_SLP_POS           13
#define NVIC_IP11_BLE_SLP_MASK          (0x7 << NVIC_IP11_BLE_SLP_POS)
#define NVIC_IP11_BLE_CSCNT_POS         5
#define NVIC_IP11_BLE_CSCNT_MASK        (0x7 << NVIC_IP11_BLE_CSCNT_POS)

/* NVIC External Interrupt Priority Register 12 */
#define NVIC_IP12_BASE                  0xE000E430
#define NVIC_IP12                       REG32_POINTER(NVIC_IP12_BASE)

/* NVIC_IP12 bit positions */
#define NVIC_IP12_BLE_FINETGTIM_POS     29
#define NVIC_IP12_BLE_FINETGTIM_MASK    (0x7 << NVIC_IP12_BLE_FINETGTIM_POS)
#define NVIC_IP12_BLE_GROSSTGTIM_POS    21
#define NVIC_IP12_BLE_GROSSTGTIM_MASK   (0x7 << NVIC_IP12_BLE_GROSSTGTIM_POS)
#define NVIC_IP12_BLE_ERROR_POS         13
#define NVIC_IP12_BLE_ERROR_MASK        (0x7 << NVIC_IP12_BLE_ERROR_POS)
#define NVIC_IP12_BLE_CRYPT_POS         5
#define NVIC_IP12_BLE_CRYPT_MASK        (0x7 << NVIC_IP12_BLE_CRYPT_POS)

/* NVIC External Interrupt Priority Register 13 */
#define NVIC_IP13_BASE                  0xE000E434
#define NVIC_IP13                       REG32_POINTER(NVIC_IP13_BASE)

/* NVIC_IP13 bit positions */
#define NVIC_IP13_RF_TX_POS             29
#define NVIC_IP13_RF_TX_MASK            (0x7 << NVIC_IP13_RF_TX_POS)
#define NVIC_IP13_BLE_COEX_IN_PROCESS_POS 21
#define NVIC_IP13_BLE_COEX_IN_PROCESS_MASK (0x7 << NVIC_IP13_BLE_COEX_IN_PROCESS_POS)
#define NVIC_IP13_BLE_COEX_RX_TX_POS    13
#define NVIC_IP13_BLE_COEX_RX_TX_MASK   (0x7 << NVIC_IP13_BLE_COEX_RX_TX_POS)
#define NVIC_IP13_BLE_SW_POS            5
#define NVIC_IP13_BLE_SW_MASK           (0x7 << NVIC_IP13_BLE_SW_POS)

/* NVIC External Interrupt Priority Register 14 */
#define NVIC_IP14_BASE                  0xE000E438
#define NVIC_IP14                       REG32_POINTER(NVIC_IP14_BASE)

/* NVIC_IP14 bit positions */
#define NVIC_IP14_RF_TXFIFO_POS         29
#define NVIC_IP14_RF_TXFIFO_MASK        (0x7 << NVIC_IP14_RF_TXFIFO_POS)
#define NVIC_IP14_RF_SYNC_POS           21
#define NVIC_IP14_RF_SYNC_MASK          (0x7 << NVIC_IP14_RF_SYNC_POS)
#define NVIC_IP14_RF_RECEIVED_POS       13
#define NVIC_IP14_RF_RECEIVED_MASK      (0x7 << NVIC_IP14_RF_RECEIVED_POS)
#define NVIC_IP14_RF_RXSTOP_POS         5
#define NVIC_IP14_RF_RXSTOP_MASK        (0x7 << NVIC_IP14_RF_RXSTOP_POS)

/* NVIC External Interrupt Priority Register 15 */
#define NVIC_IP15_BASE                  0xE000E43C
#define NVIC_IP15                       REG32_POINTER(NVIC_IP15_BASE)

/* NVIC_IP15 bit positions */
#define NVIC_IP15_ASRC_OUT_POS          29
#define NVIC_IP15_ASRC_OUT_MASK         (0x7 << NVIC_IP15_ASRC_OUT_POS)
#define NVIC_IP15_ASRC_IN_POS           21
#define NVIC_IP15_ASRC_IN_MASK          (0x7 << NVIC_IP15_ASRC_IN_POS)
#define NVIC_IP15_ASRC_ERROR_POS        13
#define NVIC_IP15_ASRC_ERROR_MASK       (0x7 << NVIC_IP15_ASRC_ERROR_POS)
#define NVIC_IP15_RF_RXFIFO_POS         5
#define NVIC_IP15_RF_RXFIFO_MASK        (0x7 << NVIC_IP15_RF_RXFIFO_POS)

/* NVIC External Interrupt Priority Register 16 */
#define NVIC_IP16_BASE                  0xE000E440
#define NVIC_IP16                       REG32_POINTER(NVIC_IP16_BASE)

/* NVIC_IP16 bit positions */
#define NVIC_IP16_FLASH_COPY_POS        29
#define NVIC_IP16_FLASH_COPY_MASK       (0x7 << NVIC_IP16_FLASH_COPY_POS)
#define NVIC_IP16_CLKDET_POS            21
#define NVIC_IP16_CLKDET_MASK           (0x7 << NVIC_IP16_CLKDET_POS)
#define NVIC_IP16_AUDIO_SINK_PERIOD_POS 13
#define NVIC_IP16_AUDIO_SINK_PERIOD_MASK (0x7 << NVIC_IP16_AUDIO_SINK_PERIOD_POS)
#define NVIC_IP16_AUDIO_SINK_DELAY_POS  5
#define NVIC_IP16_AUDIO_SINK_DELAY_MASK (0x7 << NVIC_IP16_AUDIO_SINK_DELAY_POS)

/* NVIC External Interrupt Priority Register 17 */
#define NVIC_IP17_BASE                  0xE000E444
#define NVIC_IP17                       REG32_POINTER(NVIC_IP17_BASE)

/* NVIC_IP17 bit positions */
#define NVIC_IP17_BLE_AUDIO1_POS        29
#define NVIC_IP17_BLE_AUDIO1_MASK       (0x7 << NVIC_IP17_BLE_AUDIO1_POS)
#define NVIC_IP17_BLE_AUDIO0_POS        21
#define NVIC_IP17_BLE_AUDIO0_MASK       (0x7 << NVIC_IP17_BLE_AUDIO0_POS)
#define NVIC_IP17_MEM_ERROR_POS         13
#define NVIC_IP17_MEM_ERROR_MASK        (0x7 << NVIC_IP17_MEM_ERROR_POS)
#define NVIC_IP17_FLASH_ECC_POS         5
#define NVIC_IP17_FLASH_ECC_MASK        (0x7 << NVIC_IP17_FLASH_ECC_POS)

/* NVIC External Interrupt Priority Register 18 */
#define NVIC_IP18_BASE                  0xE000E448
#define NVIC_IP18                       REG32_POINTER(NVIC_IP18_BASE)

/* NVIC_IP18 bit positions */
#define NVIC_IP18_BLE_AUDIO2_POS        5
#define NVIC_IP18_BLE_AUDIO2_MASK       (0x7 << NVIC_IP18_BLE_AUDIO2_POS)

/* NVIC Software Trigger Interrupt Register */
#define NVIC_STIR_BASE                  0xE000EF00
#define NVIC_STIR                       REG32_POINTER(NVIC_STIR_BASE)

/* NVIC_STIR bit positions */
#define NVIC_STIR_INTID_POS             0
#define NVIC_STIR_INTID_MASK            (0xFF << NVIC_STIR_INTID_POS)

/* ----------------------------------------------------------------------------
 * System Control Block
 * ------------------------------------------------------------------------- */

/* CPU ID Base Register */
#define SCB_CPUID_BASE                  0xE000ED00
#define SCB_CPUID                       READONLY_REG32_POINTER(SCB_CPUID_BASE)

/* SCB_CPUID bit positions */
#define SCB_CPUID_IMPLEMENTER_POS       24
#define SCB_CPUID_IMPLEMENTER_MASK      (0xFF << SCB_CPUID_IMPLEMENTER_POS)
#define SCB_CPUID_VARIANT_POS           20
#define SCB_CPUID_VARIANT_MASK          (0xF << SCB_CPUID_VARIANT_POS)
#define SCB_CPUID_ARCHITECTURE_POS      16
#define SCB_CPUID_ARCHITECTURE_MASK     (0xF << SCB_CPUID_ARCHITECTURE_POS)
#define SCB_CPUID_PARTNO_POS            4
#define SCB_CPUID_PARTNO_MASK           (0xFFF << SCB_CPUID_PARTNO_POS)
#define SCB_CPUID_REVISION_POS          0
#define SCB_CPUID_REVISION_MASK         (0xF << SCB_CPUID_REVISION_POS)

/* Interrupt Control and State Register */
#define SCB_ICSR_BASE                   0xE000ED04
#define SCB_ICSR                        REG32_POINTER(SCB_ICSR_BASE)

/* SCB_ICSR bit positions */
#define SCB_ICSR_NMIPENDSET_POS         31
#define SCB_ICSR_PENDSVSET_POS          28
#define SCB_ICSR_PENDSVCLR_POS          27
#define SCB_ICSR_PENDSTSET_POS          26
#define SCB_ICSR_PENDSTCLR_POS          25
#define SCB_ICSR_ISRPREEMPT_POS         23
#define SCB_ICSR_ISRPENDING_POS         22
#define SCB_ICSR_VECTPENDING_POS        12
#define SCB_ICSR_VECTPENDING_MASK       (0x3FF << SCB_ICSR_VECTPENDING_POS)
#define SCB_ICSR_RETTOBASE_POS          11
#define SCB_ICSR_VECTACTIVE_POS         0
#define SCB_ICSR_VECTACTIVE_MASK        (0x3FF << SCB_ICSR_VECTACTIVE_POS)

/* SCB_ICSR settings */
#define NVIC_NMI_SETPEND                (0x1 << SCB_ICSR_NMIPENDSET_POS)

#define NVIC_PENDSV_SETPEND             (0x1 << SCB_ICSR_PENDSVSET_POS)

#define NVIC_PENDSV_CLRPEND             (0x1 << SCB_ICSR_PENDSVCLR_POS)

#define NVIC_SYSTICK_SETPEND            (0x1 << SCB_ICSR_PENDSTSET_POS)

#define NVIC_SYSTICK_CLRPEND            (0x1 << SCB_ICSR_PENDSTCLR_POS)

/* Vector Table Offset Register */
#define SCB_VTOR_BASE                   0xE000ED08
#define SCB_VTOR                        REG32_POINTER(SCB_VTOR_BASE)

/* SCB_VTOR bit positions */
#define SCB_VTOR_TBLOFF_POS             7
#define SCB_VTOR_TBLOFF_MASK            (0x1FFFFFF << SCB_VTOR_TBLOFF_POS)

/* Application Interrupt and Reset Control Register */
#define SCB_AIRCR_BASE                  0xE000ED0C
#define SCB_AIRCR                       REG32_POINTER(SCB_AIRCR_BASE)

/* SCB_AIRCR bit positions */
#define SCB_AIRCR_VECTKEY_POS           16
#define SCB_AIRCR_VECTKEY_MASK          (0xFFFF << SCB_AIRCR_VECTKEY_POS)
#define SCB_AIRCR_ENDIANESS_POS         15
#define SCB_AIRCR_PRIGROUP_POS          8
#define SCB_AIRCR_PRIGROUP_MASK         (0x7 << SCB_AIRCR_PRIGROUP_POS)
#define SCB_AIRCR_SYSRESETREQ_POS       2
#define SCB_AIRCR_VECTCLRACTIVE_POS     1
#define SCB_AIRCR_VECTRESET_POS         0

/* SCB_AIRCR settings */
#define NVIC_VECTKEY_UNLOCK             (0x5FA << SCB_AIRCR_VECTKEY_POS)
#define NVIC_VECTKEY_UNLOCK_READBACK    (0xFA05 << SCB_AIRCR_VECTKEY_POS)

#define NVIC_LITTLE_ENDIAN              (0x0 << SCB_AIRCR_ENDIANESS_POS)

#define NVIC_PRIGROUP_4_0               (0x3 << SCB_AIRCR_PRIGROUP_POS)
#define NVIC_PRIGROUP_3_1               (0x4 << SCB_AIRCR_PRIGROUP_POS)
#define NVIC_PRIGROUP_2_2               (0x5 << SCB_AIRCR_PRIGROUP_POS)
#define NVIC_PRIGROUP_1_3               (0x6 << SCB_AIRCR_PRIGROUP_POS)
#define NVIC_PRIGROUP_0_4               (0x7 << SCB_AIRCR_PRIGROUP_POS)

#define NVIC_SYSRESET                   (0x1 << SCB_AIRCR_SYSRESETREQ_POS)

#define NVIC_CLEAR_ACTIVE_VECTORS       (0x1 << SCB_AIRCR_VECTCLRACTIVE_POS)

#define NVIC_VECTRESET                  (0x1 << SCB_AIRCR_VECTRESET_POS)

/* System Control Register */
#define SCB_SCR_BASE                    0xE000ED10
#define SCB_SCR                         REG32_POINTER(SCB_SCR_BASE)

/* SCB_SCR bit positions */
#define SCB_SCR_SEVONPEND_POS           4
#define SCB_SCR_SLEEPDEEP_POS           2
#define SCB_SCR_SLEEPONEXIT_POS         1

/* SCB_SCR settings */
#define NVIC_SLEEPDEEP_DISABLE          (0x0 << SCB_SCR_SLEEPDEEP_POS)
#define NVIC_SLEEPDEEP_ENABLE           (0x1 << SCB_SCR_SLEEPDEEP_POS)

#define NVIC_SLEEPONEXIT_DISABLE        (0x0 << SCB_SCR_SLEEPONEXIT_POS)
#define NVIC_SLEEPONEXIT_ENABLE         (0x1 << SCB_SCR_SLEEPONEXIT_POS)

/* Configuration Control Register */
#define SCB_CCR_BASE                    0xE000ED14
#define SCB_CCR                         REG32_POINTER(SCB_CCR_BASE)

/* SCB_CCR bit positions */
#define SCB_CCR_STKALIGN_POS            9
#define SCB_CCR_BFHFNMIGN_POS           8
#define SCB_CCR_DIV_0_TRP_POS           4
#define SCB_CCR_UNALIGN_TRP_POS         3
#define SCB_CCR_USERSETMPEND_POS        1
#define SCB_CCR_NONBASETHRDENA_POS      0

/* SCB_CCR settings */
#define NVIC_STKALIGN_DISABLE           (0x0 << SCB_CCR_STKALIGN_POS)
#define NVIC_STKALIGN_ENABLE            (0x1 << SCB_CCR_STKALIGN_POS)

#define NVIC_BFHFNMIGN_DISABLE          (0x0 << SCB_CCR_BFHFNMIGN_POS)
#define NVIC_BFHFNMIGN_ENABLE           (0x1 << SCB_CCR_BFHFNMIGN_POS)

#define NVIC_DIV_0_TRP_DISABLE          (0x0 << SCB_CCR_DIV_0_TRP_POS)
#define NVIC_DIV_0_TRP_ENABLE           (0x1 << SCB_CCR_DIV_0_TRP_POS)

#define NVIC_UNALIGN_TRP_DISABLE        (0x0 << SCB_CCR_UNALIGN_TRP_POS)
#define NVIC_UNALIGN_TRP_ENABLE         (0x1 << SCB_CCR_UNALIGN_TRP_POS)

#define NVIC_USERSETMPEND_DISABLE       (0x0 << SCB_CCR_USERSETMPEND_POS)
#define NVIC_USERSETMPEND_ENABLE        (0x1 << SCB_CCR_USERSETMPEND_POS)

#define NVIC_NONBASETHRDENA_DISABLE     (0x0 << SCB_CCR_NONBASETHRDENA_POS)
#define NVIC_NONBASETHRDENA_ENABLE      (0x1 << SCB_CCR_NONBASETHRDENA_POS)

/* System Interrupt Priority Register 0 */
#define SCB_SHP0_BASE                   0xE000ED18
#define SCB_SHP0                        REG32_POINTER(SCB_SHP0_BASE)

/* SCB_SHP0 bit positions */
#define SCB_SHP0_NVIC_USAGE_FAULT_PRIORITY_POS 16
#define SCB_SHP0_NVIC_USAGE_FAULT_PRIORITY_MASK (0xFF << SCB_SHP0_NVIC_USAGE_FAULT_PRIORITY_POS)
#define SCB_SHP0_NVIC_BUS_FAULT_PRIORITY_POS 8
#define SCB_SHP0_NVIC_BUS_FAULT_PRIORITY_MASK (0xFF << SCB_SHP0_NVIC_BUS_FAULT_PRIORITY_POS)
#define SCB_SHP0_NVIC_MEM_FAULT_PRIORITY_POS 0
#define SCB_SHP0_NVIC_MEM_FAULT_PRIORITY_MASK (0xFF << SCB_SHP0_NVIC_MEM_FAULT_PRIORITY_POS)

/* System Interrupt Priority Register 1 */
#define SCB_SHP1_BASE                   0xE000ED1C
#define SCB_SHP1                        REG32_POINTER(SCB_SHP1_BASE)

/* SCB_SHP1 bit positions */
#define SCB_SHP1_NVIC_SVC_PRIORITY_POS  24
#define SCB_SHP1_NVIC_SVC_PRIORITY_MASK (0xFF << SCB_SHP1_NVIC_SVC_PRIORITY_POS)

/* System Interrupt Priority Register 2 */
#define SCB_SHP2_BASE                   0xE000ED20
#define SCB_SHP2                        REG32_POINTER(SCB_SHP2_BASE)

/* SCB_SHP2 bit positions */
#define SCB_SHP2_NVIC_SYSTICK_PRIORITY_POS 24
#define SCB_SHP2_NVIC_SYSTICK_PRIORITY_MASK (0xFF << SCB_SHP2_NVIC_SYSTICK_PRIORITY_POS)
#define SCB_SHP2_NVIC_PENDSV_PRIORITY_POS 16
#define SCB_SHP2_NVIC_PENDSV_PRIORITY_MASK (0xFF << SCB_SHP2_NVIC_PENDSV_PRIORITY_POS)
#define SCB_SHP2_NVIC_MONITOR_PRIORITY_POS 0
#define SCB_SHP2_NVIC_MONITOR_PRIORITY_MASK (0xFF << SCB_SHP2_NVIC_MONITOR_PRIORITY_POS)

/* System Handler Control and State Register */
#define SCB_SHCSR_BASE                  0xE000ED24
#define SCB_SHCSR                       REG32_POINTER(SCB_SHCSR_BASE)

/* SCB_SHCSR bit positions */
#define SCB_SHCSR_USGFAULTENA_POS       18
#define SCB_SHCSR_BUSFAULTENA_POS       17
#define SCB_SHCSR_MEMFAULTENA_POS       16
#define SCB_SHCSR_SVCALLPENDED_POS      15
#define SCB_SHCSR_BUSFAULTPENDED_POS    14
#define SCB_SHCSR_MEMFAULTPENDED_POS    13
#define SCB_SHCSR_USGFAULTPENDED_POS    12
#define SCB_SHCSR_SYSTICKACT_POS        11
#define SCB_SHCSR_PENDSVACT_POS         10
#define SCB_SHCSR_MONITORACT_POS        8
#define SCB_SHCSR_SVCALLACT_POS         7
#define SCB_SHCSR_USGFAULTACT_POS       3
#define SCB_SHCSR_BUSFAULTACT_POS       1
#define SCB_SHCSR_MEMFAULTACT_POS       0

/* SCB_SHCSR settings */
#define NVIC_USGFAULT_DISABLE           (0x0 << SCB_SHCSR_USGFAULTENA_POS)
#define NVIC_USGFAULT_ENABLE            (0x1 << SCB_SHCSR_USGFAULTENA_POS)

#define NVIC_BUSFAULT_DISABLE           (0x0 << SCB_SHCSR_BUSFAULTENA_POS)
#define NVIC_BUSFAULT_ENABLE            (0x1 << SCB_SHCSR_BUSFAULTENA_POS)

#define NVIC_MEMFAULT_DISABLE           (0x0 << SCB_SHCSR_MEMFAULTENA_POS)
#define NVIC_MEMFAULT_ENABLE            (0x1 << SCB_SHCSR_MEMFAULTENA_POS)

#define NVIC_SVCALL_CLRPEND             (0x0 << SCB_SHCSR_SVCALLPENDED_POS)
#define NVIC_SVCALL_SETPEND             (0x1 << SCB_SHCSR_SVCALLPENDED_POS)

#define NVIC_BUSFAULT_CLRPEND           (0x0 << SCB_SHCSR_BUSFAULTPENDED_POS)
#define NVIC_BUSFAULT_SETPEND           (0x1 << SCB_SHCSR_BUSFAULTPENDED_POS)

#define NVIC_MEMFAULT_CLRPEND           (0x0 << SCB_SHCSR_MEMFAULTPENDED_POS)
#define NVIC_MEMFAULT_SETPEND           (0x1 << SCB_SHCSR_MEMFAULTPENDED_POS)

#define NVIC_USGFAULT_CLRPEND           (0x0 << SCB_SHCSR_USGFAULTPENDED_POS)
#define NVIC_USGFAULT_SETPEND           (0x1 << SCB_SHCSR_USGFAULTPENDED_POS)

#define NVIC_SYSTICK_ACTIVE             (0x1 << SCB_SHCSR_SYSTICKACT_POS)

#define NVIC_PENDSV_ACTIVE              (0x1 << SCB_SHCSR_PENDSVACT_POS)

#define NVIC_MONITOR_ACTIVE             (0x1 << SCB_SHCSR_MONITORACT_POS)

#define NVIC_SVCALL_ACTIVE              (0x1 << SCB_SHCSR_SVCALLACT_POS)

#define NVIC_USGFAULT_ACTIVE            (0x1 << SCB_SHCSR_USGFAULTACT_POS)

#define NVIC_BUSFAULT_ACTIVE            (0x1 << SCB_SHCSR_BUSFAULTACT_POS)

#define NVIC_MEMFAULT_ACTIVE            (0x1 << SCB_SHCSR_MEMFAULTACT_POS)

/* Fault Status Register */
#define SCB_CFSR_BASE                   0xE000ED28
#define SCB_CFSR                        REG32_POINTER(SCB_CFSR_BASE)

/* SCB_CFSR bit positions */
#define SCB_CFSR_DIVBYZERO_POS          25
#define SCB_CFSR_UNALIGNED_POS          24
#define SCB_CFSR_NOCP_POS               19
#define SCB_CFSR_INVPC_POS              18
#define SCB_CFSR_INVSTATE_POS           17
#define SCB_CFSR_UNDEFINSTR_POS         16
#define SCB_CFSR_BFARVALID_POS          15
#define SCB_CFSR_STKERR_POS             12
#define SCB_CFSR_UNSTKERR_POS           11
#define SCB_CFSR_IMPRECISERR_POS        10
#define SCB_CFSR_PRECISERR_POS          9
#define SCB_CFSR_IBUSERR_POS            8
#define SCB_CFSR_MMARVALID_POS          7
#define SCB_CFSR_MSTKERR_POS            4
#define SCB_CFSR_MUNSTKERR_POS          3
#define SCB_CFSR_DACCVIOL_POS           1
#define SCB_CFSR_IACCVIOL_POS           0

/* Hard Fault Status Register */
#define SCB_HFSR_BASE                   0xE000ED2C
#define SCB_HFSR                        REG32_POINTER(SCB_HFSR_BASE)

/* SCB_HFSR bit positions */
#define SCB_HFSR_DEBUGEVT_POS           31
#define SCB_HFSR_FORCED_POS             30
#define SCB_HFSR_VECTBL_POS             1

/* Debug Fault Status Register */
#define SCB_DFSR_BASE                   0xE000ED30
#define SCB_DFSR                        REG32_POINTER(SCB_DFSR_BASE)

/* SCB_DFSR bit positions */
#define SCB_DFSR_EXTERNAL_POS           4
#define SCB_DFSR_VCATCH_POS             3
#define SCB_DFSR_DWTTRAP_POS            2
#define SCB_DFSR_BKPT_POS               1
#define SCB_DFSR_HALTED_POS             0

/* Memory Management Fault Address Register */
#define SCB_MMFAR_BASE                  0xE000ED34
#define SCB_MMFAR                       READONLY_REG32_POINTER(SCB_MMFAR_BASE)

/* Bus Fault Address Register */
#define SCB_BFAR_BASE                   0xE000ED38
#define SCB_BFAR                        READONLY_REG32_POINTER(SCB_BFAR_BASE)

/* ----------------------------------------------------------------------------
 * Debug Controller
 * ------------------------------------------------------------------------- */

/* Debug Halting Control and Status Register */
#define DEBUG_DHCSR_BASE                0xE000EDF0
#define DEBUG_DHCSR                     REG32_POINTER(DEBUG_DHCSR_BASE)

/* DEBUG_DHCSR bit positions */
#define DEBUG_DHCSR_DBGKEY_POS          16
#define DEBUG_DHCSR_DBGKEY_MASK         (0xFFFF << DEBUG_DHCSR_DBGKEY_POS)
#define DEBUG_DHCSR_S_RESET_ST_POS      25
#define DEBUG_DHCSR_S_RETIRE_ST_POS     24
#define DEBUG_DHCSR_S_LOCKUP_POS        19
#define DEBUG_DHCSR_S_SLEEP_POS         18
#define DEBUG_DHCSR_S_HALT_POS          17
#define DEBUG_DHCSR_S_REGRDY_POS        16
#define DEBUG_DHCSR_C_SNAPSTALL_POS     5
#define DEBUG_DHCSR_C_MASKINTS_POS      3
#define DEBUG_DHCSR_C_STEP_POS          2
#define DEBUG_DHCSR_C_HALT_POS          1
#define DEBUG_DHCSR_C_DEBUGEN_POS       0

/* DEBUG_DHCSR settings */
#define DEBUG_HALT_KEY                  (0xA05F << DEBUG_DHCSR_DBGKEY_POS)

/* Debug Core Register Selector Register */
#define DEBUG_DCRSR_BASE                0xE000EDF4
#define DEBUG_DCRSR                     REG32_POINTER(DEBUG_DCRSR_BASE)

/* DEBUG_DCRSR bit positions */
#define DEBUG_DCRSR_REGWnR_POS          16
#define DEBUG_DCRSR_REGSEL_POS          0
#define DEBUG_DCRSR_REGSEL_MASK         (0x1F << DEBUG_DCRSR_REGSEL_POS)

/* DEBUG_DCRSR settings */
#define REGWNR_READ                     (0x0 << DEBUG_DCRSR_REGWnR_POS)
#define REGWNR_WRITE                    (0x1 << DEBUG_DCRSR_REGWnR_POS)

#define REGSEL_R0                       (0x0 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R1                       (0x1 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R2                       (0x2 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R3                       (0x3 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R4                       (0x4 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R5                       (0x5 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R6                       (0x6 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R7                       (0x7 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R8                       (0x8 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R9                       (0x9 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R10                      (0xA << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R11                      (0xB << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R12                      (0xC << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R13                      (0xD << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R14                      (0xE << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_R15                      (0xF << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_PSRFLGS                  (0x10 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_MSP                      (0x11 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_PSP                      (0x12 << DEBUG_DCRSR_REGSEL_POS)
#define REGSEL_SPECREG                  (0x14 << DEBUG_DCRSR_REGSEL_POS)

/* Debug Core Register Data Register */
#define DEBUG_DCRDR_BASE                0xE000EDF8
#define DEBUG_DCRDR                     REG32_POINTER(DEBUG_DCRDR_BASE)

/* Debug Exception and Monitor Control Register */
#define DEBUG_DEMCR_BASE                0xE000EDFC
#define DEBUG_DEMCR                     REG32_POINTER(DEBUG_DEMCR_BASE)

/* DEBUG_DEMCR bit positions */
#define DEBUG_DEMCR_TRCENA_POS          24
#define DEBUG_DEMCR_MON_REQ_POS         19
#define DEBUG_DEMCR_MON_STEP_POS        18
#define DEBUG_DEMCR_MON_PEND_POS        17
#define DEBUG_DEMCR_MON_EN_POS          16
#define DEBUG_DEMCR_VC_HARDERR_POS      10
#define DEBUG_DEMCR_VC_INTERR_POS       9
#define DEBUG_DEMCR_VC_BUSERR_POS       8
#define DEBUG_DEMCR_VC_STATERR_POS      7
#define DEBUG_DEMCR_VC_CHKERR_POS       6
#define DEBUG_DEMCR_VC_NOCPERR_POS      5
#define DEBUG_DEMCR_VC_MMERR_POS        4
#define DEBUG_DEMCR_VC_CORERESET_POS    0

/* DEBUG_DEMCR settings */
#define DEBUG_MON_CLRPEND               (0x0 << DEBUG_DEMCR_MON_PEND_POS)
#define DEBUG_MON_SETPEND               (0x1 << DEBUG_DEMCR_MON_PEND_POS)

#define DEBUG_MON_DISABLE               (0x0 << DEBUG_DEMCR_MON_EN_POS)
#define DEBUG_MON_ENABLE                (0x1 << DEBUG_DEMCR_MON_EN_POS)


#endif /* RSL10_HW_H */
